// This code conforms with the UFC specification version 1.0
// and was automatically generated by FFC version 0.7.0.
//
// Warning: This code was generated with the option '-l dolfin'
// and contains DOLFIN-specific wrappers that depend on DOLFIN.

#ifndef __POISSON3D_5_H
#define __POISSON3D_5_H

#include <cmath>
#include <stdexcept>
#include <fstream>
#include <ufc.h>
    
/// This class defines the interface for a finite element.

class poisson3d_5_0_finite_element_0: public ufc::finite_element
{
public:

  /// Constructor
  poisson3d_5_0_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_0_finite_element_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "FiniteElement('Lagrange', 'tetrahedron', 5)";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 56;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
    
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
    
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    static const double coefficients0[56][56] = \
    {{0.00945067404923492, 0.00609788120679252, 0.00352061335622803, 0.00248944957812476, 0.0070594623332504, 0.00546823600997146, 0.00315708753234943, 0.00446479600584755, 0.00257775117585285, 0.00182274533665715, -0.00298267418725716, -0.0025208197798293, -0.00195261860521383, -0.00112734487734488, -0.00218309396769447, -0.00169101731601731, -0.000976309302606919, -0.00138070985681772, -0.000797153207506487, -0.00056367243867244, 0.00278687112460004, 0.00245778931055954, 0.00207721109304566, 0.00160900079398463, 0.000928957041533343, 0.00219831358911339, 0.0018579140830667, 0.00143913406048032, 0.00083088443721827, 0.00160900079398463, 0.0012463266558274, 0.000719567030240151, 0.00101762145320217, 0.000587524019939407, 0.000415442218609132, -0.00705593524717798, -0.00638233357089081, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363026, -0.00582624677716565, -0.00513826668309062, -0.00434262794914383, -0.00336378514516178, -0.00194208225905522, -0.00459580543596519, -0.00388416451811043, -0.00300866089851429, -0.00173705117965752, -0.00336378514516178, -0.00260557676948629, -0.00150433044925714, -0.00212744452363027, -0.00122828066840393, -0.000868525589828762},
    {0.00945067404923495, -0.00609788120679253, 0.00352061335622802, 0.00248944957812476, 0.00705946233325039, -0.00546823600997146, 0.00315708753234944, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521383, -0.00112734487734489, -0.00218309396769448, 0.00169101731601732, -0.000976309302606925, 0.00138070985681772, -0.000797153207506488, -0.000563672438672441, 0.00278687112460004, -0.00245778931055954, 0.00207721109304566, -0.00160900079398463, 0.000928957041533346, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218268, 0.00160900079398463, -0.0012463266558274, 0.000719567030240148, -0.00101762145320216, 0.000587524019939402, 0.000415442218609128, 0.00705593524717798, -0.0063823335708908, 0.00562868913761196, -0.00475711057319694, 0.00368484200521179, -0.00212744452363027, -0.00582624677716565, 0.00513826668309062, -0.00434262794914382, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811043, 0.00300866089851429, -0.00173705117965752, -0.00336378514516178, 0.00260557676948629, -0.00150433044925714, 0.00212744452363026, -0.00122828066840393, -0.00086852558982876},
    {0.00945067404923495, 0, -0.00704122671245605, 0.00248944957812476, 0, 0, 0.00947126259704828, 0, -0.00515550235170572, 0.00182274533665715, 0, 0, 0, 0.00450937950937951, 0, 0, -0.00292892790782075, 0, 0.00159430641501296, -0.00056367243867244, 0, 0, 0, 0, 0.00464478520766673, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987881, 0.00041544221860913, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527608, 0, 0, 0, 0.00694820471863012, 0, 0, -0.00451299134777144, 0, 0.00245656133680786, -0.000868525589828763},
    {0.00945067404923493, 0, 0, -0.0074683487343743, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00563672438672439, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.018239037386404},
    {-0.010023442173431, 0, 0.0374718748311399, -0.0289614183593723, 0, 0, 0.0284137877911448, 0, 0.00515550235170572, -0.00619733414463434, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835494, -0.000187890812890802, 0, 0, 0, 0, 0.0232239260383337, 0, 0, 0, -0.00166176887443652, 0, 0, -0.00647610327216137, 0, 0.00646276421933342, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496074, 0, 0, 0, -0.0277928188745205, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.010023442173431, 0, -0.0287574853355259, 0.05360943398437, 0, 0, -0.0189425251940966, 0, 0.0154665070551171, 0.0153110608279201, 0, 0, 0, -0.00450937950937953, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144092, 0, -0.0123380044187274, 0.00872428659079174, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559316, 0, 0.041270230458372, -0.018239037386404},
    {0.010023442173431, 0, 0.0409576306293854, -0.0449826285156208, 0, 0, 0.0189425251940965, 0, -0.00515550235170569, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883226, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957583, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.055026973944496, 0.0303983956440067},
    {-0.0100234421734311, 0, -0.0148144621425437, 0.0449826285156207, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0187359374155699, -0.0289614183593723, 0.0211783869997512, -0.0164047080299144, 0.00947126259704827, 0.00446479600584757, -0.00257775117585285, -0.00619733414463432, 0.0208787193108001, -0.0176457384588052, 0.0136683302364968, -0.00789141414141413, -0.0084898098743674, 0.00657617845117844, -0.00379675839902689, 0.0023011830946962, -0.00132858867917746, -0.0001878908128908, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992314, 0.00464478520766674, -0.0010991567945567, 0.000928957041533349, -0.000719567030240152, 0.000415442218609127, -0.00482700238195388, 0.00373897996748219, -0.00215870109072045, 0.00559691799261185, -0.00323138210966672, -0.00290809553026392, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935591, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863011, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778515, 0.00607967912880135},
    {0.010023442173431, -0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, 0.0109364720199429, -0.00631417506469883, 0.0133943880175427, -0.00773325352755858, 0.0153110608279201, -0.00298267418725716, 0.00252081977982931, -0.00195261860521382, 0.00112734487734488, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476002, 0.0129522065443227, -0.00747795993496437, -0.00321800158796926, 0.0024926533116548, -0.00143913406048031, -0.0106850252586226, 0.00616900220936373, 0.00872428659079178, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845888, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.020635115229186, -0.0182390373864041},
    {0.0100234421734311, 0.0354703486038674, -0.0204788153146927, -0.0449826285156208, 0.0141189246665008, -0.0109364720199429, 0.00631417506469885, -0.00446479600584758, 0.00257775117585285, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974696, -0.00263047138047138, 0.00151870335961075, 0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620757, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, -0.0128297005588457, 0.00740723107127184, 0.0449826285156208, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0.00920473237878476, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489395, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.010023442173431, 0.0324515955311979, -0.0335503995581136, -0.00801060507812427, 0.0211783869997512, 0.00109364720199429, -0.00568275755822898, -0.0169662248222207, 0.0056710525868763, 0.00838462854862291, 0.0208787193108001, -0.0109235523792603, 0.00499002532443534, -0.00187890812890812, -0.0162519217595033, 0.00958243145743146, -0.00466458889023305, 0.0108922666482287, -0.00558007245254534, -0.00469727032227033, 0.0139343556230002, -0.00245778931055954, -0.00415442218609131, 0.00643600317593851, -0.00464478520766673, -0.0120907247401236, 0.00464478520766674, -0.000719567030240153, -0.000415442218609137, 0.00965400476390778, -0.00498530662330958, 0.00215870109072046, -0.006614539445814, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543313, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438609, 0.00310733161448835, -0.00481385743762287, 0.00347410235931505, -0.00201827108709706, -0.000521115353897262, 0.000902598269554289, 0.00170195561890421, -0.000491312267361565, -0.000868525589828764},
    {0.0100234421734309, -0.024904712849524, 0.0553363732971484, -0.0043134027343746, -0.0141189246665008, 0.0153110608279201, 0.00883984509057843, 0.00803663281052563, -0.0159820572902877, 0.000729098134662861, -0.00298267418725715, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846298, -0.0116492303992304, 0.00976309302606916, 0.00352848074520084, 0.00132858867917745, -0.00263047138047137, 0, 0.0196623144844763, -0.00727023882565982, -0.00724050357293082, 0.00928957041533346, -0.00219831358911338, -0.0148633126645336, 0.0129522065443228, -0.0066470754977461, 0.00402250198496157, 0.00685479660705067, -0.00575653624192123, -0.0045792965394097, -0.000293762009969684, 0.00290809553026392, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.00411061334647249, -0.00303983956440068, 0.00908221989193681, -0.00776832903622088, 0.00045958054359652, 0.00427258096992148, -0.00270779480866287, 0.000868525589828773, -0.00100913554354854, -0.00234501909253766, 0.00180519653910858, 0.00127646671417816, 0.000245656133680781, -0.000868525589828764},
    {0.0100234421734309, 0.0354703486038675, -0.0492363006502187, -0.00431340273437461, 0.0141189246665008, -0.00656188321196574, -0.016416855168217, -0.00982255121286466, 0.0149509568199466, 0.000729098134662872, 0, 0.0033610930397724, 0.0282044909641998, -0.0353234728234728, -0.000485131992820993, -0.0124007936507936, 0.00889526253486299, 0.00291483191994851, 0.00239145962251946, -0.00263047138047138, 0, 0, 0.0218107164769794, -0.00402250198496158, -0.00928957041533347, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992316, 0.00934744991870547, -0.0028782681209606, -0.00254405363300539, -0.00381890612960613, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320202, 0.00840946286290443, -0.00388416451811042, 0, 0.00233049871086626, 0.0030086608985143, -0.00347410235931507, -0.000336378514516178, -0.00286613444643493, 0.00120346435940572, 0.00085097780945211, 0.00098262453472315, -0.000868525589828765},
    {-0.010023442173431, -0.0128297005588457, 0.0448791059024117, -0.00801060507812424, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467804, -0.0175287079957994, 0.0083846285486229, 0, 0, 0.00867830491206149, 0.0225468975468975, 0, -0.00300625300625301, -0.019200749617936, 0.000613648825252316, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240152, -0.0108014976838374, 0, 0.00249265331165478, 0.0107935054536023, -0.000508810726601074, -0.00734405024924252, 0.00290809553026393, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931505, 0, -0.00208446141558903, -0.000902598269554287, 0.000425488904726055, 0.0017195929357655, -0.000868525589828767},
    {-0.0100234421734311, -0.0324515955311978, -0.0187359374155699, -0.0289614183593723, 0.0211783869997511, 0.0164047080299144, 0.00947126259704826, -0.00446479600584755, -0.00257775117585285, -0.00619733414463435, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141413, -0.00848980987436742, -0.00657617845117845, -0.0037967583990269, -0.00230118309469619, -0.00132858867917746, -0.00018789081289081, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992313, 0.00464478520766674, 0.00109915679455668, 0.000928957041533345, 0.000719567030240154, 0.000415442218609131, -0.00482700238195388, -0.00373897996748219, -0.00215870109072045, -0.00559691799261187, -0.00323138210966673, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.00955378148811641, 0.00740032731935591, 0.00427258096992148, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.0069482047186301, 0.0181644397838736, 0.014070114555226, 0.00812338442598859, 0.0136156449512337, 0.00786099627778516, 0.00607967912880135},
    {0.0100234421734308, 0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, -0.0109364720199429, -0.00631417506469886, -0.0133943880175427, -0.00773325352755858, 0.0153110608279201, 0.00298267418725716, 0.00252081977982933, 0.00195261860521383, 0.00112734487734487, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476002, -0.0129522065443228, -0.00747795993496438, -0.00321800158796927, -0.00249265331165479, -0.00143913406048031, 0.0106850252586227, 0.00616900220936374, 0.00872428659079177, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.0206351152291861, -0.0182390373864041},
    {0.010023442173431, -0.0354703486038674, -0.0204788153146927, -0.0449826285156208, 0.0141189246665008, 0.0109364720199429, 0.00631417506469886, 0.00446479600584758, 0.00257775117585287, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974696, 0.00263047138047138, 0.00151870335961075, -0.0308358534689291, -0.0178030883009781, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620757, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588457, 0.00740723107127183, 0.0449826285156208, 0, 0, 0, -0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878479, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489395, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.010023442173431, -0.0324515955311978, -0.0335503995581136, -0.00801060507812424, 0.0211783869997512, -0.00109364720199429, -0.00568275755822899, 0.0169662248222207, 0.00567105258687632, 0.0083846285486229, -0.0208787193108001, -0.0109235523792603, -0.00499002532443534, -0.00187890812890812, -0.0162519217595033, -0.00958243145743146, -0.00466458889023305, -0.0108922666482287, -0.00558007245254535, -0.00469727032227032, 0.0139343556230002, 0.00245778931055953, -0.00415442218609133, -0.00643600317593852, -0.00464478520766672, 0.0120907247401236, 0.00464478520766673, 0.000719567030240156, -0.000415442218609149, 0.00965400476390777, 0.00498530662330958, 0.00215870109072047, 0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543313, 0.00719357335632687, 0.0112908326677739, 0.0114368694935501, 0.00737991258440984, -0.00183832217438608, 0.00310733161448834, 0.00481385743762287, 0.00347410235931505, -0.00201827108709706, 0.000521115353897262, 0.000902598269554287, -0.00170195561890421, -0.000491312267361565, -0.000868525589828767},
    {0.010023442173431, 0.0249047128495239, 0.0553363732971484, -0.00431340273437459, -0.0141189246665008, -0.0153110608279201, 0.00883984509057843, -0.00803663281052563, -0.0159820572902878, 0.000729098134662854, 0.00298267418725716, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846298, 0.0116492303992304, 0.00976309302606918, -0.00352848074520084, 0.00132858867917746, -0.00263047138047138, 0, -0.0196623144844763, -0.0072702388256598, 0.00724050357293083, 0.00928957041533346, 0.00219831358911339, -0.0148633126645336, -0.0129522065443228, -0.00664707549774609, 0.00402250198496157, -0.00685479660705067, -0.00575653624192124, 0.0045792965394097, -0.000293762009969679, 0.00290809553026392, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.0041106133464725, -0.00303983956440067, -0.00908221989193682, -0.00776832903622089, -0.00045958054359652, 0.00427258096992148, 0.00270779480866287, 0.000868525589828776, -0.00100913554354853, 0.00234501909253766, 0.00180519653910858, -0.00127646671417816, 0.000245656133680779, -0.000868525589828761},
    {0.0100234421734309, -0.0354703486038674, -0.0492363006502187, -0.00431340273437459, 0.0141189246665008, 0.00656188321196577, -0.016416855168217, 0.00982255121286466, 0.0149509568199466, 0.000729098134662868, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820994, 0.0124007936507936, 0.00889526253486299, -0.00291483191994852, 0.00239145962251945, -0.00263047138047138, 0, 0, 0.0218107164769794, 0.00402250198496157, -0.00928957041533346, 0, -0.00557374224920009, 0.0143913406048031, 0.0141250354327105, 0.000804500396992309, -0.00934744991870548, -0.0028782681209606, 0.00254405363300539, -0.00381890612960614, 0.00290809553026393, 0, 0, 0, 0.0266398192099028, 0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320203, -0.00840946286290443, -0.00388416451811041, 0, 0.00233049871086626, -0.0030086608985143, -0.00347410235931507, -0.000336378514516177, 0.00286613444643493, 0.00120346435940572, -0.000850977809452108, 0.000982624534723152, -0.000868525589828766},
    {-0.010023442173431, 0.0128297005588457, 0.0448791059024117, -0.00801060507812424, 0, -0.0174983552319087, 0.0132597676358676, 0.00357183680467805, -0.0175287079957994, 0.0083846285486229, 0, 0, -0.00867830491206148, 0.0225468975468975, 0, 0.00300625300625301, -0.019200749617936, -0.000613648825252319, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766673, 0, 0, 0.00719567030240153, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601077, -0.00734405024924252, 0.00290809553026393, 0, 0, 0, 0, -0.0257938940364826, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702859, -0.00347410235931505, 0, 0.00208446141558903, -0.000902598269554286, -0.000425488904726054, 0.0017195929357655, -0.000868525589828767},
    {-0.0100234421734311, -0.0452812960900435, -0.0113287063442981, -0.00801060507812424, -0.0014118924666501, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.00838462854862289, -0.00894802256177147, -0.0142846454190328, -0.014536160727703, -0.00939454064454064, -0.0123708658169354, -0.0125886844636845, -0.00813591085505764, -0.0102786178229763, -0.00664294339588733, -0.00469727032227033, -0.00836061337380012, 0.00245778931055953, 0.00830884437218264, 0.00965400476390777, 0.00650269929073344, 0.00219831358911338, 0.00743165633226677, 0.00863480436288183, 0.00581619106052785, 0.00643600317593852, 0.00747795993496437, 0.00503696921168108, 0.00610572871921293, 0.00411266813957582, 0.00290809553026394, 0.0352796762358899, 0.0191470007126724, 0.00788016479265675, 0.000951422114639391, -0.00221090520312707, -0.00212744452363027, 0.017478740331497, 0.00719357335632686, 0.000868525589828764, -0.00201827108709706, -0.00194208225905521, 0.00643412761035128, 0.000776832903622088, -0.00180519653910857, -0.00173705117965753, 0.000672757029032363, -0.00156334606169177, -0.00150433044925715, -0.00127646671417816, -0.00122828066840393, -0.000868525589828768},
    {0.010023442173431, 0.0603750614533914, -0.00610007264692975, -0.0043134027343746, -0.00564756986660025, -0.0218729440398859, 0.00126283501293978, -0.0178591840233903, 0.00103110047034114, 0.000729098134662884, 0.0417574386216002, 0.0168054651988621, -0.000867830491206148, -0.00526094276094276, 0.0145539597846299, -0.000751563251563252, -0.00455611007883227, -0.000613648825252328, -0.0037200483016969, -0.00263047138047138, 0.00557374224920005, -0.0196623144844763, -0.0103860554652283, 0.00321800158796928, 0.00650269929073341, -0.0175865087129071, -0.00928957041533347, 0.00287826812096063, 0.00581619106052784, -0.00804500396992315, 0.0024926533116548, 0.00503696921168106, 0.00203524290640433, 0.00411266813957581, 0.00290809553026391, -0.0705593524717798, -0.0127646671417816, 0.00450295131008958, 0.00380568845855754, -0.000736968401042372, -0.00212744452363026, -0.0116524935543313, 0.00411061334647251, 0.00347410235931506, -0.000672757029032362, -0.00194208225905521, 0.00367664434877218, 0.00310733161448835, -0.000601732179702865, -0.00173705117965753, 0.00269102811612942, -0.000521115353897265, -0.00150433044925715, -0.00042548890472606, -0.00122828066840393, -0.000868525589828759},
    {0.0100234421734309, -0.0603750614533914, -0.00610007264692972, -0.00431340273437457, -0.00564756986660032, 0.0218729440398859, 0.00126283501293976, 0.0178591840233903, 0.00103110047034113, 0.000729098134662861, -0.0417574386216002, 0.016805465198862, 0.000867830491206157, -0.00526094276094276, 0.0145539597846298, 0.000751563251563257, -0.00455611007883228, 0.00061364882525233, -0.00372004830169691, -0.00263047138047138, 0.00557374224920011, 0.0196623144844763, -0.0103860554652283, -0.00321800158796927, 0.00650269929073344, 0.0175865087129071, -0.00928957041533346, -0.00287826812096062, 0.00581619106052786, -0.00804500396992314, -0.0024926533116548, 0.00503696921168108, -0.00203524290640433, 0.00411266813957583, 0.00290809553026393, 0.0705593524717798, -0.0127646671417816, -0.00450295131008958, 0.00380568845855756, 0.000736968401042369, -0.00212744452363027, -0.0116524935543313, -0.00411061334647251, 0.00347410235931506, 0.000672757029032361, -0.00194208225905522, -0.00367664434877217, 0.00310733161448835, 0.000601732179702866, -0.00173705117965753, 0.00269102811612942, 0.000521115353897263, -0.00150433044925715, 0.000425488904726061, -0.00122828066840394, -0.000868525589828769},
    {-0.0100234421734309, 0.0452812960900435, -0.0113287063442981, -0.00801060507812426, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089231, 0.00838462854862291, 0.00894802256177147, -0.0142846454190327, 0.014536160727703, -0.00939454064454063, -0.0123708658169354, 0.0125886844636845, -0.00813591085505763, 0.0102786178229763, -0.00664294339588732, -0.00469727032227032, -0.00836061337380015, -0.00245778931055953, 0.00830884437218263, -0.00965400476390777, 0.00650269929073343, -0.00219831358911339, 0.00743165633226677, -0.00863480436288183, 0.00581619106052785, 0.00643600317593852, -0.00747795993496437, 0.00503696921168107, -0.00610572871921293, 0.00411266813957582, 0.00290809553026393, -0.0352796762358899, 0.0191470007126724, -0.00788016479265675, 0.000951422114639385, 0.00221090520312707, -0.00212744452363027, 0.0174787403314969, -0.00719357335632687, 0.000868525589828768, 0.00201827108709706, -0.00194208225905521, -0.00643412761035128, 0.000776832903622089, 0.00180519653910857, -0.00173705117965753, 0.000672757029032358, 0.00156334606169177, -0.00150433044925715, 0.00127646671417816, -0.00122828066840393, -0.000868525589828765},
    {0.0787556170769579, 0.0679219441350653, -0.00435719474780691, -0.00308100195312472, 0.0705946233325039, -0.0328094160598287, 0.0126283501293977, -0.0267887760350854, 0.03608851646194, -0.00729098134662862, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015873, 0.0130985638061669, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, 0.0245778931055954, -0.036351194128299, 0.0362025178646541, -0.0232239260383337, 0.0219831358911338, -0.00743165633226679, 0, 0.00166176887443652, -0.0345935170706695, 0.0130864298861876, -0.00359783515120079, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596521, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598858, -0.0102117337134253, 0.00196524906944628, 0.00607967912880136},
    {-0.0214788046573522, 0.00377344134083702, 0.00217859737390344, -0.0215670136718729, 0, 0.0984282481794862, -0.0852413633734345, -0.0312535720409329, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, 0.0236742424242424, -0.0335199527228374, -0.00138070985681772, -0.00079715320750645, 0.00394570707070704, 0, 0, 0.036351194128299, -0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.00724050357293084, -0.0392592896585629, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724418, 0, 0.00694820471863008, 0.00302740663064561, 0.0164151336477636, -0.0108311792346515, -0.00680782247561686, -0.00393049813889256, 0.00607967912880134},
    {0.0787556170769579, 0.0301875307266957, 0.0610007264692975, -0.00308100195312472, 0, 0.0218729440398858, 0.0757701007763863, 0.0178591840233903, -0.0412440188136458, -0.00729098134662862, 0, 0, 0.0520698294723688, -0.00901875901875902, 0, -0.0127765752765753, 0.0286384062098028, 0.00184094647575699, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.0143913406048031, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515274, 0.00407048581280861, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967795, -0.0427258096992148, 0, 0, -0.0300866089851429, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723144, 0.00607967912880136},
    {-0.0214788046573522, 0.00377344134083686, -0.0196073763651313, 0.00924300585937417, 0, -0.0109364720199429, 0.00947126259704823, 0.102690308134494, -0.0695992817480272, -0.060150596109686, 0, -0.00504163955965862, 0.0065087286840461, -0.00450937950937951, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196709, -0.0263060558477138, 0.000563672438672465, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165478, -0.0043174021814409, -0.0605484764655283, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573521, -0.0150937653633479, 0.0130715842434208, 0.00924300585937414, 0, -0.0109364720199429, 0.00947126259704832, -0.00892959201169514, 0.123732056440937, -0.0601505961096861, 0, 0, -0.00650872868404609, 0.00676406926406927, 0, 0.0574945887445887, 0.0126920209338899, -0.0266937238984759, 0.00637722566005184, 0.000563672438672453, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097862, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448302, -0.0616900220936373, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769578, 0.0301875307266958, 0.0174287789912279, 0.0585390371093694, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094276, -0.00455611007883226, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700864, 0.0284934006896604, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769578, -0.0679219441350653, -0.00435719474780693, -0.00308100195312472, 0.0705946233325039, 0.0328094160598287, 0.0126283501293978, 0.0267887760350854, 0.03608851646194, -0.00729098134662859, -0.0298267418725716, 0.0151249186789758, 0.0325436434202305, 0.0248015873015873, 0.0130985638061669, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646541, -0.0232239260383337, -0.0219831358911339, -0.00743165633226677, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861877, -0.00359783515120079, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.0045958054359652, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.00908221989193679, -0.00703505727761299, -0.00812338442598858, 0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573524, -0.00377344134083698, 0.00217859737390344, -0.021567013671873, 0, -0.0984282481794863, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242424, -0.0335199527228374, 0.0013807098568177, -0.00079715320750645, 0.00394570707070706, 0, 0, 0.036351194128299, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838375, -0.00724050357293083, 0.039259289658563, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951694, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.00302740663064561, -0.0164151336477636, -0.0108311792346515, 0.00680782247561685, -0.00393049813889256, 0.00607967912880134},
    {0.0787556170769579, -0.0301875307266956, 0.0610007264692975, -0.00308100195312471, 0, -0.0218729440398858, 0.0757701007763863, -0.0178591840233903, -0.0412440188136458, -0.00729098134662861, 0, 0, -0.0520698294723689, -0.00901875901875903, 0, 0.0127765752765753, 0.0286384062098028, -0.00184094647575695, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766674, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515274, -0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967796, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780843, -0.00982624534723145, 0.00607967912880137},
    {-0.0214788046573523, -0.0037734413408369, -0.0196073763651314, 0.00924300585937414, 0, 0.0109364720199429, 0.00947126259704826, -0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965861, -0.00650872868404608, -0.00450937950937948, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196707, -0.0263060558477138, 0.000563672438672444, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.0024926533116548, -0.00431740218144091, 0.0605484764655282, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, 0.0150937653633478, 0.0130715842434208, 0.00924300585937414, 0, 0.0109364720199429, 0.00947126259704831, 0.00892959201169514, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404611, 0.00676406926406927, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005185, 0.000563672438672461, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097862, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448302, -0.0616900220936373, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769578, -0.0301875307266956, 0.0174287789912279, 0.0585390371093695, 0, -0.0218729440398858, -0.0189425251940966, -0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, -0.00526094276094275, -0.00455611007883226, -0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700864, -0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769576, -0.0377344134083696, -0.0566435317214905, -0.00308100195312473, 0.0423567739995023, 0.0546823600997146, 0.0378850503881931, 0.0446479600584756, 0.00515550235170574, -0.00729098134662862, 0.0417574386216002, -0.0050416395596586, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801536, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055954, -0.00519302773261415, 0.00402250198496157, 0.00464478520766674, -0.00219831358911335, -0.0130053985814669, -0.0158304746652834, -0.0108014976838374, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.00477689074405821, 0.00370016365967795, 0.00427258096992148, -0.0367664434877216, -0.00776832903622087, 0.00601732179702858, 0.00694820471863011, -0.00908221989193682, 0.00703505727761298, 0.0081233844259886, 0.00680782247561684, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573522, 0, -0.0043571947478069, -0.0215670136718729, -0.127070321998507, 0, 0.0284137877911448, 0, 0.03608851646194, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984381, 0, 0.00159430641501295, 0.00394570707070706, 0.0836061337380013, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377579, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.0095537814881164, 0, 0.00427258096992147, 0, -0.0155366580724417, 0, 0.00694820471863011, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778513, 0.00607967912880133},
    {0.0787556170769579, 0.0377344134083696, -0.0566435317214905, -0.0030810019531247, 0.0423567739995023, -0.0546823600997146, 0.0378850503881931, -0.0446479600584757, 0.00515550235170574, -0.00729098134662864, -0.0417574386216002, -0.00504163955965862, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801536, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261416, -0.00402250198496158, 0.00464478520766674, 0.00219831358911339, -0.0130053985814669, 0.0158304746652834, -0.0108014976838374, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405821, -0.00370016365967795, 0.00427258096992148, 0.0367664434877216, -0.00776832903622088, -0.00601732179702859, 0.00694820471863012, -0.00908221989193681, -0.00703505727761299, 0.00812338442598861, -0.00680782247561685, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573522, -0.0188672067041847, 0.00653579212171051, 0.00924300585937415, 0.0141189246665008, 0, -0.00315708753234942, -0.111619900146189, -0.0541327746929101, -0.0601505961096861, -0.00894802256177146, -0.00252081977982933, 0.000650872868404606, 0.00112734487734486, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672439, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476002, -0.00431740218144093, -0.00747795993496438, 0.00321800158796924, 0.0224338798048931, 0.018708742786244, 0.046301776120698, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.017478740331497, -0.00451299134777145, -0.00781673030845887, 0.0363288795677472, -0.00938007637015066, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {-0.0214788046573521, 0.0188672067041847, 0.00653579212171047, 0.00924300585937414, 0.0141189246665007, 0, -0.00315708753234944, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177148, -0.00252081977982932, -0.000650872868404592, 0.00112734487734487, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.019928830187662, 0.000563672438672456, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476002, 0.00431740218144092, -0.00747795993496438, 0.00321800158796927, -0.0224338798048931, 0.018708742786244, -0.0463017761206981, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777143, -0.00781673030845888, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769579, 0, -0.0348575579824557, 0.0585390371093695, -0.0282378493330015, 0, 0.00631417506469885, 0, -0.0206220094068228, 0.083846285486229, 0, 0, 0, 0, -0.0067918478994939, 0, 0.00151870335961074, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233621, 0, -0.0329013451166066, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.0275134869722481, 0.0303983956440068},
    {0.0787556170769578, -0.0377344134083696, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, 0.0546823600997145, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865145, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.0092175078635989, -0.00789141414141412, -0.00347132196482459, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, -0.00491557862111904, -0.0197335053839338, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097694, -0.000623163327913701, -0.00215870109072047, 0.00712335017241508, 0.00411266813957581, 0.00540074884191871, 0, -0.0510586685671264, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173251, -0.0143871467126537, -0.00564541633388696, 0.00571843474677502, 0.00737991258440982, 0.00367664434877216, -0.00155366580724418, 0.00240692871881142, 0.00347410235931506, 0.00100913554354853, 0.00026055767694863, 0.000902598269554296, -0.000850977809452108, -0.000491312267361565, -0.000868525589828763},
    {-0.0214788046573521, 0, -0.0217859737390348, 0.00308100195312476, -0.127070321998507, 0, 0.0473563129852414, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468977, 0.0509388592462045, 0, -0.00943765659186683, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533349, 0, -0.0141250354327105, -0.0144810071458616, 0, -0.00215870109072044, 0, 0.00411266813957584, 0.00540074884191874, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259877, 0, -0.0112908326677739, 0, 0.00737991258440982, 0, -0.00310733161448833, 0, 0.00347410235931505, 0.00201827108709706, 0, 0.000902598269554279, 0, -0.000491312267361592, -0.00086852558982878},
    {0.078755617076958, 0.0377344134083695, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584757, 0.0257775117585286, 0.0291639253865145, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.0092175078635989, 0.00789141414141413, -0.00347132196482461, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111908, -0.0197335053839338, 0.0394205194526234, -0.0325134964536672, 0.0241814494802473, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097695, 0.000623163327913697, -0.00215870109072046, -0.0071233501724151, 0.00411266813957581, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908062, -0.0114170653756727, -0.010317557614593, 0.0127646671417816, 0.00466099742173252, 0.0143871467126538, -0.00564541633388697, -0.00571843474677502, 0.00737991258440983, -0.00367664434877217, -0.00155366580724418, -0.00240692871881144, 0.00347410235931506, 0.00100913554354853, -0.000260557676948628, 0.000902598269554287, 0.000850977809452107, -0.000491312267361569, -0.000868525589828763},
    {-0.0214788046573521, -0.0188672067041848, 0.0108929868695174, 0.0030810019531247, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792643, 0.0127592173566001, -0.0089480225617715, -0.0277290175781224, -0.00585785581564147, 0.0191648629148629, 0.00145539597846298, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755831, -0.0129644660894661, 0, 0.0589869434534289, 0.00727023882565979, 0.0506835250105158, 0.0650269929073344, -0.00659494076734015, 0.0148633126645335, -0.00431740218144092, -0.00166176887443655, -0.00402250198496158, -0.00810112326287807, -0.0115130724838425, 0.00356167508620755, -0.00205633406978791, 0.00540074884191871, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440066, -0.00302740663064559, -0.00776832903622087, 0.00137874163078956, -0.00427258096992148, 0.000902598269554293, 0.000868525589828758, 0.00100913554354854, 0.000781673030845887, 0.00180519653910857, -0.000425488904726058, 0.00024565613368079, -0.000868525589828762},
    {-0.0214788046573521, 0.0188672067041848, 0.0108929868695174, 0.00308100195312469, 0.0141189246665007, 0.109364720199429, -0.0789271883087357, -0.0223239800292378, -0.0128887558792643, 0.0127592173566001, 0.00894802256177149, -0.0277290175781224, 0.00585785581564149, 0.0191648629148629, 0.00145539597846298, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755832, -0.0129644660894661, 0, -0.0589869434534289, 0.00727023882565983, -0.0506835250105158, 0.0650269929073343, 0.00659494076734015, 0.0148633126645336, 0.00431740218144092, -0.00166176887443653, -0.00402250198496157, 0.00810112326287806, -0.0115130724838425, -0.00356167508620755, -0.00205633406978791, 0.00540074884191872, 0, 0, -0.0607898426862092, 0.0399597288148543, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440068, 0.0030274066306456, -0.00776832903622088, -0.00137874163078956, -0.00427258096992148, -0.000902598269554298, 0.000868525589828757, 0.00100913554354854, -0.000781673030845891, 0.00180519653910858, 0.000425488904726056, 0.000245656133680792, -0.000868525589828762},
    {0.078755617076958, 0, 0.0435719474780696, -0.0523770332031201, -0.0282378493330015, 0, 0.0631417506469885, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.00672218607954481, 0, -0.0601250601250601, 0.000970263985641994, 0, -0.0125835421224892, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539589, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398463, 0, -0.0028782681209606, 0, -0.00822533627915163, 0.00540074884191871, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.018239037386404, 0, -0.00388416451811042, 0, -0.00466099742173252, 0, -0.00347410235931505, 0.000672757029032356, 0, 0.00120346435940571, 0, 0.000982624534723144, -0.000868525589828765},
    {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, -0.0546823600997146, -0.0315708753234943, -0.0446479600584757, -0.0257775117585285, -0.0182274533665716, 0.0894802256177147, -0.0151249186789759, 0.0585785581564149, 0.0744047619047619, -0.0130985638061669, -0.0281836219336219, 0.0097630930260691, 0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226673, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978424, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905521, -0.00902598269554287, -0.0191075629762328, -0.00908221989193678, -0.00234501909253766, -0.00812338442598858, 0.00340391123780844, 0.00196524906944629, 0.00607967912880135},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, 0.0546823600997146, -0.0315708753234943, 0.0446479600584756, -0.0257775117585286, -0.0182274533665715, -0.0894802256177148, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061669, 0.0281836219336219, 0.00976309302606918, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.0363511941282991, 0.0120675059548847, -0.0232239260383337, -0.0659494076734016, 0.00743165633226682, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978425, 0.0251848460584054, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554289, -0.0191075629762328, -0.00908221989193679, 0.00234501909253767, -0.00812338442598859, -0.00340391123780842, 0.00196524906944628, 0.00607967912880136},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281177, 0, 0, -0.0947126259704827, 0, 0.0515550235170571, -0.0182274533665715, 0, -0.0907495120738551, 0, -0.0946969696969696, 0.0232863356554078, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992785, 0, 0, -0.072702388256598, 0, 0.0464478520766673, 0, -0.048305766159734, 0, -0.0781031370985167, 0.0144810071458617, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992147, 0, 0.0310733161448835, 0, 0.00694820471863012, -0.00605481326129122, 0, -0.0108311792346515, 0, -0.00393049813889258, 0.00607967912880135},
    {0.107394023286761, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937952, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398575, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259878, 0, 0.0312669212338355, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    const double coeff0_6 = coefficients0[dof][6];
    const double coeff0_7 = coefficients0[dof][7];
    const double coeff0_8 = coefficients0[dof][8];
    const double coeff0_9 = coefficients0[dof][9];
    const double coeff0_10 = coefficients0[dof][10];
    const double coeff0_11 = coefficients0[dof][11];
    const double coeff0_12 = coefficients0[dof][12];
    const double coeff0_13 = coefficients0[dof][13];
    const double coeff0_14 = coefficients0[dof][14];
    const double coeff0_15 = coefficients0[dof][15];
    const double coeff0_16 = coefficients0[dof][16];
    const double coeff0_17 = coefficients0[dof][17];
    const double coeff0_18 = coefficients0[dof][18];
    const double coeff0_19 = coefficients0[dof][19];
    const double coeff0_20 = coefficients0[dof][20];
    const double coeff0_21 = coefficients0[dof][21];
    const double coeff0_22 = coefficients0[dof][22];
    const double coeff0_23 = coefficients0[dof][23];
    const double coeff0_24 = coefficients0[dof][24];
    const double coeff0_25 = coefficients0[dof][25];
    const double coeff0_26 = coefficients0[dof][26];
    const double coeff0_27 = coefficients0[dof][27];
    const double coeff0_28 = coefficients0[dof][28];
    const double coeff0_29 = coefficients0[dof][29];
    const double coeff0_30 = coefficients0[dof][30];
    const double coeff0_31 = coefficients0[dof][31];
    const double coeff0_32 = coefficients0[dof][32];
    const double coeff0_33 = coefficients0[dof][33];
    const double coeff0_34 = coefficients0[dof][34];
    const double coeff0_35 = coefficients0[dof][35];
    const double coeff0_36 = coefficients0[dof][36];
    const double coeff0_37 = coefficients0[dof][37];
    const double coeff0_38 = coefficients0[dof][38];
    const double coeff0_39 = coefficients0[dof][39];
    const double coeff0_40 = coefficients0[dof][40];
    const double coeff0_41 = coefficients0[dof][41];
    const double coeff0_42 = coefficients0[dof][42];
    const double coeff0_43 = coefficients0[dof][43];
    const double coeff0_44 = coefficients0[dof][44];
    const double coeff0_45 = coefficients0[dof][45];
    const double coeff0_46 = coefficients0[dof][46];
    const double coeff0_47 = coefficients0[dof][47];
    const double coeff0_48 = coefficients0[dof][48];
    const double coeff0_49 = coefficients0[dof][49];
    const double coeff0_50 = coefficients0[dof][50];
    const double coeff0_51 = coefficients0[dof][51];
    const double coeff0_52 = coefficients0[dof][52];
    const double coeff0_53 = coefficients0[dof][53];
    const double coeff0_54 = coefficients0[dof][54];
    const double coeff0_55 = coefficients0[dof][55];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9 + coeff0_10*basisvalue10 + coeff0_11*basisvalue11 + coeff0_12*basisvalue12 + coeff0_13*basisvalue13 + coeff0_14*basisvalue14 + coeff0_15*basisvalue15 + coeff0_16*basisvalue16 + coeff0_17*basisvalue17 + coeff0_18*basisvalue18 + coeff0_19*basisvalue19 + coeff0_20*basisvalue20 + coeff0_21*basisvalue21 + coeff0_22*basisvalue22 + coeff0_23*basisvalue23 + coeff0_24*basisvalue24 + coeff0_25*basisvalue25 + coeff0_26*basisvalue26 + coeff0_27*basisvalue27 + coeff0_28*basisvalue28 + coeff0_29*basisvalue29 + coeff0_30*basisvalue30 + coeff0_31*basisvalue31 + coeff0_32*basisvalue32 + coeff0_33*basisvalue33 + coeff0_34*basisvalue34 + coeff0_35*basisvalue35 + coeff0_36*basisvalue36 + coeff0_37*basisvalue37 + coeff0_38*basisvalue38 + coeff0_39*basisvalue39 + coeff0_40*basisvalue40 + coeff0_41*basisvalue41 + coeff0_42*basisvalue42 + coeff0_43*basisvalue43 + coeff0_44*basisvalue44 + coeff0_45*basisvalue45 + coeff0_46*basisvalue46 + coeff0_47*basisvalue47 + coeff0_48*basisvalue48 + coeff0_49*basisvalue49 + coeff0_50*basisvalue50 + coeff0_51*basisvalue51 + coeff0_52*basisvalue52 + coeff0_53*basisvalue53 + coeff0_54*basisvalue54 + coeff0_55*basisvalue55;
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
    
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
    
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
    
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
    
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
    
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    static const double coefficients0[56][56] = \
    {{0.00945067404923492, 0.00609788120679252, 0.00352061335622803, 0.00248944957812476, 0.0070594623332504, 0.00546823600997146, 0.00315708753234943, 0.00446479600584755, 0.00257775117585285, 0.00182274533665715, -0.00298267418725716, -0.0025208197798293, -0.00195261860521383, -0.00112734487734488, -0.00218309396769447, -0.00169101731601731, -0.000976309302606919, -0.00138070985681772, -0.000797153207506487, -0.00056367243867244, 0.00278687112460004, 0.00245778931055954, 0.00207721109304566, 0.00160900079398463, 0.000928957041533343, 0.00219831358911339, 0.0018579140830667, 0.00143913406048032, 0.00083088443721827, 0.00160900079398463, 0.0012463266558274, 0.000719567030240151, 0.00101762145320217, 0.000587524019939407, 0.000415442218609132, -0.00705593524717798, -0.00638233357089081, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363026, -0.00582624677716565, -0.00513826668309062, -0.00434262794914383, -0.00336378514516178, -0.00194208225905522, -0.00459580543596519, -0.00388416451811043, -0.00300866089851429, -0.00173705117965752, -0.00336378514516178, -0.00260557676948629, -0.00150433044925714, -0.00212744452363027, -0.00122828066840393, -0.000868525589828762},
    {0.00945067404923495, -0.00609788120679253, 0.00352061335622802, 0.00248944957812476, 0.00705946233325039, -0.00546823600997146, 0.00315708753234944, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521383, -0.00112734487734489, -0.00218309396769448, 0.00169101731601732, -0.000976309302606925, 0.00138070985681772, -0.000797153207506488, -0.000563672438672441, 0.00278687112460004, -0.00245778931055954, 0.00207721109304566, -0.00160900079398463, 0.000928957041533346, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218268, 0.00160900079398463, -0.0012463266558274, 0.000719567030240148, -0.00101762145320216, 0.000587524019939402, 0.000415442218609128, 0.00705593524717798, -0.0063823335708908, 0.00562868913761196, -0.00475711057319694, 0.00368484200521179, -0.00212744452363027, -0.00582624677716565, 0.00513826668309062, -0.00434262794914382, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811043, 0.00300866089851429, -0.00173705117965752, -0.00336378514516178, 0.00260557676948629, -0.00150433044925714, 0.00212744452363026, -0.00122828066840393, -0.00086852558982876},
    {0.00945067404923495, 0, -0.00704122671245605, 0.00248944957812476, 0, 0, 0.00947126259704828, 0, -0.00515550235170572, 0.00182274533665715, 0, 0, 0, 0.00450937950937951, 0, 0, -0.00292892790782075, 0, 0.00159430641501296, -0.00056367243867244, 0, 0, 0, 0, 0.00464478520766673, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987881, 0.00041544221860913, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527608, 0, 0, 0, 0.00694820471863012, 0, 0, -0.00451299134777144, 0, 0.00245656133680786, -0.000868525589828763},
    {0.00945067404923493, 0, 0, -0.0074683487343743, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00563672438672439, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.018239037386404},
    {-0.010023442173431, 0, 0.0374718748311399, -0.0289614183593723, 0, 0, 0.0284137877911448, 0, 0.00515550235170572, -0.00619733414463434, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835494, -0.000187890812890802, 0, 0, 0, 0, 0.0232239260383337, 0, 0, 0, -0.00166176887443652, 0, 0, -0.00647610327216137, 0, 0.00646276421933342, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496074, 0, 0, 0, -0.0277928188745205, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.010023442173431, 0, -0.0287574853355259, 0.05360943398437, 0, 0, -0.0189425251940966, 0, 0.0154665070551171, 0.0153110608279201, 0, 0, 0, -0.00450937950937953, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144092, 0, -0.0123380044187274, 0.00872428659079174, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559316, 0, 0.041270230458372, -0.018239037386404},
    {0.010023442173431, 0, 0.0409576306293854, -0.0449826285156208, 0, 0, 0.0189425251940965, 0, -0.00515550235170569, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883226, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957583, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.055026973944496, 0.0303983956440067},
    {-0.0100234421734311, 0, -0.0148144621425437, 0.0449826285156207, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0187359374155699, -0.0289614183593723, 0.0211783869997512, -0.0164047080299144, 0.00947126259704827, 0.00446479600584757, -0.00257775117585285, -0.00619733414463432, 0.0208787193108001, -0.0176457384588052, 0.0136683302364968, -0.00789141414141413, -0.0084898098743674, 0.00657617845117844, -0.00379675839902689, 0.0023011830946962, -0.00132858867917746, -0.0001878908128908, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992314, 0.00464478520766674, -0.0010991567945567, 0.000928957041533349, -0.000719567030240152, 0.000415442218609127, -0.00482700238195388, 0.00373897996748219, -0.00215870109072045, 0.00559691799261185, -0.00323138210966672, -0.00290809553026392, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935591, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863011, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778515, 0.00607967912880135},
    {0.010023442173431, -0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, 0.0109364720199429, -0.00631417506469883, 0.0133943880175427, -0.00773325352755858, 0.0153110608279201, -0.00298267418725716, 0.00252081977982931, -0.00195261860521382, 0.00112734487734488, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476002, 0.0129522065443227, -0.00747795993496437, -0.00321800158796926, 0.0024926533116548, -0.00143913406048031, -0.0106850252586226, 0.00616900220936373, 0.00872428659079178, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845888, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.020635115229186, -0.0182390373864041},
    {0.0100234421734311, 0.0354703486038674, -0.0204788153146927, -0.0449826285156208, 0.0141189246665008, -0.0109364720199429, 0.00631417506469885, -0.00446479600584758, 0.00257775117585285, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974696, -0.00263047138047138, 0.00151870335961075, 0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620757, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, -0.0128297005588457, 0.00740723107127184, 0.0449826285156208, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0.00920473237878476, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489395, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.010023442173431, 0.0324515955311979, -0.0335503995581136, -0.00801060507812427, 0.0211783869997512, 0.00109364720199429, -0.00568275755822898, -0.0169662248222207, 0.0056710525868763, 0.00838462854862291, 0.0208787193108001, -0.0109235523792603, 0.00499002532443534, -0.00187890812890812, -0.0162519217595033, 0.00958243145743146, -0.00466458889023305, 0.0108922666482287, -0.00558007245254534, -0.00469727032227033, 0.0139343556230002, -0.00245778931055954, -0.00415442218609131, 0.00643600317593851, -0.00464478520766673, -0.0120907247401236, 0.00464478520766674, -0.000719567030240153, -0.000415442218609137, 0.00965400476390778, -0.00498530662330958, 0.00215870109072046, -0.006614539445814, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543313, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438609, 0.00310733161448835, -0.00481385743762287, 0.00347410235931505, -0.00201827108709706, -0.000521115353897262, 0.000902598269554289, 0.00170195561890421, -0.000491312267361565, -0.000868525589828764},
    {0.0100234421734309, -0.024904712849524, 0.0553363732971484, -0.0043134027343746, -0.0141189246665008, 0.0153110608279201, 0.00883984509057843, 0.00803663281052563, -0.0159820572902877, 0.000729098134662861, -0.00298267418725715, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846298, -0.0116492303992304, 0.00976309302606916, 0.00352848074520084, 0.00132858867917745, -0.00263047138047137, 0, 0.0196623144844763, -0.00727023882565982, -0.00724050357293082, 0.00928957041533346, -0.00219831358911338, -0.0148633126645336, 0.0129522065443228, -0.0066470754977461, 0.00402250198496157, 0.00685479660705067, -0.00575653624192123, -0.0045792965394097, -0.000293762009969684, 0.00290809553026392, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.00411061334647249, -0.00303983956440068, 0.00908221989193681, -0.00776832903622088, 0.00045958054359652, 0.00427258096992148, -0.00270779480866287, 0.000868525589828773, -0.00100913554354854, -0.00234501909253766, 0.00180519653910858, 0.00127646671417816, 0.000245656133680781, -0.000868525589828764},
    {0.0100234421734309, 0.0354703486038675, -0.0492363006502187, -0.00431340273437461, 0.0141189246665008, -0.00656188321196574, -0.016416855168217, -0.00982255121286466, 0.0149509568199466, 0.000729098134662872, 0, 0.0033610930397724, 0.0282044909641998, -0.0353234728234728, -0.000485131992820993, -0.0124007936507936, 0.00889526253486299, 0.00291483191994851, 0.00239145962251946, -0.00263047138047138, 0, 0, 0.0218107164769794, -0.00402250198496158, -0.00928957041533347, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992316, 0.00934744991870547, -0.0028782681209606, -0.00254405363300539, -0.00381890612960613, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320202, 0.00840946286290443, -0.00388416451811042, 0, 0.00233049871086626, 0.0030086608985143, -0.00347410235931507, -0.000336378514516178, -0.00286613444643493, 0.00120346435940572, 0.00085097780945211, 0.00098262453472315, -0.000868525589828765},
    {-0.010023442173431, -0.0128297005588457, 0.0448791059024117, -0.00801060507812424, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467804, -0.0175287079957994, 0.0083846285486229, 0, 0, 0.00867830491206149, 0.0225468975468975, 0, -0.00300625300625301, -0.019200749617936, 0.000613648825252316, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240152, -0.0108014976838374, 0, 0.00249265331165478, 0.0107935054536023, -0.000508810726601074, -0.00734405024924252, 0.00290809553026393, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931505, 0, -0.00208446141558903, -0.000902598269554287, 0.000425488904726055, 0.0017195929357655, -0.000868525589828767},
    {-0.0100234421734311, -0.0324515955311978, -0.0187359374155699, -0.0289614183593723, 0.0211783869997511, 0.0164047080299144, 0.00947126259704826, -0.00446479600584755, -0.00257775117585285, -0.00619733414463435, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141413, -0.00848980987436742, -0.00657617845117845, -0.0037967583990269, -0.00230118309469619, -0.00132858867917746, -0.00018789081289081, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992313, 0.00464478520766674, 0.00109915679455668, 0.000928957041533345, 0.000719567030240154, 0.000415442218609131, -0.00482700238195388, -0.00373897996748219, -0.00215870109072045, -0.00559691799261187, -0.00323138210966673, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.00955378148811641, 0.00740032731935591, 0.00427258096992148, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.0069482047186301, 0.0181644397838736, 0.014070114555226, 0.00812338442598859, 0.0136156449512337, 0.00786099627778516, 0.00607967912880135},
    {0.0100234421734308, 0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, -0.0109364720199429, -0.00631417506469886, -0.0133943880175427, -0.00773325352755858, 0.0153110608279201, 0.00298267418725716, 0.00252081977982933, 0.00195261860521383, 0.00112734487734487, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476002, -0.0129522065443228, -0.00747795993496438, -0.00321800158796927, -0.00249265331165479, -0.00143913406048031, 0.0106850252586227, 0.00616900220936374, 0.00872428659079177, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.0206351152291861, -0.0182390373864041},
    {0.010023442173431, -0.0354703486038674, -0.0204788153146927, -0.0449826285156208, 0.0141189246665008, 0.0109364720199429, 0.00631417506469886, 0.00446479600584758, 0.00257775117585287, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974696, 0.00263047138047138, 0.00151870335961075, -0.0308358534689291, -0.0178030883009781, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620757, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588457, 0.00740723107127183, 0.0449826285156208, 0, 0, 0, -0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878479, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489395, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.010023442173431, -0.0324515955311978, -0.0335503995581136, -0.00801060507812424, 0.0211783869997512, -0.00109364720199429, -0.00568275755822899, 0.0169662248222207, 0.00567105258687632, 0.0083846285486229, -0.0208787193108001, -0.0109235523792603, -0.00499002532443534, -0.00187890812890812, -0.0162519217595033, -0.00958243145743146, -0.00466458889023305, -0.0108922666482287, -0.00558007245254535, -0.00469727032227032, 0.0139343556230002, 0.00245778931055953, -0.00415442218609133, -0.00643600317593852, -0.00464478520766672, 0.0120907247401236, 0.00464478520766673, 0.000719567030240156, -0.000415442218609149, 0.00965400476390777, 0.00498530662330958, 0.00215870109072047, 0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543313, 0.00719357335632687, 0.0112908326677739, 0.0114368694935501, 0.00737991258440984, -0.00183832217438608, 0.00310733161448834, 0.00481385743762287, 0.00347410235931505, -0.00201827108709706, 0.000521115353897262, 0.000902598269554287, -0.00170195561890421, -0.000491312267361565, -0.000868525589828767},
    {0.010023442173431, 0.0249047128495239, 0.0553363732971484, -0.00431340273437459, -0.0141189246665008, -0.0153110608279201, 0.00883984509057843, -0.00803663281052563, -0.0159820572902878, 0.000729098134662854, 0.00298267418725716, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846298, 0.0116492303992304, 0.00976309302606918, -0.00352848074520084, 0.00132858867917746, -0.00263047138047138, 0, -0.0196623144844763, -0.0072702388256598, 0.00724050357293083, 0.00928957041533346, 0.00219831358911339, -0.0148633126645336, -0.0129522065443228, -0.00664707549774609, 0.00402250198496157, -0.00685479660705067, -0.00575653624192124, 0.0045792965394097, -0.000293762009969679, 0.00290809553026392, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.0041106133464725, -0.00303983956440067, -0.00908221989193682, -0.00776832903622089, -0.00045958054359652, 0.00427258096992148, 0.00270779480866287, 0.000868525589828776, -0.00100913554354853, 0.00234501909253766, 0.00180519653910858, -0.00127646671417816, 0.000245656133680779, -0.000868525589828761},
    {0.0100234421734309, -0.0354703486038674, -0.0492363006502187, -0.00431340273437459, 0.0141189246665008, 0.00656188321196577, -0.016416855168217, 0.00982255121286466, 0.0149509568199466, 0.000729098134662868, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820994, 0.0124007936507936, 0.00889526253486299, -0.00291483191994852, 0.00239145962251945, -0.00263047138047138, 0, 0, 0.0218107164769794, 0.00402250198496157, -0.00928957041533346, 0, -0.00557374224920009, 0.0143913406048031, 0.0141250354327105, 0.000804500396992309, -0.00934744991870548, -0.0028782681209606, 0.00254405363300539, -0.00381890612960614, 0.00290809553026393, 0, 0, 0, 0.0266398192099028, 0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320203, -0.00840946286290443, -0.00388416451811041, 0, 0.00233049871086626, -0.0030086608985143, -0.00347410235931507, -0.000336378514516177, 0.00286613444643493, 0.00120346435940572, -0.000850977809452108, 0.000982624534723152, -0.000868525589828766},
    {-0.010023442173431, 0.0128297005588457, 0.0448791059024117, -0.00801060507812424, 0, -0.0174983552319087, 0.0132597676358676, 0.00357183680467805, -0.0175287079957994, 0.0083846285486229, 0, 0, -0.00867830491206148, 0.0225468975468975, 0, 0.00300625300625301, -0.019200749617936, -0.000613648825252319, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766673, 0, 0, 0.00719567030240153, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601077, -0.00734405024924252, 0.00290809553026393, 0, 0, 0, 0, -0.0257938940364826, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702859, -0.00347410235931505, 0, 0.00208446141558903, -0.000902598269554286, -0.000425488904726054, 0.0017195929357655, -0.000868525589828767},
    {-0.0100234421734311, -0.0452812960900435, -0.0113287063442981, -0.00801060507812424, -0.0014118924666501, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.00838462854862289, -0.00894802256177147, -0.0142846454190328, -0.014536160727703, -0.00939454064454064, -0.0123708658169354, -0.0125886844636845, -0.00813591085505764, -0.0102786178229763, -0.00664294339588733, -0.00469727032227033, -0.00836061337380012, 0.00245778931055953, 0.00830884437218264, 0.00965400476390777, 0.00650269929073344, 0.00219831358911338, 0.00743165633226677, 0.00863480436288183, 0.00581619106052785, 0.00643600317593852, 0.00747795993496437, 0.00503696921168108, 0.00610572871921293, 0.00411266813957582, 0.00290809553026394, 0.0352796762358899, 0.0191470007126724, 0.00788016479265675, 0.000951422114639391, -0.00221090520312707, -0.00212744452363027, 0.017478740331497, 0.00719357335632686, 0.000868525589828764, -0.00201827108709706, -0.00194208225905521, 0.00643412761035128, 0.000776832903622088, -0.00180519653910857, -0.00173705117965753, 0.000672757029032363, -0.00156334606169177, -0.00150433044925715, -0.00127646671417816, -0.00122828066840393, -0.000868525589828768},
    {0.010023442173431, 0.0603750614533914, -0.00610007264692975, -0.0043134027343746, -0.00564756986660025, -0.0218729440398859, 0.00126283501293978, -0.0178591840233903, 0.00103110047034114, 0.000729098134662884, 0.0417574386216002, 0.0168054651988621, -0.000867830491206148, -0.00526094276094276, 0.0145539597846299, -0.000751563251563252, -0.00455611007883227, -0.000613648825252328, -0.0037200483016969, -0.00263047138047138, 0.00557374224920005, -0.0196623144844763, -0.0103860554652283, 0.00321800158796928, 0.00650269929073341, -0.0175865087129071, -0.00928957041533347, 0.00287826812096063, 0.00581619106052784, -0.00804500396992315, 0.0024926533116548, 0.00503696921168106, 0.00203524290640433, 0.00411266813957581, 0.00290809553026391, -0.0705593524717798, -0.0127646671417816, 0.00450295131008958, 0.00380568845855754, -0.000736968401042372, -0.00212744452363026, -0.0116524935543313, 0.00411061334647251, 0.00347410235931506, -0.000672757029032362, -0.00194208225905521, 0.00367664434877218, 0.00310733161448835, -0.000601732179702865, -0.00173705117965753, 0.00269102811612942, -0.000521115353897265, -0.00150433044925715, -0.00042548890472606, -0.00122828066840393, -0.000868525589828759},
    {0.0100234421734309, -0.0603750614533914, -0.00610007264692972, -0.00431340273437457, -0.00564756986660032, 0.0218729440398859, 0.00126283501293976, 0.0178591840233903, 0.00103110047034113, 0.000729098134662861, -0.0417574386216002, 0.016805465198862, 0.000867830491206157, -0.00526094276094276, 0.0145539597846298, 0.000751563251563257, -0.00455611007883228, 0.00061364882525233, -0.00372004830169691, -0.00263047138047138, 0.00557374224920011, 0.0196623144844763, -0.0103860554652283, -0.00321800158796927, 0.00650269929073344, 0.0175865087129071, -0.00928957041533346, -0.00287826812096062, 0.00581619106052786, -0.00804500396992314, -0.0024926533116548, 0.00503696921168108, -0.00203524290640433, 0.00411266813957583, 0.00290809553026393, 0.0705593524717798, -0.0127646671417816, -0.00450295131008958, 0.00380568845855756, 0.000736968401042369, -0.00212744452363027, -0.0116524935543313, -0.00411061334647251, 0.00347410235931506, 0.000672757029032361, -0.00194208225905522, -0.00367664434877217, 0.00310733161448835, 0.000601732179702866, -0.00173705117965753, 0.00269102811612942, 0.000521115353897263, -0.00150433044925715, 0.000425488904726061, -0.00122828066840394, -0.000868525589828769},
    {-0.0100234421734309, 0.0452812960900435, -0.0113287063442981, -0.00801060507812426, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089231, 0.00838462854862291, 0.00894802256177147, -0.0142846454190327, 0.014536160727703, -0.00939454064454063, -0.0123708658169354, 0.0125886844636845, -0.00813591085505763, 0.0102786178229763, -0.00664294339588732, -0.00469727032227032, -0.00836061337380015, -0.00245778931055953, 0.00830884437218263, -0.00965400476390777, 0.00650269929073343, -0.00219831358911339, 0.00743165633226677, -0.00863480436288183, 0.00581619106052785, 0.00643600317593852, -0.00747795993496437, 0.00503696921168107, -0.00610572871921293, 0.00411266813957582, 0.00290809553026393, -0.0352796762358899, 0.0191470007126724, -0.00788016479265675, 0.000951422114639385, 0.00221090520312707, -0.00212744452363027, 0.0174787403314969, -0.00719357335632687, 0.000868525589828768, 0.00201827108709706, -0.00194208225905521, -0.00643412761035128, 0.000776832903622089, 0.00180519653910857, -0.00173705117965753, 0.000672757029032358, 0.00156334606169177, -0.00150433044925715, 0.00127646671417816, -0.00122828066840393, -0.000868525589828765},
    {0.0787556170769579, 0.0679219441350653, -0.00435719474780691, -0.00308100195312472, 0.0705946233325039, -0.0328094160598287, 0.0126283501293977, -0.0267887760350854, 0.03608851646194, -0.00729098134662862, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015873, 0.0130985638061669, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, 0.0245778931055954, -0.036351194128299, 0.0362025178646541, -0.0232239260383337, 0.0219831358911338, -0.00743165633226679, 0, 0.00166176887443652, -0.0345935170706695, 0.0130864298861876, -0.00359783515120079, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596521, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598858, -0.0102117337134253, 0.00196524906944628, 0.00607967912880136},
    {-0.0214788046573522, 0.00377344134083702, 0.00217859737390344, -0.0215670136718729, 0, 0.0984282481794862, -0.0852413633734345, -0.0312535720409329, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, 0.0236742424242424, -0.0335199527228374, -0.00138070985681772, -0.00079715320750645, 0.00394570707070704, 0, 0, 0.036351194128299, -0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.00724050357293084, -0.0392592896585629, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724418, 0, 0.00694820471863008, 0.00302740663064561, 0.0164151336477636, -0.0108311792346515, -0.00680782247561686, -0.00393049813889256, 0.00607967912880134},
    {0.0787556170769579, 0.0301875307266957, 0.0610007264692975, -0.00308100195312472, 0, 0.0218729440398858, 0.0757701007763863, 0.0178591840233903, -0.0412440188136458, -0.00729098134662862, 0, 0, 0.0520698294723688, -0.00901875901875902, 0, -0.0127765752765753, 0.0286384062098028, 0.00184094647575699, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.0143913406048031, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515274, 0.00407048581280861, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967795, -0.0427258096992148, 0, 0, -0.0300866089851429, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723144, 0.00607967912880136},
    {-0.0214788046573522, 0.00377344134083686, -0.0196073763651313, 0.00924300585937417, 0, -0.0109364720199429, 0.00947126259704823, 0.102690308134494, -0.0695992817480272, -0.060150596109686, 0, -0.00504163955965862, 0.0065087286840461, -0.00450937950937951, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196709, -0.0263060558477138, 0.000563672438672465, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165478, -0.0043174021814409, -0.0605484764655283, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573521, -0.0150937653633479, 0.0130715842434208, 0.00924300585937414, 0, -0.0109364720199429, 0.00947126259704832, -0.00892959201169514, 0.123732056440937, -0.0601505961096861, 0, 0, -0.00650872868404609, 0.00676406926406927, 0, 0.0574945887445887, 0.0126920209338899, -0.0266937238984759, 0.00637722566005184, 0.000563672438672453, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097862, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448302, -0.0616900220936373, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769578, 0.0301875307266958, 0.0174287789912279, 0.0585390371093694, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094276, -0.00455611007883226, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700864, 0.0284934006896604, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769578, -0.0679219441350653, -0.00435719474780693, -0.00308100195312472, 0.0705946233325039, 0.0328094160598287, 0.0126283501293978, 0.0267887760350854, 0.03608851646194, -0.00729098134662859, -0.0298267418725716, 0.0151249186789758, 0.0325436434202305, 0.0248015873015873, 0.0130985638061669, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646541, -0.0232239260383337, -0.0219831358911339, -0.00743165633226677, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861877, -0.00359783515120079, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.0045958054359652, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.00908221989193679, -0.00703505727761299, -0.00812338442598858, 0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573524, -0.00377344134083698, 0.00217859737390344, -0.021567013671873, 0, -0.0984282481794863, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242424, -0.0335199527228374, 0.0013807098568177, -0.00079715320750645, 0.00394570707070706, 0, 0, 0.036351194128299, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838375, -0.00724050357293083, 0.039259289658563, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951694, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.00302740663064561, -0.0164151336477636, -0.0108311792346515, 0.00680782247561685, -0.00393049813889256, 0.00607967912880134},
    {0.0787556170769579, -0.0301875307266956, 0.0610007264692975, -0.00308100195312471, 0, -0.0218729440398858, 0.0757701007763863, -0.0178591840233903, -0.0412440188136458, -0.00729098134662861, 0, 0, -0.0520698294723689, -0.00901875901875903, 0, 0.0127765752765753, 0.0286384062098028, -0.00184094647575695, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766674, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515274, -0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967796, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780843, -0.00982624534723145, 0.00607967912880137},
    {-0.0214788046573523, -0.0037734413408369, -0.0196073763651314, 0.00924300585937414, 0, 0.0109364720199429, 0.00947126259704826, -0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965861, -0.00650872868404608, -0.00450937950937948, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196707, -0.0263060558477138, 0.000563672438672444, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.0024926533116548, -0.00431740218144091, 0.0605484764655282, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, 0.0150937653633478, 0.0130715842434208, 0.00924300585937414, 0, 0.0109364720199429, 0.00947126259704831, 0.00892959201169514, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404611, 0.00676406926406927, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005185, 0.000563672438672461, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097862, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448302, -0.0616900220936373, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769578, -0.0301875307266956, 0.0174287789912279, 0.0585390371093695, 0, -0.0218729440398858, -0.0189425251940966, -0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, -0.00526094276094275, -0.00455611007883226, -0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700864, -0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769576, -0.0377344134083696, -0.0566435317214905, -0.00308100195312473, 0.0423567739995023, 0.0546823600997146, 0.0378850503881931, 0.0446479600584756, 0.00515550235170574, -0.00729098134662862, 0.0417574386216002, -0.0050416395596586, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801536, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055954, -0.00519302773261415, 0.00402250198496157, 0.00464478520766674, -0.00219831358911335, -0.0130053985814669, -0.0158304746652834, -0.0108014976838374, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.00477689074405821, 0.00370016365967795, 0.00427258096992148, -0.0367664434877216, -0.00776832903622087, 0.00601732179702858, 0.00694820471863011, -0.00908221989193682, 0.00703505727761298, 0.0081233844259886, 0.00680782247561684, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573522, 0, -0.0043571947478069, -0.0215670136718729, -0.127070321998507, 0, 0.0284137877911448, 0, 0.03608851646194, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984381, 0, 0.00159430641501295, 0.00394570707070706, 0.0836061337380013, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377579, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.0095537814881164, 0, 0.00427258096992147, 0, -0.0155366580724417, 0, 0.00694820471863011, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778513, 0.00607967912880133},
    {0.0787556170769579, 0.0377344134083696, -0.0566435317214905, -0.0030810019531247, 0.0423567739995023, -0.0546823600997146, 0.0378850503881931, -0.0446479600584757, 0.00515550235170574, -0.00729098134662864, -0.0417574386216002, -0.00504163955965862, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801536, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261416, -0.00402250198496158, 0.00464478520766674, 0.00219831358911339, -0.0130053985814669, 0.0158304746652834, -0.0108014976838374, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405821, -0.00370016365967795, 0.00427258096992148, 0.0367664434877216, -0.00776832903622088, -0.00601732179702859, 0.00694820471863012, -0.00908221989193681, -0.00703505727761299, 0.00812338442598861, -0.00680782247561685, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573522, -0.0188672067041847, 0.00653579212171051, 0.00924300585937415, 0.0141189246665008, 0, -0.00315708753234942, -0.111619900146189, -0.0541327746929101, -0.0601505961096861, -0.00894802256177146, -0.00252081977982933, 0.000650872868404606, 0.00112734487734486, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672439, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476002, -0.00431740218144093, -0.00747795993496438, 0.00321800158796924, 0.0224338798048931, 0.018708742786244, 0.046301776120698, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.017478740331497, -0.00451299134777145, -0.00781673030845887, 0.0363288795677472, -0.00938007637015066, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {-0.0214788046573521, 0.0188672067041847, 0.00653579212171047, 0.00924300585937414, 0.0141189246665007, 0, -0.00315708753234944, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177148, -0.00252081977982932, -0.000650872868404592, 0.00112734487734487, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.019928830187662, 0.000563672438672456, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476002, 0.00431740218144092, -0.00747795993496438, 0.00321800158796927, -0.0224338798048931, 0.018708742786244, -0.0463017761206981, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777143, -0.00781673030845888, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769579, 0, -0.0348575579824557, 0.0585390371093695, -0.0282378493330015, 0, 0.00631417506469885, 0, -0.0206220094068228, 0.083846285486229, 0, 0, 0, 0, -0.0067918478994939, 0, 0.00151870335961074, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233621, 0, -0.0329013451166066, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.0275134869722481, 0.0303983956440068},
    {0.0787556170769578, -0.0377344134083696, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, 0.0546823600997145, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865145, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.0092175078635989, -0.00789141414141412, -0.00347132196482459, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, -0.00491557862111904, -0.0197335053839338, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097694, -0.000623163327913701, -0.00215870109072047, 0.00712335017241508, 0.00411266813957581, 0.00540074884191871, 0, -0.0510586685671264, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173251, -0.0143871467126537, -0.00564541633388696, 0.00571843474677502, 0.00737991258440982, 0.00367664434877216, -0.00155366580724418, 0.00240692871881142, 0.00347410235931506, 0.00100913554354853, 0.00026055767694863, 0.000902598269554296, -0.000850977809452108, -0.000491312267361565, -0.000868525589828763},
    {-0.0214788046573521, 0, -0.0217859737390348, 0.00308100195312476, -0.127070321998507, 0, 0.0473563129852414, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468977, 0.0509388592462045, 0, -0.00943765659186683, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533349, 0, -0.0141250354327105, -0.0144810071458616, 0, -0.00215870109072044, 0, 0.00411266813957584, 0.00540074884191874, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259877, 0, -0.0112908326677739, 0, 0.00737991258440982, 0, -0.00310733161448833, 0, 0.00347410235931505, 0.00201827108709706, 0, 0.000902598269554279, 0, -0.000491312267361592, -0.00086852558982878},
    {0.078755617076958, 0.0377344134083695, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584757, 0.0257775117585286, 0.0291639253865145, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.0092175078635989, 0.00789141414141413, -0.00347132196482461, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111908, -0.0197335053839338, 0.0394205194526234, -0.0325134964536672, 0.0241814494802473, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097695, 0.000623163327913697, -0.00215870109072046, -0.0071233501724151, 0.00411266813957581, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908062, -0.0114170653756727, -0.010317557614593, 0.0127646671417816, 0.00466099742173252, 0.0143871467126538, -0.00564541633388697, -0.00571843474677502, 0.00737991258440983, -0.00367664434877217, -0.00155366580724418, -0.00240692871881144, 0.00347410235931506, 0.00100913554354853, -0.000260557676948628, 0.000902598269554287, 0.000850977809452107, -0.000491312267361569, -0.000868525589828763},
    {-0.0214788046573521, -0.0188672067041848, 0.0108929868695174, 0.0030810019531247, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792643, 0.0127592173566001, -0.0089480225617715, -0.0277290175781224, -0.00585785581564147, 0.0191648629148629, 0.00145539597846298, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755831, -0.0129644660894661, 0, 0.0589869434534289, 0.00727023882565979, 0.0506835250105158, 0.0650269929073344, -0.00659494076734015, 0.0148633126645335, -0.00431740218144092, -0.00166176887443655, -0.00402250198496158, -0.00810112326287807, -0.0115130724838425, 0.00356167508620755, -0.00205633406978791, 0.00540074884191871, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440066, -0.00302740663064559, -0.00776832903622087, 0.00137874163078956, -0.00427258096992148, 0.000902598269554293, 0.000868525589828758, 0.00100913554354854, 0.000781673030845887, 0.00180519653910857, -0.000425488904726058, 0.00024565613368079, -0.000868525589828762},
    {-0.0214788046573521, 0.0188672067041848, 0.0108929868695174, 0.00308100195312469, 0.0141189246665007, 0.109364720199429, -0.0789271883087357, -0.0223239800292378, -0.0128887558792643, 0.0127592173566001, 0.00894802256177149, -0.0277290175781224, 0.00585785581564149, 0.0191648629148629, 0.00145539597846298, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755832, -0.0129644660894661, 0, -0.0589869434534289, 0.00727023882565983, -0.0506835250105158, 0.0650269929073343, 0.00659494076734015, 0.0148633126645336, 0.00431740218144092, -0.00166176887443653, -0.00402250198496157, 0.00810112326287806, -0.0115130724838425, -0.00356167508620755, -0.00205633406978791, 0.00540074884191872, 0, 0, -0.0607898426862092, 0.0399597288148543, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440068, 0.0030274066306456, -0.00776832903622088, -0.00137874163078956, -0.00427258096992148, -0.000902598269554298, 0.000868525589828757, 0.00100913554354854, -0.000781673030845891, 0.00180519653910858, 0.000425488904726056, 0.000245656133680792, -0.000868525589828762},
    {0.078755617076958, 0, 0.0435719474780696, -0.0523770332031201, -0.0282378493330015, 0, 0.0631417506469885, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.00672218607954481, 0, -0.0601250601250601, 0.000970263985641994, 0, -0.0125835421224892, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539589, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398463, 0, -0.0028782681209606, 0, -0.00822533627915163, 0.00540074884191871, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.018239037386404, 0, -0.00388416451811042, 0, -0.00466099742173252, 0, -0.00347410235931505, 0.000672757029032356, 0, 0.00120346435940571, 0, 0.000982624534723144, -0.000868525589828765},
    {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, -0.0546823600997146, -0.0315708753234943, -0.0446479600584757, -0.0257775117585285, -0.0182274533665716, 0.0894802256177147, -0.0151249186789759, 0.0585785581564149, 0.0744047619047619, -0.0130985638061669, -0.0281836219336219, 0.0097630930260691, 0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226673, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978424, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905521, -0.00902598269554287, -0.0191075629762328, -0.00908221989193678, -0.00234501909253766, -0.00812338442598858, 0.00340391123780844, 0.00196524906944629, 0.00607967912880135},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, 0.0546823600997146, -0.0315708753234943, 0.0446479600584756, -0.0257775117585286, -0.0182274533665715, -0.0894802256177148, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061669, 0.0281836219336219, 0.00976309302606918, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.0363511941282991, 0.0120675059548847, -0.0232239260383337, -0.0659494076734016, 0.00743165633226682, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978425, 0.0251848460584054, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554289, -0.0191075629762328, -0.00908221989193679, 0.00234501909253767, -0.00812338442598859, -0.00340391123780842, 0.00196524906944628, 0.00607967912880136},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281177, 0, 0, -0.0947126259704827, 0, 0.0515550235170571, -0.0182274533665715, 0, -0.0907495120738551, 0, -0.0946969696969696, 0.0232863356554078, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992785, 0, 0, -0.072702388256598, 0, 0.0464478520766673, 0, -0.048305766159734, 0, -0.0781031370985167, 0.0144810071458617, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992147, 0, 0.0310733161448835, 0, 0.00694820471863012, -0.00605481326129122, 0, -0.0108311792346515, 0, -0.00393049813889258, 0.00607967912880135},
    {0.107394023286761, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937952, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398575, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259878, 0, 0.0312669212338355, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    static const double dmats0[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {6.32455532033677, 1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 11.2249721603218, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.58257569495583, 0, 8.36660026534075, -1.18321595661993, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.74165738677394, 0, 0, 8.69482604771365, 0, 0, 0, 0, 0, 0, 0, 0, 1.18793863634892e-14, 0, 0, 0, -2.04281036531029e-14, 1.06581410364015e-14, 1.4210854715202e-14, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.33226762955019e-14, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.49909083394701, -1.24344978758018e-14, -3.3466401061363, -2.36643191323985, 15.4919333848297, 0, 0.692820323027545, 0, 0.565685424949238, 0.400000000000001, -1.24344978758018e-14, 2.22044604925031e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 4.89897948556636, -1.4210854715202e-14, 0, 0, 14.1985914794391, 0, -0.828078671210839, 0, 0, 0, -1.24344978758018e-14, 1.80966353013901e-14, -1.06581410364015e-14, 0, 1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 2.48689957516035e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.6, 0, 8.76356092008264, -1.54919333848296, 0, 0, 9.52470471983252, 0, -1.48131215963607, 0.261861468283189, 0, -1.06581410364015e-14, 0, 0, 0, -1.06581410364015e-14, 2.48689957516035e-14, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 4.24264068711928, 0, -2.48689957516035e-14, 0, 0, 0, 14.3427433120127, 0, 0, 0, 0, 0, 0, 0, 2.22044604925031e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, -1.06581410364015e-14, 0},
    {3.11769145362397, 0, 3.1622776601684, 4.91934955049952, 0, 0, 0, 0, 10.690449676497, -2.41897262725905, 0, 0, 1.50990331349021e-14, 0, 0, 0, -2.30926389122033e-14, 0, 1.77635683940025e-14, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, -1.33226762955019e-14, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.37945210809676e-14, 0, 0, 1.95399252334028e-14, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.54558441227157, 0, 0, 7.66811580507232, 0, 0, 0, 0, 0, 10.3691851174526, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 11.6057691494799, 0, 0, 0, -4.20458932936041, 0, -3.43303281162798, 0, 1.25455201782643e-14, 19.6214168703486, 0, 0.611677741841184, 0, 0, 0.529728463363969, 0, 0.432521479156776, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.53137212479106, 0, 3.54562104171168, -2.50713268211204, 4.7871355387817, 0, -5.56626745075489, 0, -1.04880884817016, 0.741619848709568, 0, 18.8034951158403, -1.24344978758018e-14, 1.68183573174417, -0.603122387046461, 0, 0.701283781217647, 0, 0.13213749452868, -0.0934353184302319, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, -1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.08246744900953e-14, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 2.5951288749407, 0, 0, 0, 10.0285307284481, -1.4210854715202e-14, -0.767649473578743, 0, 0, 0, 0, 16.4130361329658, -1.06581410364015e-14, 0, -1.26347611061158, 1.4210854715202e-14, 0.0967147429123241, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.96647939483827, 0, 8.12403840463597, -1.64130361329658, 0, 0, 11.7728501222091, 0, -2.40312296813957, 0.485504156227611, 1.06581410364015e-14, -1.06581410364015e-14, 0, 10.2761860629321, 0, 0, -1.48323969741913, 0, 0.302765035409733, -0.0611677741841268, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.05298244095219, 0, -1.5856499343442, 4.4848952846511, 4.28174419288838, 0, 0.191485421551275, -1.06581410364015e-14, -3.75233260785875, -4.31161222746203, 0, 0, 0, 0, 18.8807171866294, 0, 0.844371341865028, 0, 1.18187368057054, 1.04463861754668, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, -1.15463194561016e-14, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0},
    {0, 2.32115382989599, -1.4210854715202e-14, -1.06581410364015e-14, 0, 3.92428337406972, 0, 5.49285249860473, -1.59872115546023e-14, 0, 0, 0, 0, 0, -1.68753899743024e-14, 17.3044631365566, -1.77635683940025e-14, -1.73008591662713, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.77635683940025e-14, 2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0},
    {2.65329983228431, 0, 4.15220619990511, 2.93605316083775, 0, 0, 2.63248931621763, 0, 9.82591326180786, -2.82261277137741, 0, 0, 0, 0, 0, 1.4210854715202e-14, 11.6081867662439, 0, -3.09487177319468, 0.683876505540245, 0, 0, 0, 0, 0, 0, 0, 0, 1.50990331349021e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.26565424807268e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 2.01017818278146, 0, 0, 0, 0, 0, 8.91928088068603, 0, 0, 0, 0, 0, 0, 0, 0, 0, 16.8558564884489, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.2978250586152, 0, 1.49829835452881, 5.50917601898534, 0, 0, 0, 0, 6.64803945320938, 4.13676892548487, 0, 0, 1.06581410364015e-14, 0, 0, 0, -2.57571741713036e-14, 1.24344978758018e-14, 12.5636136423842, -3.55352656109506, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87616630392937, -1.19904086659517e-14, 0, 6.22830929985771, 0, 0, 0, 0, 0, 11.0541524195338, 0, 0, -1.19904086659517e-14, 0, -1.4210854715202e-14, 0, 2.99760216648792e-14, -1.06581410364015e-14, 0, 11.60576914948, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.5709655798446e-14, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.83186700722508, 0, -4.41087092453693, -3.11895674167864, 16.6749979177077, 0, 2.13065307820925, 0, 1.739670953501, 1.23013312825382, 0, -4.36653943835009, 0, -0.557935943461983, -3.78153408023783, -1.06581410364015e-14, -0.483186700722515, 0, -0.394520289089675, -0.278967971730983, 23.6981011897578, 0, 0.490653381462667, 0, 0.0626933893908967, 0, 0.438853725736256, 0, 0.0560746721671611, 0.380058475033045, 0, 0.0485620906056452, 0, 0.0396507809422086, 0.0280373360835817, 0, 0, 0, 0, 0, 0, -1.46549439250521e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 6.91050664139875, 0, 0, 0, 10.5706241650196, 0, -2.72554057547698, 0, 1.15463194561016e-14, 4.67332857821916, 0, -7.86705867811458, -2.48689957516035e-14, 0, -2.39719223514613, 0, 0.618094504365252, 0, 0, 0, 23.10555488766, 0, 1.62065746295906, 0, -0.469687194424183, 0, 0.790669146636837, 0, 0, 0.240926884676763, 0, -0.0621208350291009, 0, 0, 0, 0, 0, 0, 0, -1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.85449644663775, 1.15463194561016e-14, 5.98171817966646, -2.4880667576406, 1.90029237516522, 0, -0.33993463423953, 0, -2.3592214660616, 0.981306762925316, 0, 9.95226703056241, 0, -6.45364422706459, -0.43094580368567, 0, 0.077089928932768, 0, 0.535021138879229, -0.222539456105679, 0, 0, 21.5273297444417, 0, 2.75066129614997, 0, -1.00024047132728, 0, 0.648615649442924, 0.0433116829031337, 0, -0.0077478293753126, 0, -0.0537716476536687, 0.0223660568773445, 0, 0, 0, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.06581410364015e-14, 1.54523626091315, -1.4210854715202e-14, 0, 0, 6.71778024562961, 0, -0.609449400220043, 0, 0, 0, 1.06581410364015e-14, 14.6593983227032, 1.4210854715202e-14, 0, -1.52344936219873, 0, 0.138210132827973, 0, 0, 0, 0, 0, 18.119501277094, 0, 0, 0, -1.47332496632642, 0, 0, 0.153112421864228, 0, -0.0138906409944104, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.52336024752224, 1.4210854715202e-14, 7.37120708415867, -1.62882203585591, 0, 0, 12.0171306297064, 0, -2.90724328050879, 0.642416074439625, 0, 0, 2.1316282072803e-14, 13.9858820944259, 0, 0, -2.72522906732402, 0, 0.659300804656272, -0.145686271816946, 0, 0, 0, 0, 10.8043830863937, 0, 0, 0, -1.40563403846548, 0, 0, 0.273895826790425, 0, -0.0662622240310384, 0.0146420212340128, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 6.30840061880559, 0, 0, 1.06581410364015e-14, -1.5236235005501, 0, 11.1963004093827, 0, 0, 4.26614580154029, 0, 0.132992762321608, 0, 0, -4.72217953828917, 0, -6.67684597013021, 0, 0, 0, 0, 0, 0, 0, 23.1532444221696, 0, 0.721778878560592, 0, 0, 1.11125127955292, 0, 1.1908743922773, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0},
    {3.51865775274499, -1.04360964314765e-14, 1.92724822331885, 2.725540575477, 1.73472166622177, 0, -2.01705689521107, -1.77635683940025e-14, 3.42052627529743, -4.03112887414928, -1.28785870856518e-14, 4.08831086321549, -2.30926389122033e-14, 0.365669640132028, 5.37643791520787, 0, -6.25148194071026, 0, -2.03981013744551, 1.8080332206528, 0, 0, 0, 0, 0, 0, 22.1880978975596, 1.765254609154e-14, 1.98456380761453, -1.26521523890981, 0, 1.4711357857929, 0, 0.363818136386342, -0.294009452979941, 0, 0, 0, 0, 0, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, -2.30926389122033e-14, 0},
    {0, 1.4106012612951, 0, 0, 0, 3.634054100636, 0, 2.50356888118883, -1.24344978758018e-14, 0, 0, 0, 3.56857028479908, 0, 0, 11.263055412878, 0, -1.4929881464507, 0, 0, 0, 0, 0, 0, 0, 0, 0, 19.3673596462205, 0, 0, -2.65048896124152, 0, 0.266287609379582, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, -1.24344978758018e-14, 0, 0, -1.59872115546023e-14, 0, -1.24344978758018e-14, 0, 0},
    {2.30350221379959, 0, 4.41588043316389, 1.78428514239955, 0, 0, 4.2661458015403, 0, 7.8374102865679, -2.63899331455872, 0, 0, 0, 2.23427840700302, 1.77635683940025e-14, 0, 13.2221027072096, -1.4210854715202e-14, -4.67379217909115, 1.18363558466232, 0, 0, 0, 0, 0, 0, 0, 0, 12.1258851598179, 0, 0, -3.11150358274815, 0, 0.833612074594101, -0.192474367616158, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.8421709430404e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.14718316987778, 0, -0.861891607371343, 6.09449400220045, 1.55158222708545, 0, 0.0693888666488798, -1.95399252334028e-14, -3.05941170815566, 1.80277563773199, 0, -1.26842980563424e-14, 0, 0, 9.03122156537879, 0, 0.403888506800988, -1.4210854715202e-14, -3.67461994579465, -5.49650682721193, 0, 0, 0, 0, 0, 0, 0, 0, 0, 21.7841260588564, 0, 0.974215733960557, 0, 1.78974858569576, 1.8079190879493, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1.26168012376112, 1.24344978758018e-14, 0, 0, 1.42204860051343, 0, 4.47852016375307, 0, 0, 0, 0, 0, 0, 0, 8.27725128825318, 0, 5.37909280973869, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 19.9654813226339, 0, -2.61992366301004, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 1.06581410364015e-14, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 1.33226762955019e-14, 0, 0},
    {2.06031501455084, 0, 2.25696201807219, 3.98978286964828, 0, 0, 0.953939201416947, 0, 8.01142041980429, 1.18019368870418, 0, 0, 0, 0, 0, 1.77635683940025e-14, 5.55254896421454, 0, 9.62241374388481, -3.59830837050572, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 13.3932520322736, 0, -4.68666192508059, 1.18356086726904, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1.09264703862702, 0, 0, 0, 0, 0, 5.38682254607085, 0, 0, 0, 0, 0, 0, 0, 0, 0, 13.4377817958113, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 18.907670401189, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.78428514239954, 0, 0.814411017927931, 5.0677048703591, 0, 0, 0, 0, 4.01510046524764, 6.58672305740272, 0, 0, -1.06581410364015e-14, 0, 0, 0, 1.59872115546023e-14, 0, 10.0159311874144, 2.83293314500743, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 14.0929454377398, -4.55552790213386, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.22124532708767e-14, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.45686271816936, 0, 0, 5.07816454066238, 0, 0, 0, 0, 1.06581410364015e-14, 10.0142755247553, 0, 0, 0, 0, 0, 0, 0, 0, 0, 13.8784813613723, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.5535587278944, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    static const double dmats1[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.47722557505167, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.95803989154981, 5.6124860801609, -1.08012344973464, -0.763762615825972, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.29128784747791, 7.24568837309471, 4.18330013267037, -0.591607978309964, 0, 0, 0, 0, 0, 0, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-2.64575131106461, 0, 9.66091783079295, 0.683130051063968, 0, 0, 0, 0, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338697, 0, 0, 4.34741302385683, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.02140518265514e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.24037034920393, 0, 0, 7.52994023880668, 0, 0, 0, 0, 0, 0, 0, 0, 1.0547118733939e-14, 0, 0, 0, -1.84297022087776e-14, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.7495454169735, 5.79655069847576, -1.67332005306815, -1.18321595661993, 7.74596669241484, -1.2, 0.346410161513775, -0.979795897113271, 0.28284271247462, 0.200000000000002, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.32379000772444, 2.44948974278317, 2.82842712474617, -1.00000000000001, 9.16515138991169, 7.09929573971954, -2.04939015319193, -0.414039335605414, -0.478091443733755, 0.169030850945704, -1.4210854715202e-14, 1.06581410364015e-14, 0, -1.15463194561016e-14, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.80000000000001, -5.69209978830307, 4.38178046004132, -0.77459666924149, 0, 10.998181667894, 4.76235235991626, 0.962140470884713, -0.740656079818035, 0.130930734141589, 1.19904086659517e-14, -1.4210854715202e-14, 1.53210777398272e-14, -1.33226762955019e-14, 0, 1.06581410364015e-14, 1.4210854715202e-14, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.19615242270665, 0, -3.1622776601684, -2.23606797749978, 0, 0, 13.7477270848675, -1.4210854715202e-14, 0.534522483824862, 0.377964473009222, 1.4210854715202e-14, 0, 0, -3.01980662698043e-14, 0, 0, 1.77635683940025e-14, 0, -1.39055433834301e-14, 0, 0, 0, 0, 0, -1.24344978758018e-14, 0, 0, 0, 1.14908083048704e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.01246117974981, 2.12132034355964, -0.40824829046386, 3.17542648054293, 0, 0, 0, 7.17137165600636, -1.38013111868471, -1.56144011671765, 0, 0, 0, 0, 0, 0, -1.55431223447522e-14, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.55884572681199, 2.73861278752584, 1.5811388300842, 2.45967477524976, 0, 0, 0, 9.25820099772551, 5.34522483824848, -1.20948631362952, 0, 0, 0, 0, 0, 1.35447209004269e-14, -1.33226762955019e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.80000000000002, 0, 3.65148371670112, -2.84018778721878, 0, 0, 0, 0, 12.3442679969673, 1.39659449751035, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.27279220613578, 0, 0, 3.83405790253616, 0, 0, 0, 0, 0, 5.18459255872629, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.20454076850486, 0, 0, 6.64078308635359, 0, 0, 0, 0, 0, 8.97997772825746, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.56904651573302, 5.80288457473996, -2.01017818278147, -1.42141062443803, 8.14215222512715, -2.10229466468021, 0.728256234341417, -1.71651640581399, 0.594618725379072, 0.42045893293605, 9.81070843517429, -1.18450885369836, 0.305838870920592, -0.105945692672798, -1.02581475831037, 0.264864231681986, -0.09175166127618, 0.216260739578392, -0.0749149177264366, -0.0529728463363877, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239553, 3.07059789431495, 1.77281052085583, -1.25356634105602, 2.39356776939085, 6.30376871403132, -2.78313372537745, -0.908295106229253, -0.524404424085079, 0.370809924354787, 11.1242977306435, 9.40174755792014, -2.42752078113807, 0.840917865872087, -0.301561193523231, -0.794200206656965, 0.350641890608825, 0.11443442705426, 0.0660687472643419, -0.0467176592151146, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551269, 1.29756443747037, 3.74574588632197, -1.05945692672795, -8.09173593712687, 5.01426536422408, 1.80936716113936, -0.383824736789376, -1.10800657553489, 0.313391585264003, 2.1316282072803e-14, 12.7134831207354, 8.20651806648291, -2.84282124887606, 1.01946290306866, -0.631738055305796, -0.227958835180072, 0.0483573714561647, 0.139595707137603, -0.0394836284566167, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741913, 7.03562363973513, 4.06201920231799, -0.820651806648276, 0, -6.79705818718656, 5.88642506110458, -2.08116553882674, -1.2015614840698, 0.242752078113814, 0, -1.06581410364015e-14, 14.8323969741913, 5.13809303146608, 0, 0.856348838577665, -0.741619848709565, 0.262202212042526, 0.151382517704867, -0.0305838870920541, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-3.4253953543107, 0, 9.38083151964687, 1.89521416591737, 0, 0, -3.39852909359329, 0, -2.77488738510233, -0.560611910581384, -1.06581410364015e-14, 0, 0, 17.7988763690296, 0, 0, 0.428174419288809, 0, 0.349602949390039, 0.0706304617818535, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 1.50990331349021e-14, 0, 0, 0, 0, -1.73194791841524e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047609, 2.74642624930236, -0.792824967172096, 2.24244764232554, 2.14087209644419, -0.331662479035538, 0.0957427107756352, 6.49923072370876, -1.87616630392937, -2.15580611373101, -1.06581410364015e-14, 0, 0, 0, 9.44035859331469, -1.46249406456535, 0.422185670932514, -2.04706526287665, 0.590936840285273, 0.522319308773341, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.71269767715535, 1.16057691494799, 1.34011878852097, 1.89521416591735, 2.53311402559511, 1.96214168703486, -0.566421515598876, 2.74642624930236, 3.17129986868836, -1.82198870938951, -1.77635683940025e-14, 0, 0, 0, 11.1699829238239, 8.6522315682783, -2.49768411251824, -0.865042958313563, -0.998865569685877, 0.441440386136644, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 1.59872115546023e-14, 0, -1.77635683940025e-14, 0, 0, 0, 0, 1.06581410364015e-14, -1.09356967925578e-14, 1.02140518265514e-14, -1.33226762955019e-14, 0},
    {1.32664991614216, -2.6969370381518, 2.07610309995255, 1.46802658041887, 0, 3.03973683071412, 1.31624465810881, -6.38211787508103, 4.91295663090393, -1.4113063856887, 0, 0, 0, 0, 0, 13.4039795085888, 5.80409338312195, 2.01017818278148, -1.54743588659734, 0.341938252770126, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.11022302462516e-14, 1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 1.6228338117763e-14, 0, 0, 0, 0},
    {3.82970843102536, 0, -1.4982983545288, 4.2378277069118, 0, 0, 3.79967103839266, -1.33226762955019e-14, -3.54562104171166, -4.07409060843206, 0, 0, -1.4210854715202e-14, 0, 1.4210854715202e-14, 1.25455201782643e-14, 16.7549743857359, -1.06581410364015e-14, 1.11676565710081, 0.987090711415277, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 1.00508909139073, -0.193429485824663, 3.55615782880926, 0, 0, 0, 4.45964044034302, -0.858258202906996, 2.67027285925187, 0, 0, 0, 0, 0, 0, 0, 8.42792824422443, -1.62195554683793, -2.29379153190449, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.14891252930761, 1.29756443747035, 0.749149177264408, 2.75458800949267, 0, 0, 0, 5.75737105184053, 3.32401972660469, 2.06838446274244, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 10.8804085776374, 6.2818068211921, -1.77676328054754, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.32664991614216, -1.06581410364015e-14, 1.73008591662715, -3.18072425757424, 0, 0, 0, 0, 7.67649473578738, -2.38836465270395, 0, 0, 0, 0, 0, 1.06581410364015e-14, -1.47659662275146e-14, 0, 14.5072114368499, 2.05162951662074, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.938083151964684, 0, 0, 3.11415464992885, 0, 0, 0, 0, 0, 5.52707620976691, 0, 0, 0, 0, 0, 0, 1.58761892521397e-14, 0, 0, 5.80288457473998, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.62480768092719, 0, 0, 5.39387407630365, 0, 0, 0, 0, 0, 9.57317681262151, 0, 0, -1.15463194561016e-14, 0, -1.24344978758018e-14, 0, 2.63122856836162e-14, 0, 0, 10.0508909139074, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.32116539930394e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.41593350361254, 5.72988941019469, -2.20543546226846, -1.55947837083932, 8.33749895885385, -2.76779953857108, 1.06532653910463, -2.25989885993663, 0.869835476750496, 0.615066564126918, 10.333118277332, -2.18326971917505, 0.724780051083766, -0.278967971730992, -1.89076704011891, 0.627677936394715, -0.241593350361261, 0.51249688899006, -0.197260144544837, -0.139483985865487, 11.8490505948789, -1.16109782023228, 0.245326690731332, -0.0814411017927985, 0.0313466946954515, -1.03851746182648, 0.21942686286813, -0.0728431359084674, 0.0280373360835804, 0.190029237516525, -0.0630840061880544, 0.0242810453028245, -0.0515078753637725, 0.0198253904711041, 0.0140186680417935, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.18529407725406, 3.45525332069938, 0.997445717412069, -1.41060126129511, 3.23209352764251, 5.28531208250978, -2.89087233497829, -1.36277028773849, -0.393397896234718, 0.556348640264195, 2.33666428910958, 9.02784807228869, -3.93352933905728, 1.64018416088946, -0.732969916135164, -1.19859611757308, 0.655588223176204, 0.309047252182618, 0.0892142571199805, -0.126168012376108, 13.0996183150503, 11.55277744383, -2.44096975264613, 0.810328731479533, -0.311895674167856, -0.234843597212092, -0.907332870306513, 0.395334573318419, -0.164844710568311, 0.0736662483163206, 0.120463442338383, -0.0658890955530712, -0.0310604175145501, -0.00896637020658453, 0.0126803623514089, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.92724822331887, 1.82834820066012, 2.99085908983323, -1.2440333788203, 0.9501461875826, 5.7406445631131, -0.169967317119757, -0.721110255092796, -1.1796107330308, 0.49065338146266, -10.3037210107158, 4.97613351528119, 5.7817446699566, -3.2268221135323, -0.215472901842833, -1.3018558182182, 0.0385449644663813, 0.163532434528613, 0.267510569439617, -0.111269728052837, 0, 14.555131461167, 10.7636648722209, -3.57321383950879, 1.37533064807499, 1.03556292536503, -0.500120235663638, -0.581087203147974, 0.32430782472146, 0.0216558414515668, 0.130841432732455, -0.00387391468765474, -0.0164356280722655, -0.0268858238268351, 0.0111830284386718, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.62882203585592, 0.772618130456564, 3.86595114186566, -1.05140010313427, 9.1544290607646, 3.3588901228148, 3.878512233171, -0.304724700110022, -1.52475169285615, 0.414677792940102, 0, -9.4626009282084, 7.32969916135161, 0.564240504518009, -2.07602937339135, -0.761724681099364, -0.879563899362182, 0.0691050664139832, 0.34578118203618, -0.0940400840863436, 0, 0, 16.3745228938128, 9.05975063854702, -3.48709964663735, 0, 0.951027176355639, -0.736662483163211, -0.0567083043941548, 0.208648802584721, 0.0765562109321125, 0.0883994979795828, -0.00694532049720742, -0.0347523163751395, 0.00945138406569365, 0, -1.4210854715202e-14, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376112, -6.78263087840204, 3.68560354207933, -0.814411017927966, 0, 10.0216907612581, 6.00856531485322, 2.67510569439611, -1.45362164025439, 0.321208037219813, 1.33226762955019e-14, -1.06581410364015e-14, -7.5700807425667, 6.99294104721295, 0, -2.27270584034421, -1.36261453366202, -0.606656948415834, 0.329650402328125, -0.0728431359084816, 0, 0, 0, 18.7137404500718, 5.40219154319687, 0, 0, 0.760821741084501, -0.702817019232737, 0, 0.228415531250638, 0.136947913395213, 0.0609713174047632, -0.0331311120155173, 0.00732101061700419, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.09901951359278, 0, -4.65474668125631, -3.2914029430219, 1.24344978758018e-14, 0, 13.490737563232, 0, 1.83585684909538, 1.29814682728311, -1.4210854715202e-14, -3.5527136788005e-14, 4.2632564145606e-14, -3.53270434653113, -2.1316282072803e-14, 0, -3.05941170815567, 0, -0.416333199893195, -0.29439202887759, 0, 0, 0, 0, 21.8326971917504, 0, 0, 0, 0.35505014583943, 0, 0, 0.307482445914323, 0, 0.0418430609640534, 0.0295875121532867, 0, 0, 0, -1.77635683940025e-14, 1.4210854715202e-14, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 1.15463194561016e-14, 0, 0, 1.15463194561016e-14, 0, 0, 0},
    {1.99489143482413, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176042, -0.761811750275057, 0.263899331455877, 5.59815020469134, -1.93925611658551, -2.28543525082516, 2.13307290077014, -0.257539376818858, 0.0664963811608033, -0.0230350221379969, 9.14446135479766, -2.36108976914457, 0.817905488277891, -3.3384229850651, 1.15646364545769, 1.02505848514082, 0, 0, 0, 0, 0, 11.5766222110848, -1.39771878815471, 0.360889439280296, -0.125015768949702, -2.15192884958005, 0.555625639776456, -0.192474367616153, 0.595437196138647, -0.206265495285698, -0.166687691932932, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.7593288763725, 1.66904592079255, 0.96362411165942, 1.3627702877385, 0.867360833110887, 2.28430636201792, -1.00852844760553, 2.96226264871971, 1.71026313764871, -2.01556443707464, 2.41867732448956, 2.04415543160774, -0.527798662911756, 0.182834820066013, 2.68821895760393, 7.07977053255619, -3.12574097035514, -1.76652739792485, -1.01990506872276, 0.904016610326401, 0, 0, 0, 0, 0, 13.1266557397187, 11.0940489487798, -2.8644711213756, 0.992281903807267, -0.6326076194549, -1.66605356688627, 0.735567892896449, 0.31507574846809, 0.181909068193174, -0.147004726489965, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, -1.24344978758018e-14, 0},
    {1.48690428533296, 0.705300630647549, 2.03602754481988, 1.1517511068998, -2.93221479395416, 1.81702705031799, 0.655663160391206, 1.25178444059441, 3.61359041872287, -1.70346285967313, 1.06581410364015e-14, 2.7642026565595, 1.78428514239953, -0.618094504365255, -9.08783875864327, 5.63152770643899, 2.03210252328837, -0.74649407322535, -2.15494277062553, 0.764033484562597, 0, 0, 0, 0, 0, 0, 15.0018922739643, 9.68367982311024, -3.3545250915713, 2.13860408462397, -1.32524448062076, -0.4782064110178, 0.133143804689791, 0.384353057392901, -0.124241670058201, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 1.06581410364015e-14, 0, 0, 1.06581410364015e-14, -1.86517468137026e-14, 0, 1.13797860024079e-14, 0, 0, 0, 0, 0},
    {1.15175110689979, 3.82426463519456, 2.20794021658195, 0.89214257119979, 0, -2.46306042692148, 2.13307290077015, 6.78739640804928, 3.91870514328395, -1.31949665727937, 0, 0, 3.2249030993194, 1.11713920350152, 2.1316282072803e-14, -7.63378455726039, 6.61105135360481, -4.04762275910198, -2.33689608954558, 0.591817792331174, 0, 0, 0, 0, 0, 0, 0, 17.5022076529583, 6.06294257990893, 0, 1.79642743108413, -1.55575179137407, 0.721929233499942, 0.416806037297052, -0.0962371838080749, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.1316282072803e-14, 0, 0, 0, -1.59872115546023e-14, 0, 0, -1.24344978758018e-14, 0, -2.44249065417534e-14, -1.06581410364015e-14, 0},
    {-2.65985524643218, 0, 5.09901951359278, -2.06031501455085, 0, 0, -1.23153021346075, 1.59872115546023e-14, 9.04986187739903, 3.04724700110024, 0, 2.1316282072803e-14, 0, 3.86988371918331, 0, -4.2632564145606e-14, -3.81689227863023, 0, -5.39683034546928, -1.36674464685442, 0, 0, 0, 0, 1.28785870856518e-14, 0, 0, 0, 21.00264918355, 0, 0, 0.898213715542074, 0, 0.962572311333254, 0.222250255910579, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -3.10862446895044e-14, 0, 0, 1.50990331349021e-14, 0, 0, 0, 0, 0, -2.30926389122033e-14, 0},
    {1.57359158493889, 1.49284005458435, -0.430945803685675, 3.04724700110022, 0.775791113542725, -0.120185042515472, 0.0346944333244361, 5.29905651979669, -1.52970585407783, 0.901387818865992, 0, 0, 0, 0, 4.51561078268939, -0.699555414372423, 0.201944253400492, 6.36462844462229, -1.83730997289733, -2.74825341360597, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.8920630294282, -1.68739114875268, 0.487107866980278, -3.09993548319961, 0.894874292847881, 0.903959543974651, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32992762321609, 0.630840061880562, 0.7284313590847, 2.57539376818856, 0.917928424547682, 0.711024300256705, -0.205255035576785, 2.23926008187652, 2.58567482211401, 0.761811750275061, 0, 0, 0, 0, 5.34294273196308, 4.13862564412659, -1.19471831485579, 2.68954640486935, 3.10562068169861, -2.32269806558124, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.8876627769294, 9.98274066131696, -2.88176900403079, -1.30996183150502, -1.51261363209513, 0.763985254692628, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.03015750727541, -1.46593983227032, 1.1284810090361, 1.99489143482414, 0, 1.10151410945721, 0.476969600708482, -5.20357020296093, 4.00571020990214, 0.590096844352088, 0, 0, 0, 0, 0, 6.4115312783557, 2.77627448210726, -6.24994106094658, 4.81120687194241, -1.79915418525285, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.4651953323153, 6.69662601613678, 3.0440762145518, -2.34333096254029, 0.591780433634522, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.01030295240889e-14, 0, 1.06581410364015e-14, 0, 0},
    {2.9738085706659, -2.1316282072803e-14, -0.814411017927963, 5.758755534499, 0, 0, 1.37689263682153, 0, -2.8908723349783, 1.70346285967311, 2.1316282072803e-14, 0, 0, 0, 0, 0, 8.01441409794462, 0, -3.47218947830368, -5.19371076584692, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 19.3314941653941, 0, 1.69115345252877, 1.70832296330021, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, -1.77635683940025e-14, 0, 0, 0, -1.4210854715202e-14, 0, 0, 0},
    {1.15175110689979, 0.546323519313513, -0.105140010313428, 3.27118942773249, 0, 0, 0, 2.69341127303542, -0.518347241175123, 4.25171145123352, 0, 0, 0, 0, 0, 0, 1.19904086659517e-14, 6.71889089790563, -1.29305115618717, 1.82865048192213, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.45383520059452, -1.81939143264587, -2.94058061635795, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.892142571199769, 0.705300630647548, 0.407205508963976, 2.53385243517955, 0, 0, 0, 3.47717900165117, 2.00755023262382, 3.29336152870135, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 8.67405085085771, 5.0079655937072, 1.41646657250371, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.2048487632307, 7.0464727188699, -2.27776395106694, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.03015750727543, 0, 0.940400840863387, -2.92584077107541, 0, 0, 0, 0, 4.63623866886823, -3.80284633026897, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 11.5654011344769, -1.63559471386627, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 16.2731316843076, 2.63013526059784, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, -1.06581410364015e-14, -1.06581410364015e-14},
    {0.728431359084679, 0, 0, 2.53908227033119, 0, 0, 0, 0, 0, 5.00713776237767, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.93924068068616, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.2767793639472, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376111, 0, 0, 4.39781949681096, 0, 0, 0, 0, 1.06581410364015e-14, 8.67261700493486, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.0191174248973, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.8717007662564, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    static const double dmats2[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.82574185835056, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.16397779494323, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.95803989154981, 5.61248608016089, -1.08012344973464, -0.763762615825974, 0, 0, 0, 0, 0, 0, 0, 0, -1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.29128784747792, 1.44913767461895, 4.18330013267037, -0.591607978309966, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32287565553229, 0, 3.86436713231718, -0.341565025531991, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338697, 7.09929573971953, 0, 4.34741302385682, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.08012344973463, 1.06581410364015e-14, 7.09929573971953, 2.50998007960221, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-3.81881307912987, 0, 0, 8.87411967464941, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, -1.33226762955019e-14, 0, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.7495454169735, 5.79655069847577, -1.67332005306815, -1.18321595661993, 7.74596669241484, -1.2, 0.346410161513777, -0.979795897113275, 0.282842712474621, 0.200000000000002, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.32379000772445, 2.44948974278317, 2.82842712474618, -1.00000000000001, 1.30930734141595, 7.09929573971954, -2.04939015319192, -0.414039335605415, -0.478091443733755, 0.169030850945703, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.8, 0.632455532033677, 4.38178046004131, -0.774596669241482, 0, 3.14233761939829, 4.76235235991626, -0.106904496764974, -0.740656079818036, 0.130930734141593, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.03923048454133, 0, 3.16227766016837, -0.447213595499959, 0, 0, 5.89188303637179, 0, -0.534522483824841, 0.0755928946018432, 0, -1.24344978758018e-14, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.0124611797498, 2.12132034355962, -0.408248290463868, 3.17542648054291, 9.07114735222146, 0, 0, 7.17137165600636, -1.38013111868471, -1.56144011671765, -1.06581410364015e-14, 0, 0, 0, 0, 0, -2.33146835171283e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.11022302462516e-14, 0, 0, 0},
    {1.55884572681199, 0.547722557505173, 1.58113883008419, 2.45967477524976, 0, 9.07114735222146, 0, 1.8516401995451, 5.34522483824849, -1.20948631362953, 0, 0, 1.95399252334028e-14, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.02140518265514e-14, 0, 0, 0, 0, 0, 0, 0},
    {0.899999999999999, 1.4210854715202e-14, 1.46059348668045, 1.42009389360938, 0, 0, 9.07114735222145, 0, 4.93770719878694, -0.698297248755174, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.27279220613579, -6.2609903369994, 0, 3.83405790253615, 0, 0, 0, 10.5830052442584, 0, 5.18459255872628, 0, 1.06581410364015e-14, 1.95399252334028e-14, 0, 0, 0, 0, 1.06581410364015e-14, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.734846922834958, 0, -6.26099033699939, 2.21359436211786, 0, 0, 0, 0, 10.5830052442584, 2.99332590941916, 0, 0, 0, 0, 0, 0, 0, 0, 1.68753899743024e-14, 1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.7157676649773, 0, 0, -4.69574275274955, 0, 0, 0, 0, 0, 12.69960629311, 1.77635683940025e-14, 0, -1.33226762955019e-14, 0, -1.28785870856518e-14, 0, 2.79776202205539e-14, 0, 1.06581410364015e-14, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.49880108324396e-14, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0},
    {2.56904651573302, 5.80288457473996, -2.01017818278147, -1.42141062443803, 8.14215222512716, -2.10229466468021, 0.728256234341419, -1.71651640581399, 0.594618725379072, 0.420458932936049, 9.81070843517429, -1.18450885369835, 0.30583887092059, -0.105945692672798, -1.02581475831037, 0.264864231681985, -0.0917516612761808, 0.216260739578389, -0.0749149177264375, -0.0529728463363888, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239553, 3.07059789431495, 1.77281052085583, -1.25356634105602, 2.39356776939085, 6.30376871403131, -2.78313372537745, -0.908295106229252, -0.524404424085074, 0.370809924354786, 1.23603308118261, 9.40174755792014, -2.42752078113807, 0.840917865872084, -0.301561193523235, -0.794200206656964, 0.350641890608822, 0.114434427054266, 0.0660687472643433, -0.0467176592151159, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551268, 1.29756443747036, 3.74574588632196, -1.05945692672795, 0.404586796856345, 5.01426536422408, 1.80936716113937, -0.38382473678937, -1.10800657553489, 0.313391585264002, -1.06581410364015e-14, 2.82521847127454, 8.20651806648291, -2.84282124887606, -0.0509731451534328, -0.631738055305796, -0.227958835180075, 0.0483573714561643, 0.139595707137599, -0.0394836284566149, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 0.335029697130249, 4.06201920231797, -0.820651806648284, 0, 1.69926454679665, 5.88642506110456, -0.0991031208965145, -1.20156148406978, 0.242752078113805, 0, 0, 4.94413232473045, 5.13809303146606, 0, -0.214087209644424, -0.741619848709568, 0.0124858196210687, 0.151382517704866, -0.0305838870920656, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.856348838577684, 0, 2.68023757704195, -0.473803541479342, 0, 0, 5.09779364038993, 0, -0.792824967172093, 0.140152977645347, 0, -1.4210854715202e-14, 0, 7.91061171956872, 0, 0, -0.642261628933261, 0, 0.0998865569685847, -0.0176576154454708, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047609, 2.74642624930237, -0.792824967172094, 2.24244764232555, 2.14087209644419, -0.331662479035534, 0.0957427107756326, 6.49923072370876, -1.87616630392937, -2.15580611373101, 11.0554159678513, 0, 0, 0, 9.4403585933147, -1.46249406456535, 0.422185670932516, -2.04706526287665, 0.590936840285272, 0.522319308773344, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.71269767715536, 1.160576914948, 1.34011878852098, 1.89521416591736, 0.361873432227874, 1.96214168703486, -0.566421515598876, 2.74642624930237, 3.17129986868837, -1.82198870938951, 0, 11.0554159678513, 0, 0, 1.59571184626055, 8.6522315682783, -2.49768411251825, -0.865042958313561, -0.998865569685869, 0.441440386136648, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.04360964314765e-14, 1.95399252334028e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32664991614215, 0.299659670905754, 2.07610309995255, 1.46802658041887, 0, 0.868496237346892, 1.31624465810881, 0.709124208342329, 4.91295663090393, -1.4113063856887, 0, 0, 11.0554159678513, 0, 0, 3.82970843102537, 5.80409338312195, -0.223353131420178, -1.54743588659734, 0.341938252770122, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.76594168620508, 0, 1.49829835452878, 0.847565541382357, 0, 0, 1.62843044502542, 0, 3.54562104171167, -0.814818121686407, 0, 0, 0, 11.0554159678514, 0, 1.06581410364015e-14, 7.18070330817255, 0, -1.11676565710083, 0.197418142283059, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 1.00508909139074, -0.193429485824666, 3.55615782880927, -8.46157280212812, 0, 0, 4.459640440343, -0.858258202906995, 2.67027285925187, 0, -1.29340982368831e-14, 0, 0, 12.4373429638327, 0, 0, 8.42792824422443, -1.62195554683794, -2.29379153190448, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.1489125293076, 0.259512887494068, 0.749149177264412, 2.75458800949266, -1.24344978758018e-14, -8.46157280212812, 0, 1.1514742103681, 3.32401972660468, 2.06838446274244, 0, 0, 0, 1.24344978758018e-14, 0, 12.4373429638328, -2.22044604925031e-14, 2.17608171552749, 6.2818068211921, -1.77676328054753, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.663324958071075, 0, 0.692034366650866, 1.59036212878711, 0, 0, -8.46157280212812, 0, 3.07059789431494, 1.19418232635199, 0, 0, -1.4210854715202e-14, 0, 0, 1.50990331349021e-14, 12.4373429638327, 0, 5.80288457473998, -1.02581475831035, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.37667655053519e-14, 0, 0, 0, 0, 0, 0, 0},
    {0.938083151964683, 7.62808987244124, 0, 3.11415464992887, 0, 0, 0, -7.5213980463361, 0, 5.52707620976692, 0, 0, 0, 0, 0, 0, 2.39808173319034e-14, 14.2141062443803, 1.59872115546023e-14, 5.80288457473999, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.541602560309058, 0, 7.62808987244124, 1.79795802543455, 0, 0, 0, 0, -7.5213980463361, 3.19105893754051, 0, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 14.2141062443803, 3.35029697130246, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-5.36159180343551, 0, 0, 8.89943818451482, 0, 0, 0, 0, 1.4210854715202e-14, -5.26497863243527, 0, 0, 0, 0, 1.4210854715202e-14, 0, 1.01585406753202e-14, -1.4210854715202e-14, -1.06581410364015e-14, 16.583123951777, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14},
    {2.41593350361254, 5.72988941019469, -2.20543546226846, -1.55947837083932, 8.33749895885385, -2.76779953857108, 1.06532653910463, -2.25989885993663, 0.869835476750495, 0.615066564126917, 10.333118277332, -2.18326971917505, 0.724780051083766, -0.27896797173099, -1.89076704011891, 0.627677936394712, -0.24159335036126, 0.512496888990059, -0.197260144544837, -0.139483985865486, 11.8490505948789, -1.16109782023228, 0.245326690731332, -0.0814411017927985, 0.031346694695451, -1.03851746182648, 0.21942686286813, -0.0728431359084674, 0.0280373360835801, 0.190029237516525, -0.063084006188054, 0.0242810453028244, -0.0515078753637721, 0.0198253904711043, 0.0140186680417933, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.18529407725405, 3.45525332069938, 0.997445717412057, -1.41060126129511, 3.23209352764252, 5.28531208250977, -2.89087233497829, -1.36277028773849, -0.393397896234717, 0.556348640264194, 2.33666428910958, 9.02784807228868, -3.93352933905729, 1.64018416088945, -0.732969916135168, -1.19859611757306, 0.655588223176204, 0.309047252182627, 0.0892142571199752, -0.126168012376108, 1.1908743922773, 11.55277744383, -2.44096975264614, 0.810328731479533, -0.311895674167854, -0.234843597212092, -0.907332870306512, 0.395334573318419, -0.164844710568313, 0.0736662483163242, 0.120463442338381, -0.0658890955530682, -0.0310604175145517, -0.00896637020658309, 0.0126803623514131, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.92724822331887, 1.82834820066013, 2.99085908983323, -1.2440333788203, 0.950146187582614, 5.7406445631131, -0.169967317119762, -0.721110255092803, -1.1796107330308, 0.490653381462659, 0.294392028877592, 4.9761335152812, 5.78174466995659, -3.22682211353229, -0.215472901842833, -1.30185581821819, 0.0385449644663778, 0.163532434528617, 0.267510569439617, -0.111269728052837, 0, 2.64638753839399, 10.7636648722209, -3.57321383950879, 1.37533064807499, -0.0295875121532885, -0.500120235663637, -0.581087203147972, 0.324307824721458, 0.0216558414515666, 0.130841432732452, -0.00387391468765502, -0.016435628072264, -0.0268858238268355, 0.0111830284386731, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.62882203585592, 0.772618130456575, 3.86595114186567, -1.05140010313427, 0.160604018609906, 3.35889012281481, 3.878512233171, -0.304724700110032, -1.52475169285615, 0.414677792940099, 0, 1.135512111385, 7.32969916135161, 0.564240504518033, -0.0364215679542301, -0.761724681099366, -0.879563899362179, 0.0691050664139837, 0.34578118203618, -0.0940400840863451, 0, 0, 4.46577897103986, 9.05975063854702, -3.48709964663735, 0, -0.114123261162675, -0.736662483163209, -0.0567083043941503, 0.00366050530850437, 0.0765562109321125, 0.0883994979795846, -0.00694532049720609, -0.0347523163751409, 0.00945138406569246, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376112, 0.199489143482424, 3.68560354207932, -0.814411017927961, 0, 1.02786571910339, 6.00856531485322, -0.078679579246943, -1.45362164025439, 0.321208037219817, 0, 0, 3.0280322970267, 6.99294104721296, 0, -0.233098034907105, -1.36261453366201, 0.0178428514239979, 0.329650402328133, -0.0728431359084714, 0, 0, 0, 6.80499652729884, 5.40219154319687, 0, 0, -0.304328696433807, -0.702817019232741, 0, 0.0234272339744261, 0.136947913395214, -0.00179327404131646, -0.0331311120155191, 0.00732101061700707, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.72843135908468, 0, 2.32737334062817, -0.470200420431707, 0, 0, 4.49691252107735, 0, -0.91792842454769, 0.185449546754732, 0, 0, 1.77635683940025e-14, 7.0654086930623, 0, 0, -1.01980390271858, 0, 0.208166599946617, -0.0420560041253766, 0, 0, 0, 0, 9.92395326897747, 0, 0, 0, -0.710100291678885, 0, 0, 0.102494148638109, 0, -0.02092153048203, 0.00422678745046823, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.99489143482414, 3.1542003094028, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275054, 0.263899331455879, 5.59815020469134, -1.93925611658551, -2.28543525082517, 2.13307290077014, -0.257539376818855, 0.0664963811608086, -0.023035022137995, 9.14446135479766, -2.36108976914458, 0.817905488277891, -3.3384229850651, 1.15646364545769, 1.02505848514082, 13.0453753561106, 0, 0, 0, 0, 11.5766222110848, -1.39771878815471, 0.360889439280295, -0.125015768949702, -2.15192884958005, 0.555625639776459, -0.192474367616154, 0.595437196138649, -0.206265495285697, -0.166687691932934, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.7593288763725, 1.66904592079255, 0.963624111659421, 1.36277028773849, 0.867360833110883, 2.28430636201793, -1.00852844760553, 2.96226264871971, 1.71026313764871, -2.01556443707463, 0.268741924943274, 2.04415543160775, -0.52779866291175, 0.182834820066012, 2.68821895760393, 7.07977053255619, -3.12574097035514, -1.76652739792483, -1.01990506872276, 0.904016610326405, 0, 13.0453753561106, 0, 0, 1.77635683940025e-14, 1.4585173044132, 11.0940489487798, -2.8644711213756, 0.992281903807275, -0.632607619454901, -1.66605356688627, 0.735567892896449, 0.31507574846809, 0.181909068193172, -0.147004726489962, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, -1.33226762955019e-14, 0},
    {1.48690428533297, 0.705300630647553, 2.03602754481991, 1.15175110689979, 0.146610739697709, 1.817027050318, 0.655663160391203, 1.25178444059441, 3.61359041872286, -1.70346285967311, 0, 0.614267257013221, 1.78428514239955, -0.618094504365256, 0.454391937932176, 5.63152770643899, 2.03210252328836, -0.746494073225344, -2.15494277062551, 0.764033484562614, 0, 0, 13.0453753561106, 0, 0, 0, 3.33375383865873, 9.68367982311023, -3.3545250915713, -0.106930204231195, -1.32524448062076, -0.478206411017788, 0.133143804689793, 0.384353057392904, -0.124241670058193, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.37667655053519e-14, 0},
    {1.15175110689979, 0.182107839771145, 2.20794021658194, 0.892142571199776, 0, 0.61576510673037, 2.13307290077016, 0.323209352764256, 3.91870514328395, -1.31949665727936, 0, -1.06581410364015e-14, 1.07496769977314, 1.11713920350151, 0, 1.90844613931508, 6.61105135360481, -0.192743940909633, -2.33689608954559, 0.591817792331167, 0, 0, 0, 13.0453753561106, 0, 0, 0, 5.83406921765278, 6.06294257990893, 0, -0.44910685777103, -1.55575179137407, 0.0343775825476159, 0.41680603729705, -0.0962371838080759, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 0},
    {0.66496381160804, 0, 1.45686271816934, 0.515078753637713, 0, -1.24344978758018e-14, 1.84729532019112, 0, 2.58567482211401, -0.761811750275046, 1.4210854715202e-14, -1.24344978758018e-14, 0, 1.71994831963701, 0, -1.4210854715202e-14, 5.72533841794525, 0, -1.54195152727693, 0.341686161713598, 0, 0, 0, 0, 13.0453753561106, 0, 0, 0, 9.33451074824443, 0, 0, -1.34732057331309, 0, 0.27502066038093, -0.0555625639776447, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0},
    {1.57359158493889, 1.49284005458435, -0.430945803685675, 3.04724700110023, 0.775791113542727, -0.120185042515474, 0.0346944333244412, 5.2990565197967, -1.52970585407783, 0.901387818865993, -10.5762837413611, 0, 0, 0, 4.5156107826894, -0.69955541437243, 0.201944253400492, 6.36462844462228, -1.83730997289733, -2.74825341360596, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 0, 10.8920630294282, -1.68739114875268, 0.487107866980279, -3.09993548319961, 0.894874292847885, 0.903959543974656, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32992762321608, 0.63084006188056, 0.728431359084682, 2.57539376818856, 0.131132632078239, 0.71102430025671, -0.205255035576787, 2.23926008187652, 2.58567482211401, 0.761811750275053, 0, -10.576283741361, 0, 0, 0.763277533137579, 4.1386256441266, -1.19471831485579, 2.68954640486935, 3.1056206816986, -2.32269806558125, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 1.84109468241849, 9.98274066131697, -2.88176900403079, -1.30996183150502, -1.51261363209512, 0.763985254692621, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.28785870856518e-14, 0, 0},
    {1.03015750727542, 0.162882203585586, 1.1284810090361, 1.99489143482414, 0, 0.314718316987764, 0.476969600708468, 0.57817446699566, 4.00571020990214, 0.590096844352083, 0, 0, -10.5762837413611, 0, 0, 1.83186607953019, 2.77627448210727, 0.694437895660723, 4.81120687194241, -1.79915418525285, 0, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 4.41862723780437, 6.69662601613678, -0.338230690505752, -2.34333096254029, 0.591780433634522, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.14352971536391e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.594761714133174, -1.19904086659517e-14, 0.814411017927931, 1.15175110689982, 0, 0, 0.590096844352076, 0, 2.89087233497831, 0.340692571934628, 0, 0, 0, -10.576283741361, 0, 0, 3.43474889911912, -1.24344978758018e-14, 3.47218947830365, -1.03874215316938, 0, 0, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 8.28492607088319, 0, -1.69115345252878, 0.341664592660043, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.68753899743024e-14, 0},
    {1.15175110689979, 0.546323519313511, -0.105140010313427, 3.27118942773251, 9.65295620481434, 0, 0, 2.69341127303542, -0.518347241175114, 4.25171145123353, 0, 0, 0, 0, -9.91526600752598, 0, 1.53210777398272e-14, 6.71889089790563, -1.29305115618718, 1.82865048192216, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.9443476574685, 0, 0, 9.45383520059451, -1.81939143264587, -2.94058061635793, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 1.68753899743024e-14},
    {0.892142571199775, 0.141060126129508, 0.407205508963975, 2.53385243517955, 0, 9.65295620481433, 0, 0.69543580033025, 2.00755023262383, 3.29336152870135, 0, 0, 0, 0, 0, -9.91526600752598, 2.12052597703405e-14, 1.73481017017154, 5.0079655937072, 1.41646657250371, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.9443476574685, 0, 2.44096975264614, 7.0464727188699, -2.27776395106693, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.515078753637709, 0, 0.376160336345346, 1.46292038553773, 0, 0, 9.65295620481433, 0, 1.8544954675473, 1.9014231651345, 0, 0, 0, -1.06581410364015e-14, 0, 0, -9.91526600752595, 0, 4.62616045379076, 0.81779735693317, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.9443476574685, 0, 6.50925267372302, -1.3150676302989, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 1.86517468137026e-14},
    {0.728431359084674, -7.94708263760857, 0, 2.53908227033117, 0, 0, 0, 10.2207771580387, 0, 5.00713776237767, 0, 0, 0, 0, -1.33226762955019e-14, 2.1316282072803e-14, 0, -8.49879943502227, -1.01030295240889e-14, 6.93924068068615, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 17.9373911146521, 0, 6.2767793639472, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0},
    {0.42056004125371, 0, -7.94708263760858, 1.46593983227034, 0, 0, 0, 0, 10.2207771580387, 2.89087233497828, 0, 0, 0, -1.06581410364015e-14, 2.1316282072803e-14, -1.77635683940025e-14, 2.62012633811537e-14, 0, -8.49879943502228, 4.00637247496576, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 17.9373911146521, 3.6239002554188, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.22124532708767e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {6.54237885546499, 0, 1.50990331349021e-14, -6.91050664139879, 0, 0, 0, 0, 0, 12.2649325896464, 0, -1.79856129989275e-14, 1.77635683940025e-14, 1.33226762955019e-14, 0, 0, -1.75415237890775e-14, 0, 1.06581410364015e-14, -5.66586629001485, 0, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 20.4998755596024, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.46549439250521e-14, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, -1.59872115546023e-14}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    double coeff0_6 = 0;
    double coeff0_7 = 0;
    double coeff0_8 = 0;
    double coeff0_9 = 0;
    double coeff0_10 = 0;
    double coeff0_11 = 0;
    double coeff0_12 = 0;
    double coeff0_13 = 0;
    double coeff0_14 = 0;
    double coeff0_15 = 0;
    double coeff0_16 = 0;
    double coeff0_17 = 0;
    double coeff0_18 = 0;
    double coeff0_19 = 0;
    double coeff0_20 = 0;
    double coeff0_21 = 0;
    double coeff0_22 = 0;
    double coeff0_23 = 0;
    double coeff0_24 = 0;
    double coeff0_25 = 0;
    double coeff0_26 = 0;
    double coeff0_27 = 0;
    double coeff0_28 = 0;
    double coeff0_29 = 0;
    double coeff0_30 = 0;
    double coeff0_31 = 0;
    double coeff0_32 = 0;
    double coeff0_33 = 0;
    double coeff0_34 = 0;
    double coeff0_35 = 0;
    double coeff0_36 = 0;
    double coeff0_37 = 0;
    double coeff0_38 = 0;
    double coeff0_39 = 0;
    double coeff0_40 = 0;
    double coeff0_41 = 0;
    double coeff0_42 = 0;
    double coeff0_43 = 0;
    double coeff0_44 = 0;
    double coeff0_45 = 0;
    double coeff0_46 = 0;
    double coeff0_47 = 0;
    double coeff0_48 = 0;
    double coeff0_49 = 0;
    double coeff0_50 = 0;
    double coeff0_51 = 0;
    double coeff0_52 = 0;
    double coeff0_53 = 0;
    double coeff0_54 = 0;
    double coeff0_55 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    double new_coeff0_6 = 0;
    double new_coeff0_7 = 0;
    double new_coeff0_8 = 0;
    double new_coeff0_9 = 0;
    double new_coeff0_10 = 0;
    double new_coeff0_11 = 0;
    double new_coeff0_12 = 0;
    double new_coeff0_13 = 0;
    double new_coeff0_14 = 0;
    double new_coeff0_15 = 0;
    double new_coeff0_16 = 0;
    double new_coeff0_17 = 0;
    double new_coeff0_18 = 0;
    double new_coeff0_19 = 0;
    double new_coeff0_20 = 0;
    double new_coeff0_21 = 0;
    double new_coeff0_22 = 0;
    double new_coeff0_23 = 0;
    double new_coeff0_24 = 0;
    double new_coeff0_25 = 0;
    double new_coeff0_26 = 0;
    double new_coeff0_27 = 0;
    double new_coeff0_28 = 0;
    double new_coeff0_29 = 0;
    double new_coeff0_30 = 0;
    double new_coeff0_31 = 0;
    double new_coeff0_32 = 0;
    double new_coeff0_33 = 0;
    double new_coeff0_34 = 0;
    double new_coeff0_35 = 0;
    double new_coeff0_36 = 0;
    double new_coeff0_37 = 0;
    double new_coeff0_38 = 0;
    double new_coeff0_39 = 0;
    double new_coeff0_40 = 0;
    double new_coeff0_41 = 0;
    double new_coeff0_42 = 0;
    double new_coeff0_43 = 0;
    double new_coeff0_44 = 0;
    double new_coeff0_45 = 0;
    double new_coeff0_46 = 0;
    double new_coeff0_47 = 0;
    double new_coeff0_48 = 0;
    double new_coeff0_49 = 0;
    double new_coeff0_50 = 0;
    double new_coeff0_51 = 0;
    double new_coeff0_52 = 0;
    double new_coeff0_53 = 0;
    double new_coeff0_54 = 0;
    double new_coeff0_55 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
      new_coeff0_6 = coefficients0[dof][6];
      new_coeff0_7 = coefficients0[dof][7];
      new_coeff0_8 = coefficients0[dof][8];
      new_coeff0_9 = coefficients0[dof][9];
      new_coeff0_10 = coefficients0[dof][10];
      new_coeff0_11 = coefficients0[dof][11];
      new_coeff0_12 = coefficients0[dof][12];
      new_coeff0_13 = coefficients0[dof][13];
      new_coeff0_14 = coefficients0[dof][14];
      new_coeff0_15 = coefficients0[dof][15];
      new_coeff0_16 = coefficients0[dof][16];
      new_coeff0_17 = coefficients0[dof][17];
      new_coeff0_18 = coefficients0[dof][18];
      new_coeff0_19 = coefficients0[dof][19];
      new_coeff0_20 = coefficients0[dof][20];
      new_coeff0_21 = coefficients0[dof][21];
      new_coeff0_22 = coefficients0[dof][22];
      new_coeff0_23 = coefficients0[dof][23];
      new_coeff0_24 = coefficients0[dof][24];
      new_coeff0_25 = coefficients0[dof][25];
      new_coeff0_26 = coefficients0[dof][26];
      new_coeff0_27 = coefficients0[dof][27];
      new_coeff0_28 = coefficients0[dof][28];
      new_coeff0_29 = coefficients0[dof][29];
      new_coeff0_30 = coefficients0[dof][30];
      new_coeff0_31 = coefficients0[dof][31];
      new_coeff0_32 = coefficients0[dof][32];
      new_coeff0_33 = coefficients0[dof][33];
      new_coeff0_34 = coefficients0[dof][34];
      new_coeff0_35 = coefficients0[dof][35];
      new_coeff0_36 = coefficients0[dof][36];
      new_coeff0_37 = coefficients0[dof][37];
      new_coeff0_38 = coefficients0[dof][38];
      new_coeff0_39 = coefficients0[dof][39];
      new_coeff0_40 = coefficients0[dof][40];
      new_coeff0_41 = coefficients0[dof][41];
      new_coeff0_42 = coefficients0[dof][42];
      new_coeff0_43 = coefficients0[dof][43];
      new_coeff0_44 = coefficients0[dof][44];
      new_coeff0_45 = coefficients0[dof][45];
      new_coeff0_46 = coefficients0[dof][46];
      new_coeff0_47 = coefficients0[dof][47];
      new_coeff0_48 = coefficients0[dof][48];
      new_coeff0_49 = coefficients0[dof][49];
      new_coeff0_50 = coefficients0[dof][50];
      new_coeff0_51 = coefficients0[dof][51];
      new_coeff0_52 = coefficients0[dof][52];
      new_coeff0_53 = coefficients0[dof][53];
      new_coeff0_54 = coefficients0[dof][54];
      new_coeff0_55 = coefficients0[dof][55];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
        coeff0_6 = new_coeff0_6;
        coeff0_7 = new_coeff0_7;
        coeff0_8 = new_coeff0_8;
        coeff0_9 = new_coeff0_9;
        coeff0_10 = new_coeff0_10;
        coeff0_11 = new_coeff0_11;
        coeff0_12 = new_coeff0_12;
        coeff0_13 = new_coeff0_13;
        coeff0_14 = new_coeff0_14;
        coeff0_15 = new_coeff0_15;
        coeff0_16 = new_coeff0_16;
        coeff0_17 = new_coeff0_17;
        coeff0_18 = new_coeff0_18;
        coeff0_19 = new_coeff0_19;
        coeff0_20 = new_coeff0_20;
        coeff0_21 = new_coeff0_21;
        coeff0_22 = new_coeff0_22;
        coeff0_23 = new_coeff0_23;
        coeff0_24 = new_coeff0_24;
        coeff0_25 = new_coeff0_25;
        coeff0_26 = new_coeff0_26;
        coeff0_27 = new_coeff0_27;
        coeff0_28 = new_coeff0_28;
        coeff0_29 = new_coeff0_29;
        coeff0_30 = new_coeff0_30;
        coeff0_31 = new_coeff0_31;
        coeff0_32 = new_coeff0_32;
        coeff0_33 = new_coeff0_33;
        coeff0_34 = new_coeff0_34;
        coeff0_35 = new_coeff0_35;
        coeff0_36 = new_coeff0_36;
        coeff0_37 = new_coeff0_37;
        coeff0_38 = new_coeff0_38;
        coeff0_39 = new_coeff0_39;
        coeff0_40 = new_coeff0_40;
        coeff0_41 = new_coeff0_41;
        coeff0_42 = new_coeff0_42;
        coeff0_43 = new_coeff0_43;
        coeff0_44 = new_coeff0_44;
        coeff0_45 = new_coeff0_45;
        coeff0_46 = new_coeff0_46;
        coeff0_47 = new_coeff0_47;
        coeff0_48 = new_coeff0_48;
        coeff0_49 = new_coeff0_49;
        coeff0_50 = new_coeff0_50;
        coeff0_51 = new_coeff0_51;
        coeff0_52 = new_coeff0_52;
        coeff0_53 = new_coeff0_53;
        coeff0_54 = new_coeff0_54;
        coeff0_55 = new_coeff0_55;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0] + coeff0_10*dmats0[10][0] + coeff0_11*dmats0[11][0] + coeff0_12*dmats0[12][0] + coeff0_13*dmats0[13][0] + coeff0_14*dmats0[14][0] + coeff0_15*dmats0[15][0] + coeff0_16*dmats0[16][0] + coeff0_17*dmats0[17][0] + coeff0_18*dmats0[18][0] + coeff0_19*dmats0[19][0] + coeff0_20*dmats0[20][0] + coeff0_21*dmats0[21][0] + coeff0_22*dmats0[22][0] + coeff0_23*dmats0[23][0] + coeff0_24*dmats0[24][0] + coeff0_25*dmats0[25][0] + coeff0_26*dmats0[26][0] + coeff0_27*dmats0[27][0] + coeff0_28*dmats0[28][0] + coeff0_29*dmats0[29][0] + coeff0_30*dmats0[30][0] + coeff0_31*dmats0[31][0] + coeff0_32*dmats0[32][0] + coeff0_33*dmats0[33][0] + coeff0_34*dmats0[34][0] + coeff0_35*dmats0[35][0] + coeff0_36*dmats0[36][0] + coeff0_37*dmats0[37][0] + coeff0_38*dmats0[38][0] + coeff0_39*dmats0[39][0] + coeff0_40*dmats0[40][0] + coeff0_41*dmats0[41][0] + coeff0_42*dmats0[42][0] + coeff0_43*dmats0[43][0] + coeff0_44*dmats0[44][0] + coeff0_45*dmats0[45][0] + coeff0_46*dmats0[46][0] + coeff0_47*dmats0[47][0] + coeff0_48*dmats0[48][0] + coeff0_49*dmats0[49][0] + coeff0_50*dmats0[50][0] + coeff0_51*dmats0[51][0] + coeff0_52*dmats0[52][0] + coeff0_53*dmats0[53][0] + coeff0_54*dmats0[54][0] + coeff0_55*dmats0[55][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1] + coeff0_10*dmats0[10][1] + coeff0_11*dmats0[11][1] + coeff0_12*dmats0[12][1] + coeff0_13*dmats0[13][1] + coeff0_14*dmats0[14][1] + coeff0_15*dmats0[15][1] + coeff0_16*dmats0[16][1] + coeff0_17*dmats0[17][1] + coeff0_18*dmats0[18][1] + coeff0_19*dmats0[19][1] + coeff0_20*dmats0[20][1] + coeff0_21*dmats0[21][1] + coeff0_22*dmats0[22][1] + coeff0_23*dmats0[23][1] + coeff0_24*dmats0[24][1] + coeff0_25*dmats0[25][1] + coeff0_26*dmats0[26][1] + coeff0_27*dmats0[27][1] + coeff0_28*dmats0[28][1] + coeff0_29*dmats0[29][1] + coeff0_30*dmats0[30][1] + coeff0_31*dmats0[31][1] + coeff0_32*dmats0[32][1] + coeff0_33*dmats0[33][1] + coeff0_34*dmats0[34][1] + coeff0_35*dmats0[35][1] + coeff0_36*dmats0[36][1] + coeff0_37*dmats0[37][1] + coeff0_38*dmats0[38][1] + coeff0_39*dmats0[39][1] + coeff0_40*dmats0[40][1] + coeff0_41*dmats0[41][1] + coeff0_42*dmats0[42][1] + coeff0_43*dmats0[43][1] + coeff0_44*dmats0[44][1] + coeff0_45*dmats0[45][1] + coeff0_46*dmats0[46][1] + coeff0_47*dmats0[47][1] + coeff0_48*dmats0[48][1] + coeff0_49*dmats0[49][1] + coeff0_50*dmats0[50][1] + coeff0_51*dmats0[51][1] + coeff0_52*dmats0[52][1] + coeff0_53*dmats0[53][1] + coeff0_54*dmats0[54][1] + coeff0_55*dmats0[55][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2] + coeff0_10*dmats0[10][2] + coeff0_11*dmats0[11][2] + coeff0_12*dmats0[12][2] + coeff0_13*dmats0[13][2] + coeff0_14*dmats0[14][2] + coeff0_15*dmats0[15][2] + coeff0_16*dmats0[16][2] + coeff0_17*dmats0[17][2] + coeff0_18*dmats0[18][2] + coeff0_19*dmats0[19][2] + coeff0_20*dmats0[20][2] + coeff0_21*dmats0[21][2] + coeff0_22*dmats0[22][2] + coeff0_23*dmats0[23][2] + coeff0_24*dmats0[24][2] + coeff0_25*dmats0[25][2] + coeff0_26*dmats0[26][2] + coeff0_27*dmats0[27][2] + coeff0_28*dmats0[28][2] + coeff0_29*dmats0[29][2] + coeff0_30*dmats0[30][2] + coeff0_31*dmats0[31][2] + coeff0_32*dmats0[32][2] + coeff0_33*dmats0[33][2] + coeff0_34*dmats0[34][2] + coeff0_35*dmats0[35][2] + coeff0_36*dmats0[36][2] + coeff0_37*dmats0[37][2] + coeff0_38*dmats0[38][2] + coeff0_39*dmats0[39][2] + coeff0_40*dmats0[40][2] + coeff0_41*dmats0[41][2] + coeff0_42*dmats0[42][2] + coeff0_43*dmats0[43][2] + coeff0_44*dmats0[44][2] + coeff0_45*dmats0[45][2] + coeff0_46*dmats0[46][2] + coeff0_47*dmats0[47][2] + coeff0_48*dmats0[48][2] + coeff0_49*dmats0[49][2] + coeff0_50*dmats0[50][2] + coeff0_51*dmats0[51][2] + coeff0_52*dmats0[52][2] + coeff0_53*dmats0[53][2] + coeff0_54*dmats0[54][2] + coeff0_55*dmats0[55][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3] + coeff0_10*dmats0[10][3] + coeff0_11*dmats0[11][3] + coeff0_12*dmats0[12][3] + coeff0_13*dmats0[13][3] + coeff0_14*dmats0[14][3] + coeff0_15*dmats0[15][3] + coeff0_16*dmats0[16][3] + coeff0_17*dmats0[17][3] + coeff0_18*dmats0[18][3] + coeff0_19*dmats0[19][3] + coeff0_20*dmats0[20][3] + coeff0_21*dmats0[21][3] + coeff0_22*dmats0[22][3] + coeff0_23*dmats0[23][3] + coeff0_24*dmats0[24][3] + coeff0_25*dmats0[25][3] + coeff0_26*dmats0[26][3] + coeff0_27*dmats0[27][3] + coeff0_28*dmats0[28][3] + coeff0_29*dmats0[29][3] + coeff0_30*dmats0[30][3] + coeff0_31*dmats0[31][3] + coeff0_32*dmats0[32][3] + coeff0_33*dmats0[33][3] + coeff0_34*dmats0[34][3] + coeff0_35*dmats0[35][3] + coeff0_36*dmats0[36][3] + coeff0_37*dmats0[37][3] + coeff0_38*dmats0[38][3] + coeff0_39*dmats0[39][3] + coeff0_40*dmats0[40][3] + coeff0_41*dmats0[41][3] + coeff0_42*dmats0[42][3] + coeff0_43*dmats0[43][3] + coeff0_44*dmats0[44][3] + coeff0_45*dmats0[45][3] + coeff0_46*dmats0[46][3] + coeff0_47*dmats0[47][3] + coeff0_48*dmats0[48][3] + coeff0_49*dmats0[49][3] + coeff0_50*dmats0[50][3] + coeff0_51*dmats0[51][3] + coeff0_52*dmats0[52][3] + coeff0_53*dmats0[53][3] + coeff0_54*dmats0[54][3] + coeff0_55*dmats0[55][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4] + coeff0_10*dmats0[10][4] + coeff0_11*dmats0[11][4] + coeff0_12*dmats0[12][4] + coeff0_13*dmats0[13][4] + coeff0_14*dmats0[14][4] + coeff0_15*dmats0[15][4] + coeff0_16*dmats0[16][4] + coeff0_17*dmats0[17][4] + coeff0_18*dmats0[18][4] + coeff0_19*dmats0[19][4] + coeff0_20*dmats0[20][4] + coeff0_21*dmats0[21][4] + coeff0_22*dmats0[22][4] + coeff0_23*dmats0[23][4] + coeff0_24*dmats0[24][4] + coeff0_25*dmats0[25][4] + coeff0_26*dmats0[26][4] + coeff0_27*dmats0[27][4] + coeff0_28*dmats0[28][4] + coeff0_29*dmats0[29][4] + coeff0_30*dmats0[30][4] + coeff0_31*dmats0[31][4] + coeff0_32*dmats0[32][4] + coeff0_33*dmats0[33][4] + coeff0_34*dmats0[34][4] + coeff0_35*dmats0[35][4] + coeff0_36*dmats0[36][4] + coeff0_37*dmats0[37][4] + coeff0_38*dmats0[38][4] + coeff0_39*dmats0[39][4] + coeff0_40*dmats0[40][4] + coeff0_41*dmats0[41][4] + coeff0_42*dmats0[42][4] + coeff0_43*dmats0[43][4] + coeff0_44*dmats0[44][4] + coeff0_45*dmats0[45][4] + coeff0_46*dmats0[46][4] + coeff0_47*dmats0[47][4] + coeff0_48*dmats0[48][4] + coeff0_49*dmats0[49][4] + coeff0_50*dmats0[50][4] + coeff0_51*dmats0[51][4] + coeff0_52*dmats0[52][4] + coeff0_53*dmats0[53][4] + coeff0_54*dmats0[54][4] + coeff0_55*dmats0[55][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5] + coeff0_10*dmats0[10][5] + coeff0_11*dmats0[11][5] + coeff0_12*dmats0[12][5] + coeff0_13*dmats0[13][5] + coeff0_14*dmats0[14][5] + coeff0_15*dmats0[15][5] + coeff0_16*dmats0[16][5] + coeff0_17*dmats0[17][5] + coeff0_18*dmats0[18][5] + coeff0_19*dmats0[19][5] + coeff0_20*dmats0[20][5] + coeff0_21*dmats0[21][5] + coeff0_22*dmats0[22][5] + coeff0_23*dmats0[23][5] + coeff0_24*dmats0[24][5] + coeff0_25*dmats0[25][5] + coeff0_26*dmats0[26][5] + coeff0_27*dmats0[27][5] + coeff0_28*dmats0[28][5] + coeff0_29*dmats0[29][5] + coeff0_30*dmats0[30][5] + coeff0_31*dmats0[31][5] + coeff0_32*dmats0[32][5] + coeff0_33*dmats0[33][5] + coeff0_34*dmats0[34][5] + coeff0_35*dmats0[35][5] + coeff0_36*dmats0[36][5] + coeff0_37*dmats0[37][5] + coeff0_38*dmats0[38][5] + coeff0_39*dmats0[39][5] + coeff0_40*dmats0[40][5] + coeff0_41*dmats0[41][5] + coeff0_42*dmats0[42][5] + coeff0_43*dmats0[43][5] + coeff0_44*dmats0[44][5] + coeff0_45*dmats0[45][5] + coeff0_46*dmats0[46][5] + coeff0_47*dmats0[47][5] + coeff0_48*dmats0[48][5] + coeff0_49*dmats0[49][5] + coeff0_50*dmats0[50][5] + coeff0_51*dmats0[51][5] + coeff0_52*dmats0[52][5] + coeff0_53*dmats0[53][5] + coeff0_54*dmats0[54][5] + coeff0_55*dmats0[55][5];
          new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6] + coeff0_10*dmats0[10][6] + coeff0_11*dmats0[11][6] + coeff0_12*dmats0[12][6] + coeff0_13*dmats0[13][6] + coeff0_14*dmats0[14][6] + coeff0_15*dmats0[15][6] + coeff0_16*dmats0[16][6] + coeff0_17*dmats0[17][6] + coeff0_18*dmats0[18][6] + coeff0_19*dmats0[19][6] + coeff0_20*dmats0[20][6] + coeff0_21*dmats0[21][6] + coeff0_22*dmats0[22][6] + coeff0_23*dmats0[23][6] + coeff0_24*dmats0[24][6] + coeff0_25*dmats0[25][6] + coeff0_26*dmats0[26][6] + coeff0_27*dmats0[27][6] + coeff0_28*dmats0[28][6] + coeff0_29*dmats0[29][6] + coeff0_30*dmats0[30][6] + coeff0_31*dmats0[31][6] + coeff0_32*dmats0[32][6] + coeff0_33*dmats0[33][6] + coeff0_34*dmats0[34][6] + coeff0_35*dmats0[35][6] + coeff0_36*dmats0[36][6] + coeff0_37*dmats0[37][6] + coeff0_38*dmats0[38][6] + coeff0_39*dmats0[39][6] + coeff0_40*dmats0[40][6] + coeff0_41*dmats0[41][6] + coeff0_42*dmats0[42][6] + coeff0_43*dmats0[43][6] + coeff0_44*dmats0[44][6] + coeff0_45*dmats0[45][6] + coeff0_46*dmats0[46][6] + coeff0_47*dmats0[47][6] + coeff0_48*dmats0[48][6] + coeff0_49*dmats0[49][6] + coeff0_50*dmats0[50][6] + coeff0_51*dmats0[51][6] + coeff0_52*dmats0[52][6] + coeff0_53*dmats0[53][6] + coeff0_54*dmats0[54][6] + coeff0_55*dmats0[55][6];
          new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7] + coeff0_10*dmats0[10][7] + coeff0_11*dmats0[11][7] + coeff0_12*dmats0[12][7] + coeff0_13*dmats0[13][7] + coeff0_14*dmats0[14][7] + coeff0_15*dmats0[15][7] + coeff0_16*dmats0[16][7] + coeff0_17*dmats0[17][7] + coeff0_18*dmats0[18][7] + coeff0_19*dmats0[19][7] + coeff0_20*dmats0[20][7] + coeff0_21*dmats0[21][7] + coeff0_22*dmats0[22][7] + coeff0_23*dmats0[23][7] + coeff0_24*dmats0[24][7] + coeff0_25*dmats0[25][7] + coeff0_26*dmats0[26][7] + coeff0_27*dmats0[27][7] + coeff0_28*dmats0[28][7] + coeff0_29*dmats0[29][7] + coeff0_30*dmats0[30][7] + coeff0_31*dmats0[31][7] + coeff0_32*dmats0[32][7] + coeff0_33*dmats0[33][7] + coeff0_34*dmats0[34][7] + coeff0_35*dmats0[35][7] + coeff0_36*dmats0[36][7] + coeff0_37*dmats0[37][7] + coeff0_38*dmats0[38][7] + coeff0_39*dmats0[39][7] + coeff0_40*dmats0[40][7] + coeff0_41*dmats0[41][7] + coeff0_42*dmats0[42][7] + coeff0_43*dmats0[43][7] + coeff0_44*dmats0[44][7] + coeff0_45*dmats0[45][7] + coeff0_46*dmats0[46][7] + coeff0_47*dmats0[47][7] + coeff0_48*dmats0[48][7] + coeff0_49*dmats0[49][7] + coeff0_50*dmats0[50][7] + coeff0_51*dmats0[51][7] + coeff0_52*dmats0[52][7] + coeff0_53*dmats0[53][7] + coeff0_54*dmats0[54][7] + coeff0_55*dmats0[55][7];
          new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8] + coeff0_10*dmats0[10][8] + coeff0_11*dmats0[11][8] + coeff0_12*dmats0[12][8] + coeff0_13*dmats0[13][8] + coeff0_14*dmats0[14][8] + coeff0_15*dmats0[15][8] + coeff0_16*dmats0[16][8] + coeff0_17*dmats0[17][8] + coeff0_18*dmats0[18][8] + coeff0_19*dmats0[19][8] + coeff0_20*dmats0[20][8] + coeff0_21*dmats0[21][8] + coeff0_22*dmats0[22][8] + coeff0_23*dmats0[23][8] + coeff0_24*dmats0[24][8] + coeff0_25*dmats0[25][8] + coeff0_26*dmats0[26][8] + coeff0_27*dmats0[27][8] + coeff0_28*dmats0[28][8] + coeff0_29*dmats0[29][8] + coeff0_30*dmats0[30][8] + coeff0_31*dmats0[31][8] + coeff0_32*dmats0[32][8] + coeff0_33*dmats0[33][8] + coeff0_34*dmats0[34][8] + coeff0_35*dmats0[35][8] + coeff0_36*dmats0[36][8] + coeff0_37*dmats0[37][8] + coeff0_38*dmats0[38][8] + coeff0_39*dmats0[39][8] + coeff0_40*dmats0[40][8] + coeff0_41*dmats0[41][8] + coeff0_42*dmats0[42][8] + coeff0_43*dmats0[43][8] + coeff0_44*dmats0[44][8] + coeff0_45*dmats0[45][8] + coeff0_46*dmats0[46][8] + coeff0_47*dmats0[47][8] + coeff0_48*dmats0[48][8] + coeff0_49*dmats0[49][8] + coeff0_50*dmats0[50][8] + coeff0_51*dmats0[51][8] + coeff0_52*dmats0[52][8] + coeff0_53*dmats0[53][8] + coeff0_54*dmats0[54][8] + coeff0_55*dmats0[55][8];
          new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9] + coeff0_10*dmats0[10][9] + coeff0_11*dmats0[11][9] + coeff0_12*dmats0[12][9] + coeff0_13*dmats0[13][9] + coeff0_14*dmats0[14][9] + coeff0_15*dmats0[15][9] + coeff0_16*dmats0[16][9] + coeff0_17*dmats0[17][9] + coeff0_18*dmats0[18][9] + coeff0_19*dmats0[19][9] + coeff0_20*dmats0[20][9] + coeff0_21*dmats0[21][9] + coeff0_22*dmats0[22][9] + coeff0_23*dmats0[23][9] + coeff0_24*dmats0[24][9] + coeff0_25*dmats0[25][9] + coeff0_26*dmats0[26][9] + coeff0_27*dmats0[27][9] + coeff0_28*dmats0[28][9] + coeff0_29*dmats0[29][9] + coeff0_30*dmats0[30][9] + coeff0_31*dmats0[31][9] + coeff0_32*dmats0[32][9] + coeff0_33*dmats0[33][9] + coeff0_34*dmats0[34][9] + coeff0_35*dmats0[35][9] + coeff0_36*dmats0[36][9] + coeff0_37*dmats0[37][9] + coeff0_38*dmats0[38][9] + coeff0_39*dmats0[39][9] + coeff0_40*dmats0[40][9] + coeff0_41*dmats0[41][9] + coeff0_42*dmats0[42][9] + coeff0_43*dmats0[43][9] + coeff0_44*dmats0[44][9] + coeff0_45*dmats0[45][9] + coeff0_46*dmats0[46][9] + coeff0_47*dmats0[47][9] + coeff0_48*dmats0[48][9] + coeff0_49*dmats0[49][9] + coeff0_50*dmats0[50][9] + coeff0_51*dmats0[51][9] + coeff0_52*dmats0[52][9] + coeff0_53*dmats0[53][9] + coeff0_54*dmats0[54][9] + coeff0_55*dmats0[55][9];
          new_coeff0_10 = coeff0_0*dmats0[0][10] + coeff0_1*dmats0[1][10] + coeff0_2*dmats0[2][10] + coeff0_3*dmats0[3][10] + coeff0_4*dmats0[4][10] + coeff0_5*dmats0[5][10] + coeff0_6*dmats0[6][10] + coeff0_7*dmats0[7][10] + coeff0_8*dmats0[8][10] + coeff0_9*dmats0[9][10] + coeff0_10*dmats0[10][10] + coeff0_11*dmats0[11][10] + coeff0_12*dmats0[12][10] + coeff0_13*dmats0[13][10] + coeff0_14*dmats0[14][10] + coeff0_15*dmats0[15][10] + coeff0_16*dmats0[16][10] + coeff0_17*dmats0[17][10] + coeff0_18*dmats0[18][10] + coeff0_19*dmats0[19][10] + coeff0_20*dmats0[20][10] + coeff0_21*dmats0[21][10] + coeff0_22*dmats0[22][10] + coeff0_23*dmats0[23][10] + coeff0_24*dmats0[24][10] + coeff0_25*dmats0[25][10] + coeff0_26*dmats0[26][10] + coeff0_27*dmats0[27][10] + coeff0_28*dmats0[28][10] + coeff0_29*dmats0[29][10] + coeff0_30*dmats0[30][10] + coeff0_31*dmats0[31][10] + coeff0_32*dmats0[32][10] + coeff0_33*dmats0[33][10] + coeff0_34*dmats0[34][10] + coeff0_35*dmats0[35][10] + coeff0_36*dmats0[36][10] + coeff0_37*dmats0[37][10] + coeff0_38*dmats0[38][10] + coeff0_39*dmats0[39][10] + coeff0_40*dmats0[40][10] + coeff0_41*dmats0[41][10] + coeff0_42*dmats0[42][10] + coeff0_43*dmats0[43][10] + coeff0_44*dmats0[44][10] + coeff0_45*dmats0[45][10] + coeff0_46*dmats0[46][10] + coeff0_47*dmats0[47][10] + coeff0_48*dmats0[48][10] + coeff0_49*dmats0[49][10] + coeff0_50*dmats0[50][10] + coeff0_51*dmats0[51][10] + coeff0_52*dmats0[52][10] + coeff0_53*dmats0[53][10] + coeff0_54*dmats0[54][10] + coeff0_55*dmats0[55][10];
          new_coeff0_11 = coeff0_0*dmats0[0][11] + coeff0_1*dmats0[1][11] + coeff0_2*dmats0[2][11] + coeff0_3*dmats0[3][11] + coeff0_4*dmats0[4][11] + coeff0_5*dmats0[5][11] + coeff0_6*dmats0[6][11] + coeff0_7*dmats0[7][11] + coeff0_8*dmats0[8][11] + coeff0_9*dmats0[9][11] + coeff0_10*dmats0[10][11] + coeff0_11*dmats0[11][11] + coeff0_12*dmats0[12][11] + coeff0_13*dmats0[13][11] + coeff0_14*dmats0[14][11] + coeff0_15*dmats0[15][11] + coeff0_16*dmats0[16][11] + coeff0_17*dmats0[17][11] + coeff0_18*dmats0[18][11] + coeff0_19*dmats0[19][11] + coeff0_20*dmats0[20][11] + coeff0_21*dmats0[21][11] + coeff0_22*dmats0[22][11] + coeff0_23*dmats0[23][11] + coeff0_24*dmats0[24][11] + coeff0_25*dmats0[25][11] + coeff0_26*dmats0[26][11] + coeff0_27*dmats0[27][11] + coeff0_28*dmats0[28][11] + coeff0_29*dmats0[29][11] + coeff0_30*dmats0[30][11] + coeff0_31*dmats0[31][11] + coeff0_32*dmats0[32][11] + coeff0_33*dmats0[33][11] + coeff0_34*dmats0[34][11] + coeff0_35*dmats0[35][11] + coeff0_36*dmats0[36][11] + coeff0_37*dmats0[37][11] + coeff0_38*dmats0[38][11] + coeff0_39*dmats0[39][11] + coeff0_40*dmats0[40][11] + coeff0_41*dmats0[41][11] + coeff0_42*dmats0[42][11] + coeff0_43*dmats0[43][11] + coeff0_44*dmats0[44][11] + coeff0_45*dmats0[45][11] + coeff0_46*dmats0[46][11] + coeff0_47*dmats0[47][11] + coeff0_48*dmats0[48][11] + coeff0_49*dmats0[49][11] + coeff0_50*dmats0[50][11] + coeff0_51*dmats0[51][11] + coeff0_52*dmats0[52][11] + coeff0_53*dmats0[53][11] + coeff0_54*dmats0[54][11] + coeff0_55*dmats0[55][11];
          new_coeff0_12 = coeff0_0*dmats0[0][12] + coeff0_1*dmats0[1][12] + coeff0_2*dmats0[2][12] + coeff0_3*dmats0[3][12] + coeff0_4*dmats0[4][12] + coeff0_5*dmats0[5][12] + coeff0_6*dmats0[6][12] + coeff0_7*dmats0[7][12] + coeff0_8*dmats0[8][12] + coeff0_9*dmats0[9][12] + coeff0_10*dmats0[10][12] + coeff0_11*dmats0[11][12] + coeff0_12*dmats0[12][12] + coeff0_13*dmats0[13][12] + coeff0_14*dmats0[14][12] + coeff0_15*dmats0[15][12] + coeff0_16*dmats0[16][12] + coeff0_17*dmats0[17][12] + coeff0_18*dmats0[18][12] + coeff0_19*dmats0[19][12] + coeff0_20*dmats0[20][12] + coeff0_21*dmats0[21][12] + coeff0_22*dmats0[22][12] + coeff0_23*dmats0[23][12] + coeff0_24*dmats0[24][12] + coeff0_25*dmats0[25][12] + coeff0_26*dmats0[26][12] + coeff0_27*dmats0[27][12] + coeff0_28*dmats0[28][12] + coeff0_29*dmats0[29][12] + coeff0_30*dmats0[30][12] + coeff0_31*dmats0[31][12] + coeff0_32*dmats0[32][12] + coeff0_33*dmats0[33][12] + coeff0_34*dmats0[34][12] + coeff0_35*dmats0[35][12] + coeff0_36*dmats0[36][12] + coeff0_37*dmats0[37][12] + coeff0_38*dmats0[38][12] + coeff0_39*dmats0[39][12] + coeff0_40*dmats0[40][12] + coeff0_41*dmats0[41][12] + coeff0_42*dmats0[42][12] + coeff0_43*dmats0[43][12] + coeff0_44*dmats0[44][12] + coeff0_45*dmats0[45][12] + coeff0_46*dmats0[46][12] + coeff0_47*dmats0[47][12] + coeff0_48*dmats0[48][12] + coeff0_49*dmats0[49][12] + coeff0_50*dmats0[50][12] + coeff0_51*dmats0[51][12] + coeff0_52*dmats0[52][12] + coeff0_53*dmats0[53][12] + coeff0_54*dmats0[54][12] + coeff0_55*dmats0[55][12];
          new_coeff0_13 = coeff0_0*dmats0[0][13] + coeff0_1*dmats0[1][13] + coeff0_2*dmats0[2][13] + coeff0_3*dmats0[3][13] + coeff0_4*dmats0[4][13] + coeff0_5*dmats0[5][13] + coeff0_6*dmats0[6][13] + coeff0_7*dmats0[7][13] + coeff0_8*dmats0[8][13] + coeff0_9*dmats0[9][13] + coeff0_10*dmats0[10][13] + coeff0_11*dmats0[11][13] + coeff0_12*dmats0[12][13] + coeff0_13*dmats0[13][13] + coeff0_14*dmats0[14][13] + coeff0_15*dmats0[15][13] + coeff0_16*dmats0[16][13] + coeff0_17*dmats0[17][13] + coeff0_18*dmats0[18][13] + coeff0_19*dmats0[19][13] + coeff0_20*dmats0[20][13] + coeff0_21*dmats0[21][13] + coeff0_22*dmats0[22][13] + coeff0_23*dmats0[23][13] + coeff0_24*dmats0[24][13] + coeff0_25*dmats0[25][13] + coeff0_26*dmats0[26][13] + coeff0_27*dmats0[27][13] + coeff0_28*dmats0[28][13] + coeff0_29*dmats0[29][13] + coeff0_30*dmats0[30][13] + coeff0_31*dmats0[31][13] + coeff0_32*dmats0[32][13] + coeff0_33*dmats0[33][13] + coeff0_34*dmats0[34][13] + coeff0_35*dmats0[35][13] + coeff0_36*dmats0[36][13] + coeff0_37*dmats0[37][13] + coeff0_38*dmats0[38][13] + coeff0_39*dmats0[39][13] + coeff0_40*dmats0[40][13] + coeff0_41*dmats0[41][13] + coeff0_42*dmats0[42][13] + coeff0_43*dmats0[43][13] + coeff0_44*dmats0[44][13] + coeff0_45*dmats0[45][13] + coeff0_46*dmats0[46][13] + coeff0_47*dmats0[47][13] + coeff0_48*dmats0[48][13] + coeff0_49*dmats0[49][13] + coeff0_50*dmats0[50][13] + coeff0_51*dmats0[51][13] + coeff0_52*dmats0[52][13] + coeff0_53*dmats0[53][13] + coeff0_54*dmats0[54][13] + coeff0_55*dmats0[55][13];
          new_coeff0_14 = coeff0_0*dmats0[0][14] + coeff0_1*dmats0[1][14] + coeff0_2*dmats0[2][14] + coeff0_3*dmats0[3][14] + coeff0_4*dmats0[4][14] + coeff0_5*dmats0[5][14] + coeff0_6*dmats0[6][14] + coeff0_7*dmats0[7][14] + coeff0_8*dmats0[8][14] + coeff0_9*dmats0[9][14] + coeff0_10*dmats0[10][14] + coeff0_11*dmats0[11][14] + coeff0_12*dmats0[12][14] + coeff0_13*dmats0[13][14] + coeff0_14*dmats0[14][14] + coeff0_15*dmats0[15][14] + coeff0_16*dmats0[16][14] + coeff0_17*dmats0[17][14] + coeff0_18*dmats0[18][14] + coeff0_19*dmats0[19][14] + coeff0_20*dmats0[20][14] + coeff0_21*dmats0[21][14] + coeff0_22*dmats0[22][14] + coeff0_23*dmats0[23][14] + coeff0_24*dmats0[24][14] + coeff0_25*dmats0[25][14] + coeff0_26*dmats0[26][14] + coeff0_27*dmats0[27][14] + coeff0_28*dmats0[28][14] + coeff0_29*dmats0[29][14] + coeff0_30*dmats0[30][14] + coeff0_31*dmats0[31][14] + coeff0_32*dmats0[32][14] + coeff0_33*dmats0[33][14] + coeff0_34*dmats0[34][14] + coeff0_35*dmats0[35][14] + coeff0_36*dmats0[36][14] + coeff0_37*dmats0[37][14] + coeff0_38*dmats0[38][14] + coeff0_39*dmats0[39][14] + coeff0_40*dmats0[40][14] + coeff0_41*dmats0[41][14] + coeff0_42*dmats0[42][14] + coeff0_43*dmats0[43][14] + coeff0_44*dmats0[44][14] + coeff0_45*dmats0[45][14] + coeff0_46*dmats0[46][14] + coeff0_47*dmats0[47][14] + coeff0_48*dmats0[48][14] + coeff0_49*dmats0[49][14] + coeff0_50*dmats0[50][14] + coeff0_51*dmats0[51][14] + coeff0_52*dmats0[52][14] + coeff0_53*dmats0[53][14] + coeff0_54*dmats0[54][14] + coeff0_55*dmats0[55][14];
          new_coeff0_15 = coeff0_0*dmats0[0][15] + coeff0_1*dmats0[1][15] + coeff0_2*dmats0[2][15] + coeff0_3*dmats0[3][15] + coeff0_4*dmats0[4][15] + coeff0_5*dmats0[5][15] + coeff0_6*dmats0[6][15] + coeff0_7*dmats0[7][15] + coeff0_8*dmats0[8][15] + coeff0_9*dmats0[9][15] + coeff0_10*dmats0[10][15] + coeff0_11*dmats0[11][15] + coeff0_12*dmats0[12][15] + coeff0_13*dmats0[13][15] + coeff0_14*dmats0[14][15] + coeff0_15*dmats0[15][15] + coeff0_16*dmats0[16][15] + coeff0_17*dmats0[17][15] + coeff0_18*dmats0[18][15] + coeff0_19*dmats0[19][15] + coeff0_20*dmats0[20][15] + coeff0_21*dmats0[21][15] + coeff0_22*dmats0[22][15] + coeff0_23*dmats0[23][15] + coeff0_24*dmats0[24][15] + coeff0_25*dmats0[25][15] + coeff0_26*dmats0[26][15] + coeff0_27*dmats0[27][15] + coeff0_28*dmats0[28][15] + coeff0_29*dmats0[29][15] + coeff0_30*dmats0[30][15] + coeff0_31*dmats0[31][15] + coeff0_32*dmats0[32][15] + coeff0_33*dmats0[33][15] + coeff0_34*dmats0[34][15] + coeff0_35*dmats0[35][15] + coeff0_36*dmats0[36][15] + coeff0_37*dmats0[37][15] + coeff0_38*dmats0[38][15] + coeff0_39*dmats0[39][15] + coeff0_40*dmats0[40][15] + coeff0_41*dmats0[41][15] + coeff0_42*dmats0[42][15] + coeff0_43*dmats0[43][15] + coeff0_44*dmats0[44][15] + coeff0_45*dmats0[45][15] + coeff0_46*dmats0[46][15] + coeff0_47*dmats0[47][15] + coeff0_48*dmats0[48][15] + coeff0_49*dmats0[49][15] + coeff0_50*dmats0[50][15] + coeff0_51*dmats0[51][15] + coeff0_52*dmats0[52][15] + coeff0_53*dmats0[53][15] + coeff0_54*dmats0[54][15] + coeff0_55*dmats0[55][15];
          new_coeff0_16 = coeff0_0*dmats0[0][16] + coeff0_1*dmats0[1][16] + coeff0_2*dmats0[2][16] + coeff0_3*dmats0[3][16] + coeff0_4*dmats0[4][16] + coeff0_5*dmats0[5][16] + coeff0_6*dmats0[6][16] + coeff0_7*dmats0[7][16] + coeff0_8*dmats0[8][16] + coeff0_9*dmats0[9][16] + coeff0_10*dmats0[10][16] + coeff0_11*dmats0[11][16] + coeff0_12*dmats0[12][16] + coeff0_13*dmats0[13][16] + coeff0_14*dmats0[14][16] + coeff0_15*dmats0[15][16] + coeff0_16*dmats0[16][16] + coeff0_17*dmats0[17][16] + coeff0_18*dmats0[18][16] + coeff0_19*dmats0[19][16] + coeff0_20*dmats0[20][16] + coeff0_21*dmats0[21][16] + coeff0_22*dmats0[22][16] + coeff0_23*dmats0[23][16] + coeff0_24*dmats0[24][16] + coeff0_25*dmats0[25][16] + coeff0_26*dmats0[26][16] + coeff0_27*dmats0[27][16] + coeff0_28*dmats0[28][16] + coeff0_29*dmats0[29][16] + coeff0_30*dmats0[30][16] + coeff0_31*dmats0[31][16] + coeff0_32*dmats0[32][16] + coeff0_33*dmats0[33][16] + coeff0_34*dmats0[34][16] + coeff0_35*dmats0[35][16] + coeff0_36*dmats0[36][16] + coeff0_37*dmats0[37][16] + coeff0_38*dmats0[38][16] + coeff0_39*dmats0[39][16] + coeff0_40*dmats0[40][16] + coeff0_41*dmats0[41][16] + coeff0_42*dmats0[42][16] + coeff0_43*dmats0[43][16] + coeff0_44*dmats0[44][16] + coeff0_45*dmats0[45][16] + coeff0_46*dmats0[46][16] + coeff0_47*dmats0[47][16] + coeff0_48*dmats0[48][16] + coeff0_49*dmats0[49][16] + coeff0_50*dmats0[50][16] + coeff0_51*dmats0[51][16] + coeff0_52*dmats0[52][16] + coeff0_53*dmats0[53][16] + coeff0_54*dmats0[54][16] + coeff0_55*dmats0[55][16];
          new_coeff0_17 = coeff0_0*dmats0[0][17] + coeff0_1*dmats0[1][17] + coeff0_2*dmats0[2][17] + coeff0_3*dmats0[3][17] + coeff0_4*dmats0[4][17] + coeff0_5*dmats0[5][17] + coeff0_6*dmats0[6][17] + coeff0_7*dmats0[7][17] + coeff0_8*dmats0[8][17] + coeff0_9*dmats0[9][17] + coeff0_10*dmats0[10][17] + coeff0_11*dmats0[11][17] + coeff0_12*dmats0[12][17] + coeff0_13*dmats0[13][17] + coeff0_14*dmats0[14][17] + coeff0_15*dmats0[15][17] + coeff0_16*dmats0[16][17] + coeff0_17*dmats0[17][17] + coeff0_18*dmats0[18][17] + coeff0_19*dmats0[19][17] + coeff0_20*dmats0[20][17] + coeff0_21*dmats0[21][17] + coeff0_22*dmats0[22][17] + coeff0_23*dmats0[23][17] + coeff0_24*dmats0[24][17] + coeff0_25*dmats0[25][17] + coeff0_26*dmats0[26][17] + coeff0_27*dmats0[27][17] + coeff0_28*dmats0[28][17] + coeff0_29*dmats0[29][17] + coeff0_30*dmats0[30][17] + coeff0_31*dmats0[31][17] + coeff0_32*dmats0[32][17] + coeff0_33*dmats0[33][17] + coeff0_34*dmats0[34][17] + coeff0_35*dmats0[35][17] + coeff0_36*dmats0[36][17] + coeff0_37*dmats0[37][17] + coeff0_38*dmats0[38][17] + coeff0_39*dmats0[39][17] + coeff0_40*dmats0[40][17] + coeff0_41*dmats0[41][17] + coeff0_42*dmats0[42][17] + coeff0_43*dmats0[43][17] + coeff0_44*dmats0[44][17] + coeff0_45*dmats0[45][17] + coeff0_46*dmats0[46][17] + coeff0_47*dmats0[47][17] + coeff0_48*dmats0[48][17] + coeff0_49*dmats0[49][17] + coeff0_50*dmats0[50][17] + coeff0_51*dmats0[51][17] + coeff0_52*dmats0[52][17] + coeff0_53*dmats0[53][17] + coeff0_54*dmats0[54][17] + coeff0_55*dmats0[55][17];
          new_coeff0_18 = coeff0_0*dmats0[0][18] + coeff0_1*dmats0[1][18] + coeff0_2*dmats0[2][18] + coeff0_3*dmats0[3][18] + coeff0_4*dmats0[4][18] + coeff0_5*dmats0[5][18] + coeff0_6*dmats0[6][18] + coeff0_7*dmats0[7][18] + coeff0_8*dmats0[8][18] + coeff0_9*dmats0[9][18] + coeff0_10*dmats0[10][18] + coeff0_11*dmats0[11][18] + coeff0_12*dmats0[12][18] + coeff0_13*dmats0[13][18] + coeff0_14*dmats0[14][18] + coeff0_15*dmats0[15][18] + coeff0_16*dmats0[16][18] + coeff0_17*dmats0[17][18] + coeff0_18*dmats0[18][18] + coeff0_19*dmats0[19][18] + coeff0_20*dmats0[20][18] + coeff0_21*dmats0[21][18] + coeff0_22*dmats0[22][18] + coeff0_23*dmats0[23][18] + coeff0_24*dmats0[24][18] + coeff0_25*dmats0[25][18] + coeff0_26*dmats0[26][18] + coeff0_27*dmats0[27][18] + coeff0_28*dmats0[28][18] + coeff0_29*dmats0[29][18] + coeff0_30*dmats0[30][18] + coeff0_31*dmats0[31][18] + coeff0_32*dmats0[32][18] + coeff0_33*dmats0[33][18] + coeff0_34*dmats0[34][18] + coeff0_35*dmats0[35][18] + coeff0_36*dmats0[36][18] + coeff0_37*dmats0[37][18] + coeff0_38*dmats0[38][18] + coeff0_39*dmats0[39][18] + coeff0_40*dmats0[40][18] + coeff0_41*dmats0[41][18] + coeff0_42*dmats0[42][18] + coeff0_43*dmats0[43][18] + coeff0_44*dmats0[44][18] + coeff0_45*dmats0[45][18] + coeff0_46*dmats0[46][18] + coeff0_47*dmats0[47][18] + coeff0_48*dmats0[48][18] + coeff0_49*dmats0[49][18] + coeff0_50*dmats0[50][18] + coeff0_51*dmats0[51][18] + coeff0_52*dmats0[52][18] + coeff0_53*dmats0[53][18] + coeff0_54*dmats0[54][18] + coeff0_55*dmats0[55][18];
          new_coeff0_19 = coeff0_0*dmats0[0][19] + coeff0_1*dmats0[1][19] + coeff0_2*dmats0[2][19] + coeff0_3*dmats0[3][19] + coeff0_4*dmats0[4][19] + coeff0_5*dmats0[5][19] + coeff0_6*dmats0[6][19] + coeff0_7*dmats0[7][19] + coeff0_8*dmats0[8][19] + coeff0_9*dmats0[9][19] + coeff0_10*dmats0[10][19] + coeff0_11*dmats0[11][19] + coeff0_12*dmats0[12][19] + coeff0_13*dmats0[13][19] + coeff0_14*dmats0[14][19] + coeff0_15*dmats0[15][19] + coeff0_16*dmats0[16][19] + coeff0_17*dmats0[17][19] + coeff0_18*dmats0[18][19] + coeff0_19*dmats0[19][19] + coeff0_20*dmats0[20][19] + coeff0_21*dmats0[21][19] + coeff0_22*dmats0[22][19] + coeff0_23*dmats0[23][19] + coeff0_24*dmats0[24][19] + coeff0_25*dmats0[25][19] + coeff0_26*dmats0[26][19] + coeff0_27*dmats0[27][19] + coeff0_28*dmats0[28][19] + coeff0_29*dmats0[29][19] + coeff0_30*dmats0[30][19] + coeff0_31*dmats0[31][19] + coeff0_32*dmats0[32][19] + coeff0_33*dmats0[33][19] + coeff0_34*dmats0[34][19] + coeff0_35*dmats0[35][19] + coeff0_36*dmats0[36][19] + coeff0_37*dmats0[37][19] + coeff0_38*dmats0[38][19] + coeff0_39*dmats0[39][19] + coeff0_40*dmats0[40][19] + coeff0_41*dmats0[41][19] + coeff0_42*dmats0[42][19] + coeff0_43*dmats0[43][19] + coeff0_44*dmats0[44][19] + coeff0_45*dmats0[45][19] + coeff0_46*dmats0[46][19] + coeff0_47*dmats0[47][19] + coeff0_48*dmats0[48][19] + coeff0_49*dmats0[49][19] + coeff0_50*dmats0[50][19] + coeff0_51*dmats0[51][19] + coeff0_52*dmats0[52][19] + coeff0_53*dmats0[53][19] + coeff0_54*dmats0[54][19] + coeff0_55*dmats0[55][19];
          new_coeff0_20 = coeff0_0*dmats0[0][20] + coeff0_1*dmats0[1][20] + coeff0_2*dmats0[2][20] + coeff0_3*dmats0[3][20] + coeff0_4*dmats0[4][20] + coeff0_5*dmats0[5][20] + coeff0_6*dmats0[6][20] + coeff0_7*dmats0[7][20] + coeff0_8*dmats0[8][20] + coeff0_9*dmats0[9][20] + coeff0_10*dmats0[10][20] + coeff0_11*dmats0[11][20] + coeff0_12*dmats0[12][20] + coeff0_13*dmats0[13][20] + coeff0_14*dmats0[14][20] + coeff0_15*dmats0[15][20] + coeff0_16*dmats0[16][20] + coeff0_17*dmats0[17][20] + coeff0_18*dmats0[18][20] + coeff0_19*dmats0[19][20] + coeff0_20*dmats0[20][20] + coeff0_21*dmats0[21][20] + coeff0_22*dmats0[22][20] + coeff0_23*dmats0[23][20] + coeff0_24*dmats0[24][20] + coeff0_25*dmats0[25][20] + coeff0_26*dmats0[26][20] + coeff0_27*dmats0[27][20] + coeff0_28*dmats0[28][20] + coeff0_29*dmats0[29][20] + coeff0_30*dmats0[30][20] + coeff0_31*dmats0[31][20] + coeff0_32*dmats0[32][20] + coeff0_33*dmats0[33][20] + coeff0_34*dmats0[34][20] + coeff0_35*dmats0[35][20] + coeff0_36*dmats0[36][20] + coeff0_37*dmats0[37][20] + coeff0_38*dmats0[38][20] + coeff0_39*dmats0[39][20] + coeff0_40*dmats0[40][20] + coeff0_41*dmats0[41][20] + coeff0_42*dmats0[42][20] + coeff0_43*dmats0[43][20] + coeff0_44*dmats0[44][20] + coeff0_45*dmats0[45][20] + coeff0_46*dmats0[46][20] + coeff0_47*dmats0[47][20] + coeff0_48*dmats0[48][20] + coeff0_49*dmats0[49][20] + coeff0_50*dmats0[50][20] + coeff0_51*dmats0[51][20] + coeff0_52*dmats0[52][20] + coeff0_53*dmats0[53][20] + coeff0_54*dmats0[54][20] + coeff0_55*dmats0[55][20];
          new_coeff0_21 = coeff0_0*dmats0[0][21] + coeff0_1*dmats0[1][21] + coeff0_2*dmats0[2][21] + coeff0_3*dmats0[3][21] + coeff0_4*dmats0[4][21] + coeff0_5*dmats0[5][21] + coeff0_6*dmats0[6][21] + coeff0_7*dmats0[7][21] + coeff0_8*dmats0[8][21] + coeff0_9*dmats0[9][21] + coeff0_10*dmats0[10][21] + coeff0_11*dmats0[11][21] + coeff0_12*dmats0[12][21] + coeff0_13*dmats0[13][21] + coeff0_14*dmats0[14][21] + coeff0_15*dmats0[15][21] + coeff0_16*dmats0[16][21] + coeff0_17*dmats0[17][21] + coeff0_18*dmats0[18][21] + coeff0_19*dmats0[19][21] + coeff0_20*dmats0[20][21] + coeff0_21*dmats0[21][21] + coeff0_22*dmats0[22][21] + coeff0_23*dmats0[23][21] + coeff0_24*dmats0[24][21] + coeff0_25*dmats0[25][21] + coeff0_26*dmats0[26][21] + coeff0_27*dmats0[27][21] + coeff0_28*dmats0[28][21] + coeff0_29*dmats0[29][21] + coeff0_30*dmats0[30][21] + coeff0_31*dmats0[31][21] + coeff0_32*dmats0[32][21] + coeff0_33*dmats0[33][21] + coeff0_34*dmats0[34][21] + coeff0_35*dmats0[35][21] + coeff0_36*dmats0[36][21] + coeff0_37*dmats0[37][21] + coeff0_38*dmats0[38][21] + coeff0_39*dmats0[39][21] + coeff0_40*dmats0[40][21] + coeff0_41*dmats0[41][21] + coeff0_42*dmats0[42][21] + coeff0_43*dmats0[43][21] + coeff0_44*dmats0[44][21] + coeff0_45*dmats0[45][21] + coeff0_46*dmats0[46][21] + coeff0_47*dmats0[47][21] + coeff0_48*dmats0[48][21] + coeff0_49*dmats0[49][21] + coeff0_50*dmats0[50][21] + coeff0_51*dmats0[51][21] + coeff0_52*dmats0[52][21] + coeff0_53*dmats0[53][21] + coeff0_54*dmats0[54][21] + coeff0_55*dmats0[55][21];
          new_coeff0_22 = coeff0_0*dmats0[0][22] + coeff0_1*dmats0[1][22] + coeff0_2*dmats0[2][22] + coeff0_3*dmats0[3][22] + coeff0_4*dmats0[4][22] + coeff0_5*dmats0[5][22] + coeff0_6*dmats0[6][22] + coeff0_7*dmats0[7][22] + coeff0_8*dmats0[8][22] + coeff0_9*dmats0[9][22] + coeff0_10*dmats0[10][22] + coeff0_11*dmats0[11][22] + coeff0_12*dmats0[12][22] + coeff0_13*dmats0[13][22] + coeff0_14*dmats0[14][22] + coeff0_15*dmats0[15][22] + coeff0_16*dmats0[16][22] + coeff0_17*dmats0[17][22] + coeff0_18*dmats0[18][22] + coeff0_19*dmats0[19][22] + coeff0_20*dmats0[20][22] + coeff0_21*dmats0[21][22] + coeff0_22*dmats0[22][22] + coeff0_23*dmats0[23][22] + coeff0_24*dmats0[24][22] + coeff0_25*dmats0[25][22] + coeff0_26*dmats0[26][22] + coeff0_27*dmats0[27][22] + coeff0_28*dmats0[28][22] + coeff0_29*dmats0[29][22] + coeff0_30*dmats0[30][22] + coeff0_31*dmats0[31][22] + coeff0_32*dmats0[32][22] + coeff0_33*dmats0[33][22] + coeff0_34*dmats0[34][22] + coeff0_35*dmats0[35][22] + coeff0_36*dmats0[36][22] + coeff0_37*dmats0[37][22] + coeff0_38*dmats0[38][22] + coeff0_39*dmats0[39][22] + coeff0_40*dmats0[40][22] + coeff0_41*dmats0[41][22] + coeff0_42*dmats0[42][22] + coeff0_43*dmats0[43][22] + coeff0_44*dmats0[44][22] + coeff0_45*dmats0[45][22] + coeff0_46*dmats0[46][22] + coeff0_47*dmats0[47][22] + coeff0_48*dmats0[48][22] + coeff0_49*dmats0[49][22] + coeff0_50*dmats0[50][22] + coeff0_51*dmats0[51][22] + coeff0_52*dmats0[52][22] + coeff0_53*dmats0[53][22] + coeff0_54*dmats0[54][22] + coeff0_55*dmats0[55][22];
          new_coeff0_23 = coeff0_0*dmats0[0][23] + coeff0_1*dmats0[1][23] + coeff0_2*dmats0[2][23] + coeff0_3*dmats0[3][23] + coeff0_4*dmats0[4][23] + coeff0_5*dmats0[5][23] + coeff0_6*dmats0[6][23] + coeff0_7*dmats0[7][23] + coeff0_8*dmats0[8][23] + coeff0_9*dmats0[9][23] + coeff0_10*dmats0[10][23] + coeff0_11*dmats0[11][23] + coeff0_12*dmats0[12][23] + coeff0_13*dmats0[13][23] + coeff0_14*dmats0[14][23] + coeff0_15*dmats0[15][23] + coeff0_16*dmats0[16][23] + coeff0_17*dmats0[17][23] + coeff0_18*dmats0[18][23] + coeff0_19*dmats0[19][23] + coeff0_20*dmats0[20][23] + coeff0_21*dmats0[21][23] + coeff0_22*dmats0[22][23] + coeff0_23*dmats0[23][23] + coeff0_24*dmats0[24][23] + coeff0_25*dmats0[25][23] + coeff0_26*dmats0[26][23] + coeff0_27*dmats0[27][23] + coeff0_28*dmats0[28][23] + coeff0_29*dmats0[29][23] + coeff0_30*dmats0[30][23] + coeff0_31*dmats0[31][23] + coeff0_32*dmats0[32][23] + coeff0_33*dmats0[33][23] + coeff0_34*dmats0[34][23] + coeff0_35*dmats0[35][23] + coeff0_36*dmats0[36][23] + coeff0_37*dmats0[37][23] + coeff0_38*dmats0[38][23] + coeff0_39*dmats0[39][23] + coeff0_40*dmats0[40][23] + coeff0_41*dmats0[41][23] + coeff0_42*dmats0[42][23] + coeff0_43*dmats0[43][23] + coeff0_44*dmats0[44][23] + coeff0_45*dmats0[45][23] + coeff0_46*dmats0[46][23] + coeff0_47*dmats0[47][23] + coeff0_48*dmats0[48][23] + coeff0_49*dmats0[49][23] + coeff0_50*dmats0[50][23] + coeff0_51*dmats0[51][23] + coeff0_52*dmats0[52][23] + coeff0_53*dmats0[53][23] + coeff0_54*dmats0[54][23] + coeff0_55*dmats0[55][23];
          new_coeff0_24 = coeff0_0*dmats0[0][24] + coeff0_1*dmats0[1][24] + coeff0_2*dmats0[2][24] + coeff0_3*dmats0[3][24] + coeff0_4*dmats0[4][24] + coeff0_5*dmats0[5][24] + coeff0_6*dmats0[6][24] + coeff0_7*dmats0[7][24] + coeff0_8*dmats0[8][24] + coeff0_9*dmats0[9][24] + coeff0_10*dmats0[10][24] + coeff0_11*dmats0[11][24] + coeff0_12*dmats0[12][24] + coeff0_13*dmats0[13][24] + coeff0_14*dmats0[14][24] + coeff0_15*dmats0[15][24] + coeff0_16*dmats0[16][24] + coeff0_17*dmats0[17][24] + coeff0_18*dmats0[18][24] + coeff0_19*dmats0[19][24] + coeff0_20*dmats0[20][24] + coeff0_21*dmats0[21][24] + coeff0_22*dmats0[22][24] + coeff0_23*dmats0[23][24] + coeff0_24*dmats0[24][24] + coeff0_25*dmats0[25][24] + coeff0_26*dmats0[26][24] + coeff0_27*dmats0[27][24] + coeff0_28*dmats0[28][24] + coeff0_29*dmats0[29][24] + coeff0_30*dmats0[30][24] + coeff0_31*dmats0[31][24] + coeff0_32*dmats0[32][24] + coeff0_33*dmats0[33][24] + coeff0_34*dmats0[34][24] + coeff0_35*dmats0[35][24] + coeff0_36*dmats0[36][24] + coeff0_37*dmats0[37][24] + coeff0_38*dmats0[38][24] + coeff0_39*dmats0[39][24] + coeff0_40*dmats0[40][24] + coeff0_41*dmats0[41][24] + coeff0_42*dmats0[42][24] + coeff0_43*dmats0[43][24] + coeff0_44*dmats0[44][24] + coeff0_45*dmats0[45][24] + coeff0_46*dmats0[46][24] + coeff0_47*dmats0[47][24] + coeff0_48*dmats0[48][24] + coeff0_49*dmats0[49][24] + coeff0_50*dmats0[50][24] + coeff0_51*dmats0[51][24] + coeff0_52*dmats0[52][24] + coeff0_53*dmats0[53][24] + coeff0_54*dmats0[54][24] + coeff0_55*dmats0[55][24];
          new_coeff0_25 = coeff0_0*dmats0[0][25] + coeff0_1*dmats0[1][25] + coeff0_2*dmats0[2][25] + coeff0_3*dmats0[3][25] + coeff0_4*dmats0[4][25] + coeff0_5*dmats0[5][25] + coeff0_6*dmats0[6][25] + coeff0_7*dmats0[7][25] + coeff0_8*dmats0[8][25] + coeff0_9*dmats0[9][25] + coeff0_10*dmats0[10][25] + coeff0_11*dmats0[11][25] + coeff0_12*dmats0[12][25] + coeff0_13*dmats0[13][25] + coeff0_14*dmats0[14][25] + coeff0_15*dmats0[15][25] + coeff0_16*dmats0[16][25] + coeff0_17*dmats0[17][25] + coeff0_18*dmats0[18][25] + coeff0_19*dmats0[19][25] + coeff0_20*dmats0[20][25] + coeff0_21*dmats0[21][25] + coeff0_22*dmats0[22][25] + coeff0_23*dmats0[23][25] + coeff0_24*dmats0[24][25] + coeff0_25*dmats0[25][25] + coeff0_26*dmats0[26][25] + coeff0_27*dmats0[27][25] + coeff0_28*dmats0[28][25] + coeff0_29*dmats0[29][25] + coeff0_30*dmats0[30][25] + coeff0_31*dmats0[31][25] + coeff0_32*dmats0[32][25] + coeff0_33*dmats0[33][25] + coeff0_34*dmats0[34][25] + coeff0_35*dmats0[35][25] + coeff0_36*dmats0[36][25] + coeff0_37*dmats0[37][25] + coeff0_38*dmats0[38][25] + coeff0_39*dmats0[39][25] + coeff0_40*dmats0[40][25] + coeff0_41*dmats0[41][25] + coeff0_42*dmats0[42][25] + coeff0_43*dmats0[43][25] + coeff0_44*dmats0[44][25] + coeff0_45*dmats0[45][25] + coeff0_46*dmats0[46][25] + coeff0_47*dmats0[47][25] + coeff0_48*dmats0[48][25] + coeff0_49*dmats0[49][25] + coeff0_50*dmats0[50][25] + coeff0_51*dmats0[51][25] + coeff0_52*dmats0[52][25] + coeff0_53*dmats0[53][25] + coeff0_54*dmats0[54][25] + coeff0_55*dmats0[55][25];
          new_coeff0_26 = coeff0_0*dmats0[0][26] + coeff0_1*dmats0[1][26] + coeff0_2*dmats0[2][26] + coeff0_3*dmats0[3][26] + coeff0_4*dmats0[4][26] + coeff0_5*dmats0[5][26] + coeff0_6*dmats0[6][26] + coeff0_7*dmats0[7][26] + coeff0_8*dmats0[8][26] + coeff0_9*dmats0[9][26] + coeff0_10*dmats0[10][26] + coeff0_11*dmats0[11][26] + coeff0_12*dmats0[12][26] + coeff0_13*dmats0[13][26] + coeff0_14*dmats0[14][26] + coeff0_15*dmats0[15][26] + coeff0_16*dmats0[16][26] + coeff0_17*dmats0[17][26] + coeff0_18*dmats0[18][26] + coeff0_19*dmats0[19][26] + coeff0_20*dmats0[20][26] + coeff0_21*dmats0[21][26] + coeff0_22*dmats0[22][26] + coeff0_23*dmats0[23][26] + coeff0_24*dmats0[24][26] + coeff0_25*dmats0[25][26] + coeff0_26*dmats0[26][26] + coeff0_27*dmats0[27][26] + coeff0_28*dmats0[28][26] + coeff0_29*dmats0[29][26] + coeff0_30*dmats0[30][26] + coeff0_31*dmats0[31][26] + coeff0_32*dmats0[32][26] + coeff0_33*dmats0[33][26] + coeff0_34*dmats0[34][26] + coeff0_35*dmats0[35][26] + coeff0_36*dmats0[36][26] + coeff0_37*dmats0[37][26] + coeff0_38*dmats0[38][26] + coeff0_39*dmats0[39][26] + coeff0_40*dmats0[40][26] + coeff0_41*dmats0[41][26] + coeff0_42*dmats0[42][26] + coeff0_43*dmats0[43][26] + coeff0_44*dmats0[44][26] + coeff0_45*dmats0[45][26] + coeff0_46*dmats0[46][26] + coeff0_47*dmats0[47][26] + coeff0_48*dmats0[48][26] + coeff0_49*dmats0[49][26] + coeff0_50*dmats0[50][26] + coeff0_51*dmats0[51][26] + coeff0_52*dmats0[52][26] + coeff0_53*dmats0[53][26] + coeff0_54*dmats0[54][26] + coeff0_55*dmats0[55][26];
          new_coeff0_27 = coeff0_0*dmats0[0][27] + coeff0_1*dmats0[1][27] + coeff0_2*dmats0[2][27] + coeff0_3*dmats0[3][27] + coeff0_4*dmats0[4][27] + coeff0_5*dmats0[5][27] + coeff0_6*dmats0[6][27] + coeff0_7*dmats0[7][27] + coeff0_8*dmats0[8][27] + coeff0_9*dmats0[9][27] + coeff0_10*dmats0[10][27] + coeff0_11*dmats0[11][27] + coeff0_12*dmats0[12][27] + coeff0_13*dmats0[13][27] + coeff0_14*dmats0[14][27] + coeff0_15*dmats0[15][27] + coeff0_16*dmats0[16][27] + coeff0_17*dmats0[17][27] + coeff0_18*dmats0[18][27] + coeff0_19*dmats0[19][27] + coeff0_20*dmats0[20][27] + coeff0_21*dmats0[21][27] + coeff0_22*dmats0[22][27] + coeff0_23*dmats0[23][27] + coeff0_24*dmats0[24][27] + coeff0_25*dmats0[25][27] + coeff0_26*dmats0[26][27] + coeff0_27*dmats0[27][27] + coeff0_28*dmats0[28][27] + coeff0_29*dmats0[29][27] + coeff0_30*dmats0[30][27] + coeff0_31*dmats0[31][27] + coeff0_32*dmats0[32][27] + coeff0_33*dmats0[33][27] + coeff0_34*dmats0[34][27] + coeff0_35*dmats0[35][27] + coeff0_36*dmats0[36][27] + coeff0_37*dmats0[37][27] + coeff0_38*dmats0[38][27] + coeff0_39*dmats0[39][27] + coeff0_40*dmats0[40][27] + coeff0_41*dmats0[41][27] + coeff0_42*dmats0[42][27] + coeff0_43*dmats0[43][27] + coeff0_44*dmats0[44][27] + coeff0_45*dmats0[45][27] + coeff0_46*dmats0[46][27] + coeff0_47*dmats0[47][27] + coeff0_48*dmats0[48][27] + coeff0_49*dmats0[49][27] + coeff0_50*dmats0[50][27] + coeff0_51*dmats0[51][27] + coeff0_52*dmats0[52][27] + coeff0_53*dmats0[53][27] + coeff0_54*dmats0[54][27] + coeff0_55*dmats0[55][27];
          new_coeff0_28 = coeff0_0*dmats0[0][28] + coeff0_1*dmats0[1][28] + coeff0_2*dmats0[2][28] + coeff0_3*dmats0[3][28] + coeff0_4*dmats0[4][28] + coeff0_5*dmats0[5][28] + coeff0_6*dmats0[6][28] + coeff0_7*dmats0[7][28] + coeff0_8*dmats0[8][28] + coeff0_9*dmats0[9][28] + coeff0_10*dmats0[10][28] + coeff0_11*dmats0[11][28] + coeff0_12*dmats0[12][28] + coeff0_13*dmats0[13][28] + coeff0_14*dmats0[14][28] + coeff0_15*dmats0[15][28] + coeff0_16*dmats0[16][28] + coeff0_17*dmats0[17][28] + coeff0_18*dmats0[18][28] + coeff0_19*dmats0[19][28] + coeff0_20*dmats0[20][28] + coeff0_21*dmats0[21][28] + coeff0_22*dmats0[22][28] + coeff0_23*dmats0[23][28] + coeff0_24*dmats0[24][28] + coeff0_25*dmats0[25][28] + coeff0_26*dmats0[26][28] + coeff0_27*dmats0[27][28] + coeff0_28*dmats0[28][28] + coeff0_29*dmats0[29][28] + coeff0_30*dmats0[30][28] + coeff0_31*dmats0[31][28] + coeff0_32*dmats0[32][28] + coeff0_33*dmats0[33][28] + coeff0_34*dmats0[34][28] + coeff0_35*dmats0[35][28] + coeff0_36*dmats0[36][28] + coeff0_37*dmats0[37][28] + coeff0_38*dmats0[38][28] + coeff0_39*dmats0[39][28] + coeff0_40*dmats0[40][28] + coeff0_41*dmats0[41][28] + coeff0_42*dmats0[42][28] + coeff0_43*dmats0[43][28] + coeff0_44*dmats0[44][28] + coeff0_45*dmats0[45][28] + coeff0_46*dmats0[46][28] + coeff0_47*dmats0[47][28] + coeff0_48*dmats0[48][28] + coeff0_49*dmats0[49][28] + coeff0_50*dmats0[50][28] + coeff0_51*dmats0[51][28] + coeff0_52*dmats0[52][28] + coeff0_53*dmats0[53][28] + coeff0_54*dmats0[54][28] + coeff0_55*dmats0[55][28];
          new_coeff0_29 = coeff0_0*dmats0[0][29] + coeff0_1*dmats0[1][29] + coeff0_2*dmats0[2][29] + coeff0_3*dmats0[3][29] + coeff0_4*dmats0[4][29] + coeff0_5*dmats0[5][29] + coeff0_6*dmats0[6][29] + coeff0_7*dmats0[7][29] + coeff0_8*dmats0[8][29] + coeff0_9*dmats0[9][29] + coeff0_10*dmats0[10][29] + coeff0_11*dmats0[11][29] + coeff0_12*dmats0[12][29] + coeff0_13*dmats0[13][29] + coeff0_14*dmats0[14][29] + coeff0_15*dmats0[15][29] + coeff0_16*dmats0[16][29] + coeff0_17*dmats0[17][29] + coeff0_18*dmats0[18][29] + coeff0_19*dmats0[19][29] + coeff0_20*dmats0[20][29] + coeff0_21*dmats0[21][29] + coeff0_22*dmats0[22][29] + coeff0_23*dmats0[23][29] + coeff0_24*dmats0[24][29] + coeff0_25*dmats0[25][29] + coeff0_26*dmats0[26][29] + coeff0_27*dmats0[27][29] + coeff0_28*dmats0[28][29] + coeff0_29*dmats0[29][29] + coeff0_30*dmats0[30][29] + coeff0_31*dmats0[31][29] + coeff0_32*dmats0[32][29] + coeff0_33*dmats0[33][29] + coeff0_34*dmats0[34][29] + coeff0_35*dmats0[35][29] + coeff0_36*dmats0[36][29] + coeff0_37*dmats0[37][29] + coeff0_38*dmats0[38][29] + coeff0_39*dmats0[39][29] + coeff0_40*dmats0[40][29] + coeff0_41*dmats0[41][29] + coeff0_42*dmats0[42][29] + coeff0_43*dmats0[43][29] + coeff0_44*dmats0[44][29] + coeff0_45*dmats0[45][29] + coeff0_46*dmats0[46][29] + coeff0_47*dmats0[47][29] + coeff0_48*dmats0[48][29] + coeff0_49*dmats0[49][29] + coeff0_50*dmats0[50][29] + coeff0_51*dmats0[51][29] + coeff0_52*dmats0[52][29] + coeff0_53*dmats0[53][29] + coeff0_54*dmats0[54][29] + coeff0_55*dmats0[55][29];
          new_coeff0_30 = coeff0_0*dmats0[0][30] + coeff0_1*dmats0[1][30] + coeff0_2*dmats0[2][30] + coeff0_3*dmats0[3][30] + coeff0_4*dmats0[4][30] + coeff0_5*dmats0[5][30] + coeff0_6*dmats0[6][30] + coeff0_7*dmats0[7][30] + coeff0_8*dmats0[8][30] + coeff0_9*dmats0[9][30] + coeff0_10*dmats0[10][30] + coeff0_11*dmats0[11][30] + coeff0_12*dmats0[12][30] + coeff0_13*dmats0[13][30] + coeff0_14*dmats0[14][30] + coeff0_15*dmats0[15][30] + coeff0_16*dmats0[16][30] + coeff0_17*dmats0[17][30] + coeff0_18*dmats0[18][30] + coeff0_19*dmats0[19][30] + coeff0_20*dmats0[20][30] + coeff0_21*dmats0[21][30] + coeff0_22*dmats0[22][30] + coeff0_23*dmats0[23][30] + coeff0_24*dmats0[24][30] + coeff0_25*dmats0[25][30] + coeff0_26*dmats0[26][30] + coeff0_27*dmats0[27][30] + coeff0_28*dmats0[28][30] + coeff0_29*dmats0[29][30] + coeff0_30*dmats0[30][30] + coeff0_31*dmats0[31][30] + coeff0_32*dmats0[32][30] + coeff0_33*dmats0[33][30] + coeff0_34*dmats0[34][30] + coeff0_35*dmats0[35][30] + coeff0_36*dmats0[36][30] + coeff0_37*dmats0[37][30] + coeff0_38*dmats0[38][30] + coeff0_39*dmats0[39][30] + coeff0_40*dmats0[40][30] + coeff0_41*dmats0[41][30] + coeff0_42*dmats0[42][30] + coeff0_43*dmats0[43][30] + coeff0_44*dmats0[44][30] + coeff0_45*dmats0[45][30] + coeff0_46*dmats0[46][30] + coeff0_47*dmats0[47][30] + coeff0_48*dmats0[48][30] + coeff0_49*dmats0[49][30] + coeff0_50*dmats0[50][30] + coeff0_51*dmats0[51][30] + coeff0_52*dmats0[52][30] + coeff0_53*dmats0[53][30] + coeff0_54*dmats0[54][30] + coeff0_55*dmats0[55][30];
          new_coeff0_31 = coeff0_0*dmats0[0][31] + coeff0_1*dmats0[1][31] + coeff0_2*dmats0[2][31] + coeff0_3*dmats0[3][31] + coeff0_4*dmats0[4][31] + coeff0_5*dmats0[5][31] + coeff0_6*dmats0[6][31] + coeff0_7*dmats0[7][31] + coeff0_8*dmats0[8][31] + coeff0_9*dmats0[9][31] + coeff0_10*dmats0[10][31] + coeff0_11*dmats0[11][31] + coeff0_12*dmats0[12][31] + coeff0_13*dmats0[13][31] + coeff0_14*dmats0[14][31] + coeff0_15*dmats0[15][31] + coeff0_16*dmats0[16][31] + coeff0_17*dmats0[17][31] + coeff0_18*dmats0[18][31] + coeff0_19*dmats0[19][31] + coeff0_20*dmats0[20][31] + coeff0_21*dmats0[21][31] + coeff0_22*dmats0[22][31] + coeff0_23*dmats0[23][31] + coeff0_24*dmats0[24][31] + coeff0_25*dmats0[25][31] + coeff0_26*dmats0[26][31] + coeff0_27*dmats0[27][31] + coeff0_28*dmats0[28][31] + coeff0_29*dmats0[29][31] + coeff0_30*dmats0[30][31] + coeff0_31*dmats0[31][31] + coeff0_32*dmats0[32][31] + coeff0_33*dmats0[33][31] + coeff0_34*dmats0[34][31] + coeff0_35*dmats0[35][31] + coeff0_36*dmats0[36][31] + coeff0_37*dmats0[37][31] + coeff0_38*dmats0[38][31] + coeff0_39*dmats0[39][31] + coeff0_40*dmats0[40][31] + coeff0_41*dmats0[41][31] + coeff0_42*dmats0[42][31] + coeff0_43*dmats0[43][31] + coeff0_44*dmats0[44][31] + coeff0_45*dmats0[45][31] + coeff0_46*dmats0[46][31] + coeff0_47*dmats0[47][31] + coeff0_48*dmats0[48][31] + coeff0_49*dmats0[49][31] + coeff0_50*dmats0[50][31] + coeff0_51*dmats0[51][31] + coeff0_52*dmats0[52][31] + coeff0_53*dmats0[53][31] + coeff0_54*dmats0[54][31] + coeff0_55*dmats0[55][31];
          new_coeff0_32 = coeff0_0*dmats0[0][32] + coeff0_1*dmats0[1][32] + coeff0_2*dmats0[2][32] + coeff0_3*dmats0[3][32] + coeff0_4*dmats0[4][32] + coeff0_5*dmats0[5][32] + coeff0_6*dmats0[6][32] + coeff0_7*dmats0[7][32] + coeff0_8*dmats0[8][32] + coeff0_9*dmats0[9][32] + coeff0_10*dmats0[10][32] + coeff0_11*dmats0[11][32] + coeff0_12*dmats0[12][32] + coeff0_13*dmats0[13][32] + coeff0_14*dmats0[14][32] + coeff0_15*dmats0[15][32] + coeff0_16*dmats0[16][32] + coeff0_17*dmats0[17][32] + coeff0_18*dmats0[18][32] + coeff0_19*dmats0[19][32] + coeff0_20*dmats0[20][32] + coeff0_21*dmats0[21][32] + coeff0_22*dmats0[22][32] + coeff0_23*dmats0[23][32] + coeff0_24*dmats0[24][32] + coeff0_25*dmats0[25][32] + coeff0_26*dmats0[26][32] + coeff0_27*dmats0[27][32] + coeff0_28*dmats0[28][32] + coeff0_29*dmats0[29][32] + coeff0_30*dmats0[30][32] + coeff0_31*dmats0[31][32] + coeff0_32*dmats0[32][32] + coeff0_33*dmats0[33][32] + coeff0_34*dmats0[34][32] + coeff0_35*dmats0[35][32] + coeff0_36*dmats0[36][32] + coeff0_37*dmats0[37][32] + coeff0_38*dmats0[38][32] + coeff0_39*dmats0[39][32] + coeff0_40*dmats0[40][32] + coeff0_41*dmats0[41][32] + coeff0_42*dmats0[42][32] + coeff0_43*dmats0[43][32] + coeff0_44*dmats0[44][32] + coeff0_45*dmats0[45][32] + coeff0_46*dmats0[46][32] + coeff0_47*dmats0[47][32] + coeff0_48*dmats0[48][32] + coeff0_49*dmats0[49][32] + coeff0_50*dmats0[50][32] + coeff0_51*dmats0[51][32] + coeff0_52*dmats0[52][32] + coeff0_53*dmats0[53][32] + coeff0_54*dmats0[54][32] + coeff0_55*dmats0[55][32];
          new_coeff0_33 = coeff0_0*dmats0[0][33] + coeff0_1*dmats0[1][33] + coeff0_2*dmats0[2][33] + coeff0_3*dmats0[3][33] + coeff0_4*dmats0[4][33] + coeff0_5*dmats0[5][33] + coeff0_6*dmats0[6][33] + coeff0_7*dmats0[7][33] + coeff0_8*dmats0[8][33] + coeff0_9*dmats0[9][33] + coeff0_10*dmats0[10][33] + coeff0_11*dmats0[11][33] + coeff0_12*dmats0[12][33] + coeff0_13*dmats0[13][33] + coeff0_14*dmats0[14][33] + coeff0_15*dmats0[15][33] + coeff0_16*dmats0[16][33] + coeff0_17*dmats0[17][33] + coeff0_18*dmats0[18][33] + coeff0_19*dmats0[19][33] + coeff0_20*dmats0[20][33] + coeff0_21*dmats0[21][33] + coeff0_22*dmats0[22][33] + coeff0_23*dmats0[23][33] + coeff0_24*dmats0[24][33] + coeff0_25*dmats0[25][33] + coeff0_26*dmats0[26][33] + coeff0_27*dmats0[27][33] + coeff0_28*dmats0[28][33] + coeff0_29*dmats0[29][33] + coeff0_30*dmats0[30][33] + coeff0_31*dmats0[31][33] + coeff0_32*dmats0[32][33] + coeff0_33*dmats0[33][33] + coeff0_34*dmats0[34][33] + coeff0_35*dmats0[35][33] + coeff0_36*dmats0[36][33] + coeff0_37*dmats0[37][33] + coeff0_38*dmats0[38][33] + coeff0_39*dmats0[39][33] + coeff0_40*dmats0[40][33] + coeff0_41*dmats0[41][33] + coeff0_42*dmats0[42][33] + coeff0_43*dmats0[43][33] + coeff0_44*dmats0[44][33] + coeff0_45*dmats0[45][33] + coeff0_46*dmats0[46][33] + coeff0_47*dmats0[47][33] + coeff0_48*dmats0[48][33] + coeff0_49*dmats0[49][33] + coeff0_50*dmats0[50][33] + coeff0_51*dmats0[51][33] + coeff0_52*dmats0[52][33] + coeff0_53*dmats0[53][33] + coeff0_54*dmats0[54][33] + coeff0_55*dmats0[55][33];
          new_coeff0_34 = coeff0_0*dmats0[0][34] + coeff0_1*dmats0[1][34] + coeff0_2*dmats0[2][34] + coeff0_3*dmats0[3][34] + coeff0_4*dmats0[4][34] + coeff0_5*dmats0[5][34] + coeff0_6*dmats0[6][34] + coeff0_7*dmats0[7][34] + coeff0_8*dmats0[8][34] + coeff0_9*dmats0[9][34] + coeff0_10*dmats0[10][34] + coeff0_11*dmats0[11][34] + coeff0_12*dmats0[12][34] + coeff0_13*dmats0[13][34] + coeff0_14*dmats0[14][34] + coeff0_15*dmats0[15][34] + coeff0_16*dmats0[16][34] + coeff0_17*dmats0[17][34] + coeff0_18*dmats0[18][34] + coeff0_19*dmats0[19][34] + coeff0_20*dmats0[20][34] + coeff0_21*dmats0[21][34] + coeff0_22*dmats0[22][34] + coeff0_23*dmats0[23][34] + coeff0_24*dmats0[24][34] + coeff0_25*dmats0[25][34] + coeff0_26*dmats0[26][34] + coeff0_27*dmats0[27][34] + coeff0_28*dmats0[28][34] + coeff0_29*dmats0[29][34] + coeff0_30*dmats0[30][34] + coeff0_31*dmats0[31][34] + coeff0_32*dmats0[32][34] + coeff0_33*dmats0[33][34] + coeff0_34*dmats0[34][34] + coeff0_35*dmats0[35][34] + coeff0_36*dmats0[36][34] + coeff0_37*dmats0[37][34] + coeff0_38*dmats0[38][34] + coeff0_39*dmats0[39][34] + coeff0_40*dmats0[40][34] + coeff0_41*dmats0[41][34] + coeff0_42*dmats0[42][34] + coeff0_43*dmats0[43][34] + coeff0_44*dmats0[44][34] + coeff0_45*dmats0[45][34] + coeff0_46*dmats0[46][34] + coeff0_47*dmats0[47][34] + coeff0_48*dmats0[48][34] + coeff0_49*dmats0[49][34] + coeff0_50*dmats0[50][34] + coeff0_51*dmats0[51][34] + coeff0_52*dmats0[52][34] + coeff0_53*dmats0[53][34] + coeff0_54*dmats0[54][34] + coeff0_55*dmats0[55][34];
          new_coeff0_35 = coeff0_0*dmats0[0][35] + coeff0_1*dmats0[1][35] + coeff0_2*dmats0[2][35] + coeff0_3*dmats0[3][35] + coeff0_4*dmats0[4][35] + coeff0_5*dmats0[5][35] + coeff0_6*dmats0[6][35] + coeff0_7*dmats0[7][35] + coeff0_8*dmats0[8][35] + coeff0_9*dmats0[9][35] + coeff0_10*dmats0[10][35] + coeff0_11*dmats0[11][35] + coeff0_12*dmats0[12][35] + coeff0_13*dmats0[13][35] + coeff0_14*dmats0[14][35] + coeff0_15*dmats0[15][35] + coeff0_16*dmats0[16][35] + coeff0_17*dmats0[17][35] + coeff0_18*dmats0[18][35] + coeff0_19*dmats0[19][35] + coeff0_20*dmats0[20][35] + coeff0_21*dmats0[21][35] + coeff0_22*dmats0[22][35] + coeff0_23*dmats0[23][35] + coeff0_24*dmats0[24][35] + coeff0_25*dmats0[25][35] + coeff0_26*dmats0[26][35] + coeff0_27*dmats0[27][35] + coeff0_28*dmats0[28][35] + coeff0_29*dmats0[29][35] + coeff0_30*dmats0[30][35] + coeff0_31*dmats0[31][35] + coeff0_32*dmats0[32][35] + coeff0_33*dmats0[33][35] + coeff0_34*dmats0[34][35] + coeff0_35*dmats0[35][35] + coeff0_36*dmats0[36][35] + coeff0_37*dmats0[37][35] + coeff0_38*dmats0[38][35] + coeff0_39*dmats0[39][35] + coeff0_40*dmats0[40][35] + coeff0_41*dmats0[41][35] + coeff0_42*dmats0[42][35] + coeff0_43*dmats0[43][35] + coeff0_44*dmats0[44][35] + coeff0_45*dmats0[45][35] + coeff0_46*dmats0[46][35] + coeff0_47*dmats0[47][35] + coeff0_48*dmats0[48][35] + coeff0_49*dmats0[49][35] + coeff0_50*dmats0[50][35] + coeff0_51*dmats0[51][35] + coeff0_52*dmats0[52][35] + coeff0_53*dmats0[53][35] + coeff0_54*dmats0[54][35] + coeff0_55*dmats0[55][35];
          new_coeff0_36 = coeff0_0*dmats0[0][36] + coeff0_1*dmats0[1][36] + coeff0_2*dmats0[2][36] + coeff0_3*dmats0[3][36] + coeff0_4*dmats0[4][36] + coeff0_5*dmats0[5][36] + coeff0_6*dmats0[6][36] + coeff0_7*dmats0[7][36] + coeff0_8*dmats0[8][36] + coeff0_9*dmats0[9][36] + coeff0_10*dmats0[10][36] + coeff0_11*dmats0[11][36] + coeff0_12*dmats0[12][36] + coeff0_13*dmats0[13][36] + coeff0_14*dmats0[14][36] + coeff0_15*dmats0[15][36] + coeff0_16*dmats0[16][36] + coeff0_17*dmats0[17][36] + coeff0_18*dmats0[18][36] + coeff0_19*dmats0[19][36] + coeff0_20*dmats0[20][36] + coeff0_21*dmats0[21][36] + coeff0_22*dmats0[22][36] + coeff0_23*dmats0[23][36] + coeff0_24*dmats0[24][36] + coeff0_25*dmats0[25][36] + coeff0_26*dmats0[26][36] + coeff0_27*dmats0[27][36] + coeff0_28*dmats0[28][36] + coeff0_29*dmats0[29][36] + coeff0_30*dmats0[30][36] + coeff0_31*dmats0[31][36] + coeff0_32*dmats0[32][36] + coeff0_33*dmats0[33][36] + coeff0_34*dmats0[34][36] + coeff0_35*dmats0[35][36] + coeff0_36*dmats0[36][36] + coeff0_37*dmats0[37][36] + coeff0_38*dmats0[38][36] + coeff0_39*dmats0[39][36] + coeff0_40*dmats0[40][36] + coeff0_41*dmats0[41][36] + coeff0_42*dmats0[42][36] + coeff0_43*dmats0[43][36] + coeff0_44*dmats0[44][36] + coeff0_45*dmats0[45][36] + coeff0_46*dmats0[46][36] + coeff0_47*dmats0[47][36] + coeff0_48*dmats0[48][36] + coeff0_49*dmats0[49][36] + coeff0_50*dmats0[50][36] + coeff0_51*dmats0[51][36] + coeff0_52*dmats0[52][36] + coeff0_53*dmats0[53][36] + coeff0_54*dmats0[54][36] + coeff0_55*dmats0[55][36];
          new_coeff0_37 = coeff0_0*dmats0[0][37] + coeff0_1*dmats0[1][37] + coeff0_2*dmats0[2][37] + coeff0_3*dmats0[3][37] + coeff0_4*dmats0[4][37] + coeff0_5*dmats0[5][37] + coeff0_6*dmats0[6][37] + coeff0_7*dmats0[7][37] + coeff0_8*dmats0[8][37] + coeff0_9*dmats0[9][37] + coeff0_10*dmats0[10][37] + coeff0_11*dmats0[11][37] + coeff0_12*dmats0[12][37] + coeff0_13*dmats0[13][37] + coeff0_14*dmats0[14][37] + coeff0_15*dmats0[15][37] + coeff0_16*dmats0[16][37] + coeff0_17*dmats0[17][37] + coeff0_18*dmats0[18][37] + coeff0_19*dmats0[19][37] + coeff0_20*dmats0[20][37] + coeff0_21*dmats0[21][37] + coeff0_22*dmats0[22][37] + coeff0_23*dmats0[23][37] + coeff0_24*dmats0[24][37] + coeff0_25*dmats0[25][37] + coeff0_26*dmats0[26][37] + coeff0_27*dmats0[27][37] + coeff0_28*dmats0[28][37] + coeff0_29*dmats0[29][37] + coeff0_30*dmats0[30][37] + coeff0_31*dmats0[31][37] + coeff0_32*dmats0[32][37] + coeff0_33*dmats0[33][37] + coeff0_34*dmats0[34][37] + coeff0_35*dmats0[35][37] + coeff0_36*dmats0[36][37] + coeff0_37*dmats0[37][37] + coeff0_38*dmats0[38][37] + coeff0_39*dmats0[39][37] + coeff0_40*dmats0[40][37] + coeff0_41*dmats0[41][37] + coeff0_42*dmats0[42][37] + coeff0_43*dmats0[43][37] + coeff0_44*dmats0[44][37] + coeff0_45*dmats0[45][37] + coeff0_46*dmats0[46][37] + coeff0_47*dmats0[47][37] + coeff0_48*dmats0[48][37] + coeff0_49*dmats0[49][37] + coeff0_50*dmats0[50][37] + coeff0_51*dmats0[51][37] + coeff0_52*dmats0[52][37] + coeff0_53*dmats0[53][37] + coeff0_54*dmats0[54][37] + coeff0_55*dmats0[55][37];
          new_coeff0_38 = coeff0_0*dmats0[0][38] + coeff0_1*dmats0[1][38] + coeff0_2*dmats0[2][38] + coeff0_3*dmats0[3][38] + coeff0_4*dmats0[4][38] + coeff0_5*dmats0[5][38] + coeff0_6*dmats0[6][38] + coeff0_7*dmats0[7][38] + coeff0_8*dmats0[8][38] + coeff0_9*dmats0[9][38] + coeff0_10*dmats0[10][38] + coeff0_11*dmats0[11][38] + coeff0_12*dmats0[12][38] + coeff0_13*dmats0[13][38] + coeff0_14*dmats0[14][38] + coeff0_15*dmats0[15][38] + coeff0_16*dmats0[16][38] + coeff0_17*dmats0[17][38] + coeff0_18*dmats0[18][38] + coeff0_19*dmats0[19][38] + coeff0_20*dmats0[20][38] + coeff0_21*dmats0[21][38] + coeff0_22*dmats0[22][38] + coeff0_23*dmats0[23][38] + coeff0_24*dmats0[24][38] + coeff0_25*dmats0[25][38] + coeff0_26*dmats0[26][38] + coeff0_27*dmats0[27][38] + coeff0_28*dmats0[28][38] + coeff0_29*dmats0[29][38] + coeff0_30*dmats0[30][38] + coeff0_31*dmats0[31][38] + coeff0_32*dmats0[32][38] + coeff0_33*dmats0[33][38] + coeff0_34*dmats0[34][38] + coeff0_35*dmats0[35][38] + coeff0_36*dmats0[36][38] + coeff0_37*dmats0[37][38] + coeff0_38*dmats0[38][38] + coeff0_39*dmats0[39][38] + coeff0_40*dmats0[40][38] + coeff0_41*dmats0[41][38] + coeff0_42*dmats0[42][38] + coeff0_43*dmats0[43][38] + coeff0_44*dmats0[44][38] + coeff0_45*dmats0[45][38] + coeff0_46*dmats0[46][38] + coeff0_47*dmats0[47][38] + coeff0_48*dmats0[48][38] + coeff0_49*dmats0[49][38] + coeff0_50*dmats0[50][38] + coeff0_51*dmats0[51][38] + coeff0_52*dmats0[52][38] + coeff0_53*dmats0[53][38] + coeff0_54*dmats0[54][38] + coeff0_55*dmats0[55][38];
          new_coeff0_39 = coeff0_0*dmats0[0][39] + coeff0_1*dmats0[1][39] + coeff0_2*dmats0[2][39] + coeff0_3*dmats0[3][39] + coeff0_4*dmats0[4][39] + coeff0_5*dmats0[5][39] + coeff0_6*dmats0[6][39] + coeff0_7*dmats0[7][39] + coeff0_8*dmats0[8][39] + coeff0_9*dmats0[9][39] + coeff0_10*dmats0[10][39] + coeff0_11*dmats0[11][39] + coeff0_12*dmats0[12][39] + coeff0_13*dmats0[13][39] + coeff0_14*dmats0[14][39] + coeff0_15*dmats0[15][39] + coeff0_16*dmats0[16][39] + coeff0_17*dmats0[17][39] + coeff0_18*dmats0[18][39] + coeff0_19*dmats0[19][39] + coeff0_20*dmats0[20][39] + coeff0_21*dmats0[21][39] + coeff0_22*dmats0[22][39] + coeff0_23*dmats0[23][39] + coeff0_24*dmats0[24][39] + coeff0_25*dmats0[25][39] + coeff0_26*dmats0[26][39] + coeff0_27*dmats0[27][39] + coeff0_28*dmats0[28][39] + coeff0_29*dmats0[29][39] + coeff0_30*dmats0[30][39] + coeff0_31*dmats0[31][39] + coeff0_32*dmats0[32][39] + coeff0_33*dmats0[33][39] + coeff0_34*dmats0[34][39] + coeff0_35*dmats0[35][39] + coeff0_36*dmats0[36][39] + coeff0_37*dmats0[37][39] + coeff0_38*dmats0[38][39] + coeff0_39*dmats0[39][39] + coeff0_40*dmats0[40][39] + coeff0_41*dmats0[41][39] + coeff0_42*dmats0[42][39] + coeff0_43*dmats0[43][39] + coeff0_44*dmats0[44][39] + coeff0_45*dmats0[45][39] + coeff0_46*dmats0[46][39] + coeff0_47*dmats0[47][39] + coeff0_48*dmats0[48][39] + coeff0_49*dmats0[49][39] + coeff0_50*dmats0[50][39] + coeff0_51*dmats0[51][39] + coeff0_52*dmats0[52][39] + coeff0_53*dmats0[53][39] + coeff0_54*dmats0[54][39] + coeff0_55*dmats0[55][39];
          new_coeff0_40 = coeff0_0*dmats0[0][40] + coeff0_1*dmats0[1][40] + coeff0_2*dmats0[2][40] + coeff0_3*dmats0[3][40] + coeff0_4*dmats0[4][40] + coeff0_5*dmats0[5][40] + coeff0_6*dmats0[6][40] + coeff0_7*dmats0[7][40] + coeff0_8*dmats0[8][40] + coeff0_9*dmats0[9][40] + coeff0_10*dmats0[10][40] + coeff0_11*dmats0[11][40] + coeff0_12*dmats0[12][40] + coeff0_13*dmats0[13][40] + coeff0_14*dmats0[14][40] + coeff0_15*dmats0[15][40] + coeff0_16*dmats0[16][40] + coeff0_17*dmats0[17][40] + coeff0_18*dmats0[18][40] + coeff0_19*dmats0[19][40] + coeff0_20*dmats0[20][40] + coeff0_21*dmats0[21][40] + coeff0_22*dmats0[22][40] + coeff0_23*dmats0[23][40] + coeff0_24*dmats0[24][40] + coeff0_25*dmats0[25][40] + coeff0_26*dmats0[26][40] + coeff0_27*dmats0[27][40] + coeff0_28*dmats0[28][40] + coeff0_29*dmats0[29][40] + coeff0_30*dmats0[30][40] + coeff0_31*dmats0[31][40] + coeff0_32*dmats0[32][40] + coeff0_33*dmats0[33][40] + coeff0_34*dmats0[34][40] + coeff0_35*dmats0[35][40] + coeff0_36*dmats0[36][40] + coeff0_37*dmats0[37][40] + coeff0_38*dmats0[38][40] + coeff0_39*dmats0[39][40] + coeff0_40*dmats0[40][40] + coeff0_41*dmats0[41][40] + coeff0_42*dmats0[42][40] + coeff0_43*dmats0[43][40] + coeff0_44*dmats0[44][40] + coeff0_45*dmats0[45][40] + coeff0_46*dmats0[46][40] + coeff0_47*dmats0[47][40] + coeff0_48*dmats0[48][40] + coeff0_49*dmats0[49][40] + coeff0_50*dmats0[50][40] + coeff0_51*dmats0[51][40] + coeff0_52*dmats0[52][40] + coeff0_53*dmats0[53][40] + coeff0_54*dmats0[54][40] + coeff0_55*dmats0[55][40];
          new_coeff0_41 = coeff0_0*dmats0[0][41] + coeff0_1*dmats0[1][41] + coeff0_2*dmats0[2][41] + coeff0_3*dmats0[3][41] + coeff0_4*dmats0[4][41] + coeff0_5*dmats0[5][41] + coeff0_6*dmats0[6][41] + coeff0_7*dmats0[7][41] + coeff0_8*dmats0[8][41] + coeff0_9*dmats0[9][41] + coeff0_10*dmats0[10][41] + coeff0_11*dmats0[11][41] + coeff0_12*dmats0[12][41] + coeff0_13*dmats0[13][41] + coeff0_14*dmats0[14][41] + coeff0_15*dmats0[15][41] + coeff0_16*dmats0[16][41] + coeff0_17*dmats0[17][41] + coeff0_18*dmats0[18][41] + coeff0_19*dmats0[19][41] + coeff0_20*dmats0[20][41] + coeff0_21*dmats0[21][41] + coeff0_22*dmats0[22][41] + coeff0_23*dmats0[23][41] + coeff0_24*dmats0[24][41] + coeff0_25*dmats0[25][41] + coeff0_26*dmats0[26][41] + coeff0_27*dmats0[27][41] + coeff0_28*dmats0[28][41] + coeff0_29*dmats0[29][41] + coeff0_30*dmats0[30][41] + coeff0_31*dmats0[31][41] + coeff0_32*dmats0[32][41] + coeff0_33*dmats0[33][41] + coeff0_34*dmats0[34][41] + coeff0_35*dmats0[35][41] + coeff0_36*dmats0[36][41] + coeff0_37*dmats0[37][41] + coeff0_38*dmats0[38][41] + coeff0_39*dmats0[39][41] + coeff0_40*dmats0[40][41] + coeff0_41*dmats0[41][41] + coeff0_42*dmats0[42][41] + coeff0_43*dmats0[43][41] + coeff0_44*dmats0[44][41] + coeff0_45*dmats0[45][41] + coeff0_46*dmats0[46][41] + coeff0_47*dmats0[47][41] + coeff0_48*dmats0[48][41] + coeff0_49*dmats0[49][41] + coeff0_50*dmats0[50][41] + coeff0_51*dmats0[51][41] + coeff0_52*dmats0[52][41] + coeff0_53*dmats0[53][41] + coeff0_54*dmats0[54][41] + coeff0_55*dmats0[55][41];
          new_coeff0_42 = coeff0_0*dmats0[0][42] + coeff0_1*dmats0[1][42] + coeff0_2*dmats0[2][42] + coeff0_3*dmats0[3][42] + coeff0_4*dmats0[4][42] + coeff0_5*dmats0[5][42] + coeff0_6*dmats0[6][42] + coeff0_7*dmats0[7][42] + coeff0_8*dmats0[8][42] + coeff0_9*dmats0[9][42] + coeff0_10*dmats0[10][42] + coeff0_11*dmats0[11][42] + coeff0_12*dmats0[12][42] + coeff0_13*dmats0[13][42] + coeff0_14*dmats0[14][42] + coeff0_15*dmats0[15][42] + coeff0_16*dmats0[16][42] + coeff0_17*dmats0[17][42] + coeff0_18*dmats0[18][42] + coeff0_19*dmats0[19][42] + coeff0_20*dmats0[20][42] + coeff0_21*dmats0[21][42] + coeff0_22*dmats0[22][42] + coeff0_23*dmats0[23][42] + coeff0_24*dmats0[24][42] + coeff0_25*dmats0[25][42] + coeff0_26*dmats0[26][42] + coeff0_27*dmats0[27][42] + coeff0_28*dmats0[28][42] + coeff0_29*dmats0[29][42] + coeff0_30*dmats0[30][42] + coeff0_31*dmats0[31][42] + coeff0_32*dmats0[32][42] + coeff0_33*dmats0[33][42] + coeff0_34*dmats0[34][42] + coeff0_35*dmats0[35][42] + coeff0_36*dmats0[36][42] + coeff0_37*dmats0[37][42] + coeff0_38*dmats0[38][42] + coeff0_39*dmats0[39][42] + coeff0_40*dmats0[40][42] + coeff0_41*dmats0[41][42] + coeff0_42*dmats0[42][42] + coeff0_43*dmats0[43][42] + coeff0_44*dmats0[44][42] + coeff0_45*dmats0[45][42] + coeff0_46*dmats0[46][42] + coeff0_47*dmats0[47][42] + coeff0_48*dmats0[48][42] + coeff0_49*dmats0[49][42] + coeff0_50*dmats0[50][42] + coeff0_51*dmats0[51][42] + coeff0_52*dmats0[52][42] + coeff0_53*dmats0[53][42] + coeff0_54*dmats0[54][42] + coeff0_55*dmats0[55][42];
          new_coeff0_43 = coeff0_0*dmats0[0][43] + coeff0_1*dmats0[1][43] + coeff0_2*dmats0[2][43] + coeff0_3*dmats0[3][43] + coeff0_4*dmats0[4][43] + coeff0_5*dmats0[5][43] + coeff0_6*dmats0[6][43] + coeff0_7*dmats0[7][43] + coeff0_8*dmats0[8][43] + coeff0_9*dmats0[9][43] + coeff0_10*dmats0[10][43] + coeff0_11*dmats0[11][43] + coeff0_12*dmats0[12][43] + coeff0_13*dmats0[13][43] + coeff0_14*dmats0[14][43] + coeff0_15*dmats0[15][43] + coeff0_16*dmats0[16][43] + coeff0_17*dmats0[17][43] + coeff0_18*dmats0[18][43] + coeff0_19*dmats0[19][43] + coeff0_20*dmats0[20][43] + coeff0_21*dmats0[21][43] + coeff0_22*dmats0[22][43] + coeff0_23*dmats0[23][43] + coeff0_24*dmats0[24][43] + coeff0_25*dmats0[25][43] + coeff0_26*dmats0[26][43] + coeff0_27*dmats0[27][43] + coeff0_28*dmats0[28][43] + coeff0_29*dmats0[29][43] + coeff0_30*dmats0[30][43] + coeff0_31*dmats0[31][43] + coeff0_32*dmats0[32][43] + coeff0_33*dmats0[33][43] + coeff0_34*dmats0[34][43] + coeff0_35*dmats0[35][43] + coeff0_36*dmats0[36][43] + coeff0_37*dmats0[37][43] + coeff0_38*dmats0[38][43] + coeff0_39*dmats0[39][43] + coeff0_40*dmats0[40][43] + coeff0_41*dmats0[41][43] + coeff0_42*dmats0[42][43] + coeff0_43*dmats0[43][43] + coeff0_44*dmats0[44][43] + coeff0_45*dmats0[45][43] + coeff0_46*dmats0[46][43] + coeff0_47*dmats0[47][43] + coeff0_48*dmats0[48][43] + coeff0_49*dmats0[49][43] + coeff0_50*dmats0[50][43] + coeff0_51*dmats0[51][43] + coeff0_52*dmats0[52][43] + coeff0_53*dmats0[53][43] + coeff0_54*dmats0[54][43] + coeff0_55*dmats0[55][43];
          new_coeff0_44 = coeff0_0*dmats0[0][44] + coeff0_1*dmats0[1][44] + coeff0_2*dmats0[2][44] + coeff0_3*dmats0[3][44] + coeff0_4*dmats0[4][44] + coeff0_5*dmats0[5][44] + coeff0_6*dmats0[6][44] + coeff0_7*dmats0[7][44] + coeff0_8*dmats0[8][44] + coeff0_9*dmats0[9][44] + coeff0_10*dmats0[10][44] + coeff0_11*dmats0[11][44] + coeff0_12*dmats0[12][44] + coeff0_13*dmats0[13][44] + coeff0_14*dmats0[14][44] + coeff0_15*dmats0[15][44] + coeff0_16*dmats0[16][44] + coeff0_17*dmats0[17][44] + coeff0_18*dmats0[18][44] + coeff0_19*dmats0[19][44] + coeff0_20*dmats0[20][44] + coeff0_21*dmats0[21][44] + coeff0_22*dmats0[22][44] + coeff0_23*dmats0[23][44] + coeff0_24*dmats0[24][44] + coeff0_25*dmats0[25][44] + coeff0_26*dmats0[26][44] + coeff0_27*dmats0[27][44] + coeff0_28*dmats0[28][44] + coeff0_29*dmats0[29][44] + coeff0_30*dmats0[30][44] + coeff0_31*dmats0[31][44] + coeff0_32*dmats0[32][44] + coeff0_33*dmats0[33][44] + coeff0_34*dmats0[34][44] + coeff0_35*dmats0[35][44] + coeff0_36*dmats0[36][44] + coeff0_37*dmats0[37][44] + coeff0_38*dmats0[38][44] + coeff0_39*dmats0[39][44] + coeff0_40*dmats0[40][44] + coeff0_41*dmats0[41][44] + coeff0_42*dmats0[42][44] + coeff0_43*dmats0[43][44] + coeff0_44*dmats0[44][44] + coeff0_45*dmats0[45][44] + coeff0_46*dmats0[46][44] + coeff0_47*dmats0[47][44] + coeff0_48*dmats0[48][44] + coeff0_49*dmats0[49][44] + coeff0_50*dmats0[50][44] + coeff0_51*dmats0[51][44] + coeff0_52*dmats0[52][44] + coeff0_53*dmats0[53][44] + coeff0_54*dmats0[54][44] + coeff0_55*dmats0[55][44];
          new_coeff0_45 = coeff0_0*dmats0[0][45] + coeff0_1*dmats0[1][45] + coeff0_2*dmats0[2][45] + coeff0_3*dmats0[3][45] + coeff0_4*dmats0[4][45] + coeff0_5*dmats0[5][45] + coeff0_6*dmats0[6][45] + coeff0_7*dmats0[7][45] + coeff0_8*dmats0[8][45] + coeff0_9*dmats0[9][45] + coeff0_10*dmats0[10][45] + coeff0_11*dmats0[11][45] + coeff0_12*dmats0[12][45] + coeff0_13*dmats0[13][45] + coeff0_14*dmats0[14][45] + coeff0_15*dmats0[15][45] + coeff0_16*dmats0[16][45] + coeff0_17*dmats0[17][45] + coeff0_18*dmats0[18][45] + coeff0_19*dmats0[19][45] + coeff0_20*dmats0[20][45] + coeff0_21*dmats0[21][45] + coeff0_22*dmats0[22][45] + coeff0_23*dmats0[23][45] + coeff0_24*dmats0[24][45] + coeff0_25*dmats0[25][45] + coeff0_26*dmats0[26][45] + coeff0_27*dmats0[27][45] + coeff0_28*dmats0[28][45] + coeff0_29*dmats0[29][45] + coeff0_30*dmats0[30][45] + coeff0_31*dmats0[31][45] + coeff0_32*dmats0[32][45] + coeff0_33*dmats0[33][45] + coeff0_34*dmats0[34][45] + coeff0_35*dmats0[35][45] + coeff0_36*dmats0[36][45] + coeff0_37*dmats0[37][45] + coeff0_38*dmats0[38][45] + coeff0_39*dmats0[39][45] + coeff0_40*dmats0[40][45] + coeff0_41*dmats0[41][45] + coeff0_42*dmats0[42][45] + coeff0_43*dmats0[43][45] + coeff0_44*dmats0[44][45] + coeff0_45*dmats0[45][45] + coeff0_46*dmats0[46][45] + coeff0_47*dmats0[47][45] + coeff0_48*dmats0[48][45] + coeff0_49*dmats0[49][45] + coeff0_50*dmats0[50][45] + coeff0_51*dmats0[51][45] + coeff0_52*dmats0[52][45] + coeff0_53*dmats0[53][45] + coeff0_54*dmats0[54][45] + coeff0_55*dmats0[55][45];
          new_coeff0_46 = coeff0_0*dmats0[0][46] + coeff0_1*dmats0[1][46] + coeff0_2*dmats0[2][46] + coeff0_3*dmats0[3][46] + coeff0_4*dmats0[4][46] + coeff0_5*dmats0[5][46] + coeff0_6*dmats0[6][46] + coeff0_7*dmats0[7][46] + coeff0_8*dmats0[8][46] + coeff0_9*dmats0[9][46] + coeff0_10*dmats0[10][46] + coeff0_11*dmats0[11][46] + coeff0_12*dmats0[12][46] + coeff0_13*dmats0[13][46] + coeff0_14*dmats0[14][46] + coeff0_15*dmats0[15][46] + coeff0_16*dmats0[16][46] + coeff0_17*dmats0[17][46] + coeff0_18*dmats0[18][46] + coeff0_19*dmats0[19][46] + coeff0_20*dmats0[20][46] + coeff0_21*dmats0[21][46] + coeff0_22*dmats0[22][46] + coeff0_23*dmats0[23][46] + coeff0_24*dmats0[24][46] + coeff0_25*dmats0[25][46] + coeff0_26*dmats0[26][46] + coeff0_27*dmats0[27][46] + coeff0_28*dmats0[28][46] + coeff0_29*dmats0[29][46] + coeff0_30*dmats0[30][46] + coeff0_31*dmats0[31][46] + coeff0_32*dmats0[32][46] + coeff0_33*dmats0[33][46] + coeff0_34*dmats0[34][46] + coeff0_35*dmats0[35][46] + coeff0_36*dmats0[36][46] + coeff0_37*dmats0[37][46] + coeff0_38*dmats0[38][46] + coeff0_39*dmats0[39][46] + coeff0_40*dmats0[40][46] + coeff0_41*dmats0[41][46] + coeff0_42*dmats0[42][46] + coeff0_43*dmats0[43][46] + coeff0_44*dmats0[44][46] + coeff0_45*dmats0[45][46] + coeff0_46*dmats0[46][46] + coeff0_47*dmats0[47][46] + coeff0_48*dmats0[48][46] + coeff0_49*dmats0[49][46] + coeff0_50*dmats0[50][46] + coeff0_51*dmats0[51][46] + coeff0_52*dmats0[52][46] + coeff0_53*dmats0[53][46] + coeff0_54*dmats0[54][46] + coeff0_55*dmats0[55][46];
          new_coeff0_47 = coeff0_0*dmats0[0][47] + coeff0_1*dmats0[1][47] + coeff0_2*dmats0[2][47] + coeff0_3*dmats0[3][47] + coeff0_4*dmats0[4][47] + coeff0_5*dmats0[5][47] + coeff0_6*dmats0[6][47] + coeff0_7*dmats0[7][47] + coeff0_8*dmats0[8][47] + coeff0_9*dmats0[9][47] + coeff0_10*dmats0[10][47] + coeff0_11*dmats0[11][47] + coeff0_12*dmats0[12][47] + coeff0_13*dmats0[13][47] + coeff0_14*dmats0[14][47] + coeff0_15*dmats0[15][47] + coeff0_16*dmats0[16][47] + coeff0_17*dmats0[17][47] + coeff0_18*dmats0[18][47] + coeff0_19*dmats0[19][47] + coeff0_20*dmats0[20][47] + coeff0_21*dmats0[21][47] + coeff0_22*dmats0[22][47] + coeff0_23*dmats0[23][47] + coeff0_24*dmats0[24][47] + coeff0_25*dmats0[25][47] + coeff0_26*dmats0[26][47] + coeff0_27*dmats0[27][47] + coeff0_28*dmats0[28][47] + coeff0_29*dmats0[29][47] + coeff0_30*dmats0[30][47] + coeff0_31*dmats0[31][47] + coeff0_32*dmats0[32][47] + coeff0_33*dmats0[33][47] + coeff0_34*dmats0[34][47] + coeff0_35*dmats0[35][47] + coeff0_36*dmats0[36][47] + coeff0_37*dmats0[37][47] + coeff0_38*dmats0[38][47] + coeff0_39*dmats0[39][47] + coeff0_40*dmats0[40][47] + coeff0_41*dmats0[41][47] + coeff0_42*dmats0[42][47] + coeff0_43*dmats0[43][47] + coeff0_44*dmats0[44][47] + coeff0_45*dmats0[45][47] + coeff0_46*dmats0[46][47] + coeff0_47*dmats0[47][47] + coeff0_48*dmats0[48][47] + coeff0_49*dmats0[49][47] + coeff0_50*dmats0[50][47] + coeff0_51*dmats0[51][47] + coeff0_52*dmats0[52][47] + coeff0_53*dmats0[53][47] + coeff0_54*dmats0[54][47] + coeff0_55*dmats0[55][47];
          new_coeff0_48 = coeff0_0*dmats0[0][48] + coeff0_1*dmats0[1][48] + coeff0_2*dmats0[2][48] + coeff0_3*dmats0[3][48] + coeff0_4*dmats0[4][48] + coeff0_5*dmats0[5][48] + coeff0_6*dmats0[6][48] + coeff0_7*dmats0[7][48] + coeff0_8*dmats0[8][48] + coeff0_9*dmats0[9][48] + coeff0_10*dmats0[10][48] + coeff0_11*dmats0[11][48] + coeff0_12*dmats0[12][48] + coeff0_13*dmats0[13][48] + coeff0_14*dmats0[14][48] + coeff0_15*dmats0[15][48] + coeff0_16*dmats0[16][48] + coeff0_17*dmats0[17][48] + coeff0_18*dmats0[18][48] + coeff0_19*dmats0[19][48] + coeff0_20*dmats0[20][48] + coeff0_21*dmats0[21][48] + coeff0_22*dmats0[22][48] + coeff0_23*dmats0[23][48] + coeff0_24*dmats0[24][48] + coeff0_25*dmats0[25][48] + coeff0_26*dmats0[26][48] + coeff0_27*dmats0[27][48] + coeff0_28*dmats0[28][48] + coeff0_29*dmats0[29][48] + coeff0_30*dmats0[30][48] + coeff0_31*dmats0[31][48] + coeff0_32*dmats0[32][48] + coeff0_33*dmats0[33][48] + coeff0_34*dmats0[34][48] + coeff0_35*dmats0[35][48] + coeff0_36*dmats0[36][48] + coeff0_37*dmats0[37][48] + coeff0_38*dmats0[38][48] + coeff0_39*dmats0[39][48] + coeff0_40*dmats0[40][48] + coeff0_41*dmats0[41][48] + coeff0_42*dmats0[42][48] + coeff0_43*dmats0[43][48] + coeff0_44*dmats0[44][48] + coeff0_45*dmats0[45][48] + coeff0_46*dmats0[46][48] + coeff0_47*dmats0[47][48] + coeff0_48*dmats0[48][48] + coeff0_49*dmats0[49][48] + coeff0_50*dmats0[50][48] + coeff0_51*dmats0[51][48] + coeff0_52*dmats0[52][48] + coeff0_53*dmats0[53][48] + coeff0_54*dmats0[54][48] + coeff0_55*dmats0[55][48];
          new_coeff0_49 = coeff0_0*dmats0[0][49] + coeff0_1*dmats0[1][49] + coeff0_2*dmats0[2][49] + coeff0_3*dmats0[3][49] + coeff0_4*dmats0[4][49] + coeff0_5*dmats0[5][49] + coeff0_6*dmats0[6][49] + coeff0_7*dmats0[7][49] + coeff0_8*dmats0[8][49] + coeff0_9*dmats0[9][49] + coeff0_10*dmats0[10][49] + coeff0_11*dmats0[11][49] + coeff0_12*dmats0[12][49] + coeff0_13*dmats0[13][49] + coeff0_14*dmats0[14][49] + coeff0_15*dmats0[15][49] + coeff0_16*dmats0[16][49] + coeff0_17*dmats0[17][49] + coeff0_18*dmats0[18][49] + coeff0_19*dmats0[19][49] + coeff0_20*dmats0[20][49] + coeff0_21*dmats0[21][49] + coeff0_22*dmats0[22][49] + coeff0_23*dmats0[23][49] + coeff0_24*dmats0[24][49] + coeff0_25*dmats0[25][49] + coeff0_26*dmats0[26][49] + coeff0_27*dmats0[27][49] + coeff0_28*dmats0[28][49] + coeff0_29*dmats0[29][49] + coeff0_30*dmats0[30][49] + coeff0_31*dmats0[31][49] + coeff0_32*dmats0[32][49] + coeff0_33*dmats0[33][49] + coeff0_34*dmats0[34][49] + coeff0_35*dmats0[35][49] + coeff0_36*dmats0[36][49] + coeff0_37*dmats0[37][49] + coeff0_38*dmats0[38][49] + coeff0_39*dmats0[39][49] + coeff0_40*dmats0[40][49] + coeff0_41*dmats0[41][49] + coeff0_42*dmats0[42][49] + coeff0_43*dmats0[43][49] + coeff0_44*dmats0[44][49] + coeff0_45*dmats0[45][49] + coeff0_46*dmats0[46][49] + coeff0_47*dmats0[47][49] + coeff0_48*dmats0[48][49] + coeff0_49*dmats0[49][49] + coeff0_50*dmats0[50][49] + coeff0_51*dmats0[51][49] + coeff0_52*dmats0[52][49] + coeff0_53*dmats0[53][49] + coeff0_54*dmats0[54][49] + coeff0_55*dmats0[55][49];
          new_coeff0_50 = coeff0_0*dmats0[0][50] + coeff0_1*dmats0[1][50] + coeff0_2*dmats0[2][50] + coeff0_3*dmats0[3][50] + coeff0_4*dmats0[4][50] + coeff0_5*dmats0[5][50] + coeff0_6*dmats0[6][50] + coeff0_7*dmats0[7][50] + coeff0_8*dmats0[8][50] + coeff0_9*dmats0[9][50] + coeff0_10*dmats0[10][50] + coeff0_11*dmats0[11][50] + coeff0_12*dmats0[12][50] + coeff0_13*dmats0[13][50] + coeff0_14*dmats0[14][50] + coeff0_15*dmats0[15][50] + coeff0_16*dmats0[16][50] + coeff0_17*dmats0[17][50] + coeff0_18*dmats0[18][50] + coeff0_19*dmats0[19][50] + coeff0_20*dmats0[20][50] + coeff0_21*dmats0[21][50] + coeff0_22*dmats0[22][50] + coeff0_23*dmats0[23][50] + coeff0_24*dmats0[24][50] + coeff0_25*dmats0[25][50] + coeff0_26*dmats0[26][50] + coeff0_27*dmats0[27][50] + coeff0_28*dmats0[28][50] + coeff0_29*dmats0[29][50] + coeff0_30*dmats0[30][50] + coeff0_31*dmats0[31][50] + coeff0_32*dmats0[32][50] + coeff0_33*dmats0[33][50] + coeff0_34*dmats0[34][50] + coeff0_35*dmats0[35][50] + coeff0_36*dmats0[36][50] + coeff0_37*dmats0[37][50] + coeff0_38*dmats0[38][50] + coeff0_39*dmats0[39][50] + coeff0_40*dmats0[40][50] + coeff0_41*dmats0[41][50] + coeff0_42*dmats0[42][50] + coeff0_43*dmats0[43][50] + coeff0_44*dmats0[44][50] + coeff0_45*dmats0[45][50] + coeff0_46*dmats0[46][50] + coeff0_47*dmats0[47][50] + coeff0_48*dmats0[48][50] + coeff0_49*dmats0[49][50] + coeff0_50*dmats0[50][50] + coeff0_51*dmats0[51][50] + coeff0_52*dmats0[52][50] + coeff0_53*dmats0[53][50] + coeff0_54*dmats0[54][50] + coeff0_55*dmats0[55][50];
          new_coeff0_51 = coeff0_0*dmats0[0][51] + coeff0_1*dmats0[1][51] + coeff0_2*dmats0[2][51] + coeff0_3*dmats0[3][51] + coeff0_4*dmats0[4][51] + coeff0_5*dmats0[5][51] + coeff0_6*dmats0[6][51] + coeff0_7*dmats0[7][51] + coeff0_8*dmats0[8][51] + coeff0_9*dmats0[9][51] + coeff0_10*dmats0[10][51] + coeff0_11*dmats0[11][51] + coeff0_12*dmats0[12][51] + coeff0_13*dmats0[13][51] + coeff0_14*dmats0[14][51] + coeff0_15*dmats0[15][51] + coeff0_16*dmats0[16][51] + coeff0_17*dmats0[17][51] + coeff0_18*dmats0[18][51] + coeff0_19*dmats0[19][51] + coeff0_20*dmats0[20][51] + coeff0_21*dmats0[21][51] + coeff0_22*dmats0[22][51] + coeff0_23*dmats0[23][51] + coeff0_24*dmats0[24][51] + coeff0_25*dmats0[25][51] + coeff0_26*dmats0[26][51] + coeff0_27*dmats0[27][51] + coeff0_28*dmats0[28][51] + coeff0_29*dmats0[29][51] + coeff0_30*dmats0[30][51] + coeff0_31*dmats0[31][51] + coeff0_32*dmats0[32][51] + coeff0_33*dmats0[33][51] + coeff0_34*dmats0[34][51] + coeff0_35*dmats0[35][51] + coeff0_36*dmats0[36][51] + coeff0_37*dmats0[37][51] + coeff0_38*dmats0[38][51] + coeff0_39*dmats0[39][51] + coeff0_40*dmats0[40][51] + coeff0_41*dmats0[41][51] + coeff0_42*dmats0[42][51] + coeff0_43*dmats0[43][51] + coeff0_44*dmats0[44][51] + coeff0_45*dmats0[45][51] + coeff0_46*dmats0[46][51] + coeff0_47*dmats0[47][51] + coeff0_48*dmats0[48][51] + coeff0_49*dmats0[49][51] + coeff0_50*dmats0[50][51] + coeff0_51*dmats0[51][51] + coeff0_52*dmats0[52][51] + coeff0_53*dmats0[53][51] + coeff0_54*dmats0[54][51] + coeff0_55*dmats0[55][51];
          new_coeff0_52 = coeff0_0*dmats0[0][52] + coeff0_1*dmats0[1][52] + coeff0_2*dmats0[2][52] + coeff0_3*dmats0[3][52] + coeff0_4*dmats0[4][52] + coeff0_5*dmats0[5][52] + coeff0_6*dmats0[6][52] + coeff0_7*dmats0[7][52] + coeff0_8*dmats0[8][52] + coeff0_9*dmats0[9][52] + coeff0_10*dmats0[10][52] + coeff0_11*dmats0[11][52] + coeff0_12*dmats0[12][52] + coeff0_13*dmats0[13][52] + coeff0_14*dmats0[14][52] + coeff0_15*dmats0[15][52] + coeff0_16*dmats0[16][52] + coeff0_17*dmats0[17][52] + coeff0_18*dmats0[18][52] + coeff0_19*dmats0[19][52] + coeff0_20*dmats0[20][52] + coeff0_21*dmats0[21][52] + coeff0_22*dmats0[22][52] + coeff0_23*dmats0[23][52] + coeff0_24*dmats0[24][52] + coeff0_25*dmats0[25][52] + coeff0_26*dmats0[26][52] + coeff0_27*dmats0[27][52] + coeff0_28*dmats0[28][52] + coeff0_29*dmats0[29][52] + coeff0_30*dmats0[30][52] + coeff0_31*dmats0[31][52] + coeff0_32*dmats0[32][52] + coeff0_33*dmats0[33][52] + coeff0_34*dmats0[34][52] + coeff0_35*dmats0[35][52] + coeff0_36*dmats0[36][52] + coeff0_37*dmats0[37][52] + coeff0_38*dmats0[38][52] + coeff0_39*dmats0[39][52] + coeff0_40*dmats0[40][52] + coeff0_41*dmats0[41][52] + coeff0_42*dmats0[42][52] + coeff0_43*dmats0[43][52] + coeff0_44*dmats0[44][52] + coeff0_45*dmats0[45][52] + coeff0_46*dmats0[46][52] + coeff0_47*dmats0[47][52] + coeff0_48*dmats0[48][52] + coeff0_49*dmats0[49][52] + coeff0_50*dmats0[50][52] + coeff0_51*dmats0[51][52] + coeff0_52*dmats0[52][52] + coeff0_53*dmats0[53][52] + coeff0_54*dmats0[54][52] + coeff0_55*dmats0[55][52];
          new_coeff0_53 = coeff0_0*dmats0[0][53] + coeff0_1*dmats0[1][53] + coeff0_2*dmats0[2][53] + coeff0_3*dmats0[3][53] + coeff0_4*dmats0[4][53] + coeff0_5*dmats0[5][53] + coeff0_6*dmats0[6][53] + coeff0_7*dmats0[7][53] + coeff0_8*dmats0[8][53] + coeff0_9*dmats0[9][53] + coeff0_10*dmats0[10][53] + coeff0_11*dmats0[11][53] + coeff0_12*dmats0[12][53] + coeff0_13*dmats0[13][53] + coeff0_14*dmats0[14][53] + coeff0_15*dmats0[15][53] + coeff0_16*dmats0[16][53] + coeff0_17*dmats0[17][53] + coeff0_18*dmats0[18][53] + coeff0_19*dmats0[19][53] + coeff0_20*dmats0[20][53] + coeff0_21*dmats0[21][53] + coeff0_22*dmats0[22][53] + coeff0_23*dmats0[23][53] + coeff0_24*dmats0[24][53] + coeff0_25*dmats0[25][53] + coeff0_26*dmats0[26][53] + coeff0_27*dmats0[27][53] + coeff0_28*dmats0[28][53] + coeff0_29*dmats0[29][53] + coeff0_30*dmats0[30][53] + coeff0_31*dmats0[31][53] + coeff0_32*dmats0[32][53] + coeff0_33*dmats0[33][53] + coeff0_34*dmats0[34][53] + coeff0_35*dmats0[35][53] + coeff0_36*dmats0[36][53] + coeff0_37*dmats0[37][53] + coeff0_38*dmats0[38][53] + coeff0_39*dmats0[39][53] + coeff0_40*dmats0[40][53] + coeff0_41*dmats0[41][53] + coeff0_42*dmats0[42][53] + coeff0_43*dmats0[43][53] + coeff0_44*dmats0[44][53] + coeff0_45*dmats0[45][53] + coeff0_46*dmats0[46][53] + coeff0_47*dmats0[47][53] + coeff0_48*dmats0[48][53] + coeff0_49*dmats0[49][53] + coeff0_50*dmats0[50][53] + coeff0_51*dmats0[51][53] + coeff0_52*dmats0[52][53] + coeff0_53*dmats0[53][53] + coeff0_54*dmats0[54][53] + coeff0_55*dmats0[55][53];
          new_coeff0_54 = coeff0_0*dmats0[0][54] + coeff0_1*dmats0[1][54] + coeff0_2*dmats0[2][54] + coeff0_3*dmats0[3][54] + coeff0_4*dmats0[4][54] + coeff0_5*dmats0[5][54] + coeff0_6*dmats0[6][54] + coeff0_7*dmats0[7][54] + coeff0_8*dmats0[8][54] + coeff0_9*dmats0[9][54] + coeff0_10*dmats0[10][54] + coeff0_11*dmats0[11][54] + coeff0_12*dmats0[12][54] + coeff0_13*dmats0[13][54] + coeff0_14*dmats0[14][54] + coeff0_15*dmats0[15][54] + coeff0_16*dmats0[16][54] + coeff0_17*dmats0[17][54] + coeff0_18*dmats0[18][54] + coeff0_19*dmats0[19][54] + coeff0_20*dmats0[20][54] + coeff0_21*dmats0[21][54] + coeff0_22*dmats0[22][54] + coeff0_23*dmats0[23][54] + coeff0_24*dmats0[24][54] + coeff0_25*dmats0[25][54] + coeff0_26*dmats0[26][54] + coeff0_27*dmats0[27][54] + coeff0_28*dmats0[28][54] + coeff0_29*dmats0[29][54] + coeff0_30*dmats0[30][54] + coeff0_31*dmats0[31][54] + coeff0_32*dmats0[32][54] + coeff0_33*dmats0[33][54] + coeff0_34*dmats0[34][54] + coeff0_35*dmats0[35][54] + coeff0_36*dmats0[36][54] + coeff0_37*dmats0[37][54] + coeff0_38*dmats0[38][54] + coeff0_39*dmats0[39][54] + coeff0_40*dmats0[40][54] + coeff0_41*dmats0[41][54] + coeff0_42*dmats0[42][54] + coeff0_43*dmats0[43][54] + coeff0_44*dmats0[44][54] + coeff0_45*dmats0[45][54] + coeff0_46*dmats0[46][54] + coeff0_47*dmats0[47][54] + coeff0_48*dmats0[48][54] + coeff0_49*dmats0[49][54] + coeff0_50*dmats0[50][54] + coeff0_51*dmats0[51][54] + coeff0_52*dmats0[52][54] + coeff0_53*dmats0[53][54] + coeff0_54*dmats0[54][54] + coeff0_55*dmats0[55][54];
          new_coeff0_55 = coeff0_0*dmats0[0][55] + coeff0_1*dmats0[1][55] + coeff0_2*dmats0[2][55] + coeff0_3*dmats0[3][55] + coeff0_4*dmats0[4][55] + coeff0_5*dmats0[5][55] + coeff0_6*dmats0[6][55] + coeff0_7*dmats0[7][55] + coeff0_8*dmats0[8][55] + coeff0_9*dmats0[9][55] + coeff0_10*dmats0[10][55] + coeff0_11*dmats0[11][55] + coeff0_12*dmats0[12][55] + coeff0_13*dmats0[13][55] + coeff0_14*dmats0[14][55] + coeff0_15*dmats0[15][55] + coeff0_16*dmats0[16][55] + coeff0_17*dmats0[17][55] + coeff0_18*dmats0[18][55] + coeff0_19*dmats0[19][55] + coeff0_20*dmats0[20][55] + coeff0_21*dmats0[21][55] + coeff0_22*dmats0[22][55] + coeff0_23*dmats0[23][55] + coeff0_24*dmats0[24][55] + coeff0_25*dmats0[25][55] + coeff0_26*dmats0[26][55] + coeff0_27*dmats0[27][55] + coeff0_28*dmats0[28][55] + coeff0_29*dmats0[29][55] + coeff0_30*dmats0[30][55] + coeff0_31*dmats0[31][55] + coeff0_32*dmats0[32][55] + coeff0_33*dmats0[33][55] + coeff0_34*dmats0[34][55] + coeff0_35*dmats0[35][55] + coeff0_36*dmats0[36][55] + coeff0_37*dmats0[37][55] + coeff0_38*dmats0[38][55] + coeff0_39*dmats0[39][55] + coeff0_40*dmats0[40][55] + coeff0_41*dmats0[41][55] + coeff0_42*dmats0[42][55] + coeff0_43*dmats0[43][55] + coeff0_44*dmats0[44][55] + coeff0_45*dmats0[45][55] + coeff0_46*dmats0[46][55] + coeff0_47*dmats0[47][55] + coeff0_48*dmats0[48][55] + coeff0_49*dmats0[49][55] + coeff0_50*dmats0[50][55] + coeff0_51*dmats0[51][55] + coeff0_52*dmats0[52][55] + coeff0_53*dmats0[53][55] + coeff0_54*dmats0[54][55] + coeff0_55*dmats0[55][55];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0] + coeff0_10*dmats1[10][0] + coeff0_11*dmats1[11][0] + coeff0_12*dmats1[12][0] + coeff0_13*dmats1[13][0] + coeff0_14*dmats1[14][0] + coeff0_15*dmats1[15][0] + coeff0_16*dmats1[16][0] + coeff0_17*dmats1[17][0] + coeff0_18*dmats1[18][0] + coeff0_19*dmats1[19][0] + coeff0_20*dmats1[20][0] + coeff0_21*dmats1[21][0] + coeff0_22*dmats1[22][0] + coeff0_23*dmats1[23][0] + coeff0_24*dmats1[24][0] + coeff0_25*dmats1[25][0] + coeff0_26*dmats1[26][0] + coeff0_27*dmats1[27][0] + coeff0_28*dmats1[28][0] + coeff0_29*dmats1[29][0] + coeff0_30*dmats1[30][0] + coeff0_31*dmats1[31][0] + coeff0_32*dmats1[32][0] + coeff0_33*dmats1[33][0] + coeff0_34*dmats1[34][0] + coeff0_35*dmats1[35][0] + coeff0_36*dmats1[36][0] + coeff0_37*dmats1[37][0] + coeff0_38*dmats1[38][0] + coeff0_39*dmats1[39][0] + coeff0_40*dmats1[40][0] + coeff0_41*dmats1[41][0] + coeff0_42*dmats1[42][0] + coeff0_43*dmats1[43][0] + coeff0_44*dmats1[44][0] + coeff0_45*dmats1[45][0] + coeff0_46*dmats1[46][0] + coeff0_47*dmats1[47][0] + coeff0_48*dmats1[48][0] + coeff0_49*dmats1[49][0] + coeff0_50*dmats1[50][0] + coeff0_51*dmats1[51][0] + coeff0_52*dmats1[52][0] + coeff0_53*dmats1[53][0] + coeff0_54*dmats1[54][0] + coeff0_55*dmats1[55][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1] + coeff0_10*dmats1[10][1] + coeff0_11*dmats1[11][1] + coeff0_12*dmats1[12][1] + coeff0_13*dmats1[13][1] + coeff0_14*dmats1[14][1] + coeff0_15*dmats1[15][1] + coeff0_16*dmats1[16][1] + coeff0_17*dmats1[17][1] + coeff0_18*dmats1[18][1] + coeff0_19*dmats1[19][1] + coeff0_20*dmats1[20][1] + coeff0_21*dmats1[21][1] + coeff0_22*dmats1[22][1] + coeff0_23*dmats1[23][1] + coeff0_24*dmats1[24][1] + coeff0_25*dmats1[25][1] + coeff0_26*dmats1[26][1] + coeff0_27*dmats1[27][1] + coeff0_28*dmats1[28][1] + coeff0_29*dmats1[29][1] + coeff0_30*dmats1[30][1] + coeff0_31*dmats1[31][1] + coeff0_32*dmats1[32][1] + coeff0_33*dmats1[33][1] + coeff0_34*dmats1[34][1] + coeff0_35*dmats1[35][1] + coeff0_36*dmats1[36][1] + coeff0_37*dmats1[37][1] + coeff0_38*dmats1[38][1] + coeff0_39*dmats1[39][1] + coeff0_40*dmats1[40][1] + coeff0_41*dmats1[41][1] + coeff0_42*dmats1[42][1] + coeff0_43*dmats1[43][1] + coeff0_44*dmats1[44][1] + coeff0_45*dmats1[45][1] + coeff0_46*dmats1[46][1] + coeff0_47*dmats1[47][1] + coeff0_48*dmats1[48][1] + coeff0_49*dmats1[49][1] + coeff0_50*dmats1[50][1] + coeff0_51*dmats1[51][1] + coeff0_52*dmats1[52][1] + coeff0_53*dmats1[53][1] + coeff0_54*dmats1[54][1] + coeff0_55*dmats1[55][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2] + coeff0_10*dmats1[10][2] + coeff0_11*dmats1[11][2] + coeff0_12*dmats1[12][2] + coeff0_13*dmats1[13][2] + coeff0_14*dmats1[14][2] + coeff0_15*dmats1[15][2] + coeff0_16*dmats1[16][2] + coeff0_17*dmats1[17][2] + coeff0_18*dmats1[18][2] + coeff0_19*dmats1[19][2] + coeff0_20*dmats1[20][2] + coeff0_21*dmats1[21][2] + coeff0_22*dmats1[22][2] + coeff0_23*dmats1[23][2] + coeff0_24*dmats1[24][2] + coeff0_25*dmats1[25][2] + coeff0_26*dmats1[26][2] + coeff0_27*dmats1[27][2] + coeff0_28*dmats1[28][2] + coeff0_29*dmats1[29][2] + coeff0_30*dmats1[30][2] + coeff0_31*dmats1[31][2] + coeff0_32*dmats1[32][2] + coeff0_33*dmats1[33][2] + coeff0_34*dmats1[34][2] + coeff0_35*dmats1[35][2] + coeff0_36*dmats1[36][2] + coeff0_37*dmats1[37][2] + coeff0_38*dmats1[38][2] + coeff0_39*dmats1[39][2] + coeff0_40*dmats1[40][2] + coeff0_41*dmats1[41][2] + coeff0_42*dmats1[42][2] + coeff0_43*dmats1[43][2] + coeff0_44*dmats1[44][2] + coeff0_45*dmats1[45][2] + coeff0_46*dmats1[46][2] + coeff0_47*dmats1[47][2] + coeff0_48*dmats1[48][2] + coeff0_49*dmats1[49][2] + coeff0_50*dmats1[50][2] + coeff0_51*dmats1[51][2] + coeff0_52*dmats1[52][2] + coeff0_53*dmats1[53][2] + coeff0_54*dmats1[54][2] + coeff0_55*dmats1[55][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3] + coeff0_10*dmats1[10][3] + coeff0_11*dmats1[11][3] + coeff0_12*dmats1[12][3] + coeff0_13*dmats1[13][3] + coeff0_14*dmats1[14][3] + coeff0_15*dmats1[15][3] + coeff0_16*dmats1[16][3] + coeff0_17*dmats1[17][3] + coeff0_18*dmats1[18][3] + coeff0_19*dmats1[19][3] + coeff0_20*dmats1[20][3] + coeff0_21*dmats1[21][3] + coeff0_22*dmats1[22][3] + coeff0_23*dmats1[23][3] + coeff0_24*dmats1[24][3] + coeff0_25*dmats1[25][3] + coeff0_26*dmats1[26][3] + coeff0_27*dmats1[27][3] + coeff0_28*dmats1[28][3] + coeff0_29*dmats1[29][3] + coeff0_30*dmats1[30][3] + coeff0_31*dmats1[31][3] + coeff0_32*dmats1[32][3] + coeff0_33*dmats1[33][3] + coeff0_34*dmats1[34][3] + coeff0_35*dmats1[35][3] + coeff0_36*dmats1[36][3] + coeff0_37*dmats1[37][3] + coeff0_38*dmats1[38][3] + coeff0_39*dmats1[39][3] + coeff0_40*dmats1[40][3] + coeff0_41*dmats1[41][3] + coeff0_42*dmats1[42][3] + coeff0_43*dmats1[43][3] + coeff0_44*dmats1[44][3] + coeff0_45*dmats1[45][3] + coeff0_46*dmats1[46][3] + coeff0_47*dmats1[47][3] + coeff0_48*dmats1[48][3] + coeff0_49*dmats1[49][3] + coeff0_50*dmats1[50][3] + coeff0_51*dmats1[51][3] + coeff0_52*dmats1[52][3] + coeff0_53*dmats1[53][3] + coeff0_54*dmats1[54][3] + coeff0_55*dmats1[55][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4] + coeff0_10*dmats1[10][4] + coeff0_11*dmats1[11][4] + coeff0_12*dmats1[12][4] + coeff0_13*dmats1[13][4] + coeff0_14*dmats1[14][4] + coeff0_15*dmats1[15][4] + coeff0_16*dmats1[16][4] + coeff0_17*dmats1[17][4] + coeff0_18*dmats1[18][4] + coeff0_19*dmats1[19][4] + coeff0_20*dmats1[20][4] + coeff0_21*dmats1[21][4] + coeff0_22*dmats1[22][4] + coeff0_23*dmats1[23][4] + coeff0_24*dmats1[24][4] + coeff0_25*dmats1[25][4] + coeff0_26*dmats1[26][4] + coeff0_27*dmats1[27][4] + coeff0_28*dmats1[28][4] + coeff0_29*dmats1[29][4] + coeff0_30*dmats1[30][4] + coeff0_31*dmats1[31][4] + coeff0_32*dmats1[32][4] + coeff0_33*dmats1[33][4] + coeff0_34*dmats1[34][4] + coeff0_35*dmats1[35][4] + coeff0_36*dmats1[36][4] + coeff0_37*dmats1[37][4] + coeff0_38*dmats1[38][4] + coeff0_39*dmats1[39][4] + coeff0_40*dmats1[40][4] + coeff0_41*dmats1[41][4] + coeff0_42*dmats1[42][4] + coeff0_43*dmats1[43][4] + coeff0_44*dmats1[44][4] + coeff0_45*dmats1[45][4] + coeff0_46*dmats1[46][4] + coeff0_47*dmats1[47][4] + coeff0_48*dmats1[48][4] + coeff0_49*dmats1[49][4] + coeff0_50*dmats1[50][4] + coeff0_51*dmats1[51][4] + coeff0_52*dmats1[52][4] + coeff0_53*dmats1[53][4] + coeff0_54*dmats1[54][4] + coeff0_55*dmats1[55][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5] + coeff0_10*dmats1[10][5] + coeff0_11*dmats1[11][5] + coeff0_12*dmats1[12][5] + coeff0_13*dmats1[13][5] + coeff0_14*dmats1[14][5] + coeff0_15*dmats1[15][5] + coeff0_16*dmats1[16][5] + coeff0_17*dmats1[17][5] + coeff0_18*dmats1[18][5] + coeff0_19*dmats1[19][5] + coeff0_20*dmats1[20][5] + coeff0_21*dmats1[21][5] + coeff0_22*dmats1[22][5] + coeff0_23*dmats1[23][5] + coeff0_24*dmats1[24][5] + coeff0_25*dmats1[25][5] + coeff0_26*dmats1[26][5] + coeff0_27*dmats1[27][5] + coeff0_28*dmats1[28][5] + coeff0_29*dmats1[29][5] + coeff0_30*dmats1[30][5] + coeff0_31*dmats1[31][5] + coeff0_32*dmats1[32][5] + coeff0_33*dmats1[33][5] + coeff0_34*dmats1[34][5] + coeff0_35*dmats1[35][5] + coeff0_36*dmats1[36][5] + coeff0_37*dmats1[37][5] + coeff0_38*dmats1[38][5] + coeff0_39*dmats1[39][5] + coeff0_40*dmats1[40][5] + coeff0_41*dmats1[41][5] + coeff0_42*dmats1[42][5] + coeff0_43*dmats1[43][5] + coeff0_44*dmats1[44][5] + coeff0_45*dmats1[45][5] + coeff0_46*dmats1[46][5] + coeff0_47*dmats1[47][5] + coeff0_48*dmats1[48][5] + coeff0_49*dmats1[49][5] + coeff0_50*dmats1[50][5] + coeff0_51*dmats1[51][5] + coeff0_52*dmats1[52][5] + coeff0_53*dmats1[53][5] + coeff0_54*dmats1[54][5] + coeff0_55*dmats1[55][5];
          new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6] + coeff0_10*dmats1[10][6] + coeff0_11*dmats1[11][6] + coeff0_12*dmats1[12][6] + coeff0_13*dmats1[13][6] + coeff0_14*dmats1[14][6] + coeff0_15*dmats1[15][6] + coeff0_16*dmats1[16][6] + coeff0_17*dmats1[17][6] + coeff0_18*dmats1[18][6] + coeff0_19*dmats1[19][6] + coeff0_20*dmats1[20][6] + coeff0_21*dmats1[21][6] + coeff0_22*dmats1[22][6] + coeff0_23*dmats1[23][6] + coeff0_24*dmats1[24][6] + coeff0_25*dmats1[25][6] + coeff0_26*dmats1[26][6] + coeff0_27*dmats1[27][6] + coeff0_28*dmats1[28][6] + coeff0_29*dmats1[29][6] + coeff0_30*dmats1[30][6] + coeff0_31*dmats1[31][6] + coeff0_32*dmats1[32][6] + coeff0_33*dmats1[33][6] + coeff0_34*dmats1[34][6] + coeff0_35*dmats1[35][6] + coeff0_36*dmats1[36][6] + coeff0_37*dmats1[37][6] + coeff0_38*dmats1[38][6] + coeff0_39*dmats1[39][6] + coeff0_40*dmats1[40][6] + coeff0_41*dmats1[41][6] + coeff0_42*dmats1[42][6] + coeff0_43*dmats1[43][6] + coeff0_44*dmats1[44][6] + coeff0_45*dmats1[45][6] + coeff0_46*dmats1[46][6] + coeff0_47*dmats1[47][6] + coeff0_48*dmats1[48][6] + coeff0_49*dmats1[49][6] + coeff0_50*dmats1[50][6] + coeff0_51*dmats1[51][6] + coeff0_52*dmats1[52][6] + coeff0_53*dmats1[53][6] + coeff0_54*dmats1[54][6] + coeff0_55*dmats1[55][6];
          new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7] + coeff0_10*dmats1[10][7] + coeff0_11*dmats1[11][7] + coeff0_12*dmats1[12][7] + coeff0_13*dmats1[13][7] + coeff0_14*dmats1[14][7] + coeff0_15*dmats1[15][7] + coeff0_16*dmats1[16][7] + coeff0_17*dmats1[17][7] + coeff0_18*dmats1[18][7] + coeff0_19*dmats1[19][7] + coeff0_20*dmats1[20][7] + coeff0_21*dmats1[21][7] + coeff0_22*dmats1[22][7] + coeff0_23*dmats1[23][7] + coeff0_24*dmats1[24][7] + coeff0_25*dmats1[25][7] + coeff0_26*dmats1[26][7] + coeff0_27*dmats1[27][7] + coeff0_28*dmats1[28][7] + coeff0_29*dmats1[29][7] + coeff0_30*dmats1[30][7] + coeff0_31*dmats1[31][7] + coeff0_32*dmats1[32][7] + coeff0_33*dmats1[33][7] + coeff0_34*dmats1[34][7] + coeff0_35*dmats1[35][7] + coeff0_36*dmats1[36][7] + coeff0_37*dmats1[37][7] + coeff0_38*dmats1[38][7] + coeff0_39*dmats1[39][7] + coeff0_40*dmats1[40][7] + coeff0_41*dmats1[41][7] + coeff0_42*dmats1[42][7] + coeff0_43*dmats1[43][7] + coeff0_44*dmats1[44][7] + coeff0_45*dmats1[45][7] + coeff0_46*dmats1[46][7] + coeff0_47*dmats1[47][7] + coeff0_48*dmats1[48][7] + coeff0_49*dmats1[49][7] + coeff0_50*dmats1[50][7] + coeff0_51*dmats1[51][7] + coeff0_52*dmats1[52][7] + coeff0_53*dmats1[53][7] + coeff0_54*dmats1[54][7] + coeff0_55*dmats1[55][7];
          new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8] + coeff0_10*dmats1[10][8] + coeff0_11*dmats1[11][8] + coeff0_12*dmats1[12][8] + coeff0_13*dmats1[13][8] + coeff0_14*dmats1[14][8] + coeff0_15*dmats1[15][8] + coeff0_16*dmats1[16][8] + coeff0_17*dmats1[17][8] + coeff0_18*dmats1[18][8] + coeff0_19*dmats1[19][8] + coeff0_20*dmats1[20][8] + coeff0_21*dmats1[21][8] + coeff0_22*dmats1[22][8] + coeff0_23*dmats1[23][8] + coeff0_24*dmats1[24][8] + coeff0_25*dmats1[25][8] + coeff0_26*dmats1[26][8] + coeff0_27*dmats1[27][8] + coeff0_28*dmats1[28][8] + coeff0_29*dmats1[29][8] + coeff0_30*dmats1[30][8] + coeff0_31*dmats1[31][8] + coeff0_32*dmats1[32][8] + coeff0_33*dmats1[33][8] + coeff0_34*dmats1[34][8] + coeff0_35*dmats1[35][8] + coeff0_36*dmats1[36][8] + coeff0_37*dmats1[37][8] + coeff0_38*dmats1[38][8] + coeff0_39*dmats1[39][8] + coeff0_40*dmats1[40][8] + coeff0_41*dmats1[41][8] + coeff0_42*dmats1[42][8] + coeff0_43*dmats1[43][8] + coeff0_44*dmats1[44][8] + coeff0_45*dmats1[45][8] + coeff0_46*dmats1[46][8] + coeff0_47*dmats1[47][8] + coeff0_48*dmats1[48][8] + coeff0_49*dmats1[49][8] + coeff0_50*dmats1[50][8] + coeff0_51*dmats1[51][8] + coeff0_52*dmats1[52][8] + coeff0_53*dmats1[53][8] + coeff0_54*dmats1[54][8] + coeff0_55*dmats1[55][8];
          new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9] + coeff0_10*dmats1[10][9] + coeff0_11*dmats1[11][9] + coeff0_12*dmats1[12][9] + coeff0_13*dmats1[13][9] + coeff0_14*dmats1[14][9] + coeff0_15*dmats1[15][9] + coeff0_16*dmats1[16][9] + coeff0_17*dmats1[17][9] + coeff0_18*dmats1[18][9] + coeff0_19*dmats1[19][9] + coeff0_20*dmats1[20][9] + coeff0_21*dmats1[21][9] + coeff0_22*dmats1[22][9] + coeff0_23*dmats1[23][9] + coeff0_24*dmats1[24][9] + coeff0_25*dmats1[25][9] + coeff0_26*dmats1[26][9] + coeff0_27*dmats1[27][9] + coeff0_28*dmats1[28][9] + coeff0_29*dmats1[29][9] + coeff0_30*dmats1[30][9] + coeff0_31*dmats1[31][9] + coeff0_32*dmats1[32][9] + coeff0_33*dmats1[33][9] + coeff0_34*dmats1[34][9] + coeff0_35*dmats1[35][9] + coeff0_36*dmats1[36][9] + coeff0_37*dmats1[37][9] + coeff0_38*dmats1[38][9] + coeff0_39*dmats1[39][9] + coeff0_40*dmats1[40][9] + coeff0_41*dmats1[41][9] + coeff0_42*dmats1[42][9] + coeff0_43*dmats1[43][9] + coeff0_44*dmats1[44][9] + coeff0_45*dmats1[45][9] + coeff0_46*dmats1[46][9] + coeff0_47*dmats1[47][9] + coeff0_48*dmats1[48][9] + coeff0_49*dmats1[49][9] + coeff0_50*dmats1[50][9] + coeff0_51*dmats1[51][9] + coeff0_52*dmats1[52][9] + coeff0_53*dmats1[53][9] + coeff0_54*dmats1[54][9] + coeff0_55*dmats1[55][9];
          new_coeff0_10 = coeff0_0*dmats1[0][10] + coeff0_1*dmats1[1][10] + coeff0_2*dmats1[2][10] + coeff0_3*dmats1[3][10] + coeff0_4*dmats1[4][10] + coeff0_5*dmats1[5][10] + coeff0_6*dmats1[6][10] + coeff0_7*dmats1[7][10] + coeff0_8*dmats1[8][10] + coeff0_9*dmats1[9][10] + coeff0_10*dmats1[10][10] + coeff0_11*dmats1[11][10] + coeff0_12*dmats1[12][10] + coeff0_13*dmats1[13][10] + coeff0_14*dmats1[14][10] + coeff0_15*dmats1[15][10] + coeff0_16*dmats1[16][10] + coeff0_17*dmats1[17][10] + coeff0_18*dmats1[18][10] + coeff0_19*dmats1[19][10] + coeff0_20*dmats1[20][10] + coeff0_21*dmats1[21][10] + coeff0_22*dmats1[22][10] + coeff0_23*dmats1[23][10] + coeff0_24*dmats1[24][10] + coeff0_25*dmats1[25][10] + coeff0_26*dmats1[26][10] + coeff0_27*dmats1[27][10] + coeff0_28*dmats1[28][10] + coeff0_29*dmats1[29][10] + coeff0_30*dmats1[30][10] + coeff0_31*dmats1[31][10] + coeff0_32*dmats1[32][10] + coeff0_33*dmats1[33][10] + coeff0_34*dmats1[34][10] + coeff0_35*dmats1[35][10] + coeff0_36*dmats1[36][10] + coeff0_37*dmats1[37][10] + coeff0_38*dmats1[38][10] + coeff0_39*dmats1[39][10] + coeff0_40*dmats1[40][10] + coeff0_41*dmats1[41][10] + coeff0_42*dmats1[42][10] + coeff0_43*dmats1[43][10] + coeff0_44*dmats1[44][10] + coeff0_45*dmats1[45][10] + coeff0_46*dmats1[46][10] + coeff0_47*dmats1[47][10] + coeff0_48*dmats1[48][10] + coeff0_49*dmats1[49][10] + coeff0_50*dmats1[50][10] + coeff0_51*dmats1[51][10] + coeff0_52*dmats1[52][10] + coeff0_53*dmats1[53][10] + coeff0_54*dmats1[54][10] + coeff0_55*dmats1[55][10];
          new_coeff0_11 = coeff0_0*dmats1[0][11] + coeff0_1*dmats1[1][11] + coeff0_2*dmats1[2][11] + coeff0_3*dmats1[3][11] + coeff0_4*dmats1[4][11] + coeff0_5*dmats1[5][11] + coeff0_6*dmats1[6][11] + coeff0_7*dmats1[7][11] + coeff0_8*dmats1[8][11] + coeff0_9*dmats1[9][11] + coeff0_10*dmats1[10][11] + coeff0_11*dmats1[11][11] + coeff0_12*dmats1[12][11] + coeff0_13*dmats1[13][11] + coeff0_14*dmats1[14][11] + coeff0_15*dmats1[15][11] + coeff0_16*dmats1[16][11] + coeff0_17*dmats1[17][11] + coeff0_18*dmats1[18][11] + coeff0_19*dmats1[19][11] + coeff0_20*dmats1[20][11] + coeff0_21*dmats1[21][11] + coeff0_22*dmats1[22][11] + coeff0_23*dmats1[23][11] + coeff0_24*dmats1[24][11] + coeff0_25*dmats1[25][11] + coeff0_26*dmats1[26][11] + coeff0_27*dmats1[27][11] + coeff0_28*dmats1[28][11] + coeff0_29*dmats1[29][11] + coeff0_30*dmats1[30][11] + coeff0_31*dmats1[31][11] + coeff0_32*dmats1[32][11] + coeff0_33*dmats1[33][11] + coeff0_34*dmats1[34][11] + coeff0_35*dmats1[35][11] + coeff0_36*dmats1[36][11] + coeff0_37*dmats1[37][11] + coeff0_38*dmats1[38][11] + coeff0_39*dmats1[39][11] + coeff0_40*dmats1[40][11] + coeff0_41*dmats1[41][11] + coeff0_42*dmats1[42][11] + coeff0_43*dmats1[43][11] + coeff0_44*dmats1[44][11] + coeff0_45*dmats1[45][11] + coeff0_46*dmats1[46][11] + coeff0_47*dmats1[47][11] + coeff0_48*dmats1[48][11] + coeff0_49*dmats1[49][11] + coeff0_50*dmats1[50][11] + coeff0_51*dmats1[51][11] + coeff0_52*dmats1[52][11] + coeff0_53*dmats1[53][11] + coeff0_54*dmats1[54][11] + coeff0_55*dmats1[55][11];
          new_coeff0_12 = coeff0_0*dmats1[0][12] + coeff0_1*dmats1[1][12] + coeff0_2*dmats1[2][12] + coeff0_3*dmats1[3][12] + coeff0_4*dmats1[4][12] + coeff0_5*dmats1[5][12] + coeff0_6*dmats1[6][12] + coeff0_7*dmats1[7][12] + coeff0_8*dmats1[8][12] + coeff0_9*dmats1[9][12] + coeff0_10*dmats1[10][12] + coeff0_11*dmats1[11][12] + coeff0_12*dmats1[12][12] + coeff0_13*dmats1[13][12] + coeff0_14*dmats1[14][12] + coeff0_15*dmats1[15][12] + coeff0_16*dmats1[16][12] + coeff0_17*dmats1[17][12] + coeff0_18*dmats1[18][12] + coeff0_19*dmats1[19][12] + coeff0_20*dmats1[20][12] + coeff0_21*dmats1[21][12] + coeff0_22*dmats1[22][12] + coeff0_23*dmats1[23][12] + coeff0_24*dmats1[24][12] + coeff0_25*dmats1[25][12] + coeff0_26*dmats1[26][12] + coeff0_27*dmats1[27][12] + coeff0_28*dmats1[28][12] + coeff0_29*dmats1[29][12] + coeff0_30*dmats1[30][12] + coeff0_31*dmats1[31][12] + coeff0_32*dmats1[32][12] + coeff0_33*dmats1[33][12] + coeff0_34*dmats1[34][12] + coeff0_35*dmats1[35][12] + coeff0_36*dmats1[36][12] + coeff0_37*dmats1[37][12] + coeff0_38*dmats1[38][12] + coeff0_39*dmats1[39][12] + coeff0_40*dmats1[40][12] + coeff0_41*dmats1[41][12] + coeff0_42*dmats1[42][12] + coeff0_43*dmats1[43][12] + coeff0_44*dmats1[44][12] + coeff0_45*dmats1[45][12] + coeff0_46*dmats1[46][12] + coeff0_47*dmats1[47][12] + coeff0_48*dmats1[48][12] + coeff0_49*dmats1[49][12] + coeff0_50*dmats1[50][12] + coeff0_51*dmats1[51][12] + coeff0_52*dmats1[52][12] + coeff0_53*dmats1[53][12] + coeff0_54*dmats1[54][12] + coeff0_55*dmats1[55][12];
          new_coeff0_13 = coeff0_0*dmats1[0][13] + coeff0_1*dmats1[1][13] + coeff0_2*dmats1[2][13] + coeff0_3*dmats1[3][13] + coeff0_4*dmats1[4][13] + coeff0_5*dmats1[5][13] + coeff0_6*dmats1[6][13] + coeff0_7*dmats1[7][13] + coeff0_8*dmats1[8][13] + coeff0_9*dmats1[9][13] + coeff0_10*dmats1[10][13] + coeff0_11*dmats1[11][13] + coeff0_12*dmats1[12][13] + coeff0_13*dmats1[13][13] + coeff0_14*dmats1[14][13] + coeff0_15*dmats1[15][13] + coeff0_16*dmats1[16][13] + coeff0_17*dmats1[17][13] + coeff0_18*dmats1[18][13] + coeff0_19*dmats1[19][13] + coeff0_20*dmats1[20][13] + coeff0_21*dmats1[21][13] + coeff0_22*dmats1[22][13] + coeff0_23*dmats1[23][13] + coeff0_24*dmats1[24][13] + coeff0_25*dmats1[25][13] + coeff0_26*dmats1[26][13] + coeff0_27*dmats1[27][13] + coeff0_28*dmats1[28][13] + coeff0_29*dmats1[29][13] + coeff0_30*dmats1[30][13] + coeff0_31*dmats1[31][13] + coeff0_32*dmats1[32][13] + coeff0_33*dmats1[33][13] + coeff0_34*dmats1[34][13] + coeff0_35*dmats1[35][13] + coeff0_36*dmats1[36][13] + coeff0_37*dmats1[37][13] + coeff0_38*dmats1[38][13] + coeff0_39*dmats1[39][13] + coeff0_40*dmats1[40][13] + coeff0_41*dmats1[41][13] + coeff0_42*dmats1[42][13] + coeff0_43*dmats1[43][13] + coeff0_44*dmats1[44][13] + coeff0_45*dmats1[45][13] + coeff0_46*dmats1[46][13] + coeff0_47*dmats1[47][13] + coeff0_48*dmats1[48][13] + coeff0_49*dmats1[49][13] + coeff0_50*dmats1[50][13] + coeff0_51*dmats1[51][13] + coeff0_52*dmats1[52][13] + coeff0_53*dmats1[53][13] + coeff0_54*dmats1[54][13] + coeff0_55*dmats1[55][13];
          new_coeff0_14 = coeff0_0*dmats1[0][14] + coeff0_1*dmats1[1][14] + coeff0_2*dmats1[2][14] + coeff0_3*dmats1[3][14] + coeff0_4*dmats1[4][14] + coeff0_5*dmats1[5][14] + coeff0_6*dmats1[6][14] + coeff0_7*dmats1[7][14] + coeff0_8*dmats1[8][14] + coeff0_9*dmats1[9][14] + coeff0_10*dmats1[10][14] + coeff0_11*dmats1[11][14] + coeff0_12*dmats1[12][14] + coeff0_13*dmats1[13][14] + coeff0_14*dmats1[14][14] + coeff0_15*dmats1[15][14] + coeff0_16*dmats1[16][14] + coeff0_17*dmats1[17][14] + coeff0_18*dmats1[18][14] + coeff0_19*dmats1[19][14] + coeff0_20*dmats1[20][14] + coeff0_21*dmats1[21][14] + coeff0_22*dmats1[22][14] + coeff0_23*dmats1[23][14] + coeff0_24*dmats1[24][14] + coeff0_25*dmats1[25][14] + coeff0_26*dmats1[26][14] + coeff0_27*dmats1[27][14] + coeff0_28*dmats1[28][14] + coeff0_29*dmats1[29][14] + coeff0_30*dmats1[30][14] + coeff0_31*dmats1[31][14] + coeff0_32*dmats1[32][14] + coeff0_33*dmats1[33][14] + coeff0_34*dmats1[34][14] + coeff0_35*dmats1[35][14] + coeff0_36*dmats1[36][14] + coeff0_37*dmats1[37][14] + coeff0_38*dmats1[38][14] + coeff0_39*dmats1[39][14] + coeff0_40*dmats1[40][14] + coeff0_41*dmats1[41][14] + coeff0_42*dmats1[42][14] + coeff0_43*dmats1[43][14] + coeff0_44*dmats1[44][14] + coeff0_45*dmats1[45][14] + coeff0_46*dmats1[46][14] + coeff0_47*dmats1[47][14] + coeff0_48*dmats1[48][14] + coeff0_49*dmats1[49][14] + coeff0_50*dmats1[50][14] + coeff0_51*dmats1[51][14] + coeff0_52*dmats1[52][14] + coeff0_53*dmats1[53][14] + coeff0_54*dmats1[54][14] + coeff0_55*dmats1[55][14];
          new_coeff0_15 = coeff0_0*dmats1[0][15] + coeff0_1*dmats1[1][15] + coeff0_2*dmats1[2][15] + coeff0_3*dmats1[3][15] + coeff0_4*dmats1[4][15] + coeff0_5*dmats1[5][15] + coeff0_6*dmats1[6][15] + coeff0_7*dmats1[7][15] + coeff0_8*dmats1[8][15] + coeff0_9*dmats1[9][15] + coeff0_10*dmats1[10][15] + coeff0_11*dmats1[11][15] + coeff0_12*dmats1[12][15] + coeff0_13*dmats1[13][15] + coeff0_14*dmats1[14][15] + coeff0_15*dmats1[15][15] + coeff0_16*dmats1[16][15] + coeff0_17*dmats1[17][15] + coeff0_18*dmats1[18][15] + coeff0_19*dmats1[19][15] + coeff0_20*dmats1[20][15] + coeff0_21*dmats1[21][15] + coeff0_22*dmats1[22][15] + coeff0_23*dmats1[23][15] + coeff0_24*dmats1[24][15] + coeff0_25*dmats1[25][15] + coeff0_26*dmats1[26][15] + coeff0_27*dmats1[27][15] + coeff0_28*dmats1[28][15] + coeff0_29*dmats1[29][15] + coeff0_30*dmats1[30][15] + coeff0_31*dmats1[31][15] + coeff0_32*dmats1[32][15] + coeff0_33*dmats1[33][15] + coeff0_34*dmats1[34][15] + coeff0_35*dmats1[35][15] + coeff0_36*dmats1[36][15] + coeff0_37*dmats1[37][15] + coeff0_38*dmats1[38][15] + coeff0_39*dmats1[39][15] + coeff0_40*dmats1[40][15] + coeff0_41*dmats1[41][15] + coeff0_42*dmats1[42][15] + coeff0_43*dmats1[43][15] + coeff0_44*dmats1[44][15] + coeff0_45*dmats1[45][15] + coeff0_46*dmats1[46][15] + coeff0_47*dmats1[47][15] + coeff0_48*dmats1[48][15] + coeff0_49*dmats1[49][15] + coeff0_50*dmats1[50][15] + coeff0_51*dmats1[51][15] + coeff0_52*dmats1[52][15] + coeff0_53*dmats1[53][15] + coeff0_54*dmats1[54][15] + coeff0_55*dmats1[55][15];
          new_coeff0_16 = coeff0_0*dmats1[0][16] + coeff0_1*dmats1[1][16] + coeff0_2*dmats1[2][16] + coeff0_3*dmats1[3][16] + coeff0_4*dmats1[4][16] + coeff0_5*dmats1[5][16] + coeff0_6*dmats1[6][16] + coeff0_7*dmats1[7][16] + coeff0_8*dmats1[8][16] + coeff0_9*dmats1[9][16] + coeff0_10*dmats1[10][16] + coeff0_11*dmats1[11][16] + coeff0_12*dmats1[12][16] + coeff0_13*dmats1[13][16] + coeff0_14*dmats1[14][16] + coeff0_15*dmats1[15][16] + coeff0_16*dmats1[16][16] + coeff0_17*dmats1[17][16] + coeff0_18*dmats1[18][16] + coeff0_19*dmats1[19][16] + coeff0_20*dmats1[20][16] + coeff0_21*dmats1[21][16] + coeff0_22*dmats1[22][16] + coeff0_23*dmats1[23][16] + coeff0_24*dmats1[24][16] + coeff0_25*dmats1[25][16] + coeff0_26*dmats1[26][16] + coeff0_27*dmats1[27][16] + coeff0_28*dmats1[28][16] + coeff0_29*dmats1[29][16] + coeff0_30*dmats1[30][16] + coeff0_31*dmats1[31][16] + coeff0_32*dmats1[32][16] + coeff0_33*dmats1[33][16] + coeff0_34*dmats1[34][16] + coeff0_35*dmats1[35][16] + coeff0_36*dmats1[36][16] + coeff0_37*dmats1[37][16] + coeff0_38*dmats1[38][16] + coeff0_39*dmats1[39][16] + coeff0_40*dmats1[40][16] + coeff0_41*dmats1[41][16] + coeff0_42*dmats1[42][16] + coeff0_43*dmats1[43][16] + coeff0_44*dmats1[44][16] + coeff0_45*dmats1[45][16] + coeff0_46*dmats1[46][16] + coeff0_47*dmats1[47][16] + coeff0_48*dmats1[48][16] + coeff0_49*dmats1[49][16] + coeff0_50*dmats1[50][16] + coeff0_51*dmats1[51][16] + coeff0_52*dmats1[52][16] + coeff0_53*dmats1[53][16] + coeff0_54*dmats1[54][16] + coeff0_55*dmats1[55][16];
          new_coeff0_17 = coeff0_0*dmats1[0][17] + coeff0_1*dmats1[1][17] + coeff0_2*dmats1[2][17] + coeff0_3*dmats1[3][17] + coeff0_4*dmats1[4][17] + coeff0_5*dmats1[5][17] + coeff0_6*dmats1[6][17] + coeff0_7*dmats1[7][17] + coeff0_8*dmats1[8][17] + coeff0_9*dmats1[9][17] + coeff0_10*dmats1[10][17] + coeff0_11*dmats1[11][17] + coeff0_12*dmats1[12][17] + coeff0_13*dmats1[13][17] + coeff0_14*dmats1[14][17] + coeff0_15*dmats1[15][17] + coeff0_16*dmats1[16][17] + coeff0_17*dmats1[17][17] + coeff0_18*dmats1[18][17] + coeff0_19*dmats1[19][17] + coeff0_20*dmats1[20][17] + coeff0_21*dmats1[21][17] + coeff0_22*dmats1[22][17] + coeff0_23*dmats1[23][17] + coeff0_24*dmats1[24][17] + coeff0_25*dmats1[25][17] + coeff0_26*dmats1[26][17] + coeff0_27*dmats1[27][17] + coeff0_28*dmats1[28][17] + coeff0_29*dmats1[29][17] + coeff0_30*dmats1[30][17] + coeff0_31*dmats1[31][17] + coeff0_32*dmats1[32][17] + coeff0_33*dmats1[33][17] + coeff0_34*dmats1[34][17] + coeff0_35*dmats1[35][17] + coeff0_36*dmats1[36][17] + coeff0_37*dmats1[37][17] + coeff0_38*dmats1[38][17] + coeff0_39*dmats1[39][17] + coeff0_40*dmats1[40][17] + coeff0_41*dmats1[41][17] + coeff0_42*dmats1[42][17] + coeff0_43*dmats1[43][17] + coeff0_44*dmats1[44][17] + coeff0_45*dmats1[45][17] + coeff0_46*dmats1[46][17] + coeff0_47*dmats1[47][17] + coeff0_48*dmats1[48][17] + coeff0_49*dmats1[49][17] + coeff0_50*dmats1[50][17] + coeff0_51*dmats1[51][17] + coeff0_52*dmats1[52][17] + coeff0_53*dmats1[53][17] + coeff0_54*dmats1[54][17] + coeff0_55*dmats1[55][17];
          new_coeff0_18 = coeff0_0*dmats1[0][18] + coeff0_1*dmats1[1][18] + coeff0_2*dmats1[2][18] + coeff0_3*dmats1[3][18] + coeff0_4*dmats1[4][18] + coeff0_5*dmats1[5][18] + coeff0_6*dmats1[6][18] + coeff0_7*dmats1[7][18] + coeff0_8*dmats1[8][18] + coeff0_9*dmats1[9][18] + coeff0_10*dmats1[10][18] + coeff0_11*dmats1[11][18] + coeff0_12*dmats1[12][18] + coeff0_13*dmats1[13][18] + coeff0_14*dmats1[14][18] + coeff0_15*dmats1[15][18] + coeff0_16*dmats1[16][18] + coeff0_17*dmats1[17][18] + coeff0_18*dmats1[18][18] + coeff0_19*dmats1[19][18] + coeff0_20*dmats1[20][18] + coeff0_21*dmats1[21][18] + coeff0_22*dmats1[22][18] + coeff0_23*dmats1[23][18] + coeff0_24*dmats1[24][18] + coeff0_25*dmats1[25][18] + coeff0_26*dmats1[26][18] + coeff0_27*dmats1[27][18] + coeff0_28*dmats1[28][18] + coeff0_29*dmats1[29][18] + coeff0_30*dmats1[30][18] + coeff0_31*dmats1[31][18] + coeff0_32*dmats1[32][18] + coeff0_33*dmats1[33][18] + coeff0_34*dmats1[34][18] + coeff0_35*dmats1[35][18] + coeff0_36*dmats1[36][18] + coeff0_37*dmats1[37][18] + coeff0_38*dmats1[38][18] + coeff0_39*dmats1[39][18] + coeff0_40*dmats1[40][18] + coeff0_41*dmats1[41][18] + coeff0_42*dmats1[42][18] + coeff0_43*dmats1[43][18] + coeff0_44*dmats1[44][18] + coeff0_45*dmats1[45][18] + coeff0_46*dmats1[46][18] + coeff0_47*dmats1[47][18] + coeff0_48*dmats1[48][18] + coeff0_49*dmats1[49][18] + coeff0_50*dmats1[50][18] + coeff0_51*dmats1[51][18] + coeff0_52*dmats1[52][18] + coeff0_53*dmats1[53][18] + coeff0_54*dmats1[54][18] + coeff0_55*dmats1[55][18];
          new_coeff0_19 = coeff0_0*dmats1[0][19] + coeff0_1*dmats1[1][19] + coeff0_2*dmats1[2][19] + coeff0_3*dmats1[3][19] + coeff0_4*dmats1[4][19] + coeff0_5*dmats1[5][19] + coeff0_6*dmats1[6][19] + coeff0_7*dmats1[7][19] + coeff0_8*dmats1[8][19] + coeff0_9*dmats1[9][19] + coeff0_10*dmats1[10][19] + coeff0_11*dmats1[11][19] + coeff0_12*dmats1[12][19] + coeff0_13*dmats1[13][19] + coeff0_14*dmats1[14][19] + coeff0_15*dmats1[15][19] + coeff0_16*dmats1[16][19] + coeff0_17*dmats1[17][19] + coeff0_18*dmats1[18][19] + coeff0_19*dmats1[19][19] + coeff0_20*dmats1[20][19] + coeff0_21*dmats1[21][19] + coeff0_22*dmats1[22][19] + coeff0_23*dmats1[23][19] + coeff0_24*dmats1[24][19] + coeff0_25*dmats1[25][19] + coeff0_26*dmats1[26][19] + coeff0_27*dmats1[27][19] + coeff0_28*dmats1[28][19] + coeff0_29*dmats1[29][19] + coeff0_30*dmats1[30][19] + coeff0_31*dmats1[31][19] + coeff0_32*dmats1[32][19] + coeff0_33*dmats1[33][19] + coeff0_34*dmats1[34][19] + coeff0_35*dmats1[35][19] + coeff0_36*dmats1[36][19] + coeff0_37*dmats1[37][19] + coeff0_38*dmats1[38][19] + coeff0_39*dmats1[39][19] + coeff0_40*dmats1[40][19] + coeff0_41*dmats1[41][19] + coeff0_42*dmats1[42][19] + coeff0_43*dmats1[43][19] + coeff0_44*dmats1[44][19] + coeff0_45*dmats1[45][19] + coeff0_46*dmats1[46][19] + coeff0_47*dmats1[47][19] + coeff0_48*dmats1[48][19] + coeff0_49*dmats1[49][19] + coeff0_50*dmats1[50][19] + coeff0_51*dmats1[51][19] + coeff0_52*dmats1[52][19] + coeff0_53*dmats1[53][19] + coeff0_54*dmats1[54][19] + coeff0_55*dmats1[55][19];
          new_coeff0_20 = coeff0_0*dmats1[0][20] + coeff0_1*dmats1[1][20] + coeff0_2*dmats1[2][20] + coeff0_3*dmats1[3][20] + coeff0_4*dmats1[4][20] + coeff0_5*dmats1[5][20] + coeff0_6*dmats1[6][20] + coeff0_7*dmats1[7][20] + coeff0_8*dmats1[8][20] + coeff0_9*dmats1[9][20] + coeff0_10*dmats1[10][20] + coeff0_11*dmats1[11][20] + coeff0_12*dmats1[12][20] + coeff0_13*dmats1[13][20] + coeff0_14*dmats1[14][20] + coeff0_15*dmats1[15][20] + coeff0_16*dmats1[16][20] + coeff0_17*dmats1[17][20] + coeff0_18*dmats1[18][20] + coeff0_19*dmats1[19][20] + coeff0_20*dmats1[20][20] + coeff0_21*dmats1[21][20] + coeff0_22*dmats1[22][20] + coeff0_23*dmats1[23][20] + coeff0_24*dmats1[24][20] + coeff0_25*dmats1[25][20] + coeff0_26*dmats1[26][20] + coeff0_27*dmats1[27][20] + coeff0_28*dmats1[28][20] + coeff0_29*dmats1[29][20] + coeff0_30*dmats1[30][20] + coeff0_31*dmats1[31][20] + coeff0_32*dmats1[32][20] + coeff0_33*dmats1[33][20] + coeff0_34*dmats1[34][20] + coeff0_35*dmats1[35][20] + coeff0_36*dmats1[36][20] + coeff0_37*dmats1[37][20] + coeff0_38*dmats1[38][20] + coeff0_39*dmats1[39][20] + coeff0_40*dmats1[40][20] + coeff0_41*dmats1[41][20] + coeff0_42*dmats1[42][20] + coeff0_43*dmats1[43][20] + coeff0_44*dmats1[44][20] + coeff0_45*dmats1[45][20] + coeff0_46*dmats1[46][20] + coeff0_47*dmats1[47][20] + coeff0_48*dmats1[48][20] + coeff0_49*dmats1[49][20] + coeff0_50*dmats1[50][20] + coeff0_51*dmats1[51][20] + coeff0_52*dmats1[52][20] + coeff0_53*dmats1[53][20] + coeff0_54*dmats1[54][20] + coeff0_55*dmats1[55][20];
          new_coeff0_21 = coeff0_0*dmats1[0][21] + coeff0_1*dmats1[1][21] + coeff0_2*dmats1[2][21] + coeff0_3*dmats1[3][21] + coeff0_4*dmats1[4][21] + coeff0_5*dmats1[5][21] + coeff0_6*dmats1[6][21] + coeff0_7*dmats1[7][21] + coeff0_8*dmats1[8][21] + coeff0_9*dmats1[9][21] + coeff0_10*dmats1[10][21] + coeff0_11*dmats1[11][21] + coeff0_12*dmats1[12][21] + coeff0_13*dmats1[13][21] + coeff0_14*dmats1[14][21] + coeff0_15*dmats1[15][21] + coeff0_16*dmats1[16][21] + coeff0_17*dmats1[17][21] + coeff0_18*dmats1[18][21] + coeff0_19*dmats1[19][21] + coeff0_20*dmats1[20][21] + coeff0_21*dmats1[21][21] + coeff0_22*dmats1[22][21] + coeff0_23*dmats1[23][21] + coeff0_24*dmats1[24][21] + coeff0_25*dmats1[25][21] + coeff0_26*dmats1[26][21] + coeff0_27*dmats1[27][21] + coeff0_28*dmats1[28][21] + coeff0_29*dmats1[29][21] + coeff0_30*dmats1[30][21] + coeff0_31*dmats1[31][21] + coeff0_32*dmats1[32][21] + coeff0_33*dmats1[33][21] + coeff0_34*dmats1[34][21] + coeff0_35*dmats1[35][21] + coeff0_36*dmats1[36][21] + coeff0_37*dmats1[37][21] + coeff0_38*dmats1[38][21] + coeff0_39*dmats1[39][21] + coeff0_40*dmats1[40][21] + coeff0_41*dmats1[41][21] + coeff0_42*dmats1[42][21] + coeff0_43*dmats1[43][21] + coeff0_44*dmats1[44][21] + coeff0_45*dmats1[45][21] + coeff0_46*dmats1[46][21] + coeff0_47*dmats1[47][21] + coeff0_48*dmats1[48][21] + coeff0_49*dmats1[49][21] + coeff0_50*dmats1[50][21] + coeff0_51*dmats1[51][21] + coeff0_52*dmats1[52][21] + coeff0_53*dmats1[53][21] + coeff0_54*dmats1[54][21] + coeff0_55*dmats1[55][21];
          new_coeff0_22 = coeff0_0*dmats1[0][22] + coeff0_1*dmats1[1][22] + coeff0_2*dmats1[2][22] + coeff0_3*dmats1[3][22] + coeff0_4*dmats1[4][22] + coeff0_5*dmats1[5][22] + coeff0_6*dmats1[6][22] + coeff0_7*dmats1[7][22] + coeff0_8*dmats1[8][22] + coeff0_9*dmats1[9][22] + coeff0_10*dmats1[10][22] + coeff0_11*dmats1[11][22] + coeff0_12*dmats1[12][22] + coeff0_13*dmats1[13][22] + coeff0_14*dmats1[14][22] + coeff0_15*dmats1[15][22] + coeff0_16*dmats1[16][22] + coeff0_17*dmats1[17][22] + coeff0_18*dmats1[18][22] + coeff0_19*dmats1[19][22] + coeff0_20*dmats1[20][22] + coeff0_21*dmats1[21][22] + coeff0_22*dmats1[22][22] + coeff0_23*dmats1[23][22] + coeff0_24*dmats1[24][22] + coeff0_25*dmats1[25][22] + coeff0_26*dmats1[26][22] + coeff0_27*dmats1[27][22] + coeff0_28*dmats1[28][22] + coeff0_29*dmats1[29][22] + coeff0_30*dmats1[30][22] + coeff0_31*dmats1[31][22] + coeff0_32*dmats1[32][22] + coeff0_33*dmats1[33][22] + coeff0_34*dmats1[34][22] + coeff0_35*dmats1[35][22] + coeff0_36*dmats1[36][22] + coeff0_37*dmats1[37][22] + coeff0_38*dmats1[38][22] + coeff0_39*dmats1[39][22] + coeff0_40*dmats1[40][22] + coeff0_41*dmats1[41][22] + coeff0_42*dmats1[42][22] + coeff0_43*dmats1[43][22] + coeff0_44*dmats1[44][22] + coeff0_45*dmats1[45][22] + coeff0_46*dmats1[46][22] + coeff0_47*dmats1[47][22] + coeff0_48*dmats1[48][22] + coeff0_49*dmats1[49][22] + coeff0_50*dmats1[50][22] + coeff0_51*dmats1[51][22] + coeff0_52*dmats1[52][22] + coeff0_53*dmats1[53][22] + coeff0_54*dmats1[54][22] + coeff0_55*dmats1[55][22];
          new_coeff0_23 = coeff0_0*dmats1[0][23] + coeff0_1*dmats1[1][23] + coeff0_2*dmats1[2][23] + coeff0_3*dmats1[3][23] + coeff0_4*dmats1[4][23] + coeff0_5*dmats1[5][23] + coeff0_6*dmats1[6][23] + coeff0_7*dmats1[7][23] + coeff0_8*dmats1[8][23] + coeff0_9*dmats1[9][23] + coeff0_10*dmats1[10][23] + coeff0_11*dmats1[11][23] + coeff0_12*dmats1[12][23] + coeff0_13*dmats1[13][23] + coeff0_14*dmats1[14][23] + coeff0_15*dmats1[15][23] + coeff0_16*dmats1[16][23] + coeff0_17*dmats1[17][23] + coeff0_18*dmats1[18][23] + coeff0_19*dmats1[19][23] + coeff0_20*dmats1[20][23] + coeff0_21*dmats1[21][23] + coeff0_22*dmats1[22][23] + coeff0_23*dmats1[23][23] + coeff0_24*dmats1[24][23] + coeff0_25*dmats1[25][23] + coeff0_26*dmats1[26][23] + coeff0_27*dmats1[27][23] + coeff0_28*dmats1[28][23] + coeff0_29*dmats1[29][23] + coeff0_30*dmats1[30][23] + coeff0_31*dmats1[31][23] + coeff0_32*dmats1[32][23] + coeff0_33*dmats1[33][23] + coeff0_34*dmats1[34][23] + coeff0_35*dmats1[35][23] + coeff0_36*dmats1[36][23] + coeff0_37*dmats1[37][23] + coeff0_38*dmats1[38][23] + coeff0_39*dmats1[39][23] + coeff0_40*dmats1[40][23] + coeff0_41*dmats1[41][23] + coeff0_42*dmats1[42][23] + coeff0_43*dmats1[43][23] + coeff0_44*dmats1[44][23] + coeff0_45*dmats1[45][23] + coeff0_46*dmats1[46][23] + coeff0_47*dmats1[47][23] + coeff0_48*dmats1[48][23] + coeff0_49*dmats1[49][23] + coeff0_50*dmats1[50][23] + coeff0_51*dmats1[51][23] + coeff0_52*dmats1[52][23] + coeff0_53*dmats1[53][23] + coeff0_54*dmats1[54][23] + coeff0_55*dmats1[55][23];
          new_coeff0_24 = coeff0_0*dmats1[0][24] + coeff0_1*dmats1[1][24] + coeff0_2*dmats1[2][24] + coeff0_3*dmats1[3][24] + coeff0_4*dmats1[4][24] + coeff0_5*dmats1[5][24] + coeff0_6*dmats1[6][24] + coeff0_7*dmats1[7][24] + coeff0_8*dmats1[8][24] + coeff0_9*dmats1[9][24] + coeff0_10*dmats1[10][24] + coeff0_11*dmats1[11][24] + coeff0_12*dmats1[12][24] + coeff0_13*dmats1[13][24] + coeff0_14*dmats1[14][24] + coeff0_15*dmats1[15][24] + coeff0_16*dmats1[16][24] + coeff0_17*dmats1[17][24] + coeff0_18*dmats1[18][24] + coeff0_19*dmats1[19][24] + coeff0_20*dmats1[20][24] + coeff0_21*dmats1[21][24] + coeff0_22*dmats1[22][24] + coeff0_23*dmats1[23][24] + coeff0_24*dmats1[24][24] + coeff0_25*dmats1[25][24] + coeff0_26*dmats1[26][24] + coeff0_27*dmats1[27][24] + coeff0_28*dmats1[28][24] + coeff0_29*dmats1[29][24] + coeff0_30*dmats1[30][24] + coeff0_31*dmats1[31][24] + coeff0_32*dmats1[32][24] + coeff0_33*dmats1[33][24] + coeff0_34*dmats1[34][24] + coeff0_35*dmats1[35][24] + coeff0_36*dmats1[36][24] + coeff0_37*dmats1[37][24] + coeff0_38*dmats1[38][24] + coeff0_39*dmats1[39][24] + coeff0_40*dmats1[40][24] + coeff0_41*dmats1[41][24] + coeff0_42*dmats1[42][24] + coeff0_43*dmats1[43][24] + coeff0_44*dmats1[44][24] + coeff0_45*dmats1[45][24] + coeff0_46*dmats1[46][24] + coeff0_47*dmats1[47][24] + coeff0_48*dmats1[48][24] + coeff0_49*dmats1[49][24] + coeff0_50*dmats1[50][24] + coeff0_51*dmats1[51][24] + coeff0_52*dmats1[52][24] + coeff0_53*dmats1[53][24] + coeff0_54*dmats1[54][24] + coeff0_55*dmats1[55][24];
          new_coeff0_25 = coeff0_0*dmats1[0][25] + coeff0_1*dmats1[1][25] + coeff0_2*dmats1[2][25] + coeff0_3*dmats1[3][25] + coeff0_4*dmats1[4][25] + coeff0_5*dmats1[5][25] + coeff0_6*dmats1[6][25] + coeff0_7*dmats1[7][25] + coeff0_8*dmats1[8][25] + coeff0_9*dmats1[9][25] + coeff0_10*dmats1[10][25] + coeff0_11*dmats1[11][25] + coeff0_12*dmats1[12][25] + coeff0_13*dmats1[13][25] + coeff0_14*dmats1[14][25] + coeff0_15*dmats1[15][25] + coeff0_16*dmats1[16][25] + coeff0_17*dmats1[17][25] + coeff0_18*dmats1[18][25] + coeff0_19*dmats1[19][25] + coeff0_20*dmats1[20][25] + coeff0_21*dmats1[21][25] + coeff0_22*dmats1[22][25] + coeff0_23*dmats1[23][25] + coeff0_24*dmats1[24][25] + coeff0_25*dmats1[25][25] + coeff0_26*dmats1[26][25] + coeff0_27*dmats1[27][25] + coeff0_28*dmats1[28][25] + coeff0_29*dmats1[29][25] + coeff0_30*dmats1[30][25] + coeff0_31*dmats1[31][25] + coeff0_32*dmats1[32][25] + coeff0_33*dmats1[33][25] + coeff0_34*dmats1[34][25] + coeff0_35*dmats1[35][25] + coeff0_36*dmats1[36][25] + coeff0_37*dmats1[37][25] + coeff0_38*dmats1[38][25] + coeff0_39*dmats1[39][25] + coeff0_40*dmats1[40][25] + coeff0_41*dmats1[41][25] + coeff0_42*dmats1[42][25] + coeff0_43*dmats1[43][25] + coeff0_44*dmats1[44][25] + coeff0_45*dmats1[45][25] + coeff0_46*dmats1[46][25] + coeff0_47*dmats1[47][25] + coeff0_48*dmats1[48][25] + coeff0_49*dmats1[49][25] + coeff0_50*dmats1[50][25] + coeff0_51*dmats1[51][25] + coeff0_52*dmats1[52][25] + coeff0_53*dmats1[53][25] + coeff0_54*dmats1[54][25] + coeff0_55*dmats1[55][25];
          new_coeff0_26 = coeff0_0*dmats1[0][26] + coeff0_1*dmats1[1][26] + coeff0_2*dmats1[2][26] + coeff0_3*dmats1[3][26] + coeff0_4*dmats1[4][26] + coeff0_5*dmats1[5][26] + coeff0_6*dmats1[6][26] + coeff0_7*dmats1[7][26] + coeff0_8*dmats1[8][26] + coeff0_9*dmats1[9][26] + coeff0_10*dmats1[10][26] + coeff0_11*dmats1[11][26] + coeff0_12*dmats1[12][26] + coeff0_13*dmats1[13][26] + coeff0_14*dmats1[14][26] + coeff0_15*dmats1[15][26] + coeff0_16*dmats1[16][26] + coeff0_17*dmats1[17][26] + coeff0_18*dmats1[18][26] + coeff0_19*dmats1[19][26] + coeff0_20*dmats1[20][26] + coeff0_21*dmats1[21][26] + coeff0_22*dmats1[22][26] + coeff0_23*dmats1[23][26] + coeff0_24*dmats1[24][26] + coeff0_25*dmats1[25][26] + coeff0_26*dmats1[26][26] + coeff0_27*dmats1[27][26] + coeff0_28*dmats1[28][26] + coeff0_29*dmats1[29][26] + coeff0_30*dmats1[30][26] + coeff0_31*dmats1[31][26] + coeff0_32*dmats1[32][26] + coeff0_33*dmats1[33][26] + coeff0_34*dmats1[34][26] + coeff0_35*dmats1[35][26] + coeff0_36*dmats1[36][26] + coeff0_37*dmats1[37][26] + coeff0_38*dmats1[38][26] + coeff0_39*dmats1[39][26] + coeff0_40*dmats1[40][26] + coeff0_41*dmats1[41][26] + coeff0_42*dmats1[42][26] + coeff0_43*dmats1[43][26] + coeff0_44*dmats1[44][26] + coeff0_45*dmats1[45][26] + coeff0_46*dmats1[46][26] + coeff0_47*dmats1[47][26] + coeff0_48*dmats1[48][26] + coeff0_49*dmats1[49][26] + coeff0_50*dmats1[50][26] + coeff0_51*dmats1[51][26] + coeff0_52*dmats1[52][26] + coeff0_53*dmats1[53][26] + coeff0_54*dmats1[54][26] + coeff0_55*dmats1[55][26];
          new_coeff0_27 = coeff0_0*dmats1[0][27] + coeff0_1*dmats1[1][27] + coeff0_2*dmats1[2][27] + coeff0_3*dmats1[3][27] + coeff0_4*dmats1[4][27] + coeff0_5*dmats1[5][27] + coeff0_6*dmats1[6][27] + coeff0_7*dmats1[7][27] + coeff0_8*dmats1[8][27] + coeff0_9*dmats1[9][27] + coeff0_10*dmats1[10][27] + coeff0_11*dmats1[11][27] + coeff0_12*dmats1[12][27] + coeff0_13*dmats1[13][27] + coeff0_14*dmats1[14][27] + coeff0_15*dmats1[15][27] + coeff0_16*dmats1[16][27] + coeff0_17*dmats1[17][27] + coeff0_18*dmats1[18][27] + coeff0_19*dmats1[19][27] + coeff0_20*dmats1[20][27] + coeff0_21*dmats1[21][27] + coeff0_22*dmats1[22][27] + coeff0_23*dmats1[23][27] + coeff0_24*dmats1[24][27] + coeff0_25*dmats1[25][27] + coeff0_26*dmats1[26][27] + coeff0_27*dmats1[27][27] + coeff0_28*dmats1[28][27] + coeff0_29*dmats1[29][27] + coeff0_30*dmats1[30][27] + coeff0_31*dmats1[31][27] + coeff0_32*dmats1[32][27] + coeff0_33*dmats1[33][27] + coeff0_34*dmats1[34][27] + coeff0_35*dmats1[35][27] + coeff0_36*dmats1[36][27] + coeff0_37*dmats1[37][27] + coeff0_38*dmats1[38][27] + coeff0_39*dmats1[39][27] + coeff0_40*dmats1[40][27] + coeff0_41*dmats1[41][27] + coeff0_42*dmats1[42][27] + coeff0_43*dmats1[43][27] + coeff0_44*dmats1[44][27] + coeff0_45*dmats1[45][27] + coeff0_46*dmats1[46][27] + coeff0_47*dmats1[47][27] + coeff0_48*dmats1[48][27] + coeff0_49*dmats1[49][27] + coeff0_50*dmats1[50][27] + coeff0_51*dmats1[51][27] + coeff0_52*dmats1[52][27] + coeff0_53*dmats1[53][27] + coeff0_54*dmats1[54][27] + coeff0_55*dmats1[55][27];
          new_coeff0_28 = coeff0_0*dmats1[0][28] + coeff0_1*dmats1[1][28] + coeff0_2*dmats1[2][28] + coeff0_3*dmats1[3][28] + coeff0_4*dmats1[4][28] + coeff0_5*dmats1[5][28] + coeff0_6*dmats1[6][28] + coeff0_7*dmats1[7][28] + coeff0_8*dmats1[8][28] + coeff0_9*dmats1[9][28] + coeff0_10*dmats1[10][28] + coeff0_11*dmats1[11][28] + coeff0_12*dmats1[12][28] + coeff0_13*dmats1[13][28] + coeff0_14*dmats1[14][28] + coeff0_15*dmats1[15][28] + coeff0_16*dmats1[16][28] + coeff0_17*dmats1[17][28] + coeff0_18*dmats1[18][28] + coeff0_19*dmats1[19][28] + coeff0_20*dmats1[20][28] + coeff0_21*dmats1[21][28] + coeff0_22*dmats1[22][28] + coeff0_23*dmats1[23][28] + coeff0_24*dmats1[24][28] + coeff0_25*dmats1[25][28] + coeff0_26*dmats1[26][28] + coeff0_27*dmats1[27][28] + coeff0_28*dmats1[28][28] + coeff0_29*dmats1[29][28] + coeff0_30*dmats1[30][28] + coeff0_31*dmats1[31][28] + coeff0_32*dmats1[32][28] + coeff0_33*dmats1[33][28] + coeff0_34*dmats1[34][28] + coeff0_35*dmats1[35][28] + coeff0_36*dmats1[36][28] + coeff0_37*dmats1[37][28] + coeff0_38*dmats1[38][28] + coeff0_39*dmats1[39][28] + coeff0_40*dmats1[40][28] + coeff0_41*dmats1[41][28] + coeff0_42*dmats1[42][28] + coeff0_43*dmats1[43][28] + coeff0_44*dmats1[44][28] + coeff0_45*dmats1[45][28] + coeff0_46*dmats1[46][28] + coeff0_47*dmats1[47][28] + coeff0_48*dmats1[48][28] + coeff0_49*dmats1[49][28] + coeff0_50*dmats1[50][28] + coeff0_51*dmats1[51][28] + coeff0_52*dmats1[52][28] + coeff0_53*dmats1[53][28] + coeff0_54*dmats1[54][28] + coeff0_55*dmats1[55][28];
          new_coeff0_29 = coeff0_0*dmats1[0][29] + coeff0_1*dmats1[1][29] + coeff0_2*dmats1[2][29] + coeff0_3*dmats1[3][29] + coeff0_4*dmats1[4][29] + coeff0_5*dmats1[5][29] + coeff0_6*dmats1[6][29] + coeff0_7*dmats1[7][29] + coeff0_8*dmats1[8][29] + coeff0_9*dmats1[9][29] + coeff0_10*dmats1[10][29] + coeff0_11*dmats1[11][29] + coeff0_12*dmats1[12][29] + coeff0_13*dmats1[13][29] + coeff0_14*dmats1[14][29] + coeff0_15*dmats1[15][29] + coeff0_16*dmats1[16][29] + coeff0_17*dmats1[17][29] + coeff0_18*dmats1[18][29] + coeff0_19*dmats1[19][29] + coeff0_20*dmats1[20][29] + coeff0_21*dmats1[21][29] + coeff0_22*dmats1[22][29] + coeff0_23*dmats1[23][29] + coeff0_24*dmats1[24][29] + coeff0_25*dmats1[25][29] + coeff0_26*dmats1[26][29] + coeff0_27*dmats1[27][29] + coeff0_28*dmats1[28][29] + coeff0_29*dmats1[29][29] + coeff0_30*dmats1[30][29] + coeff0_31*dmats1[31][29] + coeff0_32*dmats1[32][29] + coeff0_33*dmats1[33][29] + coeff0_34*dmats1[34][29] + coeff0_35*dmats1[35][29] + coeff0_36*dmats1[36][29] + coeff0_37*dmats1[37][29] + coeff0_38*dmats1[38][29] + coeff0_39*dmats1[39][29] + coeff0_40*dmats1[40][29] + coeff0_41*dmats1[41][29] + coeff0_42*dmats1[42][29] + coeff0_43*dmats1[43][29] + coeff0_44*dmats1[44][29] + coeff0_45*dmats1[45][29] + coeff0_46*dmats1[46][29] + coeff0_47*dmats1[47][29] + coeff0_48*dmats1[48][29] + coeff0_49*dmats1[49][29] + coeff0_50*dmats1[50][29] + coeff0_51*dmats1[51][29] + coeff0_52*dmats1[52][29] + coeff0_53*dmats1[53][29] + coeff0_54*dmats1[54][29] + coeff0_55*dmats1[55][29];
          new_coeff0_30 = coeff0_0*dmats1[0][30] + coeff0_1*dmats1[1][30] + coeff0_2*dmats1[2][30] + coeff0_3*dmats1[3][30] + coeff0_4*dmats1[4][30] + coeff0_5*dmats1[5][30] + coeff0_6*dmats1[6][30] + coeff0_7*dmats1[7][30] + coeff0_8*dmats1[8][30] + coeff0_9*dmats1[9][30] + coeff0_10*dmats1[10][30] + coeff0_11*dmats1[11][30] + coeff0_12*dmats1[12][30] + coeff0_13*dmats1[13][30] + coeff0_14*dmats1[14][30] + coeff0_15*dmats1[15][30] + coeff0_16*dmats1[16][30] + coeff0_17*dmats1[17][30] + coeff0_18*dmats1[18][30] + coeff0_19*dmats1[19][30] + coeff0_20*dmats1[20][30] + coeff0_21*dmats1[21][30] + coeff0_22*dmats1[22][30] + coeff0_23*dmats1[23][30] + coeff0_24*dmats1[24][30] + coeff0_25*dmats1[25][30] + coeff0_26*dmats1[26][30] + coeff0_27*dmats1[27][30] + coeff0_28*dmats1[28][30] + coeff0_29*dmats1[29][30] + coeff0_30*dmats1[30][30] + coeff0_31*dmats1[31][30] + coeff0_32*dmats1[32][30] + coeff0_33*dmats1[33][30] + coeff0_34*dmats1[34][30] + coeff0_35*dmats1[35][30] + coeff0_36*dmats1[36][30] + coeff0_37*dmats1[37][30] + coeff0_38*dmats1[38][30] + coeff0_39*dmats1[39][30] + coeff0_40*dmats1[40][30] + coeff0_41*dmats1[41][30] + coeff0_42*dmats1[42][30] + coeff0_43*dmats1[43][30] + coeff0_44*dmats1[44][30] + coeff0_45*dmats1[45][30] + coeff0_46*dmats1[46][30] + coeff0_47*dmats1[47][30] + coeff0_48*dmats1[48][30] + coeff0_49*dmats1[49][30] + coeff0_50*dmats1[50][30] + coeff0_51*dmats1[51][30] + coeff0_52*dmats1[52][30] + coeff0_53*dmats1[53][30] + coeff0_54*dmats1[54][30] + coeff0_55*dmats1[55][30];
          new_coeff0_31 = coeff0_0*dmats1[0][31] + coeff0_1*dmats1[1][31] + coeff0_2*dmats1[2][31] + coeff0_3*dmats1[3][31] + coeff0_4*dmats1[4][31] + coeff0_5*dmats1[5][31] + coeff0_6*dmats1[6][31] + coeff0_7*dmats1[7][31] + coeff0_8*dmats1[8][31] + coeff0_9*dmats1[9][31] + coeff0_10*dmats1[10][31] + coeff0_11*dmats1[11][31] + coeff0_12*dmats1[12][31] + coeff0_13*dmats1[13][31] + coeff0_14*dmats1[14][31] + coeff0_15*dmats1[15][31] + coeff0_16*dmats1[16][31] + coeff0_17*dmats1[17][31] + coeff0_18*dmats1[18][31] + coeff0_19*dmats1[19][31] + coeff0_20*dmats1[20][31] + coeff0_21*dmats1[21][31] + coeff0_22*dmats1[22][31] + coeff0_23*dmats1[23][31] + coeff0_24*dmats1[24][31] + coeff0_25*dmats1[25][31] + coeff0_26*dmats1[26][31] + coeff0_27*dmats1[27][31] + coeff0_28*dmats1[28][31] + coeff0_29*dmats1[29][31] + coeff0_30*dmats1[30][31] + coeff0_31*dmats1[31][31] + coeff0_32*dmats1[32][31] + coeff0_33*dmats1[33][31] + coeff0_34*dmats1[34][31] + coeff0_35*dmats1[35][31] + coeff0_36*dmats1[36][31] + coeff0_37*dmats1[37][31] + coeff0_38*dmats1[38][31] + coeff0_39*dmats1[39][31] + coeff0_40*dmats1[40][31] + coeff0_41*dmats1[41][31] + coeff0_42*dmats1[42][31] + coeff0_43*dmats1[43][31] + coeff0_44*dmats1[44][31] + coeff0_45*dmats1[45][31] + coeff0_46*dmats1[46][31] + coeff0_47*dmats1[47][31] + coeff0_48*dmats1[48][31] + coeff0_49*dmats1[49][31] + coeff0_50*dmats1[50][31] + coeff0_51*dmats1[51][31] + coeff0_52*dmats1[52][31] + coeff0_53*dmats1[53][31] + coeff0_54*dmats1[54][31] + coeff0_55*dmats1[55][31];
          new_coeff0_32 = coeff0_0*dmats1[0][32] + coeff0_1*dmats1[1][32] + coeff0_2*dmats1[2][32] + coeff0_3*dmats1[3][32] + coeff0_4*dmats1[4][32] + coeff0_5*dmats1[5][32] + coeff0_6*dmats1[6][32] + coeff0_7*dmats1[7][32] + coeff0_8*dmats1[8][32] + coeff0_9*dmats1[9][32] + coeff0_10*dmats1[10][32] + coeff0_11*dmats1[11][32] + coeff0_12*dmats1[12][32] + coeff0_13*dmats1[13][32] + coeff0_14*dmats1[14][32] + coeff0_15*dmats1[15][32] + coeff0_16*dmats1[16][32] + coeff0_17*dmats1[17][32] + coeff0_18*dmats1[18][32] + coeff0_19*dmats1[19][32] + coeff0_20*dmats1[20][32] + coeff0_21*dmats1[21][32] + coeff0_22*dmats1[22][32] + coeff0_23*dmats1[23][32] + coeff0_24*dmats1[24][32] + coeff0_25*dmats1[25][32] + coeff0_26*dmats1[26][32] + coeff0_27*dmats1[27][32] + coeff0_28*dmats1[28][32] + coeff0_29*dmats1[29][32] + coeff0_30*dmats1[30][32] + coeff0_31*dmats1[31][32] + coeff0_32*dmats1[32][32] + coeff0_33*dmats1[33][32] + coeff0_34*dmats1[34][32] + coeff0_35*dmats1[35][32] + coeff0_36*dmats1[36][32] + coeff0_37*dmats1[37][32] + coeff0_38*dmats1[38][32] + coeff0_39*dmats1[39][32] + coeff0_40*dmats1[40][32] + coeff0_41*dmats1[41][32] + coeff0_42*dmats1[42][32] + coeff0_43*dmats1[43][32] + coeff0_44*dmats1[44][32] + coeff0_45*dmats1[45][32] + coeff0_46*dmats1[46][32] + coeff0_47*dmats1[47][32] + coeff0_48*dmats1[48][32] + coeff0_49*dmats1[49][32] + coeff0_50*dmats1[50][32] + coeff0_51*dmats1[51][32] + coeff0_52*dmats1[52][32] + coeff0_53*dmats1[53][32] + coeff0_54*dmats1[54][32] + coeff0_55*dmats1[55][32];
          new_coeff0_33 = coeff0_0*dmats1[0][33] + coeff0_1*dmats1[1][33] + coeff0_2*dmats1[2][33] + coeff0_3*dmats1[3][33] + coeff0_4*dmats1[4][33] + coeff0_5*dmats1[5][33] + coeff0_6*dmats1[6][33] + coeff0_7*dmats1[7][33] + coeff0_8*dmats1[8][33] + coeff0_9*dmats1[9][33] + coeff0_10*dmats1[10][33] + coeff0_11*dmats1[11][33] + coeff0_12*dmats1[12][33] + coeff0_13*dmats1[13][33] + coeff0_14*dmats1[14][33] + coeff0_15*dmats1[15][33] + coeff0_16*dmats1[16][33] + coeff0_17*dmats1[17][33] + coeff0_18*dmats1[18][33] + coeff0_19*dmats1[19][33] + coeff0_20*dmats1[20][33] + coeff0_21*dmats1[21][33] + coeff0_22*dmats1[22][33] + coeff0_23*dmats1[23][33] + coeff0_24*dmats1[24][33] + coeff0_25*dmats1[25][33] + coeff0_26*dmats1[26][33] + coeff0_27*dmats1[27][33] + coeff0_28*dmats1[28][33] + coeff0_29*dmats1[29][33] + coeff0_30*dmats1[30][33] + coeff0_31*dmats1[31][33] + coeff0_32*dmats1[32][33] + coeff0_33*dmats1[33][33] + coeff0_34*dmats1[34][33] + coeff0_35*dmats1[35][33] + coeff0_36*dmats1[36][33] + coeff0_37*dmats1[37][33] + coeff0_38*dmats1[38][33] + coeff0_39*dmats1[39][33] + coeff0_40*dmats1[40][33] + coeff0_41*dmats1[41][33] + coeff0_42*dmats1[42][33] + coeff0_43*dmats1[43][33] + coeff0_44*dmats1[44][33] + coeff0_45*dmats1[45][33] + coeff0_46*dmats1[46][33] + coeff0_47*dmats1[47][33] + coeff0_48*dmats1[48][33] + coeff0_49*dmats1[49][33] + coeff0_50*dmats1[50][33] + coeff0_51*dmats1[51][33] + coeff0_52*dmats1[52][33] + coeff0_53*dmats1[53][33] + coeff0_54*dmats1[54][33] + coeff0_55*dmats1[55][33];
          new_coeff0_34 = coeff0_0*dmats1[0][34] + coeff0_1*dmats1[1][34] + coeff0_2*dmats1[2][34] + coeff0_3*dmats1[3][34] + coeff0_4*dmats1[4][34] + coeff0_5*dmats1[5][34] + coeff0_6*dmats1[6][34] + coeff0_7*dmats1[7][34] + coeff0_8*dmats1[8][34] + coeff0_9*dmats1[9][34] + coeff0_10*dmats1[10][34] + coeff0_11*dmats1[11][34] + coeff0_12*dmats1[12][34] + coeff0_13*dmats1[13][34] + coeff0_14*dmats1[14][34] + coeff0_15*dmats1[15][34] + coeff0_16*dmats1[16][34] + coeff0_17*dmats1[17][34] + coeff0_18*dmats1[18][34] + coeff0_19*dmats1[19][34] + coeff0_20*dmats1[20][34] + coeff0_21*dmats1[21][34] + coeff0_22*dmats1[22][34] + coeff0_23*dmats1[23][34] + coeff0_24*dmats1[24][34] + coeff0_25*dmats1[25][34] + coeff0_26*dmats1[26][34] + coeff0_27*dmats1[27][34] + coeff0_28*dmats1[28][34] + coeff0_29*dmats1[29][34] + coeff0_30*dmats1[30][34] + coeff0_31*dmats1[31][34] + coeff0_32*dmats1[32][34] + coeff0_33*dmats1[33][34] + coeff0_34*dmats1[34][34] + coeff0_35*dmats1[35][34] + coeff0_36*dmats1[36][34] + coeff0_37*dmats1[37][34] + coeff0_38*dmats1[38][34] + coeff0_39*dmats1[39][34] + coeff0_40*dmats1[40][34] + coeff0_41*dmats1[41][34] + coeff0_42*dmats1[42][34] + coeff0_43*dmats1[43][34] + coeff0_44*dmats1[44][34] + coeff0_45*dmats1[45][34] + coeff0_46*dmats1[46][34] + coeff0_47*dmats1[47][34] + coeff0_48*dmats1[48][34] + coeff0_49*dmats1[49][34] + coeff0_50*dmats1[50][34] + coeff0_51*dmats1[51][34] + coeff0_52*dmats1[52][34] + coeff0_53*dmats1[53][34] + coeff0_54*dmats1[54][34] + coeff0_55*dmats1[55][34];
          new_coeff0_35 = coeff0_0*dmats1[0][35] + coeff0_1*dmats1[1][35] + coeff0_2*dmats1[2][35] + coeff0_3*dmats1[3][35] + coeff0_4*dmats1[4][35] + coeff0_5*dmats1[5][35] + coeff0_6*dmats1[6][35] + coeff0_7*dmats1[7][35] + coeff0_8*dmats1[8][35] + coeff0_9*dmats1[9][35] + coeff0_10*dmats1[10][35] + coeff0_11*dmats1[11][35] + coeff0_12*dmats1[12][35] + coeff0_13*dmats1[13][35] + coeff0_14*dmats1[14][35] + coeff0_15*dmats1[15][35] + coeff0_16*dmats1[16][35] + coeff0_17*dmats1[17][35] + coeff0_18*dmats1[18][35] + coeff0_19*dmats1[19][35] + coeff0_20*dmats1[20][35] + coeff0_21*dmats1[21][35] + coeff0_22*dmats1[22][35] + coeff0_23*dmats1[23][35] + coeff0_24*dmats1[24][35] + coeff0_25*dmats1[25][35] + coeff0_26*dmats1[26][35] + coeff0_27*dmats1[27][35] + coeff0_28*dmats1[28][35] + coeff0_29*dmats1[29][35] + coeff0_30*dmats1[30][35] + coeff0_31*dmats1[31][35] + coeff0_32*dmats1[32][35] + coeff0_33*dmats1[33][35] + coeff0_34*dmats1[34][35] + coeff0_35*dmats1[35][35] + coeff0_36*dmats1[36][35] + coeff0_37*dmats1[37][35] + coeff0_38*dmats1[38][35] + coeff0_39*dmats1[39][35] + coeff0_40*dmats1[40][35] + coeff0_41*dmats1[41][35] + coeff0_42*dmats1[42][35] + coeff0_43*dmats1[43][35] + coeff0_44*dmats1[44][35] + coeff0_45*dmats1[45][35] + coeff0_46*dmats1[46][35] + coeff0_47*dmats1[47][35] + coeff0_48*dmats1[48][35] + coeff0_49*dmats1[49][35] + coeff0_50*dmats1[50][35] + coeff0_51*dmats1[51][35] + coeff0_52*dmats1[52][35] + coeff0_53*dmats1[53][35] + coeff0_54*dmats1[54][35] + coeff0_55*dmats1[55][35];
          new_coeff0_36 = coeff0_0*dmats1[0][36] + coeff0_1*dmats1[1][36] + coeff0_2*dmats1[2][36] + coeff0_3*dmats1[3][36] + coeff0_4*dmats1[4][36] + coeff0_5*dmats1[5][36] + coeff0_6*dmats1[6][36] + coeff0_7*dmats1[7][36] + coeff0_8*dmats1[8][36] + coeff0_9*dmats1[9][36] + coeff0_10*dmats1[10][36] + coeff0_11*dmats1[11][36] + coeff0_12*dmats1[12][36] + coeff0_13*dmats1[13][36] + coeff0_14*dmats1[14][36] + coeff0_15*dmats1[15][36] + coeff0_16*dmats1[16][36] + coeff0_17*dmats1[17][36] + coeff0_18*dmats1[18][36] + coeff0_19*dmats1[19][36] + coeff0_20*dmats1[20][36] + coeff0_21*dmats1[21][36] + coeff0_22*dmats1[22][36] + coeff0_23*dmats1[23][36] + coeff0_24*dmats1[24][36] + coeff0_25*dmats1[25][36] + coeff0_26*dmats1[26][36] + coeff0_27*dmats1[27][36] + coeff0_28*dmats1[28][36] + coeff0_29*dmats1[29][36] + coeff0_30*dmats1[30][36] + coeff0_31*dmats1[31][36] + coeff0_32*dmats1[32][36] + coeff0_33*dmats1[33][36] + coeff0_34*dmats1[34][36] + coeff0_35*dmats1[35][36] + coeff0_36*dmats1[36][36] + coeff0_37*dmats1[37][36] + coeff0_38*dmats1[38][36] + coeff0_39*dmats1[39][36] + coeff0_40*dmats1[40][36] + coeff0_41*dmats1[41][36] + coeff0_42*dmats1[42][36] + coeff0_43*dmats1[43][36] + coeff0_44*dmats1[44][36] + coeff0_45*dmats1[45][36] + coeff0_46*dmats1[46][36] + coeff0_47*dmats1[47][36] + coeff0_48*dmats1[48][36] + coeff0_49*dmats1[49][36] + coeff0_50*dmats1[50][36] + coeff0_51*dmats1[51][36] + coeff0_52*dmats1[52][36] + coeff0_53*dmats1[53][36] + coeff0_54*dmats1[54][36] + coeff0_55*dmats1[55][36];
          new_coeff0_37 = coeff0_0*dmats1[0][37] + coeff0_1*dmats1[1][37] + coeff0_2*dmats1[2][37] + coeff0_3*dmats1[3][37] + coeff0_4*dmats1[4][37] + coeff0_5*dmats1[5][37] + coeff0_6*dmats1[6][37] + coeff0_7*dmats1[7][37] + coeff0_8*dmats1[8][37] + coeff0_9*dmats1[9][37] + coeff0_10*dmats1[10][37] + coeff0_11*dmats1[11][37] + coeff0_12*dmats1[12][37] + coeff0_13*dmats1[13][37] + coeff0_14*dmats1[14][37] + coeff0_15*dmats1[15][37] + coeff0_16*dmats1[16][37] + coeff0_17*dmats1[17][37] + coeff0_18*dmats1[18][37] + coeff0_19*dmats1[19][37] + coeff0_20*dmats1[20][37] + coeff0_21*dmats1[21][37] + coeff0_22*dmats1[22][37] + coeff0_23*dmats1[23][37] + coeff0_24*dmats1[24][37] + coeff0_25*dmats1[25][37] + coeff0_26*dmats1[26][37] + coeff0_27*dmats1[27][37] + coeff0_28*dmats1[28][37] + coeff0_29*dmats1[29][37] + coeff0_30*dmats1[30][37] + coeff0_31*dmats1[31][37] + coeff0_32*dmats1[32][37] + coeff0_33*dmats1[33][37] + coeff0_34*dmats1[34][37] + coeff0_35*dmats1[35][37] + coeff0_36*dmats1[36][37] + coeff0_37*dmats1[37][37] + coeff0_38*dmats1[38][37] + coeff0_39*dmats1[39][37] + coeff0_40*dmats1[40][37] + coeff0_41*dmats1[41][37] + coeff0_42*dmats1[42][37] + coeff0_43*dmats1[43][37] + coeff0_44*dmats1[44][37] + coeff0_45*dmats1[45][37] + coeff0_46*dmats1[46][37] + coeff0_47*dmats1[47][37] + coeff0_48*dmats1[48][37] + coeff0_49*dmats1[49][37] + coeff0_50*dmats1[50][37] + coeff0_51*dmats1[51][37] + coeff0_52*dmats1[52][37] + coeff0_53*dmats1[53][37] + coeff0_54*dmats1[54][37] + coeff0_55*dmats1[55][37];
          new_coeff0_38 = coeff0_0*dmats1[0][38] + coeff0_1*dmats1[1][38] + coeff0_2*dmats1[2][38] + coeff0_3*dmats1[3][38] + coeff0_4*dmats1[4][38] + coeff0_5*dmats1[5][38] + coeff0_6*dmats1[6][38] + coeff0_7*dmats1[7][38] + coeff0_8*dmats1[8][38] + coeff0_9*dmats1[9][38] + coeff0_10*dmats1[10][38] + coeff0_11*dmats1[11][38] + coeff0_12*dmats1[12][38] + coeff0_13*dmats1[13][38] + coeff0_14*dmats1[14][38] + coeff0_15*dmats1[15][38] + coeff0_16*dmats1[16][38] + coeff0_17*dmats1[17][38] + coeff0_18*dmats1[18][38] + coeff0_19*dmats1[19][38] + coeff0_20*dmats1[20][38] + coeff0_21*dmats1[21][38] + coeff0_22*dmats1[22][38] + coeff0_23*dmats1[23][38] + coeff0_24*dmats1[24][38] + coeff0_25*dmats1[25][38] + coeff0_26*dmats1[26][38] + coeff0_27*dmats1[27][38] + coeff0_28*dmats1[28][38] + coeff0_29*dmats1[29][38] + coeff0_30*dmats1[30][38] + coeff0_31*dmats1[31][38] + coeff0_32*dmats1[32][38] + coeff0_33*dmats1[33][38] + coeff0_34*dmats1[34][38] + coeff0_35*dmats1[35][38] + coeff0_36*dmats1[36][38] + coeff0_37*dmats1[37][38] + coeff0_38*dmats1[38][38] + coeff0_39*dmats1[39][38] + coeff0_40*dmats1[40][38] + coeff0_41*dmats1[41][38] + coeff0_42*dmats1[42][38] + coeff0_43*dmats1[43][38] + coeff0_44*dmats1[44][38] + coeff0_45*dmats1[45][38] + coeff0_46*dmats1[46][38] + coeff0_47*dmats1[47][38] + coeff0_48*dmats1[48][38] + coeff0_49*dmats1[49][38] + coeff0_50*dmats1[50][38] + coeff0_51*dmats1[51][38] + coeff0_52*dmats1[52][38] + coeff0_53*dmats1[53][38] + coeff0_54*dmats1[54][38] + coeff0_55*dmats1[55][38];
          new_coeff0_39 = coeff0_0*dmats1[0][39] + coeff0_1*dmats1[1][39] + coeff0_2*dmats1[2][39] + coeff0_3*dmats1[3][39] + coeff0_4*dmats1[4][39] + coeff0_5*dmats1[5][39] + coeff0_6*dmats1[6][39] + coeff0_7*dmats1[7][39] + coeff0_8*dmats1[8][39] + coeff0_9*dmats1[9][39] + coeff0_10*dmats1[10][39] + coeff0_11*dmats1[11][39] + coeff0_12*dmats1[12][39] + coeff0_13*dmats1[13][39] + coeff0_14*dmats1[14][39] + coeff0_15*dmats1[15][39] + coeff0_16*dmats1[16][39] + coeff0_17*dmats1[17][39] + coeff0_18*dmats1[18][39] + coeff0_19*dmats1[19][39] + coeff0_20*dmats1[20][39] + coeff0_21*dmats1[21][39] + coeff0_22*dmats1[22][39] + coeff0_23*dmats1[23][39] + coeff0_24*dmats1[24][39] + coeff0_25*dmats1[25][39] + coeff0_26*dmats1[26][39] + coeff0_27*dmats1[27][39] + coeff0_28*dmats1[28][39] + coeff0_29*dmats1[29][39] + coeff0_30*dmats1[30][39] + coeff0_31*dmats1[31][39] + coeff0_32*dmats1[32][39] + coeff0_33*dmats1[33][39] + coeff0_34*dmats1[34][39] + coeff0_35*dmats1[35][39] + coeff0_36*dmats1[36][39] + coeff0_37*dmats1[37][39] + coeff0_38*dmats1[38][39] + coeff0_39*dmats1[39][39] + coeff0_40*dmats1[40][39] + coeff0_41*dmats1[41][39] + coeff0_42*dmats1[42][39] + coeff0_43*dmats1[43][39] + coeff0_44*dmats1[44][39] + coeff0_45*dmats1[45][39] + coeff0_46*dmats1[46][39] + coeff0_47*dmats1[47][39] + coeff0_48*dmats1[48][39] + coeff0_49*dmats1[49][39] + coeff0_50*dmats1[50][39] + coeff0_51*dmats1[51][39] + coeff0_52*dmats1[52][39] + coeff0_53*dmats1[53][39] + coeff0_54*dmats1[54][39] + coeff0_55*dmats1[55][39];
          new_coeff0_40 = coeff0_0*dmats1[0][40] + coeff0_1*dmats1[1][40] + coeff0_2*dmats1[2][40] + coeff0_3*dmats1[3][40] + coeff0_4*dmats1[4][40] + coeff0_5*dmats1[5][40] + coeff0_6*dmats1[6][40] + coeff0_7*dmats1[7][40] + coeff0_8*dmats1[8][40] + coeff0_9*dmats1[9][40] + coeff0_10*dmats1[10][40] + coeff0_11*dmats1[11][40] + coeff0_12*dmats1[12][40] + coeff0_13*dmats1[13][40] + coeff0_14*dmats1[14][40] + coeff0_15*dmats1[15][40] + coeff0_16*dmats1[16][40] + coeff0_17*dmats1[17][40] + coeff0_18*dmats1[18][40] + coeff0_19*dmats1[19][40] + coeff0_20*dmats1[20][40] + coeff0_21*dmats1[21][40] + coeff0_22*dmats1[22][40] + coeff0_23*dmats1[23][40] + coeff0_24*dmats1[24][40] + coeff0_25*dmats1[25][40] + coeff0_26*dmats1[26][40] + coeff0_27*dmats1[27][40] + coeff0_28*dmats1[28][40] + coeff0_29*dmats1[29][40] + coeff0_30*dmats1[30][40] + coeff0_31*dmats1[31][40] + coeff0_32*dmats1[32][40] + coeff0_33*dmats1[33][40] + coeff0_34*dmats1[34][40] + coeff0_35*dmats1[35][40] + coeff0_36*dmats1[36][40] + coeff0_37*dmats1[37][40] + coeff0_38*dmats1[38][40] + coeff0_39*dmats1[39][40] + coeff0_40*dmats1[40][40] + coeff0_41*dmats1[41][40] + coeff0_42*dmats1[42][40] + coeff0_43*dmats1[43][40] + coeff0_44*dmats1[44][40] + coeff0_45*dmats1[45][40] + coeff0_46*dmats1[46][40] + coeff0_47*dmats1[47][40] + coeff0_48*dmats1[48][40] + coeff0_49*dmats1[49][40] + coeff0_50*dmats1[50][40] + coeff0_51*dmats1[51][40] + coeff0_52*dmats1[52][40] + coeff0_53*dmats1[53][40] + coeff0_54*dmats1[54][40] + coeff0_55*dmats1[55][40];
          new_coeff0_41 = coeff0_0*dmats1[0][41] + coeff0_1*dmats1[1][41] + coeff0_2*dmats1[2][41] + coeff0_3*dmats1[3][41] + coeff0_4*dmats1[4][41] + coeff0_5*dmats1[5][41] + coeff0_6*dmats1[6][41] + coeff0_7*dmats1[7][41] + coeff0_8*dmats1[8][41] + coeff0_9*dmats1[9][41] + coeff0_10*dmats1[10][41] + coeff0_11*dmats1[11][41] + coeff0_12*dmats1[12][41] + coeff0_13*dmats1[13][41] + coeff0_14*dmats1[14][41] + coeff0_15*dmats1[15][41] + coeff0_16*dmats1[16][41] + coeff0_17*dmats1[17][41] + coeff0_18*dmats1[18][41] + coeff0_19*dmats1[19][41] + coeff0_20*dmats1[20][41] + coeff0_21*dmats1[21][41] + coeff0_22*dmats1[22][41] + coeff0_23*dmats1[23][41] + coeff0_24*dmats1[24][41] + coeff0_25*dmats1[25][41] + coeff0_26*dmats1[26][41] + coeff0_27*dmats1[27][41] + coeff0_28*dmats1[28][41] + coeff0_29*dmats1[29][41] + coeff0_30*dmats1[30][41] + coeff0_31*dmats1[31][41] + coeff0_32*dmats1[32][41] + coeff0_33*dmats1[33][41] + coeff0_34*dmats1[34][41] + coeff0_35*dmats1[35][41] + coeff0_36*dmats1[36][41] + coeff0_37*dmats1[37][41] + coeff0_38*dmats1[38][41] + coeff0_39*dmats1[39][41] + coeff0_40*dmats1[40][41] + coeff0_41*dmats1[41][41] + coeff0_42*dmats1[42][41] + coeff0_43*dmats1[43][41] + coeff0_44*dmats1[44][41] + coeff0_45*dmats1[45][41] + coeff0_46*dmats1[46][41] + coeff0_47*dmats1[47][41] + coeff0_48*dmats1[48][41] + coeff0_49*dmats1[49][41] + coeff0_50*dmats1[50][41] + coeff0_51*dmats1[51][41] + coeff0_52*dmats1[52][41] + coeff0_53*dmats1[53][41] + coeff0_54*dmats1[54][41] + coeff0_55*dmats1[55][41];
          new_coeff0_42 = coeff0_0*dmats1[0][42] + coeff0_1*dmats1[1][42] + coeff0_2*dmats1[2][42] + coeff0_3*dmats1[3][42] + coeff0_4*dmats1[4][42] + coeff0_5*dmats1[5][42] + coeff0_6*dmats1[6][42] + coeff0_7*dmats1[7][42] + coeff0_8*dmats1[8][42] + coeff0_9*dmats1[9][42] + coeff0_10*dmats1[10][42] + coeff0_11*dmats1[11][42] + coeff0_12*dmats1[12][42] + coeff0_13*dmats1[13][42] + coeff0_14*dmats1[14][42] + coeff0_15*dmats1[15][42] + coeff0_16*dmats1[16][42] + coeff0_17*dmats1[17][42] + coeff0_18*dmats1[18][42] + coeff0_19*dmats1[19][42] + coeff0_20*dmats1[20][42] + coeff0_21*dmats1[21][42] + coeff0_22*dmats1[22][42] + coeff0_23*dmats1[23][42] + coeff0_24*dmats1[24][42] + coeff0_25*dmats1[25][42] + coeff0_26*dmats1[26][42] + coeff0_27*dmats1[27][42] + coeff0_28*dmats1[28][42] + coeff0_29*dmats1[29][42] + coeff0_30*dmats1[30][42] + coeff0_31*dmats1[31][42] + coeff0_32*dmats1[32][42] + coeff0_33*dmats1[33][42] + coeff0_34*dmats1[34][42] + coeff0_35*dmats1[35][42] + coeff0_36*dmats1[36][42] + coeff0_37*dmats1[37][42] + coeff0_38*dmats1[38][42] + coeff0_39*dmats1[39][42] + coeff0_40*dmats1[40][42] + coeff0_41*dmats1[41][42] + coeff0_42*dmats1[42][42] + coeff0_43*dmats1[43][42] + coeff0_44*dmats1[44][42] + coeff0_45*dmats1[45][42] + coeff0_46*dmats1[46][42] + coeff0_47*dmats1[47][42] + coeff0_48*dmats1[48][42] + coeff0_49*dmats1[49][42] + coeff0_50*dmats1[50][42] + coeff0_51*dmats1[51][42] + coeff0_52*dmats1[52][42] + coeff0_53*dmats1[53][42] + coeff0_54*dmats1[54][42] + coeff0_55*dmats1[55][42];
          new_coeff0_43 = coeff0_0*dmats1[0][43] + coeff0_1*dmats1[1][43] + coeff0_2*dmats1[2][43] + coeff0_3*dmats1[3][43] + coeff0_4*dmats1[4][43] + coeff0_5*dmats1[5][43] + coeff0_6*dmats1[6][43] + coeff0_7*dmats1[7][43] + coeff0_8*dmats1[8][43] + coeff0_9*dmats1[9][43] + coeff0_10*dmats1[10][43] + coeff0_11*dmats1[11][43] + coeff0_12*dmats1[12][43] + coeff0_13*dmats1[13][43] + coeff0_14*dmats1[14][43] + coeff0_15*dmats1[15][43] + coeff0_16*dmats1[16][43] + coeff0_17*dmats1[17][43] + coeff0_18*dmats1[18][43] + coeff0_19*dmats1[19][43] + coeff0_20*dmats1[20][43] + coeff0_21*dmats1[21][43] + coeff0_22*dmats1[22][43] + coeff0_23*dmats1[23][43] + coeff0_24*dmats1[24][43] + coeff0_25*dmats1[25][43] + coeff0_26*dmats1[26][43] + coeff0_27*dmats1[27][43] + coeff0_28*dmats1[28][43] + coeff0_29*dmats1[29][43] + coeff0_30*dmats1[30][43] + coeff0_31*dmats1[31][43] + coeff0_32*dmats1[32][43] + coeff0_33*dmats1[33][43] + coeff0_34*dmats1[34][43] + coeff0_35*dmats1[35][43] + coeff0_36*dmats1[36][43] + coeff0_37*dmats1[37][43] + coeff0_38*dmats1[38][43] + coeff0_39*dmats1[39][43] + coeff0_40*dmats1[40][43] + coeff0_41*dmats1[41][43] + coeff0_42*dmats1[42][43] + coeff0_43*dmats1[43][43] + coeff0_44*dmats1[44][43] + coeff0_45*dmats1[45][43] + coeff0_46*dmats1[46][43] + coeff0_47*dmats1[47][43] + coeff0_48*dmats1[48][43] + coeff0_49*dmats1[49][43] + coeff0_50*dmats1[50][43] + coeff0_51*dmats1[51][43] + coeff0_52*dmats1[52][43] + coeff0_53*dmats1[53][43] + coeff0_54*dmats1[54][43] + coeff0_55*dmats1[55][43];
          new_coeff0_44 = coeff0_0*dmats1[0][44] + coeff0_1*dmats1[1][44] + coeff0_2*dmats1[2][44] + coeff0_3*dmats1[3][44] + coeff0_4*dmats1[4][44] + coeff0_5*dmats1[5][44] + coeff0_6*dmats1[6][44] + coeff0_7*dmats1[7][44] + coeff0_8*dmats1[8][44] + coeff0_9*dmats1[9][44] + coeff0_10*dmats1[10][44] + coeff0_11*dmats1[11][44] + coeff0_12*dmats1[12][44] + coeff0_13*dmats1[13][44] + coeff0_14*dmats1[14][44] + coeff0_15*dmats1[15][44] + coeff0_16*dmats1[16][44] + coeff0_17*dmats1[17][44] + coeff0_18*dmats1[18][44] + coeff0_19*dmats1[19][44] + coeff0_20*dmats1[20][44] + coeff0_21*dmats1[21][44] + coeff0_22*dmats1[22][44] + coeff0_23*dmats1[23][44] + coeff0_24*dmats1[24][44] + coeff0_25*dmats1[25][44] + coeff0_26*dmats1[26][44] + coeff0_27*dmats1[27][44] + coeff0_28*dmats1[28][44] + coeff0_29*dmats1[29][44] + coeff0_30*dmats1[30][44] + coeff0_31*dmats1[31][44] + coeff0_32*dmats1[32][44] + coeff0_33*dmats1[33][44] + coeff0_34*dmats1[34][44] + coeff0_35*dmats1[35][44] + coeff0_36*dmats1[36][44] + coeff0_37*dmats1[37][44] + coeff0_38*dmats1[38][44] + coeff0_39*dmats1[39][44] + coeff0_40*dmats1[40][44] + coeff0_41*dmats1[41][44] + coeff0_42*dmats1[42][44] + coeff0_43*dmats1[43][44] + coeff0_44*dmats1[44][44] + coeff0_45*dmats1[45][44] + coeff0_46*dmats1[46][44] + coeff0_47*dmats1[47][44] + coeff0_48*dmats1[48][44] + coeff0_49*dmats1[49][44] + coeff0_50*dmats1[50][44] + coeff0_51*dmats1[51][44] + coeff0_52*dmats1[52][44] + coeff0_53*dmats1[53][44] + coeff0_54*dmats1[54][44] + coeff0_55*dmats1[55][44];
          new_coeff0_45 = coeff0_0*dmats1[0][45] + coeff0_1*dmats1[1][45] + coeff0_2*dmats1[2][45] + coeff0_3*dmats1[3][45] + coeff0_4*dmats1[4][45] + coeff0_5*dmats1[5][45] + coeff0_6*dmats1[6][45] + coeff0_7*dmats1[7][45] + coeff0_8*dmats1[8][45] + coeff0_9*dmats1[9][45] + coeff0_10*dmats1[10][45] + coeff0_11*dmats1[11][45] + coeff0_12*dmats1[12][45] + coeff0_13*dmats1[13][45] + coeff0_14*dmats1[14][45] + coeff0_15*dmats1[15][45] + coeff0_16*dmats1[16][45] + coeff0_17*dmats1[17][45] + coeff0_18*dmats1[18][45] + coeff0_19*dmats1[19][45] + coeff0_20*dmats1[20][45] + coeff0_21*dmats1[21][45] + coeff0_22*dmats1[22][45] + coeff0_23*dmats1[23][45] + coeff0_24*dmats1[24][45] + coeff0_25*dmats1[25][45] + coeff0_26*dmats1[26][45] + coeff0_27*dmats1[27][45] + coeff0_28*dmats1[28][45] + coeff0_29*dmats1[29][45] + coeff0_30*dmats1[30][45] + coeff0_31*dmats1[31][45] + coeff0_32*dmats1[32][45] + coeff0_33*dmats1[33][45] + coeff0_34*dmats1[34][45] + coeff0_35*dmats1[35][45] + coeff0_36*dmats1[36][45] + coeff0_37*dmats1[37][45] + coeff0_38*dmats1[38][45] + coeff0_39*dmats1[39][45] + coeff0_40*dmats1[40][45] + coeff0_41*dmats1[41][45] + coeff0_42*dmats1[42][45] + coeff0_43*dmats1[43][45] + coeff0_44*dmats1[44][45] + coeff0_45*dmats1[45][45] + coeff0_46*dmats1[46][45] + coeff0_47*dmats1[47][45] + coeff0_48*dmats1[48][45] + coeff0_49*dmats1[49][45] + coeff0_50*dmats1[50][45] + coeff0_51*dmats1[51][45] + coeff0_52*dmats1[52][45] + coeff0_53*dmats1[53][45] + coeff0_54*dmats1[54][45] + coeff0_55*dmats1[55][45];
          new_coeff0_46 = coeff0_0*dmats1[0][46] + coeff0_1*dmats1[1][46] + coeff0_2*dmats1[2][46] + coeff0_3*dmats1[3][46] + coeff0_4*dmats1[4][46] + coeff0_5*dmats1[5][46] + coeff0_6*dmats1[6][46] + coeff0_7*dmats1[7][46] + coeff0_8*dmats1[8][46] + coeff0_9*dmats1[9][46] + coeff0_10*dmats1[10][46] + coeff0_11*dmats1[11][46] + coeff0_12*dmats1[12][46] + coeff0_13*dmats1[13][46] + coeff0_14*dmats1[14][46] + coeff0_15*dmats1[15][46] + coeff0_16*dmats1[16][46] + coeff0_17*dmats1[17][46] + coeff0_18*dmats1[18][46] + coeff0_19*dmats1[19][46] + coeff0_20*dmats1[20][46] + coeff0_21*dmats1[21][46] + coeff0_22*dmats1[22][46] + coeff0_23*dmats1[23][46] + coeff0_24*dmats1[24][46] + coeff0_25*dmats1[25][46] + coeff0_26*dmats1[26][46] + coeff0_27*dmats1[27][46] + coeff0_28*dmats1[28][46] + coeff0_29*dmats1[29][46] + coeff0_30*dmats1[30][46] + coeff0_31*dmats1[31][46] + coeff0_32*dmats1[32][46] + coeff0_33*dmats1[33][46] + coeff0_34*dmats1[34][46] + coeff0_35*dmats1[35][46] + coeff0_36*dmats1[36][46] + coeff0_37*dmats1[37][46] + coeff0_38*dmats1[38][46] + coeff0_39*dmats1[39][46] + coeff0_40*dmats1[40][46] + coeff0_41*dmats1[41][46] + coeff0_42*dmats1[42][46] + coeff0_43*dmats1[43][46] + coeff0_44*dmats1[44][46] + coeff0_45*dmats1[45][46] + coeff0_46*dmats1[46][46] + coeff0_47*dmats1[47][46] + coeff0_48*dmats1[48][46] + coeff0_49*dmats1[49][46] + coeff0_50*dmats1[50][46] + coeff0_51*dmats1[51][46] + coeff0_52*dmats1[52][46] + coeff0_53*dmats1[53][46] + coeff0_54*dmats1[54][46] + coeff0_55*dmats1[55][46];
          new_coeff0_47 = coeff0_0*dmats1[0][47] + coeff0_1*dmats1[1][47] + coeff0_2*dmats1[2][47] + coeff0_3*dmats1[3][47] + coeff0_4*dmats1[4][47] + coeff0_5*dmats1[5][47] + coeff0_6*dmats1[6][47] + coeff0_7*dmats1[7][47] + coeff0_8*dmats1[8][47] + coeff0_9*dmats1[9][47] + coeff0_10*dmats1[10][47] + coeff0_11*dmats1[11][47] + coeff0_12*dmats1[12][47] + coeff0_13*dmats1[13][47] + coeff0_14*dmats1[14][47] + coeff0_15*dmats1[15][47] + coeff0_16*dmats1[16][47] + coeff0_17*dmats1[17][47] + coeff0_18*dmats1[18][47] + coeff0_19*dmats1[19][47] + coeff0_20*dmats1[20][47] + coeff0_21*dmats1[21][47] + coeff0_22*dmats1[22][47] + coeff0_23*dmats1[23][47] + coeff0_24*dmats1[24][47] + coeff0_25*dmats1[25][47] + coeff0_26*dmats1[26][47] + coeff0_27*dmats1[27][47] + coeff0_28*dmats1[28][47] + coeff0_29*dmats1[29][47] + coeff0_30*dmats1[30][47] + coeff0_31*dmats1[31][47] + coeff0_32*dmats1[32][47] + coeff0_33*dmats1[33][47] + coeff0_34*dmats1[34][47] + coeff0_35*dmats1[35][47] + coeff0_36*dmats1[36][47] + coeff0_37*dmats1[37][47] + coeff0_38*dmats1[38][47] + coeff0_39*dmats1[39][47] + coeff0_40*dmats1[40][47] + coeff0_41*dmats1[41][47] + coeff0_42*dmats1[42][47] + coeff0_43*dmats1[43][47] + coeff0_44*dmats1[44][47] + coeff0_45*dmats1[45][47] + coeff0_46*dmats1[46][47] + coeff0_47*dmats1[47][47] + coeff0_48*dmats1[48][47] + coeff0_49*dmats1[49][47] + coeff0_50*dmats1[50][47] + coeff0_51*dmats1[51][47] + coeff0_52*dmats1[52][47] + coeff0_53*dmats1[53][47] + coeff0_54*dmats1[54][47] + coeff0_55*dmats1[55][47];
          new_coeff0_48 = coeff0_0*dmats1[0][48] + coeff0_1*dmats1[1][48] + coeff0_2*dmats1[2][48] + coeff0_3*dmats1[3][48] + coeff0_4*dmats1[4][48] + coeff0_5*dmats1[5][48] + coeff0_6*dmats1[6][48] + coeff0_7*dmats1[7][48] + coeff0_8*dmats1[8][48] + coeff0_9*dmats1[9][48] + coeff0_10*dmats1[10][48] + coeff0_11*dmats1[11][48] + coeff0_12*dmats1[12][48] + coeff0_13*dmats1[13][48] + coeff0_14*dmats1[14][48] + coeff0_15*dmats1[15][48] + coeff0_16*dmats1[16][48] + coeff0_17*dmats1[17][48] + coeff0_18*dmats1[18][48] + coeff0_19*dmats1[19][48] + coeff0_20*dmats1[20][48] + coeff0_21*dmats1[21][48] + coeff0_22*dmats1[22][48] + coeff0_23*dmats1[23][48] + coeff0_24*dmats1[24][48] + coeff0_25*dmats1[25][48] + coeff0_26*dmats1[26][48] + coeff0_27*dmats1[27][48] + coeff0_28*dmats1[28][48] + coeff0_29*dmats1[29][48] + coeff0_30*dmats1[30][48] + coeff0_31*dmats1[31][48] + coeff0_32*dmats1[32][48] + coeff0_33*dmats1[33][48] + coeff0_34*dmats1[34][48] + coeff0_35*dmats1[35][48] + coeff0_36*dmats1[36][48] + coeff0_37*dmats1[37][48] + coeff0_38*dmats1[38][48] + coeff0_39*dmats1[39][48] + coeff0_40*dmats1[40][48] + coeff0_41*dmats1[41][48] + coeff0_42*dmats1[42][48] + coeff0_43*dmats1[43][48] + coeff0_44*dmats1[44][48] + coeff0_45*dmats1[45][48] + coeff0_46*dmats1[46][48] + coeff0_47*dmats1[47][48] + coeff0_48*dmats1[48][48] + coeff0_49*dmats1[49][48] + coeff0_50*dmats1[50][48] + coeff0_51*dmats1[51][48] + coeff0_52*dmats1[52][48] + coeff0_53*dmats1[53][48] + coeff0_54*dmats1[54][48] + coeff0_55*dmats1[55][48];
          new_coeff0_49 = coeff0_0*dmats1[0][49] + coeff0_1*dmats1[1][49] + coeff0_2*dmats1[2][49] + coeff0_3*dmats1[3][49] + coeff0_4*dmats1[4][49] + coeff0_5*dmats1[5][49] + coeff0_6*dmats1[6][49] + coeff0_7*dmats1[7][49] + coeff0_8*dmats1[8][49] + coeff0_9*dmats1[9][49] + coeff0_10*dmats1[10][49] + coeff0_11*dmats1[11][49] + coeff0_12*dmats1[12][49] + coeff0_13*dmats1[13][49] + coeff0_14*dmats1[14][49] + coeff0_15*dmats1[15][49] + coeff0_16*dmats1[16][49] + coeff0_17*dmats1[17][49] + coeff0_18*dmats1[18][49] + coeff0_19*dmats1[19][49] + coeff0_20*dmats1[20][49] + coeff0_21*dmats1[21][49] + coeff0_22*dmats1[22][49] + coeff0_23*dmats1[23][49] + coeff0_24*dmats1[24][49] + coeff0_25*dmats1[25][49] + coeff0_26*dmats1[26][49] + coeff0_27*dmats1[27][49] + coeff0_28*dmats1[28][49] + coeff0_29*dmats1[29][49] + coeff0_30*dmats1[30][49] + coeff0_31*dmats1[31][49] + coeff0_32*dmats1[32][49] + coeff0_33*dmats1[33][49] + coeff0_34*dmats1[34][49] + coeff0_35*dmats1[35][49] + coeff0_36*dmats1[36][49] + coeff0_37*dmats1[37][49] + coeff0_38*dmats1[38][49] + coeff0_39*dmats1[39][49] + coeff0_40*dmats1[40][49] + coeff0_41*dmats1[41][49] + coeff0_42*dmats1[42][49] + coeff0_43*dmats1[43][49] + coeff0_44*dmats1[44][49] + coeff0_45*dmats1[45][49] + coeff0_46*dmats1[46][49] + coeff0_47*dmats1[47][49] + coeff0_48*dmats1[48][49] + coeff0_49*dmats1[49][49] + coeff0_50*dmats1[50][49] + coeff0_51*dmats1[51][49] + coeff0_52*dmats1[52][49] + coeff0_53*dmats1[53][49] + coeff0_54*dmats1[54][49] + coeff0_55*dmats1[55][49];
          new_coeff0_50 = coeff0_0*dmats1[0][50] + coeff0_1*dmats1[1][50] + coeff0_2*dmats1[2][50] + coeff0_3*dmats1[3][50] + coeff0_4*dmats1[4][50] + coeff0_5*dmats1[5][50] + coeff0_6*dmats1[6][50] + coeff0_7*dmats1[7][50] + coeff0_8*dmats1[8][50] + coeff0_9*dmats1[9][50] + coeff0_10*dmats1[10][50] + coeff0_11*dmats1[11][50] + coeff0_12*dmats1[12][50] + coeff0_13*dmats1[13][50] + coeff0_14*dmats1[14][50] + coeff0_15*dmats1[15][50] + coeff0_16*dmats1[16][50] + coeff0_17*dmats1[17][50] + coeff0_18*dmats1[18][50] + coeff0_19*dmats1[19][50] + coeff0_20*dmats1[20][50] + coeff0_21*dmats1[21][50] + coeff0_22*dmats1[22][50] + coeff0_23*dmats1[23][50] + coeff0_24*dmats1[24][50] + coeff0_25*dmats1[25][50] + coeff0_26*dmats1[26][50] + coeff0_27*dmats1[27][50] + coeff0_28*dmats1[28][50] + coeff0_29*dmats1[29][50] + coeff0_30*dmats1[30][50] + coeff0_31*dmats1[31][50] + coeff0_32*dmats1[32][50] + coeff0_33*dmats1[33][50] + coeff0_34*dmats1[34][50] + coeff0_35*dmats1[35][50] + coeff0_36*dmats1[36][50] + coeff0_37*dmats1[37][50] + coeff0_38*dmats1[38][50] + coeff0_39*dmats1[39][50] + coeff0_40*dmats1[40][50] + coeff0_41*dmats1[41][50] + coeff0_42*dmats1[42][50] + coeff0_43*dmats1[43][50] + coeff0_44*dmats1[44][50] + coeff0_45*dmats1[45][50] + coeff0_46*dmats1[46][50] + coeff0_47*dmats1[47][50] + coeff0_48*dmats1[48][50] + coeff0_49*dmats1[49][50] + coeff0_50*dmats1[50][50] + coeff0_51*dmats1[51][50] + coeff0_52*dmats1[52][50] + coeff0_53*dmats1[53][50] + coeff0_54*dmats1[54][50] + coeff0_55*dmats1[55][50];
          new_coeff0_51 = coeff0_0*dmats1[0][51] + coeff0_1*dmats1[1][51] + coeff0_2*dmats1[2][51] + coeff0_3*dmats1[3][51] + coeff0_4*dmats1[4][51] + coeff0_5*dmats1[5][51] + coeff0_6*dmats1[6][51] + coeff0_7*dmats1[7][51] + coeff0_8*dmats1[8][51] + coeff0_9*dmats1[9][51] + coeff0_10*dmats1[10][51] + coeff0_11*dmats1[11][51] + coeff0_12*dmats1[12][51] + coeff0_13*dmats1[13][51] + coeff0_14*dmats1[14][51] + coeff0_15*dmats1[15][51] + coeff0_16*dmats1[16][51] + coeff0_17*dmats1[17][51] + coeff0_18*dmats1[18][51] + coeff0_19*dmats1[19][51] + coeff0_20*dmats1[20][51] + coeff0_21*dmats1[21][51] + coeff0_22*dmats1[22][51] + coeff0_23*dmats1[23][51] + coeff0_24*dmats1[24][51] + coeff0_25*dmats1[25][51] + coeff0_26*dmats1[26][51] + coeff0_27*dmats1[27][51] + coeff0_28*dmats1[28][51] + coeff0_29*dmats1[29][51] + coeff0_30*dmats1[30][51] + coeff0_31*dmats1[31][51] + coeff0_32*dmats1[32][51] + coeff0_33*dmats1[33][51] + coeff0_34*dmats1[34][51] + coeff0_35*dmats1[35][51] + coeff0_36*dmats1[36][51] + coeff0_37*dmats1[37][51] + coeff0_38*dmats1[38][51] + coeff0_39*dmats1[39][51] + coeff0_40*dmats1[40][51] + coeff0_41*dmats1[41][51] + coeff0_42*dmats1[42][51] + coeff0_43*dmats1[43][51] + coeff0_44*dmats1[44][51] + coeff0_45*dmats1[45][51] + coeff0_46*dmats1[46][51] + coeff0_47*dmats1[47][51] + coeff0_48*dmats1[48][51] + coeff0_49*dmats1[49][51] + coeff0_50*dmats1[50][51] + coeff0_51*dmats1[51][51] + coeff0_52*dmats1[52][51] + coeff0_53*dmats1[53][51] + coeff0_54*dmats1[54][51] + coeff0_55*dmats1[55][51];
          new_coeff0_52 = coeff0_0*dmats1[0][52] + coeff0_1*dmats1[1][52] + coeff0_2*dmats1[2][52] + coeff0_3*dmats1[3][52] + coeff0_4*dmats1[4][52] + coeff0_5*dmats1[5][52] + coeff0_6*dmats1[6][52] + coeff0_7*dmats1[7][52] + coeff0_8*dmats1[8][52] + coeff0_9*dmats1[9][52] + coeff0_10*dmats1[10][52] + coeff0_11*dmats1[11][52] + coeff0_12*dmats1[12][52] + coeff0_13*dmats1[13][52] + coeff0_14*dmats1[14][52] + coeff0_15*dmats1[15][52] + coeff0_16*dmats1[16][52] + coeff0_17*dmats1[17][52] + coeff0_18*dmats1[18][52] + coeff0_19*dmats1[19][52] + coeff0_20*dmats1[20][52] + coeff0_21*dmats1[21][52] + coeff0_22*dmats1[22][52] + coeff0_23*dmats1[23][52] + coeff0_24*dmats1[24][52] + coeff0_25*dmats1[25][52] + coeff0_26*dmats1[26][52] + coeff0_27*dmats1[27][52] + coeff0_28*dmats1[28][52] + coeff0_29*dmats1[29][52] + coeff0_30*dmats1[30][52] + coeff0_31*dmats1[31][52] + coeff0_32*dmats1[32][52] + coeff0_33*dmats1[33][52] + coeff0_34*dmats1[34][52] + coeff0_35*dmats1[35][52] + coeff0_36*dmats1[36][52] + coeff0_37*dmats1[37][52] + coeff0_38*dmats1[38][52] + coeff0_39*dmats1[39][52] + coeff0_40*dmats1[40][52] + coeff0_41*dmats1[41][52] + coeff0_42*dmats1[42][52] + coeff0_43*dmats1[43][52] + coeff0_44*dmats1[44][52] + coeff0_45*dmats1[45][52] + coeff0_46*dmats1[46][52] + coeff0_47*dmats1[47][52] + coeff0_48*dmats1[48][52] + coeff0_49*dmats1[49][52] + coeff0_50*dmats1[50][52] + coeff0_51*dmats1[51][52] + coeff0_52*dmats1[52][52] + coeff0_53*dmats1[53][52] + coeff0_54*dmats1[54][52] + coeff0_55*dmats1[55][52];
          new_coeff0_53 = coeff0_0*dmats1[0][53] + coeff0_1*dmats1[1][53] + coeff0_2*dmats1[2][53] + coeff0_3*dmats1[3][53] + coeff0_4*dmats1[4][53] + coeff0_5*dmats1[5][53] + coeff0_6*dmats1[6][53] + coeff0_7*dmats1[7][53] + coeff0_8*dmats1[8][53] + coeff0_9*dmats1[9][53] + coeff0_10*dmats1[10][53] + coeff0_11*dmats1[11][53] + coeff0_12*dmats1[12][53] + coeff0_13*dmats1[13][53] + coeff0_14*dmats1[14][53] + coeff0_15*dmats1[15][53] + coeff0_16*dmats1[16][53] + coeff0_17*dmats1[17][53] + coeff0_18*dmats1[18][53] + coeff0_19*dmats1[19][53] + coeff0_20*dmats1[20][53] + coeff0_21*dmats1[21][53] + coeff0_22*dmats1[22][53] + coeff0_23*dmats1[23][53] + coeff0_24*dmats1[24][53] + coeff0_25*dmats1[25][53] + coeff0_26*dmats1[26][53] + coeff0_27*dmats1[27][53] + coeff0_28*dmats1[28][53] + coeff0_29*dmats1[29][53] + coeff0_30*dmats1[30][53] + coeff0_31*dmats1[31][53] + coeff0_32*dmats1[32][53] + coeff0_33*dmats1[33][53] + coeff0_34*dmats1[34][53] + coeff0_35*dmats1[35][53] + coeff0_36*dmats1[36][53] + coeff0_37*dmats1[37][53] + coeff0_38*dmats1[38][53] + coeff0_39*dmats1[39][53] + coeff0_40*dmats1[40][53] + coeff0_41*dmats1[41][53] + coeff0_42*dmats1[42][53] + coeff0_43*dmats1[43][53] + coeff0_44*dmats1[44][53] + coeff0_45*dmats1[45][53] + coeff0_46*dmats1[46][53] + coeff0_47*dmats1[47][53] + coeff0_48*dmats1[48][53] + coeff0_49*dmats1[49][53] + coeff0_50*dmats1[50][53] + coeff0_51*dmats1[51][53] + coeff0_52*dmats1[52][53] + coeff0_53*dmats1[53][53] + coeff0_54*dmats1[54][53] + coeff0_55*dmats1[55][53];
          new_coeff0_54 = coeff0_0*dmats1[0][54] + coeff0_1*dmats1[1][54] + coeff0_2*dmats1[2][54] + coeff0_3*dmats1[3][54] + coeff0_4*dmats1[4][54] + coeff0_5*dmats1[5][54] + coeff0_6*dmats1[6][54] + coeff0_7*dmats1[7][54] + coeff0_8*dmats1[8][54] + coeff0_9*dmats1[9][54] + coeff0_10*dmats1[10][54] + coeff0_11*dmats1[11][54] + coeff0_12*dmats1[12][54] + coeff0_13*dmats1[13][54] + coeff0_14*dmats1[14][54] + coeff0_15*dmats1[15][54] + coeff0_16*dmats1[16][54] + coeff0_17*dmats1[17][54] + coeff0_18*dmats1[18][54] + coeff0_19*dmats1[19][54] + coeff0_20*dmats1[20][54] + coeff0_21*dmats1[21][54] + coeff0_22*dmats1[22][54] + coeff0_23*dmats1[23][54] + coeff0_24*dmats1[24][54] + coeff0_25*dmats1[25][54] + coeff0_26*dmats1[26][54] + coeff0_27*dmats1[27][54] + coeff0_28*dmats1[28][54] + coeff0_29*dmats1[29][54] + coeff0_30*dmats1[30][54] + coeff0_31*dmats1[31][54] + coeff0_32*dmats1[32][54] + coeff0_33*dmats1[33][54] + coeff0_34*dmats1[34][54] + coeff0_35*dmats1[35][54] + coeff0_36*dmats1[36][54] + coeff0_37*dmats1[37][54] + coeff0_38*dmats1[38][54] + coeff0_39*dmats1[39][54] + coeff0_40*dmats1[40][54] + coeff0_41*dmats1[41][54] + coeff0_42*dmats1[42][54] + coeff0_43*dmats1[43][54] + coeff0_44*dmats1[44][54] + coeff0_45*dmats1[45][54] + coeff0_46*dmats1[46][54] + coeff0_47*dmats1[47][54] + coeff0_48*dmats1[48][54] + coeff0_49*dmats1[49][54] + coeff0_50*dmats1[50][54] + coeff0_51*dmats1[51][54] + coeff0_52*dmats1[52][54] + coeff0_53*dmats1[53][54] + coeff0_54*dmats1[54][54] + coeff0_55*dmats1[55][54];
          new_coeff0_55 = coeff0_0*dmats1[0][55] + coeff0_1*dmats1[1][55] + coeff0_2*dmats1[2][55] + coeff0_3*dmats1[3][55] + coeff0_4*dmats1[4][55] + coeff0_5*dmats1[5][55] + coeff0_6*dmats1[6][55] + coeff0_7*dmats1[7][55] + coeff0_8*dmats1[8][55] + coeff0_9*dmats1[9][55] + coeff0_10*dmats1[10][55] + coeff0_11*dmats1[11][55] + coeff0_12*dmats1[12][55] + coeff0_13*dmats1[13][55] + coeff0_14*dmats1[14][55] + coeff0_15*dmats1[15][55] + coeff0_16*dmats1[16][55] + coeff0_17*dmats1[17][55] + coeff0_18*dmats1[18][55] + coeff0_19*dmats1[19][55] + coeff0_20*dmats1[20][55] + coeff0_21*dmats1[21][55] + coeff0_22*dmats1[22][55] + coeff0_23*dmats1[23][55] + coeff0_24*dmats1[24][55] + coeff0_25*dmats1[25][55] + coeff0_26*dmats1[26][55] + coeff0_27*dmats1[27][55] + coeff0_28*dmats1[28][55] + coeff0_29*dmats1[29][55] + coeff0_30*dmats1[30][55] + coeff0_31*dmats1[31][55] + coeff0_32*dmats1[32][55] + coeff0_33*dmats1[33][55] + coeff0_34*dmats1[34][55] + coeff0_35*dmats1[35][55] + coeff0_36*dmats1[36][55] + coeff0_37*dmats1[37][55] + coeff0_38*dmats1[38][55] + coeff0_39*dmats1[39][55] + coeff0_40*dmats1[40][55] + coeff0_41*dmats1[41][55] + coeff0_42*dmats1[42][55] + coeff0_43*dmats1[43][55] + coeff0_44*dmats1[44][55] + coeff0_45*dmats1[45][55] + coeff0_46*dmats1[46][55] + coeff0_47*dmats1[47][55] + coeff0_48*dmats1[48][55] + coeff0_49*dmats1[49][55] + coeff0_50*dmats1[50][55] + coeff0_51*dmats1[51][55] + coeff0_52*dmats1[52][55] + coeff0_53*dmats1[53][55] + coeff0_54*dmats1[54][55] + coeff0_55*dmats1[55][55];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0] + coeff0_4*dmats2[4][0] + coeff0_5*dmats2[5][0] + coeff0_6*dmats2[6][0] + coeff0_7*dmats2[7][0] + coeff0_8*dmats2[8][0] + coeff0_9*dmats2[9][0] + coeff0_10*dmats2[10][0] + coeff0_11*dmats2[11][0] + coeff0_12*dmats2[12][0] + coeff0_13*dmats2[13][0] + coeff0_14*dmats2[14][0] + coeff0_15*dmats2[15][0] + coeff0_16*dmats2[16][0] + coeff0_17*dmats2[17][0] + coeff0_18*dmats2[18][0] + coeff0_19*dmats2[19][0] + coeff0_20*dmats2[20][0] + coeff0_21*dmats2[21][0] + coeff0_22*dmats2[22][0] + coeff0_23*dmats2[23][0] + coeff0_24*dmats2[24][0] + coeff0_25*dmats2[25][0] + coeff0_26*dmats2[26][0] + coeff0_27*dmats2[27][0] + coeff0_28*dmats2[28][0] + coeff0_29*dmats2[29][0] + coeff0_30*dmats2[30][0] + coeff0_31*dmats2[31][0] + coeff0_32*dmats2[32][0] + coeff0_33*dmats2[33][0] + coeff0_34*dmats2[34][0] + coeff0_35*dmats2[35][0] + coeff0_36*dmats2[36][0] + coeff0_37*dmats2[37][0] + coeff0_38*dmats2[38][0] + coeff0_39*dmats2[39][0] + coeff0_40*dmats2[40][0] + coeff0_41*dmats2[41][0] + coeff0_42*dmats2[42][0] + coeff0_43*dmats2[43][0] + coeff0_44*dmats2[44][0] + coeff0_45*dmats2[45][0] + coeff0_46*dmats2[46][0] + coeff0_47*dmats2[47][0] + coeff0_48*dmats2[48][0] + coeff0_49*dmats2[49][0] + coeff0_50*dmats2[50][0] + coeff0_51*dmats2[51][0] + coeff0_52*dmats2[52][0] + coeff0_53*dmats2[53][0] + coeff0_54*dmats2[54][0] + coeff0_55*dmats2[55][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1] + coeff0_4*dmats2[4][1] + coeff0_5*dmats2[5][1] + coeff0_6*dmats2[6][1] + coeff0_7*dmats2[7][1] + coeff0_8*dmats2[8][1] + coeff0_9*dmats2[9][1] + coeff0_10*dmats2[10][1] + coeff0_11*dmats2[11][1] + coeff0_12*dmats2[12][1] + coeff0_13*dmats2[13][1] + coeff0_14*dmats2[14][1] + coeff0_15*dmats2[15][1] + coeff0_16*dmats2[16][1] + coeff0_17*dmats2[17][1] + coeff0_18*dmats2[18][1] + coeff0_19*dmats2[19][1] + coeff0_20*dmats2[20][1] + coeff0_21*dmats2[21][1] + coeff0_22*dmats2[22][1] + coeff0_23*dmats2[23][1] + coeff0_24*dmats2[24][1] + coeff0_25*dmats2[25][1] + coeff0_26*dmats2[26][1] + coeff0_27*dmats2[27][1] + coeff0_28*dmats2[28][1] + coeff0_29*dmats2[29][1] + coeff0_30*dmats2[30][1] + coeff0_31*dmats2[31][1] + coeff0_32*dmats2[32][1] + coeff0_33*dmats2[33][1] + coeff0_34*dmats2[34][1] + coeff0_35*dmats2[35][1] + coeff0_36*dmats2[36][1] + coeff0_37*dmats2[37][1] + coeff0_38*dmats2[38][1] + coeff0_39*dmats2[39][1] + coeff0_40*dmats2[40][1] + coeff0_41*dmats2[41][1] + coeff0_42*dmats2[42][1] + coeff0_43*dmats2[43][1] + coeff0_44*dmats2[44][1] + coeff0_45*dmats2[45][1] + coeff0_46*dmats2[46][1] + coeff0_47*dmats2[47][1] + coeff0_48*dmats2[48][1] + coeff0_49*dmats2[49][1] + coeff0_50*dmats2[50][1] + coeff0_51*dmats2[51][1] + coeff0_52*dmats2[52][1] + coeff0_53*dmats2[53][1] + coeff0_54*dmats2[54][1] + coeff0_55*dmats2[55][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2] + coeff0_4*dmats2[4][2] + coeff0_5*dmats2[5][2] + coeff0_6*dmats2[6][2] + coeff0_7*dmats2[7][2] + coeff0_8*dmats2[8][2] + coeff0_9*dmats2[9][2] + coeff0_10*dmats2[10][2] + coeff0_11*dmats2[11][2] + coeff0_12*dmats2[12][2] + coeff0_13*dmats2[13][2] + coeff0_14*dmats2[14][2] + coeff0_15*dmats2[15][2] + coeff0_16*dmats2[16][2] + coeff0_17*dmats2[17][2] + coeff0_18*dmats2[18][2] + coeff0_19*dmats2[19][2] + coeff0_20*dmats2[20][2] + coeff0_21*dmats2[21][2] + coeff0_22*dmats2[22][2] + coeff0_23*dmats2[23][2] + coeff0_24*dmats2[24][2] + coeff0_25*dmats2[25][2] + coeff0_26*dmats2[26][2] + coeff0_27*dmats2[27][2] + coeff0_28*dmats2[28][2] + coeff0_29*dmats2[29][2] + coeff0_30*dmats2[30][2] + coeff0_31*dmats2[31][2] + coeff0_32*dmats2[32][2] + coeff0_33*dmats2[33][2] + coeff0_34*dmats2[34][2] + coeff0_35*dmats2[35][2] + coeff0_36*dmats2[36][2] + coeff0_37*dmats2[37][2] + coeff0_38*dmats2[38][2] + coeff0_39*dmats2[39][2] + coeff0_40*dmats2[40][2] + coeff0_41*dmats2[41][2] + coeff0_42*dmats2[42][2] + coeff0_43*dmats2[43][2] + coeff0_44*dmats2[44][2] + coeff0_45*dmats2[45][2] + coeff0_46*dmats2[46][2] + coeff0_47*dmats2[47][2] + coeff0_48*dmats2[48][2] + coeff0_49*dmats2[49][2] + coeff0_50*dmats2[50][2] + coeff0_51*dmats2[51][2] + coeff0_52*dmats2[52][2] + coeff0_53*dmats2[53][2] + coeff0_54*dmats2[54][2] + coeff0_55*dmats2[55][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3] + coeff0_4*dmats2[4][3] + coeff0_5*dmats2[5][3] + coeff0_6*dmats2[6][3] + coeff0_7*dmats2[7][3] + coeff0_8*dmats2[8][3] + coeff0_9*dmats2[9][3] + coeff0_10*dmats2[10][3] + coeff0_11*dmats2[11][3] + coeff0_12*dmats2[12][3] + coeff0_13*dmats2[13][3] + coeff0_14*dmats2[14][3] + coeff0_15*dmats2[15][3] + coeff0_16*dmats2[16][3] + coeff0_17*dmats2[17][3] + coeff0_18*dmats2[18][3] + coeff0_19*dmats2[19][3] + coeff0_20*dmats2[20][3] + coeff0_21*dmats2[21][3] + coeff0_22*dmats2[22][3] + coeff0_23*dmats2[23][3] + coeff0_24*dmats2[24][3] + coeff0_25*dmats2[25][3] + coeff0_26*dmats2[26][3] + coeff0_27*dmats2[27][3] + coeff0_28*dmats2[28][3] + coeff0_29*dmats2[29][3] + coeff0_30*dmats2[30][3] + coeff0_31*dmats2[31][3] + coeff0_32*dmats2[32][3] + coeff0_33*dmats2[33][3] + coeff0_34*dmats2[34][3] + coeff0_35*dmats2[35][3] + coeff0_36*dmats2[36][3] + coeff0_37*dmats2[37][3] + coeff0_38*dmats2[38][3] + coeff0_39*dmats2[39][3] + coeff0_40*dmats2[40][3] + coeff0_41*dmats2[41][3] + coeff0_42*dmats2[42][3] + coeff0_43*dmats2[43][3] + coeff0_44*dmats2[44][3] + coeff0_45*dmats2[45][3] + coeff0_46*dmats2[46][3] + coeff0_47*dmats2[47][3] + coeff0_48*dmats2[48][3] + coeff0_49*dmats2[49][3] + coeff0_50*dmats2[50][3] + coeff0_51*dmats2[51][3] + coeff0_52*dmats2[52][3] + coeff0_53*dmats2[53][3] + coeff0_54*dmats2[54][3] + coeff0_55*dmats2[55][3];
          new_coeff0_4 = coeff0_0*dmats2[0][4] + coeff0_1*dmats2[1][4] + coeff0_2*dmats2[2][4] + coeff0_3*dmats2[3][4] + coeff0_4*dmats2[4][4] + coeff0_5*dmats2[5][4] + coeff0_6*dmats2[6][4] + coeff0_7*dmats2[7][4] + coeff0_8*dmats2[8][4] + coeff0_9*dmats2[9][4] + coeff0_10*dmats2[10][4] + coeff0_11*dmats2[11][4] + coeff0_12*dmats2[12][4] + coeff0_13*dmats2[13][4] + coeff0_14*dmats2[14][4] + coeff0_15*dmats2[15][4] + coeff0_16*dmats2[16][4] + coeff0_17*dmats2[17][4] + coeff0_18*dmats2[18][4] + coeff0_19*dmats2[19][4] + coeff0_20*dmats2[20][4] + coeff0_21*dmats2[21][4] + coeff0_22*dmats2[22][4] + coeff0_23*dmats2[23][4] + coeff0_24*dmats2[24][4] + coeff0_25*dmats2[25][4] + coeff0_26*dmats2[26][4] + coeff0_27*dmats2[27][4] + coeff0_28*dmats2[28][4] + coeff0_29*dmats2[29][4] + coeff0_30*dmats2[30][4] + coeff0_31*dmats2[31][4] + coeff0_32*dmats2[32][4] + coeff0_33*dmats2[33][4] + coeff0_34*dmats2[34][4] + coeff0_35*dmats2[35][4] + coeff0_36*dmats2[36][4] + coeff0_37*dmats2[37][4] + coeff0_38*dmats2[38][4] + coeff0_39*dmats2[39][4] + coeff0_40*dmats2[40][4] + coeff0_41*dmats2[41][4] + coeff0_42*dmats2[42][4] + coeff0_43*dmats2[43][4] + coeff0_44*dmats2[44][4] + coeff0_45*dmats2[45][4] + coeff0_46*dmats2[46][4] + coeff0_47*dmats2[47][4] + coeff0_48*dmats2[48][4] + coeff0_49*dmats2[49][4] + coeff0_50*dmats2[50][4] + coeff0_51*dmats2[51][4] + coeff0_52*dmats2[52][4] + coeff0_53*dmats2[53][4] + coeff0_54*dmats2[54][4] + coeff0_55*dmats2[55][4];
          new_coeff0_5 = coeff0_0*dmats2[0][5] + coeff0_1*dmats2[1][5] + coeff0_2*dmats2[2][5] + coeff0_3*dmats2[3][5] + coeff0_4*dmats2[4][5] + coeff0_5*dmats2[5][5] + coeff0_6*dmats2[6][5] + coeff0_7*dmats2[7][5] + coeff0_8*dmats2[8][5] + coeff0_9*dmats2[9][5] + coeff0_10*dmats2[10][5] + coeff0_11*dmats2[11][5] + coeff0_12*dmats2[12][5] + coeff0_13*dmats2[13][5] + coeff0_14*dmats2[14][5] + coeff0_15*dmats2[15][5] + coeff0_16*dmats2[16][5] + coeff0_17*dmats2[17][5] + coeff0_18*dmats2[18][5] + coeff0_19*dmats2[19][5] + coeff0_20*dmats2[20][5] + coeff0_21*dmats2[21][5] + coeff0_22*dmats2[22][5] + coeff0_23*dmats2[23][5] + coeff0_24*dmats2[24][5] + coeff0_25*dmats2[25][5] + coeff0_26*dmats2[26][5] + coeff0_27*dmats2[27][5] + coeff0_28*dmats2[28][5] + coeff0_29*dmats2[29][5] + coeff0_30*dmats2[30][5] + coeff0_31*dmats2[31][5] + coeff0_32*dmats2[32][5] + coeff0_33*dmats2[33][5] + coeff0_34*dmats2[34][5] + coeff0_35*dmats2[35][5] + coeff0_36*dmats2[36][5] + coeff0_37*dmats2[37][5] + coeff0_38*dmats2[38][5] + coeff0_39*dmats2[39][5] + coeff0_40*dmats2[40][5] + coeff0_41*dmats2[41][5] + coeff0_42*dmats2[42][5] + coeff0_43*dmats2[43][5] + coeff0_44*dmats2[44][5] + coeff0_45*dmats2[45][5] + coeff0_46*dmats2[46][5] + coeff0_47*dmats2[47][5] + coeff0_48*dmats2[48][5] + coeff0_49*dmats2[49][5] + coeff0_50*dmats2[50][5] + coeff0_51*dmats2[51][5] + coeff0_52*dmats2[52][5] + coeff0_53*dmats2[53][5] + coeff0_54*dmats2[54][5] + coeff0_55*dmats2[55][5];
          new_coeff0_6 = coeff0_0*dmats2[0][6] + coeff0_1*dmats2[1][6] + coeff0_2*dmats2[2][6] + coeff0_3*dmats2[3][6] + coeff0_4*dmats2[4][6] + coeff0_5*dmats2[5][6] + coeff0_6*dmats2[6][6] + coeff0_7*dmats2[7][6] + coeff0_8*dmats2[8][6] + coeff0_9*dmats2[9][6] + coeff0_10*dmats2[10][6] + coeff0_11*dmats2[11][6] + coeff0_12*dmats2[12][6] + coeff0_13*dmats2[13][6] + coeff0_14*dmats2[14][6] + coeff0_15*dmats2[15][6] + coeff0_16*dmats2[16][6] + coeff0_17*dmats2[17][6] + coeff0_18*dmats2[18][6] + coeff0_19*dmats2[19][6] + coeff0_20*dmats2[20][6] + coeff0_21*dmats2[21][6] + coeff0_22*dmats2[22][6] + coeff0_23*dmats2[23][6] + coeff0_24*dmats2[24][6] + coeff0_25*dmats2[25][6] + coeff0_26*dmats2[26][6] + coeff0_27*dmats2[27][6] + coeff0_28*dmats2[28][6] + coeff0_29*dmats2[29][6] + coeff0_30*dmats2[30][6] + coeff0_31*dmats2[31][6] + coeff0_32*dmats2[32][6] + coeff0_33*dmats2[33][6] + coeff0_34*dmats2[34][6] + coeff0_35*dmats2[35][6] + coeff0_36*dmats2[36][6] + coeff0_37*dmats2[37][6] + coeff0_38*dmats2[38][6] + coeff0_39*dmats2[39][6] + coeff0_40*dmats2[40][6] + coeff0_41*dmats2[41][6] + coeff0_42*dmats2[42][6] + coeff0_43*dmats2[43][6] + coeff0_44*dmats2[44][6] + coeff0_45*dmats2[45][6] + coeff0_46*dmats2[46][6] + coeff0_47*dmats2[47][6] + coeff0_48*dmats2[48][6] + coeff0_49*dmats2[49][6] + coeff0_50*dmats2[50][6] + coeff0_51*dmats2[51][6] + coeff0_52*dmats2[52][6] + coeff0_53*dmats2[53][6] + coeff0_54*dmats2[54][6] + coeff0_55*dmats2[55][6];
          new_coeff0_7 = coeff0_0*dmats2[0][7] + coeff0_1*dmats2[1][7] + coeff0_2*dmats2[2][7] + coeff0_3*dmats2[3][7] + coeff0_4*dmats2[4][7] + coeff0_5*dmats2[5][7] + coeff0_6*dmats2[6][7] + coeff0_7*dmats2[7][7] + coeff0_8*dmats2[8][7] + coeff0_9*dmats2[9][7] + coeff0_10*dmats2[10][7] + coeff0_11*dmats2[11][7] + coeff0_12*dmats2[12][7] + coeff0_13*dmats2[13][7] + coeff0_14*dmats2[14][7] + coeff0_15*dmats2[15][7] + coeff0_16*dmats2[16][7] + coeff0_17*dmats2[17][7] + coeff0_18*dmats2[18][7] + coeff0_19*dmats2[19][7] + coeff0_20*dmats2[20][7] + coeff0_21*dmats2[21][7] + coeff0_22*dmats2[22][7] + coeff0_23*dmats2[23][7] + coeff0_24*dmats2[24][7] + coeff0_25*dmats2[25][7] + coeff0_26*dmats2[26][7] + coeff0_27*dmats2[27][7] + coeff0_28*dmats2[28][7] + coeff0_29*dmats2[29][7] + coeff0_30*dmats2[30][7] + coeff0_31*dmats2[31][7] + coeff0_32*dmats2[32][7] + coeff0_33*dmats2[33][7] + coeff0_34*dmats2[34][7] + coeff0_35*dmats2[35][7] + coeff0_36*dmats2[36][7] + coeff0_37*dmats2[37][7] + coeff0_38*dmats2[38][7] + coeff0_39*dmats2[39][7] + coeff0_40*dmats2[40][7] + coeff0_41*dmats2[41][7] + coeff0_42*dmats2[42][7] + coeff0_43*dmats2[43][7] + coeff0_44*dmats2[44][7] + coeff0_45*dmats2[45][7] + coeff0_46*dmats2[46][7] + coeff0_47*dmats2[47][7] + coeff0_48*dmats2[48][7] + coeff0_49*dmats2[49][7] + coeff0_50*dmats2[50][7] + coeff0_51*dmats2[51][7] + coeff0_52*dmats2[52][7] + coeff0_53*dmats2[53][7] + coeff0_54*dmats2[54][7] + coeff0_55*dmats2[55][7];
          new_coeff0_8 = coeff0_0*dmats2[0][8] + coeff0_1*dmats2[1][8] + coeff0_2*dmats2[2][8] + coeff0_3*dmats2[3][8] + coeff0_4*dmats2[4][8] + coeff0_5*dmats2[5][8] + coeff0_6*dmats2[6][8] + coeff0_7*dmats2[7][8] + coeff0_8*dmats2[8][8] + coeff0_9*dmats2[9][8] + coeff0_10*dmats2[10][8] + coeff0_11*dmats2[11][8] + coeff0_12*dmats2[12][8] + coeff0_13*dmats2[13][8] + coeff0_14*dmats2[14][8] + coeff0_15*dmats2[15][8] + coeff0_16*dmats2[16][8] + coeff0_17*dmats2[17][8] + coeff0_18*dmats2[18][8] + coeff0_19*dmats2[19][8] + coeff0_20*dmats2[20][8] + coeff0_21*dmats2[21][8] + coeff0_22*dmats2[22][8] + coeff0_23*dmats2[23][8] + coeff0_24*dmats2[24][8] + coeff0_25*dmats2[25][8] + coeff0_26*dmats2[26][8] + coeff0_27*dmats2[27][8] + coeff0_28*dmats2[28][8] + coeff0_29*dmats2[29][8] + coeff0_30*dmats2[30][8] + coeff0_31*dmats2[31][8] + coeff0_32*dmats2[32][8] + coeff0_33*dmats2[33][8] + coeff0_34*dmats2[34][8] + coeff0_35*dmats2[35][8] + coeff0_36*dmats2[36][8] + coeff0_37*dmats2[37][8] + coeff0_38*dmats2[38][8] + coeff0_39*dmats2[39][8] + coeff0_40*dmats2[40][8] + coeff0_41*dmats2[41][8] + coeff0_42*dmats2[42][8] + coeff0_43*dmats2[43][8] + coeff0_44*dmats2[44][8] + coeff0_45*dmats2[45][8] + coeff0_46*dmats2[46][8] + coeff0_47*dmats2[47][8] + coeff0_48*dmats2[48][8] + coeff0_49*dmats2[49][8] + coeff0_50*dmats2[50][8] + coeff0_51*dmats2[51][8] + coeff0_52*dmats2[52][8] + coeff0_53*dmats2[53][8] + coeff0_54*dmats2[54][8] + coeff0_55*dmats2[55][8];
          new_coeff0_9 = coeff0_0*dmats2[0][9] + coeff0_1*dmats2[1][9] + coeff0_2*dmats2[2][9] + coeff0_3*dmats2[3][9] + coeff0_4*dmats2[4][9] + coeff0_5*dmats2[5][9] + coeff0_6*dmats2[6][9] + coeff0_7*dmats2[7][9] + coeff0_8*dmats2[8][9] + coeff0_9*dmats2[9][9] + coeff0_10*dmats2[10][9] + coeff0_11*dmats2[11][9] + coeff0_12*dmats2[12][9] + coeff0_13*dmats2[13][9] + coeff0_14*dmats2[14][9] + coeff0_15*dmats2[15][9] + coeff0_16*dmats2[16][9] + coeff0_17*dmats2[17][9] + coeff0_18*dmats2[18][9] + coeff0_19*dmats2[19][9] + coeff0_20*dmats2[20][9] + coeff0_21*dmats2[21][9] + coeff0_22*dmats2[22][9] + coeff0_23*dmats2[23][9] + coeff0_24*dmats2[24][9] + coeff0_25*dmats2[25][9] + coeff0_26*dmats2[26][9] + coeff0_27*dmats2[27][9] + coeff0_28*dmats2[28][9] + coeff0_29*dmats2[29][9] + coeff0_30*dmats2[30][9] + coeff0_31*dmats2[31][9] + coeff0_32*dmats2[32][9] + coeff0_33*dmats2[33][9] + coeff0_34*dmats2[34][9] + coeff0_35*dmats2[35][9] + coeff0_36*dmats2[36][9] + coeff0_37*dmats2[37][9] + coeff0_38*dmats2[38][9] + coeff0_39*dmats2[39][9] + coeff0_40*dmats2[40][9] + coeff0_41*dmats2[41][9] + coeff0_42*dmats2[42][9] + coeff0_43*dmats2[43][9] + coeff0_44*dmats2[44][9] + coeff0_45*dmats2[45][9] + coeff0_46*dmats2[46][9] + coeff0_47*dmats2[47][9] + coeff0_48*dmats2[48][9] + coeff0_49*dmats2[49][9] + coeff0_50*dmats2[50][9] + coeff0_51*dmats2[51][9] + coeff0_52*dmats2[52][9] + coeff0_53*dmats2[53][9] + coeff0_54*dmats2[54][9] + coeff0_55*dmats2[55][9];
          new_coeff0_10 = coeff0_0*dmats2[0][10] + coeff0_1*dmats2[1][10] + coeff0_2*dmats2[2][10] + coeff0_3*dmats2[3][10] + coeff0_4*dmats2[4][10] + coeff0_5*dmats2[5][10] + coeff0_6*dmats2[6][10] + coeff0_7*dmats2[7][10] + coeff0_8*dmats2[8][10] + coeff0_9*dmats2[9][10] + coeff0_10*dmats2[10][10] + coeff0_11*dmats2[11][10] + coeff0_12*dmats2[12][10] + coeff0_13*dmats2[13][10] + coeff0_14*dmats2[14][10] + coeff0_15*dmats2[15][10] + coeff0_16*dmats2[16][10] + coeff0_17*dmats2[17][10] + coeff0_18*dmats2[18][10] + coeff0_19*dmats2[19][10] + coeff0_20*dmats2[20][10] + coeff0_21*dmats2[21][10] + coeff0_22*dmats2[22][10] + coeff0_23*dmats2[23][10] + coeff0_24*dmats2[24][10] + coeff0_25*dmats2[25][10] + coeff0_26*dmats2[26][10] + coeff0_27*dmats2[27][10] + coeff0_28*dmats2[28][10] + coeff0_29*dmats2[29][10] + coeff0_30*dmats2[30][10] + coeff0_31*dmats2[31][10] + coeff0_32*dmats2[32][10] + coeff0_33*dmats2[33][10] + coeff0_34*dmats2[34][10] + coeff0_35*dmats2[35][10] + coeff0_36*dmats2[36][10] + coeff0_37*dmats2[37][10] + coeff0_38*dmats2[38][10] + coeff0_39*dmats2[39][10] + coeff0_40*dmats2[40][10] + coeff0_41*dmats2[41][10] + coeff0_42*dmats2[42][10] + coeff0_43*dmats2[43][10] + coeff0_44*dmats2[44][10] + coeff0_45*dmats2[45][10] + coeff0_46*dmats2[46][10] + coeff0_47*dmats2[47][10] + coeff0_48*dmats2[48][10] + coeff0_49*dmats2[49][10] + coeff0_50*dmats2[50][10] + coeff0_51*dmats2[51][10] + coeff0_52*dmats2[52][10] + coeff0_53*dmats2[53][10] + coeff0_54*dmats2[54][10] + coeff0_55*dmats2[55][10];
          new_coeff0_11 = coeff0_0*dmats2[0][11] + coeff0_1*dmats2[1][11] + coeff0_2*dmats2[2][11] + coeff0_3*dmats2[3][11] + coeff0_4*dmats2[4][11] + coeff0_5*dmats2[5][11] + coeff0_6*dmats2[6][11] + coeff0_7*dmats2[7][11] + coeff0_8*dmats2[8][11] + coeff0_9*dmats2[9][11] + coeff0_10*dmats2[10][11] + coeff0_11*dmats2[11][11] + coeff0_12*dmats2[12][11] + coeff0_13*dmats2[13][11] + coeff0_14*dmats2[14][11] + coeff0_15*dmats2[15][11] + coeff0_16*dmats2[16][11] + coeff0_17*dmats2[17][11] + coeff0_18*dmats2[18][11] + coeff0_19*dmats2[19][11] + coeff0_20*dmats2[20][11] + coeff0_21*dmats2[21][11] + coeff0_22*dmats2[22][11] + coeff0_23*dmats2[23][11] + coeff0_24*dmats2[24][11] + coeff0_25*dmats2[25][11] + coeff0_26*dmats2[26][11] + coeff0_27*dmats2[27][11] + coeff0_28*dmats2[28][11] + coeff0_29*dmats2[29][11] + coeff0_30*dmats2[30][11] + coeff0_31*dmats2[31][11] + coeff0_32*dmats2[32][11] + coeff0_33*dmats2[33][11] + coeff0_34*dmats2[34][11] + coeff0_35*dmats2[35][11] + coeff0_36*dmats2[36][11] + coeff0_37*dmats2[37][11] + coeff0_38*dmats2[38][11] + coeff0_39*dmats2[39][11] + coeff0_40*dmats2[40][11] + coeff0_41*dmats2[41][11] + coeff0_42*dmats2[42][11] + coeff0_43*dmats2[43][11] + coeff0_44*dmats2[44][11] + coeff0_45*dmats2[45][11] + coeff0_46*dmats2[46][11] + coeff0_47*dmats2[47][11] + coeff0_48*dmats2[48][11] + coeff0_49*dmats2[49][11] + coeff0_50*dmats2[50][11] + coeff0_51*dmats2[51][11] + coeff0_52*dmats2[52][11] + coeff0_53*dmats2[53][11] + coeff0_54*dmats2[54][11] + coeff0_55*dmats2[55][11];
          new_coeff0_12 = coeff0_0*dmats2[0][12] + coeff0_1*dmats2[1][12] + coeff0_2*dmats2[2][12] + coeff0_3*dmats2[3][12] + coeff0_4*dmats2[4][12] + coeff0_5*dmats2[5][12] + coeff0_6*dmats2[6][12] + coeff0_7*dmats2[7][12] + coeff0_8*dmats2[8][12] + coeff0_9*dmats2[9][12] + coeff0_10*dmats2[10][12] + coeff0_11*dmats2[11][12] + coeff0_12*dmats2[12][12] + coeff0_13*dmats2[13][12] + coeff0_14*dmats2[14][12] + coeff0_15*dmats2[15][12] + coeff0_16*dmats2[16][12] + coeff0_17*dmats2[17][12] + coeff0_18*dmats2[18][12] + coeff0_19*dmats2[19][12] + coeff0_20*dmats2[20][12] + coeff0_21*dmats2[21][12] + coeff0_22*dmats2[22][12] + coeff0_23*dmats2[23][12] + coeff0_24*dmats2[24][12] + coeff0_25*dmats2[25][12] + coeff0_26*dmats2[26][12] + coeff0_27*dmats2[27][12] + coeff0_28*dmats2[28][12] + coeff0_29*dmats2[29][12] + coeff0_30*dmats2[30][12] + coeff0_31*dmats2[31][12] + coeff0_32*dmats2[32][12] + coeff0_33*dmats2[33][12] + coeff0_34*dmats2[34][12] + coeff0_35*dmats2[35][12] + coeff0_36*dmats2[36][12] + coeff0_37*dmats2[37][12] + coeff0_38*dmats2[38][12] + coeff0_39*dmats2[39][12] + coeff0_40*dmats2[40][12] + coeff0_41*dmats2[41][12] + coeff0_42*dmats2[42][12] + coeff0_43*dmats2[43][12] + coeff0_44*dmats2[44][12] + coeff0_45*dmats2[45][12] + coeff0_46*dmats2[46][12] + coeff0_47*dmats2[47][12] + coeff0_48*dmats2[48][12] + coeff0_49*dmats2[49][12] + coeff0_50*dmats2[50][12] + coeff0_51*dmats2[51][12] + coeff0_52*dmats2[52][12] + coeff0_53*dmats2[53][12] + coeff0_54*dmats2[54][12] + coeff0_55*dmats2[55][12];
          new_coeff0_13 = coeff0_0*dmats2[0][13] + coeff0_1*dmats2[1][13] + coeff0_2*dmats2[2][13] + coeff0_3*dmats2[3][13] + coeff0_4*dmats2[4][13] + coeff0_5*dmats2[5][13] + coeff0_6*dmats2[6][13] + coeff0_7*dmats2[7][13] + coeff0_8*dmats2[8][13] + coeff0_9*dmats2[9][13] + coeff0_10*dmats2[10][13] + coeff0_11*dmats2[11][13] + coeff0_12*dmats2[12][13] + coeff0_13*dmats2[13][13] + coeff0_14*dmats2[14][13] + coeff0_15*dmats2[15][13] + coeff0_16*dmats2[16][13] + coeff0_17*dmats2[17][13] + coeff0_18*dmats2[18][13] + coeff0_19*dmats2[19][13] + coeff0_20*dmats2[20][13] + coeff0_21*dmats2[21][13] + coeff0_22*dmats2[22][13] + coeff0_23*dmats2[23][13] + coeff0_24*dmats2[24][13] + coeff0_25*dmats2[25][13] + coeff0_26*dmats2[26][13] + coeff0_27*dmats2[27][13] + coeff0_28*dmats2[28][13] + coeff0_29*dmats2[29][13] + coeff0_30*dmats2[30][13] + coeff0_31*dmats2[31][13] + coeff0_32*dmats2[32][13] + coeff0_33*dmats2[33][13] + coeff0_34*dmats2[34][13] + coeff0_35*dmats2[35][13] + coeff0_36*dmats2[36][13] + coeff0_37*dmats2[37][13] + coeff0_38*dmats2[38][13] + coeff0_39*dmats2[39][13] + coeff0_40*dmats2[40][13] + coeff0_41*dmats2[41][13] + coeff0_42*dmats2[42][13] + coeff0_43*dmats2[43][13] + coeff0_44*dmats2[44][13] + coeff0_45*dmats2[45][13] + coeff0_46*dmats2[46][13] + coeff0_47*dmats2[47][13] + coeff0_48*dmats2[48][13] + coeff0_49*dmats2[49][13] + coeff0_50*dmats2[50][13] + coeff0_51*dmats2[51][13] + coeff0_52*dmats2[52][13] + coeff0_53*dmats2[53][13] + coeff0_54*dmats2[54][13] + coeff0_55*dmats2[55][13];
          new_coeff0_14 = coeff0_0*dmats2[0][14] + coeff0_1*dmats2[1][14] + coeff0_2*dmats2[2][14] + coeff0_3*dmats2[3][14] + coeff0_4*dmats2[4][14] + coeff0_5*dmats2[5][14] + coeff0_6*dmats2[6][14] + coeff0_7*dmats2[7][14] + coeff0_8*dmats2[8][14] + coeff0_9*dmats2[9][14] + coeff0_10*dmats2[10][14] + coeff0_11*dmats2[11][14] + coeff0_12*dmats2[12][14] + coeff0_13*dmats2[13][14] + coeff0_14*dmats2[14][14] + coeff0_15*dmats2[15][14] + coeff0_16*dmats2[16][14] + coeff0_17*dmats2[17][14] + coeff0_18*dmats2[18][14] + coeff0_19*dmats2[19][14] + coeff0_20*dmats2[20][14] + coeff0_21*dmats2[21][14] + coeff0_22*dmats2[22][14] + coeff0_23*dmats2[23][14] + coeff0_24*dmats2[24][14] + coeff0_25*dmats2[25][14] + coeff0_26*dmats2[26][14] + coeff0_27*dmats2[27][14] + coeff0_28*dmats2[28][14] + coeff0_29*dmats2[29][14] + coeff0_30*dmats2[30][14] + coeff0_31*dmats2[31][14] + coeff0_32*dmats2[32][14] + coeff0_33*dmats2[33][14] + coeff0_34*dmats2[34][14] + coeff0_35*dmats2[35][14] + coeff0_36*dmats2[36][14] + coeff0_37*dmats2[37][14] + coeff0_38*dmats2[38][14] + coeff0_39*dmats2[39][14] + coeff0_40*dmats2[40][14] + coeff0_41*dmats2[41][14] + coeff0_42*dmats2[42][14] + coeff0_43*dmats2[43][14] + coeff0_44*dmats2[44][14] + coeff0_45*dmats2[45][14] + coeff0_46*dmats2[46][14] + coeff0_47*dmats2[47][14] + coeff0_48*dmats2[48][14] + coeff0_49*dmats2[49][14] + coeff0_50*dmats2[50][14] + coeff0_51*dmats2[51][14] + coeff0_52*dmats2[52][14] + coeff0_53*dmats2[53][14] + coeff0_54*dmats2[54][14] + coeff0_55*dmats2[55][14];
          new_coeff0_15 = coeff0_0*dmats2[0][15] + coeff0_1*dmats2[1][15] + coeff0_2*dmats2[2][15] + coeff0_3*dmats2[3][15] + coeff0_4*dmats2[4][15] + coeff0_5*dmats2[5][15] + coeff0_6*dmats2[6][15] + coeff0_7*dmats2[7][15] + coeff0_8*dmats2[8][15] + coeff0_9*dmats2[9][15] + coeff0_10*dmats2[10][15] + coeff0_11*dmats2[11][15] + coeff0_12*dmats2[12][15] + coeff0_13*dmats2[13][15] + coeff0_14*dmats2[14][15] + coeff0_15*dmats2[15][15] + coeff0_16*dmats2[16][15] + coeff0_17*dmats2[17][15] + coeff0_18*dmats2[18][15] + coeff0_19*dmats2[19][15] + coeff0_20*dmats2[20][15] + coeff0_21*dmats2[21][15] + coeff0_22*dmats2[22][15] + coeff0_23*dmats2[23][15] + coeff0_24*dmats2[24][15] + coeff0_25*dmats2[25][15] + coeff0_26*dmats2[26][15] + coeff0_27*dmats2[27][15] + coeff0_28*dmats2[28][15] + coeff0_29*dmats2[29][15] + coeff0_30*dmats2[30][15] + coeff0_31*dmats2[31][15] + coeff0_32*dmats2[32][15] + coeff0_33*dmats2[33][15] + coeff0_34*dmats2[34][15] + coeff0_35*dmats2[35][15] + coeff0_36*dmats2[36][15] + coeff0_37*dmats2[37][15] + coeff0_38*dmats2[38][15] + coeff0_39*dmats2[39][15] + coeff0_40*dmats2[40][15] + coeff0_41*dmats2[41][15] + coeff0_42*dmats2[42][15] + coeff0_43*dmats2[43][15] + coeff0_44*dmats2[44][15] + coeff0_45*dmats2[45][15] + coeff0_46*dmats2[46][15] + coeff0_47*dmats2[47][15] + coeff0_48*dmats2[48][15] + coeff0_49*dmats2[49][15] + coeff0_50*dmats2[50][15] + coeff0_51*dmats2[51][15] + coeff0_52*dmats2[52][15] + coeff0_53*dmats2[53][15] + coeff0_54*dmats2[54][15] + coeff0_55*dmats2[55][15];
          new_coeff0_16 = coeff0_0*dmats2[0][16] + coeff0_1*dmats2[1][16] + coeff0_2*dmats2[2][16] + coeff0_3*dmats2[3][16] + coeff0_4*dmats2[4][16] + coeff0_5*dmats2[5][16] + coeff0_6*dmats2[6][16] + coeff0_7*dmats2[7][16] + coeff0_8*dmats2[8][16] + coeff0_9*dmats2[9][16] + coeff0_10*dmats2[10][16] + coeff0_11*dmats2[11][16] + coeff0_12*dmats2[12][16] + coeff0_13*dmats2[13][16] + coeff0_14*dmats2[14][16] + coeff0_15*dmats2[15][16] + coeff0_16*dmats2[16][16] + coeff0_17*dmats2[17][16] + coeff0_18*dmats2[18][16] + coeff0_19*dmats2[19][16] + coeff0_20*dmats2[20][16] + coeff0_21*dmats2[21][16] + coeff0_22*dmats2[22][16] + coeff0_23*dmats2[23][16] + coeff0_24*dmats2[24][16] + coeff0_25*dmats2[25][16] + coeff0_26*dmats2[26][16] + coeff0_27*dmats2[27][16] + coeff0_28*dmats2[28][16] + coeff0_29*dmats2[29][16] + coeff0_30*dmats2[30][16] + coeff0_31*dmats2[31][16] + coeff0_32*dmats2[32][16] + coeff0_33*dmats2[33][16] + coeff0_34*dmats2[34][16] + coeff0_35*dmats2[35][16] + coeff0_36*dmats2[36][16] + coeff0_37*dmats2[37][16] + coeff0_38*dmats2[38][16] + coeff0_39*dmats2[39][16] + coeff0_40*dmats2[40][16] + coeff0_41*dmats2[41][16] + coeff0_42*dmats2[42][16] + coeff0_43*dmats2[43][16] + coeff0_44*dmats2[44][16] + coeff0_45*dmats2[45][16] + coeff0_46*dmats2[46][16] + coeff0_47*dmats2[47][16] + coeff0_48*dmats2[48][16] + coeff0_49*dmats2[49][16] + coeff0_50*dmats2[50][16] + coeff0_51*dmats2[51][16] + coeff0_52*dmats2[52][16] + coeff0_53*dmats2[53][16] + coeff0_54*dmats2[54][16] + coeff0_55*dmats2[55][16];
          new_coeff0_17 = coeff0_0*dmats2[0][17] + coeff0_1*dmats2[1][17] + coeff0_2*dmats2[2][17] + coeff0_3*dmats2[3][17] + coeff0_4*dmats2[4][17] + coeff0_5*dmats2[5][17] + coeff0_6*dmats2[6][17] + coeff0_7*dmats2[7][17] + coeff0_8*dmats2[8][17] + coeff0_9*dmats2[9][17] + coeff0_10*dmats2[10][17] + coeff0_11*dmats2[11][17] + coeff0_12*dmats2[12][17] + coeff0_13*dmats2[13][17] + coeff0_14*dmats2[14][17] + coeff0_15*dmats2[15][17] + coeff0_16*dmats2[16][17] + coeff0_17*dmats2[17][17] + coeff0_18*dmats2[18][17] + coeff0_19*dmats2[19][17] + coeff0_20*dmats2[20][17] + coeff0_21*dmats2[21][17] + coeff0_22*dmats2[22][17] + coeff0_23*dmats2[23][17] + coeff0_24*dmats2[24][17] + coeff0_25*dmats2[25][17] + coeff0_26*dmats2[26][17] + coeff0_27*dmats2[27][17] + coeff0_28*dmats2[28][17] + coeff0_29*dmats2[29][17] + coeff0_30*dmats2[30][17] + coeff0_31*dmats2[31][17] + coeff0_32*dmats2[32][17] + coeff0_33*dmats2[33][17] + coeff0_34*dmats2[34][17] + coeff0_35*dmats2[35][17] + coeff0_36*dmats2[36][17] + coeff0_37*dmats2[37][17] + coeff0_38*dmats2[38][17] + coeff0_39*dmats2[39][17] + coeff0_40*dmats2[40][17] + coeff0_41*dmats2[41][17] + coeff0_42*dmats2[42][17] + coeff0_43*dmats2[43][17] + coeff0_44*dmats2[44][17] + coeff0_45*dmats2[45][17] + coeff0_46*dmats2[46][17] + coeff0_47*dmats2[47][17] + coeff0_48*dmats2[48][17] + coeff0_49*dmats2[49][17] + coeff0_50*dmats2[50][17] + coeff0_51*dmats2[51][17] + coeff0_52*dmats2[52][17] + coeff0_53*dmats2[53][17] + coeff0_54*dmats2[54][17] + coeff0_55*dmats2[55][17];
          new_coeff0_18 = coeff0_0*dmats2[0][18] + coeff0_1*dmats2[1][18] + coeff0_2*dmats2[2][18] + coeff0_3*dmats2[3][18] + coeff0_4*dmats2[4][18] + coeff0_5*dmats2[5][18] + coeff0_6*dmats2[6][18] + coeff0_7*dmats2[7][18] + coeff0_8*dmats2[8][18] + coeff0_9*dmats2[9][18] + coeff0_10*dmats2[10][18] + coeff0_11*dmats2[11][18] + coeff0_12*dmats2[12][18] + coeff0_13*dmats2[13][18] + coeff0_14*dmats2[14][18] + coeff0_15*dmats2[15][18] + coeff0_16*dmats2[16][18] + coeff0_17*dmats2[17][18] + coeff0_18*dmats2[18][18] + coeff0_19*dmats2[19][18] + coeff0_20*dmats2[20][18] + coeff0_21*dmats2[21][18] + coeff0_22*dmats2[22][18] + coeff0_23*dmats2[23][18] + coeff0_24*dmats2[24][18] + coeff0_25*dmats2[25][18] + coeff0_26*dmats2[26][18] + coeff0_27*dmats2[27][18] + coeff0_28*dmats2[28][18] + coeff0_29*dmats2[29][18] + coeff0_30*dmats2[30][18] + coeff0_31*dmats2[31][18] + coeff0_32*dmats2[32][18] + coeff0_33*dmats2[33][18] + coeff0_34*dmats2[34][18] + coeff0_35*dmats2[35][18] + coeff0_36*dmats2[36][18] + coeff0_37*dmats2[37][18] + coeff0_38*dmats2[38][18] + coeff0_39*dmats2[39][18] + coeff0_40*dmats2[40][18] + coeff0_41*dmats2[41][18] + coeff0_42*dmats2[42][18] + coeff0_43*dmats2[43][18] + coeff0_44*dmats2[44][18] + coeff0_45*dmats2[45][18] + coeff0_46*dmats2[46][18] + coeff0_47*dmats2[47][18] + coeff0_48*dmats2[48][18] + coeff0_49*dmats2[49][18] + coeff0_50*dmats2[50][18] + coeff0_51*dmats2[51][18] + coeff0_52*dmats2[52][18] + coeff0_53*dmats2[53][18] + coeff0_54*dmats2[54][18] + coeff0_55*dmats2[55][18];
          new_coeff0_19 = coeff0_0*dmats2[0][19] + coeff0_1*dmats2[1][19] + coeff0_2*dmats2[2][19] + coeff0_3*dmats2[3][19] + coeff0_4*dmats2[4][19] + coeff0_5*dmats2[5][19] + coeff0_6*dmats2[6][19] + coeff0_7*dmats2[7][19] + coeff0_8*dmats2[8][19] + coeff0_9*dmats2[9][19] + coeff0_10*dmats2[10][19] + coeff0_11*dmats2[11][19] + coeff0_12*dmats2[12][19] + coeff0_13*dmats2[13][19] + coeff0_14*dmats2[14][19] + coeff0_15*dmats2[15][19] + coeff0_16*dmats2[16][19] + coeff0_17*dmats2[17][19] + coeff0_18*dmats2[18][19] + coeff0_19*dmats2[19][19] + coeff0_20*dmats2[20][19] + coeff0_21*dmats2[21][19] + coeff0_22*dmats2[22][19] + coeff0_23*dmats2[23][19] + coeff0_24*dmats2[24][19] + coeff0_25*dmats2[25][19] + coeff0_26*dmats2[26][19] + coeff0_27*dmats2[27][19] + coeff0_28*dmats2[28][19] + coeff0_29*dmats2[29][19] + coeff0_30*dmats2[30][19] + coeff0_31*dmats2[31][19] + coeff0_32*dmats2[32][19] + coeff0_33*dmats2[33][19] + coeff0_34*dmats2[34][19] + coeff0_35*dmats2[35][19] + coeff0_36*dmats2[36][19] + coeff0_37*dmats2[37][19] + coeff0_38*dmats2[38][19] + coeff0_39*dmats2[39][19] + coeff0_40*dmats2[40][19] + coeff0_41*dmats2[41][19] + coeff0_42*dmats2[42][19] + coeff0_43*dmats2[43][19] + coeff0_44*dmats2[44][19] + coeff0_45*dmats2[45][19] + coeff0_46*dmats2[46][19] + coeff0_47*dmats2[47][19] + coeff0_48*dmats2[48][19] + coeff0_49*dmats2[49][19] + coeff0_50*dmats2[50][19] + coeff0_51*dmats2[51][19] + coeff0_52*dmats2[52][19] + coeff0_53*dmats2[53][19] + coeff0_54*dmats2[54][19] + coeff0_55*dmats2[55][19];
          new_coeff0_20 = coeff0_0*dmats2[0][20] + coeff0_1*dmats2[1][20] + coeff0_2*dmats2[2][20] + coeff0_3*dmats2[3][20] + coeff0_4*dmats2[4][20] + coeff0_5*dmats2[5][20] + coeff0_6*dmats2[6][20] + coeff0_7*dmats2[7][20] + coeff0_8*dmats2[8][20] + coeff0_9*dmats2[9][20] + coeff0_10*dmats2[10][20] + coeff0_11*dmats2[11][20] + coeff0_12*dmats2[12][20] + coeff0_13*dmats2[13][20] + coeff0_14*dmats2[14][20] + coeff0_15*dmats2[15][20] + coeff0_16*dmats2[16][20] + coeff0_17*dmats2[17][20] + coeff0_18*dmats2[18][20] + coeff0_19*dmats2[19][20] + coeff0_20*dmats2[20][20] + coeff0_21*dmats2[21][20] + coeff0_22*dmats2[22][20] + coeff0_23*dmats2[23][20] + coeff0_24*dmats2[24][20] + coeff0_25*dmats2[25][20] + coeff0_26*dmats2[26][20] + coeff0_27*dmats2[27][20] + coeff0_28*dmats2[28][20] + coeff0_29*dmats2[29][20] + coeff0_30*dmats2[30][20] + coeff0_31*dmats2[31][20] + coeff0_32*dmats2[32][20] + coeff0_33*dmats2[33][20] + coeff0_34*dmats2[34][20] + coeff0_35*dmats2[35][20] + coeff0_36*dmats2[36][20] + coeff0_37*dmats2[37][20] + coeff0_38*dmats2[38][20] + coeff0_39*dmats2[39][20] + coeff0_40*dmats2[40][20] + coeff0_41*dmats2[41][20] + coeff0_42*dmats2[42][20] + coeff0_43*dmats2[43][20] + coeff0_44*dmats2[44][20] + coeff0_45*dmats2[45][20] + coeff0_46*dmats2[46][20] + coeff0_47*dmats2[47][20] + coeff0_48*dmats2[48][20] + coeff0_49*dmats2[49][20] + coeff0_50*dmats2[50][20] + coeff0_51*dmats2[51][20] + coeff0_52*dmats2[52][20] + coeff0_53*dmats2[53][20] + coeff0_54*dmats2[54][20] + coeff0_55*dmats2[55][20];
          new_coeff0_21 = coeff0_0*dmats2[0][21] + coeff0_1*dmats2[1][21] + coeff0_2*dmats2[2][21] + coeff0_3*dmats2[3][21] + coeff0_4*dmats2[4][21] + coeff0_5*dmats2[5][21] + coeff0_6*dmats2[6][21] + coeff0_7*dmats2[7][21] + coeff0_8*dmats2[8][21] + coeff0_9*dmats2[9][21] + coeff0_10*dmats2[10][21] + coeff0_11*dmats2[11][21] + coeff0_12*dmats2[12][21] + coeff0_13*dmats2[13][21] + coeff0_14*dmats2[14][21] + coeff0_15*dmats2[15][21] + coeff0_16*dmats2[16][21] + coeff0_17*dmats2[17][21] + coeff0_18*dmats2[18][21] + coeff0_19*dmats2[19][21] + coeff0_20*dmats2[20][21] + coeff0_21*dmats2[21][21] + coeff0_22*dmats2[22][21] + coeff0_23*dmats2[23][21] + coeff0_24*dmats2[24][21] + coeff0_25*dmats2[25][21] + coeff0_26*dmats2[26][21] + coeff0_27*dmats2[27][21] + coeff0_28*dmats2[28][21] + coeff0_29*dmats2[29][21] + coeff0_30*dmats2[30][21] + coeff0_31*dmats2[31][21] + coeff0_32*dmats2[32][21] + coeff0_33*dmats2[33][21] + coeff0_34*dmats2[34][21] + coeff0_35*dmats2[35][21] + coeff0_36*dmats2[36][21] + coeff0_37*dmats2[37][21] + coeff0_38*dmats2[38][21] + coeff0_39*dmats2[39][21] + coeff0_40*dmats2[40][21] + coeff0_41*dmats2[41][21] + coeff0_42*dmats2[42][21] + coeff0_43*dmats2[43][21] + coeff0_44*dmats2[44][21] + coeff0_45*dmats2[45][21] + coeff0_46*dmats2[46][21] + coeff0_47*dmats2[47][21] + coeff0_48*dmats2[48][21] + coeff0_49*dmats2[49][21] + coeff0_50*dmats2[50][21] + coeff0_51*dmats2[51][21] + coeff0_52*dmats2[52][21] + coeff0_53*dmats2[53][21] + coeff0_54*dmats2[54][21] + coeff0_55*dmats2[55][21];
          new_coeff0_22 = coeff0_0*dmats2[0][22] + coeff0_1*dmats2[1][22] + coeff0_2*dmats2[2][22] + coeff0_3*dmats2[3][22] + coeff0_4*dmats2[4][22] + coeff0_5*dmats2[5][22] + coeff0_6*dmats2[6][22] + coeff0_7*dmats2[7][22] + coeff0_8*dmats2[8][22] + coeff0_9*dmats2[9][22] + coeff0_10*dmats2[10][22] + coeff0_11*dmats2[11][22] + coeff0_12*dmats2[12][22] + coeff0_13*dmats2[13][22] + coeff0_14*dmats2[14][22] + coeff0_15*dmats2[15][22] + coeff0_16*dmats2[16][22] + coeff0_17*dmats2[17][22] + coeff0_18*dmats2[18][22] + coeff0_19*dmats2[19][22] + coeff0_20*dmats2[20][22] + coeff0_21*dmats2[21][22] + coeff0_22*dmats2[22][22] + coeff0_23*dmats2[23][22] + coeff0_24*dmats2[24][22] + coeff0_25*dmats2[25][22] + coeff0_26*dmats2[26][22] + coeff0_27*dmats2[27][22] + coeff0_28*dmats2[28][22] + coeff0_29*dmats2[29][22] + coeff0_30*dmats2[30][22] + coeff0_31*dmats2[31][22] + coeff0_32*dmats2[32][22] + coeff0_33*dmats2[33][22] + coeff0_34*dmats2[34][22] + coeff0_35*dmats2[35][22] + coeff0_36*dmats2[36][22] + coeff0_37*dmats2[37][22] + coeff0_38*dmats2[38][22] + coeff0_39*dmats2[39][22] + coeff0_40*dmats2[40][22] + coeff0_41*dmats2[41][22] + coeff0_42*dmats2[42][22] + coeff0_43*dmats2[43][22] + coeff0_44*dmats2[44][22] + coeff0_45*dmats2[45][22] + coeff0_46*dmats2[46][22] + coeff0_47*dmats2[47][22] + coeff0_48*dmats2[48][22] + coeff0_49*dmats2[49][22] + coeff0_50*dmats2[50][22] + coeff0_51*dmats2[51][22] + coeff0_52*dmats2[52][22] + coeff0_53*dmats2[53][22] + coeff0_54*dmats2[54][22] + coeff0_55*dmats2[55][22];
          new_coeff0_23 = coeff0_0*dmats2[0][23] + coeff0_1*dmats2[1][23] + coeff0_2*dmats2[2][23] + coeff0_3*dmats2[3][23] + coeff0_4*dmats2[4][23] + coeff0_5*dmats2[5][23] + coeff0_6*dmats2[6][23] + coeff0_7*dmats2[7][23] + coeff0_8*dmats2[8][23] + coeff0_9*dmats2[9][23] + coeff0_10*dmats2[10][23] + coeff0_11*dmats2[11][23] + coeff0_12*dmats2[12][23] + coeff0_13*dmats2[13][23] + coeff0_14*dmats2[14][23] + coeff0_15*dmats2[15][23] + coeff0_16*dmats2[16][23] + coeff0_17*dmats2[17][23] + coeff0_18*dmats2[18][23] + coeff0_19*dmats2[19][23] + coeff0_20*dmats2[20][23] + coeff0_21*dmats2[21][23] + coeff0_22*dmats2[22][23] + coeff0_23*dmats2[23][23] + coeff0_24*dmats2[24][23] + coeff0_25*dmats2[25][23] + coeff0_26*dmats2[26][23] + coeff0_27*dmats2[27][23] + coeff0_28*dmats2[28][23] + coeff0_29*dmats2[29][23] + coeff0_30*dmats2[30][23] + coeff0_31*dmats2[31][23] + coeff0_32*dmats2[32][23] + coeff0_33*dmats2[33][23] + coeff0_34*dmats2[34][23] + coeff0_35*dmats2[35][23] + coeff0_36*dmats2[36][23] + coeff0_37*dmats2[37][23] + coeff0_38*dmats2[38][23] + coeff0_39*dmats2[39][23] + coeff0_40*dmats2[40][23] + coeff0_41*dmats2[41][23] + coeff0_42*dmats2[42][23] + coeff0_43*dmats2[43][23] + coeff0_44*dmats2[44][23] + coeff0_45*dmats2[45][23] + coeff0_46*dmats2[46][23] + coeff0_47*dmats2[47][23] + coeff0_48*dmats2[48][23] + coeff0_49*dmats2[49][23] + coeff0_50*dmats2[50][23] + coeff0_51*dmats2[51][23] + coeff0_52*dmats2[52][23] + coeff0_53*dmats2[53][23] + coeff0_54*dmats2[54][23] + coeff0_55*dmats2[55][23];
          new_coeff0_24 = coeff0_0*dmats2[0][24] + coeff0_1*dmats2[1][24] + coeff0_2*dmats2[2][24] + coeff0_3*dmats2[3][24] + coeff0_4*dmats2[4][24] + coeff0_5*dmats2[5][24] + coeff0_6*dmats2[6][24] + coeff0_7*dmats2[7][24] + coeff0_8*dmats2[8][24] + coeff0_9*dmats2[9][24] + coeff0_10*dmats2[10][24] + coeff0_11*dmats2[11][24] + coeff0_12*dmats2[12][24] + coeff0_13*dmats2[13][24] + coeff0_14*dmats2[14][24] + coeff0_15*dmats2[15][24] + coeff0_16*dmats2[16][24] + coeff0_17*dmats2[17][24] + coeff0_18*dmats2[18][24] + coeff0_19*dmats2[19][24] + coeff0_20*dmats2[20][24] + coeff0_21*dmats2[21][24] + coeff0_22*dmats2[22][24] + coeff0_23*dmats2[23][24] + coeff0_24*dmats2[24][24] + coeff0_25*dmats2[25][24] + coeff0_26*dmats2[26][24] + coeff0_27*dmats2[27][24] + coeff0_28*dmats2[28][24] + coeff0_29*dmats2[29][24] + coeff0_30*dmats2[30][24] + coeff0_31*dmats2[31][24] + coeff0_32*dmats2[32][24] + coeff0_33*dmats2[33][24] + coeff0_34*dmats2[34][24] + coeff0_35*dmats2[35][24] + coeff0_36*dmats2[36][24] + coeff0_37*dmats2[37][24] + coeff0_38*dmats2[38][24] + coeff0_39*dmats2[39][24] + coeff0_40*dmats2[40][24] + coeff0_41*dmats2[41][24] + coeff0_42*dmats2[42][24] + coeff0_43*dmats2[43][24] + coeff0_44*dmats2[44][24] + coeff0_45*dmats2[45][24] + coeff0_46*dmats2[46][24] + coeff0_47*dmats2[47][24] + coeff0_48*dmats2[48][24] + coeff0_49*dmats2[49][24] + coeff0_50*dmats2[50][24] + coeff0_51*dmats2[51][24] + coeff0_52*dmats2[52][24] + coeff0_53*dmats2[53][24] + coeff0_54*dmats2[54][24] + coeff0_55*dmats2[55][24];
          new_coeff0_25 = coeff0_0*dmats2[0][25] + coeff0_1*dmats2[1][25] + coeff0_2*dmats2[2][25] + coeff0_3*dmats2[3][25] + coeff0_4*dmats2[4][25] + coeff0_5*dmats2[5][25] + coeff0_6*dmats2[6][25] + coeff0_7*dmats2[7][25] + coeff0_8*dmats2[8][25] + coeff0_9*dmats2[9][25] + coeff0_10*dmats2[10][25] + coeff0_11*dmats2[11][25] + coeff0_12*dmats2[12][25] + coeff0_13*dmats2[13][25] + coeff0_14*dmats2[14][25] + coeff0_15*dmats2[15][25] + coeff0_16*dmats2[16][25] + coeff0_17*dmats2[17][25] + coeff0_18*dmats2[18][25] + coeff0_19*dmats2[19][25] + coeff0_20*dmats2[20][25] + coeff0_21*dmats2[21][25] + coeff0_22*dmats2[22][25] + coeff0_23*dmats2[23][25] + coeff0_24*dmats2[24][25] + coeff0_25*dmats2[25][25] + coeff0_26*dmats2[26][25] + coeff0_27*dmats2[27][25] + coeff0_28*dmats2[28][25] + coeff0_29*dmats2[29][25] + coeff0_30*dmats2[30][25] + coeff0_31*dmats2[31][25] + coeff0_32*dmats2[32][25] + coeff0_33*dmats2[33][25] + coeff0_34*dmats2[34][25] + coeff0_35*dmats2[35][25] + coeff0_36*dmats2[36][25] + coeff0_37*dmats2[37][25] + coeff0_38*dmats2[38][25] + coeff0_39*dmats2[39][25] + coeff0_40*dmats2[40][25] + coeff0_41*dmats2[41][25] + coeff0_42*dmats2[42][25] + coeff0_43*dmats2[43][25] + coeff0_44*dmats2[44][25] + coeff0_45*dmats2[45][25] + coeff0_46*dmats2[46][25] + coeff0_47*dmats2[47][25] + coeff0_48*dmats2[48][25] + coeff0_49*dmats2[49][25] + coeff0_50*dmats2[50][25] + coeff0_51*dmats2[51][25] + coeff0_52*dmats2[52][25] + coeff0_53*dmats2[53][25] + coeff0_54*dmats2[54][25] + coeff0_55*dmats2[55][25];
          new_coeff0_26 = coeff0_0*dmats2[0][26] + coeff0_1*dmats2[1][26] + coeff0_2*dmats2[2][26] + coeff0_3*dmats2[3][26] + coeff0_4*dmats2[4][26] + coeff0_5*dmats2[5][26] + coeff0_6*dmats2[6][26] + coeff0_7*dmats2[7][26] + coeff0_8*dmats2[8][26] + coeff0_9*dmats2[9][26] + coeff0_10*dmats2[10][26] + coeff0_11*dmats2[11][26] + coeff0_12*dmats2[12][26] + coeff0_13*dmats2[13][26] + coeff0_14*dmats2[14][26] + coeff0_15*dmats2[15][26] + coeff0_16*dmats2[16][26] + coeff0_17*dmats2[17][26] + coeff0_18*dmats2[18][26] + coeff0_19*dmats2[19][26] + coeff0_20*dmats2[20][26] + coeff0_21*dmats2[21][26] + coeff0_22*dmats2[22][26] + coeff0_23*dmats2[23][26] + coeff0_24*dmats2[24][26] + coeff0_25*dmats2[25][26] + coeff0_26*dmats2[26][26] + coeff0_27*dmats2[27][26] + coeff0_28*dmats2[28][26] + coeff0_29*dmats2[29][26] + coeff0_30*dmats2[30][26] + coeff0_31*dmats2[31][26] + coeff0_32*dmats2[32][26] + coeff0_33*dmats2[33][26] + coeff0_34*dmats2[34][26] + coeff0_35*dmats2[35][26] + coeff0_36*dmats2[36][26] + coeff0_37*dmats2[37][26] + coeff0_38*dmats2[38][26] + coeff0_39*dmats2[39][26] + coeff0_40*dmats2[40][26] + coeff0_41*dmats2[41][26] + coeff0_42*dmats2[42][26] + coeff0_43*dmats2[43][26] + coeff0_44*dmats2[44][26] + coeff0_45*dmats2[45][26] + coeff0_46*dmats2[46][26] + coeff0_47*dmats2[47][26] + coeff0_48*dmats2[48][26] + coeff0_49*dmats2[49][26] + coeff0_50*dmats2[50][26] + coeff0_51*dmats2[51][26] + coeff0_52*dmats2[52][26] + coeff0_53*dmats2[53][26] + coeff0_54*dmats2[54][26] + coeff0_55*dmats2[55][26];
          new_coeff0_27 = coeff0_0*dmats2[0][27] + coeff0_1*dmats2[1][27] + coeff0_2*dmats2[2][27] + coeff0_3*dmats2[3][27] + coeff0_4*dmats2[4][27] + coeff0_5*dmats2[5][27] + coeff0_6*dmats2[6][27] + coeff0_7*dmats2[7][27] + coeff0_8*dmats2[8][27] + coeff0_9*dmats2[9][27] + coeff0_10*dmats2[10][27] + coeff0_11*dmats2[11][27] + coeff0_12*dmats2[12][27] + coeff0_13*dmats2[13][27] + coeff0_14*dmats2[14][27] + coeff0_15*dmats2[15][27] + coeff0_16*dmats2[16][27] + coeff0_17*dmats2[17][27] + coeff0_18*dmats2[18][27] + coeff0_19*dmats2[19][27] + coeff0_20*dmats2[20][27] + coeff0_21*dmats2[21][27] + coeff0_22*dmats2[22][27] + coeff0_23*dmats2[23][27] + coeff0_24*dmats2[24][27] + coeff0_25*dmats2[25][27] + coeff0_26*dmats2[26][27] + coeff0_27*dmats2[27][27] + coeff0_28*dmats2[28][27] + coeff0_29*dmats2[29][27] + coeff0_30*dmats2[30][27] + coeff0_31*dmats2[31][27] + coeff0_32*dmats2[32][27] + coeff0_33*dmats2[33][27] + coeff0_34*dmats2[34][27] + coeff0_35*dmats2[35][27] + coeff0_36*dmats2[36][27] + coeff0_37*dmats2[37][27] + coeff0_38*dmats2[38][27] + coeff0_39*dmats2[39][27] + coeff0_40*dmats2[40][27] + coeff0_41*dmats2[41][27] + coeff0_42*dmats2[42][27] + coeff0_43*dmats2[43][27] + coeff0_44*dmats2[44][27] + coeff0_45*dmats2[45][27] + coeff0_46*dmats2[46][27] + coeff0_47*dmats2[47][27] + coeff0_48*dmats2[48][27] + coeff0_49*dmats2[49][27] + coeff0_50*dmats2[50][27] + coeff0_51*dmats2[51][27] + coeff0_52*dmats2[52][27] + coeff0_53*dmats2[53][27] + coeff0_54*dmats2[54][27] + coeff0_55*dmats2[55][27];
          new_coeff0_28 = coeff0_0*dmats2[0][28] + coeff0_1*dmats2[1][28] + coeff0_2*dmats2[2][28] + coeff0_3*dmats2[3][28] + coeff0_4*dmats2[4][28] + coeff0_5*dmats2[5][28] + coeff0_6*dmats2[6][28] + coeff0_7*dmats2[7][28] + coeff0_8*dmats2[8][28] + coeff0_9*dmats2[9][28] + coeff0_10*dmats2[10][28] + coeff0_11*dmats2[11][28] + coeff0_12*dmats2[12][28] + coeff0_13*dmats2[13][28] + coeff0_14*dmats2[14][28] + coeff0_15*dmats2[15][28] + coeff0_16*dmats2[16][28] + coeff0_17*dmats2[17][28] + coeff0_18*dmats2[18][28] + coeff0_19*dmats2[19][28] + coeff0_20*dmats2[20][28] + coeff0_21*dmats2[21][28] + coeff0_22*dmats2[22][28] + coeff0_23*dmats2[23][28] + coeff0_24*dmats2[24][28] + coeff0_25*dmats2[25][28] + coeff0_26*dmats2[26][28] + coeff0_27*dmats2[27][28] + coeff0_28*dmats2[28][28] + coeff0_29*dmats2[29][28] + coeff0_30*dmats2[30][28] + coeff0_31*dmats2[31][28] + coeff0_32*dmats2[32][28] + coeff0_33*dmats2[33][28] + coeff0_34*dmats2[34][28] + coeff0_35*dmats2[35][28] + coeff0_36*dmats2[36][28] + coeff0_37*dmats2[37][28] + coeff0_38*dmats2[38][28] + coeff0_39*dmats2[39][28] + coeff0_40*dmats2[40][28] + coeff0_41*dmats2[41][28] + coeff0_42*dmats2[42][28] + coeff0_43*dmats2[43][28] + coeff0_44*dmats2[44][28] + coeff0_45*dmats2[45][28] + coeff0_46*dmats2[46][28] + coeff0_47*dmats2[47][28] + coeff0_48*dmats2[48][28] + coeff0_49*dmats2[49][28] + coeff0_50*dmats2[50][28] + coeff0_51*dmats2[51][28] + coeff0_52*dmats2[52][28] + coeff0_53*dmats2[53][28] + coeff0_54*dmats2[54][28] + coeff0_55*dmats2[55][28];
          new_coeff0_29 = coeff0_0*dmats2[0][29] + coeff0_1*dmats2[1][29] + coeff0_2*dmats2[2][29] + coeff0_3*dmats2[3][29] + coeff0_4*dmats2[4][29] + coeff0_5*dmats2[5][29] + coeff0_6*dmats2[6][29] + coeff0_7*dmats2[7][29] + coeff0_8*dmats2[8][29] + coeff0_9*dmats2[9][29] + coeff0_10*dmats2[10][29] + coeff0_11*dmats2[11][29] + coeff0_12*dmats2[12][29] + coeff0_13*dmats2[13][29] + coeff0_14*dmats2[14][29] + coeff0_15*dmats2[15][29] + coeff0_16*dmats2[16][29] + coeff0_17*dmats2[17][29] + coeff0_18*dmats2[18][29] + coeff0_19*dmats2[19][29] + coeff0_20*dmats2[20][29] + coeff0_21*dmats2[21][29] + coeff0_22*dmats2[22][29] + coeff0_23*dmats2[23][29] + coeff0_24*dmats2[24][29] + coeff0_25*dmats2[25][29] + coeff0_26*dmats2[26][29] + coeff0_27*dmats2[27][29] + coeff0_28*dmats2[28][29] + coeff0_29*dmats2[29][29] + coeff0_30*dmats2[30][29] + coeff0_31*dmats2[31][29] + coeff0_32*dmats2[32][29] + coeff0_33*dmats2[33][29] + coeff0_34*dmats2[34][29] + coeff0_35*dmats2[35][29] + coeff0_36*dmats2[36][29] + coeff0_37*dmats2[37][29] + coeff0_38*dmats2[38][29] + coeff0_39*dmats2[39][29] + coeff0_40*dmats2[40][29] + coeff0_41*dmats2[41][29] + coeff0_42*dmats2[42][29] + coeff0_43*dmats2[43][29] + coeff0_44*dmats2[44][29] + coeff0_45*dmats2[45][29] + coeff0_46*dmats2[46][29] + coeff0_47*dmats2[47][29] + coeff0_48*dmats2[48][29] + coeff0_49*dmats2[49][29] + coeff0_50*dmats2[50][29] + coeff0_51*dmats2[51][29] + coeff0_52*dmats2[52][29] + coeff0_53*dmats2[53][29] + coeff0_54*dmats2[54][29] + coeff0_55*dmats2[55][29];
          new_coeff0_30 = coeff0_0*dmats2[0][30] + coeff0_1*dmats2[1][30] + coeff0_2*dmats2[2][30] + coeff0_3*dmats2[3][30] + coeff0_4*dmats2[4][30] + coeff0_5*dmats2[5][30] + coeff0_6*dmats2[6][30] + coeff0_7*dmats2[7][30] + coeff0_8*dmats2[8][30] + coeff0_9*dmats2[9][30] + coeff0_10*dmats2[10][30] + coeff0_11*dmats2[11][30] + coeff0_12*dmats2[12][30] + coeff0_13*dmats2[13][30] + coeff0_14*dmats2[14][30] + coeff0_15*dmats2[15][30] + coeff0_16*dmats2[16][30] + coeff0_17*dmats2[17][30] + coeff0_18*dmats2[18][30] + coeff0_19*dmats2[19][30] + coeff0_20*dmats2[20][30] + coeff0_21*dmats2[21][30] + coeff0_22*dmats2[22][30] + coeff0_23*dmats2[23][30] + coeff0_24*dmats2[24][30] + coeff0_25*dmats2[25][30] + coeff0_26*dmats2[26][30] + coeff0_27*dmats2[27][30] + coeff0_28*dmats2[28][30] + coeff0_29*dmats2[29][30] + coeff0_30*dmats2[30][30] + coeff0_31*dmats2[31][30] + coeff0_32*dmats2[32][30] + coeff0_33*dmats2[33][30] + coeff0_34*dmats2[34][30] + coeff0_35*dmats2[35][30] + coeff0_36*dmats2[36][30] + coeff0_37*dmats2[37][30] + coeff0_38*dmats2[38][30] + coeff0_39*dmats2[39][30] + coeff0_40*dmats2[40][30] + coeff0_41*dmats2[41][30] + coeff0_42*dmats2[42][30] + coeff0_43*dmats2[43][30] + coeff0_44*dmats2[44][30] + coeff0_45*dmats2[45][30] + coeff0_46*dmats2[46][30] + coeff0_47*dmats2[47][30] + coeff0_48*dmats2[48][30] + coeff0_49*dmats2[49][30] + coeff0_50*dmats2[50][30] + coeff0_51*dmats2[51][30] + coeff0_52*dmats2[52][30] + coeff0_53*dmats2[53][30] + coeff0_54*dmats2[54][30] + coeff0_55*dmats2[55][30];
          new_coeff0_31 = coeff0_0*dmats2[0][31] + coeff0_1*dmats2[1][31] + coeff0_2*dmats2[2][31] + coeff0_3*dmats2[3][31] + coeff0_4*dmats2[4][31] + coeff0_5*dmats2[5][31] + coeff0_6*dmats2[6][31] + coeff0_7*dmats2[7][31] + coeff0_8*dmats2[8][31] + coeff0_9*dmats2[9][31] + coeff0_10*dmats2[10][31] + coeff0_11*dmats2[11][31] + coeff0_12*dmats2[12][31] + coeff0_13*dmats2[13][31] + coeff0_14*dmats2[14][31] + coeff0_15*dmats2[15][31] + coeff0_16*dmats2[16][31] + coeff0_17*dmats2[17][31] + coeff0_18*dmats2[18][31] + coeff0_19*dmats2[19][31] + coeff0_20*dmats2[20][31] + coeff0_21*dmats2[21][31] + coeff0_22*dmats2[22][31] + coeff0_23*dmats2[23][31] + coeff0_24*dmats2[24][31] + coeff0_25*dmats2[25][31] + coeff0_26*dmats2[26][31] + coeff0_27*dmats2[27][31] + coeff0_28*dmats2[28][31] + coeff0_29*dmats2[29][31] + coeff0_30*dmats2[30][31] + coeff0_31*dmats2[31][31] + coeff0_32*dmats2[32][31] + coeff0_33*dmats2[33][31] + coeff0_34*dmats2[34][31] + coeff0_35*dmats2[35][31] + coeff0_36*dmats2[36][31] + coeff0_37*dmats2[37][31] + coeff0_38*dmats2[38][31] + coeff0_39*dmats2[39][31] + coeff0_40*dmats2[40][31] + coeff0_41*dmats2[41][31] + coeff0_42*dmats2[42][31] + coeff0_43*dmats2[43][31] + coeff0_44*dmats2[44][31] + coeff0_45*dmats2[45][31] + coeff0_46*dmats2[46][31] + coeff0_47*dmats2[47][31] + coeff0_48*dmats2[48][31] + coeff0_49*dmats2[49][31] + coeff0_50*dmats2[50][31] + coeff0_51*dmats2[51][31] + coeff0_52*dmats2[52][31] + coeff0_53*dmats2[53][31] + coeff0_54*dmats2[54][31] + coeff0_55*dmats2[55][31];
          new_coeff0_32 = coeff0_0*dmats2[0][32] + coeff0_1*dmats2[1][32] + coeff0_2*dmats2[2][32] + coeff0_3*dmats2[3][32] + coeff0_4*dmats2[4][32] + coeff0_5*dmats2[5][32] + coeff0_6*dmats2[6][32] + coeff0_7*dmats2[7][32] + coeff0_8*dmats2[8][32] + coeff0_9*dmats2[9][32] + coeff0_10*dmats2[10][32] + coeff0_11*dmats2[11][32] + coeff0_12*dmats2[12][32] + coeff0_13*dmats2[13][32] + coeff0_14*dmats2[14][32] + coeff0_15*dmats2[15][32] + coeff0_16*dmats2[16][32] + coeff0_17*dmats2[17][32] + coeff0_18*dmats2[18][32] + coeff0_19*dmats2[19][32] + coeff0_20*dmats2[20][32] + coeff0_21*dmats2[21][32] + coeff0_22*dmats2[22][32] + coeff0_23*dmats2[23][32] + coeff0_24*dmats2[24][32] + coeff0_25*dmats2[25][32] + coeff0_26*dmats2[26][32] + coeff0_27*dmats2[27][32] + coeff0_28*dmats2[28][32] + coeff0_29*dmats2[29][32] + coeff0_30*dmats2[30][32] + coeff0_31*dmats2[31][32] + coeff0_32*dmats2[32][32] + coeff0_33*dmats2[33][32] + coeff0_34*dmats2[34][32] + coeff0_35*dmats2[35][32] + coeff0_36*dmats2[36][32] + coeff0_37*dmats2[37][32] + coeff0_38*dmats2[38][32] + coeff0_39*dmats2[39][32] + coeff0_40*dmats2[40][32] + coeff0_41*dmats2[41][32] + coeff0_42*dmats2[42][32] + coeff0_43*dmats2[43][32] + coeff0_44*dmats2[44][32] + coeff0_45*dmats2[45][32] + coeff0_46*dmats2[46][32] + coeff0_47*dmats2[47][32] + coeff0_48*dmats2[48][32] + coeff0_49*dmats2[49][32] + coeff0_50*dmats2[50][32] + coeff0_51*dmats2[51][32] + coeff0_52*dmats2[52][32] + coeff0_53*dmats2[53][32] + coeff0_54*dmats2[54][32] + coeff0_55*dmats2[55][32];
          new_coeff0_33 = coeff0_0*dmats2[0][33] + coeff0_1*dmats2[1][33] + coeff0_2*dmats2[2][33] + coeff0_3*dmats2[3][33] + coeff0_4*dmats2[4][33] + coeff0_5*dmats2[5][33] + coeff0_6*dmats2[6][33] + coeff0_7*dmats2[7][33] + coeff0_8*dmats2[8][33] + coeff0_9*dmats2[9][33] + coeff0_10*dmats2[10][33] + coeff0_11*dmats2[11][33] + coeff0_12*dmats2[12][33] + coeff0_13*dmats2[13][33] + coeff0_14*dmats2[14][33] + coeff0_15*dmats2[15][33] + coeff0_16*dmats2[16][33] + coeff0_17*dmats2[17][33] + coeff0_18*dmats2[18][33] + coeff0_19*dmats2[19][33] + coeff0_20*dmats2[20][33] + coeff0_21*dmats2[21][33] + coeff0_22*dmats2[22][33] + coeff0_23*dmats2[23][33] + coeff0_24*dmats2[24][33] + coeff0_25*dmats2[25][33] + coeff0_26*dmats2[26][33] + coeff0_27*dmats2[27][33] + coeff0_28*dmats2[28][33] + coeff0_29*dmats2[29][33] + coeff0_30*dmats2[30][33] + coeff0_31*dmats2[31][33] + coeff0_32*dmats2[32][33] + coeff0_33*dmats2[33][33] + coeff0_34*dmats2[34][33] + coeff0_35*dmats2[35][33] + coeff0_36*dmats2[36][33] + coeff0_37*dmats2[37][33] + coeff0_38*dmats2[38][33] + coeff0_39*dmats2[39][33] + coeff0_40*dmats2[40][33] + coeff0_41*dmats2[41][33] + coeff0_42*dmats2[42][33] + coeff0_43*dmats2[43][33] + coeff0_44*dmats2[44][33] + coeff0_45*dmats2[45][33] + coeff0_46*dmats2[46][33] + coeff0_47*dmats2[47][33] + coeff0_48*dmats2[48][33] + coeff0_49*dmats2[49][33] + coeff0_50*dmats2[50][33] + coeff0_51*dmats2[51][33] + coeff0_52*dmats2[52][33] + coeff0_53*dmats2[53][33] + coeff0_54*dmats2[54][33] + coeff0_55*dmats2[55][33];
          new_coeff0_34 = coeff0_0*dmats2[0][34] + coeff0_1*dmats2[1][34] + coeff0_2*dmats2[2][34] + coeff0_3*dmats2[3][34] + coeff0_4*dmats2[4][34] + coeff0_5*dmats2[5][34] + coeff0_6*dmats2[6][34] + coeff0_7*dmats2[7][34] + coeff0_8*dmats2[8][34] + coeff0_9*dmats2[9][34] + coeff0_10*dmats2[10][34] + coeff0_11*dmats2[11][34] + coeff0_12*dmats2[12][34] + coeff0_13*dmats2[13][34] + coeff0_14*dmats2[14][34] + coeff0_15*dmats2[15][34] + coeff0_16*dmats2[16][34] + coeff0_17*dmats2[17][34] + coeff0_18*dmats2[18][34] + coeff0_19*dmats2[19][34] + coeff0_20*dmats2[20][34] + coeff0_21*dmats2[21][34] + coeff0_22*dmats2[22][34] + coeff0_23*dmats2[23][34] + coeff0_24*dmats2[24][34] + coeff0_25*dmats2[25][34] + coeff0_26*dmats2[26][34] + coeff0_27*dmats2[27][34] + coeff0_28*dmats2[28][34] + coeff0_29*dmats2[29][34] + coeff0_30*dmats2[30][34] + coeff0_31*dmats2[31][34] + coeff0_32*dmats2[32][34] + coeff0_33*dmats2[33][34] + coeff0_34*dmats2[34][34] + coeff0_35*dmats2[35][34] + coeff0_36*dmats2[36][34] + coeff0_37*dmats2[37][34] + coeff0_38*dmats2[38][34] + coeff0_39*dmats2[39][34] + coeff0_40*dmats2[40][34] + coeff0_41*dmats2[41][34] + coeff0_42*dmats2[42][34] + coeff0_43*dmats2[43][34] + coeff0_44*dmats2[44][34] + coeff0_45*dmats2[45][34] + coeff0_46*dmats2[46][34] + coeff0_47*dmats2[47][34] + coeff0_48*dmats2[48][34] + coeff0_49*dmats2[49][34] + coeff0_50*dmats2[50][34] + coeff0_51*dmats2[51][34] + coeff0_52*dmats2[52][34] + coeff0_53*dmats2[53][34] + coeff0_54*dmats2[54][34] + coeff0_55*dmats2[55][34];
          new_coeff0_35 = coeff0_0*dmats2[0][35] + coeff0_1*dmats2[1][35] + coeff0_2*dmats2[2][35] + coeff0_3*dmats2[3][35] + coeff0_4*dmats2[4][35] + coeff0_5*dmats2[5][35] + coeff0_6*dmats2[6][35] + coeff0_7*dmats2[7][35] + coeff0_8*dmats2[8][35] + coeff0_9*dmats2[9][35] + coeff0_10*dmats2[10][35] + coeff0_11*dmats2[11][35] + coeff0_12*dmats2[12][35] + coeff0_13*dmats2[13][35] + coeff0_14*dmats2[14][35] + coeff0_15*dmats2[15][35] + coeff0_16*dmats2[16][35] + coeff0_17*dmats2[17][35] + coeff0_18*dmats2[18][35] + coeff0_19*dmats2[19][35] + coeff0_20*dmats2[20][35] + coeff0_21*dmats2[21][35] + coeff0_22*dmats2[22][35] + coeff0_23*dmats2[23][35] + coeff0_24*dmats2[24][35] + coeff0_25*dmats2[25][35] + coeff0_26*dmats2[26][35] + coeff0_27*dmats2[27][35] + coeff0_28*dmats2[28][35] + coeff0_29*dmats2[29][35] + coeff0_30*dmats2[30][35] + coeff0_31*dmats2[31][35] + coeff0_32*dmats2[32][35] + coeff0_33*dmats2[33][35] + coeff0_34*dmats2[34][35] + coeff0_35*dmats2[35][35] + coeff0_36*dmats2[36][35] + coeff0_37*dmats2[37][35] + coeff0_38*dmats2[38][35] + coeff0_39*dmats2[39][35] + coeff0_40*dmats2[40][35] + coeff0_41*dmats2[41][35] + coeff0_42*dmats2[42][35] + coeff0_43*dmats2[43][35] + coeff0_44*dmats2[44][35] + coeff0_45*dmats2[45][35] + coeff0_46*dmats2[46][35] + coeff0_47*dmats2[47][35] + coeff0_48*dmats2[48][35] + coeff0_49*dmats2[49][35] + coeff0_50*dmats2[50][35] + coeff0_51*dmats2[51][35] + coeff0_52*dmats2[52][35] + coeff0_53*dmats2[53][35] + coeff0_54*dmats2[54][35] + coeff0_55*dmats2[55][35];
          new_coeff0_36 = coeff0_0*dmats2[0][36] + coeff0_1*dmats2[1][36] + coeff0_2*dmats2[2][36] + coeff0_3*dmats2[3][36] + coeff0_4*dmats2[4][36] + coeff0_5*dmats2[5][36] + coeff0_6*dmats2[6][36] + coeff0_7*dmats2[7][36] + coeff0_8*dmats2[8][36] + coeff0_9*dmats2[9][36] + coeff0_10*dmats2[10][36] + coeff0_11*dmats2[11][36] + coeff0_12*dmats2[12][36] + coeff0_13*dmats2[13][36] + coeff0_14*dmats2[14][36] + coeff0_15*dmats2[15][36] + coeff0_16*dmats2[16][36] + coeff0_17*dmats2[17][36] + coeff0_18*dmats2[18][36] + coeff0_19*dmats2[19][36] + coeff0_20*dmats2[20][36] + coeff0_21*dmats2[21][36] + coeff0_22*dmats2[22][36] + coeff0_23*dmats2[23][36] + coeff0_24*dmats2[24][36] + coeff0_25*dmats2[25][36] + coeff0_26*dmats2[26][36] + coeff0_27*dmats2[27][36] + coeff0_28*dmats2[28][36] + coeff0_29*dmats2[29][36] + coeff0_30*dmats2[30][36] + coeff0_31*dmats2[31][36] + coeff0_32*dmats2[32][36] + coeff0_33*dmats2[33][36] + coeff0_34*dmats2[34][36] + coeff0_35*dmats2[35][36] + coeff0_36*dmats2[36][36] + coeff0_37*dmats2[37][36] + coeff0_38*dmats2[38][36] + coeff0_39*dmats2[39][36] + coeff0_40*dmats2[40][36] + coeff0_41*dmats2[41][36] + coeff0_42*dmats2[42][36] + coeff0_43*dmats2[43][36] + coeff0_44*dmats2[44][36] + coeff0_45*dmats2[45][36] + coeff0_46*dmats2[46][36] + coeff0_47*dmats2[47][36] + coeff0_48*dmats2[48][36] + coeff0_49*dmats2[49][36] + coeff0_50*dmats2[50][36] + coeff0_51*dmats2[51][36] + coeff0_52*dmats2[52][36] + coeff0_53*dmats2[53][36] + coeff0_54*dmats2[54][36] + coeff0_55*dmats2[55][36];
          new_coeff0_37 = coeff0_0*dmats2[0][37] + coeff0_1*dmats2[1][37] + coeff0_2*dmats2[2][37] + coeff0_3*dmats2[3][37] + coeff0_4*dmats2[4][37] + coeff0_5*dmats2[5][37] + coeff0_6*dmats2[6][37] + coeff0_7*dmats2[7][37] + coeff0_8*dmats2[8][37] + coeff0_9*dmats2[9][37] + coeff0_10*dmats2[10][37] + coeff0_11*dmats2[11][37] + coeff0_12*dmats2[12][37] + coeff0_13*dmats2[13][37] + coeff0_14*dmats2[14][37] + coeff0_15*dmats2[15][37] + coeff0_16*dmats2[16][37] + coeff0_17*dmats2[17][37] + coeff0_18*dmats2[18][37] + coeff0_19*dmats2[19][37] + coeff0_20*dmats2[20][37] + coeff0_21*dmats2[21][37] + coeff0_22*dmats2[22][37] + coeff0_23*dmats2[23][37] + coeff0_24*dmats2[24][37] + coeff0_25*dmats2[25][37] + coeff0_26*dmats2[26][37] + coeff0_27*dmats2[27][37] + coeff0_28*dmats2[28][37] + coeff0_29*dmats2[29][37] + coeff0_30*dmats2[30][37] + coeff0_31*dmats2[31][37] + coeff0_32*dmats2[32][37] + coeff0_33*dmats2[33][37] + coeff0_34*dmats2[34][37] + coeff0_35*dmats2[35][37] + coeff0_36*dmats2[36][37] + coeff0_37*dmats2[37][37] + coeff0_38*dmats2[38][37] + coeff0_39*dmats2[39][37] + coeff0_40*dmats2[40][37] + coeff0_41*dmats2[41][37] + coeff0_42*dmats2[42][37] + coeff0_43*dmats2[43][37] + coeff0_44*dmats2[44][37] + coeff0_45*dmats2[45][37] + coeff0_46*dmats2[46][37] + coeff0_47*dmats2[47][37] + coeff0_48*dmats2[48][37] + coeff0_49*dmats2[49][37] + coeff0_50*dmats2[50][37] + coeff0_51*dmats2[51][37] + coeff0_52*dmats2[52][37] + coeff0_53*dmats2[53][37] + coeff0_54*dmats2[54][37] + coeff0_55*dmats2[55][37];
          new_coeff0_38 = coeff0_0*dmats2[0][38] + coeff0_1*dmats2[1][38] + coeff0_2*dmats2[2][38] + coeff0_3*dmats2[3][38] + coeff0_4*dmats2[4][38] + coeff0_5*dmats2[5][38] + coeff0_6*dmats2[6][38] + coeff0_7*dmats2[7][38] + coeff0_8*dmats2[8][38] + coeff0_9*dmats2[9][38] + coeff0_10*dmats2[10][38] + coeff0_11*dmats2[11][38] + coeff0_12*dmats2[12][38] + coeff0_13*dmats2[13][38] + coeff0_14*dmats2[14][38] + coeff0_15*dmats2[15][38] + coeff0_16*dmats2[16][38] + coeff0_17*dmats2[17][38] + coeff0_18*dmats2[18][38] + coeff0_19*dmats2[19][38] + coeff0_20*dmats2[20][38] + coeff0_21*dmats2[21][38] + coeff0_22*dmats2[22][38] + coeff0_23*dmats2[23][38] + coeff0_24*dmats2[24][38] + coeff0_25*dmats2[25][38] + coeff0_26*dmats2[26][38] + coeff0_27*dmats2[27][38] + coeff0_28*dmats2[28][38] + coeff0_29*dmats2[29][38] + coeff0_30*dmats2[30][38] + coeff0_31*dmats2[31][38] + coeff0_32*dmats2[32][38] + coeff0_33*dmats2[33][38] + coeff0_34*dmats2[34][38] + coeff0_35*dmats2[35][38] + coeff0_36*dmats2[36][38] + coeff0_37*dmats2[37][38] + coeff0_38*dmats2[38][38] + coeff0_39*dmats2[39][38] + coeff0_40*dmats2[40][38] + coeff0_41*dmats2[41][38] + coeff0_42*dmats2[42][38] + coeff0_43*dmats2[43][38] + coeff0_44*dmats2[44][38] + coeff0_45*dmats2[45][38] + coeff0_46*dmats2[46][38] + coeff0_47*dmats2[47][38] + coeff0_48*dmats2[48][38] + coeff0_49*dmats2[49][38] + coeff0_50*dmats2[50][38] + coeff0_51*dmats2[51][38] + coeff0_52*dmats2[52][38] + coeff0_53*dmats2[53][38] + coeff0_54*dmats2[54][38] + coeff0_55*dmats2[55][38];
          new_coeff0_39 = coeff0_0*dmats2[0][39] + coeff0_1*dmats2[1][39] + coeff0_2*dmats2[2][39] + coeff0_3*dmats2[3][39] + coeff0_4*dmats2[4][39] + coeff0_5*dmats2[5][39] + coeff0_6*dmats2[6][39] + coeff0_7*dmats2[7][39] + coeff0_8*dmats2[8][39] + coeff0_9*dmats2[9][39] + coeff0_10*dmats2[10][39] + coeff0_11*dmats2[11][39] + coeff0_12*dmats2[12][39] + coeff0_13*dmats2[13][39] + coeff0_14*dmats2[14][39] + coeff0_15*dmats2[15][39] + coeff0_16*dmats2[16][39] + coeff0_17*dmats2[17][39] + coeff0_18*dmats2[18][39] + coeff0_19*dmats2[19][39] + coeff0_20*dmats2[20][39] + coeff0_21*dmats2[21][39] + coeff0_22*dmats2[22][39] + coeff0_23*dmats2[23][39] + coeff0_24*dmats2[24][39] + coeff0_25*dmats2[25][39] + coeff0_26*dmats2[26][39] + coeff0_27*dmats2[27][39] + coeff0_28*dmats2[28][39] + coeff0_29*dmats2[29][39] + coeff0_30*dmats2[30][39] + coeff0_31*dmats2[31][39] + coeff0_32*dmats2[32][39] + coeff0_33*dmats2[33][39] + coeff0_34*dmats2[34][39] + coeff0_35*dmats2[35][39] + coeff0_36*dmats2[36][39] + coeff0_37*dmats2[37][39] + coeff0_38*dmats2[38][39] + coeff0_39*dmats2[39][39] + coeff0_40*dmats2[40][39] + coeff0_41*dmats2[41][39] + coeff0_42*dmats2[42][39] + coeff0_43*dmats2[43][39] + coeff0_44*dmats2[44][39] + coeff0_45*dmats2[45][39] + coeff0_46*dmats2[46][39] + coeff0_47*dmats2[47][39] + coeff0_48*dmats2[48][39] + coeff0_49*dmats2[49][39] + coeff0_50*dmats2[50][39] + coeff0_51*dmats2[51][39] + coeff0_52*dmats2[52][39] + coeff0_53*dmats2[53][39] + coeff0_54*dmats2[54][39] + coeff0_55*dmats2[55][39];
          new_coeff0_40 = coeff0_0*dmats2[0][40] + coeff0_1*dmats2[1][40] + coeff0_2*dmats2[2][40] + coeff0_3*dmats2[3][40] + coeff0_4*dmats2[4][40] + coeff0_5*dmats2[5][40] + coeff0_6*dmats2[6][40] + coeff0_7*dmats2[7][40] + coeff0_8*dmats2[8][40] + coeff0_9*dmats2[9][40] + coeff0_10*dmats2[10][40] + coeff0_11*dmats2[11][40] + coeff0_12*dmats2[12][40] + coeff0_13*dmats2[13][40] + coeff0_14*dmats2[14][40] + coeff0_15*dmats2[15][40] + coeff0_16*dmats2[16][40] + coeff0_17*dmats2[17][40] + coeff0_18*dmats2[18][40] + coeff0_19*dmats2[19][40] + coeff0_20*dmats2[20][40] + coeff0_21*dmats2[21][40] + coeff0_22*dmats2[22][40] + coeff0_23*dmats2[23][40] + coeff0_24*dmats2[24][40] + coeff0_25*dmats2[25][40] + coeff0_26*dmats2[26][40] + coeff0_27*dmats2[27][40] + coeff0_28*dmats2[28][40] + coeff0_29*dmats2[29][40] + coeff0_30*dmats2[30][40] + coeff0_31*dmats2[31][40] + coeff0_32*dmats2[32][40] + coeff0_33*dmats2[33][40] + coeff0_34*dmats2[34][40] + coeff0_35*dmats2[35][40] + coeff0_36*dmats2[36][40] + coeff0_37*dmats2[37][40] + coeff0_38*dmats2[38][40] + coeff0_39*dmats2[39][40] + coeff0_40*dmats2[40][40] + coeff0_41*dmats2[41][40] + coeff0_42*dmats2[42][40] + coeff0_43*dmats2[43][40] + coeff0_44*dmats2[44][40] + coeff0_45*dmats2[45][40] + coeff0_46*dmats2[46][40] + coeff0_47*dmats2[47][40] + coeff0_48*dmats2[48][40] + coeff0_49*dmats2[49][40] + coeff0_50*dmats2[50][40] + coeff0_51*dmats2[51][40] + coeff0_52*dmats2[52][40] + coeff0_53*dmats2[53][40] + coeff0_54*dmats2[54][40] + coeff0_55*dmats2[55][40];
          new_coeff0_41 = coeff0_0*dmats2[0][41] + coeff0_1*dmats2[1][41] + coeff0_2*dmats2[2][41] + coeff0_3*dmats2[3][41] + coeff0_4*dmats2[4][41] + coeff0_5*dmats2[5][41] + coeff0_6*dmats2[6][41] + coeff0_7*dmats2[7][41] + coeff0_8*dmats2[8][41] + coeff0_9*dmats2[9][41] + coeff0_10*dmats2[10][41] + coeff0_11*dmats2[11][41] + coeff0_12*dmats2[12][41] + coeff0_13*dmats2[13][41] + coeff0_14*dmats2[14][41] + coeff0_15*dmats2[15][41] + coeff0_16*dmats2[16][41] + coeff0_17*dmats2[17][41] + coeff0_18*dmats2[18][41] + coeff0_19*dmats2[19][41] + coeff0_20*dmats2[20][41] + coeff0_21*dmats2[21][41] + coeff0_22*dmats2[22][41] + coeff0_23*dmats2[23][41] + coeff0_24*dmats2[24][41] + coeff0_25*dmats2[25][41] + coeff0_26*dmats2[26][41] + coeff0_27*dmats2[27][41] + coeff0_28*dmats2[28][41] + coeff0_29*dmats2[29][41] + coeff0_30*dmats2[30][41] + coeff0_31*dmats2[31][41] + coeff0_32*dmats2[32][41] + coeff0_33*dmats2[33][41] + coeff0_34*dmats2[34][41] + coeff0_35*dmats2[35][41] + coeff0_36*dmats2[36][41] + coeff0_37*dmats2[37][41] + coeff0_38*dmats2[38][41] + coeff0_39*dmats2[39][41] + coeff0_40*dmats2[40][41] + coeff0_41*dmats2[41][41] + coeff0_42*dmats2[42][41] + coeff0_43*dmats2[43][41] + coeff0_44*dmats2[44][41] + coeff0_45*dmats2[45][41] + coeff0_46*dmats2[46][41] + coeff0_47*dmats2[47][41] + coeff0_48*dmats2[48][41] + coeff0_49*dmats2[49][41] + coeff0_50*dmats2[50][41] + coeff0_51*dmats2[51][41] + coeff0_52*dmats2[52][41] + coeff0_53*dmats2[53][41] + coeff0_54*dmats2[54][41] + coeff0_55*dmats2[55][41];
          new_coeff0_42 = coeff0_0*dmats2[0][42] + coeff0_1*dmats2[1][42] + coeff0_2*dmats2[2][42] + coeff0_3*dmats2[3][42] + coeff0_4*dmats2[4][42] + coeff0_5*dmats2[5][42] + coeff0_6*dmats2[6][42] + coeff0_7*dmats2[7][42] + coeff0_8*dmats2[8][42] + coeff0_9*dmats2[9][42] + coeff0_10*dmats2[10][42] + coeff0_11*dmats2[11][42] + coeff0_12*dmats2[12][42] + coeff0_13*dmats2[13][42] + coeff0_14*dmats2[14][42] + coeff0_15*dmats2[15][42] + coeff0_16*dmats2[16][42] + coeff0_17*dmats2[17][42] + coeff0_18*dmats2[18][42] + coeff0_19*dmats2[19][42] + coeff0_20*dmats2[20][42] + coeff0_21*dmats2[21][42] + coeff0_22*dmats2[22][42] + coeff0_23*dmats2[23][42] + coeff0_24*dmats2[24][42] + coeff0_25*dmats2[25][42] + coeff0_26*dmats2[26][42] + coeff0_27*dmats2[27][42] + coeff0_28*dmats2[28][42] + coeff0_29*dmats2[29][42] + coeff0_30*dmats2[30][42] + coeff0_31*dmats2[31][42] + coeff0_32*dmats2[32][42] + coeff0_33*dmats2[33][42] + coeff0_34*dmats2[34][42] + coeff0_35*dmats2[35][42] + coeff0_36*dmats2[36][42] + coeff0_37*dmats2[37][42] + coeff0_38*dmats2[38][42] + coeff0_39*dmats2[39][42] + coeff0_40*dmats2[40][42] + coeff0_41*dmats2[41][42] + coeff0_42*dmats2[42][42] + coeff0_43*dmats2[43][42] + coeff0_44*dmats2[44][42] + coeff0_45*dmats2[45][42] + coeff0_46*dmats2[46][42] + coeff0_47*dmats2[47][42] + coeff0_48*dmats2[48][42] + coeff0_49*dmats2[49][42] + coeff0_50*dmats2[50][42] + coeff0_51*dmats2[51][42] + coeff0_52*dmats2[52][42] + coeff0_53*dmats2[53][42] + coeff0_54*dmats2[54][42] + coeff0_55*dmats2[55][42];
          new_coeff0_43 = coeff0_0*dmats2[0][43] + coeff0_1*dmats2[1][43] + coeff0_2*dmats2[2][43] + coeff0_3*dmats2[3][43] + coeff0_4*dmats2[4][43] + coeff0_5*dmats2[5][43] + coeff0_6*dmats2[6][43] + coeff0_7*dmats2[7][43] + coeff0_8*dmats2[8][43] + coeff0_9*dmats2[9][43] + coeff0_10*dmats2[10][43] + coeff0_11*dmats2[11][43] + coeff0_12*dmats2[12][43] + coeff0_13*dmats2[13][43] + coeff0_14*dmats2[14][43] + coeff0_15*dmats2[15][43] + coeff0_16*dmats2[16][43] + coeff0_17*dmats2[17][43] + coeff0_18*dmats2[18][43] + coeff0_19*dmats2[19][43] + coeff0_20*dmats2[20][43] + coeff0_21*dmats2[21][43] + coeff0_22*dmats2[22][43] + coeff0_23*dmats2[23][43] + coeff0_24*dmats2[24][43] + coeff0_25*dmats2[25][43] + coeff0_26*dmats2[26][43] + coeff0_27*dmats2[27][43] + coeff0_28*dmats2[28][43] + coeff0_29*dmats2[29][43] + coeff0_30*dmats2[30][43] + coeff0_31*dmats2[31][43] + coeff0_32*dmats2[32][43] + coeff0_33*dmats2[33][43] + coeff0_34*dmats2[34][43] + coeff0_35*dmats2[35][43] + coeff0_36*dmats2[36][43] + coeff0_37*dmats2[37][43] + coeff0_38*dmats2[38][43] + coeff0_39*dmats2[39][43] + coeff0_40*dmats2[40][43] + coeff0_41*dmats2[41][43] + coeff0_42*dmats2[42][43] + coeff0_43*dmats2[43][43] + coeff0_44*dmats2[44][43] + coeff0_45*dmats2[45][43] + coeff0_46*dmats2[46][43] + coeff0_47*dmats2[47][43] + coeff0_48*dmats2[48][43] + coeff0_49*dmats2[49][43] + coeff0_50*dmats2[50][43] + coeff0_51*dmats2[51][43] + coeff0_52*dmats2[52][43] + coeff0_53*dmats2[53][43] + coeff0_54*dmats2[54][43] + coeff0_55*dmats2[55][43];
          new_coeff0_44 = coeff0_0*dmats2[0][44] + coeff0_1*dmats2[1][44] + coeff0_2*dmats2[2][44] + coeff0_3*dmats2[3][44] + coeff0_4*dmats2[4][44] + coeff0_5*dmats2[5][44] + coeff0_6*dmats2[6][44] + coeff0_7*dmats2[7][44] + coeff0_8*dmats2[8][44] + coeff0_9*dmats2[9][44] + coeff0_10*dmats2[10][44] + coeff0_11*dmats2[11][44] + coeff0_12*dmats2[12][44] + coeff0_13*dmats2[13][44] + coeff0_14*dmats2[14][44] + coeff0_15*dmats2[15][44] + coeff0_16*dmats2[16][44] + coeff0_17*dmats2[17][44] + coeff0_18*dmats2[18][44] + coeff0_19*dmats2[19][44] + coeff0_20*dmats2[20][44] + coeff0_21*dmats2[21][44] + coeff0_22*dmats2[22][44] + coeff0_23*dmats2[23][44] + coeff0_24*dmats2[24][44] + coeff0_25*dmats2[25][44] + coeff0_26*dmats2[26][44] + coeff0_27*dmats2[27][44] + coeff0_28*dmats2[28][44] + coeff0_29*dmats2[29][44] + coeff0_30*dmats2[30][44] + coeff0_31*dmats2[31][44] + coeff0_32*dmats2[32][44] + coeff0_33*dmats2[33][44] + coeff0_34*dmats2[34][44] + coeff0_35*dmats2[35][44] + coeff0_36*dmats2[36][44] + coeff0_37*dmats2[37][44] + coeff0_38*dmats2[38][44] + coeff0_39*dmats2[39][44] + coeff0_40*dmats2[40][44] + coeff0_41*dmats2[41][44] + coeff0_42*dmats2[42][44] + coeff0_43*dmats2[43][44] + coeff0_44*dmats2[44][44] + coeff0_45*dmats2[45][44] + coeff0_46*dmats2[46][44] + coeff0_47*dmats2[47][44] + coeff0_48*dmats2[48][44] + coeff0_49*dmats2[49][44] + coeff0_50*dmats2[50][44] + coeff0_51*dmats2[51][44] + coeff0_52*dmats2[52][44] + coeff0_53*dmats2[53][44] + coeff0_54*dmats2[54][44] + coeff0_55*dmats2[55][44];
          new_coeff0_45 = coeff0_0*dmats2[0][45] + coeff0_1*dmats2[1][45] + coeff0_2*dmats2[2][45] + coeff0_3*dmats2[3][45] + coeff0_4*dmats2[4][45] + coeff0_5*dmats2[5][45] + coeff0_6*dmats2[6][45] + coeff0_7*dmats2[7][45] + coeff0_8*dmats2[8][45] + coeff0_9*dmats2[9][45] + coeff0_10*dmats2[10][45] + coeff0_11*dmats2[11][45] + coeff0_12*dmats2[12][45] + coeff0_13*dmats2[13][45] + coeff0_14*dmats2[14][45] + coeff0_15*dmats2[15][45] + coeff0_16*dmats2[16][45] + coeff0_17*dmats2[17][45] + coeff0_18*dmats2[18][45] + coeff0_19*dmats2[19][45] + coeff0_20*dmats2[20][45] + coeff0_21*dmats2[21][45] + coeff0_22*dmats2[22][45] + coeff0_23*dmats2[23][45] + coeff0_24*dmats2[24][45] + coeff0_25*dmats2[25][45] + coeff0_26*dmats2[26][45] + coeff0_27*dmats2[27][45] + coeff0_28*dmats2[28][45] + coeff0_29*dmats2[29][45] + coeff0_30*dmats2[30][45] + coeff0_31*dmats2[31][45] + coeff0_32*dmats2[32][45] + coeff0_33*dmats2[33][45] + coeff0_34*dmats2[34][45] + coeff0_35*dmats2[35][45] + coeff0_36*dmats2[36][45] + coeff0_37*dmats2[37][45] + coeff0_38*dmats2[38][45] + coeff0_39*dmats2[39][45] + coeff0_40*dmats2[40][45] + coeff0_41*dmats2[41][45] + coeff0_42*dmats2[42][45] + coeff0_43*dmats2[43][45] + coeff0_44*dmats2[44][45] + coeff0_45*dmats2[45][45] + coeff0_46*dmats2[46][45] + coeff0_47*dmats2[47][45] + coeff0_48*dmats2[48][45] + coeff0_49*dmats2[49][45] + coeff0_50*dmats2[50][45] + coeff0_51*dmats2[51][45] + coeff0_52*dmats2[52][45] + coeff0_53*dmats2[53][45] + coeff0_54*dmats2[54][45] + coeff0_55*dmats2[55][45];
          new_coeff0_46 = coeff0_0*dmats2[0][46] + coeff0_1*dmats2[1][46] + coeff0_2*dmats2[2][46] + coeff0_3*dmats2[3][46] + coeff0_4*dmats2[4][46] + coeff0_5*dmats2[5][46] + coeff0_6*dmats2[6][46] + coeff0_7*dmats2[7][46] + coeff0_8*dmats2[8][46] + coeff0_9*dmats2[9][46] + coeff0_10*dmats2[10][46] + coeff0_11*dmats2[11][46] + coeff0_12*dmats2[12][46] + coeff0_13*dmats2[13][46] + coeff0_14*dmats2[14][46] + coeff0_15*dmats2[15][46] + coeff0_16*dmats2[16][46] + coeff0_17*dmats2[17][46] + coeff0_18*dmats2[18][46] + coeff0_19*dmats2[19][46] + coeff0_20*dmats2[20][46] + coeff0_21*dmats2[21][46] + coeff0_22*dmats2[22][46] + coeff0_23*dmats2[23][46] + coeff0_24*dmats2[24][46] + coeff0_25*dmats2[25][46] + coeff0_26*dmats2[26][46] + coeff0_27*dmats2[27][46] + coeff0_28*dmats2[28][46] + coeff0_29*dmats2[29][46] + coeff0_30*dmats2[30][46] + coeff0_31*dmats2[31][46] + coeff0_32*dmats2[32][46] + coeff0_33*dmats2[33][46] + coeff0_34*dmats2[34][46] + coeff0_35*dmats2[35][46] + coeff0_36*dmats2[36][46] + coeff0_37*dmats2[37][46] + coeff0_38*dmats2[38][46] + coeff0_39*dmats2[39][46] + coeff0_40*dmats2[40][46] + coeff0_41*dmats2[41][46] + coeff0_42*dmats2[42][46] + coeff0_43*dmats2[43][46] + coeff0_44*dmats2[44][46] + coeff0_45*dmats2[45][46] + coeff0_46*dmats2[46][46] + coeff0_47*dmats2[47][46] + coeff0_48*dmats2[48][46] + coeff0_49*dmats2[49][46] + coeff0_50*dmats2[50][46] + coeff0_51*dmats2[51][46] + coeff0_52*dmats2[52][46] + coeff0_53*dmats2[53][46] + coeff0_54*dmats2[54][46] + coeff0_55*dmats2[55][46];
          new_coeff0_47 = coeff0_0*dmats2[0][47] + coeff0_1*dmats2[1][47] + coeff0_2*dmats2[2][47] + coeff0_3*dmats2[3][47] + coeff0_4*dmats2[4][47] + coeff0_5*dmats2[5][47] + coeff0_6*dmats2[6][47] + coeff0_7*dmats2[7][47] + coeff0_8*dmats2[8][47] + coeff0_9*dmats2[9][47] + coeff0_10*dmats2[10][47] + coeff0_11*dmats2[11][47] + coeff0_12*dmats2[12][47] + coeff0_13*dmats2[13][47] + coeff0_14*dmats2[14][47] + coeff0_15*dmats2[15][47] + coeff0_16*dmats2[16][47] + coeff0_17*dmats2[17][47] + coeff0_18*dmats2[18][47] + coeff0_19*dmats2[19][47] + coeff0_20*dmats2[20][47] + coeff0_21*dmats2[21][47] + coeff0_22*dmats2[22][47] + coeff0_23*dmats2[23][47] + coeff0_24*dmats2[24][47] + coeff0_25*dmats2[25][47] + coeff0_26*dmats2[26][47] + coeff0_27*dmats2[27][47] + coeff0_28*dmats2[28][47] + coeff0_29*dmats2[29][47] + coeff0_30*dmats2[30][47] + coeff0_31*dmats2[31][47] + coeff0_32*dmats2[32][47] + coeff0_33*dmats2[33][47] + coeff0_34*dmats2[34][47] + coeff0_35*dmats2[35][47] + coeff0_36*dmats2[36][47] + coeff0_37*dmats2[37][47] + coeff0_38*dmats2[38][47] + coeff0_39*dmats2[39][47] + coeff0_40*dmats2[40][47] + coeff0_41*dmats2[41][47] + coeff0_42*dmats2[42][47] + coeff0_43*dmats2[43][47] + coeff0_44*dmats2[44][47] + coeff0_45*dmats2[45][47] + coeff0_46*dmats2[46][47] + coeff0_47*dmats2[47][47] + coeff0_48*dmats2[48][47] + coeff0_49*dmats2[49][47] + coeff0_50*dmats2[50][47] + coeff0_51*dmats2[51][47] + coeff0_52*dmats2[52][47] + coeff0_53*dmats2[53][47] + coeff0_54*dmats2[54][47] + coeff0_55*dmats2[55][47];
          new_coeff0_48 = coeff0_0*dmats2[0][48] + coeff0_1*dmats2[1][48] + coeff0_2*dmats2[2][48] + coeff0_3*dmats2[3][48] + coeff0_4*dmats2[4][48] + coeff0_5*dmats2[5][48] + coeff0_6*dmats2[6][48] + coeff0_7*dmats2[7][48] + coeff0_8*dmats2[8][48] + coeff0_9*dmats2[9][48] + coeff0_10*dmats2[10][48] + coeff0_11*dmats2[11][48] + coeff0_12*dmats2[12][48] + coeff0_13*dmats2[13][48] + coeff0_14*dmats2[14][48] + coeff0_15*dmats2[15][48] + coeff0_16*dmats2[16][48] + coeff0_17*dmats2[17][48] + coeff0_18*dmats2[18][48] + coeff0_19*dmats2[19][48] + coeff0_20*dmats2[20][48] + coeff0_21*dmats2[21][48] + coeff0_22*dmats2[22][48] + coeff0_23*dmats2[23][48] + coeff0_24*dmats2[24][48] + coeff0_25*dmats2[25][48] + coeff0_26*dmats2[26][48] + coeff0_27*dmats2[27][48] + coeff0_28*dmats2[28][48] + coeff0_29*dmats2[29][48] + coeff0_30*dmats2[30][48] + coeff0_31*dmats2[31][48] + coeff0_32*dmats2[32][48] + coeff0_33*dmats2[33][48] + coeff0_34*dmats2[34][48] + coeff0_35*dmats2[35][48] + coeff0_36*dmats2[36][48] + coeff0_37*dmats2[37][48] + coeff0_38*dmats2[38][48] + coeff0_39*dmats2[39][48] + coeff0_40*dmats2[40][48] + coeff0_41*dmats2[41][48] + coeff0_42*dmats2[42][48] + coeff0_43*dmats2[43][48] + coeff0_44*dmats2[44][48] + coeff0_45*dmats2[45][48] + coeff0_46*dmats2[46][48] + coeff0_47*dmats2[47][48] + coeff0_48*dmats2[48][48] + coeff0_49*dmats2[49][48] + coeff0_50*dmats2[50][48] + coeff0_51*dmats2[51][48] + coeff0_52*dmats2[52][48] + coeff0_53*dmats2[53][48] + coeff0_54*dmats2[54][48] + coeff0_55*dmats2[55][48];
          new_coeff0_49 = coeff0_0*dmats2[0][49] + coeff0_1*dmats2[1][49] + coeff0_2*dmats2[2][49] + coeff0_3*dmats2[3][49] + coeff0_4*dmats2[4][49] + coeff0_5*dmats2[5][49] + coeff0_6*dmats2[6][49] + coeff0_7*dmats2[7][49] + coeff0_8*dmats2[8][49] + coeff0_9*dmats2[9][49] + coeff0_10*dmats2[10][49] + coeff0_11*dmats2[11][49] + coeff0_12*dmats2[12][49] + coeff0_13*dmats2[13][49] + coeff0_14*dmats2[14][49] + coeff0_15*dmats2[15][49] + coeff0_16*dmats2[16][49] + coeff0_17*dmats2[17][49] + coeff0_18*dmats2[18][49] + coeff0_19*dmats2[19][49] + coeff0_20*dmats2[20][49] + coeff0_21*dmats2[21][49] + coeff0_22*dmats2[22][49] + coeff0_23*dmats2[23][49] + coeff0_24*dmats2[24][49] + coeff0_25*dmats2[25][49] + coeff0_26*dmats2[26][49] + coeff0_27*dmats2[27][49] + coeff0_28*dmats2[28][49] + coeff0_29*dmats2[29][49] + coeff0_30*dmats2[30][49] + coeff0_31*dmats2[31][49] + coeff0_32*dmats2[32][49] + coeff0_33*dmats2[33][49] + coeff0_34*dmats2[34][49] + coeff0_35*dmats2[35][49] + coeff0_36*dmats2[36][49] + coeff0_37*dmats2[37][49] + coeff0_38*dmats2[38][49] + coeff0_39*dmats2[39][49] + coeff0_40*dmats2[40][49] + coeff0_41*dmats2[41][49] + coeff0_42*dmats2[42][49] + coeff0_43*dmats2[43][49] + coeff0_44*dmats2[44][49] + coeff0_45*dmats2[45][49] + coeff0_46*dmats2[46][49] + coeff0_47*dmats2[47][49] + coeff0_48*dmats2[48][49] + coeff0_49*dmats2[49][49] + coeff0_50*dmats2[50][49] + coeff0_51*dmats2[51][49] + coeff0_52*dmats2[52][49] + coeff0_53*dmats2[53][49] + coeff0_54*dmats2[54][49] + coeff0_55*dmats2[55][49];
          new_coeff0_50 = coeff0_0*dmats2[0][50] + coeff0_1*dmats2[1][50] + coeff0_2*dmats2[2][50] + coeff0_3*dmats2[3][50] + coeff0_4*dmats2[4][50] + coeff0_5*dmats2[5][50] + coeff0_6*dmats2[6][50] + coeff0_7*dmats2[7][50] + coeff0_8*dmats2[8][50] + coeff0_9*dmats2[9][50] + coeff0_10*dmats2[10][50] + coeff0_11*dmats2[11][50] + coeff0_12*dmats2[12][50] + coeff0_13*dmats2[13][50] + coeff0_14*dmats2[14][50] + coeff0_15*dmats2[15][50] + coeff0_16*dmats2[16][50] + coeff0_17*dmats2[17][50] + coeff0_18*dmats2[18][50] + coeff0_19*dmats2[19][50] + coeff0_20*dmats2[20][50] + coeff0_21*dmats2[21][50] + coeff0_22*dmats2[22][50] + coeff0_23*dmats2[23][50] + coeff0_24*dmats2[24][50] + coeff0_25*dmats2[25][50] + coeff0_26*dmats2[26][50] + coeff0_27*dmats2[27][50] + coeff0_28*dmats2[28][50] + coeff0_29*dmats2[29][50] + coeff0_30*dmats2[30][50] + coeff0_31*dmats2[31][50] + coeff0_32*dmats2[32][50] + coeff0_33*dmats2[33][50] + coeff0_34*dmats2[34][50] + coeff0_35*dmats2[35][50] + coeff0_36*dmats2[36][50] + coeff0_37*dmats2[37][50] + coeff0_38*dmats2[38][50] + coeff0_39*dmats2[39][50] + coeff0_40*dmats2[40][50] + coeff0_41*dmats2[41][50] + coeff0_42*dmats2[42][50] + coeff0_43*dmats2[43][50] + coeff0_44*dmats2[44][50] + coeff0_45*dmats2[45][50] + coeff0_46*dmats2[46][50] + coeff0_47*dmats2[47][50] + coeff0_48*dmats2[48][50] + coeff0_49*dmats2[49][50] + coeff0_50*dmats2[50][50] + coeff0_51*dmats2[51][50] + coeff0_52*dmats2[52][50] + coeff0_53*dmats2[53][50] + coeff0_54*dmats2[54][50] + coeff0_55*dmats2[55][50];
          new_coeff0_51 = coeff0_0*dmats2[0][51] + coeff0_1*dmats2[1][51] + coeff0_2*dmats2[2][51] + coeff0_3*dmats2[3][51] + coeff0_4*dmats2[4][51] + coeff0_5*dmats2[5][51] + coeff0_6*dmats2[6][51] + coeff0_7*dmats2[7][51] + coeff0_8*dmats2[8][51] + coeff0_9*dmats2[9][51] + coeff0_10*dmats2[10][51] + coeff0_11*dmats2[11][51] + coeff0_12*dmats2[12][51] + coeff0_13*dmats2[13][51] + coeff0_14*dmats2[14][51] + coeff0_15*dmats2[15][51] + coeff0_16*dmats2[16][51] + coeff0_17*dmats2[17][51] + coeff0_18*dmats2[18][51] + coeff0_19*dmats2[19][51] + coeff0_20*dmats2[20][51] + coeff0_21*dmats2[21][51] + coeff0_22*dmats2[22][51] + coeff0_23*dmats2[23][51] + coeff0_24*dmats2[24][51] + coeff0_25*dmats2[25][51] + coeff0_26*dmats2[26][51] + coeff0_27*dmats2[27][51] + coeff0_28*dmats2[28][51] + coeff0_29*dmats2[29][51] + coeff0_30*dmats2[30][51] + coeff0_31*dmats2[31][51] + coeff0_32*dmats2[32][51] + coeff0_33*dmats2[33][51] + coeff0_34*dmats2[34][51] + coeff0_35*dmats2[35][51] + coeff0_36*dmats2[36][51] + coeff0_37*dmats2[37][51] + coeff0_38*dmats2[38][51] + coeff0_39*dmats2[39][51] + coeff0_40*dmats2[40][51] + coeff0_41*dmats2[41][51] + coeff0_42*dmats2[42][51] + coeff0_43*dmats2[43][51] + coeff0_44*dmats2[44][51] + coeff0_45*dmats2[45][51] + coeff0_46*dmats2[46][51] + coeff0_47*dmats2[47][51] + coeff0_48*dmats2[48][51] + coeff0_49*dmats2[49][51] + coeff0_50*dmats2[50][51] + coeff0_51*dmats2[51][51] + coeff0_52*dmats2[52][51] + coeff0_53*dmats2[53][51] + coeff0_54*dmats2[54][51] + coeff0_55*dmats2[55][51];
          new_coeff0_52 = coeff0_0*dmats2[0][52] + coeff0_1*dmats2[1][52] + coeff0_2*dmats2[2][52] + coeff0_3*dmats2[3][52] + coeff0_4*dmats2[4][52] + coeff0_5*dmats2[5][52] + coeff0_6*dmats2[6][52] + coeff0_7*dmats2[7][52] + coeff0_8*dmats2[8][52] + coeff0_9*dmats2[9][52] + coeff0_10*dmats2[10][52] + coeff0_11*dmats2[11][52] + coeff0_12*dmats2[12][52] + coeff0_13*dmats2[13][52] + coeff0_14*dmats2[14][52] + coeff0_15*dmats2[15][52] + coeff0_16*dmats2[16][52] + coeff0_17*dmats2[17][52] + coeff0_18*dmats2[18][52] + coeff0_19*dmats2[19][52] + coeff0_20*dmats2[20][52] + coeff0_21*dmats2[21][52] + coeff0_22*dmats2[22][52] + coeff0_23*dmats2[23][52] + coeff0_24*dmats2[24][52] + coeff0_25*dmats2[25][52] + coeff0_26*dmats2[26][52] + coeff0_27*dmats2[27][52] + coeff0_28*dmats2[28][52] + coeff0_29*dmats2[29][52] + coeff0_30*dmats2[30][52] + coeff0_31*dmats2[31][52] + coeff0_32*dmats2[32][52] + coeff0_33*dmats2[33][52] + coeff0_34*dmats2[34][52] + coeff0_35*dmats2[35][52] + coeff0_36*dmats2[36][52] + coeff0_37*dmats2[37][52] + coeff0_38*dmats2[38][52] + coeff0_39*dmats2[39][52] + coeff0_40*dmats2[40][52] + coeff0_41*dmats2[41][52] + coeff0_42*dmats2[42][52] + coeff0_43*dmats2[43][52] + coeff0_44*dmats2[44][52] + coeff0_45*dmats2[45][52] + coeff0_46*dmats2[46][52] + coeff0_47*dmats2[47][52] + coeff0_48*dmats2[48][52] + coeff0_49*dmats2[49][52] + coeff0_50*dmats2[50][52] + coeff0_51*dmats2[51][52] + coeff0_52*dmats2[52][52] + coeff0_53*dmats2[53][52] + coeff0_54*dmats2[54][52] + coeff0_55*dmats2[55][52];
          new_coeff0_53 = coeff0_0*dmats2[0][53] + coeff0_1*dmats2[1][53] + coeff0_2*dmats2[2][53] + coeff0_3*dmats2[3][53] + coeff0_4*dmats2[4][53] + coeff0_5*dmats2[5][53] + coeff0_6*dmats2[6][53] + coeff0_7*dmats2[7][53] + coeff0_8*dmats2[8][53] + coeff0_9*dmats2[9][53] + coeff0_10*dmats2[10][53] + coeff0_11*dmats2[11][53] + coeff0_12*dmats2[12][53] + coeff0_13*dmats2[13][53] + coeff0_14*dmats2[14][53] + coeff0_15*dmats2[15][53] + coeff0_16*dmats2[16][53] + coeff0_17*dmats2[17][53] + coeff0_18*dmats2[18][53] + coeff0_19*dmats2[19][53] + coeff0_20*dmats2[20][53] + coeff0_21*dmats2[21][53] + coeff0_22*dmats2[22][53] + coeff0_23*dmats2[23][53] + coeff0_24*dmats2[24][53] + coeff0_25*dmats2[25][53] + coeff0_26*dmats2[26][53] + coeff0_27*dmats2[27][53] + coeff0_28*dmats2[28][53] + coeff0_29*dmats2[29][53] + coeff0_30*dmats2[30][53] + coeff0_31*dmats2[31][53] + coeff0_32*dmats2[32][53] + coeff0_33*dmats2[33][53] + coeff0_34*dmats2[34][53] + coeff0_35*dmats2[35][53] + coeff0_36*dmats2[36][53] + coeff0_37*dmats2[37][53] + coeff0_38*dmats2[38][53] + coeff0_39*dmats2[39][53] + coeff0_40*dmats2[40][53] + coeff0_41*dmats2[41][53] + coeff0_42*dmats2[42][53] + coeff0_43*dmats2[43][53] + coeff0_44*dmats2[44][53] + coeff0_45*dmats2[45][53] + coeff0_46*dmats2[46][53] + coeff0_47*dmats2[47][53] + coeff0_48*dmats2[48][53] + coeff0_49*dmats2[49][53] + coeff0_50*dmats2[50][53] + coeff0_51*dmats2[51][53] + coeff0_52*dmats2[52][53] + coeff0_53*dmats2[53][53] + coeff0_54*dmats2[54][53] + coeff0_55*dmats2[55][53];
          new_coeff0_54 = coeff0_0*dmats2[0][54] + coeff0_1*dmats2[1][54] + coeff0_2*dmats2[2][54] + coeff0_3*dmats2[3][54] + coeff0_4*dmats2[4][54] + coeff0_5*dmats2[5][54] + coeff0_6*dmats2[6][54] + coeff0_7*dmats2[7][54] + coeff0_8*dmats2[8][54] + coeff0_9*dmats2[9][54] + coeff0_10*dmats2[10][54] + coeff0_11*dmats2[11][54] + coeff0_12*dmats2[12][54] + coeff0_13*dmats2[13][54] + coeff0_14*dmats2[14][54] + coeff0_15*dmats2[15][54] + coeff0_16*dmats2[16][54] + coeff0_17*dmats2[17][54] + coeff0_18*dmats2[18][54] + coeff0_19*dmats2[19][54] + coeff0_20*dmats2[20][54] + coeff0_21*dmats2[21][54] + coeff0_22*dmats2[22][54] + coeff0_23*dmats2[23][54] + coeff0_24*dmats2[24][54] + coeff0_25*dmats2[25][54] + coeff0_26*dmats2[26][54] + coeff0_27*dmats2[27][54] + coeff0_28*dmats2[28][54] + coeff0_29*dmats2[29][54] + coeff0_30*dmats2[30][54] + coeff0_31*dmats2[31][54] + coeff0_32*dmats2[32][54] + coeff0_33*dmats2[33][54] + coeff0_34*dmats2[34][54] + coeff0_35*dmats2[35][54] + coeff0_36*dmats2[36][54] + coeff0_37*dmats2[37][54] + coeff0_38*dmats2[38][54] + coeff0_39*dmats2[39][54] + coeff0_40*dmats2[40][54] + coeff0_41*dmats2[41][54] + coeff0_42*dmats2[42][54] + coeff0_43*dmats2[43][54] + coeff0_44*dmats2[44][54] + coeff0_45*dmats2[45][54] + coeff0_46*dmats2[46][54] + coeff0_47*dmats2[47][54] + coeff0_48*dmats2[48][54] + coeff0_49*dmats2[49][54] + coeff0_50*dmats2[50][54] + coeff0_51*dmats2[51][54] + coeff0_52*dmats2[52][54] + coeff0_53*dmats2[53][54] + coeff0_54*dmats2[54][54] + coeff0_55*dmats2[55][54];
          new_coeff0_55 = coeff0_0*dmats2[0][55] + coeff0_1*dmats2[1][55] + coeff0_2*dmats2[2][55] + coeff0_3*dmats2[3][55] + coeff0_4*dmats2[4][55] + coeff0_5*dmats2[5][55] + coeff0_6*dmats2[6][55] + coeff0_7*dmats2[7][55] + coeff0_8*dmats2[8][55] + coeff0_9*dmats2[9][55] + coeff0_10*dmats2[10][55] + coeff0_11*dmats2[11][55] + coeff0_12*dmats2[12][55] + coeff0_13*dmats2[13][55] + coeff0_14*dmats2[14][55] + coeff0_15*dmats2[15][55] + coeff0_16*dmats2[16][55] + coeff0_17*dmats2[17][55] + coeff0_18*dmats2[18][55] + coeff0_19*dmats2[19][55] + coeff0_20*dmats2[20][55] + coeff0_21*dmats2[21][55] + coeff0_22*dmats2[22][55] + coeff0_23*dmats2[23][55] + coeff0_24*dmats2[24][55] + coeff0_25*dmats2[25][55] + coeff0_26*dmats2[26][55] + coeff0_27*dmats2[27][55] + coeff0_28*dmats2[28][55] + coeff0_29*dmats2[29][55] + coeff0_30*dmats2[30][55] + coeff0_31*dmats2[31][55] + coeff0_32*dmats2[32][55] + coeff0_33*dmats2[33][55] + coeff0_34*dmats2[34][55] + coeff0_35*dmats2[35][55] + coeff0_36*dmats2[36][55] + coeff0_37*dmats2[37][55] + coeff0_38*dmats2[38][55] + coeff0_39*dmats2[39][55] + coeff0_40*dmats2[40][55] + coeff0_41*dmats2[41][55] + coeff0_42*dmats2[42][55] + coeff0_43*dmats2[43][55] + coeff0_44*dmats2[44][55] + coeff0_45*dmats2[45][55] + coeff0_46*dmats2[46][55] + coeff0_47*dmats2[47][55] + coeff0_48*dmats2[48][55] + coeff0_49*dmats2[49][55] + coeff0_50*dmats2[50][55] + coeff0_51*dmats2[51][55] + coeff0_52*dmats2[52][55] + coeff0_53*dmats2[53][55] + coeff0_54*dmats2[54][55] + coeff0_55*dmats2[55][55];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9 + new_coeff0_10*basisvalue10 + new_coeff0_11*basisvalue11 + new_coeff0_12*basisvalue12 + new_coeff0_13*basisvalue13 + new_coeff0_14*basisvalue14 + new_coeff0_15*basisvalue15 + new_coeff0_16*basisvalue16 + new_coeff0_17*basisvalue17 + new_coeff0_18*basisvalue18 + new_coeff0_19*basisvalue19 + new_coeff0_20*basisvalue20 + new_coeff0_21*basisvalue21 + new_coeff0_22*basisvalue22 + new_coeff0_23*basisvalue23 + new_coeff0_24*basisvalue24 + new_coeff0_25*basisvalue25 + new_coeff0_26*basisvalue26 + new_coeff0_27*basisvalue27 + new_coeff0_28*basisvalue28 + new_coeff0_29*basisvalue29 + new_coeff0_30*basisvalue30 + new_coeff0_31*basisvalue31 + new_coeff0_32*basisvalue32 + new_coeff0_33*basisvalue33 + new_coeff0_34*basisvalue34 + new_coeff0_35*basisvalue35 + new_coeff0_36*basisvalue36 + new_coeff0_37*basisvalue37 + new_coeff0_38*basisvalue38 + new_coeff0_39*basisvalue39 + new_coeff0_40*basisvalue40 + new_coeff0_41*basisvalue41 + new_coeff0_42*basisvalue42 + new_coeff0_43*basisvalue43 + new_coeff0_44*basisvalue44 + new_coeff0_45*basisvalue45 + new_coeff0_46*basisvalue46 + new_coeff0_47*basisvalue47 + new_coeff0_48*basisvalue48 + new_coeff0_49*basisvalue49 + new_coeff0_50*basisvalue50 + new_coeff0_51*basisvalue51 + new_coeff0_52*basisvalue52 + new_coeff0_53*basisvalue53 + new_coeff0_54*basisvalue54 + new_coeff0_55*basisvalue55;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    static const double X[56][1][3] = {{{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}, {{0, 0.8, 0.2}}, {{0, 0.6, 0.4}}, {{0, 0.4, 0.6}}, {{0, 0.2, 0.8}}, {{0.8, 0, 0.2}}, {{0.6, 0, 0.4}}, {{0.4, 0, 0.6}}, {{0.2, 0, 0.8}}, {{0.8, 0.2, 0}}, {{0.6, 0.4, 0}}, {{0.4, 0.6, 0}}, {{0.2, 0.8, 0}}, {{0, 0, 0.2}}, {{0, 0, 0.4}}, {{0, 0, 0.6}}, {{0, 0, 0.8}}, {{0, 0.2, 0}}, {{0, 0.4, 0}}, {{0, 0.6, 0}}, {{0, 0.8, 0}}, {{0.2, 0, 0}}, {{0.4, 0, 0}}, {{0.6, 0, 0}}, {{0.8, 0, 0}}, {{0.6, 0.2, 0.2}}, {{0.4, 0.4, 0.2}}, {{0.2, 0.6, 0.2}}, {{0.4, 0.2, 0.4}}, {{0.2, 0.4, 0.4}}, {{0.2, 0.2, 0.6}}, {{0, 0.2, 0.2}}, {{0, 0.4, 0.2}}, {{0, 0.6, 0.2}}, {{0, 0.2, 0.4}}, {{0, 0.4, 0.4}}, {{0, 0.2, 0.6}}, {{0.2, 0, 0.2}}, {{0.4, 0, 0.2}}, {{0.6, 0, 0.2}}, {{0.2, 0, 0.4}}, {{0.4, 0, 0.4}}, {{0.2, 0, 0.6}}, {{0.2, 0.2, 0}}, {{0.4, 0.2, 0}}, {{0.6, 0.2, 0}}, {{0.2, 0.4, 0}}, {{0.4, 0.4, 0}}, {{0.2, 0.6, 0}}, {{0.2, 0.2, 0.2}}, {{0.4, 0.2, 0.2}}, {{0.2, 0.4, 0.2}}, {{0.2, 0.2, 0.4}}};
    static const double W[56][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    static const double D[56][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new poisson3d_5_0_finite_element_0();
  }

};

/// This class defines the interface for a finite element.

class poisson3d_5_0_finite_element_1: public ufc::finite_element
{
public:

  /// Constructor
  poisson3d_5_0_finite_element_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_0_finite_element_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "FiniteElement('Lagrange', 'tetrahedron', 5)";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 56;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
    
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
    
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    static const double coefficients0[56][56] = \
    {{0.00945067404923492, 0.00609788120679252, 0.00352061335622803, 0.00248944957812476, 0.0070594623332504, 0.00546823600997146, 0.00315708753234943, 0.00446479600584755, 0.00257775117585285, 0.00182274533665715, -0.00298267418725716, -0.0025208197798293, -0.00195261860521383, -0.00112734487734488, -0.00218309396769447, -0.00169101731601731, -0.000976309302606919, -0.00138070985681772, -0.000797153207506487, -0.00056367243867244, 0.00278687112460004, 0.00245778931055954, 0.00207721109304566, 0.00160900079398463, 0.000928957041533343, 0.00219831358911339, 0.0018579140830667, 0.00143913406048032, 0.00083088443721827, 0.00160900079398463, 0.0012463266558274, 0.000719567030240151, 0.00101762145320217, 0.000587524019939407, 0.000415442218609132, -0.00705593524717798, -0.00638233357089081, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363026, -0.00582624677716565, -0.00513826668309062, -0.00434262794914383, -0.00336378514516178, -0.00194208225905522, -0.00459580543596519, -0.00388416451811043, -0.00300866089851429, -0.00173705117965752, -0.00336378514516178, -0.00260557676948629, -0.00150433044925714, -0.00212744452363027, -0.00122828066840393, -0.000868525589828762},
    {0.00945067404923495, -0.00609788120679253, 0.00352061335622802, 0.00248944957812476, 0.00705946233325039, -0.00546823600997146, 0.00315708753234944, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521383, -0.00112734487734489, -0.00218309396769448, 0.00169101731601732, -0.000976309302606925, 0.00138070985681772, -0.000797153207506488, -0.000563672438672441, 0.00278687112460004, -0.00245778931055954, 0.00207721109304566, -0.00160900079398463, 0.000928957041533346, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218268, 0.00160900079398463, -0.0012463266558274, 0.000719567030240148, -0.00101762145320216, 0.000587524019939402, 0.000415442218609128, 0.00705593524717798, -0.0063823335708908, 0.00562868913761196, -0.00475711057319694, 0.00368484200521179, -0.00212744452363027, -0.00582624677716565, 0.00513826668309062, -0.00434262794914382, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811043, 0.00300866089851429, -0.00173705117965752, -0.00336378514516178, 0.00260557676948629, -0.00150433044925714, 0.00212744452363026, -0.00122828066840393, -0.00086852558982876},
    {0.00945067404923495, 0, -0.00704122671245605, 0.00248944957812476, 0, 0, 0.00947126259704828, 0, -0.00515550235170572, 0.00182274533665715, 0, 0, 0, 0.00450937950937951, 0, 0, -0.00292892790782075, 0, 0.00159430641501296, -0.00056367243867244, 0, 0, 0, 0, 0.00464478520766673, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987881, 0.00041544221860913, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527608, 0, 0, 0, 0.00694820471863012, 0, 0, -0.00451299134777144, 0, 0.00245656133680786, -0.000868525589828763},
    {0.00945067404923493, 0, 0, -0.0074683487343743, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00563672438672439, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.018239037386404},
    {-0.010023442173431, 0, 0.0374718748311399, -0.0289614183593723, 0, 0, 0.0284137877911448, 0, 0.00515550235170572, -0.00619733414463434, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835494, -0.000187890812890802, 0, 0, 0, 0, 0.0232239260383337, 0, 0, 0, -0.00166176887443652, 0, 0, -0.00647610327216137, 0, 0.00646276421933342, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496074, 0, 0, 0, -0.0277928188745205, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.010023442173431, 0, -0.0287574853355259, 0.05360943398437, 0, 0, -0.0189425251940966, 0, 0.0154665070551171, 0.0153110608279201, 0, 0, 0, -0.00450937950937953, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144092, 0, -0.0123380044187274, 0.00872428659079174, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559316, 0, 0.041270230458372, -0.018239037386404},
    {0.010023442173431, 0, 0.0409576306293854, -0.0449826285156208, 0, 0, 0.0189425251940965, 0, -0.00515550235170569, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883226, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957583, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.055026973944496, 0.0303983956440067},
    {-0.0100234421734311, 0, -0.0148144621425437, 0.0449826285156207, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0187359374155699, -0.0289614183593723, 0.0211783869997512, -0.0164047080299144, 0.00947126259704827, 0.00446479600584757, -0.00257775117585285, -0.00619733414463432, 0.0208787193108001, -0.0176457384588052, 0.0136683302364968, -0.00789141414141413, -0.0084898098743674, 0.00657617845117844, -0.00379675839902689, 0.0023011830946962, -0.00132858867917746, -0.0001878908128908, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992314, 0.00464478520766674, -0.0010991567945567, 0.000928957041533349, -0.000719567030240152, 0.000415442218609127, -0.00482700238195388, 0.00373897996748219, -0.00215870109072045, 0.00559691799261185, -0.00323138210966672, -0.00290809553026392, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935591, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863011, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778515, 0.00607967912880135},
    {0.010023442173431, -0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, 0.0109364720199429, -0.00631417506469883, 0.0133943880175427, -0.00773325352755858, 0.0153110608279201, -0.00298267418725716, 0.00252081977982931, -0.00195261860521382, 0.00112734487734488, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476002, 0.0129522065443227, -0.00747795993496437, -0.00321800158796926, 0.0024926533116548, -0.00143913406048031, -0.0106850252586226, 0.00616900220936373, 0.00872428659079178, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845888, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.020635115229186, -0.0182390373864041},
    {0.0100234421734311, 0.0354703486038674, -0.0204788153146927, -0.0449826285156208, 0.0141189246665008, -0.0109364720199429, 0.00631417506469885, -0.00446479600584758, 0.00257775117585285, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974696, -0.00263047138047138, 0.00151870335961075, 0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620757, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, -0.0128297005588457, 0.00740723107127184, 0.0449826285156208, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0.00920473237878476, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489395, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.010023442173431, 0.0324515955311979, -0.0335503995581136, -0.00801060507812427, 0.0211783869997512, 0.00109364720199429, -0.00568275755822898, -0.0169662248222207, 0.0056710525868763, 0.00838462854862291, 0.0208787193108001, -0.0109235523792603, 0.00499002532443534, -0.00187890812890812, -0.0162519217595033, 0.00958243145743146, -0.00466458889023305, 0.0108922666482287, -0.00558007245254534, -0.00469727032227033, 0.0139343556230002, -0.00245778931055954, -0.00415442218609131, 0.00643600317593851, -0.00464478520766673, -0.0120907247401236, 0.00464478520766674, -0.000719567030240153, -0.000415442218609137, 0.00965400476390778, -0.00498530662330958, 0.00215870109072046, -0.006614539445814, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543313, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438609, 0.00310733161448835, -0.00481385743762287, 0.00347410235931505, -0.00201827108709706, -0.000521115353897262, 0.000902598269554289, 0.00170195561890421, -0.000491312267361565, -0.000868525589828764},
    {0.0100234421734309, -0.024904712849524, 0.0553363732971484, -0.0043134027343746, -0.0141189246665008, 0.0153110608279201, 0.00883984509057843, 0.00803663281052563, -0.0159820572902877, 0.000729098134662861, -0.00298267418725715, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846298, -0.0116492303992304, 0.00976309302606916, 0.00352848074520084, 0.00132858867917745, -0.00263047138047137, 0, 0.0196623144844763, -0.00727023882565982, -0.00724050357293082, 0.00928957041533346, -0.00219831358911338, -0.0148633126645336, 0.0129522065443228, -0.0066470754977461, 0.00402250198496157, 0.00685479660705067, -0.00575653624192123, -0.0045792965394097, -0.000293762009969684, 0.00290809553026392, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.00411061334647249, -0.00303983956440068, 0.00908221989193681, -0.00776832903622088, 0.00045958054359652, 0.00427258096992148, -0.00270779480866287, 0.000868525589828773, -0.00100913554354854, -0.00234501909253766, 0.00180519653910858, 0.00127646671417816, 0.000245656133680781, -0.000868525589828764},
    {0.0100234421734309, 0.0354703486038675, -0.0492363006502187, -0.00431340273437461, 0.0141189246665008, -0.00656188321196574, -0.016416855168217, -0.00982255121286466, 0.0149509568199466, 0.000729098134662872, 0, 0.0033610930397724, 0.0282044909641998, -0.0353234728234728, -0.000485131992820993, -0.0124007936507936, 0.00889526253486299, 0.00291483191994851, 0.00239145962251946, -0.00263047138047138, 0, 0, 0.0218107164769794, -0.00402250198496158, -0.00928957041533347, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992316, 0.00934744991870547, -0.0028782681209606, -0.00254405363300539, -0.00381890612960613, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320202, 0.00840946286290443, -0.00388416451811042, 0, 0.00233049871086626, 0.0030086608985143, -0.00347410235931507, -0.000336378514516178, -0.00286613444643493, 0.00120346435940572, 0.00085097780945211, 0.00098262453472315, -0.000868525589828765},
    {-0.010023442173431, -0.0128297005588457, 0.0448791059024117, -0.00801060507812424, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467804, -0.0175287079957994, 0.0083846285486229, 0, 0, 0.00867830491206149, 0.0225468975468975, 0, -0.00300625300625301, -0.019200749617936, 0.000613648825252316, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240152, -0.0108014976838374, 0, 0.00249265331165478, 0.0107935054536023, -0.000508810726601074, -0.00734405024924252, 0.00290809553026393, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931505, 0, -0.00208446141558903, -0.000902598269554287, 0.000425488904726055, 0.0017195929357655, -0.000868525589828767},
    {-0.0100234421734311, -0.0324515955311978, -0.0187359374155699, -0.0289614183593723, 0.0211783869997511, 0.0164047080299144, 0.00947126259704826, -0.00446479600584755, -0.00257775117585285, -0.00619733414463435, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141413, -0.00848980987436742, -0.00657617845117845, -0.0037967583990269, -0.00230118309469619, -0.00132858867917746, -0.00018789081289081, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992313, 0.00464478520766674, 0.00109915679455668, 0.000928957041533345, 0.000719567030240154, 0.000415442218609131, -0.00482700238195388, -0.00373897996748219, -0.00215870109072045, -0.00559691799261187, -0.00323138210966673, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.00955378148811641, 0.00740032731935591, 0.00427258096992148, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.0069482047186301, 0.0181644397838736, 0.014070114555226, 0.00812338442598859, 0.0136156449512337, 0.00786099627778516, 0.00607967912880135},
    {0.0100234421734308, 0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, -0.0109364720199429, -0.00631417506469886, -0.0133943880175427, -0.00773325352755858, 0.0153110608279201, 0.00298267418725716, 0.00252081977982933, 0.00195261860521383, 0.00112734487734487, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476002, -0.0129522065443228, -0.00747795993496438, -0.00321800158796927, -0.00249265331165479, -0.00143913406048031, 0.0106850252586227, 0.00616900220936374, 0.00872428659079177, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.0206351152291861, -0.0182390373864041},
    {0.010023442173431, -0.0354703486038674, -0.0204788153146927, -0.0449826285156208, 0.0141189246665008, 0.0109364720199429, 0.00631417506469886, 0.00446479600584758, 0.00257775117585287, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974696, 0.00263047138047138, 0.00151870335961075, -0.0308358534689291, -0.0178030883009781, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620757, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588457, 0.00740723107127183, 0.0449826285156208, 0, 0, 0, -0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878479, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489395, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.010023442173431, -0.0324515955311978, -0.0335503995581136, -0.00801060507812424, 0.0211783869997512, -0.00109364720199429, -0.00568275755822899, 0.0169662248222207, 0.00567105258687632, 0.0083846285486229, -0.0208787193108001, -0.0109235523792603, -0.00499002532443534, -0.00187890812890812, -0.0162519217595033, -0.00958243145743146, -0.00466458889023305, -0.0108922666482287, -0.00558007245254535, -0.00469727032227032, 0.0139343556230002, 0.00245778931055953, -0.00415442218609133, -0.00643600317593852, -0.00464478520766672, 0.0120907247401236, 0.00464478520766673, 0.000719567030240156, -0.000415442218609149, 0.00965400476390777, 0.00498530662330958, 0.00215870109072047, 0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543313, 0.00719357335632687, 0.0112908326677739, 0.0114368694935501, 0.00737991258440984, -0.00183832217438608, 0.00310733161448834, 0.00481385743762287, 0.00347410235931505, -0.00201827108709706, 0.000521115353897262, 0.000902598269554287, -0.00170195561890421, -0.000491312267361565, -0.000868525589828767},
    {0.010023442173431, 0.0249047128495239, 0.0553363732971484, -0.00431340273437459, -0.0141189246665008, -0.0153110608279201, 0.00883984509057843, -0.00803663281052563, -0.0159820572902878, 0.000729098134662854, 0.00298267418725716, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846298, 0.0116492303992304, 0.00976309302606918, -0.00352848074520084, 0.00132858867917746, -0.00263047138047138, 0, -0.0196623144844763, -0.0072702388256598, 0.00724050357293083, 0.00928957041533346, 0.00219831358911339, -0.0148633126645336, -0.0129522065443228, -0.00664707549774609, 0.00402250198496157, -0.00685479660705067, -0.00575653624192124, 0.0045792965394097, -0.000293762009969679, 0.00290809553026392, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.0041106133464725, -0.00303983956440067, -0.00908221989193682, -0.00776832903622089, -0.00045958054359652, 0.00427258096992148, 0.00270779480866287, 0.000868525589828776, -0.00100913554354853, 0.00234501909253766, 0.00180519653910858, -0.00127646671417816, 0.000245656133680779, -0.000868525589828761},
    {0.0100234421734309, -0.0354703486038674, -0.0492363006502187, -0.00431340273437459, 0.0141189246665008, 0.00656188321196577, -0.016416855168217, 0.00982255121286466, 0.0149509568199466, 0.000729098134662868, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820994, 0.0124007936507936, 0.00889526253486299, -0.00291483191994852, 0.00239145962251945, -0.00263047138047138, 0, 0, 0.0218107164769794, 0.00402250198496157, -0.00928957041533346, 0, -0.00557374224920009, 0.0143913406048031, 0.0141250354327105, 0.000804500396992309, -0.00934744991870548, -0.0028782681209606, 0.00254405363300539, -0.00381890612960614, 0.00290809553026393, 0, 0, 0, 0.0266398192099028, 0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320203, -0.00840946286290443, -0.00388416451811041, 0, 0.00233049871086626, -0.0030086608985143, -0.00347410235931507, -0.000336378514516177, 0.00286613444643493, 0.00120346435940572, -0.000850977809452108, 0.000982624534723152, -0.000868525589828766},
    {-0.010023442173431, 0.0128297005588457, 0.0448791059024117, -0.00801060507812424, 0, -0.0174983552319087, 0.0132597676358676, 0.00357183680467805, -0.0175287079957994, 0.0083846285486229, 0, 0, -0.00867830491206148, 0.0225468975468975, 0, 0.00300625300625301, -0.019200749617936, -0.000613648825252319, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766673, 0, 0, 0.00719567030240153, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601077, -0.00734405024924252, 0.00290809553026393, 0, 0, 0, 0, -0.0257938940364826, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702859, -0.00347410235931505, 0, 0.00208446141558903, -0.000902598269554286, -0.000425488904726054, 0.0017195929357655, -0.000868525589828767},
    {-0.0100234421734311, -0.0452812960900435, -0.0113287063442981, -0.00801060507812424, -0.0014118924666501, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.00838462854862289, -0.00894802256177147, -0.0142846454190328, -0.014536160727703, -0.00939454064454064, -0.0123708658169354, -0.0125886844636845, -0.00813591085505764, -0.0102786178229763, -0.00664294339588733, -0.00469727032227033, -0.00836061337380012, 0.00245778931055953, 0.00830884437218264, 0.00965400476390777, 0.00650269929073344, 0.00219831358911338, 0.00743165633226677, 0.00863480436288183, 0.00581619106052785, 0.00643600317593852, 0.00747795993496437, 0.00503696921168108, 0.00610572871921293, 0.00411266813957582, 0.00290809553026394, 0.0352796762358899, 0.0191470007126724, 0.00788016479265675, 0.000951422114639391, -0.00221090520312707, -0.00212744452363027, 0.017478740331497, 0.00719357335632686, 0.000868525589828764, -0.00201827108709706, -0.00194208225905521, 0.00643412761035128, 0.000776832903622088, -0.00180519653910857, -0.00173705117965753, 0.000672757029032363, -0.00156334606169177, -0.00150433044925715, -0.00127646671417816, -0.00122828066840393, -0.000868525589828768},
    {0.010023442173431, 0.0603750614533914, -0.00610007264692975, -0.0043134027343746, -0.00564756986660025, -0.0218729440398859, 0.00126283501293978, -0.0178591840233903, 0.00103110047034114, 0.000729098134662884, 0.0417574386216002, 0.0168054651988621, -0.000867830491206148, -0.00526094276094276, 0.0145539597846299, -0.000751563251563252, -0.00455611007883227, -0.000613648825252328, -0.0037200483016969, -0.00263047138047138, 0.00557374224920005, -0.0196623144844763, -0.0103860554652283, 0.00321800158796928, 0.00650269929073341, -0.0175865087129071, -0.00928957041533347, 0.00287826812096063, 0.00581619106052784, -0.00804500396992315, 0.0024926533116548, 0.00503696921168106, 0.00203524290640433, 0.00411266813957581, 0.00290809553026391, -0.0705593524717798, -0.0127646671417816, 0.00450295131008958, 0.00380568845855754, -0.000736968401042372, -0.00212744452363026, -0.0116524935543313, 0.00411061334647251, 0.00347410235931506, -0.000672757029032362, -0.00194208225905521, 0.00367664434877218, 0.00310733161448835, -0.000601732179702865, -0.00173705117965753, 0.00269102811612942, -0.000521115353897265, -0.00150433044925715, -0.00042548890472606, -0.00122828066840393, -0.000868525589828759},
    {0.0100234421734309, -0.0603750614533914, -0.00610007264692972, -0.00431340273437457, -0.00564756986660032, 0.0218729440398859, 0.00126283501293976, 0.0178591840233903, 0.00103110047034113, 0.000729098134662861, -0.0417574386216002, 0.016805465198862, 0.000867830491206157, -0.00526094276094276, 0.0145539597846298, 0.000751563251563257, -0.00455611007883228, 0.00061364882525233, -0.00372004830169691, -0.00263047138047138, 0.00557374224920011, 0.0196623144844763, -0.0103860554652283, -0.00321800158796927, 0.00650269929073344, 0.0175865087129071, -0.00928957041533346, -0.00287826812096062, 0.00581619106052786, -0.00804500396992314, -0.0024926533116548, 0.00503696921168108, -0.00203524290640433, 0.00411266813957583, 0.00290809553026393, 0.0705593524717798, -0.0127646671417816, -0.00450295131008958, 0.00380568845855756, 0.000736968401042369, -0.00212744452363027, -0.0116524935543313, -0.00411061334647251, 0.00347410235931506, 0.000672757029032361, -0.00194208225905522, -0.00367664434877217, 0.00310733161448835, 0.000601732179702866, -0.00173705117965753, 0.00269102811612942, 0.000521115353897263, -0.00150433044925715, 0.000425488904726061, -0.00122828066840394, -0.000868525589828769},
    {-0.0100234421734309, 0.0452812960900435, -0.0113287063442981, -0.00801060507812426, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089231, 0.00838462854862291, 0.00894802256177147, -0.0142846454190327, 0.014536160727703, -0.00939454064454063, -0.0123708658169354, 0.0125886844636845, -0.00813591085505763, 0.0102786178229763, -0.00664294339588732, -0.00469727032227032, -0.00836061337380015, -0.00245778931055953, 0.00830884437218263, -0.00965400476390777, 0.00650269929073343, -0.00219831358911339, 0.00743165633226677, -0.00863480436288183, 0.00581619106052785, 0.00643600317593852, -0.00747795993496437, 0.00503696921168107, -0.00610572871921293, 0.00411266813957582, 0.00290809553026393, -0.0352796762358899, 0.0191470007126724, -0.00788016479265675, 0.000951422114639385, 0.00221090520312707, -0.00212744452363027, 0.0174787403314969, -0.00719357335632687, 0.000868525589828768, 0.00201827108709706, -0.00194208225905521, -0.00643412761035128, 0.000776832903622089, 0.00180519653910857, -0.00173705117965753, 0.000672757029032358, 0.00156334606169177, -0.00150433044925715, 0.00127646671417816, -0.00122828066840393, -0.000868525589828765},
    {0.0787556170769579, 0.0679219441350653, -0.00435719474780691, -0.00308100195312472, 0.0705946233325039, -0.0328094160598287, 0.0126283501293977, -0.0267887760350854, 0.03608851646194, -0.00729098134662862, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015873, 0.0130985638061669, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, 0.0245778931055954, -0.036351194128299, 0.0362025178646541, -0.0232239260383337, 0.0219831358911338, -0.00743165633226679, 0, 0.00166176887443652, -0.0345935170706695, 0.0130864298861876, -0.00359783515120079, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596521, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598858, -0.0102117337134253, 0.00196524906944628, 0.00607967912880136},
    {-0.0214788046573522, 0.00377344134083702, 0.00217859737390344, -0.0215670136718729, 0, 0.0984282481794862, -0.0852413633734345, -0.0312535720409329, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, 0.0236742424242424, -0.0335199527228374, -0.00138070985681772, -0.00079715320750645, 0.00394570707070704, 0, 0, 0.036351194128299, -0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.00724050357293084, -0.0392592896585629, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724418, 0, 0.00694820471863008, 0.00302740663064561, 0.0164151336477636, -0.0108311792346515, -0.00680782247561686, -0.00393049813889256, 0.00607967912880134},
    {0.0787556170769579, 0.0301875307266957, 0.0610007264692975, -0.00308100195312472, 0, 0.0218729440398858, 0.0757701007763863, 0.0178591840233903, -0.0412440188136458, -0.00729098134662862, 0, 0, 0.0520698294723688, -0.00901875901875902, 0, -0.0127765752765753, 0.0286384062098028, 0.00184094647575699, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.0143913406048031, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515274, 0.00407048581280861, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967795, -0.0427258096992148, 0, 0, -0.0300866089851429, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723144, 0.00607967912880136},
    {-0.0214788046573522, 0.00377344134083686, -0.0196073763651313, 0.00924300585937417, 0, -0.0109364720199429, 0.00947126259704823, 0.102690308134494, -0.0695992817480272, -0.060150596109686, 0, -0.00504163955965862, 0.0065087286840461, -0.00450937950937951, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196709, -0.0263060558477138, 0.000563672438672465, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165478, -0.0043174021814409, -0.0605484764655283, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573521, -0.0150937653633479, 0.0130715842434208, 0.00924300585937414, 0, -0.0109364720199429, 0.00947126259704832, -0.00892959201169514, 0.123732056440937, -0.0601505961096861, 0, 0, -0.00650872868404609, 0.00676406926406927, 0, 0.0574945887445887, 0.0126920209338899, -0.0266937238984759, 0.00637722566005184, 0.000563672438672453, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097862, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448302, -0.0616900220936373, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769578, 0.0301875307266958, 0.0174287789912279, 0.0585390371093694, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094276, -0.00455611007883226, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700864, 0.0284934006896604, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769578, -0.0679219441350653, -0.00435719474780693, -0.00308100195312472, 0.0705946233325039, 0.0328094160598287, 0.0126283501293978, 0.0267887760350854, 0.03608851646194, -0.00729098134662859, -0.0298267418725716, 0.0151249186789758, 0.0325436434202305, 0.0248015873015873, 0.0130985638061669, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646541, -0.0232239260383337, -0.0219831358911339, -0.00743165633226677, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861877, -0.00359783515120079, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.0045958054359652, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.00908221989193679, -0.00703505727761299, -0.00812338442598858, 0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573524, -0.00377344134083698, 0.00217859737390344, -0.021567013671873, 0, -0.0984282481794863, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242424, -0.0335199527228374, 0.0013807098568177, -0.00079715320750645, 0.00394570707070706, 0, 0, 0.036351194128299, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838375, -0.00724050357293083, 0.039259289658563, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951694, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.00302740663064561, -0.0164151336477636, -0.0108311792346515, 0.00680782247561685, -0.00393049813889256, 0.00607967912880134},
    {0.0787556170769579, -0.0301875307266956, 0.0610007264692975, -0.00308100195312471, 0, -0.0218729440398858, 0.0757701007763863, -0.0178591840233903, -0.0412440188136458, -0.00729098134662861, 0, 0, -0.0520698294723689, -0.00901875901875903, 0, 0.0127765752765753, 0.0286384062098028, -0.00184094647575695, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766674, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515274, -0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967796, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780843, -0.00982624534723145, 0.00607967912880137},
    {-0.0214788046573523, -0.0037734413408369, -0.0196073763651314, 0.00924300585937414, 0, 0.0109364720199429, 0.00947126259704826, -0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965861, -0.00650872868404608, -0.00450937950937948, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196707, -0.0263060558477138, 0.000563672438672444, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.0024926533116548, -0.00431740218144091, 0.0605484764655282, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, 0.0150937653633478, 0.0130715842434208, 0.00924300585937414, 0, 0.0109364720199429, 0.00947126259704831, 0.00892959201169514, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404611, 0.00676406926406927, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005185, 0.000563672438672461, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097862, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448302, -0.0616900220936373, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769578, -0.0301875307266956, 0.0174287789912279, 0.0585390371093695, 0, -0.0218729440398858, -0.0189425251940966, -0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, -0.00526094276094275, -0.00455611007883226, -0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700864, -0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769576, -0.0377344134083696, -0.0566435317214905, -0.00308100195312473, 0.0423567739995023, 0.0546823600997146, 0.0378850503881931, 0.0446479600584756, 0.00515550235170574, -0.00729098134662862, 0.0417574386216002, -0.0050416395596586, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801536, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055954, -0.00519302773261415, 0.00402250198496157, 0.00464478520766674, -0.00219831358911335, -0.0130053985814669, -0.0158304746652834, -0.0108014976838374, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.00477689074405821, 0.00370016365967795, 0.00427258096992148, -0.0367664434877216, -0.00776832903622087, 0.00601732179702858, 0.00694820471863011, -0.00908221989193682, 0.00703505727761298, 0.0081233844259886, 0.00680782247561684, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573522, 0, -0.0043571947478069, -0.0215670136718729, -0.127070321998507, 0, 0.0284137877911448, 0, 0.03608851646194, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984381, 0, 0.00159430641501295, 0.00394570707070706, 0.0836061337380013, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377579, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.0095537814881164, 0, 0.00427258096992147, 0, -0.0155366580724417, 0, 0.00694820471863011, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778513, 0.00607967912880133},
    {0.0787556170769579, 0.0377344134083696, -0.0566435317214905, -0.0030810019531247, 0.0423567739995023, -0.0546823600997146, 0.0378850503881931, -0.0446479600584757, 0.00515550235170574, -0.00729098134662864, -0.0417574386216002, -0.00504163955965862, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801536, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261416, -0.00402250198496158, 0.00464478520766674, 0.00219831358911339, -0.0130053985814669, 0.0158304746652834, -0.0108014976838374, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405821, -0.00370016365967795, 0.00427258096992148, 0.0367664434877216, -0.00776832903622088, -0.00601732179702859, 0.00694820471863012, -0.00908221989193681, -0.00703505727761299, 0.00812338442598861, -0.00680782247561685, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573522, -0.0188672067041847, 0.00653579212171051, 0.00924300585937415, 0.0141189246665008, 0, -0.00315708753234942, -0.111619900146189, -0.0541327746929101, -0.0601505961096861, -0.00894802256177146, -0.00252081977982933, 0.000650872868404606, 0.00112734487734486, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672439, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476002, -0.00431740218144093, -0.00747795993496438, 0.00321800158796924, 0.0224338798048931, 0.018708742786244, 0.046301776120698, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.017478740331497, -0.00451299134777145, -0.00781673030845887, 0.0363288795677472, -0.00938007637015066, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {-0.0214788046573521, 0.0188672067041847, 0.00653579212171047, 0.00924300585937414, 0.0141189246665007, 0, -0.00315708753234944, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177148, -0.00252081977982932, -0.000650872868404592, 0.00112734487734487, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.019928830187662, 0.000563672438672456, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476002, 0.00431740218144092, -0.00747795993496438, 0.00321800158796927, -0.0224338798048931, 0.018708742786244, -0.0463017761206981, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777143, -0.00781673030845888, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769579, 0, -0.0348575579824557, 0.0585390371093695, -0.0282378493330015, 0, 0.00631417506469885, 0, -0.0206220094068228, 0.083846285486229, 0, 0, 0, 0, -0.0067918478994939, 0, 0.00151870335961074, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233621, 0, -0.0329013451166066, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.0275134869722481, 0.0303983956440068},
    {0.0787556170769578, -0.0377344134083696, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, 0.0546823600997145, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865145, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.0092175078635989, -0.00789141414141412, -0.00347132196482459, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, -0.00491557862111904, -0.0197335053839338, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097694, -0.000623163327913701, -0.00215870109072047, 0.00712335017241508, 0.00411266813957581, 0.00540074884191871, 0, -0.0510586685671264, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173251, -0.0143871467126537, -0.00564541633388696, 0.00571843474677502, 0.00737991258440982, 0.00367664434877216, -0.00155366580724418, 0.00240692871881142, 0.00347410235931506, 0.00100913554354853, 0.00026055767694863, 0.000902598269554296, -0.000850977809452108, -0.000491312267361565, -0.000868525589828763},
    {-0.0214788046573521, 0, -0.0217859737390348, 0.00308100195312476, -0.127070321998507, 0, 0.0473563129852414, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468977, 0.0509388592462045, 0, -0.00943765659186683, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533349, 0, -0.0141250354327105, -0.0144810071458616, 0, -0.00215870109072044, 0, 0.00411266813957584, 0.00540074884191874, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259877, 0, -0.0112908326677739, 0, 0.00737991258440982, 0, -0.00310733161448833, 0, 0.00347410235931505, 0.00201827108709706, 0, 0.000902598269554279, 0, -0.000491312267361592, -0.00086852558982878},
    {0.078755617076958, 0.0377344134083695, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584757, 0.0257775117585286, 0.0291639253865145, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.0092175078635989, 0.00789141414141413, -0.00347132196482461, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111908, -0.0197335053839338, 0.0394205194526234, -0.0325134964536672, 0.0241814494802473, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097695, 0.000623163327913697, -0.00215870109072046, -0.0071233501724151, 0.00411266813957581, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908062, -0.0114170653756727, -0.010317557614593, 0.0127646671417816, 0.00466099742173252, 0.0143871467126538, -0.00564541633388697, -0.00571843474677502, 0.00737991258440983, -0.00367664434877217, -0.00155366580724418, -0.00240692871881144, 0.00347410235931506, 0.00100913554354853, -0.000260557676948628, 0.000902598269554287, 0.000850977809452107, -0.000491312267361569, -0.000868525589828763},
    {-0.0214788046573521, -0.0188672067041848, 0.0108929868695174, 0.0030810019531247, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792643, 0.0127592173566001, -0.0089480225617715, -0.0277290175781224, -0.00585785581564147, 0.0191648629148629, 0.00145539597846298, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755831, -0.0129644660894661, 0, 0.0589869434534289, 0.00727023882565979, 0.0506835250105158, 0.0650269929073344, -0.00659494076734015, 0.0148633126645335, -0.00431740218144092, -0.00166176887443655, -0.00402250198496158, -0.00810112326287807, -0.0115130724838425, 0.00356167508620755, -0.00205633406978791, 0.00540074884191871, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440066, -0.00302740663064559, -0.00776832903622087, 0.00137874163078956, -0.00427258096992148, 0.000902598269554293, 0.000868525589828758, 0.00100913554354854, 0.000781673030845887, 0.00180519653910857, -0.000425488904726058, 0.00024565613368079, -0.000868525589828762},
    {-0.0214788046573521, 0.0188672067041848, 0.0108929868695174, 0.00308100195312469, 0.0141189246665007, 0.109364720199429, -0.0789271883087357, -0.0223239800292378, -0.0128887558792643, 0.0127592173566001, 0.00894802256177149, -0.0277290175781224, 0.00585785581564149, 0.0191648629148629, 0.00145539597846298, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755832, -0.0129644660894661, 0, -0.0589869434534289, 0.00727023882565983, -0.0506835250105158, 0.0650269929073343, 0.00659494076734015, 0.0148633126645336, 0.00431740218144092, -0.00166176887443653, -0.00402250198496157, 0.00810112326287806, -0.0115130724838425, -0.00356167508620755, -0.00205633406978791, 0.00540074884191872, 0, 0, -0.0607898426862092, 0.0399597288148543, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440068, 0.0030274066306456, -0.00776832903622088, -0.00137874163078956, -0.00427258096992148, -0.000902598269554298, 0.000868525589828757, 0.00100913554354854, -0.000781673030845891, 0.00180519653910858, 0.000425488904726056, 0.000245656133680792, -0.000868525589828762},
    {0.078755617076958, 0, 0.0435719474780696, -0.0523770332031201, -0.0282378493330015, 0, 0.0631417506469885, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.00672218607954481, 0, -0.0601250601250601, 0.000970263985641994, 0, -0.0125835421224892, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539589, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398463, 0, -0.0028782681209606, 0, -0.00822533627915163, 0.00540074884191871, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.018239037386404, 0, -0.00388416451811042, 0, -0.00466099742173252, 0, -0.00347410235931505, 0.000672757029032356, 0, 0.00120346435940571, 0, 0.000982624534723144, -0.000868525589828765},
    {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, -0.0546823600997146, -0.0315708753234943, -0.0446479600584757, -0.0257775117585285, -0.0182274533665716, 0.0894802256177147, -0.0151249186789759, 0.0585785581564149, 0.0744047619047619, -0.0130985638061669, -0.0281836219336219, 0.0097630930260691, 0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226673, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978424, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905521, -0.00902598269554287, -0.0191075629762328, -0.00908221989193678, -0.00234501909253766, -0.00812338442598858, 0.00340391123780844, 0.00196524906944629, 0.00607967912880135},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, 0.0546823600997146, -0.0315708753234943, 0.0446479600584756, -0.0257775117585286, -0.0182274533665715, -0.0894802256177148, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061669, 0.0281836219336219, 0.00976309302606918, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.0363511941282991, 0.0120675059548847, -0.0232239260383337, -0.0659494076734016, 0.00743165633226682, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978425, 0.0251848460584054, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554289, -0.0191075629762328, -0.00908221989193679, 0.00234501909253767, -0.00812338442598859, -0.00340391123780842, 0.00196524906944628, 0.00607967912880136},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281177, 0, 0, -0.0947126259704827, 0, 0.0515550235170571, -0.0182274533665715, 0, -0.0907495120738551, 0, -0.0946969696969696, 0.0232863356554078, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992785, 0, 0, -0.072702388256598, 0, 0.0464478520766673, 0, -0.048305766159734, 0, -0.0781031370985167, 0.0144810071458617, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992147, 0, 0.0310733161448835, 0, 0.00694820471863012, -0.00605481326129122, 0, -0.0108311792346515, 0, -0.00393049813889258, 0.00607967912880135},
    {0.107394023286761, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937952, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398575, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259878, 0, 0.0312669212338355, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    const double coeff0_6 = coefficients0[dof][6];
    const double coeff0_7 = coefficients0[dof][7];
    const double coeff0_8 = coefficients0[dof][8];
    const double coeff0_9 = coefficients0[dof][9];
    const double coeff0_10 = coefficients0[dof][10];
    const double coeff0_11 = coefficients0[dof][11];
    const double coeff0_12 = coefficients0[dof][12];
    const double coeff0_13 = coefficients0[dof][13];
    const double coeff0_14 = coefficients0[dof][14];
    const double coeff0_15 = coefficients0[dof][15];
    const double coeff0_16 = coefficients0[dof][16];
    const double coeff0_17 = coefficients0[dof][17];
    const double coeff0_18 = coefficients0[dof][18];
    const double coeff0_19 = coefficients0[dof][19];
    const double coeff0_20 = coefficients0[dof][20];
    const double coeff0_21 = coefficients0[dof][21];
    const double coeff0_22 = coefficients0[dof][22];
    const double coeff0_23 = coefficients0[dof][23];
    const double coeff0_24 = coefficients0[dof][24];
    const double coeff0_25 = coefficients0[dof][25];
    const double coeff0_26 = coefficients0[dof][26];
    const double coeff0_27 = coefficients0[dof][27];
    const double coeff0_28 = coefficients0[dof][28];
    const double coeff0_29 = coefficients0[dof][29];
    const double coeff0_30 = coefficients0[dof][30];
    const double coeff0_31 = coefficients0[dof][31];
    const double coeff0_32 = coefficients0[dof][32];
    const double coeff0_33 = coefficients0[dof][33];
    const double coeff0_34 = coefficients0[dof][34];
    const double coeff0_35 = coefficients0[dof][35];
    const double coeff0_36 = coefficients0[dof][36];
    const double coeff0_37 = coefficients0[dof][37];
    const double coeff0_38 = coefficients0[dof][38];
    const double coeff0_39 = coefficients0[dof][39];
    const double coeff0_40 = coefficients0[dof][40];
    const double coeff0_41 = coefficients0[dof][41];
    const double coeff0_42 = coefficients0[dof][42];
    const double coeff0_43 = coefficients0[dof][43];
    const double coeff0_44 = coefficients0[dof][44];
    const double coeff0_45 = coefficients0[dof][45];
    const double coeff0_46 = coefficients0[dof][46];
    const double coeff0_47 = coefficients0[dof][47];
    const double coeff0_48 = coefficients0[dof][48];
    const double coeff0_49 = coefficients0[dof][49];
    const double coeff0_50 = coefficients0[dof][50];
    const double coeff0_51 = coefficients0[dof][51];
    const double coeff0_52 = coefficients0[dof][52];
    const double coeff0_53 = coefficients0[dof][53];
    const double coeff0_54 = coefficients0[dof][54];
    const double coeff0_55 = coefficients0[dof][55];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9 + coeff0_10*basisvalue10 + coeff0_11*basisvalue11 + coeff0_12*basisvalue12 + coeff0_13*basisvalue13 + coeff0_14*basisvalue14 + coeff0_15*basisvalue15 + coeff0_16*basisvalue16 + coeff0_17*basisvalue17 + coeff0_18*basisvalue18 + coeff0_19*basisvalue19 + coeff0_20*basisvalue20 + coeff0_21*basisvalue21 + coeff0_22*basisvalue22 + coeff0_23*basisvalue23 + coeff0_24*basisvalue24 + coeff0_25*basisvalue25 + coeff0_26*basisvalue26 + coeff0_27*basisvalue27 + coeff0_28*basisvalue28 + coeff0_29*basisvalue29 + coeff0_30*basisvalue30 + coeff0_31*basisvalue31 + coeff0_32*basisvalue32 + coeff0_33*basisvalue33 + coeff0_34*basisvalue34 + coeff0_35*basisvalue35 + coeff0_36*basisvalue36 + coeff0_37*basisvalue37 + coeff0_38*basisvalue38 + coeff0_39*basisvalue39 + coeff0_40*basisvalue40 + coeff0_41*basisvalue41 + coeff0_42*basisvalue42 + coeff0_43*basisvalue43 + coeff0_44*basisvalue44 + coeff0_45*basisvalue45 + coeff0_46*basisvalue46 + coeff0_47*basisvalue47 + coeff0_48*basisvalue48 + coeff0_49*basisvalue49 + coeff0_50*basisvalue50 + coeff0_51*basisvalue51 + coeff0_52*basisvalue52 + coeff0_53*basisvalue53 + coeff0_54*basisvalue54 + coeff0_55*basisvalue55;
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
    
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
    
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
    
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
    
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
    
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    static const double coefficients0[56][56] = \
    {{0.00945067404923492, 0.00609788120679252, 0.00352061335622803, 0.00248944957812476, 0.0070594623332504, 0.00546823600997146, 0.00315708753234943, 0.00446479600584755, 0.00257775117585285, 0.00182274533665715, -0.00298267418725716, -0.0025208197798293, -0.00195261860521383, -0.00112734487734488, -0.00218309396769447, -0.00169101731601731, -0.000976309302606919, -0.00138070985681772, -0.000797153207506487, -0.00056367243867244, 0.00278687112460004, 0.00245778931055954, 0.00207721109304566, 0.00160900079398463, 0.000928957041533343, 0.00219831358911339, 0.0018579140830667, 0.00143913406048032, 0.00083088443721827, 0.00160900079398463, 0.0012463266558274, 0.000719567030240151, 0.00101762145320217, 0.000587524019939407, 0.000415442218609132, -0.00705593524717798, -0.00638233357089081, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363026, -0.00582624677716565, -0.00513826668309062, -0.00434262794914383, -0.00336378514516178, -0.00194208225905522, -0.00459580543596519, -0.00388416451811043, -0.00300866089851429, -0.00173705117965752, -0.00336378514516178, -0.00260557676948629, -0.00150433044925714, -0.00212744452363027, -0.00122828066840393, -0.000868525589828762},
    {0.00945067404923495, -0.00609788120679253, 0.00352061335622802, 0.00248944957812476, 0.00705946233325039, -0.00546823600997146, 0.00315708753234944, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521383, -0.00112734487734489, -0.00218309396769448, 0.00169101731601732, -0.000976309302606925, 0.00138070985681772, -0.000797153207506488, -0.000563672438672441, 0.00278687112460004, -0.00245778931055954, 0.00207721109304566, -0.00160900079398463, 0.000928957041533346, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218268, 0.00160900079398463, -0.0012463266558274, 0.000719567030240148, -0.00101762145320216, 0.000587524019939402, 0.000415442218609128, 0.00705593524717798, -0.0063823335708908, 0.00562868913761196, -0.00475711057319694, 0.00368484200521179, -0.00212744452363027, -0.00582624677716565, 0.00513826668309062, -0.00434262794914382, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811043, 0.00300866089851429, -0.00173705117965752, -0.00336378514516178, 0.00260557676948629, -0.00150433044925714, 0.00212744452363026, -0.00122828066840393, -0.00086852558982876},
    {0.00945067404923495, 0, -0.00704122671245605, 0.00248944957812476, 0, 0, 0.00947126259704828, 0, -0.00515550235170572, 0.00182274533665715, 0, 0, 0, 0.00450937950937951, 0, 0, -0.00292892790782075, 0, 0.00159430641501296, -0.00056367243867244, 0, 0, 0, 0, 0.00464478520766673, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987881, 0.00041544221860913, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527608, 0, 0, 0, 0.00694820471863012, 0, 0, -0.00451299134777144, 0, 0.00245656133680786, -0.000868525589828763},
    {0.00945067404923493, 0, 0, -0.0074683487343743, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00563672438672439, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.018239037386404},
    {-0.010023442173431, 0, 0.0374718748311399, -0.0289614183593723, 0, 0, 0.0284137877911448, 0, 0.00515550235170572, -0.00619733414463434, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835494, -0.000187890812890802, 0, 0, 0, 0, 0.0232239260383337, 0, 0, 0, -0.00166176887443652, 0, 0, -0.00647610327216137, 0, 0.00646276421933342, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496074, 0, 0, 0, -0.0277928188745205, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.010023442173431, 0, -0.0287574853355259, 0.05360943398437, 0, 0, -0.0189425251940966, 0, 0.0154665070551171, 0.0153110608279201, 0, 0, 0, -0.00450937950937953, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144092, 0, -0.0123380044187274, 0.00872428659079174, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559316, 0, 0.041270230458372, -0.018239037386404},
    {0.010023442173431, 0, 0.0409576306293854, -0.0449826285156208, 0, 0, 0.0189425251940965, 0, -0.00515550235170569, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883226, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957583, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.055026973944496, 0.0303983956440067},
    {-0.0100234421734311, 0, -0.0148144621425437, 0.0449826285156207, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0187359374155699, -0.0289614183593723, 0.0211783869997512, -0.0164047080299144, 0.00947126259704827, 0.00446479600584757, -0.00257775117585285, -0.00619733414463432, 0.0208787193108001, -0.0176457384588052, 0.0136683302364968, -0.00789141414141413, -0.0084898098743674, 0.00657617845117844, -0.00379675839902689, 0.0023011830946962, -0.00132858867917746, -0.0001878908128908, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992314, 0.00464478520766674, -0.0010991567945567, 0.000928957041533349, -0.000719567030240152, 0.000415442218609127, -0.00482700238195388, 0.00373897996748219, -0.00215870109072045, 0.00559691799261185, -0.00323138210966672, -0.00290809553026392, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935591, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863011, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778515, 0.00607967912880135},
    {0.010023442173431, -0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, 0.0109364720199429, -0.00631417506469883, 0.0133943880175427, -0.00773325352755858, 0.0153110608279201, -0.00298267418725716, 0.00252081977982931, -0.00195261860521382, 0.00112734487734488, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476002, 0.0129522065443227, -0.00747795993496437, -0.00321800158796926, 0.0024926533116548, -0.00143913406048031, -0.0106850252586226, 0.00616900220936373, 0.00872428659079178, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845888, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.020635115229186, -0.0182390373864041},
    {0.0100234421734311, 0.0354703486038674, -0.0204788153146927, -0.0449826285156208, 0.0141189246665008, -0.0109364720199429, 0.00631417506469885, -0.00446479600584758, 0.00257775117585285, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974696, -0.00263047138047138, 0.00151870335961075, 0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620757, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, -0.0128297005588457, 0.00740723107127184, 0.0449826285156208, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0.00920473237878476, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489395, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.010023442173431, 0.0324515955311979, -0.0335503995581136, -0.00801060507812427, 0.0211783869997512, 0.00109364720199429, -0.00568275755822898, -0.0169662248222207, 0.0056710525868763, 0.00838462854862291, 0.0208787193108001, -0.0109235523792603, 0.00499002532443534, -0.00187890812890812, -0.0162519217595033, 0.00958243145743146, -0.00466458889023305, 0.0108922666482287, -0.00558007245254534, -0.00469727032227033, 0.0139343556230002, -0.00245778931055954, -0.00415442218609131, 0.00643600317593851, -0.00464478520766673, -0.0120907247401236, 0.00464478520766674, -0.000719567030240153, -0.000415442218609137, 0.00965400476390778, -0.00498530662330958, 0.00215870109072046, -0.006614539445814, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543313, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438609, 0.00310733161448835, -0.00481385743762287, 0.00347410235931505, -0.00201827108709706, -0.000521115353897262, 0.000902598269554289, 0.00170195561890421, -0.000491312267361565, -0.000868525589828764},
    {0.0100234421734309, -0.024904712849524, 0.0553363732971484, -0.0043134027343746, -0.0141189246665008, 0.0153110608279201, 0.00883984509057843, 0.00803663281052563, -0.0159820572902877, 0.000729098134662861, -0.00298267418725715, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846298, -0.0116492303992304, 0.00976309302606916, 0.00352848074520084, 0.00132858867917745, -0.00263047138047137, 0, 0.0196623144844763, -0.00727023882565982, -0.00724050357293082, 0.00928957041533346, -0.00219831358911338, -0.0148633126645336, 0.0129522065443228, -0.0066470754977461, 0.00402250198496157, 0.00685479660705067, -0.00575653624192123, -0.0045792965394097, -0.000293762009969684, 0.00290809553026392, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.00411061334647249, -0.00303983956440068, 0.00908221989193681, -0.00776832903622088, 0.00045958054359652, 0.00427258096992148, -0.00270779480866287, 0.000868525589828773, -0.00100913554354854, -0.00234501909253766, 0.00180519653910858, 0.00127646671417816, 0.000245656133680781, -0.000868525589828764},
    {0.0100234421734309, 0.0354703486038675, -0.0492363006502187, -0.00431340273437461, 0.0141189246665008, -0.00656188321196574, -0.016416855168217, -0.00982255121286466, 0.0149509568199466, 0.000729098134662872, 0, 0.0033610930397724, 0.0282044909641998, -0.0353234728234728, -0.000485131992820993, -0.0124007936507936, 0.00889526253486299, 0.00291483191994851, 0.00239145962251946, -0.00263047138047138, 0, 0, 0.0218107164769794, -0.00402250198496158, -0.00928957041533347, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992316, 0.00934744991870547, -0.0028782681209606, -0.00254405363300539, -0.00381890612960613, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320202, 0.00840946286290443, -0.00388416451811042, 0, 0.00233049871086626, 0.0030086608985143, -0.00347410235931507, -0.000336378514516178, -0.00286613444643493, 0.00120346435940572, 0.00085097780945211, 0.00098262453472315, -0.000868525589828765},
    {-0.010023442173431, -0.0128297005588457, 0.0448791059024117, -0.00801060507812424, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467804, -0.0175287079957994, 0.0083846285486229, 0, 0, 0.00867830491206149, 0.0225468975468975, 0, -0.00300625300625301, -0.019200749617936, 0.000613648825252316, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240152, -0.0108014976838374, 0, 0.00249265331165478, 0.0107935054536023, -0.000508810726601074, -0.00734405024924252, 0.00290809553026393, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931505, 0, -0.00208446141558903, -0.000902598269554287, 0.000425488904726055, 0.0017195929357655, -0.000868525589828767},
    {-0.0100234421734311, -0.0324515955311978, -0.0187359374155699, -0.0289614183593723, 0.0211783869997511, 0.0164047080299144, 0.00947126259704826, -0.00446479600584755, -0.00257775117585285, -0.00619733414463435, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141413, -0.00848980987436742, -0.00657617845117845, -0.0037967583990269, -0.00230118309469619, -0.00132858867917746, -0.00018789081289081, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992313, 0.00464478520766674, 0.00109915679455668, 0.000928957041533345, 0.000719567030240154, 0.000415442218609131, -0.00482700238195388, -0.00373897996748219, -0.00215870109072045, -0.00559691799261187, -0.00323138210966673, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.00955378148811641, 0.00740032731935591, 0.00427258096992148, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.0069482047186301, 0.0181644397838736, 0.014070114555226, 0.00812338442598859, 0.0136156449512337, 0.00786099627778516, 0.00607967912880135},
    {0.0100234421734308, 0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, -0.0109364720199429, -0.00631417506469886, -0.0133943880175427, -0.00773325352755858, 0.0153110608279201, 0.00298267418725716, 0.00252081977982933, 0.00195261860521383, 0.00112734487734487, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476002, -0.0129522065443228, -0.00747795993496438, -0.00321800158796927, -0.00249265331165479, -0.00143913406048031, 0.0106850252586227, 0.00616900220936374, 0.00872428659079177, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.0206351152291861, -0.0182390373864041},
    {0.010023442173431, -0.0354703486038674, -0.0204788153146927, -0.0449826285156208, 0.0141189246665008, 0.0109364720199429, 0.00631417506469886, 0.00446479600584758, 0.00257775117585287, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974696, 0.00263047138047138, 0.00151870335961075, -0.0308358534689291, -0.0178030883009781, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620757, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588457, 0.00740723107127183, 0.0449826285156208, 0, 0, 0, -0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878479, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489395, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.010023442173431, -0.0324515955311978, -0.0335503995581136, -0.00801060507812424, 0.0211783869997512, -0.00109364720199429, -0.00568275755822899, 0.0169662248222207, 0.00567105258687632, 0.0083846285486229, -0.0208787193108001, -0.0109235523792603, -0.00499002532443534, -0.00187890812890812, -0.0162519217595033, -0.00958243145743146, -0.00466458889023305, -0.0108922666482287, -0.00558007245254535, -0.00469727032227032, 0.0139343556230002, 0.00245778931055953, -0.00415442218609133, -0.00643600317593852, -0.00464478520766672, 0.0120907247401236, 0.00464478520766673, 0.000719567030240156, -0.000415442218609149, 0.00965400476390777, 0.00498530662330958, 0.00215870109072047, 0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543313, 0.00719357335632687, 0.0112908326677739, 0.0114368694935501, 0.00737991258440984, -0.00183832217438608, 0.00310733161448834, 0.00481385743762287, 0.00347410235931505, -0.00201827108709706, 0.000521115353897262, 0.000902598269554287, -0.00170195561890421, -0.000491312267361565, -0.000868525589828767},
    {0.010023442173431, 0.0249047128495239, 0.0553363732971484, -0.00431340273437459, -0.0141189246665008, -0.0153110608279201, 0.00883984509057843, -0.00803663281052563, -0.0159820572902878, 0.000729098134662854, 0.00298267418725716, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846298, 0.0116492303992304, 0.00976309302606918, -0.00352848074520084, 0.00132858867917746, -0.00263047138047138, 0, -0.0196623144844763, -0.0072702388256598, 0.00724050357293083, 0.00928957041533346, 0.00219831358911339, -0.0148633126645336, -0.0129522065443228, -0.00664707549774609, 0.00402250198496157, -0.00685479660705067, -0.00575653624192124, 0.0045792965394097, -0.000293762009969679, 0.00290809553026392, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.0041106133464725, -0.00303983956440067, -0.00908221989193682, -0.00776832903622089, -0.00045958054359652, 0.00427258096992148, 0.00270779480866287, 0.000868525589828776, -0.00100913554354853, 0.00234501909253766, 0.00180519653910858, -0.00127646671417816, 0.000245656133680779, -0.000868525589828761},
    {0.0100234421734309, -0.0354703486038674, -0.0492363006502187, -0.00431340273437459, 0.0141189246665008, 0.00656188321196577, -0.016416855168217, 0.00982255121286466, 0.0149509568199466, 0.000729098134662868, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820994, 0.0124007936507936, 0.00889526253486299, -0.00291483191994852, 0.00239145962251945, -0.00263047138047138, 0, 0, 0.0218107164769794, 0.00402250198496157, -0.00928957041533346, 0, -0.00557374224920009, 0.0143913406048031, 0.0141250354327105, 0.000804500396992309, -0.00934744991870548, -0.0028782681209606, 0.00254405363300539, -0.00381890612960614, 0.00290809553026393, 0, 0, 0, 0.0266398192099028, 0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320203, -0.00840946286290443, -0.00388416451811041, 0, 0.00233049871086626, -0.0030086608985143, -0.00347410235931507, -0.000336378514516177, 0.00286613444643493, 0.00120346435940572, -0.000850977809452108, 0.000982624534723152, -0.000868525589828766},
    {-0.010023442173431, 0.0128297005588457, 0.0448791059024117, -0.00801060507812424, 0, -0.0174983552319087, 0.0132597676358676, 0.00357183680467805, -0.0175287079957994, 0.0083846285486229, 0, 0, -0.00867830491206148, 0.0225468975468975, 0, 0.00300625300625301, -0.019200749617936, -0.000613648825252319, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766673, 0, 0, 0.00719567030240153, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601077, -0.00734405024924252, 0.00290809553026393, 0, 0, 0, 0, -0.0257938940364826, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702859, -0.00347410235931505, 0, 0.00208446141558903, -0.000902598269554286, -0.000425488904726054, 0.0017195929357655, -0.000868525589828767},
    {-0.0100234421734311, -0.0452812960900435, -0.0113287063442981, -0.00801060507812424, -0.0014118924666501, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.00838462854862289, -0.00894802256177147, -0.0142846454190328, -0.014536160727703, -0.00939454064454064, -0.0123708658169354, -0.0125886844636845, -0.00813591085505764, -0.0102786178229763, -0.00664294339588733, -0.00469727032227033, -0.00836061337380012, 0.00245778931055953, 0.00830884437218264, 0.00965400476390777, 0.00650269929073344, 0.00219831358911338, 0.00743165633226677, 0.00863480436288183, 0.00581619106052785, 0.00643600317593852, 0.00747795993496437, 0.00503696921168108, 0.00610572871921293, 0.00411266813957582, 0.00290809553026394, 0.0352796762358899, 0.0191470007126724, 0.00788016479265675, 0.000951422114639391, -0.00221090520312707, -0.00212744452363027, 0.017478740331497, 0.00719357335632686, 0.000868525589828764, -0.00201827108709706, -0.00194208225905521, 0.00643412761035128, 0.000776832903622088, -0.00180519653910857, -0.00173705117965753, 0.000672757029032363, -0.00156334606169177, -0.00150433044925715, -0.00127646671417816, -0.00122828066840393, -0.000868525589828768},
    {0.010023442173431, 0.0603750614533914, -0.00610007264692975, -0.0043134027343746, -0.00564756986660025, -0.0218729440398859, 0.00126283501293978, -0.0178591840233903, 0.00103110047034114, 0.000729098134662884, 0.0417574386216002, 0.0168054651988621, -0.000867830491206148, -0.00526094276094276, 0.0145539597846299, -0.000751563251563252, -0.00455611007883227, -0.000613648825252328, -0.0037200483016969, -0.00263047138047138, 0.00557374224920005, -0.0196623144844763, -0.0103860554652283, 0.00321800158796928, 0.00650269929073341, -0.0175865087129071, -0.00928957041533347, 0.00287826812096063, 0.00581619106052784, -0.00804500396992315, 0.0024926533116548, 0.00503696921168106, 0.00203524290640433, 0.00411266813957581, 0.00290809553026391, -0.0705593524717798, -0.0127646671417816, 0.00450295131008958, 0.00380568845855754, -0.000736968401042372, -0.00212744452363026, -0.0116524935543313, 0.00411061334647251, 0.00347410235931506, -0.000672757029032362, -0.00194208225905521, 0.00367664434877218, 0.00310733161448835, -0.000601732179702865, -0.00173705117965753, 0.00269102811612942, -0.000521115353897265, -0.00150433044925715, -0.00042548890472606, -0.00122828066840393, -0.000868525589828759},
    {0.0100234421734309, -0.0603750614533914, -0.00610007264692972, -0.00431340273437457, -0.00564756986660032, 0.0218729440398859, 0.00126283501293976, 0.0178591840233903, 0.00103110047034113, 0.000729098134662861, -0.0417574386216002, 0.016805465198862, 0.000867830491206157, -0.00526094276094276, 0.0145539597846298, 0.000751563251563257, -0.00455611007883228, 0.00061364882525233, -0.00372004830169691, -0.00263047138047138, 0.00557374224920011, 0.0196623144844763, -0.0103860554652283, -0.00321800158796927, 0.00650269929073344, 0.0175865087129071, -0.00928957041533346, -0.00287826812096062, 0.00581619106052786, -0.00804500396992314, -0.0024926533116548, 0.00503696921168108, -0.00203524290640433, 0.00411266813957583, 0.00290809553026393, 0.0705593524717798, -0.0127646671417816, -0.00450295131008958, 0.00380568845855756, 0.000736968401042369, -0.00212744452363027, -0.0116524935543313, -0.00411061334647251, 0.00347410235931506, 0.000672757029032361, -0.00194208225905522, -0.00367664434877217, 0.00310733161448835, 0.000601732179702866, -0.00173705117965753, 0.00269102811612942, 0.000521115353897263, -0.00150433044925715, 0.000425488904726061, -0.00122828066840394, -0.000868525589828769},
    {-0.0100234421734309, 0.0452812960900435, -0.0113287063442981, -0.00801060507812426, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089231, 0.00838462854862291, 0.00894802256177147, -0.0142846454190327, 0.014536160727703, -0.00939454064454063, -0.0123708658169354, 0.0125886844636845, -0.00813591085505763, 0.0102786178229763, -0.00664294339588732, -0.00469727032227032, -0.00836061337380015, -0.00245778931055953, 0.00830884437218263, -0.00965400476390777, 0.00650269929073343, -0.00219831358911339, 0.00743165633226677, -0.00863480436288183, 0.00581619106052785, 0.00643600317593852, -0.00747795993496437, 0.00503696921168107, -0.00610572871921293, 0.00411266813957582, 0.00290809553026393, -0.0352796762358899, 0.0191470007126724, -0.00788016479265675, 0.000951422114639385, 0.00221090520312707, -0.00212744452363027, 0.0174787403314969, -0.00719357335632687, 0.000868525589828768, 0.00201827108709706, -0.00194208225905521, -0.00643412761035128, 0.000776832903622089, 0.00180519653910857, -0.00173705117965753, 0.000672757029032358, 0.00156334606169177, -0.00150433044925715, 0.00127646671417816, -0.00122828066840393, -0.000868525589828765},
    {0.0787556170769579, 0.0679219441350653, -0.00435719474780691, -0.00308100195312472, 0.0705946233325039, -0.0328094160598287, 0.0126283501293977, -0.0267887760350854, 0.03608851646194, -0.00729098134662862, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015873, 0.0130985638061669, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, 0.0245778931055954, -0.036351194128299, 0.0362025178646541, -0.0232239260383337, 0.0219831358911338, -0.00743165633226679, 0, 0.00166176887443652, -0.0345935170706695, 0.0130864298861876, -0.00359783515120079, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596521, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598858, -0.0102117337134253, 0.00196524906944628, 0.00607967912880136},
    {-0.0214788046573522, 0.00377344134083702, 0.00217859737390344, -0.0215670136718729, 0, 0.0984282481794862, -0.0852413633734345, -0.0312535720409329, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, 0.0236742424242424, -0.0335199527228374, -0.00138070985681772, -0.00079715320750645, 0.00394570707070704, 0, 0, 0.036351194128299, -0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.00724050357293084, -0.0392592896585629, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724418, 0, 0.00694820471863008, 0.00302740663064561, 0.0164151336477636, -0.0108311792346515, -0.00680782247561686, -0.00393049813889256, 0.00607967912880134},
    {0.0787556170769579, 0.0301875307266957, 0.0610007264692975, -0.00308100195312472, 0, 0.0218729440398858, 0.0757701007763863, 0.0178591840233903, -0.0412440188136458, -0.00729098134662862, 0, 0, 0.0520698294723688, -0.00901875901875902, 0, -0.0127765752765753, 0.0286384062098028, 0.00184094647575699, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.0143913406048031, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515274, 0.00407048581280861, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967795, -0.0427258096992148, 0, 0, -0.0300866089851429, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723144, 0.00607967912880136},
    {-0.0214788046573522, 0.00377344134083686, -0.0196073763651313, 0.00924300585937417, 0, -0.0109364720199429, 0.00947126259704823, 0.102690308134494, -0.0695992817480272, -0.060150596109686, 0, -0.00504163955965862, 0.0065087286840461, -0.00450937950937951, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196709, -0.0263060558477138, 0.000563672438672465, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165478, -0.0043174021814409, -0.0605484764655283, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573521, -0.0150937653633479, 0.0130715842434208, 0.00924300585937414, 0, -0.0109364720199429, 0.00947126259704832, -0.00892959201169514, 0.123732056440937, -0.0601505961096861, 0, 0, -0.00650872868404609, 0.00676406926406927, 0, 0.0574945887445887, 0.0126920209338899, -0.0266937238984759, 0.00637722566005184, 0.000563672438672453, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097862, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448302, -0.0616900220936373, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769578, 0.0301875307266958, 0.0174287789912279, 0.0585390371093694, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094276, -0.00455611007883226, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700864, 0.0284934006896604, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769578, -0.0679219441350653, -0.00435719474780693, -0.00308100195312472, 0.0705946233325039, 0.0328094160598287, 0.0126283501293978, 0.0267887760350854, 0.03608851646194, -0.00729098134662859, -0.0298267418725716, 0.0151249186789758, 0.0325436434202305, 0.0248015873015873, 0.0130985638061669, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646541, -0.0232239260383337, -0.0219831358911339, -0.00743165633226677, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861877, -0.00359783515120079, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.0045958054359652, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.00908221989193679, -0.00703505727761299, -0.00812338442598858, 0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573524, -0.00377344134083698, 0.00217859737390344, -0.021567013671873, 0, -0.0984282481794863, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242424, -0.0335199527228374, 0.0013807098568177, -0.00079715320750645, 0.00394570707070706, 0, 0, 0.036351194128299, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838375, -0.00724050357293083, 0.039259289658563, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951694, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.00302740663064561, -0.0164151336477636, -0.0108311792346515, 0.00680782247561685, -0.00393049813889256, 0.00607967912880134},
    {0.0787556170769579, -0.0301875307266956, 0.0610007264692975, -0.00308100195312471, 0, -0.0218729440398858, 0.0757701007763863, -0.0178591840233903, -0.0412440188136458, -0.00729098134662861, 0, 0, -0.0520698294723689, -0.00901875901875903, 0, 0.0127765752765753, 0.0286384062098028, -0.00184094647575695, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766674, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515274, -0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967796, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780843, -0.00982624534723145, 0.00607967912880137},
    {-0.0214788046573523, -0.0037734413408369, -0.0196073763651314, 0.00924300585937414, 0, 0.0109364720199429, 0.00947126259704826, -0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965861, -0.00650872868404608, -0.00450937950937948, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196707, -0.0263060558477138, 0.000563672438672444, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.0024926533116548, -0.00431740218144091, 0.0605484764655282, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, 0.0150937653633478, 0.0130715842434208, 0.00924300585937414, 0, 0.0109364720199429, 0.00947126259704831, 0.00892959201169514, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404611, 0.00676406926406927, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005185, 0.000563672438672461, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097862, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448302, -0.0616900220936373, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769578, -0.0301875307266956, 0.0174287789912279, 0.0585390371093695, 0, -0.0218729440398858, -0.0189425251940966, -0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, -0.00526094276094275, -0.00455611007883226, -0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700864, -0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769576, -0.0377344134083696, -0.0566435317214905, -0.00308100195312473, 0.0423567739995023, 0.0546823600997146, 0.0378850503881931, 0.0446479600584756, 0.00515550235170574, -0.00729098134662862, 0.0417574386216002, -0.0050416395596586, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801536, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055954, -0.00519302773261415, 0.00402250198496157, 0.00464478520766674, -0.00219831358911335, -0.0130053985814669, -0.0158304746652834, -0.0108014976838374, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.00477689074405821, 0.00370016365967795, 0.00427258096992148, -0.0367664434877216, -0.00776832903622087, 0.00601732179702858, 0.00694820471863011, -0.00908221989193682, 0.00703505727761298, 0.0081233844259886, 0.00680782247561684, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573522, 0, -0.0043571947478069, -0.0215670136718729, -0.127070321998507, 0, 0.0284137877911448, 0, 0.03608851646194, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984381, 0, 0.00159430641501295, 0.00394570707070706, 0.0836061337380013, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377579, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.0095537814881164, 0, 0.00427258096992147, 0, -0.0155366580724417, 0, 0.00694820471863011, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778513, 0.00607967912880133},
    {0.0787556170769579, 0.0377344134083696, -0.0566435317214905, -0.0030810019531247, 0.0423567739995023, -0.0546823600997146, 0.0378850503881931, -0.0446479600584757, 0.00515550235170574, -0.00729098134662864, -0.0417574386216002, -0.00504163955965862, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801536, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261416, -0.00402250198496158, 0.00464478520766674, 0.00219831358911339, -0.0130053985814669, 0.0158304746652834, -0.0108014976838374, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405821, -0.00370016365967795, 0.00427258096992148, 0.0367664434877216, -0.00776832903622088, -0.00601732179702859, 0.00694820471863012, -0.00908221989193681, -0.00703505727761299, 0.00812338442598861, -0.00680782247561685, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573522, -0.0188672067041847, 0.00653579212171051, 0.00924300585937415, 0.0141189246665008, 0, -0.00315708753234942, -0.111619900146189, -0.0541327746929101, -0.0601505961096861, -0.00894802256177146, -0.00252081977982933, 0.000650872868404606, 0.00112734487734486, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672439, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476002, -0.00431740218144093, -0.00747795993496438, 0.00321800158796924, 0.0224338798048931, 0.018708742786244, 0.046301776120698, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.017478740331497, -0.00451299134777145, -0.00781673030845887, 0.0363288795677472, -0.00938007637015066, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {-0.0214788046573521, 0.0188672067041847, 0.00653579212171047, 0.00924300585937414, 0.0141189246665007, 0, -0.00315708753234944, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177148, -0.00252081977982932, -0.000650872868404592, 0.00112734487734487, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.019928830187662, 0.000563672438672456, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476002, 0.00431740218144092, -0.00747795993496438, 0.00321800158796927, -0.0224338798048931, 0.018708742786244, -0.0463017761206981, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777143, -0.00781673030845888, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769579, 0, -0.0348575579824557, 0.0585390371093695, -0.0282378493330015, 0, 0.00631417506469885, 0, -0.0206220094068228, 0.083846285486229, 0, 0, 0, 0, -0.0067918478994939, 0, 0.00151870335961074, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233621, 0, -0.0329013451166066, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.0275134869722481, 0.0303983956440068},
    {0.0787556170769578, -0.0377344134083696, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, 0.0546823600997145, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865145, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.0092175078635989, -0.00789141414141412, -0.00347132196482459, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, -0.00491557862111904, -0.0197335053839338, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097694, -0.000623163327913701, -0.00215870109072047, 0.00712335017241508, 0.00411266813957581, 0.00540074884191871, 0, -0.0510586685671264, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173251, -0.0143871467126537, -0.00564541633388696, 0.00571843474677502, 0.00737991258440982, 0.00367664434877216, -0.00155366580724418, 0.00240692871881142, 0.00347410235931506, 0.00100913554354853, 0.00026055767694863, 0.000902598269554296, -0.000850977809452108, -0.000491312267361565, -0.000868525589828763},
    {-0.0214788046573521, 0, -0.0217859737390348, 0.00308100195312476, -0.127070321998507, 0, 0.0473563129852414, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468977, 0.0509388592462045, 0, -0.00943765659186683, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533349, 0, -0.0141250354327105, -0.0144810071458616, 0, -0.00215870109072044, 0, 0.00411266813957584, 0.00540074884191874, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259877, 0, -0.0112908326677739, 0, 0.00737991258440982, 0, -0.00310733161448833, 0, 0.00347410235931505, 0.00201827108709706, 0, 0.000902598269554279, 0, -0.000491312267361592, -0.00086852558982878},
    {0.078755617076958, 0.0377344134083695, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584757, 0.0257775117585286, 0.0291639253865145, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.0092175078635989, 0.00789141414141413, -0.00347132196482461, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111908, -0.0197335053839338, 0.0394205194526234, -0.0325134964536672, 0.0241814494802473, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097695, 0.000623163327913697, -0.00215870109072046, -0.0071233501724151, 0.00411266813957581, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908062, -0.0114170653756727, -0.010317557614593, 0.0127646671417816, 0.00466099742173252, 0.0143871467126538, -0.00564541633388697, -0.00571843474677502, 0.00737991258440983, -0.00367664434877217, -0.00155366580724418, -0.00240692871881144, 0.00347410235931506, 0.00100913554354853, -0.000260557676948628, 0.000902598269554287, 0.000850977809452107, -0.000491312267361569, -0.000868525589828763},
    {-0.0214788046573521, -0.0188672067041848, 0.0108929868695174, 0.0030810019531247, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792643, 0.0127592173566001, -0.0089480225617715, -0.0277290175781224, -0.00585785581564147, 0.0191648629148629, 0.00145539597846298, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755831, -0.0129644660894661, 0, 0.0589869434534289, 0.00727023882565979, 0.0506835250105158, 0.0650269929073344, -0.00659494076734015, 0.0148633126645335, -0.00431740218144092, -0.00166176887443655, -0.00402250198496158, -0.00810112326287807, -0.0115130724838425, 0.00356167508620755, -0.00205633406978791, 0.00540074884191871, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440066, -0.00302740663064559, -0.00776832903622087, 0.00137874163078956, -0.00427258096992148, 0.000902598269554293, 0.000868525589828758, 0.00100913554354854, 0.000781673030845887, 0.00180519653910857, -0.000425488904726058, 0.00024565613368079, -0.000868525589828762},
    {-0.0214788046573521, 0.0188672067041848, 0.0108929868695174, 0.00308100195312469, 0.0141189246665007, 0.109364720199429, -0.0789271883087357, -0.0223239800292378, -0.0128887558792643, 0.0127592173566001, 0.00894802256177149, -0.0277290175781224, 0.00585785581564149, 0.0191648629148629, 0.00145539597846298, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755832, -0.0129644660894661, 0, -0.0589869434534289, 0.00727023882565983, -0.0506835250105158, 0.0650269929073343, 0.00659494076734015, 0.0148633126645336, 0.00431740218144092, -0.00166176887443653, -0.00402250198496157, 0.00810112326287806, -0.0115130724838425, -0.00356167508620755, -0.00205633406978791, 0.00540074884191872, 0, 0, -0.0607898426862092, 0.0399597288148543, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440068, 0.0030274066306456, -0.00776832903622088, -0.00137874163078956, -0.00427258096992148, -0.000902598269554298, 0.000868525589828757, 0.00100913554354854, -0.000781673030845891, 0.00180519653910858, 0.000425488904726056, 0.000245656133680792, -0.000868525589828762},
    {0.078755617076958, 0, 0.0435719474780696, -0.0523770332031201, -0.0282378493330015, 0, 0.0631417506469885, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.00672218607954481, 0, -0.0601250601250601, 0.000970263985641994, 0, -0.0125835421224892, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539589, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398463, 0, -0.0028782681209606, 0, -0.00822533627915163, 0.00540074884191871, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.018239037386404, 0, -0.00388416451811042, 0, -0.00466099742173252, 0, -0.00347410235931505, 0.000672757029032356, 0, 0.00120346435940571, 0, 0.000982624534723144, -0.000868525589828765},
    {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, -0.0546823600997146, -0.0315708753234943, -0.0446479600584757, -0.0257775117585285, -0.0182274533665716, 0.0894802256177147, -0.0151249186789759, 0.0585785581564149, 0.0744047619047619, -0.0130985638061669, -0.0281836219336219, 0.0097630930260691, 0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226673, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978424, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905521, -0.00902598269554287, -0.0191075629762328, -0.00908221989193678, -0.00234501909253766, -0.00812338442598858, 0.00340391123780844, 0.00196524906944629, 0.00607967912880135},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, 0.0546823600997146, -0.0315708753234943, 0.0446479600584756, -0.0257775117585286, -0.0182274533665715, -0.0894802256177148, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061669, 0.0281836219336219, 0.00976309302606918, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.0363511941282991, 0.0120675059548847, -0.0232239260383337, -0.0659494076734016, 0.00743165633226682, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978425, 0.0251848460584054, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554289, -0.0191075629762328, -0.00908221989193679, 0.00234501909253767, -0.00812338442598859, -0.00340391123780842, 0.00196524906944628, 0.00607967912880136},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281177, 0, 0, -0.0947126259704827, 0, 0.0515550235170571, -0.0182274533665715, 0, -0.0907495120738551, 0, -0.0946969696969696, 0.0232863356554078, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992785, 0, 0, -0.072702388256598, 0, 0.0464478520766673, 0, -0.048305766159734, 0, -0.0781031370985167, 0.0144810071458617, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992147, 0, 0.0310733161448835, 0, 0.00694820471863012, -0.00605481326129122, 0, -0.0108311792346515, 0, -0.00393049813889258, 0.00607967912880135},
    {0.107394023286761, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937952, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398575, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259878, 0, 0.0312669212338355, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    static const double dmats0[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {6.32455532033677, 1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 11.2249721603218, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.58257569495583, 0, 8.36660026534075, -1.18321595661993, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.74165738677394, 0, 0, 8.69482604771365, 0, 0, 0, 0, 0, 0, 0, 0, 1.18793863634892e-14, 0, 0, 0, -2.04281036531029e-14, 1.06581410364015e-14, 1.4210854715202e-14, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.33226762955019e-14, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.49909083394701, -1.24344978758018e-14, -3.3466401061363, -2.36643191323985, 15.4919333848297, 0, 0.692820323027545, 0, 0.565685424949238, 0.400000000000001, -1.24344978758018e-14, 2.22044604925031e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 4.89897948556636, -1.4210854715202e-14, 0, 0, 14.1985914794391, 0, -0.828078671210839, 0, 0, 0, -1.24344978758018e-14, 1.80966353013901e-14, -1.06581410364015e-14, 0, 1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 2.48689957516035e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.6, 0, 8.76356092008264, -1.54919333848296, 0, 0, 9.52470471983252, 0, -1.48131215963607, 0.261861468283189, 0, -1.06581410364015e-14, 0, 0, 0, -1.06581410364015e-14, 2.48689957516035e-14, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 4.24264068711928, 0, -2.48689957516035e-14, 0, 0, 0, 14.3427433120127, 0, 0, 0, 0, 0, 0, 0, 2.22044604925031e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, -1.06581410364015e-14, 0},
    {3.11769145362397, 0, 3.1622776601684, 4.91934955049952, 0, 0, 0, 0, 10.690449676497, -2.41897262725905, 0, 0, 1.50990331349021e-14, 0, 0, 0, -2.30926389122033e-14, 0, 1.77635683940025e-14, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, -1.33226762955019e-14, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.37945210809676e-14, 0, 0, 1.95399252334028e-14, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.54558441227157, 0, 0, 7.66811580507232, 0, 0, 0, 0, 0, 10.3691851174526, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 11.6057691494799, 0, 0, 0, -4.20458932936041, 0, -3.43303281162798, 0, 1.25455201782643e-14, 19.6214168703486, 0, 0.611677741841184, 0, 0, 0.529728463363969, 0, 0.432521479156776, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.53137212479106, 0, 3.54562104171168, -2.50713268211204, 4.7871355387817, 0, -5.56626745075489, 0, -1.04880884817016, 0.741619848709568, 0, 18.8034951158403, -1.24344978758018e-14, 1.68183573174417, -0.603122387046461, 0, 0.701283781217647, 0, 0.13213749452868, -0.0934353184302319, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, -1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.08246744900953e-14, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 2.5951288749407, 0, 0, 0, 10.0285307284481, -1.4210854715202e-14, -0.767649473578743, 0, 0, 0, 0, 16.4130361329658, -1.06581410364015e-14, 0, -1.26347611061158, 1.4210854715202e-14, 0.0967147429123241, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.96647939483827, 0, 8.12403840463597, -1.64130361329658, 0, 0, 11.7728501222091, 0, -2.40312296813957, 0.485504156227611, 1.06581410364015e-14, -1.06581410364015e-14, 0, 10.2761860629321, 0, 0, -1.48323969741913, 0, 0.302765035409733, -0.0611677741841268, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.05298244095219, 0, -1.5856499343442, 4.4848952846511, 4.28174419288838, 0, 0.191485421551275, -1.06581410364015e-14, -3.75233260785875, -4.31161222746203, 0, 0, 0, 0, 18.8807171866294, 0, 0.844371341865028, 0, 1.18187368057054, 1.04463861754668, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, -1.15463194561016e-14, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0},
    {0, 2.32115382989599, -1.4210854715202e-14, -1.06581410364015e-14, 0, 3.92428337406972, 0, 5.49285249860473, -1.59872115546023e-14, 0, 0, 0, 0, 0, -1.68753899743024e-14, 17.3044631365566, -1.77635683940025e-14, -1.73008591662713, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.77635683940025e-14, 2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0},
    {2.65329983228431, 0, 4.15220619990511, 2.93605316083775, 0, 0, 2.63248931621763, 0, 9.82591326180786, -2.82261277137741, 0, 0, 0, 0, 0, 1.4210854715202e-14, 11.6081867662439, 0, -3.09487177319468, 0.683876505540245, 0, 0, 0, 0, 0, 0, 0, 0, 1.50990331349021e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.26565424807268e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 2.01017818278146, 0, 0, 0, 0, 0, 8.91928088068603, 0, 0, 0, 0, 0, 0, 0, 0, 0, 16.8558564884489, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.2978250586152, 0, 1.49829835452881, 5.50917601898534, 0, 0, 0, 0, 6.64803945320938, 4.13676892548487, 0, 0, 1.06581410364015e-14, 0, 0, 0, -2.57571741713036e-14, 1.24344978758018e-14, 12.5636136423842, -3.55352656109506, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87616630392937, -1.19904086659517e-14, 0, 6.22830929985771, 0, 0, 0, 0, 0, 11.0541524195338, 0, 0, -1.19904086659517e-14, 0, -1.4210854715202e-14, 0, 2.99760216648792e-14, -1.06581410364015e-14, 0, 11.60576914948, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.5709655798446e-14, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.83186700722508, 0, -4.41087092453693, -3.11895674167864, 16.6749979177077, 0, 2.13065307820925, 0, 1.739670953501, 1.23013312825382, 0, -4.36653943835009, 0, -0.557935943461983, -3.78153408023783, -1.06581410364015e-14, -0.483186700722515, 0, -0.394520289089675, -0.278967971730983, 23.6981011897578, 0, 0.490653381462667, 0, 0.0626933893908967, 0, 0.438853725736256, 0, 0.0560746721671611, 0.380058475033045, 0, 0.0485620906056452, 0, 0.0396507809422086, 0.0280373360835817, 0, 0, 0, 0, 0, 0, -1.46549439250521e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 6.91050664139875, 0, 0, 0, 10.5706241650196, 0, -2.72554057547698, 0, 1.15463194561016e-14, 4.67332857821916, 0, -7.86705867811458, -2.48689957516035e-14, 0, -2.39719223514613, 0, 0.618094504365252, 0, 0, 0, 23.10555488766, 0, 1.62065746295906, 0, -0.469687194424183, 0, 0.790669146636837, 0, 0, 0.240926884676763, 0, -0.0621208350291009, 0, 0, 0, 0, 0, 0, 0, -1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.85449644663775, 1.15463194561016e-14, 5.98171817966646, -2.4880667576406, 1.90029237516522, 0, -0.33993463423953, 0, -2.3592214660616, 0.981306762925316, 0, 9.95226703056241, 0, -6.45364422706459, -0.43094580368567, 0, 0.077089928932768, 0, 0.535021138879229, -0.222539456105679, 0, 0, 21.5273297444417, 0, 2.75066129614997, 0, -1.00024047132728, 0, 0.648615649442924, 0.0433116829031337, 0, -0.0077478293753126, 0, -0.0537716476536687, 0.0223660568773445, 0, 0, 0, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.06581410364015e-14, 1.54523626091315, -1.4210854715202e-14, 0, 0, 6.71778024562961, 0, -0.609449400220043, 0, 0, 0, 1.06581410364015e-14, 14.6593983227032, 1.4210854715202e-14, 0, -1.52344936219873, 0, 0.138210132827973, 0, 0, 0, 0, 0, 18.119501277094, 0, 0, 0, -1.47332496632642, 0, 0, 0.153112421864228, 0, -0.0138906409944104, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.52336024752224, 1.4210854715202e-14, 7.37120708415867, -1.62882203585591, 0, 0, 12.0171306297064, 0, -2.90724328050879, 0.642416074439625, 0, 0, 2.1316282072803e-14, 13.9858820944259, 0, 0, -2.72522906732402, 0, 0.659300804656272, -0.145686271816946, 0, 0, 0, 0, 10.8043830863937, 0, 0, 0, -1.40563403846548, 0, 0, 0.273895826790425, 0, -0.0662622240310384, 0.0146420212340128, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 6.30840061880559, 0, 0, 1.06581410364015e-14, -1.5236235005501, 0, 11.1963004093827, 0, 0, 4.26614580154029, 0, 0.132992762321608, 0, 0, -4.72217953828917, 0, -6.67684597013021, 0, 0, 0, 0, 0, 0, 0, 23.1532444221696, 0, 0.721778878560592, 0, 0, 1.11125127955292, 0, 1.1908743922773, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0},
    {3.51865775274499, -1.04360964314765e-14, 1.92724822331885, 2.725540575477, 1.73472166622177, 0, -2.01705689521107, -1.77635683940025e-14, 3.42052627529743, -4.03112887414928, -1.28785870856518e-14, 4.08831086321549, -2.30926389122033e-14, 0.365669640132028, 5.37643791520787, 0, -6.25148194071026, 0, -2.03981013744551, 1.8080332206528, 0, 0, 0, 0, 0, 0, 22.1880978975596, 1.765254609154e-14, 1.98456380761453, -1.26521523890981, 0, 1.4711357857929, 0, 0.363818136386342, -0.294009452979941, 0, 0, 0, 0, 0, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, -2.30926389122033e-14, 0},
    {0, 1.4106012612951, 0, 0, 0, 3.634054100636, 0, 2.50356888118883, -1.24344978758018e-14, 0, 0, 0, 3.56857028479908, 0, 0, 11.263055412878, 0, -1.4929881464507, 0, 0, 0, 0, 0, 0, 0, 0, 0, 19.3673596462205, 0, 0, -2.65048896124152, 0, 0.266287609379582, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, -1.24344978758018e-14, 0, 0, -1.59872115546023e-14, 0, -1.24344978758018e-14, 0, 0},
    {2.30350221379959, 0, 4.41588043316389, 1.78428514239955, 0, 0, 4.2661458015403, 0, 7.8374102865679, -2.63899331455872, 0, 0, 0, 2.23427840700302, 1.77635683940025e-14, 0, 13.2221027072096, -1.4210854715202e-14, -4.67379217909115, 1.18363558466232, 0, 0, 0, 0, 0, 0, 0, 0, 12.1258851598179, 0, 0, -3.11150358274815, 0, 0.833612074594101, -0.192474367616158, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.8421709430404e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.14718316987778, 0, -0.861891607371343, 6.09449400220045, 1.55158222708545, 0, 0.0693888666488798, -1.95399252334028e-14, -3.05941170815566, 1.80277563773199, 0, -1.26842980563424e-14, 0, 0, 9.03122156537879, 0, 0.403888506800988, -1.4210854715202e-14, -3.67461994579465, -5.49650682721193, 0, 0, 0, 0, 0, 0, 0, 0, 0, 21.7841260588564, 0, 0.974215733960557, 0, 1.78974858569576, 1.8079190879493, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1.26168012376112, 1.24344978758018e-14, 0, 0, 1.42204860051343, 0, 4.47852016375307, 0, 0, 0, 0, 0, 0, 0, 8.27725128825318, 0, 5.37909280973869, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 19.9654813226339, 0, -2.61992366301004, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 1.06581410364015e-14, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 1.33226762955019e-14, 0, 0},
    {2.06031501455084, 0, 2.25696201807219, 3.98978286964828, 0, 0, 0.953939201416947, 0, 8.01142041980429, 1.18019368870418, 0, 0, 0, 0, 0, 1.77635683940025e-14, 5.55254896421454, 0, 9.62241374388481, -3.59830837050572, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 13.3932520322736, 0, -4.68666192508059, 1.18356086726904, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1.09264703862702, 0, 0, 0, 0, 0, 5.38682254607085, 0, 0, 0, 0, 0, 0, 0, 0, 0, 13.4377817958113, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 18.907670401189, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.78428514239954, 0, 0.814411017927931, 5.0677048703591, 0, 0, 0, 0, 4.01510046524764, 6.58672305740272, 0, 0, -1.06581410364015e-14, 0, 0, 0, 1.59872115546023e-14, 0, 10.0159311874144, 2.83293314500743, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 14.0929454377398, -4.55552790213386, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.22124532708767e-14, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.45686271816936, 0, 0, 5.07816454066238, 0, 0, 0, 0, 1.06581410364015e-14, 10.0142755247553, 0, 0, 0, 0, 0, 0, 0, 0, 0, 13.8784813613723, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.5535587278944, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    static const double dmats1[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.47722557505167, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.95803989154981, 5.6124860801609, -1.08012344973464, -0.763762615825972, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.29128784747791, 7.24568837309471, 4.18330013267037, -0.591607978309964, 0, 0, 0, 0, 0, 0, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-2.64575131106461, 0, 9.66091783079295, 0.683130051063968, 0, 0, 0, 0, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338697, 0, 0, 4.34741302385683, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.02140518265514e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.24037034920393, 0, 0, 7.52994023880668, 0, 0, 0, 0, 0, 0, 0, 0, 1.0547118733939e-14, 0, 0, 0, -1.84297022087776e-14, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.7495454169735, 5.79655069847576, -1.67332005306815, -1.18321595661993, 7.74596669241484, -1.2, 0.346410161513775, -0.979795897113271, 0.28284271247462, 0.200000000000002, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.32379000772444, 2.44948974278317, 2.82842712474617, -1.00000000000001, 9.16515138991169, 7.09929573971954, -2.04939015319193, -0.414039335605414, -0.478091443733755, 0.169030850945704, -1.4210854715202e-14, 1.06581410364015e-14, 0, -1.15463194561016e-14, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.80000000000001, -5.69209978830307, 4.38178046004132, -0.77459666924149, 0, 10.998181667894, 4.76235235991626, 0.962140470884713, -0.740656079818035, 0.130930734141589, 1.19904086659517e-14, -1.4210854715202e-14, 1.53210777398272e-14, -1.33226762955019e-14, 0, 1.06581410364015e-14, 1.4210854715202e-14, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.19615242270665, 0, -3.1622776601684, -2.23606797749978, 0, 0, 13.7477270848675, -1.4210854715202e-14, 0.534522483824862, 0.377964473009222, 1.4210854715202e-14, 0, 0, -3.01980662698043e-14, 0, 0, 1.77635683940025e-14, 0, -1.39055433834301e-14, 0, 0, 0, 0, 0, -1.24344978758018e-14, 0, 0, 0, 1.14908083048704e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.01246117974981, 2.12132034355964, -0.40824829046386, 3.17542648054293, 0, 0, 0, 7.17137165600636, -1.38013111868471, -1.56144011671765, 0, 0, 0, 0, 0, 0, -1.55431223447522e-14, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.55884572681199, 2.73861278752584, 1.5811388300842, 2.45967477524976, 0, 0, 0, 9.25820099772551, 5.34522483824848, -1.20948631362952, 0, 0, 0, 0, 0, 1.35447209004269e-14, -1.33226762955019e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.80000000000002, 0, 3.65148371670112, -2.84018778721878, 0, 0, 0, 0, 12.3442679969673, 1.39659449751035, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.27279220613578, 0, 0, 3.83405790253616, 0, 0, 0, 0, 0, 5.18459255872629, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.20454076850486, 0, 0, 6.64078308635359, 0, 0, 0, 0, 0, 8.97997772825746, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.56904651573302, 5.80288457473996, -2.01017818278147, -1.42141062443803, 8.14215222512715, -2.10229466468021, 0.728256234341417, -1.71651640581399, 0.594618725379072, 0.42045893293605, 9.81070843517429, -1.18450885369836, 0.305838870920592, -0.105945692672798, -1.02581475831037, 0.264864231681986, -0.09175166127618, 0.216260739578392, -0.0749149177264366, -0.0529728463363877, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239553, 3.07059789431495, 1.77281052085583, -1.25356634105602, 2.39356776939085, 6.30376871403132, -2.78313372537745, -0.908295106229253, -0.524404424085079, 0.370809924354787, 11.1242977306435, 9.40174755792014, -2.42752078113807, 0.840917865872087, -0.301561193523231, -0.794200206656965, 0.350641890608825, 0.11443442705426, 0.0660687472643419, -0.0467176592151146, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551269, 1.29756443747037, 3.74574588632197, -1.05945692672795, -8.09173593712687, 5.01426536422408, 1.80936716113936, -0.383824736789376, -1.10800657553489, 0.313391585264003, 2.1316282072803e-14, 12.7134831207354, 8.20651806648291, -2.84282124887606, 1.01946290306866, -0.631738055305796, -0.227958835180072, 0.0483573714561647, 0.139595707137603, -0.0394836284566167, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741913, 7.03562363973513, 4.06201920231799, -0.820651806648276, 0, -6.79705818718656, 5.88642506110458, -2.08116553882674, -1.2015614840698, 0.242752078113814, 0, -1.06581410364015e-14, 14.8323969741913, 5.13809303146608, 0, 0.856348838577665, -0.741619848709565, 0.262202212042526, 0.151382517704867, -0.0305838870920541, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-3.4253953543107, 0, 9.38083151964687, 1.89521416591737, 0, 0, -3.39852909359329, 0, -2.77488738510233, -0.560611910581384, -1.06581410364015e-14, 0, 0, 17.7988763690296, 0, 0, 0.428174419288809, 0, 0.349602949390039, 0.0706304617818535, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 1.50990331349021e-14, 0, 0, 0, 0, -1.73194791841524e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047609, 2.74642624930236, -0.792824967172096, 2.24244764232554, 2.14087209644419, -0.331662479035538, 0.0957427107756352, 6.49923072370876, -1.87616630392937, -2.15580611373101, -1.06581410364015e-14, 0, 0, 0, 9.44035859331469, -1.46249406456535, 0.422185670932514, -2.04706526287665, 0.590936840285273, 0.522319308773341, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.71269767715535, 1.16057691494799, 1.34011878852097, 1.89521416591735, 2.53311402559511, 1.96214168703486, -0.566421515598876, 2.74642624930236, 3.17129986868836, -1.82198870938951, -1.77635683940025e-14, 0, 0, 0, 11.1699829238239, 8.6522315682783, -2.49768411251824, -0.865042958313563, -0.998865569685877, 0.441440386136644, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 1.59872115546023e-14, 0, -1.77635683940025e-14, 0, 0, 0, 0, 1.06581410364015e-14, -1.09356967925578e-14, 1.02140518265514e-14, -1.33226762955019e-14, 0},
    {1.32664991614216, -2.6969370381518, 2.07610309995255, 1.46802658041887, 0, 3.03973683071412, 1.31624465810881, -6.38211787508103, 4.91295663090393, -1.4113063856887, 0, 0, 0, 0, 0, 13.4039795085888, 5.80409338312195, 2.01017818278148, -1.54743588659734, 0.341938252770126, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.11022302462516e-14, 1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 1.6228338117763e-14, 0, 0, 0, 0},
    {3.82970843102536, 0, -1.4982983545288, 4.2378277069118, 0, 0, 3.79967103839266, -1.33226762955019e-14, -3.54562104171166, -4.07409060843206, 0, 0, -1.4210854715202e-14, 0, 1.4210854715202e-14, 1.25455201782643e-14, 16.7549743857359, -1.06581410364015e-14, 1.11676565710081, 0.987090711415277, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 1.00508909139073, -0.193429485824663, 3.55615782880926, 0, 0, 0, 4.45964044034302, -0.858258202906996, 2.67027285925187, 0, 0, 0, 0, 0, 0, 0, 8.42792824422443, -1.62195554683793, -2.29379153190449, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.14891252930761, 1.29756443747035, 0.749149177264408, 2.75458800949267, 0, 0, 0, 5.75737105184053, 3.32401972660469, 2.06838446274244, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 10.8804085776374, 6.2818068211921, -1.77676328054754, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.32664991614216, -1.06581410364015e-14, 1.73008591662715, -3.18072425757424, 0, 0, 0, 0, 7.67649473578738, -2.38836465270395, 0, 0, 0, 0, 0, 1.06581410364015e-14, -1.47659662275146e-14, 0, 14.5072114368499, 2.05162951662074, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.938083151964684, 0, 0, 3.11415464992885, 0, 0, 0, 0, 0, 5.52707620976691, 0, 0, 0, 0, 0, 0, 1.58761892521397e-14, 0, 0, 5.80288457473998, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.62480768092719, 0, 0, 5.39387407630365, 0, 0, 0, 0, 0, 9.57317681262151, 0, 0, -1.15463194561016e-14, 0, -1.24344978758018e-14, 0, 2.63122856836162e-14, 0, 0, 10.0508909139074, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.32116539930394e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.41593350361254, 5.72988941019469, -2.20543546226846, -1.55947837083932, 8.33749895885385, -2.76779953857108, 1.06532653910463, -2.25989885993663, 0.869835476750496, 0.615066564126918, 10.333118277332, -2.18326971917505, 0.724780051083766, -0.278967971730992, -1.89076704011891, 0.627677936394715, -0.241593350361261, 0.51249688899006, -0.197260144544837, -0.139483985865487, 11.8490505948789, -1.16109782023228, 0.245326690731332, -0.0814411017927985, 0.0313466946954515, -1.03851746182648, 0.21942686286813, -0.0728431359084674, 0.0280373360835804, 0.190029237516525, -0.0630840061880544, 0.0242810453028245, -0.0515078753637725, 0.0198253904711041, 0.0140186680417935, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.18529407725406, 3.45525332069938, 0.997445717412069, -1.41060126129511, 3.23209352764251, 5.28531208250978, -2.89087233497829, -1.36277028773849, -0.393397896234718, 0.556348640264195, 2.33666428910958, 9.02784807228869, -3.93352933905728, 1.64018416088946, -0.732969916135164, -1.19859611757308, 0.655588223176204, 0.309047252182618, 0.0892142571199805, -0.126168012376108, 13.0996183150503, 11.55277744383, -2.44096975264613, 0.810328731479533, -0.311895674167856, -0.234843597212092, -0.907332870306513, 0.395334573318419, -0.164844710568311, 0.0736662483163206, 0.120463442338383, -0.0658890955530712, -0.0310604175145501, -0.00896637020658453, 0.0126803623514089, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.92724822331887, 1.82834820066012, 2.99085908983323, -1.2440333788203, 0.9501461875826, 5.7406445631131, -0.169967317119757, -0.721110255092796, -1.1796107330308, 0.49065338146266, -10.3037210107158, 4.97613351528119, 5.7817446699566, -3.2268221135323, -0.215472901842833, -1.3018558182182, 0.0385449644663813, 0.163532434528613, 0.267510569439617, -0.111269728052837, 0, 14.555131461167, 10.7636648722209, -3.57321383950879, 1.37533064807499, 1.03556292536503, -0.500120235663638, -0.581087203147974, 0.32430782472146, 0.0216558414515668, 0.130841432732455, -0.00387391468765474, -0.0164356280722655, -0.0268858238268351, 0.0111830284386718, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.62882203585592, 0.772618130456564, 3.86595114186566, -1.05140010313427, 9.1544290607646, 3.3588901228148, 3.878512233171, -0.304724700110022, -1.52475169285615, 0.414677792940102, 0, -9.4626009282084, 7.32969916135161, 0.564240504518009, -2.07602937339135, -0.761724681099364, -0.879563899362182, 0.0691050664139832, 0.34578118203618, -0.0940400840863436, 0, 0, 16.3745228938128, 9.05975063854702, -3.48709964663735, 0, 0.951027176355639, -0.736662483163211, -0.0567083043941548, 0.208648802584721, 0.0765562109321125, 0.0883994979795828, -0.00694532049720742, -0.0347523163751395, 0.00945138406569365, 0, -1.4210854715202e-14, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376112, -6.78263087840204, 3.68560354207933, -0.814411017927966, 0, 10.0216907612581, 6.00856531485322, 2.67510569439611, -1.45362164025439, 0.321208037219813, 1.33226762955019e-14, -1.06581410364015e-14, -7.5700807425667, 6.99294104721295, 0, -2.27270584034421, -1.36261453366202, -0.606656948415834, 0.329650402328125, -0.0728431359084816, 0, 0, 0, 18.7137404500718, 5.40219154319687, 0, 0, 0.760821741084501, -0.702817019232737, 0, 0.228415531250638, 0.136947913395213, 0.0609713174047632, -0.0331311120155173, 0.00732101061700419, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.09901951359278, 0, -4.65474668125631, -3.2914029430219, 1.24344978758018e-14, 0, 13.490737563232, 0, 1.83585684909538, 1.29814682728311, -1.4210854715202e-14, -3.5527136788005e-14, 4.2632564145606e-14, -3.53270434653113, -2.1316282072803e-14, 0, -3.05941170815567, 0, -0.416333199893195, -0.29439202887759, 0, 0, 0, 0, 21.8326971917504, 0, 0, 0, 0.35505014583943, 0, 0, 0.307482445914323, 0, 0.0418430609640534, 0.0295875121532867, 0, 0, 0, -1.77635683940025e-14, 1.4210854715202e-14, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 1.15463194561016e-14, 0, 0, 1.15463194561016e-14, 0, 0, 0},
    {1.99489143482413, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176042, -0.761811750275057, 0.263899331455877, 5.59815020469134, -1.93925611658551, -2.28543525082516, 2.13307290077014, -0.257539376818858, 0.0664963811608033, -0.0230350221379969, 9.14446135479766, -2.36108976914457, 0.817905488277891, -3.3384229850651, 1.15646364545769, 1.02505848514082, 0, 0, 0, 0, 0, 11.5766222110848, -1.39771878815471, 0.360889439280296, -0.125015768949702, -2.15192884958005, 0.555625639776456, -0.192474367616153, 0.595437196138647, -0.206265495285698, -0.166687691932932, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.7593288763725, 1.66904592079255, 0.96362411165942, 1.3627702877385, 0.867360833110887, 2.28430636201792, -1.00852844760553, 2.96226264871971, 1.71026313764871, -2.01556443707464, 2.41867732448956, 2.04415543160774, -0.527798662911756, 0.182834820066013, 2.68821895760393, 7.07977053255619, -3.12574097035514, -1.76652739792485, -1.01990506872276, 0.904016610326401, 0, 0, 0, 0, 0, 13.1266557397187, 11.0940489487798, -2.8644711213756, 0.992281903807267, -0.6326076194549, -1.66605356688627, 0.735567892896449, 0.31507574846809, 0.181909068193174, -0.147004726489965, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, -1.24344978758018e-14, 0},
    {1.48690428533296, 0.705300630647549, 2.03602754481988, 1.1517511068998, -2.93221479395416, 1.81702705031799, 0.655663160391206, 1.25178444059441, 3.61359041872287, -1.70346285967313, 1.06581410364015e-14, 2.7642026565595, 1.78428514239953, -0.618094504365255, -9.08783875864327, 5.63152770643899, 2.03210252328837, -0.74649407322535, -2.15494277062553, 0.764033484562597, 0, 0, 0, 0, 0, 0, 15.0018922739643, 9.68367982311024, -3.3545250915713, 2.13860408462397, -1.32524448062076, -0.4782064110178, 0.133143804689791, 0.384353057392901, -0.124241670058201, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 1.06581410364015e-14, 0, 0, 1.06581410364015e-14, -1.86517468137026e-14, 0, 1.13797860024079e-14, 0, 0, 0, 0, 0},
    {1.15175110689979, 3.82426463519456, 2.20794021658195, 0.89214257119979, 0, -2.46306042692148, 2.13307290077015, 6.78739640804928, 3.91870514328395, -1.31949665727937, 0, 0, 3.2249030993194, 1.11713920350152, 2.1316282072803e-14, -7.63378455726039, 6.61105135360481, -4.04762275910198, -2.33689608954558, 0.591817792331174, 0, 0, 0, 0, 0, 0, 0, 17.5022076529583, 6.06294257990893, 0, 1.79642743108413, -1.55575179137407, 0.721929233499942, 0.416806037297052, -0.0962371838080749, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.1316282072803e-14, 0, 0, 0, -1.59872115546023e-14, 0, 0, -1.24344978758018e-14, 0, -2.44249065417534e-14, -1.06581410364015e-14, 0},
    {-2.65985524643218, 0, 5.09901951359278, -2.06031501455085, 0, 0, -1.23153021346075, 1.59872115546023e-14, 9.04986187739903, 3.04724700110024, 0, 2.1316282072803e-14, 0, 3.86988371918331, 0, -4.2632564145606e-14, -3.81689227863023, 0, -5.39683034546928, -1.36674464685442, 0, 0, 0, 0, 1.28785870856518e-14, 0, 0, 0, 21.00264918355, 0, 0, 0.898213715542074, 0, 0.962572311333254, 0.222250255910579, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -3.10862446895044e-14, 0, 0, 1.50990331349021e-14, 0, 0, 0, 0, 0, -2.30926389122033e-14, 0},
    {1.57359158493889, 1.49284005458435, -0.430945803685675, 3.04724700110022, 0.775791113542725, -0.120185042515472, 0.0346944333244361, 5.29905651979669, -1.52970585407783, 0.901387818865992, 0, 0, 0, 0, 4.51561078268939, -0.699555414372423, 0.201944253400492, 6.36462844462229, -1.83730997289733, -2.74825341360597, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.8920630294282, -1.68739114875268, 0.487107866980278, -3.09993548319961, 0.894874292847881, 0.903959543974651, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32992762321609, 0.630840061880562, 0.7284313590847, 2.57539376818856, 0.917928424547682, 0.711024300256705, -0.205255035576785, 2.23926008187652, 2.58567482211401, 0.761811750275061, 0, 0, 0, 0, 5.34294273196308, 4.13862564412659, -1.19471831485579, 2.68954640486935, 3.10562068169861, -2.32269806558124, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.8876627769294, 9.98274066131696, -2.88176900403079, -1.30996183150502, -1.51261363209513, 0.763985254692628, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.03015750727541, -1.46593983227032, 1.1284810090361, 1.99489143482414, 0, 1.10151410945721, 0.476969600708482, -5.20357020296093, 4.00571020990214, 0.590096844352088, 0, 0, 0, 0, 0, 6.4115312783557, 2.77627448210726, -6.24994106094658, 4.81120687194241, -1.79915418525285, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.4651953323153, 6.69662601613678, 3.0440762145518, -2.34333096254029, 0.591780433634522, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.01030295240889e-14, 0, 1.06581410364015e-14, 0, 0},
    {2.9738085706659, -2.1316282072803e-14, -0.814411017927963, 5.758755534499, 0, 0, 1.37689263682153, 0, -2.8908723349783, 1.70346285967311, 2.1316282072803e-14, 0, 0, 0, 0, 0, 8.01441409794462, 0, -3.47218947830368, -5.19371076584692, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 19.3314941653941, 0, 1.69115345252877, 1.70832296330021, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, -1.77635683940025e-14, 0, 0, 0, -1.4210854715202e-14, 0, 0, 0},
    {1.15175110689979, 0.546323519313513, -0.105140010313428, 3.27118942773249, 0, 0, 0, 2.69341127303542, -0.518347241175123, 4.25171145123352, 0, 0, 0, 0, 0, 0, 1.19904086659517e-14, 6.71889089790563, -1.29305115618717, 1.82865048192213, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.45383520059452, -1.81939143264587, -2.94058061635795, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.892142571199769, 0.705300630647548, 0.407205508963976, 2.53385243517955, 0, 0, 0, 3.47717900165117, 2.00755023262382, 3.29336152870135, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 8.67405085085771, 5.0079655937072, 1.41646657250371, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.2048487632307, 7.0464727188699, -2.27776395106694, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.03015750727543, 0, 0.940400840863387, -2.92584077107541, 0, 0, 0, 0, 4.63623866886823, -3.80284633026897, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 11.5654011344769, -1.63559471386627, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 16.2731316843076, 2.63013526059784, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, -1.06581410364015e-14, -1.06581410364015e-14},
    {0.728431359084679, 0, 0, 2.53908227033119, 0, 0, 0, 0, 0, 5.00713776237767, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.93924068068616, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.2767793639472, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376111, 0, 0, 4.39781949681096, 0, 0, 0, 0, 1.06581410364015e-14, 8.67261700493486, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.0191174248973, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.8717007662564, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    static const double dmats2[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.82574185835056, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.16397779494323, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.95803989154981, 5.61248608016089, -1.08012344973464, -0.763762615825974, 0, 0, 0, 0, 0, 0, 0, 0, -1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.29128784747792, 1.44913767461895, 4.18330013267037, -0.591607978309966, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32287565553229, 0, 3.86436713231718, -0.341565025531991, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338697, 7.09929573971953, 0, 4.34741302385682, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.08012344973463, 1.06581410364015e-14, 7.09929573971953, 2.50998007960221, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-3.81881307912987, 0, 0, 8.87411967464941, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, -1.33226762955019e-14, 0, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.7495454169735, 5.79655069847577, -1.67332005306815, -1.18321595661993, 7.74596669241484, -1.2, 0.346410161513777, -0.979795897113275, 0.282842712474621, 0.200000000000002, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.32379000772445, 2.44948974278317, 2.82842712474618, -1.00000000000001, 1.30930734141595, 7.09929573971954, -2.04939015319192, -0.414039335605415, -0.478091443733755, 0.169030850945703, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.8, 0.632455532033677, 4.38178046004131, -0.774596669241482, 0, 3.14233761939829, 4.76235235991626, -0.106904496764974, -0.740656079818036, 0.130930734141593, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.03923048454133, 0, 3.16227766016837, -0.447213595499959, 0, 0, 5.89188303637179, 0, -0.534522483824841, 0.0755928946018432, 0, -1.24344978758018e-14, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.0124611797498, 2.12132034355962, -0.408248290463868, 3.17542648054291, 9.07114735222146, 0, 0, 7.17137165600636, -1.38013111868471, -1.56144011671765, -1.06581410364015e-14, 0, 0, 0, 0, 0, -2.33146835171283e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.11022302462516e-14, 0, 0, 0},
    {1.55884572681199, 0.547722557505173, 1.58113883008419, 2.45967477524976, 0, 9.07114735222146, 0, 1.8516401995451, 5.34522483824849, -1.20948631362953, 0, 0, 1.95399252334028e-14, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.02140518265514e-14, 0, 0, 0, 0, 0, 0, 0},
    {0.899999999999999, 1.4210854715202e-14, 1.46059348668045, 1.42009389360938, 0, 0, 9.07114735222145, 0, 4.93770719878694, -0.698297248755174, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.27279220613579, -6.2609903369994, 0, 3.83405790253615, 0, 0, 0, 10.5830052442584, 0, 5.18459255872628, 0, 1.06581410364015e-14, 1.95399252334028e-14, 0, 0, 0, 0, 1.06581410364015e-14, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.734846922834958, 0, -6.26099033699939, 2.21359436211786, 0, 0, 0, 0, 10.5830052442584, 2.99332590941916, 0, 0, 0, 0, 0, 0, 0, 0, 1.68753899743024e-14, 1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.7157676649773, 0, 0, -4.69574275274955, 0, 0, 0, 0, 0, 12.69960629311, 1.77635683940025e-14, 0, -1.33226762955019e-14, 0, -1.28785870856518e-14, 0, 2.79776202205539e-14, 0, 1.06581410364015e-14, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.49880108324396e-14, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0},
    {2.56904651573302, 5.80288457473996, -2.01017818278147, -1.42141062443803, 8.14215222512716, -2.10229466468021, 0.728256234341419, -1.71651640581399, 0.594618725379072, 0.420458932936049, 9.81070843517429, -1.18450885369835, 0.30583887092059, -0.105945692672798, -1.02581475831037, 0.264864231681985, -0.0917516612761808, 0.216260739578389, -0.0749149177264375, -0.0529728463363888, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239553, 3.07059789431495, 1.77281052085583, -1.25356634105602, 2.39356776939085, 6.30376871403131, -2.78313372537745, -0.908295106229252, -0.524404424085074, 0.370809924354786, 1.23603308118261, 9.40174755792014, -2.42752078113807, 0.840917865872084, -0.301561193523235, -0.794200206656964, 0.350641890608822, 0.114434427054266, 0.0660687472643433, -0.0467176592151159, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551268, 1.29756443747036, 3.74574588632196, -1.05945692672795, 0.404586796856345, 5.01426536422408, 1.80936716113937, -0.38382473678937, -1.10800657553489, 0.313391585264002, -1.06581410364015e-14, 2.82521847127454, 8.20651806648291, -2.84282124887606, -0.0509731451534328, -0.631738055305796, -0.227958835180075, 0.0483573714561643, 0.139595707137599, -0.0394836284566149, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 0.335029697130249, 4.06201920231797, -0.820651806648284, 0, 1.69926454679665, 5.88642506110456, -0.0991031208965145, -1.20156148406978, 0.242752078113805, 0, 0, 4.94413232473045, 5.13809303146606, 0, -0.214087209644424, -0.741619848709568, 0.0124858196210687, 0.151382517704866, -0.0305838870920656, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.856348838577684, 0, 2.68023757704195, -0.473803541479342, 0, 0, 5.09779364038993, 0, -0.792824967172093, 0.140152977645347, 0, -1.4210854715202e-14, 0, 7.91061171956872, 0, 0, -0.642261628933261, 0, 0.0998865569685847, -0.0176576154454708, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047609, 2.74642624930237, -0.792824967172094, 2.24244764232555, 2.14087209644419, -0.331662479035534, 0.0957427107756326, 6.49923072370876, -1.87616630392937, -2.15580611373101, 11.0554159678513, 0, 0, 0, 9.4403585933147, -1.46249406456535, 0.422185670932516, -2.04706526287665, 0.590936840285272, 0.522319308773344, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.71269767715536, 1.160576914948, 1.34011878852098, 1.89521416591736, 0.361873432227874, 1.96214168703486, -0.566421515598876, 2.74642624930237, 3.17129986868837, -1.82198870938951, 0, 11.0554159678513, 0, 0, 1.59571184626055, 8.6522315682783, -2.49768411251825, -0.865042958313561, -0.998865569685869, 0.441440386136648, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.04360964314765e-14, 1.95399252334028e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32664991614215, 0.299659670905754, 2.07610309995255, 1.46802658041887, 0, 0.868496237346892, 1.31624465810881, 0.709124208342329, 4.91295663090393, -1.4113063856887, 0, 0, 11.0554159678513, 0, 0, 3.82970843102537, 5.80409338312195, -0.223353131420178, -1.54743588659734, 0.341938252770122, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.76594168620508, 0, 1.49829835452878, 0.847565541382357, 0, 0, 1.62843044502542, 0, 3.54562104171167, -0.814818121686407, 0, 0, 0, 11.0554159678514, 0, 1.06581410364015e-14, 7.18070330817255, 0, -1.11676565710083, 0.197418142283059, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 1.00508909139074, -0.193429485824666, 3.55615782880927, -8.46157280212812, 0, 0, 4.459640440343, -0.858258202906995, 2.67027285925187, 0, -1.29340982368831e-14, 0, 0, 12.4373429638327, 0, 0, 8.42792824422443, -1.62195554683794, -2.29379153190448, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.1489125293076, 0.259512887494068, 0.749149177264412, 2.75458800949266, -1.24344978758018e-14, -8.46157280212812, 0, 1.1514742103681, 3.32401972660468, 2.06838446274244, 0, 0, 0, 1.24344978758018e-14, 0, 12.4373429638328, -2.22044604925031e-14, 2.17608171552749, 6.2818068211921, -1.77676328054753, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.663324958071075, 0, 0.692034366650866, 1.59036212878711, 0, 0, -8.46157280212812, 0, 3.07059789431494, 1.19418232635199, 0, 0, -1.4210854715202e-14, 0, 0, 1.50990331349021e-14, 12.4373429638327, 0, 5.80288457473998, -1.02581475831035, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.37667655053519e-14, 0, 0, 0, 0, 0, 0, 0},
    {0.938083151964683, 7.62808987244124, 0, 3.11415464992887, 0, 0, 0, -7.5213980463361, 0, 5.52707620976692, 0, 0, 0, 0, 0, 0, 2.39808173319034e-14, 14.2141062443803, 1.59872115546023e-14, 5.80288457473999, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.541602560309058, 0, 7.62808987244124, 1.79795802543455, 0, 0, 0, 0, -7.5213980463361, 3.19105893754051, 0, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 14.2141062443803, 3.35029697130246, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-5.36159180343551, 0, 0, 8.89943818451482, 0, 0, 0, 0, 1.4210854715202e-14, -5.26497863243527, 0, 0, 0, 0, 1.4210854715202e-14, 0, 1.01585406753202e-14, -1.4210854715202e-14, -1.06581410364015e-14, 16.583123951777, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14},
    {2.41593350361254, 5.72988941019469, -2.20543546226846, -1.55947837083932, 8.33749895885385, -2.76779953857108, 1.06532653910463, -2.25989885993663, 0.869835476750495, 0.615066564126917, 10.333118277332, -2.18326971917505, 0.724780051083766, -0.27896797173099, -1.89076704011891, 0.627677936394712, -0.24159335036126, 0.512496888990059, -0.197260144544837, -0.139483985865486, 11.8490505948789, -1.16109782023228, 0.245326690731332, -0.0814411017927985, 0.031346694695451, -1.03851746182648, 0.21942686286813, -0.0728431359084674, 0.0280373360835801, 0.190029237516525, -0.063084006188054, 0.0242810453028244, -0.0515078753637721, 0.0198253904711043, 0.0140186680417933, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.18529407725405, 3.45525332069938, 0.997445717412057, -1.41060126129511, 3.23209352764252, 5.28531208250977, -2.89087233497829, -1.36277028773849, -0.393397896234717, 0.556348640264194, 2.33666428910958, 9.02784807228868, -3.93352933905729, 1.64018416088945, -0.732969916135168, -1.19859611757306, 0.655588223176204, 0.309047252182627, 0.0892142571199752, -0.126168012376108, 1.1908743922773, 11.55277744383, -2.44096975264614, 0.810328731479533, -0.311895674167854, -0.234843597212092, -0.907332870306512, 0.395334573318419, -0.164844710568313, 0.0736662483163242, 0.120463442338381, -0.0658890955530682, -0.0310604175145517, -0.00896637020658309, 0.0126803623514131, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.92724822331887, 1.82834820066013, 2.99085908983323, -1.2440333788203, 0.950146187582614, 5.7406445631131, -0.169967317119762, -0.721110255092803, -1.1796107330308, 0.490653381462659, 0.294392028877592, 4.9761335152812, 5.78174466995659, -3.22682211353229, -0.215472901842833, -1.30185581821819, 0.0385449644663778, 0.163532434528617, 0.267510569439617, -0.111269728052837, 0, 2.64638753839399, 10.7636648722209, -3.57321383950879, 1.37533064807499, -0.0295875121532885, -0.500120235663637, -0.581087203147972, 0.324307824721458, 0.0216558414515666, 0.130841432732452, -0.00387391468765502, -0.016435628072264, -0.0268858238268355, 0.0111830284386731, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.62882203585592, 0.772618130456575, 3.86595114186567, -1.05140010313427, 0.160604018609906, 3.35889012281481, 3.878512233171, -0.304724700110032, -1.52475169285615, 0.414677792940099, 0, 1.135512111385, 7.32969916135161, 0.564240504518033, -0.0364215679542301, -0.761724681099366, -0.879563899362179, 0.0691050664139837, 0.34578118203618, -0.0940400840863451, 0, 0, 4.46577897103986, 9.05975063854702, -3.48709964663735, 0, -0.114123261162675, -0.736662483163209, -0.0567083043941503, 0.00366050530850437, 0.0765562109321125, 0.0883994979795846, -0.00694532049720609, -0.0347523163751409, 0.00945138406569246, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376112, 0.199489143482424, 3.68560354207932, -0.814411017927961, 0, 1.02786571910339, 6.00856531485322, -0.078679579246943, -1.45362164025439, 0.321208037219817, 0, 0, 3.0280322970267, 6.99294104721296, 0, -0.233098034907105, -1.36261453366201, 0.0178428514239979, 0.329650402328133, -0.0728431359084714, 0, 0, 0, 6.80499652729884, 5.40219154319687, 0, 0, -0.304328696433807, -0.702817019232741, 0, 0.0234272339744261, 0.136947913395214, -0.00179327404131646, -0.0331311120155191, 0.00732101061700707, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.72843135908468, 0, 2.32737334062817, -0.470200420431707, 0, 0, 4.49691252107735, 0, -0.91792842454769, 0.185449546754732, 0, 0, 1.77635683940025e-14, 7.0654086930623, 0, 0, -1.01980390271858, 0, 0.208166599946617, -0.0420560041253766, 0, 0, 0, 0, 9.92395326897747, 0, 0, 0, -0.710100291678885, 0, 0, 0.102494148638109, 0, -0.02092153048203, 0.00422678745046823, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.99489143482414, 3.1542003094028, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275054, 0.263899331455879, 5.59815020469134, -1.93925611658551, -2.28543525082517, 2.13307290077014, -0.257539376818855, 0.0664963811608086, -0.023035022137995, 9.14446135479766, -2.36108976914458, 0.817905488277891, -3.3384229850651, 1.15646364545769, 1.02505848514082, 13.0453753561106, 0, 0, 0, 0, 11.5766222110848, -1.39771878815471, 0.360889439280295, -0.125015768949702, -2.15192884958005, 0.555625639776459, -0.192474367616154, 0.595437196138649, -0.206265495285697, -0.166687691932934, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.7593288763725, 1.66904592079255, 0.963624111659421, 1.36277028773849, 0.867360833110883, 2.28430636201793, -1.00852844760553, 2.96226264871971, 1.71026313764871, -2.01556443707463, 0.268741924943274, 2.04415543160775, -0.52779866291175, 0.182834820066012, 2.68821895760393, 7.07977053255619, -3.12574097035514, -1.76652739792483, -1.01990506872276, 0.904016610326405, 0, 13.0453753561106, 0, 0, 1.77635683940025e-14, 1.4585173044132, 11.0940489487798, -2.8644711213756, 0.992281903807275, -0.632607619454901, -1.66605356688627, 0.735567892896449, 0.31507574846809, 0.181909068193172, -0.147004726489962, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, -1.33226762955019e-14, 0},
    {1.48690428533297, 0.705300630647553, 2.03602754481991, 1.15175110689979, 0.146610739697709, 1.817027050318, 0.655663160391203, 1.25178444059441, 3.61359041872286, -1.70346285967311, 0, 0.614267257013221, 1.78428514239955, -0.618094504365256, 0.454391937932176, 5.63152770643899, 2.03210252328836, -0.746494073225344, -2.15494277062551, 0.764033484562614, 0, 0, 13.0453753561106, 0, 0, 0, 3.33375383865873, 9.68367982311023, -3.3545250915713, -0.106930204231195, -1.32524448062076, -0.478206411017788, 0.133143804689793, 0.384353057392904, -0.124241670058193, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.37667655053519e-14, 0},
    {1.15175110689979, 0.182107839771145, 2.20794021658194, 0.892142571199776, 0, 0.61576510673037, 2.13307290077016, 0.323209352764256, 3.91870514328395, -1.31949665727936, 0, -1.06581410364015e-14, 1.07496769977314, 1.11713920350151, 0, 1.90844613931508, 6.61105135360481, -0.192743940909633, -2.33689608954559, 0.591817792331167, 0, 0, 0, 13.0453753561106, 0, 0, 0, 5.83406921765278, 6.06294257990893, 0, -0.44910685777103, -1.55575179137407, 0.0343775825476159, 0.41680603729705, -0.0962371838080759, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 0},
    {0.66496381160804, 0, 1.45686271816934, 0.515078753637713, 0, -1.24344978758018e-14, 1.84729532019112, 0, 2.58567482211401, -0.761811750275046, 1.4210854715202e-14, -1.24344978758018e-14, 0, 1.71994831963701, 0, -1.4210854715202e-14, 5.72533841794525, 0, -1.54195152727693, 0.341686161713598, 0, 0, 0, 0, 13.0453753561106, 0, 0, 0, 9.33451074824443, 0, 0, -1.34732057331309, 0, 0.27502066038093, -0.0555625639776447, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0},
    {1.57359158493889, 1.49284005458435, -0.430945803685675, 3.04724700110023, 0.775791113542727, -0.120185042515474, 0.0346944333244412, 5.2990565197967, -1.52970585407783, 0.901387818865993, -10.5762837413611, 0, 0, 0, 4.5156107826894, -0.69955541437243, 0.201944253400492, 6.36462844462228, -1.83730997289733, -2.74825341360596, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 0, 10.8920630294282, -1.68739114875268, 0.487107866980279, -3.09993548319961, 0.894874292847885, 0.903959543974656, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32992762321608, 0.63084006188056, 0.728431359084682, 2.57539376818856, 0.131132632078239, 0.71102430025671, -0.205255035576787, 2.23926008187652, 2.58567482211401, 0.761811750275053, 0, -10.576283741361, 0, 0, 0.763277533137579, 4.1386256441266, -1.19471831485579, 2.68954640486935, 3.1056206816986, -2.32269806558125, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 1.84109468241849, 9.98274066131697, -2.88176900403079, -1.30996183150502, -1.51261363209512, 0.763985254692621, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.28785870856518e-14, 0, 0},
    {1.03015750727542, 0.162882203585586, 1.1284810090361, 1.99489143482414, 0, 0.314718316987764, 0.476969600708468, 0.57817446699566, 4.00571020990214, 0.590096844352083, 0, 0, -10.5762837413611, 0, 0, 1.83186607953019, 2.77627448210727, 0.694437895660723, 4.81120687194241, -1.79915418525285, 0, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 4.41862723780437, 6.69662601613678, -0.338230690505752, -2.34333096254029, 0.591780433634522, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.14352971536391e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.594761714133174, -1.19904086659517e-14, 0.814411017927931, 1.15175110689982, 0, 0, 0.590096844352076, 0, 2.89087233497831, 0.340692571934628, 0, 0, 0, -10.576283741361, 0, 0, 3.43474889911912, -1.24344978758018e-14, 3.47218947830365, -1.03874215316938, 0, 0, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 8.28492607088319, 0, -1.69115345252878, 0.341664592660043, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.68753899743024e-14, 0},
    {1.15175110689979, 0.546323519313511, -0.105140010313427, 3.27118942773251, 9.65295620481434, 0, 0, 2.69341127303542, -0.518347241175114, 4.25171145123353, 0, 0, 0, 0, -9.91526600752598, 0, 1.53210777398272e-14, 6.71889089790563, -1.29305115618718, 1.82865048192216, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.9443476574685, 0, 0, 9.45383520059451, -1.81939143264587, -2.94058061635793, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 1.68753899743024e-14},
    {0.892142571199775, 0.141060126129508, 0.407205508963975, 2.53385243517955, 0, 9.65295620481433, 0, 0.69543580033025, 2.00755023262383, 3.29336152870135, 0, 0, 0, 0, 0, -9.91526600752598, 2.12052597703405e-14, 1.73481017017154, 5.0079655937072, 1.41646657250371, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.9443476574685, 0, 2.44096975264614, 7.0464727188699, -2.27776395106693, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.515078753637709, 0, 0.376160336345346, 1.46292038553773, 0, 0, 9.65295620481433, 0, 1.8544954675473, 1.9014231651345, 0, 0, 0, -1.06581410364015e-14, 0, 0, -9.91526600752595, 0, 4.62616045379076, 0.81779735693317, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.9443476574685, 0, 6.50925267372302, -1.3150676302989, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 1.86517468137026e-14},
    {0.728431359084674, -7.94708263760857, 0, 2.53908227033117, 0, 0, 0, 10.2207771580387, 0, 5.00713776237767, 0, 0, 0, 0, -1.33226762955019e-14, 2.1316282072803e-14, 0, -8.49879943502227, -1.01030295240889e-14, 6.93924068068615, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 17.9373911146521, 0, 6.2767793639472, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0},
    {0.42056004125371, 0, -7.94708263760858, 1.46593983227034, 0, 0, 0, 0, 10.2207771580387, 2.89087233497828, 0, 0, 0, -1.06581410364015e-14, 2.1316282072803e-14, -1.77635683940025e-14, 2.62012633811537e-14, 0, -8.49879943502228, 4.00637247496576, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 17.9373911146521, 3.6239002554188, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.22124532708767e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {6.54237885546499, 0, 1.50990331349021e-14, -6.91050664139879, 0, 0, 0, 0, 0, 12.2649325896464, 0, -1.79856129989275e-14, 1.77635683940025e-14, 1.33226762955019e-14, 0, 0, -1.75415237890775e-14, 0, 1.06581410364015e-14, -5.66586629001485, 0, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 20.4998755596024, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.46549439250521e-14, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, -1.59872115546023e-14}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    double coeff0_6 = 0;
    double coeff0_7 = 0;
    double coeff0_8 = 0;
    double coeff0_9 = 0;
    double coeff0_10 = 0;
    double coeff0_11 = 0;
    double coeff0_12 = 0;
    double coeff0_13 = 0;
    double coeff0_14 = 0;
    double coeff0_15 = 0;
    double coeff0_16 = 0;
    double coeff0_17 = 0;
    double coeff0_18 = 0;
    double coeff0_19 = 0;
    double coeff0_20 = 0;
    double coeff0_21 = 0;
    double coeff0_22 = 0;
    double coeff0_23 = 0;
    double coeff0_24 = 0;
    double coeff0_25 = 0;
    double coeff0_26 = 0;
    double coeff0_27 = 0;
    double coeff0_28 = 0;
    double coeff0_29 = 0;
    double coeff0_30 = 0;
    double coeff0_31 = 0;
    double coeff0_32 = 0;
    double coeff0_33 = 0;
    double coeff0_34 = 0;
    double coeff0_35 = 0;
    double coeff0_36 = 0;
    double coeff0_37 = 0;
    double coeff0_38 = 0;
    double coeff0_39 = 0;
    double coeff0_40 = 0;
    double coeff0_41 = 0;
    double coeff0_42 = 0;
    double coeff0_43 = 0;
    double coeff0_44 = 0;
    double coeff0_45 = 0;
    double coeff0_46 = 0;
    double coeff0_47 = 0;
    double coeff0_48 = 0;
    double coeff0_49 = 0;
    double coeff0_50 = 0;
    double coeff0_51 = 0;
    double coeff0_52 = 0;
    double coeff0_53 = 0;
    double coeff0_54 = 0;
    double coeff0_55 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    double new_coeff0_6 = 0;
    double new_coeff0_7 = 0;
    double new_coeff0_8 = 0;
    double new_coeff0_9 = 0;
    double new_coeff0_10 = 0;
    double new_coeff0_11 = 0;
    double new_coeff0_12 = 0;
    double new_coeff0_13 = 0;
    double new_coeff0_14 = 0;
    double new_coeff0_15 = 0;
    double new_coeff0_16 = 0;
    double new_coeff0_17 = 0;
    double new_coeff0_18 = 0;
    double new_coeff0_19 = 0;
    double new_coeff0_20 = 0;
    double new_coeff0_21 = 0;
    double new_coeff0_22 = 0;
    double new_coeff0_23 = 0;
    double new_coeff0_24 = 0;
    double new_coeff0_25 = 0;
    double new_coeff0_26 = 0;
    double new_coeff0_27 = 0;
    double new_coeff0_28 = 0;
    double new_coeff0_29 = 0;
    double new_coeff0_30 = 0;
    double new_coeff0_31 = 0;
    double new_coeff0_32 = 0;
    double new_coeff0_33 = 0;
    double new_coeff0_34 = 0;
    double new_coeff0_35 = 0;
    double new_coeff0_36 = 0;
    double new_coeff0_37 = 0;
    double new_coeff0_38 = 0;
    double new_coeff0_39 = 0;
    double new_coeff0_40 = 0;
    double new_coeff0_41 = 0;
    double new_coeff0_42 = 0;
    double new_coeff0_43 = 0;
    double new_coeff0_44 = 0;
    double new_coeff0_45 = 0;
    double new_coeff0_46 = 0;
    double new_coeff0_47 = 0;
    double new_coeff0_48 = 0;
    double new_coeff0_49 = 0;
    double new_coeff0_50 = 0;
    double new_coeff0_51 = 0;
    double new_coeff0_52 = 0;
    double new_coeff0_53 = 0;
    double new_coeff0_54 = 0;
    double new_coeff0_55 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
      new_coeff0_6 = coefficients0[dof][6];
      new_coeff0_7 = coefficients0[dof][7];
      new_coeff0_8 = coefficients0[dof][8];
      new_coeff0_9 = coefficients0[dof][9];
      new_coeff0_10 = coefficients0[dof][10];
      new_coeff0_11 = coefficients0[dof][11];
      new_coeff0_12 = coefficients0[dof][12];
      new_coeff0_13 = coefficients0[dof][13];
      new_coeff0_14 = coefficients0[dof][14];
      new_coeff0_15 = coefficients0[dof][15];
      new_coeff0_16 = coefficients0[dof][16];
      new_coeff0_17 = coefficients0[dof][17];
      new_coeff0_18 = coefficients0[dof][18];
      new_coeff0_19 = coefficients0[dof][19];
      new_coeff0_20 = coefficients0[dof][20];
      new_coeff0_21 = coefficients0[dof][21];
      new_coeff0_22 = coefficients0[dof][22];
      new_coeff0_23 = coefficients0[dof][23];
      new_coeff0_24 = coefficients0[dof][24];
      new_coeff0_25 = coefficients0[dof][25];
      new_coeff0_26 = coefficients0[dof][26];
      new_coeff0_27 = coefficients0[dof][27];
      new_coeff0_28 = coefficients0[dof][28];
      new_coeff0_29 = coefficients0[dof][29];
      new_coeff0_30 = coefficients0[dof][30];
      new_coeff0_31 = coefficients0[dof][31];
      new_coeff0_32 = coefficients0[dof][32];
      new_coeff0_33 = coefficients0[dof][33];
      new_coeff0_34 = coefficients0[dof][34];
      new_coeff0_35 = coefficients0[dof][35];
      new_coeff0_36 = coefficients0[dof][36];
      new_coeff0_37 = coefficients0[dof][37];
      new_coeff0_38 = coefficients0[dof][38];
      new_coeff0_39 = coefficients0[dof][39];
      new_coeff0_40 = coefficients0[dof][40];
      new_coeff0_41 = coefficients0[dof][41];
      new_coeff0_42 = coefficients0[dof][42];
      new_coeff0_43 = coefficients0[dof][43];
      new_coeff0_44 = coefficients0[dof][44];
      new_coeff0_45 = coefficients0[dof][45];
      new_coeff0_46 = coefficients0[dof][46];
      new_coeff0_47 = coefficients0[dof][47];
      new_coeff0_48 = coefficients0[dof][48];
      new_coeff0_49 = coefficients0[dof][49];
      new_coeff0_50 = coefficients0[dof][50];
      new_coeff0_51 = coefficients0[dof][51];
      new_coeff0_52 = coefficients0[dof][52];
      new_coeff0_53 = coefficients0[dof][53];
      new_coeff0_54 = coefficients0[dof][54];
      new_coeff0_55 = coefficients0[dof][55];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
        coeff0_6 = new_coeff0_6;
        coeff0_7 = new_coeff0_7;
        coeff0_8 = new_coeff0_8;
        coeff0_9 = new_coeff0_9;
        coeff0_10 = new_coeff0_10;
        coeff0_11 = new_coeff0_11;
        coeff0_12 = new_coeff0_12;
        coeff0_13 = new_coeff0_13;
        coeff0_14 = new_coeff0_14;
        coeff0_15 = new_coeff0_15;
        coeff0_16 = new_coeff0_16;
        coeff0_17 = new_coeff0_17;
        coeff0_18 = new_coeff0_18;
        coeff0_19 = new_coeff0_19;
        coeff0_20 = new_coeff0_20;
        coeff0_21 = new_coeff0_21;
        coeff0_22 = new_coeff0_22;
        coeff0_23 = new_coeff0_23;
        coeff0_24 = new_coeff0_24;
        coeff0_25 = new_coeff0_25;
        coeff0_26 = new_coeff0_26;
        coeff0_27 = new_coeff0_27;
        coeff0_28 = new_coeff0_28;
        coeff0_29 = new_coeff0_29;
        coeff0_30 = new_coeff0_30;
        coeff0_31 = new_coeff0_31;
        coeff0_32 = new_coeff0_32;
        coeff0_33 = new_coeff0_33;
        coeff0_34 = new_coeff0_34;
        coeff0_35 = new_coeff0_35;
        coeff0_36 = new_coeff0_36;
        coeff0_37 = new_coeff0_37;
        coeff0_38 = new_coeff0_38;
        coeff0_39 = new_coeff0_39;
        coeff0_40 = new_coeff0_40;
        coeff0_41 = new_coeff0_41;
        coeff0_42 = new_coeff0_42;
        coeff0_43 = new_coeff0_43;
        coeff0_44 = new_coeff0_44;
        coeff0_45 = new_coeff0_45;
        coeff0_46 = new_coeff0_46;
        coeff0_47 = new_coeff0_47;
        coeff0_48 = new_coeff0_48;
        coeff0_49 = new_coeff0_49;
        coeff0_50 = new_coeff0_50;
        coeff0_51 = new_coeff0_51;
        coeff0_52 = new_coeff0_52;
        coeff0_53 = new_coeff0_53;
        coeff0_54 = new_coeff0_54;
        coeff0_55 = new_coeff0_55;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0] + coeff0_10*dmats0[10][0] + coeff0_11*dmats0[11][0] + coeff0_12*dmats0[12][0] + coeff0_13*dmats0[13][0] + coeff0_14*dmats0[14][0] + coeff0_15*dmats0[15][0] + coeff0_16*dmats0[16][0] + coeff0_17*dmats0[17][0] + coeff0_18*dmats0[18][0] + coeff0_19*dmats0[19][0] + coeff0_20*dmats0[20][0] + coeff0_21*dmats0[21][0] + coeff0_22*dmats0[22][0] + coeff0_23*dmats0[23][0] + coeff0_24*dmats0[24][0] + coeff0_25*dmats0[25][0] + coeff0_26*dmats0[26][0] + coeff0_27*dmats0[27][0] + coeff0_28*dmats0[28][0] + coeff0_29*dmats0[29][0] + coeff0_30*dmats0[30][0] + coeff0_31*dmats0[31][0] + coeff0_32*dmats0[32][0] + coeff0_33*dmats0[33][0] + coeff0_34*dmats0[34][0] + coeff0_35*dmats0[35][0] + coeff0_36*dmats0[36][0] + coeff0_37*dmats0[37][0] + coeff0_38*dmats0[38][0] + coeff0_39*dmats0[39][0] + coeff0_40*dmats0[40][0] + coeff0_41*dmats0[41][0] + coeff0_42*dmats0[42][0] + coeff0_43*dmats0[43][0] + coeff0_44*dmats0[44][0] + coeff0_45*dmats0[45][0] + coeff0_46*dmats0[46][0] + coeff0_47*dmats0[47][0] + coeff0_48*dmats0[48][0] + coeff0_49*dmats0[49][0] + coeff0_50*dmats0[50][0] + coeff0_51*dmats0[51][0] + coeff0_52*dmats0[52][0] + coeff0_53*dmats0[53][0] + coeff0_54*dmats0[54][0] + coeff0_55*dmats0[55][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1] + coeff0_10*dmats0[10][1] + coeff0_11*dmats0[11][1] + coeff0_12*dmats0[12][1] + coeff0_13*dmats0[13][1] + coeff0_14*dmats0[14][1] + coeff0_15*dmats0[15][1] + coeff0_16*dmats0[16][1] + coeff0_17*dmats0[17][1] + coeff0_18*dmats0[18][1] + coeff0_19*dmats0[19][1] + coeff0_20*dmats0[20][1] + coeff0_21*dmats0[21][1] + coeff0_22*dmats0[22][1] + coeff0_23*dmats0[23][1] + coeff0_24*dmats0[24][1] + coeff0_25*dmats0[25][1] + coeff0_26*dmats0[26][1] + coeff0_27*dmats0[27][1] + coeff0_28*dmats0[28][1] + coeff0_29*dmats0[29][1] + coeff0_30*dmats0[30][1] + coeff0_31*dmats0[31][1] + coeff0_32*dmats0[32][1] + coeff0_33*dmats0[33][1] + coeff0_34*dmats0[34][1] + coeff0_35*dmats0[35][1] + coeff0_36*dmats0[36][1] + coeff0_37*dmats0[37][1] + coeff0_38*dmats0[38][1] + coeff0_39*dmats0[39][1] + coeff0_40*dmats0[40][1] + coeff0_41*dmats0[41][1] + coeff0_42*dmats0[42][1] + coeff0_43*dmats0[43][1] + coeff0_44*dmats0[44][1] + coeff0_45*dmats0[45][1] + coeff0_46*dmats0[46][1] + coeff0_47*dmats0[47][1] + coeff0_48*dmats0[48][1] + coeff0_49*dmats0[49][1] + coeff0_50*dmats0[50][1] + coeff0_51*dmats0[51][1] + coeff0_52*dmats0[52][1] + coeff0_53*dmats0[53][1] + coeff0_54*dmats0[54][1] + coeff0_55*dmats0[55][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2] + coeff0_10*dmats0[10][2] + coeff0_11*dmats0[11][2] + coeff0_12*dmats0[12][2] + coeff0_13*dmats0[13][2] + coeff0_14*dmats0[14][2] + coeff0_15*dmats0[15][2] + coeff0_16*dmats0[16][2] + coeff0_17*dmats0[17][2] + coeff0_18*dmats0[18][2] + coeff0_19*dmats0[19][2] + coeff0_20*dmats0[20][2] + coeff0_21*dmats0[21][2] + coeff0_22*dmats0[22][2] + coeff0_23*dmats0[23][2] + coeff0_24*dmats0[24][2] + coeff0_25*dmats0[25][2] + coeff0_26*dmats0[26][2] + coeff0_27*dmats0[27][2] + coeff0_28*dmats0[28][2] + coeff0_29*dmats0[29][2] + coeff0_30*dmats0[30][2] + coeff0_31*dmats0[31][2] + coeff0_32*dmats0[32][2] + coeff0_33*dmats0[33][2] + coeff0_34*dmats0[34][2] + coeff0_35*dmats0[35][2] + coeff0_36*dmats0[36][2] + coeff0_37*dmats0[37][2] + coeff0_38*dmats0[38][2] + coeff0_39*dmats0[39][2] + coeff0_40*dmats0[40][2] + coeff0_41*dmats0[41][2] + coeff0_42*dmats0[42][2] + coeff0_43*dmats0[43][2] + coeff0_44*dmats0[44][2] + coeff0_45*dmats0[45][2] + coeff0_46*dmats0[46][2] + coeff0_47*dmats0[47][2] + coeff0_48*dmats0[48][2] + coeff0_49*dmats0[49][2] + coeff0_50*dmats0[50][2] + coeff0_51*dmats0[51][2] + coeff0_52*dmats0[52][2] + coeff0_53*dmats0[53][2] + coeff0_54*dmats0[54][2] + coeff0_55*dmats0[55][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3] + coeff0_10*dmats0[10][3] + coeff0_11*dmats0[11][3] + coeff0_12*dmats0[12][3] + coeff0_13*dmats0[13][3] + coeff0_14*dmats0[14][3] + coeff0_15*dmats0[15][3] + coeff0_16*dmats0[16][3] + coeff0_17*dmats0[17][3] + coeff0_18*dmats0[18][3] + coeff0_19*dmats0[19][3] + coeff0_20*dmats0[20][3] + coeff0_21*dmats0[21][3] + coeff0_22*dmats0[22][3] + coeff0_23*dmats0[23][3] + coeff0_24*dmats0[24][3] + coeff0_25*dmats0[25][3] + coeff0_26*dmats0[26][3] + coeff0_27*dmats0[27][3] + coeff0_28*dmats0[28][3] + coeff0_29*dmats0[29][3] + coeff0_30*dmats0[30][3] + coeff0_31*dmats0[31][3] + coeff0_32*dmats0[32][3] + coeff0_33*dmats0[33][3] + coeff0_34*dmats0[34][3] + coeff0_35*dmats0[35][3] + coeff0_36*dmats0[36][3] + coeff0_37*dmats0[37][3] + coeff0_38*dmats0[38][3] + coeff0_39*dmats0[39][3] + coeff0_40*dmats0[40][3] + coeff0_41*dmats0[41][3] + coeff0_42*dmats0[42][3] + coeff0_43*dmats0[43][3] + coeff0_44*dmats0[44][3] + coeff0_45*dmats0[45][3] + coeff0_46*dmats0[46][3] + coeff0_47*dmats0[47][3] + coeff0_48*dmats0[48][3] + coeff0_49*dmats0[49][3] + coeff0_50*dmats0[50][3] + coeff0_51*dmats0[51][3] + coeff0_52*dmats0[52][3] + coeff0_53*dmats0[53][3] + coeff0_54*dmats0[54][3] + coeff0_55*dmats0[55][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4] + coeff0_10*dmats0[10][4] + coeff0_11*dmats0[11][4] + coeff0_12*dmats0[12][4] + coeff0_13*dmats0[13][4] + coeff0_14*dmats0[14][4] + coeff0_15*dmats0[15][4] + coeff0_16*dmats0[16][4] + coeff0_17*dmats0[17][4] + coeff0_18*dmats0[18][4] + coeff0_19*dmats0[19][4] + coeff0_20*dmats0[20][4] + coeff0_21*dmats0[21][4] + coeff0_22*dmats0[22][4] + coeff0_23*dmats0[23][4] + coeff0_24*dmats0[24][4] + coeff0_25*dmats0[25][4] + coeff0_26*dmats0[26][4] + coeff0_27*dmats0[27][4] + coeff0_28*dmats0[28][4] + coeff0_29*dmats0[29][4] + coeff0_30*dmats0[30][4] + coeff0_31*dmats0[31][4] + coeff0_32*dmats0[32][4] + coeff0_33*dmats0[33][4] + coeff0_34*dmats0[34][4] + coeff0_35*dmats0[35][4] + coeff0_36*dmats0[36][4] + coeff0_37*dmats0[37][4] + coeff0_38*dmats0[38][4] + coeff0_39*dmats0[39][4] + coeff0_40*dmats0[40][4] + coeff0_41*dmats0[41][4] + coeff0_42*dmats0[42][4] + coeff0_43*dmats0[43][4] + coeff0_44*dmats0[44][4] + coeff0_45*dmats0[45][4] + coeff0_46*dmats0[46][4] + coeff0_47*dmats0[47][4] + coeff0_48*dmats0[48][4] + coeff0_49*dmats0[49][4] + coeff0_50*dmats0[50][4] + coeff0_51*dmats0[51][4] + coeff0_52*dmats0[52][4] + coeff0_53*dmats0[53][4] + coeff0_54*dmats0[54][4] + coeff0_55*dmats0[55][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5] + coeff0_10*dmats0[10][5] + coeff0_11*dmats0[11][5] + coeff0_12*dmats0[12][5] + coeff0_13*dmats0[13][5] + coeff0_14*dmats0[14][5] + coeff0_15*dmats0[15][5] + coeff0_16*dmats0[16][5] + coeff0_17*dmats0[17][5] + coeff0_18*dmats0[18][5] + coeff0_19*dmats0[19][5] + coeff0_20*dmats0[20][5] + coeff0_21*dmats0[21][5] + coeff0_22*dmats0[22][5] + coeff0_23*dmats0[23][5] + coeff0_24*dmats0[24][5] + coeff0_25*dmats0[25][5] + coeff0_26*dmats0[26][5] + coeff0_27*dmats0[27][5] + coeff0_28*dmats0[28][5] + coeff0_29*dmats0[29][5] + coeff0_30*dmats0[30][5] + coeff0_31*dmats0[31][5] + coeff0_32*dmats0[32][5] + coeff0_33*dmats0[33][5] + coeff0_34*dmats0[34][5] + coeff0_35*dmats0[35][5] + coeff0_36*dmats0[36][5] + coeff0_37*dmats0[37][5] + coeff0_38*dmats0[38][5] + coeff0_39*dmats0[39][5] + coeff0_40*dmats0[40][5] + coeff0_41*dmats0[41][5] + coeff0_42*dmats0[42][5] + coeff0_43*dmats0[43][5] + coeff0_44*dmats0[44][5] + coeff0_45*dmats0[45][5] + coeff0_46*dmats0[46][5] + coeff0_47*dmats0[47][5] + coeff0_48*dmats0[48][5] + coeff0_49*dmats0[49][5] + coeff0_50*dmats0[50][5] + coeff0_51*dmats0[51][5] + coeff0_52*dmats0[52][5] + coeff0_53*dmats0[53][5] + coeff0_54*dmats0[54][5] + coeff0_55*dmats0[55][5];
          new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6] + coeff0_10*dmats0[10][6] + coeff0_11*dmats0[11][6] + coeff0_12*dmats0[12][6] + coeff0_13*dmats0[13][6] + coeff0_14*dmats0[14][6] + coeff0_15*dmats0[15][6] + coeff0_16*dmats0[16][6] + coeff0_17*dmats0[17][6] + coeff0_18*dmats0[18][6] + coeff0_19*dmats0[19][6] + coeff0_20*dmats0[20][6] + coeff0_21*dmats0[21][6] + coeff0_22*dmats0[22][6] + coeff0_23*dmats0[23][6] + coeff0_24*dmats0[24][6] + coeff0_25*dmats0[25][6] + coeff0_26*dmats0[26][6] + coeff0_27*dmats0[27][6] + coeff0_28*dmats0[28][6] + coeff0_29*dmats0[29][6] + coeff0_30*dmats0[30][6] + coeff0_31*dmats0[31][6] + coeff0_32*dmats0[32][6] + coeff0_33*dmats0[33][6] + coeff0_34*dmats0[34][6] + coeff0_35*dmats0[35][6] + coeff0_36*dmats0[36][6] + coeff0_37*dmats0[37][6] + coeff0_38*dmats0[38][6] + coeff0_39*dmats0[39][6] + coeff0_40*dmats0[40][6] + coeff0_41*dmats0[41][6] + coeff0_42*dmats0[42][6] + coeff0_43*dmats0[43][6] + coeff0_44*dmats0[44][6] + coeff0_45*dmats0[45][6] + coeff0_46*dmats0[46][6] + coeff0_47*dmats0[47][6] + coeff0_48*dmats0[48][6] + coeff0_49*dmats0[49][6] + coeff0_50*dmats0[50][6] + coeff0_51*dmats0[51][6] + coeff0_52*dmats0[52][6] + coeff0_53*dmats0[53][6] + coeff0_54*dmats0[54][6] + coeff0_55*dmats0[55][6];
          new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7] + coeff0_10*dmats0[10][7] + coeff0_11*dmats0[11][7] + coeff0_12*dmats0[12][7] + coeff0_13*dmats0[13][7] + coeff0_14*dmats0[14][7] + coeff0_15*dmats0[15][7] + coeff0_16*dmats0[16][7] + coeff0_17*dmats0[17][7] + coeff0_18*dmats0[18][7] + coeff0_19*dmats0[19][7] + coeff0_20*dmats0[20][7] + coeff0_21*dmats0[21][7] + coeff0_22*dmats0[22][7] + coeff0_23*dmats0[23][7] + coeff0_24*dmats0[24][7] + coeff0_25*dmats0[25][7] + coeff0_26*dmats0[26][7] + coeff0_27*dmats0[27][7] + coeff0_28*dmats0[28][7] + coeff0_29*dmats0[29][7] + coeff0_30*dmats0[30][7] + coeff0_31*dmats0[31][7] + coeff0_32*dmats0[32][7] + coeff0_33*dmats0[33][7] + coeff0_34*dmats0[34][7] + coeff0_35*dmats0[35][7] + coeff0_36*dmats0[36][7] + coeff0_37*dmats0[37][7] + coeff0_38*dmats0[38][7] + coeff0_39*dmats0[39][7] + coeff0_40*dmats0[40][7] + coeff0_41*dmats0[41][7] + coeff0_42*dmats0[42][7] + coeff0_43*dmats0[43][7] + coeff0_44*dmats0[44][7] + coeff0_45*dmats0[45][7] + coeff0_46*dmats0[46][7] + coeff0_47*dmats0[47][7] + coeff0_48*dmats0[48][7] + coeff0_49*dmats0[49][7] + coeff0_50*dmats0[50][7] + coeff0_51*dmats0[51][7] + coeff0_52*dmats0[52][7] + coeff0_53*dmats0[53][7] + coeff0_54*dmats0[54][7] + coeff0_55*dmats0[55][7];
          new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8] + coeff0_10*dmats0[10][8] + coeff0_11*dmats0[11][8] + coeff0_12*dmats0[12][8] + coeff0_13*dmats0[13][8] + coeff0_14*dmats0[14][8] + coeff0_15*dmats0[15][8] + coeff0_16*dmats0[16][8] + coeff0_17*dmats0[17][8] + coeff0_18*dmats0[18][8] + coeff0_19*dmats0[19][8] + coeff0_20*dmats0[20][8] + coeff0_21*dmats0[21][8] + coeff0_22*dmats0[22][8] + coeff0_23*dmats0[23][8] + coeff0_24*dmats0[24][8] + coeff0_25*dmats0[25][8] + coeff0_26*dmats0[26][8] + coeff0_27*dmats0[27][8] + coeff0_28*dmats0[28][8] + coeff0_29*dmats0[29][8] + coeff0_30*dmats0[30][8] + coeff0_31*dmats0[31][8] + coeff0_32*dmats0[32][8] + coeff0_33*dmats0[33][8] + coeff0_34*dmats0[34][8] + coeff0_35*dmats0[35][8] + coeff0_36*dmats0[36][8] + coeff0_37*dmats0[37][8] + coeff0_38*dmats0[38][8] + coeff0_39*dmats0[39][8] + coeff0_40*dmats0[40][8] + coeff0_41*dmats0[41][8] + coeff0_42*dmats0[42][8] + coeff0_43*dmats0[43][8] + coeff0_44*dmats0[44][8] + coeff0_45*dmats0[45][8] + coeff0_46*dmats0[46][8] + coeff0_47*dmats0[47][8] + coeff0_48*dmats0[48][8] + coeff0_49*dmats0[49][8] + coeff0_50*dmats0[50][8] + coeff0_51*dmats0[51][8] + coeff0_52*dmats0[52][8] + coeff0_53*dmats0[53][8] + coeff0_54*dmats0[54][8] + coeff0_55*dmats0[55][8];
          new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9] + coeff0_10*dmats0[10][9] + coeff0_11*dmats0[11][9] + coeff0_12*dmats0[12][9] + coeff0_13*dmats0[13][9] + coeff0_14*dmats0[14][9] + coeff0_15*dmats0[15][9] + coeff0_16*dmats0[16][9] + coeff0_17*dmats0[17][9] + coeff0_18*dmats0[18][9] + coeff0_19*dmats0[19][9] + coeff0_20*dmats0[20][9] + coeff0_21*dmats0[21][9] + coeff0_22*dmats0[22][9] + coeff0_23*dmats0[23][9] + coeff0_24*dmats0[24][9] + coeff0_25*dmats0[25][9] + coeff0_26*dmats0[26][9] + coeff0_27*dmats0[27][9] + coeff0_28*dmats0[28][9] + coeff0_29*dmats0[29][9] + coeff0_30*dmats0[30][9] + coeff0_31*dmats0[31][9] + coeff0_32*dmats0[32][9] + coeff0_33*dmats0[33][9] + coeff0_34*dmats0[34][9] + coeff0_35*dmats0[35][9] + coeff0_36*dmats0[36][9] + coeff0_37*dmats0[37][9] + coeff0_38*dmats0[38][9] + coeff0_39*dmats0[39][9] + coeff0_40*dmats0[40][9] + coeff0_41*dmats0[41][9] + coeff0_42*dmats0[42][9] + coeff0_43*dmats0[43][9] + coeff0_44*dmats0[44][9] + coeff0_45*dmats0[45][9] + coeff0_46*dmats0[46][9] + coeff0_47*dmats0[47][9] + coeff0_48*dmats0[48][9] + coeff0_49*dmats0[49][9] + coeff0_50*dmats0[50][9] + coeff0_51*dmats0[51][9] + coeff0_52*dmats0[52][9] + coeff0_53*dmats0[53][9] + coeff0_54*dmats0[54][9] + coeff0_55*dmats0[55][9];
          new_coeff0_10 = coeff0_0*dmats0[0][10] + coeff0_1*dmats0[1][10] + coeff0_2*dmats0[2][10] + coeff0_3*dmats0[3][10] + coeff0_4*dmats0[4][10] + coeff0_5*dmats0[5][10] + coeff0_6*dmats0[6][10] + coeff0_7*dmats0[7][10] + coeff0_8*dmats0[8][10] + coeff0_9*dmats0[9][10] + coeff0_10*dmats0[10][10] + coeff0_11*dmats0[11][10] + coeff0_12*dmats0[12][10] + coeff0_13*dmats0[13][10] + coeff0_14*dmats0[14][10] + coeff0_15*dmats0[15][10] + coeff0_16*dmats0[16][10] + coeff0_17*dmats0[17][10] + coeff0_18*dmats0[18][10] + coeff0_19*dmats0[19][10] + coeff0_20*dmats0[20][10] + coeff0_21*dmats0[21][10] + coeff0_22*dmats0[22][10] + coeff0_23*dmats0[23][10] + coeff0_24*dmats0[24][10] + coeff0_25*dmats0[25][10] + coeff0_26*dmats0[26][10] + coeff0_27*dmats0[27][10] + coeff0_28*dmats0[28][10] + coeff0_29*dmats0[29][10] + coeff0_30*dmats0[30][10] + coeff0_31*dmats0[31][10] + coeff0_32*dmats0[32][10] + coeff0_33*dmats0[33][10] + coeff0_34*dmats0[34][10] + coeff0_35*dmats0[35][10] + coeff0_36*dmats0[36][10] + coeff0_37*dmats0[37][10] + coeff0_38*dmats0[38][10] + coeff0_39*dmats0[39][10] + coeff0_40*dmats0[40][10] + coeff0_41*dmats0[41][10] + coeff0_42*dmats0[42][10] + coeff0_43*dmats0[43][10] + coeff0_44*dmats0[44][10] + coeff0_45*dmats0[45][10] + coeff0_46*dmats0[46][10] + coeff0_47*dmats0[47][10] + coeff0_48*dmats0[48][10] + coeff0_49*dmats0[49][10] + coeff0_50*dmats0[50][10] + coeff0_51*dmats0[51][10] + coeff0_52*dmats0[52][10] + coeff0_53*dmats0[53][10] + coeff0_54*dmats0[54][10] + coeff0_55*dmats0[55][10];
          new_coeff0_11 = coeff0_0*dmats0[0][11] + coeff0_1*dmats0[1][11] + coeff0_2*dmats0[2][11] + coeff0_3*dmats0[3][11] + coeff0_4*dmats0[4][11] + coeff0_5*dmats0[5][11] + coeff0_6*dmats0[6][11] + coeff0_7*dmats0[7][11] + coeff0_8*dmats0[8][11] + coeff0_9*dmats0[9][11] + coeff0_10*dmats0[10][11] + coeff0_11*dmats0[11][11] + coeff0_12*dmats0[12][11] + coeff0_13*dmats0[13][11] + coeff0_14*dmats0[14][11] + coeff0_15*dmats0[15][11] + coeff0_16*dmats0[16][11] + coeff0_17*dmats0[17][11] + coeff0_18*dmats0[18][11] + coeff0_19*dmats0[19][11] + coeff0_20*dmats0[20][11] + coeff0_21*dmats0[21][11] + coeff0_22*dmats0[22][11] + coeff0_23*dmats0[23][11] + coeff0_24*dmats0[24][11] + coeff0_25*dmats0[25][11] + coeff0_26*dmats0[26][11] + coeff0_27*dmats0[27][11] + coeff0_28*dmats0[28][11] + coeff0_29*dmats0[29][11] + coeff0_30*dmats0[30][11] + coeff0_31*dmats0[31][11] + coeff0_32*dmats0[32][11] + coeff0_33*dmats0[33][11] + coeff0_34*dmats0[34][11] + coeff0_35*dmats0[35][11] + coeff0_36*dmats0[36][11] + coeff0_37*dmats0[37][11] + coeff0_38*dmats0[38][11] + coeff0_39*dmats0[39][11] + coeff0_40*dmats0[40][11] + coeff0_41*dmats0[41][11] + coeff0_42*dmats0[42][11] + coeff0_43*dmats0[43][11] + coeff0_44*dmats0[44][11] + coeff0_45*dmats0[45][11] + coeff0_46*dmats0[46][11] + coeff0_47*dmats0[47][11] + coeff0_48*dmats0[48][11] + coeff0_49*dmats0[49][11] + coeff0_50*dmats0[50][11] + coeff0_51*dmats0[51][11] + coeff0_52*dmats0[52][11] + coeff0_53*dmats0[53][11] + coeff0_54*dmats0[54][11] + coeff0_55*dmats0[55][11];
          new_coeff0_12 = coeff0_0*dmats0[0][12] + coeff0_1*dmats0[1][12] + coeff0_2*dmats0[2][12] + coeff0_3*dmats0[3][12] + coeff0_4*dmats0[4][12] + coeff0_5*dmats0[5][12] + coeff0_6*dmats0[6][12] + coeff0_7*dmats0[7][12] + coeff0_8*dmats0[8][12] + coeff0_9*dmats0[9][12] + coeff0_10*dmats0[10][12] + coeff0_11*dmats0[11][12] + coeff0_12*dmats0[12][12] + coeff0_13*dmats0[13][12] + coeff0_14*dmats0[14][12] + coeff0_15*dmats0[15][12] + coeff0_16*dmats0[16][12] + coeff0_17*dmats0[17][12] + coeff0_18*dmats0[18][12] + coeff0_19*dmats0[19][12] + coeff0_20*dmats0[20][12] + coeff0_21*dmats0[21][12] + coeff0_22*dmats0[22][12] + coeff0_23*dmats0[23][12] + coeff0_24*dmats0[24][12] + coeff0_25*dmats0[25][12] + coeff0_26*dmats0[26][12] + coeff0_27*dmats0[27][12] + coeff0_28*dmats0[28][12] + coeff0_29*dmats0[29][12] + coeff0_30*dmats0[30][12] + coeff0_31*dmats0[31][12] + coeff0_32*dmats0[32][12] + coeff0_33*dmats0[33][12] + coeff0_34*dmats0[34][12] + coeff0_35*dmats0[35][12] + coeff0_36*dmats0[36][12] + coeff0_37*dmats0[37][12] + coeff0_38*dmats0[38][12] + coeff0_39*dmats0[39][12] + coeff0_40*dmats0[40][12] + coeff0_41*dmats0[41][12] + coeff0_42*dmats0[42][12] + coeff0_43*dmats0[43][12] + coeff0_44*dmats0[44][12] + coeff0_45*dmats0[45][12] + coeff0_46*dmats0[46][12] + coeff0_47*dmats0[47][12] + coeff0_48*dmats0[48][12] + coeff0_49*dmats0[49][12] + coeff0_50*dmats0[50][12] + coeff0_51*dmats0[51][12] + coeff0_52*dmats0[52][12] + coeff0_53*dmats0[53][12] + coeff0_54*dmats0[54][12] + coeff0_55*dmats0[55][12];
          new_coeff0_13 = coeff0_0*dmats0[0][13] + coeff0_1*dmats0[1][13] + coeff0_2*dmats0[2][13] + coeff0_3*dmats0[3][13] + coeff0_4*dmats0[4][13] + coeff0_5*dmats0[5][13] + coeff0_6*dmats0[6][13] + coeff0_7*dmats0[7][13] + coeff0_8*dmats0[8][13] + coeff0_9*dmats0[9][13] + coeff0_10*dmats0[10][13] + coeff0_11*dmats0[11][13] + coeff0_12*dmats0[12][13] + coeff0_13*dmats0[13][13] + coeff0_14*dmats0[14][13] + coeff0_15*dmats0[15][13] + coeff0_16*dmats0[16][13] + coeff0_17*dmats0[17][13] + coeff0_18*dmats0[18][13] + coeff0_19*dmats0[19][13] + coeff0_20*dmats0[20][13] + coeff0_21*dmats0[21][13] + coeff0_22*dmats0[22][13] + coeff0_23*dmats0[23][13] + coeff0_24*dmats0[24][13] + coeff0_25*dmats0[25][13] + coeff0_26*dmats0[26][13] + coeff0_27*dmats0[27][13] + coeff0_28*dmats0[28][13] + coeff0_29*dmats0[29][13] + coeff0_30*dmats0[30][13] + coeff0_31*dmats0[31][13] + coeff0_32*dmats0[32][13] + coeff0_33*dmats0[33][13] + coeff0_34*dmats0[34][13] + coeff0_35*dmats0[35][13] + coeff0_36*dmats0[36][13] + coeff0_37*dmats0[37][13] + coeff0_38*dmats0[38][13] + coeff0_39*dmats0[39][13] + coeff0_40*dmats0[40][13] + coeff0_41*dmats0[41][13] + coeff0_42*dmats0[42][13] + coeff0_43*dmats0[43][13] + coeff0_44*dmats0[44][13] + coeff0_45*dmats0[45][13] + coeff0_46*dmats0[46][13] + coeff0_47*dmats0[47][13] + coeff0_48*dmats0[48][13] + coeff0_49*dmats0[49][13] + coeff0_50*dmats0[50][13] + coeff0_51*dmats0[51][13] + coeff0_52*dmats0[52][13] + coeff0_53*dmats0[53][13] + coeff0_54*dmats0[54][13] + coeff0_55*dmats0[55][13];
          new_coeff0_14 = coeff0_0*dmats0[0][14] + coeff0_1*dmats0[1][14] + coeff0_2*dmats0[2][14] + coeff0_3*dmats0[3][14] + coeff0_4*dmats0[4][14] + coeff0_5*dmats0[5][14] + coeff0_6*dmats0[6][14] + coeff0_7*dmats0[7][14] + coeff0_8*dmats0[8][14] + coeff0_9*dmats0[9][14] + coeff0_10*dmats0[10][14] + coeff0_11*dmats0[11][14] + coeff0_12*dmats0[12][14] + coeff0_13*dmats0[13][14] + coeff0_14*dmats0[14][14] + coeff0_15*dmats0[15][14] + coeff0_16*dmats0[16][14] + coeff0_17*dmats0[17][14] + coeff0_18*dmats0[18][14] + coeff0_19*dmats0[19][14] + coeff0_20*dmats0[20][14] + coeff0_21*dmats0[21][14] + coeff0_22*dmats0[22][14] + coeff0_23*dmats0[23][14] + coeff0_24*dmats0[24][14] + coeff0_25*dmats0[25][14] + coeff0_26*dmats0[26][14] + coeff0_27*dmats0[27][14] + coeff0_28*dmats0[28][14] + coeff0_29*dmats0[29][14] + coeff0_30*dmats0[30][14] + coeff0_31*dmats0[31][14] + coeff0_32*dmats0[32][14] + coeff0_33*dmats0[33][14] + coeff0_34*dmats0[34][14] + coeff0_35*dmats0[35][14] + coeff0_36*dmats0[36][14] + coeff0_37*dmats0[37][14] + coeff0_38*dmats0[38][14] + coeff0_39*dmats0[39][14] + coeff0_40*dmats0[40][14] + coeff0_41*dmats0[41][14] + coeff0_42*dmats0[42][14] + coeff0_43*dmats0[43][14] + coeff0_44*dmats0[44][14] + coeff0_45*dmats0[45][14] + coeff0_46*dmats0[46][14] + coeff0_47*dmats0[47][14] + coeff0_48*dmats0[48][14] + coeff0_49*dmats0[49][14] + coeff0_50*dmats0[50][14] + coeff0_51*dmats0[51][14] + coeff0_52*dmats0[52][14] + coeff0_53*dmats0[53][14] + coeff0_54*dmats0[54][14] + coeff0_55*dmats0[55][14];
          new_coeff0_15 = coeff0_0*dmats0[0][15] + coeff0_1*dmats0[1][15] + coeff0_2*dmats0[2][15] + coeff0_3*dmats0[3][15] + coeff0_4*dmats0[4][15] + coeff0_5*dmats0[5][15] + coeff0_6*dmats0[6][15] + coeff0_7*dmats0[7][15] + coeff0_8*dmats0[8][15] + coeff0_9*dmats0[9][15] + coeff0_10*dmats0[10][15] + coeff0_11*dmats0[11][15] + coeff0_12*dmats0[12][15] + coeff0_13*dmats0[13][15] + coeff0_14*dmats0[14][15] + coeff0_15*dmats0[15][15] + coeff0_16*dmats0[16][15] + coeff0_17*dmats0[17][15] + coeff0_18*dmats0[18][15] + coeff0_19*dmats0[19][15] + coeff0_20*dmats0[20][15] + coeff0_21*dmats0[21][15] + coeff0_22*dmats0[22][15] + coeff0_23*dmats0[23][15] + coeff0_24*dmats0[24][15] + coeff0_25*dmats0[25][15] + coeff0_26*dmats0[26][15] + coeff0_27*dmats0[27][15] + coeff0_28*dmats0[28][15] + coeff0_29*dmats0[29][15] + coeff0_30*dmats0[30][15] + coeff0_31*dmats0[31][15] + coeff0_32*dmats0[32][15] + coeff0_33*dmats0[33][15] + coeff0_34*dmats0[34][15] + coeff0_35*dmats0[35][15] + coeff0_36*dmats0[36][15] + coeff0_37*dmats0[37][15] + coeff0_38*dmats0[38][15] + coeff0_39*dmats0[39][15] + coeff0_40*dmats0[40][15] + coeff0_41*dmats0[41][15] + coeff0_42*dmats0[42][15] + coeff0_43*dmats0[43][15] + coeff0_44*dmats0[44][15] + coeff0_45*dmats0[45][15] + coeff0_46*dmats0[46][15] + coeff0_47*dmats0[47][15] + coeff0_48*dmats0[48][15] + coeff0_49*dmats0[49][15] + coeff0_50*dmats0[50][15] + coeff0_51*dmats0[51][15] + coeff0_52*dmats0[52][15] + coeff0_53*dmats0[53][15] + coeff0_54*dmats0[54][15] + coeff0_55*dmats0[55][15];
          new_coeff0_16 = coeff0_0*dmats0[0][16] + coeff0_1*dmats0[1][16] + coeff0_2*dmats0[2][16] + coeff0_3*dmats0[3][16] + coeff0_4*dmats0[4][16] + coeff0_5*dmats0[5][16] + coeff0_6*dmats0[6][16] + coeff0_7*dmats0[7][16] + coeff0_8*dmats0[8][16] + coeff0_9*dmats0[9][16] + coeff0_10*dmats0[10][16] + coeff0_11*dmats0[11][16] + coeff0_12*dmats0[12][16] + coeff0_13*dmats0[13][16] + coeff0_14*dmats0[14][16] + coeff0_15*dmats0[15][16] + coeff0_16*dmats0[16][16] + coeff0_17*dmats0[17][16] + coeff0_18*dmats0[18][16] + coeff0_19*dmats0[19][16] + coeff0_20*dmats0[20][16] + coeff0_21*dmats0[21][16] + coeff0_22*dmats0[22][16] + coeff0_23*dmats0[23][16] + coeff0_24*dmats0[24][16] + coeff0_25*dmats0[25][16] + coeff0_26*dmats0[26][16] + coeff0_27*dmats0[27][16] + coeff0_28*dmats0[28][16] + coeff0_29*dmats0[29][16] + coeff0_30*dmats0[30][16] + coeff0_31*dmats0[31][16] + coeff0_32*dmats0[32][16] + coeff0_33*dmats0[33][16] + coeff0_34*dmats0[34][16] + coeff0_35*dmats0[35][16] + coeff0_36*dmats0[36][16] + coeff0_37*dmats0[37][16] + coeff0_38*dmats0[38][16] + coeff0_39*dmats0[39][16] + coeff0_40*dmats0[40][16] + coeff0_41*dmats0[41][16] + coeff0_42*dmats0[42][16] + coeff0_43*dmats0[43][16] + coeff0_44*dmats0[44][16] + coeff0_45*dmats0[45][16] + coeff0_46*dmats0[46][16] + coeff0_47*dmats0[47][16] + coeff0_48*dmats0[48][16] + coeff0_49*dmats0[49][16] + coeff0_50*dmats0[50][16] + coeff0_51*dmats0[51][16] + coeff0_52*dmats0[52][16] + coeff0_53*dmats0[53][16] + coeff0_54*dmats0[54][16] + coeff0_55*dmats0[55][16];
          new_coeff0_17 = coeff0_0*dmats0[0][17] + coeff0_1*dmats0[1][17] + coeff0_2*dmats0[2][17] + coeff0_3*dmats0[3][17] + coeff0_4*dmats0[4][17] + coeff0_5*dmats0[5][17] + coeff0_6*dmats0[6][17] + coeff0_7*dmats0[7][17] + coeff0_8*dmats0[8][17] + coeff0_9*dmats0[9][17] + coeff0_10*dmats0[10][17] + coeff0_11*dmats0[11][17] + coeff0_12*dmats0[12][17] + coeff0_13*dmats0[13][17] + coeff0_14*dmats0[14][17] + coeff0_15*dmats0[15][17] + coeff0_16*dmats0[16][17] + coeff0_17*dmats0[17][17] + coeff0_18*dmats0[18][17] + coeff0_19*dmats0[19][17] + coeff0_20*dmats0[20][17] + coeff0_21*dmats0[21][17] + coeff0_22*dmats0[22][17] + coeff0_23*dmats0[23][17] + coeff0_24*dmats0[24][17] + coeff0_25*dmats0[25][17] + coeff0_26*dmats0[26][17] + coeff0_27*dmats0[27][17] + coeff0_28*dmats0[28][17] + coeff0_29*dmats0[29][17] + coeff0_30*dmats0[30][17] + coeff0_31*dmats0[31][17] + coeff0_32*dmats0[32][17] + coeff0_33*dmats0[33][17] + coeff0_34*dmats0[34][17] + coeff0_35*dmats0[35][17] + coeff0_36*dmats0[36][17] + coeff0_37*dmats0[37][17] + coeff0_38*dmats0[38][17] + coeff0_39*dmats0[39][17] + coeff0_40*dmats0[40][17] + coeff0_41*dmats0[41][17] + coeff0_42*dmats0[42][17] + coeff0_43*dmats0[43][17] + coeff0_44*dmats0[44][17] + coeff0_45*dmats0[45][17] + coeff0_46*dmats0[46][17] + coeff0_47*dmats0[47][17] + coeff0_48*dmats0[48][17] + coeff0_49*dmats0[49][17] + coeff0_50*dmats0[50][17] + coeff0_51*dmats0[51][17] + coeff0_52*dmats0[52][17] + coeff0_53*dmats0[53][17] + coeff0_54*dmats0[54][17] + coeff0_55*dmats0[55][17];
          new_coeff0_18 = coeff0_0*dmats0[0][18] + coeff0_1*dmats0[1][18] + coeff0_2*dmats0[2][18] + coeff0_3*dmats0[3][18] + coeff0_4*dmats0[4][18] + coeff0_5*dmats0[5][18] + coeff0_6*dmats0[6][18] + coeff0_7*dmats0[7][18] + coeff0_8*dmats0[8][18] + coeff0_9*dmats0[9][18] + coeff0_10*dmats0[10][18] + coeff0_11*dmats0[11][18] + coeff0_12*dmats0[12][18] + coeff0_13*dmats0[13][18] + coeff0_14*dmats0[14][18] + coeff0_15*dmats0[15][18] + coeff0_16*dmats0[16][18] + coeff0_17*dmats0[17][18] + coeff0_18*dmats0[18][18] + coeff0_19*dmats0[19][18] + coeff0_20*dmats0[20][18] + coeff0_21*dmats0[21][18] + coeff0_22*dmats0[22][18] + coeff0_23*dmats0[23][18] + coeff0_24*dmats0[24][18] + coeff0_25*dmats0[25][18] + coeff0_26*dmats0[26][18] + coeff0_27*dmats0[27][18] + coeff0_28*dmats0[28][18] + coeff0_29*dmats0[29][18] + coeff0_30*dmats0[30][18] + coeff0_31*dmats0[31][18] + coeff0_32*dmats0[32][18] + coeff0_33*dmats0[33][18] + coeff0_34*dmats0[34][18] + coeff0_35*dmats0[35][18] + coeff0_36*dmats0[36][18] + coeff0_37*dmats0[37][18] + coeff0_38*dmats0[38][18] + coeff0_39*dmats0[39][18] + coeff0_40*dmats0[40][18] + coeff0_41*dmats0[41][18] + coeff0_42*dmats0[42][18] + coeff0_43*dmats0[43][18] + coeff0_44*dmats0[44][18] + coeff0_45*dmats0[45][18] + coeff0_46*dmats0[46][18] + coeff0_47*dmats0[47][18] + coeff0_48*dmats0[48][18] + coeff0_49*dmats0[49][18] + coeff0_50*dmats0[50][18] + coeff0_51*dmats0[51][18] + coeff0_52*dmats0[52][18] + coeff0_53*dmats0[53][18] + coeff0_54*dmats0[54][18] + coeff0_55*dmats0[55][18];
          new_coeff0_19 = coeff0_0*dmats0[0][19] + coeff0_1*dmats0[1][19] + coeff0_2*dmats0[2][19] + coeff0_3*dmats0[3][19] + coeff0_4*dmats0[4][19] + coeff0_5*dmats0[5][19] + coeff0_6*dmats0[6][19] + coeff0_7*dmats0[7][19] + coeff0_8*dmats0[8][19] + coeff0_9*dmats0[9][19] + coeff0_10*dmats0[10][19] + coeff0_11*dmats0[11][19] + coeff0_12*dmats0[12][19] + coeff0_13*dmats0[13][19] + coeff0_14*dmats0[14][19] + coeff0_15*dmats0[15][19] + coeff0_16*dmats0[16][19] + coeff0_17*dmats0[17][19] + coeff0_18*dmats0[18][19] + coeff0_19*dmats0[19][19] + coeff0_20*dmats0[20][19] + coeff0_21*dmats0[21][19] + coeff0_22*dmats0[22][19] + coeff0_23*dmats0[23][19] + coeff0_24*dmats0[24][19] + coeff0_25*dmats0[25][19] + coeff0_26*dmats0[26][19] + coeff0_27*dmats0[27][19] + coeff0_28*dmats0[28][19] + coeff0_29*dmats0[29][19] + coeff0_30*dmats0[30][19] + coeff0_31*dmats0[31][19] + coeff0_32*dmats0[32][19] + coeff0_33*dmats0[33][19] + coeff0_34*dmats0[34][19] + coeff0_35*dmats0[35][19] + coeff0_36*dmats0[36][19] + coeff0_37*dmats0[37][19] + coeff0_38*dmats0[38][19] + coeff0_39*dmats0[39][19] + coeff0_40*dmats0[40][19] + coeff0_41*dmats0[41][19] + coeff0_42*dmats0[42][19] + coeff0_43*dmats0[43][19] + coeff0_44*dmats0[44][19] + coeff0_45*dmats0[45][19] + coeff0_46*dmats0[46][19] + coeff0_47*dmats0[47][19] + coeff0_48*dmats0[48][19] + coeff0_49*dmats0[49][19] + coeff0_50*dmats0[50][19] + coeff0_51*dmats0[51][19] + coeff0_52*dmats0[52][19] + coeff0_53*dmats0[53][19] + coeff0_54*dmats0[54][19] + coeff0_55*dmats0[55][19];
          new_coeff0_20 = coeff0_0*dmats0[0][20] + coeff0_1*dmats0[1][20] + coeff0_2*dmats0[2][20] + coeff0_3*dmats0[3][20] + coeff0_4*dmats0[4][20] + coeff0_5*dmats0[5][20] + coeff0_6*dmats0[6][20] + coeff0_7*dmats0[7][20] + coeff0_8*dmats0[8][20] + coeff0_9*dmats0[9][20] + coeff0_10*dmats0[10][20] + coeff0_11*dmats0[11][20] + coeff0_12*dmats0[12][20] + coeff0_13*dmats0[13][20] + coeff0_14*dmats0[14][20] + coeff0_15*dmats0[15][20] + coeff0_16*dmats0[16][20] + coeff0_17*dmats0[17][20] + coeff0_18*dmats0[18][20] + coeff0_19*dmats0[19][20] + coeff0_20*dmats0[20][20] + coeff0_21*dmats0[21][20] + coeff0_22*dmats0[22][20] + coeff0_23*dmats0[23][20] + coeff0_24*dmats0[24][20] + coeff0_25*dmats0[25][20] + coeff0_26*dmats0[26][20] + coeff0_27*dmats0[27][20] + coeff0_28*dmats0[28][20] + coeff0_29*dmats0[29][20] + coeff0_30*dmats0[30][20] + coeff0_31*dmats0[31][20] + coeff0_32*dmats0[32][20] + coeff0_33*dmats0[33][20] + coeff0_34*dmats0[34][20] + coeff0_35*dmats0[35][20] + coeff0_36*dmats0[36][20] + coeff0_37*dmats0[37][20] + coeff0_38*dmats0[38][20] + coeff0_39*dmats0[39][20] + coeff0_40*dmats0[40][20] + coeff0_41*dmats0[41][20] + coeff0_42*dmats0[42][20] + coeff0_43*dmats0[43][20] + coeff0_44*dmats0[44][20] + coeff0_45*dmats0[45][20] + coeff0_46*dmats0[46][20] + coeff0_47*dmats0[47][20] + coeff0_48*dmats0[48][20] + coeff0_49*dmats0[49][20] + coeff0_50*dmats0[50][20] + coeff0_51*dmats0[51][20] + coeff0_52*dmats0[52][20] + coeff0_53*dmats0[53][20] + coeff0_54*dmats0[54][20] + coeff0_55*dmats0[55][20];
          new_coeff0_21 = coeff0_0*dmats0[0][21] + coeff0_1*dmats0[1][21] + coeff0_2*dmats0[2][21] + coeff0_3*dmats0[3][21] + coeff0_4*dmats0[4][21] + coeff0_5*dmats0[5][21] + coeff0_6*dmats0[6][21] + coeff0_7*dmats0[7][21] + coeff0_8*dmats0[8][21] + coeff0_9*dmats0[9][21] + coeff0_10*dmats0[10][21] + coeff0_11*dmats0[11][21] + coeff0_12*dmats0[12][21] + coeff0_13*dmats0[13][21] + coeff0_14*dmats0[14][21] + coeff0_15*dmats0[15][21] + coeff0_16*dmats0[16][21] + coeff0_17*dmats0[17][21] + coeff0_18*dmats0[18][21] + coeff0_19*dmats0[19][21] + coeff0_20*dmats0[20][21] + coeff0_21*dmats0[21][21] + coeff0_22*dmats0[22][21] + coeff0_23*dmats0[23][21] + coeff0_24*dmats0[24][21] + coeff0_25*dmats0[25][21] + coeff0_26*dmats0[26][21] + coeff0_27*dmats0[27][21] + coeff0_28*dmats0[28][21] + coeff0_29*dmats0[29][21] + coeff0_30*dmats0[30][21] + coeff0_31*dmats0[31][21] + coeff0_32*dmats0[32][21] + coeff0_33*dmats0[33][21] + coeff0_34*dmats0[34][21] + coeff0_35*dmats0[35][21] + coeff0_36*dmats0[36][21] + coeff0_37*dmats0[37][21] + coeff0_38*dmats0[38][21] + coeff0_39*dmats0[39][21] + coeff0_40*dmats0[40][21] + coeff0_41*dmats0[41][21] + coeff0_42*dmats0[42][21] + coeff0_43*dmats0[43][21] + coeff0_44*dmats0[44][21] + coeff0_45*dmats0[45][21] + coeff0_46*dmats0[46][21] + coeff0_47*dmats0[47][21] + coeff0_48*dmats0[48][21] + coeff0_49*dmats0[49][21] + coeff0_50*dmats0[50][21] + coeff0_51*dmats0[51][21] + coeff0_52*dmats0[52][21] + coeff0_53*dmats0[53][21] + coeff0_54*dmats0[54][21] + coeff0_55*dmats0[55][21];
          new_coeff0_22 = coeff0_0*dmats0[0][22] + coeff0_1*dmats0[1][22] + coeff0_2*dmats0[2][22] + coeff0_3*dmats0[3][22] + coeff0_4*dmats0[4][22] + coeff0_5*dmats0[5][22] + coeff0_6*dmats0[6][22] + coeff0_7*dmats0[7][22] + coeff0_8*dmats0[8][22] + coeff0_9*dmats0[9][22] + coeff0_10*dmats0[10][22] + coeff0_11*dmats0[11][22] + coeff0_12*dmats0[12][22] + coeff0_13*dmats0[13][22] + coeff0_14*dmats0[14][22] + coeff0_15*dmats0[15][22] + coeff0_16*dmats0[16][22] + coeff0_17*dmats0[17][22] + coeff0_18*dmats0[18][22] + coeff0_19*dmats0[19][22] + coeff0_20*dmats0[20][22] + coeff0_21*dmats0[21][22] + coeff0_22*dmats0[22][22] + coeff0_23*dmats0[23][22] + coeff0_24*dmats0[24][22] + coeff0_25*dmats0[25][22] + coeff0_26*dmats0[26][22] + coeff0_27*dmats0[27][22] + coeff0_28*dmats0[28][22] + coeff0_29*dmats0[29][22] + coeff0_30*dmats0[30][22] + coeff0_31*dmats0[31][22] + coeff0_32*dmats0[32][22] + coeff0_33*dmats0[33][22] + coeff0_34*dmats0[34][22] + coeff0_35*dmats0[35][22] + coeff0_36*dmats0[36][22] + coeff0_37*dmats0[37][22] + coeff0_38*dmats0[38][22] + coeff0_39*dmats0[39][22] + coeff0_40*dmats0[40][22] + coeff0_41*dmats0[41][22] + coeff0_42*dmats0[42][22] + coeff0_43*dmats0[43][22] + coeff0_44*dmats0[44][22] + coeff0_45*dmats0[45][22] + coeff0_46*dmats0[46][22] + coeff0_47*dmats0[47][22] + coeff0_48*dmats0[48][22] + coeff0_49*dmats0[49][22] + coeff0_50*dmats0[50][22] + coeff0_51*dmats0[51][22] + coeff0_52*dmats0[52][22] + coeff0_53*dmats0[53][22] + coeff0_54*dmats0[54][22] + coeff0_55*dmats0[55][22];
          new_coeff0_23 = coeff0_0*dmats0[0][23] + coeff0_1*dmats0[1][23] + coeff0_2*dmats0[2][23] + coeff0_3*dmats0[3][23] + coeff0_4*dmats0[4][23] + coeff0_5*dmats0[5][23] + coeff0_6*dmats0[6][23] + coeff0_7*dmats0[7][23] + coeff0_8*dmats0[8][23] + coeff0_9*dmats0[9][23] + coeff0_10*dmats0[10][23] + coeff0_11*dmats0[11][23] + coeff0_12*dmats0[12][23] + coeff0_13*dmats0[13][23] + coeff0_14*dmats0[14][23] + coeff0_15*dmats0[15][23] + coeff0_16*dmats0[16][23] + coeff0_17*dmats0[17][23] + coeff0_18*dmats0[18][23] + coeff0_19*dmats0[19][23] + coeff0_20*dmats0[20][23] + coeff0_21*dmats0[21][23] + coeff0_22*dmats0[22][23] + coeff0_23*dmats0[23][23] + coeff0_24*dmats0[24][23] + coeff0_25*dmats0[25][23] + coeff0_26*dmats0[26][23] + coeff0_27*dmats0[27][23] + coeff0_28*dmats0[28][23] + coeff0_29*dmats0[29][23] + coeff0_30*dmats0[30][23] + coeff0_31*dmats0[31][23] + coeff0_32*dmats0[32][23] + coeff0_33*dmats0[33][23] + coeff0_34*dmats0[34][23] + coeff0_35*dmats0[35][23] + coeff0_36*dmats0[36][23] + coeff0_37*dmats0[37][23] + coeff0_38*dmats0[38][23] + coeff0_39*dmats0[39][23] + coeff0_40*dmats0[40][23] + coeff0_41*dmats0[41][23] + coeff0_42*dmats0[42][23] + coeff0_43*dmats0[43][23] + coeff0_44*dmats0[44][23] + coeff0_45*dmats0[45][23] + coeff0_46*dmats0[46][23] + coeff0_47*dmats0[47][23] + coeff0_48*dmats0[48][23] + coeff0_49*dmats0[49][23] + coeff0_50*dmats0[50][23] + coeff0_51*dmats0[51][23] + coeff0_52*dmats0[52][23] + coeff0_53*dmats0[53][23] + coeff0_54*dmats0[54][23] + coeff0_55*dmats0[55][23];
          new_coeff0_24 = coeff0_0*dmats0[0][24] + coeff0_1*dmats0[1][24] + coeff0_2*dmats0[2][24] + coeff0_3*dmats0[3][24] + coeff0_4*dmats0[4][24] + coeff0_5*dmats0[5][24] + coeff0_6*dmats0[6][24] + coeff0_7*dmats0[7][24] + coeff0_8*dmats0[8][24] + coeff0_9*dmats0[9][24] + coeff0_10*dmats0[10][24] + coeff0_11*dmats0[11][24] + coeff0_12*dmats0[12][24] + coeff0_13*dmats0[13][24] + coeff0_14*dmats0[14][24] + coeff0_15*dmats0[15][24] + coeff0_16*dmats0[16][24] + coeff0_17*dmats0[17][24] + coeff0_18*dmats0[18][24] + coeff0_19*dmats0[19][24] + coeff0_20*dmats0[20][24] + coeff0_21*dmats0[21][24] + coeff0_22*dmats0[22][24] + coeff0_23*dmats0[23][24] + coeff0_24*dmats0[24][24] + coeff0_25*dmats0[25][24] + coeff0_26*dmats0[26][24] + coeff0_27*dmats0[27][24] + coeff0_28*dmats0[28][24] + coeff0_29*dmats0[29][24] + coeff0_30*dmats0[30][24] + coeff0_31*dmats0[31][24] + coeff0_32*dmats0[32][24] + coeff0_33*dmats0[33][24] + coeff0_34*dmats0[34][24] + coeff0_35*dmats0[35][24] + coeff0_36*dmats0[36][24] + coeff0_37*dmats0[37][24] + coeff0_38*dmats0[38][24] + coeff0_39*dmats0[39][24] + coeff0_40*dmats0[40][24] + coeff0_41*dmats0[41][24] + coeff0_42*dmats0[42][24] + coeff0_43*dmats0[43][24] + coeff0_44*dmats0[44][24] + coeff0_45*dmats0[45][24] + coeff0_46*dmats0[46][24] + coeff0_47*dmats0[47][24] + coeff0_48*dmats0[48][24] + coeff0_49*dmats0[49][24] + coeff0_50*dmats0[50][24] + coeff0_51*dmats0[51][24] + coeff0_52*dmats0[52][24] + coeff0_53*dmats0[53][24] + coeff0_54*dmats0[54][24] + coeff0_55*dmats0[55][24];
          new_coeff0_25 = coeff0_0*dmats0[0][25] + coeff0_1*dmats0[1][25] + coeff0_2*dmats0[2][25] + coeff0_3*dmats0[3][25] + coeff0_4*dmats0[4][25] + coeff0_5*dmats0[5][25] + coeff0_6*dmats0[6][25] + coeff0_7*dmats0[7][25] + coeff0_8*dmats0[8][25] + coeff0_9*dmats0[9][25] + coeff0_10*dmats0[10][25] + coeff0_11*dmats0[11][25] + coeff0_12*dmats0[12][25] + coeff0_13*dmats0[13][25] + coeff0_14*dmats0[14][25] + coeff0_15*dmats0[15][25] + coeff0_16*dmats0[16][25] + coeff0_17*dmats0[17][25] + coeff0_18*dmats0[18][25] + coeff0_19*dmats0[19][25] + coeff0_20*dmats0[20][25] + coeff0_21*dmats0[21][25] + coeff0_22*dmats0[22][25] + coeff0_23*dmats0[23][25] + coeff0_24*dmats0[24][25] + coeff0_25*dmats0[25][25] + coeff0_26*dmats0[26][25] + coeff0_27*dmats0[27][25] + coeff0_28*dmats0[28][25] + coeff0_29*dmats0[29][25] + coeff0_30*dmats0[30][25] + coeff0_31*dmats0[31][25] + coeff0_32*dmats0[32][25] + coeff0_33*dmats0[33][25] + coeff0_34*dmats0[34][25] + coeff0_35*dmats0[35][25] + coeff0_36*dmats0[36][25] + coeff0_37*dmats0[37][25] + coeff0_38*dmats0[38][25] + coeff0_39*dmats0[39][25] + coeff0_40*dmats0[40][25] + coeff0_41*dmats0[41][25] + coeff0_42*dmats0[42][25] + coeff0_43*dmats0[43][25] + coeff0_44*dmats0[44][25] + coeff0_45*dmats0[45][25] + coeff0_46*dmats0[46][25] + coeff0_47*dmats0[47][25] + coeff0_48*dmats0[48][25] + coeff0_49*dmats0[49][25] + coeff0_50*dmats0[50][25] + coeff0_51*dmats0[51][25] + coeff0_52*dmats0[52][25] + coeff0_53*dmats0[53][25] + coeff0_54*dmats0[54][25] + coeff0_55*dmats0[55][25];
          new_coeff0_26 = coeff0_0*dmats0[0][26] + coeff0_1*dmats0[1][26] + coeff0_2*dmats0[2][26] + coeff0_3*dmats0[3][26] + coeff0_4*dmats0[4][26] + coeff0_5*dmats0[5][26] + coeff0_6*dmats0[6][26] + coeff0_7*dmats0[7][26] + coeff0_8*dmats0[8][26] + coeff0_9*dmats0[9][26] + coeff0_10*dmats0[10][26] + coeff0_11*dmats0[11][26] + coeff0_12*dmats0[12][26] + coeff0_13*dmats0[13][26] + coeff0_14*dmats0[14][26] + coeff0_15*dmats0[15][26] + coeff0_16*dmats0[16][26] + coeff0_17*dmats0[17][26] + coeff0_18*dmats0[18][26] + coeff0_19*dmats0[19][26] + coeff0_20*dmats0[20][26] + coeff0_21*dmats0[21][26] + coeff0_22*dmats0[22][26] + coeff0_23*dmats0[23][26] + coeff0_24*dmats0[24][26] + coeff0_25*dmats0[25][26] + coeff0_26*dmats0[26][26] + coeff0_27*dmats0[27][26] + coeff0_28*dmats0[28][26] + coeff0_29*dmats0[29][26] + coeff0_30*dmats0[30][26] + coeff0_31*dmats0[31][26] + coeff0_32*dmats0[32][26] + coeff0_33*dmats0[33][26] + coeff0_34*dmats0[34][26] + coeff0_35*dmats0[35][26] + coeff0_36*dmats0[36][26] + coeff0_37*dmats0[37][26] + coeff0_38*dmats0[38][26] + coeff0_39*dmats0[39][26] + coeff0_40*dmats0[40][26] + coeff0_41*dmats0[41][26] + coeff0_42*dmats0[42][26] + coeff0_43*dmats0[43][26] + coeff0_44*dmats0[44][26] + coeff0_45*dmats0[45][26] + coeff0_46*dmats0[46][26] + coeff0_47*dmats0[47][26] + coeff0_48*dmats0[48][26] + coeff0_49*dmats0[49][26] + coeff0_50*dmats0[50][26] + coeff0_51*dmats0[51][26] + coeff0_52*dmats0[52][26] + coeff0_53*dmats0[53][26] + coeff0_54*dmats0[54][26] + coeff0_55*dmats0[55][26];
          new_coeff0_27 = coeff0_0*dmats0[0][27] + coeff0_1*dmats0[1][27] + coeff0_2*dmats0[2][27] + coeff0_3*dmats0[3][27] + coeff0_4*dmats0[4][27] + coeff0_5*dmats0[5][27] + coeff0_6*dmats0[6][27] + coeff0_7*dmats0[7][27] + coeff0_8*dmats0[8][27] + coeff0_9*dmats0[9][27] + coeff0_10*dmats0[10][27] + coeff0_11*dmats0[11][27] + coeff0_12*dmats0[12][27] + coeff0_13*dmats0[13][27] + coeff0_14*dmats0[14][27] + coeff0_15*dmats0[15][27] + coeff0_16*dmats0[16][27] + coeff0_17*dmats0[17][27] + coeff0_18*dmats0[18][27] + coeff0_19*dmats0[19][27] + coeff0_20*dmats0[20][27] + coeff0_21*dmats0[21][27] + coeff0_22*dmats0[22][27] + coeff0_23*dmats0[23][27] + coeff0_24*dmats0[24][27] + coeff0_25*dmats0[25][27] + coeff0_26*dmats0[26][27] + coeff0_27*dmats0[27][27] + coeff0_28*dmats0[28][27] + coeff0_29*dmats0[29][27] + coeff0_30*dmats0[30][27] + coeff0_31*dmats0[31][27] + coeff0_32*dmats0[32][27] + coeff0_33*dmats0[33][27] + coeff0_34*dmats0[34][27] + coeff0_35*dmats0[35][27] + coeff0_36*dmats0[36][27] + coeff0_37*dmats0[37][27] + coeff0_38*dmats0[38][27] + coeff0_39*dmats0[39][27] + coeff0_40*dmats0[40][27] + coeff0_41*dmats0[41][27] + coeff0_42*dmats0[42][27] + coeff0_43*dmats0[43][27] + coeff0_44*dmats0[44][27] + coeff0_45*dmats0[45][27] + coeff0_46*dmats0[46][27] + coeff0_47*dmats0[47][27] + coeff0_48*dmats0[48][27] + coeff0_49*dmats0[49][27] + coeff0_50*dmats0[50][27] + coeff0_51*dmats0[51][27] + coeff0_52*dmats0[52][27] + coeff0_53*dmats0[53][27] + coeff0_54*dmats0[54][27] + coeff0_55*dmats0[55][27];
          new_coeff0_28 = coeff0_0*dmats0[0][28] + coeff0_1*dmats0[1][28] + coeff0_2*dmats0[2][28] + coeff0_3*dmats0[3][28] + coeff0_4*dmats0[4][28] + coeff0_5*dmats0[5][28] + coeff0_6*dmats0[6][28] + coeff0_7*dmats0[7][28] + coeff0_8*dmats0[8][28] + coeff0_9*dmats0[9][28] + coeff0_10*dmats0[10][28] + coeff0_11*dmats0[11][28] + coeff0_12*dmats0[12][28] + coeff0_13*dmats0[13][28] + coeff0_14*dmats0[14][28] + coeff0_15*dmats0[15][28] + coeff0_16*dmats0[16][28] + coeff0_17*dmats0[17][28] + coeff0_18*dmats0[18][28] + coeff0_19*dmats0[19][28] + coeff0_20*dmats0[20][28] + coeff0_21*dmats0[21][28] + coeff0_22*dmats0[22][28] + coeff0_23*dmats0[23][28] + coeff0_24*dmats0[24][28] + coeff0_25*dmats0[25][28] + coeff0_26*dmats0[26][28] + coeff0_27*dmats0[27][28] + coeff0_28*dmats0[28][28] + coeff0_29*dmats0[29][28] + coeff0_30*dmats0[30][28] + coeff0_31*dmats0[31][28] + coeff0_32*dmats0[32][28] + coeff0_33*dmats0[33][28] + coeff0_34*dmats0[34][28] + coeff0_35*dmats0[35][28] + coeff0_36*dmats0[36][28] + coeff0_37*dmats0[37][28] + coeff0_38*dmats0[38][28] + coeff0_39*dmats0[39][28] + coeff0_40*dmats0[40][28] + coeff0_41*dmats0[41][28] + coeff0_42*dmats0[42][28] + coeff0_43*dmats0[43][28] + coeff0_44*dmats0[44][28] + coeff0_45*dmats0[45][28] + coeff0_46*dmats0[46][28] + coeff0_47*dmats0[47][28] + coeff0_48*dmats0[48][28] + coeff0_49*dmats0[49][28] + coeff0_50*dmats0[50][28] + coeff0_51*dmats0[51][28] + coeff0_52*dmats0[52][28] + coeff0_53*dmats0[53][28] + coeff0_54*dmats0[54][28] + coeff0_55*dmats0[55][28];
          new_coeff0_29 = coeff0_0*dmats0[0][29] + coeff0_1*dmats0[1][29] + coeff0_2*dmats0[2][29] + coeff0_3*dmats0[3][29] + coeff0_4*dmats0[4][29] + coeff0_5*dmats0[5][29] + coeff0_6*dmats0[6][29] + coeff0_7*dmats0[7][29] + coeff0_8*dmats0[8][29] + coeff0_9*dmats0[9][29] + coeff0_10*dmats0[10][29] + coeff0_11*dmats0[11][29] + coeff0_12*dmats0[12][29] + coeff0_13*dmats0[13][29] + coeff0_14*dmats0[14][29] + coeff0_15*dmats0[15][29] + coeff0_16*dmats0[16][29] + coeff0_17*dmats0[17][29] + coeff0_18*dmats0[18][29] + coeff0_19*dmats0[19][29] + coeff0_20*dmats0[20][29] + coeff0_21*dmats0[21][29] + coeff0_22*dmats0[22][29] + coeff0_23*dmats0[23][29] + coeff0_24*dmats0[24][29] + coeff0_25*dmats0[25][29] + coeff0_26*dmats0[26][29] + coeff0_27*dmats0[27][29] + coeff0_28*dmats0[28][29] + coeff0_29*dmats0[29][29] + coeff0_30*dmats0[30][29] + coeff0_31*dmats0[31][29] + coeff0_32*dmats0[32][29] + coeff0_33*dmats0[33][29] + coeff0_34*dmats0[34][29] + coeff0_35*dmats0[35][29] + coeff0_36*dmats0[36][29] + coeff0_37*dmats0[37][29] + coeff0_38*dmats0[38][29] + coeff0_39*dmats0[39][29] + coeff0_40*dmats0[40][29] + coeff0_41*dmats0[41][29] + coeff0_42*dmats0[42][29] + coeff0_43*dmats0[43][29] + coeff0_44*dmats0[44][29] + coeff0_45*dmats0[45][29] + coeff0_46*dmats0[46][29] + coeff0_47*dmats0[47][29] + coeff0_48*dmats0[48][29] + coeff0_49*dmats0[49][29] + coeff0_50*dmats0[50][29] + coeff0_51*dmats0[51][29] + coeff0_52*dmats0[52][29] + coeff0_53*dmats0[53][29] + coeff0_54*dmats0[54][29] + coeff0_55*dmats0[55][29];
          new_coeff0_30 = coeff0_0*dmats0[0][30] + coeff0_1*dmats0[1][30] + coeff0_2*dmats0[2][30] + coeff0_3*dmats0[3][30] + coeff0_4*dmats0[4][30] + coeff0_5*dmats0[5][30] + coeff0_6*dmats0[6][30] + coeff0_7*dmats0[7][30] + coeff0_8*dmats0[8][30] + coeff0_9*dmats0[9][30] + coeff0_10*dmats0[10][30] + coeff0_11*dmats0[11][30] + coeff0_12*dmats0[12][30] + coeff0_13*dmats0[13][30] + coeff0_14*dmats0[14][30] + coeff0_15*dmats0[15][30] + coeff0_16*dmats0[16][30] + coeff0_17*dmats0[17][30] + coeff0_18*dmats0[18][30] + coeff0_19*dmats0[19][30] + coeff0_20*dmats0[20][30] + coeff0_21*dmats0[21][30] + coeff0_22*dmats0[22][30] + coeff0_23*dmats0[23][30] + coeff0_24*dmats0[24][30] + coeff0_25*dmats0[25][30] + coeff0_26*dmats0[26][30] + coeff0_27*dmats0[27][30] + coeff0_28*dmats0[28][30] + coeff0_29*dmats0[29][30] + coeff0_30*dmats0[30][30] + coeff0_31*dmats0[31][30] + coeff0_32*dmats0[32][30] + coeff0_33*dmats0[33][30] + coeff0_34*dmats0[34][30] + coeff0_35*dmats0[35][30] + coeff0_36*dmats0[36][30] + coeff0_37*dmats0[37][30] + coeff0_38*dmats0[38][30] + coeff0_39*dmats0[39][30] + coeff0_40*dmats0[40][30] + coeff0_41*dmats0[41][30] + coeff0_42*dmats0[42][30] + coeff0_43*dmats0[43][30] + coeff0_44*dmats0[44][30] + coeff0_45*dmats0[45][30] + coeff0_46*dmats0[46][30] + coeff0_47*dmats0[47][30] + coeff0_48*dmats0[48][30] + coeff0_49*dmats0[49][30] + coeff0_50*dmats0[50][30] + coeff0_51*dmats0[51][30] + coeff0_52*dmats0[52][30] + coeff0_53*dmats0[53][30] + coeff0_54*dmats0[54][30] + coeff0_55*dmats0[55][30];
          new_coeff0_31 = coeff0_0*dmats0[0][31] + coeff0_1*dmats0[1][31] + coeff0_2*dmats0[2][31] + coeff0_3*dmats0[3][31] + coeff0_4*dmats0[4][31] + coeff0_5*dmats0[5][31] + coeff0_6*dmats0[6][31] + coeff0_7*dmats0[7][31] + coeff0_8*dmats0[8][31] + coeff0_9*dmats0[9][31] + coeff0_10*dmats0[10][31] + coeff0_11*dmats0[11][31] + coeff0_12*dmats0[12][31] + coeff0_13*dmats0[13][31] + coeff0_14*dmats0[14][31] + coeff0_15*dmats0[15][31] + coeff0_16*dmats0[16][31] + coeff0_17*dmats0[17][31] + coeff0_18*dmats0[18][31] + coeff0_19*dmats0[19][31] + coeff0_20*dmats0[20][31] + coeff0_21*dmats0[21][31] + coeff0_22*dmats0[22][31] + coeff0_23*dmats0[23][31] + coeff0_24*dmats0[24][31] + coeff0_25*dmats0[25][31] + coeff0_26*dmats0[26][31] + coeff0_27*dmats0[27][31] + coeff0_28*dmats0[28][31] + coeff0_29*dmats0[29][31] + coeff0_30*dmats0[30][31] + coeff0_31*dmats0[31][31] + coeff0_32*dmats0[32][31] + coeff0_33*dmats0[33][31] + coeff0_34*dmats0[34][31] + coeff0_35*dmats0[35][31] + coeff0_36*dmats0[36][31] + coeff0_37*dmats0[37][31] + coeff0_38*dmats0[38][31] + coeff0_39*dmats0[39][31] + coeff0_40*dmats0[40][31] + coeff0_41*dmats0[41][31] + coeff0_42*dmats0[42][31] + coeff0_43*dmats0[43][31] + coeff0_44*dmats0[44][31] + coeff0_45*dmats0[45][31] + coeff0_46*dmats0[46][31] + coeff0_47*dmats0[47][31] + coeff0_48*dmats0[48][31] + coeff0_49*dmats0[49][31] + coeff0_50*dmats0[50][31] + coeff0_51*dmats0[51][31] + coeff0_52*dmats0[52][31] + coeff0_53*dmats0[53][31] + coeff0_54*dmats0[54][31] + coeff0_55*dmats0[55][31];
          new_coeff0_32 = coeff0_0*dmats0[0][32] + coeff0_1*dmats0[1][32] + coeff0_2*dmats0[2][32] + coeff0_3*dmats0[3][32] + coeff0_4*dmats0[4][32] + coeff0_5*dmats0[5][32] + coeff0_6*dmats0[6][32] + coeff0_7*dmats0[7][32] + coeff0_8*dmats0[8][32] + coeff0_9*dmats0[9][32] + coeff0_10*dmats0[10][32] + coeff0_11*dmats0[11][32] + coeff0_12*dmats0[12][32] + coeff0_13*dmats0[13][32] + coeff0_14*dmats0[14][32] + coeff0_15*dmats0[15][32] + coeff0_16*dmats0[16][32] + coeff0_17*dmats0[17][32] + coeff0_18*dmats0[18][32] + coeff0_19*dmats0[19][32] + coeff0_20*dmats0[20][32] + coeff0_21*dmats0[21][32] + coeff0_22*dmats0[22][32] + coeff0_23*dmats0[23][32] + coeff0_24*dmats0[24][32] + coeff0_25*dmats0[25][32] + coeff0_26*dmats0[26][32] + coeff0_27*dmats0[27][32] + coeff0_28*dmats0[28][32] + coeff0_29*dmats0[29][32] + coeff0_30*dmats0[30][32] + coeff0_31*dmats0[31][32] + coeff0_32*dmats0[32][32] + coeff0_33*dmats0[33][32] + coeff0_34*dmats0[34][32] + coeff0_35*dmats0[35][32] + coeff0_36*dmats0[36][32] + coeff0_37*dmats0[37][32] + coeff0_38*dmats0[38][32] + coeff0_39*dmats0[39][32] + coeff0_40*dmats0[40][32] + coeff0_41*dmats0[41][32] + coeff0_42*dmats0[42][32] + coeff0_43*dmats0[43][32] + coeff0_44*dmats0[44][32] + coeff0_45*dmats0[45][32] + coeff0_46*dmats0[46][32] + coeff0_47*dmats0[47][32] + coeff0_48*dmats0[48][32] + coeff0_49*dmats0[49][32] + coeff0_50*dmats0[50][32] + coeff0_51*dmats0[51][32] + coeff0_52*dmats0[52][32] + coeff0_53*dmats0[53][32] + coeff0_54*dmats0[54][32] + coeff0_55*dmats0[55][32];
          new_coeff0_33 = coeff0_0*dmats0[0][33] + coeff0_1*dmats0[1][33] + coeff0_2*dmats0[2][33] + coeff0_3*dmats0[3][33] + coeff0_4*dmats0[4][33] + coeff0_5*dmats0[5][33] + coeff0_6*dmats0[6][33] + coeff0_7*dmats0[7][33] + coeff0_8*dmats0[8][33] + coeff0_9*dmats0[9][33] + coeff0_10*dmats0[10][33] + coeff0_11*dmats0[11][33] + coeff0_12*dmats0[12][33] + coeff0_13*dmats0[13][33] + coeff0_14*dmats0[14][33] + coeff0_15*dmats0[15][33] + coeff0_16*dmats0[16][33] + coeff0_17*dmats0[17][33] + coeff0_18*dmats0[18][33] + coeff0_19*dmats0[19][33] + coeff0_20*dmats0[20][33] + coeff0_21*dmats0[21][33] + coeff0_22*dmats0[22][33] + coeff0_23*dmats0[23][33] + coeff0_24*dmats0[24][33] + coeff0_25*dmats0[25][33] + coeff0_26*dmats0[26][33] + coeff0_27*dmats0[27][33] + coeff0_28*dmats0[28][33] + coeff0_29*dmats0[29][33] + coeff0_30*dmats0[30][33] + coeff0_31*dmats0[31][33] + coeff0_32*dmats0[32][33] + coeff0_33*dmats0[33][33] + coeff0_34*dmats0[34][33] + coeff0_35*dmats0[35][33] + coeff0_36*dmats0[36][33] + coeff0_37*dmats0[37][33] + coeff0_38*dmats0[38][33] + coeff0_39*dmats0[39][33] + coeff0_40*dmats0[40][33] + coeff0_41*dmats0[41][33] + coeff0_42*dmats0[42][33] + coeff0_43*dmats0[43][33] + coeff0_44*dmats0[44][33] + coeff0_45*dmats0[45][33] + coeff0_46*dmats0[46][33] + coeff0_47*dmats0[47][33] + coeff0_48*dmats0[48][33] + coeff0_49*dmats0[49][33] + coeff0_50*dmats0[50][33] + coeff0_51*dmats0[51][33] + coeff0_52*dmats0[52][33] + coeff0_53*dmats0[53][33] + coeff0_54*dmats0[54][33] + coeff0_55*dmats0[55][33];
          new_coeff0_34 = coeff0_0*dmats0[0][34] + coeff0_1*dmats0[1][34] + coeff0_2*dmats0[2][34] + coeff0_3*dmats0[3][34] + coeff0_4*dmats0[4][34] + coeff0_5*dmats0[5][34] + coeff0_6*dmats0[6][34] + coeff0_7*dmats0[7][34] + coeff0_8*dmats0[8][34] + coeff0_9*dmats0[9][34] + coeff0_10*dmats0[10][34] + coeff0_11*dmats0[11][34] + coeff0_12*dmats0[12][34] + coeff0_13*dmats0[13][34] + coeff0_14*dmats0[14][34] + coeff0_15*dmats0[15][34] + coeff0_16*dmats0[16][34] + coeff0_17*dmats0[17][34] + coeff0_18*dmats0[18][34] + coeff0_19*dmats0[19][34] + coeff0_20*dmats0[20][34] + coeff0_21*dmats0[21][34] + coeff0_22*dmats0[22][34] + coeff0_23*dmats0[23][34] + coeff0_24*dmats0[24][34] + coeff0_25*dmats0[25][34] + coeff0_26*dmats0[26][34] + coeff0_27*dmats0[27][34] + coeff0_28*dmats0[28][34] + coeff0_29*dmats0[29][34] + coeff0_30*dmats0[30][34] + coeff0_31*dmats0[31][34] + coeff0_32*dmats0[32][34] + coeff0_33*dmats0[33][34] + coeff0_34*dmats0[34][34] + coeff0_35*dmats0[35][34] + coeff0_36*dmats0[36][34] + coeff0_37*dmats0[37][34] + coeff0_38*dmats0[38][34] + coeff0_39*dmats0[39][34] + coeff0_40*dmats0[40][34] + coeff0_41*dmats0[41][34] + coeff0_42*dmats0[42][34] + coeff0_43*dmats0[43][34] + coeff0_44*dmats0[44][34] + coeff0_45*dmats0[45][34] + coeff0_46*dmats0[46][34] + coeff0_47*dmats0[47][34] + coeff0_48*dmats0[48][34] + coeff0_49*dmats0[49][34] + coeff0_50*dmats0[50][34] + coeff0_51*dmats0[51][34] + coeff0_52*dmats0[52][34] + coeff0_53*dmats0[53][34] + coeff0_54*dmats0[54][34] + coeff0_55*dmats0[55][34];
          new_coeff0_35 = coeff0_0*dmats0[0][35] + coeff0_1*dmats0[1][35] + coeff0_2*dmats0[2][35] + coeff0_3*dmats0[3][35] + coeff0_4*dmats0[4][35] + coeff0_5*dmats0[5][35] + coeff0_6*dmats0[6][35] + coeff0_7*dmats0[7][35] + coeff0_8*dmats0[8][35] + coeff0_9*dmats0[9][35] + coeff0_10*dmats0[10][35] + coeff0_11*dmats0[11][35] + coeff0_12*dmats0[12][35] + coeff0_13*dmats0[13][35] + coeff0_14*dmats0[14][35] + coeff0_15*dmats0[15][35] + coeff0_16*dmats0[16][35] + coeff0_17*dmats0[17][35] + coeff0_18*dmats0[18][35] + coeff0_19*dmats0[19][35] + coeff0_20*dmats0[20][35] + coeff0_21*dmats0[21][35] + coeff0_22*dmats0[22][35] + coeff0_23*dmats0[23][35] + coeff0_24*dmats0[24][35] + coeff0_25*dmats0[25][35] + coeff0_26*dmats0[26][35] + coeff0_27*dmats0[27][35] + coeff0_28*dmats0[28][35] + coeff0_29*dmats0[29][35] + coeff0_30*dmats0[30][35] + coeff0_31*dmats0[31][35] + coeff0_32*dmats0[32][35] + coeff0_33*dmats0[33][35] + coeff0_34*dmats0[34][35] + coeff0_35*dmats0[35][35] + coeff0_36*dmats0[36][35] + coeff0_37*dmats0[37][35] + coeff0_38*dmats0[38][35] + coeff0_39*dmats0[39][35] + coeff0_40*dmats0[40][35] + coeff0_41*dmats0[41][35] + coeff0_42*dmats0[42][35] + coeff0_43*dmats0[43][35] + coeff0_44*dmats0[44][35] + coeff0_45*dmats0[45][35] + coeff0_46*dmats0[46][35] + coeff0_47*dmats0[47][35] + coeff0_48*dmats0[48][35] + coeff0_49*dmats0[49][35] + coeff0_50*dmats0[50][35] + coeff0_51*dmats0[51][35] + coeff0_52*dmats0[52][35] + coeff0_53*dmats0[53][35] + coeff0_54*dmats0[54][35] + coeff0_55*dmats0[55][35];
          new_coeff0_36 = coeff0_0*dmats0[0][36] + coeff0_1*dmats0[1][36] + coeff0_2*dmats0[2][36] + coeff0_3*dmats0[3][36] + coeff0_4*dmats0[4][36] + coeff0_5*dmats0[5][36] + coeff0_6*dmats0[6][36] + coeff0_7*dmats0[7][36] + coeff0_8*dmats0[8][36] + coeff0_9*dmats0[9][36] + coeff0_10*dmats0[10][36] + coeff0_11*dmats0[11][36] + coeff0_12*dmats0[12][36] + coeff0_13*dmats0[13][36] + coeff0_14*dmats0[14][36] + coeff0_15*dmats0[15][36] + coeff0_16*dmats0[16][36] + coeff0_17*dmats0[17][36] + coeff0_18*dmats0[18][36] + coeff0_19*dmats0[19][36] + coeff0_20*dmats0[20][36] + coeff0_21*dmats0[21][36] + coeff0_22*dmats0[22][36] + coeff0_23*dmats0[23][36] + coeff0_24*dmats0[24][36] + coeff0_25*dmats0[25][36] + coeff0_26*dmats0[26][36] + coeff0_27*dmats0[27][36] + coeff0_28*dmats0[28][36] + coeff0_29*dmats0[29][36] + coeff0_30*dmats0[30][36] + coeff0_31*dmats0[31][36] + coeff0_32*dmats0[32][36] + coeff0_33*dmats0[33][36] + coeff0_34*dmats0[34][36] + coeff0_35*dmats0[35][36] + coeff0_36*dmats0[36][36] + coeff0_37*dmats0[37][36] + coeff0_38*dmats0[38][36] + coeff0_39*dmats0[39][36] + coeff0_40*dmats0[40][36] + coeff0_41*dmats0[41][36] + coeff0_42*dmats0[42][36] + coeff0_43*dmats0[43][36] + coeff0_44*dmats0[44][36] + coeff0_45*dmats0[45][36] + coeff0_46*dmats0[46][36] + coeff0_47*dmats0[47][36] + coeff0_48*dmats0[48][36] + coeff0_49*dmats0[49][36] + coeff0_50*dmats0[50][36] + coeff0_51*dmats0[51][36] + coeff0_52*dmats0[52][36] + coeff0_53*dmats0[53][36] + coeff0_54*dmats0[54][36] + coeff0_55*dmats0[55][36];
          new_coeff0_37 = coeff0_0*dmats0[0][37] + coeff0_1*dmats0[1][37] + coeff0_2*dmats0[2][37] + coeff0_3*dmats0[3][37] + coeff0_4*dmats0[4][37] + coeff0_5*dmats0[5][37] + coeff0_6*dmats0[6][37] + coeff0_7*dmats0[7][37] + coeff0_8*dmats0[8][37] + coeff0_9*dmats0[9][37] + coeff0_10*dmats0[10][37] + coeff0_11*dmats0[11][37] + coeff0_12*dmats0[12][37] + coeff0_13*dmats0[13][37] + coeff0_14*dmats0[14][37] + coeff0_15*dmats0[15][37] + coeff0_16*dmats0[16][37] + coeff0_17*dmats0[17][37] + coeff0_18*dmats0[18][37] + coeff0_19*dmats0[19][37] + coeff0_20*dmats0[20][37] + coeff0_21*dmats0[21][37] + coeff0_22*dmats0[22][37] + coeff0_23*dmats0[23][37] + coeff0_24*dmats0[24][37] + coeff0_25*dmats0[25][37] + coeff0_26*dmats0[26][37] + coeff0_27*dmats0[27][37] + coeff0_28*dmats0[28][37] + coeff0_29*dmats0[29][37] + coeff0_30*dmats0[30][37] + coeff0_31*dmats0[31][37] + coeff0_32*dmats0[32][37] + coeff0_33*dmats0[33][37] + coeff0_34*dmats0[34][37] + coeff0_35*dmats0[35][37] + coeff0_36*dmats0[36][37] + coeff0_37*dmats0[37][37] + coeff0_38*dmats0[38][37] + coeff0_39*dmats0[39][37] + coeff0_40*dmats0[40][37] + coeff0_41*dmats0[41][37] + coeff0_42*dmats0[42][37] + coeff0_43*dmats0[43][37] + coeff0_44*dmats0[44][37] + coeff0_45*dmats0[45][37] + coeff0_46*dmats0[46][37] + coeff0_47*dmats0[47][37] + coeff0_48*dmats0[48][37] + coeff0_49*dmats0[49][37] + coeff0_50*dmats0[50][37] + coeff0_51*dmats0[51][37] + coeff0_52*dmats0[52][37] + coeff0_53*dmats0[53][37] + coeff0_54*dmats0[54][37] + coeff0_55*dmats0[55][37];
          new_coeff0_38 = coeff0_0*dmats0[0][38] + coeff0_1*dmats0[1][38] + coeff0_2*dmats0[2][38] + coeff0_3*dmats0[3][38] + coeff0_4*dmats0[4][38] + coeff0_5*dmats0[5][38] + coeff0_6*dmats0[6][38] + coeff0_7*dmats0[7][38] + coeff0_8*dmats0[8][38] + coeff0_9*dmats0[9][38] + coeff0_10*dmats0[10][38] + coeff0_11*dmats0[11][38] + coeff0_12*dmats0[12][38] + coeff0_13*dmats0[13][38] + coeff0_14*dmats0[14][38] + coeff0_15*dmats0[15][38] + coeff0_16*dmats0[16][38] + coeff0_17*dmats0[17][38] + coeff0_18*dmats0[18][38] + coeff0_19*dmats0[19][38] + coeff0_20*dmats0[20][38] + coeff0_21*dmats0[21][38] + coeff0_22*dmats0[22][38] + coeff0_23*dmats0[23][38] + coeff0_24*dmats0[24][38] + coeff0_25*dmats0[25][38] + coeff0_26*dmats0[26][38] + coeff0_27*dmats0[27][38] + coeff0_28*dmats0[28][38] + coeff0_29*dmats0[29][38] + coeff0_30*dmats0[30][38] + coeff0_31*dmats0[31][38] + coeff0_32*dmats0[32][38] + coeff0_33*dmats0[33][38] + coeff0_34*dmats0[34][38] + coeff0_35*dmats0[35][38] + coeff0_36*dmats0[36][38] + coeff0_37*dmats0[37][38] + coeff0_38*dmats0[38][38] + coeff0_39*dmats0[39][38] + coeff0_40*dmats0[40][38] + coeff0_41*dmats0[41][38] + coeff0_42*dmats0[42][38] + coeff0_43*dmats0[43][38] + coeff0_44*dmats0[44][38] + coeff0_45*dmats0[45][38] + coeff0_46*dmats0[46][38] + coeff0_47*dmats0[47][38] + coeff0_48*dmats0[48][38] + coeff0_49*dmats0[49][38] + coeff0_50*dmats0[50][38] + coeff0_51*dmats0[51][38] + coeff0_52*dmats0[52][38] + coeff0_53*dmats0[53][38] + coeff0_54*dmats0[54][38] + coeff0_55*dmats0[55][38];
          new_coeff0_39 = coeff0_0*dmats0[0][39] + coeff0_1*dmats0[1][39] + coeff0_2*dmats0[2][39] + coeff0_3*dmats0[3][39] + coeff0_4*dmats0[4][39] + coeff0_5*dmats0[5][39] + coeff0_6*dmats0[6][39] + coeff0_7*dmats0[7][39] + coeff0_8*dmats0[8][39] + coeff0_9*dmats0[9][39] + coeff0_10*dmats0[10][39] + coeff0_11*dmats0[11][39] + coeff0_12*dmats0[12][39] + coeff0_13*dmats0[13][39] + coeff0_14*dmats0[14][39] + coeff0_15*dmats0[15][39] + coeff0_16*dmats0[16][39] + coeff0_17*dmats0[17][39] + coeff0_18*dmats0[18][39] + coeff0_19*dmats0[19][39] + coeff0_20*dmats0[20][39] + coeff0_21*dmats0[21][39] + coeff0_22*dmats0[22][39] + coeff0_23*dmats0[23][39] + coeff0_24*dmats0[24][39] + coeff0_25*dmats0[25][39] + coeff0_26*dmats0[26][39] + coeff0_27*dmats0[27][39] + coeff0_28*dmats0[28][39] + coeff0_29*dmats0[29][39] + coeff0_30*dmats0[30][39] + coeff0_31*dmats0[31][39] + coeff0_32*dmats0[32][39] + coeff0_33*dmats0[33][39] + coeff0_34*dmats0[34][39] + coeff0_35*dmats0[35][39] + coeff0_36*dmats0[36][39] + coeff0_37*dmats0[37][39] + coeff0_38*dmats0[38][39] + coeff0_39*dmats0[39][39] + coeff0_40*dmats0[40][39] + coeff0_41*dmats0[41][39] + coeff0_42*dmats0[42][39] + coeff0_43*dmats0[43][39] + coeff0_44*dmats0[44][39] + coeff0_45*dmats0[45][39] + coeff0_46*dmats0[46][39] + coeff0_47*dmats0[47][39] + coeff0_48*dmats0[48][39] + coeff0_49*dmats0[49][39] + coeff0_50*dmats0[50][39] + coeff0_51*dmats0[51][39] + coeff0_52*dmats0[52][39] + coeff0_53*dmats0[53][39] + coeff0_54*dmats0[54][39] + coeff0_55*dmats0[55][39];
          new_coeff0_40 = coeff0_0*dmats0[0][40] + coeff0_1*dmats0[1][40] + coeff0_2*dmats0[2][40] + coeff0_3*dmats0[3][40] + coeff0_4*dmats0[4][40] + coeff0_5*dmats0[5][40] + coeff0_6*dmats0[6][40] + coeff0_7*dmats0[7][40] + coeff0_8*dmats0[8][40] + coeff0_9*dmats0[9][40] + coeff0_10*dmats0[10][40] + coeff0_11*dmats0[11][40] + coeff0_12*dmats0[12][40] + coeff0_13*dmats0[13][40] + coeff0_14*dmats0[14][40] + coeff0_15*dmats0[15][40] + coeff0_16*dmats0[16][40] + coeff0_17*dmats0[17][40] + coeff0_18*dmats0[18][40] + coeff0_19*dmats0[19][40] + coeff0_20*dmats0[20][40] + coeff0_21*dmats0[21][40] + coeff0_22*dmats0[22][40] + coeff0_23*dmats0[23][40] + coeff0_24*dmats0[24][40] + coeff0_25*dmats0[25][40] + coeff0_26*dmats0[26][40] + coeff0_27*dmats0[27][40] + coeff0_28*dmats0[28][40] + coeff0_29*dmats0[29][40] + coeff0_30*dmats0[30][40] + coeff0_31*dmats0[31][40] + coeff0_32*dmats0[32][40] + coeff0_33*dmats0[33][40] + coeff0_34*dmats0[34][40] + coeff0_35*dmats0[35][40] + coeff0_36*dmats0[36][40] + coeff0_37*dmats0[37][40] + coeff0_38*dmats0[38][40] + coeff0_39*dmats0[39][40] + coeff0_40*dmats0[40][40] + coeff0_41*dmats0[41][40] + coeff0_42*dmats0[42][40] + coeff0_43*dmats0[43][40] + coeff0_44*dmats0[44][40] + coeff0_45*dmats0[45][40] + coeff0_46*dmats0[46][40] + coeff0_47*dmats0[47][40] + coeff0_48*dmats0[48][40] + coeff0_49*dmats0[49][40] + coeff0_50*dmats0[50][40] + coeff0_51*dmats0[51][40] + coeff0_52*dmats0[52][40] + coeff0_53*dmats0[53][40] + coeff0_54*dmats0[54][40] + coeff0_55*dmats0[55][40];
          new_coeff0_41 = coeff0_0*dmats0[0][41] + coeff0_1*dmats0[1][41] + coeff0_2*dmats0[2][41] + coeff0_3*dmats0[3][41] + coeff0_4*dmats0[4][41] + coeff0_5*dmats0[5][41] + coeff0_6*dmats0[6][41] + coeff0_7*dmats0[7][41] + coeff0_8*dmats0[8][41] + coeff0_9*dmats0[9][41] + coeff0_10*dmats0[10][41] + coeff0_11*dmats0[11][41] + coeff0_12*dmats0[12][41] + coeff0_13*dmats0[13][41] + coeff0_14*dmats0[14][41] + coeff0_15*dmats0[15][41] + coeff0_16*dmats0[16][41] + coeff0_17*dmats0[17][41] + coeff0_18*dmats0[18][41] + coeff0_19*dmats0[19][41] + coeff0_20*dmats0[20][41] + coeff0_21*dmats0[21][41] + coeff0_22*dmats0[22][41] + coeff0_23*dmats0[23][41] + coeff0_24*dmats0[24][41] + coeff0_25*dmats0[25][41] + coeff0_26*dmats0[26][41] + coeff0_27*dmats0[27][41] + coeff0_28*dmats0[28][41] + coeff0_29*dmats0[29][41] + coeff0_30*dmats0[30][41] + coeff0_31*dmats0[31][41] + coeff0_32*dmats0[32][41] + coeff0_33*dmats0[33][41] + coeff0_34*dmats0[34][41] + coeff0_35*dmats0[35][41] + coeff0_36*dmats0[36][41] + coeff0_37*dmats0[37][41] + coeff0_38*dmats0[38][41] + coeff0_39*dmats0[39][41] + coeff0_40*dmats0[40][41] + coeff0_41*dmats0[41][41] + coeff0_42*dmats0[42][41] + coeff0_43*dmats0[43][41] + coeff0_44*dmats0[44][41] + coeff0_45*dmats0[45][41] + coeff0_46*dmats0[46][41] + coeff0_47*dmats0[47][41] + coeff0_48*dmats0[48][41] + coeff0_49*dmats0[49][41] + coeff0_50*dmats0[50][41] + coeff0_51*dmats0[51][41] + coeff0_52*dmats0[52][41] + coeff0_53*dmats0[53][41] + coeff0_54*dmats0[54][41] + coeff0_55*dmats0[55][41];
          new_coeff0_42 = coeff0_0*dmats0[0][42] + coeff0_1*dmats0[1][42] + coeff0_2*dmats0[2][42] + coeff0_3*dmats0[3][42] + coeff0_4*dmats0[4][42] + coeff0_5*dmats0[5][42] + coeff0_6*dmats0[6][42] + coeff0_7*dmats0[7][42] + coeff0_8*dmats0[8][42] + coeff0_9*dmats0[9][42] + coeff0_10*dmats0[10][42] + coeff0_11*dmats0[11][42] + coeff0_12*dmats0[12][42] + coeff0_13*dmats0[13][42] + coeff0_14*dmats0[14][42] + coeff0_15*dmats0[15][42] + coeff0_16*dmats0[16][42] + coeff0_17*dmats0[17][42] + coeff0_18*dmats0[18][42] + coeff0_19*dmats0[19][42] + coeff0_20*dmats0[20][42] + coeff0_21*dmats0[21][42] + coeff0_22*dmats0[22][42] + coeff0_23*dmats0[23][42] + coeff0_24*dmats0[24][42] + coeff0_25*dmats0[25][42] + coeff0_26*dmats0[26][42] + coeff0_27*dmats0[27][42] + coeff0_28*dmats0[28][42] + coeff0_29*dmats0[29][42] + coeff0_30*dmats0[30][42] + coeff0_31*dmats0[31][42] + coeff0_32*dmats0[32][42] + coeff0_33*dmats0[33][42] + coeff0_34*dmats0[34][42] + coeff0_35*dmats0[35][42] + coeff0_36*dmats0[36][42] + coeff0_37*dmats0[37][42] + coeff0_38*dmats0[38][42] + coeff0_39*dmats0[39][42] + coeff0_40*dmats0[40][42] + coeff0_41*dmats0[41][42] + coeff0_42*dmats0[42][42] + coeff0_43*dmats0[43][42] + coeff0_44*dmats0[44][42] + coeff0_45*dmats0[45][42] + coeff0_46*dmats0[46][42] + coeff0_47*dmats0[47][42] + coeff0_48*dmats0[48][42] + coeff0_49*dmats0[49][42] + coeff0_50*dmats0[50][42] + coeff0_51*dmats0[51][42] + coeff0_52*dmats0[52][42] + coeff0_53*dmats0[53][42] + coeff0_54*dmats0[54][42] + coeff0_55*dmats0[55][42];
          new_coeff0_43 = coeff0_0*dmats0[0][43] + coeff0_1*dmats0[1][43] + coeff0_2*dmats0[2][43] + coeff0_3*dmats0[3][43] + coeff0_4*dmats0[4][43] + coeff0_5*dmats0[5][43] + coeff0_6*dmats0[6][43] + coeff0_7*dmats0[7][43] + coeff0_8*dmats0[8][43] + coeff0_9*dmats0[9][43] + coeff0_10*dmats0[10][43] + coeff0_11*dmats0[11][43] + coeff0_12*dmats0[12][43] + coeff0_13*dmats0[13][43] + coeff0_14*dmats0[14][43] + coeff0_15*dmats0[15][43] + coeff0_16*dmats0[16][43] + coeff0_17*dmats0[17][43] + coeff0_18*dmats0[18][43] + coeff0_19*dmats0[19][43] + coeff0_20*dmats0[20][43] + coeff0_21*dmats0[21][43] + coeff0_22*dmats0[22][43] + coeff0_23*dmats0[23][43] + coeff0_24*dmats0[24][43] + coeff0_25*dmats0[25][43] + coeff0_26*dmats0[26][43] + coeff0_27*dmats0[27][43] + coeff0_28*dmats0[28][43] + coeff0_29*dmats0[29][43] + coeff0_30*dmats0[30][43] + coeff0_31*dmats0[31][43] + coeff0_32*dmats0[32][43] + coeff0_33*dmats0[33][43] + coeff0_34*dmats0[34][43] + coeff0_35*dmats0[35][43] + coeff0_36*dmats0[36][43] + coeff0_37*dmats0[37][43] + coeff0_38*dmats0[38][43] + coeff0_39*dmats0[39][43] + coeff0_40*dmats0[40][43] + coeff0_41*dmats0[41][43] + coeff0_42*dmats0[42][43] + coeff0_43*dmats0[43][43] + coeff0_44*dmats0[44][43] + coeff0_45*dmats0[45][43] + coeff0_46*dmats0[46][43] + coeff0_47*dmats0[47][43] + coeff0_48*dmats0[48][43] + coeff0_49*dmats0[49][43] + coeff0_50*dmats0[50][43] + coeff0_51*dmats0[51][43] + coeff0_52*dmats0[52][43] + coeff0_53*dmats0[53][43] + coeff0_54*dmats0[54][43] + coeff0_55*dmats0[55][43];
          new_coeff0_44 = coeff0_0*dmats0[0][44] + coeff0_1*dmats0[1][44] + coeff0_2*dmats0[2][44] + coeff0_3*dmats0[3][44] + coeff0_4*dmats0[4][44] + coeff0_5*dmats0[5][44] + coeff0_6*dmats0[6][44] + coeff0_7*dmats0[7][44] + coeff0_8*dmats0[8][44] + coeff0_9*dmats0[9][44] + coeff0_10*dmats0[10][44] + coeff0_11*dmats0[11][44] + coeff0_12*dmats0[12][44] + coeff0_13*dmats0[13][44] + coeff0_14*dmats0[14][44] + coeff0_15*dmats0[15][44] + coeff0_16*dmats0[16][44] + coeff0_17*dmats0[17][44] + coeff0_18*dmats0[18][44] + coeff0_19*dmats0[19][44] + coeff0_20*dmats0[20][44] + coeff0_21*dmats0[21][44] + coeff0_22*dmats0[22][44] + coeff0_23*dmats0[23][44] + coeff0_24*dmats0[24][44] + coeff0_25*dmats0[25][44] + coeff0_26*dmats0[26][44] + coeff0_27*dmats0[27][44] + coeff0_28*dmats0[28][44] + coeff0_29*dmats0[29][44] + coeff0_30*dmats0[30][44] + coeff0_31*dmats0[31][44] + coeff0_32*dmats0[32][44] + coeff0_33*dmats0[33][44] + coeff0_34*dmats0[34][44] + coeff0_35*dmats0[35][44] + coeff0_36*dmats0[36][44] + coeff0_37*dmats0[37][44] + coeff0_38*dmats0[38][44] + coeff0_39*dmats0[39][44] + coeff0_40*dmats0[40][44] + coeff0_41*dmats0[41][44] + coeff0_42*dmats0[42][44] + coeff0_43*dmats0[43][44] + coeff0_44*dmats0[44][44] + coeff0_45*dmats0[45][44] + coeff0_46*dmats0[46][44] + coeff0_47*dmats0[47][44] + coeff0_48*dmats0[48][44] + coeff0_49*dmats0[49][44] + coeff0_50*dmats0[50][44] + coeff0_51*dmats0[51][44] + coeff0_52*dmats0[52][44] + coeff0_53*dmats0[53][44] + coeff0_54*dmats0[54][44] + coeff0_55*dmats0[55][44];
          new_coeff0_45 = coeff0_0*dmats0[0][45] + coeff0_1*dmats0[1][45] + coeff0_2*dmats0[2][45] + coeff0_3*dmats0[3][45] + coeff0_4*dmats0[4][45] + coeff0_5*dmats0[5][45] + coeff0_6*dmats0[6][45] + coeff0_7*dmats0[7][45] + coeff0_8*dmats0[8][45] + coeff0_9*dmats0[9][45] + coeff0_10*dmats0[10][45] + coeff0_11*dmats0[11][45] + coeff0_12*dmats0[12][45] + coeff0_13*dmats0[13][45] + coeff0_14*dmats0[14][45] + coeff0_15*dmats0[15][45] + coeff0_16*dmats0[16][45] + coeff0_17*dmats0[17][45] + coeff0_18*dmats0[18][45] + coeff0_19*dmats0[19][45] + coeff0_20*dmats0[20][45] + coeff0_21*dmats0[21][45] + coeff0_22*dmats0[22][45] + coeff0_23*dmats0[23][45] + coeff0_24*dmats0[24][45] + coeff0_25*dmats0[25][45] + coeff0_26*dmats0[26][45] + coeff0_27*dmats0[27][45] + coeff0_28*dmats0[28][45] + coeff0_29*dmats0[29][45] + coeff0_30*dmats0[30][45] + coeff0_31*dmats0[31][45] + coeff0_32*dmats0[32][45] + coeff0_33*dmats0[33][45] + coeff0_34*dmats0[34][45] + coeff0_35*dmats0[35][45] + coeff0_36*dmats0[36][45] + coeff0_37*dmats0[37][45] + coeff0_38*dmats0[38][45] + coeff0_39*dmats0[39][45] + coeff0_40*dmats0[40][45] + coeff0_41*dmats0[41][45] + coeff0_42*dmats0[42][45] + coeff0_43*dmats0[43][45] + coeff0_44*dmats0[44][45] + coeff0_45*dmats0[45][45] + coeff0_46*dmats0[46][45] + coeff0_47*dmats0[47][45] + coeff0_48*dmats0[48][45] + coeff0_49*dmats0[49][45] + coeff0_50*dmats0[50][45] + coeff0_51*dmats0[51][45] + coeff0_52*dmats0[52][45] + coeff0_53*dmats0[53][45] + coeff0_54*dmats0[54][45] + coeff0_55*dmats0[55][45];
          new_coeff0_46 = coeff0_0*dmats0[0][46] + coeff0_1*dmats0[1][46] + coeff0_2*dmats0[2][46] + coeff0_3*dmats0[3][46] + coeff0_4*dmats0[4][46] + coeff0_5*dmats0[5][46] + coeff0_6*dmats0[6][46] + coeff0_7*dmats0[7][46] + coeff0_8*dmats0[8][46] + coeff0_9*dmats0[9][46] + coeff0_10*dmats0[10][46] + coeff0_11*dmats0[11][46] + coeff0_12*dmats0[12][46] + coeff0_13*dmats0[13][46] + coeff0_14*dmats0[14][46] + coeff0_15*dmats0[15][46] + coeff0_16*dmats0[16][46] + coeff0_17*dmats0[17][46] + coeff0_18*dmats0[18][46] + coeff0_19*dmats0[19][46] + coeff0_20*dmats0[20][46] + coeff0_21*dmats0[21][46] + coeff0_22*dmats0[22][46] + coeff0_23*dmats0[23][46] + coeff0_24*dmats0[24][46] + coeff0_25*dmats0[25][46] + coeff0_26*dmats0[26][46] + coeff0_27*dmats0[27][46] + coeff0_28*dmats0[28][46] + coeff0_29*dmats0[29][46] + coeff0_30*dmats0[30][46] + coeff0_31*dmats0[31][46] + coeff0_32*dmats0[32][46] + coeff0_33*dmats0[33][46] + coeff0_34*dmats0[34][46] + coeff0_35*dmats0[35][46] + coeff0_36*dmats0[36][46] + coeff0_37*dmats0[37][46] + coeff0_38*dmats0[38][46] + coeff0_39*dmats0[39][46] + coeff0_40*dmats0[40][46] + coeff0_41*dmats0[41][46] + coeff0_42*dmats0[42][46] + coeff0_43*dmats0[43][46] + coeff0_44*dmats0[44][46] + coeff0_45*dmats0[45][46] + coeff0_46*dmats0[46][46] + coeff0_47*dmats0[47][46] + coeff0_48*dmats0[48][46] + coeff0_49*dmats0[49][46] + coeff0_50*dmats0[50][46] + coeff0_51*dmats0[51][46] + coeff0_52*dmats0[52][46] + coeff0_53*dmats0[53][46] + coeff0_54*dmats0[54][46] + coeff0_55*dmats0[55][46];
          new_coeff0_47 = coeff0_0*dmats0[0][47] + coeff0_1*dmats0[1][47] + coeff0_2*dmats0[2][47] + coeff0_3*dmats0[3][47] + coeff0_4*dmats0[4][47] + coeff0_5*dmats0[5][47] + coeff0_6*dmats0[6][47] + coeff0_7*dmats0[7][47] + coeff0_8*dmats0[8][47] + coeff0_9*dmats0[9][47] + coeff0_10*dmats0[10][47] + coeff0_11*dmats0[11][47] + coeff0_12*dmats0[12][47] + coeff0_13*dmats0[13][47] + coeff0_14*dmats0[14][47] + coeff0_15*dmats0[15][47] + coeff0_16*dmats0[16][47] + coeff0_17*dmats0[17][47] + coeff0_18*dmats0[18][47] + coeff0_19*dmats0[19][47] + coeff0_20*dmats0[20][47] + coeff0_21*dmats0[21][47] + coeff0_22*dmats0[22][47] + coeff0_23*dmats0[23][47] + coeff0_24*dmats0[24][47] + coeff0_25*dmats0[25][47] + coeff0_26*dmats0[26][47] + coeff0_27*dmats0[27][47] + coeff0_28*dmats0[28][47] + coeff0_29*dmats0[29][47] + coeff0_30*dmats0[30][47] + coeff0_31*dmats0[31][47] + coeff0_32*dmats0[32][47] + coeff0_33*dmats0[33][47] + coeff0_34*dmats0[34][47] + coeff0_35*dmats0[35][47] + coeff0_36*dmats0[36][47] + coeff0_37*dmats0[37][47] + coeff0_38*dmats0[38][47] + coeff0_39*dmats0[39][47] + coeff0_40*dmats0[40][47] + coeff0_41*dmats0[41][47] + coeff0_42*dmats0[42][47] + coeff0_43*dmats0[43][47] + coeff0_44*dmats0[44][47] + coeff0_45*dmats0[45][47] + coeff0_46*dmats0[46][47] + coeff0_47*dmats0[47][47] + coeff0_48*dmats0[48][47] + coeff0_49*dmats0[49][47] + coeff0_50*dmats0[50][47] + coeff0_51*dmats0[51][47] + coeff0_52*dmats0[52][47] + coeff0_53*dmats0[53][47] + coeff0_54*dmats0[54][47] + coeff0_55*dmats0[55][47];
          new_coeff0_48 = coeff0_0*dmats0[0][48] + coeff0_1*dmats0[1][48] + coeff0_2*dmats0[2][48] + coeff0_3*dmats0[3][48] + coeff0_4*dmats0[4][48] + coeff0_5*dmats0[5][48] + coeff0_6*dmats0[6][48] + coeff0_7*dmats0[7][48] + coeff0_8*dmats0[8][48] + coeff0_9*dmats0[9][48] + coeff0_10*dmats0[10][48] + coeff0_11*dmats0[11][48] + coeff0_12*dmats0[12][48] + coeff0_13*dmats0[13][48] + coeff0_14*dmats0[14][48] + coeff0_15*dmats0[15][48] + coeff0_16*dmats0[16][48] + coeff0_17*dmats0[17][48] + coeff0_18*dmats0[18][48] + coeff0_19*dmats0[19][48] + coeff0_20*dmats0[20][48] + coeff0_21*dmats0[21][48] + coeff0_22*dmats0[22][48] + coeff0_23*dmats0[23][48] + coeff0_24*dmats0[24][48] + coeff0_25*dmats0[25][48] + coeff0_26*dmats0[26][48] + coeff0_27*dmats0[27][48] + coeff0_28*dmats0[28][48] + coeff0_29*dmats0[29][48] + coeff0_30*dmats0[30][48] + coeff0_31*dmats0[31][48] + coeff0_32*dmats0[32][48] + coeff0_33*dmats0[33][48] + coeff0_34*dmats0[34][48] + coeff0_35*dmats0[35][48] + coeff0_36*dmats0[36][48] + coeff0_37*dmats0[37][48] + coeff0_38*dmats0[38][48] + coeff0_39*dmats0[39][48] + coeff0_40*dmats0[40][48] + coeff0_41*dmats0[41][48] + coeff0_42*dmats0[42][48] + coeff0_43*dmats0[43][48] + coeff0_44*dmats0[44][48] + coeff0_45*dmats0[45][48] + coeff0_46*dmats0[46][48] + coeff0_47*dmats0[47][48] + coeff0_48*dmats0[48][48] + coeff0_49*dmats0[49][48] + coeff0_50*dmats0[50][48] + coeff0_51*dmats0[51][48] + coeff0_52*dmats0[52][48] + coeff0_53*dmats0[53][48] + coeff0_54*dmats0[54][48] + coeff0_55*dmats0[55][48];
          new_coeff0_49 = coeff0_0*dmats0[0][49] + coeff0_1*dmats0[1][49] + coeff0_2*dmats0[2][49] + coeff0_3*dmats0[3][49] + coeff0_4*dmats0[4][49] + coeff0_5*dmats0[5][49] + coeff0_6*dmats0[6][49] + coeff0_7*dmats0[7][49] + coeff0_8*dmats0[8][49] + coeff0_9*dmats0[9][49] + coeff0_10*dmats0[10][49] + coeff0_11*dmats0[11][49] + coeff0_12*dmats0[12][49] + coeff0_13*dmats0[13][49] + coeff0_14*dmats0[14][49] + coeff0_15*dmats0[15][49] + coeff0_16*dmats0[16][49] + coeff0_17*dmats0[17][49] + coeff0_18*dmats0[18][49] + coeff0_19*dmats0[19][49] + coeff0_20*dmats0[20][49] + coeff0_21*dmats0[21][49] + coeff0_22*dmats0[22][49] + coeff0_23*dmats0[23][49] + coeff0_24*dmats0[24][49] + coeff0_25*dmats0[25][49] + coeff0_26*dmats0[26][49] + coeff0_27*dmats0[27][49] + coeff0_28*dmats0[28][49] + coeff0_29*dmats0[29][49] + coeff0_30*dmats0[30][49] + coeff0_31*dmats0[31][49] + coeff0_32*dmats0[32][49] + coeff0_33*dmats0[33][49] + coeff0_34*dmats0[34][49] + coeff0_35*dmats0[35][49] + coeff0_36*dmats0[36][49] + coeff0_37*dmats0[37][49] + coeff0_38*dmats0[38][49] + coeff0_39*dmats0[39][49] + coeff0_40*dmats0[40][49] + coeff0_41*dmats0[41][49] + coeff0_42*dmats0[42][49] + coeff0_43*dmats0[43][49] + coeff0_44*dmats0[44][49] + coeff0_45*dmats0[45][49] + coeff0_46*dmats0[46][49] + coeff0_47*dmats0[47][49] + coeff0_48*dmats0[48][49] + coeff0_49*dmats0[49][49] + coeff0_50*dmats0[50][49] + coeff0_51*dmats0[51][49] + coeff0_52*dmats0[52][49] + coeff0_53*dmats0[53][49] + coeff0_54*dmats0[54][49] + coeff0_55*dmats0[55][49];
          new_coeff0_50 = coeff0_0*dmats0[0][50] + coeff0_1*dmats0[1][50] + coeff0_2*dmats0[2][50] + coeff0_3*dmats0[3][50] + coeff0_4*dmats0[4][50] + coeff0_5*dmats0[5][50] + coeff0_6*dmats0[6][50] + coeff0_7*dmats0[7][50] + coeff0_8*dmats0[8][50] + coeff0_9*dmats0[9][50] + coeff0_10*dmats0[10][50] + coeff0_11*dmats0[11][50] + coeff0_12*dmats0[12][50] + coeff0_13*dmats0[13][50] + coeff0_14*dmats0[14][50] + coeff0_15*dmats0[15][50] + coeff0_16*dmats0[16][50] + coeff0_17*dmats0[17][50] + coeff0_18*dmats0[18][50] + coeff0_19*dmats0[19][50] + coeff0_20*dmats0[20][50] + coeff0_21*dmats0[21][50] + coeff0_22*dmats0[22][50] + coeff0_23*dmats0[23][50] + coeff0_24*dmats0[24][50] + coeff0_25*dmats0[25][50] + coeff0_26*dmats0[26][50] + coeff0_27*dmats0[27][50] + coeff0_28*dmats0[28][50] + coeff0_29*dmats0[29][50] + coeff0_30*dmats0[30][50] + coeff0_31*dmats0[31][50] + coeff0_32*dmats0[32][50] + coeff0_33*dmats0[33][50] + coeff0_34*dmats0[34][50] + coeff0_35*dmats0[35][50] + coeff0_36*dmats0[36][50] + coeff0_37*dmats0[37][50] + coeff0_38*dmats0[38][50] + coeff0_39*dmats0[39][50] + coeff0_40*dmats0[40][50] + coeff0_41*dmats0[41][50] + coeff0_42*dmats0[42][50] + coeff0_43*dmats0[43][50] + coeff0_44*dmats0[44][50] + coeff0_45*dmats0[45][50] + coeff0_46*dmats0[46][50] + coeff0_47*dmats0[47][50] + coeff0_48*dmats0[48][50] + coeff0_49*dmats0[49][50] + coeff0_50*dmats0[50][50] + coeff0_51*dmats0[51][50] + coeff0_52*dmats0[52][50] + coeff0_53*dmats0[53][50] + coeff0_54*dmats0[54][50] + coeff0_55*dmats0[55][50];
          new_coeff0_51 = coeff0_0*dmats0[0][51] + coeff0_1*dmats0[1][51] + coeff0_2*dmats0[2][51] + coeff0_3*dmats0[3][51] + coeff0_4*dmats0[4][51] + coeff0_5*dmats0[5][51] + coeff0_6*dmats0[6][51] + coeff0_7*dmats0[7][51] + coeff0_8*dmats0[8][51] + coeff0_9*dmats0[9][51] + coeff0_10*dmats0[10][51] + coeff0_11*dmats0[11][51] + coeff0_12*dmats0[12][51] + coeff0_13*dmats0[13][51] + coeff0_14*dmats0[14][51] + coeff0_15*dmats0[15][51] + coeff0_16*dmats0[16][51] + coeff0_17*dmats0[17][51] + coeff0_18*dmats0[18][51] + coeff0_19*dmats0[19][51] + coeff0_20*dmats0[20][51] + coeff0_21*dmats0[21][51] + coeff0_22*dmats0[22][51] + coeff0_23*dmats0[23][51] + coeff0_24*dmats0[24][51] + coeff0_25*dmats0[25][51] + coeff0_26*dmats0[26][51] + coeff0_27*dmats0[27][51] + coeff0_28*dmats0[28][51] + coeff0_29*dmats0[29][51] + coeff0_30*dmats0[30][51] + coeff0_31*dmats0[31][51] + coeff0_32*dmats0[32][51] + coeff0_33*dmats0[33][51] + coeff0_34*dmats0[34][51] + coeff0_35*dmats0[35][51] + coeff0_36*dmats0[36][51] + coeff0_37*dmats0[37][51] + coeff0_38*dmats0[38][51] + coeff0_39*dmats0[39][51] + coeff0_40*dmats0[40][51] + coeff0_41*dmats0[41][51] + coeff0_42*dmats0[42][51] + coeff0_43*dmats0[43][51] + coeff0_44*dmats0[44][51] + coeff0_45*dmats0[45][51] + coeff0_46*dmats0[46][51] + coeff0_47*dmats0[47][51] + coeff0_48*dmats0[48][51] + coeff0_49*dmats0[49][51] + coeff0_50*dmats0[50][51] + coeff0_51*dmats0[51][51] + coeff0_52*dmats0[52][51] + coeff0_53*dmats0[53][51] + coeff0_54*dmats0[54][51] + coeff0_55*dmats0[55][51];
          new_coeff0_52 = coeff0_0*dmats0[0][52] + coeff0_1*dmats0[1][52] + coeff0_2*dmats0[2][52] + coeff0_3*dmats0[3][52] + coeff0_4*dmats0[4][52] + coeff0_5*dmats0[5][52] + coeff0_6*dmats0[6][52] + coeff0_7*dmats0[7][52] + coeff0_8*dmats0[8][52] + coeff0_9*dmats0[9][52] + coeff0_10*dmats0[10][52] + coeff0_11*dmats0[11][52] + coeff0_12*dmats0[12][52] + coeff0_13*dmats0[13][52] + coeff0_14*dmats0[14][52] + coeff0_15*dmats0[15][52] + coeff0_16*dmats0[16][52] + coeff0_17*dmats0[17][52] + coeff0_18*dmats0[18][52] + coeff0_19*dmats0[19][52] + coeff0_20*dmats0[20][52] + coeff0_21*dmats0[21][52] + coeff0_22*dmats0[22][52] + coeff0_23*dmats0[23][52] + coeff0_24*dmats0[24][52] + coeff0_25*dmats0[25][52] + coeff0_26*dmats0[26][52] + coeff0_27*dmats0[27][52] + coeff0_28*dmats0[28][52] + coeff0_29*dmats0[29][52] + coeff0_30*dmats0[30][52] + coeff0_31*dmats0[31][52] + coeff0_32*dmats0[32][52] + coeff0_33*dmats0[33][52] + coeff0_34*dmats0[34][52] + coeff0_35*dmats0[35][52] + coeff0_36*dmats0[36][52] + coeff0_37*dmats0[37][52] + coeff0_38*dmats0[38][52] + coeff0_39*dmats0[39][52] + coeff0_40*dmats0[40][52] + coeff0_41*dmats0[41][52] + coeff0_42*dmats0[42][52] + coeff0_43*dmats0[43][52] + coeff0_44*dmats0[44][52] + coeff0_45*dmats0[45][52] + coeff0_46*dmats0[46][52] + coeff0_47*dmats0[47][52] + coeff0_48*dmats0[48][52] + coeff0_49*dmats0[49][52] + coeff0_50*dmats0[50][52] + coeff0_51*dmats0[51][52] + coeff0_52*dmats0[52][52] + coeff0_53*dmats0[53][52] + coeff0_54*dmats0[54][52] + coeff0_55*dmats0[55][52];
          new_coeff0_53 = coeff0_0*dmats0[0][53] + coeff0_1*dmats0[1][53] + coeff0_2*dmats0[2][53] + coeff0_3*dmats0[3][53] + coeff0_4*dmats0[4][53] + coeff0_5*dmats0[5][53] + coeff0_6*dmats0[6][53] + coeff0_7*dmats0[7][53] + coeff0_8*dmats0[8][53] + coeff0_9*dmats0[9][53] + coeff0_10*dmats0[10][53] + coeff0_11*dmats0[11][53] + coeff0_12*dmats0[12][53] + coeff0_13*dmats0[13][53] + coeff0_14*dmats0[14][53] + coeff0_15*dmats0[15][53] + coeff0_16*dmats0[16][53] + coeff0_17*dmats0[17][53] + coeff0_18*dmats0[18][53] + coeff0_19*dmats0[19][53] + coeff0_20*dmats0[20][53] + coeff0_21*dmats0[21][53] + coeff0_22*dmats0[22][53] + coeff0_23*dmats0[23][53] + coeff0_24*dmats0[24][53] + coeff0_25*dmats0[25][53] + coeff0_26*dmats0[26][53] + coeff0_27*dmats0[27][53] + coeff0_28*dmats0[28][53] + coeff0_29*dmats0[29][53] + coeff0_30*dmats0[30][53] + coeff0_31*dmats0[31][53] + coeff0_32*dmats0[32][53] + coeff0_33*dmats0[33][53] + coeff0_34*dmats0[34][53] + coeff0_35*dmats0[35][53] + coeff0_36*dmats0[36][53] + coeff0_37*dmats0[37][53] + coeff0_38*dmats0[38][53] + coeff0_39*dmats0[39][53] + coeff0_40*dmats0[40][53] + coeff0_41*dmats0[41][53] + coeff0_42*dmats0[42][53] + coeff0_43*dmats0[43][53] + coeff0_44*dmats0[44][53] + coeff0_45*dmats0[45][53] + coeff0_46*dmats0[46][53] + coeff0_47*dmats0[47][53] + coeff0_48*dmats0[48][53] + coeff0_49*dmats0[49][53] + coeff0_50*dmats0[50][53] + coeff0_51*dmats0[51][53] + coeff0_52*dmats0[52][53] + coeff0_53*dmats0[53][53] + coeff0_54*dmats0[54][53] + coeff0_55*dmats0[55][53];
          new_coeff0_54 = coeff0_0*dmats0[0][54] + coeff0_1*dmats0[1][54] + coeff0_2*dmats0[2][54] + coeff0_3*dmats0[3][54] + coeff0_4*dmats0[4][54] + coeff0_5*dmats0[5][54] + coeff0_6*dmats0[6][54] + coeff0_7*dmats0[7][54] + coeff0_8*dmats0[8][54] + coeff0_9*dmats0[9][54] + coeff0_10*dmats0[10][54] + coeff0_11*dmats0[11][54] + coeff0_12*dmats0[12][54] + coeff0_13*dmats0[13][54] + coeff0_14*dmats0[14][54] + coeff0_15*dmats0[15][54] + coeff0_16*dmats0[16][54] + coeff0_17*dmats0[17][54] + coeff0_18*dmats0[18][54] + coeff0_19*dmats0[19][54] + coeff0_20*dmats0[20][54] + coeff0_21*dmats0[21][54] + coeff0_22*dmats0[22][54] + coeff0_23*dmats0[23][54] + coeff0_24*dmats0[24][54] + coeff0_25*dmats0[25][54] + coeff0_26*dmats0[26][54] + coeff0_27*dmats0[27][54] + coeff0_28*dmats0[28][54] + coeff0_29*dmats0[29][54] + coeff0_30*dmats0[30][54] + coeff0_31*dmats0[31][54] + coeff0_32*dmats0[32][54] + coeff0_33*dmats0[33][54] + coeff0_34*dmats0[34][54] + coeff0_35*dmats0[35][54] + coeff0_36*dmats0[36][54] + coeff0_37*dmats0[37][54] + coeff0_38*dmats0[38][54] + coeff0_39*dmats0[39][54] + coeff0_40*dmats0[40][54] + coeff0_41*dmats0[41][54] + coeff0_42*dmats0[42][54] + coeff0_43*dmats0[43][54] + coeff0_44*dmats0[44][54] + coeff0_45*dmats0[45][54] + coeff0_46*dmats0[46][54] + coeff0_47*dmats0[47][54] + coeff0_48*dmats0[48][54] + coeff0_49*dmats0[49][54] + coeff0_50*dmats0[50][54] + coeff0_51*dmats0[51][54] + coeff0_52*dmats0[52][54] + coeff0_53*dmats0[53][54] + coeff0_54*dmats0[54][54] + coeff0_55*dmats0[55][54];
          new_coeff0_55 = coeff0_0*dmats0[0][55] + coeff0_1*dmats0[1][55] + coeff0_2*dmats0[2][55] + coeff0_3*dmats0[3][55] + coeff0_4*dmats0[4][55] + coeff0_5*dmats0[5][55] + coeff0_6*dmats0[6][55] + coeff0_7*dmats0[7][55] + coeff0_8*dmats0[8][55] + coeff0_9*dmats0[9][55] + coeff0_10*dmats0[10][55] + coeff0_11*dmats0[11][55] + coeff0_12*dmats0[12][55] + coeff0_13*dmats0[13][55] + coeff0_14*dmats0[14][55] + coeff0_15*dmats0[15][55] + coeff0_16*dmats0[16][55] + coeff0_17*dmats0[17][55] + coeff0_18*dmats0[18][55] + coeff0_19*dmats0[19][55] + coeff0_20*dmats0[20][55] + coeff0_21*dmats0[21][55] + coeff0_22*dmats0[22][55] + coeff0_23*dmats0[23][55] + coeff0_24*dmats0[24][55] + coeff0_25*dmats0[25][55] + coeff0_26*dmats0[26][55] + coeff0_27*dmats0[27][55] + coeff0_28*dmats0[28][55] + coeff0_29*dmats0[29][55] + coeff0_30*dmats0[30][55] + coeff0_31*dmats0[31][55] + coeff0_32*dmats0[32][55] + coeff0_33*dmats0[33][55] + coeff0_34*dmats0[34][55] + coeff0_35*dmats0[35][55] + coeff0_36*dmats0[36][55] + coeff0_37*dmats0[37][55] + coeff0_38*dmats0[38][55] + coeff0_39*dmats0[39][55] + coeff0_40*dmats0[40][55] + coeff0_41*dmats0[41][55] + coeff0_42*dmats0[42][55] + coeff0_43*dmats0[43][55] + coeff0_44*dmats0[44][55] + coeff0_45*dmats0[45][55] + coeff0_46*dmats0[46][55] + coeff0_47*dmats0[47][55] + coeff0_48*dmats0[48][55] + coeff0_49*dmats0[49][55] + coeff0_50*dmats0[50][55] + coeff0_51*dmats0[51][55] + coeff0_52*dmats0[52][55] + coeff0_53*dmats0[53][55] + coeff0_54*dmats0[54][55] + coeff0_55*dmats0[55][55];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0] + coeff0_10*dmats1[10][0] + coeff0_11*dmats1[11][0] + coeff0_12*dmats1[12][0] + coeff0_13*dmats1[13][0] + coeff0_14*dmats1[14][0] + coeff0_15*dmats1[15][0] + coeff0_16*dmats1[16][0] + coeff0_17*dmats1[17][0] + coeff0_18*dmats1[18][0] + coeff0_19*dmats1[19][0] + coeff0_20*dmats1[20][0] + coeff0_21*dmats1[21][0] + coeff0_22*dmats1[22][0] + coeff0_23*dmats1[23][0] + coeff0_24*dmats1[24][0] + coeff0_25*dmats1[25][0] + coeff0_26*dmats1[26][0] + coeff0_27*dmats1[27][0] + coeff0_28*dmats1[28][0] + coeff0_29*dmats1[29][0] + coeff0_30*dmats1[30][0] + coeff0_31*dmats1[31][0] + coeff0_32*dmats1[32][0] + coeff0_33*dmats1[33][0] + coeff0_34*dmats1[34][0] + coeff0_35*dmats1[35][0] + coeff0_36*dmats1[36][0] + coeff0_37*dmats1[37][0] + coeff0_38*dmats1[38][0] + coeff0_39*dmats1[39][0] + coeff0_40*dmats1[40][0] + coeff0_41*dmats1[41][0] + coeff0_42*dmats1[42][0] + coeff0_43*dmats1[43][0] + coeff0_44*dmats1[44][0] + coeff0_45*dmats1[45][0] + coeff0_46*dmats1[46][0] + coeff0_47*dmats1[47][0] + coeff0_48*dmats1[48][0] + coeff0_49*dmats1[49][0] + coeff0_50*dmats1[50][0] + coeff0_51*dmats1[51][0] + coeff0_52*dmats1[52][0] + coeff0_53*dmats1[53][0] + coeff0_54*dmats1[54][0] + coeff0_55*dmats1[55][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1] + coeff0_10*dmats1[10][1] + coeff0_11*dmats1[11][1] + coeff0_12*dmats1[12][1] + coeff0_13*dmats1[13][1] + coeff0_14*dmats1[14][1] + coeff0_15*dmats1[15][1] + coeff0_16*dmats1[16][1] + coeff0_17*dmats1[17][1] + coeff0_18*dmats1[18][1] + coeff0_19*dmats1[19][1] + coeff0_20*dmats1[20][1] + coeff0_21*dmats1[21][1] + coeff0_22*dmats1[22][1] + coeff0_23*dmats1[23][1] + coeff0_24*dmats1[24][1] + coeff0_25*dmats1[25][1] + coeff0_26*dmats1[26][1] + coeff0_27*dmats1[27][1] + coeff0_28*dmats1[28][1] + coeff0_29*dmats1[29][1] + coeff0_30*dmats1[30][1] + coeff0_31*dmats1[31][1] + coeff0_32*dmats1[32][1] + coeff0_33*dmats1[33][1] + coeff0_34*dmats1[34][1] + coeff0_35*dmats1[35][1] + coeff0_36*dmats1[36][1] + coeff0_37*dmats1[37][1] + coeff0_38*dmats1[38][1] + coeff0_39*dmats1[39][1] + coeff0_40*dmats1[40][1] + coeff0_41*dmats1[41][1] + coeff0_42*dmats1[42][1] + coeff0_43*dmats1[43][1] + coeff0_44*dmats1[44][1] + coeff0_45*dmats1[45][1] + coeff0_46*dmats1[46][1] + coeff0_47*dmats1[47][1] + coeff0_48*dmats1[48][1] + coeff0_49*dmats1[49][1] + coeff0_50*dmats1[50][1] + coeff0_51*dmats1[51][1] + coeff0_52*dmats1[52][1] + coeff0_53*dmats1[53][1] + coeff0_54*dmats1[54][1] + coeff0_55*dmats1[55][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2] + coeff0_10*dmats1[10][2] + coeff0_11*dmats1[11][2] + coeff0_12*dmats1[12][2] + coeff0_13*dmats1[13][2] + coeff0_14*dmats1[14][2] + coeff0_15*dmats1[15][2] + coeff0_16*dmats1[16][2] + coeff0_17*dmats1[17][2] + coeff0_18*dmats1[18][2] + coeff0_19*dmats1[19][2] + coeff0_20*dmats1[20][2] + coeff0_21*dmats1[21][2] + coeff0_22*dmats1[22][2] + coeff0_23*dmats1[23][2] + coeff0_24*dmats1[24][2] + coeff0_25*dmats1[25][2] + coeff0_26*dmats1[26][2] + coeff0_27*dmats1[27][2] + coeff0_28*dmats1[28][2] + coeff0_29*dmats1[29][2] + coeff0_30*dmats1[30][2] + coeff0_31*dmats1[31][2] + coeff0_32*dmats1[32][2] + coeff0_33*dmats1[33][2] + coeff0_34*dmats1[34][2] + coeff0_35*dmats1[35][2] + coeff0_36*dmats1[36][2] + coeff0_37*dmats1[37][2] + coeff0_38*dmats1[38][2] + coeff0_39*dmats1[39][2] + coeff0_40*dmats1[40][2] + coeff0_41*dmats1[41][2] + coeff0_42*dmats1[42][2] + coeff0_43*dmats1[43][2] + coeff0_44*dmats1[44][2] + coeff0_45*dmats1[45][2] + coeff0_46*dmats1[46][2] + coeff0_47*dmats1[47][2] + coeff0_48*dmats1[48][2] + coeff0_49*dmats1[49][2] + coeff0_50*dmats1[50][2] + coeff0_51*dmats1[51][2] + coeff0_52*dmats1[52][2] + coeff0_53*dmats1[53][2] + coeff0_54*dmats1[54][2] + coeff0_55*dmats1[55][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3] + coeff0_10*dmats1[10][3] + coeff0_11*dmats1[11][3] + coeff0_12*dmats1[12][3] + coeff0_13*dmats1[13][3] + coeff0_14*dmats1[14][3] + coeff0_15*dmats1[15][3] + coeff0_16*dmats1[16][3] + coeff0_17*dmats1[17][3] + coeff0_18*dmats1[18][3] + coeff0_19*dmats1[19][3] + coeff0_20*dmats1[20][3] + coeff0_21*dmats1[21][3] + coeff0_22*dmats1[22][3] + coeff0_23*dmats1[23][3] + coeff0_24*dmats1[24][3] + coeff0_25*dmats1[25][3] + coeff0_26*dmats1[26][3] + coeff0_27*dmats1[27][3] + coeff0_28*dmats1[28][3] + coeff0_29*dmats1[29][3] + coeff0_30*dmats1[30][3] + coeff0_31*dmats1[31][3] + coeff0_32*dmats1[32][3] + coeff0_33*dmats1[33][3] + coeff0_34*dmats1[34][3] + coeff0_35*dmats1[35][3] + coeff0_36*dmats1[36][3] + coeff0_37*dmats1[37][3] + coeff0_38*dmats1[38][3] + coeff0_39*dmats1[39][3] + coeff0_40*dmats1[40][3] + coeff0_41*dmats1[41][3] + coeff0_42*dmats1[42][3] + coeff0_43*dmats1[43][3] + coeff0_44*dmats1[44][3] + coeff0_45*dmats1[45][3] + coeff0_46*dmats1[46][3] + coeff0_47*dmats1[47][3] + coeff0_48*dmats1[48][3] + coeff0_49*dmats1[49][3] + coeff0_50*dmats1[50][3] + coeff0_51*dmats1[51][3] + coeff0_52*dmats1[52][3] + coeff0_53*dmats1[53][3] + coeff0_54*dmats1[54][3] + coeff0_55*dmats1[55][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4] + coeff0_10*dmats1[10][4] + coeff0_11*dmats1[11][4] + coeff0_12*dmats1[12][4] + coeff0_13*dmats1[13][4] + coeff0_14*dmats1[14][4] + coeff0_15*dmats1[15][4] + coeff0_16*dmats1[16][4] + coeff0_17*dmats1[17][4] + coeff0_18*dmats1[18][4] + coeff0_19*dmats1[19][4] + coeff0_20*dmats1[20][4] + coeff0_21*dmats1[21][4] + coeff0_22*dmats1[22][4] + coeff0_23*dmats1[23][4] + coeff0_24*dmats1[24][4] + coeff0_25*dmats1[25][4] + coeff0_26*dmats1[26][4] + coeff0_27*dmats1[27][4] + coeff0_28*dmats1[28][4] + coeff0_29*dmats1[29][4] + coeff0_30*dmats1[30][4] + coeff0_31*dmats1[31][4] + coeff0_32*dmats1[32][4] + coeff0_33*dmats1[33][4] + coeff0_34*dmats1[34][4] + coeff0_35*dmats1[35][4] + coeff0_36*dmats1[36][4] + coeff0_37*dmats1[37][4] + coeff0_38*dmats1[38][4] + coeff0_39*dmats1[39][4] + coeff0_40*dmats1[40][4] + coeff0_41*dmats1[41][4] + coeff0_42*dmats1[42][4] + coeff0_43*dmats1[43][4] + coeff0_44*dmats1[44][4] + coeff0_45*dmats1[45][4] + coeff0_46*dmats1[46][4] + coeff0_47*dmats1[47][4] + coeff0_48*dmats1[48][4] + coeff0_49*dmats1[49][4] + coeff0_50*dmats1[50][4] + coeff0_51*dmats1[51][4] + coeff0_52*dmats1[52][4] + coeff0_53*dmats1[53][4] + coeff0_54*dmats1[54][4] + coeff0_55*dmats1[55][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5] + coeff0_10*dmats1[10][5] + coeff0_11*dmats1[11][5] + coeff0_12*dmats1[12][5] + coeff0_13*dmats1[13][5] + coeff0_14*dmats1[14][5] + coeff0_15*dmats1[15][5] + coeff0_16*dmats1[16][5] + coeff0_17*dmats1[17][5] + coeff0_18*dmats1[18][5] + coeff0_19*dmats1[19][5] + coeff0_20*dmats1[20][5] + coeff0_21*dmats1[21][5] + coeff0_22*dmats1[22][5] + coeff0_23*dmats1[23][5] + coeff0_24*dmats1[24][5] + coeff0_25*dmats1[25][5] + coeff0_26*dmats1[26][5] + coeff0_27*dmats1[27][5] + coeff0_28*dmats1[28][5] + coeff0_29*dmats1[29][5] + coeff0_30*dmats1[30][5] + coeff0_31*dmats1[31][5] + coeff0_32*dmats1[32][5] + coeff0_33*dmats1[33][5] + coeff0_34*dmats1[34][5] + coeff0_35*dmats1[35][5] + coeff0_36*dmats1[36][5] + coeff0_37*dmats1[37][5] + coeff0_38*dmats1[38][5] + coeff0_39*dmats1[39][5] + coeff0_40*dmats1[40][5] + coeff0_41*dmats1[41][5] + coeff0_42*dmats1[42][5] + coeff0_43*dmats1[43][5] + coeff0_44*dmats1[44][5] + coeff0_45*dmats1[45][5] + coeff0_46*dmats1[46][5] + coeff0_47*dmats1[47][5] + coeff0_48*dmats1[48][5] + coeff0_49*dmats1[49][5] + coeff0_50*dmats1[50][5] + coeff0_51*dmats1[51][5] + coeff0_52*dmats1[52][5] + coeff0_53*dmats1[53][5] + coeff0_54*dmats1[54][5] + coeff0_55*dmats1[55][5];
          new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6] + coeff0_10*dmats1[10][6] + coeff0_11*dmats1[11][6] + coeff0_12*dmats1[12][6] + coeff0_13*dmats1[13][6] + coeff0_14*dmats1[14][6] + coeff0_15*dmats1[15][6] + coeff0_16*dmats1[16][6] + coeff0_17*dmats1[17][6] + coeff0_18*dmats1[18][6] + coeff0_19*dmats1[19][6] + coeff0_20*dmats1[20][6] + coeff0_21*dmats1[21][6] + coeff0_22*dmats1[22][6] + coeff0_23*dmats1[23][6] + coeff0_24*dmats1[24][6] + coeff0_25*dmats1[25][6] + coeff0_26*dmats1[26][6] + coeff0_27*dmats1[27][6] + coeff0_28*dmats1[28][6] + coeff0_29*dmats1[29][6] + coeff0_30*dmats1[30][6] + coeff0_31*dmats1[31][6] + coeff0_32*dmats1[32][6] + coeff0_33*dmats1[33][6] + coeff0_34*dmats1[34][6] + coeff0_35*dmats1[35][6] + coeff0_36*dmats1[36][6] + coeff0_37*dmats1[37][6] + coeff0_38*dmats1[38][6] + coeff0_39*dmats1[39][6] + coeff0_40*dmats1[40][6] + coeff0_41*dmats1[41][6] + coeff0_42*dmats1[42][6] + coeff0_43*dmats1[43][6] + coeff0_44*dmats1[44][6] + coeff0_45*dmats1[45][6] + coeff0_46*dmats1[46][6] + coeff0_47*dmats1[47][6] + coeff0_48*dmats1[48][6] + coeff0_49*dmats1[49][6] + coeff0_50*dmats1[50][6] + coeff0_51*dmats1[51][6] + coeff0_52*dmats1[52][6] + coeff0_53*dmats1[53][6] + coeff0_54*dmats1[54][6] + coeff0_55*dmats1[55][6];
          new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7] + coeff0_10*dmats1[10][7] + coeff0_11*dmats1[11][7] + coeff0_12*dmats1[12][7] + coeff0_13*dmats1[13][7] + coeff0_14*dmats1[14][7] + coeff0_15*dmats1[15][7] + coeff0_16*dmats1[16][7] + coeff0_17*dmats1[17][7] + coeff0_18*dmats1[18][7] + coeff0_19*dmats1[19][7] + coeff0_20*dmats1[20][7] + coeff0_21*dmats1[21][7] + coeff0_22*dmats1[22][7] + coeff0_23*dmats1[23][7] + coeff0_24*dmats1[24][7] + coeff0_25*dmats1[25][7] + coeff0_26*dmats1[26][7] + coeff0_27*dmats1[27][7] + coeff0_28*dmats1[28][7] + coeff0_29*dmats1[29][7] + coeff0_30*dmats1[30][7] + coeff0_31*dmats1[31][7] + coeff0_32*dmats1[32][7] + coeff0_33*dmats1[33][7] + coeff0_34*dmats1[34][7] + coeff0_35*dmats1[35][7] + coeff0_36*dmats1[36][7] + coeff0_37*dmats1[37][7] + coeff0_38*dmats1[38][7] + coeff0_39*dmats1[39][7] + coeff0_40*dmats1[40][7] + coeff0_41*dmats1[41][7] + coeff0_42*dmats1[42][7] + coeff0_43*dmats1[43][7] + coeff0_44*dmats1[44][7] + coeff0_45*dmats1[45][7] + coeff0_46*dmats1[46][7] + coeff0_47*dmats1[47][7] + coeff0_48*dmats1[48][7] + coeff0_49*dmats1[49][7] + coeff0_50*dmats1[50][7] + coeff0_51*dmats1[51][7] + coeff0_52*dmats1[52][7] + coeff0_53*dmats1[53][7] + coeff0_54*dmats1[54][7] + coeff0_55*dmats1[55][7];
          new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8] + coeff0_10*dmats1[10][8] + coeff0_11*dmats1[11][8] + coeff0_12*dmats1[12][8] + coeff0_13*dmats1[13][8] + coeff0_14*dmats1[14][8] + coeff0_15*dmats1[15][8] + coeff0_16*dmats1[16][8] + coeff0_17*dmats1[17][8] + coeff0_18*dmats1[18][8] + coeff0_19*dmats1[19][8] + coeff0_20*dmats1[20][8] + coeff0_21*dmats1[21][8] + coeff0_22*dmats1[22][8] + coeff0_23*dmats1[23][8] + coeff0_24*dmats1[24][8] + coeff0_25*dmats1[25][8] + coeff0_26*dmats1[26][8] + coeff0_27*dmats1[27][8] + coeff0_28*dmats1[28][8] + coeff0_29*dmats1[29][8] + coeff0_30*dmats1[30][8] + coeff0_31*dmats1[31][8] + coeff0_32*dmats1[32][8] + coeff0_33*dmats1[33][8] + coeff0_34*dmats1[34][8] + coeff0_35*dmats1[35][8] + coeff0_36*dmats1[36][8] + coeff0_37*dmats1[37][8] + coeff0_38*dmats1[38][8] + coeff0_39*dmats1[39][8] + coeff0_40*dmats1[40][8] + coeff0_41*dmats1[41][8] + coeff0_42*dmats1[42][8] + coeff0_43*dmats1[43][8] + coeff0_44*dmats1[44][8] + coeff0_45*dmats1[45][8] + coeff0_46*dmats1[46][8] + coeff0_47*dmats1[47][8] + coeff0_48*dmats1[48][8] + coeff0_49*dmats1[49][8] + coeff0_50*dmats1[50][8] + coeff0_51*dmats1[51][8] + coeff0_52*dmats1[52][8] + coeff0_53*dmats1[53][8] + coeff0_54*dmats1[54][8] + coeff0_55*dmats1[55][8];
          new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9] + coeff0_10*dmats1[10][9] + coeff0_11*dmats1[11][9] + coeff0_12*dmats1[12][9] + coeff0_13*dmats1[13][9] + coeff0_14*dmats1[14][9] + coeff0_15*dmats1[15][9] + coeff0_16*dmats1[16][9] + coeff0_17*dmats1[17][9] + coeff0_18*dmats1[18][9] + coeff0_19*dmats1[19][9] + coeff0_20*dmats1[20][9] + coeff0_21*dmats1[21][9] + coeff0_22*dmats1[22][9] + coeff0_23*dmats1[23][9] + coeff0_24*dmats1[24][9] + coeff0_25*dmats1[25][9] + coeff0_26*dmats1[26][9] + coeff0_27*dmats1[27][9] + coeff0_28*dmats1[28][9] + coeff0_29*dmats1[29][9] + coeff0_30*dmats1[30][9] + coeff0_31*dmats1[31][9] + coeff0_32*dmats1[32][9] + coeff0_33*dmats1[33][9] + coeff0_34*dmats1[34][9] + coeff0_35*dmats1[35][9] + coeff0_36*dmats1[36][9] + coeff0_37*dmats1[37][9] + coeff0_38*dmats1[38][9] + coeff0_39*dmats1[39][9] + coeff0_40*dmats1[40][9] + coeff0_41*dmats1[41][9] + coeff0_42*dmats1[42][9] + coeff0_43*dmats1[43][9] + coeff0_44*dmats1[44][9] + coeff0_45*dmats1[45][9] + coeff0_46*dmats1[46][9] + coeff0_47*dmats1[47][9] + coeff0_48*dmats1[48][9] + coeff0_49*dmats1[49][9] + coeff0_50*dmats1[50][9] + coeff0_51*dmats1[51][9] + coeff0_52*dmats1[52][9] + coeff0_53*dmats1[53][9] + coeff0_54*dmats1[54][9] + coeff0_55*dmats1[55][9];
          new_coeff0_10 = coeff0_0*dmats1[0][10] + coeff0_1*dmats1[1][10] + coeff0_2*dmats1[2][10] + coeff0_3*dmats1[3][10] + coeff0_4*dmats1[4][10] + coeff0_5*dmats1[5][10] + coeff0_6*dmats1[6][10] + coeff0_7*dmats1[7][10] + coeff0_8*dmats1[8][10] + coeff0_9*dmats1[9][10] + coeff0_10*dmats1[10][10] + coeff0_11*dmats1[11][10] + coeff0_12*dmats1[12][10] + coeff0_13*dmats1[13][10] + coeff0_14*dmats1[14][10] + coeff0_15*dmats1[15][10] + coeff0_16*dmats1[16][10] + coeff0_17*dmats1[17][10] + coeff0_18*dmats1[18][10] + coeff0_19*dmats1[19][10] + coeff0_20*dmats1[20][10] + coeff0_21*dmats1[21][10] + coeff0_22*dmats1[22][10] + coeff0_23*dmats1[23][10] + coeff0_24*dmats1[24][10] + coeff0_25*dmats1[25][10] + coeff0_26*dmats1[26][10] + coeff0_27*dmats1[27][10] + coeff0_28*dmats1[28][10] + coeff0_29*dmats1[29][10] + coeff0_30*dmats1[30][10] + coeff0_31*dmats1[31][10] + coeff0_32*dmats1[32][10] + coeff0_33*dmats1[33][10] + coeff0_34*dmats1[34][10] + coeff0_35*dmats1[35][10] + coeff0_36*dmats1[36][10] + coeff0_37*dmats1[37][10] + coeff0_38*dmats1[38][10] + coeff0_39*dmats1[39][10] + coeff0_40*dmats1[40][10] + coeff0_41*dmats1[41][10] + coeff0_42*dmats1[42][10] + coeff0_43*dmats1[43][10] + coeff0_44*dmats1[44][10] + coeff0_45*dmats1[45][10] + coeff0_46*dmats1[46][10] + coeff0_47*dmats1[47][10] + coeff0_48*dmats1[48][10] + coeff0_49*dmats1[49][10] + coeff0_50*dmats1[50][10] + coeff0_51*dmats1[51][10] + coeff0_52*dmats1[52][10] + coeff0_53*dmats1[53][10] + coeff0_54*dmats1[54][10] + coeff0_55*dmats1[55][10];
          new_coeff0_11 = coeff0_0*dmats1[0][11] + coeff0_1*dmats1[1][11] + coeff0_2*dmats1[2][11] + coeff0_3*dmats1[3][11] + coeff0_4*dmats1[4][11] + coeff0_5*dmats1[5][11] + coeff0_6*dmats1[6][11] + coeff0_7*dmats1[7][11] + coeff0_8*dmats1[8][11] + coeff0_9*dmats1[9][11] + coeff0_10*dmats1[10][11] + coeff0_11*dmats1[11][11] + coeff0_12*dmats1[12][11] + coeff0_13*dmats1[13][11] + coeff0_14*dmats1[14][11] + coeff0_15*dmats1[15][11] + coeff0_16*dmats1[16][11] + coeff0_17*dmats1[17][11] + coeff0_18*dmats1[18][11] + coeff0_19*dmats1[19][11] + coeff0_20*dmats1[20][11] + coeff0_21*dmats1[21][11] + coeff0_22*dmats1[22][11] + coeff0_23*dmats1[23][11] + coeff0_24*dmats1[24][11] + coeff0_25*dmats1[25][11] + coeff0_26*dmats1[26][11] + coeff0_27*dmats1[27][11] + coeff0_28*dmats1[28][11] + coeff0_29*dmats1[29][11] + coeff0_30*dmats1[30][11] + coeff0_31*dmats1[31][11] + coeff0_32*dmats1[32][11] + coeff0_33*dmats1[33][11] + coeff0_34*dmats1[34][11] + coeff0_35*dmats1[35][11] + coeff0_36*dmats1[36][11] + coeff0_37*dmats1[37][11] + coeff0_38*dmats1[38][11] + coeff0_39*dmats1[39][11] + coeff0_40*dmats1[40][11] + coeff0_41*dmats1[41][11] + coeff0_42*dmats1[42][11] + coeff0_43*dmats1[43][11] + coeff0_44*dmats1[44][11] + coeff0_45*dmats1[45][11] + coeff0_46*dmats1[46][11] + coeff0_47*dmats1[47][11] + coeff0_48*dmats1[48][11] + coeff0_49*dmats1[49][11] + coeff0_50*dmats1[50][11] + coeff0_51*dmats1[51][11] + coeff0_52*dmats1[52][11] + coeff0_53*dmats1[53][11] + coeff0_54*dmats1[54][11] + coeff0_55*dmats1[55][11];
          new_coeff0_12 = coeff0_0*dmats1[0][12] + coeff0_1*dmats1[1][12] + coeff0_2*dmats1[2][12] + coeff0_3*dmats1[3][12] + coeff0_4*dmats1[4][12] + coeff0_5*dmats1[5][12] + coeff0_6*dmats1[6][12] + coeff0_7*dmats1[7][12] + coeff0_8*dmats1[8][12] + coeff0_9*dmats1[9][12] + coeff0_10*dmats1[10][12] + coeff0_11*dmats1[11][12] + coeff0_12*dmats1[12][12] + coeff0_13*dmats1[13][12] + coeff0_14*dmats1[14][12] + coeff0_15*dmats1[15][12] + coeff0_16*dmats1[16][12] + coeff0_17*dmats1[17][12] + coeff0_18*dmats1[18][12] + coeff0_19*dmats1[19][12] + coeff0_20*dmats1[20][12] + coeff0_21*dmats1[21][12] + coeff0_22*dmats1[22][12] + coeff0_23*dmats1[23][12] + coeff0_24*dmats1[24][12] + coeff0_25*dmats1[25][12] + coeff0_26*dmats1[26][12] + coeff0_27*dmats1[27][12] + coeff0_28*dmats1[28][12] + coeff0_29*dmats1[29][12] + coeff0_30*dmats1[30][12] + coeff0_31*dmats1[31][12] + coeff0_32*dmats1[32][12] + coeff0_33*dmats1[33][12] + coeff0_34*dmats1[34][12] + coeff0_35*dmats1[35][12] + coeff0_36*dmats1[36][12] + coeff0_37*dmats1[37][12] + coeff0_38*dmats1[38][12] + coeff0_39*dmats1[39][12] + coeff0_40*dmats1[40][12] + coeff0_41*dmats1[41][12] + coeff0_42*dmats1[42][12] + coeff0_43*dmats1[43][12] + coeff0_44*dmats1[44][12] + coeff0_45*dmats1[45][12] + coeff0_46*dmats1[46][12] + coeff0_47*dmats1[47][12] + coeff0_48*dmats1[48][12] + coeff0_49*dmats1[49][12] + coeff0_50*dmats1[50][12] + coeff0_51*dmats1[51][12] + coeff0_52*dmats1[52][12] + coeff0_53*dmats1[53][12] + coeff0_54*dmats1[54][12] + coeff0_55*dmats1[55][12];
          new_coeff0_13 = coeff0_0*dmats1[0][13] + coeff0_1*dmats1[1][13] + coeff0_2*dmats1[2][13] + coeff0_3*dmats1[3][13] + coeff0_4*dmats1[4][13] + coeff0_5*dmats1[5][13] + coeff0_6*dmats1[6][13] + coeff0_7*dmats1[7][13] + coeff0_8*dmats1[8][13] + coeff0_9*dmats1[9][13] + coeff0_10*dmats1[10][13] + coeff0_11*dmats1[11][13] + coeff0_12*dmats1[12][13] + coeff0_13*dmats1[13][13] + coeff0_14*dmats1[14][13] + coeff0_15*dmats1[15][13] + coeff0_16*dmats1[16][13] + coeff0_17*dmats1[17][13] + coeff0_18*dmats1[18][13] + coeff0_19*dmats1[19][13] + coeff0_20*dmats1[20][13] + coeff0_21*dmats1[21][13] + coeff0_22*dmats1[22][13] + coeff0_23*dmats1[23][13] + coeff0_24*dmats1[24][13] + coeff0_25*dmats1[25][13] + coeff0_26*dmats1[26][13] + coeff0_27*dmats1[27][13] + coeff0_28*dmats1[28][13] + coeff0_29*dmats1[29][13] + coeff0_30*dmats1[30][13] + coeff0_31*dmats1[31][13] + coeff0_32*dmats1[32][13] + coeff0_33*dmats1[33][13] + coeff0_34*dmats1[34][13] + coeff0_35*dmats1[35][13] + coeff0_36*dmats1[36][13] + coeff0_37*dmats1[37][13] + coeff0_38*dmats1[38][13] + coeff0_39*dmats1[39][13] + coeff0_40*dmats1[40][13] + coeff0_41*dmats1[41][13] + coeff0_42*dmats1[42][13] + coeff0_43*dmats1[43][13] + coeff0_44*dmats1[44][13] + coeff0_45*dmats1[45][13] + coeff0_46*dmats1[46][13] + coeff0_47*dmats1[47][13] + coeff0_48*dmats1[48][13] + coeff0_49*dmats1[49][13] + coeff0_50*dmats1[50][13] + coeff0_51*dmats1[51][13] + coeff0_52*dmats1[52][13] + coeff0_53*dmats1[53][13] + coeff0_54*dmats1[54][13] + coeff0_55*dmats1[55][13];
          new_coeff0_14 = coeff0_0*dmats1[0][14] + coeff0_1*dmats1[1][14] + coeff0_2*dmats1[2][14] + coeff0_3*dmats1[3][14] + coeff0_4*dmats1[4][14] + coeff0_5*dmats1[5][14] + coeff0_6*dmats1[6][14] + coeff0_7*dmats1[7][14] + coeff0_8*dmats1[8][14] + coeff0_9*dmats1[9][14] + coeff0_10*dmats1[10][14] + coeff0_11*dmats1[11][14] + coeff0_12*dmats1[12][14] + coeff0_13*dmats1[13][14] + coeff0_14*dmats1[14][14] + coeff0_15*dmats1[15][14] + coeff0_16*dmats1[16][14] + coeff0_17*dmats1[17][14] + coeff0_18*dmats1[18][14] + coeff0_19*dmats1[19][14] + coeff0_20*dmats1[20][14] + coeff0_21*dmats1[21][14] + coeff0_22*dmats1[22][14] + coeff0_23*dmats1[23][14] + coeff0_24*dmats1[24][14] + coeff0_25*dmats1[25][14] + coeff0_26*dmats1[26][14] + coeff0_27*dmats1[27][14] + coeff0_28*dmats1[28][14] + coeff0_29*dmats1[29][14] + coeff0_30*dmats1[30][14] + coeff0_31*dmats1[31][14] + coeff0_32*dmats1[32][14] + coeff0_33*dmats1[33][14] + coeff0_34*dmats1[34][14] + coeff0_35*dmats1[35][14] + coeff0_36*dmats1[36][14] + coeff0_37*dmats1[37][14] + coeff0_38*dmats1[38][14] + coeff0_39*dmats1[39][14] + coeff0_40*dmats1[40][14] + coeff0_41*dmats1[41][14] + coeff0_42*dmats1[42][14] + coeff0_43*dmats1[43][14] + coeff0_44*dmats1[44][14] + coeff0_45*dmats1[45][14] + coeff0_46*dmats1[46][14] + coeff0_47*dmats1[47][14] + coeff0_48*dmats1[48][14] + coeff0_49*dmats1[49][14] + coeff0_50*dmats1[50][14] + coeff0_51*dmats1[51][14] + coeff0_52*dmats1[52][14] + coeff0_53*dmats1[53][14] + coeff0_54*dmats1[54][14] + coeff0_55*dmats1[55][14];
          new_coeff0_15 = coeff0_0*dmats1[0][15] + coeff0_1*dmats1[1][15] + coeff0_2*dmats1[2][15] + coeff0_3*dmats1[3][15] + coeff0_4*dmats1[4][15] + coeff0_5*dmats1[5][15] + coeff0_6*dmats1[6][15] + coeff0_7*dmats1[7][15] + coeff0_8*dmats1[8][15] + coeff0_9*dmats1[9][15] + coeff0_10*dmats1[10][15] + coeff0_11*dmats1[11][15] + coeff0_12*dmats1[12][15] + coeff0_13*dmats1[13][15] + coeff0_14*dmats1[14][15] + coeff0_15*dmats1[15][15] + coeff0_16*dmats1[16][15] + coeff0_17*dmats1[17][15] + coeff0_18*dmats1[18][15] + coeff0_19*dmats1[19][15] + coeff0_20*dmats1[20][15] + coeff0_21*dmats1[21][15] + coeff0_22*dmats1[22][15] + coeff0_23*dmats1[23][15] + coeff0_24*dmats1[24][15] + coeff0_25*dmats1[25][15] + coeff0_26*dmats1[26][15] + coeff0_27*dmats1[27][15] + coeff0_28*dmats1[28][15] + coeff0_29*dmats1[29][15] + coeff0_30*dmats1[30][15] + coeff0_31*dmats1[31][15] + coeff0_32*dmats1[32][15] + coeff0_33*dmats1[33][15] + coeff0_34*dmats1[34][15] + coeff0_35*dmats1[35][15] + coeff0_36*dmats1[36][15] + coeff0_37*dmats1[37][15] + coeff0_38*dmats1[38][15] + coeff0_39*dmats1[39][15] + coeff0_40*dmats1[40][15] + coeff0_41*dmats1[41][15] + coeff0_42*dmats1[42][15] + coeff0_43*dmats1[43][15] + coeff0_44*dmats1[44][15] + coeff0_45*dmats1[45][15] + coeff0_46*dmats1[46][15] + coeff0_47*dmats1[47][15] + coeff0_48*dmats1[48][15] + coeff0_49*dmats1[49][15] + coeff0_50*dmats1[50][15] + coeff0_51*dmats1[51][15] + coeff0_52*dmats1[52][15] + coeff0_53*dmats1[53][15] + coeff0_54*dmats1[54][15] + coeff0_55*dmats1[55][15];
          new_coeff0_16 = coeff0_0*dmats1[0][16] + coeff0_1*dmats1[1][16] + coeff0_2*dmats1[2][16] + coeff0_3*dmats1[3][16] + coeff0_4*dmats1[4][16] + coeff0_5*dmats1[5][16] + coeff0_6*dmats1[6][16] + coeff0_7*dmats1[7][16] + coeff0_8*dmats1[8][16] + coeff0_9*dmats1[9][16] + coeff0_10*dmats1[10][16] + coeff0_11*dmats1[11][16] + coeff0_12*dmats1[12][16] + coeff0_13*dmats1[13][16] + coeff0_14*dmats1[14][16] + coeff0_15*dmats1[15][16] + coeff0_16*dmats1[16][16] + coeff0_17*dmats1[17][16] + coeff0_18*dmats1[18][16] + coeff0_19*dmats1[19][16] + coeff0_20*dmats1[20][16] + coeff0_21*dmats1[21][16] + coeff0_22*dmats1[22][16] + coeff0_23*dmats1[23][16] + coeff0_24*dmats1[24][16] + coeff0_25*dmats1[25][16] + coeff0_26*dmats1[26][16] + coeff0_27*dmats1[27][16] + coeff0_28*dmats1[28][16] + coeff0_29*dmats1[29][16] + coeff0_30*dmats1[30][16] + coeff0_31*dmats1[31][16] + coeff0_32*dmats1[32][16] + coeff0_33*dmats1[33][16] + coeff0_34*dmats1[34][16] + coeff0_35*dmats1[35][16] + coeff0_36*dmats1[36][16] + coeff0_37*dmats1[37][16] + coeff0_38*dmats1[38][16] + coeff0_39*dmats1[39][16] + coeff0_40*dmats1[40][16] + coeff0_41*dmats1[41][16] + coeff0_42*dmats1[42][16] + coeff0_43*dmats1[43][16] + coeff0_44*dmats1[44][16] + coeff0_45*dmats1[45][16] + coeff0_46*dmats1[46][16] + coeff0_47*dmats1[47][16] + coeff0_48*dmats1[48][16] + coeff0_49*dmats1[49][16] + coeff0_50*dmats1[50][16] + coeff0_51*dmats1[51][16] + coeff0_52*dmats1[52][16] + coeff0_53*dmats1[53][16] + coeff0_54*dmats1[54][16] + coeff0_55*dmats1[55][16];
          new_coeff0_17 = coeff0_0*dmats1[0][17] + coeff0_1*dmats1[1][17] + coeff0_2*dmats1[2][17] + coeff0_3*dmats1[3][17] + coeff0_4*dmats1[4][17] + coeff0_5*dmats1[5][17] + coeff0_6*dmats1[6][17] + coeff0_7*dmats1[7][17] + coeff0_8*dmats1[8][17] + coeff0_9*dmats1[9][17] + coeff0_10*dmats1[10][17] + coeff0_11*dmats1[11][17] + coeff0_12*dmats1[12][17] + coeff0_13*dmats1[13][17] + coeff0_14*dmats1[14][17] + coeff0_15*dmats1[15][17] + coeff0_16*dmats1[16][17] + coeff0_17*dmats1[17][17] + coeff0_18*dmats1[18][17] + coeff0_19*dmats1[19][17] + coeff0_20*dmats1[20][17] + coeff0_21*dmats1[21][17] + coeff0_22*dmats1[22][17] + coeff0_23*dmats1[23][17] + coeff0_24*dmats1[24][17] + coeff0_25*dmats1[25][17] + coeff0_26*dmats1[26][17] + coeff0_27*dmats1[27][17] + coeff0_28*dmats1[28][17] + coeff0_29*dmats1[29][17] + coeff0_30*dmats1[30][17] + coeff0_31*dmats1[31][17] + coeff0_32*dmats1[32][17] + coeff0_33*dmats1[33][17] + coeff0_34*dmats1[34][17] + coeff0_35*dmats1[35][17] + coeff0_36*dmats1[36][17] + coeff0_37*dmats1[37][17] + coeff0_38*dmats1[38][17] + coeff0_39*dmats1[39][17] + coeff0_40*dmats1[40][17] + coeff0_41*dmats1[41][17] + coeff0_42*dmats1[42][17] + coeff0_43*dmats1[43][17] + coeff0_44*dmats1[44][17] + coeff0_45*dmats1[45][17] + coeff0_46*dmats1[46][17] + coeff0_47*dmats1[47][17] + coeff0_48*dmats1[48][17] + coeff0_49*dmats1[49][17] + coeff0_50*dmats1[50][17] + coeff0_51*dmats1[51][17] + coeff0_52*dmats1[52][17] + coeff0_53*dmats1[53][17] + coeff0_54*dmats1[54][17] + coeff0_55*dmats1[55][17];
          new_coeff0_18 = coeff0_0*dmats1[0][18] + coeff0_1*dmats1[1][18] + coeff0_2*dmats1[2][18] + coeff0_3*dmats1[3][18] + coeff0_4*dmats1[4][18] + coeff0_5*dmats1[5][18] + coeff0_6*dmats1[6][18] + coeff0_7*dmats1[7][18] + coeff0_8*dmats1[8][18] + coeff0_9*dmats1[9][18] + coeff0_10*dmats1[10][18] + coeff0_11*dmats1[11][18] + coeff0_12*dmats1[12][18] + coeff0_13*dmats1[13][18] + coeff0_14*dmats1[14][18] + coeff0_15*dmats1[15][18] + coeff0_16*dmats1[16][18] + coeff0_17*dmats1[17][18] + coeff0_18*dmats1[18][18] + coeff0_19*dmats1[19][18] + coeff0_20*dmats1[20][18] + coeff0_21*dmats1[21][18] + coeff0_22*dmats1[22][18] + coeff0_23*dmats1[23][18] + coeff0_24*dmats1[24][18] + coeff0_25*dmats1[25][18] + coeff0_26*dmats1[26][18] + coeff0_27*dmats1[27][18] + coeff0_28*dmats1[28][18] + coeff0_29*dmats1[29][18] + coeff0_30*dmats1[30][18] + coeff0_31*dmats1[31][18] + coeff0_32*dmats1[32][18] + coeff0_33*dmats1[33][18] + coeff0_34*dmats1[34][18] + coeff0_35*dmats1[35][18] + coeff0_36*dmats1[36][18] + coeff0_37*dmats1[37][18] + coeff0_38*dmats1[38][18] + coeff0_39*dmats1[39][18] + coeff0_40*dmats1[40][18] + coeff0_41*dmats1[41][18] + coeff0_42*dmats1[42][18] + coeff0_43*dmats1[43][18] + coeff0_44*dmats1[44][18] + coeff0_45*dmats1[45][18] + coeff0_46*dmats1[46][18] + coeff0_47*dmats1[47][18] + coeff0_48*dmats1[48][18] + coeff0_49*dmats1[49][18] + coeff0_50*dmats1[50][18] + coeff0_51*dmats1[51][18] + coeff0_52*dmats1[52][18] + coeff0_53*dmats1[53][18] + coeff0_54*dmats1[54][18] + coeff0_55*dmats1[55][18];
          new_coeff0_19 = coeff0_0*dmats1[0][19] + coeff0_1*dmats1[1][19] + coeff0_2*dmats1[2][19] + coeff0_3*dmats1[3][19] + coeff0_4*dmats1[4][19] + coeff0_5*dmats1[5][19] + coeff0_6*dmats1[6][19] + coeff0_7*dmats1[7][19] + coeff0_8*dmats1[8][19] + coeff0_9*dmats1[9][19] + coeff0_10*dmats1[10][19] + coeff0_11*dmats1[11][19] + coeff0_12*dmats1[12][19] + coeff0_13*dmats1[13][19] + coeff0_14*dmats1[14][19] + coeff0_15*dmats1[15][19] + coeff0_16*dmats1[16][19] + coeff0_17*dmats1[17][19] + coeff0_18*dmats1[18][19] + coeff0_19*dmats1[19][19] + coeff0_20*dmats1[20][19] + coeff0_21*dmats1[21][19] + coeff0_22*dmats1[22][19] + coeff0_23*dmats1[23][19] + coeff0_24*dmats1[24][19] + coeff0_25*dmats1[25][19] + coeff0_26*dmats1[26][19] + coeff0_27*dmats1[27][19] + coeff0_28*dmats1[28][19] + coeff0_29*dmats1[29][19] + coeff0_30*dmats1[30][19] + coeff0_31*dmats1[31][19] + coeff0_32*dmats1[32][19] + coeff0_33*dmats1[33][19] + coeff0_34*dmats1[34][19] + coeff0_35*dmats1[35][19] + coeff0_36*dmats1[36][19] + coeff0_37*dmats1[37][19] + coeff0_38*dmats1[38][19] + coeff0_39*dmats1[39][19] + coeff0_40*dmats1[40][19] + coeff0_41*dmats1[41][19] + coeff0_42*dmats1[42][19] + coeff0_43*dmats1[43][19] + coeff0_44*dmats1[44][19] + coeff0_45*dmats1[45][19] + coeff0_46*dmats1[46][19] + coeff0_47*dmats1[47][19] + coeff0_48*dmats1[48][19] + coeff0_49*dmats1[49][19] + coeff0_50*dmats1[50][19] + coeff0_51*dmats1[51][19] + coeff0_52*dmats1[52][19] + coeff0_53*dmats1[53][19] + coeff0_54*dmats1[54][19] + coeff0_55*dmats1[55][19];
          new_coeff0_20 = coeff0_0*dmats1[0][20] + coeff0_1*dmats1[1][20] + coeff0_2*dmats1[2][20] + coeff0_3*dmats1[3][20] + coeff0_4*dmats1[4][20] + coeff0_5*dmats1[5][20] + coeff0_6*dmats1[6][20] + coeff0_7*dmats1[7][20] + coeff0_8*dmats1[8][20] + coeff0_9*dmats1[9][20] + coeff0_10*dmats1[10][20] + coeff0_11*dmats1[11][20] + coeff0_12*dmats1[12][20] + coeff0_13*dmats1[13][20] + coeff0_14*dmats1[14][20] + coeff0_15*dmats1[15][20] + coeff0_16*dmats1[16][20] + coeff0_17*dmats1[17][20] + coeff0_18*dmats1[18][20] + coeff0_19*dmats1[19][20] + coeff0_20*dmats1[20][20] + coeff0_21*dmats1[21][20] + coeff0_22*dmats1[22][20] + coeff0_23*dmats1[23][20] + coeff0_24*dmats1[24][20] + coeff0_25*dmats1[25][20] + coeff0_26*dmats1[26][20] + coeff0_27*dmats1[27][20] + coeff0_28*dmats1[28][20] + coeff0_29*dmats1[29][20] + coeff0_30*dmats1[30][20] + coeff0_31*dmats1[31][20] + coeff0_32*dmats1[32][20] + coeff0_33*dmats1[33][20] + coeff0_34*dmats1[34][20] + coeff0_35*dmats1[35][20] + coeff0_36*dmats1[36][20] + coeff0_37*dmats1[37][20] + coeff0_38*dmats1[38][20] + coeff0_39*dmats1[39][20] + coeff0_40*dmats1[40][20] + coeff0_41*dmats1[41][20] + coeff0_42*dmats1[42][20] + coeff0_43*dmats1[43][20] + coeff0_44*dmats1[44][20] + coeff0_45*dmats1[45][20] + coeff0_46*dmats1[46][20] + coeff0_47*dmats1[47][20] + coeff0_48*dmats1[48][20] + coeff0_49*dmats1[49][20] + coeff0_50*dmats1[50][20] + coeff0_51*dmats1[51][20] + coeff0_52*dmats1[52][20] + coeff0_53*dmats1[53][20] + coeff0_54*dmats1[54][20] + coeff0_55*dmats1[55][20];
          new_coeff0_21 = coeff0_0*dmats1[0][21] + coeff0_1*dmats1[1][21] + coeff0_2*dmats1[2][21] + coeff0_3*dmats1[3][21] + coeff0_4*dmats1[4][21] + coeff0_5*dmats1[5][21] + coeff0_6*dmats1[6][21] + coeff0_7*dmats1[7][21] + coeff0_8*dmats1[8][21] + coeff0_9*dmats1[9][21] + coeff0_10*dmats1[10][21] + coeff0_11*dmats1[11][21] + coeff0_12*dmats1[12][21] + coeff0_13*dmats1[13][21] + coeff0_14*dmats1[14][21] + coeff0_15*dmats1[15][21] + coeff0_16*dmats1[16][21] + coeff0_17*dmats1[17][21] + coeff0_18*dmats1[18][21] + coeff0_19*dmats1[19][21] + coeff0_20*dmats1[20][21] + coeff0_21*dmats1[21][21] + coeff0_22*dmats1[22][21] + coeff0_23*dmats1[23][21] + coeff0_24*dmats1[24][21] + coeff0_25*dmats1[25][21] + coeff0_26*dmats1[26][21] + coeff0_27*dmats1[27][21] + coeff0_28*dmats1[28][21] + coeff0_29*dmats1[29][21] + coeff0_30*dmats1[30][21] + coeff0_31*dmats1[31][21] + coeff0_32*dmats1[32][21] + coeff0_33*dmats1[33][21] + coeff0_34*dmats1[34][21] + coeff0_35*dmats1[35][21] + coeff0_36*dmats1[36][21] + coeff0_37*dmats1[37][21] + coeff0_38*dmats1[38][21] + coeff0_39*dmats1[39][21] + coeff0_40*dmats1[40][21] + coeff0_41*dmats1[41][21] + coeff0_42*dmats1[42][21] + coeff0_43*dmats1[43][21] + coeff0_44*dmats1[44][21] + coeff0_45*dmats1[45][21] + coeff0_46*dmats1[46][21] + coeff0_47*dmats1[47][21] + coeff0_48*dmats1[48][21] + coeff0_49*dmats1[49][21] + coeff0_50*dmats1[50][21] + coeff0_51*dmats1[51][21] + coeff0_52*dmats1[52][21] + coeff0_53*dmats1[53][21] + coeff0_54*dmats1[54][21] + coeff0_55*dmats1[55][21];
          new_coeff0_22 = coeff0_0*dmats1[0][22] + coeff0_1*dmats1[1][22] + coeff0_2*dmats1[2][22] + coeff0_3*dmats1[3][22] + coeff0_4*dmats1[4][22] + coeff0_5*dmats1[5][22] + coeff0_6*dmats1[6][22] + coeff0_7*dmats1[7][22] + coeff0_8*dmats1[8][22] + coeff0_9*dmats1[9][22] + coeff0_10*dmats1[10][22] + coeff0_11*dmats1[11][22] + coeff0_12*dmats1[12][22] + coeff0_13*dmats1[13][22] + coeff0_14*dmats1[14][22] + coeff0_15*dmats1[15][22] + coeff0_16*dmats1[16][22] + coeff0_17*dmats1[17][22] + coeff0_18*dmats1[18][22] + coeff0_19*dmats1[19][22] + coeff0_20*dmats1[20][22] + coeff0_21*dmats1[21][22] + coeff0_22*dmats1[22][22] + coeff0_23*dmats1[23][22] + coeff0_24*dmats1[24][22] + coeff0_25*dmats1[25][22] + coeff0_26*dmats1[26][22] + coeff0_27*dmats1[27][22] + coeff0_28*dmats1[28][22] + coeff0_29*dmats1[29][22] + coeff0_30*dmats1[30][22] + coeff0_31*dmats1[31][22] + coeff0_32*dmats1[32][22] + coeff0_33*dmats1[33][22] + coeff0_34*dmats1[34][22] + coeff0_35*dmats1[35][22] + coeff0_36*dmats1[36][22] + coeff0_37*dmats1[37][22] + coeff0_38*dmats1[38][22] + coeff0_39*dmats1[39][22] + coeff0_40*dmats1[40][22] + coeff0_41*dmats1[41][22] + coeff0_42*dmats1[42][22] + coeff0_43*dmats1[43][22] + coeff0_44*dmats1[44][22] + coeff0_45*dmats1[45][22] + coeff0_46*dmats1[46][22] + coeff0_47*dmats1[47][22] + coeff0_48*dmats1[48][22] + coeff0_49*dmats1[49][22] + coeff0_50*dmats1[50][22] + coeff0_51*dmats1[51][22] + coeff0_52*dmats1[52][22] + coeff0_53*dmats1[53][22] + coeff0_54*dmats1[54][22] + coeff0_55*dmats1[55][22];
          new_coeff0_23 = coeff0_0*dmats1[0][23] + coeff0_1*dmats1[1][23] + coeff0_2*dmats1[2][23] + coeff0_3*dmats1[3][23] + coeff0_4*dmats1[4][23] + coeff0_5*dmats1[5][23] + coeff0_6*dmats1[6][23] + coeff0_7*dmats1[7][23] + coeff0_8*dmats1[8][23] + coeff0_9*dmats1[9][23] + coeff0_10*dmats1[10][23] + coeff0_11*dmats1[11][23] + coeff0_12*dmats1[12][23] + coeff0_13*dmats1[13][23] + coeff0_14*dmats1[14][23] + coeff0_15*dmats1[15][23] + coeff0_16*dmats1[16][23] + coeff0_17*dmats1[17][23] + coeff0_18*dmats1[18][23] + coeff0_19*dmats1[19][23] + coeff0_20*dmats1[20][23] + coeff0_21*dmats1[21][23] + coeff0_22*dmats1[22][23] + coeff0_23*dmats1[23][23] + coeff0_24*dmats1[24][23] + coeff0_25*dmats1[25][23] + coeff0_26*dmats1[26][23] + coeff0_27*dmats1[27][23] + coeff0_28*dmats1[28][23] + coeff0_29*dmats1[29][23] + coeff0_30*dmats1[30][23] + coeff0_31*dmats1[31][23] + coeff0_32*dmats1[32][23] + coeff0_33*dmats1[33][23] + coeff0_34*dmats1[34][23] + coeff0_35*dmats1[35][23] + coeff0_36*dmats1[36][23] + coeff0_37*dmats1[37][23] + coeff0_38*dmats1[38][23] + coeff0_39*dmats1[39][23] + coeff0_40*dmats1[40][23] + coeff0_41*dmats1[41][23] + coeff0_42*dmats1[42][23] + coeff0_43*dmats1[43][23] + coeff0_44*dmats1[44][23] + coeff0_45*dmats1[45][23] + coeff0_46*dmats1[46][23] + coeff0_47*dmats1[47][23] + coeff0_48*dmats1[48][23] + coeff0_49*dmats1[49][23] + coeff0_50*dmats1[50][23] + coeff0_51*dmats1[51][23] + coeff0_52*dmats1[52][23] + coeff0_53*dmats1[53][23] + coeff0_54*dmats1[54][23] + coeff0_55*dmats1[55][23];
          new_coeff0_24 = coeff0_0*dmats1[0][24] + coeff0_1*dmats1[1][24] + coeff0_2*dmats1[2][24] + coeff0_3*dmats1[3][24] + coeff0_4*dmats1[4][24] + coeff0_5*dmats1[5][24] + coeff0_6*dmats1[6][24] + coeff0_7*dmats1[7][24] + coeff0_8*dmats1[8][24] + coeff0_9*dmats1[9][24] + coeff0_10*dmats1[10][24] + coeff0_11*dmats1[11][24] + coeff0_12*dmats1[12][24] + coeff0_13*dmats1[13][24] + coeff0_14*dmats1[14][24] + coeff0_15*dmats1[15][24] + coeff0_16*dmats1[16][24] + coeff0_17*dmats1[17][24] + coeff0_18*dmats1[18][24] + coeff0_19*dmats1[19][24] + coeff0_20*dmats1[20][24] + coeff0_21*dmats1[21][24] + coeff0_22*dmats1[22][24] + coeff0_23*dmats1[23][24] + coeff0_24*dmats1[24][24] + coeff0_25*dmats1[25][24] + coeff0_26*dmats1[26][24] + coeff0_27*dmats1[27][24] + coeff0_28*dmats1[28][24] + coeff0_29*dmats1[29][24] + coeff0_30*dmats1[30][24] + coeff0_31*dmats1[31][24] + coeff0_32*dmats1[32][24] + coeff0_33*dmats1[33][24] + coeff0_34*dmats1[34][24] + coeff0_35*dmats1[35][24] + coeff0_36*dmats1[36][24] + coeff0_37*dmats1[37][24] + coeff0_38*dmats1[38][24] + coeff0_39*dmats1[39][24] + coeff0_40*dmats1[40][24] + coeff0_41*dmats1[41][24] + coeff0_42*dmats1[42][24] + coeff0_43*dmats1[43][24] + coeff0_44*dmats1[44][24] + coeff0_45*dmats1[45][24] + coeff0_46*dmats1[46][24] + coeff0_47*dmats1[47][24] + coeff0_48*dmats1[48][24] + coeff0_49*dmats1[49][24] + coeff0_50*dmats1[50][24] + coeff0_51*dmats1[51][24] + coeff0_52*dmats1[52][24] + coeff0_53*dmats1[53][24] + coeff0_54*dmats1[54][24] + coeff0_55*dmats1[55][24];
          new_coeff0_25 = coeff0_0*dmats1[0][25] + coeff0_1*dmats1[1][25] + coeff0_2*dmats1[2][25] + coeff0_3*dmats1[3][25] + coeff0_4*dmats1[4][25] + coeff0_5*dmats1[5][25] + coeff0_6*dmats1[6][25] + coeff0_7*dmats1[7][25] + coeff0_8*dmats1[8][25] + coeff0_9*dmats1[9][25] + coeff0_10*dmats1[10][25] + coeff0_11*dmats1[11][25] + coeff0_12*dmats1[12][25] + coeff0_13*dmats1[13][25] + coeff0_14*dmats1[14][25] + coeff0_15*dmats1[15][25] + coeff0_16*dmats1[16][25] + coeff0_17*dmats1[17][25] + coeff0_18*dmats1[18][25] + coeff0_19*dmats1[19][25] + coeff0_20*dmats1[20][25] + coeff0_21*dmats1[21][25] + coeff0_22*dmats1[22][25] + coeff0_23*dmats1[23][25] + coeff0_24*dmats1[24][25] + coeff0_25*dmats1[25][25] + coeff0_26*dmats1[26][25] + coeff0_27*dmats1[27][25] + coeff0_28*dmats1[28][25] + coeff0_29*dmats1[29][25] + coeff0_30*dmats1[30][25] + coeff0_31*dmats1[31][25] + coeff0_32*dmats1[32][25] + coeff0_33*dmats1[33][25] + coeff0_34*dmats1[34][25] + coeff0_35*dmats1[35][25] + coeff0_36*dmats1[36][25] + coeff0_37*dmats1[37][25] + coeff0_38*dmats1[38][25] + coeff0_39*dmats1[39][25] + coeff0_40*dmats1[40][25] + coeff0_41*dmats1[41][25] + coeff0_42*dmats1[42][25] + coeff0_43*dmats1[43][25] + coeff0_44*dmats1[44][25] + coeff0_45*dmats1[45][25] + coeff0_46*dmats1[46][25] + coeff0_47*dmats1[47][25] + coeff0_48*dmats1[48][25] + coeff0_49*dmats1[49][25] + coeff0_50*dmats1[50][25] + coeff0_51*dmats1[51][25] + coeff0_52*dmats1[52][25] + coeff0_53*dmats1[53][25] + coeff0_54*dmats1[54][25] + coeff0_55*dmats1[55][25];
          new_coeff0_26 = coeff0_0*dmats1[0][26] + coeff0_1*dmats1[1][26] + coeff0_2*dmats1[2][26] + coeff0_3*dmats1[3][26] + coeff0_4*dmats1[4][26] + coeff0_5*dmats1[5][26] + coeff0_6*dmats1[6][26] + coeff0_7*dmats1[7][26] + coeff0_8*dmats1[8][26] + coeff0_9*dmats1[9][26] + coeff0_10*dmats1[10][26] + coeff0_11*dmats1[11][26] + coeff0_12*dmats1[12][26] + coeff0_13*dmats1[13][26] + coeff0_14*dmats1[14][26] + coeff0_15*dmats1[15][26] + coeff0_16*dmats1[16][26] + coeff0_17*dmats1[17][26] + coeff0_18*dmats1[18][26] + coeff0_19*dmats1[19][26] + coeff0_20*dmats1[20][26] + coeff0_21*dmats1[21][26] + coeff0_22*dmats1[22][26] + coeff0_23*dmats1[23][26] + coeff0_24*dmats1[24][26] + coeff0_25*dmats1[25][26] + coeff0_26*dmats1[26][26] + coeff0_27*dmats1[27][26] + coeff0_28*dmats1[28][26] + coeff0_29*dmats1[29][26] + coeff0_30*dmats1[30][26] + coeff0_31*dmats1[31][26] + coeff0_32*dmats1[32][26] + coeff0_33*dmats1[33][26] + coeff0_34*dmats1[34][26] + coeff0_35*dmats1[35][26] + coeff0_36*dmats1[36][26] + coeff0_37*dmats1[37][26] + coeff0_38*dmats1[38][26] + coeff0_39*dmats1[39][26] + coeff0_40*dmats1[40][26] + coeff0_41*dmats1[41][26] + coeff0_42*dmats1[42][26] + coeff0_43*dmats1[43][26] + coeff0_44*dmats1[44][26] + coeff0_45*dmats1[45][26] + coeff0_46*dmats1[46][26] + coeff0_47*dmats1[47][26] + coeff0_48*dmats1[48][26] + coeff0_49*dmats1[49][26] + coeff0_50*dmats1[50][26] + coeff0_51*dmats1[51][26] + coeff0_52*dmats1[52][26] + coeff0_53*dmats1[53][26] + coeff0_54*dmats1[54][26] + coeff0_55*dmats1[55][26];
          new_coeff0_27 = coeff0_0*dmats1[0][27] + coeff0_1*dmats1[1][27] + coeff0_2*dmats1[2][27] + coeff0_3*dmats1[3][27] + coeff0_4*dmats1[4][27] + coeff0_5*dmats1[5][27] + coeff0_6*dmats1[6][27] + coeff0_7*dmats1[7][27] + coeff0_8*dmats1[8][27] + coeff0_9*dmats1[9][27] + coeff0_10*dmats1[10][27] + coeff0_11*dmats1[11][27] + coeff0_12*dmats1[12][27] + coeff0_13*dmats1[13][27] + coeff0_14*dmats1[14][27] + coeff0_15*dmats1[15][27] + coeff0_16*dmats1[16][27] + coeff0_17*dmats1[17][27] + coeff0_18*dmats1[18][27] + coeff0_19*dmats1[19][27] + coeff0_20*dmats1[20][27] + coeff0_21*dmats1[21][27] + coeff0_22*dmats1[22][27] + coeff0_23*dmats1[23][27] + coeff0_24*dmats1[24][27] + coeff0_25*dmats1[25][27] + coeff0_26*dmats1[26][27] + coeff0_27*dmats1[27][27] + coeff0_28*dmats1[28][27] + coeff0_29*dmats1[29][27] + coeff0_30*dmats1[30][27] + coeff0_31*dmats1[31][27] + coeff0_32*dmats1[32][27] + coeff0_33*dmats1[33][27] + coeff0_34*dmats1[34][27] + coeff0_35*dmats1[35][27] + coeff0_36*dmats1[36][27] + coeff0_37*dmats1[37][27] + coeff0_38*dmats1[38][27] + coeff0_39*dmats1[39][27] + coeff0_40*dmats1[40][27] + coeff0_41*dmats1[41][27] + coeff0_42*dmats1[42][27] + coeff0_43*dmats1[43][27] + coeff0_44*dmats1[44][27] + coeff0_45*dmats1[45][27] + coeff0_46*dmats1[46][27] + coeff0_47*dmats1[47][27] + coeff0_48*dmats1[48][27] + coeff0_49*dmats1[49][27] + coeff0_50*dmats1[50][27] + coeff0_51*dmats1[51][27] + coeff0_52*dmats1[52][27] + coeff0_53*dmats1[53][27] + coeff0_54*dmats1[54][27] + coeff0_55*dmats1[55][27];
          new_coeff0_28 = coeff0_0*dmats1[0][28] + coeff0_1*dmats1[1][28] + coeff0_2*dmats1[2][28] + coeff0_3*dmats1[3][28] + coeff0_4*dmats1[4][28] + coeff0_5*dmats1[5][28] + coeff0_6*dmats1[6][28] + coeff0_7*dmats1[7][28] + coeff0_8*dmats1[8][28] + coeff0_9*dmats1[9][28] + coeff0_10*dmats1[10][28] + coeff0_11*dmats1[11][28] + coeff0_12*dmats1[12][28] + coeff0_13*dmats1[13][28] + coeff0_14*dmats1[14][28] + coeff0_15*dmats1[15][28] + coeff0_16*dmats1[16][28] + coeff0_17*dmats1[17][28] + coeff0_18*dmats1[18][28] + coeff0_19*dmats1[19][28] + coeff0_20*dmats1[20][28] + coeff0_21*dmats1[21][28] + coeff0_22*dmats1[22][28] + coeff0_23*dmats1[23][28] + coeff0_24*dmats1[24][28] + coeff0_25*dmats1[25][28] + coeff0_26*dmats1[26][28] + coeff0_27*dmats1[27][28] + coeff0_28*dmats1[28][28] + coeff0_29*dmats1[29][28] + coeff0_30*dmats1[30][28] + coeff0_31*dmats1[31][28] + coeff0_32*dmats1[32][28] + coeff0_33*dmats1[33][28] + coeff0_34*dmats1[34][28] + coeff0_35*dmats1[35][28] + coeff0_36*dmats1[36][28] + coeff0_37*dmats1[37][28] + coeff0_38*dmats1[38][28] + coeff0_39*dmats1[39][28] + coeff0_40*dmats1[40][28] + coeff0_41*dmats1[41][28] + coeff0_42*dmats1[42][28] + coeff0_43*dmats1[43][28] + coeff0_44*dmats1[44][28] + coeff0_45*dmats1[45][28] + coeff0_46*dmats1[46][28] + coeff0_47*dmats1[47][28] + coeff0_48*dmats1[48][28] + coeff0_49*dmats1[49][28] + coeff0_50*dmats1[50][28] + coeff0_51*dmats1[51][28] + coeff0_52*dmats1[52][28] + coeff0_53*dmats1[53][28] + coeff0_54*dmats1[54][28] + coeff0_55*dmats1[55][28];
          new_coeff0_29 = coeff0_0*dmats1[0][29] + coeff0_1*dmats1[1][29] + coeff0_2*dmats1[2][29] + coeff0_3*dmats1[3][29] + coeff0_4*dmats1[4][29] + coeff0_5*dmats1[5][29] + coeff0_6*dmats1[6][29] + coeff0_7*dmats1[7][29] + coeff0_8*dmats1[8][29] + coeff0_9*dmats1[9][29] + coeff0_10*dmats1[10][29] + coeff0_11*dmats1[11][29] + coeff0_12*dmats1[12][29] + coeff0_13*dmats1[13][29] + coeff0_14*dmats1[14][29] + coeff0_15*dmats1[15][29] + coeff0_16*dmats1[16][29] + coeff0_17*dmats1[17][29] + coeff0_18*dmats1[18][29] + coeff0_19*dmats1[19][29] + coeff0_20*dmats1[20][29] + coeff0_21*dmats1[21][29] + coeff0_22*dmats1[22][29] + coeff0_23*dmats1[23][29] + coeff0_24*dmats1[24][29] + coeff0_25*dmats1[25][29] + coeff0_26*dmats1[26][29] + coeff0_27*dmats1[27][29] + coeff0_28*dmats1[28][29] + coeff0_29*dmats1[29][29] + coeff0_30*dmats1[30][29] + coeff0_31*dmats1[31][29] + coeff0_32*dmats1[32][29] + coeff0_33*dmats1[33][29] + coeff0_34*dmats1[34][29] + coeff0_35*dmats1[35][29] + coeff0_36*dmats1[36][29] + coeff0_37*dmats1[37][29] + coeff0_38*dmats1[38][29] + coeff0_39*dmats1[39][29] + coeff0_40*dmats1[40][29] + coeff0_41*dmats1[41][29] + coeff0_42*dmats1[42][29] + coeff0_43*dmats1[43][29] + coeff0_44*dmats1[44][29] + coeff0_45*dmats1[45][29] + coeff0_46*dmats1[46][29] + coeff0_47*dmats1[47][29] + coeff0_48*dmats1[48][29] + coeff0_49*dmats1[49][29] + coeff0_50*dmats1[50][29] + coeff0_51*dmats1[51][29] + coeff0_52*dmats1[52][29] + coeff0_53*dmats1[53][29] + coeff0_54*dmats1[54][29] + coeff0_55*dmats1[55][29];
          new_coeff0_30 = coeff0_0*dmats1[0][30] + coeff0_1*dmats1[1][30] + coeff0_2*dmats1[2][30] + coeff0_3*dmats1[3][30] + coeff0_4*dmats1[4][30] + coeff0_5*dmats1[5][30] + coeff0_6*dmats1[6][30] + coeff0_7*dmats1[7][30] + coeff0_8*dmats1[8][30] + coeff0_9*dmats1[9][30] + coeff0_10*dmats1[10][30] + coeff0_11*dmats1[11][30] + coeff0_12*dmats1[12][30] + coeff0_13*dmats1[13][30] + coeff0_14*dmats1[14][30] + coeff0_15*dmats1[15][30] + coeff0_16*dmats1[16][30] + coeff0_17*dmats1[17][30] + coeff0_18*dmats1[18][30] + coeff0_19*dmats1[19][30] + coeff0_20*dmats1[20][30] + coeff0_21*dmats1[21][30] + coeff0_22*dmats1[22][30] + coeff0_23*dmats1[23][30] + coeff0_24*dmats1[24][30] + coeff0_25*dmats1[25][30] + coeff0_26*dmats1[26][30] + coeff0_27*dmats1[27][30] + coeff0_28*dmats1[28][30] + coeff0_29*dmats1[29][30] + coeff0_30*dmats1[30][30] + coeff0_31*dmats1[31][30] + coeff0_32*dmats1[32][30] + coeff0_33*dmats1[33][30] + coeff0_34*dmats1[34][30] + coeff0_35*dmats1[35][30] + coeff0_36*dmats1[36][30] + coeff0_37*dmats1[37][30] + coeff0_38*dmats1[38][30] + coeff0_39*dmats1[39][30] + coeff0_40*dmats1[40][30] + coeff0_41*dmats1[41][30] + coeff0_42*dmats1[42][30] + coeff0_43*dmats1[43][30] + coeff0_44*dmats1[44][30] + coeff0_45*dmats1[45][30] + coeff0_46*dmats1[46][30] + coeff0_47*dmats1[47][30] + coeff0_48*dmats1[48][30] + coeff0_49*dmats1[49][30] + coeff0_50*dmats1[50][30] + coeff0_51*dmats1[51][30] + coeff0_52*dmats1[52][30] + coeff0_53*dmats1[53][30] + coeff0_54*dmats1[54][30] + coeff0_55*dmats1[55][30];
          new_coeff0_31 = coeff0_0*dmats1[0][31] + coeff0_1*dmats1[1][31] + coeff0_2*dmats1[2][31] + coeff0_3*dmats1[3][31] + coeff0_4*dmats1[4][31] + coeff0_5*dmats1[5][31] + coeff0_6*dmats1[6][31] + coeff0_7*dmats1[7][31] + coeff0_8*dmats1[8][31] + coeff0_9*dmats1[9][31] + coeff0_10*dmats1[10][31] + coeff0_11*dmats1[11][31] + coeff0_12*dmats1[12][31] + coeff0_13*dmats1[13][31] + coeff0_14*dmats1[14][31] + coeff0_15*dmats1[15][31] + coeff0_16*dmats1[16][31] + coeff0_17*dmats1[17][31] + coeff0_18*dmats1[18][31] + coeff0_19*dmats1[19][31] + coeff0_20*dmats1[20][31] + coeff0_21*dmats1[21][31] + coeff0_22*dmats1[22][31] + coeff0_23*dmats1[23][31] + coeff0_24*dmats1[24][31] + coeff0_25*dmats1[25][31] + coeff0_26*dmats1[26][31] + coeff0_27*dmats1[27][31] + coeff0_28*dmats1[28][31] + coeff0_29*dmats1[29][31] + coeff0_30*dmats1[30][31] + coeff0_31*dmats1[31][31] + coeff0_32*dmats1[32][31] + coeff0_33*dmats1[33][31] + coeff0_34*dmats1[34][31] + coeff0_35*dmats1[35][31] + coeff0_36*dmats1[36][31] + coeff0_37*dmats1[37][31] + coeff0_38*dmats1[38][31] + coeff0_39*dmats1[39][31] + coeff0_40*dmats1[40][31] + coeff0_41*dmats1[41][31] + coeff0_42*dmats1[42][31] + coeff0_43*dmats1[43][31] + coeff0_44*dmats1[44][31] + coeff0_45*dmats1[45][31] + coeff0_46*dmats1[46][31] + coeff0_47*dmats1[47][31] + coeff0_48*dmats1[48][31] + coeff0_49*dmats1[49][31] + coeff0_50*dmats1[50][31] + coeff0_51*dmats1[51][31] + coeff0_52*dmats1[52][31] + coeff0_53*dmats1[53][31] + coeff0_54*dmats1[54][31] + coeff0_55*dmats1[55][31];
          new_coeff0_32 = coeff0_0*dmats1[0][32] + coeff0_1*dmats1[1][32] + coeff0_2*dmats1[2][32] + coeff0_3*dmats1[3][32] + coeff0_4*dmats1[4][32] + coeff0_5*dmats1[5][32] + coeff0_6*dmats1[6][32] + coeff0_7*dmats1[7][32] + coeff0_8*dmats1[8][32] + coeff0_9*dmats1[9][32] + coeff0_10*dmats1[10][32] + coeff0_11*dmats1[11][32] + coeff0_12*dmats1[12][32] + coeff0_13*dmats1[13][32] + coeff0_14*dmats1[14][32] + coeff0_15*dmats1[15][32] + coeff0_16*dmats1[16][32] + coeff0_17*dmats1[17][32] + coeff0_18*dmats1[18][32] + coeff0_19*dmats1[19][32] + coeff0_20*dmats1[20][32] + coeff0_21*dmats1[21][32] + coeff0_22*dmats1[22][32] + coeff0_23*dmats1[23][32] + coeff0_24*dmats1[24][32] + coeff0_25*dmats1[25][32] + coeff0_26*dmats1[26][32] + coeff0_27*dmats1[27][32] + coeff0_28*dmats1[28][32] + coeff0_29*dmats1[29][32] + coeff0_30*dmats1[30][32] + coeff0_31*dmats1[31][32] + coeff0_32*dmats1[32][32] + coeff0_33*dmats1[33][32] + coeff0_34*dmats1[34][32] + coeff0_35*dmats1[35][32] + coeff0_36*dmats1[36][32] + coeff0_37*dmats1[37][32] + coeff0_38*dmats1[38][32] + coeff0_39*dmats1[39][32] + coeff0_40*dmats1[40][32] + coeff0_41*dmats1[41][32] + coeff0_42*dmats1[42][32] + coeff0_43*dmats1[43][32] + coeff0_44*dmats1[44][32] + coeff0_45*dmats1[45][32] + coeff0_46*dmats1[46][32] + coeff0_47*dmats1[47][32] + coeff0_48*dmats1[48][32] + coeff0_49*dmats1[49][32] + coeff0_50*dmats1[50][32] + coeff0_51*dmats1[51][32] + coeff0_52*dmats1[52][32] + coeff0_53*dmats1[53][32] + coeff0_54*dmats1[54][32] + coeff0_55*dmats1[55][32];
          new_coeff0_33 = coeff0_0*dmats1[0][33] + coeff0_1*dmats1[1][33] + coeff0_2*dmats1[2][33] + coeff0_3*dmats1[3][33] + coeff0_4*dmats1[4][33] + coeff0_5*dmats1[5][33] + coeff0_6*dmats1[6][33] + coeff0_7*dmats1[7][33] + coeff0_8*dmats1[8][33] + coeff0_9*dmats1[9][33] + coeff0_10*dmats1[10][33] + coeff0_11*dmats1[11][33] + coeff0_12*dmats1[12][33] + coeff0_13*dmats1[13][33] + coeff0_14*dmats1[14][33] + coeff0_15*dmats1[15][33] + coeff0_16*dmats1[16][33] + coeff0_17*dmats1[17][33] + coeff0_18*dmats1[18][33] + coeff0_19*dmats1[19][33] + coeff0_20*dmats1[20][33] + coeff0_21*dmats1[21][33] + coeff0_22*dmats1[22][33] + coeff0_23*dmats1[23][33] + coeff0_24*dmats1[24][33] + coeff0_25*dmats1[25][33] + coeff0_26*dmats1[26][33] + coeff0_27*dmats1[27][33] + coeff0_28*dmats1[28][33] + coeff0_29*dmats1[29][33] + coeff0_30*dmats1[30][33] + coeff0_31*dmats1[31][33] + coeff0_32*dmats1[32][33] + coeff0_33*dmats1[33][33] + coeff0_34*dmats1[34][33] + coeff0_35*dmats1[35][33] + coeff0_36*dmats1[36][33] + coeff0_37*dmats1[37][33] + coeff0_38*dmats1[38][33] + coeff0_39*dmats1[39][33] + coeff0_40*dmats1[40][33] + coeff0_41*dmats1[41][33] + coeff0_42*dmats1[42][33] + coeff0_43*dmats1[43][33] + coeff0_44*dmats1[44][33] + coeff0_45*dmats1[45][33] + coeff0_46*dmats1[46][33] + coeff0_47*dmats1[47][33] + coeff0_48*dmats1[48][33] + coeff0_49*dmats1[49][33] + coeff0_50*dmats1[50][33] + coeff0_51*dmats1[51][33] + coeff0_52*dmats1[52][33] + coeff0_53*dmats1[53][33] + coeff0_54*dmats1[54][33] + coeff0_55*dmats1[55][33];
          new_coeff0_34 = coeff0_0*dmats1[0][34] + coeff0_1*dmats1[1][34] + coeff0_2*dmats1[2][34] + coeff0_3*dmats1[3][34] + coeff0_4*dmats1[4][34] + coeff0_5*dmats1[5][34] + coeff0_6*dmats1[6][34] + coeff0_7*dmats1[7][34] + coeff0_8*dmats1[8][34] + coeff0_9*dmats1[9][34] + coeff0_10*dmats1[10][34] + coeff0_11*dmats1[11][34] + coeff0_12*dmats1[12][34] + coeff0_13*dmats1[13][34] + coeff0_14*dmats1[14][34] + coeff0_15*dmats1[15][34] + coeff0_16*dmats1[16][34] + coeff0_17*dmats1[17][34] + coeff0_18*dmats1[18][34] + coeff0_19*dmats1[19][34] + coeff0_20*dmats1[20][34] + coeff0_21*dmats1[21][34] + coeff0_22*dmats1[22][34] + coeff0_23*dmats1[23][34] + coeff0_24*dmats1[24][34] + coeff0_25*dmats1[25][34] + coeff0_26*dmats1[26][34] + coeff0_27*dmats1[27][34] + coeff0_28*dmats1[28][34] + coeff0_29*dmats1[29][34] + coeff0_30*dmats1[30][34] + coeff0_31*dmats1[31][34] + coeff0_32*dmats1[32][34] + coeff0_33*dmats1[33][34] + coeff0_34*dmats1[34][34] + coeff0_35*dmats1[35][34] + coeff0_36*dmats1[36][34] + coeff0_37*dmats1[37][34] + coeff0_38*dmats1[38][34] + coeff0_39*dmats1[39][34] + coeff0_40*dmats1[40][34] + coeff0_41*dmats1[41][34] + coeff0_42*dmats1[42][34] + coeff0_43*dmats1[43][34] + coeff0_44*dmats1[44][34] + coeff0_45*dmats1[45][34] + coeff0_46*dmats1[46][34] + coeff0_47*dmats1[47][34] + coeff0_48*dmats1[48][34] + coeff0_49*dmats1[49][34] + coeff0_50*dmats1[50][34] + coeff0_51*dmats1[51][34] + coeff0_52*dmats1[52][34] + coeff0_53*dmats1[53][34] + coeff0_54*dmats1[54][34] + coeff0_55*dmats1[55][34];
          new_coeff0_35 = coeff0_0*dmats1[0][35] + coeff0_1*dmats1[1][35] + coeff0_2*dmats1[2][35] + coeff0_3*dmats1[3][35] + coeff0_4*dmats1[4][35] + coeff0_5*dmats1[5][35] + coeff0_6*dmats1[6][35] + coeff0_7*dmats1[7][35] + coeff0_8*dmats1[8][35] + coeff0_9*dmats1[9][35] + coeff0_10*dmats1[10][35] + coeff0_11*dmats1[11][35] + coeff0_12*dmats1[12][35] + coeff0_13*dmats1[13][35] + coeff0_14*dmats1[14][35] + coeff0_15*dmats1[15][35] + coeff0_16*dmats1[16][35] + coeff0_17*dmats1[17][35] + coeff0_18*dmats1[18][35] + coeff0_19*dmats1[19][35] + coeff0_20*dmats1[20][35] + coeff0_21*dmats1[21][35] + coeff0_22*dmats1[22][35] + coeff0_23*dmats1[23][35] + coeff0_24*dmats1[24][35] + coeff0_25*dmats1[25][35] + coeff0_26*dmats1[26][35] + coeff0_27*dmats1[27][35] + coeff0_28*dmats1[28][35] + coeff0_29*dmats1[29][35] + coeff0_30*dmats1[30][35] + coeff0_31*dmats1[31][35] + coeff0_32*dmats1[32][35] + coeff0_33*dmats1[33][35] + coeff0_34*dmats1[34][35] + coeff0_35*dmats1[35][35] + coeff0_36*dmats1[36][35] + coeff0_37*dmats1[37][35] + coeff0_38*dmats1[38][35] + coeff0_39*dmats1[39][35] + coeff0_40*dmats1[40][35] + coeff0_41*dmats1[41][35] + coeff0_42*dmats1[42][35] + coeff0_43*dmats1[43][35] + coeff0_44*dmats1[44][35] + coeff0_45*dmats1[45][35] + coeff0_46*dmats1[46][35] + coeff0_47*dmats1[47][35] + coeff0_48*dmats1[48][35] + coeff0_49*dmats1[49][35] + coeff0_50*dmats1[50][35] + coeff0_51*dmats1[51][35] + coeff0_52*dmats1[52][35] + coeff0_53*dmats1[53][35] + coeff0_54*dmats1[54][35] + coeff0_55*dmats1[55][35];
          new_coeff0_36 = coeff0_0*dmats1[0][36] + coeff0_1*dmats1[1][36] + coeff0_2*dmats1[2][36] + coeff0_3*dmats1[3][36] + coeff0_4*dmats1[4][36] + coeff0_5*dmats1[5][36] + coeff0_6*dmats1[6][36] + coeff0_7*dmats1[7][36] + coeff0_8*dmats1[8][36] + coeff0_9*dmats1[9][36] + coeff0_10*dmats1[10][36] + coeff0_11*dmats1[11][36] + coeff0_12*dmats1[12][36] + coeff0_13*dmats1[13][36] + coeff0_14*dmats1[14][36] + coeff0_15*dmats1[15][36] + coeff0_16*dmats1[16][36] + coeff0_17*dmats1[17][36] + coeff0_18*dmats1[18][36] + coeff0_19*dmats1[19][36] + coeff0_20*dmats1[20][36] + coeff0_21*dmats1[21][36] + coeff0_22*dmats1[22][36] + coeff0_23*dmats1[23][36] + coeff0_24*dmats1[24][36] + coeff0_25*dmats1[25][36] + coeff0_26*dmats1[26][36] + coeff0_27*dmats1[27][36] + coeff0_28*dmats1[28][36] + coeff0_29*dmats1[29][36] + coeff0_30*dmats1[30][36] + coeff0_31*dmats1[31][36] + coeff0_32*dmats1[32][36] + coeff0_33*dmats1[33][36] + coeff0_34*dmats1[34][36] + coeff0_35*dmats1[35][36] + coeff0_36*dmats1[36][36] + coeff0_37*dmats1[37][36] + coeff0_38*dmats1[38][36] + coeff0_39*dmats1[39][36] + coeff0_40*dmats1[40][36] + coeff0_41*dmats1[41][36] + coeff0_42*dmats1[42][36] + coeff0_43*dmats1[43][36] + coeff0_44*dmats1[44][36] + coeff0_45*dmats1[45][36] + coeff0_46*dmats1[46][36] + coeff0_47*dmats1[47][36] + coeff0_48*dmats1[48][36] + coeff0_49*dmats1[49][36] + coeff0_50*dmats1[50][36] + coeff0_51*dmats1[51][36] + coeff0_52*dmats1[52][36] + coeff0_53*dmats1[53][36] + coeff0_54*dmats1[54][36] + coeff0_55*dmats1[55][36];
          new_coeff0_37 = coeff0_0*dmats1[0][37] + coeff0_1*dmats1[1][37] + coeff0_2*dmats1[2][37] + coeff0_3*dmats1[3][37] + coeff0_4*dmats1[4][37] + coeff0_5*dmats1[5][37] + coeff0_6*dmats1[6][37] + coeff0_7*dmats1[7][37] + coeff0_8*dmats1[8][37] + coeff0_9*dmats1[9][37] + coeff0_10*dmats1[10][37] + coeff0_11*dmats1[11][37] + coeff0_12*dmats1[12][37] + coeff0_13*dmats1[13][37] + coeff0_14*dmats1[14][37] + coeff0_15*dmats1[15][37] + coeff0_16*dmats1[16][37] + coeff0_17*dmats1[17][37] + coeff0_18*dmats1[18][37] + coeff0_19*dmats1[19][37] + coeff0_20*dmats1[20][37] + coeff0_21*dmats1[21][37] + coeff0_22*dmats1[22][37] + coeff0_23*dmats1[23][37] + coeff0_24*dmats1[24][37] + coeff0_25*dmats1[25][37] + coeff0_26*dmats1[26][37] + coeff0_27*dmats1[27][37] + coeff0_28*dmats1[28][37] + coeff0_29*dmats1[29][37] + coeff0_30*dmats1[30][37] + coeff0_31*dmats1[31][37] + coeff0_32*dmats1[32][37] + coeff0_33*dmats1[33][37] + coeff0_34*dmats1[34][37] + coeff0_35*dmats1[35][37] + coeff0_36*dmats1[36][37] + coeff0_37*dmats1[37][37] + coeff0_38*dmats1[38][37] + coeff0_39*dmats1[39][37] + coeff0_40*dmats1[40][37] + coeff0_41*dmats1[41][37] + coeff0_42*dmats1[42][37] + coeff0_43*dmats1[43][37] + coeff0_44*dmats1[44][37] + coeff0_45*dmats1[45][37] + coeff0_46*dmats1[46][37] + coeff0_47*dmats1[47][37] + coeff0_48*dmats1[48][37] + coeff0_49*dmats1[49][37] + coeff0_50*dmats1[50][37] + coeff0_51*dmats1[51][37] + coeff0_52*dmats1[52][37] + coeff0_53*dmats1[53][37] + coeff0_54*dmats1[54][37] + coeff0_55*dmats1[55][37];
          new_coeff0_38 = coeff0_0*dmats1[0][38] + coeff0_1*dmats1[1][38] + coeff0_2*dmats1[2][38] + coeff0_3*dmats1[3][38] + coeff0_4*dmats1[4][38] + coeff0_5*dmats1[5][38] + coeff0_6*dmats1[6][38] + coeff0_7*dmats1[7][38] + coeff0_8*dmats1[8][38] + coeff0_9*dmats1[9][38] + coeff0_10*dmats1[10][38] + coeff0_11*dmats1[11][38] + coeff0_12*dmats1[12][38] + coeff0_13*dmats1[13][38] + coeff0_14*dmats1[14][38] + coeff0_15*dmats1[15][38] + coeff0_16*dmats1[16][38] + coeff0_17*dmats1[17][38] + coeff0_18*dmats1[18][38] + coeff0_19*dmats1[19][38] + coeff0_20*dmats1[20][38] + coeff0_21*dmats1[21][38] + coeff0_22*dmats1[22][38] + coeff0_23*dmats1[23][38] + coeff0_24*dmats1[24][38] + coeff0_25*dmats1[25][38] + coeff0_26*dmats1[26][38] + coeff0_27*dmats1[27][38] + coeff0_28*dmats1[28][38] + coeff0_29*dmats1[29][38] + coeff0_30*dmats1[30][38] + coeff0_31*dmats1[31][38] + coeff0_32*dmats1[32][38] + coeff0_33*dmats1[33][38] + coeff0_34*dmats1[34][38] + coeff0_35*dmats1[35][38] + coeff0_36*dmats1[36][38] + coeff0_37*dmats1[37][38] + coeff0_38*dmats1[38][38] + coeff0_39*dmats1[39][38] + coeff0_40*dmats1[40][38] + coeff0_41*dmats1[41][38] + coeff0_42*dmats1[42][38] + coeff0_43*dmats1[43][38] + coeff0_44*dmats1[44][38] + coeff0_45*dmats1[45][38] + coeff0_46*dmats1[46][38] + coeff0_47*dmats1[47][38] + coeff0_48*dmats1[48][38] + coeff0_49*dmats1[49][38] + coeff0_50*dmats1[50][38] + coeff0_51*dmats1[51][38] + coeff0_52*dmats1[52][38] + coeff0_53*dmats1[53][38] + coeff0_54*dmats1[54][38] + coeff0_55*dmats1[55][38];
          new_coeff0_39 = coeff0_0*dmats1[0][39] + coeff0_1*dmats1[1][39] + coeff0_2*dmats1[2][39] + coeff0_3*dmats1[3][39] + coeff0_4*dmats1[4][39] + coeff0_5*dmats1[5][39] + coeff0_6*dmats1[6][39] + coeff0_7*dmats1[7][39] + coeff0_8*dmats1[8][39] + coeff0_9*dmats1[9][39] + coeff0_10*dmats1[10][39] + coeff0_11*dmats1[11][39] + coeff0_12*dmats1[12][39] + coeff0_13*dmats1[13][39] + coeff0_14*dmats1[14][39] + coeff0_15*dmats1[15][39] + coeff0_16*dmats1[16][39] + coeff0_17*dmats1[17][39] + coeff0_18*dmats1[18][39] + coeff0_19*dmats1[19][39] + coeff0_20*dmats1[20][39] + coeff0_21*dmats1[21][39] + coeff0_22*dmats1[22][39] + coeff0_23*dmats1[23][39] + coeff0_24*dmats1[24][39] + coeff0_25*dmats1[25][39] + coeff0_26*dmats1[26][39] + coeff0_27*dmats1[27][39] + coeff0_28*dmats1[28][39] + coeff0_29*dmats1[29][39] + coeff0_30*dmats1[30][39] + coeff0_31*dmats1[31][39] + coeff0_32*dmats1[32][39] + coeff0_33*dmats1[33][39] + coeff0_34*dmats1[34][39] + coeff0_35*dmats1[35][39] + coeff0_36*dmats1[36][39] + coeff0_37*dmats1[37][39] + coeff0_38*dmats1[38][39] + coeff0_39*dmats1[39][39] + coeff0_40*dmats1[40][39] + coeff0_41*dmats1[41][39] + coeff0_42*dmats1[42][39] + coeff0_43*dmats1[43][39] + coeff0_44*dmats1[44][39] + coeff0_45*dmats1[45][39] + coeff0_46*dmats1[46][39] + coeff0_47*dmats1[47][39] + coeff0_48*dmats1[48][39] + coeff0_49*dmats1[49][39] + coeff0_50*dmats1[50][39] + coeff0_51*dmats1[51][39] + coeff0_52*dmats1[52][39] + coeff0_53*dmats1[53][39] + coeff0_54*dmats1[54][39] + coeff0_55*dmats1[55][39];
          new_coeff0_40 = coeff0_0*dmats1[0][40] + coeff0_1*dmats1[1][40] + coeff0_2*dmats1[2][40] + coeff0_3*dmats1[3][40] + coeff0_4*dmats1[4][40] + coeff0_5*dmats1[5][40] + coeff0_6*dmats1[6][40] + coeff0_7*dmats1[7][40] + coeff0_8*dmats1[8][40] + coeff0_9*dmats1[9][40] + coeff0_10*dmats1[10][40] + coeff0_11*dmats1[11][40] + coeff0_12*dmats1[12][40] + coeff0_13*dmats1[13][40] + coeff0_14*dmats1[14][40] + coeff0_15*dmats1[15][40] + coeff0_16*dmats1[16][40] + coeff0_17*dmats1[17][40] + coeff0_18*dmats1[18][40] + coeff0_19*dmats1[19][40] + coeff0_20*dmats1[20][40] + coeff0_21*dmats1[21][40] + coeff0_22*dmats1[22][40] + coeff0_23*dmats1[23][40] + coeff0_24*dmats1[24][40] + coeff0_25*dmats1[25][40] + coeff0_26*dmats1[26][40] + coeff0_27*dmats1[27][40] + coeff0_28*dmats1[28][40] + coeff0_29*dmats1[29][40] + coeff0_30*dmats1[30][40] + coeff0_31*dmats1[31][40] + coeff0_32*dmats1[32][40] + coeff0_33*dmats1[33][40] + coeff0_34*dmats1[34][40] + coeff0_35*dmats1[35][40] + coeff0_36*dmats1[36][40] + coeff0_37*dmats1[37][40] + coeff0_38*dmats1[38][40] + coeff0_39*dmats1[39][40] + coeff0_40*dmats1[40][40] + coeff0_41*dmats1[41][40] + coeff0_42*dmats1[42][40] + coeff0_43*dmats1[43][40] + coeff0_44*dmats1[44][40] + coeff0_45*dmats1[45][40] + coeff0_46*dmats1[46][40] + coeff0_47*dmats1[47][40] + coeff0_48*dmats1[48][40] + coeff0_49*dmats1[49][40] + coeff0_50*dmats1[50][40] + coeff0_51*dmats1[51][40] + coeff0_52*dmats1[52][40] + coeff0_53*dmats1[53][40] + coeff0_54*dmats1[54][40] + coeff0_55*dmats1[55][40];
          new_coeff0_41 = coeff0_0*dmats1[0][41] + coeff0_1*dmats1[1][41] + coeff0_2*dmats1[2][41] + coeff0_3*dmats1[3][41] + coeff0_4*dmats1[4][41] + coeff0_5*dmats1[5][41] + coeff0_6*dmats1[6][41] + coeff0_7*dmats1[7][41] + coeff0_8*dmats1[8][41] + coeff0_9*dmats1[9][41] + coeff0_10*dmats1[10][41] + coeff0_11*dmats1[11][41] + coeff0_12*dmats1[12][41] + coeff0_13*dmats1[13][41] + coeff0_14*dmats1[14][41] + coeff0_15*dmats1[15][41] + coeff0_16*dmats1[16][41] + coeff0_17*dmats1[17][41] + coeff0_18*dmats1[18][41] + coeff0_19*dmats1[19][41] + coeff0_20*dmats1[20][41] + coeff0_21*dmats1[21][41] + coeff0_22*dmats1[22][41] + coeff0_23*dmats1[23][41] + coeff0_24*dmats1[24][41] + coeff0_25*dmats1[25][41] + coeff0_26*dmats1[26][41] + coeff0_27*dmats1[27][41] + coeff0_28*dmats1[28][41] + coeff0_29*dmats1[29][41] + coeff0_30*dmats1[30][41] + coeff0_31*dmats1[31][41] + coeff0_32*dmats1[32][41] + coeff0_33*dmats1[33][41] + coeff0_34*dmats1[34][41] + coeff0_35*dmats1[35][41] + coeff0_36*dmats1[36][41] + coeff0_37*dmats1[37][41] + coeff0_38*dmats1[38][41] + coeff0_39*dmats1[39][41] + coeff0_40*dmats1[40][41] + coeff0_41*dmats1[41][41] + coeff0_42*dmats1[42][41] + coeff0_43*dmats1[43][41] + coeff0_44*dmats1[44][41] + coeff0_45*dmats1[45][41] + coeff0_46*dmats1[46][41] + coeff0_47*dmats1[47][41] + coeff0_48*dmats1[48][41] + coeff0_49*dmats1[49][41] + coeff0_50*dmats1[50][41] + coeff0_51*dmats1[51][41] + coeff0_52*dmats1[52][41] + coeff0_53*dmats1[53][41] + coeff0_54*dmats1[54][41] + coeff0_55*dmats1[55][41];
          new_coeff0_42 = coeff0_0*dmats1[0][42] + coeff0_1*dmats1[1][42] + coeff0_2*dmats1[2][42] + coeff0_3*dmats1[3][42] + coeff0_4*dmats1[4][42] + coeff0_5*dmats1[5][42] + coeff0_6*dmats1[6][42] + coeff0_7*dmats1[7][42] + coeff0_8*dmats1[8][42] + coeff0_9*dmats1[9][42] + coeff0_10*dmats1[10][42] + coeff0_11*dmats1[11][42] + coeff0_12*dmats1[12][42] + coeff0_13*dmats1[13][42] + coeff0_14*dmats1[14][42] + coeff0_15*dmats1[15][42] + coeff0_16*dmats1[16][42] + coeff0_17*dmats1[17][42] + coeff0_18*dmats1[18][42] + coeff0_19*dmats1[19][42] + coeff0_20*dmats1[20][42] + coeff0_21*dmats1[21][42] + coeff0_22*dmats1[22][42] + coeff0_23*dmats1[23][42] + coeff0_24*dmats1[24][42] + coeff0_25*dmats1[25][42] + coeff0_26*dmats1[26][42] + coeff0_27*dmats1[27][42] + coeff0_28*dmats1[28][42] + coeff0_29*dmats1[29][42] + coeff0_30*dmats1[30][42] + coeff0_31*dmats1[31][42] + coeff0_32*dmats1[32][42] + coeff0_33*dmats1[33][42] + coeff0_34*dmats1[34][42] + coeff0_35*dmats1[35][42] + coeff0_36*dmats1[36][42] + coeff0_37*dmats1[37][42] + coeff0_38*dmats1[38][42] + coeff0_39*dmats1[39][42] + coeff0_40*dmats1[40][42] + coeff0_41*dmats1[41][42] + coeff0_42*dmats1[42][42] + coeff0_43*dmats1[43][42] + coeff0_44*dmats1[44][42] + coeff0_45*dmats1[45][42] + coeff0_46*dmats1[46][42] + coeff0_47*dmats1[47][42] + coeff0_48*dmats1[48][42] + coeff0_49*dmats1[49][42] + coeff0_50*dmats1[50][42] + coeff0_51*dmats1[51][42] + coeff0_52*dmats1[52][42] + coeff0_53*dmats1[53][42] + coeff0_54*dmats1[54][42] + coeff0_55*dmats1[55][42];
          new_coeff0_43 = coeff0_0*dmats1[0][43] + coeff0_1*dmats1[1][43] + coeff0_2*dmats1[2][43] + coeff0_3*dmats1[3][43] + coeff0_4*dmats1[4][43] + coeff0_5*dmats1[5][43] + coeff0_6*dmats1[6][43] + coeff0_7*dmats1[7][43] + coeff0_8*dmats1[8][43] + coeff0_9*dmats1[9][43] + coeff0_10*dmats1[10][43] + coeff0_11*dmats1[11][43] + coeff0_12*dmats1[12][43] + coeff0_13*dmats1[13][43] + coeff0_14*dmats1[14][43] + coeff0_15*dmats1[15][43] + coeff0_16*dmats1[16][43] + coeff0_17*dmats1[17][43] + coeff0_18*dmats1[18][43] + coeff0_19*dmats1[19][43] + coeff0_20*dmats1[20][43] + coeff0_21*dmats1[21][43] + coeff0_22*dmats1[22][43] + coeff0_23*dmats1[23][43] + coeff0_24*dmats1[24][43] + coeff0_25*dmats1[25][43] + coeff0_26*dmats1[26][43] + coeff0_27*dmats1[27][43] + coeff0_28*dmats1[28][43] + coeff0_29*dmats1[29][43] + coeff0_30*dmats1[30][43] + coeff0_31*dmats1[31][43] + coeff0_32*dmats1[32][43] + coeff0_33*dmats1[33][43] + coeff0_34*dmats1[34][43] + coeff0_35*dmats1[35][43] + coeff0_36*dmats1[36][43] + coeff0_37*dmats1[37][43] + coeff0_38*dmats1[38][43] + coeff0_39*dmats1[39][43] + coeff0_40*dmats1[40][43] + coeff0_41*dmats1[41][43] + coeff0_42*dmats1[42][43] + coeff0_43*dmats1[43][43] + coeff0_44*dmats1[44][43] + coeff0_45*dmats1[45][43] + coeff0_46*dmats1[46][43] + coeff0_47*dmats1[47][43] + coeff0_48*dmats1[48][43] + coeff0_49*dmats1[49][43] + coeff0_50*dmats1[50][43] + coeff0_51*dmats1[51][43] + coeff0_52*dmats1[52][43] + coeff0_53*dmats1[53][43] + coeff0_54*dmats1[54][43] + coeff0_55*dmats1[55][43];
          new_coeff0_44 = coeff0_0*dmats1[0][44] + coeff0_1*dmats1[1][44] + coeff0_2*dmats1[2][44] + coeff0_3*dmats1[3][44] + coeff0_4*dmats1[4][44] + coeff0_5*dmats1[5][44] + coeff0_6*dmats1[6][44] + coeff0_7*dmats1[7][44] + coeff0_8*dmats1[8][44] + coeff0_9*dmats1[9][44] + coeff0_10*dmats1[10][44] + coeff0_11*dmats1[11][44] + coeff0_12*dmats1[12][44] + coeff0_13*dmats1[13][44] + coeff0_14*dmats1[14][44] + coeff0_15*dmats1[15][44] + coeff0_16*dmats1[16][44] + coeff0_17*dmats1[17][44] + coeff0_18*dmats1[18][44] + coeff0_19*dmats1[19][44] + coeff0_20*dmats1[20][44] + coeff0_21*dmats1[21][44] + coeff0_22*dmats1[22][44] + coeff0_23*dmats1[23][44] + coeff0_24*dmats1[24][44] + coeff0_25*dmats1[25][44] + coeff0_26*dmats1[26][44] + coeff0_27*dmats1[27][44] + coeff0_28*dmats1[28][44] + coeff0_29*dmats1[29][44] + coeff0_30*dmats1[30][44] + coeff0_31*dmats1[31][44] + coeff0_32*dmats1[32][44] + coeff0_33*dmats1[33][44] + coeff0_34*dmats1[34][44] + coeff0_35*dmats1[35][44] + coeff0_36*dmats1[36][44] + coeff0_37*dmats1[37][44] + coeff0_38*dmats1[38][44] + coeff0_39*dmats1[39][44] + coeff0_40*dmats1[40][44] + coeff0_41*dmats1[41][44] + coeff0_42*dmats1[42][44] + coeff0_43*dmats1[43][44] + coeff0_44*dmats1[44][44] + coeff0_45*dmats1[45][44] + coeff0_46*dmats1[46][44] + coeff0_47*dmats1[47][44] + coeff0_48*dmats1[48][44] + coeff0_49*dmats1[49][44] + coeff0_50*dmats1[50][44] + coeff0_51*dmats1[51][44] + coeff0_52*dmats1[52][44] + coeff0_53*dmats1[53][44] + coeff0_54*dmats1[54][44] + coeff0_55*dmats1[55][44];
          new_coeff0_45 = coeff0_0*dmats1[0][45] + coeff0_1*dmats1[1][45] + coeff0_2*dmats1[2][45] + coeff0_3*dmats1[3][45] + coeff0_4*dmats1[4][45] + coeff0_5*dmats1[5][45] + coeff0_6*dmats1[6][45] + coeff0_7*dmats1[7][45] + coeff0_8*dmats1[8][45] + coeff0_9*dmats1[9][45] + coeff0_10*dmats1[10][45] + coeff0_11*dmats1[11][45] + coeff0_12*dmats1[12][45] + coeff0_13*dmats1[13][45] + coeff0_14*dmats1[14][45] + coeff0_15*dmats1[15][45] + coeff0_16*dmats1[16][45] + coeff0_17*dmats1[17][45] + coeff0_18*dmats1[18][45] + coeff0_19*dmats1[19][45] + coeff0_20*dmats1[20][45] + coeff0_21*dmats1[21][45] + coeff0_22*dmats1[22][45] + coeff0_23*dmats1[23][45] + coeff0_24*dmats1[24][45] + coeff0_25*dmats1[25][45] + coeff0_26*dmats1[26][45] + coeff0_27*dmats1[27][45] + coeff0_28*dmats1[28][45] + coeff0_29*dmats1[29][45] + coeff0_30*dmats1[30][45] + coeff0_31*dmats1[31][45] + coeff0_32*dmats1[32][45] + coeff0_33*dmats1[33][45] + coeff0_34*dmats1[34][45] + coeff0_35*dmats1[35][45] + coeff0_36*dmats1[36][45] + coeff0_37*dmats1[37][45] + coeff0_38*dmats1[38][45] + coeff0_39*dmats1[39][45] + coeff0_40*dmats1[40][45] + coeff0_41*dmats1[41][45] + coeff0_42*dmats1[42][45] + coeff0_43*dmats1[43][45] + coeff0_44*dmats1[44][45] + coeff0_45*dmats1[45][45] + coeff0_46*dmats1[46][45] + coeff0_47*dmats1[47][45] + coeff0_48*dmats1[48][45] + coeff0_49*dmats1[49][45] + coeff0_50*dmats1[50][45] + coeff0_51*dmats1[51][45] + coeff0_52*dmats1[52][45] + coeff0_53*dmats1[53][45] + coeff0_54*dmats1[54][45] + coeff0_55*dmats1[55][45];
          new_coeff0_46 = coeff0_0*dmats1[0][46] + coeff0_1*dmats1[1][46] + coeff0_2*dmats1[2][46] + coeff0_3*dmats1[3][46] + coeff0_4*dmats1[4][46] + coeff0_5*dmats1[5][46] + coeff0_6*dmats1[6][46] + coeff0_7*dmats1[7][46] + coeff0_8*dmats1[8][46] + coeff0_9*dmats1[9][46] + coeff0_10*dmats1[10][46] + coeff0_11*dmats1[11][46] + coeff0_12*dmats1[12][46] + coeff0_13*dmats1[13][46] + coeff0_14*dmats1[14][46] + coeff0_15*dmats1[15][46] + coeff0_16*dmats1[16][46] + coeff0_17*dmats1[17][46] + coeff0_18*dmats1[18][46] + coeff0_19*dmats1[19][46] + coeff0_20*dmats1[20][46] + coeff0_21*dmats1[21][46] + coeff0_22*dmats1[22][46] + coeff0_23*dmats1[23][46] + coeff0_24*dmats1[24][46] + coeff0_25*dmats1[25][46] + coeff0_26*dmats1[26][46] + coeff0_27*dmats1[27][46] + coeff0_28*dmats1[28][46] + coeff0_29*dmats1[29][46] + coeff0_30*dmats1[30][46] + coeff0_31*dmats1[31][46] + coeff0_32*dmats1[32][46] + coeff0_33*dmats1[33][46] + coeff0_34*dmats1[34][46] + coeff0_35*dmats1[35][46] + coeff0_36*dmats1[36][46] + coeff0_37*dmats1[37][46] + coeff0_38*dmats1[38][46] + coeff0_39*dmats1[39][46] + coeff0_40*dmats1[40][46] + coeff0_41*dmats1[41][46] + coeff0_42*dmats1[42][46] + coeff0_43*dmats1[43][46] + coeff0_44*dmats1[44][46] + coeff0_45*dmats1[45][46] + coeff0_46*dmats1[46][46] + coeff0_47*dmats1[47][46] + coeff0_48*dmats1[48][46] + coeff0_49*dmats1[49][46] + coeff0_50*dmats1[50][46] + coeff0_51*dmats1[51][46] + coeff0_52*dmats1[52][46] + coeff0_53*dmats1[53][46] + coeff0_54*dmats1[54][46] + coeff0_55*dmats1[55][46];
          new_coeff0_47 = coeff0_0*dmats1[0][47] + coeff0_1*dmats1[1][47] + coeff0_2*dmats1[2][47] + coeff0_3*dmats1[3][47] + coeff0_4*dmats1[4][47] + coeff0_5*dmats1[5][47] + coeff0_6*dmats1[6][47] + coeff0_7*dmats1[7][47] + coeff0_8*dmats1[8][47] + coeff0_9*dmats1[9][47] + coeff0_10*dmats1[10][47] + coeff0_11*dmats1[11][47] + coeff0_12*dmats1[12][47] + coeff0_13*dmats1[13][47] + coeff0_14*dmats1[14][47] + coeff0_15*dmats1[15][47] + coeff0_16*dmats1[16][47] + coeff0_17*dmats1[17][47] + coeff0_18*dmats1[18][47] + coeff0_19*dmats1[19][47] + coeff0_20*dmats1[20][47] + coeff0_21*dmats1[21][47] + coeff0_22*dmats1[22][47] + coeff0_23*dmats1[23][47] + coeff0_24*dmats1[24][47] + coeff0_25*dmats1[25][47] + coeff0_26*dmats1[26][47] + coeff0_27*dmats1[27][47] + coeff0_28*dmats1[28][47] + coeff0_29*dmats1[29][47] + coeff0_30*dmats1[30][47] + coeff0_31*dmats1[31][47] + coeff0_32*dmats1[32][47] + coeff0_33*dmats1[33][47] + coeff0_34*dmats1[34][47] + coeff0_35*dmats1[35][47] + coeff0_36*dmats1[36][47] + coeff0_37*dmats1[37][47] + coeff0_38*dmats1[38][47] + coeff0_39*dmats1[39][47] + coeff0_40*dmats1[40][47] + coeff0_41*dmats1[41][47] + coeff0_42*dmats1[42][47] + coeff0_43*dmats1[43][47] + coeff0_44*dmats1[44][47] + coeff0_45*dmats1[45][47] + coeff0_46*dmats1[46][47] + coeff0_47*dmats1[47][47] + coeff0_48*dmats1[48][47] + coeff0_49*dmats1[49][47] + coeff0_50*dmats1[50][47] + coeff0_51*dmats1[51][47] + coeff0_52*dmats1[52][47] + coeff0_53*dmats1[53][47] + coeff0_54*dmats1[54][47] + coeff0_55*dmats1[55][47];
          new_coeff0_48 = coeff0_0*dmats1[0][48] + coeff0_1*dmats1[1][48] + coeff0_2*dmats1[2][48] + coeff0_3*dmats1[3][48] + coeff0_4*dmats1[4][48] + coeff0_5*dmats1[5][48] + coeff0_6*dmats1[6][48] + coeff0_7*dmats1[7][48] + coeff0_8*dmats1[8][48] + coeff0_9*dmats1[9][48] + coeff0_10*dmats1[10][48] + coeff0_11*dmats1[11][48] + coeff0_12*dmats1[12][48] + coeff0_13*dmats1[13][48] + coeff0_14*dmats1[14][48] + coeff0_15*dmats1[15][48] + coeff0_16*dmats1[16][48] + coeff0_17*dmats1[17][48] + coeff0_18*dmats1[18][48] + coeff0_19*dmats1[19][48] + coeff0_20*dmats1[20][48] + coeff0_21*dmats1[21][48] + coeff0_22*dmats1[22][48] + coeff0_23*dmats1[23][48] + coeff0_24*dmats1[24][48] + coeff0_25*dmats1[25][48] + coeff0_26*dmats1[26][48] + coeff0_27*dmats1[27][48] + coeff0_28*dmats1[28][48] + coeff0_29*dmats1[29][48] + coeff0_30*dmats1[30][48] + coeff0_31*dmats1[31][48] + coeff0_32*dmats1[32][48] + coeff0_33*dmats1[33][48] + coeff0_34*dmats1[34][48] + coeff0_35*dmats1[35][48] + coeff0_36*dmats1[36][48] + coeff0_37*dmats1[37][48] + coeff0_38*dmats1[38][48] + coeff0_39*dmats1[39][48] + coeff0_40*dmats1[40][48] + coeff0_41*dmats1[41][48] + coeff0_42*dmats1[42][48] + coeff0_43*dmats1[43][48] + coeff0_44*dmats1[44][48] + coeff0_45*dmats1[45][48] + coeff0_46*dmats1[46][48] + coeff0_47*dmats1[47][48] + coeff0_48*dmats1[48][48] + coeff0_49*dmats1[49][48] + coeff0_50*dmats1[50][48] + coeff0_51*dmats1[51][48] + coeff0_52*dmats1[52][48] + coeff0_53*dmats1[53][48] + coeff0_54*dmats1[54][48] + coeff0_55*dmats1[55][48];
          new_coeff0_49 = coeff0_0*dmats1[0][49] + coeff0_1*dmats1[1][49] + coeff0_2*dmats1[2][49] + coeff0_3*dmats1[3][49] + coeff0_4*dmats1[4][49] + coeff0_5*dmats1[5][49] + coeff0_6*dmats1[6][49] + coeff0_7*dmats1[7][49] + coeff0_8*dmats1[8][49] + coeff0_9*dmats1[9][49] + coeff0_10*dmats1[10][49] + coeff0_11*dmats1[11][49] + coeff0_12*dmats1[12][49] + coeff0_13*dmats1[13][49] + coeff0_14*dmats1[14][49] + coeff0_15*dmats1[15][49] + coeff0_16*dmats1[16][49] + coeff0_17*dmats1[17][49] + coeff0_18*dmats1[18][49] + coeff0_19*dmats1[19][49] + coeff0_20*dmats1[20][49] + coeff0_21*dmats1[21][49] + coeff0_22*dmats1[22][49] + coeff0_23*dmats1[23][49] + coeff0_24*dmats1[24][49] + coeff0_25*dmats1[25][49] + coeff0_26*dmats1[26][49] + coeff0_27*dmats1[27][49] + coeff0_28*dmats1[28][49] + coeff0_29*dmats1[29][49] + coeff0_30*dmats1[30][49] + coeff0_31*dmats1[31][49] + coeff0_32*dmats1[32][49] + coeff0_33*dmats1[33][49] + coeff0_34*dmats1[34][49] + coeff0_35*dmats1[35][49] + coeff0_36*dmats1[36][49] + coeff0_37*dmats1[37][49] + coeff0_38*dmats1[38][49] + coeff0_39*dmats1[39][49] + coeff0_40*dmats1[40][49] + coeff0_41*dmats1[41][49] + coeff0_42*dmats1[42][49] + coeff0_43*dmats1[43][49] + coeff0_44*dmats1[44][49] + coeff0_45*dmats1[45][49] + coeff0_46*dmats1[46][49] + coeff0_47*dmats1[47][49] + coeff0_48*dmats1[48][49] + coeff0_49*dmats1[49][49] + coeff0_50*dmats1[50][49] + coeff0_51*dmats1[51][49] + coeff0_52*dmats1[52][49] + coeff0_53*dmats1[53][49] + coeff0_54*dmats1[54][49] + coeff0_55*dmats1[55][49];
          new_coeff0_50 = coeff0_0*dmats1[0][50] + coeff0_1*dmats1[1][50] + coeff0_2*dmats1[2][50] + coeff0_3*dmats1[3][50] + coeff0_4*dmats1[4][50] + coeff0_5*dmats1[5][50] + coeff0_6*dmats1[6][50] + coeff0_7*dmats1[7][50] + coeff0_8*dmats1[8][50] + coeff0_9*dmats1[9][50] + coeff0_10*dmats1[10][50] + coeff0_11*dmats1[11][50] + coeff0_12*dmats1[12][50] + coeff0_13*dmats1[13][50] + coeff0_14*dmats1[14][50] + coeff0_15*dmats1[15][50] + coeff0_16*dmats1[16][50] + coeff0_17*dmats1[17][50] + coeff0_18*dmats1[18][50] + coeff0_19*dmats1[19][50] + coeff0_20*dmats1[20][50] + coeff0_21*dmats1[21][50] + coeff0_22*dmats1[22][50] + coeff0_23*dmats1[23][50] + coeff0_24*dmats1[24][50] + coeff0_25*dmats1[25][50] + coeff0_26*dmats1[26][50] + coeff0_27*dmats1[27][50] + coeff0_28*dmats1[28][50] + coeff0_29*dmats1[29][50] + coeff0_30*dmats1[30][50] + coeff0_31*dmats1[31][50] + coeff0_32*dmats1[32][50] + coeff0_33*dmats1[33][50] + coeff0_34*dmats1[34][50] + coeff0_35*dmats1[35][50] + coeff0_36*dmats1[36][50] + coeff0_37*dmats1[37][50] + coeff0_38*dmats1[38][50] + coeff0_39*dmats1[39][50] + coeff0_40*dmats1[40][50] + coeff0_41*dmats1[41][50] + coeff0_42*dmats1[42][50] + coeff0_43*dmats1[43][50] + coeff0_44*dmats1[44][50] + coeff0_45*dmats1[45][50] + coeff0_46*dmats1[46][50] + coeff0_47*dmats1[47][50] + coeff0_48*dmats1[48][50] + coeff0_49*dmats1[49][50] + coeff0_50*dmats1[50][50] + coeff0_51*dmats1[51][50] + coeff0_52*dmats1[52][50] + coeff0_53*dmats1[53][50] + coeff0_54*dmats1[54][50] + coeff0_55*dmats1[55][50];
          new_coeff0_51 = coeff0_0*dmats1[0][51] + coeff0_1*dmats1[1][51] + coeff0_2*dmats1[2][51] + coeff0_3*dmats1[3][51] + coeff0_4*dmats1[4][51] + coeff0_5*dmats1[5][51] + coeff0_6*dmats1[6][51] + coeff0_7*dmats1[7][51] + coeff0_8*dmats1[8][51] + coeff0_9*dmats1[9][51] + coeff0_10*dmats1[10][51] + coeff0_11*dmats1[11][51] + coeff0_12*dmats1[12][51] + coeff0_13*dmats1[13][51] + coeff0_14*dmats1[14][51] + coeff0_15*dmats1[15][51] + coeff0_16*dmats1[16][51] + coeff0_17*dmats1[17][51] + coeff0_18*dmats1[18][51] + coeff0_19*dmats1[19][51] + coeff0_20*dmats1[20][51] + coeff0_21*dmats1[21][51] + coeff0_22*dmats1[22][51] + coeff0_23*dmats1[23][51] + coeff0_24*dmats1[24][51] + coeff0_25*dmats1[25][51] + coeff0_26*dmats1[26][51] + coeff0_27*dmats1[27][51] + coeff0_28*dmats1[28][51] + coeff0_29*dmats1[29][51] + coeff0_30*dmats1[30][51] + coeff0_31*dmats1[31][51] + coeff0_32*dmats1[32][51] + coeff0_33*dmats1[33][51] + coeff0_34*dmats1[34][51] + coeff0_35*dmats1[35][51] + coeff0_36*dmats1[36][51] + coeff0_37*dmats1[37][51] + coeff0_38*dmats1[38][51] + coeff0_39*dmats1[39][51] + coeff0_40*dmats1[40][51] + coeff0_41*dmats1[41][51] + coeff0_42*dmats1[42][51] + coeff0_43*dmats1[43][51] + coeff0_44*dmats1[44][51] + coeff0_45*dmats1[45][51] + coeff0_46*dmats1[46][51] + coeff0_47*dmats1[47][51] + coeff0_48*dmats1[48][51] + coeff0_49*dmats1[49][51] + coeff0_50*dmats1[50][51] + coeff0_51*dmats1[51][51] + coeff0_52*dmats1[52][51] + coeff0_53*dmats1[53][51] + coeff0_54*dmats1[54][51] + coeff0_55*dmats1[55][51];
          new_coeff0_52 = coeff0_0*dmats1[0][52] + coeff0_1*dmats1[1][52] + coeff0_2*dmats1[2][52] + coeff0_3*dmats1[3][52] + coeff0_4*dmats1[4][52] + coeff0_5*dmats1[5][52] + coeff0_6*dmats1[6][52] + coeff0_7*dmats1[7][52] + coeff0_8*dmats1[8][52] + coeff0_9*dmats1[9][52] + coeff0_10*dmats1[10][52] + coeff0_11*dmats1[11][52] + coeff0_12*dmats1[12][52] + coeff0_13*dmats1[13][52] + coeff0_14*dmats1[14][52] + coeff0_15*dmats1[15][52] + coeff0_16*dmats1[16][52] + coeff0_17*dmats1[17][52] + coeff0_18*dmats1[18][52] + coeff0_19*dmats1[19][52] + coeff0_20*dmats1[20][52] + coeff0_21*dmats1[21][52] + coeff0_22*dmats1[22][52] + coeff0_23*dmats1[23][52] + coeff0_24*dmats1[24][52] + coeff0_25*dmats1[25][52] + coeff0_26*dmats1[26][52] + coeff0_27*dmats1[27][52] + coeff0_28*dmats1[28][52] + coeff0_29*dmats1[29][52] + coeff0_30*dmats1[30][52] + coeff0_31*dmats1[31][52] + coeff0_32*dmats1[32][52] + coeff0_33*dmats1[33][52] + coeff0_34*dmats1[34][52] + coeff0_35*dmats1[35][52] + coeff0_36*dmats1[36][52] + coeff0_37*dmats1[37][52] + coeff0_38*dmats1[38][52] + coeff0_39*dmats1[39][52] + coeff0_40*dmats1[40][52] + coeff0_41*dmats1[41][52] + coeff0_42*dmats1[42][52] + coeff0_43*dmats1[43][52] + coeff0_44*dmats1[44][52] + coeff0_45*dmats1[45][52] + coeff0_46*dmats1[46][52] + coeff0_47*dmats1[47][52] + coeff0_48*dmats1[48][52] + coeff0_49*dmats1[49][52] + coeff0_50*dmats1[50][52] + coeff0_51*dmats1[51][52] + coeff0_52*dmats1[52][52] + coeff0_53*dmats1[53][52] + coeff0_54*dmats1[54][52] + coeff0_55*dmats1[55][52];
          new_coeff0_53 = coeff0_0*dmats1[0][53] + coeff0_1*dmats1[1][53] + coeff0_2*dmats1[2][53] + coeff0_3*dmats1[3][53] + coeff0_4*dmats1[4][53] + coeff0_5*dmats1[5][53] + coeff0_6*dmats1[6][53] + coeff0_7*dmats1[7][53] + coeff0_8*dmats1[8][53] + coeff0_9*dmats1[9][53] + coeff0_10*dmats1[10][53] + coeff0_11*dmats1[11][53] + coeff0_12*dmats1[12][53] + coeff0_13*dmats1[13][53] + coeff0_14*dmats1[14][53] + coeff0_15*dmats1[15][53] + coeff0_16*dmats1[16][53] + coeff0_17*dmats1[17][53] + coeff0_18*dmats1[18][53] + coeff0_19*dmats1[19][53] + coeff0_20*dmats1[20][53] + coeff0_21*dmats1[21][53] + coeff0_22*dmats1[22][53] + coeff0_23*dmats1[23][53] + coeff0_24*dmats1[24][53] + coeff0_25*dmats1[25][53] + coeff0_26*dmats1[26][53] + coeff0_27*dmats1[27][53] + coeff0_28*dmats1[28][53] + coeff0_29*dmats1[29][53] + coeff0_30*dmats1[30][53] + coeff0_31*dmats1[31][53] + coeff0_32*dmats1[32][53] + coeff0_33*dmats1[33][53] + coeff0_34*dmats1[34][53] + coeff0_35*dmats1[35][53] + coeff0_36*dmats1[36][53] + coeff0_37*dmats1[37][53] + coeff0_38*dmats1[38][53] + coeff0_39*dmats1[39][53] + coeff0_40*dmats1[40][53] + coeff0_41*dmats1[41][53] + coeff0_42*dmats1[42][53] + coeff0_43*dmats1[43][53] + coeff0_44*dmats1[44][53] + coeff0_45*dmats1[45][53] + coeff0_46*dmats1[46][53] + coeff0_47*dmats1[47][53] + coeff0_48*dmats1[48][53] + coeff0_49*dmats1[49][53] + coeff0_50*dmats1[50][53] + coeff0_51*dmats1[51][53] + coeff0_52*dmats1[52][53] + coeff0_53*dmats1[53][53] + coeff0_54*dmats1[54][53] + coeff0_55*dmats1[55][53];
          new_coeff0_54 = coeff0_0*dmats1[0][54] + coeff0_1*dmats1[1][54] + coeff0_2*dmats1[2][54] + coeff0_3*dmats1[3][54] + coeff0_4*dmats1[4][54] + coeff0_5*dmats1[5][54] + coeff0_6*dmats1[6][54] + coeff0_7*dmats1[7][54] + coeff0_8*dmats1[8][54] + coeff0_9*dmats1[9][54] + coeff0_10*dmats1[10][54] + coeff0_11*dmats1[11][54] + coeff0_12*dmats1[12][54] + coeff0_13*dmats1[13][54] + coeff0_14*dmats1[14][54] + coeff0_15*dmats1[15][54] + coeff0_16*dmats1[16][54] + coeff0_17*dmats1[17][54] + coeff0_18*dmats1[18][54] + coeff0_19*dmats1[19][54] + coeff0_20*dmats1[20][54] + coeff0_21*dmats1[21][54] + coeff0_22*dmats1[22][54] + coeff0_23*dmats1[23][54] + coeff0_24*dmats1[24][54] + coeff0_25*dmats1[25][54] + coeff0_26*dmats1[26][54] + coeff0_27*dmats1[27][54] + coeff0_28*dmats1[28][54] + coeff0_29*dmats1[29][54] + coeff0_30*dmats1[30][54] + coeff0_31*dmats1[31][54] + coeff0_32*dmats1[32][54] + coeff0_33*dmats1[33][54] + coeff0_34*dmats1[34][54] + coeff0_35*dmats1[35][54] + coeff0_36*dmats1[36][54] + coeff0_37*dmats1[37][54] + coeff0_38*dmats1[38][54] + coeff0_39*dmats1[39][54] + coeff0_40*dmats1[40][54] + coeff0_41*dmats1[41][54] + coeff0_42*dmats1[42][54] + coeff0_43*dmats1[43][54] + coeff0_44*dmats1[44][54] + coeff0_45*dmats1[45][54] + coeff0_46*dmats1[46][54] + coeff0_47*dmats1[47][54] + coeff0_48*dmats1[48][54] + coeff0_49*dmats1[49][54] + coeff0_50*dmats1[50][54] + coeff0_51*dmats1[51][54] + coeff0_52*dmats1[52][54] + coeff0_53*dmats1[53][54] + coeff0_54*dmats1[54][54] + coeff0_55*dmats1[55][54];
          new_coeff0_55 = coeff0_0*dmats1[0][55] + coeff0_1*dmats1[1][55] + coeff0_2*dmats1[2][55] + coeff0_3*dmats1[3][55] + coeff0_4*dmats1[4][55] + coeff0_5*dmats1[5][55] + coeff0_6*dmats1[6][55] + coeff0_7*dmats1[7][55] + coeff0_8*dmats1[8][55] + coeff0_9*dmats1[9][55] + coeff0_10*dmats1[10][55] + coeff0_11*dmats1[11][55] + coeff0_12*dmats1[12][55] + coeff0_13*dmats1[13][55] + coeff0_14*dmats1[14][55] + coeff0_15*dmats1[15][55] + coeff0_16*dmats1[16][55] + coeff0_17*dmats1[17][55] + coeff0_18*dmats1[18][55] + coeff0_19*dmats1[19][55] + coeff0_20*dmats1[20][55] + coeff0_21*dmats1[21][55] + coeff0_22*dmats1[22][55] + coeff0_23*dmats1[23][55] + coeff0_24*dmats1[24][55] + coeff0_25*dmats1[25][55] + coeff0_26*dmats1[26][55] + coeff0_27*dmats1[27][55] + coeff0_28*dmats1[28][55] + coeff0_29*dmats1[29][55] + coeff0_30*dmats1[30][55] + coeff0_31*dmats1[31][55] + coeff0_32*dmats1[32][55] + coeff0_33*dmats1[33][55] + coeff0_34*dmats1[34][55] + coeff0_35*dmats1[35][55] + coeff0_36*dmats1[36][55] + coeff0_37*dmats1[37][55] + coeff0_38*dmats1[38][55] + coeff0_39*dmats1[39][55] + coeff0_40*dmats1[40][55] + coeff0_41*dmats1[41][55] + coeff0_42*dmats1[42][55] + coeff0_43*dmats1[43][55] + coeff0_44*dmats1[44][55] + coeff0_45*dmats1[45][55] + coeff0_46*dmats1[46][55] + coeff0_47*dmats1[47][55] + coeff0_48*dmats1[48][55] + coeff0_49*dmats1[49][55] + coeff0_50*dmats1[50][55] + coeff0_51*dmats1[51][55] + coeff0_52*dmats1[52][55] + coeff0_53*dmats1[53][55] + coeff0_54*dmats1[54][55] + coeff0_55*dmats1[55][55];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0] + coeff0_4*dmats2[4][0] + coeff0_5*dmats2[5][0] + coeff0_6*dmats2[6][0] + coeff0_7*dmats2[7][0] + coeff0_8*dmats2[8][0] + coeff0_9*dmats2[9][0] + coeff0_10*dmats2[10][0] + coeff0_11*dmats2[11][0] + coeff0_12*dmats2[12][0] + coeff0_13*dmats2[13][0] + coeff0_14*dmats2[14][0] + coeff0_15*dmats2[15][0] + coeff0_16*dmats2[16][0] + coeff0_17*dmats2[17][0] + coeff0_18*dmats2[18][0] + coeff0_19*dmats2[19][0] + coeff0_20*dmats2[20][0] + coeff0_21*dmats2[21][0] + coeff0_22*dmats2[22][0] + coeff0_23*dmats2[23][0] + coeff0_24*dmats2[24][0] + coeff0_25*dmats2[25][0] + coeff0_26*dmats2[26][0] + coeff0_27*dmats2[27][0] + coeff0_28*dmats2[28][0] + coeff0_29*dmats2[29][0] + coeff0_30*dmats2[30][0] + coeff0_31*dmats2[31][0] + coeff0_32*dmats2[32][0] + coeff0_33*dmats2[33][0] + coeff0_34*dmats2[34][0] + coeff0_35*dmats2[35][0] + coeff0_36*dmats2[36][0] + coeff0_37*dmats2[37][0] + coeff0_38*dmats2[38][0] + coeff0_39*dmats2[39][0] + coeff0_40*dmats2[40][0] + coeff0_41*dmats2[41][0] + coeff0_42*dmats2[42][0] + coeff0_43*dmats2[43][0] + coeff0_44*dmats2[44][0] + coeff0_45*dmats2[45][0] + coeff0_46*dmats2[46][0] + coeff0_47*dmats2[47][0] + coeff0_48*dmats2[48][0] + coeff0_49*dmats2[49][0] + coeff0_50*dmats2[50][0] + coeff0_51*dmats2[51][0] + coeff0_52*dmats2[52][0] + coeff0_53*dmats2[53][0] + coeff0_54*dmats2[54][0] + coeff0_55*dmats2[55][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1] + coeff0_4*dmats2[4][1] + coeff0_5*dmats2[5][1] + coeff0_6*dmats2[6][1] + coeff0_7*dmats2[7][1] + coeff0_8*dmats2[8][1] + coeff0_9*dmats2[9][1] + coeff0_10*dmats2[10][1] + coeff0_11*dmats2[11][1] + coeff0_12*dmats2[12][1] + coeff0_13*dmats2[13][1] + coeff0_14*dmats2[14][1] + coeff0_15*dmats2[15][1] + coeff0_16*dmats2[16][1] + coeff0_17*dmats2[17][1] + coeff0_18*dmats2[18][1] + coeff0_19*dmats2[19][1] + coeff0_20*dmats2[20][1] + coeff0_21*dmats2[21][1] + coeff0_22*dmats2[22][1] + coeff0_23*dmats2[23][1] + coeff0_24*dmats2[24][1] + coeff0_25*dmats2[25][1] + coeff0_26*dmats2[26][1] + coeff0_27*dmats2[27][1] + coeff0_28*dmats2[28][1] + coeff0_29*dmats2[29][1] + coeff0_30*dmats2[30][1] + coeff0_31*dmats2[31][1] + coeff0_32*dmats2[32][1] + coeff0_33*dmats2[33][1] + coeff0_34*dmats2[34][1] + coeff0_35*dmats2[35][1] + coeff0_36*dmats2[36][1] + coeff0_37*dmats2[37][1] + coeff0_38*dmats2[38][1] + coeff0_39*dmats2[39][1] + coeff0_40*dmats2[40][1] + coeff0_41*dmats2[41][1] + coeff0_42*dmats2[42][1] + coeff0_43*dmats2[43][1] + coeff0_44*dmats2[44][1] + coeff0_45*dmats2[45][1] + coeff0_46*dmats2[46][1] + coeff0_47*dmats2[47][1] + coeff0_48*dmats2[48][1] + coeff0_49*dmats2[49][1] + coeff0_50*dmats2[50][1] + coeff0_51*dmats2[51][1] + coeff0_52*dmats2[52][1] + coeff0_53*dmats2[53][1] + coeff0_54*dmats2[54][1] + coeff0_55*dmats2[55][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2] + coeff0_4*dmats2[4][2] + coeff0_5*dmats2[5][2] + coeff0_6*dmats2[6][2] + coeff0_7*dmats2[7][2] + coeff0_8*dmats2[8][2] + coeff0_9*dmats2[9][2] + coeff0_10*dmats2[10][2] + coeff0_11*dmats2[11][2] + coeff0_12*dmats2[12][2] + coeff0_13*dmats2[13][2] + coeff0_14*dmats2[14][2] + coeff0_15*dmats2[15][2] + coeff0_16*dmats2[16][2] + coeff0_17*dmats2[17][2] + coeff0_18*dmats2[18][2] + coeff0_19*dmats2[19][2] + coeff0_20*dmats2[20][2] + coeff0_21*dmats2[21][2] + coeff0_22*dmats2[22][2] + coeff0_23*dmats2[23][2] + coeff0_24*dmats2[24][2] + coeff0_25*dmats2[25][2] + coeff0_26*dmats2[26][2] + coeff0_27*dmats2[27][2] + coeff0_28*dmats2[28][2] + coeff0_29*dmats2[29][2] + coeff0_30*dmats2[30][2] + coeff0_31*dmats2[31][2] + coeff0_32*dmats2[32][2] + coeff0_33*dmats2[33][2] + coeff0_34*dmats2[34][2] + coeff0_35*dmats2[35][2] + coeff0_36*dmats2[36][2] + coeff0_37*dmats2[37][2] + coeff0_38*dmats2[38][2] + coeff0_39*dmats2[39][2] + coeff0_40*dmats2[40][2] + coeff0_41*dmats2[41][2] + coeff0_42*dmats2[42][2] + coeff0_43*dmats2[43][2] + coeff0_44*dmats2[44][2] + coeff0_45*dmats2[45][2] + coeff0_46*dmats2[46][2] + coeff0_47*dmats2[47][2] + coeff0_48*dmats2[48][2] + coeff0_49*dmats2[49][2] + coeff0_50*dmats2[50][2] + coeff0_51*dmats2[51][2] + coeff0_52*dmats2[52][2] + coeff0_53*dmats2[53][2] + coeff0_54*dmats2[54][2] + coeff0_55*dmats2[55][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3] + coeff0_4*dmats2[4][3] + coeff0_5*dmats2[5][3] + coeff0_6*dmats2[6][3] + coeff0_7*dmats2[7][3] + coeff0_8*dmats2[8][3] + coeff0_9*dmats2[9][3] + coeff0_10*dmats2[10][3] + coeff0_11*dmats2[11][3] + coeff0_12*dmats2[12][3] + coeff0_13*dmats2[13][3] + coeff0_14*dmats2[14][3] + coeff0_15*dmats2[15][3] + coeff0_16*dmats2[16][3] + coeff0_17*dmats2[17][3] + coeff0_18*dmats2[18][3] + coeff0_19*dmats2[19][3] + coeff0_20*dmats2[20][3] + coeff0_21*dmats2[21][3] + coeff0_22*dmats2[22][3] + coeff0_23*dmats2[23][3] + coeff0_24*dmats2[24][3] + coeff0_25*dmats2[25][3] + coeff0_26*dmats2[26][3] + coeff0_27*dmats2[27][3] + coeff0_28*dmats2[28][3] + coeff0_29*dmats2[29][3] + coeff0_30*dmats2[30][3] + coeff0_31*dmats2[31][3] + coeff0_32*dmats2[32][3] + coeff0_33*dmats2[33][3] + coeff0_34*dmats2[34][3] + coeff0_35*dmats2[35][3] + coeff0_36*dmats2[36][3] + coeff0_37*dmats2[37][3] + coeff0_38*dmats2[38][3] + coeff0_39*dmats2[39][3] + coeff0_40*dmats2[40][3] + coeff0_41*dmats2[41][3] + coeff0_42*dmats2[42][3] + coeff0_43*dmats2[43][3] + coeff0_44*dmats2[44][3] + coeff0_45*dmats2[45][3] + coeff0_46*dmats2[46][3] + coeff0_47*dmats2[47][3] + coeff0_48*dmats2[48][3] + coeff0_49*dmats2[49][3] + coeff0_50*dmats2[50][3] + coeff0_51*dmats2[51][3] + coeff0_52*dmats2[52][3] + coeff0_53*dmats2[53][3] + coeff0_54*dmats2[54][3] + coeff0_55*dmats2[55][3];
          new_coeff0_4 = coeff0_0*dmats2[0][4] + coeff0_1*dmats2[1][4] + coeff0_2*dmats2[2][4] + coeff0_3*dmats2[3][4] + coeff0_4*dmats2[4][4] + coeff0_5*dmats2[5][4] + coeff0_6*dmats2[6][4] + coeff0_7*dmats2[7][4] + coeff0_8*dmats2[8][4] + coeff0_9*dmats2[9][4] + coeff0_10*dmats2[10][4] + coeff0_11*dmats2[11][4] + coeff0_12*dmats2[12][4] + coeff0_13*dmats2[13][4] + coeff0_14*dmats2[14][4] + coeff0_15*dmats2[15][4] + coeff0_16*dmats2[16][4] + coeff0_17*dmats2[17][4] + coeff0_18*dmats2[18][4] + coeff0_19*dmats2[19][4] + coeff0_20*dmats2[20][4] + coeff0_21*dmats2[21][4] + coeff0_22*dmats2[22][4] + coeff0_23*dmats2[23][4] + coeff0_24*dmats2[24][4] + coeff0_25*dmats2[25][4] + coeff0_26*dmats2[26][4] + coeff0_27*dmats2[27][4] + coeff0_28*dmats2[28][4] + coeff0_29*dmats2[29][4] + coeff0_30*dmats2[30][4] + coeff0_31*dmats2[31][4] + coeff0_32*dmats2[32][4] + coeff0_33*dmats2[33][4] + coeff0_34*dmats2[34][4] + coeff0_35*dmats2[35][4] + coeff0_36*dmats2[36][4] + coeff0_37*dmats2[37][4] + coeff0_38*dmats2[38][4] + coeff0_39*dmats2[39][4] + coeff0_40*dmats2[40][4] + coeff0_41*dmats2[41][4] + coeff0_42*dmats2[42][4] + coeff0_43*dmats2[43][4] + coeff0_44*dmats2[44][4] + coeff0_45*dmats2[45][4] + coeff0_46*dmats2[46][4] + coeff0_47*dmats2[47][4] + coeff0_48*dmats2[48][4] + coeff0_49*dmats2[49][4] + coeff0_50*dmats2[50][4] + coeff0_51*dmats2[51][4] + coeff0_52*dmats2[52][4] + coeff0_53*dmats2[53][4] + coeff0_54*dmats2[54][4] + coeff0_55*dmats2[55][4];
          new_coeff0_5 = coeff0_0*dmats2[0][5] + coeff0_1*dmats2[1][5] + coeff0_2*dmats2[2][5] + coeff0_3*dmats2[3][5] + coeff0_4*dmats2[4][5] + coeff0_5*dmats2[5][5] + coeff0_6*dmats2[6][5] + coeff0_7*dmats2[7][5] + coeff0_8*dmats2[8][5] + coeff0_9*dmats2[9][5] + coeff0_10*dmats2[10][5] + coeff0_11*dmats2[11][5] + coeff0_12*dmats2[12][5] + coeff0_13*dmats2[13][5] + coeff0_14*dmats2[14][5] + coeff0_15*dmats2[15][5] + coeff0_16*dmats2[16][5] + coeff0_17*dmats2[17][5] + coeff0_18*dmats2[18][5] + coeff0_19*dmats2[19][5] + coeff0_20*dmats2[20][5] + coeff0_21*dmats2[21][5] + coeff0_22*dmats2[22][5] + coeff0_23*dmats2[23][5] + coeff0_24*dmats2[24][5] + coeff0_25*dmats2[25][5] + coeff0_26*dmats2[26][5] + coeff0_27*dmats2[27][5] + coeff0_28*dmats2[28][5] + coeff0_29*dmats2[29][5] + coeff0_30*dmats2[30][5] + coeff0_31*dmats2[31][5] + coeff0_32*dmats2[32][5] + coeff0_33*dmats2[33][5] + coeff0_34*dmats2[34][5] + coeff0_35*dmats2[35][5] + coeff0_36*dmats2[36][5] + coeff0_37*dmats2[37][5] + coeff0_38*dmats2[38][5] + coeff0_39*dmats2[39][5] + coeff0_40*dmats2[40][5] + coeff0_41*dmats2[41][5] + coeff0_42*dmats2[42][5] + coeff0_43*dmats2[43][5] + coeff0_44*dmats2[44][5] + coeff0_45*dmats2[45][5] + coeff0_46*dmats2[46][5] + coeff0_47*dmats2[47][5] + coeff0_48*dmats2[48][5] + coeff0_49*dmats2[49][5] + coeff0_50*dmats2[50][5] + coeff0_51*dmats2[51][5] + coeff0_52*dmats2[52][5] + coeff0_53*dmats2[53][5] + coeff0_54*dmats2[54][5] + coeff0_55*dmats2[55][5];
          new_coeff0_6 = coeff0_0*dmats2[0][6] + coeff0_1*dmats2[1][6] + coeff0_2*dmats2[2][6] + coeff0_3*dmats2[3][6] + coeff0_4*dmats2[4][6] + coeff0_5*dmats2[5][6] + coeff0_6*dmats2[6][6] + coeff0_7*dmats2[7][6] + coeff0_8*dmats2[8][6] + coeff0_9*dmats2[9][6] + coeff0_10*dmats2[10][6] + coeff0_11*dmats2[11][6] + coeff0_12*dmats2[12][6] + coeff0_13*dmats2[13][6] + coeff0_14*dmats2[14][6] + coeff0_15*dmats2[15][6] + coeff0_16*dmats2[16][6] + coeff0_17*dmats2[17][6] + coeff0_18*dmats2[18][6] + coeff0_19*dmats2[19][6] + coeff0_20*dmats2[20][6] + coeff0_21*dmats2[21][6] + coeff0_22*dmats2[22][6] + coeff0_23*dmats2[23][6] + coeff0_24*dmats2[24][6] + coeff0_25*dmats2[25][6] + coeff0_26*dmats2[26][6] + coeff0_27*dmats2[27][6] + coeff0_28*dmats2[28][6] + coeff0_29*dmats2[29][6] + coeff0_30*dmats2[30][6] + coeff0_31*dmats2[31][6] + coeff0_32*dmats2[32][6] + coeff0_33*dmats2[33][6] + coeff0_34*dmats2[34][6] + coeff0_35*dmats2[35][6] + coeff0_36*dmats2[36][6] + coeff0_37*dmats2[37][6] + coeff0_38*dmats2[38][6] + coeff0_39*dmats2[39][6] + coeff0_40*dmats2[40][6] + coeff0_41*dmats2[41][6] + coeff0_42*dmats2[42][6] + coeff0_43*dmats2[43][6] + coeff0_44*dmats2[44][6] + coeff0_45*dmats2[45][6] + coeff0_46*dmats2[46][6] + coeff0_47*dmats2[47][6] + coeff0_48*dmats2[48][6] + coeff0_49*dmats2[49][6] + coeff0_50*dmats2[50][6] + coeff0_51*dmats2[51][6] + coeff0_52*dmats2[52][6] + coeff0_53*dmats2[53][6] + coeff0_54*dmats2[54][6] + coeff0_55*dmats2[55][6];
          new_coeff0_7 = coeff0_0*dmats2[0][7] + coeff0_1*dmats2[1][7] + coeff0_2*dmats2[2][7] + coeff0_3*dmats2[3][7] + coeff0_4*dmats2[4][7] + coeff0_5*dmats2[5][7] + coeff0_6*dmats2[6][7] + coeff0_7*dmats2[7][7] + coeff0_8*dmats2[8][7] + coeff0_9*dmats2[9][7] + coeff0_10*dmats2[10][7] + coeff0_11*dmats2[11][7] + coeff0_12*dmats2[12][7] + coeff0_13*dmats2[13][7] + coeff0_14*dmats2[14][7] + coeff0_15*dmats2[15][7] + coeff0_16*dmats2[16][7] + coeff0_17*dmats2[17][7] + coeff0_18*dmats2[18][7] + coeff0_19*dmats2[19][7] + coeff0_20*dmats2[20][7] + coeff0_21*dmats2[21][7] + coeff0_22*dmats2[22][7] + coeff0_23*dmats2[23][7] + coeff0_24*dmats2[24][7] + coeff0_25*dmats2[25][7] + coeff0_26*dmats2[26][7] + coeff0_27*dmats2[27][7] + coeff0_28*dmats2[28][7] + coeff0_29*dmats2[29][7] + coeff0_30*dmats2[30][7] + coeff0_31*dmats2[31][7] + coeff0_32*dmats2[32][7] + coeff0_33*dmats2[33][7] + coeff0_34*dmats2[34][7] + coeff0_35*dmats2[35][7] + coeff0_36*dmats2[36][7] + coeff0_37*dmats2[37][7] + coeff0_38*dmats2[38][7] + coeff0_39*dmats2[39][7] + coeff0_40*dmats2[40][7] + coeff0_41*dmats2[41][7] + coeff0_42*dmats2[42][7] + coeff0_43*dmats2[43][7] + coeff0_44*dmats2[44][7] + coeff0_45*dmats2[45][7] + coeff0_46*dmats2[46][7] + coeff0_47*dmats2[47][7] + coeff0_48*dmats2[48][7] + coeff0_49*dmats2[49][7] + coeff0_50*dmats2[50][7] + coeff0_51*dmats2[51][7] + coeff0_52*dmats2[52][7] + coeff0_53*dmats2[53][7] + coeff0_54*dmats2[54][7] + coeff0_55*dmats2[55][7];
          new_coeff0_8 = coeff0_0*dmats2[0][8] + coeff0_1*dmats2[1][8] + coeff0_2*dmats2[2][8] + coeff0_3*dmats2[3][8] + coeff0_4*dmats2[4][8] + coeff0_5*dmats2[5][8] + coeff0_6*dmats2[6][8] + coeff0_7*dmats2[7][8] + coeff0_8*dmats2[8][8] + coeff0_9*dmats2[9][8] + coeff0_10*dmats2[10][8] + coeff0_11*dmats2[11][8] + coeff0_12*dmats2[12][8] + coeff0_13*dmats2[13][8] + coeff0_14*dmats2[14][8] + coeff0_15*dmats2[15][8] + coeff0_16*dmats2[16][8] + coeff0_17*dmats2[17][8] + coeff0_18*dmats2[18][8] + coeff0_19*dmats2[19][8] + coeff0_20*dmats2[20][8] + coeff0_21*dmats2[21][8] + coeff0_22*dmats2[22][8] + coeff0_23*dmats2[23][8] + coeff0_24*dmats2[24][8] + coeff0_25*dmats2[25][8] + coeff0_26*dmats2[26][8] + coeff0_27*dmats2[27][8] + coeff0_28*dmats2[28][8] + coeff0_29*dmats2[29][8] + coeff0_30*dmats2[30][8] + coeff0_31*dmats2[31][8] + coeff0_32*dmats2[32][8] + coeff0_33*dmats2[33][8] + coeff0_34*dmats2[34][8] + coeff0_35*dmats2[35][8] + coeff0_36*dmats2[36][8] + coeff0_37*dmats2[37][8] + coeff0_38*dmats2[38][8] + coeff0_39*dmats2[39][8] + coeff0_40*dmats2[40][8] + coeff0_41*dmats2[41][8] + coeff0_42*dmats2[42][8] + coeff0_43*dmats2[43][8] + coeff0_44*dmats2[44][8] + coeff0_45*dmats2[45][8] + coeff0_46*dmats2[46][8] + coeff0_47*dmats2[47][8] + coeff0_48*dmats2[48][8] + coeff0_49*dmats2[49][8] + coeff0_50*dmats2[50][8] + coeff0_51*dmats2[51][8] + coeff0_52*dmats2[52][8] + coeff0_53*dmats2[53][8] + coeff0_54*dmats2[54][8] + coeff0_55*dmats2[55][8];
          new_coeff0_9 = coeff0_0*dmats2[0][9] + coeff0_1*dmats2[1][9] + coeff0_2*dmats2[2][9] + coeff0_3*dmats2[3][9] + coeff0_4*dmats2[4][9] + coeff0_5*dmats2[5][9] + coeff0_6*dmats2[6][9] + coeff0_7*dmats2[7][9] + coeff0_8*dmats2[8][9] + coeff0_9*dmats2[9][9] + coeff0_10*dmats2[10][9] + coeff0_11*dmats2[11][9] + coeff0_12*dmats2[12][9] + coeff0_13*dmats2[13][9] + coeff0_14*dmats2[14][9] + coeff0_15*dmats2[15][9] + coeff0_16*dmats2[16][9] + coeff0_17*dmats2[17][9] + coeff0_18*dmats2[18][9] + coeff0_19*dmats2[19][9] + coeff0_20*dmats2[20][9] + coeff0_21*dmats2[21][9] + coeff0_22*dmats2[22][9] + coeff0_23*dmats2[23][9] + coeff0_24*dmats2[24][9] + coeff0_25*dmats2[25][9] + coeff0_26*dmats2[26][9] + coeff0_27*dmats2[27][9] + coeff0_28*dmats2[28][9] + coeff0_29*dmats2[29][9] + coeff0_30*dmats2[30][9] + coeff0_31*dmats2[31][9] + coeff0_32*dmats2[32][9] + coeff0_33*dmats2[33][9] + coeff0_34*dmats2[34][9] + coeff0_35*dmats2[35][9] + coeff0_36*dmats2[36][9] + coeff0_37*dmats2[37][9] + coeff0_38*dmats2[38][9] + coeff0_39*dmats2[39][9] + coeff0_40*dmats2[40][9] + coeff0_41*dmats2[41][9] + coeff0_42*dmats2[42][9] + coeff0_43*dmats2[43][9] + coeff0_44*dmats2[44][9] + coeff0_45*dmats2[45][9] + coeff0_46*dmats2[46][9] + coeff0_47*dmats2[47][9] + coeff0_48*dmats2[48][9] + coeff0_49*dmats2[49][9] + coeff0_50*dmats2[50][9] + coeff0_51*dmats2[51][9] + coeff0_52*dmats2[52][9] + coeff0_53*dmats2[53][9] + coeff0_54*dmats2[54][9] + coeff0_55*dmats2[55][9];
          new_coeff0_10 = coeff0_0*dmats2[0][10] + coeff0_1*dmats2[1][10] + coeff0_2*dmats2[2][10] + coeff0_3*dmats2[3][10] + coeff0_4*dmats2[4][10] + coeff0_5*dmats2[5][10] + coeff0_6*dmats2[6][10] + coeff0_7*dmats2[7][10] + coeff0_8*dmats2[8][10] + coeff0_9*dmats2[9][10] + coeff0_10*dmats2[10][10] + coeff0_11*dmats2[11][10] + coeff0_12*dmats2[12][10] + coeff0_13*dmats2[13][10] + coeff0_14*dmats2[14][10] + coeff0_15*dmats2[15][10] + coeff0_16*dmats2[16][10] + coeff0_17*dmats2[17][10] + coeff0_18*dmats2[18][10] + coeff0_19*dmats2[19][10] + coeff0_20*dmats2[20][10] + coeff0_21*dmats2[21][10] + coeff0_22*dmats2[22][10] + coeff0_23*dmats2[23][10] + coeff0_24*dmats2[24][10] + coeff0_25*dmats2[25][10] + coeff0_26*dmats2[26][10] + coeff0_27*dmats2[27][10] + coeff0_28*dmats2[28][10] + coeff0_29*dmats2[29][10] + coeff0_30*dmats2[30][10] + coeff0_31*dmats2[31][10] + coeff0_32*dmats2[32][10] + coeff0_33*dmats2[33][10] + coeff0_34*dmats2[34][10] + coeff0_35*dmats2[35][10] + coeff0_36*dmats2[36][10] + coeff0_37*dmats2[37][10] + coeff0_38*dmats2[38][10] + coeff0_39*dmats2[39][10] + coeff0_40*dmats2[40][10] + coeff0_41*dmats2[41][10] + coeff0_42*dmats2[42][10] + coeff0_43*dmats2[43][10] + coeff0_44*dmats2[44][10] + coeff0_45*dmats2[45][10] + coeff0_46*dmats2[46][10] + coeff0_47*dmats2[47][10] + coeff0_48*dmats2[48][10] + coeff0_49*dmats2[49][10] + coeff0_50*dmats2[50][10] + coeff0_51*dmats2[51][10] + coeff0_52*dmats2[52][10] + coeff0_53*dmats2[53][10] + coeff0_54*dmats2[54][10] + coeff0_55*dmats2[55][10];
          new_coeff0_11 = coeff0_0*dmats2[0][11] + coeff0_1*dmats2[1][11] + coeff0_2*dmats2[2][11] + coeff0_3*dmats2[3][11] + coeff0_4*dmats2[4][11] + coeff0_5*dmats2[5][11] + coeff0_6*dmats2[6][11] + coeff0_7*dmats2[7][11] + coeff0_8*dmats2[8][11] + coeff0_9*dmats2[9][11] + coeff0_10*dmats2[10][11] + coeff0_11*dmats2[11][11] + coeff0_12*dmats2[12][11] + coeff0_13*dmats2[13][11] + coeff0_14*dmats2[14][11] + coeff0_15*dmats2[15][11] + coeff0_16*dmats2[16][11] + coeff0_17*dmats2[17][11] + coeff0_18*dmats2[18][11] + coeff0_19*dmats2[19][11] + coeff0_20*dmats2[20][11] + coeff0_21*dmats2[21][11] + coeff0_22*dmats2[22][11] + coeff0_23*dmats2[23][11] + coeff0_24*dmats2[24][11] + coeff0_25*dmats2[25][11] + coeff0_26*dmats2[26][11] + coeff0_27*dmats2[27][11] + coeff0_28*dmats2[28][11] + coeff0_29*dmats2[29][11] + coeff0_30*dmats2[30][11] + coeff0_31*dmats2[31][11] + coeff0_32*dmats2[32][11] + coeff0_33*dmats2[33][11] + coeff0_34*dmats2[34][11] + coeff0_35*dmats2[35][11] + coeff0_36*dmats2[36][11] + coeff0_37*dmats2[37][11] + coeff0_38*dmats2[38][11] + coeff0_39*dmats2[39][11] + coeff0_40*dmats2[40][11] + coeff0_41*dmats2[41][11] + coeff0_42*dmats2[42][11] + coeff0_43*dmats2[43][11] + coeff0_44*dmats2[44][11] + coeff0_45*dmats2[45][11] + coeff0_46*dmats2[46][11] + coeff0_47*dmats2[47][11] + coeff0_48*dmats2[48][11] + coeff0_49*dmats2[49][11] + coeff0_50*dmats2[50][11] + coeff0_51*dmats2[51][11] + coeff0_52*dmats2[52][11] + coeff0_53*dmats2[53][11] + coeff0_54*dmats2[54][11] + coeff0_55*dmats2[55][11];
          new_coeff0_12 = coeff0_0*dmats2[0][12] + coeff0_1*dmats2[1][12] + coeff0_2*dmats2[2][12] + coeff0_3*dmats2[3][12] + coeff0_4*dmats2[4][12] + coeff0_5*dmats2[5][12] + coeff0_6*dmats2[6][12] + coeff0_7*dmats2[7][12] + coeff0_8*dmats2[8][12] + coeff0_9*dmats2[9][12] + coeff0_10*dmats2[10][12] + coeff0_11*dmats2[11][12] + coeff0_12*dmats2[12][12] + coeff0_13*dmats2[13][12] + coeff0_14*dmats2[14][12] + coeff0_15*dmats2[15][12] + coeff0_16*dmats2[16][12] + coeff0_17*dmats2[17][12] + coeff0_18*dmats2[18][12] + coeff0_19*dmats2[19][12] + coeff0_20*dmats2[20][12] + coeff0_21*dmats2[21][12] + coeff0_22*dmats2[22][12] + coeff0_23*dmats2[23][12] + coeff0_24*dmats2[24][12] + coeff0_25*dmats2[25][12] + coeff0_26*dmats2[26][12] + coeff0_27*dmats2[27][12] + coeff0_28*dmats2[28][12] + coeff0_29*dmats2[29][12] + coeff0_30*dmats2[30][12] + coeff0_31*dmats2[31][12] + coeff0_32*dmats2[32][12] + coeff0_33*dmats2[33][12] + coeff0_34*dmats2[34][12] + coeff0_35*dmats2[35][12] + coeff0_36*dmats2[36][12] + coeff0_37*dmats2[37][12] + coeff0_38*dmats2[38][12] + coeff0_39*dmats2[39][12] + coeff0_40*dmats2[40][12] + coeff0_41*dmats2[41][12] + coeff0_42*dmats2[42][12] + coeff0_43*dmats2[43][12] + coeff0_44*dmats2[44][12] + coeff0_45*dmats2[45][12] + coeff0_46*dmats2[46][12] + coeff0_47*dmats2[47][12] + coeff0_48*dmats2[48][12] + coeff0_49*dmats2[49][12] + coeff0_50*dmats2[50][12] + coeff0_51*dmats2[51][12] + coeff0_52*dmats2[52][12] + coeff0_53*dmats2[53][12] + coeff0_54*dmats2[54][12] + coeff0_55*dmats2[55][12];
          new_coeff0_13 = coeff0_0*dmats2[0][13] + coeff0_1*dmats2[1][13] + coeff0_2*dmats2[2][13] + coeff0_3*dmats2[3][13] + coeff0_4*dmats2[4][13] + coeff0_5*dmats2[5][13] + coeff0_6*dmats2[6][13] + coeff0_7*dmats2[7][13] + coeff0_8*dmats2[8][13] + coeff0_9*dmats2[9][13] + coeff0_10*dmats2[10][13] + coeff0_11*dmats2[11][13] + coeff0_12*dmats2[12][13] + coeff0_13*dmats2[13][13] + coeff0_14*dmats2[14][13] + coeff0_15*dmats2[15][13] + coeff0_16*dmats2[16][13] + coeff0_17*dmats2[17][13] + coeff0_18*dmats2[18][13] + coeff0_19*dmats2[19][13] + coeff0_20*dmats2[20][13] + coeff0_21*dmats2[21][13] + coeff0_22*dmats2[22][13] + coeff0_23*dmats2[23][13] + coeff0_24*dmats2[24][13] + coeff0_25*dmats2[25][13] + coeff0_26*dmats2[26][13] + coeff0_27*dmats2[27][13] + coeff0_28*dmats2[28][13] + coeff0_29*dmats2[29][13] + coeff0_30*dmats2[30][13] + coeff0_31*dmats2[31][13] + coeff0_32*dmats2[32][13] + coeff0_33*dmats2[33][13] + coeff0_34*dmats2[34][13] + coeff0_35*dmats2[35][13] + coeff0_36*dmats2[36][13] + coeff0_37*dmats2[37][13] + coeff0_38*dmats2[38][13] + coeff0_39*dmats2[39][13] + coeff0_40*dmats2[40][13] + coeff0_41*dmats2[41][13] + coeff0_42*dmats2[42][13] + coeff0_43*dmats2[43][13] + coeff0_44*dmats2[44][13] + coeff0_45*dmats2[45][13] + coeff0_46*dmats2[46][13] + coeff0_47*dmats2[47][13] + coeff0_48*dmats2[48][13] + coeff0_49*dmats2[49][13] + coeff0_50*dmats2[50][13] + coeff0_51*dmats2[51][13] + coeff0_52*dmats2[52][13] + coeff0_53*dmats2[53][13] + coeff0_54*dmats2[54][13] + coeff0_55*dmats2[55][13];
          new_coeff0_14 = coeff0_0*dmats2[0][14] + coeff0_1*dmats2[1][14] + coeff0_2*dmats2[2][14] + coeff0_3*dmats2[3][14] + coeff0_4*dmats2[4][14] + coeff0_5*dmats2[5][14] + coeff0_6*dmats2[6][14] + coeff0_7*dmats2[7][14] + coeff0_8*dmats2[8][14] + coeff0_9*dmats2[9][14] + coeff0_10*dmats2[10][14] + coeff0_11*dmats2[11][14] + coeff0_12*dmats2[12][14] + coeff0_13*dmats2[13][14] + coeff0_14*dmats2[14][14] + coeff0_15*dmats2[15][14] + coeff0_16*dmats2[16][14] + coeff0_17*dmats2[17][14] + coeff0_18*dmats2[18][14] + coeff0_19*dmats2[19][14] + coeff0_20*dmats2[20][14] + coeff0_21*dmats2[21][14] + coeff0_22*dmats2[22][14] + coeff0_23*dmats2[23][14] + coeff0_24*dmats2[24][14] + coeff0_25*dmats2[25][14] + coeff0_26*dmats2[26][14] + coeff0_27*dmats2[27][14] + coeff0_28*dmats2[28][14] + coeff0_29*dmats2[29][14] + coeff0_30*dmats2[30][14] + coeff0_31*dmats2[31][14] + coeff0_32*dmats2[32][14] + coeff0_33*dmats2[33][14] + coeff0_34*dmats2[34][14] + coeff0_35*dmats2[35][14] + coeff0_36*dmats2[36][14] + coeff0_37*dmats2[37][14] + coeff0_38*dmats2[38][14] + coeff0_39*dmats2[39][14] + coeff0_40*dmats2[40][14] + coeff0_41*dmats2[41][14] + coeff0_42*dmats2[42][14] + coeff0_43*dmats2[43][14] + coeff0_44*dmats2[44][14] + coeff0_45*dmats2[45][14] + coeff0_46*dmats2[46][14] + coeff0_47*dmats2[47][14] + coeff0_48*dmats2[48][14] + coeff0_49*dmats2[49][14] + coeff0_50*dmats2[50][14] + coeff0_51*dmats2[51][14] + coeff0_52*dmats2[52][14] + coeff0_53*dmats2[53][14] + coeff0_54*dmats2[54][14] + coeff0_55*dmats2[55][14];
          new_coeff0_15 = coeff0_0*dmats2[0][15] + coeff0_1*dmats2[1][15] + coeff0_2*dmats2[2][15] + coeff0_3*dmats2[3][15] + coeff0_4*dmats2[4][15] + coeff0_5*dmats2[5][15] + coeff0_6*dmats2[6][15] + coeff0_7*dmats2[7][15] + coeff0_8*dmats2[8][15] + coeff0_9*dmats2[9][15] + coeff0_10*dmats2[10][15] + coeff0_11*dmats2[11][15] + coeff0_12*dmats2[12][15] + coeff0_13*dmats2[13][15] + coeff0_14*dmats2[14][15] + coeff0_15*dmats2[15][15] + coeff0_16*dmats2[16][15] + coeff0_17*dmats2[17][15] + coeff0_18*dmats2[18][15] + coeff0_19*dmats2[19][15] + coeff0_20*dmats2[20][15] + coeff0_21*dmats2[21][15] + coeff0_22*dmats2[22][15] + coeff0_23*dmats2[23][15] + coeff0_24*dmats2[24][15] + coeff0_25*dmats2[25][15] + coeff0_26*dmats2[26][15] + coeff0_27*dmats2[27][15] + coeff0_28*dmats2[28][15] + coeff0_29*dmats2[29][15] + coeff0_30*dmats2[30][15] + coeff0_31*dmats2[31][15] + coeff0_32*dmats2[32][15] + coeff0_33*dmats2[33][15] + coeff0_34*dmats2[34][15] + coeff0_35*dmats2[35][15] + coeff0_36*dmats2[36][15] + coeff0_37*dmats2[37][15] + coeff0_38*dmats2[38][15] + coeff0_39*dmats2[39][15] + coeff0_40*dmats2[40][15] + coeff0_41*dmats2[41][15] + coeff0_42*dmats2[42][15] + coeff0_43*dmats2[43][15] + coeff0_44*dmats2[44][15] + coeff0_45*dmats2[45][15] + coeff0_46*dmats2[46][15] + coeff0_47*dmats2[47][15] + coeff0_48*dmats2[48][15] + coeff0_49*dmats2[49][15] + coeff0_50*dmats2[50][15] + coeff0_51*dmats2[51][15] + coeff0_52*dmats2[52][15] + coeff0_53*dmats2[53][15] + coeff0_54*dmats2[54][15] + coeff0_55*dmats2[55][15];
          new_coeff0_16 = coeff0_0*dmats2[0][16] + coeff0_1*dmats2[1][16] + coeff0_2*dmats2[2][16] + coeff0_3*dmats2[3][16] + coeff0_4*dmats2[4][16] + coeff0_5*dmats2[5][16] + coeff0_6*dmats2[6][16] + coeff0_7*dmats2[7][16] + coeff0_8*dmats2[8][16] + coeff0_9*dmats2[9][16] + coeff0_10*dmats2[10][16] + coeff0_11*dmats2[11][16] + coeff0_12*dmats2[12][16] + coeff0_13*dmats2[13][16] + coeff0_14*dmats2[14][16] + coeff0_15*dmats2[15][16] + coeff0_16*dmats2[16][16] + coeff0_17*dmats2[17][16] + coeff0_18*dmats2[18][16] + coeff0_19*dmats2[19][16] + coeff0_20*dmats2[20][16] + coeff0_21*dmats2[21][16] + coeff0_22*dmats2[22][16] + coeff0_23*dmats2[23][16] + coeff0_24*dmats2[24][16] + coeff0_25*dmats2[25][16] + coeff0_26*dmats2[26][16] + coeff0_27*dmats2[27][16] + coeff0_28*dmats2[28][16] + coeff0_29*dmats2[29][16] + coeff0_30*dmats2[30][16] + coeff0_31*dmats2[31][16] + coeff0_32*dmats2[32][16] + coeff0_33*dmats2[33][16] + coeff0_34*dmats2[34][16] + coeff0_35*dmats2[35][16] + coeff0_36*dmats2[36][16] + coeff0_37*dmats2[37][16] + coeff0_38*dmats2[38][16] + coeff0_39*dmats2[39][16] + coeff0_40*dmats2[40][16] + coeff0_41*dmats2[41][16] + coeff0_42*dmats2[42][16] + coeff0_43*dmats2[43][16] + coeff0_44*dmats2[44][16] + coeff0_45*dmats2[45][16] + coeff0_46*dmats2[46][16] + coeff0_47*dmats2[47][16] + coeff0_48*dmats2[48][16] + coeff0_49*dmats2[49][16] + coeff0_50*dmats2[50][16] + coeff0_51*dmats2[51][16] + coeff0_52*dmats2[52][16] + coeff0_53*dmats2[53][16] + coeff0_54*dmats2[54][16] + coeff0_55*dmats2[55][16];
          new_coeff0_17 = coeff0_0*dmats2[0][17] + coeff0_1*dmats2[1][17] + coeff0_2*dmats2[2][17] + coeff0_3*dmats2[3][17] + coeff0_4*dmats2[4][17] + coeff0_5*dmats2[5][17] + coeff0_6*dmats2[6][17] + coeff0_7*dmats2[7][17] + coeff0_8*dmats2[8][17] + coeff0_9*dmats2[9][17] + coeff0_10*dmats2[10][17] + coeff0_11*dmats2[11][17] + coeff0_12*dmats2[12][17] + coeff0_13*dmats2[13][17] + coeff0_14*dmats2[14][17] + coeff0_15*dmats2[15][17] + coeff0_16*dmats2[16][17] + coeff0_17*dmats2[17][17] + coeff0_18*dmats2[18][17] + coeff0_19*dmats2[19][17] + coeff0_20*dmats2[20][17] + coeff0_21*dmats2[21][17] + coeff0_22*dmats2[22][17] + coeff0_23*dmats2[23][17] + coeff0_24*dmats2[24][17] + coeff0_25*dmats2[25][17] + coeff0_26*dmats2[26][17] + coeff0_27*dmats2[27][17] + coeff0_28*dmats2[28][17] + coeff0_29*dmats2[29][17] + coeff0_30*dmats2[30][17] + coeff0_31*dmats2[31][17] + coeff0_32*dmats2[32][17] + coeff0_33*dmats2[33][17] + coeff0_34*dmats2[34][17] + coeff0_35*dmats2[35][17] + coeff0_36*dmats2[36][17] + coeff0_37*dmats2[37][17] + coeff0_38*dmats2[38][17] + coeff0_39*dmats2[39][17] + coeff0_40*dmats2[40][17] + coeff0_41*dmats2[41][17] + coeff0_42*dmats2[42][17] + coeff0_43*dmats2[43][17] + coeff0_44*dmats2[44][17] + coeff0_45*dmats2[45][17] + coeff0_46*dmats2[46][17] + coeff0_47*dmats2[47][17] + coeff0_48*dmats2[48][17] + coeff0_49*dmats2[49][17] + coeff0_50*dmats2[50][17] + coeff0_51*dmats2[51][17] + coeff0_52*dmats2[52][17] + coeff0_53*dmats2[53][17] + coeff0_54*dmats2[54][17] + coeff0_55*dmats2[55][17];
          new_coeff0_18 = coeff0_0*dmats2[0][18] + coeff0_1*dmats2[1][18] + coeff0_2*dmats2[2][18] + coeff0_3*dmats2[3][18] + coeff0_4*dmats2[4][18] + coeff0_5*dmats2[5][18] + coeff0_6*dmats2[6][18] + coeff0_7*dmats2[7][18] + coeff0_8*dmats2[8][18] + coeff0_9*dmats2[9][18] + coeff0_10*dmats2[10][18] + coeff0_11*dmats2[11][18] + coeff0_12*dmats2[12][18] + coeff0_13*dmats2[13][18] + coeff0_14*dmats2[14][18] + coeff0_15*dmats2[15][18] + coeff0_16*dmats2[16][18] + coeff0_17*dmats2[17][18] + coeff0_18*dmats2[18][18] + coeff0_19*dmats2[19][18] + coeff0_20*dmats2[20][18] + coeff0_21*dmats2[21][18] + coeff0_22*dmats2[22][18] + coeff0_23*dmats2[23][18] + coeff0_24*dmats2[24][18] + coeff0_25*dmats2[25][18] + coeff0_26*dmats2[26][18] + coeff0_27*dmats2[27][18] + coeff0_28*dmats2[28][18] + coeff0_29*dmats2[29][18] + coeff0_30*dmats2[30][18] + coeff0_31*dmats2[31][18] + coeff0_32*dmats2[32][18] + coeff0_33*dmats2[33][18] + coeff0_34*dmats2[34][18] + coeff0_35*dmats2[35][18] + coeff0_36*dmats2[36][18] + coeff0_37*dmats2[37][18] + coeff0_38*dmats2[38][18] + coeff0_39*dmats2[39][18] + coeff0_40*dmats2[40][18] + coeff0_41*dmats2[41][18] + coeff0_42*dmats2[42][18] + coeff0_43*dmats2[43][18] + coeff0_44*dmats2[44][18] + coeff0_45*dmats2[45][18] + coeff0_46*dmats2[46][18] + coeff0_47*dmats2[47][18] + coeff0_48*dmats2[48][18] + coeff0_49*dmats2[49][18] + coeff0_50*dmats2[50][18] + coeff0_51*dmats2[51][18] + coeff0_52*dmats2[52][18] + coeff0_53*dmats2[53][18] + coeff0_54*dmats2[54][18] + coeff0_55*dmats2[55][18];
          new_coeff0_19 = coeff0_0*dmats2[0][19] + coeff0_1*dmats2[1][19] + coeff0_2*dmats2[2][19] + coeff0_3*dmats2[3][19] + coeff0_4*dmats2[4][19] + coeff0_5*dmats2[5][19] + coeff0_6*dmats2[6][19] + coeff0_7*dmats2[7][19] + coeff0_8*dmats2[8][19] + coeff0_9*dmats2[9][19] + coeff0_10*dmats2[10][19] + coeff0_11*dmats2[11][19] + coeff0_12*dmats2[12][19] + coeff0_13*dmats2[13][19] + coeff0_14*dmats2[14][19] + coeff0_15*dmats2[15][19] + coeff0_16*dmats2[16][19] + coeff0_17*dmats2[17][19] + coeff0_18*dmats2[18][19] + coeff0_19*dmats2[19][19] + coeff0_20*dmats2[20][19] + coeff0_21*dmats2[21][19] + coeff0_22*dmats2[22][19] + coeff0_23*dmats2[23][19] + coeff0_24*dmats2[24][19] + coeff0_25*dmats2[25][19] + coeff0_26*dmats2[26][19] + coeff0_27*dmats2[27][19] + coeff0_28*dmats2[28][19] + coeff0_29*dmats2[29][19] + coeff0_30*dmats2[30][19] + coeff0_31*dmats2[31][19] + coeff0_32*dmats2[32][19] + coeff0_33*dmats2[33][19] + coeff0_34*dmats2[34][19] + coeff0_35*dmats2[35][19] + coeff0_36*dmats2[36][19] + coeff0_37*dmats2[37][19] + coeff0_38*dmats2[38][19] + coeff0_39*dmats2[39][19] + coeff0_40*dmats2[40][19] + coeff0_41*dmats2[41][19] + coeff0_42*dmats2[42][19] + coeff0_43*dmats2[43][19] + coeff0_44*dmats2[44][19] + coeff0_45*dmats2[45][19] + coeff0_46*dmats2[46][19] + coeff0_47*dmats2[47][19] + coeff0_48*dmats2[48][19] + coeff0_49*dmats2[49][19] + coeff0_50*dmats2[50][19] + coeff0_51*dmats2[51][19] + coeff0_52*dmats2[52][19] + coeff0_53*dmats2[53][19] + coeff0_54*dmats2[54][19] + coeff0_55*dmats2[55][19];
          new_coeff0_20 = coeff0_0*dmats2[0][20] + coeff0_1*dmats2[1][20] + coeff0_2*dmats2[2][20] + coeff0_3*dmats2[3][20] + coeff0_4*dmats2[4][20] + coeff0_5*dmats2[5][20] + coeff0_6*dmats2[6][20] + coeff0_7*dmats2[7][20] + coeff0_8*dmats2[8][20] + coeff0_9*dmats2[9][20] + coeff0_10*dmats2[10][20] + coeff0_11*dmats2[11][20] + coeff0_12*dmats2[12][20] + coeff0_13*dmats2[13][20] + coeff0_14*dmats2[14][20] + coeff0_15*dmats2[15][20] + coeff0_16*dmats2[16][20] + coeff0_17*dmats2[17][20] + coeff0_18*dmats2[18][20] + coeff0_19*dmats2[19][20] + coeff0_20*dmats2[20][20] + coeff0_21*dmats2[21][20] + coeff0_22*dmats2[22][20] + coeff0_23*dmats2[23][20] + coeff0_24*dmats2[24][20] + coeff0_25*dmats2[25][20] + coeff0_26*dmats2[26][20] + coeff0_27*dmats2[27][20] + coeff0_28*dmats2[28][20] + coeff0_29*dmats2[29][20] + coeff0_30*dmats2[30][20] + coeff0_31*dmats2[31][20] + coeff0_32*dmats2[32][20] + coeff0_33*dmats2[33][20] + coeff0_34*dmats2[34][20] + coeff0_35*dmats2[35][20] + coeff0_36*dmats2[36][20] + coeff0_37*dmats2[37][20] + coeff0_38*dmats2[38][20] + coeff0_39*dmats2[39][20] + coeff0_40*dmats2[40][20] + coeff0_41*dmats2[41][20] + coeff0_42*dmats2[42][20] + coeff0_43*dmats2[43][20] + coeff0_44*dmats2[44][20] + coeff0_45*dmats2[45][20] + coeff0_46*dmats2[46][20] + coeff0_47*dmats2[47][20] + coeff0_48*dmats2[48][20] + coeff0_49*dmats2[49][20] + coeff0_50*dmats2[50][20] + coeff0_51*dmats2[51][20] + coeff0_52*dmats2[52][20] + coeff0_53*dmats2[53][20] + coeff0_54*dmats2[54][20] + coeff0_55*dmats2[55][20];
          new_coeff0_21 = coeff0_0*dmats2[0][21] + coeff0_1*dmats2[1][21] + coeff0_2*dmats2[2][21] + coeff0_3*dmats2[3][21] + coeff0_4*dmats2[4][21] + coeff0_5*dmats2[5][21] + coeff0_6*dmats2[6][21] + coeff0_7*dmats2[7][21] + coeff0_8*dmats2[8][21] + coeff0_9*dmats2[9][21] + coeff0_10*dmats2[10][21] + coeff0_11*dmats2[11][21] + coeff0_12*dmats2[12][21] + coeff0_13*dmats2[13][21] + coeff0_14*dmats2[14][21] + coeff0_15*dmats2[15][21] + coeff0_16*dmats2[16][21] + coeff0_17*dmats2[17][21] + coeff0_18*dmats2[18][21] + coeff0_19*dmats2[19][21] + coeff0_20*dmats2[20][21] + coeff0_21*dmats2[21][21] + coeff0_22*dmats2[22][21] + coeff0_23*dmats2[23][21] + coeff0_24*dmats2[24][21] + coeff0_25*dmats2[25][21] + coeff0_26*dmats2[26][21] + coeff0_27*dmats2[27][21] + coeff0_28*dmats2[28][21] + coeff0_29*dmats2[29][21] + coeff0_30*dmats2[30][21] + coeff0_31*dmats2[31][21] + coeff0_32*dmats2[32][21] + coeff0_33*dmats2[33][21] + coeff0_34*dmats2[34][21] + coeff0_35*dmats2[35][21] + coeff0_36*dmats2[36][21] + coeff0_37*dmats2[37][21] + coeff0_38*dmats2[38][21] + coeff0_39*dmats2[39][21] + coeff0_40*dmats2[40][21] + coeff0_41*dmats2[41][21] + coeff0_42*dmats2[42][21] + coeff0_43*dmats2[43][21] + coeff0_44*dmats2[44][21] + coeff0_45*dmats2[45][21] + coeff0_46*dmats2[46][21] + coeff0_47*dmats2[47][21] + coeff0_48*dmats2[48][21] + coeff0_49*dmats2[49][21] + coeff0_50*dmats2[50][21] + coeff0_51*dmats2[51][21] + coeff0_52*dmats2[52][21] + coeff0_53*dmats2[53][21] + coeff0_54*dmats2[54][21] + coeff0_55*dmats2[55][21];
          new_coeff0_22 = coeff0_0*dmats2[0][22] + coeff0_1*dmats2[1][22] + coeff0_2*dmats2[2][22] + coeff0_3*dmats2[3][22] + coeff0_4*dmats2[4][22] + coeff0_5*dmats2[5][22] + coeff0_6*dmats2[6][22] + coeff0_7*dmats2[7][22] + coeff0_8*dmats2[8][22] + coeff0_9*dmats2[9][22] + coeff0_10*dmats2[10][22] + coeff0_11*dmats2[11][22] + coeff0_12*dmats2[12][22] + coeff0_13*dmats2[13][22] + coeff0_14*dmats2[14][22] + coeff0_15*dmats2[15][22] + coeff0_16*dmats2[16][22] + coeff0_17*dmats2[17][22] + coeff0_18*dmats2[18][22] + coeff0_19*dmats2[19][22] + coeff0_20*dmats2[20][22] + coeff0_21*dmats2[21][22] + coeff0_22*dmats2[22][22] + coeff0_23*dmats2[23][22] + coeff0_24*dmats2[24][22] + coeff0_25*dmats2[25][22] + coeff0_26*dmats2[26][22] + coeff0_27*dmats2[27][22] + coeff0_28*dmats2[28][22] + coeff0_29*dmats2[29][22] + coeff0_30*dmats2[30][22] + coeff0_31*dmats2[31][22] + coeff0_32*dmats2[32][22] + coeff0_33*dmats2[33][22] + coeff0_34*dmats2[34][22] + coeff0_35*dmats2[35][22] + coeff0_36*dmats2[36][22] + coeff0_37*dmats2[37][22] + coeff0_38*dmats2[38][22] + coeff0_39*dmats2[39][22] + coeff0_40*dmats2[40][22] + coeff0_41*dmats2[41][22] + coeff0_42*dmats2[42][22] + coeff0_43*dmats2[43][22] + coeff0_44*dmats2[44][22] + coeff0_45*dmats2[45][22] + coeff0_46*dmats2[46][22] + coeff0_47*dmats2[47][22] + coeff0_48*dmats2[48][22] + coeff0_49*dmats2[49][22] + coeff0_50*dmats2[50][22] + coeff0_51*dmats2[51][22] + coeff0_52*dmats2[52][22] + coeff0_53*dmats2[53][22] + coeff0_54*dmats2[54][22] + coeff0_55*dmats2[55][22];
          new_coeff0_23 = coeff0_0*dmats2[0][23] + coeff0_1*dmats2[1][23] + coeff0_2*dmats2[2][23] + coeff0_3*dmats2[3][23] + coeff0_4*dmats2[4][23] + coeff0_5*dmats2[5][23] + coeff0_6*dmats2[6][23] + coeff0_7*dmats2[7][23] + coeff0_8*dmats2[8][23] + coeff0_9*dmats2[9][23] + coeff0_10*dmats2[10][23] + coeff0_11*dmats2[11][23] + coeff0_12*dmats2[12][23] + coeff0_13*dmats2[13][23] + coeff0_14*dmats2[14][23] + coeff0_15*dmats2[15][23] + coeff0_16*dmats2[16][23] + coeff0_17*dmats2[17][23] + coeff0_18*dmats2[18][23] + coeff0_19*dmats2[19][23] + coeff0_20*dmats2[20][23] + coeff0_21*dmats2[21][23] + coeff0_22*dmats2[22][23] + coeff0_23*dmats2[23][23] + coeff0_24*dmats2[24][23] + coeff0_25*dmats2[25][23] + coeff0_26*dmats2[26][23] + coeff0_27*dmats2[27][23] + coeff0_28*dmats2[28][23] + coeff0_29*dmats2[29][23] + coeff0_30*dmats2[30][23] + coeff0_31*dmats2[31][23] + coeff0_32*dmats2[32][23] + coeff0_33*dmats2[33][23] + coeff0_34*dmats2[34][23] + coeff0_35*dmats2[35][23] + coeff0_36*dmats2[36][23] + coeff0_37*dmats2[37][23] + coeff0_38*dmats2[38][23] + coeff0_39*dmats2[39][23] + coeff0_40*dmats2[40][23] + coeff0_41*dmats2[41][23] + coeff0_42*dmats2[42][23] + coeff0_43*dmats2[43][23] + coeff0_44*dmats2[44][23] + coeff0_45*dmats2[45][23] + coeff0_46*dmats2[46][23] + coeff0_47*dmats2[47][23] + coeff0_48*dmats2[48][23] + coeff0_49*dmats2[49][23] + coeff0_50*dmats2[50][23] + coeff0_51*dmats2[51][23] + coeff0_52*dmats2[52][23] + coeff0_53*dmats2[53][23] + coeff0_54*dmats2[54][23] + coeff0_55*dmats2[55][23];
          new_coeff0_24 = coeff0_0*dmats2[0][24] + coeff0_1*dmats2[1][24] + coeff0_2*dmats2[2][24] + coeff0_3*dmats2[3][24] + coeff0_4*dmats2[4][24] + coeff0_5*dmats2[5][24] + coeff0_6*dmats2[6][24] + coeff0_7*dmats2[7][24] + coeff0_8*dmats2[8][24] + coeff0_9*dmats2[9][24] + coeff0_10*dmats2[10][24] + coeff0_11*dmats2[11][24] + coeff0_12*dmats2[12][24] + coeff0_13*dmats2[13][24] + coeff0_14*dmats2[14][24] + coeff0_15*dmats2[15][24] + coeff0_16*dmats2[16][24] + coeff0_17*dmats2[17][24] + coeff0_18*dmats2[18][24] + coeff0_19*dmats2[19][24] + coeff0_20*dmats2[20][24] + coeff0_21*dmats2[21][24] + coeff0_22*dmats2[22][24] + coeff0_23*dmats2[23][24] + coeff0_24*dmats2[24][24] + coeff0_25*dmats2[25][24] + coeff0_26*dmats2[26][24] + coeff0_27*dmats2[27][24] + coeff0_28*dmats2[28][24] + coeff0_29*dmats2[29][24] + coeff0_30*dmats2[30][24] + coeff0_31*dmats2[31][24] + coeff0_32*dmats2[32][24] + coeff0_33*dmats2[33][24] + coeff0_34*dmats2[34][24] + coeff0_35*dmats2[35][24] + coeff0_36*dmats2[36][24] + coeff0_37*dmats2[37][24] + coeff0_38*dmats2[38][24] + coeff0_39*dmats2[39][24] + coeff0_40*dmats2[40][24] + coeff0_41*dmats2[41][24] + coeff0_42*dmats2[42][24] + coeff0_43*dmats2[43][24] + coeff0_44*dmats2[44][24] + coeff0_45*dmats2[45][24] + coeff0_46*dmats2[46][24] + coeff0_47*dmats2[47][24] + coeff0_48*dmats2[48][24] + coeff0_49*dmats2[49][24] + coeff0_50*dmats2[50][24] + coeff0_51*dmats2[51][24] + coeff0_52*dmats2[52][24] + coeff0_53*dmats2[53][24] + coeff0_54*dmats2[54][24] + coeff0_55*dmats2[55][24];
          new_coeff0_25 = coeff0_0*dmats2[0][25] + coeff0_1*dmats2[1][25] + coeff0_2*dmats2[2][25] + coeff0_3*dmats2[3][25] + coeff0_4*dmats2[4][25] + coeff0_5*dmats2[5][25] + coeff0_6*dmats2[6][25] + coeff0_7*dmats2[7][25] + coeff0_8*dmats2[8][25] + coeff0_9*dmats2[9][25] + coeff0_10*dmats2[10][25] + coeff0_11*dmats2[11][25] + coeff0_12*dmats2[12][25] + coeff0_13*dmats2[13][25] + coeff0_14*dmats2[14][25] + coeff0_15*dmats2[15][25] + coeff0_16*dmats2[16][25] + coeff0_17*dmats2[17][25] + coeff0_18*dmats2[18][25] + coeff0_19*dmats2[19][25] + coeff0_20*dmats2[20][25] + coeff0_21*dmats2[21][25] + coeff0_22*dmats2[22][25] + coeff0_23*dmats2[23][25] + coeff0_24*dmats2[24][25] + coeff0_25*dmats2[25][25] + coeff0_26*dmats2[26][25] + coeff0_27*dmats2[27][25] + coeff0_28*dmats2[28][25] + coeff0_29*dmats2[29][25] + coeff0_30*dmats2[30][25] + coeff0_31*dmats2[31][25] + coeff0_32*dmats2[32][25] + coeff0_33*dmats2[33][25] + coeff0_34*dmats2[34][25] + coeff0_35*dmats2[35][25] + coeff0_36*dmats2[36][25] + coeff0_37*dmats2[37][25] + coeff0_38*dmats2[38][25] + coeff0_39*dmats2[39][25] + coeff0_40*dmats2[40][25] + coeff0_41*dmats2[41][25] + coeff0_42*dmats2[42][25] + coeff0_43*dmats2[43][25] + coeff0_44*dmats2[44][25] + coeff0_45*dmats2[45][25] + coeff0_46*dmats2[46][25] + coeff0_47*dmats2[47][25] + coeff0_48*dmats2[48][25] + coeff0_49*dmats2[49][25] + coeff0_50*dmats2[50][25] + coeff0_51*dmats2[51][25] + coeff0_52*dmats2[52][25] + coeff0_53*dmats2[53][25] + coeff0_54*dmats2[54][25] + coeff0_55*dmats2[55][25];
          new_coeff0_26 = coeff0_0*dmats2[0][26] + coeff0_1*dmats2[1][26] + coeff0_2*dmats2[2][26] + coeff0_3*dmats2[3][26] + coeff0_4*dmats2[4][26] + coeff0_5*dmats2[5][26] + coeff0_6*dmats2[6][26] + coeff0_7*dmats2[7][26] + coeff0_8*dmats2[8][26] + coeff0_9*dmats2[9][26] + coeff0_10*dmats2[10][26] + coeff0_11*dmats2[11][26] + coeff0_12*dmats2[12][26] + coeff0_13*dmats2[13][26] + coeff0_14*dmats2[14][26] + coeff0_15*dmats2[15][26] + coeff0_16*dmats2[16][26] + coeff0_17*dmats2[17][26] + coeff0_18*dmats2[18][26] + coeff0_19*dmats2[19][26] + coeff0_20*dmats2[20][26] + coeff0_21*dmats2[21][26] + coeff0_22*dmats2[22][26] + coeff0_23*dmats2[23][26] + coeff0_24*dmats2[24][26] + coeff0_25*dmats2[25][26] + coeff0_26*dmats2[26][26] + coeff0_27*dmats2[27][26] + coeff0_28*dmats2[28][26] + coeff0_29*dmats2[29][26] + coeff0_30*dmats2[30][26] + coeff0_31*dmats2[31][26] + coeff0_32*dmats2[32][26] + coeff0_33*dmats2[33][26] + coeff0_34*dmats2[34][26] + coeff0_35*dmats2[35][26] + coeff0_36*dmats2[36][26] + coeff0_37*dmats2[37][26] + coeff0_38*dmats2[38][26] + coeff0_39*dmats2[39][26] + coeff0_40*dmats2[40][26] + coeff0_41*dmats2[41][26] + coeff0_42*dmats2[42][26] + coeff0_43*dmats2[43][26] + coeff0_44*dmats2[44][26] + coeff0_45*dmats2[45][26] + coeff0_46*dmats2[46][26] + coeff0_47*dmats2[47][26] + coeff0_48*dmats2[48][26] + coeff0_49*dmats2[49][26] + coeff0_50*dmats2[50][26] + coeff0_51*dmats2[51][26] + coeff0_52*dmats2[52][26] + coeff0_53*dmats2[53][26] + coeff0_54*dmats2[54][26] + coeff0_55*dmats2[55][26];
          new_coeff0_27 = coeff0_0*dmats2[0][27] + coeff0_1*dmats2[1][27] + coeff0_2*dmats2[2][27] + coeff0_3*dmats2[3][27] + coeff0_4*dmats2[4][27] + coeff0_5*dmats2[5][27] + coeff0_6*dmats2[6][27] + coeff0_7*dmats2[7][27] + coeff0_8*dmats2[8][27] + coeff0_9*dmats2[9][27] + coeff0_10*dmats2[10][27] + coeff0_11*dmats2[11][27] + coeff0_12*dmats2[12][27] + coeff0_13*dmats2[13][27] + coeff0_14*dmats2[14][27] + coeff0_15*dmats2[15][27] + coeff0_16*dmats2[16][27] + coeff0_17*dmats2[17][27] + coeff0_18*dmats2[18][27] + coeff0_19*dmats2[19][27] + coeff0_20*dmats2[20][27] + coeff0_21*dmats2[21][27] + coeff0_22*dmats2[22][27] + coeff0_23*dmats2[23][27] + coeff0_24*dmats2[24][27] + coeff0_25*dmats2[25][27] + coeff0_26*dmats2[26][27] + coeff0_27*dmats2[27][27] + coeff0_28*dmats2[28][27] + coeff0_29*dmats2[29][27] + coeff0_30*dmats2[30][27] + coeff0_31*dmats2[31][27] + coeff0_32*dmats2[32][27] + coeff0_33*dmats2[33][27] + coeff0_34*dmats2[34][27] + coeff0_35*dmats2[35][27] + coeff0_36*dmats2[36][27] + coeff0_37*dmats2[37][27] + coeff0_38*dmats2[38][27] + coeff0_39*dmats2[39][27] + coeff0_40*dmats2[40][27] + coeff0_41*dmats2[41][27] + coeff0_42*dmats2[42][27] + coeff0_43*dmats2[43][27] + coeff0_44*dmats2[44][27] + coeff0_45*dmats2[45][27] + coeff0_46*dmats2[46][27] + coeff0_47*dmats2[47][27] + coeff0_48*dmats2[48][27] + coeff0_49*dmats2[49][27] + coeff0_50*dmats2[50][27] + coeff0_51*dmats2[51][27] + coeff0_52*dmats2[52][27] + coeff0_53*dmats2[53][27] + coeff0_54*dmats2[54][27] + coeff0_55*dmats2[55][27];
          new_coeff0_28 = coeff0_0*dmats2[0][28] + coeff0_1*dmats2[1][28] + coeff0_2*dmats2[2][28] + coeff0_3*dmats2[3][28] + coeff0_4*dmats2[4][28] + coeff0_5*dmats2[5][28] + coeff0_6*dmats2[6][28] + coeff0_7*dmats2[7][28] + coeff0_8*dmats2[8][28] + coeff0_9*dmats2[9][28] + coeff0_10*dmats2[10][28] + coeff0_11*dmats2[11][28] + coeff0_12*dmats2[12][28] + coeff0_13*dmats2[13][28] + coeff0_14*dmats2[14][28] + coeff0_15*dmats2[15][28] + coeff0_16*dmats2[16][28] + coeff0_17*dmats2[17][28] + coeff0_18*dmats2[18][28] + coeff0_19*dmats2[19][28] + coeff0_20*dmats2[20][28] + coeff0_21*dmats2[21][28] + coeff0_22*dmats2[22][28] + coeff0_23*dmats2[23][28] + coeff0_24*dmats2[24][28] + coeff0_25*dmats2[25][28] + coeff0_26*dmats2[26][28] + coeff0_27*dmats2[27][28] + coeff0_28*dmats2[28][28] + coeff0_29*dmats2[29][28] + coeff0_30*dmats2[30][28] + coeff0_31*dmats2[31][28] + coeff0_32*dmats2[32][28] + coeff0_33*dmats2[33][28] + coeff0_34*dmats2[34][28] + coeff0_35*dmats2[35][28] + coeff0_36*dmats2[36][28] + coeff0_37*dmats2[37][28] + coeff0_38*dmats2[38][28] + coeff0_39*dmats2[39][28] + coeff0_40*dmats2[40][28] + coeff0_41*dmats2[41][28] + coeff0_42*dmats2[42][28] + coeff0_43*dmats2[43][28] + coeff0_44*dmats2[44][28] + coeff0_45*dmats2[45][28] + coeff0_46*dmats2[46][28] + coeff0_47*dmats2[47][28] + coeff0_48*dmats2[48][28] + coeff0_49*dmats2[49][28] + coeff0_50*dmats2[50][28] + coeff0_51*dmats2[51][28] + coeff0_52*dmats2[52][28] + coeff0_53*dmats2[53][28] + coeff0_54*dmats2[54][28] + coeff0_55*dmats2[55][28];
          new_coeff0_29 = coeff0_0*dmats2[0][29] + coeff0_1*dmats2[1][29] + coeff0_2*dmats2[2][29] + coeff0_3*dmats2[3][29] + coeff0_4*dmats2[4][29] + coeff0_5*dmats2[5][29] + coeff0_6*dmats2[6][29] + coeff0_7*dmats2[7][29] + coeff0_8*dmats2[8][29] + coeff0_9*dmats2[9][29] + coeff0_10*dmats2[10][29] + coeff0_11*dmats2[11][29] + coeff0_12*dmats2[12][29] + coeff0_13*dmats2[13][29] + coeff0_14*dmats2[14][29] + coeff0_15*dmats2[15][29] + coeff0_16*dmats2[16][29] + coeff0_17*dmats2[17][29] + coeff0_18*dmats2[18][29] + coeff0_19*dmats2[19][29] + coeff0_20*dmats2[20][29] + coeff0_21*dmats2[21][29] + coeff0_22*dmats2[22][29] + coeff0_23*dmats2[23][29] + coeff0_24*dmats2[24][29] + coeff0_25*dmats2[25][29] + coeff0_26*dmats2[26][29] + coeff0_27*dmats2[27][29] + coeff0_28*dmats2[28][29] + coeff0_29*dmats2[29][29] + coeff0_30*dmats2[30][29] + coeff0_31*dmats2[31][29] + coeff0_32*dmats2[32][29] + coeff0_33*dmats2[33][29] + coeff0_34*dmats2[34][29] + coeff0_35*dmats2[35][29] + coeff0_36*dmats2[36][29] + coeff0_37*dmats2[37][29] + coeff0_38*dmats2[38][29] + coeff0_39*dmats2[39][29] + coeff0_40*dmats2[40][29] + coeff0_41*dmats2[41][29] + coeff0_42*dmats2[42][29] + coeff0_43*dmats2[43][29] + coeff0_44*dmats2[44][29] + coeff0_45*dmats2[45][29] + coeff0_46*dmats2[46][29] + coeff0_47*dmats2[47][29] + coeff0_48*dmats2[48][29] + coeff0_49*dmats2[49][29] + coeff0_50*dmats2[50][29] + coeff0_51*dmats2[51][29] + coeff0_52*dmats2[52][29] + coeff0_53*dmats2[53][29] + coeff0_54*dmats2[54][29] + coeff0_55*dmats2[55][29];
          new_coeff0_30 = coeff0_0*dmats2[0][30] + coeff0_1*dmats2[1][30] + coeff0_2*dmats2[2][30] + coeff0_3*dmats2[3][30] + coeff0_4*dmats2[4][30] + coeff0_5*dmats2[5][30] + coeff0_6*dmats2[6][30] + coeff0_7*dmats2[7][30] + coeff0_8*dmats2[8][30] + coeff0_9*dmats2[9][30] + coeff0_10*dmats2[10][30] + coeff0_11*dmats2[11][30] + coeff0_12*dmats2[12][30] + coeff0_13*dmats2[13][30] + coeff0_14*dmats2[14][30] + coeff0_15*dmats2[15][30] + coeff0_16*dmats2[16][30] + coeff0_17*dmats2[17][30] + coeff0_18*dmats2[18][30] + coeff0_19*dmats2[19][30] + coeff0_20*dmats2[20][30] + coeff0_21*dmats2[21][30] + coeff0_22*dmats2[22][30] + coeff0_23*dmats2[23][30] + coeff0_24*dmats2[24][30] + coeff0_25*dmats2[25][30] + coeff0_26*dmats2[26][30] + coeff0_27*dmats2[27][30] + coeff0_28*dmats2[28][30] + coeff0_29*dmats2[29][30] + coeff0_30*dmats2[30][30] + coeff0_31*dmats2[31][30] + coeff0_32*dmats2[32][30] + coeff0_33*dmats2[33][30] + coeff0_34*dmats2[34][30] + coeff0_35*dmats2[35][30] + coeff0_36*dmats2[36][30] + coeff0_37*dmats2[37][30] + coeff0_38*dmats2[38][30] + coeff0_39*dmats2[39][30] + coeff0_40*dmats2[40][30] + coeff0_41*dmats2[41][30] + coeff0_42*dmats2[42][30] + coeff0_43*dmats2[43][30] + coeff0_44*dmats2[44][30] + coeff0_45*dmats2[45][30] + coeff0_46*dmats2[46][30] + coeff0_47*dmats2[47][30] + coeff0_48*dmats2[48][30] + coeff0_49*dmats2[49][30] + coeff0_50*dmats2[50][30] + coeff0_51*dmats2[51][30] + coeff0_52*dmats2[52][30] + coeff0_53*dmats2[53][30] + coeff0_54*dmats2[54][30] + coeff0_55*dmats2[55][30];
          new_coeff0_31 = coeff0_0*dmats2[0][31] + coeff0_1*dmats2[1][31] + coeff0_2*dmats2[2][31] + coeff0_3*dmats2[3][31] + coeff0_4*dmats2[4][31] + coeff0_5*dmats2[5][31] + coeff0_6*dmats2[6][31] + coeff0_7*dmats2[7][31] + coeff0_8*dmats2[8][31] + coeff0_9*dmats2[9][31] + coeff0_10*dmats2[10][31] + coeff0_11*dmats2[11][31] + coeff0_12*dmats2[12][31] + coeff0_13*dmats2[13][31] + coeff0_14*dmats2[14][31] + coeff0_15*dmats2[15][31] + coeff0_16*dmats2[16][31] + coeff0_17*dmats2[17][31] + coeff0_18*dmats2[18][31] + coeff0_19*dmats2[19][31] + coeff0_20*dmats2[20][31] + coeff0_21*dmats2[21][31] + coeff0_22*dmats2[22][31] + coeff0_23*dmats2[23][31] + coeff0_24*dmats2[24][31] + coeff0_25*dmats2[25][31] + coeff0_26*dmats2[26][31] + coeff0_27*dmats2[27][31] + coeff0_28*dmats2[28][31] + coeff0_29*dmats2[29][31] + coeff0_30*dmats2[30][31] + coeff0_31*dmats2[31][31] + coeff0_32*dmats2[32][31] + coeff0_33*dmats2[33][31] + coeff0_34*dmats2[34][31] + coeff0_35*dmats2[35][31] + coeff0_36*dmats2[36][31] + coeff0_37*dmats2[37][31] + coeff0_38*dmats2[38][31] + coeff0_39*dmats2[39][31] + coeff0_40*dmats2[40][31] + coeff0_41*dmats2[41][31] + coeff0_42*dmats2[42][31] + coeff0_43*dmats2[43][31] + coeff0_44*dmats2[44][31] + coeff0_45*dmats2[45][31] + coeff0_46*dmats2[46][31] + coeff0_47*dmats2[47][31] + coeff0_48*dmats2[48][31] + coeff0_49*dmats2[49][31] + coeff0_50*dmats2[50][31] + coeff0_51*dmats2[51][31] + coeff0_52*dmats2[52][31] + coeff0_53*dmats2[53][31] + coeff0_54*dmats2[54][31] + coeff0_55*dmats2[55][31];
          new_coeff0_32 = coeff0_0*dmats2[0][32] + coeff0_1*dmats2[1][32] + coeff0_2*dmats2[2][32] + coeff0_3*dmats2[3][32] + coeff0_4*dmats2[4][32] + coeff0_5*dmats2[5][32] + coeff0_6*dmats2[6][32] + coeff0_7*dmats2[7][32] + coeff0_8*dmats2[8][32] + coeff0_9*dmats2[9][32] + coeff0_10*dmats2[10][32] + coeff0_11*dmats2[11][32] + coeff0_12*dmats2[12][32] + coeff0_13*dmats2[13][32] + coeff0_14*dmats2[14][32] + coeff0_15*dmats2[15][32] + coeff0_16*dmats2[16][32] + coeff0_17*dmats2[17][32] + coeff0_18*dmats2[18][32] + coeff0_19*dmats2[19][32] + coeff0_20*dmats2[20][32] + coeff0_21*dmats2[21][32] + coeff0_22*dmats2[22][32] + coeff0_23*dmats2[23][32] + coeff0_24*dmats2[24][32] + coeff0_25*dmats2[25][32] + coeff0_26*dmats2[26][32] + coeff0_27*dmats2[27][32] + coeff0_28*dmats2[28][32] + coeff0_29*dmats2[29][32] + coeff0_30*dmats2[30][32] + coeff0_31*dmats2[31][32] + coeff0_32*dmats2[32][32] + coeff0_33*dmats2[33][32] + coeff0_34*dmats2[34][32] + coeff0_35*dmats2[35][32] + coeff0_36*dmats2[36][32] + coeff0_37*dmats2[37][32] + coeff0_38*dmats2[38][32] + coeff0_39*dmats2[39][32] + coeff0_40*dmats2[40][32] + coeff0_41*dmats2[41][32] + coeff0_42*dmats2[42][32] + coeff0_43*dmats2[43][32] + coeff0_44*dmats2[44][32] + coeff0_45*dmats2[45][32] + coeff0_46*dmats2[46][32] + coeff0_47*dmats2[47][32] + coeff0_48*dmats2[48][32] + coeff0_49*dmats2[49][32] + coeff0_50*dmats2[50][32] + coeff0_51*dmats2[51][32] + coeff0_52*dmats2[52][32] + coeff0_53*dmats2[53][32] + coeff0_54*dmats2[54][32] + coeff0_55*dmats2[55][32];
          new_coeff0_33 = coeff0_0*dmats2[0][33] + coeff0_1*dmats2[1][33] + coeff0_2*dmats2[2][33] + coeff0_3*dmats2[3][33] + coeff0_4*dmats2[4][33] + coeff0_5*dmats2[5][33] + coeff0_6*dmats2[6][33] + coeff0_7*dmats2[7][33] + coeff0_8*dmats2[8][33] + coeff0_9*dmats2[9][33] + coeff0_10*dmats2[10][33] + coeff0_11*dmats2[11][33] + coeff0_12*dmats2[12][33] + coeff0_13*dmats2[13][33] + coeff0_14*dmats2[14][33] + coeff0_15*dmats2[15][33] + coeff0_16*dmats2[16][33] + coeff0_17*dmats2[17][33] + coeff0_18*dmats2[18][33] + coeff0_19*dmats2[19][33] + coeff0_20*dmats2[20][33] + coeff0_21*dmats2[21][33] + coeff0_22*dmats2[22][33] + coeff0_23*dmats2[23][33] + coeff0_24*dmats2[24][33] + coeff0_25*dmats2[25][33] + coeff0_26*dmats2[26][33] + coeff0_27*dmats2[27][33] + coeff0_28*dmats2[28][33] + coeff0_29*dmats2[29][33] + coeff0_30*dmats2[30][33] + coeff0_31*dmats2[31][33] + coeff0_32*dmats2[32][33] + coeff0_33*dmats2[33][33] + coeff0_34*dmats2[34][33] + coeff0_35*dmats2[35][33] + coeff0_36*dmats2[36][33] + coeff0_37*dmats2[37][33] + coeff0_38*dmats2[38][33] + coeff0_39*dmats2[39][33] + coeff0_40*dmats2[40][33] + coeff0_41*dmats2[41][33] + coeff0_42*dmats2[42][33] + coeff0_43*dmats2[43][33] + coeff0_44*dmats2[44][33] + coeff0_45*dmats2[45][33] + coeff0_46*dmats2[46][33] + coeff0_47*dmats2[47][33] + coeff0_48*dmats2[48][33] + coeff0_49*dmats2[49][33] + coeff0_50*dmats2[50][33] + coeff0_51*dmats2[51][33] + coeff0_52*dmats2[52][33] + coeff0_53*dmats2[53][33] + coeff0_54*dmats2[54][33] + coeff0_55*dmats2[55][33];
          new_coeff0_34 = coeff0_0*dmats2[0][34] + coeff0_1*dmats2[1][34] + coeff0_2*dmats2[2][34] + coeff0_3*dmats2[3][34] + coeff0_4*dmats2[4][34] + coeff0_5*dmats2[5][34] + coeff0_6*dmats2[6][34] + coeff0_7*dmats2[7][34] + coeff0_8*dmats2[8][34] + coeff0_9*dmats2[9][34] + coeff0_10*dmats2[10][34] + coeff0_11*dmats2[11][34] + coeff0_12*dmats2[12][34] + coeff0_13*dmats2[13][34] + coeff0_14*dmats2[14][34] + coeff0_15*dmats2[15][34] + coeff0_16*dmats2[16][34] + coeff0_17*dmats2[17][34] + coeff0_18*dmats2[18][34] + coeff0_19*dmats2[19][34] + coeff0_20*dmats2[20][34] + coeff0_21*dmats2[21][34] + coeff0_22*dmats2[22][34] + coeff0_23*dmats2[23][34] + coeff0_24*dmats2[24][34] + coeff0_25*dmats2[25][34] + coeff0_26*dmats2[26][34] + coeff0_27*dmats2[27][34] + coeff0_28*dmats2[28][34] + coeff0_29*dmats2[29][34] + coeff0_30*dmats2[30][34] + coeff0_31*dmats2[31][34] + coeff0_32*dmats2[32][34] + coeff0_33*dmats2[33][34] + coeff0_34*dmats2[34][34] + coeff0_35*dmats2[35][34] + coeff0_36*dmats2[36][34] + coeff0_37*dmats2[37][34] + coeff0_38*dmats2[38][34] + coeff0_39*dmats2[39][34] + coeff0_40*dmats2[40][34] + coeff0_41*dmats2[41][34] + coeff0_42*dmats2[42][34] + coeff0_43*dmats2[43][34] + coeff0_44*dmats2[44][34] + coeff0_45*dmats2[45][34] + coeff0_46*dmats2[46][34] + coeff0_47*dmats2[47][34] + coeff0_48*dmats2[48][34] + coeff0_49*dmats2[49][34] + coeff0_50*dmats2[50][34] + coeff0_51*dmats2[51][34] + coeff0_52*dmats2[52][34] + coeff0_53*dmats2[53][34] + coeff0_54*dmats2[54][34] + coeff0_55*dmats2[55][34];
          new_coeff0_35 = coeff0_0*dmats2[0][35] + coeff0_1*dmats2[1][35] + coeff0_2*dmats2[2][35] + coeff0_3*dmats2[3][35] + coeff0_4*dmats2[4][35] + coeff0_5*dmats2[5][35] + coeff0_6*dmats2[6][35] + coeff0_7*dmats2[7][35] + coeff0_8*dmats2[8][35] + coeff0_9*dmats2[9][35] + coeff0_10*dmats2[10][35] + coeff0_11*dmats2[11][35] + coeff0_12*dmats2[12][35] + coeff0_13*dmats2[13][35] + coeff0_14*dmats2[14][35] + coeff0_15*dmats2[15][35] + coeff0_16*dmats2[16][35] + coeff0_17*dmats2[17][35] + coeff0_18*dmats2[18][35] + coeff0_19*dmats2[19][35] + coeff0_20*dmats2[20][35] + coeff0_21*dmats2[21][35] + coeff0_22*dmats2[22][35] + coeff0_23*dmats2[23][35] + coeff0_24*dmats2[24][35] + coeff0_25*dmats2[25][35] + coeff0_26*dmats2[26][35] + coeff0_27*dmats2[27][35] + coeff0_28*dmats2[28][35] + coeff0_29*dmats2[29][35] + coeff0_30*dmats2[30][35] + coeff0_31*dmats2[31][35] + coeff0_32*dmats2[32][35] + coeff0_33*dmats2[33][35] + coeff0_34*dmats2[34][35] + coeff0_35*dmats2[35][35] + coeff0_36*dmats2[36][35] + coeff0_37*dmats2[37][35] + coeff0_38*dmats2[38][35] + coeff0_39*dmats2[39][35] + coeff0_40*dmats2[40][35] + coeff0_41*dmats2[41][35] + coeff0_42*dmats2[42][35] + coeff0_43*dmats2[43][35] + coeff0_44*dmats2[44][35] + coeff0_45*dmats2[45][35] + coeff0_46*dmats2[46][35] + coeff0_47*dmats2[47][35] + coeff0_48*dmats2[48][35] + coeff0_49*dmats2[49][35] + coeff0_50*dmats2[50][35] + coeff0_51*dmats2[51][35] + coeff0_52*dmats2[52][35] + coeff0_53*dmats2[53][35] + coeff0_54*dmats2[54][35] + coeff0_55*dmats2[55][35];
          new_coeff0_36 = coeff0_0*dmats2[0][36] + coeff0_1*dmats2[1][36] + coeff0_2*dmats2[2][36] + coeff0_3*dmats2[3][36] + coeff0_4*dmats2[4][36] + coeff0_5*dmats2[5][36] + coeff0_6*dmats2[6][36] + coeff0_7*dmats2[7][36] + coeff0_8*dmats2[8][36] + coeff0_9*dmats2[9][36] + coeff0_10*dmats2[10][36] + coeff0_11*dmats2[11][36] + coeff0_12*dmats2[12][36] + coeff0_13*dmats2[13][36] + coeff0_14*dmats2[14][36] + coeff0_15*dmats2[15][36] + coeff0_16*dmats2[16][36] + coeff0_17*dmats2[17][36] + coeff0_18*dmats2[18][36] + coeff0_19*dmats2[19][36] + coeff0_20*dmats2[20][36] + coeff0_21*dmats2[21][36] + coeff0_22*dmats2[22][36] + coeff0_23*dmats2[23][36] + coeff0_24*dmats2[24][36] + coeff0_25*dmats2[25][36] + coeff0_26*dmats2[26][36] + coeff0_27*dmats2[27][36] + coeff0_28*dmats2[28][36] + coeff0_29*dmats2[29][36] + coeff0_30*dmats2[30][36] + coeff0_31*dmats2[31][36] + coeff0_32*dmats2[32][36] + coeff0_33*dmats2[33][36] + coeff0_34*dmats2[34][36] + coeff0_35*dmats2[35][36] + coeff0_36*dmats2[36][36] + coeff0_37*dmats2[37][36] + coeff0_38*dmats2[38][36] + coeff0_39*dmats2[39][36] + coeff0_40*dmats2[40][36] + coeff0_41*dmats2[41][36] + coeff0_42*dmats2[42][36] + coeff0_43*dmats2[43][36] + coeff0_44*dmats2[44][36] + coeff0_45*dmats2[45][36] + coeff0_46*dmats2[46][36] + coeff0_47*dmats2[47][36] + coeff0_48*dmats2[48][36] + coeff0_49*dmats2[49][36] + coeff0_50*dmats2[50][36] + coeff0_51*dmats2[51][36] + coeff0_52*dmats2[52][36] + coeff0_53*dmats2[53][36] + coeff0_54*dmats2[54][36] + coeff0_55*dmats2[55][36];
          new_coeff0_37 = coeff0_0*dmats2[0][37] + coeff0_1*dmats2[1][37] + coeff0_2*dmats2[2][37] + coeff0_3*dmats2[3][37] + coeff0_4*dmats2[4][37] + coeff0_5*dmats2[5][37] + coeff0_6*dmats2[6][37] + coeff0_7*dmats2[7][37] + coeff0_8*dmats2[8][37] + coeff0_9*dmats2[9][37] + coeff0_10*dmats2[10][37] + coeff0_11*dmats2[11][37] + coeff0_12*dmats2[12][37] + coeff0_13*dmats2[13][37] + coeff0_14*dmats2[14][37] + coeff0_15*dmats2[15][37] + coeff0_16*dmats2[16][37] + coeff0_17*dmats2[17][37] + coeff0_18*dmats2[18][37] + coeff0_19*dmats2[19][37] + coeff0_20*dmats2[20][37] + coeff0_21*dmats2[21][37] + coeff0_22*dmats2[22][37] + coeff0_23*dmats2[23][37] + coeff0_24*dmats2[24][37] + coeff0_25*dmats2[25][37] + coeff0_26*dmats2[26][37] + coeff0_27*dmats2[27][37] + coeff0_28*dmats2[28][37] + coeff0_29*dmats2[29][37] + coeff0_30*dmats2[30][37] + coeff0_31*dmats2[31][37] + coeff0_32*dmats2[32][37] + coeff0_33*dmats2[33][37] + coeff0_34*dmats2[34][37] + coeff0_35*dmats2[35][37] + coeff0_36*dmats2[36][37] + coeff0_37*dmats2[37][37] + coeff0_38*dmats2[38][37] + coeff0_39*dmats2[39][37] + coeff0_40*dmats2[40][37] + coeff0_41*dmats2[41][37] + coeff0_42*dmats2[42][37] + coeff0_43*dmats2[43][37] + coeff0_44*dmats2[44][37] + coeff0_45*dmats2[45][37] + coeff0_46*dmats2[46][37] + coeff0_47*dmats2[47][37] + coeff0_48*dmats2[48][37] + coeff0_49*dmats2[49][37] + coeff0_50*dmats2[50][37] + coeff0_51*dmats2[51][37] + coeff0_52*dmats2[52][37] + coeff0_53*dmats2[53][37] + coeff0_54*dmats2[54][37] + coeff0_55*dmats2[55][37];
          new_coeff0_38 = coeff0_0*dmats2[0][38] + coeff0_1*dmats2[1][38] + coeff0_2*dmats2[2][38] + coeff0_3*dmats2[3][38] + coeff0_4*dmats2[4][38] + coeff0_5*dmats2[5][38] + coeff0_6*dmats2[6][38] + coeff0_7*dmats2[7][38] + coeff0_8*dmats2[8][38] + coeff0_9*dmats2[9][38] + coeff0_10*dmats2[10][38] + coeff0_11*dmats2[11][38] + coeff0_12*dmats2[12][38] + coeff0_13*dmats2[13][38] + coeff0_14*dmats2[14][38] + coeff0_15*dmats2[15][38] + coeff0_16*dmats2[16][38] + coeff0_17*dmats2[17][38] + coeff0_18*dmats2[18][38] + coeff0_19*dmats2[19][38] + coeff0_20*dmats2[20][38] + coeff0_21*dmats2[21][38] + coeff0_22*dmats2[22][38] + coeff0_23*dmats2[23][38] + coeff0_24*dmats2[24][38] + coeff0_25*dmats2[25][38] + coeff0_26*dmats2[26][38] + coeff0_27*dmats2[27][38] + coeff0_28*dmats2[28][38] + coeff0_29*dmats2[29][38] + coeff0_30*dmats2[30][38] + coeff0_31*dmats2[31][38] + coeff0_32*dmats2[32][38] + coeff0_33*dmats2[33][38] + coeff0_34*dmats2[34][38] + coeff0_35*dmats2[35][38] + coeff0_36*dmats2[36][38] + coeff0_37*dmats2[37][38] + coeff0_38*dmats2[38][38] + coeff0_39*dmats2[39][38] + coeff0_40*dmats2[40][38] + coeff0_41*dmats2[41][38] + coeff0_42*dmats2[42][38] + coeff0_43*dmats2[43][38] + coeff0_44*dmats2[44][38] + coeff0_45*dmats2[45][38] + coeff0_46*dmats2[46][38] + coeff0_47*dmats2[47][38] + coeff0_48*dmats2[48][38] + coeff0_49*dmats2[49][38] + coeff0_50*dmats2[50][38] + coeff0_51*dmats2[51][38] + coeff0_52*dmats2[52][38] + coeff0_53*dmats2[53][38] + coeff0_54*dmats2[54][38] + coeff0_55*dmats2[55][38];
          new_coeff0_39 = coeff0_0*dmats2[0][39] + coeff0_1*dmats2[1][39] + coeff0_2*dmats2[2][39] + coeff0_3*dmats2[3][39] + coeff0_4*dmats2[4][39] + coeff0_5*dmats2[5][39] + coeff0_6*dmats2[6][39] + coeff0_7*dmats2[7][39] + coeff0_8*dmats2[8][39] + coeff0_9*dmats2[9][39] + coeff0_10*dmats2[10][39] + coeff0_11*dmats2[11][39] + coeff0_12*dmats2[12][39] + coeff0_13*dmats2[13][39] + coeff0_14*dmats2[14][39] + coeff0_15*dmats2[15][39] + coeff0_16*dmats2[16][39] + coeff0_17*dmats2[17][39] + coeff0_18*dmats2[18][39] + coeff0_19*dmats2[19][39] + coeff0_20*dmats2[20][39] + coeff0_21*dmats2[21][39] + coeff0_22*dmats2[22][39] + coeff0_23*dmats2[23][39] + coeff0_24*dmats2[24][39] + coeff0_25*dmats2[25][39] + coeff0_26*dmats2[26][39] + coeff0_27*dmats2[27][39] + coeff0_28*dmats2[28][39] + coeff0_29*dmats2[29][39] + coeff0_30*dmats2[30][39] + coeff0_31*dmats2[31][39] + coeff0_32*dmats2[32][39] + coeff0_33*dmats2[33][39] + coeff0_34*dmats2[34][39] + coeff0_35*dmats2[35][39] + coeff0_36*dmats2[36][39] + coeff0_37*dmats2[37][39] + coeff0_38*dmats2[38][39] + coeff0_39*dmats2[39][39] + coeff0_40*dmats2[40][39] + coeff0_41*dmats2[41][39] + coeff0_42*dmats2[42][39] + coeff0_43*dmats2[43][39] + coeff0_44*dmats2[44][39] + coeff0_45*dmats2[45][39] + coeff0_46*dmats2[46][39] + coeff0_47*dmats2[47][39] + coeff0_48*dmats2[48][39] + coeff0_49*dmats2[49][39] + coeff0_50*dmats2[50][39] + coeff0_51*dmats2[51][39] + coeff0_52*dmats2[52][39] + coeff0_53*dmats2[53][39] + coeff0_54*dmats2[54][39] + coeff0_55*dmats2[55][39];
          new_coeff0_40 = coeff0_0*dmats2[0][40] + coeff0_1*dmats2[1][40] + coeff0_2*dmats2[2][40] + coeff0_3*dmats2[3][40] + coeff0_4*dmats2[4][40] + coeff0_5*dmats2[5][40] + coeff0_6*dmats2[6][40] + coeff0_7*dmats2[7][40] + coeff0_8*dmats2[8][40] + coeff0_9*dmats2[9][40] + coeff0_10*dmats2[10][40] + coeff0_11*dmats2[11][40] + coeff0_12*dmats2[12][40] + coeff0_13*dmats2[13][40] + coeff0_14*dmats2[14][40] + coeff0_15*dmats2[15][40] + coeff0_16*dmats2[16][40] + coeff0_17*dmats2[17][40] + coeff0_18*dmats2[18][40] + coeff0_19*dmats2[19][40] + coeff0_20*dmats2[20][40] + coeff0_21*dmats2[21][40] + coeff0_22*dmats2[22][40] + coeff0_23*dmats2[23][40] + coeff0_24*dmats2[24][40] + coeff0_25*dmats2[25][40] + coeff0_26*dmats2[26][40] + coeff0_27*dmats2[27][40] + coeff0_28*dmats2[28][40] + coeff0_29*dmats2[29][40] + coeff0_30*dmats2[30][40] + coeff0_31*dmats2[31][40] + coeff0_32*dmats2[32][40] + coeff0_33*dmats2[33][40] + coeff0_34*dmats2[34][40] + coeff0_35*dmats2[35][40] + coeff0_36*dmats2[36][40] + coeff0_37*dmats2[37][40] + coeff0_38*dmats2[38][40] + coeff0_39*dmats2[39][40] + coeff0_40*dmats2[40][40] + coeff0_41*dmats2[41][40] + coeff0_42*dmats2[42][40] + coeff0_43*dmats2[43][40] + coeff0_44*dmats2[44][40] + coeff0_45*dmats2[45][40] + coeff0_46*dmats2[46][40] + coeff0_47*dmats2[47][40] + coeff0_48*dmats2[48][40] + coeff0_49*dmats2[49][40] + coeff0_50*dmats2[50][40] + coeff0_51*dmats2[51][40] + coeff0_52*dmats2[52][40] + coeff0_53*dmats2[53][40] + coeff0_54*dmats2[54][40] + coeff0_55*dmats2[55][40];
          new_coeff0_41 = coeff0_0*dmats2[0][41] + coeff0_1*dmats2[1][41] + coeff0_2*dmats2[2][41] + coeff0_3*dmats2[3][41] + coeff0_4*dmats2[4][41] + coeff0_5*dmats2[5][41] + coeff0_6*dmats2[6][41] + coeff0_7*dmats2[7][41] + coeff0_8*dmats2[8][41] + coeff0_9*dmats2[9][41] + coeff0_10*dmats2[10][41] + coeff0_11*dmats2[11][41] + coeff0_12*dmats2[12][41] + coeff0_13*dmats2[13][41] + coeff0_14*dmats2[14][41] + coeff0_15*dmats2[15][41] + coeff0_16*dmats2[16][41] + coeff0_17*dmats2[17][41] + coeff0_18*dmats2[18][41] + coeff0_19*dmats2[19][41] + coeff0_20*dmats2[20][41] + coeff0_21*dmats2[21][41] + coeff0_22*dmats2[22][41] + coeff0_23*dmats2[23][41] + coeff0_24*dmats2[24][41] + coeff0_25*dmats2[25][41] + coeff0_26*dmats2[26][41] + coeff0_27*dmats2[27][41] + coeff0_28*dmats2[28][41] + coeff0_29*dmats2[29][41] + coeff0_30*dmats2[30][41] + coeff0_31*dmats2[31][41] + coeff0_32*dmats2[32][41] + coeff0_33*dmats2[33][41] + coeff0_34*dmats2[34][41] + coeff0_35*dmats2[35][41] + coeff0_36*dmats2[36][41] + coeff0_37*dmats2[37][41] + coeff0_38*dmats2[38][41] + coeff0_39*dmats2[39][41] + coeff0_40*dmats2[40][41] + coeff0_41*dmats2[41][41] + coeff0_42*dmats2[42][41] + coeff0_43*dmats2[43][41] + coeff0_44*dmats2[44][41] + coeff0_45*dmats2[45][41] + coeff0_46*dmats2[46][41] + coeff0_47*dmats2[47][41] + coeff0_48*dmats2[48][41] + coeff0_49*dmats2[49][41] + coeff0_50*dmats2[50][41] + coeff0_51*dmats2[51][41] + coeff0_52*dmats2[52][41] + coeff0_53*dmats2[53][41] + coeff0_54*dmats2[54][41] + coeff0_55*dmats2[55][41];
          new_coeff0_42 = coeff0_0*dmats2[0][42] + coeff0_1*dmats2[1][42] + coeff0_2*dmats2[2][42] + coeff0_3*dmats2[3][42] + coeff0_4*dmats2[4][42] + coeff0_5*dmats2[5][42] + coeff0_6*dmats2[6][42] + coeff0_7*dmats2[7][42] + coeff0_8*dmats2[8][42] + coeff0_9*dmats2[9][42] + coeff0_10*dmats2[10][42] + coeff0_11*dmats2[11][42] + coeff0_12*dmats2[12][42] + coeff0_13*dmats2[13][42] + coeff0_14*dmats2[14][42] + coeff0_15*dmats2[15][42] + coeff0_16*dmats2[16][42] + coeff0_17*dmats2[17][42] + coeff0_18*dmats2[18][42] + coeff0_19*dmats2[19][42] + coeff0_20*dmats2[20][42] + coeff0_21*dmats2[21][42] + coeff0_22*dmats2[22][42] + coeff0_23*dmats2[23][42] + coeff0_24*dmats2[24][42] + coeff0_25*dmats2[25][42] + coeff0_26*dmats2[26][42] + coeff0_27*dmats2[27][42] + coeff0_28*dmats2[28][42] + coeff0_29*dmats2[29][42] + coeff0_30*dmats2[30][42] + coeff0_31*dmats2[31][42] + coeff0_32*dmats2[32][42] + coeff0_33*dmats2[33][42] + coeff0_34*dmats2[34][42] + coeff0_35*dmats2[35][42] + coeff0_36*dmats2[36][42] + coeff0_37*dmats2[37][42] + coeff0_38*dmats2[38][42] + coeff0_39*dmats2[39][42] + coeff0_40*dmats2[40][42] + coeff0_41*dmats2[41][42] + coeff0_42*dmats2[42][42] + coeff0_43*dmats2[43][42] + coeff0_44*dmats2[44][42] + coeff0_45*dmats2[45][42] + coeff0_46*dmats2[46][42] + coeff0_47*dmats2[47][42] + coeff0_48*dmats2[48][42] + coeff0_49*dmats2[49][42] + coeff0_50*dmats2[50][42] + coeff0_51*dmats2[51][42] + coeff0_52*dmats2[52][42] + coeff0_53*dmats2[53][42] + coeff0_54*dmats2[54][42] + coeff0_55*dmats2[55][42];
          new_coeff0_43 = coeff0_0*dmats2[0][43] + coeff0_1*dmats2[1][43] + coeff0_2*dmats2[2][43] + coeff0_3*dmats2[3][43] + coeff0_4*dmats2[4][43] + coeff0_5*dmats2[5][43] + coeff0_6*dmats2[6][43] + coeff0_7*dmats2[7][43] + coeff0_8*dmats2[8][43] + coeff0_9*dmats2[9][43] + coeff0_10*dmats2[10][43] + coeff0_11*dmats2[11][43] + coeff0_12*dmats2[12][43] + coeff0_13*dmats2[13][43] + coeff0_14*dmats2[14][43] + coeff0_15*dmats2[15][43] + coeff0_16*dmats2[16][43] + coeff0_17*dmats2[17][43] + coeff0_18*dmats2[18][43] + coeff0_19*dmats2[19][43] + coeff0_20*dmats2[20][43] + coeff0_21*dmats2[21][43] + coeff0_22*dmats2[22][43] + coeff0_23*dmats2[23][43] + coeff0_24*dmats2[24][43] + coeff0_25*dmats2[25][43] + coeff0_26*dmats2[26][43] + coeff0_27*dmats2[27][43] + coeff0_28*dmats2[28][43] + coeff0_29*dmats2[29][43] + coeff0_30*dmats2[30][43] + coeff0_31*dmats2[31][43] + coeff0_32*dmats2[32][43] + coeff0_33*dmats2[33][43] + coeff0_34*dmats2[34][43] + coeff0_35*dmats2[35][43] + coeff0_36*dmats2[36][43] + coeff0_37*dmats2[37][43] + coeff0_38*dmats2[38][43] + coeff0_39*dmats2[39][43] + coeff0_40*dmats2[40][43] + coeff0_41*dmats2[41][43] + coeff0_42*dmats2[42][43] + coeff0_43*dmats2[43][43] + coeff0_44*dmats2[44][43] + coeff0_45*dmats2[45][43] + coeff0_46*dmats2[46][43] + coeff0_47*dmats2[47][43] + coeff0_48*dmats2[48][43] + coeff0_49*dmats2[49][43] + coeff0_50*dmats2[50][43] + coeff0_51*dmats2[51][43] + coeff0_52*dmats2[52][43] + coeff0_53*dmats2[53][43] + coeff0_54*dmats2[54][43] + coeff0_55*dmats2[55][43];
          new_coeff0_44 = coeff0_0*dmats2[0][44] + coeff0_1*dmats2[1][44] + coeff0_2*dmats2[2][44] + coeff0_3*dmats2[3][44] + coeff0_4*dmats2[4][44] + coeff0_5*dmats2[5][44] + coeff0_6*dmats2[6][44] + coeff0_7*dmats2[7][44] + coeff0_8*dmats2[8][44] + coeff0_9*dmats2[9][44] + coeff0_10*dmats2[10][44] + coeff0_11*dmats2[11][44] + coeff0_12*dmats2[12][44] + coeff0_13*dmats2[13][44] + coeff0_14*dmats2[14][44] + coeff0_15*dmats2[15][44] + coeff0_16*dmats2[16][44] + coeff0_17*dmats2[17][44] + coeff0_18*dmats2[18][44] + coeff0_19*dmats2[19][44] + coeff0_20*dmats2[20][44] + coeff0_21*dmats2[21][44] + coeff0_22*dmats2[22][44] + coeff0_23*dmats2[23][44] + coeff0_24*dmats2[24][44] + coeff0_25*dmats2[25][44] + coeff0_26*dmats2[26][44] + coeff0_27*dmats2[27][44] + coeff0_28*dmats2[28][44] + coeff0_29*dmats2[29][44] + coeff0_30*dmats2[30][44] + coeff0_31*dmats2[31][44] + coeff0_32*dmats2[32][44] + coeff0_33*dmats2[33][44] + coeff0_34*dmats2[34][44] + coeff0_35*dmats2[35][44] + coeff0_36*dmats2[36][44] + coeff0_37*dmats2[37][44] + coeff0_38*dmats2[38][44] + coeff0_39*dmats2[39][44] + coeff0_40*dmats2[40][44] + coeff0_41*dmats2[41][44] + coeff0_42*dmats2[42][44] + coeff0_43*dmats2[43][44] + coeff0_44*dmats2[44][44] + coeff0_45*dmats2[45][44] + coeff0_46*dmats2[46][44] + coeff0_47*dmats2[47][44] + coeff0_48*dmats2[48][44] + coeff0_49*dmats2[49][44] + coeff0_50*dmats2[50][44] + coeff0_51*dmats2[51][44] + coeff0_52*dmats2[52][44] + coeff0_53*dmats2[53][44] + coeff0_54*dmats2[54][44] + coeff0_55*dmats2[55][44];
          new_coeff0_45 = coeff0_0*dmats2[0][45] + coeff0_1*dmats2[1][45] + coeff0_2*dmats2[2][45] + coeff0_3*dmats2[3][45] + coeff0_4*dmats2[4][45] + coeff0_5*dmats2[5][45] + coeff0_6*dmats2[6][45] + coeff0_7*dmats2[7][45] + coeff0_8*dmats2[8][45] + coeff0_9*dmats2[9][45] + coeff0_10*dmats2[10][45] + coeff0_11*dmats2[11][45] + coeff0_12*dmats2[12][45] + coeff0_13*dmats2[13][45] + coeff0_14*dmats2[14][45] + coeff0_15*dmats2[15][45] + coeff0_16*dmats2[16][45] + coeff0_17*dmats2[17][45] + coeff0_18*dmats2[18][45] + coeff0_19*dmats2[19][45] + coeff0_20*dmats2[20][45] + coeff0_21*dmats2[21][45] + coeff0_22*dmats2[22][45] + coeff0_23*dmats2[23][45] + coeff0_24*dmats2[24][45] + coeff0_25*dmats2[25][45] + coeff0_26*dmats2[26][45] + coeff0_27*dmats2[27][45] + coeff0_28*dmats2[28][45] + coeff0_29*dmats2[29][45] + coeff0_30*dmats2[30][45] + coeff0_31*dmats2[31][45] + coeff0_32*dmats2[32][45] + coeff0_33*dmats2[33][45] + coeff0_34*dmats2[34][45] + coeff0_35*dmats2[35][45] + coeff0_36*dmats2[36][45] + coeff0_37*dmats2[37][45] + coeff0_38*dmats2[38][45] + coeff0_39*dmats2[39][45] + coeff0_40*dmats2[40][45] + coeff0_41*dmats2[41][45] + coeff0_42*dmats2[42][45] + coeff0_43*dmats2[43][45] + coeff0_44*dmats2[44][45] + coeff0_45*dmats2[45][45] + coeff0_46*dmats2[46][45] + coeff0_47*dmats2[47][45] + coeff0_48*dmats2[48][45] + coeff0_49*dmats2[49][45] + coeff0_50*dmats2[50][45] + coeff0_51*dmats2[51][45] + coeff0_52*dmats2[52][45] + coeff0_53*dmats2[53][45] + coeff0_54*dmats2[54][45] + coeff0_55*dmats2[55][45];
          new_coeff0_46 = coeff0_0*dmats2[0][46] + coeff0_1*dmats2[1][46] + coeff0_2*dmats2[2][46] + coeff0_3*dmats2[3][46] + coeff0_4*dmats2[4][46] + coeff0_5*dmats2[5][46] + coeff0_6*dmats2[6][46] + coeff0_7*dmats2[7][46] + coeff0_8*dmats2[8][46] + coeff0_9*dmats2[9][46] + coeff0_10*dmats2[10][46] + coeff0_11*dmats2[11][46] + coeff0_12*dmats2[12][46] + coeff0_13*dmats2[13][46] + coeff0_14*dmats2[14][46] + coeff0_15*dmats2[15][46] + coeff0_16*dmats2[16][46] + coeff0_17*dmats2[17][46] + coeff0_18*dmats2[18][46] + coeff0_19*dmats2[19][46] + coeff0_20*dmats2[20][46] + coeff0_21*dmats2[21][46] + coeff0_22*dmats2[22][46] + coeff0_23*dmats2[23][46] + coeff0_24*dmats2[24][46] + coeff0_25*dmats2[25][46] + coeff0_26*dmats2[26][46] + coeff0_27*dmats2[27][46] + coeff0_28*dmats2[28][46] + coeff0_29*dmats2[29][46] + coeff0_30*dmats2[30][46] + coeff0_31*dmats2[31][46] + coeff0_32*dmats2[32][46] + coeff0_33*dmats2[33][46] + coeff0_34*dmats2[34][46] + coeff0_35*dmats2[35][46] + coeff0_36*dmats2[36][46] + coeff0_37*dmats2[37][46] + coeff0_38*dmats2[38][46] + coeff0_39*dmats2[39][46] + coeff0_40*dmats2[40][46] + coeff0_41*dmats2[41][46] + coeff0_42*dmats2[42][46] + coeff0_43*dmats2[43][46] + coeff0_44*dmats2[44][46] + coeff0_45*dmats2[45][46] + coeff0_46*dmats2[46][46] + coeff0_47*dmats2[47][46] + coeff0_48*dmats2[48][46] + coeff0_49*dmats2[49][46] + coeff0_50*dmats2[50][46] + coeff0_51*dmats2[51][46] + coeff0_52*dmats2[52][46] + coeff0_53*dmats2[53][46] + coeff0_54*dmats2[54][46] + coeff0_55*dmats2[55][46];
          new_coeff0_47 = coeff0_0*dmats2[0][47] + coeff0_1*dmats2[1][47] + coeff0_2*dmats2[2][47] + coeff0_3*dmats2[3][47] + coeff0_4*dmats2[4][47] + coeff0_5*dmats2[5][47] + coeff0_6*dmats2[6][47] + coeff0_7*dmats2[7][47] + coeff0_8*dmats2[8][47] + coeff0_9*dmats2[9][47] + coeff0_10*dmats2[10][47] + coeff0_11*dmats2[11][47] + coeff0_12*dmats2[12][47] + coeff0_13*dmats2[13][47] + coeff0_14*dmats2[14][47] + coeff0_15*dmats2[15][47] + coeff0_16*dmats2[16][47] + coeff0_17*dmats2[17][47] + coeff0_18*dmats2[18][47] + coeff0_19*dmats2[19][47] + coeff0_20*dmats2[20][47] + coeff0_21*dmats2[21][47] + coeff0_22*dmats2[22][47] + coeff0_23*dmats2[23][47] + coeff0_24*dmats2[24][47] + coeff0_25*dmats2[25][47] + coeff0_26*dmats2[26][47] + coeff0_27*dmats2[27][47] + coeff0_28*dmats2[28][47] + coeff0_29*dmats2[29][47] + coeff0_30*dmats2[30][47] + coeff0_31*dmats2[31][47] + coeff0_32*dmats2[32][47] + coeff0_33*dmats2[33][47] + coeff0_34*dmats2[34][47] + coeff0_35*dmats2[35][47] + coeff0_36*dmats2[36][47] + coeff0_37*dmats2[37][47] + coeff0_38*dmats2[38][47] + coeff0_39*dmats2[39][47] + coeff0_40*dmats2[40][47] + coeff0_41*dmats2[41][47] + coeff0_42*dmats2[42][47] + coeff0_43*dmats2[43][47] + coeff0_44*dmats2[44][47] + coeff0_45*dmats2[45][47] + coeff0_46*dmats2[46][47] + coeff0_47*dmats2[47][47] + coeff0_48*dmats2[48][47] + coeff0_49*dmats2[49][47] + coeff0_50*dmats2[50][47] + coeff0_51*dmats2[51][47] + coeff0_52*dmats2[52][47] + coeff0_53*dmats2[53][47] + coeff0_54*dmats2[54][47] + coeff0_55*dmats2[55][47];
          new_coeff0_48 = coeff0_0*dmats2[0][48] + coeff0_1*dmats2[1][48] + coeff0_2*dmats2[2][48] + coeff0_3*dmats2[3][48] + coeff0_4*dmats2[4][48] + coeff0_5*dmats2[5][48] + coeff0_6*dmats2[6][48] + coeff0_7*dmats2[7][48] + coeff0_8*dmats2[8][48] + coeff0_9*dmats2[9][48] + coeff0_10*dmats2[10][48] + coeff0_11*dmats2[11][48] + coeff0_12*dmats2[12][48] + coeff0_13*dmats2[13][48] + coeff0_14*dmats2[14][48] + coeff0_15*dmats2[15][48] + coeff0_16*dmats2[16][48] + coeff0_17*dmats2[17][48] + coeff0_18*dmats2[18][48] + coeff0_19*dmats2[19][48] + coeff0_20*dmats2[20][48] + coeff0_21*dmats2[21][48] + coeff0_22*dmats2[22][48] + coeff0_23*dmats2[23][48] + coeff0_24*dmats2[24][48] + coeff0_25*dmats2[25][48] + coeff0_26*dmats2[26][48] + coeff0_27*dmats2[27][48] + coeff0_28*dmats2[28][48] + coeff0_29*dmats2[29][48] + coeff0_30*dmats2[30][48] + coeff0_31*dmats2[31][48] + coeff0_32*dmats2[32][48] + coeff0_33*dmats2[33][48] + coeff0_34*dmats2[34][48] + coeff0_35*dmats2[35][48] + coeff0_36*dmats2[36][48] + coeff0_37*dmats2[37][48] + coeff0_38*dmats2[38][48] + coeff0_39*dmats2[39][48] + coeff0_40*dmats2[40][48] + coeff0_41*dmats2[41][48] + coeff0_42*dmats2[42][48] + coeff0_43*dmats2[43][48] + coeff0_44*dmats2[44][48] + coeff0_45*dmats2[45][48] + coeff0_46*dmats2[46][48] + coeff0_47*dmats2[47][48] + coeff0_48*dmats2[48][48] + coeff0_49*dmats2[49][48] + coeff0_50*dmats2[50][48] + coeff0_51*dmats2[51][48] + coeff0_52*dmats2[52][48] + coeff0_53*dmats2[53][48] + coeff0_54*dmats2[54][48] + coeff0_55*dmats2[55][48];
          new_coeff0_49 = coeff0_0*dmats2[0][49] + coeff0_1*dmats2[1][49] + coeff0_2*dmats2[2][49] + coeff0_3*dmats2[3][49] + coeff0_4*dmats2[4][49] + coeff0_5*dmats2[5][49] + coeff0_6*dmats2[6][49] + coeff0_7*dmats2[7][49] + coeff0_8*dmats2[8][49] + coeff0_9*dmats2[9][49] + coeff0_10*dmats2[10][49] + coeff0_11*dmats2[11][49] + coeff0_12*dmats2[12][49] + coeff0_13*dmats2[13][49] + coeff0_14*dmats2[14][49] + coeff0_15*dmats2[15][49] + coeff0_16*dmats2[16][49] + coeff0_17*dmats2[17][49] + coeff0_18*dmats2[18][49] + coeff0_19*dmats2[19][49] + coeff0_20*dmats2[20][49] + coeff0_21*dmats2[21][49] + coeff0_22*dmats2[22][49] + coeff0_23*dmats2[23][49] + coeff0_24*dmats2[24][49] + coeff0_25*dmats2[25][49] + coeff0_26*dmats2[26][49] + coeff0_27*dmats2[27][49] + coeff0_28*dmats2[28][49] + coeff0_29*dmats2[29][49] + coeff0_30*dmats2[30][49] + coeff0_31*dmats2[31][49] + coeff0_32*dmats2[32][49] + coeff0_33*dmats2[33][49] + coeff0_34*dmats2[34][49] + coeff0_35*dmats2[35][49] + coeff0_36*dmats2[36][49] + coeff0_37*dmats2[37][49] + coeff0_38*dmats2[38][49] + coeff0_39*dmats2[39][49] + coeff0_40*dmats2[40][49] + coeff0_41*dmats2[41][49] + coeff0_42*dmats2[42][49] + coeff0_43*dmats2[43][49] + coeff0_44*dmats2[44][49] + coeff0_45*dmats2[45][49] + coeff0_46*dmats2[46][49] + coeff0_47*dmats2[47][49] + coeff0_48*dmats2[48][49] + coeff0_49*dmats2[49][49] + coeff0_50*dmats2[50][49] + coeff0_51*dmats2[51][49] + coeff0_52*dmats2[52][49] + coeff0_53*dmats2[53][49] + coeff0_54*dmats2[54][49] + coeff0_55*dmats2[55][49];
          new_coeff0_50 = coeff0_0*dmats2[0][50] + coeff0_1*dmats2[1][50] + coeff0_2*dmats2[2][50] + coeff0_3*dmats2[3][50] + coeff0_4*dmats2[4][50] + coeff0_5*dmats2[5][50] + coeff0_6*dmats2[6][50] + coeff0_7*dmats2[7][50] + coeff0_8*dmats2[8][50] + coeff0_9*dmats2[9][50] + coeff0_10*dmats2[10][50] + coeff0_11*dmats2[11][50] + coeff0_12*dmats2[12][50] + coeff0_13*dmats2[13][50] + coeff0_14*dmats2[14][50] + coeff0_15*dmats2[15][50] + coeff0_16*dmats2[16][50] + coeff0_17*dmats2[17][50] + coeff0_18*dmats2[18][50] + coeff0_19*dmats2[19][50] + coeff0_20*dmats2[20][50] + coeff0_21*dmats2[21][50] + coeff0_22*dmats2[22][50] + coeff0_23*dmats2[23][50] + coeff0_24*dmats2[24][50] + coeff0_25*dmats2[25][50] + coeff0_26*dmats2[26][50] + coeff0_27*dmats2[27][50] + coeff0_28*dmats2[28][50] + coeff0_29*dmats2[29][50] + coeff0_30*dmats2[30][50] + coeff0_31*dmats2[31][50] + coeff0_32*dmats2[32][50] + coeff0_33*dmats2[33][50] + coeff0_34*dmats2[34][50] + coeff0_35*dmats2[35][50] + coeff0_36*dmats2[36][50] + coeff0_37*dmats2[37][50] + coeff0_38*dmats2[38][50] + coeff0_39*dmats2[39][50] + coeff0_40*dmats2[40][50] + coeff0_41*dmats2[41][50] + coeff0_42*dmats2[42][50] + coeff0_43*dmats2[43][50] + coeff0_44*dmats2[44][50] + coeff0_45*dmats2[45][50] + coeff0_46*dmats2[46][50] + coeff0_47*dmats2[47][50] + coeff0_48*dmats2[48][50] + coeff0_49*dmats2[49][50] + coeff0_50*dmats2[50][50] + coeff0_51*dmats2[51][50] + coeff0_52*dmats2[52][50] + coeff0_53*dmats2[53][50] + coeff0_54*dmats2[54][50] + coeff0_55*dmats2[55][50];
          new_coeff0_51 = coeff0_0*dmats2[0][51] + coeff0_1*dmats2[1][51] + coeff0_2*dmats2[2][51] + coeff0_3*dmats2[3][51] + coeff0_4*dmats2[4][51] + coeff0_5*dmats2[5][51] + coeff0_6*dmats2[6][51] + coeff0_7*dmats2[7][51] + coeff0_8*dmats2[8][51] + coeff0_9*dmats2[9][51] + coeff0_10*dmats2[10][51] + coeff0_11*dmats2[11][51] + coeff0_12*dmats2[12][51] + coeff0_13*dmats2[13][51] + coeff0_14*dmats2[14][51] + coeff0_15*dmats2[15][51] + coeff0_16*dmats2[16][51] + coeff0_17*dmats2[17][51] + coeff0_18*dmats2[18][51] + coeff0_19*dmats2[19][51] + coeff0_20*dmats2[20][51] + coeff0_21*dmats2[21][51] + coeff0_22*dmats2[22][51] + coeff0_23*dmats2[23][51] + coeff0_24*dmats2[24][51] + coeff0_25*dmats2[25][51] + coeff0_26*dmats2[26][51] + coeff0_27*dmats2[27][51] + coeff0_28*dmats2[28][51] + coeff0_29*dmats2[29][51] + coeff0_30*dmats2[30][51] + coeff0_31*dmats2[31][51] + coeff0_32*dmats2[32][51] + coeff0_33*dmats2[33][51] + coeff0_34*dmats2[34][51] + coeff0_35*dmats2[35][51] + coeff0_36*dmats2[36][51] + coeff0_37*dmats2[37][51] + coeff0_38*dmats2[38][51] + coeff0_39*dmats2[39][51] + coeff0_40*dmats2[40][51] + coeff0_41*dmats2[41][51] + coeff0_42*dmats2[42][51] + coeff0_43*dmats2[43][51] + coeff0_44*dmats2[44][51] + coeff0_45*dmats2[45][51] + coeff0_46*dmats2[46][51] + coeff0_47*dmats2[47][51] + coeff0_48*dmats2[48][51] + coeff0_49*dmats2[49][51] + coeff0_50*dmats2[50][51] + coeff0_51*dmats2[51][51] + coeff0_52*dmats2[52][51] + coeff0_53*dmats2[53][51] + coeff0_54*dmats2[54][51] + coeff0_55*dmats2[55][51];
          new_coeff0_52 = coeff0_0*dmats2[0][52] + coeff0_1*dmats2[1][52] + coeff0_2*dmats2[2][52] + coeff0_3*dmats2[3][52] + coeff0_4*dmats2[4][52] + coeff0_5*dmats2[5][52] + coeff0_6*dmats2[6][52] + coeff0_7*dmats2[7][52] + coeff0_8*dmats2[8][52] + coeff0_9*dmats2[9][52] + coeff0_10*dmats2[10][52] + coeff0_11*dmats2[11][52] + coeff0_12*dmats2[12][52] + coeff0_13*dmats2[13][52] + coeff0_14*dmats2[14][52] + coeff0_15*dmats2[15][52] + coeff0_16*dmats2[16][52] + coeff0_17*dmats2[17][52] + coeff0_18*dmats2[18][52] + coeff0_19*dmats2[19][52] + coeff0_20*dmats2[20][52] + coeff0_21*dmats2[21][52] + coeff0_22*dmats2[22][52] + coeff0_23*dmats2[23][52] + coeff0_24*dmats2[24][52] + coeff0_25*dmats2[25][52] + coeff0_26*dmats2[26][52] + coeff0_27*dmats2[27][52] + coeff0_28*dmats2[28][52] + coeff0_29*dmats2[29][52] + coeff0_30*dmats2[30][52] + coeff0_31*dmats2[31][52] + coeff0_32*dmats2[32][52] + coeff0_33*dmats2[33][52] + coeff0_34*dmats2[34][52] + coeff0_35*dmats2[35][52] + coeff0_36*dmats2[36][52] + coeff0_37*dmats2[37][52] + coeff0_38*dmats2[38][52] + coeff0_39*dmats2[39][52] + coeff0_40*dmats2[40][52] + coeff0_41*dmats2[41][52] + coeff0_42*dmats2[42][52] + coeff0_43*dmats2[43][52] + coeff0_44*dmats2[44][52] + coeff0_45*dmats2[45][52] + coeff0_46*dmats2[46][52] + coeff0_47*dmats2[47][52] + coeff0_48*dmats2[48][52] + coeff0_49*dmats2[49][52] + coeff0_50*dmats2[50][52] + coeff0_51*dmats2[51][52] + coeff0_52*dmats2[52][52] + coeff0_53*dmats2[53][52] + coeff0_54*dmats2[54][52] + coeff0_55*dmats2[55][52];
          new_coeff0_53 = coeff0_0*dmats2[0][53] + coeff0_1*dmats2[1][53] + coeff0_2*dmats2[2][53] + coeff0_3*dmats2[3][53] + coeff0_4*dmats2[4][53] + coeff0_5*dmats2[5][53] + coeff0_6*dmats2[6][53] + coeff0_7*dmats2[7][53] + coeff0_8*dmats2[8][53] + coeff0_9*dmats2[9][53] + coeff0_10*dmats2[10][53] + coeff0_11*dmats2[11][53] + coeff0_12*dmats2[12][53] + coeff0_13*dmats2[13][53] + coeff0_14*dmats2[14][53] + coeff0_15*dmats2[15][53] + coeff0_16*dmats2[16][53] + coeff0_17*dmats2[17][53] + coeff0_18*dmats2[18][53] + coeff0_19*dmats2[19][53] + coeff0_20*dmats2[20][53] + coeff0_21*dmats2[21][53] + coeff0_22*dmats2[22][53] + coeff0_23*dmats2[23][53] + coeff0_24*dmats2[24][53] + coeff0_25*dmats2[25][53] + coeff0_26*dmats2[26][53] + coeff0_27*dmats2[27][53] + coeff0_28*dmats2[28][53] + coeff0_29*dmats2[29][53] + coeff0_30*dmats2[30][53] + coeff0_31*dmats2[31][53] + coeff0_32*dmats2[32][53] + coeff0_33*dmats2[33][53] + coeff0_34*dmats2[34][53] + coeff0_35*dmats2[35][53] + coeff0_36*dmats2[36][53] + coeff0_37*dmats2[37][53] + coeff0_38*dmats2[38][53] + coeff0_39*dmats2[39][53] + coeff0_40*dmats2[40][53] + coeff0_41*dmats2[41][53] + coeff0_42*dmats2[42][53] + coeff0_43*dmats2[43][53] + coeff0_44*dmats2[44][53] + coeff0_45*dmats2[45][53] + coeff0_46*dmats2[46][53] + coeff0_47*dmats2[47][53] + coeff0_48*dmats2[48][53] + coeff0_49*dmats2[49][53] + coeff0_50*dmats2[50][53] + coeff0_51*dmats2[51][53] + coeff0_52*dmats2[52][53] + coeff0_53*dmats2[53][53] + coeff0_54*dmats2[54][53] + coeff0_55*dmats2[55][53];
          new_coeff0_54 = coeff0_0*dmats2[0][54] + coeff0_1*dmats2[1][54] + coeff0_2*dmats2[2][54] + coeff0_3*dmats2[3][54] + coeff0_4*dmats2[4][54] + coeff0_5*dmats2[5][54] + coeff0_6*dmats2[6][54] + coeff0_7*dmats2[7][54] + coeff0_8*dmats2[8][54] + coeff0_9*dmats2[9][54] + coeff0_10*dmats2[10][54] + coeff0_11*dmats2[11][54] + coeff0_12*dmats2[12][54] + coeff0_13*dmats2[13][54] + coeff0_14*dmats2[14][54] + coeff0_15*dmats2[15][54] + coeff0_16*dmats2[16][54] + coeff0_17*dmats2[17][54] + coeff0_18*dmats2[18][54] + coeff0_19*dmats2[19][54] + coeff0_20*dmats2[20][54] + coeff0_21*dmats2[21][54] + coeff0_22*dmats2[22][54] + coeff0_23*dmats2[23][54] + coeff0_24*dmats2[24][54] + coeff0_25*dmats2[25][54] + coeff0_26*dmats2[26][54] + coeff0_27*dmats2[27][54] + coeff0_28*dmats2[28][54] + coeff0_29*dmats2[29][54] + coeff0_30*dmats2[30][54] + coeff0_31*dmats2[31][54] + coeff0_32*dmats2[32][54] + coeff0_33*dmats2[33][54] + coeff0_34*dmats2[34][54] + coeff0_35*dmats2[35][54] + coeff0_36*dmats2[36][54] + coeff0_37*dmats2[37][54] + coeff0_38*dmats2[38][54] + coeff0_39*dmats2[39][54] + coeff0_40*dmats2[40][54] + coeff0_41*dmats2[41][54] + coeff0_42*dmats2[42][54] + coeff0_43*dmats2[43][54] + coeff0_44*dmats2[44][54] + coeff0_45*dmats2[45][54] + coeff0_46*dmats2[46][54] + coeff0_47*dmats2[47][54] + coeff0_48*dmats2[48][54] + coeff0_49*dmats2[49][54] + coeff0_50*dmats2[50][54] + coeff0_51*dmats2[51][54] + coeff0_52*dmats2[52][54] + coeff0_53*dmats2[53][54] + coeff0_54*dmats2[54][54] + coeff0_55*dmats2[55][54];
          new_coeff0_55 = coeff0_0*dmats2[0][55] + coeff0_1*dmats2[1][55] + coeff0_2*dmats2[2][55] + coeff0_3*dmats2[3][55] + coeff0_4*dmats2[4][55] + coeff0_5*dmats2[5][55] + coeff0_6*dmats2[6][55] + coeff0_7*dmats2[7][55] + coeff0_8*dmats2[8][55] + coeff0_9*dmats2[9][55] + coeff0_10*dmats2[10][55] + coeff0_11*dmats2[11][55] + coeff0_12*dmats2[12][55] + coeff0_13*dmats2[13][55] + coeff0_14*dmats2[14][55] + coeff0_15*dmats2[15][55] + coeff0_16*dmats2[16][55] + coeff0_17*dmats2[17][55] + coeff0_18*dmats2[18][55] + coeff0_19*dmats2[19][55] + coeff0_20*dmats2[20][55] + coeff0_21*dmats2[21][55] + coeff0_22*dmats2[22][55] + coeff0_23*dmats2[23][55] + coeff0_24*dmats2[24][55] + coeff0_25*dmats2[25][55] + coeff0_26*dmats2[26][55] + coeff0_27*dmats2[27][55] + coeff0_28*dmats2[28][55] + coeff0_29*dmats2[29][55] + coeff0_30*dmats2[30][55] + coeff0_31*dmats2[31][55] + coeff0_32*dmats2[32][55] + coeff0_33*dmats2[33][55] + coeff0_34*dmats2[34][55] + coeff0_35*dmats2[35][55] + coeff0_36*dmats2[36][55] + coeff0_37*dmats2[37][55] + coeff0_38*dmats2[38][55] + coeff0_39*dmats2[39][55] + coeff0_40*dmats2[40][55] + coeff0_41*dmats2[41][55] + coeff0_42*dmats2[42][55] + coeff0_43*dmats2[43][55] + coeff0_44*dmats2[44][55] + coeff0_45*dmats2[45][55] + coeff0_46*dmats2[46][55] + coeff0_47*dmats2[47][55] + coeff0_48*dmats2[48][55] + coeff0_49*dmats2[49][55] + coeff0_50*dmats2[50][55] + coeff0_51*dmats2[51][55] + coeff0_52*dmats2[52][55] + coeff0_53*dmats2[53][55] + coeff0_54*dmats2[54][55] + coeff0_55*dmats2[55][55];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9 + new_coeff0_10*basisvalue10 + new_coeff0_11*basisvalue11 + new_coeff0_12*basisvalue12 + new_coeff0_13*basisvalue13 + new_coeff0_14*basisvalue14 + new_coeff0_15*basisvalue15 + new_coeff0_16*basisvalue16 + new_coeff0_17*basisvalue17 + new_coeff0_18*basisvalue18 + new_coeff0_19*basisvalue19 + new_coeff0_20*basisvalue20 + new_coeff0_21*basisvalue21 + new_coeff0_22*basisvalue22 + new_coeff0_23*basisvalue23 + new_coeff0_24*basisvalue24 + new_coeff0_25*basisvalue25 + new_coeff0_26*basisvalue26 + new_coeff0_27*basisvalue27 + new_coeff0_28*basisvalue28 + new_coeff0_29*basisvalue29 + new_coeff0_30*basisvalue30 + new_coeff0_31*basisvalue31 + new_coeff0_32*basisvalue32 + new_coeff0_33*basisvalue33 + new_coeff0_34*basisvalue34 + new_coeff0_35*basisvalue35 + new_coeff0_36*basisvalue36 + new_coeff0_37*basisvalue37 + new_coeff0_38*basisvalue38 + new_coeff0_39*basisvalue39 + new_coeff0_40*basisvalue40 + new_coeff0_41*basisvalue41 + new_coeff0_42*basisvalue42 + new_coeff0_43*basisvalue43 + new_coeff0_44*basisvalue44 + new_coeff0_45*basisvalue45 + new_coeff0_46*basisvalue46 + new_coeff0_47*basisvalue47 + new_coeff0_48*basisvalue48 + new_coeff0_49*basisvalue49 + new_coeff0_50*basisvalue50 + new_coeff0_51*basisvalue51 + new_coeff0_52*basisvalue52 + new_coeff0_53*basisvalue53 + new_coeff0_54*basisvalue54 + new_coeff0_55*basisvalue55;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    static const double X[56][1][3] = {{{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}, {{0, 0.8, 0.2}}, {{0, 0.6, 0.4}}, {{0, 0.4, 0.6}}, {{0, 0.2, 0.8}}, {{0.8, 0, 0.2}}, {{0.6, 0, 0.4}}, {{0.4, 0, 0.6}}, {{0.2, 0, 0.8}}, {{0.8, 0.2, 0}}, {{0.6, 0.4, 0}}, {{0.4, 0.6, 0}}, {{0.2, 0.8, 0}}, {{0, 0, 0.2}}, {{0, 0, 0.4}}, {{0, 0, 0.6}}, {{0, 0, 0.8}}, {{0, 0.2, 0}}, {{0, 0.4, 0}}, {{0, 0.6, 0}}, {{0, 0.8, 0}}, {{0.2, 0, 0}}, {{0.4, 0, 0}}, {{0.6, 0, 0}}, {{0.8, 0, 0}}, {{0.6, 0.2, 0.2}}, {{0.4, 0.4, 0.2}}, {{0.2, 0.6, 0.2}}, {{0.4, 0.2, 0.4}}, {{0.2, 0.4, 0.4}}, {{0.2, 0.2, 0.6}}, {{0, 0.2, 0.2}}, {{0, 0.4, 0.2}}, {{0, 0.6, 0.2}}, {{0, 0.2, 0.4}}, {{0, 0.4, 0.4}}, {{0, 0.2, 0.6}}, {{0.2, 0, 0.2}}, {{0.4, 0, 0.2}}, {{0.6, 0, 0.2}}, {{0.2, 0, 0.4}}, {{0.4, 0, 0.4}}, {{0.2, 0, 0.6}}, {{0.2, 0.2, 0}}, {{0.4, 0.2, 0}}, {{0.6, 0.2, 0}}, {{0.2, 0.4, 0}}, {{0.4, 0.4, 0}}, {{0.2, 0.6, 0}}, {{0.2, 0.2, 0.2}}, {{0.4, 0.2, 0.2}}, {{0.2, 0.4, 0.2}}, {{0.2, 0.2, 0.4}}};
    static const double W[56][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    static const double D[56][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new poisson3d_5_0_finite_element_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class poisson3d_5_0_dof_map_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  poisson3d_5_0_dof_map_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~poisson3d_5_0_dof_map_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for FiniteElement('Lagrange', 'tetrahedron', 5)";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return true;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + 4*m.num_entities[1] + 6*m.num_entities[2] + 4*m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space for a cell
  virtual unsigned int local_dimension(const ufc::cell& c) const
  {
    return 56;
  }

  /// Return the maximum dimension of the local finite element function space
  virtual unsigned int max_local_dimension() const
  {
    return 56;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    return 3;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 21;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + 4*c.entity_indices[1][0];
    dofs[5] = offset + 4*c.entity_indices[1][0] + 1;
    dofs[6] = offset + 4*c.entity_indices[1][0] + 2;
    dofs[7] = offset + 4*c.entity_indices[1][0] + 3;
    dofs[8] = offset + 4*c.entity_indices[1][1];
    dofs[9] = offset + 4*c.entity_indices[1][1] + 1;
    dofs[10] = offset + 4*c.entity_indices[1][1] + 2;
    dofs[11] = offset + 4*c.entity_indices[1][1] + 3;
    dofs[12] = offset + 4*c.entity_indices[1][2];
    dofs[13] = offset + 4*c.entity_indices[1][2] + 1;
    dofs[14] = offset + 4*c.entity_indices[1][2] + 2;
    dofs[15] = offset + 4*c.entity_indices[1][2] + 3;
    dofs[16] = offset + 4*c.entity_indices[1][3];
    dofs[17] = offset + 4*c.entity_indices[1][3] + 1;
    dofs[18] = offset + 4*c.entity_indices[1][3] + 2;
    dofs[19] = offset + 4*c.entity_indices[1][3] + 3;
    dofs[20] = offset + 4*c.entity_indices[1][4];
    dofs[21] = offset + 4*c.entity_indices[1][4] + 1;
    dofs[22] = offset + 4*c.entity_indices[1][4] + 2;
    dofs[23] = offset + 4*c.entity_indices[1][4] + 3;
    dofs[24] = offset + 4*c.entity_indices[1][5];
    dofs[25] = offset + 4*c.entity_indices[1][5] + 1;
    dofs[26] = offset + 4*c.entity_indices[1][5] + 2;
    dofs[27] = offset + 4*c.entity_indices[1][5] + 3;
    offset = offset + 4*m.num_entities[1];
    dofs[28] = offset + 6*c.entity_indices[2][0];
    dofs[29] = offset + 6*c.entity_indices[2][0] + 1;
    dofs[30] = offset + 6*c.entity_indices[2][0] + 2;
    dofs[31] = offset + 6*c.entity_indices[2][0] + 3;
    dofs[32] = offset + 6*c.entity_indices[2][0] + 4;
    dofs[33] = offset + 6*c.entity_indices[2][0] + 5;
    dofs[34] = offset + 6*c.entity_indices[2][1];
    dofs[35] = offset + 6*c.entity_indices[2][1] + 1;
    dofs[36] = offset + 6*c.entity_indices[2][1] + 2;
    dofs[37] = offset + 6*c.entity_indices[2][1] + 3;
    dofs[38] = offset + 6*c.entity_indices[2][1] + 4;
    dofs[39] = offset + 6*c.entity_indices[2][1] + 5;
    dofs[40] = offset + 6*c.entity_indices[2][2];
    dofs[41] = offset + 6*c.entity_indices[2][2] + 1;
    dofs[42] = offset + 6*c.entity_indices[2][2] + 2;
    dofs[43] = offset + 6*c.entity_indices[2][2] + 3;
    dofs[44] = offset + 6*c.entity_indices[2][2] + 4;
    dofs[45] = offset + 6*c.entity_indices[2][2] + 5;
    dofs[46] = offset + 6*c.entity_indices[2][3];
    dofs[47] = offset + 6*c.entity_indices[2][3] + 1;
    dofs[48] = offset + 6*c.entity_indices[2][3] + 2;
    dofs[49] = offset + 6*c.entity_indices[2][3] + 3;
    dofs[50] = offset + 6*c.entity_indices[2][3] + 4;
    dofs[51] = offset + 6*c.entity_indices[2][3] + 5;
    offset = offset + 6*m.num_entities[2];
    dofs[52] = offset + 4*c.entity_indices[3][0];
    dofs[53] = offset + 4*c.entity_indices[3][0] + 1;
    dofs[54] = offset + 4*c.entity_indices[3][0] + 2;
    dofs[55] = offset + 4*c.entity_indices[3][0] + 3;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 8;
      dofs[8] = 9;
      dofs[9] = 10;
      dofs[10] = 11;
      dofs[11] = 12;
      dofs[12] = 13;
      dofs[13] = 14;
      dofs[14] = 15;
      dofs[15] = 28;
      dofs[16] = 29;
      dofs[17] = 30;
      dofs[18] = 31;
      dofs[19] = 32;
      dofs[20] = 33;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 20;
      dofs[12] = 21;
      dofs[13] = 22;
      dofs[14] = 23;
      dofs[15] = 34;
      dofs[16] = 35;
      dofs[17] = 36;
      dofs[18] = 37;
      dofs[19] = 38;
      dofs[20] = 39;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 8;
      dofs[4] = 9;
      dofs[5] = 10;
      dofs[6] = 11;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 40;
      dofs[16] = 41;
      dofs[17] = 42;
      dofs[18] = 43;
      dofs[19] = 44;
      dofs[20] = 45;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 12;
      dofs[4] = 13;
      dofs[5] = 14;
      dofs[6] = 15;
      dofs[7] = 20;
      dofs[8] = 21;
      dofs[9] = 22;
      dofs[10] = 23;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 46;
      dofs[16] = 47;
      dofs[17] = 48;
      dofs[18] = 49;
      dofs[19] = 50;
      dofs[20] = 51;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.8*x[2][0] + 0.2*x[3][0];
    coordinates[4][1] = 0.8*x[2][1] + 0.2*x[3][1];
    coordinates[4][2] = 0.8*x[2][2] + 0.2*x[3][2];
    coordinates[5][0] = 0.6*x[2][0] + 0.4*x[3][0];
    coordinates[5][1] = 0.6*x[2][1] + 0.4*x[3][1];
    coordinates[5][2] = 0.6*x[2][2] + 0.4*x[3][2];
    coordinates[6][0] = 0.4*x[2][0] + 0.6*x[3][0];
    coordinates[6][1] = 0.4*x[2][1] + 0.6*x[3][1];
    coordinates[6][2] = 0.4*x[2][2] + 0.6*x[3][2];
    coordinates[7][0] = 0.2*x[2][0] + 0.8*x[3][0];
    coordinates[7][1] = 0.2*x[2][1] + 0.8*x[3][1];
    coordinates[7][2] = 0.2*x[2][2] + 0.8*x[3][2];
    coordinates[8][0] = 0.8*x[1][0] + 0.2*x[3][0];
    coordinates[8][1] = 0.8*x[1][1] + 0.2*x[3][1];
    coordinates[8][2] = 0.8*x[1][2] + 0.2*x[3][2];
    coordinates[9][0] = 0.6*x[1][0] + 0.4*x[3][0];
    coordinates[9][1] = 0.6*x[1][1] + 0.4*x[3][1];
    coordinates[9][2] = 0.6*x[1][2] + 0.4*x[3][2];
    coordinates[10][0] = 0.4*x[1][0] + 0.6*x[3][0];
    coordinates[10][1] = 0.4*x[1][1] + 0.6*x[3][1];
    coordinates[10][2] = 0.4*x[1][2] + 0.6*x[3][2];
    coordinates[11][0] = 0.2*x[1][0] + 0.8*x[3][0];
    coordinates[11][1] = 0.2*x[1][1] + 0.8*x[3][1];
    coordinates[11][2] = 0.2*x[1][2] + 0.8*x[3][2];
    coordinates[12][0] = 0.8*x[1][0] + 0.2*x[2][0];
    coordinates[12][1] = 0.8*x[1][1] + 0.2*x[2][1];
    coordinates[12][2] = 0.8*x[1][2] + 0.2*x[2][2];
    coordinates[13][0] = 0.6*x[1][0] + 0.4*x[2][0];
    coordinates[13][1] = 0.6*x[1][1] + 0.4*x[2][1];
    coordinates[13][2] = 0.6*x[1][2] + 0.4*x[2][2];
    coordinates[14][0] = 0.4*x[1][0] + 0.6*x[2][0];
    coordinates[14][1] = 0.4*x[1][1] + 0.6*x[2][1];
    coordinates[14][2] = 0.4*x[1][2] + 0.6*x[2][2];
    coordinates[15][0] = 0.2*x[1][0] + 0.8*x[2][0];
    coordinates[15][1] = 0.2*x[1][1] + 0.8*x[2][1];
    coordinates[15][2] = 0.2*x[1][2] + 0.8*x[2][2];
    coordinates[16][0] = 0.8*x[0][0] + 0.2*x[3][0];
    coordinates[16][1] = 0.8*x[0][1] + 0.2*x[3][1];
    coordinates[16][2] = 0.8*x[0][2] + 0.2*x[3][2];
    coordinates[17][0] = 0.6*x[0][0] + 0.4*x[3][0];
    coordinates[17][1] = 0.6*x[0][1] + 0.4*x[3][1];
    coordinates[17][2] = 0.6*x[0][2] + 0.4*x[3][2];
    coordinates[18][0] = 0.4*x[0][0] + 0.6*x[3][0];
    coordinates[18][1] = 0.4*x[0][1] + 0.6*x[3][1];
    coordinates[18][2] = 0.4*x[0][2] + 0.6*x[3][2];
    coordinates[19][0] = 0.2*x[0][0] + 0.8*x[3][0];
    coordinates[19][1] = 0.2*x[0][1] + 0.8*x[3][1];
    coordinates[19][2] = 0.2*x[0][2] + 0.8*x[3][2];
    coordinates[20][0] = 0.8*x[0][0] + 0.2*x[2][0];
    coordinates[20][1] = 0.8*x[0][1] + 0.2*x[2][1];
    coordinates[20][2] = 0.8*x[0][2] + 0.2*x[2][2];
    coordinates[21][0] = 0.6*x[0][0] + 0.4*x[2][0];
    coordinates[21][1] = 0.6*x[0][1] + 0.4*x[2][1];
    coordinates[21][2] = 0.6*x[0][2] + 0.4*x[2][2];
    coordinates[22][0] = 0.4*x[0][0] + 0.6*x[2][0];
    coordinates[22][1] = 0.4*x[0][1] + 0.6*x[2][1];
    coordinates[22][2] = 0.4*x[0][2] + 0.6*x[2][2];
    coordinates[23][0] = 0.2*x[0][0] + 0.8*x[2][0];
    coordinates[23][1] = 0.2*x[0][1] + 0.8*x[2][1];
    coordinates[23][2] = 0.2*x[0][2] + 0.8*x[2][2];
    coordinates[24][0] = 0.8*x[0][0] + 0.2*x[1][0];
    coordinates[24][1] = 0.8*x[0][1] + 0.2*x[1][1];
    coordinates[24][2] = 0.8*x[0][2] + 0.2*x[1][2];
    coordinates[25][0] = 0.6*x[0][0] + 0.4*x[1][0];
    coordinates[25][1] = 0.6*x[0][1] + 0.4*x[1][1];
    coordinates[25][2] = 0.6*x[0][2] + 0.4*x[1][2];
    coordinates[26][0] = 0.4*x[0][0] + 0.6*x[1][0];
    coordinates[26][1] = 0.4*x[0][1] + 0.6*x[1][1];
    coordinates[26][2] = 0.4*x[0][2] + 0.6*x[1][2];
    coordinates[27][0] = 0.2*x[0][0] + 0.8*x[1][0];
    coordinates[27][1] = 0.2*x[0][1] + 0.8*x[1][1];
    coordinates[27][2] = 0.2*x[0][2] + 0.8*x[1][2];
    coordinates[28][0] = 0.6*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[28][1] = 0.6*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[28][2] = 0.6*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[29][0] = 0.4*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[29][1] = 0.4*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[29][2] = 0.4*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[30][0] = 0.2*x[1][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[30][1] = 0.2*x[1][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[30][2] = 0.2*x[1][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[31][0] = 0.4*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[31][1] = 0.4*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[31][2] = 0.4*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[32][0] = 0.2*x[1][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[32][1] = 0.2*x[1][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[32][2] = 0.2*x[1][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[33][0] = 0.2*x[1][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[33][1] = 0.2*x[1][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[33][2] = 0.2*x[1][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[34][0] = 0.6*x[0][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[34][1] = 0.6*x[0][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[34][2] = 0.6*x[0][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[35][0] = 0.4*x[0][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[35][1] = 0.4*x[0][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[35][2] = 0.4*x[0][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[36][0] = 0.2*x[0][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[36][1] = 0.2*x[0][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[36][2] = 0.2*x[0][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[37][0] = 0.4*x[0][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[37][1] = 0.4*x[0][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[37][2] = 0.4*x[0][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[38][0] = 0.2*x[0][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[38][1] = 0.2*x[0][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[38][2] = 0.2*x[0][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[39][0] = 0.2*x[0][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[39][1] = 0.2*x[0][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[39][2] = 0.2*x[0][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[40][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[3][0];
    coordinates[40][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[3][1];
    coordinates[40][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[3][2];
    coordinates[41][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[3][0];
    coordinates[41][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[3][1];
    coordinates[41][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[3][2];
    coordinates[42][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[3][0];
    coordinates[42][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[3][1];
    coordinates[42][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[3][2];
    coordinates[43][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[3][0];
    coordinates[43][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[3][1];
    coordinates[43][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[3][2];
    coordinates[44][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[3][0];
    coordinates[44][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[3][1];
    coordinates[44][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[3][2];
    coordinates[45][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[3][0];
    coordinates[45][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[3][1];
    coordinates[45][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[3][2];
    coordinates[46][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0];
    coordinates[46][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1];
    coordinates[46][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2];
    coordinates[47][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0];
    coordinates[47][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1];
    coordinates[47][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2];
    coordinates[48][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[2][0];
    coordinates[48][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[2][1];
    coordinates[48][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[2][2];
    coordinates[49][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0];
    coordinates[49][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1];
    coordinates[49][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2];
    coordinates[50][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[2][0];
    coordinates[50][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[2][1];
    coordinates[50][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[2][2];
    coordinates[51][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[2][0];
    coordinates[51][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[2][1];
    coordinates[51][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[2][2];
    coordinates[52][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[52][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[52][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[53][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[53][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[53][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[54][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[54][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[54][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[55][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[55][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[55][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new poisson3d_5_0_dof_map_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class poisson3d_5_0_dof_map_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  poisson3d_5_0_dof_map_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~poisson3d_5_0_dof_map_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for FiniteElement('Lagrange', 'tetrahedron', 5)";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return true;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + 4*m.num_entities[1] + 6*m.num_entities[2] + 4*m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space for a cell
  virtual unsigned int local_dimension(const ufc::cell& c) const
  {
    return 56;
  }

  /// Return the maximum dimension of the local finite element function space
  virtual unsigned int max_local_dimension() const
  {
    return 56;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    return 3;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 21;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + 4*c.entity_indices[1][0];
    dofs[5] = offset + 4*c.entity_indices[1][0] + 1;
    dofs[6] = offset + 4*c.entity_indices[1][0] + 2;
    dofs[7] = offset + 4*c.entity_indices[1][0] + 3;
    dofs[8] = offset + 4*c.entity_indices[1][1];
    dofs[9] = offset + 4*c.entity_indices[1][1] + 1;
    dofs[10] = offset + 4*c.entity_indices[1][1] + 2;
    dofs[11] = offset + 4*c.entity_indices[1][1] + 3;
    dofs[12] = offset + 4*c.entity_indices[1][2];
    dofs[13] = offset + 4*c.entity_indices[1][2] + 1;
    dofs[14] = offset + 4*c.entity_indices[1][2] + 2;
    dofs[15] = offset + 4*c.entity_indices[1][2] + 3;
    dofs[16] = offset + 4*c.entity_indices[1][3];
    dofs[17] = offset + 4*c.entity_indices[1][3] + 1;
    dofs[18] = offset + 4*c.entity_indices[1][3] + 2;
    dofs[19] = offset + 4*c.entity_indices[1][3] + 3;
    dofs[20] = offset + 4*c.entity_indices[1][4];
    dofs[21] = offset + 4*c.entity_indices[1][4] + 1;
    dofs[22] = offset + 4*c.entity_indices[1][4] + 2;
    dofs[23] = offset + 4*c.entity_indices[1][4] + 3;
    dofs[24] = offset + 4*c.entity_indices[1][5];
    dofs[25] = offset + 4*c.entity_indices[1][5] + 1;
    dofs[26] = offset + 4*c.entity_indices[1][5] + 2;
    dofs[27] = offset + 4*c.entity_indices[1][5] + 3;
    offset = offset + 4*m.num_entities[1];
    dofs[28] = offset + 6*c.entity_indices[2][0];
    dofs[29] = offset + 6*c.entity_indices[2][0] + 1;
    dofs[30] = offset + 6*c.entity_indices[2][0] + 2;
    dofs[31] = offset + 6*c.entity_indices[2][0] + 3;
    dofs[32] = offset + 6*c.entity_indices[2][0] + 4;
    dofs[33] = offset + 6*c.entity_indices[2][0] + 5;
    dofs[34] = offset + 6*c.entity_indices[2][1];
    dofs[35] = offset + 6*c.entity_indices[2][1] + 1;
    dofs[36] = offset + 6*c.entity_indices[2][1] + 2;
    dofs[37] = offset + 6*c.entity_indices[2][1] + 3;
    dofs[38] = offset + 6*c.entity_indices[2][1] + 4;
    dofs[39] = offset + 6*c.entity_indices[2][1] + 5;
    dofs[40] = offset + 6*c.entity_indices[2][2];
    dofs[41] = offset + 6*c.entity_indices[2][2] + 1;
    dofs[42] = offset + 6*c.entity_indices[2][2] + 2;
    dofs[43] = offset + 6*c.entity_indices[2][2] + 3;
    dofs[44] = offset + 6*c.entity_indices[2][2] + 4;
    dofs[45] = offset + 6*c.entity_indices[2][2] + 5;
    dofs[46] = offset + 6*c.entity_indices[2][3];
    dofs[47] = offset + 6*c.entity_indices[2][3] + 1;
    dofs[48] = offset + 6*c.entity_indices[2][3] + 2;
    dofs[49] = offset + 6*c.entity_indices[2][3] + 3;
    dofs[50] = offset + 6*c.entity_indices[2][3] + 4;
    dofs[51] = offset + 6*c.entity_indices[2][3] + 5;
    offset = offset + 6*m.num_entities[2];
    dofs[52] = offset + 4*c.entity_indices[3][0];
    dofs[53] = offset + 4*c.entity_indices[3][0] + 1;
    dofs[54] = offset + 4*c.entity_indices[3][0] + 2;
    dofs[55] = offset + 4*c.entity_indices[3][0] + 3;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 8;
      dofs[8] = 9;
      dofs[9] = 10;
      dofs[10] = 11;
      dofs[11] = 12;
      dofs[12] = 13;
      dofs[13] = 14;
      dofs[14] = 15;
      dofs[15] = 28;
      dofs[16] = 29;
      dofs[17] = 30;
      dofs[18] = 31;
      dofs[19] = 32;
      dofs[20] = 33;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 20;
      dofs[12] = 21;
      dofs[13] = 22;
      dofs[14] = 23;
      dofs[15] = 34;
      dofs[16] = 35;
      dofs[17] = 36;
      dofs[18] = 37;
      dofs[19] = 38;
      dofs[20] = 39;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 8;
      dofs[4] = 9;
      dofs[5] = 10;
      dofs[6] = 11;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 40;
      dofs[16] = 41;
      dofs[17] = 42;
      dofs[18] = 43;
      dofs[19] = 44;
      dofs[20] = 45;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 12;
      dofs[4] = 13;
      dofs[5] = 14;
      dofs[6] = 15;
      dofs[7] = 20;
      dofs[8] = 21;
      dofs[9] = 22;
      dofs[10] = 23;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 46;
      dofs[16] = 47;
      dofs[17] = 48;
      dofs[18] = 49;
      dofs[19] = 50;
      dofs[20] = 51;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.8*x[2][0] + 0.2*x[3][0];
    coordinates[4][1] = 0.8*x[2][1] + 0.2*x[3][1];
    coordinates[4][2] = 0.8*x[2][2] + 0.2*x[3][2];
    coordinates[5][0] = 0.6*x[2][0] + 0.4*x[3][0];
    coordinates[5][1] = 0.6*x[2][1] + 0.4*x[3][1];
    coordinates[5][2] = 0.6*x[2][2] + 0.4*x[3][2];
    coordinates[6][0] = 0.4*x[2][0] + 0.6*x[3][0];
    coordinates[6][1] = 0.4*x[2][1] + 0.6*x[3][1];
    coordinates[6][2] = 0.4*x[2][2] + 0.6*x[3][2];
    coordinates[7][0] = 0.2*x[2][0] + 0.8*x[3][0];
    coordinates[7][1] = 0.2*x[2][1] + 0.8*x[3][1];
    coordinates[7][2] = 0.2*x[2][2] + 0.8*x[3][2];
    coordinates[8][0] = 0.8*x[1][0] + 0.2*x[3][0];
    coordinates[8][1] = 0.8*x[1][1] + 0.2*x[3][1];
    coordinates[8][2] = 0.8*x[1][2] + 0.2*x[3][2];
    coordinates[9][0] = 0.6*x[1][0] + 0.4*x[3][0];
    coordinates[9][1] = 0.6*x[1][1] + 0.4*x[3][1];
    coordinates[9][2] = 0.6*x[1][2] + 0.4*x[3][2];
    coordinates[10][0] = 0.4*x[1][0] + 0.6*x[3][0];
    coordinates[10][1] = 0.4*x[1][1] + 0.6*x[3][1];
    coordinates[10][2] = 0.4*x[1][2] + 0.6*x[3][2];
    coordinates[11][0] = 0.2*x[1][0] + 0.8*x[3][0];
    coordinates[11][1] = 0.2*x[1][1] + 0.8*x[3][1];
    coordinates[11][2] = 0.2*x[1][2] + 0.8*x[3][2];
    coordinates[12][0] = 0.8*x[1][0] + 0.2*x[2][0];
    coordinates[12][1] = 0.8*x[1][1] + 0.2*x[2][1];
    coordinates[12][2] = 0.8*x[1][2] + 0.2*x[2][2];
    coordinates[13][0] = 0.6*x[1][0] + 0.4*x[2][0];
    coordinates[13][1] = 0.6*x[1][1] + 0.4*x[2][1];
    coordinates[13][2] = 0.6*x[1][2] + 0.4*x[2][2];
    coordinates[14][0] = 0.4*x[1][0] + 0.6*x[2][0];
    coordinates[14][1] = 0.4*x[1][1] + 0.6*x[2][1];
    coordinates[14][2] = 0.4*x[1][2] + 0.6*x[2][2];
    coordinates[15][0] = 0.2*x[1][0] + 0.8*x[2][0];
    coordinates[15][1] = 0.2*x[1][1] + 0.8*x[2][1];
    coordinates[15][2] = 0.2*x[1][2] + 0.8*x[2][2];
    coordinates[16][0] = 0.8*x[0][0] + 0.2*x[3][0];
    coordinates[16][1] = 0.8*x[0][1] + 0.2*x[3][1];
    coordinates[16][2] = 0.8*x[0][2] + 0.2*x[3][2];
    coordinates[17][0] = 0.6*x[0][0] + 0.4*x[3][0];
    coordinates[17][1] = 0.6*x[0][1] + 0.4*x[3][1];
    coordinates[17][2] = 0.6*x[0][2] + 0.4*x[3][2];
    coordinates[18][0] = 0.4*x[0][0] + 0.6*x[3][0];
    coordinates[18][1] = 0.4*x[0][1] + 0.6*x[3][1];
    coordinates[18][2] = 0.4*x[0][2] + 0.6*x[3][2];
    coordinates[19][0] = 0.2*x[0][0] + 0.8*x[3][0];
    coordinates[19][1] = 0.2*x[0][1] + 0.8*x[3][1];
    coordinates[19][2] = 0.2*x[0][2] + 0.8*x[3][2];
    coordinates[20][0] = 0.8*x[0][0] + 0.2*x[2][0];
    coordinates[20][1] = 0.8*x[0][1] + 0.2*x[2][1];
    coordinates[20][2] = 0.8*x[0][2] + 0.2*x[2][2];
    coordinates[21][0] = 0.6*x[0][0] + 0.4*x[2][0];
    coordinates[21][1] = 0.6*x[0][1] + 0.4*x[2][1];
    coordinates[21][2] = 0.6*x[0][2] + 0.4*x[2][2];
    coordinates[22][0] = 0.4*x[0][0] + 0.6*x[2][0];
    coordinates[22][1] = 0.4*x[0][1] + 0.6*x[2][1];
    coordinates[22][2] = 0.4*x[0][2] + 0.6*x[2][2];
    coordinates[23][0] = 0.2*x[0][0] + 0.8*x[2][0];
    coordinates[23][1] = 0.2*x[0][1] + 0.8*x[2][1];
    coordinates[23][2] = 0.2*x[0][2] + 0.8*x[2][2];
    coordinates[24][0] = 0.8*x[0][0] + 0.2*x[1][0];
    coordinates[24][1] = 0.8*x[0][1] + 0.2*x[1][1];
    coordinates[24][2] = 0.8*x[0][2] + 0.2*x[1][2];
    coordinates[25][0] = 0.6*x[0][0] + 0.4*x[1][0];
    coordinates[25][1] = 0.6*x[0][1] + 0.4*x[1][1];
    coordinates[25][2] = 0.6*x[0][2] + 0.4*x[1][2];
    coordinates[26][0] = 0.4*x[0][0] + 0.6*x[1][0];
    coordinates[26][1] = 0.4*x[0][1] + 0.6*x[1][1];
    coordinates[26][2] = 0.4*x[0][2] + 0.6*x[1][2];
    coordinates[27][0] = 0.2*x[0][0] + 0.8*x[1][0];
    coordinates[27][1] = 0.2*x[0][1] + 0.8*x[1][1];
    coordinates[27][2] = 0.2*x[0][2] + 0.8*x[1][2];
    coordinates[28][0] = 0.6*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[28][1] = 0.6*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[28][2] = 0.6*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[29][0] = 0.4*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[29][1] = 0.4*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[29][2] = 0.4*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[30][0] = 0.2*x[1][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[30][1] = 0.2*x[1][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[30][2] = 0.2*x[1][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[31][0] = 0.4*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[31][1] = 0.4*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[31][2] = 0.4*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[32][0] = 0.2*x[1][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[32][1] = 0.2*x[1][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[32][2] = 0.2*x[1][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[33][0] = 0.2*x[1][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[33][1] = 0.2*x[1][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[33][2] = 0.2*x[1][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[34][0] = 0.6*x[0][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[34][1] = 0.6*x[0][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[34][2] = 0.6*x[0][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[35][0] = 0.4*x[0][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[35][1] = 0.4*x[0][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[35][2] = 0.4*x[0][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[36][0] = 0.2*x[0][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[36][1] = 0.2*x[0][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[36][2] = 0.2*x[0][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[37][0] = 0.4*x[0][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[37][1] = 0.4*x[0][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[37][2] = 0.4*x[0][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[38][0] = 0.2*x[0][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[38][1] = 0.2*x[0][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[38][2] = 0.2*x[0][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[39][0] = 0.2*x[0][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[39][1] = 0.2*x[0][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[39][2] = 0.2*x[0][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[40][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[3][0];
    coordinates[40][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[3][1];
    coordinates[40][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[3][2];
    coordinates[41][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[3][0];
    coordinates[41][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[3][1];
    coordinates[41][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[3][2];
    coordinates[42][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[3][0];
    coordinates[42][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[3][1];
    coordinates[42][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[3][2];
    coordinates[43][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[3][0];
    coordinates[43][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[3][1];
    coordinates[43][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[3][2];
    coordinates[44][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[3][0];
    coordinates[44][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[3][1];
    coordinates[44][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[3][2];
    coordinates[45][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[3][0];
    coordinates[45][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[3][1];
    coordinates[45][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[3][2];
    coordinates[46][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0];
    coordinates[46][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1];
    coordinates[46][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2];
    coordinates[47][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0];
    coordinates[47][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1];
    coordinates[47][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2];
    coordinates[48][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[2][0];
    coordinates[48][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[2][1];
    coordinates[48][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[2][2];
    coordinates[49][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0];
    coordinates[49][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1];
    coordinates[49][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2];
    coordinates[50][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[2][0];
    coordinates[50][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[2][1];
    coordinates[50][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[2][2];
    coordinates[51][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[2][0];
    coordinates[51][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[2][1];
    coordinates[51][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[2][2];
    coordinates[52][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[52][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[52][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[53][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[53][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[53][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[54][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[54][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[54][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[55][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[55][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[55][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new poisson3d_5_0_dof_map_1();
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class poisson3d_5_0_cell_integral_0_quadrature: public ufc::cell_integral
{
public:

  /// Constructor
  poisson3d_5_0_cell_integral_0_quadrature() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_0_cell_integral_0_quadrature()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
    
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
    
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
    
    // Compute inverse of Jacobian
    const double Jinv_00 = d_00 / detJ;
    const double Jinv_01 = d_10 / detJ;
    const double Jinv_02 = d_20 / detJ;
    const double Jinv_10 = d_01 / detJ;
    const double Jinv_11 = d_11 / detJ;
    const double Jinv_12 = d_21 / detJ;
    const double Jinv_20 = d_02 / detJ;
    const double Jinv_21 = d_12 / detJ;
    const double Jinv_22 = d_22 / detJ;
    
    // Set scale factor
    const double det = std::abs(detJ);
    
    
    // Array of quadrature weights
    static const double W125[125] = {0.000937439821766984, 0.00144688123847004, 0.00102268701578053, 0.000367520038007324, 4.71653365059364e-05, 0.00161927658526931, 0.00249925473264391, 0.00176652740822439, 0.000634831781565251, 8.1470536312884e-05, 0.00141792453255091, 0.00218848010941898, 0.00154686516950305, 0.000555892406098532, 7.13399262170554e-05, 0.000715891501943862, 0.00110493490770459, 0.000780991938624509, 0.000280662785913662, 3.60185932012982e-05, 0.000152536470498617, 0.000235430746830112, 0.000166407554052789, 5.9801395389292e-05, 7.67455552179798e-06, 0.00189377231486028, 0.00292292216383614, 0.00206598473020027, 0.000742446882427904, 9.52812185081393e-05, 0.00327118722298822, 0.00504887813656484, 0.00356865648488397, 0.00128245763045954, 0.000164582987156811, 0.00286442517370845, 0.00442106570107946, 0.00312490504969682, 0.00112298797668544, 0.000144117599953649, 0.00144621070637857, 0.0022321380949974, 0.00157772357985427, 0.000566981902660164, 7.27630862707133e-05, 0.000308147081155878, 0.000475606241660779, 0.000336168798819301, 0.000120807996789371, 1.55037800172006e-05, 0.00225090157446143, 0.00347412941301361, 0.00245558995953753, 0.00088245817276838, 0.000113249435042247, 0.00388807060532275, 0.00600100004508522, 0.00424163688396193, 0.00152430462570915, 0.00019562019257218, 0.0034046010087031, 0.0052547941847441, 0.00371420241029555, 0.00133476204345558, 0.000171295424533231, 0.00171893840164765, 0.00265307667295562, 0.00187525208922536, 0.000673903851785402, 8.64848134932762e-05, 0.000366257730507922, 0.000565296487744311, 0.00039956380849458, 0.000143590075769372, 1.8427496577589e-05, 0.00189377231486028, 0.00292292216383614, 0.00206598473020027, 0.000742446882427904, 9.52812185081393e-05, 0.00327118722298822, 0.00504887813656484, 0.00356865648488397, 0.00128245763045954, 0.000164582987156811, 0.00286442517370845, 0.00442106570107946, 0.00312490504969682, 0.00112298797668544, 0.000144117599953649, 0.00144621070637857, 0.0022321380949974, 0.00157772357985427, 0.000566981902660164, 7.27630862707133e-05, 0.000308147081155878, 0.000475606241660779, 0.000336168798819301, 0.000120807996789371, 1.55037800172006e-05, 0.000937439821766984, 0.00144688123847004, 0.00102268701578053, 0.000367520038007324, 4.71653365059364e-05, 0.00161927658526931, 0.00249925473264391, 0.00176652740822439, 0.000634831781565252, 8.1470536312884e-05, 0.00141792453255091, 0.00218848010941898, 0.00154686516950305, 0.000555892406098532, 7.13399262170554e-05, 0.000715891501943862, 0.00110493490770459, 0.000780991938624509, 0.000280662785913662, 3.60185932012982e-05, 0.000152536470498617, 0.000235430746830112, 0.000166407554052789, 5.9801395389292e-05, 7.67455552179798e-06};
    // Quadrature points on the UFC reference element: (0.0434850684329929, 0.0384332743963333, 0.034578939918215), (0.0372285899889251, 0.0329036302803046, 0.173480320771696), (0.0274810994988124, 0.0242885357160768, 0.389886387065519), (0.0164705687743685, 0.0145571321830714, 0.634333472630887), (0.0067089045501621, 0.00592951049099777, 0.851054212947016), (0.0363203493206216, 0.191166323793956, 0.034578939918215), (0.0310947054204484, 0.163661986623795, 0.173480320771696), (0.0229532381913956, 0.120810681788372, 0.389886387065519), (0.0137568327003139, 0.0724068788863314, 0.634333472630887), (0.00560352704046152, 0.0294932643722359, 0.851054212947016), (0.025452983470971, 0.42283010559815, 0.034578939918215), (0.0217908978824722, 0.361994799675747, 0.173480320771696), (0.0160854287808059, 0.267214393854326, 0.389886387065519), (0.00964066816216436, 0.160152727938308, 0.634333472630887), (0.00392690279162666, 0.0652345028216781, 0.851054212947016), (0.0137918067694829, 0.671415856030076, 0.034578939918215), (0.0118074902013492, 0.574814908126993, 0.173480320771696), (0.00871595763232119, 0.42431222048264, 0.389886387065519), (0.00522383682733774, 0.254308005746508, 0.634333472630887), (0.00212780888992548, 0.103586473561889, 0.851054212947016), (0.00446245462992895, 0.870293213094632, 0.034578939918215), (0.00382041237943087, 0.745078491721125, 0.173480320771696), (0.00282012111543484, 0.54999601573695, 0.389886387065519), (0.00169021617151183, 0.329635544721039, 0.634333472630887), (0.000688470393412244, 0.134269401146344, 0.851054212947016), (0.213916656125506, 0.0384332743963333, 0.034578939918215), (0.183139081291086, 0.0329036302803046, 0.173480320771696), (0.135188126023001, 0.0242885357160768, 0.389886387065519), (0.0810238806942951, 0.0145571321830714, 0.634333472630887), (0.0330032003938849, 0.00592951049099777, 0.851054212947016), (0.178671161296432, 0.191166323793956, 0.034578939918215), (0.152964584084757, 0.163661986623795, 0.173480320771696), (0.112914159689587, 0.120810681788372, 0.389886387065519), (0.0676741639412116, 0.0724068788863314, 0.634333472630887), (0.027565502601231, 0.0294932643722359, 0.851054212947016), (0.125211188776624, 0.42283010559815, 0.034578939918215), (0.107196244066483, 0.361994799675747, 0.173480320771696), (0.0791292565731431, 0.267214393854326, 0.389886387065519), (0.0474254628170509, 0.160152727938308, 0.634333472630887), (0.0193176633816068, 0.0652345028216781, 0.851054212947016), (0.0678462123292524, 0.671415856030076, 0.034578939918215), (0.0580847383280397, 0.574814908126993, 0.173480320771696), (0.0428765224208113, 0.42431222048264, 0.389886387065519), (0.0256976876550462, 0.254308005746508, 0.634333472630887), (0.0104673576243388, 0.103586473561889, 0.851054212947016), (0.0219522104240708, 0.870293213094632, 0.034578939918215), (0.0187938037280005, 0.745078491721125, 0.173480320771696), (0.0138730580546826, 0.54999601573695, 0.389886387065519), (0.00831470213956798, 0.329635544721039, 0.634333472630887), (0.00338680125632329, 0.134269401146344, 0.851054212947016), (0.463493892842726, 0.0384332743963333, 0.034578939918215), (0.396808024474, 0.0329036302803046, 0.173480320771696), (0.292912538609202, 0.0242885357160768, 0.389886387065519), (0.175554697593021, 0.0145571321830714, 0.634333472630887), (0.0715081382809929, 0.00592951049099777, 0.851054212947016), (0.387127368143914, 0.191166323793956, 0.034578939918215), (0.331428846302255, 0.163661986623795, 0.173480320771696), (0.244651465573054, 0.120810681788372, 0.389886387065519), (0.146629824241391, 0.0724068788863314, 0.634333472630887), (0.0597262613403738, 0.0294932643722359, 0.851054212947016), (0.271295477241817, 0.42283010559815, 0.034578939918215), (0.232262439776279, 0.361994799675747, 0.173480320771696), (0.171449609540077, 0.267214393854326, 0.389886387065519), (0.102756899715403, 0.160152727938308, 0.634333472630887), (0.0418556421156527, 0.0652345028216781, 0.851054212947016), (0.147002602025855, 0.671415856030076, 0.034578939918215), (0.125852385550656, 0.574814908126993, 0.173480320771696), (0.0929006962259202, 0.42431222048264, 0.389886387065519), (0.0556792608113027, 0.254308005746508, 0.634333472630887), (0.0226796567455474, 0.103586473561889, 0.851054212947016), (0.0475639234935763, 0.870293213094632, 0.034578939918215), (0.0407205937535897, 0.745078491721125, 0.173480320771696), (0.0300587985987655, 0.54999601573695, 0.389886387065519), (0.0180154913240372, 0.329635544721039, 0.634333472630887), (0.00733819295331972, 0.134269401146344, 0.851054212947016), (0.713071129559946, 0.0384332743963333, 0.034578939918215), (0.610476967656914, 0.0329036302803046, 0.173480320771696), (0.450636951195403, 0.0242885357160768, 0.389886387065519), (0.270085514491747, 0.0145571321830714, 0.634333472630887), (0.110013076168101, 0.00592951049099777, 0.851054212947016), (0.595583574991397, 0.191166323793956, 0.034578939918215), (0.509893108519752, 0.163661986623795, 0.173480320771696), (0.376388771456521, 0.120810681788372, 0.389886387065519), (0.22558548454157, 0.0724068788863314, 0.634333472630887), (0.0918870200795167, 0.0294932643722359, 0.851054212947016), (0.417379765707011, 0.42283010559815, 0.034578939918215), (0.357328635486074, 0.361994799675747, 0.173480320771696), (0.263769962507011, 0.267214393854326, 0.389886387065519), (0.158088336613754, 0.160152727938308, 0.634333472630887), (0.0643936208496987, 0.0652345028216781, 0.851054212947016), (0.226158991722457, 0.671415856030076, 0.034578939918215), (0.193620032773272, 0.574814908126993, 0.173480320771696), (0.142924870031029, 0.42431222048264, 0.389886387065519), (0.0856608339675592, 0.254308005746508, 0.634333472630887), (0.0348919558667561, 0.103586473561889, 0.851054212947016), (0.0731756365630818, 0.870293213094632, 0.034578939918215), (0.062647383779179, 0.745078491721125, 0.173480320771696), (0.0462445391428484, 0.54999601573695, 0.389886387065519), (0.0277162805085065, 0.329635544721039, 0.634333472630887), (0.0112895846503162, 0.134269401146344, 0.851054212947016), (0.883502717252459, 0.0384332743963333, 0.034578939918215), (0.756387458959075, 0.0329036302803046, 0.173480320771696), (0.558343977719591, 0.0242885357160768, 0.389886387065519), (0.334638826411673, 0.0145571321830714, 0.634333472630887), (0.136307372011824, 0.00592951049099777, 0.851054212947016), (0.737934386967207, 0.191166323793956, 0.034578939918215), (0.631762987184061, 0.163661986623795, 0.173480320771696), (0.466349692954713, 0.120810681788372, 0.389886387065519), (0.279502815782468, 0.0724068788863314, 0.634333472630887), (0.113848995640286, 0.0294932643722359, 0.851054212947016), (0.517137971012664, 0.42283010559815, 0.034578939918215), (0.442733981670085, 0.361994799675747, 0.173480320771696), (0.326813790299348, 0.267214393854326, 0.389886387065519), (0.195873131268641, 0.160152727938308, 0.634333472630887), (0.0797843814396788, 0.0652345028216781, 0.851054212947016), (0.280213397282226, 0.671415856030076, 0.034578939918215), (0.239897280899962, 0.574814908126993, 0.173480320771696), (0.177085434819519, 0.42431222048264, 0.389886387065519), (0.106134684795268, 0.254308005746508, 0.634333472630887), (0.0432315046011695, 0.103586473561889, 0.851054212947016), (0.0906653923572237, 0.870293213094632, 0.034578939918215), (0.0776207751277486, 0.745078491721125, 0.173480320771696), (0.0572974760820962, 0.54999601573695, 0.389886387065519), (0.0343407664765626, 0.329635544721039, 0.634333472630887), (0.0139879155132272, 0.134269401146344, 0.851054212947016)
    
    // Value of basis functions at quadrature points.
    static const double FE0_D010[125][50] = \
    {{-3.62990990645797, 0.349323335271638, -0.0867591749077241, -0.0570689004231587, -0.0670256254939616, -0.153888727424808, -0.175828494830756, -0.0778069667533905, -0.067903625662378, -0.109104809660912, -3.05254952705568, 2.02813935973006, -0.852973082215369, 0.153888727424807, 7.38531978670072, -7.16081891567265, 5.10864194701882, -2.05255624686056, -3.83876213073824, 2.41318852918331, -0.9901772308779, 0.175828494830751, 0.0437002683219805, 0.0226398576078526, 0.0300039794095119, 0.030415043262991, 0.0239281275472965, 0.0473407328843838, 4.31602724927854, -1.74588576697717, 0.569167219662036, -2.42947806492689, 0.458407605619987, 0.919998707709332, -1.06629444649797, 0.28811650741601, -0.0437002683219674, 0.304511126233411, -0.0304150432630075, -0.0473407328843884, 5.42766032554618, -2.89072275534576, 1.0679841976313, -2.19555493120249, 0.545437851824812, 0.715761546055484, 1.27729830600253, -0.31075636502388, -0.241007838914061, -0.32843925378069},
    {-0.282631633534522, 0.425653570551385, -0.515016292023652, -0.0524324561977713, -0.0364185135341676, -0.0578765020203719, -0.161086287535334, -0.0768219914016639, -0.0690615677537182, -0.110521644691893, -6.52731890993827, 1.06473844170331, -0.356273456486926, 0.0578765020203719, 1.81608845625925, -3.96890023631498, 4.10626582749042, -2.09647598445155, -1.40075184518967, 1.40242344845198, -0.751530849418154, 0.161086287535333, 0.198626300742623, 0.110210755392297, 0.147210409077317, 0.0217802683865225, 0.0179561870251892, 0.0202069040997851, 10.9095820348659, -6.67581251337119, 2.80856568046723, -1.35736720998758, 0.345061224482055, 0.392691970021095, -2.98938087073942, 1.0781650033246, -0.198626300742627, 0.175661008535879, -0.0217802683865247, -0.0202069040997923, 2.34117826575307, -1.78786031281182, 0.828352840819824, -1.43261832695308, 0.45449843211355, 0.602713551081585, 3.8109712330989, -1.1883757587169, -0.968800771436785, -0.193617195561062},
    {0.143241115565024, 0.555572946106277, -1.45661822356639, 1.01456732457269, 0.0295255943487218, 0.0204847751966064, -0.131658137530841, -0.0696433278875353, -0.0649710303379895, -0.102669576732067, -0.878278855981277, -6.65596057481905, 0.178742789963476, -0.0204847751966046, -0.226551992163587, -0.865602057515811, 2.35911829376461, -1.96577830575651, -0.0619053894407148, 0.426235505482272, -0.421608540100327, 0.13165813753084, 0.35863770104175, 0.21867066709978, 0.293664290797287, -0.274215954779785, -0.240684254375373, -0.0107169476080424, 4.73065721449071, -8.03664327050678, 5.64088313556374, 10.2846225763129, -4.64322932606656, -0.208268384312198, -1.92655321577806, 1.32379401609847, -0.358637701041753, -1.45706042761475, 0.274215954779778, 0.0107169476080398, 0.333439858171673, -0.65860836362126, 0.491251867987867, -0.566461925012372, 0.297343888476969, 0.397597033013489, 2.97686148749433, -1.54246468319825, -1.34397256251355, 1.69774468199014},
    {-0.102291761943676, 0.71926725952242, -2.96850396495831, 4.53211724574026, -2.8730940690759, 0.288623553688339, -0.0880875552963097, -0.0515926220049665, -0.0497249692031567, -0.0770776741593783, 0.558471148525243, -1.03193983204808, -7.88994374560002, -0.288623553688337, 0.114568117951423, 0.14116234547003, 0.613992989586029, -1.48669895058622, 0.0145007915507629, 0.0113221202333854, -0.141681015490765, 0.0880875552963082, 0.383027020449071, 0.25599282152056, 0.343729201147287, -0.650646765379126, -0.605822457995385, 0.55383784122667, -1.04210594164298, -3.18777292613865, 6.63991168421469, 8.22749391170655, -11.7276713253987, 10.7630378146759, -0.0782653746293638, 0.70299437210014, -0.38302702044907, -1.66367859822352, 0.650646765379121, -0.553837841226666, -0.0270584453171417, -0.0902694831567534, 0.193273637495736, -0.0827710273576927, 0.128672332126517, 0.172406355283454, 0.623997517357562, -0.958987193620702, -0.889461343875475, 2.26950105621892},
    {0.290248257600272, 0.88027078582891, -4.75479662623642, 10.5319042888745, -12.2449015666311, 8.1697622215787, -0.0393989079388153, -0.0249878994264349, -0.0246488115605273, -0.037482308689118, -1.68368761382697, 4.01561825483129, -4.72603256077766, -8.1697622215787, -0.149952137540204, -0.120505154295903, -0.173596923941105, -0.726464827651967, -0.0132725968823168, -0.00939813114015681, -0.00964430997441279, 0.0393989079388114, 0.226065126168303, 0.162216159930392, 0.217055781839351, -0.56134242992677, -0.546385823335478, 0.873279986799639, 1.08790294413218, 1.14158185247206, 4.20899944345912, -3.94452348395894, -10.6029990597469, 16.9709341274087, 0.0827593126526115, 0.0626088212750271, -0.226065126168282, -0.210882641871702, 0.56134242992675, -0.873279986799639, 0.00857599539603879, 0.00923174132476907, 0.0346322094008603, 0.00899914901768549, 0.0248152013759124, 0.0331797611576971, -0.0812940950952489, -0.224824981205456, -0.218520999396668, 0.757268465207204},
    {-0.0804169334283811, -0.267111615790655, 0.0747872960979259, 0.0592354988079015, 0.0992497134834056, -0.15388872742481, -0.158684622935266, 0.102658974459709, 0.0615636388131089, 0.0785536145829163, -1.11829004720866, 1.23357237757902, -0.694497732197733, 0.153888727424809, -3.7730589397251, 6.73378637280234, -4.22797584878612, 1.61477696492791, -1.17460758636495, 1.28205562258155, -0.71835396244177, 0.158684622935265, 0.0389380460040458, -0.0292826819147862, -0.0260118513062326, 0.0265666755156739, -0.0295942242317676, 0.0395407439246528, -0.435157361976339, 2.01350413283208, -0.534844019745006, -0.683984241269904, -0.608823635117013, 0.595248018714328, -0.541693780027628, 0.204904935930991, -0.0389380460040361, 0.207084946583176, -0.0265666755156755, -0.039540743924653, -0.457072062758504, -0.710866956990591, 0.61569498798206, 2.1149050154789, -0.632752304404065, -0.561778980938365, 0.300355306155582, -0.175622254016209, 0.267350325178276, -0.177490722351413},
    {0.296938442417927, -0.299846929362777, 0.386955294238904, 0.0431773246303394, 0.0345544018425569, -0.0578765020203715, -0.143523611516616, 0.0642426662126411, 0.0492909235970536, 0.0693580737672023, -1.80057222446278, 0.622114485944269, -0.288629886037619, 0.0578765020203714, -1.43720748845573, 2.90034530056824, -2.94925917988491, 1.48902985471726, -0.322735527919581, 0.710201427666699, -0.536613352828259, 0.143523611516617, 0.175062685486992, -0.0906317028154919, -0.101252068598978, 0.0188772281077665, -0.014230007961354, 0.0168775591723737, -1.96650278130224, 5.56429304510754, -2.18417333358142, -0.354799752022244, -0.310492058552363, 0.254075484195065, -1.45887636963455, 0.757038659624996, -0.175062685486986, 0.118862007651424, -0.0188772281077656, -0.0168775591723758, -0.352477009617464, -0.405036847903526, 0.472370686615623, 0.997346859522853, -0.354455503360239, -0.391492395753008, 0.832015627142657, -0.666406956809508, 0.72811281109087, -0.104631999690074},
    {-0.100404462764307, -0.260145422130515, 0.651010728103405, -0.408482387606868, -0.00811565004606885, 0.0204847751966053, -0.115126938239244, 0.0166078053671563, 0.0224219502185271, 0.0383260478001761, 0.754231010240759, -3.33204574872114, 0.142867299242775, -0.0204847751966049, 0.0902863545719231, 0.519905220556769, -1.19498096482826, 0.945339274594389, 0.0444027916945074, 0.182898862155061, -0.292362569318101, 0.115126938239246, 0.311145847960948, -0.0515200597450769, -0.098753719891413, -0.235046552195552, 0.0552563509575357, -0.00895119393395216, -2.56631544901889, 3.70987558278027, -2.54880187210553, 2.06452148094636, 1.67600665538164, -0.134751649196705, -0.805547368791999, 0.906955296349867, -0.311145847960946, -0.972728688706471, 0.235046552195551, 0.00895119393395261, -0.151083114799006, -0.113323362953437, 0.275754763950947, 0.218406337684442, -0.0919974494201615, -0.150052062380112, 0.499113749392326, -0.855435236604797, 0.405187339291089, 0.917472337748934},
    {0.0327513821297277, -0.0105569750572722, -0.13524370810424, 0.634253162364355, -0.927843770282098, 0.288623553688337, -0.0755398622052345, -0.014221952138668, -0.00589821387692278, -0.00293303939715259, -0.335749943545727, 1.50942741166771, -6.03594528816658, -0.288623553688336, 0.0734153358266496, -0.012134003337138, -0.231796160540428, 0.14832042097846, -0.00728143161567213, -0.014036864510881, -0.0925187275588505, 0.0755398622052346, 0.326945989565526, 0.070070834009421, 0.0401779517081222, -0.551479996410729, -0.163410605106836, 0.462585999866328, -0.381363329437101, 1.03595011602074, -0.183593134933671, -0.812162016093019, -1.33151855793905, 6.96378905844868, 0.0956174998431458, 0.455835059655005, -0.326945989565527, -1.06304261937452, 0.55147999641073, -0.462585999866324, -0.00827065218448311, 0.00755267069661708, 0.106740679697519, 0.0224667198671765, 0.0123824076911832, -0.00398159666986489, -0.0514479204803368, -0.525905893664433, -0.0843475310709234, 1.22645322448135},
    {0.239734464658433, 0.475450897669485, -2.77864520754328, 6.81314195983392, -9.17767136168491, 8.1697622215787, -0.0332504769431179, -0.0157765578631612, -0.0133943891250996, -0.0182952076606327, -1.26639041080198, 2.55888156794629, -1.80268636101632, -8.1697622215787, -0.221548830513196, -0.126289880066056, -0.134043576195675, -0.233303075552986, -0.0083381913898609, -0.00503066803073199, -0.00309884550916012, 0.0332504769431172, 0.190431537011323, 0.102131030238349, 0.117279404520323, -0.471535356993021, -0.342047551053799, 0.72939598161483, 1.47589721749269, 1.01220714185214, 1.55693125900042, -4.59936445208088, -4.77265907569933, 10.9803577227012, 0.0440478281967136, 0.0200606676782911, -0.190431537011306, -0.0671852838049547, 0.471535356993006, -0.729395981614831, 0.00971763001855593, 0.00904218311647157, 0.0188754033723288, 0.00666459316414649, 0.00938287403935874, 0.0102511758677786, -0.0791720952376742, -0.122191697916668, -0.0821551374793239, 0.409232834858768},
    {0.0232138437439127, 0.158558075149596, -0.0772323978522295, -0.162307063211102, 0.35145428872232, -0.15388872742481, -0.124486375182814, 0.279788762204936, -0.126062544966409, -0.0568494855714365, 0.0111495902450409, 0.389873654755789, -0.45412409301076, 0.15388872742481, -0.0709380867274232, -3.19169673587895, 4.1862733367843, -1.10541043307144, 0.00820702823413133, 0.302811622374953, -0.361523025747754, 0.124486375182814, 0.0299687006580563, -0.0774919973017003, 0.0499476789121212, 0.0198537994431306, -0.0734404322480771, 0.0277098078727117, -1.64711011447786, 0.737126751174729, 0.976066170910321, 0.843502031359413, -1.0710686229041, 0.102669804288439, -0.119978044940106, 0.100129616053805, -0.0299687006580505, 0.0948944734924744, -0.0198537994431243, -0.0277098078727099, -1.21241040407344, 0.655141006520424, 0.0817342635428154, 0.542586761133695, -0.831890083928969, 0.718466100277049, -0.259575694307695, -0.022637618752114, 0.329606060335684, -0.0214540412444038},
    {-0.138566640473885, 0.14436964159384, -0.258546391444692, -0.0463981325961396, 0.14220532642428, -0.0578765020203706, -0.110575952246123, 0.200414170939772, -0.0391639985698002, -0.0324760640791474, 0.382088705155294, 0.163849194570803, -0.186028874620443, 0.0578765020203707, 0.703248296907838, -1.46180572737149, 1.4816501764658, -0.728895747122102, 0.0479942388799388, 0.138302756227033, -0.262175993089585, 0.110575952246123, 0.132704598354074, -0.275783272801984, 0.0762495624955098, 0.0139576891226348, -0.0410399011292801, 0.0118276460078247, -4.05814755606082, 2.18404350464616, 1.75056173770407, 0.505600743224567, -0.623051805199231, 0.0438235481961615, -0.269265780802283, 0.360771661431511, -0.132704598354071, 0.0536872057719946, -0.0139576891226341, -0.0118276460078239, -0.509744728351653, 0.426770339161937, 0.0617618221498169, 0.274338142614187, -0.525909096819176, 0.219888410936672, -0.830891962912573, -0.0849883886295268, 1.02390818121934, -0.0126473046427167},
    {-0.0867112697486279, -0.0683533897674916, 0.325725544285392, -0.509884452975201, -0.0652094687232026, 0.0204847751966044, -0.0863518149033904, 0.0989135541327083, 0.0203745791918814, 0.0134383636323566, 0.268333407534362, -0.158965367263477, 0.0884517559977631, -0.0204847751966051, 0.291799362514229, -0.302725852411402, -0.00369930727231099, 0.169690456685604, 0.0110705530113326, 0.00635213026241119, -0.134168821282001, 0.0863518149033916, 0.230632347871968, -0.301356482486248, -0.0863854453341406, -0.171108157494915, 0.311141707254319, -0.00627291850183276, -1.06333489443119, 1.74010698392144, -1.27083104130999, -4.61390588340458, 5.28275570364326, -0.0232422872745617, -0.0269321685798278, 0.408767477777853, -0.230632347871964, -0.422040401641766, 0.171108157494916, 0.0062729185018342, -0.0438696971270349, 0.184368027416386, 0.0352552671492951, 0.0717910855301205, -0.211094736870685, -0.0524303050467737, -0.781695366748292, -0.107410995291613, 0.895012980662259, 0.11089869438744},
    {0.258324173993582, -0.301286830201029, 1.40654666417307, -2.52815393023257, 2.02269013545502, 0.288623553688337, -0.0550682258188458, 0.0224439850584832, 0.0168400699068254, 0.0213768470827365, -1.34708568607264, 2.86625354516241, -3.2238211552292, -0.288623553688337, -0.34878679947778, -0.0612210877801336, -0.069012410093766, 0.521982953559126, -0.0204731527591675, -0.0190921563335267, -0.0357718627143633, 0.0550682258188468, 0.23668038790238, -0.109414350988847, -0.112232165559834, -0.395013797789652, 0.249645313218054, 0.324176227066611, 2.1682743404431, 0.55821809899773, -2.78595341754125, -6.78385006662922, 6.44575045169938, 1.20113101977418, 0.127241398781263, 0.174387709328567, -0.236680387902376, -0.397891811478675, 0.395013797789655, -0.324176227066608, 0.0329536660174139, 0.0451873234431331, 0.0133278776558776, 0.00848385854877445, -0.0429352370164308, -0.0423412188897568, -0.301154994838835, -0.0649733583397305, 0.286145761617414, 0.148246498260608},
    {0.0664733009545162, 0.0509205961060789, -0.53225801135685, 2.09010537178824, -4.5253308777861, 8.1697622215787, -0.0236691033128955, -0.00552192775210114, -0.00290443700715087, -0.00245592517947196, -0.146914305631311, -0.484156194529896, 2.63141247240989, -8.1697622215787, -0.219299564551276, -0.0479301246209541, -0.00519311782490771, 0.155028909936551, -0.000677886541337366, 0.000672789605542706, 0.00321091870429557, 0.0236691033128995, 0.135175972111273, 0.0355953982494472, 0.0252133860930542, -0.333297894072582, -0.118193338718046, 0.511154331142247, 1.2643502572938, 0.207078150205771, -0.792256090511431, -3.01014678896068, 1.40419761170234, 1.89391840537621, -0.00584047925373521, -0.0206981936666344, -0.135175972111264, 0.0687276652377291, 0.333297894072576, -0.511154331142246, 0.00583391807410127, 0.00418293826180857, 0.00231100904780673, 0.000955492322069623, -0.00195129086020076, -0.00365559867537359, -0.0363120415895013, -0.0148972045828313, 0.0169391347502096, 0.0494656734803196},
    {0.0307253509165799, -0.277763665240074, 0.588308925193378, -0.933533835729216, 0.622081242620202, -0.15388872742481, -0.0757063131817327, 0.295183781142474, -0.419128833660778, 0.23464695667999, -0.0176781856910624, -0.0308310083721297, -0.196192270366812, 0.15388872742481, -0.231145906488429, 0.904707938978344, -4.39508691957179, 3.96856320140537, -0.00705094261602132, -0.013842202697983, -0.0930951975884561, 0.0757063131817308, 0.0178628963919137, -0.0792873564973449, 0.155664801915619, 0.0114765791187428, -0.0704362054930752, 0.0150146766187935, 0.201785345580319, -2.38793509349722, 1.61551900841459, 0.980991861914712, -0.0166270178133683, -0.42588897225339, 0.00514100574335372, 0.0250056832079877, -0.0178628963919173, 0.0222142031055588, -0.0114765791187378, -0.0150146766187947, 0.0804820651454057, 0.4404360711723, -0.202088583554018, -0.952427676078984, -0.00746503481354077, 0.644349596869611, -0.163578327877601, 0.0542816732893598, 0.00277252021862884, 0.0482220023875155},
    {0.148294415168218, -0.189231932784837, 0.633760217237781, -0.457019490504955, 0.257719653538464, -0.0578765020203695, -0.0660640735715601, 0.213319323186306, -0.220737523925425, 0.0431352531615907, -0.251910113159571, -0.0422020538189083, -0.0759333334893265, 0.0578765020203704, -0.762835589501422, 1.59737491801092, -1.79350820569863, 0.999906394805756, -0.0171456115571014, -0.0203833251274647, -0.0628514243474501, 0.0660640735715606, 0.0779392221679467, -0.285992416229425, 0.406962173977063, 0.00798671967783016, -0.0403014293621209, 0.00640886002474961, 1.69100432584983, -6.93817873052023, 4.86532430059219, 0.543662362384867, -0.0444408180610023, -0.18178632004914, 0.0375796654743693, 0.0842634902647297, -0.0779392221679463, 0.0118742278046461, -0.00798671967783265, -0.0064088600247505, 0.115093844184137, 0.262585483152333, -0.150467898838856, -0.472229224683289, -0.0214646340994409, 0.331145738894659, -0.484115057458974, 0.201728925964693, 0.0395732180075392, 0.0284272015574814},
    {0.288245120659521, 0.157476470390113, -0.872076066213632, 2.13904455634209, -0.126473726995893, 0.0204847751966063, -0.0502251134543614, 0.110191494450976, -0.0481705693270831, -0.0194953665925289, -0.870895760782949, 0.74081869619401, 0.0300614087998774, -0.0204847751966064, -0.99760421558848, 1.10518808917439, -0.210145730480019, -0.343159734155521, -0.0194689807209837, -0.016682989729224, -0.0261913018588236, 0.0502251134543618, 0.132471615139589, -0.329393680842763, 0.196368175885006, -0.0964308034466874, 0.326986899708099, -0.00339900742703543, 3.49448860824024, -5.27130984328822, 3.51979306204456, -4.01834167959562, 1.13847842705952, 0.096412318196013, 0.068008502021062, 0.0782932418543342, -0.132471615139585, -0.0777211765464005, 0.0964308034466865, 0.00339900742703536, 0.0781197181191504, 0.0904917131730387, -0.0840001925921506, -0.117840773069145, -0.0256381541167314, 0.0786854022635049, -0.368891065576607, 0.251100438988418, 0.10451438767054, -0.249265723161696},
    {0.211629929169825, -0.106326707514486, 0.723274251164084, -2.23508757936844, 5.18874690712782, 0.288623553688341, -0.0311077547329553, 0.0322822324989517, 0.00825427673915247, 0.00595627825506759, -0.795255349482014, 0.973470037069573, -0.206284350599026, -0.288623553688341, -0.507082554433024, 0.284085452518064, 0.0872485939133577, 0.0304452863462651, -0.00654905402442452, -0.00359506766419834, -0.00128341572370379, 0.031107754732955, 0.132706663832992, -0.155615023567568, -0.0537639097383116, -0.219006179402854, 0.347007623924245, 0.175656260023977, 2.10718354009035, -1.60763757155523, -0.427564870217189, -2.8159429115199, 4.07756045381877, -4.9824625565288, 0.0234163330730772, 0.00618664672888825, -0.132706663832985, -0.0137956704451772, 0.219006179402852, -0.175656260023974, 0.0173529909008715, 0.0103994010292389, -0.0309988167752499, -0.0132391505629876, -0.0150586101041886, -0.00352106456853074, -0.0677360931717881, 0.149428376838666, 0.0980836698370329, -0.333211953479077},
    {-0.287383368069141, -0.201011180775915, 0.994395503589028, -1.74785530185878, 0.466841577692111, 8.16976222157871, -0.0130409161051935, 0.00031291571361361, 0.0013281541349135, 0.00248619131476435, 1.81307241877918, -4.86684701705693, 7.38940173345124, -8.16976222157871, 0.0453882041920756, 0.0922397772807378, 0.108906774455683, 0.241859792916571, 0.00453305036515159, 0.00369820258157665, 0.00495298625306788, 0.0130409161052019, 0.0742535060640361, -0.00200799333803992, -0.0114248613268688, -0.182255982056159, 0.00660685321642607, 0.276971136705397, -0.476916665310392, -0.78624310476404, -1.54430815229379, 2.15726737803241, 4.4574349408833, -7.85624331114335, -0.0318121598566348, -0.0317835217805608, -0.0742535060640328, 0.104576573602585, 0.182255982056158, -0.276971136705395, -0.0011923888099854, -0.00163925674232296, -0.00526590196668611, -0.00196576791760594, -0.00338709997416671, -0.00386108495233686, 0.0141009845682101, 0.0337915151185919, 0.0291360366153129, -0.111183426819014},
    {0.136246592488563, 3.08041774684675, 2.82690544195193, -1.94822363815396, 0.838592338884274, -0.153888727424811, -0.0267632969820106, 0.138467078595141, -0.297194184216815, 0.364815616315781, -0.0232954564072134, -0.00616373515137372, 0.0101622736935166, 0.153888727424812, -0.789669027444761, 1.85881577239572, -2.04244724899488, -2.24336383529141, -0.00300630723631878, -0.000940254601247859, 0.00167797901910213, 0.0267632969820111, 0.00621587300784401, -0.0363154763493316, 0.105111883455644, 0.00389938824136782, -0.0307222052097955, 0.00485812441505046, 0.120081100044275, -0.118349344142022, -2.80534174144699, -0.0283091750006088, 1.98269654830595, -0.848754612577791, 0.000332550020575084, -0.000440080111469648, -0.00621587300784981, -0.00037229943959971, -0.00389938824136027, -0.00485812441505032, 0.0154966133180059, -0.00431845810994735, -0.140145057614249, -0.0152731280937166, 0.302452896928007, -0.362032794303732, 0.0015273558153166, 0.0367555564608002, -0.106971789291519, 0.03109450464938},
    {0.0508201537761855, 0.152389035168535, 5.95560097140644, -1.01996684985416, 0.350135184354867, -0.0578765020203681, -0.0230500588646325, 0.0997625381024874, -0.166161970812518, 0.131155231768652, -0.0110004824460644, 0.00590658279383584, 0.0121469776576488, 0.0578765020203695, -0.255540141862247, 0.507468973892816, -0.472461497012011, 0.017323476036718, -0.000242254447822221, 0.000962236603998967, 0.00346098700029776, 0.0230500588646357, 0.0268290407030097, -0.131052969298324, 0.293871878784493, 0.0026938448206679, -0.0177158222721526, 0.00207364035532609, -0.0551992442907024, 0.802639357357642, -6.69204060202733, -0.0857207084015164, 1.09978097546184, -0.362282162012518, -0.00170179901735173, -0.00454652248951845, -0.0268290407030033, -0.000614601750806769, -0.00269384482067236, -0.00207364035532387, -0.00121560690737743, -0.013964690959797, -0.103223525102788, 0.017675856969996, 0.179164425168313, -0.147373227383438, 0.0246977689836605, 0.135599491787825, -0.316867848750785, 0.0183304240229589},
    {-0.127595092614205, -0.118966191906676, 0.634469463727495, 6.31394635781805, -0.175487291718018, 0.0204847751966117, -0.0171736078270748, 0.0517636670039687, -0.0474241264972801, 0.0045892362265359, 0.438855364505298, -0.406648411251743, -0.0166529258255799, -0.0204847751966111, 0.421065389531876, -0.400590600726752, -0.0551495321694434, 0.281236027885199, 0.00317432288256442, 0.00305434107327351, 0.00491213066563365, 0.0171736078270739, 0.0448491728896599, -0.152439361979694, 0.18754470734403, -0.0321626844245358, 0.14680077833818, -0.00109977733036504, -1.78665999919862, 2.76617861326414, -2.05284344229832, 2.24392444842935, -8.15122239499565, 0.192140217543598, -0.0120787781457286, -0.0144657847477117, -0.0448491728896562, 0.0139307094483868, 0.0321626844245373, 0.00109977733036609, -0.0129232457377264, -0.0168541433300145, -0.0566757976695991, 0.0200082869654548, 0.0612239287540213, -0.014848600336828, 0.0666518418329147, 0.166905146727387, -0.242117771031202, -0.160731487786574},
    {-0.405296087049493, 0.0924967660603461, -0.480953409540753, 0.747335856394912, 7.72170385245787, 0.288623553688349, -0.0104009318950495, 0.0159678061491313, -0.000909203888063537, -0.00128152661909069, 1.80583279730041, -3.03701648093236, 2.20785138931024, -0.288623553688348, 0.785199765675359, -0.222253028926929, -0.118257963364637, -0.131889452394642, 0.004811740084256, 0.00369481427796076, 0.00456564298039092, 0.010400931895047, 0.0441086502404844, -0.0762892176099255, 0.00581654065197895, -0.0721468465368917, 0.167086868681428, 0.0568350546031866, -3.92260546598521, 1.48614072416041, 1.11158535406514, 8.09282165215101, -5.80314102761356, -9.92955524176811, -0.023637203635841, -0.021813223908608, -0.0441086502404757, 0.0477748152742845, 0.0721468465368869, -0.0568350546031799, -0.0104519964326799, -0.00984567360008981, -0.0205334491295223, 0.0039599031006371, 0.00706006321020124, 0.0029618798668708, 0.0629867090223133, 0.0981024415185128, -0.045166046038434, -0.214861683955726},
    {-0.729420397946708, -0.287417905294881, 1.65533088393823, -3.90144231251835, 4.4607553970921, 8.16976222157871, -0.00427583515645609, 0.000977990931073203, 0.000966203266164312, 0.00133909953976219, 4.03187699731425, -9.20606324283967, 11.1959607479293, -8.16976222157871, 0.559465158668686, 0.164563918856158, 0.123382529296815, 0.169426696419943, 0.00326163468825724, 0.00227990770101716, 0.00245463987633615, 0.00427583515646796, 0.024287652374543, -0.00625318760230087, -0.00825131754117292, -0.0593994058346444, 0.0204261346086358, 0.0896163317834988, -3.35200748678032, -1.12801283727973, -1.20718755719245, 8.87209810346025, 4.23540745189776, -15.6567161450214, -0.0194702740771402, -0.0156947504877887, -0.0242876523745446, 0.0512671466943727, 0.0593994058346474, -0.0896163317834935, -0.00271164618897646, -0.00219720028601261, -0.00343263080741829, -0.00091251935545611, -0.00104891068116836, -0.000976568683600215, 0.0187639577479173, 0.021947938090086, 0.00895763387041093, -0.0716932813030171},
    {0.113928564117325, 0.349323335271632, -0.0867591749077276, -0.0570689004231543, -0.0670256254939572, -0.153888727424815, 0.0278488230036587, 0.0177636344220033, 0.0297013731175999, -0.536720692663425, -0.897392725924526, 1.11679585815058, -0.667429939275518, 0.153888727424815, 0.6889149961637, -2.46045115507582, 2.93322462338119, -1.62494036385803, -5.55156553714957, -0.581233740849927, 0.176887292825777, -0.0278488230036634, -0.00997694195012544, -0.00990278282759025, 0.147598961598273, -0.0133036865046581, 0.117709945429906, 0.232883875824241, 1.54388831955444, -1.01130865619545, 0.451572237473268, -1.4243527454648, 0.364625787737368, 0.734455564769479, -2.88840169966596, -0.0986102508192861, 0.00997694195011878, 1.17213589804522, 0.0133036865046576, -0.232883875824245, 9.5509991784434, 0.741300989338684, -0.194650927247775, -6.25628681954351, -0.189768621606353, 2.79357387091309, 3.68384504732797, 0.108513033646873, -0.943042309260274, -1.28984584347512},
    {0.269990545910721, 0.425653570551384, -0.515016292023659, -0.0524324561977673, -0.0364185135341686, -0.0578765020203712, -0.0363474142259871, -0.0234013737801547, -0.0351920029423213, -0.543690547444172, -1.30154270918084, 0.557676967080657, -0.277076277496998, 0.0578765020203713, -0.174909704706172, -1.17668753447558, 2.31926020441892, -1.66330708169927, -1.37400792758611, 0.374305742846376, -0.17804036755202, 0.0363474142259868, 0.0605051785489994, 0.0561605731551376, 0.724174057704709, 0.0110986659306151, 0.0883321016525762, 0.0994040830897167, 3.38168720858408, -3.79673023921943, 2.23160203183985, -0.779929820737541, 0.274685309854655, 0.313494791031168, -7.70238934861216, 0.427276158246274, -0.0605051785490121, 0.672040880702619, -0.0110986659306109, -0.099404083089723, 3.56996739364429, -0.523479053559308, 0.201441741332175, -4.0081185279557, 0.18436531365526, 2.35584960934168, 10.7720481542588, -0.483436731401409, -3.79383286335138, -0.7603729823552},
    {-0.127458660661963, 0.555572946106277, -1.45661822356639, 1.01456732457268, 0.0295255943487227, 0.0204847751966055, -0.140425692842851, -0.0914936594648406, -0.120072096778313, -0.50506376859421, 0.835345906031967, -2.86220224093332, 0.136739719441227, -0.020484775196604, -0.0591907819839724, -0.0605330432107258, 1.25499365364475, -1.56338411389436, 0.289645525886158, 0.338736145102242, -0.423727874132199, 0.14042569284285, 0.471158927720341, 0.404122350622944, 1.4446265206555, -0.506774857797255, -1.18400114644809, -0.0527200181302874, 0.429171571301532, -4.29782015947264, 4.48992090570553, 5.54754735035447, -3.69991243399383, -0.16626531378995, -4.07544493557717, 1.87158220056294, -0.471158927720344, -5.48337766451797, 0.506774857797256, 0.0527200181302832, 0.148809762000874, -0.656541587927229, 0.515221533597042, -1.49021425886649, 0.437877539603301, 1.55682273957367, 7.89906577199252, -2.27570455118587, -5.26824735707084, 6.66737881096607},
    {0.0599574981188586, 0.719267259522419, -2.9685039649583, 4.53211724574025, -2.8730940690759, 0.288623553688341, -0.207786421496728, -0.136837646941047, -0.158574389013702, -0.379169193294767, -0.485316152903589, 1.81264552367904, -5.71927997511735, -0.288623553688339, 0.0945023567458139, 0.0916757697803168, 0.219204547283424, -1.18460743145083, -0.0619897889177819, -0.0634227097028423, -0.272393731348949, 0.207786421496726, 1.01589169451493, 0.816368635617823, 1.69091147770454, -2.07493166787512, -2.98023020522093, 2.72450161170936, -0.690957544332397, -1.14795174546314, 5.29272940765743, 3.00850080875389, -9.35326357817318, 8.59237404419324, 0.676289459165992, 1.62509151379949, -1.01589169451493, -5.93254885644662, 2.07493166787512, -2.72450161170936, -0.0882565149914314, -0.105264526870701, 0.409231378290003, -0.146628720192623, 0.327261625587238, 0.676044217396605, 1.12245740177769, -2.44146014941732, -3.48965833864822, 8.91277906166757},
    {0.226504621301652, 0.88027078582891, -4.75479662623642, 10.5319042888745, -12.2449015666311, 8.1697622215787, -0.149329588787135, -0.099101141488931, -0.104760298900417, -0.184387202954388, -1.17098067011925, 2.26640789781616, -1.30337508285043, -8.1697622215787, -0.191316311349798, -0.145361694694602, -0.190537467699463, -0.579559933386698, -0.0454096920329887, -0.0225438403438938, -0.0105485501697049, 0.149329588787132, 0.896566440891595, 0.689437434298067, 1.06776529776709, -2.3857702266989, -2.68784280430429, 4.29593746472688, 1.31378384172699, 1.25370352709728, 3.35828992753139, -4.33677010791263, -8.46154207877803, 13.5482766494815, 0.229777221430829, 0.0733852835123956, -0.896566440891577, -0.286100081628694, 2.38577022669888, -4.29593746472688, 0.0509474845939764, 0.0431376245269564, 0.10964969165865, 0.0486176181374394, 0.0841665147173481, 0.130231792255951, -0.439678570808326, -0.762822717810493, -0.857863948389564, 2.97394288593301},
    {0.240503275451298, -0.267111615790657, 0.0747872960979263, 0.059235498807905, 0.0992497134834048, -0.153888727424811, -0.0462721460137365, 0.0400491118444152, 0.0394640129859651, 0.386429805992919, -0.198934346621587, 0.639724604203334, -0.539525186253462, 0.15388872742481, -1.12628789153541, 2.37685767348729, -2.53086221513044, 1.30690077351792, -1.02790400216701, 0.426198826646041, -0.217708482662351, 0.0462721460137359, 0.0151904318899105, -0.018771017464659, -0.127960434502698, 0.0170299814591469, -0.145583247685152, 0.194513289868925, -0.648679727349184, 1.2057222144214, -0.432895436548559, -0.206125491347612, -0.492834611663628, 0.44027547277006, -1.38193211778959, 0.102039958991733, -0.0151904318899116, 0.791396025902324, -0.0170299814591432, -0.194513289868926, -3.35176151926737, -0.137325408428824, 0.177659370817933, 6.23002870420839, -0.328337431203179, -2.23679298876692, 0.445271973153438, -0.0832689415270776, 1.06462057913886, -0.645812778217177},
    {0.00303876266691733, -0.299846929362778, 0.386955294238902, 0.0431773246303445, 0.0345544018425535, -0.0578765020203709, -0.1034560301836, 0.0589323903674588, 0.0675232169078045, 0.341194063853184, 0.112743231543428, 0.297246090911907, -0.222481451419091, 0.0578765020203707, -0.0973691697225817, 0.960002280146955, -1.78301880835979, 1.21719386463128, 0.0994102469069545, 0.464850762373005, -0.379441201277077, 0.103456030183601, 0.160592066427415, -0.124155598664817, -0.498090602613621, 0.0258597541924534, -0.0700018610853214, 0.0830259937909026, -2.09985897112538, 3.38749524490984, -1.78733479956679, -0.0857032101138527, -0.254720205428399, 0.18792704957654, -3.42901015323231, 0.799386368156972, -0.160592066427418, 0.45071292876847, -0.0258597541924518, -0.0830259937909037, -1.85153020657341, -0.134027675307745, 0.320508810909616, 2.98689095640217, -0.398346303973066, -1.5759650605889, 0.988666248708012, -0.675230769492153, 2.93843450713792, -0.380711067683153},
    {-0.157996351774294, -0.260145422130515, 0.651010728103405, -0.408482387606867, -0.00811565004606918, 0.0204847751966049, -0.179040800534354, 0.0305979835068194, 0.0542547024805229, 0.188537819609357, 0.685081437210633, -1.02836080694757, 0.10778476979344, -0.0204847751966048, 0.257921546868496, 0.115897235305268, -0.750804511054163, 0.795127502785209, 0.198404964543105, 0.136587063032125, -0.406374916328886, 0.179040800534354, 0.573250668203956, -0.124663799803553, -0.485800443701226, -0.568745387183396, 0.271823277563598, -0.0440337233832833, -1.62685937719308, 2.45252236017477, -2.16175514829572, -0.0225965342211385, 1.45943972877558, -0.0996691197473701, -1.22301022383586, 1.65567254466686, -0.573250668203958, -3.61011246546788, 0.568745387183395, 0.0440337233832825, -0.47115125226984, 0.00300127564481062, 0.375776932822067, 0.710269736533589, -0.19384304115746, -0.626061268416209, -0.0268736344179965, -1.53100874486331, 1.73568430195508, 3.33828918790428},
    {0.18815737454253, -0.0105569750572724, -0.13524370810424, 0.634253162364353, -0.9278437702821, 0.288623553688339, -0.206275655835312, -0.0427692121532615, -0.0206152809481796, -0.0144285384094618, -1.09311974803885, 2.729017019739, -4.22292603253327, -0.288623553688339, -0.0756985794411016, -0.0770435263672285, -0.184674213667699, 0.15981591999077, -0.116619992839635, -0.0887018873734195, -0.19465692951523, 0.206275655835312, 0.983213995803707, 0.244909723438415, 0.197647914309173, -1.92751827935447, -0.803867890279711, 2.27560525549965, 0.577312226379414, 0.992114327298396, -0.341063097534721, -2.67220890933718, -0.691061272766169, 5.15076980281537, 0.850424647557035, 1.11466572267242, -0.983213995803706, -3.65867051038954, 1.92751827935447, -2.27560525549965, 0.061590825549888, 0.0868554398890976, 0.237426141668494, 0.105844182170549, 0.0224617284324975, -0.0363864764713373, -0.832721930088643, -1.35957544611084, -0.215350631777561, 4.46253840066925},
    {0.143182682873854, 0.475450897669485, -2.77864520754329, 6.81314195983392, -9.17767136168491, 8.1697622215787, -0.131948528310843, -0.0650083811815061, -0.0584471218103459, -0.0899998502224726, -0.617043178048826, 0.712046529486201, 1.05604468581899, -8.1697622215787, -0.223891097108792, -0.1188637791641, -0.114280271279296, -0.161598432991147, -0.0199859245989384, -0.00610835213604982, 0.0074803022220899, 0.131948528310843, 0.784686118124592, 0.445654124962655, 0.576934082236388, -2.05756934419683, -1.68263891478267, 3.58812702845014, 1.42125680067031, 0.877155003637424, 1.09727658128436, -4.09312077734966, -3.43206771197046, 8.12162667586592, 0.0602958096896277, -0.0512799654545062, -0.784686118124579, 0.193615767451332, 2.05756934419682, -3.58812702845015, 0.0460342683696064, 0.0351131871416449, 0.0575280789594255, 0.0284109028151446, 0.0294422868047463, 0.0355406036366499, -0.346603797937062, -0.39437415950817, -0.290626093988928, 1.48902314733135},
    {-0.161958917611736, 0.158558075149595, -0.0772323978522259, -0.162307063211105, 0.35145428872232, -0.15388872742481, -0.159080911263008, 0.432664849446292, -0.265714169326551, -0.27966040514886, 0.0765730592183343, 0.151585061594867, -0.345520686816804, 0.15388872742481, 0.898837018621739, -2.23172205862127, 2.21888539595569, -0.882599513494011, 0.277272923798914, 0.248161096178579, -0.425360169840707, 0.159080911263007, 0.0463435459527745, -0.163337347329965, 0.245708258929762, 0.0418477655549009, -0.361276462405173, 0.136313214066664, -0.746529146484305, -0.0331171057744781, 0.780305590892677, 0.793954594363222, -0.783232592746987, -0.00593360190551573, -0.229476775855351, 0.160579723290477, -0.0463435459527747, 0.355177032765154, -0.041847765554898, -0.136313214066664, -2.70320033259493, 1.29978930892136, -0.00730467960558512, -0.119917851520914, -1.28223623577339, 2.8255056654658, -1.20192675038754, 0.00275762403948621, 1.18569526731313, 0.00609942964001786},
    {-0.13864766091785, 0.144369641593841, -0.258546391444692, -0.0463981325961387, 0.14220532642428, -0.0578765020203708, -0.186910868101331, 0.397478102435846, -0.100329151778505, -0.159759919491315, 0.241970447233905, 0.0410426412418445, -0.139672629582347, 0.0578765020203706, 0.580710587391402, -0.850486150117402, 0.865665473759946, -0.601611891709938, 0.149517380433584, 0.0887486877626282, -0.39039896159007, 0.18691086810133, 0.263190829725002, -0.706493280700452, 0.375095452941697, 0.0357563875395928, -0.201888113176696, 0.0581838910459217, -1.33721875556561, -0.0979211474814876, 1.45171584725789, 0.467559084506106, -0.462203593151813, -0.00253269684193314, -0.331800166194943, 0.693910536116482, -0.263190829725002, 0.198292457517916, -0.0357563875395949, -0.058183891045921, -0.826288696344618, 1.01102789552214, -0.00707914084577549, -0.060507031448885, -0.999447431506261, 0.897038266851232, -3.7798781279922, 0.0125827445839717, 3.73658284124544, 0.00359565565878373},
    {0.0784651408728714, -0.0683533897674908, 0.325725544285392, -0.509884452975201, -0.0652094687232015, 0.0204847751966044, -0.208140763677231, 0.267276533181229, 0.0658715898373908, 0.066107514961414, -0.358197104159229, 0.535979548866735, 0.063866225877687, -0.020484775196605, -0.145175181233355, -0.0695376468745371, 0.0875797716459691, 0.117021305356543, -0.0726977691427045, -0.0692427956925714, -0.261770932568324, 0.208140763677231, 0.623196840101038, -0.974294017178111, -0.424957031682426, -0.553197504703679, 1.53060340009786, -0.0308584486219029, 1.04317677449815, -0.0784457596625998, -0.93225945496171, -4.08938910669126, 4.06329401079973, 0.00134324284551413, 0.446705673804824, 0.954224639315861, -0.623196840101037, -1.49907466498073, 0.553197504703676, 0.0308584486219041, 0.211717580759074, 0.528305109813902, -0.00550560061290657, -0.0159209319672239, -0.524933903958721, -0.18920639461056, -3.40825190106056, 0.0200693778622507, 3.38650325893816, -0.0315287351171247},
    {0.301798607677328, -0.301286830201029, 1.40654666417307, -2.52815393023257, 2.02269013545501, 0.288623553688338, -0.183558386739004, 0.0799279514730895, 0.0663981872654402, 0.105159398644887, -1.4000385651718, 2.49182453126872, -1.95327310133187, -0.288623553688338, -0.445702012696598, -0.00750135259921819, 0.0144911858225418, 0.438200401996975, -0.104672825540183, -0.0654440498583804, -0.0771848910123978, 0.183558386739004, 0.84287075176702, -0.431406437543365, -0.552105134737892, -1.55748760326961, 1.22808339840383, 1.59472428096394, 2.36718295409855, -0.0276106047366207, -2.3460804483632, -5.43098296754975, 5.46731236651361, -0.0694170341231415, 0.544171421734887, 0.416600929338737, -0.842870751767019, -1.18593660306493, 1.55748760326961, -1.59472428096394, 0.176980645062193, 0.142636656653898, -0.00274306046069311, -0.00206428600221809, -0.143590794060957, -0.175402932164678, -1.18603283890328, 0.0148055082046243, 1.19396655190629, -0.042146795338905},
    {-0.0586054310955214, 0.0509205961060793, -0.532258011356852, 2.09010537178824, -4.5253308777861, 8.1697622215787, -0.100463988122248, -0.0240591474913986, -0.013166310545128, -0.0120814643052636, 0.575471048921233, -2.18852029690419, 4.63478621313925, -8.1697622215787, -0.160269004707359, -0.0183398474773682, 0.0216392381118354, 0.164654449062341, 0.0130623359121072, 0.0137862608518718, 0.0246410722452075, 0.100463988122252, 0.588964361057172, 0.16136003851149, 0.124032534315479, -1.51089645482929, -0.581430010483474, 2.51452807187161, 0.860428183121139, -0.0125659819516815, -0.891075238733852, -1.76901935835182, 1.86743428346777, -0.109455335353148, -0.129872743494111, -0.165262895032901, -0.588964361057165, 0.595493206855313, 1.51089645482929, -2.51452807187161, 0.0195304385463497, 0.0111436765566076, -0.00058192475380678, -0.000285229079066515, -0.0117636268633542, -0.0202260810741368, -0.104978417466961, 0.00390285652139888, 0.110818626645611, -0.0140631963718358},
    {0.186613680044608, -0.277763665240073, 0.588308925193382, -0.933533835729221, 0.622081242620203, -0.153888727424811, -0.206357552845445, 0.886330083065449, -1.46329810777014, 1.15430179028823, -0.0592422377411611, -0.0565262386342753, -0.137345055287173, 0.15388872742481, -1.10023813827126, 2.82174955621817, -4.67926980054856, 3.04890836779712, -0.11623726609761, -0.088603899362989, -0.195686746233608, 0.206357552845445, 0.0536358142088675, -0.276814739084716, 0.765763861072013, 0.0400680057286023, -0.346497731111145, 0.0738618916984276, 0.440338302452322, -1.97482493916272, 1.00541994925819, 0.730625566558763, 0.259434507804732, -0.484736187333027, 0.0463676292082589, 0.0611160296101372, -0.0536358142088689, 0.0765008599775493, -0.0400680057286, -0.0738618916984299, 0.863973448449906, 1.14524291259219, -0.690643336831841, -3.87473972459629, 0.406659094540939, 1.97270175195576, -0.599321238751768, 0.215698709474578, -0.212810251528506, 0.269996871133596},
    {0.262628331893209, -0.189231932784835, 0.633760217237781, -0.457019490504956, 0.257719653538466, -0.0578765020203704, -0.198874814606376, 0.696999944491397, -0.818855703119534, 0.212195805364135, -0.371819690231103, -0.0477770585860836, -0.0508150060110035, 0.057876502020371, -1.24548839225751, 2.30045745660786, -1.95921130606192, 0.830845842603203, -0.124492791552472, -0.0856036070983841, -0.13742862014097, 0.198874814606376, 0.254658756240829, -1.06092755284112, 2.0019742537807, 0.029627816970616, -0.198255339519798, 0.0315271875030749, 2.09417343488009, -5.62642618267533, 3.27031222078856, 0.391283456994369, 0.113513092096671, -0.206904647527464, 0.209287444404246, 0.209184822479261, -0.254658756240826, 0.0390903298646123, -0.0296278169706206, -0.0315271875030757, 0.701171841494476, 0.701074455144475, -0.559571324350428, -1.88384187375821, 0.203384855073444, 1.09496701845207, -1.71401750495926, 0.851742730361856, -0.497244193225689, 0.159165009655193},
    {0.297257923557779, 0.157476470390114, -0.872076066213631, 2.13904455634209, -0.126473726995891, 0.020484775196606, -0.174529674546856, 0.406388232198369, -0.194647380835119, -0.0959038074837375, -0.807857834631562, 0.582773602692335, 0.0167396352982317, -0.0204847751966059, -1.00399696465102, 1.10343805136222, -0.287424187394774, -0.266751293264316, -0.0888416105525319, -0.0530308520444718, -0.0537881737027813, 0.174529674546856, 0.488557721821178, -1.33101223704313, 0.965996491865109, -0.389657078687915, 1.60855085901936, -0.0167207809286759, 3.03032836225609, -4.10055920747536, 2.75016474606445, -2.57873262678269, -0.143085532251725, 0.109734091697658, 0.263181640645712, 0.176168283759884, -0.488557721821176, -0.212902358285947, 0.389657078687912, 0.0167207809286765, 0.333250778373189, 0.234657828977474, -0.35260005849559, -0.450946031023225, 0.0130204039021178, 0.302440670686304, -1.16456044056887, 1.15484395328324, -0.0646176919419488, -1.39564850073341},
    {0.105970427076989, -0.106326707514486, 0.723274251164084, -2.23508757936844, 5.18874690712782, 0.288623553688342, -0.125371565743527, 0.134743224057092, 0.0363371457344555, 0.0293007961857386, -0.287413545533684, 0.0559394161257737, 0.482167487303576, -0.288623553688341, -0.321414156721481, 0.250240238067649, 0.0644294306757365, 0.00710076841559382, -0.0116435027310175, -0.000909440298818676, 0.0125210966997736, 0.125371565743527, 0.553906045354711, -0.685051636690271, -0.264481492289191, -0.96411347828558, 1.70703906501424, 0.86410809792658, 1.08267343025857, -1.30168684822266, -0.216847287666314, -0.538380849486119, 2.71752901272879, -5.6709143944314, 0.00661940068537259, -0.0636588433100205, -0.553906045354706, 0.158627651615897, 0.964113478285578, -0.864108097926577, 0.0438605321075189, 0.00875277710323707, -0.147264320756868, -0.0527330552360317, -0.0441804825388695, -0.00878477032621158, -0.0637074680233538, 0.748710480000283, 0.321569559627178, -1.86566671663014},
    {-0.397585858525558, -0.201011180775915, 0.994395503589028, -1.74785530185878, 0.466841577692112, 8.16976222157871, -0.0593214248111, 0.0014435483733686, 0.00625823945879744, 0.0122303528937862, 2.38339251504387, -6.02882067961086, 8.47493826989333, -8.16976222157871, 0.135326325796608, 0.112596325648857, 0.118558756518469, 0.232115631337548, 0.0293140218738281, 0.0215863426605568, 0.0262058564159218, 0.059321424811109, 0.342547603810881, -0.00946163010062673, -0.0562024671872934, -0.858787055312676, 0.0325011779560898, 1.36250767314749, -0.986107396037847, -0.892150076161703, -1.49953054643336, 3.34513536532599, 4.43154061614364, -8.94177984758544, -0.193857349669388, -0.171764330485656, -0.34254760381088, 0.59001916289999, 0.858787055312676, -1.36250767314748, -0.0121284151036611, -0.0119773405246656, -0.0276494047892946, -0.0109728073249575, -0.0158672415946899, -0.0184431523390068, 0.107563172081135, 0.181225960586279, 0.14249664477556, -0.622520340856081},
    {0.0167231769272591, 3.08041774684675, 2.82690544195193, -1.94822363815396, 0.838592338884272, -0.153888727424811, -0.111215180556966, 0.592811184964297, -1.33122411777891, 1.79464215090037, 0.00592046743457608, 0.0122940972627764, 0.0292027831626315, 0.153888727424812, -0.0590517234941792, -0.0369354053732147, 0.672036574969377, -3.673190369876, 0.00375856944256564, 0.00840057499663072, 0.0206438047286824, 0.111215180556967, 0.0266116616364576, -0.16266818306794, 0.517078239454578, 0.0174665587254641, -0.151132139010329, 0.023898633884164, -0.0747350964838835, 0.459217284543293, -3.21730809744593, -0.167176941215313, 2.10310648210651, -0.867795122046904, -0.00326297763960803, -0.00566468766446332, -0.0266116616364603, -0.00526296136965665, -0.0174665587254573, -0.0238986338841644, -0.0474450798074719, -0.114232253281347, -0.613454989692984, 0.29153104417044, 1.43705579606362, -2.04248668470588, 0.0443704494432628, 0.168332870732398, -0.558185711258219, 0.15639510037997},
    {-0.0748919790983824, 0.152389035168536, 5.95560097140644, -1.01996684985416, 0.350135184354867, -0.0578765020203685, -0.0982464118371551, 0.4361732381481, -0.75501452175656, 0.645193617587336, 0.13588649724902, 0.0198848675421218, 0.020274222958078, 0.0578765020203697, 0.380511545357719, -0.780062924499114, 0.818769232853208, -0.496714909781971, 0.0147211173418602, 0.0147194624606256, 0.025256169256034, 0.0982464118371577, 0.117299637543452, -0.59548460129415, 1.44564771090949, 0.0122404178827665, -0.0871496722331326, 0.0102008856557561, -0.855058167091082, 2.60738713258794, -7.84381643415233, -0.16913284311078, 1.16921482542282, -0.370409407312946, -0.0281837720981979, -0.0344809322633183, -0.117299637543444, -0.00504631343699313, -0.0122404178827717, -0.0102008856557532, -0.0926318057105893, -0.125197944102678, -0.461429407404135, 0.282468477086011, 0.865493003398609, -0.849751406304607, 0.23972005317398, 0.62996553355745, -1.65718399198526, 0.0921959856701229},
    {-0.250468856650959, -0.118966191906676, 0.634469463727495, 6.31394635781805, -0.175487291718018, 0.0204847751966115, -0.0760925285239276, 0.233678626336287, -0.220216791197608, 0.0225758887620093, 0.741597708025432, -0.59617178467272, -0.0209632972536968, -0.0204847751966105, 0.864776154428896, -0.960584571721968, 0.201994090500982, 0.263249375349725, 0.0263877590958996, 0.0207931822639925, 0.0279146966014816, 0.0760925285239268, 0.20246427117246, -0.707861369872468, 0.922591089649212, -0.149349364625152, 0.722158955944213, -0.00541014875848006, -2.94278434675721, 4.35460699960778, -2.7878898246035, 3.00880599945635, -8.72658057260168, 0.196450588971714, -0.087112361313977, -0.0845594468167976, -0.202464271172458, 0.0862674026687456, 0.149349364625155, 0.00541014875848156, -0.104711063118274, -0.104940644881467, -0.261593322937766, 0.154946973567289, 0.304364253815084, -0.0991995583069223, 0.439645421146841, 0.792420816689248, -1.27512414948206, -0.808426358612968},
    {-0.502239635580981, 0.092496766060346, -0.480953409540753, 0.747335856394912, 7.72170385245787, 0.288623553688349, -0.048095265712099, 0.0746638970428194, -0.00432325040203874, -0.00630423037075758, 2.17143575456295, -3.51888276364892, 2.43060574940825, -0.288623553688348, 0.995935126434362, -0.33317187352825, -0.126153634742499, -0.126866748642975, 0.0284626971354931, 0.0203563246598785, 0.0235023902616742, 0.048095265712097, 0.20624772680061, -0.36275404782907, 0.0286133831984429, -0.343057137552263, 0.821952580939268, 0.279589414701194, -4.75396923780331, 1.97469838126244, 1.08878851151867, 9.2295536471254, -6.45800673987139, -10.1523096018661, -0.134728101274686, -0.114186206972176, -0.206247726800604, 0.258730806975471, 0.343057137552259, -0.279589414701188, -0.0623139719067021, -0.0533918868930499, -0.0981662873044943, 0.0258839073832529, 0.0373588126352182, 0.0142715977587073, 0.353373590997565, 0.476940254801228, -0.247258872921306, -1.08068338791475},
    {-0.778443022056055, -0.287417905294881, 1.65533088393823, -3.90144231251835, 4.4607553970921, 8.16976222157871, -0.0205164010902988, 0.0047138399252431, 0.00468870804797139, 0.00658744957964241, 4.26837451395494, -9.64375602134095, 11.5471951890703, -8.16976222157871, 0.609933789752645, 0.168389984706707, 0.123358806511534, 0.164178346380063, 0.0169861519353357, 0.0115897539544476, 0.0122023345513319, 0.0205164010903109, 0.117064588039241, -0.030344930579165, -0.0405908124476984, -0.288248324077245, 0.100482546550085, 0.44085077292443, -3.61066138287928, -1.13819595272799, -1.17484806228592, 9.38984729390298, 4.15535103995631, -16.0079505861624, -0.10033414839883, -0.07855145714209, -0.117064588039244, 0.260111006953053, 0.288248324077247, -0.440850772924425, -0.0143687585604613, -0.011284609395479, -0.0169161744765839, -0.0045294922744033, -0.00499385260693996, -0.00467535068012651, 0.0976924685510077, 0.108896387721253, 0.0432324922955332, -0.360593553503146},
    {-0.105791126909819, 0.349323335271631, -0.0867591749077278, -0.0570689004231547, -0.0670256254939589, -0.153888727424815, -0.137827978438839, -0.248651653930035, 1.21846090748446, -1.16291441591096, 0.068517836679807, 0.250681372099602, -0.395723587573241, 0.153888727424815, -0.0616728150825679, 0.0234798980621419, 0.793407349269107, -0.998746640610491, 0.918408688265111, -5.3522224867339, -1.46805529712027, 0.137827978438838, 0.139655154915356, -0.406249020977913, 0.319803135154601, -0.54576675183252, 0.255042509647347, 0.504590227526505, 0.023170519220358, -0.284297244909364, 0.279368063916936, -0.42090569519639, 0.227293223519938, 0.462749213067203, -1.4047701660255, -2.39852025019858, -0.139655154915363, 1.50578731876832, 0.545766751832529, -0.504590227526509, 0.310576153521044, 8.98663074865171, 1.71670695105031, -3.81070203653351, -4.85286916940227, 3.74463161065832, 2.35867451326692, 2.80476927117651, -1.27370748239603, -1.76082982841567},
    {-0.166507481624836, 0.425653570551384, -0.515016292023658, -0.0524324561977686, -0.0364185135341701, -0.0578765020203703, 0.00659520248347825, 0.00871125236244275, 0.890030660034573, -1.17801602222524, 0.355375918844869, 0.0912942330174316, -0.161101220550594, 0.0578765020203707, 0.0739985701518116, 0.117814001117102, 0.578022946722739, -1.0289816069182, 0.812864627383562, -1.5497016993325, 0.0385351639022038, -0.00659520248347614, -0.0225232879285911, -1.4203406403184, 1.56907021258007, -0.280693115987706, 0.191389442970318, 0.215379140036128, -0.249065371192063, -0.978000132593625, 1.38670587696448, -0.210489745356579, 0.171627968536913, 0.197519734084764, -2.73203034905887, -6.28302993581031, 0.0225232879285885, 0.84663183284753, 0.280693115987709, -0.215379140036131, -0.569696536575304, 3.57302214268966, -0.0472464162646473, -2.23701627264399, -2.91335110339174, 3.17186420406097, 6.29902189298244, 7.70337057612873, -5.13606175650365, -1.03802127581785},
    {-0.00460700210468695, 0.555572946106276, -1.45661822356639, 1.01456732457268, 0.0295255943487234, 0.0204847751966051, 0.116533764631271, 0.114924157681887, 0.372959141873575, -1.09432325878451, -0.0718887651732678, 0.271878352533974, 0.0752311081766617, -0.0204847751966046, 0.112055350153562, 0.0992514455241687, 0.211851884024745, -0.974124623704066, -0.0540083506450925, 0.0999436061059611, 0.292826340312173, -0.116533764631271, -0.591817435429704, -1.25525521036366, 3.13007683408069, 1.57410690042444, -2.56537901459857, -0.114228629394848, -0.522265034354513, -0.753698569186149, 2.80447059228032, 1.03208888873669, -2.31853456584334, -0.104756702525386, 0.838781333030841, -3.19838575998988, 0.591817435429706, -6.53657649976356, -1.57410690042444, 0.114228629394847, -0.392365525226444, 0.37940014127954, -0.407750497994058, -0.566236135885747, -0.852302889259079, 2.10693327054179, 3.18413321925916, 4.45364097035355, -7.15299138637069, 9.10195551436213},
    {0.290022490838602, 0.719267259522418, -2.9685039649583, 4.53211724574025, -2.87309406907589, 0.288623553688341, -0.0532327497944595, -0.0428640277409228, -0.0705940464802365, -0.821547085810462, -1.41899947092762, 2.74129544088452, -2.54059536028731, -0.28862355368834, -0.0857969510501969, -0.0697237102610323, -0.111539550114656, -0.742229538935135, -0.392714611086447, -0.0426994994341222, -0.0379034404664838, 0.0532327497944594, 0.318225362310003, 0.363430474279363, 3.66370322652156, -0.923716771140336, -6.4572741758589, 5.9031862265394, 0.501980023649316, 0.565585753396195, 3.31993765884041, -1.39719307908958, -5.87621960753519, 5.41368942936321, 2.2160267281384, 0.321371230646184, -0.318225362309996, -5.70998387691732, 0.92371677114033, -5.9031862265394, 0.138925273616722, 0.0217632306974852, 0.0807674682074126, 0.156528451018966, 0.0915303152168629, 0.918807972261225, -1.12947227849089, -0.684801704925557, -4.75025767616907, 12.1672580527762},
    {0.00324591462099711, 0.88027078582891, -4.75479662623642, 10.5319042888745, -12.2449015666311, 8.1697622215787, -0.207698228045336, -0.147879469090223, -0.174648000219722, -0.399512333614497, 0.223681572845264, -1.37517423910197, 3.7087086930266, -8.1697622215787, -0.206139781811647, -0.145023446147852, -0.167918669763816, -0.364434802726591, 0.0187943994619912, 0.0228041790188372, 0.0447894063950375, 0.207698228045335, 1.33786318998118, 1.14937500599567, 2.31353043502173, -3.97736550439588, -5.82375747302903, 9.30802124060392, 1.31281424654277, 1.10577601657163, 2.11252479027675, -3.83110263971924, -5.32562741005328, 8.53619287360446, -0.302082830260939, -0.348120158670758, -1.33786318998117, 1.76388677759247, 3.97736550439586, -9.30802124060392, 0.110306607089056, 0.0635302407153787, 0.103090062695201, 0.0929106314238247, 0.0883135804854972, 0.177500695639618, -0.841573595199246, -0.801254847324923, -1.16987400956153, 4.05987069543657},
    {-0.164081106760814, -0.267111615790657, 0.0747872960979294, 0.0592354988078989, 0.0992497134834096, -0.153888727424812, 0.0258474166283122, -0.0442783824868061, -0.750189117397994, 0.83727867822052, 0.066545891578804, 0.0968987680588138, -0.312585918084556, 0.153888727424811, 0.393739939846182, -0.0156716242350008, -0.802927494350031, 0.856051901290318, 0.745012308897368, -1.22717631740947, 0.139454295182939, -0.0258474166283137, -0.0167945735219864, 0.356826687379335, -0.27725227661892, -0.323730553319045, -0.315435681467856, 0.42145255803783, -0.240753738827887, 0.38302414558349, -0.283603594432336, 0.166847911014199, -0.322982177880915, 0.213336204601148, -0.453535541804052, -1.1238218606061, 0.016794573521988, 0.993461327268657, 0.323730553319047, -0.421452558037832, -2.6953504504096, -2.11304858779617, -0.0951759126961348, 4.28813404245509, 4.09041402260363, -3.17507457916337, -0.780933124709776, 0.766995173226756, 1.51172094313275, -0.6780256458008},
    {-0.0960766222659437, -0.299846929362777, 0.386955294238903, 0.0431773246303433, 0.0345544018425519, -0.057876502020371, 0.104452923995495, -0.0990378982800393, -0.40880692731083, 0.7392662531961, 0.139181614087523, 0.0141621012778633, -0.125614762862231, 0.0578765020203707, 0.235757381825312, -0.0807308548564222, -0.578224650628529, 0.819121675288359, 0.265902216333921, -0.134088092702238, 0.360030023483157, -0.104452923995495, -0.269880462004838, 0.75167729149964, -1.07921447808308, -0.15656313689652, -0.151673252977714, 0.179892682347763, -0.44467877762162, 1.12475279339252, -1.20621092409733, 0.115709387627796, -0.173048813536005, 0.0910603610196797, -0.353980819098922, -2.73255387695276, 0.269880462004839, 0.551374027313544, 0.156563136896518, -0.179892682347763, -0.84954520251427, -1.09554724898114, -0.260992125203118, 2.14880580708577, 1.63844226899421, -2.30442907410152, -2.89214877130876, 1.98087658545312, 4.32534406849076, -0.399700774335826},
    {0.13457946630335, -0.260145422130515, 0.651010728103406, -0.408482387606868, -0.00811565004607073, 0.0204847751966051, 0.0785206721957282, -0.0183939829723474, -0.0602733003365504, 0.408505487798716, -0.527538352011231, 0.655871713874957, 0.0564104859792678, -0.0204847751966054, -0.084943018998191, -0.103147735711715, -0.261503124058777, 0.575159834595846, -0.33102728191393, 0.096776639561456, 0.127853408251255, -0.0785206721957289, -0.344609736372977, 0.138493039370235, -1.05258535204336, 0.631837609818953, 0.588960351966716, -0.0954080071974545, 0.489955712403521, 0.981542151457888, -1.59497023995359, -1.38969198064055, 1.14230265437246, -0.0482948359331978, 1.69006297404619, -0.962641268570256, 0.34460973637298, -4.09376198928807, -0.631837609818955, 0.0954080071974555, 0.307444391704981, -0.20505491709243, -0.109459425278907, 0.615912055004753, 0.168551577867522, -1.00083465259452, -3.580985293501, 0.824148229200024, 2.94350767149817, 3.50480163732135},
    {0.29979034391558, -0.0105569750572724, -0.13524370810424, 0.63425316236435, -0.927843770282099, 0.28862355368834, -0.11704912380254, -0.0284954840174733, -0.0180153481386418, -0.0312623596336947, -1.34163924975309, 2.25682373763315, -1.56797021089041, -0.28862355368834, -0.270192407039853, -0.104172465397179, -0.0915182376362798, 0.176649741215003, -0.310127615638405, -0.0641028972870166, -0.0481547341431419, 0.11704912380254, 0.655077736825811, 0.214022498231881, 0.428244358689189, -1.68442588453943, -1.74174309072226, 4.93056107714251, 1.40753902906185, 0.641003470710214, -0.571659541914737, -3.13789082767389, 0.246813927676391, 2.49581398117251, 1.52379537311956, 0.361678239846246, -0.655077736825808, -2.94338483346865, 1.68442588453943, -4.93056107714251, 0.325360727990986, 0.0891287565218535, 0.0766502181606189, 0.148171632593408, -0.00701051109620091, -0.132142385312291, -2.11868718182591, -0.575700738078131, 0.166647450017161, 4.68512792419092},
    {-0.103138285400156, 0.475450897669485, -2.77864520754329, 6.81314195983391, -9.1776713616849, 8.1697622215787, -0.200550049418395, -0.104688629505176, -0.103018390649353, -0.195002959051495, 0.823425571056501, -2.74590631055144, 5.24232379549969, -8.1697622215787, -0.172014255001802, -0.0797670410881805, -0.0639359920172169, -0.0565953241621265, 0.0577873067345194, 0.0415195882684592, 0.0597985776505809, 0.200550049418396, 1.26364805283859, 0.785506100520589, 1.25004489380433, -3.62665390395788, -3.64577903837326, 7.77440613813085, 1.01486465155495, 0.516189215215409, 0.424165769716426, -2.59830806090261, -1.46892758837986, 3.93534756618522, -0.503806640549397, -0.448684329606801, -1.26364805283858, 2.08248404772794, 3.62665390395787, -7.77440613813085, 0.071222338696825, 0.0392878229198092, 0.0448900518546051, 0.03622572040656, 0.0222109794610765, 0.0297675932135841, -0.476725972130059, -0.336821770913797, -0.269512281124862, 1.56329499064533},
    {0.0564307431585092, 0.158558075149594, -0.0772323978522209, -0.162307063211112, 0.351454288722325, -0.153888727424811, 0.106547386759107, -0.418572933892421, 0.548832068175609, -0.605941080999178, -0.0254222755673805, -0.0368690228656162, -0.186483630465099, 0.153888727424811, -0.20431533743439, 0.188720968758327, 0.356924388011649, -0.55631883764369, -0.199455171238251, 0.124670495976096, 0.22209716265081, -0.106547386759108, -0.0448341343681067, 0.337372953699175, 0.532376858808729, -0.0864364733586971, -0.782778849414962, 0.295350270418374, 0.14364649589583, -0.534628813489934, 0.493636991013705, 0.560906291813905, -0.361730205737179, -0.164970658257226, 0.120932596476201, -0.179011994576243, 0.0448341343681056, 0.415346878312041, 0.0864364733586974, -0.295350270418374, 1.12700518727133, -1.89667260375809, 0.196475771241612, -4.19452937094157, 1.22317003960638, 3.87292043590767, -1.83980612928438, -0.158360959122933, 1.18649667399945, 0.367431971102924},
    {0.169880004840815, 0.144369641593842, -0.258546391444693, -0.0463981325961392, 0.14220532642428, -0.0578765020203712, 0.0601843095813742, -0.171513359938567, 0.0755715289825172, -0.346152320938611, -0.277936485181336, -0.0441822278518117, -0.0717893005713683, 0.057876502020371, -0.519775202991848, 0.391647589650998, 0.229097457168838, -0.415219490262645, -0.372112559302997, 0.0719623468831859, 0.0865848309499929, -0.0601843095813747, -0.113567875147134, 0.532156172876651, 0.812720499751799, -0.0269329983295821, -0.437431610935616, 0.126067220056901, 1.09261087104032, -1.57021879486209, 1.01409080044779, 0.317240455840847, -0.226660095392897, -0.0704160258529123, 0.773906196019205, -0.268647598554565, 0.113567875147137, 0.220827940742403, 0.0269329983295788, -0.1260672200569, 1.46283143531814, -0.516709293274422, 0.0849285289885736, -2.10227215775785, 0.369175417408719, 1.35770560268132, -5.55685773081993, -0.26350857432209, 3.97023103504893, 0.21660367019322},
    {0.294232960746207, -0.06835338976749, 0.325725544285391, -0.509884452975201, -0.0652094687232025, 0.0204847751966049, -0.0624268174830955, 0.0974322990067754, 0.0337123028059787, 0.143235360960612, -0.873832507491196, 0.724042594224889, 0.0278635766407018, -0.0204847751966055, -0.661320832677902, 0.273084803804743, 0.122462998537101, 0.03989345935734, -0.384261280165227, -0.047871911054568, -0.0416321798630034, 0.0624268174830955, 0.227178570980758, -0.498632187415587, -0.920755739514813, -0.283119958636485, 3.31636320966726, -0.0668610978588865, 2.26497186338523, -1.2804041530501, -0.436460747129322, -2.49169234248002, 2.27753420123033, 0.0373458920825004, 1.30748519668667, 0.213062250645694, -0.227178570980756, -1.41706016427169, 0.283119958636481, 0.0668610978588879, 0.996004360448217, 0.164744553906037, -0.0558001191437744, -0.563048106773327, -0.150584945657445, -0.191930334470275, -4.499529279735, 0.285569936769894, 4.11279982256314, -1.89930304539557},
    {0.197406415132326, -0.301286830201028, 1.40654666417307, -2.52815393023257, 2.02269013545501, 0.28862355368834, -0.192304353920289, 0.0930541336911337, 0.0916921957694094, 0.227849200383547, -0.723022422627702, 0.83452508147955, -0.0927032580218609, -0.28862355368834, -0.374848738444309, 0.073724153890696, 0.0894943993640016, 0.315510600258314, -0.117123793366607, -0.0302669217370281, -0.00426482594360902, 0.19230435392029, 0.98129135269632, -0.595748244892207, -1.1962479350275, -2.1508035702318, 2.66089216880691, 3.45529412427395, 1.59672385676892, -0.578310450240695, -1.70193764807359, -2.34087474735752, 4.03450359611053, -1.92998687743315, 0.394872675081426, 0.0273041343774023, -0.98129135269632, -0.121953120237949, 2.1508035702318, -3.45529412427395, 0.258656369720958, 0.0848998722109585, -0.0887893077475255, -0.0936816225277907, -0.14632514624334, -0.275700154210106, -1.10763294481315, 0.568444110514807, 1.90900820475921, -2.53893904856896},
    {-0.30474418230749, 0.0509205961060799, -0.532258011356853, 2.09010537178824, -4.5253308777861, 8.1697622215787, -0.172286458511462, -0.0429252705156635, -0.0249690209116749, -0.026176946776874, 1.90385142678792, -5.05440252646412, 7.56849403744908, -8.1697622215787, -0.0263535680052439, 0.0372705104135178, 0.0641567122591933, 0.17874993153395, 0.0936331936893631, 0.0603813970734722, 0.0717913590691066, 0.172286458511468, 1.05080425362138, 0.306008441931436, 0.268741682907101, -2.8653133348694, -1.25978623570319, 5.44823589618145, 0.0379195220042073, -0.373728550109812, -1.03578438732547, 0.418506645988391, 2.54579050868749, -3.04316315966299, -0.649885244728015, -0.511790879101472, -1.05080425362138, 2.10695855637515, 2.8653133348694, -5.44823589618145, 0.00186491755525416, -0.00499960496556613, -0.0288660885534398, -0.0183803196128062, -0.0304127711962372, -0.0509408448549442, 0.0538107704371328, 0.205782437170035, 0.327332791383785, -0.847172320671949},
    {0.299991988790644, -0.277763665240073, 0.588308925193387, -0.933533835729226, 0.622081242620204, -0.153888727424812, -0.116259944551924, 0.586606245733346, -1.27147304705768, 2.50102932602933, -0.073272694948824, -0.0470325451594915, -0.0511701535161626, 0.153888727424811, -1.53898861463403, 3.18911252263765, -3.3745330636102, 1.70218083205602, -0.31149817896092, -0.064058324632779, -0.0482521085530285, 0.116259944551926, 0.0354981786256144, -0.240526846789189, 1.65918297057852, 0.0348154549389182, -0.750757725754872, 0.16003679346944, 0.417292329309052, -1.04432939930527, 0.112000839751657, 0.316871878440218, 0.663694502448498, -0.570911089104039, 0.0835916117926758, 0.019784868650137, -0.0354981786256167, 0.0617546157130714, -0.0348154549389151, -0.160036793469442, 1.77400054365302, 0.431579485806823, -0.53835413718032, -4.43967164502639, 0.903951885883638, 0.476139954304963, -0.563180898689733, 0.220741978139051, -1.17959368368147, 0.6890031100418},
    {0.272170305024623, -0.189231932784834, 0.633760217237779, -0.457019490504954, 0.257719653538467, -0.057876502020371, -0.157941208263555, 0.632653950000105, -0.926989625497545, 0.459765320075944, -0.309548329113103, -0.0281626337102487, -0.014032137014275, 0.0578765020203715, -1.18360794422824, 1.95151688520918, -1.43412364111211, 0.583276327891388, -0.224563774662213, -0.0571233170980164, -0.0344462938201895, 0.157941208263556, 0.231149039983692, -1.20102825338039, 4.33768392355258, 0.0335403158984809, -0.429560468806962, 0.0683100564998054, 1.45200103969363, -2.710815478835, 0.934602551016693, 0.140363902831378, 0.344818221383826, -0.243687516524192, 0.267298454503267, 0.0653927350050992, -0.231149039983689, 0.0233884039166453, -0.0335403158984859, -0.0683100564998056, 1.05336325097049, 0.284705323125868, -0.598207656179918, -1.9665780722645, 0.699407619469696, 0.678013275880281, -1.33222818152909, 1.13563551837528, -3.27275419652676, 0.406172064890321},
    {0.148805352758508, 0.157476470390114, -0.872076066213631, 2.13904455634208, -0.126473726995889, 0.0204847751966057, -0.201751253837804, 0.516115061727203, -0.28747067189417, -0.207795081851867, -0.299004451343534, 0.107758535600082, -0.00276855249564001, -0.0204847751966053, -0.527156928138631, 0.654649660726363, -0.278914536840165, -0.154860018896189, -0.0712456823987439, -0.0144818949841153, 0.0112979325913967, 0.201751253837806, 0.620470719319342, -1.96574431384857, 2.09302764261745, -0.575476442262637, 3.48525221451189, -0.0362289687225436, 1.10205620022748, -1.55410927798243, 1.62313359531211, -0.227016204197922, -2.01978688774424, 0.12924227949153, 0.105440343947129, -0.0430308053313982, -0.620470719319343, 0.0762933452315839, 0.575476442262635, 0.0362289687225449, 0.262593903449161, 0.0305091825030718, -0.527412994318603, -0.370307450389302, 0.271443384375217, 0.38675431119075, -0.222132441935145, 2.00877511917996, -1.97633554462943, -3.56154555974347},
    {-0.144387432414536, -0.106326707514486, 0.723274251164085, -2.23508757936844, 5.18874690712782, 0.288623553688343, -0.196100657264723, 0.222368167659388, 0.0651892255343086, 0.0634861274175523, 0.782380615353444, -1.61328408499109, 1.49032512781455, -0.288623553688342, 0.117560315409561, 0.14149698248244, 0.0187414048532422, -0.0270845628162195, 0.0686742481920393, 0.0470535208723658, 0.063869152575861, 0.196100657264724, 0.914117004568581, -1.2289899149814, -0.573052882679938, -1.72962982387027, 3.69864692076087, 1.87226573843755, -0.97508143031846, -0.622297538923495, 0.0917241027244265, 3.12245050737736, 0.725921156982172, -6.67907203494237, -0.413629024660877, -0.352993619637028, -0.914117004568581, 1.06233480714807, 1.72962982387026, -1.87226573843755, -0.0855887567241026, -0.091070315196606, -0.286237320235253, -0.0546227945819205, -0.0211724312100645, 0.0080511756964297, 0.800563378721657, 1.58198353461843, 0.186118528619159, -4.76098172790894},
    {-0.581370126327112, -0.201011180775914, 0.994395503589026, -1.74785530185878, 0.466841577692114, 8.16976222157871, -0.11406723191218, 0.00283446870569962, 0.0126860417796922, 0.0264995441507845, 3.30747149106534, -7.83904169966102, 10.0645802651882, -8.16976222157871, 0.288503254143643, 0.144130385484696, 0.131901227394148, 0.217846440080549, 0.088140469751392, 0.0568962490251347, 0.0611079628742537, 0.11406723191219, 0.672606807727141, -0.0191796168156955, -0.121774062739287, -1.74084237832476, 0.0704204046832562, 2.95214966844235, -1.8218665237543, -1.0460415200187, -1.43395895088136, 5.19327561210332, 4.39362138941647, -10.5314218428803, -0.546150447771748, -0.4134910044885, -0.672606807727144, 1.51818485993808, 1.74084237832476, -2.95214966844235, -0.04855073480205, -0.0376931152815699, -0.063942431579957, -0.0278758535645572, -0.0318891755232602, -0.0382134255355787, 0.361818435163417, 0.432670621304198, 0.306106075347625, -1.58860526462133},
    {-0.234775607728654, 3.08041774684675, 2.82690544195193, -1.94822363815396, 0.838592338884269, -0.153888727424811, -0.183808928740494, 1.02518120257738, -2.46945870260461, 3.88845680297297, 0.0623938398739255, 0.0442565832486457, 0.057085394615875, 0.153888727424811, 1.38653809489939, -3.48823959814696, 5.02306438607807, -5.76700502194861, 0.0858237942878895, 0.0560971556399128, 0.0697870357124553, 0.183808928740496, 0.0460210197968569, -0.301754118595964, 1.12035505065331, 0.0324009645506681, -0.327458481785814, 0.0517812453374064, -0.419653713722612, 1.35093934054142, -3.82058490864468, -0.375465769976699, 2.27943282488202, -0.895677733500145, -0.0254504080518169, -0.0205412715312057, -0.0460210197968566, -0.0222910415303259, -0.0324009645506615, -0.0517812453374067, -0.5772408633841, -0.475919291317887, -1.09496823828984, 1.85824017711212, 2.88928083828258, -5.25527991098885, 0.215917189126837, 0.322295390127161, -1.31082183172831, 0.349749523316124},
    {-0.319307309300151, 0.152389035168537, 5.95560097140644, -1.01996684985416, 0.350135184354867, -0.0578765020203688, -0.169696803710654, 0.782920000021128, -1.43794178384007, 1.39794304412362, 0.403549372775833, 0.0432625580147502, 0.0321756283939274, 0.0578765020203698, 1.5635024241056, -3.03612507050114, 2.88900525684541, -1.24946433631826, 0.0947241162295679, 0.0609912369741345, 0.0719463341830874, 0.169696803710656, 0.210549901268386, -1.13411353710397, 3.13228945022166, 0.0233121454198678, -0.188827469421551, 0.0221022910916064, -2.22039967365903, 5.39170750294123, -9.53045817346448, -0.294188330771823, 1.27089262261123, -0.382310812748795, -0.132858096396554, -0.104219219766845, -0.210549901268377, -0.0173522763725199, -0.0233121454198739, -0.0221022910916031, -0.521188758929398, -0.414744550958173, -0.854866334204215, 1.26558176678951, 1.79169509782411, -2.23706016821329, 0.903444072703653, 1.23833275687079, -3.90287542652874, 0.206179745794066},
    {-0.467046932097563, -0.118966191906676, 0.634469463727496, 6.31394635781805, -0.175487291718016, 0.020484775196611, -0.140239979255148, 0.442904859355286, -0.435651573811344, 0.0489152493135007, 1.25354229882126, -0.899206741227323, -0.0272753348566974, -0.02048477519661, 1.62807841497893, -1.89512629921112, 0.616150993438191, 0.236910014798233, 0.0966434754978224, 0.0620437377508592, 0.0688390494280754, 0.140239979255148, 0.383742454130394, -1.4003515270026, 1.99898102087306, -0.295455607144352, 1.56470408524642, -0.0117221863614783, -4.82526839326842, 6.80153638654701, -3.86427975582735, 4.15438608531315, -9.56912570190388, 0.202762626574715, -0.284686804050662, -0.217651411918006, -0.383742454130395, 0.243196116704146, 0.295455607144356, 0.0117221863614808, -0.372010348732362, -0.286645583240605, -0.51174390878336, 0.524373302551435, 0.66025341930109, -0.297921678630394, 1.31526916246876, 1.61800293892059, -3.02956337929115, -1.80790020195058},
    {-0.658971363590883, 0.0924967660603458, -0.480953409540752, 0.747335856394911, 7.72170385245787, 0.288623553688349, -0.0949022450750675, 0.149783565838491, -0.00889314599221547, -0.013659395807882, 2.74971897010371, -4.25860765292947, 2.75680360047958, -0.288623553688348, 1.33400723179774, -0.510734702388034, -0.137286348673311, -0.11951158320585, 0.0780938424138131, 0.050676572329665, 0.0534757459605011, 0.0949022450750661, 0.4137544541582, -0.746203528965421, 0.0619967075320089, -0.705685982583165, 1.78092724695622, 0.605787265772528, -6.05034993514168, 2.72617918739362, 1.0554051871851, 10.9282532024229, -7.41698140588833, -10.4785074529375, -0.353281126407818, -0.266409670022218, -0.413754454158195, 0.635826851225117, 0.705685982583163, -0.605787265772521, -0.17183395085846, -0.130044009447199, -0.203259311798994, 0.0774252969728284, 0.0882605831097567, 0.0299742072796954, 0.906574616791938, 1.01261319898763, -0.615290197916116, -2.41675409818134},
    {-0.852928776435773, -0.28741790529488, 1.65533088393823, -3.90144231251835, 4.4607553970921, 8.16976222157871, -0.042839968042431, 0.0099085180282287, 0.00995486939805446, 0.0142730477601933, 4.62479994033094, -10.2960803115461, 12.0615371748882, -8.16976222157871, 0.686806990944582, 0.173907889260907, 0.123139053325667, 0.156492748199511, 0.0398772179449481, 0.0262713444420438, 0.0267918654817089, 0.0428399680424433, 0.2460704222985, -0.0644270911558178, -0.0879482412252948, -0.61199682109665, 0.217715850213779, 0.955192758742358, -4.00072964471365, -1.15191054604721, -1.12749063350832, 10.1594048877718, 4.03811773629262, -16.5222925719803, -0.232099332087887, -0.174205865569767, -0.246070422298504, 0.588686800138815, 0.611996821096654, -0.955192758742352, -0.0344961879517868, -0.0259226052105956, -0.0367003835099464, -0.00993231890902491, -0.0103036086295034, -0.00972175884434194, 0.229018327121786, 0.238632956725585, 0.0910292461914057, -0.806402650352603},
    {0.218256695635768, 0.349323335271635, -0.0867591749077293, -0.0570689004231575, -0.0670256254939616, -0.153888727424813, 1.68633541906575, -3.67280844988191, 3.65012267913838, -1.7891081391585, -0.0660288489812664, -0.0587187272009766, -0.124017235870966, 0.153888727424813, -0.0455399181484104, -0.0459795078401186, -0.103507687555905, -0.372552917362969, -1.36161681179276, 3.75564547866193, -6.79578218749923, -1.68633541906575, 2.062832178816, -1.21699330338851, 0.492007308710945, -1.63494420391079, 0.392375073864793, 0.776296579228781, 0.0173080114119641, 0.0283161221164372, 0.107163890360611, 0.0258269683216179, 0.089960659302506, 0.191042861364932, 0.506230937110604, -2.25179764377298, -2.06283217881599, 0.726009965990641, 1.63494420391079, -0.776296579228786, 0.356917918164782, -1.65189099676161, 10.4685906373812, 0.583922156957053, -5.75387716103873, 2.20988487582944, -0.222661678810292, 3.4687909471615, -0.77557656701128, -1.11838503985543},
    {0.280175037546943, 0.425653570551385, -0.515016292023659, -0.0524324561977681, -0.0364185135341711, -0.0578765020203708, 0.0719510337707423, -1.84314901856563, 2.85587895540874, -1.81234149700632, -0.383747306192837, -0.0469015469885909, -0.0451261636041879, 0.0578765020203717, -0.109131920433167, -0.0794518769518588, -0.122588678576166, -0.394656132137135, -1.3504061485995, 2.55462266569428, -2.28384511302603, -0.0719510337707419, 4.7655347719497, -4.5575069785119, 2.41396636745544, -0.900671851963226, 0.294446784288054, 0.331354196982535, 0.176889026815153, 0.180064849312237, 0.541809722089105, 0.0307633759671808, 0.0685706272191771, 0.0815446771383574, 2.15932583026536, -5.64720483021854, -4.76553477194971, 0.364848876878041, 0.900671851963225, -0.331354196982535, 0.62247219869976, -1.67561248114399, 4.12699413159166, 0.63364791292032, -3.73488913995902, 1.90662753398573, -1.41633179750479, 10.2047118087304, -3.15696040021602, -0.659295661166097},
    {0.2887340782237, 0.555572946106275, -1.45661822356639, 1.01456732457268, 0.0295255943487231, 0.0204847751966047, -0.111229861949402, -0.225527686738772, 1.54782147144669, -1.6835827489748, -0.764302484300345, 0.528230785883272, 0.0137224969120988, -0.0204847751966044, -0.188039020221463, -0.121944075920713, -0.149458794674029, -0.38486513351377, -0.883393092814753, 0.805867617127967, -0.104817635449234, 0.111229861949403, 1.16138521157218, -5.20944722519984, 4.81552714750589, 6.53271681876414, -3.94675688274904, -0.175737240659414, 0.606291861760848, 0.495608567250778, 1.1190202788551, -0.605641412763101, -0.937156697692845, -0.043248091260824, 2.5071866873306, -2.42117474816067, -1.16138521157219, -1.83431901477316, -6.53271681876414, 0.175737240659417, 0.700761876234934, -0.923965083408198, 0.330345322188004, 0.572832345887169, -1.42972400516646, 1.29338161966745, -2.87460732686507, 7.63062197336052, -4.44810650797142, 5.7810758975222},
    {0.0751681159726963, 0.719267259522416, -2.96850396495829, 4.53211724574024, -2.87309406907589, 0.288623553688337, 0.105639734313318, 0.109436381263989, 0.311379319648966, -1.26392497832616, -0.148269706050579, -0.176927006276891, 0.638089254542725, -0.288623553688336, -0.211189019346048, -0.135104085812627, -0.148290623916999, -0.299851646419437, -0.0631300436915674, -0.0121557779410515, -0.0243048703807554, -0.105639734313315, -0.812462895183736, -1.60303509237851, 5.63649497533857, 4.07437048996136, -9.93431814649686, 9.08187084136944, 1.00403191404936, 0.765595846936137, 1.34714591002337, -1.95601460256616, -2.39917563689718, 2.23500481453316, -0.220040243402511, 0.356020179816335, 0.812462895183735, 2.20632583134578, -4.07437048996137, -9.08187084136944, 0.427495139027525, -0.134388071434593, -0.0851315108832325, 0.325974203055834, -0.164835470273328, 0.573585679934365, -2.43265252888495, 1.24701491256217, -2.98380220305111, 7.72799231515108},
    {-0.424231227319648, 0.88027078582891, -4.75479662623642, 10.5319042888745, -12.2449015666311, 8.1697622215787, -0.183157792112608, -0.14065471810291, -0.188172536299753, -0.614637464274605, 2.51931486939399, -6.30034913894458, 8.72079246890364, -8.1697622215787, -0.131776883306163, -0.0860162965477203, -0.0889367065888942, -0.149309672066483, 0.32566383480829, 0.112567741267969, 0.100173986672977, 0.18315779211261, 1.27250098343435, 1.23838125696055, 3.55929557227637, -4.28536801916839, -8.95967214175376, 14.320105016481, 0.781804918507035, 0.586917185313282, 0.866759653022111, -2.04184240860139, -2.18971274132853, 3.52410909772741, -2.12922603348472, -0.881972458542519, -1.27250098343435, 6.38105916266257, 4.28536801916838, -14.320105016481, 0.101061439729529, 0.0364814049019537, 0.040480731429947, 0.0758689211920502, 0.03912339012982, 0.112043268544732, -0.69004810952358, -0.356408798418027, -0.740021429268082, 2.57861297909119},
    {0.286289299876281, -0.267111615790656, 0.07478729609793, 0.0592354988078972, 0.0992497134834083, -0.153888727424811, -0.0264986033169916, 2.18799616918965, -2.43983545736058, 1.28812755044812, -0.0770893893180518, -0.0575512666200261, -0.0856466499156504, 0.153888727424811, -0.47245337639004, 0.0230586463899443, 0.0250140168517534, 0.405203029062718, -1.32777853203544, 2.37046405856914, -1.88811166644526, 0.0264986033169913, 0.8298962262281, 1.16050524302495, -0.426544118735142, -1.05286688956291, -0.485288115250576, 0.648391826206741, 0.148206917989042, -0.0115930724528139, -0.134311752316107, 0.151445511910334, -0.153129744098209, -0.0136030635677579, 0.414416267213987, -1.00144758897719, -0.829896226228097, 0.418775025703631, 1.05286688956292, -0.648391826206741, 2.5526984419749, -6.23784955395089, -0.299884502744394, -0.199677709985004, 6.30722095274233, -2.31336975040257, -1.09053175400227, -0.159057654047758, 1.10265960552342, 0.0665130895469456},
    {0.30194613339729, -0.299846929362777, 0.386955294238905, 0.0431773246303416, 0.0345544018425519, -0.0578765020203712, -0.203731558094384, 0.57575865788657, -1.48295613960183, 1.13733844253902, -0.377044237587168, -0.0399716863386913, -0.0287480743053716, 0.0578765020203715, -0.459160981004961, 0.00726185199463725, 0.0287504390303683, 0.421049485945441, -1.10820787912762, 1.37285619648022, -0.479011407990876, 0.203731558094385, 1.56895506964764, 2.72672594312746, -1.66033835355253, -0.56793623000298, -0.233344644870105, 0.276759370904622, 0.653949888637556, -0.0387738966614256, -0.625087048627869, 0.0881717833519567, -0.0913774216436075, -0.00580632753718158, 1.5370689233873, -2.26854223611181, -1.56895506964764, 0.194134721823644, 0.567936230002978, -0.276759370904622, 1.92208856918342, -3.0283230510661, -0.0967472498956929, -0.113964181125394, 3.1384229941877, -1.83725495146942, -3.39055267675165, -0.458183707015649, 3.51382210691688, 0.0392099230464643},
    {0.236643324868564, -0.260145422130515, 0.651010728103406, -0.408482387606868, -0.00811565004607129, 0.0204847751966054, 0.0456413589580971, -0.0169906995074023, -0.366310212716689, 0.628473155988076, -0.569818879903483, 0.332539840484565, 0.00503620216509604, -0.0204847751966054, -0.355605588792679, -0.012373833725399, 0.0362893533735422, 0.355192166406486, -0.550092117280209, 0.298208058414284, 0.0105436529618634, -0.0456413589580964, -0.318319337733485, 0.841689677323442, -1.61937026038549, 3.83998500103422, 0.906097426369835, -0.146782291011628, 0.996391741891085, -0.0493982584795605, -1.02818533161145, -0.749223032847042, 0.825165579969348, 0.00307944788097347, 1.31830684605259, -0.522314214044908, 0.318319337733486, -0.562282724682405, -3.83998500103422, 0.14678229101163, 0.961897301525397, -0.671872415705174, 0.00644704654553951, -0.0476881225865655, 0.739974570007581, -0.992590217646699, -2.9701880291495, -0.319375463278531, 3.27125144348241, -0.343814701687429},
    {-0.0291979345553109, -0.0105569750572729, -0.135243708104239, 0.634253162364347, -0.927843770282096, 0.288623553688338, 0.0490770986832653, 0.0144672844878025, 0.0132869869789191, -0.0480961808579275, 0.305128780364225, -0.899029556149415, 1.08698561075244, -0.288623553688338, -0.171446305662551, -0.012622487867263, 0.0303401407031612, 0.193483562439237, 0.108511731976808, -0.0188337949497861, -0.0169486831388894, -0.0490770986832637, -0.332585892363579, -0.157849525044939, 0.658840803069204, 1.24232652195232, -2.67961829116481, 7.58551689878537, 0.683463097983016, -0.0510921839482455, -0.802255986294756, -0.919912734333885, 1.18468912811895, -0.159141840470346, -0.933881594769519, 0.184923548428625, 0.332585892363578, 3.13922086680566, -1.24232652195232, -7.58551689878536, 0.243057257384393, -0.0192712772251315, 0.00248139865108929, -0.0181697097339785, 0.024818085195995, -0.285303098769294, -0.955574336252139, -0.0270740233836833, 1.23061512795245, -0.459602575640847},
    {-0.504539511638759, 0.475450897669486, -2.77864520754329, 6.81314195983391, -9.1776713616849, 8.1697622215787, -0.202507479822821, -0.112401311572459, -0.122153260624199, -0.300006067880517, 2.92484054429376, -7.0993207731389, 9.42860290518039, -8.1697622215787, -0.0305882458433141, -0.000575610963326367, 0.0118446861090225, 0.0484077846668945, 0.315790554508004, 0.127284178983569, 0.115474534996162, 0.202507479822825, 1.35674427276764, 0.931407788589743, 1.92315570537226, -4.30027684116912, -5.60891916196385, 11.9606852478116, 0.141476565796424, -0.0387268606953831, -0.24894504185151, -0.208033721905753, 0.494212535210738, -0.250931543495484, -2.00393582426733, -0.956873899197069, -1.35674427276764, 5.76227557310416, 4.30027684116911, -11.9606852478116, 0.0152750081538238, 0.00372984998704914, -0.00307322342369307, -0.00418127984350895, -0.00886076834642871, -0.0268782149378049, -0.0587219878217056, 0.0254661106073344, 0.139502106716763, -0.153356411140311},
    {0.270764883720562, 0.158558075149593, -0.0772323978522171, -0.162307063211115, 0.351454288722327, -0.153888727424812, -0.0374957547900425, 0.265135099080673, 2.57445587305492, -0.932221756849496, -0.0612235461982308, -0.0345884911148328, -0.0274465741133927, 0.153888727424811, -0.907720359589551, 1.002394615068, -0.293959052555242, -0.230038161793367, -0.738989379905981, 0.548630123230591, -0.0207055380471612, 0.0374957547900442, 0.0283991192343477, 1.58254561353879, 0.819045458687697, -0.405455328483566, -1.20428123642476, 0.454387326770088, 0.223165715310629, -0.291678162394917, 0.206968391134735, 0.137123373053326, 0.0597721812726217, -0.324007714608935, 0.174542906574691, -0.123587780441425, -0.0283991192343513, 0.0940474914363878, 0.405455328483568, -0.454387326770087, 2.69368737707081, -2.17500129757813, -0.244429561033515, -3.52065631191058, -0.948084698707376, 2.49818007159525, -0.691961728327213, -1.45895783309736, -0.301626636935176, 1.11023374498838},
    {0.21585406400914, 0.144369641593842, -0.258546391444692, -0.0463981325961391, 0.142205326424279, -0.0578765020203709, 0.0758000408564921, -0.327343385724778, 0.566966782893934, -0.532544722385905, -0.223449718107216, -0.0169677308765786, -0.00390597156038855, 0.0578765020203711, -0.653174912253482, 0.613754281737722, -0.091975986271873, -0.228827088815349, -0.460253834647198, 0.207338943397586, 0.00899118188658532, -0.0758000408564909, -0.21675100279971, 3.99244103427919, 1.2503455465619, -0.202061750267644, -0.672975108694533, 0.193950549067879, 0.72641139033147, -0.820881034417261, 0.576465753637697, 0.0544824611066941, 0.00888340236602292, -0.138299354863891, 0.457249582035067, -0.109660879297124, 0.216751002799711, 0.0184846918291069, 0.202061750267643, -0.193950549067879, 1.49623651693776, -0.665754071757142, 0.318352203838191, -1.69082175211526, -0.108551654534378, 1.18738379221061, -1.46820354179852, -3.88278015498206, -0.239391586798441, 0.65449041686543},
    {0.0617978275970903, -0.0683533897674902, 0.32572554428539, -0.5098844529752, -0.0652094687232042, 0.0204847751966055, 0.100320494542055, -0.199566116235118, -0.115846086184703, 0.220363206959813, -0.0545443252607909, -0.0738260863498091, -0.00813907259628269, -0.0204847751966053, -0.245523522594397, 0.249366347937696, 0.0399471234689606, -0.0372343866418589, -0.0369008898651063, -0.0167733358255092, -0.0249087002179144, -0.100320494542054, -0.465319463510972, 1.7134571817969, -1.4165544473472, 0.972889313363564, 5.10212301923665, -0.102863747095872, 0.415415827259973, -0.745935006987639, 0.0593379607030652, 0.0919361476640752, 0.491774391660935, 0.0733485413194866, -0.205102686184766, 0.213863916794997, 0.465319463510972, 0.636817788303052, -0.972889313363563, 0.102863747095873, 0.281041402870959, 0.0208879537778332, 0.22447481645303, -0.504647649554209, 0.111731468232383, 0.0401439295885426, 0.255415826243778, -1.9273210985919, 1.36624130728819, -5.73894080753971},
    {-0.236409154506208, -0.301286830201028, 1.40654666417306, -2.52815393023257, 2.02269013545501, 0.28862355368834, -0.0922286345676678, 0.0502164067882647, 0.060799122485104, 0.350539002122207, 1.15104668812143, -2.14074232727352, 1.76786658528815, -0.288623553688339, 0.107051008856094, 0.129513646214305, 0.108310531117184, 0.192820798519655, 0.286863399364978, 0.0514823300150008, 0.0438900185639349, 0.0922286345676699, 0.529551174033434, -0.395027845145917, -1.8403907353171, -1.42615157823009, 4.09370093920998, 5.31586396758397, -0.735850415860877, -0.763948088649631, -1.05779484778399, 2.06720143179863, 2.60169482570746, -3.79055672074317, -1.55837122838992, -0.345261251563269, -0.529551174033437, 3.57796628051682, 1.42615157823009, -5.31586396758397, -0.183388348966534, -0.0497137581499039, -0.0941064253522004, -0.190390839841664, -0.062567694350201, -0.263623212678986, 1.50483652028515, 0.740289096709193, 1.89392544342187, -7.67166721972681},
    {-0.636880676372851, 0.0509205961060803, -0.532258011356855, 2.09010537178824, -4.52533087778609, 8.1697622215787, -0.204379670271958, -0.0530640494716975, -0.0329394822111662, -0.0402724292484846, 3.58097023984, -8.36005347555911, 10.5022018617589, -8.1697622215787, 0.1740224190436, 0.108585811744012, 0.110506435473606, 0.192845414005559, 0.270948238537975, 0.131752129083818, 0.123148864515013, 0.204379670271966, 1.29900005822651, 0.403690623878352, 0.413450831498723, -3.77996149537446, -1.93814246092291, 8.38194372049129, -1.08636135282508, -0.781857339740972, -1.18049353591709, 3.04580136986365, 3.22414673390721, -5.97687098397283, -1.6537305283736, -0.936868603902829, -1.29900005822651, 4.49796134496507, 3.77996149537446, -8.38194372049129, -0.0821977495621014, -0.0480009866466862, -0.0700848150433122, -0.0591579529575307, -0.0508116602259738, -0.0893201067698632, 0.602500297806847, 0.533177980024488, 0.63777939906802, -2.55981888404216},
    {-0.0276979469831464, -0.277763665240074, 0.588308925193391, -0.93353383572923, 0.622081242620204, -0.153888727424812, 0.0500625320267789, -0.306082253789322, 0.965522610606368, 3.84775686177042, 0.0162868154822797, 0.0184619773736329, 0.0350047482548482, 0.153888727424811, 0.109881932370394, -0.135247886741848, -0.0246257297202468, 0.355453296314924, 0.106521766617375, -0.019094601511483, -0.0172233649001308, -0.0500625320267761, -0.0185224119214147, 0.182649651575555, 2.55260208008503, -0.0264379249091131, -1.1550177203986, 0.246211695240455, -0.0962322175421959, 0.273054746621397, -0.781418269754866, -0.15288263873666, 1.06795449709226, -0.657085990875051, -0.0504813859366666, 0.0102777654013605, 0.0185224119214069, -0.0649932866680873, 0.026437924909118, -0.246211695240456, -0.629394115089672, 0.158121365096621, 0.323305618689447, 1.78587852395091, -1.1045493741915, -5.11076303724904, 0.418033633824483, -0.192927416976908, -2.92015432797285, 1.22001100706668},
    {-0.119570063807636, -0.189231932784834, 0.633760217237777, -0.457019490504952, 0.257719653538466, -0.057876502020371, -0.0134898570388154, 0.0630462694728009, -0.122711418295394, 0.707334834787757, 0.186296385708589, 0.0244646644642508, 0.0227507319824552, 0.0578765020203716, 0.418405228219932, -0.448686198224909, 0.00337615341786957, 0.335706813179578, 0.207923942877124, 0.00656885261329279, 0.00556553898616446, 0.0134898570388156, 0.0230348433976993, -0.158987626540096, 6.67339359332445, 0.00443994152768276, -0.660865598094126, 0.105092925496535, -0.805886036576068, 1.38693655238488, -1.40110711875518, -0.14356852463028, 0.576123350670981, -0.280470385520921, -0.357232762476244, -0.0140349594230741, -0.0230348433977001, -0.0583392686286556, -0.00443994152768626, -0.105092925496534, -0.899443119077046, -0.0385486782204708, -0.0686118084589667, 1.54794906726402, 0.154691243902574, -1.56376472585185, 2.09638602373872, 0.173022585963169, -8.41254685458693, 0.719204866722781},
    {-0.291832089112706, 0.157476470390114, -0.872076066213631, 2.13904455634208, -0.12647372699589, 0.0204847751966057, -0.124807634069925, 0.354223087492421, -0.235690806650839, -0.319686356219994, 0.841280923622991, -0.656732008696605, -0.0222767402895105, -0.0204847751966046, 0.741033906523163, -0.437907813289338, -0.125801729983173, -0.0429687445280593, 0.308397447711336, 0.0723620723206563, 0.0623918968156401, 0.124807634069927, 0.425845068656624, -1.61167001818826, 3.22005879336978, -0.471820328632921, 5.36195357000441, -0.0557371565164131, -2.44645432594538, 1.98114702397625, 0.496102444559767, 2.41417569559128, -3.89648824323676, 0.148750467285403, -0.988626287947566, -0.283875199065947, -0.425845068656629, 0.944440003157974, 0.471820328632925, 0.0557371565164154, -0.896823223822465, -0.266006154665517, -0.416614984308065, 0.7262504932407, 0.429334888995705, 0.181861639090423, 3.63423363682618, 1.89554521725421, -5.8656661422484, -6.30639357316239},
    {-0.533380965289453, -0.106326707514486, 0.723274251164084, -2.23508757936844, 5.18874690712782, 0.288623553688343, -0.206188382357002, 0.247436090352882, 0.07945677509243, 0.0976714586493664, 2.28752304127764, -3.66947089586477, 2.49848276832552, -0.288623553688342, 0.79170982413452, -0.0492011060715606, -0.0415311512109852, -0.0612698940480318, 0.308908704790786, 0.130448724413434, 0.119145300218686, 0.206188382357004, 1.01716689091037, -1.49797109054705, -0.881624273070685, -2.10818285969814, 5.69025477650749, 2.88042337894852, -3.74314165865804, 0.332048873101154, 0.400295493115167, 7.17024517399765, -1.26568669876444, -7.68722967545334, -1.44741290720917, -0.721302276671143, -1.01716689091037, 2.73996858219387, 2.10818285969814, -2.88042337894852, -0.505476456743658, -0.254900328473415, -0.366581390571571, 0.0448401111009718, 0.0449948289675545, 0.0540561822025333, 2.82828388811967, 2.21927336721819, -0.499246707839819, -8.43022335870137},
    {-0.793676417832502, -0.201011180775914, 0.994395503589025, -1.74785530185878, 0.466841577692116, 8.16976222157871, -0.15476944247681, 0.00392899389341736, 0.0181729170402965, 0.0407687354077828, 4.34154822751181, -9.77838145744786, 11.6542222604831, -8.16976222157871, 0.469112209433549, 0.17769536914179, 0.144302771209537, 0.20357724882355, 0.177997014576992, 0.101668436068696, 0.0980833119458533, 0.15476944247682, 0.932332763072866, -0.027475046299651, -0.187345658291282, -2.49377896360016, 0.108339631410423, 4.54179166373722, -2.76904596912688, -1.19851040664459, -1.36838735532937, 7.17053459661733, 4.35570216268931, -12.1210638381752, -1.04810581815357, -0.685886415207419, -0.932332763072872, 2.70458803244951, 2.49377896360017, -4.54179166373722, -0.113526762781932, -0.0745539577676488, -0.102012305839274, -0.0491371425912795, -0.0452873953413493, -0.0561018446115705, 0.768581084987387, 0.713361461507079, 0.466870391457465, -2.81292766385994},
    {-0.593196374838999, 3.08041774684675, 2.82690544195194, -1.94822363815396, 0.838592338884267, -0.15388872742481, -0.20805983764814, 1.21685398004279, -3.16086198992807, 5.98227145504557, 0.136760509442189, 0.082081797376203, 0.0849680060691187, 0.153888727424811, 3.33081466386313, -7.7781398565387, 9.82092349468903, -7.86081967402122, 0.289411339931, 0.133172202775034, 0.123294301136917, 0.208059837648142, 0.0546253296145476, -0.386239754796575, 1.72363186185203, 0.041472642234182, -0.503784824561299, 0.0796638567906496, -0.837065927417667, 2.29726169586697, -4.42386171984343, -0.589617326879771, 2.45575916765753, -0.923560344953386, -0.0726193843791446, -0.0391346549544485, -0.0546253296145458, -0.0510445779743709, -0.0414726422341759, -0.0796638567906495, -1.77139126384231, -0.956614508025881, -1.34014828117971, 4.86144420114209, 3.98430429517891, -9.36173573227634, 0.521646073380347, 0.425374409751018, -2.17265855085323, 0.554829402535655},
    {-0.645727750172869, 0.152389035168538, 5.95560097140644, -1.01996684985416, 0.350135184354866, -0.0578765020203689, -0.203105697019898, 0.975293412471049, -1.90768844364097, 2.1506924706599, 0.738792277826611, 0.0700963669874181, 0.044077033829777, 0.0578765020203699, 3.07478343736792, -5.75165284262929, 5.17242188312024, -2.00221376285454, 0.266793392796454, 0.131104289577877, 0.122775552575934, 0.203105697019899, 0.262284692814004, -1.50460561952113, 4.81893118953382, 0.0309277544569301, -0.290505266609969, 0.0340036965274564, -3.8214580631436, 8.34416472668719, -11.2170999127766, -0.422699936932905, 1.37257041979964, -0.394212218184643, -0.331177007563525, -0.189408422105036, -0.262284692813997, -0.0365704763081261, -0.0309277544569368, -0.0340036965274528, -1.38000868809125, -0.790594110906634, -1.09806896504698, 3.01325295932212, 2.56717826496972, -4.05073013468722, 1.99708638588726, 1.69401404162615, -6.48484056785754, 0.327075742918087},
    {-0.731477026246282, -0.118966191906676, 0.634469463727496, 6.31394635781805, -0.175487291718016, 0.0204847751966106, -0.181363947469802, 0.589524145791054, -0.606401112905343, 0.0752546098649925, 1.85165020848897, -1.23254689245622, -0.0335873724596976, -0.0204847751966093, 2.5359510590258, -2.97107163501472, 1.07499313989513, 0.210570654246743, 0.219624776309364, 0.118375698008269, 0.11283191429319, 0.181363947469801, 0.510776609686074, -1.94920614426792, 3.07537095209691, -0.411256654989254, 2.40724921454863, -0.0180342239644771, -6.9375512985163, 9.39210131335104, -4.9406696870512, 5.33027136584425, -10.4116708312061, 0.209074664177716, -0.600343857128859, -0.373068112951836, -0.510776609686079, 0.460735220088137, 0.411256654989259, 0.0180342239644804, -0.822865001762278, -0.511927454742023, -0.702356060084243, 1.11399990158126, 0.999952869639098, -0.586014285993338, 2.5962465938618, 2.32227425721975, -5.07127368882984, -2.86798443463678},
    {-0.834219465627172, 0.0924967660603458, -0.480953409540751, 0.74733585639491, 7.72170385245787, 0.28862355368835, -0.132465689012758, 0.212613636329341, -0.0129525142247648, -0.0210145612450063, 3.38060476427583, -5.03884374042012, 3.08300145155091, -0.288623553688348, 1.70865814718409, -0.706870494602038, -0.147908535246495, -0.112156417768724, 0.147710619063584, 0.087331143910432, 0.0848890558285615, 0.132465689012757, 0.587313024319974, -1.08681582782371, 0.0953800318655753, -1.02780362940397, 2.73990191297316, 0.931985116843863, -7.44217039338947, 3.52049717580286, 1.02202186285153, 12.6674639559305, -8.37595607190527, -10.8047053040088, -0.643091151607848, -0.433926868832533, -0.587313024319971, 1.09394529189495, 1.02780362940397, -0.931985116843855, -0.325174834870032, -0.219547216525379, -0.297502692157905, 0.153823014966021, 0.145168586839719, 0.0446557620854287, 1.61670700720934, 1.52074269665623, -1.06899588746706, -3.83384720486813},
    {-0.930688141770991, -0.28741790529488, 1.65533088393823, -3.90144231251835, 4.4607553970921, 8.16976222157871, -0.0634805252678735, 0.014781050436091, 0.0150011531563038, 0.0219586459407444, 4.99337808590907, -10.961922045512, 12.5758791607061, -8.16976222157871, 0.767280516065696, 0.179318958433263, 0.122699422547966, 0.14880715001896, 0.0662392168845516, 0.0421488922094663, 0.0416711268844585, 0.0634805252678861, 0.367076016056136, -0.0970862221490776, -0.135305670002893, -0.922227874355373, 0.334949153877476, 1.46953474456029, -4.40437365533355, -1.16420210978303, -1.08013320473073, 10.9424799254014, 3.92088443262892, -17.0366345577982, -0.380169713679585, -0.273708035798563, -0.367076016056142, 0.944297480845959, 0.922227874355376, -1.46953474456029, -0.0584258305253361, -0.0420741367221071, -0.0564521773205585, -0.0154436204750816, -0.0150759086436645, -0.0143284118248897, 0.379495442761993, 0.370794257947644, 0.135979940920492, -1.27924663472344},
    {-0.28421858194347, 0.34932333527164, -0.0867591749077303, -0.0570689004231605, -0.0670256254939564, -0.153888727424812, 10.7781211235609, -9.36250110289346, 6.0248351056853, -2.21672402216102, 0.0729924116468987, 0.0498880353538502, 0.0615259070688868, 0.153888727424813, -0.0946999989222341, -0.0236501366189072, -0.00181758342656524, 0.0550629656395333, 1.8649789201575, -5.26674221029165, 8.59427075157847, -10.7781211235609, 5.25844699302783, -2.00875001258034, 0.609602290899721, -2.69861319781277, 0.486156891747435, 0.961839722168655, 0.020518690947271, 0.00367916414171887, -0.0104310918281589, 0.0110020236493975, -0.00382115858009668, 0.00549971842507357, -0.532897259535391, 1.84392410645669, -5.25844699302788, -0.446265790466154, 2.69861319781276, -0.961839722168654, 0.52425896367002, -1.16149737992197, 0.76823035131497, 0.0940037931802777, 0.403404484528363, -0.2665176548468, -0.11752213151963, 0.164825906123725, 0.0408171001552731, -0.0398911012812735},
    {-0.365495227282889, 0.425653570551386, -0.515016292023664, -0.0524324561977605, -0.0364185135341774, -0.057876502020368, 3.05411065485141, -5.24133428669361, 4.79626721973488, -2.2455103997586, 0.452267144066581, 0.0473046632961824, 0.0340710153857432, 0.0578765020203698, -0.0753057746212404, -0.0202897371294748, -0.0030756021329239, 0.0385127706151407, 1.97191931828046, -4.32720155341021, 4.90348352510901, -3.05411065485142, 13.5516773429902, -7.65404334919408, 2.99093001608284, -1.51262113932656, 0.364822698915434, 0.410551375972463, 0.0812421394763716, 0.0166609350189886, -0.0351539265382876, 0.00693308030980067, -0.00180528740821861, 0.00234749814842994, -2.69842283984551, 7.16067196067346, -13.5516773429903, -0.341306620770079, 1.51262113932655, -0.410551375972458, 0.354221938059549, -0.634204617389306, 0.337850761584604, 0.0726429502022343, 0.165138951064651, -0.153273806783652, -0.3954870609967, 0.493371388520642, 0.102979884759388, -0.0235160781453583},
    {-0.505828786742264, 0.555572946106273, -1.45661822356639, 1.01456732457268, 0.029525594348716, 0.020484775196607, -0.171838342059686, -1.24929187044142, 2.74189559758475, -2.08597694083695, 1.34289336556039, -0.950264780748591, -0.0282805736101455, -0.0204847751966052, -0.0480006553658726, -0.0148643201310011, -0.00440824221551028, 0.0175290583483763, 1.92311516445491, -2.56811623612244, 1.19661234538719, 0.171838342059683, 6.43339682257551, -9.22829969484467, 5.9664893773641, 11.5724118162643, -4.89007377482177, -0.217740311181664, 0.115539283892146, 0.0301275251169575, -0.0319419510031098, -0.0704627382039653, 0.00616019437987747, -0.0012450207385708, -5.58833759248823, 8.83916529280076, -6.43339682257552, 4.68387154935881, -11.5724118162643, 0.217740311181669, 0.165460158372689, -0.190427453157483, 0.052679525054229, 0.0431446769384654, 0.0166480916951819, -0.0457430589291263, -0.414378788683085, 0.38913440204391, 0.0362270038072254, 0.206202225462957},
    {-0.685613082674016, 0.719267259522414, -2.96850396495829, 4.53211724574023, -2.87309406907588, 0.288623553688331, 0.101751927918442, 0.131051222146628, 0.741151263335809, -1.56601649746155, 2.84662379487985, -4.38015391749967, 2.8087530250254, -0.288623553688331, -0.02276122357102, -0.00867512327111478, -0.00445770272221457, 0.00223987271595691, 1.50171934330263, -0.716300226480562, -0.128116416583607, -0.101751927918439, -0.972932896106745, -3.81557608009214, 6.98367725189582, 9.69789785441526, -12.3087258937224, 11.2525346118521, 0.0894163108115533, 0.0325002258007787, -3.63665338963259e-05, -0.127195438568935, -0.0247678896716249, 0.0643410440504821, -6.74951233933679, 3.73012877389719, 0.972932896106734, 12.0330801070224, -9.69789785441527, -11.2525346118521, 0.0471710395290014, -0.0208006666364311, -0.00293480556302095, 0.0171452995771741, -0.00405037021881349, -1.91849472682137e-05, -0.195999318356931, 0.0854473061949581, -0.0381655942020998, 0.275645786699939},
    {-0.86509141302199, 0.88027078582891, -4.75479662623642, 10.5319042888745, -12.244901566631, 8.16976222157869, -0.138221459026591, -0.112165339938185, -0.165021692646536, -0.761542358539875, 4.68268090500079, -10.4012155852295, 12.1434499468309, -8.16976222157869, -0.00677348275462075, -0.00338619289693671, -0.00261491935414621, -0.00240477780121354, 0.749992090304533, 0.162973965047242, 0.111236026185822, 0.138221459026595, 1.01475803516351, 1.08602336549173, 4.4100050882041, -3.75813972667071, -11.1011291227226, 17.7427624944082, 0.0366058900392577, 0.0194596941020061, 0.0160501370943678, -0.0824329432852734, -0.0482557603597042, 0.101451619800175, -4.35528204551052, -1.07702543039435, -1.01475803516351, 11.0091539014452, 3.7581397266707, -17.7427624944082, 0.00586290813851102, 0.00129162053298726, 0.00092931375237415, 0.00311672244005945, 0.000756107066298639, 0.00257063765676901, -0.034516996105622, -0.00899793509735325, -0.0202060465879885, 0.0919752212773837},
    {-0.377854821367114, -0.267111615790658, 0.074787296097926, 0.0592354988079015, 0.0992497134834051, -0.15388872742481, 2.40929939068856, 6.36960726610733, -4.11080267791069, 1.59600374185812, 0.0927302983767189, 0.06013996584984, 0.0693258960286186, 0.153888727424811, 0.353965859899706, 0.120413683820875, 0.0732600557844614, 0.0973268376527269, 1.97891768943057, -4.17357053861733, 4.44916882452415, -2.40929939068856, 2.41596082622727, 1.95529909460258, -0.528492701931607, -1.77394258946499, -0.601277138703982, 0.80336437215102, -0.100061621618982, -0.035092803736021, -0.0323631691196438, -0.082234744012915, -0.0371407206448309, -0.168575609512023, -0.536562258876685, 1.36577584941787, -2.41596082622728, -0.419991906668312, 1.773942589465, -0.803364372151021, -2.135372327289, 5.70689557290478, -10.8187760906315, -0.748900535214508, 2.57747764362326, -0.690648568785197, 0.733689475579235, -3.32107494402044, 0.331365485229053, 1.02126904537229},
    {-0.451804425814343, -0.299846929362779, 0.386955294238904, 0.043177324630344, 0.034554401842552, -0.0578765020203707, 0.26145911536624, 2.09615129819623, -2.55998154197248, 1.409174432625, 0.542062083177414, 0.054617048854023, 0.0374003603131545, 0.0578765020203721, 0.367859795037022, 0.134844949406548, 0.0997331148740843, 0.149213495859468, 1.97402656038467, -3.23824225193071, 2.26879383358003, -0.26145911536624, 5.71205862214124, 4.7070630735566, -2.05717688756718, -0.980410834143371, -0.289116497994067, 0.342907805523152, -0.499058192222418, -0.201710670580673, -0.228248514613232, -0.062188804964724, -0.0356055685196439, -0.0719547621557076, -2.60222076128038, 5.09474468028488, -5.71205862214124, -0.312928617509459, 0.98041083414337, -0.342907805523153, -1.81741936430954, 3.6871713148048, -4.36494513177626, -0.734569403757634, 2.1110524790984, -0.831212224942496, 2.96297589844054, -9.80180775384147, 1.69642175040702, 0.602045115503527},
    {-0.576948661244746, -0.260145422130516, 0.651010728103406, -0.408482387606867, -0.00811565004607184, 0.0204847751966057, -0.143420606214598, 0.0893280580886725, -0.68533881331658, 0.778684927797257, 1.50519714998454, -1.04181171702516, -0.0300463272842381, -0.0204847751966045, 0.351275326492782, 0.151233907999424, 0.129604454285747, 0.204980394597311, 1.80039173466613, -1.74791870471657, 0.330716585087003, 0.143420606214596, 1.67355371563219, 1.57473798057549, -2.00641698419531, 7.18432266532967, 1.1226643529759, -0.181864820460961, -1.01450983251179, -0.500559437774523, -0.64113860780164, 0.841695451268747, 0.608598653363286, 0.0381619773303095, -5.11725544795023, 5.83010510343555, -1.67355371563219, 4.15640649712276, -7.18432266532967, 0.181864820460962, -1.21347234627072, 1.41216997169937, -0.420044643175671, -0.598727598233648, 1.02108754633378, -0.766876717959024, 4.13430811262021, -7.40484308401104, 2.98936431952533, -5.27907085009867},
    {-0.73356085670795, -0.0105569750572741, -0.135243708104237, 0.634253162364344, -0.927843770282091, 0.288623553688335, 0.111739981375583, 0.0384808976947908, 0.0511552733888945, -0.0595916798702371, 3.02009427869259, -4.59614374252045, 2.90000486638575, -0.288623553688334, 0.269193011113971, 0.139898318142587, 0.13004744105712, 0.20497906145155, 1.33072727712461, -0.370698963181918, -0.1202732552096, -0.111739981375581, -0.884630678933836, -0.607725108844567, 0.816310765670254, 4.78299203345091, -3.32007557633769, 9.39853615441867, -1.20548217579335, -0.719642445899192, -0.959725948895812, 2.13674416686427, 1.82514641329183, -1.97216109610366, -5.91543225989015, 1.89946392865065, 0.884630678933829, 10.3770005646753, -4.78299203345091, -9.39853615441867, -0.531164879432237, 0.172337701258934, 0.0817923575148111, -0.317092032288905, 0.147205988534091, -0.422878685533234, 2.75008052051686, -1.29173881980607, 2.34904097370303, -7.05692498833759},
    {-0.886657647844947, 0.475450897669486, -2.77864520754329, 6.81314195983391, -9.17767136168489, 8.1697622215787, -0.177662099660995, -0.103065710691024, -0.120604248577172, -0.371710710442358, 4.78510003641373, -10.5867211358993, 12.2873339520157, -8.1697622215787, 0.132577055868038, 0.0803075313210172, 0.0782097357576751, 0.120112427228734, 0.640122362237717, 0.187403044735946, 0.137987377882087, 0.177662099661, 1.24405855005195, 0.919596708984911, 2.38281038308833, -4.24574550407263, -6.94951052569271, 14.8194162946469, -0.768741936063631, -0.529113173239227, -0.708599719567575, 1.93877527712577, 1.83480389893961, -3.1096625903308, -3.70257205767906, -1.23118288062099, -1.24405855005195, 9.30420727296104, 4.24574550407263, -14.8194162946469, -0.102837746404373, -0.0343196335605707, -0.0349216671910539, -0.0707816287575058, -0.0324791625982124, -0.0947922766334829, 0.678062175725282, 0.311586171636099, 0.64169949886543, -2.35469674726832},
    {-0.537207852379277, 0.158558075149592, -0.0772323978522176, -0.162307063211113, 0.351454288722324, -0.153888727424812, -0.207274898435274, 3.23016842177171, 4.65360605107617, -1.15503267642692, 0.125473334825657, 0.0765656943091268, 0.0811568320805647, 0.153888727424811, 1.40979731134916, -0.981375100901085, -0.0425451910024548, -0.00722724221594029, 1.87648973454375, -2.19526230893808, 0.74590137212638, 0.207274898435276, 0.345989416244822, 2.86062150854784, 1.01480603870534, -0.732904140956684, -1.49211726658188, 0.562990732964049, -0.361447973783708, 0.301999225693177, 0.0112078111170943, -0.261866842527737, 0.347608211429722, -0.43261112080289, -0.478718100157491, 0.66056664227734, -0.345989416244823, -0.344555506460693, 0.732904140956688, -0.562990732964048, -5.40555818747604, 7.50814597253982, -3.97606979389809, 4.516485097338, -9.96648971467791, 0.167616032021202, 1.63729198148447, -3.52118815082518, -2.17337992003233, 1.83667277304257},
    {-0.595832618747913, 0.14436964159384, -0.258546391444691, -0.0463981325961381, 0.14220532642428, -0.0578765020203709, -0.0940416992472924, 0.626669039303187, 1.08381574406082, -0.659828577798066, 0.688772361232905, 0.0662246334629471, 0.0424502734777083, 0.0578765020203714, 1.3266591866883, -0.643706401528916, -0.129946574602136, -0.101543233403178, 1.75779551592058, -1.54694373190725, 0.187069449276908, 0.0940416992472928, 0.414950014620733, 7.63196465955174, 1.54919143700807, -0.386261969519167, -0.833823320741947, 0.240306794105974, -1.67523377960584, 0.967387946626125, 0.27761986319151, -0.189558115280255, 0.169731614413445, -0.184655599901986, -2.21118026393013, 2.27824790474839, -0.414950014620734, -0.248907891761944, 0.386261969519168, -0.240306794105977, -4.2753144458915, 4.4278952246524, -0.813738488580093, 2.46884208839501, -3.96476723680597, 0.708505419373975, 6.3291730200364, -9.91021256430012, -5.66718419311436, 1.08273121250389},
    {-0.692328341357121, -0.0683533897674913, 0.32572554428539, -0.509884452975199, -0.0652094687232041, 0.0204847751966056, 0.107893098474517, -0.264160904494418, -0.285434670419235, 0.273032358288873, 1.76462388414566, -1.18519576926094, -0.0327246027163566, -0.0204847751966044, 1.12305922694355, -0.188614565355217, -0.0838593924928049, -0.0899035379709125, 1.4791576191489, -0.663475738095749, -0.132566034075006, -0.107893098474517, -0.61593226685386, 4.22180931675051, -1.75512603369549, 2.39711456519615, 6.32158471208019, -0.127449277215946, -3.1050608945562, 0.616801919073809, 0.397909547051346, 2.42276752341874, -0.727687301182593, 0.0979340714395627, -4.07968499042822, 2.11866520828912, 0.615932266853857, 3.17241272304676, -2.39711456519615, 0.127449277215946, -2.60274981052279, 1.35627169158478, 0.396726938569422, 0.517020803300189, -0.407361283069787, 0.333539029784827, 8.33965877785055, -6.34047452503962, -2.50484775372684, -9.49399743512696},
    {-0.809132783013124, -0.301286830201028, 1.40654666417306, -2.52815393023257, 2.02269013545502, 0.288623553688338, -0.00868264811335495, 0.00516951492373158, 0.00741750939162993, 0.434321553684358, 3.29120129853804, -4.92980584650552, 3.03841463918548, -0.288623553688337, 0.775874628211621, 0.136632341463629, 0.088874396581399, 0.109038246957507, 1.01627603113289, 0.0144638665898124, 0.00776546518244309, 0.00868264811335731, 0.0545145077501616, -0.0481935039776857, -2.28026370449516, -0.173990878371734, 5.07213902439576, 6.58641202148129, -3.39887571944394, -0.680950364661223, -0.617921878605931, 5.8347030362164, 1.62325674052168, -5.06110477464049, -4.44642915431989, -0.0723946023331807, -0.0545145077501683, 7.61919049960716, 0.173990878371741, -6.58641202148129, -1.0495243569589, -0.017118801213403, -0.0129349801061758, -0.21026776280861, -0.00476257476803785, -0.190805465049732, 5.2625994805498, 0.120588106310876, 1.46409337826524, -12.6913295240029},
    {-0.919863564854675, 0.0509205961060805, -0.532258011356856, 2.09010537178824, -4.52533087778609, 8.1697622215787, -0.208038376322082, -0.0556119987710085, -0.0361803346632916, -0.0498979683742761, 4.94227542033299, -10.8701129799616, 12.5055756024883, -8.1697622215787, 0.355163261239761, 0.166948941881289, 0.144359812496201, 0.20247095313135, 0.46332699059249, 0.188164831664138, 0.153681592311879, 0.20803837632209, 1.36137347904748, 0.44340896978033, 0.512269979721147, -4.15186465410292, -2.40137913268834, 10.3853174612206, -2.04315695857813, -1.08754776625848, -1.27931268413951, 5.09262420250076, 3.68738340567263, -7.98024472470219, -2.66418747636995, -1.2253434155785, -1.36137347904748, 6.63611769063915, 4.15186465410292, -10.3853174612206, -0.191541280971823, -0.0881548128854482, -0.0980695935408669, -0.101955109906063, -0.0638296841154072, -0.119932631340331, 1.24816601696523, 0.781934445798191, 0.903751479683547, -4.23473855795081},
    {-0.732933512062504, -0.277763665240076, 0.588308925193392, -0.933533835729232, 0.622081242620204, -0.153888727424811, 0.112109676444357, -0.801299944278725, 3.66828774344941, 4.76741169537867, 0.164508796215618, 0.0953652459992167, 0.0938519633344865, 0.153888727424811, 3.11854471061592, -4.9817717922754, 3.43812579625538, -0.564201537293317, 1.33311110113324, -0.374734315630515, -0.120890275800025, -0.112109676444354, -0.048490258604643, 0.6939366006138, 3.16270113924141, -0.100444997186996, -1.43107924601666, 0.305058910320096, -0.756279625079238, 1.39497923258148, -1.39151732891125, -0.505847432980326, 1.34401602271034, -0.71593320595469, -0.323086063538459, 0.104692615586598, 0.0484902586046292, -0.215903627571554, 0.100444997187002, -0.305058910320096, -6.12857662901218, 1.98770935496674, 0.92219022007874, 11.304333528568, -5.28126278278563, -11.2762796960677, 1.71375068726822, -0.798629216200376, -4.55336576297118, 1.64698287358821},
    {-0.768918916307804, -0.189231932784835, 0.633760217237775, -0.45701949050495, 0.257719653538465, -0.0578765020203708, 0.071850949628521, -0.378957450051657, 0.950790563068446, 0.876395386990307, 0.860731749519539, 0.0793721793555832, 0.04786905946078, 0.0578765020203717, 2.78591037126002, -3.50369899675694, 1.50929321361253, 0.166646260977034, 1.19026299568438, -0.165127134989546, -0.0703878670507415, -0.0718509496285226, -0.138457447035778, 1.23186527430633, 8.26840567312807, -0.0344014808380364, -0.818819508251803, 0.130211252974858, -3.36284990588641, 4.8644771376879, -2.9961191985588, -0.356429949679289, 0.734077260828655, -0.305588712999244, -1.43600304080498, 0.228807664661238, 0.13845744703577, -0.152088205885712, 0.0344014808380339, -0.130211252974858, -4.65031736688181, 0.741522494315499, 0.4493453171024, 6.72684275161749, -1.5271859223944, -4.14318376741036, 6.44853770841326, -1.46067293896756, -13.2809403407364, 0.97090771413751},
    {-0.82658934733983, 0.157476470390113, -0.872076066213631, 2.13904455634208, -0.126473726995891, 0.0204847751966056, -0.0498108934158, 0.152790022173939, -0.117241776514703, -0.3960947971112, 2.06115841063896, -1.34512191896953, -0.0355985137911548, -0.020484775196604, 2.19292758691862, -1.6191000610783, 0.0618456547462395, 0.0334396963631514, 0.936173063458539, 0.073726600477435, 0.0430057517929103, 0.049810893415801, 0.183683333413712, -0.801707367261882, 3.98968710934988, -0.234701787102886, 6.64351752931566, -0.0690589300180579, -5.87903533992085, 4.96347886691586, -0.273525871420337, 4.38412956517545, -5.17805220254801, 0.162072240787049, -2.50888442912758, -0.225656280144812, -0.183683333413723, 1.86994845496093, 0.234701787102893, 0.0690589300180601, -2.67024334274654, -0.240295667132243, -0.195795773966854, 2.25439985217434, 0.283810843169517, -0.124234775775135, 8.17715795589205, 1.0273636474067, -9.65796063611436, -8.5134659842766},
    {-0.894125141570335, -0.106326707514486, 0.723274251164084, -2.23508757936844, 5.18874690712782, 0.288623553688343, -0.18826002541335, 0.235287056522915, 0.0808194877088783, 0.121015976580037, 3.59296158151086, -5.29595271223313, 3.18693460622812, -0.28862355368834, 1.40933159450745, -0.233194862616465, -0.0910704708274636, -0.0846144119787011, 0.601163049702498, 0.191498619152118, 0.144513593792824, 0.188260025413353, 0.967224318059458, -1.52366184003683, -1.09234185562157, -2.1443389631563, 7.05028621759748, 3.56887521685112, -6.06899436840763, 1.14174546006665, 0.611013075666042, 10.156758431456, -2.62571813985443, -8.37568151335594, -2.58826135779402, -0.93583493917042, -0.967224318059466, 4.33029429510406, 2.1443389631563, -3.56887521685112, -1.0154450807139, -0.367262572075591, -0.379800650315744, 0.191033265228804, 0.094944465214598, 0.10223278920256, 4.9638557587311, 2.45949677920725, -1.28325254531552, -11.3805805127016},
    {-0.956163846795197, -0.201011180775914, 0.994395503589025, -1.74785530185878, 0.466841577692117, 8.16976222157871, -0.175290778787532, 0.00451660429035489, 0.0213791403147805, 0.0505128969868047, 5.11339400277779, -11.1769121298321, 12.7397587969252, -8.16976222157871, 0.609338298546825, 0.201772750556845, 0.152230891222923, 0.193833087244528, 0.259748101819746, 0.136711987803173, 0.123254765620596, 0.175290778787543, 1.07177009482939, -0.0323224317093871, -0.232123264151706, -2.93375302702634, 0.134233956150086, 5.62732820017931, -3.48236863020847, -1.3018111266894, -1.32360974946894, 8.59495959374125, 4.32980783794964, -13.2066003746173, -1.48434559997857, -0.882055076891873, -1.0717700948294, 3.6631446414076, 2.93375302702634, -5.62732820017931, -0.176895940551797, -0.105130468728658, -0.127771369910954, -0.0661288703553199, -0.0529606593893012, -0.0672361878994402, 1.14145036721826, 0.914377508601275, 0.57501849691201, -3.79737859755769},
    {-0.909191772905899, 3.08041774684675, 2.82690544195194, -1.94822363815396, 0.838592338884265, -0.15388872742481, -0.203361938358282, 1.23021933768942, -3.37625725056366, 7.41209798963016, 0.198758544842829, 0.111280669107779, 0.104008515538234, 0.15388872742481, 4.97565184444245, -11.2102736013573, 13.3540419915798, -9.29064620860581, 0.521141524151647, 0.19284858194136, 0.152581035106698, 0.203361938358284, 0.055225308805833, -0.412559857641102, 2.13559821785096, 0.0442987734008502, -0.62419475836183, 0.0987043662597645, -1.16469683937875, 2.97486092842636, -4.83582807584237, -0.739226132411897, 2.57616910145809, -0.942600854422499, -0.121983325719704, -0.0511687698232401, -0.0552253088058323, -0.0774173185392778, -0.0442987734008447, -0.0987043662597641, -3.05381530403331, -1.28107345608763, -1.38280037279612, 7.80003475878301, 4.46448212470992, -12.6794589685315, 0.810322788436828, 0.46372862746434, -2.82393768056808, 0.701612076901091},
    {-0.921992874650238, 0.152389035168538, 5.95560097140644, -1.01996684985415, 0.350135184354865, -0.057876502020369, -0.208294305373909, 1.02887840822964, -2.1059751734363, 2.66473085647859, 1.00949188137989, 0.0904065681741683, 0.0522042791302064, 0.05787650202037, 4.31260418155548, -7.88096662753471, 6.85421843413416, -2.51625214867323, 0.45167971773004, 0.186666839336591, 0.153403194006992, 0.208294305373909, 0.276695252725785, -1.66099558399411, 5.97070702165881, 0.0341424110805648, -0.359939116570948, 0.0421309418278852, -5.05317127732501, 10.4569541694403, -12.3688757449016, -0.512443988080633, 1.44200426976062, -0.402339463485072, -0.529224191432094, -0.247650281877962, -0.276695252725779, -0.0536660208712427, -0.0341424110805717, -0.042130941827881, -2.26095426648093, -1.05806803116085, -1.18228160223663, 4.67878364817811, 2.97737636526055, -5.5342399559058, 2.99975721591098, 1.90864586587206, -8.44124004613769, 0.413605137442181},
    {-0.942112680825337, -0.118966191906676, 0.634469463727496, 6.31394635781805, -0.175487291718016, 0.0204847751966104, -0.198009063873279, 0.656737611459916, -0.697326437699304, 0.093241262400466, 2.31225141453112, -1.47759207750688, -0.0378977438878146, -0.0204847751966089, 3.24462035975901, -3.79012959121019, 1.41400410247192, 0.19258400171127, 0.339807119497161, 0.163188592583588, 0.141827214705654, 0.198009063873278, 0.569011825944304, -2.24147506987856, 3.81041733440209, -0.472921523560186, 2.98260739215466, -0.0223445953925928, -8.51468759087905, 11.2436827819768, -5.67571606935638, 6.15067472850094, -10.9870290088121, 0.213385035605832, -0.891715283316586, -0.484062676548036, -0.569011825944311, 0.644115536567862, 0.472921523560192, 0.0223445953925968, -1.2513135230416, -0.679294351711108, -0.798564826165568, 1.65236506492014, 1.21343219682683, -0.834099923776169, 3.71188418075948, 2.72553774642659, -6.63058623184497, -3.62672292872254},
    {-0.965098624043572, 0.0924967660603456, -0.480953409540751, 0.747335856394909, 7.72170385245787, 0.28862355368835, -0.15321307787896, 0.248794090236757, -0.0154312292192889, -0.0260372649966731, 3.84258032344357, -5.59493014084545, 3.30575581164892, -0.288623553688347, 1.98642110683366, -0.851816659728463, -0.154868875104906, -0.107133714017057, 0.208024894236653, 0.115525902752574, 0.106511791363549, 0.153213077878959, 0.687256057948901, -1.29479913067598, 0.118176874412039, -1.22449380271057, 3.394767625231, 1.15473947694187, -8.44838829447964, 4.08753636027175, 0.999225020305065, 13.8784160686136, -9.0308217841631, -11.0274596641068, -0.884731210125906, -0.554319335853292, -0.687256057948902, 1.45334151901369, 1.22449380271057, -1.15473947694186, -0.457368469490919, -0.286565963245457, -0.355305881600308, 0.221286023312573, 0.186471289712178, 0.0540948169383624, 2.19460610479754, 1.84911846652927, -1.42805176908367, -4.8481091442447},
    {-0.98570864166634, -0.28741790529488, 1.65533088393823, -3.90144231251835, 4.4607553970921, 8.16976222157871, -0.0766373383161938, 0.0179266998689587, 0.018320822636553, 0.0272069959806244, 5.25214046484748, -11.4243799830648, 12.9271136018471, -8.16976222157871, 0.824345590157431, 0.182949292363595, 0.122272864461128, 0.14355879997908, 0.0863241095199967, 0.0536478621586483, 0.0519509511468603, 0.0766373383162066, 0.44519579967488, -0.118570848381708, -0.167645164909417, -1.12631163354646, 0.415005565818924, 1.82076918570122, -4.68789953047446, -1.17177810848706, -1.0477937098242, 11.4849942748956, 3.84082802068747, -17.3878689989392, -0.490906160550785, -0.343614184247037, -0.445195799674886, 1.20267165920764, 1.12631163354647, -1.82076918570121, -0.0770502531674342, -0.0539325014281548, -0.0698776510158275, -0.0192593291148977, -0.0180361833670483, -0.0172215232183001, 0.493510759603061, 0.46218503262875, 0.165040565857146, -1.61767722502658}};
    
    // Array of non-zero columns
    static const unsigned int nzc2[50] = {0, 2, 4, 5, 6, 7, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55};
    static const double FE0_D001[125][51] = \
    {{-3.62990990645797, 0.401957822762748, -0.164167775300454, -0.0765004719955982, -0.0680662120055078, -0.108642456439214, -0.175828494830759, -0.0826780082397547, -0.074605046536711, -0.122922772706513, 0, 7.72557159650525, -7.92049346364189, 5.76645406988605, -2.34358011905419, -3.3928013368602, 2.20168218722082, -0.91619315866649, 0.164167775300471, -3.83876213073824, 2.41318852918331, -0.9901772308779, 0.175828494830749, 0.0485713098083429, 0.0330175826697559, 0.0508495009974803, 0.0267387390754377, 0.027601778794744, 0.0366395230945579, 4.90215896975701, -2.6666625540809, 0.992693630662062, -2.10329332423928, 0.533046578865592, 0.702578147781685, 5.54651461832333, -2.9228376030211, 1.07285523911768, -2.37975700940869, 0.584254120374478, 0.794927294530122, -1.18514873927512, 0.320231355091354, -0.0485713098083416, 0.330567383953277, -0.0330175826697592, -0.050849500997479, 1.43544412648885, -0.346970094166791, -0.358169162748018, -0.286934910308289},
    {-0.282631633534522, -0.292253144230847, -0.149034415307038, 0.077243285110048, 0.054250729917274, 0.0737033512859821, -0.161086287535331, 0.0841312964337333, 0.0597928794101201, 0.0833911584363383, 0, -3.47320825508685, 6.80476470939691, -4.50883814356456, 1.75216646701988, -1.23802219859217, 1.27243405037863, -0.690001392244449, 0.149034415307047, -1.40075184518967, 1.40242344845198, -0.751530849418156, 0.161086287535333, 0.0376730129072205, 0.0260288856266601, 0.0391350848847589, -0.0228923090116634, -0.0235005750094148, -0.024173812801302, -0.0717785195213282, -0.829453162063557, 0.612758107134389, 1.7451053455173, -0.497231618232345, -0.509007978689772, -0.0812133205517753, -0.914188490573982, 0.66739955298443, 1.97448764292228, -0.548027837288126, -0.575913635617165, -0.566989284434577, 0.204493181086761, -0.0376730129072227, 0.209926719868722, -0.0260288856266527, -0.0391350848847611, 0.369599551890683, -0.181600872075094, -0.186426144859309, 0.221563545345176},
    {0.143241115565021, 0.165069823211546, -0.12019863508044, 0.24209313321224, -0.0757646835227037, -0.0476672211627472, -0.131658137530836, 0.266652519476152, -0.0841658993569981, -0.0539327550626433, 0, -1.05011719804096, -1.87263337149761, 3.7306967564191, -1.1162571256571, -0.0547136501038982, 0.383689812925608, -0.382777303820134, 0.120198635080445, -0.0619053894407151, 0.426235505482272, -0.42160854010033, 0.13165813753084, 0.0223418536780489, 0.0158074656855302, 0.0229503204656527, -0.0521578843465159, -0.0531230966917486, 0.0236990433705241, -1.30196185927216, 0.524552634438834, 0.140684170607886, 0.913368302496435, -0.832477763841734, 0.49097442804237, -1.47309594191116, 0.582717991878106, 0.154956020624173, 1.03342438973593, -0.924787598003382, 0.555509696678585, -0.120017415695227, 0.0824676605991068, -0.0223418536780539, 0.0839937731914961, -0.0158074656855191, -0.022950320465658, -0.164079028060237, -0.0303097762525916, -0.0308706764997406, 0.260397400384933},
    {-0.102291761943679, -0.29871276705299, -0.0793175468753286, 0.289572373956813, -0.3697329961386, 0.156600961026923, -0.0880875552963095, 0.322644424074253, -0.414015466198862, 0.177185098430693, 0, 0.660223076443687, -1.85149395530345, -1.71903824819843, 3.31131365605486, 0.0128161900329826, 0.0101111233234072, -0.127158273749779, 0.07931754687533, 0.0145007915507604, 0.0113221202333887, -0.141681015490764, 0.0880875552963087, 0.00878997436984741, 0.00637514851196497, 0.00892592724339991, -0.0367385953748303, -0.03712162371112, 0.065677508256206, -0.0915005962409912, 0.532781488795775, -0.162414100207036, -0.922814737293642, -0.173159615980589, 0.844898182474736, -0.103527730897131, 0.596592077984885, -0.180963408583481, -1.04411249451119, -0.193898732019425, 0.955954335426873, -0.00179608904937001, 0.0161328109585034, -0.00878997436984397, 0.0163010080188217, -0.00637514851195653, -0.00892592724340135, -0.0946406217313094, 0.0206057844163101, 0.0208206156923136, 0.0307592025244685},
    {0.290248257600272, 2.37732743172645, -0.0350775308879017, 0.177387214604194, -0.369081269115191, 0.431683257087092, -0.0393989079388157, 0.199502173687779, -0.415917431700268, 0.488425102223403, 0, -1.82190907285356, 4.91114087352822, -7.51056431849916, 1.75375682849779, -0.0117306785136109, -0.00833981435771294, -0.00857523129456578, 0.0350775308879063, -0.0132725968823191, -0.00939813114015164, -0.00964430997440779, 0.039398907938808, 0.00157505305407457, 0.00116582089220063, 0.0015845379344096, -0.00902347074880971, -0.00905985545386756, 0.0255178519104238, 0.0802149854970493, 0.0633635584940249, -0.168811983309653, -0.246795124905892, 0.314057524978899, -0.25337243916463, 0.090758702933328, 0.0714043510671623, -0.189857863713356, -0.279234675265855, 0.353911211773259, -0.286676533008572, 0.000576605115313544, 0.000436211532629294, -0.00157505305406125, 0.00043797043764926, -0.00116582089221756, -0.00158453793439595, -0.00438088312107587, 0.00858725921616355, 0.00862188501623984, -0.0217135739046714},
    {-0.0804169334283808, 0.401957822762747, -0.0187732230697977, -0.0120162719057391, -0.0185107978054506, -0.540385364807997, -0.15868462293527, -0.0736681995781838, -0.065165386944685, -0.102669679617485, 0, 1.29100934347991, -3.33726521147606, 3.65680528243622, -1.93209030377443, -6.18235833041367, 0.364179106695019, -0.117173170875437, 0.0187732230697977, -1.17460758636495, 1.28205562258155, -0.718353962441769, 0.158684622935265, 0.215265220041943, 0.00784309727827637, 0.00667117274779272, 0.116169922080392, 0.00626960940031562, 0.152217180438976, 10.5052874931861, -0.465503857415097, 0.129189442781173, -6.66708194356736, 0.119835548525529, 2.88453814560296, 1.99593581073058, -1.63875913461504, 0.792022162019963, -1.2667018977703, 0.421868898978173, 0.548043353022147, -2.99470165351671, 1.13279711355544, -0.215265220041938, 0.0611362675002899, -0.00784309727828947, -0.0066711727477663, 3.82791089848766, -1.24896703563583, -0.0674058769006247, -0.985426425409911},
    {0.296938442417927, -0.292253144230847, -0.0798554220922585, 0.0537913438739425, 0.0586818441400101, 0.366598967637804, -0.143523611516616, 0.0741505562562731, 0.0518232284292406, 0.0696514025113293, 0, -1.53911310909653, 3.04154683397432, -2.98012962965793, 1.47301060659307, -1.69866660382198, 0.804194002372017, -0.389277637912434, 0.0798554220922585, -0.322735527919581, 0.710201427666698, -0.536613352828258, 0.143523611516616, 0.165154795443352, 0.0244021863638662, 0.0227629108632699, -0.0986889659037735, -0.0212317853742989, -0.100428971621051, -2.28978383113606, -0.410126006315278, 0.335486294038491, 5.7511375010484, -0.452749840196746, -2.12928603372817, -0.435043929103384, -0.362191305019967, 0.462462796571987, 1.09267845341915, -0.399833351075974, -0.404550398907511, -1.37630945014863, 0.714193116741434, -0.165154795443351, 0.153650358290697, -0.0244021863638694, -0.0227629108632663, 0.701895682607094, -0.615504150837663, -0.132418572916395, 0.774842739162579},
    {-0.100404462764307, 0.165069823211547, -0.166654346797131, 0.403271313827147, -0.169838959850657, -0.237095786874347, -0.115126938239243, 0.231341613116329, -0.0721435208691166, -0.0450466464533047, 0, 0.610810404026161, -2.08773341497668, 2.34797231905812, -0.935714668554841, 0.233706960786521, 0.430577162468102, -0.509642151511679, 0.166654346797132, 0.0444027916945068, 0.18289886215506, -0.2923625693181, 0.115126938239245, 0.0964120402117682, 0.030373435758233, 0.0319310700158484, -0.222374576611211, -0.0994635029161277, 0.0984565642859258, -3.72129972467468, 1.20675741154257, 0.106370837684533, 1.67719581905213, -1.46749561416001, 2.04749273171038, -0.707022571991269, 0.512601634984939, 0.0610209562017676, 0.318656219453792, -0.623356976270884, 0.389010207296274, -0.24960791159974, 0.281030298411488, -0.0964120402117693, 0.125698982012876, -0.0303734357582321, -0.0319310700158537, -0.699563803096376, -0.0586557218002812, -0.0262354790967381, 0.850715150410185},
    {0.0327513821297281, -0.298712767052989, -0.207912100848701, 0.842273262501726, -1.26534612149435, 0.778929989503397, -0.0755398622052359, 0.275404331483141, -0.350914290152959, 0.147991595766438, 0, -0.224009962170146, 0.764255032997442, -2.99246181614667, 2.71817813024264, -0.0383246455489349, -0.0506149010319225, -0.282951433929325, 0.207912100848701, -0.00728143161567441, -0.0140368645108792, -0.0925187275588496, 0.0755398622052352, 0.0373197059437171, 0.0184925126068624, 0.0216849831681613, -0.15488559873214, -0.106110399883791, 0.272854128019651, 0.402289946927961, 1.4832894610066, -0.5593218285724, -4.2518295597099, -0.167328438480332, 3.10893426882748, 0.0764324547891824, 0.411355801751532, -0.182885603924288, -0.807819765508762, -0.0464046470877029, 0.590677146568822, 0.0109143928694786, 0.0520319286000908, -0.0373197059437143, 0.0356464951917918, -0.0184925126068599, -0.0216849831681616, -0.319850549670729, 0.102853670132037, 0.0704639046920081, 0.0360820287815957},
    {0.239734464658433, 2.37732743172645, -0.138407634889364, 0.728837230454808, -1.61290224291864, 2.14718372548066, -0.0332504769431173, 0.168055578568884, -0.349376359563605, 0.407950843104913, 0, -1.44405251046299, 3.65676158658081, -4.9485015917254, 0.118730619222707, -0.0438867308521977, -0.0232241693750271, -0.0134393276185013, 0.138407634889369, -0.00833819138986303, -0.00503066803072777, -0.00309884550915585, 0.0332504769431134, 0.00659940057926056, 0.00427961072299976, 0.00543776875031554, -0.0377019670391376, -0.03306870350435, 0.10601271906982, 0.274951488167143, 0.131144421288151, -0.715397902836331, -0.69309074938494, 1.50498199100555, -1.68515773341067, 0.0522389817320213, 0.0284076488131166, -0.164956733059719, -0.131682702418215, 0.325999378781224, -0.320168931028872, 0.00152647648323301, 0.000695201981636639, -0.00659940057925357, 0.000609767341389011, -0.00427961072301186, -0.00543776875030455, -0.00861985080165373, 0.0370067650574943, 0.0324589361629763, -0.0989193447514123},
    {0.0232138437439115, 0.401957822762745, -0.0496222433554898, -0.0732932595554749, 1.03279003696238, -1.19524818143039, -0.124486375182813, -0.05669879328169, -0.0486993761139199, -0.0719500144452042, 0, -0.196125308190234, -0.264715365073909, 1.3436161590818, -1.30794715232431, 0.136336811707852, -6.42186515765067, -0.46733271429188, 0.0496222433554884, 0.0082070282341305, 0.302811622374954, -0.361523025747756, 0.124486375182812, 0.366456256144685, -0.327024976772805, 0.0285157733865296, 0.19202361006174, -0.245141063756194, 0.235942873780199, 2.23764732712356, 9.9486509475111, 0.540625973847358, -5.51215636599335, -4.55957582682282, 4.33342040859233, 0.134699019008048, -0.469110929597917, 0.418221819029451, -0.331813707244329, 0.214998683336878, 0.260857674447354, -1.46708746802159, 1.22438155217215, -0.366456256144685, -1.56306922907259, 0.327024976772798, -0.0285157733865232, 2.27278847663537, -1.41640516223389, 1.80821029282877, -1.04164388239398},
    {-0.138566640473884, -0.292253144230849, 0.0690671693240325, -0.0852935608097477, -0.578626286337774, 0.810859764011252, -0.110575952246123, 0.0562091216546029, 0.0383177290446038, 0.0488110943317455, 0, 0.288047005908002, 0.339607805362621, -1.2464251449651, 1.04959011839921, 0.797289996155129, -2.08847476667468, 0.397834432404977, -0.0690671693240333, 0.0479942388799374, 0.138302756227032, -0.262175993089586, 0.110575952246122, 0.276909647639241, -0.177909214690158, -0.0252941638539178, -0.161398096603595, 0.146713338525794, -0.155669025708709, -3.60723712754945, -0.524501306672908, -0.312540871595232, 5.3583575689491, 3.19160235968536, -3.35927020156603, -0.21714382625027, 0.0347334703368413, 0.205966871434982, 0.322555524850996, -0.211353955608473, -0.202217031812412, -0.561866682903669, 0.752808530256605, -0.276909647639241, -0.684314468812522, 0.177909214690157, 0.0252941638539172, -0.14110767056486, -0.591410433653013, 0.537601130286728, 0.858643379177238},
    {-0.0867112697486268, 0.165069823211547, 0.103341901603271, -0.360080603668167, 0.318850804445169, -0.524418917575722, -0.0863518149033911, 0.171478616035409, -0.052518723698784, -0.0315682962594101, 0, 0.376226506008878, -0.616920262429731, 0.824205091005292, -0.661869888047359, 0.183906264039709, -0.0385649479169799, 0.281735363146926, -0.103341901603271, 0.0110705530113315, 0.00635213026240974, -0.134168821282, 0.0863518149033901, 0.158067285969267, -0.0415107849939735, -0.0199804170326388, -0.358060552096523, 0.1308587069136, 0.152611713429153, -0.86954090375935, -1.9001630960178, 0.0783452405212391, -1.99482380889212, 1.61987723948962, 3.20487736618748, -0.0523435061924271, 0.312980676953429, -0.0373097947534117, -0.120081840822095, -0.266814083517052, 0.1929230902626, -0.0184583595144396, 0.28015482824081, -0.158067285969263, -0.102386868211357, 0.0415107849939705, 0.0199804170326365, -0.909475973826478, 0.077905723855705, -0.0284718387022377, 0.775322619911769},
    {0.258324173993584, -0.298712767052989, -0.0876613998747703, 0.425969859867179, -0.874047802450553, 1.72287170239396, -0.0550682258188479, 0.19936872170246, -0.251352700649438, 0.103711217302281, 0, -1.355768334186, 2.91217407039357, -3.3345339389644, 1.81851679581624, -0.340104151364429, -0.0663906027038608, -0.0764299194853974, 0.0876613998747696, -0.020473152759169, -0.0190921563335256, -0.0357718627143631, 0.055068225818847, 0.0597556512584037, 0.0091496455907944, 0.00768552650434717, -0.245385023813034, -0.0513656131766311, 0.422935091177573, 2.12752333013568, 0.610441727700277, -0.349539940381781, -6.71794340220494, 0.329996677454142, 3.20765252103973, 0.128069916117827, 0.175546665086723, -0.163596858988095, -0.404398126129996, 0.0948981918962373, 0.193090145469062, 0.03212514868084, 0.0440283676849774, -0.0597556512584008, 0.00921630859196501, -0.00914964559079556, -0.00768552650434667, -0.295381130230706, 0.201356656128046, 0.0421493045846701, -0.159679109627705},
    {0.0664733009545165, 2.37732743172645, -0.204926878310818, 1.15124252490879, -2.81967943501397, 4.74923565701963, -0.0236691033128933, 0.119292615909615, -0.246951587308039, 0.285888386558618, 0, -0.354952681953402, 0.71725409317945, -0.444843288137042, -2.36125885576996, -0.0112611882291887, 0.007681874150058, 0.0309872168383875, 0.204926878310823, -0.000677886541339351, 0.000672789605545926, 0.00321091870429835, 0.0236691033128943, 0.0103614284495394, 0.00528827784435851, 0.00601929582042138, -0.0589436233665764, -0.0405132923465005, 0.164324063305222, 0.00732797081343872, -0.170455174232783, -1.1822297417472, 0.40746196136155, 2.98245273509673, -5.15276440096544, 0.000441119772476295, -0.0149287097387609, -0.122503534613913, 0.0245278716683087, 0.261207507441263, -0.310179491458077, -0.000447680952132323, -0.00158654566608063, -0.010361428449542, -0.00109046890434084, -0.00528827784436345, -0.00601929582041616, 0.00993371320636027, 0.0605301690326614, 0.0416037612508527, -0.173810095559467},
    {0.0307253509165796, 0.401957822762742, 0.798848493973301, -2.9271659390857, 3.46950926213078, -1.89794570036165, -0.0757063131817334, -0.0337954148060366, -0.0281508959837293, -0.0389864197028714, 0, 0.0944314907532909, 0.0609093436761938, 0.0501892200265485, -0.63821322813536, -0.34325558293279, 1.70600788325706, -8.06337466302121, -0.798848493973299, -0.0070509426160251, -0.0138422026979816, -0.0930951975884574, 0.0757063131817249, 0.346842092340435, -0.635046840281537, 0.617093439996418, 0.176258125225889, -0.446225665713599, 0.203008875141375, -0.691254950746712, 2.47616763325852, 10.9905406021069, -0.461204611784973, -7.65168477864637, 3.39366084582614, -0.0141993291095608, -0.0200911230423833, 0.126890612394507, -0.00947377818052808, 0.0620842217240933, 0.0697104696089921, 0.0998223999983197, 0.485532877422675, -0.346842092340431, -1.22920422066289, 0.635046840281524, -0.617093439996418, 0.144886197992376, -0.661791002648574, 1.6754298863765, -0.447717473132075},
    {0.148294415168218, -0.292253144230848, -0.123511007815443, 1.44142868628426, -2.12588434718251, 1.28757175841062, -0.0660640735715585, 0.0330123844602191, 0.0219257613406894, 0.0264484979530274, 0, -0.180059163531047, -0.10084788509338, -0.170374942691494, 0.595240720378553, -0.834686539129948, 1.97633236806258, -2.74429876876708, 0.123511007815448, -0.0171456115571033, -0.0203833251274656, -0.0628514243474495, 0.0660640735715583, 0.258246160894036, -0.374020010534917, 0.231621948713655, -0.146648971282791, 0.292074433552564, -0.133940022417969, 1.37069270707264, -6.55901435254365, 1.30287008248281, 1.2097301566789, 6.70856633166359, -3.03330808303223, 0.0281559167638182, 0.0676477925596548, 0.029839039887232, 0.0248495242021578, -0.0691902287728751, -0.0623083273618993, 0.124517592894689, 0.279201180857412, -0.258246160894035, -0.556072954571814, 0.374020010534914, -0.231621948713657, -0.413246623967974, -0.132552209574631, 0.263998521019248, 0.42266905349126},
    {0.288245120659521, 0.165069823211547, -0.052935062955676, 0.34937328461803, 1.68967763895773, -0.832729675112352, -0.0502251134543608, 0.0984946363236177, -0.0295977865486525, -0.0171054148739841, 0, -0.920706654198747, 0.877390014541953, -0.0419762923181157, -0.368022011896158, -0.947793322172684, 0.95239806700045, -0.0929039539653112, 0.0529350629556796, -0.0194689807209842, -0.0166829897292249, -0.0261913018588223, 0.0502251134543601, 0.144168473266947, -0.123971642824033, 0.010504538613976, -0.320425624243847, 0.375752113893087, 0.13130946394046, 3.51070731206624, -4.77133558495939, -0.256469330652726, -4.56352001332116, 2.12925987900123, 2.83333569853995, 0.0721147652939257, 0.0835786477489027, -0.0723033344647947, -0.0937409887584914, -0.0372978714710229, 0.0582006190595322, 0.0740134548462827, 0.0852063072784729, -0.144168473266942, -0.0999185073056074, 0.123971642824033, -0.0105045386139817, -0.370793519128056, 0.235219316965357, -0.275833606587474, 0.165470600341319},
    {0.211629929169826, -0.298712767052989, 0.0905706902337697, -0.559977693850785, 1.89158620077828, 2.73576399498904, -0.0311077547329551, 0.111786017271075, -0.139356637565186, 0.0561963618328784, 0, -0.98351537489537, 1.78821551556096, -1.57075666147299, 0.853139358690567, -0.318822529019676, 0.0487983959951502, 0.00642910620448367, -0.0905706902337695, -0.0065490540244259, -0.00359506766419745, -0.00128341572370434, 0.0311077547329551, 0.0532028790608656, -0.0109784069985885, -0.00547454398696167, -0.216031881667492, 0.0602345814802008, 0.363899853138859, 1.52772511812188, -0.191467793818467, 0.553548587646301, -2.78682089067383, -1.74891680295496, -1.15784569341743, 0.0313815788483216, 0.0141057848368628, -0.110502601547371, -0.0572451408171761, 0.128845920392521, -0.0237837458395971, 0.00938774512561569, 0.00248026292125947, -0.0532028790608644, -0.000691553477516371, 0.0109784069985881, 0.00547454398695788, -0.0368342198852487, 0.213551618746223, -0.0595430280026772, -0.336453378379224},
    {-0.287383368069142, 2.37732743172645, -0.191346895929756, 1.15800142154501, -3.20320459390344, 7.54135545678689, -0.0130409161051892, 0.0655286205121212, -0.135039569302964, 0.154909831672685, 0, 1.63778163999163, -3.92101133140206, 5.21568572840442, -5.02240010065129, 0.220678982979618, 0.0877231729903858, 0.0875276341409103, 0.191346895929763, 0.00453305036514948, 0.00369820258157887, 0.00495298625306968, 0.0130409161051956, 0.0090378012655068, 0.00328509874800964, 0.00328072950903424, -0.0511813507042979, -0.0249381910695978, 0.141386949797572, -1.41823574667493, -0.701040826328673, -1.24552905568594, 4.26716024495868, 3.81652224724177, -10.6109589380503, -0.0291325163027676, -0.0295542318562624, -0.0704816067652, 0.0876533507875588, 0.160895598577664, -0.217963716522635, -0.0038720323638799, -0.00386854666664804, -0.00903780126552034, -0.00188495525433063, -0.00328509874800595, -0.00328072950903513, 0.0309433947200169, 0.0550498973709599, 0.0268231463239386, -0.168458312153736},
    {0.136246592488564, 0.401957822762749, 9.65546182492156, -9.34793058151145, 6.39444256476644, -2.46012846883513, -0.0267632969820128, -0.0117600193200639, -0.00956480773992907, -0.0126143827284784, 0, -0.226657394765518, -0.11064635532325, -0.09849816852623, -0.102402496636316, -0.586307089086468, 0.628596434706278, 1.3338100015688, -9.65546182492156, -0.00300630723632245, -0.000940254601245138, 0.00167797901909995, 0.0267632969820045, 0.156442970923089, -0.397671458273214, 0.637634365012876, 0.077625993688331, -0.266098137984345, 0.0851415746766406, 1.45478305790563, -4.22351296711155, 8.01412057994265, 0.809257639624936, -2.79952603236114, 0.782394860391011, 0.00745944389157141, 0.00631753106048288, 0.0100820403009863, 0.0041494928908834, 0.0041875312806956, 0.00401175318233404, 0.00836971944700693, -0.0110760692819056, -0.156442970923068, 0.037968227806938, 0.3976714582732, -0.63763436501288, -0.0562357924158094, -0.0665499244064485, 0.228129910177424, -0.0372755017078394},
    {0.0508201537761859, -0.292253144230848, 2.64740738044187, 5.3635060916448, -4.007336635459, 1.6689581466585, -0.0230500588646286, 0.0113638625296748, 0.00739535140953718, 0.00855763310187285, 0, -0.219294787819977, 0.123105212793215, 0.105877368499594, 0.23174519698183, -0.0472458364883372, -0.521409434336827, 1.6335136764243, -2.64740738044187, -0.000242254447823775, 0.000962236603999911, 0.00346098700029618, 0.0230500588646341, 0.115227716275824, -0.23780181420748, 0.278861127094667, -0.0641146324922199, 0.178140384532264, -0.0561742160924954, 0.856480533939566, -0.550320143979467, -6.9970197680691, -0.932466771342657, 5.07906621377531, -1.54572607276708, 0.00439162970204654, 0.00101558995978202, -0.0148248495299739, -0.00478125142011421, -0.00937317797331672, -0.0079257569359924, -0.00730903562677665, -0.0195268034091012, -0.115227716275818, 0.0542545770406099, 0.23780181420747, -0.278861127094664, -0.00771430141764928, 0.083641435901311, -0.232394961572871, 0.0711975531369293},
    {-0.127595092614205, 0.165069823211547, -0.187994765484734, 4.35942248656921, 3.41733871056418, -1.07938914174274, -0.0171736078270701, 0.033346033929805, -0.00987178613475859, -0.00553460029400399, 0, 0.240846300632023, 0.0164204310898302, -0.16180810247346, -0.132933359845733, 0.619074453405154, -1.05732821218667, 0.642176905529868, 0.187994765484733, 0.00317432288256408, 0.00305434107327193, 0.00491213066563387, 0.0171736078270745, 0.0632668059638175, -0.0836263284960161, 0.0424100611619966, -0.138528058270723, 0.245888519339784, 0.0550709643705989, -1.55299123008028, 3.7780084781576, -5.00159939209908, 0.679923322484457, -6.13801897653512, 1.33338259593342, -0.00796300924865112, -0.0109136655364122, -0.0382581645954413, 0.00348632728921874, 0.0177311105978957, 0.0068369593708657, -0.0170390146347994, -0.020406262541314, -0.0632668059638172, 0.0362212951237044, 0.0836263284960195, -0.0424100611620042, 0.0608832158338299, 0.158934320812024, -0.282109814463485, -0.0989151655696205},
    {-0.405296087049493, -0.298712767052989, 0.105852187397965, -0.836801817303567, 5.85275550665803, 3.54611350936128, -0.0104009318950459, 0.0371551073259593, -0.0459080285849662, 0.0181828036919185, 0, 1.65261971942144, -2.4581034456366, 1.42868917785835, 0.0808034024592929, 0.93841284355432, -0.471047119163683, -0.102826734145336, -0.105852187397967, 0.00481174008425411, 0.00369481427796026, 0.00456564298039014, 0.0104009318950491, 0.0229213490636473, -0.0111901151776017, -0.00264699113268718, -0.0922471242723029, 0.0603021493600566, 0.152618975398214, -4.25272441104422, 2.59101850214634, 0.939628551448908, 7.75167485639767, -7.9727268896407, -7.98347679826906, -0.0218059723462529, -0.0203235128010126, -0.0417207503063521, 0.0397469460087478, 0.0625367271080188, -0.0409355174386667, -0.0122832277222654, -0.0113353847076961, -0.0229213490636523, 0.00740996607849692, 0.0111901151776058, 0.00264699113267874, 0.0675645153099971, 0.103582508979994, -0.0677121154385435, -0.207900262985947},
    {-0.72942039794671, 2.37732743172645, -0.142219700439307, 0.917394780947744, -2.83066387967159, 9.77514965223253, -0.00427583515644986, 0.0214338209728042, -0.0440110118211892, 0.0501223738936725, 0, 3.95523965899805, -8.81317854073089, 10.361438686271, -7.15140683831791, 0.636102496984894, 0.146637218987202, 0.105061706660276, 0.142219700439315, 0.00326163468825494, 0.00227990770101838, 0.00245463987633757, 0.00427583515646125, 0.00383182233278823, 0.000946133374444769, 0.000840950140282626, -0.021621511723999, -0.00713052032935568, 0.0592974446860968, -3.72696548902016, -0.991501724824302, -1.02245648760804, 9.5884298752997, 3.67552838550873, -16.272716535497, -0.0191101276579502, -0.0154158162137914, -0.0238884608491581, 0.0491649626206994, 0.0571469203339796, -0.0834388435446841, -0.0030717926081919, -0.00247613456004661, -0.00383182233280988, -0.000816600062181649, -0.000946133374433389, -0.000840950140289398, 0.0207702225283888, 0.0240976462840692, 0.00794712039154888, -0.0769958746063306},
    {0.113928564117325, 0.401957822762745, -0.164167775300465, -0.0765004719955837, -0.06806621200552, -0.108642456439204, 0.0278488230036613, 0.0188757122195102, 0.0326326069046902, -0.604695575897986, 0, 0.78894269314123, -2.78594575938158, 3.34292399522299, -1.86180731586271, -0.997420422902054, 1.2123572948061, -0.716898055757191, 0.16416777530047, -5.55156553714956, -0.581233740849928, 0.176887292825777, -0.0278488230036631, -0.0110890197476325, -0.014442049783802, 0.250144603906785, -0.0116956533355431, 0.135781785234771, 0.18024127694543, 1.77382148720468, -1.56915765522615, 0.793398527752768, -1.22673500179422, 0.424866572425574, 0.558976393930805, 9.87295428418144, 0.752292560813336, -0.195763005045275, -6.82791288688567, -0.203691426868106, 3.11121971575178, -3.21035680540399, -0.109601822293927, 0.0110890197476339, 1.27243264391444, 0.014442049783796, -0.250144603906788, 4.15517436880088, 0.121297475629465, -1.40821442914921, -1.12505884034231},
    {0.269990545910721, -0.292253144230849, -0.14903441530704, 0.0772432851100511, 0.054250729917269, 0.0737033512859881, -0.0363474142259859, 0.0256279208405441, 0.0304689171788609, 0.410227197655731, 0, -1.2295916754101, 2.57619423828383, -2.7496703923541, 1.4253304278005, -0.246860738476913, 0.666461484090052, -0.536618750989814, 0.149034415307046, -1.37400792758611, 0.374305742846375, -0.178040367552021, 0.036347414225987, 0.0114758839283033, 0.0132636522649658, 0.192517726139401, -0.0116653333003971, -0.11560668073424, -0.118918548058068, -0.479979081184731, -0.3155867014998, 0.459375465879757, 1.06739971180867, -0.405125512507519, -0.414263243433012, -2.67152673483959, -0.177243422999313, 0.152412446711478, 5.94106488936636, -0.227531237025924, -2.30575742459638, -1.46089522012828, 0.081040527686283, -0.0114758839283062, 0.803134052795603, -0.0132636522649636, -0.192517726139403, 0.691771564843808, -0.069375194385885, -0.687527372061359, 0.888042203342527},
    {-0.127458660661965, 0.165069823211545, -0.120198635080441, 0.242093133212243, -0.0757646835227087, -0.0476672211627448, -0.140425692842851, 0.350313742212194, -0.15554587883946, -0.265312095263553, 0, 0.724116044082566, -1.99869488560365, 2.14184546442769, -0.904877785456182, 0.0520390799654311, 0.164994643528043, -0.292827817801941, 0.120198635080445, 0.289645525886159, 0.338736145102242, -0.423727874132201, 0.140425692842851, 0.0293515260433029, 0.0292135670273161, 0.112899806483854, -0.0963922921404767, -0.261329132431804, 0.116583008694398, -0.659863470766916, 0.535041768096351, 0.0507346845896934, 0.257401149245724, -0.624271728101687, 0.398090462718508, -3.67274944388542, 1.09844769574502, 0.0734141319200052, 1.43267504511035, -1.2816379620078, 2.21574096815246, -0.253885729690886, 0.116592916890696, -0.0293515260433083, 0.316095043930839, -0.0292135670273117, -0.112899806483856, -0.823296240433125, -0.0202006247502181, -0.0547659114990329, 0.960598961429614},
    {0.0599574981188577, -0.29871276705299, -0.0793175468753288, 0.289572373956813, -0.369732996138606, 0.156600961026926, -0.207786421496727, 0.85574064804697, -1.32030749635023, 0.871628932353358, 0, -0.379676418590269, 1.12826398163816, -3.12670211624595, 2.6168698221322, -0.0111373775675036, -0.0177606114836716, -0.0921747723655418, 0.0793175468753303, -0.0619897889177825, -0.0634227097028403, -0.272393731348948, 0.207786421496727, 0.0233133995269077, 0.0203305360738738, 0.0439094286276417, -0.117160460994639, -0.182612880705714, 0.323088210622904, 0.10286037897246, 0.415161966608262, -0.197397601591272, -0.835811442539925, -0.0276683589859923, 0.587487480108052, 0.572512976403884, 1.48253324003278, -0.583346916698023, -4.65206235346522, -0.0988030339797175, 3.26991023362576, 0.0155199677706807, 0.0372937468960063, -0.0233133995269091, 0.0581281303878, -0.020330536073872, -0.0439094286276391, -0.362785951784156, 0.0798667140986307, 0.124484750317921, 0.0241777733905678},
    {0.226504621301652, 2.37732743172645, -0.0350775308879004, 0.177387214604189, -0.369081269115189, 0.431683257087094, -0.149329588787133, 0.791218693679664, -1.76769717094972, 2.40271588387604, 0, -1.35413846223186, 3.38938832614648, -4.47854796378787, -0.16053395315484, -0.00815851923718558, -0.00470697659168706, -0.00236493139970831, 0.0350775308879051, -0.04540969203299, -0.0225438403438901, -0.0105485501697012, 0.14932958878713, 0.00624660572297797, 0.00495487357813479, 0.00779483782926227, -0.0383507939296295, -0.0445682633951124, 0.125530296925074, 0.0501486952937417, 0.0242223127867521, -0.175022283204504, -0.120288548964371, 0.349565932920145, -0.35338488417927, 0.27912378986184, 0.116011614161875, -0.780670143509961, -0.66951683323449, 1.67422939713175, -1.96691314847042, 0.0016009161629551, 0.000511293877469565, -0.00624660572296931, 0.000594185357548571, -0.00495487357814994, -0.00779483782925161, -0.00823838642263647, 0.0378395000521525, 0.0439740780375821, -0.118892826665403},
    {0.240503275451299, 0.401957822762748, -0.018773223069798, -0.0120162719057305, -0.0185107978054549, -0.540385364807992, -0.0462721460137363, -0.0287392892809517, -0.0417728341956483, -0.505064274719775, 0, -0.225432949938577, -0.93038399386609, 2.04305155426276, -1.52969570867214, -1.09978928821843, 0.188861504297645, -0.0910267577698614, 0.0187732230698002, -1.02790400216701, 0.42619882664604, -0.217708482662349, 0.0462721460137353, 0.0839788330152867, 0.0050276445448541, 0.0328175858533553, 0.0744681666312362, 0.0308421701162691, 0.748803932424149, 3.1094250399099, -0.265613694301751, 0.10304302967559, -3.75720178050126, 0.09526298780956, 2.28795139361779, 2.90618437295319, -0.599403490264056, 0.246447771943307, -3.51162062451299, 0.214977497813656, 2.13840452844658, -7.63987801001015, 0.564118040826963, -0.0839788330152805, 0.233638417164255, -0.0050276445448586, -0.0328175858533507, 10.7446789106129, -0.638586207458195, -0.264480587280531, -3.85360483302686},
    {0.00303876266691769, -0.292253144230847, -0.0798554220922575, 0.0537913438739435, 0.0586818441400097, 0.366598967637805, -0.1034560301836, 0.06802129775864, 0.0709921997545912, 0.342637039714788, 0, -0.0909883265509577, 0.961801708985776, -1.7816239702605, 1.20002496938961, 0.106362388371802, 0.384243622260385, -0.300062668757962, 0.0798554220922592, 0.0994102469069532, 0.464850762373005, -0.379441201277076, 0.1034560301836, 0.151503159036236, 0.0334283475055569, 0.11197788001774, -0.135193174824706, -0.104445794717851, -0.494041531069533, -2.18865593131267, -0.0733896355471881, 0.246271324884014, 3.45136804958275, -0.369535830853203, -1.73567347427969, -2.04559929366765, -0.0887854112807445, 0.311419903518437, 3.2257770366762, -0.447057550846852, -1.62222502963029, -3.23494106613807, 0.754144104129969, -0.151503159036235, 0.58262689954402, -0.0334283475055559, -0.111977880017741, 0.617866197658435, -0.618950929305261, -0.478181104826171, 3.11111639954916},
    {-0.157996351774294, 0.165069823211547, -0.166654346797131, 0.403271313827147, -0.169838959850658, -0.237095786874348, -0.179040800534354, 0.426220485252834, -0.17456667339389, -0.221598546954274, 0, 0.730722796168729, -1.26503601749588, 1.28640251794376, -0.759162768053871, 0.212280187910398, 0.132887934812671, -0.384494298337474, 0.166654346797131, 0.198404964543105, 0.136587063032125, -0.406374916328885, 0.179040800534354, 0.177628166457941, 0.0734950218118239, 0.157078923190059, -0.538082832924357, -0.489292088458691, 0.48433864231135, -1.40717486615218, 1.11461805365544, -0.0187770154896728, -0.229620188568829, -1.07766702861745, 1.66161065368496, -1.31519800398291, 1.14564506225483, -0.0198455689239488, -0.214611574541326, -1.10766545189307, 1.5530031609354, -0.378963472122794, 0.513028758056838, -0.177628166457942, 0.466509795721893, -0.0734950218118226, -0.15707892319006, -3.17861458453286, 0.0250540748675196, 0.022782292736798, 3.0732394143443},
    {0.18815737454253, -0.298712767052989, -0.207912100848701, 0.842273262501724, -1.26534612149435, 0.778929989503398, -0.206275655835313, 0.828214450891496, -1.22650633415315, 0.728016959426351, 0, -1.04404264935558, 2.44886189237734, -3.43241661709402, 2.13815276658273, -0.124775678124369, -0.0915108108550279, -0.19796120064662, 0.2079121008487, -0.116619992839636, -0.0887018873734179, -0.194656929515229, 0.206275655835313, 0.112230332758947, 0.0646345403512655, 0.106675216450868, -0.541352043062346, -0.521990254154906, 1.34225481939733, 0.871934638228878, 1.10830551655859, -0.644312061855103, -3.56562252705771, 0.248551415790785, 2.03953357744981, 0.814942565694007, 1.0742860891142, -0.633557521376266, -3.33256352379708, 0.240922132412366, 1.90622399151636, 0.0970729074129153, 0.12723507344732, -0.112230332758948, 0.122684432759294, -0.0646345403512651, -0.106675216450866, -1.17566916726986, 0.414116969615023, 0.399305821395616, -0.263658559540386},
    {0.143182682873853, 2.37732743172645, -0.138407634889362, 0.728837230454804, -1.61290224291864, 2.14718372548067, -0.13194852831084, 0.692484457385766, -1.52452213044933, 2.00683782653018, 0, -0.819550657871651, 1.90937154879996, -2.13017464132605, -1.48015636420256, -0.021383617285966, -0.00646246759163619, 0.00787298934490473, 0.138407634889367, -0.0199859245989397, -0.0061083521360466, 0.00748030222209317, 0.13194852831084, 0.0271932795573006, 0.0186743065990365, 0.0267500857137174, -0.164514517194193, -0.162675298233921, 0.521509731671387, 0.111530469784074, -0.0152238752248142, -0.736710219799731, -0.136675831966544, 1.63458858573512, -2.10065474601223, 0.104240528147297, -0.0143896722776304, -0.699964759607861, -0.127742319536055, 1.54502015486303, -1.9633501105425, 0.00208954991192234, -0.00177710603524273, -0.027193279557297, -0.00175723856597257, -0.0186743065990469, -0.0267500857137093, 0.00492243043145724, 0.166291623229434, 0.164432536799908, -0.52852171201478},
    {-0.161958917611737, 0.401957822762748, -0.0496222433554895, -0.0732932595554694, 1.03279003696238, -1.19524818143039, -0.159080911263009, -0.0876789141410957, -0.102648366128719, -0.353944630948841, 0, 0.0390773044282917, 0.097855812228211, 0.649020514013168, -1.02595253582068, 0.936332773411781, -2.49685715770195, -0.355570477099232, 0.0496222433554895, 0.277272923798913, 0.248161096178579, -0.425360169840707, 0.159080911263006, 0.566687309540169, -0.689302045071177, 0.140278010579183, 0.404746660098269, -1.20592558612575, 1.16067681193781, -0.427664798036973, 5.06285842519284, 0.428863736654706, -2.72210626437915, -3.59879130445328, 3.40868647043475, -0.126642869207177, -0.503194382873386, 0.51303908398181, -0.806087733173703, 0.357681652823518, 1.00940230953081, -2.80603423924311, 1.96356341508522, -0.566687309540163, -5.85035429731965, 0.689302045071177, -0.140278010579178, 5.68977446135003, -2.3683100751835, 7.05627988344538, -4.04441703404473},
    {-0.13864766091785, -0.292253144230848, 0.0690671693240334, -0.0852935608097461, -0.578626286337772, 0.810859764011251, -0.186910868101331, 0.111478619052203, 0.0981612040014786, 0.240117043793169, 0, 0.317770488090397, -0.0507656580653119, -0.694388193814171, 0.858284168937782, 0.504910546534909, -0.523142764392621, 0.29869869086601, -0.0690671693240332, 0.149517380433583, 0.088748687762628, -0.39039896159007, 0.186910868101329, 0.549190313108646, -0.455762466940976, -0.124429905392883, -0.413464781999867, 0.721728812150955, -0.765784639231633, -1.57275384198324, -1.51481783532979, -0.213405130056265, 3.00613811948018, 2.61658688606019, -2.7491545880431, -0.46573405157408, 0.256981658230577, 0.27892034253787, 0.890197085267534, -0.443891549994684, -0.814097457920204, -0.692354810965479, 1.44795677340804, -0.549190313108642, -2.52749972334539, 0.455762466940977, 0.124429905392885, -2.0047900638453, -1.03449199140818, 1.80577091119442, 3.46292951404242},
    {0.0784651408728718, 0.165069823211547, 0.103341901603272, -0.360080603668168, 0.318850804445173, -0.524418917575723, -0.208140763677233, 0.463356214530207, -0.169794516671449, -0.155294325586674, 0, -0.257876609617175, 0.190007536035743, 0.362477968217109, -0.538143858720096, -0.245495675775413, 0.130028469360583, 0.203425857830673, -0.103341901603272, -0.0726977691427049, -0.0692427956925709, -0.261770932568323, 0.208140763677231, 0.427117158752061, -0.134205540011375, -0.0982899223488917, -1.15761987536158, 0.643734919056187, 0.750744763634324, 1.18958267109402, -1.55588030115278, 0.156654745837495, -3.02641239372521, 1.10700102734702, 2.60674431598232, 0.352266923342759, 0.82853779902661, -0.201585281961885, -0.896200834636828, -0.589500486662589, 0.77192600602345, 0.306156331221138, 0.653991950103147, -0.427117158752058, -0.363675040506323, 0.134205540011374, 0.0982899223488916, -3.66337162286536, 0.503627925258423, -0.280059878549862, 2.6064705280099},
    {0.301798607677329, -0.298712767052989, -0.0876613998747696, 0.425969859867177, -0.874047802450552, 1.72287170239395, -0.183558386739005, 0.709995728118776, -0.991050736709452, 0.51018792444115, 0, -1.49226719973947, 2.93789419330861, -2.86075292287086, 1.41204008867737, -0.353473378128933, -0.0577177593874797, -0.0463079366625643, 0.0876613998747686, -0.104672825540183, -0.065444049858379, -0.0771848910123976, 0.183558386739004, 0.212802975121333, 0.0360758526960666, 0.0378075093271792, -0.96752096953415, -0.252683521184092, 2.08055003069053, 1.97132969884692, 0.400450976376433, -0.379661923204611, -4.89076560463873, 0.5313145854616, 1.55003758152678, 0.583762915164494, 0.454056670631958, -0.632810837106379, -1.44828517950099, 0.602438115935874, 0.45900716543553, 0.137389151632578, 0.105180915360673, -0.212802975121333, 0.0274696723808525, -0.0360758526960676, -0.0378075093271786, -0.953218220850293, 0.862340054173472, 0.225213848803244, -1.26472096147281},
    {-0.058605431095522, 2.37732743172645, -0.204926878310817, 1.15124252490878, -2.81967943501396, 4.74923565701962, -0.100463988122245, 0.519760266641373, -1.119473853312, 1.40637441497817, 0, 0.371091378649267, -1.04215447025366, 1.83408597516299, -3.48174488418951, 0.0441106655646064, 0.0347242019943326, 0.0545787203230051, 0.204926878310822, 0.0130623359121058, 0.0137862608518743, 0.0246410722452101, 0.100463988122248, 0.0451449469243799, 0.0239726694626586, 0.0296107993050357, -0.267201543013555, -0.199297559822288, 0.808361476937409, -0.339001693001097, -0.356281769552847, -1.20582124523181, 1.34245718501448, 3.14123700257251, -5.79680181459763, -0.100387376433183, -0.141451585053744, -0.544401338886589, 0.397537114297042, 1.24713917751389, -1.71658648875415, -0.00995492851460023, -0.0126676334225669, -0.0451449469243848, -0.00944840513031997, -0.0239726694626618, -0.0296107993050326, 0.102140851142603, 0.27986917643613, 0.20874596495262, -0.90054739956543},
    {0.186613680044609, 0.401957822762745, 0.798848493973304, -2.9271659390857, 3.46950926213078, -1.89794570036165, -0.206357552845447, -0.101475401854199, -0.0982827940163736, -0.191786395599148, 0, -0.00917970571438348, -0.0214830642039402, -0.0724954806499376, -0.485413252239094, -1.15030067029804, 3.12783181000749, -5.64479241115494, -0.798848493973303, -0.116237266097612, -0.0886038993629897, -0.195686746233608, 0.206357552845441, 1.04144129912852, -2.21712935283643, 3.03567569186269, 0.615367305726548, -2.19512365339655, 0.998664169080539, 0.0992128742326708, -0.694554281174871, 8.57195835024064, 0.351027944540032, -5.90278679096341, 2.598005551887, 0.0100254077566571, 0.0196750405294865, 0.297162148087818, 0.0354711856219591, 0.167211652849874, 0.26252706831815, 0.900315669901513, 1.18668390167284, -1.04144129912852, -4.2331106599682, 2.21712935283643, -3.03567569186269, -0.199920629024283, -1.8020512073994, 6.42823431336477, -1.69905920995778},
    {0.262628331893209, -0.292253144230847, -0.123511007815442, 1.44142868628426, -2.12588434718251, 1.28757175841062, -0.198874814606375, 0.107864725016984, 0.0813365774870581, 0.13010843596515, 0, -0.385309547269917, -0.0380202880529207, -0.038626134705953, 0.49158078236643, -1.2319985352187, 2.23741118713506, -1.83649988776653, 0.123511007815447, -0.124492791552472, -0.0856036070983851, -0.137428620140969, 0.198874814606375, 0.843793975715242, -1.3874778209926, 1.13942082971421, -0.544014195484494, 1.43680551547464, -0.658892874025887, 2.14746293381972, -5.67536208969405, 0.395071201482257, 0.305319074412651, 5.56383524974152, -2.5083552314243, 0.216999978282624, 0.21714000057778, 0.0295638951239871, 0.0308523287985069, -0.212872970966454, -0.253467951493806, 0.693459307616102, 0.693119277045957, -0.84379397571524, -1.83060958410197, 1.3874778209926, -1.13942082971421, -1.7590117935494, -0.149105081561475, 0.393804068627333, 1.72444535995919},
    {0.297257923557779, 0.165069823211548, -0.0529350629556763, 0.34937328461803, 1.68967763895773, -0.832729675112352, -0.174529674546856, 0.363250007053725, -0.119598578773167, -0.0841468872728142, 0, -0.932665468701487, 0.899395809822109, -0.128077548392621, -0.300980539497326, -0.879189330581098, 0.749214963869798, -0.0517333807439314, 0.0529350629556786, -0.0888416105525319, -0.053030852044472, -0.0537881737027803, 0.174529674546855, 0.531695946965823, -0.500943956250021, 0.0516751118353533, -1.29477416154298, 1.84844220401675, 0.645952333892759, 3.06792924261874, -3.09546239170508, -0.297639903874103, -3.67470306551296, 0.656569788877559, 2.31869282858766, 0.310012605356926, 0.219102682167284, -0.309461833350944, -0.37132677489013, -0.0464732513496461, 0.23430266735855, 0.286419813661973, 0.191723430570073, -0.531695946965819, -0.273707717600946, 0.500943956250022, -0.0516751118353585, -1.18337433738696, 1.10305073097289, -1.5747344864158, 0.251002189832235},
    {0.105970427076989, -0.298712767052989, 0.09057069023377, -0.559977693850786, 1.89158620077829, 2.73576399498904, -0.125371565743527, 0.466585090485748, -0.613478637595955, 0.276447485146131, 0, -0.493601927890689, 0.912754231227867, -0.85929819873849, 0.632888235377315, -0.11522577436448, 0.00280414771476928, -0.0150273444166897, -0.0905706902337702, -0.0116435027310185, -0.000909440298817843, 0.012521096699773, 0.125371565743527, 0.222064178926052, -0.0483293676292908, -0.0269309946081376, -0.951019964016148, 0.296312750966933, 1.79013722533244, 0.473294705509365, 0.0906046239486415, 0.575005038267477, -0.509749860522935, -1.9849949724417, -2.584083065611, 0.0478261762749617, -0.0293848629457365, -0.479106187185523, -0.0515099501467411, 0.643772940840511, -0.261120208543331, 0.00265375651792199, -0.0255212032610539, -0.222064178926054, 0.00795173417215991, 0.048329367629291, 0.0269309946081336, 0.0857453443311055, 0.976541167277198, -0.304264485139085, -1.87853632618147},
    {-0.397585858525559, 2.37732743172645, -0.191346895929756, 1.158001421545, -3.20320459390343, 7.54135545678689, -0.0593214248110956, 0.302297805548907, -0.636304129851133, 0.762049926251087, 0, 2.22862307256705, -5.20603825409458, 6.62721380355635, -5.62954019522969, 0.290095768273428, 0.108667331755443, 0.100385839478181, 0.191346895929762, 0.0293140218738264, 0.0215863426605585, 0.0262058564159236, 0.0593214248111028, 0.0416933466353216, 0.0154793288449621, 0.016138934846303, -0.241165644948299, -0.122678763899136, 0.695526639611787, -1.80496082766072, -0.819725557923269, -1.25838726102321, 5.13860823046487, 3.9142628200713, -11.1650986278645, -0.182390324058692, -0.162835292770208, -0.328503661964843, 0.519253607060167, 0.777553079960802, -1.1282272311264, -0.0235954407143826, -0.020906378240141, -0.0416933466353362, -0.0106348839224381, -0.015479328844957, -0.016138934846305, 0.177990804518497, 0.262072023188456, 0.133313647821584, -0.84992200341593},
    {0.0167231769272602, 0.40195782276275, 9.65546182492156, -9.34793058151145, 6.39444256476644, -2.46012846883513, -0.111215180556967, -0.0503474981856371, -0.042843714384419, -0.0620540951090301, 0, -0.202139370213583, -0.0910534020865996, -0.0725254431340566, -0.0529627842557695, 0.14900811415397, -1.25378938541602, 3.83289856489746, -9.65546182492157, 0.00375856944256303, 0.00840057499663299, 0.0206438047286813, 0.111215180556962, 0.669770344786421, -1.78129271809511, 3.13672292834152, 0.347710690358119, -1.30901999077107, 0.418838042767374, 1.24546638290834, -3.38404899977598, 5.51503201661399, 0.616955579472509, -1.75660417957441, 0.448698392300295, 0.0314155501873835, 0.0226736306317109, 0.0297036934569744, 0.0155620410444142, 0.0117695087560765, 0.0113179344346601, -0.0821236076344665, -0.142570571577529, -0.669770344786407, 0.536732788107341, 1.7812927180951, -3.13672292834152, -0.221656296907692, -0.205140118780608, 0.772287202663748, -0.115058138225219},
    {-0.074891979098382, -0.292253144230848, 2.64740738044187, 5.3635060916448, -4.00733663545901, 1.6689581466585, -0.0982464118371512, 0.0496841080000007, 0.0336033430537133, 0.0420976745229786, 0, -0.0672191997708805, 0.130979678041357, 0.105179489498029, 0.198205155560725, 0.583617242377618, -1.75535633697016, 2.72645767649418, -2.64740738044187, 0.0147211173418589, 0.0147194624606259, 0.0252561692560331, 0.0982464118371566, 0.503788767691548, -1.08053498733039, 1.37180512716454, -0.291327060891255, 0.876328283552676, -0.276338543320754, 0.0091404348807349, 1.38181465767428, -8.08996376813898, -0.936154078378042, 4.3808783147549, -1.32556174553882, 0.000230557640622631, -0.0115871453293054, -0.0749402772560377, -0.0236134799268652, -0.0367356601850333, -0.033435869578605, -0.12104613544941, -0.148091731036697, -0.503788767691544, 0.445468306551036, 1.08053498733038, -1.37180512716454, 0.0952873901988291, 0.439418791927945, -1.32179659010371, 0.302097288571342},
    {-0.25046885665096, 0.165069823211547, -0.187994765484735, 4.35942248656921, 3.41733871056418, -1.07938914174274, -0.0760925285239234, 0.150535227766675, -0.0458402342130894, -0.0272264303713641, 0, 0.560233760555631, -0.216401700639015, -0.147191496708831, -0.111241529768371, 1.0461401018987, -1.55010871751302, 0.808395203406331, 0.187994765484733, 0.0263877590958994, 0.020793182263991, 0.0279146966014815, 0.0760925285239276, 0.285607669742066, -0.388323899272721, 0.208628359038459, -0.643263392209406, 1.20960255398634, 0.270911302943511, -2.73303028499987, 5.23450301813051, -5.16781768997554, 1.6487370674834, -7.10173301118168, 1.11754225736051, -0.0689377499547984, -0.0702157042843054, -0.17844992436816, 0.0415876195456905, 0.0952627562333999, 0.0281888016845668, -0.122885674477448, -0.119284387413961, -0.285607669742068, 0.224304229673355, 0.388323899272726, -0.208628359038466, 0.41496794816383, 0.762547779623359, -1.4339067836597, -0.562993576629884},
    {-0.502239635580981, -0.298712767052989, 0.105852187397965, -0.836801817303566, 5.85275550665803, 3.54611350936128, -0.0480952657120957, 0.173733641433921, -0.218291964694203, 0.0894468999342647, 0, 2.03897006555019, -3.02415716371381, 1.77660019458065, 0.00953930621694732, 1.12840081544712, -0.545785509857588, -0.113201120517721, -0.105852187397967, 0.0284626971354913, 0.020356324659878, 0.0235023902616734, 0.0480952657120989, 0.107177982409502, -0.0532088243073626, -0.0130213775050737, -0.438633646781797, 0.296645138506831, 0.750780487532123, -5.03608120140908, 2.90209988198702, 0.950002937821294, 8.94320518700363, -8.20906987878748, -8.58163831040296, -0.127029732718387, -0.108240483351329, -0.197236031695597, 0.225582733700365, 0.306176123385655, -0.216462598051734, -0.0700123404629989, -0.0593376105139085, -0.107177982409509, 0.0401296476426405, 0.0532088243073667, 0.0130213775050653, 0.372275924013291, 0.497971257295704, -0.336774786149463, -1.05304407108242},
    {-0.778443022056057, 2.37732743172645, -0.142219700439307, 0.917394780947744, -2.83066387967159, 9.77514965223253, -0.0205164010902927, 0.103309343513251, -0.213572850101018, 0.246567638198118, 0, 4.20489398868706, -9.31981192455295, 10.8638856288179, -7.34785210262236, 0.673414315020533, 0.153608934270619, 0.108357653355245, 0.142219700439315, 0.0169861519353335, 0.0115897539544488, 0.0122023345513334, 0.0205164010903041, 0.0184690844512087, 0.00459131460491591, 0.00413689683525065, -0.104923011112297, -0.0350772603162119, 0.291702681889279, -3.91982442923121, -1.02642018009458, -1.02575243430301, 9.97638223467829, 3.70347512549559, -16.5051217727002, -0.0988733560152029, -0.0774431343962636, -0.115511678064602, 0.251643513693441, 0.279426230542851, -0.416323947811698, -0.0158295509441141, -0.0123929321413425, -0.0184690844512309, -0.00414313411512829, -0.00459131460490392, -0.00413689683525797, 0.105773603651419, 0.117315943253664, 0.0392203944313509, -0.381646734596623},
    {-0.105791126909818, 0.401957822762745, -0.164167775300466, -0.0765004719955856, -0.0680662120055154, -0.108642456439209, -0.137827978438839, -0.264218287259758, 1.33871103080744, -1.31019580959276, 0, -0.0693101417590309, -0.0135369151601555, 0.942987443234203, -1.15630708216795, 0.0761551633562676, 0.272131551992182, -0.425053558215356, 0.164167775300469, 0.91840868826511, -5.35222248673391, -1.46805529712027, 0.137827978438837, 0.155221788245082, -0.592466651821696, 0.541989101448609, -0.479799244311731, 0.294198821893887, 0.390529342667767, 0.0387371525500821, -0.470514875753141, 0.501554030210936, -0.354938187675602, 0.266449535766485, 0.348688328208458, 0.467158573269898, 9.25398131864231, 1.73227358438004, -4.28044929578008, -5.24046986271586, 4.20507784383782, -1.56135258577435, -2.66587082018917, -0.155221788245077, 1.63463378469046, 0.592466651821699, -0.541989101448615, 2.69957530659132, 3.1456700645009, -1.92883260658434, -1.52875206348473},
    {-0.166507481624835, -0.292253144230849, -0.149034415307041, 0.077243285110049, 0.0542507299172712, 0.0737033512859859, 0.00659520248347836, -0.00954009316137126, -0.770580478516431, 0.888840561717928, 0, 0.361971121328345, 0.0817541398560667, -0.931681699067025, 0.946717063738298, 0.0674033676683334, 0.109102748754661, -0.312007713311832, 0.149034415307042, 0.812864627383561, -1.5497016993325, 0.0385351639022047, -0.00659520248347706, -0.00427194240478018, -0.335447152559136, 0.417128763817379, 0.29502453480403, -0.250485359404177, -0.25766119276985, -0.230814025668254, 0.106893355165648, 0.234764428201783, 0.365227905435152, -0.270246833837586, -0.275520598721216, -2.78354870772829, -1.51831934610621, -0.0289950707408353, 4.40454024081496, 3.83860152395514, -3.32269672218817, -0.518178177905878, -1.19168844701444, 0.0042719424047776, 1.01178198330688, 0.335447152559138, -0.417128763817378, -0.507684770935875, 0.896663912210412, -0.761296623902712, 1.28352414161161},
    {-0.00460700210468688, 0.165069823211546, -0.120198635080441, 0.242093133212242, -0.0757646835227073, -0.0476672211627468, 0.11653376463127, -0.440025155662253, 0.48314520234492, -0.574852552761566, 0, 0.0446449994580023, -0.168146803128276, 0.558376310521585, -0.595337327958173, -0.00447841447771002, -0.0156727121577186, -0.161107257848829, 0.120198635080441, -0.054008350645093, 0.0999436061059625, 0.292826340312172, -0.116533764631271, -0.0368681220855668, -0.0907410396079683, 0.244620366436965, 0.299406669197409, -0.566222654644362, 0.252600772271737, 0.0326842789896216, 0.410815601569553, -0.0809858753634098, -0.242611342490355, -0.319378205889139, 0.262072699141195, 0.394162712951218, -2.61973755737125, 0.14719881535008, -2.92582085041875, 2.03664874892036, 3.16051904087419, 0.0522530948531797, -0.199248061339095, 0.0368681220855644, 0.376807792981306, 0.0907410396079678, -0.244620366436961, -1.36966635895271, -0.100158607858313, 0.189414861663043, 1.06481249182781},
    {0.290022490838602, -0.298712767052989, -0.0793175468753283, 0.289572373956811, -0.369732996138608, 0.156600961026926, -0.0532327497944575, 0.268058474381135, -0.587773658440494, 1.88856115408695, 0, -1.47223222072208, 3.00935391526566, -3.12836901872781, 1.59993760039861, -0.0325642012557375, -0.0268596825201084, -0.0409455036344199, 0.0793175468753284, -0.392714611086448, -0.0426994994341207, -0.0379034404664838, 0.0532327497944609, 0.00730286018793136, 0.00905073522587117, 0.0951386973587608, -0.0521574201265629, -0.395667904007707, 0.700036243953589, 0.191057521527247, 0.211206014342702, -0.24862687032239, -0.525633728075823, 0.185386664316006, 0.210539446777391, 2.30409705653375, 0.335759407549097, -0.23015503391466, -6.33898689773213, 0.294713750325524, 2.53904329819788, 0.0508549452213776, 0.00737505379456994, -0.00730286018793425, 0.0559474005762264, -0.00905073522587337, -0.0951386973587561, -0.39988820723335, 0.0447823663320008, 0.339720503431477, -0.351002981941614},
    {0.00324591462099527, 2.37732743172645, -0.0350775308878986, 0.177387214604182, -0.369081269115183, 0.431683257087092, -0.207698228045331, 1.18066248882387, -2.94696348846712, 5.20597207614995, 0, 0.0159833447999282, -0.1945117502781, 0.761745204559482, -2.96379014542875, 0.00155844623369605, 0.00285602294237489, 0.00672933045590636, 0.0350775308879028, 0.0187943994619904, 0.0228041790188392, 0.0447894063950395, 0.207698228045335, 0.00932123206705371, 0.00826036934647995, 0.0168890996848692, -0.0639353794992433, -0.0965661967253602, 0.271986889872638, -0.0157277113713552, -0.0353386200775606, -0.18411654506011, 0.0823274851773874, 0.401563866250393, -0.499841477126815, -0.18967153549872, -0.282164476542189, -1.22545189521893, 0.992845059187131, 3.20632378599051, -6.02793999930037, -0.00210468767317473, -0.00242544141320308, -0.00932123206704849, -0.0036633184081792, -0.00826036934649133, -0.0168890996848629, 0.0260420730381236, 0.0663608209124456, 0.100229515133552, -0.295924275237596},
    {-0.164081106760814, 0.401957822762747, -0.0187732230697997, -0.0120162719057295, -0.0185107978054531, -0.540385364807993, 0.0258474166283138, 0.0317742188172508, 0.794078534982992, -1.09432435540836, 0, 0.0923933082071171, 0.128672986876066, 0.481492616898436, -0.940435627983551, 0.36789252321787, 0.028606758251802, -0.0527383769520339, 0.0187732230697981, 0.745012308897366, -1.22717631740947, 0.13945429518294, -0.0258474166283132, -0.092847174826044, -0.0955727494068921, 0.0711059666711761, -1.4155987689138, 0.0668258271694491, 1.62243583973931, -0.316806340131939, -0.0693752912027381, 0.0647546488577656, -0.925020304580562, 0.0592793307563892, 1.41431948630263, -0.641558629326223, 2.97606997717111, -0.171228514000188, -1.87324142079504, -2.54297219474464, 2.86411209663226, -2.50732736288743, -6.21294042557338, 0.0928471748260451, 0.293292769258353, 0.0955727494068876, -0.0711059666711837, 6.08061089655065, 7.62853919448719, -0.360118596427793, -5.19571937340253},
    {-0.0960766222659435, -0.292253144230848, -0.0798554220922575, 0.053791343873944, 0.0586818441400089, 0.366598967637805, 0.104452923995495, -0.114312118111813, -0.42980924745243, 0.742392753542015, 0, 0.243634538083017, -0.100150016833948, -0.555424010314663, 0.800269255562386, 0.131304457829817, 0.0183070434236163, -0.169417723317698, 0.0798554220922577, 0.26590221633392, -0.134088092702238, 0.360030023483157, -0.104452923995495, -0.254606242173065, -0.202385796391837, 0.242622825458005, 0.818502271136559, -0.226303032506379, -1.07044134201057, -0.429404557789847, 0.170689705501043, 0.115626379443753, 1.09077479566088, -0.247678593064667, -1.15927366333865, -0.869579186475105, -1.25019952839658, -0.245717905371346, 2.20890775896828, 1.81409686855125, -2.34762354236911, -0.333946835138086, -2.57790159753732, 0.254606242173064, 0.712749334483394, 0.202385796391837, -0.242622825458005, -3.11362603036112, 1.75939932640077, -0.48644630197702, 4.51801420750978},
    {0.13457946630335, 0.165069823211547, -0.166654346797131, 0.403271313827146, -0.169838959850658, -0.237095786874348, 0.0785206721957266, -0.256222516966143, 0.1939317525149, -0.480138269905771, 0, -0.449017679815505, 0.399649196908814, 0.250342238494167, -0.500623045102376, -0.163463691193921, -0.0847537527393667, -0.201229823722228, 0.16665434679713, -0.331027281913931, 0.0967766395614572, 0.127853408251255, -0.0785206721957283, -0.10678120237918, -0.0816479921624376, 0.340343397805308, 0.597773588500172, -1.06015070974095, 1.04941806236603, 0.727784246397318, 0.761401119925219, -0.202041490104917, -1.42375600195933, -0.506808407335198, 1.09653123363028, 1.4738223463881, -0.869410962500858, 0.128369108714889, -2.88322180904461, 0.578702570424502, 2.22056501447621, 0.523685019363069, -0.298285223161829, 0.10678120237918, 0.529008469299668, 0.0816479921624385, -0.340343397805305, -4.70462188803937, -0.299488365338347, 0.531142240441271, 3.13151880631028},
    {0.29979034391558, -0.298712767052989, -0.2079121008487, 0.842273262501722, -1.26534612149436, 0.778929989503399, -0.11704912380254, 0.551807490955152, -1.07182330716522, 1.57739663984871, 0, -1.45868837355563, 2.80863122858831, -2.63979351805564, 1.28877308616037, -0.153143283237314, -0.0756769813797038, -0.0735028894976401, 0.207912100848699, -0.310127615638405, -0.0641028972870157, -0.0481547341431413, 0.11704912380254, 0.074774761853181, 0.0564830403784519, 0.231133527599847, -0.473078467659412, -1.13099792838139, 2.90826774640857, 0.827236054089221, 0.483464012856785, -0.768770373004081, -1.92654341079388, 0.85755909001727, 0.473520650438573, 1.67522035313336, 0.409522729277867, -0.503652756812014, -3.90140724283371, 0.726403475174377, 0.958917865490046, 0.173935747977184, 0.0412842670902297, -0.074774761853182, 0.098699103311151, -0.0564830403784527, -0.231133527599844, -1.11119224317859, 0.431794200569184, 1.03229882507024, -1.97101125120716},
    {-0.103138285400158, 2.37732743172645, -0.138407634889361, 0.728837230454798, -1.61290224291863, 2.14718372548066, -0.200550049418391, 1.11516772883395, -2.68710950212104, 4.34822184195316, 0, 0.622875521638104, -1.63073858171749, 2.55521429337864, -3.82154037962554, 0.0285357944166003, 0.0249215884169984, 0.0390823986321367, 0.138407634889365, 0.0577873067345188, 0.0415195882684609, 0.0597985776505831, 0.200550049418394, 0.043791694499442, 0.0329151710595562, 0.0579594950009424, -0.289971863025115, -0.352469081246105, 1.12995677880239, -0.204991706784202, -0.236401714245636, -0.767919629086956, 0.738373980030159, 1.8243823687473, -2.70910179314322, -0.415124894195385, -0.393847361460468, -1.17496630648455, 1.49526741908317, 3.03943727531308, -5.48615167357549, -0.0174594076572036, -0.0155491452265394, -0.0437916944994413, -0.0189004301140968, -0.0329151710595628, -0.0579594950009388, 0.165616807035397, 0.305521008251656, 0.371369511360213, -1.2781141781806},
    {0.0564307431585088, 0.401957822762748, -0.04962224335549, -0.0732932595554662, 1.03279003696238, -1.19524818143039, 0.106547386759108, 0.084823207569346, 0.21201998832074, -0.766892947097166, 0, 0.0811251111917294, 0.0479541847037276, 0.0255363578556439, -0.613004219672357, -0.310862724193498, 0.607293902650749, -0.191907680163961, 0.0496222433554894, -0.19945517123825, 0.124670495976095, 0.22209716265081, -0.106547386759108, -0.54823027582987, 1.4237519510266, 0.303940807514456, -0.836003390831256, -2.61288276712841, 2.51484210552408, -0.359749645565939, 0.551750183837496, 0.265200939719434, -0.188660625658652, -2.19183412345063, 2.05452117684848, -0.230821908112053, 0.113268005447899, -0.306920370220153, -0.121048084791406, -0.449958489744739, 1.31821811123888, 1.47875969185959, -2.18895260378223, 0.548230275829873, -6.84145136157456, -1.42375195102659, -0.303940807514452, 1.34351082445205, 3.02495599461349, 9.45433412870296, -5.33711262183572},
    {0.169880004840814, -0.292253144230848, 0.0690671693240339, -0.0852935608097445, -0.578626286337773, 0.810859764011252, 0.0601843095813742, -0.0481034612920366, -0.0739385526704499, 0.520262355355291, 0, -0.217752175599961, -0.0922856891438495, -0.145727853241818, 0.578138857375662, -0.579959512573223, 0.563160949589567, 0.15352592818632, -0.0690671693240334, -0.372112559302997, 0.0719623468831859, 0.0865848309499929, -0.0601843095813746, -0.236977773793665, 0.343296697610014, -0.269602668072571, 0.311436558590024, 1.56377209133421, -1.65922799068246, 0.969200972393791, -1.75907827012873, -0.0682323673765752, 0.655610012760456, 1.77454360687693, -1.85571123659227, 0.621856951214114, -0.224780146354139, -0.0384813696579554, 0.420651294554226, 0.226756352141404, -1.19065804182063, 1.61488068012323, -0.560576745474846, 0.236977773793667, -2.81474427277658, -0.343296697610014, 0.269602668072575, -5.04420896961302, 0.249140186884821, 1.25097218144236, 5.08855628017225},
    {0.294232960746207, 0.165069823211547, 0.103341901603272, -0.360080603668169, 0.318850804445174, -0.524418917575723, -0.0624268174830971, 0.168910681021683, -0.0868988311190472, -0.336476704555082, 0, -0.936259324974292, 0.892953275246571, -0.0590352544783466, -0.356961479751687, -0.598894015194808, 0.17565250479797, 0.0887506957311207, -0.103341901603273, -0.384261280165227, -0.047871911054567, -0.0416321798630036, 0.0624268174830956, 0.155700188965854, -0.0686848125917701, -0.212965084448444, -0.592456199535279, 1.39478247742007, 1.62664104483763, 2.19349348137032, -0.850456778226285, 0.271329907937049, -2.80102858337881, 0.355953468983138, 1.73084803477902, 1.40738526650875, 0.231781444220388, -0.127278501158678, -1.79719082495494, -0.0970107020467739, 1.1105435431665, 0.896104290626134, 0.146025360331339, -0.155700188965851, -0.343778348524107, 0.0686848125917704, 0.212965084448445, -4.33866837730096, 0.446430839203937, -1.05100412889596, 1.8159230418372},
    {0.197406415132327, -0.298712767052989, -0.0876613998747692, 0.425969859867175, -0.87404780245055, 1.72287170239396, -0.19230435392029, 0.82659490437137, -1.36858582907549, 1.10542578340343, 0, -0.915326776547994, 1.66111998585092, -1.46128908709736, 0.816802229715094, -0.182544384524022, -0.0193299798004355, -0.00219779640541003, 0.0876613998747677, -0.117123793366606, -0.0302669217370274, -0.00426482594360886, 0.192304353920289, 0.247750582016084, 0.0498187418089048, 0.081917649584331, -1.33609253208801, -0.547490181513061, 4.50793430696221, 0.863183086088688, 0.0672565364604162, -0.423772063461763, -1.52616370921373, 0.826121245790568, -0.8773466947449, 0.553833949349987, 0.105310421757656, -0.822330078427763, -0.979214361414882, 1.29354232905487, -0.562921577971929, 0.0996950954523921, 0.00689358483069949, -0.247750582016086, 0.00282478190664234, -0.049818741808905, -0.081917649584331, -0.346878108070644, 1.32919894725731, 0.544665399606422, -4.26075129434396},
    {-0.304744182307492, 2.37732743172645, -0.204926878310815, 1.15124252490878, -2.81967943501396, 4.74923565701962, -0.172286458511458, 0.927333358625987, -2.12300674191134, 3.04719587618368, 0, 1.73156496827645, -4.12706916783813, 5.44548729553774, -5.12256634539502, 0.145932890506227, 0.0801957809291841, 0.0891257331708715, 0.204926878310821, 0.0936331936893625, 0.0603813970734735, 0.0717913590691088, 0.172286458511463, 0.0805456244797069, 0.0454625525556248, 0.0641578121528971, -0.506729724493884, -0.431818650820233, 1.75147935909211, -0.93233910713747, -0.634274439485646, -1.24036825807967, 2.77709025636393, 3.37375809357045, -6.73991969675232, -0.598205708801781, -0.4775609931146, -0.999124717695112, 1.78183155945829, 2.54018633795249, -4.32445492052956, -0.0498146183710012, -0.0392294909524586, -0.0805456244797136, -0.0334301009722124, -0.0454625525556248, -0.0641578121528971, 0.393987548713446, 0.54595921544635, 0.465248751792455, -2.09565228943458},
    {0.299991988790645, 0.401957822762747, 0.798848493973308, -2.9271659390857, 3.46950926213078, -1.89794570036166, -0.116259944551924, -0.0671601987265287, -0.0853988144437375, -0.41554418763153, 0, -0.189532639500748, -0.114192743886022, -0.136568967959901, -0.26165546020672, -1.42272867008211, 2.60250627690431, -2.10306001655252, -0.798848493973307, -0.31149817896092, -0.0640583246327801, -0.0482521085530288, 0.116259944551924, 0.689264623085494, -1.92648387844072, 6.57740808646509, 0.534698253976494, -4.75618133628166, 2.16380880177052, 1.07105877376893, -2.7302864309568, 5.03022595563823, 0.816754677477797, -3.34172910807829, 1.43286091919704, 0.23450209769786, 0.0672035168893932, 0.115412307279561, 0.1788236928391, 0.0822536221871217, 0.313716576055495, 1.62309005774784, 0.384160837567568, -0.689264623085493, -3.4171396524693, 1.92648387844072, -6.57740808646509, -1.70278196837286, -0.918859091544065, 8.17332098875097, -2.08411689114552},
    {0.272170305024623, -0.292253144230847, -0.123511007815442, 1.44142868628427, -2.12588434718251, 1.28757175841062, -0.157941208263555, 0.0979068146088089, 0.0920774725226209, 0.281906358155606, 0, -0.467489537376657, 0.0697441808985594, 0.0780453355083465, 0.339782860175977, -1.02566673596469, 1.31886293520907, -0.507134015614564, 0.123511007815446, -0.224563774662213, -0.0571233170980174, -0.0344462938201885, 0.157941208263555, 0.765896175374987, -1.5707010902754, 2.46878670186617, -0.615853945226666, 3.11313103751268, -1.4276252662131, 1.98674817508493, -3.08048831573002, -0.93429467066971, -0.50903035829377, 3.88750972770347, -1.73962283923709, 0.434986973698341, 0.133423804838585, -0.0634605207886202, -0.111449240448061, -0.168377960263191, -0.38088031674367, 0.885674731775416, 0.216674253292382, -0.765896175374986, -1.09528460145879, 1.57070109027541, -2.46878670186617, -2.06868400797009, 0.399179691934278, -2.01784643605389, 2.61063454240777},
    {0.148805352758507, 0.165069823211548, -0.0529350629556773, 0.349373284618031, 1.68967763895773, -0.832729675112352, -0.201751253837805, 0.461329302767378, -0.176632655677163, -0.182321325786767, 0, -0.500755705181338, 0.569087838367462, -0.179401208172805, -0.202806100983371, -0.325405674300826, 0.138534598999158, 0.00855613505400887, 0.0529350629556781, -0.0712456823987434, -0.0144818949841152, 0.0112979325913968, 0.201751253837805, 0.675256478279167, -0.739833719142113, 0.111964627633291, -1.9122250531861, 4.00502554757522, 1.39958695713318, 1.1568419591873, -0.328198683275979, -0.357929419672042, -1.56376481512138, -1.50001355468091, 1.56505820534725, 0.253283827907698, 0.03430867739514, -0.472627235358776, -0.342377223764915, 0.156805873266136, 0.342660404042728, 0.114750419488591, -0.0468303002234681, -0.675256478279168, 0.0980828843775988, 0.739833719142117, -0.111964627633296, -0.271852207705542, 1.95905535340957, -4.10310843195281, -1.24248516891623},
    {-0.144387432414535, -0.298712767052989, 0.0905706902337698, -0.559977693850787, 1.89158620077829, 2.73576399498904, -0.196100657264724, 0.770010309271974, -1.10058719413418, 0.598979637105893, 0, 0.586279958088719, -0.843273775719113, 0.389737933680367, 0.310356083417553, 0.313660972674284, -0.0808711851769469, -0.0464478206810625, -0.090570690233771, 0.0686742481920392, 0.0470535208723661, 0.0638691525758603, 0.196100657264724, 0.366474862955995, -0.0867033990325078, -0.058351470872514, -1.70613992015067, 0.642021771151963, 3.87869596655069, -1.52272357193105, 0.519988977025383, 0.606425514531852, 3.14594041109696, -2.33070399262673, -4.67264180682924, -0.33339148194013, -0.302546724526545, -0.833879461847837, 0.68878538106617, 1.35608039778836, -1.02304778442397, -0.165826299444854, -0.141517210307097, -0.366474862956, 0.0532530356606557, 0.0867033990325095, 0.0583514708725091, 1.06623697456099, 1.84765713045777, -0.695274806812611, -4.77910664166683},
    {-0.581370126327114, 2.37732743172645, -0.191346895929755, 1.158001421545, -3.20320459390343, 7.54135545678689, -0.114067231912176, 0.593574614772199, -1.2898484995732, 1.65113597630005, 0, 3.19340425915315, -7.24546708488881, 8.77473176561499, -6.51862624527865, 0.402570486055835, 0.141295916778999, 0.119215185614465, 0.191346895929761, 0.0881404697513908, 0.0568962490251358, 0.0611079628742554, 0.114067231912184, 0.0818666616606203, 0.0313780598748269, 0.0349682809825832, -0.48886551366235, -0.265808464280528, 1.50699976153497, -2.41260666982083, -0.99548384332822, -1.27721660715949, 6.44525247676576, 4.05739252045269, -11.9765717497877, -0.528226217691116, -0.400855862941174, -0.654682577646471, 1.41115059510264, 1.63380811348926, -2.62220082346298, -0.0664749648827072, -0.0503282568289246, -0.0818666616606361, -0.0273647379161339, -0.0313780598748197, -0.0349682809825874, 0.468341584350488, 0.539193770491291, 0.293173202196672, -1.90886638100278},
    {-0.234775607728653, 0.401957822762752, 9.65546182492156, -9.34793058151145, 6.39444256476644, -2.46012846883514, -0.183808928740493, -0.0870687160530413, -0.0794763120089922, -0.134452803394807, 0, -0.121415088866584, -0.0428121328043926, -0.0223909173931203, 0.0194359240300013, 1.57034702363989, -4.51342080072435, 7.49252308868269, -9.65546182492157, 0.0858237942878881, 0.0560971556399149, 0.0697870357124558, 0.183808928740492, 1.1582709384273, -3.30434879134063, 6.79634745212674, 0.645013246699687, -2.83625773850666, 0.907497706952617, 0.692596204907851, -1.65165533220324, 1.85540749282876, 0.23714651217229, -0.229366431838823, -0.039961271884926, 0.0378522920856122, 0.0205283686863949, 0.0172816803405991, 0.0129607107029936, 0.00285078768267955, -0.00218399368169109, -0.640543563521531, -0.516988931535496, -1.15827093842729, 2.27330813017015, 3.30434879134063, -6.79634745212674, -0.234402516164493, -0.128024315164197, 0.562949608336525, -0.0325516272665904},
    {-0.319307309300151, -0.292253144230848, 2.64740738044187, 5.3635060916448, -4.00733663545901, 1.6689581466585, -0.16969680371065, 0.0891817251364754, 0.0639983068686289, 0.0912131640316667, 0, 0.233852569065178, 0.132444283151227, 0.0961739352625574, 0.149089666052038, 1.73319922781626, -3.81904507052228, 4.32694704068548, -2.64740738044188, 0.0947241162295672, 0.0609912369741345, 0.0719463341830872, 0.169696803710655, 0.904288176153031, -2.05790267923408, 2.97229449135585, -0.554838803158707, 1.89874325313739, -0.598743592509582, -1.52666139877437, 4.46791836081111, -9.69045313233029, -0.872339279350405, 3.35846334517018, -1.00315669634999, -0.0834362544477479, -0.0713539281399674, -0.161128059319569, -0.0476757466554445, -0.0536356157027974, -0.0548252791580501, -0.570610600878202, -0.447609842585059, -0.904288176153031, 1.53178934819946, 2.05790267923407, -2.97229449135584, 0.667559961576641, 1.00244864574377, -3.43053260133685, 0.501794231811151},
    {-0.467046932097564, 0.165069823211546, -0.187994765484736, 4.35942248656922, 3.41733871056418, -1.07938914174274, -0.140239979255145, 0.285318280611882, -0.0906850475397857, -0.0589915924715635, 0, 1.11330231956611, -0.613888460615439, -0.117960382396487, -0.07947636766817, 1.76831839423408, -2.33803115856641, 1.05180256724954, 0.187994765484733, 0.0966434754978223, 0.0620437377508575, 0.0688390494280752, 0.140239979255149, 0.541329032873791, -0.768215343374559, 0.452035722881667, -1.27255831704394, 2.62084966497748, 0.586984373683922, -4.66768181452502, 7.43367257017503, -5.41122505381875, 3.17728337541358, -8.51298012217281, 0.801469186620108, -0.25510167996023, -0.197265476886336, -0.354157330039962, 0.17364729623505, 0.22590678667526, 0.0438025006989162, -0.401595472822788, -0.307031518272277, -0.541329032873796, 0.632335226624898, 0.768215343374566, -0.452035722881673, 1.2768560588644, 1.57958983531621, -3.25318489160238, -1.46224495972553},
    {-0.658971363590883, -0.298712767052989, 0.105852187397964, -0.836801817303567, 5.85275550665803, 3.54611350936128, -0.0949022450750647, 0.348527807290231, -0.449037675456541, 0.193804879919779, 0, 2.65481672502864, -3.91007984563924, 2.30776592502304, -0.0948186737685673, 1.42890947687281, -0.660518268226523, -0.128393202681082, -0.105852187397967, 0.0780938424138116, 0.0506765723296642, 0.0534757459605001, 0.0949022450750679, 0.215010212706453, -0.109453258227926, -0.0282134596684365, -0.902291723856328, 0.642741956281951, 1.62671844791957, -6.24909417659342, 3.36292945813107, 0.965195019984656, 10.7316474611497, -8.5551666965626, -9.45757627079041, -0.341530225500357, -0.258012149129707, -0.402003553250735, 0.586514120898553, 0.656373252256582, -0.516882617731784, -0.183584851765916, -0.138441530339722, -0.215010212706462, 0.0986179720910108, 0.109453258227931, 0.0282134596684279, 0.934694672000328, 1.04073325419605, -0.741359928372954, -2.37782826815399},
    {-0.852928776435775, 2.37732743172645, -0.142219700439307, 0.917394780947743, -2.83066387967159, 9.77514965223253, -0.0428399680424249, 0.217156820960842, -0.453448968880608, 0.534238878577229, 0, 4.5819599722885, -10.0789234905853, 11.6080882060076, -7.63552334300147, 0.729646958987027, 0.163999371232681, 0.113184183927627, 0.142219700439314, 0.0398772179449461, 0.0262713444420447, 0.0267918654817104, 0.0428399680424365, 0.0388221193658635, 0.00974808770129243, 0.00896342740763179, -0.222768161675061, -0.0760020104497254, 0.632033119955875, -4.20797794764629, -1.07773536719015, -1.03057896487539, 10.5486335471935, 3.7443998756291, -16.8454522107668, -0.229977596231999, -0.172644302450802, -0.243948686442571, 0.576511905931632, 0.599821926889385, -0.920650406221813, -0.0366179238076995, -0.0274841683295968, -0.0388221193658864, -0.00937679798081492, -0.00974808770127966, -0.00896342740763933, 0.240637700400835, 0.250252330004682, 0.0853788084305522, -0.836052896549048},
    {0.218256695635768, 0.401957822762747, -0.164167775300462, -0.0765004719955904, -0.0680662120055064, -0.108642456439216, 1.68633541906575, -3.90274161753215, 4.01035393449851, -2.01569604328753, 0, -0.0381800259775987, -0.0398430149814679, -0.0913846289662665, -0.450806848473183, -0.0733887411520734, -0.0637431422621128, -0.133209060673515, 0.164167775300462, -1.36161681179276, 3.75564547866193, -6.79578218749924, -1.68633541906575, 2.29276534646624, -1.77484230241922, 0.833833598990446, -1.43732646024022, 0.452615858552999, 0.600817408390111, 0.0161959336144414, 0.0237768551602391, 0.209709532669108, 0.0274350014907239, 0.108032499107374, 0.138400262486129, 0.300490990115204, -1.40089483214037, 10.6985238050314, 0.509014852617868, -6.36510458102009, 2.56780701234723, 0.562657865160182, -2.5027938083942, -2.29276534646624, 0.788132828347221, 1.77484230241922, -0.833833598990446, -0.209877236827702, 3.94012026863442, -1.24074868690022, -0.953598036722592},
    {0.280175037546943, -0.292253144230849, -0.149034415307041, 0.0772432851100504, 0.0542507299172711, 0.0737033512859848, 0.0719510337707437, 2.01851727120317, -2.4725941148571, 1.36745392578012, 0, -0.420094720418819, -0.0212736261480504, -0.0146572464253234, 0.4681036996761, -0.0727845062071818, -0.0560505031716999, -0.0873966756338477, 0.14903441530704, -1.3504061485995, 2.55462266569428, -2.28384511302603, -0.0719510337707406, 0.903868482180902, -1.07636344079227, 0.641739801495356, 0.946657680582987, -0.385364038074113, -0.396403837481651, 0.127859732194447, 0.137167928422065, 0.0101533905237989, 0.00799937673617745, -0.135368155167642, -0.136777954009429, 2.37224345552584, -6.25172441146412, 0.265327841822856, 0.148416305782731, 6.16969586062695, -2.5377075384892, 0.409554573439286, -1.0710928998984, -0.903868482180903, 0.436018947000106, 1.07636344079227, -0.641739801495356, -1.00227026048927, 0.124435219315422, -0.050654908925996, 0.989119524531632},
    {0.288734078223699, 0.165069823211547, -0.120198635080441, 0.242093133212245, -0.0757646835227094, -0.0476672211627481, -0.111229861949401, 0.863507355329664, 2.00510574498646, -0.884393010259582, 0, -0.904728177143191, 0.878544528095464, -0.141823381927358, -0.285796870460161, -0.0476133273786136, -0.0304504164558693, -0.0293866978957163, 0.12019863508044, -0.883393092814753, 0.805867617127967, -0.104817635449234, 0.111229861949404, 0.0723501695037457, -0.376585297630868, 0.376340926390067, 1.2425706176554, -0.871116176856912, 0.388618535849055, 0.164484460083807, 0.120699783655144, -0.212706435316524, -0.195258847106315, -0.0144846836765733, 0.126054935563871, 3.05175974697095, -3.19430925297859, -0.758689719880432, -3.62273183457821, 0.383335890864167, 2.3387581906816, 0.156188816594585, -0.150830578590277, -0.0723501695037486, 0.105741239256558, 0.376585297630867, -0.376340926390069, -0.619103400429417, -1.09174003906512, 0.765374937600346, 0.0742960479857878},
    {0.0751681159726963, -0.29871276705299, -0.0793175468753276, 0.289572373956813, -0.369732996138608, 0.156600961026923, 0.10563973431332, -0.684381542040878, 2.5925778588714, 2.90549337582053, 0, -0.356056127547306, 0.678813641770082, -0.682218241807499, 0.583005378665018, -0.00340259784931868, 0.00173356112842082, 0.0102837650967033, 0.0793175468753262, -0.0631300436915682, -0.0121557779410514, -0.0243048703807556, -0.105639734313313, -0.0186449718788803, -0.0399213803071025, 0.146367966089875, 0.230058238667543, -0.608722927309693, 1.07698427728427, 0.0114536190613368, -0.0304422526078203, -0.299856139053514, 0.00175660431432112, 0.398441687618005, -0.166408586553268, 0.212504534414402, 0.213461906049626, 0.708686412421623, 0.0325911294909406, -2.79388398697997, -3.08745899603431, -0.00504963878938641, 0.00817020233210991, 0.0186449718788778, -0.0216179586052756, 0.0399213803071016, -0.146367966089873, 0.0886743346310045, -0.238228440999644, 0.630340885914961, -1.16060897312588},
    {-0.424231227319651, 2.37732743172645, -0.0350775308878966, 0.177387214604177, -0.369081269115175, 0.43168325708709, -0.183157792112604, 1.12298042833032, -3.17517288093745, 8.00922826842386, 0, 2.36998528060686, -5.50913044526492, 6.95309529795392, -5.76704633770266, 0.0175527054809814, 0.0130848449412128, 0.0158235923115214, 0.0350775308879001, 0.32566383480829, 0.11256774126797, 0.100173986672977, 0.183157792112612, 0.008865837001095, 0.0089000426500473, 0.0259833615404747, -0.0688864602201401, -0.148564130055605, 0.418443482820204, -0.108514916661581, -0.0975653754066459, -0.193210806915717, 0.305577024167867, 0.453561799580636, -0.646298070074359, -2.01332973610313, -0.839346127893034, -1.22315441500333, 5.66951833309545, 3.90195126756255, -11.9910807002245, -0.0148348576520702, -0.00614492574754771, -0.00886583700109067, -0.0132524671034576, -0.00890004265005241, -0.0259833615404737, 0.110614108339094, 0.0750313859676873, 0.161816597159068, -0.514222733507234},
    {0.28628929987628, 0.401957822762747, -0.0187732230697992, -0.0120162719057324, -0.0185107978054505, -0.540385364807996, -0.0264986033169914, -1.57010859806942, 2.58257674051622, -1.68358443609695, 0, -0.123361535331788, -0.0862905559009793, -0.127419484111295, -0.351175547294966, -0.426181230376304, -0.0169904654544737, -0.0144499961342084, 0.018773223069796, -1.32777853203544, 2.37046405856914, -1.88811166644526, 0.0264986033169939, 4.58800099348717, -0.310830665698194, 0.109394347489004, -4.60394317871658, 0.102809484222617, 2.49606774705447, 0.216995319114415, 0.0122055895567033, 0.0264662680399477, 0.208883697082417, 0.0232956737032143, 0.54068757898747, 0.676054471047209, -1.70289103823298, 3.45822026451468, 0.650782504976419, -3.25014976085239, 1.68452599210877, 2.29106023814168, -5.53640610469511, -4.58800099348716, 0.123632076673315, 0.310830665698196, -0.109394347489009, -1.64584901993339, 10.1403492834117, -0.226441560895929, -3.14127896526276},
    {0.30194613339729, -0.292253144230849, -0.0798554220922571, 0.0537913438739438, 0.0586818441400087, 0.366598967637805, -0.203731558094385, 0.664555618073865, -1.55914251884141, 1.14214846736924, 0, -0.480500267770768, 0.0280496114199472, 0.0422441254492192, 0.400513541735161, -0.355704950821361, -0.0516705383728244, -0.0387727778774347, 0.0798554220922571, -1.10820787912762, 1.37285619648022, -0.479011407990876, 0.203731558094386, 1.48015810946035, -0.734158937329558, 0.373267770898266, 2.96913502969362, -0.348160270294902, -1.6468411529516, 0.644860981246376, 0.118810049508945, -0.0150185659965085, -0.0728811456652004, -0.125821355276128, -0.582873852397622, 2.00908089333315, -3.15671401554952, -0.185544210082991, -0.227063074830753, 3.34300033791071, -1.81595840674401, 1.45007659923757, -2.14015127162839, -1.48015810946034, 0.250953775342113, 0.734158937329558, -0.373267770898266, -3.33427283656476, -0.828983758065233, 0.0972064949527878, 3.53103739027879},
    {0.236643324868564, 0.165069823211547, -0.166654346797131, 0.403271313827148, -0.169838959850661, -0.237095786874346, 0.0456413589580958, -0.236675210548302, 1.17861774815033, -0.738677992857266, 0, -0.748859680437838, 0.7587603257374, -0.169530471228794, -0.242083322150879, -0.176564788258325, -0.0429718172322188, -0.0179653491069812, 0.16665434679713, -0.55009211728021, 0.298208058414284, 0.0105436529618627, -0.045641358958096, -0.0986348266925834, -0.496214629195886, 0.523607872420552, 3.63296134668653, -1.63100933102319, 1.6144974824207, 0.600769240145071, 0.148760563135812, -0.385305964720166, -0.718560478588005, 0.0640502139470673, 0.531451813575603, 1.87171194533259, -1.03234169645694, 0.226131557586438, -2.23869356375886, -0.444484110107669, 1.65575172856375, 0.408492202245387, -0.161844933293144, 0.0986348266925867, 0.0726598967590723, 0.496214629195886, -0.523607872420551, -1.41412520941868, -3.47111641339339, 1.55834943426412, -0.608864475247402},
    {-0.0291979345553107, -0.29871276705299, -0.2079121008487, 0.842273262501723, -1.26534612149435, 0.778929989503399, 0.0490770986832651, -0.280155127361664, 0.790509415439254, 2.42677632027106, 0, 0.0988531245289089, -0.0708151052579159, -0.139520723400707, 0.439393405738014, 0.0348293501727633, 0.030146724285998, 0.0509554216513404, 0.207912100848698, 0.108511731976808, -0.0188337949497869, -0.0169486831388893, -0.0490770986832633, -0.037963480514116, -0.0416583357847409, 0.355591838748823, 0.348912904231793, -1.74000560260787, 4.4742806734198, -0.187520565061739, -0.231367367035399, -0.893228684153063, 0.466253501958231, 1.46656676424376, -1.09249227657266, -0.584225120342077, 0.144543914870399, 0.297103810500549, 1.45262471986356, -0.916219535359863, -3.40368765176936, -0.10659921704305, 0.0211083563330913, 0.0379634805141167, -0.105265978517745, 0.0416583357847412, -0.355591838748822, 0.81811807947373, -0.370021260564885, 1.84527158112561, -5.18579953585048},
    {-0.504539511638762, 2.37732743172645, -0.138407634889359, 0.728837230454793, -1.61290224291863, 2.14718372548066, -0.202507479822817, 1.19732501931376, -3.18621932714503, 6.68960585737614, 0, 2.79289201598291, -6.40683631575313, 7.90408077473105, -6.16292439504852, 0.101360282467539, 0.0644327702181815, 0.0702918079193688, 0.138407634889363, 0.315790554508003, 0.127284178983569, 0.115474534996164, 0.202507479822825, 0.047017941881394, 0.0390289097275267, 0.0891689042881673, -0.343831895786023, -0.542262864258289, 1.7384038259334, -0.61601627277087, -0.465706679059004, -0.799129038374181, 1.68502110509688, 2.01417615175948, -3.31754884027422, -1.91921446575065, -0.919983606020188, -1.31279955430995, 5.24972638377039, 3.97891875418169, -10.3359083299039, -0.0694463503628787, -0.0331604431898496, -0.0470179418813936, -0.0522978732473168, -0.0390289097275282, -0.0891689042881675, 0.501943794915923, 0.376992338975874, 0.594560737505612, -2.17090127048646},
    {0.270764883720562, 0.401957822762747, -0.0496222433554893, -0.0732932595554674, 1.03279003696237, -1.19524818143039, -0.0374957547900422, -0.053729249366658, 0.994541200829972, -1.17984126324549, 0, -0.220304457461238, -0.12226740525593, -0.130094940242111, -0.200055903524031, -0.748639448326542, 0.569729765621711, -0.028244883228689, 0.0496222433554884, -0.738989379905982, 0.548630123230591, -0.0207055380471627, 0.0374957547900443, 0.347263467681681, 6.67852114450611, 0.467603604449727, -3.92151618722594, -4.01983994813106, 3.86900739911035, 0.743509478898021, -0.817642860136125, 0.101538142784164, 0.500022267596691, -0.784876942447964, 0.700355883262212, 0.733925536509278, -0.787361886605281, 0.0744347874138175, 0.493576909814796, -0.755809437455284, 0.6913281968274, 2.13430474713623, -1.51122719141427, -0.347263467681677, -1.54911803106602, -6.6785211445061, -0.467603604449728, -3.06302942755019, 5.43274337864022, 5.56895797919707, -2.94028271869639},
    {0.21585406400914, -0.292253144230848, 0.0690671693240343, -0.0852935608097449, -0.578626286337775, 0.810859764011253, 0.0758000408564917, -0.0918082993071564, -0.554715564231823, 0.800407666917413, 0, -0.410360586208545, 0.0945108881756237, 0.0942552324410884, 0.297993545813542, -0.466264044152153, 0.216276179301878, 0.0083531655066313, -0.0690671693240342, -0.460253834647198, 0.207338943397586, 0.00899118188658532, -0.0758000408564912, -0.452286089217331, 2.57554434643088, -0.414775430752262, 2.33651728470642, 2.40581537051748, -2.55267134213329, 1.01241087371511, -0.570150220657786, 0.0769403953031145, -0.394738708432763, 0.932500327693684, -0.962267885141452, 0.999360754297938, -0.546589757183024, 0.0828171174205687, -0.389650470625984, 0.893966378017264, -0.949864115942167, 0.954125344674886, -0.22882519387124, 0.452286089217334, -0.235611853668027, -2.57554434643088, 0.414775430752262, -2.51527827779067, -2.10769209083518, -2.17020351684945, 4.11382427524908},
    {0.0617978275970902, 0.165069823211547, 0.103341901603271, -0.360080603668168, 0.318850804445173, -0.524418917575722, 0.100320494542054, -0.345972012830991, 0.298611742339421, -0.51765908352349, 0, -0.262685088938024, 0.3895301281804, -0.177933589267736, -0.175779100783277, -0.0373827589171671, -0.0179101852435319, -0.0259244663684316, -0.103341901603272, -0.0369008898651062, -0.0167733358255092, -0.0249087002179156, -0.100320494542054, -0.318913566915097, 0.236022640306725, -0.327640246547998, 2.03586602632961, 2.14583003578395, 2.50253732604093, 0.219336145910995, 0.094153470179094, 0.386005070036601, -0.512486222993818, -0.395094089380737, 0.854951753575712, 0.216508871954389, 0.0881770765057487, 0.370880713048905, -0.50588020306321, -0.370015483019661, 0.84393130449742, -0.140570155268201, 0.146574794067084, 0.318913566915099, 0.154491794415889, -0.236022640306724, 0.327640246547997, 0.738974373639945, -2.1824408203967, -2.30032183019984, -3.10094154441268},
    {-0.236409154506208, -0.29871276705299, -0.0876613998747698, 0.425969859867175, -0.874047802450548, 1.72287170239396, -0.0922286345676698, 0.446069662039893, -0.907479821538979, 1.70066364236571, 0, 0.967488301382425, -1.43074659915474, 0.776815848578698, 0.221564370752814, 0.290609395595098, 0.0495856947412157, 0.041912343851744, 0.0876613998747676, 0.286863399364978, 0.051482330015, 0.0438900185639349, 0.0922286345676689, 0.133697918781808, 0.0330337359671369, 0.126027789841482, -0.885934215319064, -0.84229684184203, 6.93531858323389, -1.36591819250656, -0.296465798410205, -0.467882203718918, 2.65716806553409, 1.12092790611954, -3.30473097101658, -1.34831131372928, -0.307805510270051, -0.48995968060383, 2.62291679318309, 1.16380300179403, -3.2621325211845, -0.39344826362718, -0.0871694994431247, -0.13369791878181, -0.0828758984767799, -0.0330337359671359, -0.126027789841482, 2.352371066254, 0.973103714762189, 0.925172740318812, -8.89424138586072},
    {-0.636880676372854, 2.37732743172645, -0.204926878310814, 1.15124252490877, -2.81967943501395, 4.74923565701962, -0.204379670271955, 1.14636582665052, -2.80070023797626, 4.68801733738919, 0, 3.48050625171774, -7.84029320891773, 9.38272800844693, -6.76338780660053, 0.274486407165859, 0.132644959235412, 0.123672746018738, 0.204926878310819, 0.270948238537975, 0.131752129083818, 0.123148864515015, 0.204379670271962, 0.0995701821042717, 0.0599748362772144, 0.0987048250007576, -0.668484952008194, -0.664339741818177, 2.69459724124682, -1.63018076695788, -0.91924470878982, -1.27491527092753, 4.2894962816794, 3.6062791845684, -7.68303757890702, -1.60916750620306, -0.91305729392367, -1.26951469116555, 4.23420406764972, 3.58200540281614, -7.58400213737383, -0.12676077173266, -0.0718122966258672, -0.0995701821042793, -0.0713669955568572, -0.0599748362772105, -0.0987048250007615, 0.878466617721589, 0.740297248634068, 0.735706737375042, -3.44630308723577},
    {-0.0276979469831449, 0.401957822762748, 0.798848493973307, -2.9271659390857, 3.46950926213078, -1.89794570036166, 0.0500625320267806, 0.0350431744303312, 0.0648495746372393, -0.639301979663908, 0, -0.190070737363165, -0.0830134244805706, -0.0632780457615226, -0.0378976681743427, 0.316239485215835, -1.02157796980729, 1.43867237804988, -0.798848493973309, 0.106521766617375, -0.0190946015114828, -0.0172233649001307, -0.0500625320267773, -0.359647840141065, 1.4629203095634, 10.1191404810675, -0.406035546927844, -7.31723901916677, 3.3289534344605, 0.891573267377457, -1.66725986713032, 1.48849356103583, 0.422416661261282, -0.780671425193177, 0.267716286507078, 0.300316576360022, -0.0311632236792809, -0.0178198095302043, 0.142286371910371, -0.0145917494464737, 0.0901772647761464, -0.980192077386348, 0.19956235417726, 0.359647840141061, 3.59634878224641, -1.4629203095634, -10.1191404810675, -1.59971628304948, 0.206473192750593, 3.72089023692035, -0.749045074024681},
    {-0.119570063807636, -0.292253144230848, -0.123511007815444, 1.44142868628427, -2.12588434718252, 1.28757175841062, -0.0134898570388153, 0.00975677053316198, 0.0121888713050506, 0.43370428034606, 0, -0.0125784288977855, 0.132329389481234, 0.104087309469512, 0.187984937985523, 0.617280042826309, -1.14568614271631, 0.822231856537403, 0.123511007815446, 0.207923942877125, 0.00656885261329201, 0.00556553898616571, 0.0134898570388147, 0.0763243423373354, -0.207923533558815, 3.79815257401814, -0.0815244410540353, 4.78945655955072, -2.19635765840031, -0.216750817101653, 1.0603862842334, -2.26366054282168, -0.71721053708539, 2.21118420566543, -0.97089044704989, -0.0730101111114272, -0.00607978132455161, -0.0153223095193313, -0.24158442262449, -0.0126779425937901, -0.327033689487269, -1.18366577044186, -0.0465038563189919, -0.0763243423373368, 2.73204203318317, 0.207923533558823, -3.79815257401814, 1.0955382118154, 0.12802829737303, -7.5214985927339, 2.28448521702677},
    {-0.291832089112706, 0.165069823211547, -0.0529350629556799, 0.349373284618033, 1.68967763895773, -0.832729675112352, -0.124807634069926, 0.316622207129774, -0.144817183690852, -0.28049576430072, 0, 0.666751249076135, -0.293482001642875, -0.141875319062684, -0.104631662469416, 0.915563581070021, -0.844296045487711, 0.0688456508519483, 0.0529350629556785, 0.308397447711337, 0.0723620723206561, 0.0623918968156394, 0.124807634069926, 0.463445949019271, -0.606573202417975, 0.172254143431229, -1.56779076736319, 6.16160889113368, 2.1532215803736, -2.40331610080074, 2.81121530476935, -0.41821893546998, 1.50905861273623, -3.65659689823938, 0.811423582106843, -0.809530399477331, -0.240940800658548, -0.379014103945417, 0.508309673120419, 0.313395912028741, 0.273319042946295, -1.0759191122927, -0.308940553072915, -0.463445949019276, 1.2141740455886, 0.606573202417982, -0.172254143431235, 3.58244041451583, 1.87673132043612, -7.37578293672228, -4.65974288259674},
    {-0.533380965289453, -0.298712767052989, 0.0905706902337689, -0.559977693850787, 1.89158620077829, 2.73576399498904, -0.206188382357004, 0.856814815102093, -1.34146568604206, 0.921511789065655, 0, 2.16215147553411, -3.20288580537901, 1.88500413072956, -0.0121760685422081, 0.917081389878048, -0.183944330128652, -0.0778682969454356, -0.0905706902337711, 0.308908704790787, 0.130448724413433, 0.119145300218685, 0.206188382357004, 0.407788166161158, -0.105679618375735, -0.0897719471368901, -2.07955187073496, 0.987730791336992, 5.96725470776894, -4.0749835250867, 0.968771142162115, 0.637845990796226, 7.18333868826709, -2.67641301281176, -6.76120054804747, -1.37261304903998, -0.687028296306861, -0.975960115320781, 2.41962804474882, 1.89804525793549, -2.27743548956527, -0.580276314912848, -0.289174308837702, -0.407788166161166, 0.13734996126913, 0.105679618375738, 0.0897719471368852, 3.05611457539658, 2.36872617957267, -1.12508075260612, -8.44309296825267},
    {-0.793676417832505, 2.37732743172645, -0.191346895929754, 1.158001421545, -3.20320459390343, 7.54135545678689, -0.154769442476805, 0.822782425516262, -1.84772446633698, 2.54022202634901, 0, 4.2822268027007, -9.47608365189896, 11.0179181306319, -7.40771229532761, 0.528433634244662, 0.176251820768423, 0.138044531750748, 0.19134689592976, 0.177997014576991, 0.101668436068697, 0.0980833119458551, 0.154769442476814, 0.11347933145, 0.0449494719388239, 0.0537976271188634, -0.700306098461328, -0.408938164661921, 2.31847288345816, -3.06990022630245, -1.17356944769904, -1.29604595329577, 7.78815600698174, 4.20052222083408, -12.7880448717109, -1.03406187630758, -0.676957377491971, -0.920865737462138, 2.6233540570977, 2.42301340776028, -4.30751122171614, -0.127570704627952, -0.0834829954831269, -0.113479331450017, -0.0487492298415652, -0.0449494719388145, -0.0537976271188698, 0.849427147589549, 0.783789093944471, 0.457687394503495, -3.04032932641979},
    {-0.593196374838999, 0.401957822762754, 9.65546182492156, -9.34793058151145, 6.39444256476645, -2.46012846883514, -0.208059837648138, -0.103347499349383, -0.101728226296684, -0.206851511680584, 0, 0.0255453288852115, 0.0317342991905668, 0.0421242916846992, 0.091834632315773, 3.4420298444201, -8.37095104150299, 11.1521476124679, -9.65546182492158, 0.289411339930999, 0.133172202775036, 0.12329430113692, 0.20805983764814, 1.37482680900672, -4.22950603911584, 10.455971975912, 0.825605162922037, -4.36349548624225, 1.39615737113786, -0.193907244267697, 0.678637160839806, -1.80421703095647, -0.259373195247147, 1.29787131589676, -0.528620936070149, -0.0163040292857395, -0.0107963366582678, -0.0199468017875292, -0.0218085104928656, -0.0206476398200908, -0.0444472884718097, -1.82770661893572, -0.984952826322072, -1.37482680900673, 5.20568112317358, 4.22950603911584, -10.455971975912, 0.148173083867384, 0.159347663400041, -0.842185636931318, 0.283376163930478},
    {-0.645727750172868, -0.292253144230848, 2.64740738044187, 5.3635060916448, -4.00733663545901, 1.6689581466585, -0.203105697019895, 0.111094810499234, 0.0849052665399497, 0.140328653540355, 0, 0.640545865989456, 0.119780474987419, 0.0776803768834909, 0.0999741765433517, 3.17302984920508, -6.18782608077739, 5.92743640487679, -2.64740738044188, 0.266793392796454, 0.131104289577877, 0.122775552575934, 0.203105697019899, 1.12648329478581, -2.73017809443477, 4.57278385554715, -0.736093480810324, 2.92115822272211, -0.921148641698411, -3.4349689329955, 7.85911434065094, -11.2909424965216, -0.726267415706931, 2.33604837558546, -0.680751647161165, -0.288817647276124, -0.166514635171023, -0.233870363075176, -0.061065718610386, -0.0494949209468056, -0.0572386804503062, -1.42236804837865, -0.813487897840657, -1.12648329478581, 3.22829494325388, 2.73017809443477, -4.57278385554715, 1.80653964425778, 1.54958137865099, -6.14945316597599, 0.536977045819291},
    {-0.731477026246282, 0.165069823211546, -0.187994765484738, 4.35942248656922, 3.41733871056418, -1.07938914174274, -0.181363947469799, 0.379770084033704, -0.126228199455133, -0.0907567545717631, 0, 1.77555767996504, -1.08201166468954, -0.0794276066727926, -0.0477112055679689, 2.61204358754973, -3.20475026135101, 1.29520993109275, 0.187994765484733, 0.219624776309364, 0.118375698008267, 0.11283191429319, 0.181363947469803, 0.720530671443419, -1.06931012574517, 0.695443086724875, -1.7713255869622, 4.0320967759686, 0.903057444424332, -6.8544078999467, 9.71163878395076, -5.65463241766196, 4.83635733826001, -9.92422723316394, 0.485396115879704, -0.576329510324561, -0.358724370419346, -0.4926019983269, 0.406648611696386, 0.366576871866207, 0.0408128768905686, -0.846879348566569, -0.526271197274515, -0.720530671443428, 1.19795954703889, 1.06931012574518, -0.695443086724879, 2.56637355679593, 2.29759678423673, -5.2300563230075, -2.62255165265369},
    {-0.834219465627172, -0.298712767052989, 0.105852187397964, -0.836801817303566, 5.85275550665803, 3.54611350936128, -0.132465689012756, 0.494725599935106, -0.654005554827596, 0.298162859905294, 0, 3.33250949856373, -4.8651100989862, 2.86470948685671, -0.199176653754082, 1.75675341289619, -0.781534391644855, -0.143585284844443, -0.105852187397967, 0.147710619063583, 0.0873311439104309, 0.0848890558285604, 0.132465689012759, 0.305201060714203, -0.159414327099091, -0.043405541831799, -1.31415208952575, 0.988838774057071, 2.50265640830702, -7.54124013778058, 3.83004239932452, 0.980387102148019, 12.571887446701, -8.90126351433772, -10.3335142311779, -0.634079456502839, -0.427981145211688, -0.579614655763671, 1.05706427772837, 0.994655556128853, -0.868858300194401, -0.334186529975034, -0.225492940146236, -0.305201060714214, 0.169673026638176, 0.159414327099097, 0.0434055418317905, 1.63773800970378, 1.539645029672, -1.15851180069524, -3.80620788803578},
    {-0.930688141770993, 2.37732743172645, -0.142219700439307, 0.917394780947742, -2.83066387967159, 9.77514965223253, -0.0634805252678676, 0.323944096787989, -0.68330956025239, 0.821910118956344, 0, 4.97286168481876, -10.8586127019987, 12.3623063106051, -7.92319458338059, 0.78779691715601, 0.174605118508022, 0.118010714500009, 0.142219700439314, 0.0662392168845497, 0.0421488922094669, 0.0416711268844601, 0.0634805252678793, 0.0579129697042154, 0.01468955048439, 0.0137899579800127, -0.33569293358011, -0.116926760583239, 0.972363558022478, -4.50296915892159, -1.12926586459901, -1.03540549544777, 11.1258052383664, 3.78532462576262, -17.1857826488334, -0.37861680370499, -0.272599713052735, -0.36561522367247, 0.935475387311649, 0.913760381095679, -1.44500791944756, -0.0599787404999561, -0.0431824594679714, -0.0579129697042382, -0.0150410824730598, -0.0146895504843765, -0.0137899579800207, 0.387914998294357, 0.378875393048105, 0.131967843056311, -1.30029981581692},
    {-0.284218581943473, 0.401957822762748, -0.164167775300452, -0.0765004719955957, -0.0680662120055009, -0.108642456439213, 10.7781211235609, -9.94863282337192, 6.61942715210139, -2.49746884647899, 1.22955648709578e-14, -0.102836083183848, -0.0327899728859166, -0.0130791394678107, 0.0309659547182931, 0.0811284959085212, 0.0541568301344948, 0.0660860422357885, 0.164167775300458, 1.8649789201575, -5.26674221029165, 8.59427075157847, -10.7781211235609, 5.84457871350631, -2.9295267996841, 1.03312870189975, -2.37242845712512, 0.560795864993027, 0.744419162240994, 0.0253897324336094, 0.0140568892036677, 0.0104144297597957, 0.00732571946181193, -0.000147507332675767, -0.00520149136471526, 0.58365824790429, -1.36703000398041, 1.35436207179343, 0.16840337317133, 0.0143450621706881, -0.119571694754131, -0.592296543769635, 2.04945673051513, -5.84457871350636, -0.48445164131427, 2.92952679968412, -1.03312870189975, -0.153735860662546, 0.322971726610051, -0.0763442236787579, 0.00161324219117476},
    {-0.36549522728289, -0.292253144230847, -0.149034415307039, 0.077243285110056, 0.0542507299172663, 0.0737033512859893, 3.05411065485142, 5.7400262676936, -4.15256468707764, 1.69428996499951, 0, 0.291180856531248, 0.131435959729915, 0.0938638947958608, 0.141267660456711, 0.0857805129140898, 0.0565322542722033, 0.0659859656207882, 0.14903441530704, 1.97191931828046, -4.32720155341021, 4.90348352510901, -3.05411065485142, 2.57031678860304, -1.80768399788644, 0.795122442749991, 1.5898514161783, -0.477470143798948, -0.491148572738422, -0.0797111483590371, -0.0675209347466543, -0.14322925073085, -0.037739497088384, -0.0432620494428156, -0.04203321875266, -1.83239698611847, 5.1683184667749, -10.6435097928026, -0.867554189671666, 3.31144777371294, -0.966258107489831, -0.511803915667498, 1.35814887650926, -2.57031678860305, -0.407884367537952, 1.80768399788645, -0.795122442749999, 0.611287825645056, -2.94800029268755, 0.885354511336907, 0.391664662760856},
    {-0.505828786742265, 0.165069823211547, -0.120198635080441, 0.242093133212254, -0.0757646835227197, -0.0476672211627449, -0.171838342059684, 4.78332720332143, 3.55195396645563, -1.0957723504605, 0, 1.21123522802956, -0.683612261272444, -0.112446472967149, -0.0744175302592477, 0.0836574821649647, 0.0547790077565402, 0.0605627881224788, 0.120198635080441, 1.92311516445491, -2.56811623612244, 1.19661234538719, 0.171838342059687, 0.400777748812655, -0.667103789899055, 0.466290412408256, 2.20115754244777, -1.07932221259696, 0.481502501172913, -0.220756563471549, -0.172735676297313, -0.302655921334728, 0.151595332229321, 0.193721352063491, 0.033170970240006, -5.07474387082231, 8.0980892685804, -5.97993954870863, 3.48486799657566, -9.08192699891359, 0.76253306025224, -0.348133563293237, 0.550648571062888, -0.400777748812667, -0.270006677224028, 0.667103789899062, -0.466290412408268, 1.09777611826255, -2.75180611351065, 1.349328889821, -1.23114505614223},
    {-0.685613082674016, -0.298712767052992, -0.0793175468753278, 0.289572373956817, -0.36973299613861, 0.15660096102692, 0.101751927918443, -0.819554123255372, 6.17090549740159, 3.59993720974319, 0, 2.75853623958354, -4.05750949342543, 2.39473755882654, -0.111438455257645, 0.0653263317252917, 0.0429174987338478, 0.0452672664809432, 0.0793175468753257, 1.50171934330263, -0.716300226480564, -0.128116416583609, -0.101751927918434, -0.0223275507047551, -0.0950216651577064, 0.181351467474108, 0.547589205415054, -0.754214184304276, 1.33439497965096, -0.284820735267657, -0.21711744720783, -0.334839640437758, 0.486712731435373, 0.543932944612597, -0.423819288919939, -6.54744872747121, 3.62372647978193, 0.94767053983897, 11.1885346096239, -9.07833175070297, -9.74274243519852, -0.154892572336577, 0.0856016274788255, 0.0223275507047493, -0.117902181061299, 0.0950216651577101, -0.18135146747411, 0.783593659680087, -0.633190832893871, 0.872116365365569, -1.96309606699446},
    {-0.865091413021994, 2.37732743172644, -0.0350775308878956, 0.177387214604174, -0.369081269115168, 0.431683257087086, -0.138221459026586, 0.89552261869692, -2.7845317577215, 9.92351905007649, 0, 4.64328199706198, -10.2017134115417, 11.7275325151306, -7.68133711935529, 0.0326254251842071, 0.0216017065294981, 0.0220338922063795, 0.0350775308878983, 0.749992090304532, 0.162973965047241, 0.111236026185822, 0.138221459026598, 0.00707007652837136, 0.00780507151371851, 0.0321936614353265, -0.0604113676176929, -0.184072537996847, 0.518455927834857, -0.18788418307497, -0.141590644936174, -0.199421106810571, 0.469886015892666, 0.489070207521874, -0.746310515088996, -4.31907478305564, -1.06822990060221, -1.00675864488278, 10.8017226832917, 3.68978769327651, -17.1561590406171, -0.0303443543163876, -0.00750390925917696, -0.00707007652836672, -0.022864299828075, -0.00780507151371884, -0.0321936614353289, 0.198895244316045, 0.0679152768768674, 0.206936837824922, -0.687006817834514},
    {-0.377854821367115, 0.401957822762745, -0.0187732230698006, -0.012016271905727, -0.0185107978054602, -0.540385364807988, 2.40929939068857, -4.57083758905508, 4.35130301463395, -2.08597903119923, 0, -0.065954324558547, -0.0135282337283473, 0.00416050908392901, 0.0512190478073331, 0.512650482834977, 0.0177547093611521, 0.0116964169713665, 0.0187732230697975, 1.97891768943057, -4.17357053861732, 4.44916882452415, -2.40929939068856, 13.3564056813897, -0.52370889564461, 0.135540760594579, -7.75704029176245, 0.127382044938553, 3.09265449903965, 0.0762655524189189, 0.00203297545704251, 0.000319854934366592, 0.00736850255180976, -0.00127688701274553, -0.0558991729977106, 0.294397949147327, -0.477888221128151, 0.121668764530917, 0.0284436678255435, 0.300155745111535, -0.215780275204205, -2.96633253531301, 7.55055964345081, -13.3564056813897, -0.123991328088738, 0.523708895644623, -0.13554076059459, -0.339655306040382, 0.20648064831166, -0.00339071684981973, 0.213333342313744},
    {-0.451804425814343, -0.292253144230849, -0.0798554220922592, 0.0537913438739489, 0.0586818441400052, 0.36659896763781, 0.26145911536624, 2.41943234803004, -2.69149974362031, 1.4151341045727, 0, 0.398538471660795, 0.128767605110298, 0.0892235887423916, 0.127527904531707, 0.511383406553643, 0.0706022831938977, 0.0504421912770379, 0.0798554220922573, 1.97402656038466, -3.23824225193071, 2.26879383358003, -0.26145911536624, 5.38877757230742, -1.26735597786622, 0.462482740052734, 5.12552641892727, -0.431374279638449, -2.04045371240008, -0.508966082266047, -0.0866767814013216, -0.104233535150986, -0.179754998976263, -0.0426073459325803, -0.189261292949142, -1.96469527922141, 3.97551471563992, -4.68822618161007, -0.693884740477667, 1.9542272799111, -0.730580645258291, -2.45494484636851, 4.80640127944976, -5.38877757230743, -0.404516086761275, 1.26735597786622, -0.462482740052741, 3.01387870441238, -9.93192769837703, 0.835890366399729, 1.48151985435621},
    {-0.576948661244746, 0.165069823211546, -0.166654346797133, 0.403271313827154, -0.169838959850666, -0.237095786874342, -0.143420606214597, 1.24431233374446, 2.20510501981534, -0.915229893358237, 0, 1.3900702117453, -0.810470103908833, -0.102189848153358, -0.0655314216499061, 0.466402264732029, 0.134626102632261, 0.107182504067224, 0.166654346797131, 1.80039173466613, -1.74791870471657, 0.330716585087002, 0.143420606214598, 0.518569439976398, -0.928380190662204, 0.648755725594752, 6.79699700343544, -2.02083791656575, 2.00037956044611, -1.22924364026096, -0.418665942271223, -0.510453817894376, 0.854367426853088, 0.453878799489632, 0.145569735550181, -4.74508864378752, 5.43575144207063, -1.57502891883146, 3.29800296865633, -5.8929377571694, 0.561923833823295, -1.58563915043343, 1.8065236330643, -0.518569439976399, -0.537103584571725, 0.928380190662206, -0.648755725594757, 4.93108762742472, -8.60352063649975, 2.55794150113748, -5.34582803743741},
    {-0.73356085670795, -0.298712767052992, -0.207912100848701, 0.842273262501727, -1.26534612149436, 0.778929989503398, 0.111739981375583, -0.745172378670274, 3.04348347201991, 3.00680168393097, 0, 2.94455441648735, -4.32073941103731, 2.54909057623279, -0.140631957921897, 0.344732873319208, 0.154120270281249, 0.135945654934046, 0.207912100848698, 1.33072727712461, -0.370698963181921, -0.1202732552096, -0.111739981375579, -0.100977402568773, -0.160385763858697, 0.440582072031525, 1.34332448983402, -2.15588545687898, 5.54368136479748, -1.49510845941515, -0.771220767301757, -0.978218917435771, 2.53333856454286, 1.88244661851487, -2.16189296795033, -5.77137187424893, 1.85498467074717, 0.865445633879871, 9.77911592117685, -4.52776917958516, -8.3452730079835, -0.675225265073469, 0.216816959162411, 0.100977402568774, -0.347966952586951, 0.1603857638587, -0.440582072031525, 3.37884008431334, -1.56014144899644, 2.50385240946593, -8.24729618403735},
    {-0.88665764784495, 2.37732743172645, -0.138407634889358, 0.728837230454791, -1.61290224291862, 2.14718372548066, -0.177662099660991, 1.0978800186345, -3.14581523070908, 8.28849284080141, 0, 4.75184955947061, -10.4186655573304, 11.9379575924521, -7.76181137847379, 0.165827532811168, 0.096084089184178, 0.0916041248827751, 0.138407634889362, 0.640122362237716, 0.187403044735945, 0.137987377882087, 0.177662099661001, 0.0431128207263926, 0.0385339884209562, 0.110481221251568, -0.33947180137673, -0.671869458987859, 2.15390083853497, -0.952574072495693, -0.626964592754571, -0.820441355337583, 2.37260866707963, 2.14378274648905, -3.73304585287578, -3.67709725372752, -1.22283589948615, -1.23586739651663, 9.15867129474928, 4.18124808545933, -14.4101892440609, -0.128312550355933, -0.0426666146954242, -0.0431128207263923, -0.0844441135198877, -0.0385339884209535, -0.110481221251571, 0.837260638699481, 0.382138416072155, 0.75631357250775, -2.86284892687853},
    {-0.537207852379276, 0.401957822762745, -0.0496222433554921, -0.073293259555463, 1.03279003696236, -1.19524818143038, -0.207274898435275, -0.654589019829709, 1.79774025209256, -1.46183587974912, 0, 0.000986959642842206, 0.0198669010274393, 0.0324574559666422, 0.0819387129796056, 1.53428368653198, -1.26116386310603, 0.0835173539639629, 0.0496222433554861, 1.87648973454375, -2.19526230893808, 0.745901372126379, 0.207274898435274, 4.23074685784624, 12.0721457048842, 0.579365841642382, -7.08856253830945, -4.98062447050061, 4.79374133726794, -0.0249604182970792, 0.0524662462220626, -0.0102240944084851, -0.0896970319091348, 0.175907579921592, -0.224378054895395, -0.0305275804699886, 0.0913260966256324, -0.0913123522966792, -0.109703023680691, 0.306195960219889, -0.274423250643955, -5.85374870716355, 8.07738651819153, -4.23074685784624, 5.67540015803981, -12.0721457048842, -0.579365841642391, 0.243524438002675, -0.988823979882096, -0.694775687539206, 0.816482931892949},
    {-0.595832618747913, -0.292253144230848, 0.0690671693240319, -0.0852935608097434, -0.578626286337776, 0.810859764011255, -0.0940416992472928, 0.175758610791812, -1.06039627034465, 0.991713616378837, 0, 0.578196408986783, 0.122433755117552, 0.0807680025223076, 0.106687596352119, 1.43723513893443, -0.844120572468691, -0.0907825760323327, -0.0690671693240364, 1.75779551592058, -1.54694373190725, 0.187069449276908, 0.0940416992472921, 0.865860443132101, 4.92341984823269, -0.513911172291228, 4.46649485620536, 2.98083084414265, -3.16278695565622, -1.53102873032068, 1.06526200473794, 0.176076136842082, -0.364913901006483, 0.357484854068525, -0.352152271618525, -1.87250879414107, 1.95220971365373, -0.362828060068724, -0.446304158247794, 0.655130288598173, -0.430696179910565, -4.61398591568056, 4.75393341574706, -0.865860443132102, 3.17266039990434, -4.92341984823268, 0.513911172291221, 5.82275097501292, -9.22042827195242, -6.15349124404698, 1.95402189632387},
    {-0.69232834135712, 0.165069823211547, 0.103341901603268, -0.360080603668166, 0.318850804445171, -0.52441891757572, 0.107893098474517, -0.457954895166256, 0.73575333500753, -0.641385112850754, 0, 1.67827206924226, -1.01371715322553, -0.0852433264151487, -0.052053071456012, 1.20941104184694, -0.287528119487927, -0.104233971684685, -0.103341901603272, 1.4791576191489, -0.663475738095749, -0.132566034075007, -0.107893098474516, -0.422138276182026, 0.581539236811284, -0.405949751864253, 5.01619663970861, 2.65870624792654, 3.1006703762461, -3.1776259564589, 0.876647616566075, 0.464314575352853, 2.23581512881714, -0.907970301523318, 0.256818703370547, -3.88636243730967, 2.0228784074647, 0.590520929241259, 2.73449048203479, -2.09515600437648, 0.314099448976726, -2.79607236364135, 1.4520584924092, 0.422138276182025, 0.769626325164842, -0.581539236811283, 0.40594975186425, 8.52497549699787, -6.46825513211781, -3.42833257309137, -8.82957350960262},
    {-0.809132783013123, -0.298712767052991, -0.087661399874771, 0.425969859867177, -0.874047802450548, 1.72287170239396, -0.00868264811335726, 0.0459205252311543, -0.110712783735199, 2.10714034950458, 0, 3.23613307271919, -4.73043712480305, 2.78706193853603, -0.184912336386053, 0.83094285403047, 0.114188356405143, 0.0720343266745766, 0.087661399874768, 1.01627603113289, 0.01446386658981, 0.00776546518244274, 0.00868264811335673, 0.013763497442743, 0.00403012472486476, 0.156149772664312, -0.108084213947468, -1.04361474984949, 8.59293352274684, -3.57580045608792, -0.562386368081592, -0.498004186541752, 5.98433181019302, 1.322245814127, -4.96234591052953, -4.37334562540561, -0.0712356465750289, -0.0536859904135982, 7.31907489371375, 0.167484563720415, -6.06914564894561, -1.12260788587319, -0.0182777569715523, -0.0137634974427443, -0.176482171382981, -0.00403012472486268, -0.156149772664313, 5.52892949501758, 0.12636197091902, 1.22009692123247, -12.9992551318912},
    {-0.919863564854678, 2.37732743172645, -0.204926878310814, 1.15124252490877, -2.81967943501395, 4.74923565701962, -0.208038376322079, 1.20141028770934, -3.07625576054693, 5.80850336580874, 0, 4.91860631702009, -10.750820364052, 12.2586240151802, -7.88387383502008, 0.37883236455267, 0.172470869633391, 0.147264249503355, 0.204926878310818, 0.46332699059249, 0.188164831664137, 0.15368159231188, 0.208038376322087, 0.10435119256711, 0.0658756453418098, 0.122296328485371, -0.734255903780727, -0.823124009293965, 3.338634654879, -2.16797150223988, -1.11785488666359, -1.29850677441215, 5.36697847320677, 3.76506345204418, -8.3270749925392, -2.65151503887264, -1.21957393165062, -1.35509188002124, 6.56402730400788, 4.10766486053345, -10.1843426215365, -0.204213718469151, -0.0939242968133483, -0.104351192567118, -0.105292096890244, -0.065875645341803, -0.122296328485377, 1.32359339058075, 0.828180200594082, 0.928416106184214, -4.45801432699062},
    {-0.732933512062503, 0.401957822762748, 0.798848493973305, -2.9271659390857, 3.46950926213077, -1.89794570036166, 0.112109676444358, 0.0917403520483129, 0.246381490393368, -0.792101955560178, 0, 0.0888024830338865, 0.0615698311931759, 0.0657010673507652, 0.114902307721928, 3.19425102379765, -5.27695557341787, 3.85725462991614, -0.798848493973311, 1.33311110113324, -0.374734315630514, -0.120890275800025, -0.112109676444357, -0.941530554931677, 5.55803932736951, 12.5377227329338, -1.54264147088668, -9.06613700684972, 4.12460872839966, -0.427300429130724, 0.839219748797305, -0.930088690830428, -0.341065886873197, 0.968226562489775, -0.527939007432077, -0.178332554752042, 0.0595958093362493, 0.0291499237517039, -0.142342826728741, 0.0687570159009028, -0.220333764092268, -6.2733301377986, 2.03280616121709, 0.941530554931672, 11.9468454036631, -5.55803932736951, -12.5377227329338, 0.997678011330304, -0.490164690330401, -2.88070839681339, 1.15104339806863},
    {-0.768918916307804, -0.292253144230848, -0.123511007815448, 1.44142868628427, -2.12588434718252, 1.28757175841062, 0.0718509496285213, -0.0586458312744749, -0.0944416092021645, 0.537364218358182, 0, 0.794667675947981, 0.1123845638158, 0.0697948208014681, 0.0843249999734019, 2.85197444483158, -3.71701831994325, 1.73003073753795, 0.123511007815446, 1.19026299568438, -0.165127134989547, -0.0703878670507394, -0.0718509496285241, -0.458769065812963, 1.61102965228292, 4.70595145501869, 0.631666313455994, 5.93418764147279, -2.72131051000823, -3.18254296716032, 4.77644954338241, -3.17145942382222, -0.511065640639908, 1.06645312374336, -0.445937595441974, -1.3282247787498, 0.212191966956172, 0.129033698325208, -0.213291714980757, 0.0473767772355431, -0.186110720312009, -4.75809562893699, 0.758138192020568, 0.45876906581296, 7.12232739625184, -1.61102965228291, -4.70595145501869, 6.11425657287395, -1.38980450547656, -13.0565150377246, 1.36514956607127},
    {-0.82658934733983, 0.165069823211547, -0.0529350629556826, 0.349373284618034, 1.68967763895773, -0.832729675112352, -0.049810893415801, 0.136571318347944, -0.0720377011179925, -0.347537236699552, 0, 2.0109332971846, -1.24662728264591, -0.0651963003398178, -0.0375901900705843, 2.24315270037299, -1.72929155552928, 0.110016224073326, 0.052935062955679, 0.93617306345854, 0.0737266004774343, 0.0430057517929092, 0.0498108934158009, 0.199902037239708, -0.301733108933046, 0.213424716652608, -0.779880120828439, 7.63429898125734, 2.66786445032589, -5.8673384817935, 5.16890090493458, -0.459389508691358, 4.16013474437831, -5.12928698836303, 0.296780712154553, -2.44871614845278, -0.220370874250243, -0.17957707014086, 1.73621978004322, 0.218681974890799, 0.12386054165057, -2.73041162342134, -0.245581073026805, -0.199902037239716, 2.4040096491151, 0.301733108933054, -0.213424716652613, 8.16127683386899, 1.02546119385526, -10.0383086303724, -8.09872966077356},
    {-0.894125141570335, -0.29871276705299, 0.0905706902337681, -0.559977693850787, 1.89158620077829, 2.73576399498904, -0.188260025413352, 0.814745478491389, -1.36447231087397, 1.14176291237891, 0, 3.5618538267779, -5.18416669496205, 3.04757796866293, -0.23242719185546, 1.44043934924041, -0.265477095115417, -0.0993247475666096, -0.0905706902337709, 0.601163049702499, 0.191498619152117, 0.144513593792823, 0.188260025413352, 0.387765896090989, -0.107492062300052, -0.111228397758066, -2.11521694231024, 1.22380896082372, 7.39349207996251, -6.14849815317976, 1.28638207663561, 0.659302441417401, 10.1597327291914, -2.91249118229849, -8.18743792024105, -2.5660572955084, -0.92791580106245, -0.959259072284216, 4.24013403609374, 2.1008894927843, -3.41700270266674, -1.03764914299952, -0.375181710183564, -0.387765896090998, 0.217070282331576, 0.107492062300056, 0.111228397758061, 5.02797900063865, 2.49039865249381, -1.44087924315529, -11.3838219376017},
    {-0.9561638467952, 2.37732743172645, -0.191346895929754, 1.158001421545, -3.20320459390342, 7.54135545678689, -0.175290778787528, 0.945835685654873, -2.17371600504682, 3.14736212092741, 0, 5.10035308667259, -11.11138350932, 12.6047192276222, -8.01485238990602, 0.622379214652032, 0.201459834843234, 0.150902737088019, 0.19134689592976, 0.259748101819745, 0.136711987803173, 0.123254765620598, 0.175290778787538, 0.130451013464852, 0.052879846726023, 0.0666558324561319, -0.823860160100111, -0.506678737491459, 2.87261257327238, -3.54758433500701, -1.29651803460339, -1.30890415863303, 8.72603422509314, 4.29826279366362, -13.3421845615251, -1.48057370067979, -0.879825786967572, -1.06909045127549, 3.64178425792917, 2.91682980421125, -5.56832077999655, -0.180667839850608, -0.107359758652988, -0.130451013464869, -0.0660268691291784, -0.0528798467260121, -0.0666558324561397, 1.16270874947062, 0.931219918753115, 0.572705606620646, -3.85465348289242},
    {-0.909191772905898, 0.401957822762755, 9.65546182492156, -9.34793058151145, 6.39444256476645, -2.46012846883515, -0.203361938358278, -0.104482620171885, -0.108660442219742, -0.256291224061134, 0, 0.17199524786081, 0.0995206497877137, 0.0944437077983061, 0.141274344696321, 5.00241514142446, -11.3487406799524, 13.6512361757966, -9.65546182492158, 0.521141524151648, 0.192848581941363, 0.152581035106701, 0.203361938358282, 1.38992726666714, -4.51772348061062, 12.9550605392406, 0.881865588026429, -5.40641733902898, 1.7298538392286, -1.01446974146352, 2.6135049465025, -4.30330559428512, -0.665499526964965, 2.34079316868349, -0.86231740416087, -0.105685412410901, -0.0444111586512763, -0.0480984149348113, -0.0693303990172534, -0.0397769810703525, -0.0898344886623536, -3.07011321734212, -1.28783106725961, -1.38992726666715, 7.89525332012822, 4.51772348061062, -12.9550605392406, 0.707017307569636, 0.405965479233195, -2.48883598109923, 0.633242070543894},
    {-0.921992874650238, -0.292253144230848, 2.64740738044187, 5.36350609164481, -4.00733663545901, 1.6689581466585, -0.208294305373906, 0.117198629999377, 0.093730390841944, 0.173868694961461, 0, 0.986441822515254, 0.101770430703842, 0.0595996305397436, 0.066434135122247, 4.33565424042011, -7.98072916563719, 7.02038040494667, -2.64740738044188, 0.45167971773004, 0.186666839336591, 0.153403194006994, 0.208294305373909, 1.18837503095603, -3.0139550853312, 5.66572785561703, -0.812603651860566, 3.61934612174252, -1.14131296892667, -4.96477260175219, 10.3502053245312, -12.3838864965915, -0.579252465393524, 1.63786047656505, -0.460587319932908, -0.517220000259071, -0.242088169428662, -0.270601824006381, -0.0603453540238832, -0.0383090607498746, -0.0479830584085536, -2.27295845765395, -1.06363014361016, -1.18837503095604, 4.73742103721725, 3.0139550853312, -5.66572785561702, 2.9477991600245, 1.87623379547074, -8.35676715895977, 0.466472266556128},
    {-0.942112680825337, 0.165069823211546, -0.187994765484739, 4.35942248656922, 3.41733871056418, -1.07938914174274, -0.198009063873276, 0.423068842341573, -0.145155176647879, -0.112448584649124, 0, 2.29507780670405, -1.44424604357707, -0.04776953002258, -0.0260193754906073, 3.26179396758608, -3.84189325821416, 1.46142822896921, 0.187994765484733, 0.339807119497161, 0.163188592583586, 0.141827214705653, 0.198009063873279, 0.802680595062642, -1.22964520498508, 0.861661384601337, -2.03692264950508, 4.99581081061516, 1.11889778299724, -8.4962699578049, 11.3124958154605, -5.82085071553842, 6.04430935465477, -10.8879412678105, 0.269555777306797, -0.885124275022371, -0.48051055733674, -0.564896057047233, 0.629683962741226, 0.462477141401028, 0.0280817774331017, -1.25790453133581, -0.682846470922406, -0.802680595062653, 1.67476746466213, 1.22964520498509, -0.861661384601341, 3.70391335484413, 2.7197691204275, -6.6705782752773, -3.56490660650557},
    {-0.965098624043572, -0.298712767052989, 0.105852187397964, -0.836801817303567, 5.85275550665804, 3.54611350936128, -0.153213077878958, 0.578913035295761, -0.779162211451891, 0.36942695614764, 0, 3.83217939154853, -5.55777503351949, 3.25984778306395, -0.270440749996428, 1.99682203872871, -0.867784465877592, -0.153959671216829, -0.105852187397967, 0.208024894236652, 0.115525902752573, 0.106511791363547, 0.153213077878961, 0.357137112889894, -0.18992135269002, -0.0537799282041856, -1.56564059846392, 1.22518176320385, 3.10081792044093, -8.46957559565648, 4.15263546270403, 0.990761488520405, 13.8583157908783, -9.13760650348449, -10.9316757433118, -0.882343310191781, -0.552829624745701, -0.685424826659313, 1.44373139958484, 1.21646593344502, -1.13883993977735, -0.459756369425038, -0.28805567435306, -0.357137112889906, 0.225416075279983, 0.189921352690027, 0.053779928204177, 2.20008617225899, 1.85369627281699, -1.45059783848382, -4.8411477232749},
    {-0.985708641666342, 2.37732743172645, -0.142219700439307, 0.917394780947742, -2.83066387967159, 9.77514965223253, -0.0766373383161878, 0.392884702108768, -0.834522061658295, 1.01835538326078, 0, 5.24786462969102, -11.402946162092, 12.8831025900259, -8.11963984768503, 0.828621425313903, 0.181971301432524, 0.121306661194978, 0.142219700439313, 0.086324109519995, 0.0536478621586488, 0.0519509511468617, 0.0766373383161998, 0.0702377974350468, 0.0179402640737755, 0.0170859046749803, -0.40997986170706, -0.144873500570096, 1.20476879522565, -4.70835536051623, -1.16457878751037, -1.03870144214274, 11.5227721690063, 3.81327136574947, -17.4181878860366, -0.490506969025445, -0.343335249973039, -0.444835653255651, 1.20041917370706, 1.12420944947271, -1.81459169746241, -0.0774494446927982, -0.0542114357021901, -0.0702377974350703, -0.0191565518081778, -0.0179402640737614, -0.0170859046749894, 0.495660467796999, 0.464191297409274, 0.164030052378285, -1.62297981832989}};
    
    // Array of non-zero columns
    static const unsigned int nzc0[51] = {0, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55};
    static const double FE0_D100[125][50] = \
    {{-3.62990990645797, 0.284218581943474, -0.0729924116469172, -0.0498880353538378, -0.0615259070688939, -0.153888727424807, -0.0811284959085292, -0.0541568301345019, -0.0660860422357863, -0.164167775300466, -3.05254952705567, 2.02813935973006, -0.852973082215368, 0.153888727424807, -3.39280133686021, 2.20168218722082, -0.916193158666491, 0.164167775300465, 6.93935899282269, -6.18108222239521, 4.27656497941378, -1.68915042532677, 0.0231815773747428, 0.0189606934660911, 0.0404350712376758, 0.0194130196135851, 0.0277492861273936, 0.0418410144593158, -0.942419743749323, 0.262864245603203, -0.0404350712376753, 0.269135132885879, -0.0277492861273945, -0.0418410144593163, 4.19215254652988, -1.55580759904069, 0.489196991213394, -2.39410207157935, 0.415850747203131, 0.914498989284263, 4.65942997423121, -1.72922537542377, 0.543725233961301, -2.59895941573084, 0.451434058644518, 0.982279200902278, 1.11247239987882, -0.193234233504243, -0.281824939069297, -0.288548152499463},
    {-0.282631633534523, 0.365495227282887, -0.452267144066576, -0.0473046632961878, -0.03407101538574, -0.0578765020203711, -0.0857805129140872, -0.0565322542722027, -0.0659859656207897, -0.149034415307042, -6.52731890993827, 1.06473844170331, -0.35627345648693, 0.0578765020203716, -1.23802219859217, 1.27243405037863, -0.690001392244448, 0.149034415307042, 1.65335880966175, -3.50106007665704, 3.57567070399205, -1.81083303074512, 0.117384161266235, 0.0935498203733121, 0.182364335615596, 0.0148471880767296, 0.0197614744334073, 0.0178594059513562, -2.64209530812438, 0.978230835822898, -0.182364335615596, 0.15525392933897, -0.0197614744334088, -0.0178594059513577, 10.5622964722508, -6.08250695734887, 2.49979653910288, -1.33696013079068, 0.319526352383561, 0.390344471872671, 2.00332750416847, -1.15365569542249, 0.474130902760275, -1.59775727801775, 0.381855481911313, 0.755987357865237, 3.31759984457828, -0.792888697720136, -1.07178065619621, -0.170101117415695},
    {0.143241115565021, 0.505828786742266, -1.3428933655604, 0.95026478074861, 0.0282805736101464, 0.0204847751966061, -0.0836574821649677, -0.0547790077565391, -0.0605627881224791, -0.120198635080441, -0.878278855981279, -6.65596057481905, 0.178742789963476, -0.0204847751966048, -0.0547136501039005, 0.383689812925607, -0.382777303820132, 0.120198635080441, -0.233743731500401, -0.770376839904916, 2.14650769602211, -1.79145702692408, 0.2430984171496, 0.188543141982841, 0.325606241800371, -0.203753216575855, -0.246844448755224, -0.0094719268694694, -1.70273960808483, 1.1916564243379, -0.325606241800372, -1.28778923995135, 0.246844448755222, 0.0094719268694694, 4.50684360679748, -7.51537127670231, 5.22969989144651, 10.1153513886495, -4.40965559457908, -0.207023363573621, 0.280760333117444, -0.468180910463754, 0.32579170961518, -0.583110016707578, 0.254199211538508, 0.443340091942612, 2.58772708545043, -1.12808589451521, -1.38019956632074, 1.49154245652721},
    {-0.102291761943678, 0.68561308267402, -2.84662379487986, 4.3801539174997, -2.80875302502541, 0.288623553688339, -0.0653263317252945, -0.0429174987338489, -0.0452672664809421, -0.079317546875327, 0.558471148525243, -1.03193983204807, -7.88994374560002, -0.288623553688338, 0.0128161900329828, 0.0101111233234064, -0.127158273749778, 0.0793175468753272, 0.116252719469202, 0.139438536816996, 0.574619210989792, -1.41363178800633, 0.293610709637523, 0.223492595719795, 0.343765567681152, -0.523451326810272, -0.581054568323697, 0.489496797176187, -0.0691730455362489, 0.627803153953486, -0.343765567681151, -1.47040394270873, 0.581054568323699, -0.48949679717619, -1.0511982707361, -3.02713440179703, 6.36648531888775, 8.03421925619176, -11.3824333416434, 10.6986967706254, -0.0241236397541215, -0.0694688165202924, 0.146102597966726, -0.0787206571389024, 0.111527032549345, 0.17242554023072, 0.538550211162609, -0.762987875263888, -0.851295749673278, 1.993855269519},
    {0.290248257600274, 0.865091413021992, -4.68268090500078, 10.4012155852295, -12.1434499468309, 8.1697622215787, -0.0326254251841883, -0.0216017065295067, -0.0220338922063713, -0.035077530887907, -1.68368761382697, 4.01561825483129, -4.72603256077766, -8.1697622215787, -0.0117306785136116, -0.00833981435771908, -0.00857523129456642, 0.0350775308879069, -0.151494055908918, -0.120634157325973, -0.172618275021671, -0.710593182365706, 0.189459236129027, 0.142756465828401, 0.201005644744987, -0.478909486641503, -0.498130062975762, 0.771828366999447, 0.0731449089686982, 0.0555584869803152, -0.201005644744988, -0.186383757288265, 0.498130062975771, -0.771828366999453, 1.09751734781612, 1.13963425166939, 4.12921691934224, -3.9690223685424, -10.4478114715184, 16.8694825076085, 0.00764668164368537, 0.0079401207918075, 0.0287693012623067, 0.00824304195135561, 0.0216984789358701, 0.0306091235009386, -0.0722961599978937, -0.190307985099834, -0.198314952808717, 0.665293243929773},
    {-0.0804169334283805, 0.377854821367114, -0.0927302983767186, -0.0601399658498372, -0.0693258960286219, -0.153888727424809, -0.512650482834975, -0.0177547093611561, -0.0116964169713651, -0.0187732230698003, -1.11829004720866, 1.23357237757902, -0.694497732197734, 0.153888727424809, -6.18235833041367, 0.364179106695015, -0.117173170875438, 0.0187732230697999, 1.23469180432362, -3.1671132019426, 3.45521657617556, -1.8202330664953, 0.138999667623016, 0.00581012182124541, 0.00635131781340881, 0.108801419528594, 0.00754649641305671, 0.208116353436679, -2.85111818820362, 0.0582050382295243, -0.00635131781340759, 1.08995834819509, -0.0075464964130576, -0.208116353436679, 1.87426704619965, -1.1608709134869, 0.497624212872628, -1.56685764288182, 0.393425231152639, 0.763823628226357, 10.361704027873, -6.41776252989538, 2.75106731527105, -0.462572628144319, 0.11614823081046, 0.128869587846803, 3.62143025017603, -0.90931172959542, -0.0640151600507721, -1.19875976772368},
    {0.296938442417927, 0.451804425814346, -0.542062083177416, -0.0546170488540245, -0.0374003603131548, -0.0578765020203726, -0.511383406553643, -0.0706022831938999, -0.0504421912770374, -0.079855422092258, -1.80057222446278, 0.62211448594427, -0.28862988603762, 0.0578765020203722, -1.69866660382198, 0.804194002372015, -0.389277637912434, 0.0798554220922591, -0.0612764125533407, -1.55859240591334, 2.70162889910246, -1.83050294886806, 0.674120877709403, 0.111078967765189, 0.126996446014248, 0.0810660330724962, 0.0213755605582862, 0.0888323213280853, -7.67856140344349, 0.857229971550943, -0.12699644601425, 0.625611082121128, -0.021375560558288, -0.0888323213280868, 4.2531822525067, -4.33770602065991, 2.4271580757934, -0.861548826491947, 0.2940513894017, 0.326030246350775, 4.01246812215881, -4.09220816270835, 2.28979005092516, -1.11370561957553, 0.380113900397412, 0.439719829189471, 10.6338233809842, -3.62938285525007, -0.968308939316131, -0.706677115193625},
    {-0.100404462764308, 0.57694866124475, -1.50519714998455, 1.04181171702517, 0.0300463272842365, 0.0204847751966049, -0.466402264732031, -0.134626102632261, -0.107182504067225, -0.16665434679713, 0.754231010240759, -3.33204574872114, 0.142867299242776, -0.0204847751966047, 0.23370696078652, 0.4305771624681, -0.509642151511679, 0.166654346797131, -0.0990178145200931, -0.147817722931937, 1.45555613539848, -1.68526479642689, 1.32565568047273, 0.449039378029455, 0.54238488791022, -1.0767420034643, -0.553342302405746, -0.0471131712642592, -4.23986916465108, 2.13513760220477, -0.542384887910221, -5.11980118438331, 0.553342302405745, 0.0471131712642582, 0.868006346840195, -4.92314980708571, 4.8061095999893, 6.2115939766232, -3.92135994492723, -0.172913626527011, 0.268961528376667, -1.52549333465282, 1.48922711022166, -0.802681208649336, 0.506730148813495, 0.616824655578903, 7.90395683340339, -4.98974334922504, -2.58417698023423, 6.19654318784761},
    {0.032751382129728, 0.733560856707953, -3.02009427869259, 4.59614374252045, -2.90000486638576, 0.288623553688338, -0.344732873319207, -0.154120270281249, -0.135945654934047, -0.207912100848699, -0.335749943545728, 1.50942741166772, -6.03594528816658, -0.288623553688337, -0.0383246455489333, -0.0506149010319234, -0.282951433929324, 0.207912100848699, 0.104458549759909, 0.10623639069872, 0.278180235623068, -1.25518741491938, 1.53242816535887, 0.789713279908622, 0.999903900603928, -2.68822416327502, -1.98855701839865, 2.43474709596998, 0.503267349496731, 1.64367522486531, -0.999903900603927, -5.59515404954394, 1.98855701839865, -2.43474709596999, -0.789013179090685, -1.44362886899564, 5.58848627032465, 3.7199494140764, -9.82552056826457, 8.93595015455234, -0.0900630096992914, -0.164785030562313, 0.637905559129746, -0.124739268666916, 0.329474439980089, 0.418897088863371, 1.24029089932574, -3.27598641418135, -2.43338850477393, 8.28337821281897},
    {0.239734464658436, 0.886657647844945, -4.7851000364137, 10.5867211358993, -12.2873339520157, 8.1697622215787, -0.165827532811147, -0.0960840891841836, -0.0916041248827689, -0.138407634889368, -1.26639041080199, 2.55888156794629, -1.80268636101632, -8.1697622215787, -0.0438867308521986, -0.0232241693750327, -0.0134393276185009, 0.138407634889369, -0.186000291050867, -0.143018045912814, -0.190501890245121, -0.606871885294581, 0.959173473074921, 0.631244203477591, 0.825879124087905, -2.41031063411877, -2.17685144999341, 3.83905857194561, 0.231838667440774, 0.0926104328672054, -0.825879124087906, -0.353618948008272, 2.17685144999342, -3.83905857194562, 1.28810637824866, 1.25124354829929, 3.51214052066774, -4.31293078787758, -8.83267191596799, 14.090020313032, 0.0446392972096266, 0.0433618166770592, 0.12171314977666, 0.0391437557623348, 0.0801645027968814, 0.10504345250127, -0.390758266873767, -0.800253873641932, -0.723854636344798, 2.76392958212705},
    {0.0232138437439123, 0.537207852379281, -0.125473334825659, -0.0765656943091292, -0.0811568320805657, -0.15388872742481, -1.53428368653199, 1.26116386310604, -0.0835173539639663, -0.0496222433554875, 0.0111495902450423, 0.389873654755793, -0.454124093010761, 0.15388872742481, 0.136336811707853, -6.42186515765067, -0.467332714291879, 0.0496222433554881, -0.199067870201143, -0.443089749751426, 1.83373928319032, -1.75200335936095, 0.391416674441761, -0.379491222994875, 0.0387398677950158, 0.281720641970858, -0.421048643677781, 0.460320928675598, -1.9930995307227, -2.12349475737312, -0.0387398677950154, 1.5764061723161, 0.421048643677781, -0.460320928675598, 0.22601137130472, -0.560437026223539, 0.448749399499436, -0.638009667464227, 0.324701707017564, 0.535280925091327, 2.76365938982465, -6.85300496601941, 5.48729245101889, 10.5090764758116, -5.34837518126702, 0.550850068255844, 3.26161245651751, -1.65992960023657, 2.50298598036799, -1.85812681428696},
    {-0.138566640473885, 0.595832618747915, -0.68877236123291, -0.0662246334629458, -0.0424502734777076, -0.0578765020203719, -1.43723513893444, 0.844120572468694, 0.0907825760323359, 0.0690671693240353, 0.382088705155293, 0.163849194570805, -0.186028874620443, 0.0578765020203716, 0.797289996155129, -2.08847476667468, 0.397834432404977, -0.0690671693240345, -0.0460474603673536, -0.0487666930498748, 1.28476773881767, -1.64721956367447, 1.80793837795993, -1.24317121942812, -0.201370300695997, 0.203515804402888, -0.210771515542724, 0.196483245909811, -4.47309757068157, -5.4479211549056, 0.201370300695996, 0.891862712743735, 0.210771515542723, -0.196483245909814, 0.14568423381845, -1.91747624331689, 2.07847566557605, -0.332574763747174, 0.234950202639315, 0.22847914809815, 0.303993760228439, -4.00112488549047, 4.33707626804133, 4.23910537942018, -2.99475118521419, -0.488617008437314, 9.0793206013876, -6.41416140866596, 6.69109237433371, -1.09537851714662},
    {-0.0867112697486274, 0.692328341357125, -1.76462388414567, 1.18519576926095, 0.0327246027163563, 0.0204847751966047, -1.20941104184695, 0.287528119487929, 0.104233971684686, 0.103341901603273, 0.268333407534361, -0.158965367263477, 0.0884517559977629, -0.020484775196605, 0.183906264039709, -0.0385649479169801, 0.281735363146926, -0.103341901603272, 0.11896365148585, 0.138918164337415, 0.529306916813754, -1.39280580424552, 3.33569324242818, -0.918158401560055, -0.484294992385489, -2.59387568091367, 1.03882900843692, -0.104206989941394, -0.447402627577328, -2.48170233282908, 0.484294992385489, -7.01102044860074, -1.03882900843692, 0.104206989941392, -0.642864435433686, -1.70989773051129, 3.84905264255628, 1.97507416355439, -3.00130456555186, -0.121176358714119, -0.440596635696458, -1.17190366416841, 2.6380050776721, 0.479152368599923, -0.728115540170872, -0.385969334831612, 5.55877915829136, -8.44706977314222, 3.39986073438914, 9.60489612951441},
    {0.258324173993583, 0.809132783013126, -3.29120129853804, 4.92980584650552, -3.03841463918548, 0.288623553688337, -0.830942854030469, -0.114188356405141, -0.0720343266745775, -0.0876613998747685, -1.34708568607265, 2.86625354516241, -3.2238211552292, -0.288623553688337, -0.340104151364426, -0.0663906027038615, -0.076429919485396, 0.0876613998747685, -0.0291558008725222, -0.0268576215159714, -0.0502357293041729, -0.961207805314045, 3.63555610734631, 0.571536013672383, 0.5056897130461, -6.22971683400606, -1.37361142730362, 5.3852810017071, 2.11375983269294, 0.606411602975419, -0.505689713046099, -6.60985918825749, 1.37361142730363, -5.3852810017071, 0.181755906531425, 0.246782311661741, 4.20974876641752, -0.571882689850408, -7.22417670181752, 6.26223579441468, 0.0458886461235919, 0.0623061246565249, 1.06285223461478, 0.0132464333168218, 0.167332525792181, 0.148464246159974, -0.4217431011497, -5.32757283888956, -1.1779476166478, 12.8395760222636},
    {0.0664733009545186, 0.919863564854675, -4.94227542033297, 10.8701129799616, -12.5055756024883, 8.16976222157871, -0.378832364552647, -0.172470869633392, -0.147264249503351, -0.204926878310822, -0.146914305631318, -0.484156194529895, 2.63141247240989, -8.1697622215787, -0.0112611882291904, 0.00768187415005356, 0.0309872168383891, 0.204926878310822, -0.208716262863432, -0.153008802706332, -0.184953912959859, -0.439657887279572, 2.17833293068937, 1.12314316450795, 1.30452607023257, -5.42592209657333, -3.80557674439069, 8.49139905584443, -0.0970232217536438, -0.236330819574577, -1.30452607023257, 1.14171786514221, 3.8055767443907, -8.49139905584444, 1.35553299979373, 1.20464522191189, 2.52901150425867, -4.08313698886518, -6.30281979656655, 9.87416313007839, 0.103903511614979, 0.0923377511472672, 0.193852290019592, 0.0647851764374573, 0.100003819045881, 0.116277032664963, -0.818246487387684, -1.26306322154805, -0.88681234493338, 4.28420423143111},
    {0.0307253509165792, 0.732933512062509, -0.164508796215635, -0.0953652459992073, -0.0938519633344896, -0.15388872742481, -3.19425102379766, 5.27695557341788, -3.85725462991615, 0.798848493973306, -0.0176781856910608, -0.0308310083721262, -0.196192270366815, 0.15388872742481, -0.343255582932786, 1.70600788325705, -8.0633746630212, -0.798848493973306, 0.105058733828339, 0.107048073102041, 0.281639118042046, -1.25740478795151, 0.774142521471181, -1.47426658907885, 1.54718213082686, 0.51732401209903, -1.41445222820338, 0.730947882573483, 0.250275604184931, -3.08187169411102, -1.54718213082686, 1.08143685910171, 1.41445222820338, -0.730947882573483, -0.0433492528612682, -0.0796869323786037, 0.305223167146569, -0.0782307940814472, 0.204427048452782, 0.290044233701304, -0.84170815493334, -1.54727328379441, 5.9264880454582, 4.32883510670666, -11.3117985633816, 11.9206292929373, 0.635050888322814, -1.65946901397893, 4.55613828318987, -1.59876087120074},
    {0.148294415168218, 0.768918916307805, -0.860731749519541, -0.0793721793555842, -0.0478690594607818, -0.0578765020203709, -2.85197444483158, 3.71701831994326, -1.73003073753795, -0.123511007815444, -0.251910113159571, -0.0422020538189061, -0.0759333334893278, 0.0578765020203709, -0.834686539129945, 1.97633236806257, -2.74429876876708, 0.123511007815443, 0.054705338071427, 0.0500045419232755, 0.102275710642092, -1.12419892211282, 3.44078912805436, -5.15046955391734, 3.40308137253588, 0.364416669357127, -0.774378690190786, 0.311997573023993, 1.82946177288559, -8.17004400482657, -3.40308137253588, 0.5780638432229, 0.774378690190787, -0.311997573023994, -0.100877781561402, -0.144544174396511, 1.35806381863702, -0.0225272530333861, 0.144101486207881, 0.123802392950109, -0.334251472918275, -0.478937011163157, 4.49984946804296, 1.05495669771112, -6.74830738571695, 4.47432950630503, 0.976557881508633, -6.24680878244858, 13.3205135587439, -0.942480512580024},
    {0.288245120659521, 0.826589347339835, -2.06115841063897, 1.34512191896954, 0.0355985137911532, 0.0204847751966062, -2.243152700373, 1.72929155552928, -0.110016224073327, -0.0529350629556762, -0.870895760782947, 0.74081869619401, 0.030061408799877, -0.0204847751966066, -0.947793322172682, 0.952398067000447, -0.0929039539653085, 0.0529350629556752, -0.0692798741367781, -0.0596887415221383, -0.0999179023362548, -0.885947950004185, 6.01150695506047, -5.29287254775863, 0.469894047305341, -4.48056036862217, 5.50503910225613, -0.165471248214082, 3.31080527482653, -4.46960247602635, -0.469894047305341, -3.78363989249273, -5.50503910225613, 0.165471248214082, 0.251691835434776, 0.303949521999141, 2.376412813988, -0.312422963649291, -1.77351765151426, -0.0656599225910297, 0.273915492085994, 0.330787380305278, 2.58624315015441, -0.401651616238653, -2.28003800629108, 0.202920178038638, -1.39625471298327, -7.92605751690372, 9.76247502378498, 8.26420026111491},
    {0.211629929169825, 0.894125141570338, -3.59296158151086, 5.29595271223312, -3.18693460622812, 0.288623553688341, -1.44043934924041, 0.265477095115421, 0.0993247475666122, 0.0905706902337726, -0.795255349482016, 0.973470037069573, -0.206284350599025, -0.288623553688341, -0.318822529019674, 0.0487983959951498, 0.0064291062044855, -0.090570690233773, -0.194809079437774, -0.148108661457024, -0.192782034875823, -0.570055294969543, 6.20170103224062, -1.29736048363422, -0.664776985404357, -10.3757646108589, 2.97272576377869, 8.55133777337991, 1.13995922203089, -0.0839757315184051, 0.664776985404358, -0.671603948363604, -2.97272576377869, -8.55133777337991, 0.990640651132532, 0.942021585899311, 2.45555469396103, -2.15813463360148, -4.11128811570122, 3.39321895682715, 0.397153641216615, 0.377661973104824, 0.984446263938647, -0.108183615777582, -0.206091875332989, -0.105753853771096, -2.52723287237903, -4.81442738189249, 1.38133621515262, 11.0473685592225},
    {-0.287383368069139, 0.956163846795196, -5.11339400277777, 11.1769121298321, -12.7397587969251, 8.16976222157871, -0.622379214652007, -0.201459834843229, -0.150902737088016, -0.191346895929758, 1.81307241877917, -4.86684701705693, 7.38940173345124, -8.16976222157871, 0.220678982979614, 0.087723172990383, 0.0875276341409132, 0.191346895929759, -0.170757728422396, -0.119556563039015, -0.131759001550123, -0.246707185714522, 3.55662213627246, 1.29980313335139, 1.31218488814207, -8.77721557579741, -4.32320098473321, 13.4835715113227, -1.54868676013975, -0.753920673054671, -1.31218488814207, 5.09102040505872, 4.32320098473322, -13.4835715113227, 1.03995793497274, 0.850271555111339, 1.41009209391452, -2.82917645342374, -3.48088865935143, 5.3503570634739, 0.126578981100973, 0.103491211986346, 0.171630038585074, 0.0509948914716731, 0.0627417703811724, 0.0633751029471036, -0.900276524033051, -1.10765885209965, -0.545882460296721, 3.68619517073868},
    {0.136246592488564, 0.909191772905896, -0.198758544842833, -0.111280669107774, -0.104008515538229, -0.153888727424812, -5.00241514142444, 11.3487406799524, -13.6512361757966, 9.65546182492157, -0.0232954564072219, -0.00616373515136334, 0.0101622736935095, 0.153888727424812, -0.586307089086471, 0.62859643470628, 1.33381000156881, -9.65546182492157, -0.206368245594619, -0.153521289707944, -0.191170602922265, -0.494378227169631, 1.1709127123866, -3.01117640477566, 4.94093995929799, 0.743125520653232, -2.60689130666785, 0.94745897883755, 0.0648557912384806, 0.294210513499024, -4.94093995929798, -0.072607948401481, 2.60689130666785, -0.947458978837549, 0.0555578588263913, 0.0507286897117839, 0.115767452711881, 0.0439264739612475, 0.0735179302978892, 0.0938462418447205, 1.39829698611416, 1.27675499797771, 2.91367024641904, -4.47975525280369, -7.49758186185498, 12.3174261742278, -0.462201271649009, -0.773567231976073, 2.71696589127663, -0.670517572251752},
    {0.050820153776186, 0.921992874650236, -1.00949188137988, -0.0904065681741688, -0.0522042791302105, -0.0578765020203706, -4.33565424042011, 7.9807291656372, -7.02038040494668, 2.64740738044188, -0.0110004824460647, 0.00590658279383621, 0.0121469776576465, 0.0578765020203701, -0.0472458364883348, -0.521409434336835, 1.6335136764243, -2.64740738044188, -0.20853655982173, -0.152440957402994, -0.183205852336292, -0.428629658865404, 5.080000318028, -10.5880071387386, 12.6627476236861, 0.515137832901314, -1.45972009203279, 0.404413103840402, -0.331894497016483, 2.46363494135172, -12.6627476236861, -0.119863119482079, 1.45972009203278, -0.404413103840399, 0.274993453708417, 0.243103759388448, 0.502395150729079, 0.0335278093297636, 0.0509721760505694, 0.0400573014725653, 1.18106599532926, 1.04410334020106, 2.15773074137812, -2.95970050829058, -4.49961922300978, 5.38686672852238, -1.88394809688836, -2.86415772412316, 8.12437219738692, -0.395274713419239},
    {-0.127595092614206, 0.942112680825341, -2.31225141453113, 1.47759207750689, 0.0378977438878119, 0.02048477519661, -3.26179396758609, 3.84189325821417, -1.46142822896921, -0.187994765484728, 0.4388553645053, -0.406648411251743, -0.0166529258255803, -0.020484775196611, 0.619074453405155, -1.05732821218667, 0.642176905529871, 0.187994765484727, -0.194834740990709, -0.138772873632381, -0.158276461917952, -0.322633511670092, 8.55953676376873, -11.3961221439565, 5.86326077670042, -6.18283741292549, 11.1338297871503, -0.214484812936193, -2.35567182514293, 5.00765368314269, -5.86326077670042, 2.71684597198954, -11.1338297871503, 0.2144848129362, 0.55693304779857, 0.469596891800321, 0.846866110426937, -0.458990814111809, -0.611952852143339, -0.0212448180622289, 0.78564158042784, 0.662440208381099, 1.194637725372, -1.19342390986138, -1.59114113616612, 0.819251323439348, -2.65888590459365, -3.54497903403215, 6.38846846081384, 3.46599144093596},
    {-0.405296087049496, 0.965098624043576, -3.84258032344358, 5.59493014084545, -3.30575581164892, 0.288623553688349, -1.99682203872871, 0.867784465877601, 0.153959671216836, 0.105852187397972, 1.80583279730041, -3.03701648093236, 2.20785138931024, -0.288623553688348, 0.938412843554319, -0.471047119163682, -0.102826734145335, -0.105852187397973, -0.148401337794703, -0.102816977085585, -0.110960259772189, -0.197623962341602, 8.49249694472013, -4.16382557788168, -0.993408479653091, -13.9505629151506, 9.19790865284456, 11.08429471871, -4.60986152393411, 2.78093985483639, 0.993408479653092, 9.31731545486156, -9.19790865284457, -11.08429471871, 0.663618854313048, 0.532506111944691, 0.840622559885429, -1.17671898743628, -1.38119467247682, 1.09790442233868, 0.344853885167629, 0.276720289645371, 0.436835020361389, -0.182511386611551, -0.214225960102369, -0.0511329370714992, -1.78613175750695, -2.09650366327907, 1.38288572304529, 4.633247460289},
    {-0.729420397946707, 0.985708641666341, -5.25214046484747, 11.4243799830647, -12.9271136018471, 8.16976222157872, -0.828621425313877, -0.181971301432515, -0.121306661194975, -0.142219700439308, 4.03187699731424, -9.20606324283967, 11.1959607479293, -8.16976222157871, 0.636102496984889, 0.146637218987201, 0.10506170666028, 0.142219700439308, -0.0733757036279551, -0.0496710434458389, -0.0511932222823332, -0.0820482743635052, 4.71218718284896, 1.16552492088479, 1.03954239228302, -11.5443936807303, -3.82040188607881, 17.4774853307227, -3.79720328645517, -1.00944198889804, -1.03954239228302, 9.99840973700668, 3.82040188607882, -17.4774853307227, 0.425725525597713, 0.327919433759287, 0.466618508176218, -1.07504448685206, -1.14327225337301, 1.73115285391772, 0.0671660048268503, 0.0517353011421571, 0.0736176223599809, 0.0171236640115632, 0.018210418433749, 0.0162449545346954, -0.443421074880822, -0.471562821512966, -0.156082931986747, 1.5459839437236},
    {0.113928564117326, -0.218256695635771, 0.0660288489812666, 0.0587187272009708, 0.124017235870968, -0.153888727424813, 0.0733887411520791, 0.0637431422621031, 0.133209060673525, -0.164167775300469, -0.89739272592452, 1.11679585815058, -0.667429939275515, 0.153888727424813, -0.997420422902054, 1.2123572948061, -0.716898055757191, 0.164167775300469, -3.8652301180838, 6.2145484466493, -3.57875818583616, 1.33376798878911, -0.027284953362072, -0.038218904944046, 0.040435071237679, -0.039130654826274, 0.0277492861273959, 0.0418410144593131, -0.518943859261561, 0.205684647193058, -0.0404350712376801, 0.21059145844601, -0.0277492861273968, -0.0418410144593126, -0.825569520849958, 2.15318739295371, -0.4962539951605, -0.462808305865583, -0.712706279485965, 0.543412703404547, -0.917591458937757, 2.39319198610029, -0.551568845412557, -0.502409658504781, -0.773690778563423, 0.583688995083666, 0.215054100166451, 0.331174712457182, -0.167465742249011, -0.171460803619736},
    {0.269990545910721, -0.280175037546944, 0.383747306192834, 0.0469015469885949, 0.0451261636041861, -0.0578765020203698, 0.0727845062071844, 0.0560505031716942, 0.0873966756338539, -0.149034415307043, -1.30154270918084, 0.557676967080658, -0.277076277496999, 0.0578765020203699, -0.246860738476914, 0.666461484090052, -0.536618750989813, 0.149034415307043, -1.30205689381536, 2.6581508558724, -2.7326630332463, 1.38675356282548, -0.116383848266151, -0.123904276157098, 0.182364335615599, -0.0196647100365737, 0.0197614744334016, 0.0178594059513633, -1.38384756336564, 0.760776739292475, -0.1823643356156, 0.120742031225681, -0.0197614744334029, -0.0178594059513628, -2.93685457666245, 6.07448098846482, -2.21983100881437, -0.228630971260601, -0.375947542808652, 0.231950113892814, -0.557026737947377, 1.1521334275847, -0.421030457367594, -0.273229387996678, -0.449282599265069, 0.449222075355959, 0.567336345528405, 0.932895066103381, -0.636872463135375, -0.101077321189109},
    {-0.127458660661965, -0.288734078223699, 0.764302484300338, -0.528230785883266, -0.0137224969121001, 0.0204847751966048, 0.0476133273786124, 0.0304504164558675, 0.02938669789572, -0.120198635080442, 0.835345906031968, -2.86220224093332, 0.136739719441226, -0.0204847751966044, 0.0520390799654305, 0.164994643528041, -0.292827817801938, 0.120198635080442, 0.17841566393676, 0.443553780551471, -1.22959549126017, 1.0238187856576, -0.135132934040503, -0.0914862166278273, 0.325606241800377, 0.0988665549658267, -0.246844448755233, -0.00947192686946341, -0.732213640270667, 0.911627065727221, -0.325606241800376, -0.985169468409679, 0.246844448755231, 0.00947192686946396, -2.91405972400498, 4.29275694872361, -2.97834561505093, 1.04933915424618, 2.34109387257041, -0.123017222529127, -0.181535560187132, 0.26742349548098, -0.18554034263789, -0.0604902537000337, -0.134954806283875, 0.263441119906219, 0.268443798632004, 0.598902775679166, -0.820140849099393, 0.886302913443851},
    {0.0599574981188576, -0.0751681159726939, 0.148269706050571, 0.176927006276902, -0.638089254542733, 0.288623553688338, 0.00340259784931624, -0.00173356112842094, -0.0102837650967008, -0.0793175468753283, -0.485316152903586, 1.81264552367903, -5.71927997511735, -0.288623553688339, -0.0111373775675038, -0.0177606114836739, -0.0921747723655393, 0.0793175468753281, 0.0436499453955371, -0.0391178393220895, -0.260237953407903, 0.270916465188291, 0.0118597804655858, 0.050772788681691, 0.343765567681152, -0.118917065309004, -0.581054568323708, 0.489496797176196, 0.12150535085133, 0.455083346915365, -0.34376556768115, -1.06586968120747, 0.581054568323709, -0.489496797176195, -0.136173436017741, 1.26907133398317, -0.795851451112413, -1.85817836648526, -0.131394163470674, 6.35736922966009, -0.00312500410819108, 0.0291235445639028, -0.0182637607375244, 0.0182067500806896, 0.00128742253140562, 0.10245853746224, -0.124557510784489, -0.008807620532431, -0.505856135597055, 1.18478674651648},
    {0.226504621301653, 0.424231227319648, -2.51931486939398, 6.30034913894458, -8.72079246890365, 8.1697622215787, -0.0175527054809668, -0.0130848449412189, -0.0158235923115135, -0.0350775308879063, -1.17098067011925, 2.26640789781616, -1.30337508285042, -8.1697622215787, -0.00815851923718568, -0.00470697659169361, -0.00236493139970897, 0.0350775308879061, -0.228567484145605, -0.122717827016865, -0.123116291437688, -0.176334246021144, 0.114761522384543, 0.102520248984801, 0.201005644744983, -0.343927818097511, -0.498130062975763, 0.771828366999454, 0.041282858292666, 0.015322270136707, -0.201005644744984, -0.0514020887442646, 0.49813006297577, -0.771828366999459, 1.50227820486518, 0.955357742054925, 1.23265959259313, -4.57146810079707, -3.99528893596366, 10.0241675517541, 0.010466753164048, 0.00665621962501722, 0.00858825192908641, 0.00949422800759286, 0.00829759352531967, 0.018188523711223, -0.0832697723903051, -0.0727746082869067, -0.117842519121509, 0.395329906841778},
    {0.240503275451299, -0.28628929987628, 0.0770893893180506, 0.0575512666200253, 0.0856466499156486, -0.15388872742481, 0.426181230376301, 0.0169904654544732, 0.0144499961342108, -0.0187732230698004, -0.198934346621587, 0.639724604203336, -0.539525186253462, 0.153888727424809, -1.09978928821843, 0.188861504297646, -0.0910267577698622, 0.0187732230697998, -1.054402605484, 2.31431049309129, -2.58817254123149, 1.37405067804918, -0.13301648609913, -0.00717794501185476, 0.00635131781341347, -0.134415530451183, 0.00754649641305638, 0.208116353436681, -1.47857595357728, 0.0452169713964435, -0.00635131781341391, 0.846741398215303, -0.00754649641305716, -0.20811635343668, -0.552035891561488, 1.10348754796893, -0.429606699103904, -0.261470863660596, -0.435805007162765, 0.453878536337814, -3.05187701652297, 6.10052414552206, -2.37503907115697, -0.0771922485339442, -0.128659721218175, 0.0765767616356516, 0.604329627201185, 1.00726281247522, -0.0380390263845873, -0.712325867764121},
    {0.00303876266691747, -0.301946133397289, 0.377044237587165, 0.0399716863386936, 0.0287480743053716, -0.0578765020203708, 0.355704950821359, 0.0516705383728242, 0.0387727778774354, -0.0798554220922585, 0.112743231543427, 0.297246090911908, -0.222481451419091, 0.0578765020203708, 0.106362388371802, 0.384243622260384, -0.300062668757961, 0.0798554220922589, -0.104321311187429, 0.943862170363877, -1.7522973977573, 1.21166390931122, -0.493357822210138, -0.0853817020033946, 0.126996446014249, -0.0623120291595039, 0.0213755605582874, 0.0888323213280862, -3.66881404077302, 0.660769301782364, -0.12699644601425, 0.482233019889128, -0.021375560558289, -0.0888323213280848, -1.86005508358466, 3.06792860426878, -1.69766098981471, -0.117223301234508, -0.219994476016094, 0.19373337711372, -1.75478295667772, 2.89429537575836, -1.6015797582738, -0.151532037785539, -0.28438212284767, 0.261289890880525, 1.44684995572367, 2.71532190725949, -0.575387599778966, -0.419920990729627},
    {-0.157996351774294, -0.236643324868562, 0.569818879903482, -0.332539840484566, -0.00503620216509604, 0.0204847751966045, 0.176564788258324, 0.0429718172322194, 0.0179653491069813, -0.166654346797131, 0.685081437210634, -1.02836080694758, 0.107784769793441, -0.0204847751966045, 0.212280187910398, 0.132887934812671, -0.384494298337473, 0.166654346797131, 0.244046323501203, 0.126043410070259, -0.704582974743168, 0.729132917814562, -0.423141073687128, -0.0752655413239928, 0.542384887910222, 0.180477645663648, -0.553342302405749, -0.0471131712642562, -1.3085400394596, 1.61083268285132, -0.542384887910222, -3.86258153525536, 0.553342302405748, 0.0471131712642568, -1.54132956156935, 2.17798675871177, -1.89155751425654, 0.229872535566343, 1.1310281118658, -0.102748567628345, -0.477598298815376, 0.674873691349983, -0.586120368703329, -0.0297048334739944, -0.146154918570896, 0.366528949230492, 0.292501828860533, 1.43917928428619, -1.53556714152733, 3.68210388959171},
    {0.18815737454253, 0.0291979345553123, -0.305128780364226, 0.899029556149417, -1.08698561075244, 0.288623553688338, -0.0348293501727635, -0.0301467242859978, -0.0509554216513404, -0.2079121008487, -1.09311974803885, 2.729017019739, -4.22292603253327, -0.288623553688339, -0.124775678124368, -0.0915108108550289, -0.197961200646619, 0.2079121008487, -0.0675428941563698, -0.0717532042345304, -0.175823134565446, 0.0977639238585024, 0.29975089782069, 0.296001907386663, 0.999903900603927, -1.00760554502059, -1.98855701839865, 2.43474709596999, 0.909898118742991, 1.14996385234334, -0.999903900603926, -3.91453543128951, 1.98855701839866, -2.43474709596999, 0.517838755193456, 0.929742174243801, -0.0493324010341835, -2.41634398843721, -1.2117025874512, 5.30991164328572, 0.0591094268988055, 0.106126717114229, -0.00563111571590297, 0.0810260969745486, 0.0406314381664783, 0.248916622297959, -0.805647906704963, -0.404001109858722, -1.44596575973, 4.9221409763101},
    {0.143182682873855, 0.504539511638758, -2.92484054429374, 7.09932077313889, -9.42860290518038, 8.1697622215787, -0.101360282467521, -0.0644327702181854, -0.070291807919363, -0.138407634889368, -0.617043178048828, 0.7120465294862, 1.05604468581899, -8.1697622215787, -0.0213836172859663, -0.00646246759164174, 0.00787298934490475, 0.138407634889368, -0.222493404421769, -0.121582887132207, -0.119803876761493, -0.183842026197145, 0.643209552328142, 0.484380985658056, 0.825879124087901, -1.84953562229107, -2.17685144999341, 3.83905857194562, 0.0645125279027027, -0.0542527849523353, -0.825879124087901, 0.207156063819439, 2.17685144999342, -3.83905857194562, 1.41704008245725, 0.905593933742574, 1.21924970614274, -4.10666107371777, -3.70470622890731, 8.37255821936139, 0.049107491793314, 0.0313833371546067, 0.0422530708055669, 0.0372716711615512, 0.0336235666482759, 0.0624188185744587, -0.372069908544397, -0.335652171686451, -0.430128200705723, 1.64237955847163},
    {-0.161958917611736, -0.27076488372056, 0.061223546198226, 0.034588491114838, 0.0274465741133886, -0.153888727424809, 0.748639448326537, -0.569729765621708, 0.0282448832286873, -0.0496222433554884, 0.0765730592183335, 0.151585061594869, -0.345520686816804, 0.153888727424809, 0.936332773411779, -2.49685715770195, -0.355570477099232, 0.0496222433554883, 0.239777169008871, 0.268866634225738, -0.973990293071298, 0.898070291168986, -0.176822169357853, 0.128340815064946, 0.0387398677950246, -0.0952756074984328, -0.421048643677782, 0.460320928675596, -0.774928265718658, -1.61566271931327, -0.0387398677950244, 1.19940992284679, 0.421048643677781, -0.460320928675596, -0.201077656620996, 0.284167503731905, -0.220886452527469, -0.0502782957184201, -0.135895256991287, 0.318074112703416, -2.45877077156142, 3.47479060649949, -2.70099205667639, 0.828166847186466, 2.23842007613719, 0.327325593870545, 0.257031082709816, 0.694719352366693, 1.48732190424832, -1.10413431534836},
    {-0.138647660917849, -0.215854064009139, 0.223449718107213, 0.0169677308765799, 0.0039059715603883, -0.0578765020203709, 0.46626404415215, -0.216276179301875, -0.0083531655066301, 0.0690671693240343, 0.241970447233904, 0.0410426412418458, -0.139672629582347, 0.0578765020203711, 0.504910546534908, -0.52314276439262, 0.29869869086601, -0.0690671693240345, 0.225317421290076, 0.0797575058760406, -0.597737904987654, 0.647164702748526, -0.463220560606467, 0.1143877537168, -0.201370300695995, -0.0187260735670991, -0.21077151554272, 0.196483245909813, -1.12046775276591, -4.09036218176068, 0.201370300695996, 0.669620834773751, 0.21077151554272, -0.196483245909813, -0.548551168994651, 0.803571415413603, -0.72044041176007, -0.00376929274972829, -0.0542410793686963, 0.135766658021959, -1.14464090018281, 1.67678196727928, -1.50331565778353, 0.0480446230854973, 0.691374320608998, -0.29034552535938, 0.102902026989879, 1.48078628638249, 3.97597442804388, -0.650894761206651},
    {0.0784651408728718, -0.0617978275970892, 0.0545443252607889, 0.07382608634981, 0.00813907259628335, 0.0204847751966047, 0.0373827589171654, 0.017910185243533, 0.0259244663684316, 0.103341901603272, -0.35819710415923, 0.535979548866735, 0.0638662258776869, -0.0204847751966047, -0.245495675775413, 0.130028469360585, 0.203425857830673, -0.103341901603273, 0.0276227253993508, -0.0443340954746589, -0.244997596742811, 0.245041653542336, 0.207781012841067, -0.228359010190472, -0.48429499238549, -0.645133652367756, 1.03882900843693, -0.10420698994139, 1.50849623800912, -1.7919029414595, 0.484294992385491, -5.06227842005483, -1.03882900843693, 0.104206989941389, -0.0186137897061487, 0.740360722520859, -0.418094153916271, -0.526185351617165, -0.0836202835993776, -0.0720052984739706, -0.0127572356939561, 0.507417156036062, -0.28654700348386, -0.127652400199602, -0.020286254404515, -0.229350324199104, -1.48093080246866, -0.23534644838153, 2.02026195164997, 5.70741207242258},
    {0.301798607677329, 0.236409154506209, -1.15104668812143, 2.14074232727352, -1.76786658528815, 0.288623553688338, -0.290609395595098, -0.0495856947412141, -0.0419123438517448, -0.0876613998747682, -1.4000385651718, 2.49182453126872, -1.95327310133188, -0.288623553688338, -0.353473378128932, -0.0577177593874797, -0.0463079366625637, 0.0876613998747681, -0.19690146010785, -0.109334068422316, -0.128667221027399, -0.103305012625974, 1.57872116762789, 0.332541651106268, 0.505689713046098, -3.62468903506824, -1.37361142730363, 5.3852810017071, 1.83763178006511, 0.367417240409298, -0.505689713046096, -4.00483138931967, 1.37361142730363, -5.3852810017071, 1.07372259576832, 0.761862180902006, 0.715500476622902, -2.61208818129502, -2.02047867724722, 3.72113968662003, 0.271087070414396, 0.192350414803796, 0.180645288505838, 0.0605034083479854, 0.0468000457807086, 0.0882202805143087, -1.92632193561247, -1.49003101208053, -0.699958891515568, 7.62952042438791},
    {-0.0586054310955196, 0.63688067637285, -3.58097023983998, 8.36005347555909, -10.5022018617589, 8.16976222157871, -0.274486407165838, -0.132644959235412, -0.123672746018734, -0.204926878310821, 0.575471048921228, -2.18852029690419, 4.63478621313925, -8.1697622215787, 0.044110665564605, 0.0347242019943284, 0.0545787203230063, 0.204926878310821, -0.191317334359864, -0.109362603663137, -0.107111056838625, -0.170484250415705, 1.67532571388221, 0.943217378252485, 1.30452607023257, -4.55669782469293, -3.80557674439069, 8.49139905584443, -0.438571875105341, -0.416256605830049, -1.30452607023257, 2.01094213702262, 3.8055767443907, -8.49139905584444, 1.16912731473238, 0.771605708869946, 1.06476616731642, -3.18446828851913, -2.98706489013577, 5.86741564861967, 0.0896152535896704, 0.0591446632033025, 0.0816158248082608, 0.0505264311468873, 0.0473943260941962, 0.0690940256957285, -0.638156397491441, -0.598597441285432, -0.526960772422438, 2.54575568767031},
    {0.186613680044609, 0.0276979469831487, -0.0162868154822906, -0.018461977373623, -0.0350047482548506, -0.15388872742481, -0.316239485215844, 1.0215779698073, -1.43867237804988, 0.798848493973307, -0.0592422377411612, -0.0565262386342738, -0.137345055287175, 0.15388872742481, -1.15030067029804, 3.12783181000749, -5.64479241115493, -0.798848493973308, -0.0661747340708308, -0.071380534462859, -0.176592144722132, 0.0998357862280644, 0.149868031751084, -0.549869485706124, 1.54718213082687, 0.192950644465233, -1.41445222820338, 0.730947882573478, 0.458860714373717, -2.15747459073827, -1.54718213082687, 0.757063491467877, 1.41445222820338, -0.730947882573478, 0.0278452172868567, 0.0508382642087821, -0.00315442827218665, 0.0500629350684283, 0.0249252809394695, 0.172349803542026, 0.540667829760455, 0.987121547495586, -0.0612492217421561, -2.77019035040479, -1.37921942941, 7.08346478920482, -0.406393821774847, -0.202334924349955, 2.70734407644439, -0.950014135933112},
    {0.26262833189321, 0.119570063807636, -0.18629638570859, -0.0244646644642514, -0.022750731982457, -0.0578765020203708, -0.61728004282631, 1.14568614271631, -0.822231856537404, -0.123511007815444, -0.371819690231103, -0.0477770585860817, -0.0508150060110037, 0.057876502020371, -1.2319985352187, 2.23741118713505, -1.83649988776653, 0.123511007815443, -0.137982648591284, -0.0911691460845495, -0.143997472754264, -0.00904912827074769, 1.0605447928169, -2.44786410522601, 3.40308137253589, 0.173196341600902, -0.774378690190786, 0.311997573023996, 2.07113859148238, -5.46743855613524, -3.40308137253589, 0.386843515466682, 0.774378690190787, -0.311997573023996, 0.232322287801947, 0.223219781902334, 0.102574006235411, 0.0435302713922993, 0.0287114516580373, 0.0735657379934605, 0.769783649953434, 0.739623133364952, 0.339871794726618, -2.03853311766078, -1.34456421219058, 2.65873174430393, -1.88704009092241, -1.24464329337687, 7.91530266136124, -0.560039857067583},
    {0.297257923557779, 0.291832089112708, -0.841280923622999, 0.656732008696609, 0.0222767402895098, 0.0204847751966059, -0.915563581070026, 0.844296045487715, -0.0688456508519486, -0.052935062955676, -0.807857834631561, 0.582773602692336, 0.016739635298231, -0.0204847751966061, -0.879189330581097, 0.749214963869798, -0.0517333807439302, 0.0529350629556752, -0.213649244622453, -0.115422748860114, -0.126150246023436, -0.133867773164483, 2.93501204776657, -3.31215926101939, 0.469894047305339, -2.80383277427921, 5.50503910225613, -0.165471248214078, 2.60448329359947, -2.48888918928711, -0.469894047305338, -2.10691229814977, -5.50503910225613, 0.165471248214079, 0.689026709302335, 0.460043482825832, 0.500068566126392, -0.684722686918872, -0.554782924470071, -0.0390163755877407, 0.749865762681246, 0.500663983642992, 0.544223165326883, -0.880280920018925, -0.713230089338587, 0.120579031595881, -3.06010565782306, -2.47938968354298, 5.80104845030649, 4.91074507242899},
    {0.105970427076989, 0.533380965289455, -2.28752304127764, 3.66947089586476, -2.49848276832552, 0.288623553688342, -0.917081389878049, 0.183944330128657, 0.0778682969454375, 0.0905706902337729, -0.287413545533684, 0.0559394161257741, 0.482167487303576, -0.288623553688341, -0.11522577436448, 0.00280414771476983, -0.0150273444166888, -0.0905706902337732, -0.217831885088019, -0.120054740517504, -0.117927627713663, -0.183537139047258, 4.29704770401275, -1.01710050979142, -0.66477698540436, -8.13435865228324, 2.9727257637787, 8.55133777337992, 0.0655065393482051, 0.196284242324386, 0.664776985404361, 1.56980201021201, -2.9727257637787, -8.55133777337992, 1.02378629159574, 0.657643433361127, 0.893506861854458, -1.94955520808224, -1.7758551039083, 2.01631528102194, 0.410441922679089, 0.263653105576651, 0.358212135986787, -0.0977278842035871, -0.0890205936398393, -0.0628409525287474, -2.28298083524154, -2.07957340811942, 0.820816267467038, 6.56455664207123},
    {-0.397585858525557, 0.793676417832501, -4.34154822751179, 9.77838145744784, -11.6542222604831, 8.16976222157871, -0.528433634244638, -0.176251820768419, -0.138044531750746, -0.191346895929758, 2.38339251504386, -6.02882067961085, 8.47493826989333, -8.16976222157871, 0.290095768273425, 0.10866733175544, 0.100385839478184, 0.191346895929759, -0.125455420602997, -0.0764969692852927, -0.0754625796527919, -0.118675589765863, 3.11159357293771, 1.18904877654399, 1.31218488814207, -8.02932165193, -4.32320098473321, 13.4835715113227, -1.91844015911069, -0.864675029862071, -1.31218488814207, 5.83891432892613, 4.32320098473322, -13.4835715113227, 0.738475413403457, 0.51412208472179, 0.705558214342672, -1.90375980070016, -1.84580097713683, 3.17928399058972, 0.0898838907356157, 0.0625766172429938, 0.0858773579926041, 0.0343145880163686, 0.033269900996609, 0.0376586922725624, -0.605798289425931, -0.587355124401096, -0.324373746681923, 2.19040732300386},
    {0.0167231769272602, 0.593196374838998, -0.136760509442188, -0.0820817973762034, -0.0849680060691138, -0.153888727424812, -3.44202984442008, 8.37095104150297, -11.1521476124679, 9.65546182492157, 0.00592046743456753, 0.012294097262785, 0.0292027831626261, 0.153888727424812, 0.149008114153968, -1.25378938541602, 3.83289856489746, -9.65546182492158, -0.204301268205592, -0.114893726140283, -0.112528398046355, -0.178196159374027, 0.863677589054117, -2.45992987893488, 4.94093995929798, 0.607083885605219, -2.60689130666785, 0.947458978837547, -0.129360426098388, 0.84545703933982, -4.94093995929798, -0.208649583449513, 2.60689130666784, -0.947458978837546, 0.0513623519749208, 0.0334699672899939, 0.0460077227427061, 0.0362096808645223, 0.0335780192488951, 0.0557652229064882, 1.29270320137226, 0.842382254744544, 1.15793627414931, -3.69277325100851, -3.42438840507845, 7.31924904757045, -0.381003960307747, -0.35331320264798, 1.61447283959506, -0.398434302155707},
    {-0.0748919790983819, 0.645727750172866, -0.738792277826606, -0.0700963669874178, -0.0440770338297802, -0.0578765020203707, -3.17302984920508, 6.18782608077739, -5.9274364048768, 2.64740738044188, 0.135886497249019, 0.0198848675421218, 0.0202742229580763, 0.0578765020203701, 0.583617242377619, -1.75535633697017, 2.72645767649418, -2.64740738044188, -0.188384579678038, -0.108056090115309, -0.105848120321839, -0.168546980959297, 3.93875770068703, -8.93964932798132, 12.6627476236861, 0.434940354815682, -1.45972009203279, 0.404413103840402, -1.11734285990508, 4.11199275210903, -12.6627476236861, -0.200060597567708, 1.45972009203278, -0.404413103840399, 0.234100920715793, 0.154927489841719, 0.213877370020074, 0.0258814410199424, 0.0243300584253529, 0.0238028108717053, 1.0054371593364, 0.66539616680396, 0.918579280687101, -2.28470978788373, -2.14775995592349, 3.20097872838263, -1.45429398845214, -1.36712085232981, 4.82765657587229, -0.234879757247978},
    {-0.25046885665096, 0.731477026246286, -1.85165020848897, 1.23254689245623, 0.033587372459696, 0.0204847751966095, -2.61204358754974, 3.20475026135102, -1.29520993109275, -0.187994765484727, 0.741597708025434, -0.596171784672719, -0.0209632972536968, -0.0204847751966106, 1.0461401018987, -1.55010871751303, 0.808395203406333, 0.187994765484728, -0.154976188373899, -0.0920387320291988, -0.0904610014067849, -0.143532247785442, 7.14001556968877, -10.0999626832235, 5.86326077670042, -5.47962073046942, 11.1338297871503, -0.214484812936192, -3.45356095644329, 6.3038131438757, -5.86326077670042, 3.42006265444561, -11.1338297871503, 0.214484812936198, 0.423664248372092, 0.288508666135035, 0.397879585956409, -0.324989252320517, -0.311385855462993, -0.0126240752059966, 0.59764499696597, 0.40698680986056, 0.561271678824511, -0.845005896071819, -0.809635647766178, 0.486814727686421, -1.88262883607289, -1.8038257771726, 3.79614953934783, 2.05955807602381},
    {-0.502239635580984, 0.834219465627176, -3.38060476427583, 5.03884374042011, -3.08300145155091, 0.288623553688348, -1.75675341289619, 0.781534391644865, 0.14358528484445, 0.105852187397972, 2.17143575456295, -3.51888276364892, 2.43060574940825, -0.288623553688348, 1.12840081544712, -0.545785509857587, -0.11320112051772, -0.105852187397973, -0.104002991877265, -0.0645327311686799, -0.0638287536487634, -0.0996153533514823, 7.64841812019008, -3.88325122363193, -0.993408479653092, -13.0105210934828, 9.19790865284456, 11.08429471871, -5.34128226212328, 3.06151420908614, 0.993408479653092, 10.2573572765294, -9.19790865284457, -11.08429471871, 0.452584923045275, 0.319740661860363, 0.436843424807242, -0.769072822428493, -0.750888154342712, 0.652395702142667, 0.235188720251204, 0.166155329632336, 0.227008547565529, -0.119284679456477, -0.116464202330801, -0.0303841643267281, -1.16736910565866, -1.13976675240814, 0.821737014545791, 2.7531638169534},
    {-0.778443022056055, 0.930688141770991, -4.99337808590904, 10.961922045512, -12.5758791607061, 8.16976222157872, -0.787796917155983, -0.174605118508012, -0.118010714500006, -0.142219700439308, 4.26837451395493, -9.64375602134095, 11.5471951890703, -8.16976222157871, 0.673414315020528, 0.153608934270618, 0.108357653355249, 0.142219700439308, -0.0464943733325561, -0.0300813729300064, -0.0299465576581549, -0.0457228157942172, 4.52143824337274, 1.1338571792039, 1.03954239228302, -11.2307282494787, -3.82040188607881, 17.4774853307227, -3.97773739893538, -1.04110973057893, -1.03954239228302, 10.3120751682583, 3.82040188607882, -17.4774853307227, 0.266741867657277, 0.19515657865651, 0.263105125640317, -0.66211686740229, -0.656049156768722, 1.02868397163585, 0.0420834187600956, 0.0307895273266429, 0.0415096560487177, 0.0105464163692319, 0.010449767868161, 0.0096530611447582, -0.273101789396623, -0.270599055040729, -0.0927474486249693, 0.918653081220328},
    {-0.105791126909819, 0.105791126909818, -0.0685178366798086, -0.250681372099603, 0.395723587573243, -0.153888727424815, -0.0761551633562687, -0.272131551992181, 0.425053558215357, -0.16416777530047, 0.0685178366798071, 0.250681372099602, -0.395723587573242, 0.153888727424815, 0.0761551633562701, 0.272131551992179, -0.425053558215356, 0.16416777530047, 0.780580709826277, -3.88416718961364, 3.88416718961364, -0.780580709826278, 0.116484635695008, -0.121951776068556, 0.0404350712376738, -0.124861056636133, 0.0277492861274045, 0.0418410144593038, -0.116484635695005, 0.121951776068554, -0.0404350712376746, 0.124861056636132, -0.0277492861274051, -0.0418410144593028, -1.26511501111013, 0, 1.26511501111014, 0.960020566935791, -0.96002056693579, 0, -1.40613079752927, 0, 1.40613079752926, 1.04216713286877, -1.04216713286877, 0, -0.446094757909592, 0.44609475790959, 0, 0},
    {-0.166507481624836, 0.166507481624835, -0.355375918844868, -0.091294233017434, 0.161101220550595, -0.0578765020203704, -0.0674033676683349, -0.10910274875466, 0.312007713311833, -0.149034415307043, 0.355375918844868, 0.0912942330174338, -0.161101220550595, 0.0578765020203702, 0.0674033676683333, 0.10910274875466, -0.312007713311832, 0.149034415307043, 0.819459829867038, -1.5882368632347, 1.5882368632347, -0.819459829867037, 0.226542083263474, -0.442340507724783, 0.182364335615597, -0.0702033706311268, 0.0197614744334011, 0.0178594059513653, -0.226542083263477, 0.44234050772479, -0.182364335615597, 0.0702033706311268, -0.0197614744334017, -0.0178594059513648, -2.75455363698745, 0, 2.75455363698745, 0.565938716859822, -0.565938716859824, 0, -0.52245012031066, 0, 0.522450120310654, 0.676334830747747, -0.676334830747747, 0, -1.40434868314629, 1.40434868314629, 0, 0},
    {-0.0046070021046874, 0.00460700210468684, 0.0718887651732712, -0.271878352533979, -0.0752311081766624, 0.0204847751966039, 0.00447841447770747, 0.0156727121577206, 0.161107257848827, -0.120198635080441, -0.0718887651732669, 0.271878352533971, 0.0752311081766612, -0.0204847751966043, -0.00447841447770947, -0.0156727121577204, -0.161107257848828, 0.120198635080441, 0.0625254139861795, -0.192882734206211, 0.19288273420621, -0.0625254139861773, -0.0695524010751921, -0.501556641177518, 0.325606241800375, 0.542018011687761, -0.246844448755231, -0.00947192686946186, 0.0695524010751853, 0.501556641177526, -0.325606241800375, -0.542018011687761, 0.24684444875523, 0.0094719268694613, 0.246963897601136, 0, -0.246963897601144, -4.96246959933911, 4.96246959933912, 0, 0.0153849727676139, 0, -0.0153849727676139, 0.286066753373337, -0.286066753373337, 0, -1.26950775109439, 1.2695077510944, 0, 0},
    {0.290022490838601, -0.290022490838602, 1.41899947092762, -2.74129544088452, 2.54059536028731, 0.288623553688338, 0.0325642012557354, 0.0268596825201096, 0.0409455036344194, -0.0793175468753282, -1.41899947092761, 2.74129544088452, -2.54059536028731, -0.28862355368834, -0.0325642012557362, -0.0268596825201113, -0.0409455036344185, 0.0793175468753278, -0.445947360880905, -0.00479605896763807, 0.00479605896763524, 0.445947360880908, -0.183754661339309, -0.202155279116832, 0.343765567681149, 0.473476307949247, -0.58105456832371, 0.489496797176202, 0.183754661339307, 0.202155279116832, -0.343765567681146, -0.473476307949248, 0.581054568323709, -0.489496797176201, 2.5342520904484, 0, -2.53425209044841, -6.63370064805765, 6.63370064805766, 0, 0.0581578054093145, 0, -0.0581578054093127, 0.064998135802096, -0.0649981358020938, 0, -0.444670573565339, 0.44467057356534, 0, 0},
    {0.00324591462099669, -0.00324591462099744, -0.223681572845258, 1.37517423910197, -3.70870869302661, 8.1697622215787, -0.00155844623368379, -0.00285602294237797, -0.00672933045589987, -0.0350775308879049, 0.223681572845268, -1.37517423910198, 3.70870869302661, -8.1697622215787, 0.00155844623369772, 0.0028560229423685, 0.00672933045590519, 0.035077530887905, -0.188903828583351, -0.0219852273761958, 0.0219852273761908, 0.188903828583357, 0.0250489434383911, 0.0435989894240532, 0.201005644744976, -0.146262864676622, -0.498130062975761, 0.771828366999466, -0.0250489434383909, -0.0435989894240448, -0.201005644744977, 0.146262864676634, 0.498130062975765, -0.771828366999467, 1.03578035972022, 0, -1.03578035972024, -2.2134787268034, 2.21347872680341, 0, 0.00721654439386805, 0, -0.00721654439388363, 0.00459705093830921, -0.00459705093829955, 0, -0.0403187478743311, 0.0403187478743314, 0, -1.72533883297915e-14},
    {-0.164081106760815, 0.164081106760815, -0.0665458915788056, -0.0968987680588136, 0.312585918084555, -0.153888727424811, -0.36789252321787, -0.0286067582518039, 0.0527383769520375, -0.0187732230698003, 0.0665458915788039, 0.096898768058816, -0.312585918084557, 0.153888727424811, 0.36789252321787, 0.0286067582518039, -0.0527383769520355, 0.0187732230698001, 0.770859725525681, -1.3666306125924, 1.36663061259241, -0.770859725525682, 0.2239591653059, -0.026197458204149, 0.00635131781341047, -0.490578464333248, 0.00754649641305838, 0.208116353436683, -0.223959165305898, 0.0261974582041505, -0.00635131781341047, 0.490578464333247, -0.00754649641305905, -0.208116353436682, -0.470330115326035, 0, 0.470330115326041, 0.669730773949605, -0.669730773949608, 0, -2.60017453771347, 0, 2.60017453771347, 0.197720019851459, -0.197720019851459, 0, -1.54792829793654, 1.54792829793654, 0, 0},
    {-0.0960766222659433, 0.0960766222659434, -0.139181614087522, -0.0141621012778636, 0.125614762862232, -0.0578765020203709, -0.131304457829817, -0.0183070434236159, 0.169417723317698, -0.0798554220922589, 0.139181614087521, 0.0141621012778642, -0.125614762862232, 0.057876502020371, 0.131304457829817, 0.0183070434236173, -0.169417723317698, 0.0798554220922589, 0.370355140329414, -0.494118116185395, 0.494118116185395, -0.370355140329414, 0.17479831561678, -0.373075501892876, 0.12699644601425, -0.272272524524317, 0.0213755605582874, 0.0888323213280853, -0.174798315616782, 0.373075501892879, -0.12699644601425, 0.272272524524317, -0.0213755605582884, -0.0888323213280837, -0.623861281103758, 0, 0.623861281103756, 0.394810890417023, -0.394810890417024, 0, -0.58855307731115, 0, 0.588553077311147, 0.510363538091555, -0.510363538091556, 0, -4.87302535676189, 4.87302535676189, 0, 0},
    {0.13457946630335, -0.13457946630335, 0.527538352011231, -0.655871713874957, -0.0564104859792674, 0.0204847751966045, 0.163463691193919, 0.0847537527393672, 0.201229823722227, -0.166654346797131, -0.527538352011232, 0.655871713874955, 0.056410485979268, -0.0204847751966046, -0.163463691193921, -0.0847537527393658, -0.201229823722227, 0.166654346797131, -0.252506609718203, -0.0310767686897993, 0.0310767686897997, 0.252506609718202, -0.834565448776497, -0.843049112087655, 0.542384887910224, 2.0215295904595, -0.55334230240575, -0.0471131712642567, 0.834565448776497, 0.843049112087657, -0.542384887910223, -2.02152959045951, 0.55334230240575, 0.0471131712642568, 1.34545323767321, 0, -1.34545323767321, -3.46192437946911, 3.46192437946911, 0, 0.41690381698389, 0, -0.416903816983888, 0.447360477137227, -0.447360477137229, 0, -4.40513352270102, 4.40513352270102, 0, 0},
    {0.299790343915579, -0.29979034391558, 1.34163924975309, -2.25682373763315, 1.56797021089042, 0.288623553688338, 0.153143283237312, 0.0756769813797041, 0.0735028894976395, -0.2079121008487, -1.34163924975309, 2.25682373763315, -1.56797021089041, -0.28862355368834, -0.153143283237313, -0.0756769813797044, -0.0735028894976398, 0.2079121008487, -0.427176739440945, -0.0159481631438734, 0.0159481631438727, 0.427176739440946, -0.752461292236036, -0.426980972478333, 0.999903900603926, 1.45346494313446, -1.98855701839866, 2.43474709597, 0.752461292236033, 0.426980972478332, -0.999903900603924, -1.45346494313446, 1.98855701839866, -2.43474709596999, 2.17887310994537, 0, -2.17887310994537, -4.62781071800808, 4.62781071800809, 0, 0.248710509830371, 0, -0.24871050983037, 0.155182143689603, -0.155182143689602, 0, -1.54298644374778, 1.54298644374778, 0, 0},
    {-0.103138285400156, 0.103138285400155, -0.823425571056489, 2.74590631055143, -5.24232379549968, 8.1697622215787, -0.0285357944165857, -0.0249215884170003, -0.0390823986321319, -0.138407634889366, 0.823425571056501, -2.74590631055145, 5.24232379549969, -8.1697622215787, 0.0285357944166011, 0.0249215884169935, 0.0390823986321361, 0.138407634889367, -0.142762742683884, -0.0182789893821175, 0.0182789893821117, 0.14276274268389, 0.248783401283617, 0.269316885305202, 0.825879124087896, -1.02834584305526, -2.17685144999341, 3.83905857194562, -0.248783401283623, -0.269316885305193, -0.825879124087896, 1.02834584305526, 2.17685144999342, -3.83905857194563, 0.759841412289172, 0, -0.759841412289196, -1.54416985622994, 1.54416985622995, -1.15921285928425e-14, 0.026332286842229, 0, -0.0263322868422473, 0.0140147409454655, -0.0140147409454587, 0, -0.139904201216263, 0.139904201216269, -1.07274960038762e-14, -1.48077191099279e-14},
    {0.0564307431585096, -0.0564307431585096, 0.0254222755673794, 0.0368690228656186, 0.186483630465099, -0.153888727424811, 0.310862724193501, -0.60729390265075, 0.191907680163962, -0.0496222433554895, -0.025422275567381, -0.0368690228656151, -0.186483630465099, 0.153888727424811, -0.310862724193499, 0.607293902650749, -0.19190768016396, 0.0496222433554886, -0.0929077844791425, -0.0974266666747133, 0.0974266666747105, 0.0929077844791448, -0.188480630263931, 0.872001767189103, 0.0387398677950262, -0.647342765172606, -0.421048643677781, 0.460320928675599, 0.188480630263934, -0.872001767189108, -0.0387398677950252, 0.647342765172606, 0.421048643677781, -0.460320928675598, 0.0760984621081029, 0, -0.0760984621080987, 0.328910404953335, -0.328910404953338, 0, 0.930529416029721, 0, -0.930529416029729, -5.41769941054797, 5.41769941054797, 0, -1.68144517016145, 1.68144517016145, 0, 0},
    {0.169880004840815, -0.169880004840814, 0.277936485181336, 0.0441822278518115, 0.0717893005713683, -0.0578765020203708, 0.579959512573225, -0.563160949589567, -0.15352592818632, 0.0690671693240332, -0.277936485181336, -0.0441822278518107, -0.0717893005713683, 0.0578765020203712, -0.579959512573224, 0.563160949589568, 0.15352592818632, -0.0690671693240338, -0.311928249721622, -0.0146224840668081, 0.0146224840668068, 0.311928249721623, -1.20617874618746, 2.10237496773874, -0.201370300695994, -0.344173454170427, -0.21077151554272, 0.196483245909813, 1.20617874618746, -2.10237496773874, 0.201370300695996, 0.344173454170428, 0.21077151554272, -0.196483245909812, 0.660338320872071, 0, -0.660338320872069, 0.193894942412821, -0.193894942412821, 0, 1.37790290632956, 0, -1.37790290632956, -2.47144757516657, 2.47144757516657, 0, -5.29334915649784, 5.29334915649784, 0, 0},
    {0.294232960746207, -0.294232960746207, 0.873832507491196, -0.724042594224889, -0.0278635766407012, 0.0204847751966051, 0.598894015194808, -0.17565250479797, -0.0887506957311217, 0.103341901603272, -0.873832507491195, 0.724042594224888, 0.0278635766407015, -0.020484775196605, -0.598894015194809, 0.175652504797972, 0.0887506957311197, -0.103341901603272, -0.446688097648323, -0.00623973119156477, 0.00623973119156443, 0.446688097648323, -2.03779329240447, 0.781771965634515, -0.48429499238549, 2.20857238384354, 1.03882900843693, -0.104206989941388, 2.03779329240447, -0.781771965634516, 0.484294992385492, -2.20857238384353, -1.03882900843693, 0.104206989941388, 1.53466376766743, 0, -1.53466376766743, -1.70018012290817, 1.70018012290817, 0, 1.05180447959199, 0, -1.05180447959199, -0.412463161115881, 0.412463161115878, 0, -4.7850992165049, 4.7850992165049, 0, 0},
    {0.197406415132326, -0.197406415132326, 0.723022422627702, -0.834525081479549, 0.0927032580218602, 0.288623553688339, 0.182544384524021, 0.0193299798004365, 0.00219779640540869, -0.0876613998747682, -0.723022422627704, 0.83452508147955, -0.0927032580218612, -0.28862355368834, -0.182544384524022, -0.0193299798004345, -0.00219779640541066, 0.0876613998747683, -0.309428147286896, -0.0260020957934179, 0.0260020957934172, 0.309428147286897, -0.615432504072603, -0.0174377946515135, 0.505689713046096, 0.190071177125715, -1.37361142730363, 5.38528100170711, 0.6154325040726, 0.0174377946515116, -0.505689713046094, -0.190071177125715, 1.37361142730363, -5.38528100170711, 1.37616402777775, 0, -1.37616402777775, -2.27275669046975, 2.27275669046975, 0, 0.347445677468483, 0, -0.347445677468484, 0.0526435237155459, -0.0526435237155479, 0, -1.67607705532795, 1.67607705532795, 0, 0},
    {-0.30474418230749, 0.30474418230749, -1.90385142678791, 5.05440252646411, -7.56849403744909, 8.16976222157871, -0.145932890506209, -0.0801957809291829, -0.0891257331708688, -0.20492687831082, 1.90385142678792, -5.05440252646412, 7.56849403744909, -8.1697622215787, 0.145932890506226, 0.0801957809291806, 0.0891257331708721, 0.204926878310821, -0.0786532648221107, -0.0114099619956309, 0.0114099619956223, 0.0786532648221194, 1.01288473161714, 0.679736992041273, 1.30452607023257, -3.28381998085779, -3.80557674439069, 8.49139905584444, -1.01288473161715, -0.679736992041263, -1.30452607023257, 3.28381998085778, 3.80557674439069, -8.49139905584444, 0.400919008893344, 1.74646664375587e-14, -0.400919008893374, -0.758354778494244, 0.758354778494255, 0, 0.0307310061086986, 0, -0.0307310061087216, 0.0120324515834115, -0.0120324515834095, 0, -0.151971666732894, 0.151971666732906, -1.2156763771739e-14, 0},
    {0.299991988790645, -0.299991988790644, 0.073272694948822, 0.0470325451594928, 0.0511701535161641, -0.15388872742481, 1.42272867008211, -2.6025062769043, 2.10306001655252, 0.79884849397331, -0.0732726949488236, -0.0470325451594912, -0.0511701535161636, 0.153888727424811, -1.42272867008211, 2.60250627690431, -2.10306001655252, -0.798848493973309, -0.427758123512845, -0.0158062160797494, 0.0158062160797494, 0.427758123512843, -0.381794150683429, 0.803802552516068, 1.54718213082688, -0.282056423501301, -1.41445222820337, 0.730947882573478, 0.381794150683429, -0.80380255251607, -1.54718213082687, 0.282056423501305, 1.41445222820337, -0.730947882573477, 0.119089790418294, 0, -0.119089790418291, 0.0965700706519827, -0.0965700706519838, 0, 2.31235468083333, 0, -2.31235468083333, -5.34362353091002, 5.34362353091001, 0, -0.783922876828781, 0.783922876828781, 0, 0},
    {0.272170305024623, -0.272170305024622, 0.309548329113101, 0.0281626337102488, 0.014032137014274, -0.0578765020203709, 1.02566673596469, -1.31886293520906, 0.507134015614558, -0.123511007815443, -0.309548329113103, -0.0281626337102476, -0.0140321370142741, 0.0578765020203712, -1.02566673596469, 1.31886293520907, -0.507134015614563, 0.123511007815444, -0.382504982925767, -0.0226770232778285, 0.0226770232778272, 0.382504982925767, -1.22085199970994, 1.50978722545461, 3.40308137253589, -0.106823586932896, -0.774378690190787, 0.311997573023998, 1.22085199970994, -1.50978722545462, -3.40308137253589, 0.106823586932892, 0.774378690190788, -0.311997573023997, 0.498447494486958, 0, -0.498447494486957, 0.0569287198151308, -0.0569287198151308, 0, 1.6515709071504, 0, -1.6515709071504, -2.66598569173419, 2.66598569173418, 0, -2.46786369990437, 2.46786369990436, 0, 0},
    {0.148805352758508, -0.148805352758507, 0.299004451343531, -0.107758535600081, 0.00276855249564056, 0.0204847751966054, 0.325405674300823, -0.138534598999154, -0.00855613505400976, -0.0529350629556759, -0.299004451343533, 0.107758535600082, -0.00276855249564001, -0.0204847751966056, -0.325405674300826, 0.138534598999159, 0.00855613505400887, 0.052935062955676, -0.272996936236546, -0.0257798275755134, 0.0257798275755121, 0.272996936236548, -0.481585480908131, -0.411635035866146, 0.469894047305337, -0.348460238064721, 5.50503910225614, -0.165471248214074, 0.481585480908135, 0.411635035866138, -0.469894047305336, 0.348460238064715, -5.50503910225614, 0.165471248214076, 0.725911063266471, 0, -0.725911063266472, -0.499183097031053, 0.499183097031052, 0, 0.790006897767759, 0, -0.790006897767761, -0.641750834764518, 0.641750834764513, 0, -2.23090756111511, 2.2309075611151, 0, 0},
    {-0.144387432414536, 0.144387432414536, -0.782380615353444, 1.61328408499109, -1.49032512781455, 0.288623553688342, -0.313660972674284, 0.0808711851769519, 0.0464478206810633, 0.0905706902337731, 0.782380615353444, -1.61328408499109, 1.49032512781455, -0.288623553688342, 0.313660972674284, -0.0808711851769454, -0.0464478206810627, -0.0905706902337731, -0.127426409072684, -0.0168156317034939, 0.0168156317034922, 0.127426409072686, 1.88919843488704, -0.606692376057907, -0.664776985404362, -4.85208033124763, 2.9727257637787, 8.55133777337992, -1.88919843488704, 0.6066923760579, 0.664776985404364, 4.85208033124762, -2.9727257637787, -8.55133777337992, 0.500487979907702, 0, -0.500487979907708, -0.667295016722192, 0.66729501672219, 0, 0.200648563511148, 0, -0.200648563511152, -0.0334503633718622, 0.0334503633718555, 0, -0.781420155896763, 0.781420155896763, 0, 0},
    {-0.581370126327111, 0.581370126327112, -3.30747149106532, 7.839041699661, -10.0645802651882, 8.16976222157871, -0.402570486055813, -0.141295916778994, -0.119215185614463, -0.191346895929757, 3.30747149106533, -7.83904169966102, 10.0645802651882, -8.16976222157871, 0.402570486055832, 0.141295916778997, 0.119215185614467, 0.191346895929758, -0.0259267621608039, -0.00421171384911552, 0.00421171384910318, 0.0259267621608164, 2.49447333148141, 1.02686190320304, 1.31218488814207, -6.93411799042808, -4.32320098473321, 13.4835715113227, -2.49447333148142, -1.02686190320303, -1.31218488814207, 6.93411799042805, 4.32320098473321, -13.4835715113227, 0.126456359955369, 2.85111401141042e-14, -0.12645635995541, -0.222657518386664, 0.222657518386675, 0, 0.0153916967779076, 1.10346542679626e-14, -0.0153916967779368, 0.00401332195867896, -0.00401332195868343, 0, -0.070852186140768, 0.0708521861407849, -1.16564240092162e-14, 1.80076705490212e-14},
    {-0.234775607728653, 0.234775607728654, -0.0623938398739167, -0.0442565832486536, -0.0570853946158694, -0.153888727424812, -1.57034702363989, 4.51342080072434, -7.4925230886827, 9.65546182492158, 0.0623938398739188, 0.0442565832486516, 0.0570853946158718, 0.153888727424812, 1.57034702363989, -4.51342080072434, 7.49252308868269, -9.65546182492158, -0.0979851344526203, -0.0136898800725382, 0.0136898800725449, 0.0979851344526132, 0.465674733519446, -1.65269345913737, 4.94093995929798, 0.40786673452738, -2.60689130666784, 0.947458978837544, -0.465674733519425, 1.65269345913735, -4.94093995929798, -0.40786673452738, 2.60689130666784, -0.947458978837543, 0.0205706117450227, 0, -0.0205706117450253, 0.0101099230203402, -0.0101099230203397, 0, 0.517727374905762, -1.08514633134913e-14, -0.517727374905755, -1.03104066117048, 1.03104066117048, 0, -0.10637820100033, 0.106378201000312, 2.04716824557885e-14, 0},
    {-0.31930730930015, 0.31930730930015, -0.40354937277583, -0.0432625580147485, -0.0321756283939297, -0.0578765020203707, -1.73319922781626, 3.81904507052228, -4.32694704068549, 2.64740738044188, 0.403549372775832, 0.0432625580147495, 0.0321756283939273, 0.0578765020203702, 1.73319922781626, -3.81904507052228, 4.32694704068548, -2.64740738044188, -0.0749726874810879, -0.0109550972089555, 0.0109550972089591, 0.0749726874810845, 2.4309495749274, -6.52582104004519, 12.6627476236861, 0.317500476191697, -1.45972009203279, 0.404413103840403, -2.4309495749274, 6.52582104004517, -12.6627476236861, -0.317500476191689, 1.45972009203279, -0.4044131038404, 0.0776918048718218, 0, -0.077691804871824, 0.00595986904735342, -0.00595986904735666, 0, 0.333677575274831, 0, -0.333677575274824, -0.526113331034617, 0.526113331034613, 1.08232277306529e-14, -0.334888684167127, 0.334888684167127, 2.22347849634512e-14, -1.36331978027872e-14},
    {-0.467046932097564, 0.467046932097565, -1.25354229882126, 0.899206741227323, 0.0272753348566972, 0.020484775196609, -1.76831839423409, 2.33803115856642, -1.05180256724954, -0.187994765484728, 1.25354229882126, -0.899206741227323, -0.0272753348566972, -0.0204847751966101, 1.76831839423408, -2.33803115856641, 1.05180256724954, 0.187994765484728, -0.0435965037573232, -0.00679531167721729, 0.0067953116772199, 0.0435965037573209, 5.20901084739882, -8.20188791354961, 5.86326077670042, -4.44984169245751, 11.1338297871503, -0.21448481293619, -5.20901084739881, 8.2018879135496, -5.86326077670042, 4.44984169245751, -11.1338297871503, 0.214484812936197, 0.0990556500797281, 0, -0.0990556500797241, -0.052259490440213, 0.0522594904402063, 0, 0.139733560051001, 0, -0.139733560050999, -0.135880116749664, 0.135880116749657, 0, -0.302733776451806, 0.302733776451798, 2.21543860704444e-14, 0},
    {-0.658971363590885, 0.658971363590887, -2.74971897010371, 4.25860765292947, -2.75680360047958, 0.288623553688349, -1.42890947687281, 0.660518268226533, 0.128393202681089, 0.105852187397973, 2.74971897010371, -4.25860765292947, 2.75680360047958, -0.288623553688348, 1.4289094768728, -0.660518268226521, -0.128393202681082, -0.105852187397972, -0.0168084026612552, -0.00279917363083235, 0.00279917363083044, 0.016808402661258, 6.46410438929988, -3.47238271635904, -0.993408479653094, -11.6339391850061, 9.19790865284457, 11.08429471871, -6.46410438929987, 3.47238271635903, 0.993408479653094, 11.633939185006, -9.19790865284457, -11.08429471871, 0.0604733277503699, 0, -0.0604733277503793, -0.0698591313580376, 0.0698591313580324, 0, 0.0314253609405353, 0, -0.031425360940542, -0.0108352861369411, 0.0108352861369284, 0, -0.106038582195684, 0.106038582195675, 1.72543953244255e-14, 3.36471746689618e-14},
    {-0.852928776435772, 0.852928776435774, -4.62479994033092, 10.2960803115461, -12.0615371748882, 8.16976222157872, -0.729646958987002, -0.163999371232671, -0.113184183927624, -0.142219700439307, 4.62479994033093, -10.2960803115461, 12.0615371748882, -8.16976222157871, 0.729646958987022, 0.16399937123268, 0.113184183927631, 0.142219700439308, -0.00296275009750153, -0.000520521039660874, 0.000520521039645013, 0.00296275009751766, 4.2468000670121, 1.08748345489142, 1.03954239228302, -10.7714017088685, -3.82040188607881, 17.4774853307227, -4.24680006701212, -1.08748345489141, -1.03954239228302, 10.7714017088685, 3.82040188607882, -17.4774853307227, 0.0139710902105843, 3.90254011425267e-14, -0.0139710902106377, -0.0233100209578104, 0.0233100209578213, 0, 0.00220419555815771, 1.54213898149886e-14, -0.00220419555819265, 0.000371289720448626, -0.0003712897204592, 0, -0.00961462960378686, 0.0096146296038049, 0, 4.08959350716666e-14},
    {0.218256695635768, -0.113928564117326, 0.897392725924518, -1.11679585815057, 0.667429939275517, -0.153888727424814, 0.997420422902051, -1.2123572948061, 0.716898055757183, -0.164167775300466, -0.0660288489812695, -0.0587187272009719, -0.124017235870968, 0.153888727424814, -0.0733887411520724, -0.0637431422621106, -0.13320906067352, 0.164167775300467, -1.3337679887891, 3.57875818583615, -6.2145484466493, 3.8652301180838, 0.518943859261568, -0.205684647193057, 0.0404350712376716, -0.21059145844599, 0.0277492861274123, 0.0418410144593046, 0.0272849533620714, 0.0382189049440496, -0.0404350712376716, 0.0391306548262746, -0.0277492861274128, -0.0418410144593035, 0.496253995160494, -2.15318739295371, 0.825569520849965, 0.712706279485976, 0.462808305865569, -0.543412703404548, 0.551568845412547, -2.39319198610028, 0.917591458937758, 0.773690778563429, 0.502409658504779, -0.583688995083663, -0.33117471245717, -0.215054100166468, 0.167465742249008, 0.171460803619715},
    {0.280175037546942, -0.269990545910719, 1.30154270918084, -0.557676967080663, 0.277076277497003, -0.0578765020203724, 0.24686073847691, -0.666461484090045, 0.536618750989811, -0.149034415307043, -0.383747306192836, -0.0469015469885905, -0.0451261636041884, 0.0578765020203722, -0.0727845062071805, -0.0560505031717015, -0.0873966756338489, 0.149034415307042, -1.38675356282548, 2.7326630332463, -2.6581508558724, 1.30205689381536, 1.38384756336564, -0.760776739292476, 0.182364335615596, -0.12074203122568, 0.0197614744334011, 0.0178594059513624, 0.11638384826615, 0.123904276157108, -0.182364335615596, 0.019664710036573, -0.0197614744334012, -0.0178594059513622, 2.21983100881437, -6.07448098846482, 2.93685457666245, 0.375947542808647, 0.228630971260607, -0.231950113892814, 0.421030457367586, -1.15213342758469, 0.557026737947377, 0.449282599265076, 0.273229387996671, -0.449222075355961, -0.932895066103382, -0.567336345528402, 0.636872463135367, 0.101077321189109},
    {0.288734078223699, 0.127458660661965, -0.835345906031963, 2.86220224093331, -0.136739719441227, 0.0204847751966034, -0.0520390799654323, -0.164994643528037, 0.292827817801936, -0.120198635080441, -0.764302484300343, 0.528230785883268, 0.0137224969120978, -0.0204847751966036, -0.0476133273786108, -0.030450416455872, -0.0293866978957166, 0.120198635080441, -1.0238187856576, 1.22959549126017, -0.443553780551473, -0.178415663936756, 0.732213640270653, -0.911627065727219, 0.325606241800374, 0.985169468409694, -0.246844448755229, -0.00947192686946607, 0.135132934040505, 0.0914862166278332, -0.325606241800373, -0.0988665549658453, 0.246844448755229, 0.00947192686946463, 2.97834561505094, -4.29275694872361, 2.91405972400498, -2.34109387257041, -1.04933915424616, 0.12301722252913, 0.185540342637888, -0.267423495480978, 0.181535560187134, 0.134954806283879, 0.0604902537000311, -0.263441119906219, -0.598902775679185, -0.268443798631974, 0.820140849099385, -0.886302913443848},
    {0.0751681159726955, -0.0599574981188607, 0.485316152903599, -1.81264552367904, 5.71927997511736, 0.288623553688334, 0.0111373775675087, 0.0177606114836727, 0.0921747723655389, -0.0793175468753276, -0.148269706050575, -0.176927006276899, 0.638089254542729, -0.288623553688337, -0.00340259784931518, 0.00173356112841877, 0.0102837650967024, 0.0793175468753277, -0.270916465188293, 0.260237953407898, 0.0391178393220861, -0.0436499453955259, -0.121505350851351, -0.455083346915364, 0.343765567681148, 1.0658696812075, -0.581054568323704, 0.489496797176204, -0.0118597804655767, -0.0507727886816928, -0.343765567681146, 0.118917065308966, 0.581054568323702, -0.489496797176201, 0.795851451112416, -1.26907133398316, 0.13617343601772, 0.131394163470664, 1.85817836648528, -6.35736922966008, 0.0182637607375191, -0.0291235445639007, 0.00312500410818864, -0.00128742253140141, -0.0182067500806896, -0.102458537462241, 0.00880762053238948, 0.124557510784538, 0.505856135597057, -1.18478674651647},
    {-0.424231227319649, -0.226504621301655, 1.17098067011926, -2.26640789781617, 1.30337508285044, 8.1697622215787, 0.00815851923719796, 0.0047069765916861, 0.00236493139971297, -0.0350775308879033, 2.519314869394, -6.30034913894459, 8.72079246890365, -8.1697622215787, 0.0175527054809858, 0.0130848449412079, 0.0158235923115192, 0.0350775308879038, 0.176334246021151, 0.123116291437677, 0.122717827016866, 0.228567484145611, -0.0412828582926714, -0.0153222701366976, 0.201005644744972, 0.0514020887442683, -0.498130062975753, 0.771828366999475, -0.11476152238454, -0.102520248984791, -0.201005644744973, 0.343927818097526, 0.498130062975754, -0.771828366999472, -1.23265959259316, -0.955357742054917, -1.50227820486519, 3.99528893596368, 4.57146810079708, -10.0241675517541, -0.00858825192911775, -0.00665621962501209, -0.0104667531640531, -0.00829759352530626, -0.0094942280075872, -0.0181885237112316, 0.0727746082869156, 0.0832697723902998, 0.117842519121488, -0.395329906841804},
    {0.28628929987628, -0.240503275451298, 0.198934346621583, -0.63972460420333, 0.539525186253463, -0.153888727424811, 1.09978928821842, -0.188861504297644, 0.0910267577698622, -0.0187732230698009, -0.0770893893180525, -0.057551266620024, -0.0856466499156516, 0.15388872742481, -0.426181230376302, -0.0169904654544728, -0.0144499961342122, 0.0187732230698007, -1.37405067804918, 2.58817254123149, -2.3143104930913, 1.054402605484, 1.47857595357728, -0.0452169713964471, 0.00635131781340947, -0.846741398215312, 0.00754649641305438, 0.208116353436684, 0.133016486099123, 0.00717794501185165, -0.00635131781340892, 0.134415530451193, -0.00754649641305516, -0.208116353436683, 0.429606699103903, -1.10348754796893, 0.552035891561494, 0.435805007162769, 0.261470863660585, -0.453878536337811, 2.37503907115697, -6.10052414552207, 3.05187701652298, 0.128659721218171, 0.0771922485339447, -0.0765767616356501, -1.0072628124752, -0.604329627201203, 0.0380390263845942, 0.71232586776412},
    {0.30194613339729, -0.00303876266691713, -0.112743231543428, -0.297246090911908, 0.222481451419093, -0.0578765020203722, -0.106362388371803, -0.384243622260383, 0.300062668757961, -0.0798554220922592, -0.377044237587169, -0.0399716863386912, -0.0287480743053725, 0.0578765020203719, -0.35570495082136, -0.051670538372825, -0.0387727778774356, 0.079855422092259, -1.21166390931122, 1.7522973977573, -0.94386217036388, 0.10432131118743, 3.66881404077301, -0.660769301782358, 0.12699644601425, -0.482233019889125, 0.0213755605582884, 0.0888323213280839, 0.493357822210139, 0.0853817020033933, -0.126996446014251, 0.0623120291595069, -0.0213755605582887, -0.0888323213280835, 1.69766098981472, -3.06792860426878, 1.86005508358466, 0.219994476016092, 0.117223301234508, -0.193733377113719, 1.60157975827381, -2.89429537575836, 1.75478295667772, 0.284382122847669, 0.151532037785539, -0.261289890880524, -2.7153219072595, -1.44684995572365, 0.575387599778965, 0.419920990729618},
    {0.236643324868563, 0.157996351774294, -0.685081437210632, 1.02836080694757, -0.10778476979344, 0.0204847751966044, -0.212280187910398, -0.132887934812671, 0.384494298337473, -0.166654346797132, -0.569818879903484, 0.332539840484564, 0.00503620216509609, -0.0204847751966043, -0.176564788258325, -0.0429718172322188, -0.0179653491069821, 0.166654346797132, -0.729132917814566, 0.70458297474317, -0.126043410070261, -0.244046323501201, 1.30854003945959, -1.61083268285132, 0.542384887910224, 3.86258153525536, -0.55334230240575, -0.0471131712642575, 0.423141073687128, 0.0752655413239918, -0.542384887910225, -0.180477645663649, 0.55334230240575, 0.0471131712642566, 1.89155751425654, -2.17798675871177, 1.54132956156934, -1.1310281118658, -0.229872535566341, 0.102748567628345, 0.586120368703332, -0.674873691349988, 0.477598298815379, 0.146154918570893, 0.0297048334739968, -0.36652894923049, -1.43917928428619, -0.292501828860527, 1.53556714152732, -3.68210388959171},
    {-0.0291979345553113, -0.188157374542532, 1.09311974803885, -2.729017019739, 4.22292603253328, 0.288623553688337, 0.124775678124371, 0.0915108108550274, 0.197961200646619, -0.2079121008487, 0.305128780364224, -0.899029556149418, 1.08698561075244, -0.288623553688339, 0.0348293501727644, 0.0301467242859982, 0.0509554216513393, 0.2079121008487, -0.0977639238585051, 0.175823134565445, 0.0717532042345292, 0.0675428941563751, -0.909898118742996, -1.14996385234333, 0.999903900603926, 3.91453543128952, -1.98855701839866, 2.43474709597, -0.299750897820693, -0.296001907386666, -0.999903900603925, 1.00760554502058, 1.98855701839865, -2.43474709597, 0.0493324010341872, -0.9297421742438, -0.517838755193463, 1.2117025874512, 2.41634398843722, -5.30991164328572, 0.00563111571589872, -0.106126717114231, -0.059109426898802, -0.04063143816648, -0.0810260969745454, -0.248916622297958, 0.404001109858711, 0.805647906704977, 1.44596575973, -4.9221409763101},
    {-0.50453951163876, -0.143182682873856, 0.617043178048837, -0.712046529486212, -1.05604468581898, 8.1697622215787, 0.0213836172859797, 0.00646246759163618, -0.00787298934490147, -0.138407634889365, 2.92484054429376, -7.09932077313891, 9.42860290518039, -8.1697622215787, 0.101360282467542, 0.0644327702181777, 0.0702918079193675, 0.138407634889366, 0.183842026197153, 0.119803876761481, 0.121582887132207, 0.222493404421776, -0.0645125279027093, 0.0542527849523456, 0.825879124087892, -0.207156063819443, -2.1768514499934, 3.83905857194563, -0.643209552328149, -0.484380985658047, -0.825879124087892, 1.84953562229109, 2.17685144999341, -3.83905857194563, -1.21924970614277, -0.905593933742558, -1.41704008245727, 3.70470622890734, 4.10666107371778, -8.37255821936142, -0.0422530708055991, -0.0313833371545993, -0.0491074917933215, -0.0336235666482677, -0.037271671161546, -0.0624188185744657, 0.335652171686472, 0.372069908544389, 0.430128200705701, -1.64237955847165},
    {0.270764883720562, 0.161958917611736, -0.0765730592183317, -0.151585061594872, 0.345520686816809, -0.153888727424811, -0.936332773411779, 2.49685715770194, 0.355570477099233, -0.0496222433554886, -0.0612235461982323, -0.034588491114831, -0.0274465741133936, 0.153888727424811, -0.748639448326543, 0.569729765621711, -0.0282448832286885, 0.0496222433554886, -0.898070291168991, 0.973990293071302, -0.268866634225742, -0.239777169008868, 0.774928265718656, 1.61566271931327, 0.0387398677950279, -1.19940992284678, -0.421048643677786, 0.460320928675604, 0.176822169357853, -0.128340815064948, -0.0387398677950275, 0.0952756074984286, 0.421048643677785, -0.460320928675603, 0.220886452527472, -0.284167503731906, 0.201077656620997, 0.13589525699128, 0.0502782957184226, -0.318074112703416, 2.70099205667639, -3.47479060649949, 2.45877077156142, -2.2384200761372, -0.828166847186457, -0.327325593870543, -0.694719352366708, -0.257031082709801, -1.48732190424832, 1.10413431534835},
    {0.21585406400914, 0.138647660917849, -0.241970447233903, -0.0410426412418459, 0.139672629582348, -0.0578765020203709, -0.504910546534907, 0.52314276439262, -0.29869869086601, 0.0690671693240336, -0.223449718107215, -0.0169677308765787, -0.00390597156038897, 0.057876502020371, -0.466264044152152, 0.216276179301876, 0.00835316550663003, -0.0690671693240332, -0.647164702748529, 0.597737904987658, -0.0797575058760441, -0.225317421290074, 1.1204677527659, 4.09036218176068, -0.201370300695995, -0.669620834773751, -0.21077151554272, 0.19648324590981, 0.463220560606466, -0.114387753716805, 0.201370300695995, 0.0187260735671042, 0.21077151554272, -0.19648324590981, 0.720440411760071, -0.803571415413606, 0.548551168994653, 0.0542410793686982, 0.00376929274972541, -0.135766658021958, 1.50331565778353, -1.67678196727928, 1.14464090018281, -0.691374320609003, -0.0480446230854943, 0.29034552535938, -1.4807862863825, -0.102902026989869, -3.97597442804387, 0.650894761206646},
    {0.06179782759709, -0.0784651408728727, 0.358197104159231, -0.535979548866736, -0.0638662258776859, 0.0204847751966049, 0.245495675775413, -0.130028469360582, -0.203425857830674, 0.103341901603271, -0.0545443252607901, -0.0738260863498105, -0.00813907259628263, -0.0204847751966049, -0.0373827589171673, -0.0179101852435312, -0.0259244663684329, -0.103341901603271, -0.245041653542339, 0.244997596742815, 0.0443340954746555, -0.0276227253993489, -1.50849623800912, 1.7919029414595, -0.484294992385492, 5.06227842005483, 1.03882900843693, -0.104206989941389, -0.207781012841069, 0.22835901019047, 0.484294992385492, 0.645133652367756, -1.03882900843693, 0.10420698994139, 0.418094153916273, -0.740360722520861, 0.0186137897061459, 0.0836202835993762, 0.52618535161717, 0.0720052984739689, 0.286547003483862, -0.507417156036066, 0.012757235693958, 0.0202862544045118, 0.127652400199601, 0.229350324199107, 0.235346448381523, 1.48093080246867, -2.02026195164997, -5.70741207242258},
    {-0.236409154506208, -0.301798607677329, 1.4000385651718, -2.49182453126872, 1.95327310133188, 0.288623553688339, 0.353473378128934, 0.0577177593874808, 0.0463079366625623, -0.0876613998747685, 1.15104668812143, -2.14074232727352, 1.76786658528815, -0.28862355368834, 0.290609395595098, 0.0495856947412172, 0.0419123438517426, 0.0876613998747693, 0.103305012625972, 0.128667221027399, 0.109334068422314, 0.196901460107853, -1.83763178006511, -0.367417240409299, 0.505689713046095, 4.00483138931967, -1.37361142730363, 5.38528100170711, -1.5787211676279, -0.332541651106274, -0.505689713046094, 3.62468903506824, 1.37361142730362, -5.38528100170711, -0.715500476622901, -0.761862180902006, -1.07372259576832, 2.02047867724722, 2.61208818129502, -3.72113968662003, -0.180645288505842, -0.192350414803797, -0.271087070414392, -0.0468000457807141, -0.0605034083479843, -0.0882202805143052, 1.49003101208053, 1.92632193561248, 0.699958891515574, -7.62952042438791},
    {-0.636880676372851, 0.0586054310955197, -0.575471048921219, 2.18852029690417, -4.63478621313924, 8.16976222157871, -0.0441106655645895, -0.03472420199433, -0.0545787203230036, -0.204926878310819, 3.58097023983999, -8.36005347555911, 10.5022018617589, -8.1697622215787, 0.274486407165859, 0.13264495923541, 0.123672746018738, 0.20492687831082, 0.170484250415716, 0.107111056838613, 0.109362603663133, 0.191317334359872, 0.438571875105332, 0.41625660583006, 1.30452607023256, -2.01094213702264, -3.80557674439068, 8.49139905584444, -1.67532571388223, -0.943217378252476, -1.30452607023256, 4.55669782469294, 3.80557674439068, -8.49139905584444, -1.06476616731646, -0.771605708869914, -1.16912731473241, 2.9870648901358, 3.18446828851914, -5.86741564861969, -0.0816158248082943, -0.0591446632032896, -0.0896152535896842, -0.0473943260941947, -0.0505264311468853, -0.0690940256957341, 0.598597441285462, 0.638156397491439, 0.526960772422415, -2.54575568767031},
    {-0.0276979469831454, -0.18661368004461, 0.0592422377411684, 0.0565262386342644, 0.137345055287177, -0.153888727424811, 1.15030067029804, -3.12783181000748, 5.64479241115492, 0.798848493973312, 0.0162868154822804, 0.0184619773736321, 0.035004748254848, 0.153888727424811, 0.316239485215838, -1.02157796980729, 1.43867237804988, -0.79884849397331, -0.0998357862280697, 0.176592144722128, 0.0713805344628637, 0.066174734070833, -0.458860714373742, 2.15747459073827, 1.54718213082687, -0.757063491467837, -1.41445222820337, 0.730947882573482, -0.149868031751065, 0.549869485706125, -1.54718213082687, -0.192950644465256, 1.41445222820337, -0.730947882573481, 0.00315442827220136, -0.0508382642087863, -0.027845217286866, -0.0249252809394884, -0.0500629350684087, -0.172349803542025, 0.0612492217421679, -0.987121547495585, -0.54066782976046, 1.37921942940999, 2.77019035040479, -7.0834647892048, 0.202334924349901, 0.406393821774905, -2.70734407644439, 0.950014135933092},
    {-0.119570063807635, -0.262628331893209, 0.3718196902311, 0.0477770585860856, 0.050815006011004, -0.0578765020203711, 1.23199853521869, -2.23741118713505, 1.83649988776652, -0.123511007815443, 0.18629638570859, 0.0244646644642504, 0.0227507319824565, 0.0578765020203711, 0.617280042826308, -1.14568614271631, 0.822231856537402, 0.123511007815445, 0.00904912827074873, 0.143997472754261, 0.0911691460845515, 0.137982648591284, -2.07113859148238, 5.46743855613523, 3.40308137253589, -0.386843515466692, -0.774378690190788, 0.311997573023997, -1.0605447928169, 2.447864105226, -3.40308137253589, -0.173196341600899, 0.774378690190789, -0.311997573023996, -0.102574006235412, -0.223219781902334, -0.232322287801945, -0.0287114516580352, -0.0435302713923029, -0.0735657379934598, -0.339871794726616, -0.739623133364948, -0.769783649953436, 1.34456421219057, 2.03853311766078, -2.65873174430393, 1.24464329337686, 1.88704009092241, -7.91530266136123, 0.560039857067588},
    {-0.291832089112706, -0.29725792355778, 0.807857834631563, -0.582773602692339, -0.0167396352982287, 0.0204847751966048, 0.879189330581097, -0.749214963869792, 0.0517333807439295, -0.0529350629556763, 0.841280923622993, -0.656732008696606, -0.0222767402895098, -0.0204847751966051, 0.91556358107002, -0.844296045487709, 0.0688456508519484, 0.0529350629556777, 0.133867773164481, 0.126150246023437, 0.115422748860112, 0.213649244622457, -2.60448329359947, 2.48888918928709, 0.469894047305333, 2.10691229814977, 5.50503910225614, -0.165471248214076, -2.93501204776656, 3.31215926101938, -0.469894047305334, 2.8038327742792, -5.50503910225614, 0.165471248214077, -0.500068566126384, -0.460043482825831, -0.689026709302343, 0.55478292447006, 0.68472268691888, 0.0390163755877392, -0.544223165326878, -0.500663983642986, -0.749865762681252, 0.713230089338579, 0.880280920018922, -0.120579031595879, 2.47938968354294, 3.06010565782309, -5.80104845030647, -4.91074507242897},
    {-0.533380965289454, -0.105970427076989, 0.287413545533686, -0.0559394161257774, -0.482167487303574, 0.288623553688342, 0.115225774364481, -0.00280414771476367, 0.0150273444166892, 0.0905706902337731, 2.28752304127764, -3.66947089586477, 2.49848276832552, -0.288623553688342, 0.917081389878047, -0.18394433012865, -0.0778682969454364, -0.0905706902337723, 0.183537139047258, 0.117927627713662, 0.120054740517502, 0.217831885088023, -0.0655065393482086, -0.196284242324393, -0.664776985404365, -1.56980201021202, 2.9727257637787, 8.55133777337992, -4.29704770401275, 1.01710050979141, 0.664776985404366, 8.13435865228322, -2.9727257637787, -8.55133777337991, -0.893506861854459, -0.657643433361119, -1.02378629159575, 1.7758551039083, 1.94955520808224, -2.01631528102194, -0.358212135986792, -0.263653105576644, -0.410441922679092, 0.0890205936398309, 0.0977278842035819, 0.0628409525287471, 2.0795734081194, 2.28298083524156, -0.820816267467019, -6.56455664207121},
    {-0.793676417832502, 0.397585858525557, -2.38339251504385, 6.02882067961083, -8.47493826989332, 8.16976222157871, -0.290095768273407, -0.108667331755436, -0.10038583947818, -0.191346895929757, 4.3415482275118, -9.77838145744787, 11.6542222604831, -8.16976222157871, 0.52843363424466, 0.176251820768422, 0.13804453175075, 0.191346895929758, 0.118675589765877, 0.0754625796527779, 0.0764969692852827, 0.125455420603009, 1.91844015911067, 0.86467502986208, 1.31218488814207, -5.83891432892616, -4.32320098473321, 13.4835715113227, -3.11159357293773, -1.18904877654398, -1.31218488814207, 8.02932165192998, 4.32320098473321, -13.4835715113227, -0.70555821434272, -0.514122084721736, -0.738475413403497, 1.84580097713686, 1.90375980070017, -3.17928399058975, -0.0858773579926396, -0.0625766172429721, -0.0898838907356395, -0.0332699009966144, -0.0343145880163725, -0.0376586922725695, 0.587355124401124, 0.605798289425942, 0.324373746681901, -2.19040732300384},
    {-0.593196374838997, -0.0167231769272591, -0.00592046743455898, -0.0122940972627931, -0.0292027831626254, -0.153888727424811, -0.149008114153966, 1.25378938541602, -3.83289856489747, 9.65546182492158, 0.136760509442183, 0.0820817973762067, 0.0849680060691177, 0.153888727424811, 3.4420298444201, -8.37095104150299, 11.1521476124679, -9.65546182492158, 0.178196159374021, 0.112528398046357, 0.114893726140294, 0.204301268205585, 0.12936042609839, -0.845457039339862, 4.94093995929798, 0.20864958344954, -2.60689130666783, 0.947458978837542, -0.863677589054079, 2.45992987893488, -4.94093995929797, -0.607083885605245, 2.60689130666783, -0.947458978837541, -0.0460077227427006, -0.0334699672899935, -0.0513623519749307, -0.0335780192489065, -0.0362096808645092, -0.0557652229064916, -1.15793627414931, -0.842382254744568, -1.29270320137226, 3.42438840507848, 3.69277325100849, -7.31924904757045, 0.353313202647934, 0.381003960307758, -1.61447283959502, 0.398434302155702},
    {-0.645727750172868, 0.074891979098382, -0.135886497249018, -0.0198848675421183, -0.0202742229580792, -0.0578765020203707, -0.583617242377621, 1.75535633697017, -2.72645767649418, 2.64740738044188, 0.73879227782661, 0.0700963669874164, 0.0440770338297782, 0.0578765020203702, 3.17302984920508, -6.18782608077739, 5.92743640487679, -2.64740738044188, 0.168546980959297, 0.105848120321839, 0.108056090115317, 0.188384579678033, 1.11734285990507, -4.11199275210905, 12.6627476236861, 0.200060597567712, -1.45972009203279, 0.404413103840403, -3.93875770068704, 8.93964932798132, -12.6627476236861, -0.43494035481567, 1.45972009203279, -0.404413103840399, -0.213877370020078, -0.154927489841722, -0.234100920715795, -0.0243300584253537, -0.0258814410199484, -0.0238028108716971, -0.918579280687098, -0.665396166803969, -1.00543715933639, 2.1477599559235, 2.28470978788373, -3.20097872838261, 1.36712085232982, 1.45429398845214, -4.82765657587226, 0.234879757247951},
    {-0.731477026246283, 0.250468856650961, -0.741597708025434, 0.596171784672719, 0.0209632972536984, 0.0204847751966086, -1.04614010189871, 1.55010871751304, -0.808395203406334, -0.187994765484727, 1.85165020848897, -1.23254689245622, -0.0335873724596972, -0.0204847751966096, 2.61204358754973, -3.20475026135101, 1.29520993109275, 0.187994765484729, 0.143532247785438, 0.0904610014067863, 0.0920387320292021, 0.154976188373897, 3.4535609564433, -6.30381314387572, 5.86326077670041, -3.42006265444561, 11.1338297871503, -0.21448481293619, -7.14001556968876, 10.0999626832235, -5.86326077670042, 5.4796207304694, -11.1338297871503, 0.214484812936197, -0.397879585956398, -0.288508666135047, -0.423664248372091, 0.31138585546298, 0.324989252320514, 0.0126240752060018, -0.561271678824503, -0.406986809860553, -0.597644996965969, 0.809635647766167, 0.84500589607181, -0.486814727686415, 1.80382577717256, 1.88262883607289, -3.79614953934777, -2.0595580760238},
    {-0.834219465627174, 0.502239635580984, -2.17143575456295, 3.51888276364891, -2.43060574940825, 0.288623553688349, -1.12840081544712, 0.545785509857599, 0.113201120517727, 0.105852187397973, 3.38060476427583, -5.03884374042012, 3.08300145155092, -0.288623553688348, 1.75675341289619, -0.781534391644853, -0.143585284844443, -0.105852187397972, 0.099615353351484, 0.0638287536487619, 0.0645327311686777, 0.104002991877269, 5.34128226212328, -3.06151420908615, -0.993408479653096, -10.2573572765294, 9.19790865284457, 11.08429471871, -7.64841812019007, 3.88325122363192, 0.993408479653096, 13.0105210934827, -9.19790865284457, -11.08429471871, -0.436843424807248, -0.319740661860352, -0.452584923045287, 0.750888154342703, 0.769072822428492, -0.65239570214267, -0.227008547565535, -0.16615532963232, -0.235188720251211, 0.116464202330788, 0.119284679456465, 0.0303841643267173, 1.13976675240812, 1.16736910565867, -0.821737014545756, -2.75316381695333},
    {-0.93068814177099, 0.778443022056055, -4.26837451395492, 9.64375602134092, -11.5471951890703, 8.16976222157872, -0.673414315020507, -0.153608934270609, -0.108357653355242, -0.142219700439307, 4.99337808590906, -10.961922045512, 12.5758791607061, -8.16976222157871, 0.787796917156005, 0.174605118508022, 0.118010714500013, 0.142219700439308, 0.0457228157942343, 0.0299465576581389, 0.0300813729299916, 0.0464943733325726, 3.97773739893536, 1.04110973057894, 1.03954239228302, -10.3120751682584, -3.82040188607881, 17.4774853307227, -4.52143824337276, -1.13385717920389, -1.03954239228302, 11.2307282494786, 3.82040188607882, -17.4774853307227, -0.263105125640376, -0.195156578656435, -0.266741867657333, 0.656049156768744, 0.662116867402308, -1.02868397163587, -0.0415096560487552, -0.0307895273266125, -0.0420834187601292, -0.0104497678681721, -0.0105464163692422, -0.00965306114476933, 0.270599055040753, 0.273101789396643, 0.0927474486249506, -0.918653081220258},
    {-0.284218581943468, 3.62990990645796, 3.05254952705568, -2.02813935973006, 0.852973082215378, -0.153888727424813, 3.39280133686021, -2.20168218722082, 0.916193158666486, -0.164167775300459, 0.0729924116469035, 0.0498880353538467, 0.0615259070688879, 0.153888727424812, 0.0811284959085241, 0.0541568301345006, 0.0660860422357934, 0.164167775300459, 1.68915042532674, -4.27656497941373, 6.18108222239516, -6.93935899282266, 0.942419743749329, -0.262864245603182, 0.0404350712376769, -0.269135132885853, 0.0277492861274138, 0.0418410144593263, -0.0231815773746993, -0.0189606934660804, -0.0404350712376773, -0.0194130196135855, -0.0277492861274148, -0.0418410144593249, -0.48919699121338, 1.55580759904068, -4.19215254652989, -0.415850747203152, 2.39410207157937, -0.914498989284266, -0.543725233961289, 1.72922537542376, -4.6594299742312, -0.451434058644528, 2.59895941573085, -0.982279200902279, 0.193234233504121, -1.11247239987875, 0.281824939069266, 0.288548152499436},
    {-0.365495227282889, 0.282631633534525, 6.52731890993826, -1.0647384417033, 0.356273456486932, -0.0578765020203701, 1.23802219859216, -1.27243405037862, 0.690001392244446, -0.14903441530704, 0.452267144066583, 0.047304663296178, 0.0340710153857456, 0.0578765020203703, 0.0857805129140944, 0.0565322542722027, 0.0659859656207906, 0.14903441530704, 1.81083303074513, -3.57567070399205, 3.50106007665704, -1.65335880966175, 2.64209530812438, -0.9782308358229, 0.182364335615606, -0.155253929338986, 0.0197614744333962, 0.0178594059513615, -0.117384161266238, -0.0935498203732983, -0.182364335615606, -0.0148471880767287, -0.0197614744333971, -0.0178594059513606, -2.49979653910289, 6.08250695734887, -10.5622964722508, -0.319526352383545, 1.33696013079067, -0.390344471872678, -0.474130902760286, 1.15365569542249, -2.00332750416846, -0.381855481911313, 1.59775727801773, -0.755987357865236, 0.792888697720147, -3.31759984457829, 1.0717806561962, 0.170101117415712},
    {-0.505828786742264, -0.143241115565023, 0.878278855981279, 6.65596057481904, -0.178742789963476, 0.0204847751966052, 0.0547136501038983, -0.3836898129256, 0.382777303820131, -0.120198635080441, 1.34289336556039, -0.950264780748593, -0.0282805736101488, -0.0204847751966045, 0.0836574821649678, 0.054779007756538, 0.0605627881224783, 0.120198635080441, 1.79145702692408, -2.14650769602212, 0.770376839904918, 0.233743731500406, 1.7027396080848, -1.19165642433791, 0.325606241800382, 1.28778923995138, -0.246844448755236, -0.00947192686947174, -0.243098417149595, -0.188543141982837, -0.325606241800383, 0.203753216575818, 0.246844448755236, 0.0094719268694714, -5.22969989144649, 7.5153712767023, -4.50684360679748, 4.40965559457903, -10.1153513886495, 0.207023363573627, -0.325791709615184, 0.468180910463767, -0.280760333117448, -0.254199211538503, 0.583110016707564, -0.443340091942609, 1.12808589451518, -2.58772708545038, 1.38019956632074, -1.4915424565272},
    {-0.685613082674015, 0.102291761943671, -0.558471148525221, 1.03193983204806, 7.88994374560003, 0.28862355368833, -0.0128161900329664, -0.0101111233234065, 0.127158273749777, -0.0793175468753275, 2.84662379487985, -4.38015391749968, 2.8087530250254, -0.288623553688333, 0.0653263317252962, 0.0429174987338478, 0.0452672664809406, 0.0793175468753279, 1.41363178800633, -0.574619210989802, -0.139438536817, -0.116252719469182, 0.0691730455362047, -0.627803153953483, 0.343765567681152, 1.47040394270876, -0.581054568323701, 0.489496797176205, -0.293610709637506, -0.223492595719796, -0.343765567681152, 0.523451326810195, 0.581054568323697, -0.489496797176199, -6.36648531888773, 3.02713440179705, 1.05119827073606, 11.3824333416433, -8.0342192561917, -10.6986967706254, -0.146102597966737, 0.0694688165203082, 0.0241236397541019, -0.111527032549338, 0.0787206571388971, -0.172425540230718, 0.762987875263803, -0.538550211162502, 0.851295749673279, -1.99385526951897},
    {-0.865091413021992, -0.290248257600275, 1.68368761382698, -4.01561825483131, 4.72603256077767, 8.16976222157869, 0.0117306785136257, 0.00833981435771314, 0.00857523129456925, -0.0350775308879022, 4.6826809050008, -10.4012155852295, 12.1434499468309, -8.16976222157869, 0.0326254251842135, 0.0216017065294947, 0.0220338922063766, 0.0350775308879031, 0.710593182365714, 0.172618275021654, 0.120634157325978, 0.151494055908925, -0.0731449089687124, -0.0555584869803039, 0.20100564474497, 0.186383757288259, -0.498130062975745, 0.771828366999481, -0.189459236129024, -0.14275646582839, -0.201005644744971, 0.478909486641513, 0.498130062975745, -0.771828366999474, -4.12921691934226, -1.13963425166939, -1.09751734781613, 10.4478114715184, 3.96902236854241, -16.8694825076086, -0.0287693012623511, -0.00794012079179857, -0.00764668164368776, -0.0216984789358539, -0.00824304195135334, -0.0306091235009452, 0.190307985099848, 0.0722961599978929, 0.198314952808694, -0.665293243929786},
    {-0.377854821367115, 0.0804169334283818, 1.11829004720866, -1.23357237757901, 0.694497732197736, -0.153888727424811, 6.18235833041366, -0.364179106695019, 0.117173170875445, -0.018773223069803, 0.0927302983767224, 0.0601399658498403, 0.0693258960286176, 0.153888727424811, 0.51265048283498, 0.0177547093611472, 0.0116964169713665, 0.0187732230698021, 1.82023306649531, -3.45521657617556, 3.1671132019426, -1.23469180432362, 2.85111818820364, -0.0582050382295427, 0.0063513178134097, -1.0899583481951, 0.00754649641304139, 0.208116353436686, -0.138999667623031, -0.0058101218212534, -0.00635131781341047, -0.108801419528584, -0.00754649641304184, -0.208116353436685, -0.49762421287264, 1.1608709134869, -1.87426704619964, -0.39342523115264, 1.56685764288181, -0.763823628226356, -2.75106731527107, 6.41776252989538, -10.361704027873, -0.116148230810445, 0.462572628144319, -0.128869587846814, 0.909311729595437, -3.62143025017604, 0.0640151600507952, 1.19875976772368},
    {-0.451804425814343, -0.296938442417928, 1.80057222446277, -0.62211448594427, 0.288629886037621, -0.0578765020203728, 1.69866660382197, -0.804194002372014, 0.389277637912433, -0.0798554220922589, 0.542062083177414, 0.0546170488540227, 0.0374003603131547, 0.0578765020203724, 0.511383406553645, 0.0706022831938959, 0.0504421912770376, 0.0798554220922588, 1.83050294886805, -2.70162889910245, 1.55859240591333, 0.0612764125533418, 7.67856140344347, -0.857229971550937, 0.126996446014249, -0.62561108212112, 0.0213755605582897, 0.0888323213280853, -0.674120877709405, -0.11107896776519, -0.126996446014251, -0.0810660330724882, -0.0213755605582899, -0.0888323213280855, -2.42715807579339, 4.33770602065989, -4.25318225250669, -0.294051389401699, 0.861548826491944, -0.326030246350776, -2.28979005092516, 4.09220816270834, -4.01246812215879, -0.380113900397406, 1.11370561957553, -0.439719829189472, 3.62938285525003, -10.6338233809841, 0.968308939316129, 0.706677115193606},
    {-0.576948661244746, 0.100404462764304, -0.754231010240755, 3.33204574872113, -0.142867299242774, 0.0204847751966036, -0.233706960786517, -0.4305771624681, 0.509642151511679, -0.166654346797132, 1.50519714998454, -1.04181171702516, -0.0300463272842384, -0.0204847751966035, 0.466402264732028, 0.134626102632261, 0.107182504067224, 0.166654346797132, 1.68526479642688, -1.45555613539847, 0.147817722931937, 0.0990178145200961, 4.23986916465107, -2.13513760220476, 0.542384887910226, 5.11980118438332, -0.553342302405752, -0.0471131712642565, -1.32565568047273, -0.449039378029456, -0.542384887910228, 1.0767420034643, 0.553342302405752, 0.0471131712642556, -4.80610959998928, 4.92314980708569, -0.868006346840198, 3.9213599449272, -6.21159397662318, 0.172913626527014, -1.48922711022166, 1.52549333465281, -0.268961528376665, -0.506730148813497, 0.802681208649336, -0.616824655578902, 4.98974334922499, -7.90395683340333, 2.58417698023422, -6.19654318784762},
    {-0.73356085670795, -0.0327513821297315, 0.335749943545738, -1.50942741166772, 6.03594528816659, 0.288623553688333, 0.0383246455489446, 0.0506149010319215, 0.282951433929324, -0.2079121008487, 3.02009427869259, -4.59614374252045, 2.90000486638575, -0.288623553688335, 0.344732873319209, 0.154120270281251, 0.135945654934044, 0.207912100848701, 1.25518741491938, -0.278180235623071, -0.106236390698722, -0.104458549759898, -0.503267349496747, -1.6436752248653, 0.999903900603928, 5.59515404954395, -1.98855701839865, 2.43474709597, -1.53242816535887, -0.789713279908625, -0.999903900603928, 2.688224163275, 1.98855701839865, -2.43474709597, -5.58848627032463, 1.44362886899563, 0.789013179090672, 9.82552056826454, -3.71994941407637, -8.93595015455234, -0.637905559129753, 0.164785030562312, 0.0900630096992894, -0.329474439980092, 0.12473926866692, -0.418897088863368, 3.27598641418131, -1.2402908993257, 2.43338850477393, -8.28337821281896},
    {-0.886657647844948, -0.239734464658436, 1.266390410802, -2.55888156794631, 1.80268636101634, 8.1697622215787, 0.0438867308522127, 0.0232241693750282, 0.0134393276185034, -0.138407634889364, 4.78510003641373, -10.5867211358993, 12.2873339520157, -8.1697622215787, 0.165827532811172, 0.0960840891841753, 0.0916041248827732, 0.138407634889365, 0.606871885294592, 0.190501890245105, 0.143018045912816, 0.186000291050873, -0.231838667440784, -0.0926104328671937, 0.82587912408789, 0.353618948008259, -2.1768514499934, 3.83905857194564, -0.95917347307493, -0.631244203477581, -0.825879124087891, 2.41031063411879, 2.1768514499934, -3.83905857194563, -3.51214052066778, -1.25124354829928, -1.28810637824867, 8.83267191596803, 4.31293078787758, -14.090020313032, -0.121713149776704, -0.0433618166770505, -0.0446392972096292, -0.0801645027968721, -0.039143755762331, -0.105043452501276, 0.800253873641964, 0.390758266873754, 0.723854636344774, -2.76392958212706},
    {-0.537207852379277, -0.023213843743911, -0.0111495902450414, -0.389873654755793, 0.454124093010765, -0.153888727424811, -0.136336811707856, 6.42186515765065, 0.467332714291884, -0.049622243355488, 0.125473334825655, 0.0765656943091299, 0.0811568320805628, 0.15388872742481, 1.53428368653198, -1.26116386310603, 0.0835173539639602, 0.0496222433554892, 1.75200335936093, -1.83373928319032, 0.443089749751429, 0.199067870201142, 1.99309953072269, 2.12349475737311, 0.0387398677950327, -1.57640617231609, -0.421048643677798, 0.460320928675606, -0.391416674441765, 0.379491222994887, -0.0387398677950352, -0.281720641970866, 0.421048643677796, -0.460320928675604, -0.448749399499435, 0.56043702622354, -0.226011371304719, -0.324701707017567, 0.638009667464229, -0.535280925091328, -5.48729245101886, 6.85300496601938, -2.76365938982464, 5.34837518126699, -10.5090764758116, -0.550850068255848, 1.65992960023656, -3.26161245651748, -2.50298598036799, 1.85812681428695},
    {-0.595832618747913, 0.138566640473884, -0.382088705155292, -0.163849194570805, 0.186028874620443, -0.0578765020203715, -0.797289996155129, 2.08847476667467, -0.397834432404978, 0.0690671693240351, 0.688772361232906, 0.0662246334629468, 0.0424502734777083, 0.057876502020371, 1.43723513893443, -0.844120572468691, -0.0907825760323357, -0.0690671693240333, 1.64721956367446, -1.28476773881766, 0.048766693049872, 0.0460474603673556, 4.47309757068155, 5.4479211549056, -0.201370300695997, -0.891862712743732, -0.210771515542718, 0.196483245909809, -1.80793837795992, 1.24317121942811, 0.201370300695994, -0.20351580440288, 0.210771515542718, -0.19648324590981, -2.07847566557605, 1.91747624331689, -0.145684233818453, -0.234950202639317, 0.332574763747171, -0.228479148098151, -4.33707626804132, 4.00112488549046, -0.303993760228437, 2.99475118521418, -4.23910537942016, 0.488617008437312, 6.41416140866592, -9.07932060138754, -6.6910923743337, 1.09537851714661},
    {-0.692328341357121, 0.0867112697486244, -0.268333407534355, 0.158965367263468, -0.0884517559977598, 0.0204847751966037, -0.183906264039706, 0.0385649479169814, -0.281735363146928, 0.103341901603271, 1.76462388414566, -1.18519576926094, -0.0327246027163559, -0.020484775196604, 1.20941104184694, -0.287528119487927, -0.104233971684686, -0.103341901603269, 1.39280580424551, -0.529306916813743, -0.13891816433742, -0.118963651485846, 0.447402627577314, 2.48170233282908, -0.484294992385495, 7.01102044860074, 1.03882900843693, -0.104206989941391, -3.33569324242817, 0.91815840156005, 0.484294992385493, 2.59387568091366, -1.03882900843693, 0.10420698994139, -3.84905264255625, 1.70989773051127, 0.64286443543368, 3.00130456555184, -1.97507416355437, 0.121176358714116, -2.63800507767209, 1.17190366416839, 0.440596635696461, 0.728115540170867, -0.479152368599922, 0.385969334831612, 8.44706977314216, -5.5587791582913, -3.39986073438912, -9.60489612951441},
    {-0.809132783013123, -0.258324173993585, 1.34708568607265, -2.86625354516242, 3.2238211552292, 0.288623553688337, 0.340104151364433, 0.0663906027038623, 0.076429919485395, -0.087661399874769, 3.29120129853803, -4.92980584650552, 3.03841463918548, -0.288623553688338, 0.83094285403047, 0.114188356405145, 0.0720343266745751, 0.0876613998747705, 0.96120780531404, 0.050235729304175, 0.026857621515969, 0.0291558008725276, -2.11375983269295, -0.606411602975418, 0.505689713046095, 6.60985918825749, -1.37361142730362, 5.38528100170711, -3.63555610734632, -0.57153601367239, -0.505689713046094, 6.22971683400605, 1.37361142730362, -5.38528100170711, -4.2097487664175, -0.246782311661753, -0.181755906531429, 7.22417670181751, 0.571882689850426, -6.26223579441468, -1.06285223461478, -0.0623061246565313, -0.0458886461235855, -0.167332525792189, -0.013246433316819, -0.14846424615997, 5.32757283888954, 0.421743101149727, 1.17794761664781, -12.8395760222635},
    {-0.919863564854675, -0.0664733009545177, 0.146914305631322, 0.484156194529889, -2.63141247240989, 8.1697622215787, 0.0112611882292053, -0.00768187415005485, -0.0309872168383869, -0.204926878310818, 4.94227542033299, -10.8701129799616, 12.5055756024883, -8.1697622215787, 0.378832364552671, 0.172470869633389, 0.147264249503355, 0.20492687831082, 0.439657887279583, 0.184953912959843, 0.153008802706329, 0.20871626286344, 0.0970232217536349, 0.236330819574589, 1.30452607023256, -1.14171786514224, -3.80557674439068, 8.49139905584445, -2.17833293068939, -1.12314316450794, -1.30452607023256, 5.42592209657334, 3.80557674439068, -8.49139905584444, -2.52901150425871, -1.20464522191185, -1.35553299979375, 6.30281979656657, 4.08313698886518, -9.8741631300784, -0.193852290019632, -0.0923377511472544, -0.103903511614987, -0.10000381904588, -0.0647851764374552, -0.116277032664968, 1.26306322154809, 0.818246487387673, 0.886812344933355, -4.28420423143111},
    {-0.732933512062504, -0.0307253509165817, 0.0176781856910737, 0.0308310083721116, 0.196192270366814, -0.15388872742481, 0.343255582932788, -1.70600788325706, 8.06337466302118, 0.798848493973314, 0.164508796215621, 0.0953652459992159, 0.0938519633344861, 0.15388872742481, 3.19425102379765, -5.27695557341787, 3.85725462991614, -0.798848493973309, 1.2574047879515, -0.281639118042051, -0.107048073102031, -0.105058733828334, -0.250275604184983, 3.08187169411101, 1.54718213082687, -1.08143685910164, -1.41445222820337, 0.730947882573487, -0.774142521471149, 1.47426658907886, -1.54718213082687, -0.517324012099065, 1.41445222820336, -0.730947882573486, -0.305223167146542, 0.0796869323785973, 0.0433492528612491, -0.20442704845281, 0.0782307940814807, -0.290044233701301, -5.92648804545816, 1.54727328379439, 0.841708154933327, 11.3117985633816, -4.32883510670663, -11.9206292929373, 1.65946901397884, -0.635050888322712, -4.55613828318986, 1.5987608712007},
    {-0.768918916307804, -0.148294415168218, 0.251910113159567, 0.0422020538189121, 0.075933333489327, -0.0578765020203713, 0.834686539129943, -1.97633236806257, 2.74429876876707, -0.123511007815442, 0.860731749519541, 0.0793721793555812, 0.0478690594607821, 0.0578765020203709, 2.85197444483158, -3.71701831994325, 1.73003073753795, 0.123511007815447, 1.12419892211282, -0.102275710642096, -0.0500045419232713, -0.0547053380714266, -1.8294617728856, 8.17004400482655, 3.40308137253588, -0.578063843222912, -0.774378690190788, 0.311997573023995, -3.44078912805436, 5.15046955391733, -3.40308137253589, -0.364416669357118, 0.774378690190789, -0.311997573023994, -1.35806381863702, 0.144544174396513, 0.100877781561402, -0.144101486207879, 0.0225272530333807, -0.123802392950109, -4.49984946804295, 0.478937011163158, 0.334251472918269, 6.74830738571693, -1.05495669771111, -4.47432950630503, 6.24680878244856, -0.976557881508612, -13.3205135587439, 0.942480512580026},
    {-0.82658934733983, -0.288245120659523, 0.870895760782952, -0.740818696194016, -0.0300614087998723, 0.020484775196604, 0.947793322172683, -0.952398067000442, 0.0929039539653081, -0.0529350629556766, 2.06115841063896, -1.34512191896954, -0.0355985137911533, -0.0204847751966046, 2.24315270037299, -1.72929155552928, 0.110016224073327, 0.0529350629556794, 0.885947950004178, 0.0999179023362583, 0.0596887415221357, 0.0692798741367833, -3.31080527482653, 4.46960247602633, 0.46989404730533, 3.78363989249273, 5.50503910225614, -0.165471248214078, -6.01150695506044, 5.29287254775862, -0.469894047305333, 4.48056036862214, -5.50503910225614, 0.16547124821408, -2.37641281398798, -0.303949521999147, -0.251691835434788, 1.77351765151424, 0.312422963649309, 0.0656599225910264, -2.58624315015439, -0.330787380305278, -0.273915492086001, 2.28003800629106, 0.401651616238654, -0.202920178038638, 7.92605751690364, 1.39625471298334, -9.76247502378494, -8.26420026111488},
    {-0.894125141570335, -0.211629929169826, 0.79525534948202, -0.973470037069578, 0.206284350599028, 0.288623553688342, 0.318822529019678, -0.0487983959951435, -0.00642910620448534, 0.0905706902337729, 3.59296158151086, -5.29595271223313, 3.18693460622812, -0.288623553688342, 1.44043934924041, -0.265477095115415, -0.0993247475666105, -0.0905706902337716, 0.570055294969541, 0.192782034875824, 0.14810866145702, 0.194809079437779, -1.1399592220309, 0.0839757315183982, -0.664776985404367, 0.671603948363608, 2.9727257637787, 8.55133777337992, -6.20170103224063, 1.29736048363421, 0.664776985404367, 10.3757646108588, -2.97272576377871, -8.55133777337991, -2.45555469396103, -0.942021585899309, -0.990640651132542, 4.11128811570121, 2.15813463360149, -3.39321895682715, -0.984446263938651, -0.37766197310482, -0.397153641216616, 0.206091875332979, 0.108183615777578, 0.105753853771096, 4.81442738189245, 2.52723287237907, -1.3813362151526, -11.0473685592224},
    {-0.956163846795197, 0.28738336806914, -1.81307241877916, 4.86684701705692, -7.38940173345123, 8.16976222157871, -0.220678982979597, -0.0877231729903787, -0.0875276341409096, -0.191346895929756, 5.11339400277778, -11.1769121298321, 12.7397587969252, -8.16976222157871, 0.622379214652029, 0.201459834843233, 0.150902737088021, 0.191346895929758, 0.246707185714537, 0.131759001550108, 0.119556563039007, 0.170757728422408, 1.54868676013974, 0.75392067305468, 1.31218488814206, -5.09102040505876, -4.32320098473321, 13.4835715113227, -3.55662213627248, -1.29980313335138, -1.31218488814206, 8.77721557579739, 4.32320098473321, -13.4835715113227, -1.41009209391457, -0.850271555111286, -1.03995793497277, 3.48088865935147, 2.82917645342374, -5.35035706347392, -0.171630038585114, -0.103491211986324, -0.126578981100993, -0.0627417703811784, -0.0509948914716764, -0.0633751029471105, 1.10765885209969, 0.900276524033056, 0.545882460296699, -3.68619517073865},
    {-0.909191772905897, -0.136246592488562, 0.0232954564072358, 0.00616373515135121, -0.0101622736935104, -0.153888727424811, 0.586307089086472, -0.628596434706277, -1.33381000156882, 9.65546182492158, 0.198758544842824, 0.11128066910778, 0.104008515538234, 0.153888727424811, 5.00241514142446, -11.3487406799524, 13.6512361757966, -9.65546182492158, 0.494378227169626, 0.191170602922265, 0.153521289707957, 0.206368245594612, -0.0648557912384922, -0.294210513499079, 4.94093995929797, 0.072607948401529, -2.60689130666783, 0.947458978837542, -1.17091271238656, 3.01117640477567, -4.94093995929797, -0.743125520653272, 2.60689130666783, -0.947458978837541, -0.115767452711869, -0.0507286897117851, -0.0555578588264061, -0.0735179302979083, -0.0439264739612257, -0.0938462418447227, -2.91367024641904, -1.27675499797774, -1.39829698611415, 7.49758186185502, 4.47975525280367, -12.3174261742278, 0.773567231976011, 0.46220127164904, -2.71696589127659, 0.67051757225174},
    {-0.921992874650237, -0.0508201537761847, 0.0110004824460652, -0.00590658279383117, -0.0121469776576497, -0.0578765020203708, 0.0472458364883293, 0.521409434336841, -1.63351367642431, 2.64740738044188, 1.00949188137988, 0.0904065681741657, 0.0522042791302084, 0.0578765020203701, 4.33565424042011, -7.98072916563719, 7.02038040494667, -2.64740738044188, 0.428629658865405, 0.183205852336289, 0.152440957403005, 0.208536559821724, 0.331894497016479, -2.46363494135174, 12.6627476236861, 0.119863119482081, -1.45972009203279, 0.404413103840402, -5.080000318028, 10.5880071387386, -12.6627476236861, -0.515137832901299, 1.45972009203278, -0.404413103840399, -0.502395150729083, -0.243103759388451, -0.274993453708418, -0.0509721760505685, -0.0335278093297714, -0.0400573014725569, -2.15773074137811, -1.04410334020107, -1.18106599532925, 4.49961922300978, 2.95970050829058, -5.38686672852236, 2.86415772412316, 1.88394809688836, -8.12437219738687, 0.39527471341921},
    {-0.942112680825338, 0.127595092614206, -0.438855364505299, 0.406648411251741, 0.0166529258255823, 0.020484775196608, -0.619074453405162, 1.05732821218668, -0.642176905529871, -0.187994765484727, 2.31225141453113, -1.47759207750688, -0.0378977438878135, -0.0204847751966092, 3.26179396758608, -3.84189325821416, 1.46142822896921, 0.18799476548473, 0.322633511670087, 0.158276461917953, 0.138772873632385, 0.194834740990708, 2.35567182514294, -5.00765368314271, 5.86326077670041, -2.71684597198953, 11.1338297871503, -0.21448481293619, -8.55953676376871, 11.3961221439565, -5.86326077670042, 6.18283741292547, -11.1338297871503, 0.214484812936197, -0.846866110426923, -0.469596891800335, -0.556933047798572, 0.611952852143322, 0.45899081411181, 0.0212448180622336, -1.19463772537199, -0.662440208381093, -0.785641580427842, 1.59114113616611, 1.19342390986137, -0.819251323439342, 3.5449790340321, 2.65888590459367, -6.38846846081378, -3.46599144093594},
    {-0.965098624043574, 0.405296087049496, -1.8058327973004, 3.03701648093235, -2.20785138931024, 0.288623553688348, -0.93841284355432, 0.471047119163694, 0.102826734145341, 0.105852187397973, 3.84258032344358, -5.59493014084545, 3.30575581164892, -0.288623553688348, 1.99682203872871, -0.867784465877589, -0.153959671216829, -0.105852187397972, 0.197623962341602, 0.110960259772188, 0.102816977085583, 0.148401337794708, 4.60986152393411, -2.7809398548364, -0.993408479653097, -9.31731545486158, 9.19790865284457, 11.08429471871, -8.49249694472012, 4.16382557788167, 0.993408479653096, 13.9505629151505, -9.19790865284457, -11.08429471871, -0.840622559885433, -0.532506111944681, -0.663618854313062, 1.38119467247681, 1.17671898743628, -1.09790442233869, -0.436835020361394, -0.276720289645356, -0.344853885167637, 0.214225960102356, 0.182511386611539, 0.0511329370714884, 2.09650366327904, 1.78613175750697, -1.38288572304525, -4.63324746028893},
    {-0.985708641666339, 0.729420397946709, -4.03187699731423, 9.20606324283965, -11.1959607479293, 8.16976222157872, -0.636102496984869, -0.146637218987192, -0.105061706660274, -0.142219700439307, 5.25214046484748, -11.4243799830648, 12.9271136018471, -8.16976222157871, 0.828621425313898, 0.181971301432524, 0.121306661194981, 0.142219700439308, 0.0820482743635217, 0.0511932222823162, 0.0496710434458241, 0.0733757036279708, 3.79720328645516, 1.00944198889805, 1.03954239228301, -9.99840973700673, -3.82040188607881, 17.4774853307227, -4.71218718284898, -1.16552492088478, -1.03954239228302, 11.5443936807302, 3.82040188607881, -17.4774853307227, -0.466618508176272, -0.32791943375921, -0.425725525597764, 1.14327225337303, 1.07504448685207, -1.73115285391772, -0.0736176223600191, -0.0517353011421263, -0.0671660048268821, -0.0182104184337602, -0.0171236640115731, -0.0162449545347064, 0.471562821512987, 0.443421074880837, 0.156082931986727, -1.54598394372351}};
    
    // Array of non-zero columns
    static const unsigned int nzc1[50] = {0, 1, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55};
    
    // Number of operations to compute geometry constants: 36
    const double G0 = det*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21 + Jinv_02*Jinv_22);
    const double G1 = det*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11 + Jinv_02*Jinv_12);
    const double G2 = det*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21 + Jinv_12*Jinv_22);
    const double G3 = det*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11 + Jinv_12*Jinv_12);
    const double G4 = det*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21 + Jinv_22*Jinv_22);
    const double G5 = det*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01 + Jinv_02*Jinv_02);
    
    // Compute element tensor using UFL quadrature representation
    // Optimisations: ('simplify expressions', True), ('ignore zero tables', True), ('non zero columns', True), ('remove zero terms', True), ('ignore ones', True)
    // Total number of operations to compute element tensor: 8551161
    
    // Loop quadrature points for integral
    // Number of operations to compute element tensor for following IP loop = 8551125
    for (unsigned int ip = 0; ip < 125; ip++)
    {
      
      // Number of operations to compute ip constants: 6
      // Number of operations: 1
      const double Gip0 = G0*W125[ip];
      
      // Number of operations: 1
      const double Gip1 = G1*W125[ip];
      
      // Number of operations: 1
      const double Gip2 = G2*W125[ip];
      
      // Number of operations: 1
      const double Gip3 = G3*W125[ip];
      
      // Number of operations: 1
      const double Gip4 = G4*W125[ip];
      
      // Number of operations: 1
      const double Gip5 = G5*W125[ip];
      
      
      // Number of operations for primary indices = 7803
      for (unsigned int j = 0; j < 51; j++)
      {
        for (unsigned int k = 0; k < 51; k++)
        {
          // Number of operations to compute entry = 3
          A[nzc0[j]*56 + nzc0[k]] += FE0_D001[ip][j]*FE0_D001[ip][k]*Gip4;
        }// end loop over 'k'
      }// end loop over 'j'
      
      // Number of operations for primary indices = 15300
      for (unsigned int j = 0; j < 51; j++)
      {
        for (unsigned int k = 0; k < 50; k++)
        {
          // Number of operations to compute entry = 3
          A[nzc0[j]*56 + nzc2[k]] += FE0_D001[ip][j]*FE0_D010[ip][k]*Gip2;
          // Number of operations to compute entry = 3
          A[nzc0[j]*56 + nzc1[k]] += FE0_D001[ip][j]*FE0_D100[ip][k]*Gip0;
        }// end loop over 'k'
      }// end loop over 'j'
      
      // Number of operations for primary indices = 30000
      for (unsigned int j = 0; j < 50; j++)
      {
        for (unsigned int k = 0; k < 50; k++)
        {
          // Number of operations to compute entry = 3
          A[nzc1[j]*56 + nzc2[k]] += FE0_D010[ip][k]*FE0_D100[ip][j]*Gip1;
          // Number of operations to compute entry = 3
          A[nzc2[j]*56 + nzc1[k]] += FE0_D010[ip][j]*FE0_D100[ip][k]*Gip1;
          // Number of operations to compute entry = 3
          A[nzc2[j]*56 + nzc2[k]] += FE0_D010[ip][j]*FE0_D010[ip][k]*Gip3;
          // Number of operations to compute entry = 3
          A[nzc1[j]*56 + nzc1[k]] += FE0_D100[ip][j]*FE0_D100[ip][k]*Gip5;
        }// end loop over 'k'
      }// end loop over 'j'
      
      // Number of operations for primary indices = 15300
      for (unsigned int j = 0; j < 50; j++)
      {
        for (unsigned int k = 0; k < 51; k++)
        {
          // Number of operations to compute entry = 3
          A[nzc1[j]*56 + nzc0[k]] += FE0_D001[ip][k]*FE0_D100[ip][j]*Gip0;
          // Number of operations to compute entry = 3
          A[nzc2[j]*56 + nzc0[k]] += FE0_D001[ip][k]*FE0_D010[ip][j]*Gip2;
        }// end loop over 'k'
      }// end loop over 'j'
    }// end loop over 'ip'
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class poisson3d_5_0_cell_integral_0: public ufc::cell_integral
{
private:

  poisson3d_5_0_cell_integral_0_quadrature integral_0_quadrature;

public:

  /// Constructor
  poisson3d_5_0_cell_integral_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_0_cell_integral_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Reset values of the element tensor block
    for (unsigned int j = 0; j < 3136; j++)
      A[j] = 0;
    
    // Add all contributions to element tensor
    integral_0_quadrature.tabulate_tensor(A, w, c);
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class poisson3d_5_form_0: public ufc::form
{
public:

  /// Constructor
  poisson3d_5_form_0() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_form_0()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "Form([Integral(IndexSum(Product(SpatialDerivative(BasisFunction(FiniteElement('Lagrange', Cell('tetrahedron', 1, Space(3)), 5), 0), MultiIndex((Index(0),), {Index(0): 3})), SpatialDerivative(BasisFunction(FiniteElement('Lagrange', Cell('tetrahedron', 1, Space(3)), 5), 1), MultiIndex((Index(0),), {Index(0): 3}))), MultiIndex((Index(0),), {Index(0): 3})), Measure('cell', 0, None))])";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 2;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 0;
  }

  /// Return the number of cell integrals
  virtual unsigned int num_cell_integrals() const
  {
    return 1;
  }

  /// Return the number of exterior facet integrals
  virtual unsigned int num_exterior_facet_integrals() const
  {
    return 0;
  }

  /// Return the number of interior facet integrals
  virtual unsigned int num_interior_facet_integrals() const
  {
    return 0;
  }

  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new poisson3d_5_0_finite_element_0();
      break;
    case 1:
      return new poisson3d_5_0_finite_element_1();
      break;
    }
    return 0;
  }

  /// Create a new dof map for argument function i
  virtual ufc::dof_map* create_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new poisson3d_5_0_dof_map_0();
      break;
    case 1:
      return new poisson3d_5_0_dof_map_1();
      break;
    }
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    return new poisson3d_5_0_cell_integral_0();
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return 0;
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

/// This class defines the interface for a finite element.

class poisson3d_5_1_finite_element_0: public ufc::finite_element
{
public:

  /// Constructor
  poisson3d_5_1_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_1_finite_element_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "FiniteElement('Lagrange', 'tetrahedron', 5)";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 56;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
    
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
    
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    static const double coefficients0[56][56] = \
    {{0.00945067404923492, 0.00609788120679252, 0.00352061335622803, 0.00248944957812476, 0.0070594623332504, 0.00546823600997146, 0.00315708753234943, 0.00446479600584755, 0.00257775117585285, 0.00182274533665715, -0.00298267418725716, -0.0025208197798293, -0.00195261860521383, -0.00112734487734488, -0.00218309396769447, -0.00169101731601731, -0.000976309302606919, -0.00138070985681772, -0.000797153207506487, -0.00056367243867244, 0.00278687112460004, 0.00245778931055954, 0.00207721109304566, 0.00160900079398463, 0.000928957041533343, 0.00219831358911339, 0.0018579140830667, 0.00143913406048032, 0.00083088443721827, 0.00160900079398463, 0.0012463266558274, 0.000719567030240151, 0.00101762145320217, 0.000587524019939407, 0.000415442218609132, -0.00705593524717798, -0.00638233357089081, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363026, -0.00582624677716565, -0.00513826668309062, -0.00434262794914383, -0.00336378514516178, -0.00194208225905522, -0.00459580543596519, -0.00388416451811043, -0.00300866089851429, -0.00173705117965752, -0.00336378514516178, -0.00260557676948629, -0.00150433044925714, -0.00212744452363027, -0.00122828066840393, -0.000868525589828762},
    {0.00945067404923495, -0.00609788120679253, 0.00352061335622802, 0.00248944957812476, 0.00705946233325039, -0.00546823600997146, 0.00315708753234944, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521383, -0.00112734487734489, -0.00218309396769448, 0.00169101731601732, -0.000976309302606925, 0.00138070985681772, -0.000797153207506488, -0.000563672438672441, 0.00278687112460004, -0.00245778931055954, 0.00207721109304566, -0.00160900079398463, 0.000928957041533346, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218268, 0.00160900079398463, -0.0012463266558274, 0.000719567030240148, -0.00101762145320216, 0.000587524019939402, 0.000415442218609128, 0.00705593524717798, -0.0063823335708908, 0.00562868913761196, -0.00475711057319694, 0.00368484200521179, -0.00212744452363027, -0.00582624677716565, 0.00513826668309062, -0.00434262794914382, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811043, 0.00300866089851429, -0.00173705117965752, -0.00336378514516178, 0.00260557676948629, -0.00150433044925714, 0.00212744452363026, -0.00122828066840393, -0.00086852558982876},
    {0.00945067404923495, 0, -0.00704122671245605, 0.00248944957812476, 0, 0, 0.00947126259704828, 0, -0.00515550235170572, 0.00182274533665715, 0, 0, 0, 0.00450937950937951, 0, 0, -0.00292892790782075, 0, 0.00159430641501296, -0.00056367243867244, 0, 0, 0, 0, 0.00464478520766673, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987881, 0.00041544221860913, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527608, 0, 0, 0, 0.00694820471863012, 0, 0, -0.00451299134777144, 0, 0.00245656133680786, -0.000868525589828763},
    {0.00945067404923493, 0, 0, -0.0074683487343743, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00563672438672439, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.018239037386404},
    {-0.010023442173431, 0, 0.0374718748311399, -0.0289614183593723, 0, 0, 0.0284137877911448, 0, 0.00515550235170572, -0.00619733414463434, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835494, -0.000187890812890802, 0, 0, 0, 0, 0.0232239260383337, 0, 0, 0, -0.00166176887443652, 0, 0, -0.00647610327216137, 0, 0.00646276421933342, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496074, 0, 0, 0, -0.0277928188745205, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.010023442173431, 0, -0.0287574853355259, 0.05360943398437, 0, 0, -0.0189425251940966, 0, 0.0154665070551171, 0.0153110608279201, 0, 0, 0, -0.00450937950937953, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144092, 0, -0.0123380044187274, 0.00872428659079174, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559316, 0, 0.041270230458372, -0.018239037386404},
    {0.010023442173431, 0, 0.0409576306293854, -0.0449826285156208, 0, 0, 0.0189425251940965, 0, -0.00515550235170569, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883226, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957583, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.055026973944496, 0.0303983956440067},
    {-0.0100234421734311, 0, -0.0148144621425437, 0.0449826285156207, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0187359374155699, -0.0289614183593723, 0.0211783869997512, -0.0164047080299144, 0.00947126259704827, 0.00446479600584757, -0.00257775117585285, -0.00619733414463432, 0.0208787193108001, -0.0176457384588052, 0.0136683302364968, -0.00789141414141413, -0.0084898098743674, 0.00657617845117844, -0.00379675839902689, 0.0023011830946962, -0.00132858867917746, -0.0001878908128908, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992314, 0.00464478520766674, -0.0010991567945567, 0.000928957041533349, -0.000719567030240152, 0.000415442218609127, -0.00482700238195388, 0.00373897996748219, -0.00215870109072045, 0.00559691799261185, -0.00323138210966672, -0.00290809553026392, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935591, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863011, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778515, 0.00607967912880135},
    {0.010023442173431, -0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, 0.0109364720199429, -0.00631417506469883, 0.0133943880175427, -0.00773325352755858, 0.0153110608279201, -0.00298267418725716, 0.00252081977982931, -0.00195261860521382, 0.00112734487734488, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476002, 0.0129522065443227, -0.00747795993496437, -0.00321800158796926, 0.0024926533116548, -0.00143913406048031, -0.0106850252586226, 0.00616900220936373, 0.00872428659079178, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845888, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.020635115229186, -0.0182390373864041},
    {0.0100234421734311, 0.0354703486038674, -0.0204788153146927, -0.0449826285156208, 0.0141189246665008, -0.0109364720199429, 0.00631417506469885, -0.00446479600584758, 0.00257775117585285, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974696, -0.00263047138047138, 0.00151870335961075, 0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620757, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, -0.0128297005588457, 0.00740723107127184, 0.0449826285156208, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0.00920473237878476, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489395, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.010023442173431, 0.0324515955311979, -0.0335503995581136, -0.00801060507812427, 0.0211783869997512, 0.00109364720199429, -0.00568275755822898, -0.0169662248222207, 0.0056710525868763, 0.00838462854862291, 0.0208787193108001, -0.0109235523792603, 0.00499002532443534, -0.00187890812890812, -0.0162519217595033, 0.00958243145743146, -0.00466458889023305, 0.0108922666482287, -0.00558007245254534, -0.00469727032227033, 0.0139343556230002, -0.00245778931055954, -0.00415442218609131, 0.00643600317593851, -0.00464478520766673, -0.0120907247401236, 0.00464478520766674, -0.000719567030240153, -0.000415442218609137, 0.00965400476390778, -0.00498530662330958, 0.00215870109072046, -0.006614539445814, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543313, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438609, 0.00310733161448835, -0.00481385743762287, 0.00347410235931505, -0.00201827108709706, -0.000521115353897262, 0.000902598269554289, 0.00170195561890421, -0.000491312267361565, -0.000868525589828764},
    {0.0100234421734309, -0.024904712849524, 0.0553363732971484, -0.0043134027343746, -0.0141189246665008, 0.0153110608279201, 0.00883984509057843, 0.00803663281052563, -0.0159820572902877, 0.000729098134662861, -0.00298267418725715, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846298, -0.0116492303992304, 0.00976309302606916, 0.00352848074520084, 0.00132858867917745, -0.00263047138047137, 0, 0.0196623144844763, -0.00727023882565982, -0.00724050357293082, 0.00928957041533346, -0.00219831358911338, -0.0148633126645336, 0.0129522065443228, -0.0066470754977461, 0.00402250198496157, 0.00685479660705067, -0.00575653624192123, -0.0045792965394097, -0.000293762009969684, 0.00290809553026392, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.00411061334647249, -0.00303983956440068, 0.00908221989193681, -0.00776832903622088, 0.00045958054359652, 0.00427258096992148, -0.00270779480866287, 0.000868525589828773, -0.00100913554354854, -0.00234501909253766, 0.00180519653910858, 0.00127646671417816, 0.000245656133680781, -0.000868525589828764},
    {0.0100234421734309, 0.0354703486038675, -0.0492363006502187, -0.00431340273437461, 0.0141189246665008, -0.00656188321196574, -0.016416855168217, -0.00982255121286466, 0.0149509568199466, 0.000729098134662872, 0, 0.0033610930397724, 0.0282044909641998, -0.0353234728234728, -0.000485131992820993, -0.0124007936507936, 0.00889526253486299, 0.00291483191994851, 0.00239145962251946, -0.00263047138047138, 0, 0, 0.0218107164769794, -0.00402250198496158, -0.00928957041533347, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992316, 0.00934744991870547, -0.0028782681209606, -0.00254405363300539, -0.00381890612960613, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320202, 0.00840946286290443, -0.00388416451811042, 0, 0.00233049871086626, 0.0030086608985143, -0.00347410235931507, -0.000336378514516178, -0.00286613444643493, 0.00120346435940572, 0.00085097780945211, 0.00098262453472315, -0.000868525589828765},
    {-0.010023442173431, -0.0128297005588457, 0.0448791059024117, -0.00801060507812424, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467804, -0.0175287079957994, 0.0083846285486229, 0, 0, 0.00867830491206149, 0.0225468975468975, 0, -0.00300625300625301, -0.019200749617936, 0.000613648825252316, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240152, -0.0108014976838374, 0, 0.00249265331165478, 0.0107935054536023, -0.000508810726601074, -0.00734405024924252, 0.00290809553026393, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931505, 0, -0.00208446141558903, -0.000902598269554287, 0.000425488904726055, 0.0017195929357655, -0.000868525589828767},
    {-0.0100234421734311, -0.0324515955311978, -0.0187359374155699, -0.0289614183593723, 0.0211783869997511, 0.0164047080299144, 0.00947126259704826, -0.00446479600584755, -0.00257775117585285, -0.00619733414463435, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141413, -0.00848980987436742, -0.00657617845117845, -0.0037967583990269, -0.00230118309469619, -0.00132858867917746, -0.00018789081289081, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992313, 0.00464478520766674, 0.00109915679455668, 0.000928957041533345, 0.000719567030240154, 0.000415442218609131, -0.00482700238195388, -0.00373897996748219, -0.00215870109072045, -0.00559691799261187, -0.00323138210966673, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.00955378148811641, 0.00740032731935591, 0.00427258096992148, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.0069482047186301, 0.0181644397838736, 0.014070114555226, 0.00812338442598859, 0.0136156449512337, 0.00786099627778516, 0.00607967912880135},
    {0.0100234421734308, 0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, -0.0109364720199429, -0.00631417506469886, -0.0133943880175427, -0.00773325352755858, 0.0153110608279201, 0.00298267418725716, 0.00252081977982933, 0.00195261860521383, 0.00112734487734487, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476002, -0.0129522065443228, -0.00747795993496438, -0.00321800158796927, -0.00249265331165479, -0.00143913406048031, 0.0106850252586227, 0.00616900220936374, 0.00872428659079177, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.0206351152291861, -0.0182390373864041},
    {0.010023442173431, -0.0354703486038674, -0.0204788153146927, -0.0449826285156208, 0.0141189246665008, 0.0109364720199429, 0.00631417506469886, 0.00446479600584758, 0.00257775117585287, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974696, 0.00263047138047138, 0.00151870335961075, -0.0308358534689291, -0.0178030883009781, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620757, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588457, 0.00740723107127183, 0.0449826285156208, 0, 0, 0, -0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878479, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489395, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.010023442173431, -0.0324515955311978, -0.0335503995581136, -0.00801060507812424, 0.0211783869997512, -0.00109364720199429, -0.00568275755822899, 0.0169662248222207, 0.00567105258687632, 0.0083846285486229, -0.0208787193108001, -0.0109235523792603, -0.00499002532443534, -0.00187890812890812, -0.0162519217595033, -0.00958243145743146, -0.00466458889023305, -0.0108922666482287, -0.00558007245254535, -0.00469727032227032, 0.0139343556230002, 0.00245778931055953, -0.00415442218609133, -0.00643600317593852, -0.00464478520766672, 0.0120907247401236, 0.00464478520766673, 0.000719567030240156, -0.000415442218609149, 0.00965400476390777, 0.00498530662330958, 0.00215870109072047, 0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543313, 0.00719357335632687, 0.0112908326677739, 0.0114368694935501, 0.00737991258440984, -0.00183832217438608, 0.00310733161448834, 0.00481385743762287, 0.00347410235931505, -0.00201827108709706, 0.000521115353897262, 0.000902598269554287, -0.00170195561890421, -0.000491312267361565, -0.000868525589828767},
    {0.010023442173431, 0.0249047128495239, 0.0553363732971484, -0.00431340273437459, -0.0141189246665008, -0.0153110608279201, 0.00883984509057843, -0.00803663281052563, -0.0159820572902878, 0.000729098134662854, 0.00298267418725716, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846298, 0.0116492303992304, 0.00976309302606918, -0.00352848074520084, 0.00132858867917746, -0.00263047138047138, 0, -0.0196623144844763, -0.0072702388256598, 0.00724050357293083, 0.00928957041533346, 0.00219831358911339, -0.0148633126645336, -0.0129522065443228, -0.00664707549774609, 0.00402250198496157, -0.00685479660705067, -0.00575653624192124, 0.0045792965394097, -0.000293762009969679, 0.00290809553026392, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.0041106133464725, -0.00303983956440067, -0.00908221989193682, -0.00776832903622089, -0.00045958054359652, 0.00427258096992148, 0.00270779480866287, 0.000868525589828776, -0.00100913554354853, 0.00234501909253766, 0.00180519653910858, -0.00127646671417816, 0.000245656133680779, -0.000868525589828761},
    {0.0100234421734309, -0.0354703486038674, -0.0492363006502187, -0.00431340273437459, 0.0141189246665008, 0.00656188321196577, -0.016416855168217, 0.00982255121286466, 0.0149509568199466, 0.000729098134662868, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820994, 0.0124007936507936, 0.00889526253486299, -0.00291483191994852, 0.00239145962251945, -0.00263047138047138, 0, 0, 0.0218107164769794, 0.00402250198496157, -0.00928957041533346, 0, -0.00557374224920009, 0.0143913406048031, 0.0141250354327105, 0.000804500396992309, -0.00934744991870548, -0.0028782681209606, 0.00254405363300539, -0.00381890612960614, 0.00290809553026393, 0, 0, 0, 0.0266398192099028, 0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320203, -0.00840946286290443, -0.00388416451811041, 0, 0.00233049871086626, -0.0030086608985143, -0.00347410235931507, -0.000336378514516177, 0.00286613444643493, 0.00120346435940572, -0.000850977809452108, 0.000982624534723152, -0.000868525589828766},
    {-0.010023442173431, 0.0128297005588457, 0.0448791059024117, -0.00801060507812424, 0, -0.0174983552319087, 0.0132597676358676, 0.00357183680467805, -0.0175287079957994, 0.0083846285486229, 0, 0, -0.00867830491206148, 0.0225468975468975, 0, 0.00300625300625301, -0.019200749617936, -0.000613648825252319, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766673, 0, 0, 0.00719567030240153, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601077, -0.00734405024924252, 0.00290809553026393, 0, 0, 0, 0, -0.0257938940364826, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702859, -0.00347410235931505, 0, 0.00208446141558903, -0.000902598269554286, -0.000425488904726054, 0.0017195929357655, -0.000868525589828767},
    {-0.0100234421734311, -0.0452812960900435, -0.0113287063442981, -0.00801060507812424, -0.0014118924666501, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.00838462854862289, -0.00894802256177147, -0.0142846454190328, -0.014536160727703, -0.00939454064454064, -0.0123708658169354, -0.0125886844636845, -0.00813591085505764, -0.0102786178229763, -0.00664294339588733, -0.00469727032227033, -0.00836061337380012, 0.00245778931055953, 0.00830884437218264, 0.00965400476390777, 0.00650269929073344, 0.00219831358911338, 0.00743165633226677, 0.00863480436288183, 0.00581619106052785, 0.00643600317593852, 0.00747795993496437, 0.00503696921168108, 0.00610572871921293, 0.00411266813957582, 0.00290809553026394, 0.0352796762358899, 0.0191470007126724, 0.00788016479265675, 0.000951422114639391, -0.00221090520312707, -0.00212744452363027, 0.017478740331497, 0.00719357335632686, 0.000868525589828764, -0.00201827108709706, -0.00194208225905521, 0.00643412761035128, 0.000776832903622088, -0.00180519653910857, -0.00173705117965753, 0.000672757029032363, -0.00156334606169177, -0.00150433044925715, -0.00127646671417816, -0.00122828066840393, -0.000868525589828768},
    {0.010023442173431, 0.0603750614533914, -0.00610007264692975, -0.0043134027343746, -0.00564756986660025, -0.0218729440398859, 0.00126283501293978, -0.0178591840233903, 0.00103110047034114, 0.000729098134662884, 0.0417574386216002, 0.0168054651988621, -0.000867830491206148, -0.00526094276094276, 0.0145539597846299, -0.000751563251563252, -0.00455611007883227, -0.000613648825252328, -0.0037200483016969, -0.00263047138047138, 0.00557374224920005, -0.0196623144844763, -0.0103860554652283, 0.00321800158796928, 0.00650269929073341, -0.0175865087129071, -0.00928957041533347, 0.00287826812096063, 0.00581619106052784, -0.00804500396992315, 0.0024926533116548, 0.00503696921168106, 0.00203524290640433, 0.00411266813957581, 0.00290809553026391, -0.0705593524717798, -0.0127646671417816, 0.00450295131008958, 0.00380568845855754, -0.000736968401042372, -0.00212744452363026, -0.0116524935543313, 0.00411061334647251, 0.00347410235931506, -0.000672757029032362, -0.00194208225905521, 0.00367664434877218, 0.00310733161448835, -0.000601732179702865, -0.00173705117965753, 0.00269102811612942, -0.000521115353897265, -0.00150433044925715, -0.00042548890472606, -0.00122828066840393, -0.000868525589828759},
    {0.0100234421734309, -0.0603750614533914, -0.00610007264692972, -0.00431340273437457, -0.00564756986660032, 0.0218729440398859, 0.00126283501293976, 0.0178591840233903, 0.00103110047034113, 0.000729098134662861, -0.0417574386216002, 0.016805465198862, 0.000867830491206157, -0.00526094276094276, 0.0145539597846298, 0.000751563251563257, -0.00455611007883228, 0.00061364882525233, -0.00372004830169691, -0.00263047138047138, 0.00557374224920011, 0.0196623144844763, -0.0103860554652283, -0.00321800158796927, 0.00650269929073344, 0.0175865087129071, -0.00928957041533346, -0.00287826812096062, 0.00581619106052786, -0.00804500396992314, -0.0024926533116548, 0.00503696921168108, -0.00203524290640433, 0.00411266813957583, 0.00290809553026393, 0.0705593524717798, -0.0127646671417816, -0.00450295131008958, 0.00380568845855756, 0.000736968401042369, -0.00212744452363027, -0.0116524935543313, -0.00411061334647251, 0.00347410235931506, 0.000672757029032361, -0.00194208225905522, -0.00367664434877217, 0.00310733161448835, 0.000601732179702866, -0.00173705117965753, 0.00269102811612942, 0.000521115353897263, -0.00150433044925715, 0.000425488904726061, -0.00122828066840394, -0.000868525589828769},
    {-0.0100234421734309, 0.0452812960900435, -0.0113287063442981, -0.00801060507812426, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089231, 0.00838462854862291, 0.00894802256177147, -0.0142846454190327, 0.014536160727703, -0.00939454064454063, -0.0123708658169354, 0.0125886844636845, -0.00813591085505763, 0.0102786178229763, -0.00664294339588732, -0.00469727032227032, -0.00836061337380015, -0.00245778931055953, 0.00830884437218263, -0.00965400476390777, 0.00650269929073343, -0.00219831358911339, 0.00743165633226677, -0.00863480436288183, 0.00581619106052785, 0.00643600317593852, -0.00747795993496437, 0.00503696921168107, -0.00610572871921293, 0.00411266813957582, 0.00290809553026393, -0.0352796762358899, 0.0191470007126724, -0.00788016479265675, 0.000951422114639385, 0.00221090520312707, -0.00212744452363027, 0.0174787403314969, -0.00719357335632687, 0.000868525589828768, 0.00201827108709706, -0.00194208225905521, -0.00643412761035128, 0.000776832903622089, 0.00180519653910857, -0.00173705117965753, 0.000672757029032358, 0.00156334606169177, -0.00150433044925715, 0.00127646671417816, -0.00122828066840393, -0.000868525589828765},
    {0.0787556170769579, 0.0679219441350653, -0.00435719474780691, -0.00308100195312472, 0.0705946233325039, -0.0328094160598287, 0.0126283501293977, -0.0267887760350854, 0.03608851646194, -0.00729098134662862, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015873, 0.0130985638061669, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, 0.0245778931055954, -0.036351194128299, 0.0362025178646541, -0.0232239260383337, 0.0219831358911338, -0.00743165633226679, 0, 0.00166176887443652, -0.0345935170706695, 0.0130864298861876, -0.00359783515120079, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596521, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598858, -0.0102117337134253, 0.00196524906944628, 0.00607967912880136},
    {-0.0214788046573522, 0.00377344134083702, 0.00217859737390344, -0.0215670136718729, 0, 0.0984282481794862, -0.0852413633734345, -0.0312535720409329, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, 0.0236742424242424, -0.0335199527228374, -0.00138070985681772, -0.00079715320750645, 0.00394570707070704, 0, 0, 0.036351194128299, -0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.00724050357293084, -0.0392592896585629, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724418, 0, 0.00694820471863008, 0.00302740663064561, 0.0164151336477636, -0.0108311792346515, -0.00680782247561686, -0.00393049813889256, 0.00607967912880134},
    {0.0787556170769579, 0.0301875307266957, 0.0610007264692975, -0.00308100195312472, 0, 0.0218729440398858, 0.0757701007763863, 0.0178591840233903, -0.0412440188136458, -0.00729098134662862, 0, 0, 0.0520698294723688, -0.00901875901875902, 0, -0.0127765752765753, 0.0286384062098028, 0.00184094647575699, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.0143913406048031, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515274, 0.00407048581280861, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967795, -0.0427258096992148, 0, 0, -0.0300866089851429, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723144, 0.00607967912880136},
    {-0.0214788046573522, 0.00377344134083686, -0.0196073763651313, 0.00924300585937417, 0, -0.0109364720199429, 0.00947126259704823, 0.102690308134494, -0.0695992817480272, -0.060150596109686, 0, -0.00504163955965862, 0.0065087286840461, -0.00450937950937951, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196709, -0.0263060558477138, 0.000563672438672465, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165478, -0.0043174021814409, -0.0605484764655283, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573521, -0.0150937653633479, 0.0130715842434208, 0.00924300585937414, 0, -0.0109364720199429, 0.00947126259704832, -0.00892959201169514, 0.123732056440937, -0.0601505961096861, 0, 0, -0.00650872868404609, 0.00676406926406927, 0, 0.0574945887445887, 0.0126920209338899, -0.0266937238984759, 0.00637722566005184, 0.000563672438672453, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097862, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448302, -0.0616900220936373, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769578, 0.0301875307266958, 0.0174287789912279, 0.0585390371093694, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094276, -0.00455611007883226, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700864, 0.0284934006896604, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769578, -0.0679219441350653, -0.00435719474780693, -0.00308100195312472, 0.0705946233325039, 0.0328094160598287, 0.0126283501293978, 0.0267887760350854, 0.03608851646194, -0.00729098134662859, -0.0298267418725716, 0.0151249186789758, 0.0325436434202305, 0.0248015873015873, 0.0130985638061669, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646541, -0.0232239260383337, -0.0219831358911339, -0.00743165633226677, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861877, -0.00359783515120079, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.0045958054359652, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.00908221989193679, -0.00703505727761299, -0.00812338442598858, 0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573524, -0.00377344134083698, 0.00217859737390344, -0.021567013671873, 0, -0.0984282481794863, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242424, -0.0335199527228374, 0.0013807098568177, -0.00079715320750645, 0.00394570707070706, 0, 0, 0.036351194128299, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838375, -0.00724050357293083, 0.039259289658563, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951694, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.00302740663064561, -0.0164151336477636, -0.0108311792346515, 0.00680782247561685, -0.00393049813889256, 0.00607967912880134},
    {0.0787556170769579, -0.0301875307266956, 0.0610007264692975, -0.00308100195312471, 0, -0.0218729440398858, 0.0757701007763863, -0.0178591840233903, -0.0412440188136458, -0.00729098134662861, 0, 0, -0.0520698294723689, -0.00901875901875903, 0, 0.0127765752765753, 0.0286384062098028, -0.00184094647575695, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766674, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515274, -0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967796, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780843, -0.00982624534723145, 0.00607967912880137},
    {-0.0214788046573523, -0.0037734413408369, -0.0196073763651314, 0.00924300585937414, 0, 0.0109364720199429, 0.00947126259704826, -0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965861, -0.00650872868404608, -0.00450937950937948, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196707, -0.0263060558477138, 0.000563672438672444, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.0024926533116548, -0.00431740218144091, 0.0605484764655282, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, 0.0150937653633478, 0.0130715842434208, 0.00924300585937414, 0, 0.0109364720199429, 0.00947126259704831, 0.00892959201169514, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404611, 0.00676406926406927, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005185, 0.000563672438672461, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097862, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448302, -0.0616900220936373, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769578, -0.0301875307266956, 0.0174287789912279, 0.0585390371093695, 0, -0.0218729440398858, -0.0189425251940966, -0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, -0.00526094276094275, -0.00455611007883226, -0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700864, -0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769576, -0.0377344134083696, -0.0566435317214905, -0.00308100195312473, 0.0423567739995023, 0.0546823600997146, 0.0378850503881931, 0.0446479600584756, 0.00515550235170574, -0.00729098134662862, 0.0417574386216002, -0.0050416395596586, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801536, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055954, -0.00519302773261415, 0.00402250198496157, 0.00464478520766674, -0.00219831358911335, -0.0130053985814669, -0.0158304746652834, -0.0108014976838374, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.00477689074405821, 0.00370016365967795, 0.00427258096992148, -0.0367664434877216, -0.00776832903622087, 0.00601732179702858, 0.00694820471863011, -0.00908221989193682, 0.00703505727761298, 0.0081233844259886, 0.00680782247561684, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573522, 0, -0.0043571947478069, -0.0215670136718729, -0.127070321998507, 0, 0.0284137877911448, 0, 0.03608851646194, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984381, 0, 0.00159430641501295, 0.00394570707070706, 0.0836061337380013, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377579, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.0095537814881164, 0, 0.00427258096992147, 0, -0.0155366580724417, 0, 0.00694820471863011, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778513, 0.00607967912880133},
    {0.0787556170769579, 0.0377344134083696, -0.0566435317214905, -0.0030810019531247, 0.0423567739995023, -0.0546823600997146, 0.0378850503881931, -0.0446479600584757, 0.00515550235170574, -0.00729098134662864, -0.0417574386216002, -0.00504163955965862, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801536, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261416, -0.00402250198496158, 0.00464478520766674, 0.00219831358911339, -0.0130053985814669, 0.0158304746652834, -0.0108014976838374, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405821, -0.00370016365967795, 0.00427258096992148, 0.0367664434877216, -0.00776832903622088, -0.00601732179702859, 0.00694820471863012, -0.00908221989193681, -0.00703505727761299, 0.00812338442598861, -0.00680782247561685, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573522, -0.0188672067041847, 0.00653579212171051, 0.00924300585937415, 0.0141189246665008, 0, -0.00315708753234942, -0.111619900146189, -0.0541327746929101, -0.0601505961096861, -0.00894802256177146, -0.00252081977982933, 0.000650872868404606, 0.00112734487734486, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672439, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476002, -0.00431740218144093, -0.00747795993496438, 0.00321800158796924, 0.0224338798048931, 0.018708742786244, 0.046301776120698, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.017478740331497, -0.00451299134777145, -0.00781673030845887, 0.0363288795677472, -0.00938007637015066, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {-0.0214788046573521, 0.0188672067041847, 0.00653579212171047, 0.00924300585937414, 0.0141189246665007, 0, -0.00315708753234944, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177148, -0.00252081977982932, -0.000650872868404592, 0.00112734487734487, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.019928830187662, 0.000563672438672456, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476002, 0.00431740218144092, -0.00747795993496438, 0.00321800158796927, -0.0224338798048931, 0.018708742786244, -0.0463017761206981, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777143, -0.00781673030845888, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769579, 0, -0.0348575579824557, 0.0585390371093695, -0.0282378493330015, 0, 0.00631417506469885, 0, -0.0206220094068228, 0.083846285486229, 0, 0, 0, 0, -0.0067918478994939, 0, 0.00151870335961074, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233621, 0, -0.0329013451166066, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.0275134869722481, 0.0303983956440068},
    {0.0787556170769578, -0.0377344134083696, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, 0.0546823600997145, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865145, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.0092175078635989, -0.00789141414141412, -0.00347132196482459, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, -0.00491557862111904, -0.0197335053839338, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097694, -0.000623163327913701, -0.00215870109072047, 0.00712335017241508, 0.00411266813957581, 0.00540074884191871, 0, -0.0510586685671264, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173251, -0.0143871467126537, -0.00564541633388696, 0.00571843474677502, 0.00737991258440982, 0.00367664434877216, -0.00155366580724418, 0.00240692871881142, 0.00347410235931506, 0.00100913554354853, 0.00026055767694863, 0.000902598269554296, -0.000850977809452108, -0.000491312267361565, -0.000868525589828763},
    {-0.0214788046573521, 0, -0.0217859737390348, 0.00308100195312476, -0.127070321998507, 0, 0.0473563129852414, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468977, 0.0509388592462045, 0, -0.00943765659186683, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533349, 0, -0.0141250354327105, -0.0144810071458616, 0, -0.00215870109072044, 0, 0.00411266813957584, 0.00540074884191874, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259877, 0, -0.0112908326677739, 0, 0.00737991258440982, 0, -0.00310733161448833, 0, 0.00347410235931505, 0.00201827108709706, 0, 0.000902598269554279, 0, -0.000491312267361592, -0.00086852558982878},
    {0.078755617076958, 0.0377344134083695, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584757, 0.0257775117585286, 0.0291639253865145, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.0092175078635989, 0.00789141414141413, -0.00347132196482461, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111908, -0.0197335053839338, 0.0394205194526234, -0.0325134964536672, 0.0241814494802473, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097695, 0.000623163327913697, -0.00215870109072046, -0.0071233501724151, 0.00411266813957581, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908062, -0.0114170653756727, -0.010317557614593, 0.0127646671417816, 0.00466099742173252, 0.0143871467126538, -0.00564541633388697, -0.00571843474677502, 0.00737991258440983, -0.00367664434877217, -0.00155366580724418, -0.00240692871881144, 0.00347410235931506, 0.00100913554354853, -0.000260557676948628, 0.000902598269554287, 0.000850977809452107, -0.000491312267361569, -0.000868525589828763},
    {-0.0214788046573521, -0.0188672067041848, 0.0108929868695174, 0.0030810019531247, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792643, 0.0127592173566001, -0.0089480225617715, -0.0277290175781224, -0.00585785581564147, 0.0191648629148629, 0.00145539597846298, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755831, -0.0129644660894661, 0, 0.0589869434534289, 0.00727023882565979, 0.0506835250105158, 0.0650269929073344, -0.00659494076734015, 0.0148633126645335, -0.00431740218144092, -0.00166176887443655, -0.00402250198496158, -0.00810112326287807, -0.0115130724838425, 0.00356167508620755, -0.00205633406978791, 0.00540074884191871, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440066, -0.00302740663064559, -0.00776832903622087, 0.00137874163078956, -0.00427258096992148, 0.000902598269554293, 0.000868525589828758, 0.00100913554354854, 0.000781673030845887, 0.00180519653910857, -0.000425488904726058, 0.00024565613368079, -0.000868525589828762},
    {-0.0214788046573521, 0.0188672067041848, 0.0108929868695174, 0.00308100195312469, 0.0141189246665007, 0.109364720199429, -0.0789271883087357, -0.0223239800292378, -0.0128887558792643, 0.0127592173566001, 0.00894802256177149, -0.0277290175781224, 0.00585785581564149, 0.0191648629148629, 0.00145539597846298, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755832, -0.0129644660894661, 0, -0.0589869434534289, 0.00727023882565983, -0.0506835250105158, 0.0650269929073343, 0.00659494076734015, 0.0148633126645336, 0.00431740218144092, -0.00166176887443653, -0.00402250198496157, 0.00810112326287806, -0.0115130724838425, -0.00356167508620755, -0.00205633406978791, 0.00540074884191872, 0, 0, -0.0607898426862092, 0.0399597288148543, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440068, 0.0030274066306456, -0.00776832903622088, -0.00137874163078956, -0.00427258096992148, -0.000902598269554298, 0.000868525589828757, 0.00100913554354854, -0.000781673030845891, 0.00180519653910858, 0.000425488904726056, 0.000245656133680792, -0.000868525589828762},
    {0.078755617076958, 0, 0.0435719474780696, -0.0523770332031201, -0.0282378493330015, 0, 0.0631417506469885, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.00672218607954481, 0, -0.0601250601250601, 0.000970263985641994, 0, -0.0125835421224892, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539589, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398463, 0, -0.0028782681209606, 0, -0.00822533627915163, 0.00540074884191871, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.018239037386404, 0, -0.00388416451811042, 0, -0.00466099742173252, 0, -0.00347410235931505, 0.000672757029032356, 0, 0.00120346435940571, 0, 0.000982624534723144, -0.000868525589828765},
    {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, -0.0546823600997146, -0.0315708753234943, -0.0446479600584757, -0.0257775117585285, -0.0182274533665716, 0.0894802256177147, -0.0151249186789759, 0.0585785581564149, 0.0744047619047619, -0.0130985638061669, -0.0281836219336219, 0.0097630930260691, 0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226673, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978424, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905521, -0.00902598269554287, -0.0191075629762328, -0.00908221989193678, -0.00234501909253766, -0.00812338442598858, 0.00340391123780844, 0.00196524906944629, 0.00607967912880135},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, 0.0546823600997146, -0.0315708753234943, 0.0446479600584756, -0.0257775117585286, -0.0182274533665715, -0.0894802256177148, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061669, 0.0281836219336219, 0.00976309302606918, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.0363511941282991, 0.0120675059548847, -0.0232239260383337, -0.0659494076734016, 0.00743165633226682, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978425, 0.0251848460584054, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554289, -0.0191075629762328, -0.00908221989193679, 0.00234501909253767, -0.00812338442598859, -0.00340391123780842, 0.00196524906944628, 0.00607967912880136},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281177, 0, 0, -0.0947126259704827, 0, 0.0515550235170571, -0.0182274533665715, 0, -0.0907495120738551, 0, -0.0946969696969696, 0.0232863356554078, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992785, 0, 0, -0.072702388256598, 0, 0.0464478520766673, 0, -0.048305766159734, 0, -0.0781031370985167, 0.0144810071458617, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992147, 0, 0.0310733161448835, 0, 0.00694820471863012, -0.00605481326129122, 0, -0.0108311792346515, 0, -0.00393049813889258, 0.00607967912880135},
    {0.107394023286761, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937952, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398575, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259878, 0, 0.0312669212338355, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    const double coeff0_6 = coefficients0[dof][6];
    const double coeff0_7 = coefficients0[dof][7];
    const double coeff0_8 = coefficients0[dof][8];
    const double coeff0_9 = coefficients0[dof][9];
    const double coeff0_10 = coefficients0[dof][10];
    const double coeff0_11 = coefficients0[dof][11];
    const double coeff0_12 = coefficients0[dof][12];
    const double coeff0_13 = coefficients0[dof][13];
    const double coeff0_14 = coefficients0[dof][14];
    const double coeff0_15 = coefficients0[dof][15];
    const double coeff0_16 = coefficients0[dof][16];
    const double coeff0_17 = coefficients0[dof][17];
    const double coeff0_18 = coefficients0[dof][18];
    const double coeff0_19 = coefficients0[dof][19];
    const double coeff0_20 = coefficients0[dof][20];
    const double coeff0_21 = coefficients0[dof][21];
    const double coeff0_22 = coefficients0[dof][22];
    const double coeff0_23 = coefficients0[dof][23];
    const double coeff0_24 = coefficients0[dof][24];
    const double coeff0_25 = coefficients0[dof][25];
    const double coeff0_26 = coefficients0[dof][26];
    const double coeff0_27 = coefficients0[dof][27];
    const double coeff0_28 = coefficients0[dof][28];
    const double coeff0_29 = coefficients0[dof][29];
    const double coeff0_30 = coefficients0[dof][30];
    const double coeff0_31 = coefficients0[dof][31];
    const double coeff0_32 = coefficients0[dof][32];
    const double coeff0_33 = coefficients0[dof][33];
    const double coeff0_34 = coefficients0[dof][34];
    const double coeff0_35 = coefficients0[dof][35];
    const double coeff0_36 = coefficients0[dof][36];
    const double coeff0_37 = coefficients0[dof][37];
    const double coeff0_38 = coefficients0[dof][38];
    const double coeff0_39 = coefficients0[dof][39];
    const double coeff0_40 = coefficients0[dof][40];
    const double coeff0_41 = coefficients0[dof][41];
    const double coeff0_42 = coefficients0[dof][42];
    const double coeff0_43 = coefficients0[dof][43];
    const double coeff0_44 = coefficients0[dof][44];
    const double coeff0_45 = coefficients0[dof][45];
    const double coeff0_46 = coefficients0[dof][46];
    const double coeff0_47 = coefficients0[dof][47];
    const double coeff0_48 = coefficients0[dof][48];
    const double coeff0_49 = coefficients0[dof][49];
    const double coeff0_50 = coefficients0[dof][50];
    const double coeff0_51 = coefficients0[dof][51];
    const double coeff0_52 = coefficients0[dof][52];
    const double coeff0_53 = coefficients0[dof][53];
    const double coeff0_54 = coefficients0[dof][54];
    const double coeff0_55 = coefficients0[dof][55];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9 + coeff0_10*basisvalue10 + coeff0_11*basisvalue11 + coeff0_12*basisvalue12 + coeff0_13*basisvalue13 + coeff0_14*basisvalue14 + coeff0_15*basisvalue15 + coeff0_16*basisvalue16 + coeff0_17*basisvalue17 + coeff0_18*basisvalue18 + coeff0_19*basisvalue19 + coeff0_20*basisvalue20 + coeff0_21*basisvalue21 + coeff0_22*basisvalue22 + coeff0_23*basisvalue23 + coeff0_24*basisvalue24 + coeff0_25*basisvalue25 + coeff0_26*basisvalue26 + coeff0_27*basisvalue27 + coeff0_28*basisvalue28 + coeff0_29*basisvalue29 + coeff0_30*basisvalue30 + coeff0_31*basisvalue31 + coeff0_32*basisvalue32 + coeff0_33*basisvalue33 + coeff0_34*basisvalue34 + coeff0_35*basisvalue35 + coeff0_36*basisvalue36 + coeff0_37*basisvalue37 + coeff0_38*basisvalue38 + coeff0_39*basisvalue39 + coeff0_40*basisvalue40 + coeff0_41*basisvalue41 + coeff0_42*basisvalue42 + coeff0_43*basisvalue43 + coeff0_44*basisvalue44 + coeff0_45*basisvalue45 + coeff0_46*basisvalue46 + coeff0_47*basisvalue47 + coeff0_48*basisvalue48 + coeff0_49*basisvalue49 + coeff0_50*basisvalue50 + coeff0_51*basisvalue51 + coeff0_52*basisvalue52 + coeff0_53*basisvalue53 + coeff0_54*basisvalue54 + coeff0_55*basisvalue55;
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
    
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
    
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
    
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
    
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
    
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    static const double coefficients0[56][56] = \
    {{0.00945067404923492, 0.00609788120679252, 0.00352061335622803, 0.00248944957812476, 0.0070594623332504, 0.00546823600997146, 0.00315708753234943, 0.00446479600584755, 0.00257775117585285, 0.00182274533665715, -0.00298267418725716, -0.0025208197798293, -0.00195261860521383, -0.00112734487734488, -0.00218309396769447, -0.00169101731601731, -0.000976309302606919, -0.00138070985681772, -0.000797153207506487, -0.00056367243867244, 0.00278687112460004, 0.00245778931055954, 0.00207721109304566, 0.00160900079398463, 0.000928957041533343, 0.00219831358911339, 0.0018579140830667, 0.00143913406048032, 0.00083088443721827, 0.00160900079398463, 0.0012463266558274, 0.000719567030240151, 0.00101762145320217, 0.000587524019939407, 0.000415442218609132, -0.00705593524717798, -0.00638233357089081, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363026, -0.00582624677716565, -0.00513826668309062, -0.00434262794914383, -0.00336378514516178, -0.00194208225905522, -0.00459580543596519, -0.00388416451811043, -0.00300866089851429, -0.00173705117965752, -0.00336378514516178, -0.00260557676948629, -0.00150433044925714, -0.00212744452363027, -0.00122828066840393, -0.000868525589828762},
    {0.00945067404923495, -0.00609788120679253, 0.00352061335622802, 0.00248944957812476, 0.00705946233325039, -0.00546823600997146, 0.00315708753234944, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521383, -0.00112734487734489, -0.00218309396769448, 0.00169101731601732, -0.000976309302606925, 0.00138070985681772, -0.000797153207506488, -0.000563672438672441, 0.00278687112460004, -0.00245778931055954, 0.00207721109304566, -0.00160900079398463, 0.000928957041533346, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218268, 0.00160900079398463, -0.0012463266558274, 0.000719567030240148, -0.00101762145320216, 0.000587524019939402, 0.000415442218609128, 0.00705593524717798, -0.0063823335708908, 0.00562868913761196, -0.00475711057319694, 0.00368484200521179, -0.00212744452363027, -0.00582624677716565, 0.00513826668309062, -0.00434262794914382, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811043, 0.00300866089851429, -0.00173705117965752, -0.00336378514516178, 0.00260557676948629, -0.00150433044925714, 0.00212744452363026, -0.00122828066840393, -0.00086852558982876},
    {0.00945067404923495, 0, -0.00704122671245605, 0.00248944957812476, 0, 0, 0.00947126259704828, 0, -0.00515550235170572, 0.00182274533665715, 0, 0, 0, 0.00450937950937951, 0, 0, -0.00292892790782075, 0, 0.00159430641501296, -0.00056367243867244, 0, 0, 0, 0, 0.00464478520766673, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987881, 0.00041544221860913, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527608, 0, 0, 0, 0.00694820471863012, 0, 0, -0.00451299134777144, 0, 0.00245656133680786, -0.000868525589828763},
    {0.00945067404923493, 0, 0, -0.0074683487343743, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00563672438672439, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.018239037386404},
    {-0.010023442173431, 0, 0.0374718748311399, -0.0289614183593723, 0, 0, 0.0284137877911448, 0, 0.00515550235170572, -0.00619733414463434, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835494, -0.000187890812890802, 0, 0, 0, 0, 0.0232239260383337, 0, 0, 0, -0.00166176887443652, 0, 0, -0.00647610327216137, 0, 0.00646276421933342, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496074, 0, 0, 0, -0.0277928188745205, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.010023442173431, 0, -0.0287574853355259, 0.05360943398437, 0, 0, -0.0189425251940966, 0, 0.0154665070551171, 0.0153110608279201, 0, 0, 0, -0.00450937950937953, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144092, 0, -0.0123380044187274, 0.00872428659079174, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559316, 0, 0.041270230458372, -0.018239037386404},
    {0.010023442173431, 0, 0.0409576306293854, -0.0449826285156208, 0, 0, 0.0189425251940965, 0, -0.00515550235170569, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883226, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957583, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.055026973944496, 0.0303983956440067},
    {-0.0100234421734311, 0, -0.0148144621425437, 0.0449826285156207, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0187359374155699, -0.0289614183593723, 0.0211783869997512, -0.0164047080299144, 0.00947126259704827, 0.00446479600584757, -0.00257775117585285, -0.00619733414463432, 0.0208787193108001, -0.0176457384588052, 0.0136683302364968, -0.00789141414141413, -0.0084898098743674, 0.00657617845117844, -0.00379675839902689, 0.0023011830946962, -0.00132858867917746, -0.0001878908128908, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992314, 0.00464478520766674, -0.0010991567945567, 0.000928957041533349, -0.000719567030240152, 0.000415442218609127, -0.00482700238195388, 0.00373897996748219, -0.00215870109072045, 0.00559691799261185, -0.00323138210966672, -0.00290809553026392, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935591, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863011, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778515, 0.00607967912880135},
    {0.010023442173431, -0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, 0.0109364720199429, -0.00631417506469883, 0.0133943880175427, -0.00773325352755858, 0.0153110608279201, -0.00298267418725716, 0.00252081977982931, -0.00195261860521382, 0.00112734487734488, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476002, 0.0129522065443227, -0.00747795993496437, -0.00321800158796926, 0.0024926533116548, -0.00143913406048031, -0.0106850252586226, 0.00616900220936373, 0.00872428659079178, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845888, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.020635115229186, -0.0182390373864041},
    {0.0100234421734311, 0.0354703486038674, -0.0204788153146927, -0.0449826285156208, 0.0141189246665008, -0.0109364720199429, 0.00631417506469885, -0.00446479600584758, 0.00257775117585285, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974696, -0.00263047138047138, 0.00151870335961075, 0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620757, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, -0.0128297005588457, 0.00740723107127184, 0.0449826285156208, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0.00920473237878476, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489395, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.010023442173431, 0.0324515955311979, -0.0335503995581136, -0.00801060507812427, 0.0211783869997512, 0.00109364720199429, -0.00568275755822898, -0.0169662248222207, 0.0056710525868763, 0.00838462854862291, 0.0208787193108001, -0.0109235523792603, 0.00499002532443534, -0.00187890812890812, -0.0162519217595033, 0.00958243145743146, -0.00466458889023305, 0.0108922666482287, -0.00558007245254534, -0.00469727032227033, 0.0139343556230002, -0.00245778931055954, -0.00415442218609131, 0.00643600317593851, -0.00464478520766673, -0.0120907247401236, 0.00464478520766674, -0.000719567030240153, -0.000415442218609137, 0.00965400476390778, -0.00498530662330958, 0.00215870109072046, -0.006614539445814, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543313, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438609, 0.00310733161448835, -0.00481385743762287, 0.00347410235931505, -0.00201827108709706, -0.000521115353897262, 0.000902598269554289, 0.00170195561890421, -0.000491312267361565, -0.000868525589828764},
    {0.0100234421734309, -0.024904712849524, 0.0553363732971484, -0.0043134027343746, -0.0141189246665008, 0.0153110608279201, 0.00883984509057843, 0.00803663281052563, -0.0159820572902877, 0.000729098134662861, -0.00298267418725715, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846298, -0.0116492303992304, 0.00976309302606916, 0.00352848074520084, 0.00132858867917745, -0.00263047138047137, 0, 0.0196623144844763, -0.00727023882565982, -0.00724050357293082, 0.00928957041533346, -0.00219831358911338, -0.0148633126645336, 0.0129522065443228, -0.0066470754977461, 0.00402250198496157, 0.00685479660705067, -0.00575653624192123, -0.0045792965394097, -0.000293762009969684, 0.00290809553026392, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.00411061334647249, -0.00303983956440068, 0.00908221989193681, -0.00776832903622088, 0.00045958054359652, 0.00427258096992148, -0.00270779480866287, 0.000868525589828773, -0.00100913554354854, -0.00234501909253766, 0.00180519653910858, 0.00127646671417816, 0.000245656133680781, -0.000868525589828764},
    {0.0100234421734309, 0.0354703486038675, -0.0492363006502187, -0.00431340273437461, 0.0141189246665008, -0.00656188321196574, -0.016416855168217, -0.00982255121286466, 0.0149509568199466, 0.000729098134662872, 0, 0.0033610930397724, 0.0282044909641998, -0.0353234728234728, -0.000485131992820993, -0.0124007936507936, 0.00889526253486299, 0.00291483191994851, 0.00239145962251946, -0.00263047138047138, 0, 0, 0.0218107164769794, -0.00402250198496158, -0.00928957041533347, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992316, 0.00934744991870547, -0.0028782681209606, -0.00254405363300539, -0.00381890612960613, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320202, 0.00840946286290443, -0.00388416451811042, 0, 0.00233049871086626, 0.0030086608985143, -0.00347410235931507, -0.000336378514516178, -0.00286613444643493, 0.00120346435940572, 0.00085097780945211, 0.00098262453472315, -0.000868525589828765},
    {-0.010023442173431, -0.0128297005588457, 0.0448791059024117, -0.00801060507812424, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467804, -0.0175287079957994, 0.0083846285486229, 0, 0, 0.00867830491206149, 0.0225468975468975, 0, -0.00300625300625301, -0.019200749617936, 0.000613648825252316, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240152, -0.0108014976838374, 0, 0.00249265331165478, 0.0107935054536023, -0.000508810726601074, -0.00734405024924252, 0.00290809553026393, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931505, 0, -0.00208446141558903, -0.000902598269554287, 0.000425488904726055, 0.0017195929357655, -0.000868525589828767},
    {-0.0100234421734311, -0.0324515955311978, -0.0187359374155699, -0.0289614183593723, 0.0211783869997511, 0.0164047080299144, 0.00947126259704826, -0.00446479600584755, -0.00257775117585285, -0.00619733414463435, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141413, -0.00848980987436742, -0.00657617845117845, -0.0037967583990269, -0.00230118309469619, -0.00132858867917746, -0.00018789081289081, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992313, 0.00464478520766674, 0.00109915679455668, 0.000928957041533345, 0.000719567030240154, 0.000415442218609131, -0.00482700238195388, -0.00373897996748219, -0.00215870109072045, -0.00559691799261187, -0.00323138210966673, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.00955378148811641, 0.00740032731935591, 0.00427258096992148, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.0069482047186301, 0.0181644397838736, 0.014070114555226, 0.00812338442598859, 0.0136156449512337, 0.00786099627778516, 0.00607967912880135},
    {0.0100234421734308, 0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, -0.0109364720199429, -0.00631417506469886, -0.0133943880175427, -0.00773325352755858, 0.0153110608279201, 0.00298267418725716, 0.00252081977982933, 0.00195261860521383, 0.00112734487734487, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476002, -0.0129522065443228, -0.00747795993496438, -0.00321800158796927, -0.00249265331165479, -0.00143913406048031, 0.0106850252586227, 0.00616900220936374, 0.00872428659079177, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.0206351152291861, -0.0182390373864041},
    {0.010023442173431, -0.0354703486038674, -0.0204788153146927, -0.0449826285156208, 0.0141189246665008, 0.0109364720199429, 0.00631417506469886, 0.00446479600584758, 0.00257775117585287, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974696, 0.00263047138047138, 0.00151870335961075, -0.0308358534689291, -0.0178030883009781, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620757, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588457, 0.00740723107127183, 0.0449826285156208, 0, 0, 0, -0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878479, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489395, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.010023442173431, -0.0324515955311978, -0.0335503995581136, -0.00801060507812424, 0.0211783869997512, -0.00109364720199429, -0.00568275755822899, 0.0169662248222207, 0.00567105258687632, 0.0083846285486229, -0.0208787193108001, -0.0109235523792603, -0.00499002532443534, -0.00187890812890812, -0.0162519217595033, -0.00958243145743146, -0.00466458889023305, -0.0108922666482287, -0.00558007245254535, -0.00469727032227032, 0.0139343556230002, 0.00245778931055953, -0.00415442218609133, -0.00643600317593852, -0.00464478520766672, 0.0120907247401236, 0.00464478520766673, 0.000719567030240156, -0.000415442218609149, 0.00965400476390777, 0.00498530662330958, 0.00215870109072047, 0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543313, 0.00719357335632687, 0.0112908326677739, 0.0114368694935501, 0.00737991258440984, -0.00183832217438608, 0.00310733161448834, 0.00481385743762287, 0.00347410235931505, -0.00201827108709706, 0.000521115353897262, 0.000902598269554287, -0.00170195561890421, -0.000491312267361565, -0.000868525589828767},
    {0.010023442173431, 0.0249047128495239, 0.0553363732971484, -0.00431340273437459, -0.0141189246665008, -0.0153110608279201, 0.00883984509057843, -0.00803663281052563, -0.0159820572902878, 0.000729098134662854, 0.00298267418725716, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846298, 0.0116492303992304, 0.00976309302606918, -0.00352848074520084, 0.00132858867917746, -0.00263047138047138, 0, -0.0196623144844763, -0.0072702388256598, 0.00724050357293083, 0.00928957041533346, 0.00219831358911339, -0.0148633126645336, -0.0129522065443228, -0.00664707549774609, 0.00402250198496157, -0.00685479660705067, -0.00575653624192124, 0.0045792965394097, -0.000293762009969679, 0.00290809553026392, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.0041106133464725, -0.00303983956440067, -0.00908221989193682, -0.00776832903622089, -0.00045958054359652, 0.00427258096992148, 0.00270779480866287, 0.000868525589828776, -0.00100913554354853, 0.00234501909253766, 0.00180519653910858, -0.00127646671417816, 0.000245656133680779, -0.000868525589828761},
    {0.0100234421734309, -0.0354703486038674, -0.0492363006502187, -0.00431340273437459, 0.0141189246665008, 0.00656188321196577, -0.016416855168217, 0.00982255121286466, 0.0149509568199466, 0.000729098134662868, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820994, 0.0124007936507936, 0.00889526253486299, -0.00291483191994852, 0.00239145962251945, -0.00263047138047138, 0, 0, 0.0218107164769794, 0.00402250198496157, -0.00928957041533346, 0, -0.00557374224920009, 0.0143913406048031, 0.0141250354327105, 0.000804500396992309, -0.00934744991870548, -0.0028782681209606, 0.00254405363300539, -0.00381890612960614, 0.00290809553026393, 0, 0, 0, 0.0266398192099028, 0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320203, -0.00840946286290443, -0.00388416451811041, 0, 0.00233049871086626, -0.0030086608985143, -0.00347410235931507, -0.000336378514516177, 0.00286613444643493, 0.00120346435940572, -0.000850977809452108, 0.000982624534723152, -0.000868525589828766},
    {-0.010023442173431, 0.0128297005588457, 0.0448791059024117, -0.00801060507812424, 0, -0.0174983552319087, 0.0132597676358676, 0.00357183680467805, -0.0175287079957994, 0.0083846285486229, 0, 0, -0.00867830491206148, 0.0225468975468975, 0, 0.00300625300625301, -0.019200749617936, -0.000613648825252319, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766673, 0, 0, 0.00719567030240153, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601077, -0.00734405024924252, 0.00290809553026393, 0, 0, 0, 0, -0.0257938940364826, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702859, -0.00347410235931505, 0, 0.00208446141558903, -0.000902598269554286, -0.000425488904726054, 0.0017195929357655, -0.000868525589828767},
    {-0.0100234421734311, -0.0452812960900435, -0.0113287063442981, -0.00801060507812424, -0.0014118924666501, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.00838462854862289, -0.00894802256177147, -0.0142846454190328, -0.014536160727703, -0.00939454064454064, -0.0123708658169354, -0.0125886844636845, -0.00813591085505764, -0.0102786178229763, -0.00664294339588733, -0.00469727032227033, -0.00836061337380012, 0.00245778931055953, 0.00830884437218264, 0.00965400476390777, 0.00650269929073344, 0.00219831358911338, 0.00743165633226677, 0.00863480436288183, 0.00581619106052785, 0.00643600317593852, 0.00747795993496437, 0.00503696921168108, 0.00610572871921293, 0.00411266813957582, 0.00290809553026394, 0.0352796762358899, 0.0191470007126724, 0.00788016479265675, 0.000951422114639391, -0.00221090520312707, -0.00212744452363027, 0.017478740331497, 0.00719357335632686, 0.000868525589828764, -0.00201827108709706, -0.00194208225905521, 0.00643412761035128, 0.000776832903622088, -0.00180519653910857, -0.00173705117965753, 0.000672757029032363, -0.00156334606169177, -0.00150433044925715, -0.00127646671417816, -0.00122828066840393, -0.000868525589828768},
    {0.010023442173431, 0.0603750614533914, -0.00610007264692975, -0.0043134027343746, -0.00564756986660025, -0.0218729440398859, 0.00126283501293978, -0.0178591840233903, 0.00103110047034114, 0.000729098134662884, 0.0417574386216002, 0.0168054651988621, -0.000867830491206148, -0.00526094276094276, 0.0145539597846299, -0.000751563251563252, -0.00455611007883227, -0.000613648825252328, -0.0037200483016969, -0.00263047138047138, 0.00557374224920005, -0.0196623144844763, -0.0103860554652283, 0.00321800158796928, 0.00650269929073341, -0.0175865087129071, -0.00928957041533347, 0.00287826812096063, 0.00581619106052784, -0.00804500396992315, 0.0024926533116548, 0.00503696921168106, 0.00203524290640433, 0.00411266813957581, 0.00290809553026391, -0.0705593524717798, -0.0127646671417816, 0.00450295131008958, 0.00380568845855754, -0.000736968401042372, -0.00212744452363026, -0.0116524935543313, 0.00411061334647251, 0.00347410235931506, -0.000672757029032362, -0.00194208225905521, 0.00367664434877218, 0.00310733161448835, -0.000601732179702865, -0.00173705117965753, 0.00269102811612942, -0.000521115353897265, -0.00150433044925715, -0.00042548890472606, -0.00122828066840393, -0.000868525589828759},
    {0.0100234421734309, -0.0603750614533914, -0.00610007264692972, -0.00431340273437457, -0.00564756986660032, 0.0218729440398859, 0.00126283501293976, 0.0178591840233903, 0.00103110047034113, 0.000729098134662861, -0.0417574386216002, 0.016805465198862, 0.000867830491206157, -0.00526094276094276, 0.0145539597846298, 0.000751563251563257, -0.00455611007883228, 0.00061364882525233, -0.00372004830169691, -0.00263047138047138, 0.00557374224920011, 0.0196623144844763, -0.0103860554652283, -0.00321800158796927, 0.00650269929073344, 0.0175865087129071, -0.00928957041533346, -0.00287826812096062, 0.00581619106052786, -0.00804500396992314, -0.0024926533116548, 0.00503696921168108, -0.00203524290640433, 0.00411266813957583, 0.00290809553026393, 0.0705593524717798, -0.0127646671417816, -0.00450295131008958, 0.00380568845855756, 0.000736968401042369, -0.00212744452363027, -0.0116524935543313, -0.00411061334647251, 0.00347410235931506, 0.000672757029032361, -0.00194208225905522, -0.00367664434877217, 0.00310733161448835, 0.000601732179702866, -0.00173705117965753, 0.00269102811612942, 0.000521115353897263, -0.00150433044925715, 0.000425488904726061, -0.00122828066840394, -0.000868525589828769},
    {-0.0100234421734309, 0.0452812960900435, -0.0113287063442981, -0.00801060507812426, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089231, 0.00838462854862291, 0.00894802256177147, -0.0142846454190327, 0.014536160727703, -0.00939454064454063, -0.0123708658169354, 0.0125886844636845, -0.00813591085505763, 0.0102786178229763, -0.00664294339588732, -0.00469727032227032, -0.00836061337380015, -0.00245778931055953, 0.00830884437218263, -0.00965400476390777, 0.00650269929073343, -0.00219831358911339, 0.00743165633226677, -0.00863480436288183, 0.00581619106052785, 0.00643600317593852, -0.00747795993496437, 0.00503696921168107, -0.00610572871921293, 0.00411266813957582, 0.00290809553026393, -0.0352796762358899, 0.0191470007126724, -0.00788016479265675, 0.000951422114639385, 0.00221090520312707, -0.00212744452363027, 0.0174787403314969, -0.00719357335632687, 0.000868525589828768, 0.00201827108709706, -0.00194208225905521, -0.00643412761035128, 0.000776832903622089, 0.00180519653910857, -0.00173705117965753, 0.000672757029032358, 0.00156334606169177, -0.00150433044925715, 0.00127646671417816, -0.00122828066840393, -0.000868525589828765},
    {0.0787556170769579, 0.0679219441350653, -0.00435719474780691, -0.00308100195312472, 0.0705946233325039, -0.0328094160598287, 0.0126283501293977, -0.0267887760350854, 0.03608851646194, -0.00729098134662862, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015873, 0.0130985638061669, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, 0.0245778931055954, -0.036351194128299, 0.0362025178646541, -0.0232239260383337, 0.0219831358911338, -0.00743165633226679, 0, 0.00166176887443652, -0.0345935170706695, 0.0130864298861876, -0.00359783515120079, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596521, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598858, -0.0102117337134253, 0.00196524906944628, 0.00607967912880136},
    {-0.0214788046573522, 0.00377344134083702, 0.00217859737390344, -0.0215670136718729, 0, 0.0984282481794862, -0.0852413633734345, -0.0312535720409329, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, 0.0236742424242424, -0.0335199527228374, -0.00138070985681772, -0.00079715320750645, 0.00394570707070704, 0, 0, 0.036351194128299, -0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.00724050357293084, -0.0392592896585629, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724418, 0, 0.00694820471863008, 0.00302740663064561, 0.0164151336477636, -0.0108311792346515, -0.00680782247561686, -0.00393049813889256, 0.00607967912880134},
    {0.0787556170769579, 0.0301875307266957, 0.0610007264692975, -0.00308100195312472, 0, 0.0218729440398858, 0.0757701007763863, 0.0178591840233903, -0.0412440188136458, -0.00729098134662862, 0, 0, 0.0520698294723688, -0.00901875901875902, 0, -0.0127765752765753, 0.0286384062098028, 0.00184094647575699, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.0143913406048031, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515274, 0.00407048581280861, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967795, -0.0427258096992148, 0, 0, -0.0300866089851429, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723144, 0.00607967912880136},
    {-0.0214788046573522, 0.00377344134083686, -0.0196073763651313, 0.00924300585937417, 0, -0.0109364720199429, 0.00947126259704823, 0.102690308134494, -0.0695992817480272, -0.060150596109686, 0, -0.00504163955965862, 0.0065087286840461, -0.00450937950937951, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196709, -0.0263060558477138, 0.000563672438672465, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165478, -0.0043174021814409, -0.0605484764655283, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573521, -0.0150937653633479, 0.0130715842434208, 0.00924300585937414, 0, -0.0109364720199429, 0.00947126259704832, -0.00892959201169514, 0.123732056440937, -0.0601505961096861, 0, 0, -0.00650872868404609, 0.00676406926406927, 0, 0.0574945887445887, 0.0126920209338899, -0.0266937238984759, 0.00637722566005184, 0.000563672438672453, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097862, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448302, -0.0616900220936373, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769578, 0.0301875307266958, 0.0174287789912279, 0.0585390371093694, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094276, -0.00455611007883226, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700864, 0.0284934006896604, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769578, -0.0679219441350653, -0.00435719474780693, -0.00308100195312472, 0.0705946233325039, 0.0328094160598287, 0.0126283501293978, 0.0267887760350854, 0.03608851646194, -0.00729098134662859, -0.0298267418725716, 0.0151249186789758, 0.0325436434202305, 0.0248015873015873, 0.0130985638061669, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646541, -0.0232239260383337, -0.0219831358911339, -0.00743165633226677, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861877, -0.00359783515120079, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.0045958054359652, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.00908221989193679, -0.00703505727761299, -0.00812338442598858, 0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573524, -0.00377344134083698, 0.00217859737390344, -0.021567013671873, 0, -0.0984282481794863, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242424, -0.0335199527228374, 0.0013807098568177, -0.00079715320750645, 0.00394570707070706, 0, 0, 0.036351194128299, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838375, -0.00724050357293083, 0.039259289658563, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951694, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.00302740663064561, -0.0164151336477636, -0.0108311792346515, 0.00680782247561685, -0.00393049813889256, 0.00607967912880134},
    {0.0787556170769579, -0.0301875307266956, 0.0610007264692975, -0.00308100195312471, 0, -0.0218729440398858, 0.0757701007763863, -0.0178591840233903, -0.0412440188136458, -0.00729098134662861, 0, 0, -0.0520698294723689, -0.00901875901875903, 0, 0.0127765752765753, 0.0286384062098028, -0.00184094647575695, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766674, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515274, -0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967796, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780843, -0.00982624534723145, 0.00607967912880137},
    {-0.0214788046573523, -0.0037734413408369, -0.0196073763651314, 0.00924300585937414, 0, 0.0109364720199429, 0.00947126259704826, -0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965861, -0.00650872868404608, -0.00450937950937948, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196707, -0.0263060558477138, 0.000563672438672444, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.0024926533116548, -0.00431740218144091, 0.0605484764655282, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, 0.0150937653633478, 0.0130715842434208, 0.00924300585937414, 0, 0.0109364720199429, 0.00947126259704831, 0.00892959201169514, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404611, 0.00676406926406927, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005185, 0.000563672438672461, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097862, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448302, -0.0616900220936373, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769578, -0.0301875307266956, 0.0174287789912279, 0.0585390371093695, 0, -0.0218729440398858, -0.0189425251940966, -0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, -0.00526094276094275, -0.00455611007883226, -0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700864, -0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769576, -0.0377344134083696, -0.0566435317214905, -0.00308100195312473, 0.0423567739995023, 0.0546823600997146, 0.0378850503881931, 0.0446479600584756, 0.00515550235170574, -0.00729098134662862, 0.0417574386216002, -0.0050416395596586, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801536, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055954, -0.00519302773261415, 0.00402250198496157, 0.00464478520766674, -0.00219831358911335, -0.0130053985814669, -0.0158304746652834, -0.0108014976838374, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.00477689074405821, 0.00370016365967795, 0.00427258096992148, -0.0367664434877216, -0.00776832903622087, 0.00601732179702858, 0.00694820471863011, -0.00908221989193682, 0.00703505727761298, 0.0081233844259886, 0.00680782247561684, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573522, 0, -0.0043571947478069, -0.0215670136718729, -0.127070321998507, 0, 0.0284137877911448, 0, 0.03608851646194, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984381, 0, 0.00159430641501295, 0.00394570707070706, 0.0836061337380013, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377579, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.0095537814881164, 0, 0.00427258096992147, 0, -0.0155366580724417, 0, 0.00694820471863011, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778513, 0.00607967912880133},
    {0.0787556170769579, 0.0377344134083696, -0.0566435317214905, -0.0030810019531247, 0.0423567739995023, -0.0546823600997146, 0.0378850503881931, -0.0446479600584757, 0.00515550235170574, -0.00729098134662864, -0.0417574386216002, -0.00504163955965862, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801536, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261416, -0.00402250198496158, 0.00464478520766674, 0.00219831358911339, -0.0130053985814669, 0.0158304746652834, -0.0108014976838374, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405821, -0.00370016365967795, 0.00427258096992148, 0.0367664434877216, -0.00776832903622088, -0.00601732179702859, 0.00694820471863012, -0.00908221989193681, -0.00703505727761299, 0.00812338442598861, -0.00680782247561685, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573522, -0.0188672067041847, 0.00653579212171051, 0.00924300585937415, 0.0141189246665008, 0, -0.00315708753234942, -0.111619900146189, -0.0541327746929101, -0.0601505961096861, -0.00894802256177146, -0.00252081977982933, 0.000650872868404606, 0.00112734487734486, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672439, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476002, -0.00431740218144093, -0.00747795993496438, 0.00321800158796924, 0.0224338798048931, 0.018708742786244, 0.046301776120698, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.017478740331497, -0.00451299134777145, -0.00781673030845887, 0.0363288795677472, -0.00938007637015066, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {-0.0214788046573521, 0.0188672067041847, 0.00653579212171047, 0.00924300585937414, 0.0141189246665007, 0, -0.00315708753234944, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177148, -0.00252081977982932, -0.000650872868404592, 0.00112734487734487, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.019928830187662, 0.000563672438672456, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476002, 0.00431740218144092, -0.00747795993496438, 0.00321800158796927, -0.0224338798048931, 0.018708742786244, -0.0463017761206981, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777143, -0.00781673030845888, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769579, 0, -0.0348575579824557, 0.0585390371093695, -0.0282378493330015, 0, 0.00631417506469885, 0, -0.0206220094068228, 0.083846285486229, 0, 0, 0, 0, -0.0067918478994939, 0, 0.00151870335961074, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233621, 0, -0.0329013451166066, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.0275134869722481, 0.0303983956440068},
    {0.0787556170769578, -0.0377344134083696, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, 0.0546823600997145, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865145, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.0092175078635989, -0.00789141414141412, -0.00347132196482459, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, -0.00491557862111904, -0.0197335053839338, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097694, -0.000623163327913701, -0.00215870109072047, 0.00712335017241508, 0.00411266813957581, 0.00540074884191871, 0, -0.0510586685671264, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173251, -0.0143871467126537, -0.00564541633388696, 0.00571843474677502, 0.00737991258440982, 0.00367664434877216, -0.00155366580724418, 0.00240692871881142, 0.00347410235931506, 0.00100913554354853, 0.00026055767694863, 0.000902598269554296, -0.000850977809452108, -0.000491312267361565, -0.000868525589828763},
    {-0.0214788046573521, 0, -0.0217859737390348, 0.00308100195312476, -0.127070321998507, 0, 0.0473563129852414, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468977, 0.0509388592462045, 0, -0.00943765659186683, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533349, 0, -0.0141250354327105, -0.0144810071458616, 0, -0.00215870109072044, 0, 0.00411266813957584, 0.00540074884191874, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259877, 0, -0.0112908326677739, 0, 0.00737991258440982, 0, -0.00310733161448833, 0, 0.00347410235931505, 0.00201827108709706, 0, 0.000902598269554279, 0, -0.000491312267361592, -0.00086852558982878},
    {0.078755617076958, 0.0377344134083695, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584757, 0.0257775117585286, 0.0291639253865145, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.0092175078635989, 0.00789141414141413, -0.00347132196482461, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111908, -0.0197335053839338, 0.0394205194526234, -0.0325134964536672, 0.0241814494802473, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097695, 0.000623163327913697, -0.00215870109072046, -0.0071233501724151, 0.00411266813957581, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908062, -0.0114170653756727, -0.010317557614593, 0.0127646671417816, 0.00466099742173252, 0.0143871467126538, -0.00564541633388697, -0.00571843474677502, 0.00737991258440983, -0.00367664434877217, -0.00155366580724418, -0.00240692871881144, 0.00347410235931506, 0.00100913554354853, -0.000260557676948628, 0.000902598269554287, 0.000850977809452107, -0.000491312267361569, -0.000868525589828763},
    {-0.0214788046573521, -0.0188672067041848, 0.0108929868695174, 0.0030810019531247, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792643, 0.0127592173566001, -0.0089480225617715, -0.0277290175781224, -0.00585785581564147, 0.0191648629148629, 0.00145539597846298, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755831, -0.0129644660894661, 0, 0.0589869434534289, 0.00727023882565979, 0.0506835250105158, 0.0650269929073344, -0.00659494076734015, 0.0148633126645335, -0.00431740218144092, -0.00166176887443655, -0.00402250198496158, -0.00810112326287807, -0.0115130724838425, 0.00356167508620755, -0.00205633406978791, 0.00540074884191871, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440066, -0.00302740663064559, -0.00776832903622087, 0.00137874163078956, -0.00427258096992148, 0.000902598269554293, 0.000868525589828758, 0.00100913554354854, 0.000781673030845887, 0.00180519653910857, -0.000425488904726058, 0.00024565613368079, -0.000868525589828762},
    {-0.0214788046573521, 0.0188672067041848, 0.0108929868695174, 0.00308100195312469, 0.0141189246665007, 0.109364720199429, -0.0789271883087357, -0.0223239800292378, -0.0128887558792643, 0.0127592173566001, 0.00894802256177149, -0.0277290175781224, 0.00585785581564149, 0.0191648629148629, 0.00145539597846298, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755832, -0.0129644660894661, 0, -0.0589869434534289, 0.00727023882565983, -0.0506835250105158, 0.0650269929073343, 0.00659494076734015, 0.0148633126645336, 0.00431740218144092, -0.00166176887443653, -0.00402250198496157, 0.00810112326287806, -0.0115130724838425, -0.00356167508620755, -0.00205633406978791, 0.00540074884191872, 0, 0, -0.0607898426862092, 0.0399597288148543, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440068, 0.0030274066306456, -0.00776832903622088, -0.00137874163078956, -0.00427258096992148, -0.000902598269554298, 0.000868525589828757, 0.00100913554354854, -0.000781673030845891, 0.00180519653910858, 0.000425488904726056, 0.000245656133680792, -0.000868525589828762},
    {0.078755617076958, 0, 0.0435719474780696, -0.0523770332031201, -0.0282378493330015, 0, 0.0631417506469885, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.00672218607954481, 0, -0.0601250601250601, 0.000970263985641994, 0, -0.0125835421224892, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539589, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398463, 0, -0.0028782681209606, 0, -0.00822533627915163, 0.00540074884191871, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.018239037386404, 0, -0.00388416451811042, 0, -0.00466099742173252, 0, -0.00347410235931505, 0.000672757029032356, 0, 0.00120346435940571, 0, 0.000982624534723144, -0.000868525589828765},
    {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, -0.0546823600997146, -0.0315708753234943, -0.0446479600584757, -0.0257775117585285, -0.0182274533665716, 0.0894802256177147, -0.0151249186789759, 0.0585785581564149, 0.0744047619047619, -0.0130985638061669, -0.0281836219336219, 0.0097630930260691, 0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226673, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978424, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905521, -0.00902598269554287, -0.0191075629762328, -0.00908221989193678, -0.00234501909253766, -0.00812338442598858, 0.00340391123780844, 0.00196524906944629, 0.00607967912880135},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, 0.0546823600997146, -0.0315708753234943, 0.0446479600584756, -0.0257775117585286, -0.0182274533665715, -0.0894802256177148, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061669, 0.0281836219336219, 0.00976309302606918, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.0363511941282991, 0.0120675059548847, -0.0232239260383337, -0.0659494076734016, 0.00743165633226682, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978425, 0.0251848460584054, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554289, -0.0191075629762328, -0.00908221989193679, 0.00234501909253767, -0.00812338442598859, -0.00340391123780842, 0.00196524906944628, 0.00607967912880136},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281177, 0, 0, -0.0947126259704827, 0, 0.0515550235170571, -0.0182274533665715, 0, -0.0907495120738551, 0, -0.0946969696969696, 0.0232863356554078, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992785, 0, 0, -0.072702388256598, 0, 0.0464478520766673, 0, -0.048305766159734, 0, -0.0781031370985167, 0.0144810071458617, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992147, 0, 0.0310733161448835, 0, 0.00694820471863012, -0.00605481326129122, 0, -0.0108311792346515, 0, -0.00393049813889258, 0.00607967912880135},
    {0.107394023286761, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937952, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398575, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259878, 0, 0.0312669212338355, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    static const double dmats0[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {6.32455532033677, 1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 11.2249721603218, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.58257569495583, 0, 8.36660026534075, -1.18321595661993, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.74165738677394, 0, 0, 8.69482604771365, 0, 0, 0, 0, 0, 0, 0, 0, 1.18793863634892e-14, 0, 0, 0, -2.04281036531029e-14, 1.06581410364015e-14, 1.4210854715202e-14, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.33226762955019e-14, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.49909083394701, -1.24344978758018e-14, -3.3466401061363, -2.36643191323985, 15.4919333848297, 0, 0.692820323027545, 0, 0.565685424949238, 0.400000000000001, -1.24344978758018e-14, 2.22044604925031e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 4.89897948556636, -1.4210854715202e-14, 0, 0, 14.1985914794391, 0, -0.828078671210839, 0, 0, 0, -1.24344978758018e-14, 1.80966353013901e-14, -1.06581410364015e-14, 0, 1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 2.48689957516035e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.6, 0, 8.76356092008264, -1.54919333848296, 0, 0, 9.52470471983252, 0, -1.48131215963607, 0.261861468283189, 0, -1.06581410364015e-14, 0, 0, 0, -1.06581410364015e-14, 2.48689957516035e-14, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 4.24264068711928, 0, -2.48689957516035e-14, 0, 0, 0, 14.3427433120127, 0, 0, 0, 0, 0, 0, 0, 2.22044604925031e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, -1.06581410364015e-14, 0},
    {3.11769145362397, 0, 3.1622776601684, 4.91934955049952, 0, 0, 0, 0, 10.690449676497, -2.41897262725905, 0, 0, 1.50990331349021e-14, 0, 0, 0, -2.30926389122033e-14, 0, 1.77635683940025e-14, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, -1.33226762955019e-14, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.37945210809676e-14, 0, 0, 1.95399252334028e-14, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.54558441227157, 0, 0, 7.66811580507232, 0, 0, 0, 0, 0, 10.3691851174526, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 11.6057691494799, 0, 0, 0, -4.20458932936041, 0, -3.43303281162798, 0, 1.25455201782643e-14, 19.6214168703486, 0, 0.611677741841184, 0, 0, 0.529728463363969, 0, 0.432521479156776, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.53137212479106, 0, 3.54562104171168, -2.50713268211204, 4.7871355387817, 0, -5.56626745075489, 0, -1.04880884817016, 0.741619848709568, 0, 18.8034951158403, -1.24344978758018e-14, 1.68183573174417, -0.603122387046461, 0, 0.701283781217647, 0, 0.13213749452868, -0.0934353184302319, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, -1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.08246744900953e-14, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 2.5951288749407, 0, 0, 0, 10.0285307284481, -1.4210854715202e-14, -0.767649473578743, 0, 0, 0, 0, 16.4130361329658, -1.06581410364015e-14, 0, -1.26347611061158, 1.4210854715202e-14, 0.0967147429123241, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.96647939483827, 0, 8.12403840463597, -1.64130361329658, 0, 0, 11.7728501222091, 0, -2.40312296813957, 0.485504156227611, 1.06581410364015e-14, -1.06581410364015e-14, 0, 10.2761860629321, 0, 0, -1.48323969741913, 0, 0.302765035409733, -0.0611677741841268, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.05298244095219, 0, -1.5856499343442, 4.4848952846511, 4.28174419288838, 0, 0.191485421551275, -1.06581410364015e-14, -3.75233260785875, -4.31161222746203, 0, 0, 0, 0, 18.8807171866294, 0, 0.844371341865028, 0, 1.18187368057054, 1.04463861754668, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, -1.15463194561016e-14, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0},
    {0, 2.32115382989599, -1.4210854715202e-14, -1.06581410364015e-14, 0, 3.92428337406972, 0, 5.49285249860473, -1.59872115546023e-14, 0, 0, 0, 0, 0, -1.68753899743024e-14, 17.3044631365566, -1.77635683940025e-14, -1.73008591662713, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.77635683940025e-14, 2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0},
    {2.65329983228431, 0, 4.15220619990511, 2.93605316083775, 0, 0, 2.63248931621763, 0, 9.82591326180786, -2.82261277137741, 0, 0, 0, 0, 0, 1.4210854715202e-14, 11.6081867662439, 0, -3.09487177319468, 0.683876505540245, 0, 0, 0, 0, 0, 0, 0, 0, 1.50990331349021e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.26565424807268e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 2.01017818278146, 0, 0, 0, 0, 0, 8.91928088068603, 0, 0, 0, 0, 0, 0, 0, 0, 0, 16.8558564884489, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.2978250586152, 0, 1.49829835452881, 5.50917601898534, 0, 0, 0, 0, 6.64803945320938, 4.13676892548487, 0, 0, 1.06581410364015e-14, 0, 0, 0, -2.57571741713036e-14, 1.24344978758018e-14, 12.5636136423842, -3.55352656109506, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87616630392937, -1.19904086659517e-14, 0, 6.22830929985771, 0, 0, 0, 0, 0, 11.0541524195338, 0, 0, -1.19904086659517e-14, 0, -1.4210854715202e-14, 0, 2.99760216648792e-14, -1.06581410364015e-14, 0, 11.60576914948, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.5709655798446e-14, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.83186700722508, 0, -4.41087092453693, -3.11895674167864, 16.6749979177077, 0, 2.13065307820925, 0, 1.739670953501, 1.23013312825382, 0, -4.36653943835009, 0, -0.557935943461983, -3.78153408023783, -1.06581410364015e-14, -0.483186700722515, 0, -0.394520289089675, -0.278967971730983, 23.6981011897578, 0, 0.490653381462667, 0, 0.0626933893908967, 0, 0.438853725736256, 0, 0.0560746721671611, 0.380058475033045, 0, 0.0485620906056452, 0, 0.0396507809422086, 0.0280373360835817, 0, 0, 0, 0, 0, 0, -1.46549439250521e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 6.91050664139875, 0, 0, 0, 10.5706241650196, 0, -2.72554057547698, 0, 1.15463194561016e-14, 4.67332857821916, 0, -7.86705867811458, -2.48689957516035e-14, 0, -2.39719223514613, 0, 0.618094504365252, 0, 0, 0, 23.10555488766, 0, 1.62065746295906, 0, -0.469687194424183, 0, 0.790669146636837, 0, 0, 0.240926884676763, 0, -0.0621208350291009, 0, 0, 0, 0, 0, 0, 0, -1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.85449644663775, 1.15463194561016e-14, 5.98171817966646, -2.4880667576406, 1.90029237516522, 0, -0.33993463423953, 0, -2.3592214660616, 0.981306762925316, 0, 9.95226703056241, 0, -6.45364422706459, -0.43094580368567, 0, 0.077089928932768, 0, 0.535021138879229, -0.222539456105679, 0, 0, 21.5273297444417, 0, 2.75066129614997, 0, -1.00024047132728, 0, 0.648615649442924, 0.0433116829031337, 0, -0.0077478293753126, 0, -0.0537716476536687, 0.0223660568773445, 0, 0, 0, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.06581410364015e-14, 1.54523626091315, -1.4210854715202e-14, 0, 0, 6.71778024562961, 0, -0.609449400220043, 0, 0, 0, 1.06581410364015e-14, 14.6593983227032, 1.4210854715202e-14, 0, -1.52344936219873, 0, 0.138210132827973, 0, 0, 0, 0, 0, 18.119501277094, 0, 0, 0, -1.47332496632642, 0, 0, 0.153112421864228, 0, -0.0138906409944104, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.52336024752224, 1.4210854715202e-14, 7.37120708415867, -1.62882203585591, 0, 0, 12.0171306297064, 0, -2.90724328050879, 0.642416074439625, 0, 0, 2.1316282072803e-14, 13.9858820944259, 0, 0, -2.72522906732402, 0, 0.659300804656272, -0.145686271816946, 0, 0, 0, 0, 10.8043830863937, 0, 0, 0, -1.40563403846548, 0, 0, 0.273895826790425, 0, -0.0662622240310384, 0.0146420212340128, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 6.30840061880559, 0, 0, 1.06581410364015e-14, -1.5236235005501, 0, 11.1963004093827, 0, 0, 4.26614580154029, 0, 0.132992762321608, 0, 0, -4.72217953828917, 0, -6.67684597013021, 0, 0, 0, 0, 0, 0, 0, 23.1532444221696, 0, 0.721778878560592, 0, 0, 1.11125127955292, 0, 1.1908743922773, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0},
    {3.51865775274499, -1.04360964314765e-14, 1.92724822331885, 2.725540575477, 1.73472166622177, 0, -2.01705689521107, -1.77635683940025e-14, 3.42052627529743, -4.03112887414928, -1.28785870856518e-14, 4.08831086321549, -2.30926389122033e-14, 0.365669640132028, 5.37643791520787, 0, -6.25148194071026, 0, -2.03981013744551, 1.8080332206528, 0, 0, 0, 0, 0, 0, 22.1880978975596, 1.765254609154e-14, 1.98456380761453, -1.26521523890981, 0, 1.4711357857929, 0, 0.363818136386342, -0.294009452979941, 0, 0, 0, 0, 0, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, -2.30926389122033e-14, 0},
    {0, 1.4106012612951, 0, 0, 0, 3.634054100636, 0, 2.50356888118883, -1.24344978758018e-14, 0, 0, 0, 3.56857028479908, 0, 0, 11.263055412878, 0, -1.4929881464507, 0, 0, 0, 0, 0, 0, 0, 0, 0, 19.3673596462205, 0, 0, -2.65048896124152, 0, 0.266287609379582, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, -1.24344978758018e-14, 0, 0, -1.59872115546023e-14, 0, -1.24344978758018e-14, 0, 0},
    {2.30350221379959, 0, 4.41588043316389, 1.78428514239955, 0, 0, 4.2661458015403, 0, 7.8374102865679, -2.63899331455872, 0, 0, 0, 2.23427840700302, 1.77635683940025e-14, 0, 13.2221027072096, -1.4210854715202e-14, -4.67379217909115, 1.18363558466232, 0, 0, 0, 0, 0, 0, 0, 0, 12.1258851598179, 0, 0, -3.11150358274815, 0, 0.833612074594101, -0.192474367616158, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.8421709430404e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.14718316987778, 0, -0.861891607371343, 6.09449400220045, 1.55158222708545, 0, 0.0693888666488798, -1.95399252334028e-14, -3.05941170815566, 1.80277563773199, 0, -1.26842980563424e-14, 0, 0, 9.03122156537879, 0, 0.403888506800988, -1.4210854715202e-14, -3.67461994579465, -5.49650682721193, 0, 0, 0, 0, 0, 0, 0, 0, 0, 21.7841260588564, 0, 0.974215733960557, 0, 1.78974858569576, 1.8079190879493, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1.26168012376112, 1.24344978758018e-14, 0, 0, 1.42204860051343, 0, 4.47852016375307, 0, 0, 0, 0, 0, 0, 0, 8.27725128825318, 0, 5.37909280973869, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 19.9654813226339, 0, -2.61992366301004, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 1.06581410364015e-14, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 1.33226762955019e-14, 0, 0},
    {2.06031501455084, 0, 2.25696201807219, 3.98978286964828, 0, 0, 0.953939201416947, 0, 8.01142041980429, 1.18019368870418, 0, 0, 0, 0, 0, 1.77635683940025e-14, 5.55254896421454, 0, 9.62241374388481, -3.59830837050572, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 13.3932520322736, 0, -4.68666192508059, 1.18356086726904, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1.09264703862702, 0, 0, 0, 0, 0, 5.38682254607085, 0, 0, 0, 0, 0, 0, 0, 0, 0, 13.4377817958113, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 18.907670401189, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.78428514239954, 0, 0.814411017927931, 5.0677048703591, 0, 0, 0, 0, 4.01510046524764, 6.58672305740272, 0, 0, -1.06581410364015e-14, 0, 0, 0, 1.59872115546023e-14, 0, 10.0159311874144, 2.83293314500743, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 14.0929454377398, -4.55552790213386, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.22124532708767e-14, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.45686271816936, 0, 0, 5.07816454066238, 0, 0, 0, 0, 1.06581410364015e-14, 10.0142755247553, 0, 0, 0, 0, 0, 0, 0, 0, 0, 13.8784813613723, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.5535587278944, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    static const double dmats1[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.47722557505167, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.95803989154981, 5.6124860801609, -1.08012344973464, -0.763762615825972, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.29128784747791, 7.24568837309471, 4.18330013267037, -0.591607978309964, 0, 0, 0, 0, 0, 0, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-2.64575131106461, 0, 9.66091783079295, 0.683130051063968, 0, 0, 0, 0, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338697, 0, 0, 4.34741302385683, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.02140518265514e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.24037034920393, 0, 0, 7.52994023880668, 0, 0, 0, 0, 0, 0, 0, 0, 1.0547118733939e-14, 0, 0, 0, -1.84297022087776e-14, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.7495454169735, 5.79655069847576, -1.67332005306815, -1.18321595661993, 7.74596669241484, -1.2, 0.346410161513775, -0.979795897113271, 0.28284271247462, 0.200000000000002, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.32379000772444, 2.44948974278317, 2.82842712474617, -1.00000000000001, 9.16515138991169, 7.09929573971954, -2.04939015319193, -0.414039335605414, -0.478091443733755, 0.169030850945704, -1.4210854715202e-14, 1.06581410364015e-14, 0, -1.15463194561016e-14, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.80000000000001, -5.69209978830307, 4.38178046004132, -0.77459666924149, 0, 10.998181667894, 4.76235235991626, 0.962140470884713, -0.740656079818035, 0.130930734141589, 1.19904086659517e-14, -1.4210854715202e-14, 1.53210777398272e-14, -1.33226762955019e-14, 0, 1.06581410364015e-14, 1.4210854715202e-14, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.19615242270665, 0, -3.1622776601684, -2.23606797749978, 0, 0, 13.7477270848675, -1.4210854715202e-14, 0.534522483824862, 0.377964473009222, 1.4210854715202e-14, 0, 0, -3.01980662698043e-14, 0, 0, 1.77635683940025e-14, 0, -1.39055433834301e-14, 0, 0, 0, 0, 0, -1.24344978758018e-14, 0, 0, 0, 1.14908083048704e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.01246117974981, 2.12132034355964, -0.40824829046386, 3.17542648054293, 0, 0, 0, 7.17137165600636, -1.38013111868471, -1.56144011671765, 0, 0, 0, 0, 0, 0, -1.55431223447522e-14, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.55884572681199, 2.73861278752584, 1.5811388300842, 2.45967477524976, 0, 0, 0, 9.25820099772551, 5.34522483824848, -1.20948631362952, 0, 0, 0, 0, 0, 1.35447209004269e-14, -1.33226762955019e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.80000000000002, 0, 3.65148371670112, -2.84018778721878, 0, 0, 0, 0, 12.3442679969673, 1.39659449751035, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.27279220613578, 0, 0, 3.83405790253616, 0, 0, 0, 0, 0, 5.18459255872629, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.20454076850486, 0, 0, 6.64078308635359, 0, 0, 0, 0, 0, 8.97997772825746, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.56904651573302, 5.80288457473996, -2.01017818278147, -1.42141062443803, 8.14215222512715, -2.10229466468021, 0.728256234341417, -1.71651640581399, 0.594618725379072, 0.42045893293605, 9.81070843517429, -1.18450885369836, 0.305838870920592, -0.105945692672798, -1.02581475831037, 0.264864231681986, -0.09175166127618, 0.216260739578392, -0.0749149177264366, -0.0529728463363877, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239553, 3.07059789431495, 1.77281052085583, -1.25356634105602, 2.39356776939085, 6.30376871403132, -2.78313372537745, -0.908295106229253, -0.524404424085079, 0.370809924354787, 11.1242977306435, 9.40174755792014, -2.42752078113807, 0.840917865872087, -0.301561193523231, -0.794200206656965, 0.350641890608825, 0.11443442705426, 0.0660687472643419, -0.0467176592151146, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551269, 1.29756443747037, 3.74574588632197, -1.05945692672795, -8.09173593712687, 5.01426536422408, 1.80936716113936, -0.383824736789376, -1.10800657553489, 0.313391585264003, 2.1316282072803e-14, 12.7134831207354, 8.20651806648291, -2.84282124887606, 1.01946290306866, -0.631738055305796, -0.227958835180072, 0.0483573714561647, 0.139595707137603, -0.0394836284566167, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741913, 7.03562363973513, 4.06201920231799, -0.820651806648276, 0, -6.79705818718656, 5.88642506110458, -2.08116553882674, -1.2015614840698, 0.242752078113814, 0, -1.06581410364015e-14, 14.8323969741913, 5.13809303146608, 0, 0.856348838577665, -0.741619848709565, 0.262202212042526, 0.151382517704867, -0.0305838870920541, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-3.4253953543107, 0, 9.38083151964687, 1.89521416591737, 0, 0, -3.39852909359329, 0, -2.77488738510233, -0.560611910581384, -1.06581410364015e-14, 0, 0, 17.7988763690296, 0, 0, 0.428174419288809, 0, 0.349602949390039, 0.0706304617818535, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 1.50990331349021e-14, 0, 0, 0, 0, -1.73194791841524e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047609, 2.74642624930236, -0.792824967172096, 2.24244764232554, 2.14087209644419, -0.331662479035538, 0.0957427107756352, 6.49923072370876, -1.87616630392937, -2.15580611373101, -1.06581410364015e-14, 0, 0, 0, 9.44035859331469, -1.46249406456535, 0.422185670932514, -2.04706526287665, 0.590936840285273, 0.522319308773341, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.71269767715535, 1.16057691494799, 1.34011878852097, 1.89521416591735, 2.53311402559511, 1.96214168703486, -0.566421515598876, 2.74642624930236, 3.17129986868836, -1.82198870938951, -1.77635683940025e-14, 0, 0, 0, 11.1699829238239, 8.6522315682783, -2.49768411251824, -0.865042958313563, -0.998865569685877, 0.441440386136644, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 1.59872115546023e-14, 0, -1.77635683940025e-14, 0, 0, 0, 0, 1.06581410364015e-14, -1.09356967925578e-14, 1.02140518265514e-14, -1.33226762955019e-14, 0},
    {1.32664991614216, -2.6969370381518, 2.07610309995255, 1.46802658041887, 0, 3.03973683071412, 1.31624465810881, -6.38211787508103, 4.91295663090393, -1.4113063856887, 0, 0, 0, 0, 0, 13.4039795085888, 5.80409338312195, 2.01017818278148, -1.54743588659734, 0.341938252770126, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.11022302462516e-14, 1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 1.6228338117763e-14, 0, 0, 0, 0},
    {3.82970843102536, 0, -1.4982983545288, 4.2378277069118, 0, 0, 3.79967103839266, -1.33226762955019e-14, -3.54562104171166, -4.07409060843206, 0, 0, -1.4210854715202e-14, 0, 1.4210854715202e-14, 1.25455201782643e-14, 16.7549743857359, -1.06581410364015e-14, 1.11676565710081, 0.987090711415277, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 1.00508909139073, -0.193429485824663, 3.55615782880926, 0, 0, 0, 4.45964044034302, -0.858258202906996, 2.67027285925187, 0, 0, 0, 0, 0, 0, 0, 8.42792824422443, -1.62195554683793, -2.29379153190449, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.14891252930761, 1.29756443747035, 0.749149177264408, 2.75458800949267, 0, 0, 0, 5.75737105184053, 3.32401972660469, 2.06838446274244, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 10.8804085776374, 6.2818068211921, -1.77676328054754, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.32664991614216, -1.06581410364015e-14, 1.73008591662715, -3.18072425757424, 0, 0, 0, 0, 7.67649473578738, -2.38836465270395, 0, 0, 0, 0, 0, 1.06581410364015e-14, -1.47659662275146e-14, 0, 14.5072114368499, 2.05162951662074, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.938083151964684, 0, 0, 3.11415464992885, 0, 0, 0, 0, 0, 5.52707620976691, 0, 0, 0, 0, 0, 0, 1.58761892521397e-14, 0, 0, 5.80288457473998, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.62480768092719, 0, 0, 5.39387407630365, 0, 0, 0, 0, 0, 9.57317681262151, 0, 0, -1.15463194561016e-14, 0, -1.24344978758018e-14, 0, 2.63122856836162e-14, 0, 0, 10.0508909139074, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.32116539930394e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.41593350361254, 5.72988941019469, -2.20543546226846, -1.55947837083932, 8.33749895885385, -2.76779953857108, 1.06532653910463, -2.25989885993663, 0.869835476750496, 0.615066564126918, 10.333118277332, -2.18326971917505, 0.724780051083766, -0.278967971730992, -1.89076704011891, 0.627677936394715, -0.241593350361261, 0.51249688899006, -0.197260144544837, -0.139483985865487, 11.8490505948789, -1.16109782023228, 0.245326690731332, -0.0814411017927985, 0.0313466946954515, -1.03851746182648, 0.21942686286813, -0.0728431359084674, 0.0280373360835804, 0.190029237516525, -0.0630840061880544, 0.0242810453028245, -0.0515078753637725, 0.0198253904711041, 0.0140186680417935, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.18529407725406, 3.45525332069938, 0.997445717412069, -1.41060126129511, 3.23209352764251, 5.28531208250978, -2.89087233497829, -1.36277028773849, -0.393397896234718, 0.556348640264195, 2.33666428910958, 9.02784807228869, -3.93352933905728, 1.64018416088946, -0.732969916135164, -1.19859611757308, 0.655588223176204, 0.309047252182618, 0.0892142571199805, -0.126168012376108, 13.0996183150503, 11.55277744383, -2.44096975264613, 0.810328731479533, -0.311895674167856, -0.234843597212092, -0.907332870306513, 0.395334573318419, -0.164844710568311, 0.0736662483163206, 0.120463442338383, -0.0658890955530712, -0.0310604175145501, -0.00896637020658453, 0.0126803623514089, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.92724822331887, 1.82834820066012, 2.99085908983323, -1.2440333788203, 0.9501461875826, 5.7406445631131, -0.169967317119757, -0.721110255092796, -1.1796107330308, 0.49065338146266, -10.3037210107158, 4.97613351528119, 5.7817446699566, -3.2268221135323, -0.215472901842833, -1.3018558182182, 0.0385449644663813, 0.163532434528613, 0.267510569439617, -0.111269728052837, 0, 14.555131461167, 10.7636648722209, -3.57321383950879, 1.37533064807499, 1.03556292536503, -0.500120235663638, -0.581087203147974, 0.32430782472146, 0.0216558414515668, 0.130841432732455, -0.00387391468765474, -0.0164356280722655, -0.0268858238268351, 0.0111830284386718, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.62882203585592, 0.772618130456564, 3.86595114186566, -1.05140010313427, 9.1544290607646, 3.3588901228148, 3.878512233171, -0.304724700110022, -1.52475169285615, 0.414677792940102, 0, -9.4626009282084, 7.32969916135161, 0.564240504518009, -2.07602937339135, -0.761724681099364, -0.879563899362182, 0.0691050664139832, 0.34578118203618, -0.0940400840863436, 0, 0, 16.3745228938128, 9.05975063854702, -3.48709964663735, 0, 0.951027176355639, -0.736662483163211, -0.0567083043941548, 0.208648802584721, 0.0765562109321125, 0.0883994979795828, -0.00694532049720742, -0.0347523163751395, 0.00945138406569365, 0, -1.4210854715202e-14, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376112, -6.78263087840204, 3.68560354207933, -0.814411017927966, 0, 10.0216907612581, 6.00856531485322, 2.67510569439611, -1.45362164025439, 0.321208037219813, 1.33226762955019e-14, -1.06581410364015e-14, -7.5700807425667, 6.99294104721295, 0, -2.27270584034421, -1.36261453366202, -0.606656948415834, 0.329650402328125, -0.0728431359084816, 0, 0, 0, 18.7137404500718, 5.40219154319687, 0, 0, 0.760821741084501, -0.702817019232737, 0, 0.228415531250638, 0.136947913395213, 0.0609713174047632, -0.0331311120155173, 0.00732101061700419, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.09901951359278, 0, -4.65474668125631, -3.2914029430219, 1.24344978758018e-14, 0, 13.490737563232, 0, 1.83585684909538, 1.29814682728311, -1.4210854715202e-14, -3.5527136788005e-14, 4.2632564145606e-14, -3.53270434653113, -2.1316282072803e-14, 0, -3.05941170815567, 0, -0.416333199893195, -0.29439202887759, 0, 0, 0, 0, 21.8326971917504, 0, 0, 0, 0.35505014583943, 0, 0, 0.307482445914323, 0, 0.0418430609640534, 0.0295875121532867, 0, 0, 0, -1.77635683940025e-14, 1.4210854715202e-14, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 1.15463194561016e-14, 0, 0, 1.15463194561016e-14, 0, 0, 0},
    {1.99489143482413, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176042, -0.761811750275057, 0.263899331455877, 5.59815020469134, -1.93925611658551, -2.28543525082516, 2.13307290077014, -0.257539376818858, 0.0664963811608033, -0.0230350221379969, 9.14446135479766, -2.36108976914457, 0.817905488277891, -3.3384229850651, 1.15646364545769, 1.02505848514082, 0, 0, 0, 0, 0, 11.5766222110848, -1.39771878815471, 0.360889439280296, -0.125015768949702, -2.15192884958005, 0.555625639776456, -0.192474367616153, 0.595437196138647, -0.206265495285698, -0.166687691932932, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.7593288763725, 1.66904592079255, 0.96362411165942, 1.3627702877385, 0.867360833110887, 2.28430636201792, -1.00852844760553, 2.96226264871971, 1.71026313764871, -2.01556443707464, 2.41867732448956, 2.04415543160774, -0.527798662911756, 0.182834820066013, 2.68821895760393, 7.07977053255619, -3.12574097035514, -1.76652739792485, -1.01990506872276, 0.904016610326401, 0, 0, 0, 0, 0, 13.1266557397187, 11.0940489487798, -2.8644711213756, 0.992281903807267, -0.6326076194549, -1.66605356688627, 0.735567892896449, 0.31507574846809, 0.181909068193174, -0.147004726489965, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, -1.24344978758018e-14, 0},
    {1.48690428533296, 0.705300630647549, 2.03602754481988, 1.1517511068998, -2.93221479395416, 1.81702705031799, 0.655663160391206, 1.25178444059441, 3.61359041872287, -1.70346285967313, 1.06581410364015e-14, 2.7642026565595, 1.78428514239953, -0.618094504365255, -9.08783875864327, 5.63152770643899, 2.03210252328837, -0.74649407322535, -2.15494277062553, 0.764033484562597, 0, 0, 0, 0, 0, 0, 15.0018922739643, 9.68367982311024, -3.3545250915713, 2.13860408462397, -1.32524448062076, -0.4782064110178, 0.133143804689791, 0.384353057392901, -0.124241670058201, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 1.06581410364015e-14, 0, 0, 1.06581410364015e-14, -1.86517468137026e-14, 0, 1.13797860024079e-14, 0, 0, 0, 0, 0},
    {1.15175110689979, 3.82426463519456, 2.20794021658195, 0.89214257119979, 0, -2.46306042692148, 2.13307290077015, 6.78739640804928, 3.91870514328395, -1.31949665727937, 0, 0, 3.2249030993194, 1.11713920350152, 2.1316282072803e-14, -7.63378455726039, 6.61105135360481, -4.04762275910198, -2.33689608954558, 0.591817792331174, 0, 0, 0, 0, 0, 0, 0, 17.5022076529583, 6.06294257990893, 0, 1.79642743108413, -1.55575179137407, 0.721929233499942, 0.416806037297052, -0.0962371838080749, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.1316282072803e-14, 0, 0, 0, -1.59872115546023e-14, 0, 0, -1.24344978758018e-14, 0, -2.44249065417534e-14, -1.06581410364015e-14, 0},
    {-2.65985524643218, 0, 5.09901951359278, -2.06031501455085, 0, 0, -1.23153021346075, 1.59872115546023e-14, 9.04986187739903, 3.04724700110024, 0, 2.1316282072803e-14, 0, 3.86988371918331, 0, -4.2632564145606e-14, -3.81689227863023, 0, -5.39683034546928, -1.36674464685442, 0, 0, 0, 0, 1.28785870856518e-14, 0, 0, 0, 21.00264918355, 0, 0, 0.898213715542074, 0, 0.962572311333254, 0.222250255910579, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -3.10862446895044e-14, 0, 0, 1.50990331349021e-14, 0, 0, 0, 0, 0, -2.30926389122033e-14, 0},
    {1.57359158493889, 1.49284005458435, -0.430945803685675, 3.04724700110022, 0.775791113542725, -0.120185042515472, 0.0346944333244361, 5.29905651979669, -1.52970585407783, 0.901387818865992, 0, 0, 0, 0, 4.51561078268939, -0.699555414372423, 0.201944253400492, 6.36462844462229, -1.83730997289733, -2.74825341360597, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.8920630294282, -1.68739114875268, 0.487107866980278, -3.09993548319961, 0.894874292847881, 0.903959543974651, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32992762321609, 0.630840061880562, 0.7284313590847, 2.57539376818856, 0.917928424547682, 0.711024300256705, -0.205255035576785, 2.23926008187652, 2.58567482211401, 0.761811750275061, 0, 0, 0, 0, 5.34294273196308, 4.13862564412659, -1.19471831485579, 2.68954640486935, 3.10562068169861, -2.32269806558124, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.8876627769294, 9.98274066131696, -2.88176900403079, -1.30996183150502, -1.51261363209513, 0.763985254692628, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.03015750727541, -1.46593983227032, 1.1284810090361, 1.99489143482414, 0, 1.10151410945721, 0.476969600708482, -5.20357020296093, 4.00571020990214, 0.590096844352088, 0, 0, 0, 0, 0, 6.4115312783557, 2.77627448210726, -6.24994106094658, 4.81120687194241, -1.79915418525285, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.4651953323153, 6.69662601613678, 3.0440762145518, -2.34333096254029, 0.591780433634522, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.01030295240889e-14, 0, 1.06581410364015e-14, 0, 0},
    {2.9738085706659, -2.1316282072803e-14, -0.814411017927963, 5.758755534499, 0, 0, 1.37689263682153, 0, -2.8908723349783, 1.70346285967311, 2.1316282072803e-14, 0, 0, 0, 0, 0, 8.01441409794462, 0, -3.47218947830368, -5.19371076584692, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 19.3314941653941, 0, 1.69115345252877, 1.70832296330021, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, -1.77635683940025e-14, 0, 0, 0, -1.4210854715202e-14, 0, 0, 0},
    {1.15175110689979, 0.546323519313513, -0.105140010313428, 3.27118942773249, 0, 0, 0, 2.69341127303542, -0.518347241175123, 4.25171145123352, 0, 0, 0, 0, 0, 0, 1.19904086659517e-14, 6.71889089790563, -1.29305115618717, 1.82865048192213, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.45383520059452, -1.81939143264587, -2.94058061635795, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.892142571199769, 0.705300630647548, 0.407205508963976, 2.53385243517955, 0, 0, 0, 3.47717900165117, 2.00755023262382, 3.29336152870135, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 8.67405085085771, 5.0079655937072, 1.41646657250371, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.2048487632307, 7.0464727188699, -2.27776395106694, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.03015750727543, 0, 0.940400840863387, -2.92584077107541, 0, 0, 0, 0, 4.63623866886823, -3.80284633026897, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 11.5654011344769, -1.63559471386627, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 16.2731316843076, 2.63013526059784, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, -1.06581410364015e-14, -1.06581410364015e-14},
    {0.728431359084679, 0, 0, 2.53908227033119, 0, 0, 0, 0, 0, 5.00713776237767, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.93924068068616, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.2767793639472, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376111, 0, 0, 4.39781949681096, 0, 0, 0, 0, 1.06581410364015e-14, 8.67261700493486, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.0191174248973, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.8717007662564, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    static const double dmats2[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.82574185835056, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.16397779494323, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.95803989154981, 5.61248608016089, -1.08012344973464, -0.763762615825974, 0, 0, 0, 0, 0, 0, 0, 0, -1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.29128784747792, 1.44913767461895, 4.18330013267037, -0.591607978309966, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32287565553229, 0, 3.86436713231718, -0.341565025531991, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338697, 7.09929573971953, 0, 4.34741302385682, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.08012344973463, 1.06581410364015e-14, 7.09929573971953, 2.50998007960221, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-3.81881307912987, 0, 0, 8.87411967464941, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, -1.33226762955019e-14, 0, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.7495454169735, 5.79655069847577, -1.67332005306815, -1.18321595661993, 7.74596669241484, -1.2, 0.346410161513777, -0.979795897113275, 0.282842712474621, 0.200000000000002, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.32379000772445, 2.44948974278317, 2.82842712474618, -1.00000000000001, 1.30930734141595, 7.09929573971954, -2.04939015319192, -0.414039335605415, -0.478091443733755, 0.169030850945703, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.8, 0.632455532033677, 4.38178046004131, -0.774596669241482, 0, 3.14233761939829, 4.76235235991626, -0.106904496764974, -0.740656079818036, 0.130930734141593, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.03923048454133, 0, 3.16227766016837, -0.447213595499959, 0, 0, 5.89188303637179, 0, -0.534522483824841, 0.0755928946018432, 0, -1.24344978758018e-14, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.0124611797498, 2.12132034355962, -0.408248290463868, 3.17542648054291, 9.07114735222146, 0, 0, 7.17137165600636, -1.38013111868471, -1.56144011671765, -1.06581410364015e-14, 0, 0, 0, 0, 0, -2.33146835171283e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.11022302462516e-14, 0, 0, 0},
    {1.55884572681199, 0.547722557505173, 1.58113883008419, 2.45967477524976, 0, 9.07114735222146, 0, 1.8516401995451, 5.34522483824849, -1.20948631362953, 0, 0, 1.95399252334028e-14, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.02140518265514e-14, 0, 0, 0, 0, 0, 0, 0},
    {0.899999999999999, 1.4210854715202e-14, 1.46059348668045, 1.42009389360938, 0, 0, 9.07114735222145, 0, 4.93770719878694, -0.698297248755174, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.27279220613579, -6.2609903369994, 0, 3.83405790253615, 0, 0, 0, 10.5830052442584, 0, 5.18459255872628, 0, 1.06581410364015e-14, 1.95399252334028e-14, 0, 0, 0, 0, 1.06581410364015e-14, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.734846922834958, 0, -6.26099033699939, 2.21359436211786, 0, 0, 0, 0, 10.5830052442584, 2.99332590941916, 0, 0, 0, 0, 0, 0, 0, 0, 1.68753899743024e-14, 1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.7157676649773, 0, 0, -4.69574275274955, 0, 0, 0, 0, 0, 12.69960629311, 1.77635683940025e-14, 0, -1.33226762955019e-14, 0, -1.28785870856518e-14, 0, 2.79776202205539e-14, 0, 1.06581410364015e-14, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.49880108324396e-14, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0},
    {2.56904651573302, 5.80288457473996, -2.01017818278147, -1.42141062443803, 8.14215222512716, -2.10229466468021, 0.728256234341419, -1.71651640581399, 0.594618725379072, 0.420458932936049, 9.81070843517429, -1.18450885369835, 0.30583887092059, -0.105945692672798, -1.02581475831037, 0.264864231681985, -0.0917516612761808, 0.216260739578389, -0.0749149177264375, -0.0529728463363888, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239553, 3.07059789431495, 1.77281052085583, -1.25356634105602, 2.39356776939085, 6.30376871403131, -2.78313372537745, -0.908295106229252, -0.524404424085074, 0.370809924354786, 1.23603308118261, 9.40174755792014, -2.42752078113807, 0.840917865872084, -0.301561193523235, -0.794200206656964, 0.350641890608822, 0.114434427054266, 0.0660687472643433, -0.0467176592151159, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551268, 1.29756443747036, 3.74574588632196, -1.05945692672795, 0.404586796856345, 5.01426536422408, 1.80936716113937, -0.38382473678937, -1.10800657553489, 0.313391585264002, -1.06581410364015e-14, 2.82521847127454, 8.20651806648291, -2.84282124887606, -0.0509731451534328, -0.631738055305796, -0.227958835180075, 0.0483573714561643, 0.139595707137599, -0.0394836284566149, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 0.335029697130249, 4.06201920231797, -0.820651806648284, 0, 1.69926454679665, 5.88642506110456, -0.0991031208965145, -1.20156148406978, 0.242752078113805, 0, 0, 4.94413232473045, 5.13809303146606, 0, -0.214087209644424, -0.741619848709568, 0.0124858196210687, 0.151382517704866, -0.0305838870920656, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.856348838577684, 0, 2.68023757704195, -0.473803541479342, 0, 0, 5.09779364038993, 0, -0.792824967172093, 0.140152977645347, 0, -1.4210854715202e-14, 0, 7.91061171956872, 0, 0, -0.642261628933261, 0, 0.0998865569685847, -0.0176576154454708, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047609, 2.74642624930237, -0.792824967172094, 2.24244764232555, 2.14087209644419, -0.331662479035534, 0.0957427107756326, 6.49923072370876, -1.87616630392937, -2.15580611373101, 11.0554159678513, 0, 0, 0, 9.4403585933147, -1.46249406456535, 0.422185670932516, -2.04706526287665, 0.590936840285272, 0.522319308773344, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.71269767715536, 1.160576914948, 1.34011878852098, 1.89521416591736, 0.361873432227874, 1.96214168703486, -0.566421515598876, 2.74642624930237, 3.17129986868837, -1.82198870938951, 0, 11.0554159678513, 0, 0, 1.59571184626055, 8.6522315682783, -2.49768411251825, -0.865042958313561, -0.998865569685869, 0.441440386136648, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.04360964314765e-14, 1.95399252334028e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32664991614215, 0.299659670905754, 2.07610309995255, 1.46802658041887, 0, 0.868496237346892, 1.31624465810881, 0.709124208342329, 4.91295663090393, -1.4113063856887, 0, 0, 11.0554159678513, 0, 0, 3.82970843102537, 5.80409338312195, -0.223353131420178, -1.54743588659734, 0.341938252770122, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.76594168620508, 0, 1.49829835452878, 0.847565541382357, 0, 0, 1.62843044502542, 0, 3.54562104171167, -0.814818121686407, 0, 0, 0, 11.0554159678514, 0, 1.06581410364015e-14, 7.18070330817255, 0, -1.11676565710083, 0.197418142283059, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 1.00508909139074, -0.193429485824666, 3.55615782880927, -8.46157280212812, 0, 0, 4.459640440343, -0.858258202906995, 2.67027285925187, 0, -1.29340982368831e-14, 0, 0, 12.4373429638327, 0, 0, 8.42792824422443, -1.62195554683794, -2.29379153190448, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.1489125293076, 0.259512887494068, 0.749149177264412, 2.75458800949266, -1.24344978758018e-14, -8.46157280212812, 0, 1.1514742103681, 3.32401972660468, 2.06838446274244, 0, 0, 0, 1.24344978758018e-14, 0, 12.4373429638328, -2.22044604925031e-14, 2.17608171552749, 6.2818068211921, -1.77676328054753, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.663324958071075, 0, 0.692034366650866, 1.59036212878711, 0, 0, -8.46157280212812, 0, 3.07059789431494, 1.19418232635199, 0, 0, -1.4210854715202e-14, 0, 0, 1.50990331349021e-14, 12.4373429638327, 0, 5.80288457473998, -1.02581475831035, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.37667655053519e-14, 0, 0, 0, 0, 0, 0, 0},
    {0.938083151964683, 7.62808987244124, 0, 3.11415464992887, 0, 0, 0, -7.5213980463361, 0, 5.52707620976692, 0, 0, 0, 0, 0, 0, 2.39808173319034e-14, 14.2141062443803, 1.59872115546023e-14, 5.80288457473999, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.541602560309058, 0, 7.62808987244124, 1.79795802543455, 0, 0, 0, 0, -7.5213980463361, 3.19105893754051, 0, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 14.2141062443803, 3.35029697130246, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-5.36159180343551, 0, 0, 8.89943818451482, 0, 0, 0, 0, 1.4210854715202e-14, -5.26497863243527, 0, 0, 0, 0, 1.4210854715202e-14, 0, 1.01585406753202e-14, -1.4210854715202e-14, -1.06581410364015e-14, 16.583123951777, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14},
    {2.41593350361254, 5.72988941019469, -2.20543546226846, -1.55947837083932, 8.33749895885385, -2.76779953857108, 1.06532653910463, -2.25989885993663, 0.869835476750495, 0.615066564126917, 10.333118277332, -2.18326971917505, 0.724780051083766, -0.27896797173099, -1.89076704011891, 0.627677936394712, -0.24159335036126, 0.512496888990059, -0.197260144544837, -0.139483985865486, 11.8490505948789, -1.16109782023228, 0.245326690731332, -0.0814411017927985, 0.031346694695451, -1.03851746182648, 0.21942686286813, -0.0728431359084674, 0.0280373360835801, 0.190029237516525, -0.063084006188054, 0.0242810453028244, -0.0515078753637721, 0.0198253904711043, 0.0140186680417933, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.18529407725405, 3.45525332069938, 0.997445717412057, -1.41060126129511, 3.23209352764252, 5.28531208250977, -2.89087233497829, -1.36277028773849, -0.393397896234717, 0.556348640264194, 2.33666428910958, 9.02784807228868, -3.93352933905729, 1.64018416088945, -0.732969916135168, -1.19859611757306, 0.655588223176204, 0.309047252182627, 0.0892142571199752, -0.126168012376108, 1.1908743922773, 11.55277744383, -2.44096975264614, 0.810328731479533, -0.311895674167854, -0.234843597212092, -0.907332870306512, 0.395334573318419, -0.164844710568313, 0.0736662483163242, 0.120463442338381, -0.0658890955530682, -0.0310604175145517, -0.00896637020658309, 0.0126803623514131, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.92724822331887, 1.82834820066013, 2.99085908983323, -1.2440333788203, 0.950146187582614, 5.7406445631131, -0.169967317119762, -0.721110255092803, -1.1796107330308, 0.490653381462659, 0.294392028877592, 4.9761335152812, 5.78174466995659, -3.22682211353229, -0.215472901842833, -1.30185581821819, 0.0385449644663778, 0.163532434528617, 0.267510569439617, -0.111269728052837, 0, 2.64638753839399, 10.7636648722209, -3.57321383950879, 1.37533064807499, -0.0295875121532885, -0.500120235663637, -0.581087203147972, 0.324307824721458, 0.0216558414515666, 0.130841432732452, -0.00387391468765502, -0.016435628072264, -0.0268858238268355, 0.0111830284386731, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.62882203585592, 0.772618130456575, 3.86595114186567, -1.05140010313427, 0.160604018609906, 3.35889012281481, 3.878512233171, -0.304724700110032, -1.52475169285615, 0.414677792940099, 0, 1.135512111385, 7.32969916135161, 0.564240504518033, -0.0364215679542301, -0.761724681099366, -0.879563899362179, 0.0691050664139837, 0.34578118203618, -0.0940400840863451, 0, 0, 4.46577897103986, 9.05975063854702, -3.48709964663735, 0, -0.114123261162675, -0.736662483163209, -0.0567083043941503, 0.00366050530850437, 0.0765562109321125, 0.0883994979795846, -0.00694532049720609, -0.0347523163751409, 0.00945138406569246, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376112, 0.199489143482424, 3.68560354207932, -0.814411017927961, 0, 1.02786571910339, 6.00856531485322, -0.078679579246943, -1.45362164025439, 0.321208037219817, 0, 0, 3.0280322970267, 6.99294104721296, 0, -0.233098034907105, -1.36261453366201, 0.0178428514239979, 0.329650402328133, -0.0728431359084714, 0, 0, 0, 6.80499652729884, 5.40219154319687, 0, 0, -0.304328696433807, -0.702817019232741, 0, 0.0234272339744261, 0.136947913395214, -0.00179327404131646, -0.0331311120155191, 0.00732101061700707, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.72843135908468, 0, 2.32737334062817, -0.470200420431707, 0, 0, 4.49691252107735, 0, -0.91792842454769, 0.185449546754732, 0, 0, 1.77635683940025e-14, 7.0654086930623, 0, 0, -1.01980390271858, 0, 0.208166599946617, -0.0420560041253766, 0, 0, 0, 0, 9.92395326897747, 0, 0, 0, -0.710100291678885, 0, 0, 0.102494148638109, 0, -0.02092153048203, 0.00422678745046823, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.99489143482414, 3.1542003094028, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275054, 0.263899331455879, 5.59815020469134, -1.93925611658551, -2.28543525082517, 2.13307290077014, -0.257539376818855, 0.0664963811608086, -0.023035022137995, 9.14446135479766, -2.36108976914458, 0.817905488277891, -3.3384229850651, 1.15646364545769, 1.02505848514082, 13.0453753561106, 0, 0, 0, 0, 11.5766222110848, -1.39771878815471, 0.360889439280295, -0.125015768949702, -2.15192884958005, 0.555625639776459, -0.192474367616154, 0.595437196138649, -0.206265495285697, -0.166687691932934, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.7593288763725, 1.66904592079255, 0.963624111659421, 1.36277028773849, 0.867360833110883, 2.28430636201793, -1.00852844760553, 2.96226264871971, 1.71026313764871, -2.01556443707463, 0.268741924943274, 2.04415543160775, -0.52779866291175, 0.182834820066012, 2.68821895760393, 7.07977053255619, -3.12574097035514, -1.76652739792483, -1.01990506872276, 0.904016610326405, 0, 13.0453753561106, 0, 0, 1.77635683940025e-14, 1.4585173044132, 11.0940489487798, -2.8644711213756, 0.992281903807275, -0.632607619454901, -1.66605356688627, 0.735567892896449, 0.31507574846809, 0.181909068193172, -0.147004726489962, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, -1.33226762955019e-14, 0},
    {1.48690428533297, 0.705300630647553, 2.03602754481991, 1.15175110689979, 0.146610739697709, 1.817027050318, 0.655663160391203, 1.25178444059441, 3.61359041872286, -1.70346285967311, 0, 0.614267257013221, 1.78428514239955, -0.618094504365256, 0.454391937932176, 5.63152770643899, 2.03210252328836, -0.746494073225344, -2.15494277062551, 0.764033484562614, 0, 0, 13.0453753561106, 0, 0, 0, 3.33375383865873, 9.68367982311023, -3.3545250915713, -0.106930204231195, -1.32524448062076, -0.478206411017788, 0.133143804689793, 0.384353057392904, -0.124241670058193, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.37667655053519e-14, 0},
    {1.15175110689979, 0.182107839771145, 2.20794021658194, 0.892142571199776, 0, 0.61576510673037, 2.13307290077016, 0.323209352764256, 3.91870514328395, -1.31949665727936, 0, -1.06581410364015e-14, 1.07496769977314, 1.11713920350151, 0, 1.90844613931508, 6.61105135360481, -0.192743940909633, -2.33689608954559, 0.591817792331167, 0, 0, 0, 13.0453753561106, 0, 0, 0, 5.83406921765278, 6.06294257990893, 0, -0.44910685777103, -1.55575179137407, 0.0343775825476159, 0.41680603729705, -0.0962371838080759, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 0},
    {0.66496381160804, 0, 1.45686271816934, 0.515078753637713, 0, -1.24344978758018e-14, 1.84729532019112, 0, 2.58567482211401, -0.761811750275046, 1.4210854715202e-14, -1.24344978758018e-14, 0, 1.71994831963701, 0, -1.4210854715202e-14, 5.72533841794525, 0, -1.54195152727693, 0.341686161713598, 0, 0, 0, 0, 13.0453753561106, 0, 0, 0, 9.33451074824443, 0, 0, -1.34732057331309, 0, 0.27502066038093, -0.0555625639776447, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0},
    {1.57359158493889, 1.49284005458435, -0.430945803685675, 3.04724700110023, 0.775791113542727, -0.120185042515474, 0.0346944333244412, 5.2990565197967, -1.52970585407783, 0.901387818865993, -10.5762837413611, 0, 0, 0, 4.5156107826894, -0.69955541437243, 0.201944253400492, 6.36462844462228, -1.83730997289733, -2.74825341360596, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 0, 10.8920630294282, -1.68739114875268, 0.487107866980279, -3.09993548319961, 0.894874292847885, 0.903959543974656, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32992762321608, 0.63084006188056, 0.728431359084682, 2.57539376818856, 0.131132632078239, 0.71102430025671, -0.205255035576787, 2.23926008187652, 2.58567482211401, 0.761811750275053, 0, -10.576283741361, 0, 0, 0.763277533137579, 4.1386256441266, -1.19471831485579, 2.68954640486935, 3.1056206816986, -2.32269806558125, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 1.84109468241849, 9.98274066131697, -2.88176900403079, -1.30996183150502, -1.51261363209512, 0.763985254692621, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.28785870856518e-14, 0, 0},
    {1.03015750727542, 0.162882203585586, 1.1284810090361, 1.99489143482414, 0, 0.314718316987764, 0.476969600708468, 0.57817446699566, 4.00571020990214, 0.590096844352083, 0, 0, -10.5762837413611, 0, 0, 1.83186607953019, 2.77627448210727, 0.694437895660723, 4.81120687194241, -1.79915418525285, 0, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 4.41862723780437, 6.69662601613678, -0.338230690505752, -2.34333096254029, 0.591780433634522, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.14352971536391e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.594761714133174, -1.19904086659517e-14, 0.814411017927931, 1.15175110689982, 0, 0, 0.590096844352076, 0, 2.89087233497831, 0.340692571934628, 0, 0, 0, -10.576283741361, 0, 0, 3.43474889911912, -1.24344978758018e-14, 3.47218947830365, -1.03874215316938, 0, 0, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 8.28492607088319, 0, -1.69115345252878, 0.341664592660043, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.68753899743024e-14, 0},
    {1.15175110689979, 0.546323519313511, -0.105140010313427, 3.27118942773251, 9.65295620481434, 0, 0, 2.69341127303542, -0.518347241175114, 4.25171145123353, 0, 0, 0, 0, -9.91526600752598, 0, 1.53210777398272e-14, 6.71889089790563, -1.29305115618718, 1.82865048192216, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.9443476574685, 0, 0, 9.45383520059451, -1.81939143264587, -2.94058061635793, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 1.68753899743024e-14},
    {0.892142571199775, 0.141060126129508, 0.407205508963975, 2.53385243517955, 0, 9.65295620481433, 0, 0.69543580033025, 2.00755023262383, 3.29336152870135, 0, 0, 0, 0, 0, -9.91526600752598, 2.12052597703405e-14, 1.73481017017154, 5.0079655937072, 1.41646657250371, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.9443476574685, 0, 2.44096975264614, 7.0464727188699, -2.27776395106693, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.515078753637709, 0, 0.376160336345346, 1.46292038553773, 0, 0, 9.65295620481433, 0, 1.8544954675473, 1.9014231651345, 0, 0, 0, -1.06581410364015e-14, 0, 0, -9.91526600752595, 0, 4.62616045379076, 0.81779735693317, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.9443476574685, 0, 6.50925267372302, -1.3150676302989, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 1.86517468137026e-14},
    {0.728431359084674, -7.94708263760857, 0, 2.53908227033117, 0, 0, 0, 10.2207771580387, 0, 5.00713776237767, 0, 0, 0, 0, -1.33226762955019e-14, 2.1316282072803e-14, 0, -8.49879943502227, -1.01030295240889e-14, 6.93924068068615, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 17.9373911146521, 0, 6.2767793639472, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0},
    {0.42056004125371, 0, -7.94708263760858, 1.46593983227034, 0, 0, 0, 0, 10.2207771580387, 2.89087233497828, 0, 0, 0, -1.06581410364015e-14, 2.1316282072803e-14, -1.77635683940025e-14, 2.62012633811537e-14, 0, -8.49879943502228, 4.00637247496576, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 17.9373911146521, 3.6239002554188, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.22124532708767e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {6.54237885546499, 0, 1.50990331349021e-14, -6.91050664139879, 0, 0, 0, 0, 0, 12.2649325896464, 0, -1.79856129989275e-14, 1.77635683940025e-14, 1.33226762955019e-14, 0, 0, -1.75415237890775e-14, 0, 1.06581410364015e-14, -5.66586629001485, 0, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 20.4998755596024, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.46549439250521e-14, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, -1.59872115546023e-14}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    double coeff0_6 = 0;
    double coeff0_7 = 0;
    double coeff0_8 = 0;
    double coeff0_9 = 0;
    double coeff0_10 = 0;
    double coeff0_11 = 0;
    double coeff0_12 = 0;
    double coeff0_13 = 0;
    double coeff0_14 = 0;
    double coeff0_15 = 0;
    double coeff0_16 = 0;
    double coeff0_17 = 0;
    double coeff0_18 = 0;
    double coeff0_19 = 0;
    double coeff0_20 = 0;
    double coeff0_21 = 0;
    double coeff0_22 = 0;
    double coeff0_23 = 0;
    double coeff0_24 = 0;
    double coeff0_25 = 0;
    double coeff0_26 = 0;
    double coeff0_27 = 0;
    double coeff0_28 = 0;
    double coeff0_29 = 0;
    double coeff0_30 = 0;
    double coeff0_31 = 0;
    double coeff0_32 = 0;
    double coeff0_33 = 0;
    double coeff0_34 = 0;
    double coeff0_35 = 0;
    double coeff0_36 = 0;
    double coeff0_37 = 0;
    double coeff0_38 = 0;
    double coeff0_39 = 0;
    double coeff0_40 = 0;
    double coeff0_41 = 0;
    double coeff0_42 = 0;
    double coeff0_43 = 0;
    double coeff0_44 = 0;
    double coeff0_45 = 0;
    double coeff0_46 = 0;
    double coeff0_47 = 0;
    double coeff0_48 = 0;
    double coeff0_49 = 0;
    double coeff0_50 = 0;
    double coeff0_51 = 0;
    double coeff0_52 = 0;
    double coeff0_53 = 0;
    double coeff0_54 = 0;
    double coeff0_55 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    double new_coeff0_6 = 0;
    double new_coeff0_7 = 0;
    double new_coeff0_8 = 0;
    double new_coeff0_9 = 0;
    double new_coeff0_10 = 0;
    double new_coeff0_11 = 0;
    double new_coeff0_12 = 0;
    double new_coeff0_13 = 0;
    double new_coeff0_14 = 0;
    double new_coeff0_15 = 0;
    double new_coeff0_16 = 0;
    double new_coeff0_17 = 0;
    double new_coeff0_18 = 0;
    double new_coeff0_19 = 0;
    double new_coeff0_20 = 0;
    double new_coeff0_21 = 0;
    double new_coeff0_22 = 0;
    double new_coeff0_23 = 0;
    double new_coeff0_24 = 0;
    double new_coeff0_25 = 0;
    double new_coeff0_26 = 0;
    double new_coeff0_27 = 0;
    double new_coeff0_28 = 0;
    double new_coeff0_29 = 0;
    double new_coeff0_30 = 0;
    double new_coeff0_31 = 0;
    double new_coeff0_32 = 0;
    double new_coeff0_33 = 0;
    double new_coeff0_34 = 0;
    double new_coeff0_35 = 0;
    double new_coeff0_36 = 0;
    double new_coeff0_37 = 0;
    double new_coeff0_38 = 0;
    double new_coeff0_39 = 0;
    double new_coeff0_40 = 0;
    double new_coeff0_41 = 0;
    double new_coeff0_42 = 0;
    double new_coeff0_43 = 0;
    double new_coeff0_44 = 0;
    double new_coeff0_45 = 0;
    double new_coeff0_46 = 0;
    double new_coeff0_47 = 0;
    double new_coeff0_48 = 0;
    double new_coeff0_49 = 0;
    double new_coeff0_50 = 0;
    double new_coeff0_51 = 0;
    double new_coeff0_52 = 0;
    double new_coeff0_53 = 0;
    double new_coeff0_54 = 0;
    double new_coeff0_55 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
      new_coeff0_6 = coefficients0[dof][6];
      new_coeff0_7 = coefficients0[dof][7];
      new_coeff0_8 = coefficients0[dof][8];
      new_coeff0_9 = coefficients0[dof][9];
      new_coeff0_10 = coefficients0[dof][10];
      new_coeff0_11 = coefficients0[dof][11];
      new_coeff0_12 = coefficients0[dof][12];
      new_coeff0_13 = coefficients0[dof][13];
      new_coeff0_14 = coefficients0[dof][14];
      new_coeff0_15 = coefficients0[dof][15];
      new_coeff0_16 = coefficients0[dof][16];
      new_coeff0_17 = coefficients0[dof][17];
      new_coeff0_18 = coefficients0[dof][18];
      new_coeff0_19 = coefficients0[dof][19];
      new_coeff0_20 = coefficients0[dof][20];
      new_coeff0_21 = coefficients0[dof][21];
      new_coeff0_22 = coefficients0[dof][22];
      new_coeff0_23 = coefficients0[dof][23];
      new_coeff0_24 = coefficients0[dof][24];
      new_coeff0_25 = coefficients0[dof][25];
      new_coeff0_26 = coefficients0[dof][26];
      new_coeff0_27 = coefficients0[dof][27];
      new_coeff0_28 = coefficients0[dof][28];
      new_coeff0_29 = coefficients0[dof][29];
      new_coeff0_30 = coefficients0[dof][30];
      new_coeff0_31 = coefficients0[dof][31];
      new_coeff0_32 = coefficients0[dof][32];
      new_coeff0_33 = coefficients0[dof][33];
      new_coeff0_34 = coefficients0[dof][34];
      new_coeff0_35 = coefficients0[dof][35];
      new_coeff0_36 = coefficients0[dof][36];
      new_coeff0_37 = coefficients0[dof][37];
      new_coeff0_38 = coefficients0[dof][38];
      new_coeff0_39 = coefficients0[dof][39];
      new_coeff0_40 = coefficients0[dof][40];
      new_coeff0_41 = coefficients0[dof][41];
      new_coeff0_42 = coefficients0[dof][42];
      new_coeff0_43 = coefficients0[dof][43];
      new_coeff0_44 = coefficients0[dof][44];
      new_coeff0_45 = coefficients0[dof][45];
      new_coeff0_46 = coefficients0[dof][46];
      new_coeff0_47 = coefficients0[dof][47];
      new_coeff0_48 = coefficients0[dof][48];
      new_coeff0_49 = coefficients0[dof][49];
      new_coeff0_50 = coefficients0[dof][50];
      new_coeff0_51 = coefficients0[dof][51];
      new_coeff0_52 = coefficients0[dof][52];
      new_coeff0_53 = coefficients0[dof][53];
      new_coeff0_54 = coefficients0[dof][54];
      new_coeff0_55 = coefficients0[dof][55];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
        coeff0_6 = new_coeff0_6;
        coeff0_7 = new_coeff0_7;
        coeff0_8 = new_coeff0_8;
        coeff0_9 = new_coeff0_9;
        coeff0_10 = new_coeff0_10;
        coeff0_11 = new_coeff0_11;
        coeff0_12 = new_coeff0_12;
        coeff0_13 = new_coeff0_13;
        coeff0_14 = new_coeff0_14;
        coeff0_15 = new_coeff0_15;
        coeff0_16 = new_coeff0_16;
        coeff0_17 = new_coeff0_17;
        coeff0_18 = new_coeff0_18;
        coeff0_19 = new_coeff0_19;
        coeff0_20 = new_coeff0_20;
        coeff0_21 = new_coeff0_21;
        coeff0_22 = new_coeff0_22;
        coeff0_23 = new_coeff0_23;
        coeff0_24 = new_coeff0_24;
        coeff0_25 = new_coeff0_25;
        coeff0_26 = new_coeff0_26;
        coeff0_27 = new_coeff0_27;
        coeff0_28 = new_coeff0_28;
        coeff0_29 = new_coeff0_29;
        coeff0_30 = new_coeff0_30;
        coeff0_31 = new_coeff0_31;
        coeff0_32 = new_coeff0_32;
        coeff0_33 = new_coeff0_33;
        coeff0_34 = new_coeff0_34;
        coeff0_35 = new_coeff0_35;
        coeff0_36 = new_coeff0_36;
        coeff0_37 = new_coeff0_37;
        coeff0_38 = new_coeff0_38;
        coeff0_39 = new_coeff0_39;
        coeff0_40 = new_coeff0_40;
        coeff0_41 = new_coeff0_41;
        coeff0_42 = new_coeff0_42;
        coeff0_43 = new_coeff0_43;
        coeff0_44 = new_coeff0_44;
        coeff0_45 = new_coeff0_45;
        coeff0_46 = new_coeff0_46;
        coeff0_47 = new_coeff0_47;
        coeff0_48 = new_coeff0_48;
        coeff0_49 = new_coeff0_49;
        coeff0_50 = new_coeff0_50;
        coeff0_51 = new_coeff0_51;
        coeff0_52 = new_coeff0_52;
        coeff0_53 = new_coeff0_53;
        coeff0_54 = new_coeff0_54;
        coeff0_55 = new_coeff0_55;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0] + coeff0_10*dmats0[10][0] + coeff0_11*dmats0[11][0] + coeff0_12*dmats0[12][0] + coeff0_13*dmats0[13][0] + coeff0_14*dmats0[14][0] + coeff0_15*dmats0[15][0] + coeff0_16*dmats0[16][0] + coeff0_17*dmats0[17][0] + coeff0_18*dmats0[18][0] + coeff0_19*dmats0[19][0] + coeff0_20*dmats0[20][0] + coeff0_21*dmats0[21][0] + coeff0_22*dmats0[22][0] + coeff0_23*dmats0[23][0] + coeff0_24*dmats0[24][0] + coeff0_25*dmats0[25][0] + coeff0_26*dmats0[26][0] + coeff0_27*dmats0[27][0] + coeff0_28*dmats0[28][0] + coeff0_29*dmats0[29][0] + coeff0_30*dmats0[30][0] + coeff0_31*dmats0[31][0] + coeff0_32*dmats0[32][0] + coeff0_33*dmats0[33][0] + coeff0_34*dmats0[34][0] + coeff0_35*dmats0[35][0] + coeff0_36*dmats0[36][0] + coeff0_37*dmats0[37][0] + coeff0_38*dmats0[38][0] + coeff0_39*dmats0[39][0] + coeff0_40*dmats0[40][0] + coeff0_41*dmats0[41][0] + coeff0_42*dmats0[42][0] + coeff0_43*dmats0[43][0] + coeff0_44*dmats0[44][0] + coeff0_45*dmats0[45][0] + coeff0_46*dmats0[46][0] + coeff0_47*dmats0[47][0] + coeff0_48*dmats0[48][0] + coeff0_49*dmats0[49][0] + coeff0_50*dmats0[50][0] + coeff0_51*dmats0[51][0] + coeff0_52*dmats0[52][0] + coeff0_53*dmats0[53][0] + coeff0_54*dmats0[54][0] + coeff0_55*dmats0[55][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1] + coeff0_10*dmats0[10][1] + coeff0_11*dmats0[11][1] + coeff0_12*dmats0[12][1] + coeff0_13*dmats0[13][1] + coeff0_14*dmats0[14][1] + coeff0_15*dmats0[15][1] + coeff0_16*dmats0[16][1] + coeff0_17*dmats0[17][1] + coeff0_18*dmats0[18][1] + coeff0_19*dmats0[19][1] + coeff0_20*dmats0[20][1] + coeff0_21*dmats0[21][1] + coeff0_22*dmats0[22][1] + coeff0_23*dmats0[23][1] + coeff0_24*dmats0[24][1] + coeff0_25*dmats0[25][1] + coeff0_26*dmats0[26][1] + coeff0_27*dmats0[27][1] + coeff0_28*dmats0[28][1] + coeff0_29*dmats0[29][1] + coeff0_30*dmats0[30][1] + coeff0_31*dmats0[31][1] + coeff0_32*dmats0[32][1] + coeff0_33*dmats0[33][1] + coeff0_34*dmats0[34][1] + coeff0_35*dmats0[35][1] + coeff0_36*dmats0[36][1] + coeff0_37*dmats0[37][1] + coeff0_38*dmats0[38][1] + coeff0_39*dmats0[39][1] + coeff0_40*dmats0[40][1] + coeff0_41*dmats0[41][1] + coeff0_42*dmats0[42][1] + coeff0_43*dmats0[43][1] + coeff0_44*dmats0[44][1] + coeff0_45*dmats0[45][1] + coeff0_46*dmats0[46][1] + coeff0_47*dmats0[47][1] + coeff0_48*dmats0[48][1] + coeff0_49*dmats0[49][1] + coeff0_50*dmats0[50][1] + coeff0_51*dmats0[51][1] + coeff0_52*dmats0[52][1] + coeff0_53*dmats0[53][1] + coeff0_54*dmats0[54][1] + coeff0_55*dmats0[55][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2] + coeff0_10*dmats0[10][2] + coeff0_11*dmats0[11][2] + coeff0_12*dmats0[12][2] + coeff0_13*dmats0[13][2] + coeff0_14*dmats0[14][2] + coeff0_15*dmats0[15][2] + coeff0_16*dmats0[16][2] + coeff0_17*dmats0[17][2] + coeff0_18*dmats0[18][2] + coeff0_19*dmats0[19][2] + coeff0_20*dmats0[20][2] + coeff0_21*dmats0[21][2] + coeff0_22*dmats0[22][2] + coeff0_23*dmats0[23][2] + coeff0_24*dmats0[24][2] + coeff0_25*dmats0[25][2] + coeff0_26*dmats0[26][2] + coeff0_27*dmats0[27][2] + coeff0_28*dmats0[28][2] + coeff0_29*dmats0[29][2] + coeff0_30*dmats0[30][2] + coeff0_31*dmats0[31][2] + coeff0_32*dmats0[32][2] + coeff0_33*dmats0[33][2] + coeff0_34*dmats0[34][2] + coeff0_35*dmats0[35][2] + coeff0_36*dmats0[36][2] + coeff0_37*dmats0[37][2] + coeff0_38*dmats0[38][2] + coeff0_39*dmats0[39][2] + coeff0_40*dmats0[40][2] + coeff0_41*dmats0[41][2] + coeff0_42*dmats0[42][2] + coeff0_43*dmats0[43][2] + coeff0_44*dmats0[44][2] + coeff0_45*dmats0[45][2] + coeff0_46*dmats0[46][2] + coeff0_47*dmats0[47][2] + coeff0_48*dmats0[48][2] + coeff0_49*dmats0[49][2] + coeff0_50*dmats0[50][2] + coeff0_51*dmats0[51][2] + coeff0_52*dmats0[52][2] + coeff0_53*dmats0[53][2] + coeff0_54*dmats0[54][2] + coeff0_55*dmats0[55][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3] + coeff0_10*dmats0[10][3] + coeff0_11*dmats0[11][3] + coeff0_12*dmats0[12][3] + coeff0_13*dmats0[13][3] + coeff0_14*dmats0[14][3] + coeff0_15*dmats0[15][3] + coeff0_16*dmats0[16][3] + coeff0_17*dmats0[17][3] + coeff0_18*dmats0[18][3] + coeff0_19*dmats0[19][3] + coeff0_20*dmats0[20][3] + coeff0_21*dmats0[21][3] + coeff0_22*dmats0[22][3] + coeff0_23*dmats0[23][3] + coeff0_24*dmats0[24][3] + coeff0_25*dmats0[25][3] + coeff0_26*dmats0[26][3] + coeff0_27*dmats0[27][3] + coeff0_28*dmats0[28][3] + coeff0_29*dmats0[29][3] + coeff0_30*dmats0[30][3] + coeff0_31*dmats0[31][3] + coeff0_32*dmats0[32][3] + coeff0_33*dmats0[33][3] + coeff0_34*dmats0[34][3] + coeff0_35*dmats0[35][3] + coeff0_36*dmats0[36][3] + coeff0_37*dmats0[37][3] + coeff0_38*dmats0[38][3] + coeff0_39*dmats0[39][3] + coeff0_40*dmats0[40][3] + coeff0_41*dmats0[41][3] + coeff0_42*dmats0[42][3] + coeff0_43*dmats0[43][3] + coeff0_44*dmats0[44][3] + coeff0_45*dmats0[45][3] + coeff0_46*dmats0[46][3] + coeff0_47*dmats0[47][3] + coeff0_48*dmats0[48][3] + coeff0_49*dmats0[49][3] + coeff0_50*dmats0[50][3] + coeff0_51*dmats0[51][3] + coeff0_52*dmats0[52][3] + coeff0_53*dmats0[53][3] + coeff0_54*dmats0[54][3] + coeff0_55*dmats0[55][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4] + coeff0_10*dmats0[10][4] + coeff0_11*dmats0[11][4] + coeff0_12*dmats0[12][4] + coeff0_13*dmats0[13][4] + coeff0_14*dmats0[14][4] + coeff0_15*dmats0[15][4] + coeff0_16*dmats0[16][4] + coeff0_17*dmats0[17][4] + coeff0_18*dmats0[18][4] + coeff0_19*dmats0[19][4] + coeff0_20*dmats0[20][4] + coeff0_21*dmats0[21][4] + coeff0_22*dmats0[22][4] + coeff0_23*dmats0[23][4] + coeff0_24*dmats0[24][4] + coeff0_25*dmats0[25][4] + coeff0_26*dmats0[26][4] + coeff0_27*dmats0[27][4] + coeff0_28*dmats0[28][4] + coeff0_29*dmats0[29][4] + coeff0_30*dmats0[30][4] + coeff0_31*dmats0[31][4] + coeff0_32*dmats0[32][4] + coeff0_33*dmats0[33][4] + coeff0_34*dmats0[34][4] + coeff0_35*dmats0[35][4] + coeff0_36*dmats0[36][4] + coeff0_37*dmats0[37][4] + coeff0_38*dmats0[38][4] + coeff0_39*dmats0[39][4] + coeff0_40*dmats0[40][4] + coeff0_41*dmats0[41][4] + coeff0_42*dmats0[42][4] + coeff0_43*dmats0[43][4] + coeff0_44*dmats0[44][4] + coeff0_45*dmats0[45][4] + coeff0_46*dmats0[46][4] + coeff0_47*dmats0[47][4] + coeff0_48*dmats0[48][4] + coeff0_49*dmats0[49][4] + coeff0_50*dmats0[50][4] + coeff0_51*dmats0[51][4] + coeff0_52*dmats0[52][4] + coeff0_53*dmats0[53][4] + coeff0_54*dmats0[54][4] + coeff0_55*dmats0[55][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5] + coeff0_10*dmats0[10][5] + coeff0_11*dmats0[11][5] + coeff0_12*dmats0[12][5] + coeff0_13*dmats0[13][5] + coeff0_14*dmats0[14][5] + coeff0_15*dmats0[15][5] + coeff0_16*dmats0[16][5] + coeff0_17*dmats0[17][5] + coeff0_18*dmats0[18][5] + coeff0_19*dmats0[19][5] + coeff0_20*dmats0[20][5] + coeff0_21*dmats0[21][5] + coeff0_22*dmats0[22][5] + coeff0_23*dmats0[23][5] + coeff0_24*dmats0[24][5] + coeff0_25*dmats0[25][5] + coeff0_26*dmats0[26][5] + coeff0_27*dmats0[27][5] + coeff0_28*dmats0[28][5] + coeff0_29*dmats0[29][5] + coeff0_30*dmats0[30][5] + coeff0_31*dmats0[31][5] + coeff0_32*dmats0[32][5] + coeff0_33*dmats0[33][5] + coeff0_34*dmats0[34][5] + coeff0_35*dmats0[35][5] + coeff0_36*dmats0[36][5] + coeff0_37*dmats0[37][5] + coeff0_38*dmats0[38][5] + coeff0_39*dmats0[39][5] + coeff0_40*dmats0[40][5] + coeff0_41*dmats0[41][5] + coeff0_42*dmats0[42][5] + coeff0_43*dmats0[43][5] + coeff0_44*dmats0[44][5] + coeff0_45*dmats0[45][5] + coeff0_46*dmats0[46][5] + coeff0_47*dmats0[47][5] + coeff0_48*dmats0[48][5] + coeff0_49*dmats0[49][5] + coeff0_50*dmats0[50][5] + coeff0_51*dmats0[51][5] + coeff0_52*dmats0[52][5] + coeff0_53*dmats0[53][5] + coeff0_54*dmats0[54][5] + coeff0_55*dmats0[55][5];
          new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6] + coeff0_10*dmats0[10][6] + coeff0_11*dmats0[11][6] + coeff0_12*dmats0[12][6] + coeff0_13*dmats0[13][6] + coeff0_14*dmats0[14][6] + coeff0_15*dmats0[15][6] + coeff0_16*dmats0[16][6] + coeff0_17*dmats0[17][6] + coeff0_18*dmats0[18][6] + coeff0_19*dmats0[19][6] + coeff0_20*dmats0[20][6] + coeff0_21*dmats0[21][6] + coeff0_22*dmats0[22][6] + coeff0_23*dmats0[23][6] + coeff0_24*dmats0[24][6] + coeff0_25*dmats0[25][6] + coeff0_26*dmats0[26][6] + coeff0_27*dmats0[27][6] + coeff0_28*dmats0[28][6] + coeff0_29*dmats0[29][6] + coeff0_30*dmats0[30][6] + coeff0_31*dmats0[31][6] + coeff0_32*dmats0[32][6] + coeff0_33*dmats0[33][6] + coeff0_34*dmats0[34][6] + coeff0_35*dmats0[35][6] + coeff0_36*dmats0[36][6] + coeff0_37*dmats0[37][6] + coeff0_38*dmats0[38][6] + coeff0_39*dmats0[39][6] + coeff0_40*dmats0[40][6] + coeff0_41*dmats0[41][6] + coeff0_42*dmats0[42][6] + coeff0_43*dmats0[43][6] + coeff0_44*dmats0[44][6] + coeff0_45*dmats0[45][6] + coeff0_46*dmats0[46][6] + coeff0_47*dmats0[47][6] + coeff0_48*dmats0[48][6] + coeff0_49*dmats0[49][6] + coeff0_50*dmats0[50][6] + coeff0_51*dmats0[51][6] + coeff0_52*dmats0[52][6] + coeff0_53*dmats0[53][6] + coeff0_54*dmats0[54][6] + coeff0_55*dmats0[55][6];
          new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7] + coeff0_10*dmats0[10][7] + coeff0_11*dmats0[11][7] + coeff0_12*dmats0[12][7] + coeff0_13*dmats0[13][7] + coeff0_14*dmats0[14][7] + coeff0_15*dmats0[15][7] + coeff0_16*dmats0[16][7] + coeff0_17*dmats0[17][7] + coeff0_18*dmats0[18][7] + coeff0_19*dmats0[19][7] + coeff0_20*dmats0[20][7] + coeff0_21*dmats0[21][7] + coeff0_22*dmats0[22][7] + coeff0_23*dmats0[23][7] + coeff0_24*dmats0[24][7] + coeff0_25*dmats0[25][7] + coeff0_26*dmats0[26][7] + coeff0_27*dmats0[27][7] + coeff0_28*dmats0[28][7] + coeff0_29*dmats0[29][7] + coeff0_30*dmats0[30][7] + coeff0_31*dmats0[31][7] + coeff0_32*dmats0[32][7] + coeff0_33*dmats0[33][7] + coeff0_34*dmats0[34][7] + coeff0_35*dmats0[35][7] + coeff0_36*dmats0[36][7] + coeff0_37*dmats0[37][7] + coeff0_38*dmats0[38][7] + coeff0_39*dmats0[39][7] + coeff0_40*dmats0[40][7] + coeff0_41*dmats0[41][7] + coeff0_42*dmats0[42][7] + coeff0_43*dmats0[43][7] + coeff0_44*dmats0[44][7] + coeff0_45*dmats0[45][7] + coeff0_46*dmats0[46][7] + coeff0_47*dmats0[47][7] + coeff0_48*dmats0[48][7] + coeff0_49*dmats0[49][7] + coeff0_50*dmats0[50][7] + coeff0_51*dmats0[51][7] + coeff0_52*dmats0[52][7] + coeff0_53*dmats0[53][7] + coeff0_54*dmats0[54][7] + coeff0_55*dmats0[55][7];
          new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8] + coeff0_10*dmats0[10][8] + coeff0_11*dmats0[11][8] + coeff0_12*dmats0[12][8] + coeff0_13*dmats0[13][8] + coeff0_14*dmats0[14][8] + coeff0_15*dmats0[15][8] + coeff0_16*dmats0[16][8] + coeff0_17*dmats0[17][8] + coeff0_18*dmats0[18][8] + coeff0_19*dmats0[19][8] + coeff0_20*dmats0[20][8] + coeff0_21*dmats0[21][8] + coeff0_22*dmats0[22][8] + coeff0_23*dmats0[23][8] + coeff0_24*dmats0[24][8] + coeff0_25*dmats0[25][8] + coeff0_26*dmats0[26][8] + coeff0_27*dmats0[27][8] + coeff0_28*dmats0[28][8] + coeff0_29*dmats0[29][8] + coeff0_30*dmats0[30][8] + coeff0_31*dmats0[31][8] + coeff0_32*dmats0[32][8] + coeff0_33*dmats0[33][8] + coeff0_34*dmats0[34][8] + coeff0_35*dmats0[35][8] + coeff0_36*dmats0[36][8] + coeff0_37*dmats0[37][8] + coeff0_38*dmats0[38][8] + coeff0_39*dmats0[39][8] + coeff0_40*dmats0[40][8] + coeff0_41*dmats0[41][8] + coeff0_42*dmats0[42][8] + coeff0_43*dmats0[43][8] + coeff0_44*dmats0[44][8] + coeff0_45*dmats0[45][8] + coeff0_46*dmats0[46][8] + coeff0_47*dmats0[47][8] + coeff0_48*dmats0[48][8] + coeff0_49*dmats0[49][8] + coeff0_50*dmats0[50][8] + coeff0_51*dmats0[51][8] + coeff0_52*dmats0[52][8] + coeff0_53*dmats0[53][8] + coeff0_54*dmats0[54][8] + coeff0_55*dmats0[55][8];
          new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9] + coeff0_10*dmats0[10][9] + coeff0_11*dmats0[11][9] + coeff0_12*dmats0[12][9] + coeff0_13*dmats0[13][9] + coeff0_14*dmats0[14][9] + coeff0_15*dmats0[15][9] + coeff0_16*dmats0[16][9] + coeff0_17*dmats0[17][9] + coeff0_18*dmats0[18][9] + coeff0_19*dmats0[19][9] + coeff0_20*dmats0[20][9] + coeff0_21*dmats0[21][9] + coeff0_22*dmats0[22][9] + coeff0_23*dmats0[23][9] + coeff0_24*dmats0[24][9] + coeff0_25*dmats0[25][9] + coeff0_26*dmats0[26][9] + coeff0_27*dmats0[27][9] + coeff0_28*dmats0[28][9] + coeff0_29*dmats0[29][9] + coeff0_30*dmats0[30][9] + coeff0_31*dmats0[31][9] + coeff0_32*dmats0[32][9] + coeff0_33*dmats0[33][9] + coeff0_34*dmats0[34][9] + coeff0_35*dmats0[35][9] + coeff0_36*dmats0[36][9] + coeff0_37*dmats0[37][9] + coeff0_38*dmats0[38][9] + coeff0_39*dmats0[39][9] + coeff0_40*dmats0[40][9] + coeff0_41*dmats0[41][9] + coeff0_42*dmats0[42][9] + coeff0_43*dmats0[43][9] + coeff0_44*dmats0[44][9] + coeff0_45*dmats0[45][9] + coeff0_46*dmats0[46][9] + coeff0_47*dmats0[47][9] + coeff0_48*dmats0[48][9] + coeff0_49*dmats0[49][9] + coeff0_50*dmats0[50][9] + coeff0_51*dmats0[51][9] + coeff0_52*dmats0[52][9] + coeff0_53*dmats0[53][9] + coeff0_54*dmats0[54][9] + coeff0_55*dmats0[55][9];
          new_coeff0_10 = coeff0_0*dmats0[0][10] + coeff0_1*dmats0[1][10] + coeff0_2*dmats0[2][10] + coeff0_3*dmats0[3][10] + coeff0_4*dmats0[4][10] + coeff0_5*dmats0[5][10] + coeff0_6*dmats0[6][10] + coeff0_7*dmats0[7][10] + coeff0_8*dmats0[8][10] + coeff0_9*dmats0[9][10] + coeff0_10*dmats0[10][10] + coeff0_11*dmats0[11][10] + coeff0_12*dmats0[12][10] + coeff0_13*dmats0[13][10] + coeff0_14*dmats0[14][10] + coeff0_15*dmats0[15][10] + coeff0_16*dmats0[16][10] + coeff0_17*dmats0[17][10] + coeff0_18*dmats0[18][10] + coeff0_19*dmats0[19][10] + coeff0_20*dmats0[20][10] + coeff0_21*dmats0[21][10] + coeff0_22*dmats0[22][10] + coeff0_23*dmats0[23][10] + coeff0_24*dmats0[24][10] + coeff0_25*dmats0[25][10] + coeff0_26*dmats0[26][10] + coeff0_27*dmats0[27][10] + coeff0_28*dmats0[28][10] + coeff0_29*dmats0[29][10] + coeff0_30*dmats0[30][10] + coeff0_31*dmats0[31][10] + coeff0_32*dmats0[32][10] + coeff0_33*dmats0[33][10] + coeff0_34*dmats0[34][10] + coeff0_35*dmats0[35][10] + coeff0_36*dmats0[36][10] + coeff0_37*dmats0[37][10] + coeff0_38*dmats0[38][10] + coeff0_39*dmats0[39][10] + coeff0_40*dmats0[40][10] + coeff0_41*dmats0[41][10] + coeff0_42*dmats0[42][10] + coeff0_43*dmats0[43][10] + coeff0_44*dmats0[44][10] + coeff0_45*dmats0[45][10] + coeff0_46*dmats0[46][10] + coeff0_47*dmats0[47][10] + coeff0_48*dmats0[48][10] + coeff0_49*dmats0[49][10] + coeff0_50*dmats0[50][10] + coeff0_51*dmats0[51][10] + coeff0_52*dmats0[52][10] + coeff0_53*dmats0[53][10] + coeff0_54*dmats0[54][10] + coeff0_55*dmats0[55][10];
          new_coeff0_11 = coeff0_0*dmats0[0][11] + coeff0_1*dmats0[1][11] + coeff0_2*dmats0[2][11] + coeff0_3*dmats0[3][11] + coeff0_4*dmats0[4][11] + coeff0_5*dmats0[5][11] + coeff0_6*dmats0[6][11] + coeff0_7*dmats0[7][11] + coeff0_8*dmats0[8][11] + coeff0_9*dmats0[9][11] + coeff0_10*dmats0[10][11] + coeff0_11*dmats0[11][11] + coeff0_12*dmats0[12][11] + coeff0_13*dmats0[13][11] + coeff0_14*dmats0[14][11] + coeff0_15*dmats0[15][11] + coeff0_16*dmats0[16][11] + coeff0_17*dmats0[17][11] + coeff0_18*dmats0[18][11] + coeff0_19*dmats0[19][11] + coeff0_20*dmats0[20][11] + coeff0_21*dmats0[21][11] + coeff0_22*dmats0[22][11] + coeff0_23*dmats0[23][11] + coeff0_24*dmats0[24][11] + coeff0_25*dmats0[25][11] + coeff0_26*dmats0[26][11] + coeff0_27*dmats0[27][11] + coeff0_28*dmats0[28][11] + coeff0_29*dmats0[29][11] + coeff0_30*dmats0[30][11] + coeff0_31*dmats0[31][11] + coeff0_32*dmats0[32][11] + coeff0_33*dmats0[33][11] + coeff0_34*dmats0[34][11] + coeff0_35*dmats0[35][11] + coeff0_36*dmats0[36][11] + coeff0_37*dmats0[37][11] + coeff0_38*dmats0[38][11] + coeff0_39*dmats0[39][11] + coeff0_40*dmats0[40][11] + coeff0_41*dmats0[41][11] + coeff0_42*dmats0[42][11] + coeff0_43*dmats0[43][11] + coeff0_44*dmats0[44][11] + coeff0_45*dmats0[45][11] + coeff0_46*dmats0[46][11] + coeff0_47*dmats0[47][11] + coeff0_48*dmats0[48][11] + coeff0_49*dmats0[49][11] + coeff0_50*dmats0[50][11] + coeff0_51*dmats0[51][11] + coeff0_52*dmats0[52][11] + coeff0_53*dmats0[53][11] + coeff0_54*dmats0[54][11] + coeff0_55*dmats0[55][11];
          new_coeff0_12 = coeff0_0*dmats0[0][12] + coeff0_1*dmats0[1][12] + coeff0_2*dmats0[2][12] + coeff0_3*dmats0[3][12] + coeff0_4*dmats0[4][12] + coeff0_5*dmats0[5][12] + coeff0_6*dmats0[6][12] + coeff0_7*dmats0[7][12] + coeff0_8*dmats0[8][12] + coeff0_9*dmats0[9][12] + coeff0_10*dmats0[10][12] + coeff0_11*dmats0[11][12] + coeff0_12*dmats0[12][12] + coeff0_13*dmats0[13][12] + coeff0_14*dmats0[14][12] + coeff0_15*dmats0[15][12] + coeff0_16*dmats0[16][12] + coeff0_17*dmats0[17][12] + coeff0_18*dmats0[18][12] + coeff0_19*dmats0[19][12] + coeff0_20*dmats0[20][12] + coeff0_21*dmats0[21][12] + coeff0_22*dmats0[22][12] + coeff0_23*dmats0[23][12] + coeff0_24*dmats0[24][12] + coeff0_25*dmats0[25][12] + coeff0_26*dmats0[26][12] + coeff0_27*dmats0[27][12] + coeff0_28*dmats0[28][12] + coeff0_29*dmats0[29][12] + coeff0_30*dmats0[30][12] + coeff0_31*dmats0[31][12] + coeff0_32*dmats0[32][12] + coeff0_33*dmats0[33][12] + coeff0_34*dmats0[34][12] + coeff0_35*dmats0[35][12] + coeff0_36*dmats0[36][12] + coeff0_37*dmats0[37][12] + coeff0_38*dmats0[38][12] + coeff0_39*dmats0[39][12] + coeff0_40*dmats0[40][12] + coeff0_41*dmats0[41][12] + coeff0_42*dmats0[42][12] + coeff0_43*dmats0[43][12] + coeff0_44*dmats0[44][12] + coeff0_45*dmats0[45][12] + coeff0_46*dmats0[46][12] + coeff0_47*dmats0[47][12] + coeff0_48*dmats0[48][12] + coeff0_49*dmats0[49][12] + coeff0_50*dmats0[50][12] + coeff0_51*dmats0[51][12] + coeff0_52*dmats0[52][12] + coeff0_53*dmats0[53][12] + coeff0_54*dmats0[54][12] + coeff0_55*dmats0[55][12];
          new_coeff0_13 = coeff0_0*dmats0[0][13] + coeff0_1*dmats0[1][13] + coeff0_2*dmats0[2][13] + coeff0_3*dmats0[3][13] + coeff0_4*dmats0[4][13] + coeff0_5*dmats0[5][13] + coeff0_6*dmats0[6][13] + coeff0_7*dmats0[7][13] + coeff0_8*dmats0[8][13] + coeff0_9*dmats0[9][13] + coeff0_10*dmats0[10][13] + coeff0_11*dmats0[11][13] + coeff0_12*dmats0[12][13] + coeff0_13*dmats0[13][13] + coeff0_14*dmats0[14][13] + coeff0_15*dmats0[15][13] + coeff0_16*dmats0[16][13] + coeff0_17*dmats0[17][13] + coeff0_18*dmats0[18][13] + coeff0_19*dmats0[19][13] + coeff0_20*dmats0[20][13] + coeff0_21*dmats0[21][13] + coeff0_22*dmats0[22][13] + coeff0_23*dmats0[23][13] + coeff0_24*dmats0[24][13] + coeff0_25*dmats0[25][13] + coeff0_26*dmats0[26][13] + coeff0_27*dmats0[27][13] + coeff0_28*dmats0[28][13] + coeff0_29*dmats0[29][13] + coeff0_30*dmats0[30][13] + coeff0_31*dmats0[31][13] + coeff0_32*dmats0[32][13] + coeff0_33*dmats0[33][13] + coeff0_34*dmats0[34][13] + coeff0_35*dmats0[35][13] + coeff0_36*dmats0[36][13] + coeff0_37*dmats0[37][13] + coeff0_38*dmats0[38][13] + coeff0_39*dmats0[39][13] + coeff0_40*dmats0[40][13] + coeff0_41*dmats0[41][13] + coeff0_42*dmats0[42][13] + coeff0_43*dmats0[43][13] + coeff0_44*dmats0[44][13] + coeff0_45*dmats0[45][13] + coeff0_46*dmats0[46][13] + coeff0_47*dmats0[47][13] + coeff0_48*dmats0[48][13] + coeff0_49*dmats0[49][13] + coeff0_50*dmats0[50][13] + coeff0_51*dmats0[51][13] + coeff0_52*dmats0[52][13] + coeff0_53*dmats0[53][13] + coeff0_54*dmats0[54][13] + coeff0_55*dmats0[55][13];
          new_coeff0_14 = coeff0_0*dmats0[0][14] + coeff0_1*dmats0[1][14] + coeff0_2*dmats0[2][14] + coeff0_3*dmats0[3][14] + coeff0_4*dmats0[4][14] + coeff0_5*dmats0[5][14] + coeff0_6*dmats0[6][14] + coeff0_7*dmats0[7][14] + coeff0_8*dmats0[8][14] + coeff0_9*dmats0[9][14] + coeff0_10*dmats0[10][14] + coeff0_11*dmats0[11][14] + coeff0_12*dmats0[12][14] + coeff0_13*dmats0[13][14] + coeff0_14*dmats0[14][14] + coeff0_15*dmats0[15][14] + coeff0_16*dmats0[16][14] + coeff0_17*dmats0[17][14] + coeff0_18*dmats0[18][14] + coeff0_19*dmats0[19][14] + coeff0_20*dmats0[20][14] + coeff0_21*dmats0[21][14] + coeff0_22*dmats0[22][14] + coeff0_23*dmats0[23][14] + coeff0_24*dmats0[24][14] + coeff0_25*dmats0[25][14] + coeff0_26*dmats0[26][14] + coeff0_27*dmats0[27][14] + coeff0_28*dmats0[28][14] + coeff0_29*dmats0[29][14] + coeff0_30*dmats0[30][14] + coeff0_31*dmats0[31][14] + coeff0_32*dmats0[32][14] + coeff0_33*dmats0[33][14] + coeff0_34*dmats0[34][14] + coeff0_35*dmats0[35][14] + coeff0_36*dmats0[36][14] + coeff0_37*dmats0[37][14] + coeff0_38*dmats0[38][14] + coeff0_39*dmats0[39][14] + coeff0_40*dmats0[40][14] + coeff0_41*dmats0[41][14] + coeff0_42*dmats0[42][14] + coeff0_43*dmats0[43][14] + coeff0_44*dmats0[44][14] + coeff0_45*dmats0[45][14] + coeff0_46*dmats0[46][14] + coeff0_47*dmats0[47][14] + coeff0_48*dmats0[48][14] + coeff0_49*dmats0[49][14] + coeff0_50*dmats0[50][14] + coeff0_51*dmats0[51][14] + coeff0_52*dmats0[52][14] + coeff0_53*dmats0[53][14] + coeff0_54*dmats0[54][14] + coeff0_55*dmats0[55][14];
          new_coeff0_15 = coeff0_0*dmats0[0][15] + coeff0_1*dmats0[1][15] + coeff0_2*dmats0[2][15] + coeff0_3*dmats0[3][15] + coeff0_4*dmats0[4][15] + coeff0_5*dmats0[5][15] + coeff0_6*dmats0[6][15] + coeff0_7*dmats0[7][15] + coeff0_8*dmats0[8][15] + coeff0_9*dmats0[9][15] + coeff0_10*dmats0[10][15] + coeff0_11*dmats0[11][15] + coeff0_12*dmats0[12][15] + coeff0_13*dmats0[13][15] + coeff0_14*dmats0[14][15] + coeff0_15*dmats0[15][15] + coeff0_16*dmats0[16][15] + coeff0_17*dmats0[17][15] + coeff0_18*dmats0[18][15] + coeff0_19*dmats0[19][15] + coeff0_20*dmats0[20][15] + coeff0_21*dmats0[21][15] + coeff0_22*dmats0[22][15] + coeff0_23*dmats0[23][15] + coeff0_24*dmats0[24][15] + coeff0_25*dmats0[25][15] + coeff0_26*dmats0[26][15] + coeff0_27*dmats0[27][15] + coeff0_28*dmats0[28][15] + coeff0_29*dmats0[29][15] + coeff0_30*dmats0[30][15] + coeff0_31*dmats0[31][15] + coeff0_32*dmats0[32][15] + coeff0_33*dmats0[33][15] + coeff0_34*dmats0[34][15] + coeff0_35*dmats0[35][15] + coeff0_36*dmats0[36][15] + coeff0_37*dmats0[37][15] + coeff0_38*dmats0[38][15] + coeff0_39*dmats0[39][15] + coeff0_40*dmats0[40][15] + coeff0_41*dmats0[41][15] + coeff0_42*dmats0[42][15] + coeff0_43*dmats0[43][15] + coeff0_44*dmats0[44][15] + coeff0_45*dmats0[45][15] + coeff0_46*dmats0[46][15] + coeff0_47*dmats0[47][15] + coeff0_48*dmats0[48][15] + coeff0_49*dmats0[49][15] + coeff0_50*dmats0[50][15] + coeff0_51*dmats0[51][15] + coeff0_52*dmats0[52][15] + coeff0_53*dmats0[53][15] + coeff0_54*dmats0[54][15] + coeff0_55*dmats0[55][15];
          new_coeff0_16 = coeff0_0*dmats0[0][16] + coeff0_1*dmats0[1][16] + coeff0_2*dmats0[2][16] + coeff0_3*dmats0[3][16] + coeff0_4*dmats0[4][16] + coeff0_5*dmats0[5][16] + coeff0_6*dmats0[6][16] + coeff0_7*dmats0[7][16] + coeff0_8*dmats0[8][16] + coeff0_9*dmats0[9][16] + coeff0_10*dmats0[10][16] + coeff0_11*dmats0[11][16] + coeff0_12*dmats0[12][16] + coeff0_13*dmats0[13][16] + coeff0_14*dmats0[14][16] + coeff0_15*dmats0[15][16] + coeff0_16*dmats0[16][16] + coeff0_17*dmats0[17][16] + coeff0_18*dmats0[18][16] + coeff0_19*dmats0[19][16] + coeff0_20*dmats0[20][16] + coeff0_21*dmats0[21][16] + coeff0_22*dmats0[22][16] + coeff0_23*dmats0[23][16] + coeff0_24*dmats0[24][16] + coeff0_25*dmats0[25][16] + coeff0_26*dmats0[26][16] + coeff0_27*dmats0[27][16] + coeff0_28*dmats0[28][16] + coeff0_29*dmats0[29][16] + coeff0_30*dmats0[30][16] + coeff0_31*dmats0[31][16] + coeff0_32*dmats0[32][16] + coeff0_33*dmats0[33][16] + coeff0_34*dmats0[34][16] + coeff0_35*dmats0[35][16] + coeff0_36*dmats0[36][16] + coeff0_37*dmats0[37][16] + coeff0_38*dmats0[38][16] + coeff0_39*dmats0[39][16] + coeff0_40*dmats0[40][16] + coeff0_41*dmats0[41][16] + coeff0_42*dmats0[42][16] + coeff0_43*dmats0[43][16] + coeff0_44*dmats0[44][16] + coeff0_45*dmats0[45][16] + coeff0_46*dmats0[46][16] + coeff0_47*dmats0[47][16] + coeff0_48*dmats0[48][16] + coeff0_49*dmats0[49][16] + coeff0_50*dmats0[50][16] + coeff0_51*dmats0[51][16] + coeff0_52*dmats0[52][16] + coeff0_53*dmats0[53][16] + coeff0_54*dmats0[54][16] + coeff0_55*dmats0[55][16];
          new_coeff0_17 = coeff0_0*dmats0[0][17] + coeff0_1*dmats0[1][17] + coeff0_2*dmats0[2][17] + coeff0_3*dmats0[3][17] + coeff0_4*dmats0[4][17] + coeff0_5*dmats0[5][17] + coeff0_6*dmats0[6][17] + coeff0_7*dmats0[7][17] + coeff0_8*dmats0[8][17] + coeff0_9*dmats0[9][17] + coeff0_10*dmats0[10][17] + coeff0_11*dmats0[11][17] + coeff0_12*dmats0[12][17] + coeff0_13*dmats0[13][17] + coeff0_14*dmats0[14][17] + coeff0_15*dmats0[15][17] + coeff0_16*dmats0[16][17] + coeff0_17*dmats0[17][17] + coeff0_18*dmats0[18][17] + coeff0_19*dmats0[19][17] + coeff0_20*dmats0[20][17] + coeff0_21*dmats0[21][17] + coeff0_22*dmats0[22][17] + coeff0_23*dmats0[23][17] + coeff0_24*dmats0[24][17] + coeff0_25*dmats0[25][17] + coeff0_26*dmats0[26][17] + coeff0_27*dmats0[27][17] + coeff0_28*dmats0[28][17] + coeff0_29*dmats0[29][17] + coeff0_30*dmats0[30][17] + coeff0_31*dmats0[31][17] + coeff0_32*dmats0[32][17] + coeff0_33*dmats0[33][17] + coeff0_34*dmats0[34][17] + coeff0_35*dmats0[35][17] + coeff0_36*dmats0[36][17] + coeff0_37*dmats0[37][17] + coeff0_38*dmats0[38][17] + coeff0_39*dmats0[39][17] + coeff0_40*dmats0[40][17] + coeff0_41*dmats0[41][17] + coeff0_42*dmats0[42][17] + coeff0_43*dmats0[43][17] + coeff0_44*dmats0[44][17] + coeff0_45*dmats0[45][17] + coeff0_46*dmats0[46][17] + coeff0_47*dmats0[47][17] + coeff0_48*dmats0[48][17] + coeff0_49*dmats0[49][17] + coeff0_50*dmats0[50][17] + coeff0_51*dmats0[51][17] + coeff0_52*dmats0[52][17] + coeff0_53*dmats0[53][17] + coeff0_54*dmats0[54][17] + coeff0_55*dmats0[55][17];
          new_coeff0_18 = coeff0_0*dmats0[0][18] + coeff0_1*dmats0[1][18] + coeff0_2*dmats0[2][18] + coeff0_3*dmats0[3][18] + coeff0_4*dmats0[4][18] + coeff0_5*dmats0[5][18] + coeff0_6*dmats0[6][18] + coeff0_7*dmats0[7][18] + coeff0_8*dmats0[8][18] + coeff0_9*dmats0[9][18] + coeff0_10*dmats0[10][18] + coeff0_11*dmats0[11][18] + coeff0_12*dmats0[12][18] + coeff0_13*dmats0[13][18] + coeff0_14*dmats0[14][18] + coeff0_15*dmats0[15][18] + coeff0_16*dmats0[16][18] + coeff0_17*dmats0[17][18] + coeff0_18*dmats0[18][18] + coeff0_19*dmats0[19][18] + coeff0_20*dmats0[20][18] + coeff0_21*dmats0[21][18] + coeff0_22*dmats0[22][18] + coeff0_23*dmats0[23][18] + coeff0_24*dmats0[24][18] + coeff0_25*dmats0[25][18] + coeff0_26*dmats0[26][18] + coeff0_27*dmats0[27][18] + coeff0_28*dmats0[28][18] + coeff0_29*dmats0[29][18] + coeff0_30*dmats0[30][18] + coeff0_31*dmats0[31][18] + coeff0_32*dmats0[32][18] + coeff0_33*dmats0[33][18] + coeff0_34*dmats0[34][18] + coeff0_35*dmats0[35][18] + coeff0_36*dmats0[36][18] + coeff0_37*dmats0[37][18] + coeff0_38*dmats0[38][18] + coeff0_39*dmats0[39][18] + coeff0_40*dmats0[40][18] + coeff0_41*dmats0[41][18] + coeff0_42*dmats0[42][18] + coeff0_43*dmats0[43][18] + coeff0_44*dmats0[44][18] + coeff0_45*dmats0[45][18] + coeff0_46*dmats0[46][18] + coeff0_47*dmats0[47][18] + coeff0_48*dmats0[48][18] + coeff0_49*dmats0[49][18] + coeff0_50*dmats0[50][18] + coeff0_51*dmats0[51][18] + coeff0_52*dmats0[52][18] + coeff0_53*dmats0[53][18] + coeff0_54*dmats0[54][18] + coeff0_55*dmats0[55][18];
          new_coeff0_19 = coeff0_0*dmats0[0][19] + coeff0_1*dmats0[1][19] + coeff0_2*dmats0[2][19] + coeff0_3*dmats0[3][19] + coeff0_4*dmats0[4][19] + coeff0_5*dmats0[5][19] + coeff0_6*dmats0[6][19] + coeff0_7*dmats0[7][19] + coeff0_8*dmats0[8][19] + coeff0_9*dmats0[9][19] + coeff0_10*dmats0[10][19] + coeff0_11*dmats0[11][19] + coeff0_12*dmats0[12][19] + coeff0_13*dmats0[13][19] + coeff0_14*dmats0[14][19] + coeff0_15*dmats0[15][19] + coeff0_16*dmats0[16][19] + coeff0_17*dmats0[17][19] + coeff0_18*dmats0[18][19] + coeff0_19*dmats0[19][19] + coeff0_20*dmats0[20][19] + coeff0_21*dmats0[21][19] + coeff0_22*dmats0[22][19] + coeff0_23*dmats0[23][19] + coeff0_24*dmats0[24][19] + coeff0_25*dmats0[25][19] + coeff0_26*dmats0[26][19] + coeff0_27*dmats0[27][19] + coeff0_28*dmats0[28][19] + coeff0_29*dmats0[29][19] + coeff0_30*dmats0[30][19] + coeff0_31*dmats0[31][19] + coeff0_32*dmats0[32][19] + coeff0_33*dmats0[33][19] + coeff0_34*dmats0[34][19] + coeff0_35*dmats0[35][19] + coeff0_36*dmats0[36][19] + coeff0_37*dmats0[37][19] + coeff0_38*dmats0[38][19] + coeff0_39*dmats0[39][19] + coeff0_40*dmats0[40][19] + coeff0_41*dmats0[41][19] + coeff0_42*dmats0[42][19] + coeff0_43*dmats0[43][19] + coeff0_44*dmats0[44][19] + coeff0_45*dmats0[45][19] + coeff0_46*dmats0[46][19] + coeff0_47*dmats0[47][19] + coeff0_48*dmats0[48][19] + coeff0_49*dmats0[49][19] + coeff0_50*dmats0[50][19] + coeff0_51*dmats0[51][19] + coeff0_52*dmats0[52][19] + coeff0_53*dmats0[53][19] + coeff0_54*dmats0[54][19] + coeff0_55*dmats0[55][19];
          new_coeff0_20 = coeff0_0*dmats0[0][20] + coeff0_1*dmats0[1][20] + coeff0_2*dmats0[2][20] + coeff0_3*dmats0[3][20] + coeff0_4*dmats0[4][20] + coeff0_5*dmats0[5][20] + coeff0_6*dmats0[6][20] + coeff0_7*dmats0[7][20] + coeff0_8*dmats0[8][20] + coeff0_9*dmats0[9][20] + coeff0_10*dmats0[10][20] + coeff0_11*dmats0[11][20] + coeff0_12*dmats0[12][20] + coeff0_13*dmats0[13][20] + coeff0_14*dmats0[14][20] + coeff0_15*dmats0[15][20] + coeff0_16*dmats0[16][20] + coeff0_17*dmats0[17][20] + coeff0_18*dmats0[18][20] + coeff0_19*dmats0[19][20] + coeff0_20*dmats0[20][20] + coeff0_21*dmats0[21][20] + coeff0_22*dmats0[22][20] + coeff0_23*dmats0[23][20] + coeff0_24*dmats0[24][20] + coeff0_25*dmats0[25][20] + coeff0_26*dmats0[26][20] + coeff0_27*dmats0[27][20] + coeff0_28*dmats0[28][20] + coeff0_29*dmats0[29][20] + coeff0_30*dmats0[30][20] + coeff0_31*dmats0[31][20] + coeff0_32*dmats0[32][20] + coeff0_33*dmats0[33][20] + coeff0_34*dmats0[34][20] + coeff0_35*dmats0[35][20] + coeff0_36*dmats0[36][20] + coeff0_37*dmats0[37][20] + coeff0_38*dmats0[38][20] + coeff0_39*dmats0[39][20] + coeff0_40*dmats0[40][20] + coeff0_41*dmats0[41][20] + coeff0_42*dmats0[42][20] + coeff0_43*dmats0[43][20] + coeff0_44*dmats0[44][20] + coeff0_45*dmats0[45][20] + coeff0_46*dmats0[46][20] + coeff0_47*dmats0[47][20] + coeff0_48*dmats0[48][20] + coeff0_49*dmats0[49][20] + coeff0_50*dmats0[50][20] + coeff0_51*dmats0[51][20] + coeff0_52*dmats0[52][20] + coeff0_53*dmats0[53][20] + coeff0_54*dmats0[54][20] + coeff0_55*dmats0[55][20];
          new_coeff0_21 = coeff0_0*dmats0[0][21] + coeff0_1*dmats0[1][21] + coeff0_2*dmats0[2][21] + coeff0_3*dmats0[3][21] + coeff0_4*dmats0[4][21] + coeff0_5*dmats0[5][21] + coeff0_6*dmats0[6][21] + coeff0_7*dmats0[7][21] + coeff0_8*dmats0[8][21] + coeff0_9*dmats0[9][21] + coeff0_10*dmats0[10][21] + coeff0_11*dmats0[11][21] + coeff0_12*dmats0[12][21] + coeff0_13*dmats0[13][21] + coeff0_14*dmats0[14][21] + coeff0_15*dmats0[15][21] + coeff0_16*dmats0[16][21] + coeff0_17*dmats0[17][21] + coeff0_18*dmats0[18][21] + coeff0_19*dmats0[19][21] + coeff0_20*dmats0[20][21] + coeff0_21*dmats0[21][21] + coeff0_22*dmats0[22][21] + coeff0_23*dmats0[23][21] + coeff0_24*dmats0[24][21] + coeff0_25*dmats0[25][21] + coeff0_26*dmats0[26][21] + coeff0_27*dmats0[27][21] + coeff0_28*dmats0[28][21] + coeff0_29*dmats0[29][21] + coeff0_30*dmats0[30][21] + coeff0_31*dmats0[31][21] + coeff0_32*dmats0[32][21] + coeff0_33*dmats0[33][21] + coeff0_34*dmats0[34][21] + coeff0_35*dmats0[35][21] + coeff0_36*dmats0[36][21] + coeff0_37*dmats0[37][21] + coeff0_38*dmats0[38][21] + coeff0_39*dmats0[39][21] + coeff0_40*dmats0[40][21] + coeff0_41*dmats0[41][21] + coeff0_42*dmats0[42][21] + coeff0_43*dmats0[43][21] + coeff0_44*dmats0[44][21] + coeff0_45*dmats0[45][21] + coeff0_46*dmats0[46][21] + coeff0_47*dmats0[47][21] + coeff0_48*dmats0[48][21] + coeff0_49*dmats0[49][21] + coeff0_50*dmats0[50][21] + coeff0_51*dmats0[51][21] + coeff0_52*dmats0[52][21] + coeff0_53*dmats0[53][21] + coeff0_54*dmats0[54][21] + coeff0_55*dmats0[55][21];
          new_coeff0_22 = coeff0_0*dmats0[0][22] + coeff0_1*dmats0[1][22] + coeff0_2*dmats0[2][22] + coeff0_3*dmats0[3][22] + coeff0_4*dmats0[4][22] + coeff0_5*dmats0[5][22] + coeff0_6*dmats0[6][22] + coeff0_7*dmats0[7][22] + coeff0_8*dmats0[8][22] + coeff0_9*dmats0[9][22] + coeff0_10*dmats0[10][22] + coeff0_11*dmats0[11][22] + coeff0_12*dmats0[12][22] + coeff0_13*dmats0[13][22] + coeff0_14*dmats0[14][22] + coeff0_15*dmats0[15][22] + coeff0_16*dmats0[16][22] + coeff0_17*dmats0[17][22] + coeff0_18*dmats0[18][22] + coeff0_19*dmats0[19][22] + coeff0_20*dmats0[20][22] + coeff0_21*dmats0[21][22] + coeff0_22*dmats0[22][22] + coeff0_23*dmats0[23][22] + coeff0_24*dmats0[24][22] + coeff0_25*dmats0[25][22] + coeff0_26*dmats0[26][22] + coeff0_27*dmats0[27][22] + coeff0_28*dmats0[28][22] + coeff0_29*dmats0[29][22] + coeff0_30*dmats0[30][22] + coeff0_31*dmats0[31][22] + coeff0_32*dmats0[32][22] + coeff0_33*dmats0[33][22] + coeff0_34*dmats0[34][22] + coeff0_35*dmats0[35][22] + coeff0_36*dmats0[36][22] + coeff0_37*dmats0[37][22] + coeff0_38*dmats0[38][22] + coeff0_39*dmats0[39][22] + coeff0_40*dmats0[40][22] + coeff0_41*dmats0[41][22] + coeff0_42*dmats0[42][22] + coeff0_43*dmats0[43][22] + coeff0_44*dmats0[44][22] + coeff0_45*dmats0[45][22] + coeff0_46*dmats0[46][22] + coeff0_47*dmats0[47][22] + coeff0_48*dmats0[48][22] + coeff0_49*dmats0[49][22] + coeff0_50*dmats0[50][22] + coeff0_51*dmats0[51][22] + coeff0_52*dmats0[52][22] + coeff0_53*dmats0[53][22] + coeff0_54*dmats0[54][22] + coeff0_55*dmats0[55][22];
          new_coeff0_23 = coeff0_0*dmats0[0][23] + coeff0_1*dmats0[1][23] + coeff0_2*dmats0[2][23] + coeff0_3*dmats0[3][23] + coeff0_4*dmats0[4][23] + coeff0_5*dmats0[5][23] + coeff0_6*dmats0[6][23] + coeff0_7*dmats0[7][23] + coeff0_8*dmats0[8][23] + coeff0_9*dmats0[9][23] + coeff0_10*dmats0[10][23] + coeff0_11*dmats0[11][23] + coeff0_12*dmats0[12][23] + coeff0_13*dmats0[13][23] + coeff0_14*dmats0[14][23] + coeff0_15*dmats0[15][23] + coeff0_16*dmats0[16][23] + coeff0_17*dmats0[17][23] + coeff0_18*dmats0[18][23] + coeff0_19*dmats0[19][23] + coeff0_20*dmats0[20][23] + coeff0_21*dmats0[21][23] + coeff0_22*dmats0[22][23] + coeff0_23*dmats0[23][23] + coeff0_24*dmats0[24][23] + coeff0_25*dmats0[25][23] + coeff0_26*dmats0[26][23] + coeff0_27*dmats0[27][23] + coeff0_28*dmats0[28][23] + coeff0_29*dmats0[29][23] + coeff0_30*dmats0[30][23] + coeff0_31*dmats0[31][23] + coeff0_32*dmats0[32][23] + coeff0_33*dmats0[33][23] + coeff0_34*dmats0[34][23] + coeff0_35*dmats0[35][23] + coeff0_36*dmats0[36][23] + coeff0_37*dmats0[37][23] + coeff0_38*dmats0[38][23] + coeff0_39*dmats0[39][23] + coeff0_40*dmats0[40][23] + coeff0_41*dmats0[41][23] + coeff0_42*dmats0[42][23] + coeff0_43*dmats0[43][23] + coeff0_44*dmats0[44][23] + coeff0_45*dmats0[45][23] + coeff0_46*dmats0[46][23] + coeff0_47*dmats0[47][23] + coeff0_48*dmats0[48][23] + coeff0_49*dmats0[49][23] + coeff0_50*dmats0[50][23] + coeff0_51*dmats0[51][23] + coeff0_52*dmats0[52][23] + coeff0_53*dmats0[53][23] + coeff0_54*dmats0[54][23] + coeff0_55*dmats0[55][23];
          new_coeff0_24 = coeff0_0*dmats0[0][24] + coeff0_1*dmats0[1][24] + coeff0_2*dmats0[2][24] + coeff0_3*dmats0[3][24] + coeff0_4*dmats0[4][24] + coeff0_5*dmats0[5][24] + coeff0_6*dmats0[6][24] + coeff0_7*dmats0[7][24] + coeff0_8*dmats0[8][24] + coeff0_9*dmats0[9][24] + coeff0_10*dmats0[10][24] + coeff0_11*dmats0[11][24] + coeff0_12*dmats0[12][24] + coeff0_13*dmats0[13][24] + coeff0_14*dmats0[14][24] + coeff0_15*dmats0[15][24] + coeff0_16*dmats0[16][24] + coeff0_17*dmats0[17][24] + coeff0_18*dmats0[18][24] + coeff0_19*dmats0[19][24] + coeff0_20*dmats0[20][24] + coeff0_21*dmats0[21][24] + coeff0_22*dmats0[22][24] + coeff0_23*dmats0[23][24] + coeff0_24*dmats0[24][24] + coeff0_25*dmats0[25][24] + coeff0_26*dmats0[26][24] + coeff0_27*dmats0[27][24] + coeff0_28*dmats0[28][24] + coeff0_29*dmats0[29][24] + coeff0_30*dmats0[30][24] + coeff0_31*dmats0[31][24] + coeff0_32*dmats0[32][24] + coeff0_33*dmats0[33][24] + coeff0_34*dmats0[34][24] + coeff0_35*dmats0[35][24] + coeff0_36*dmats0[36][24] + coeff0_37*dmats0[37][24] + coeff0_38*dmats0[38][24] + coeff0_39*dmats0[39][24] + coeff0_40*dmats0[40][24] + coeff0_41*dmats0[41][24] + coeff0_42*dmats0[42][24] + coeff0_43*dmats0[43][24] + coeff0_44*dmats0[44][24] + coeff0_45*dmats0[45][24] + coeff0_46*dmats0[46][24] + coeff0_47*dmats0[47][24] + coeff0_48*dmats0[48][24] + coeff0_49*dmats0[49][24] + coeff0_50*dmats0[50][24] + coeff0_51*dmats0[51][24] + coeff0_52*dmats0[52][24] + coeff0_53*dmats0[53][24] + coeff0_54*dmats0[54][24] + coeff0_55*dmats0[55][24];
          new_coeff0_25 = coeff0_0*dmats0[0][25] + coeff0_1*dmats0[1][25] + coeff0_2*dmats0[2][25] + coeff0_3*dmats0[3][25] + coeff0_4*dmats0[4][25] + coeff0_5*dmats0[5][25] + coeff0_6*dmats0[6][25] + coeff0_7*dmats0[7][25] + coeff0_8*dmats0[8][25] + coeff0_9*dmats0[9][25] + coeff0_10*dmats0[10][25] + coeff0_11*dmats0[11][25] + coeff0_12*dmats0[12][25] + coeff0_13*dmats0[13][25] + coeff0_14*dmats0[14][25] + coeff0_15*dmats0[15][25] + coeff0_16*dmats0[16][25] + coeff0_17*dmats0[17][25] + coeff0_18*dmats0[18][25] + coeff0_19*dmats0[19][25] + coeff0_20*dmats0[20][25] + coeff0_21*dmats0[21][25] + coeff0_22*dmats0[22][25] + coeff0_23*dmats0[23][25] + coeff0_24*dmats0[24][25] + coeff0_25*dmats0[25][25] + coeff0_26*dmats0[26][25] + coeff0_27*dmats0[27][25] + coeff0_28*dmats0[28][25] + coeff0_29*dmats0[29][25] + coeff0_30*dmats0[30][25] + coeff0_31*dmats0[31][25] + coeff0_32*dmats0[32][25] + coeff0_33*dmats0[33][25] + coeff0_34*dmats0[34][25] + coeff0_35*dmats0[35][25] + coeff0_36*dmats0[36][25] + coeff0_37*dmats0[37][25] + coeff0_38*dmats0[38][25] + coeff0_39*dmats0[39][25] + coeff0_40*dmats0[40][25] + coeff0_41*dmats0[41][25] + coeff0_42*dmats0[42][25] + coeff0_43*dmats0[43][25] + coeff0_44*dmats0[44][25] + coeff0_45*dmats0[45][25] + coeff0_46*dmats0[46][25] + coeff0_47*dmats0[47][25] + coeff0_48*dmats0[48][25] + coeff0_49*dmats0[49][25] + coeff0_50*dmats0[50][25] + coeff0_51*dmats0[51][25] + coeff0_52*dmats0[52][25] + coeff0_53*dmats0[53][25] + coeff0_54*dmats0[54][25] + coeff0_55*dmats0[55][25];
          new_coeff0_26 = coeff0_0*dmats0[0][26] + coeff0_1*dmats0[1][26] + coeff0_2*dmats0[2][26] + coeff0_3*dmats0[3][26] + coeff0_4*dmats0[4][26] + coeff0_5*dmats0[5][26] + coeff0_6*dmats0[6][26] + coeff0_7*dmats0[7][26] + coeff0_8*dmats0[8][26] + coeff0_9*dmats0[9][26] + coeff0_10*dmats0[10][26] + coeff0_11*dmats0[11][26] + coeff0_12*dmats0[12][26] + coeff0_13*dmats0[13][26] + coeff0_14*dmats0[14][26] + coeff0_15*dmats0[15][26] + coeff0_16*dmats0[16][26] + coeff0_17*dmats0[17][26] + coeff0_18*dmats0[18][26] + coeff0_19*dmats0[19][26] + coeff0_20*dmats0[20][26] + coeff0_21*dmats0[21][26] + coeff0_22*dmats0[22][26] + coeff0_23*dmats0[23][26] + coeff0_24*dmats0[24][26] + coeff0_25*dmats0[25][26] + coeff0_26*dmats0[26][26] + coeff0_27*dmats0[27][26] + coeff0_28*dmats0[28][26] + coeff0_29*dmats0[29][26] + coeff0_30*dmats0[30][26] + coeff0_31*dmats0[31][26] + coeff0_32*dmats0[32][26] + coeff0_33*dmats0[33][26] + coeff0_34*dmats0[34][26] + coeff0_35*dmats0[35][26] + coeff0_36*dmats0[36][26] + coeff0_37*dmats0[37][26] + coeff0_38*dmats0[38][26] + coeff0_39*dmats0[39][26] + coeff0_40*dmats0[40][26] + coeff0_41*dmats0[41][26] + coeff0_42*dmats0[42][26] + coeff0_43*dmats0[43][26] + coeff0_44*dmats0[44][26] + coeff0_45*dmats0[45][26] + coeff0_46*dmats0[46][26] + coeff0_47*dmats0[47][26] + coeff0_48*dmats0[48][26] + coeff0_49*dmats0[49][26] + coeff0_50*dmats0[50][26] + coeff0_51*dmats0[51][26] + coeff0_52*dmats0[52][26] + coeff0_53*dmats0[53][26] + coeff0_54*dmats0[54][26] + coeff0_55*dmats0[55][26];
          new_coeff0_27 = coeff0_0*dmats0[0][27] + coeff0_1*dmats0[1][27] + coeff0_2*dmats0[2][27] + coeff0_3*dmats0[3][27] + coeff0_4*dmats0[4][27] + coeff0_5*dmats0[5][27] + coeff0_6*dmats0[6][27] + coeff0_7*dmats0[7][27] + coeff0_8*dmats0[8][27] + coeff0_9*dmats0[9][27] + coeff0_10*dmats0[10][27] + coeff0_11*dmats0[11][27] + coeff0_12*dmats0[12][27] + coeff0_13*dmats0[13][27] + coeff0_14*dmats0[14][27] + coeff0_15*dmats0[15][27] + coeff0_16*dmats0[16][27] + coeff0_17*dmats0[17][27] + coeff0_18*dmats0[18][27] + coeff0_19*dmats0[19][27] + coeff0_20*dmats0[20][27] + coeff0_21*dmats0[21][27] + coeff0_22*dmats0[22][27] + coeff0_23*dmats0[23][27] + coeff0_24*dmats0[24][27] + coeff0_25*dmats0[25][27] + coeff0_26*dmats0[26][27] + coeff0_27*dmats0[27][27] + coeff0_28*dmats0[28][27] + coeff0_29*dmats0[29][27] + coeff0_30*dmats0[30][27] + coeff0_31*dmats0[31][27] + coeff0_32*dmats0[32][27] + coeff0_33*dmats0[33][27] + coeff0_34*dmats0[34][27] + coeff0_35*dmats0[35][27] + coeff0_36*dmats0[36][27] + coeff0_37*dmats0[37][27] + coeff0_38*dmats0[38][27] + coeff0_39*dmats0[39][27] + coeff0_40*dmats0[40][27] + coeff0_41*dmats0[41][27] + coeff0_42*dmats0[42][27] + coeff0_43*dmats0[43][27] + coeff0_44*dmats0[44][27] + coeff0_45*dmats0[45][27] + coeff0_46*dmats0[46][27] + coeff0_47*dmats0[47][27] + coeff0_48*dmats0[48][27] + coeff0_49*dmats0[49][27] + coeff0_50*dmats0[50][27] + coeff0_51*dmats0[51][27] + coeff0_52*dmats0[52][27] + coeff0_53*dmats0[53][27] + coeff0_54*dmats0[54][27] + coeff0_55*dmats0[55][27];
          new_coeff0_28 = coeff0_0*dmats0[0][28] + coeff0_1*dmats0[1][28] + coeff0_2*dmats0[2][28] + coeff0_3*dmats0[3][28] + coeff0_4*dmats0[4][28] + coeff0_5*dmats0[5][28] + coeff0_6*dmats0[6][28] + coeff0_7*dmats0[7][28] + coeff0_8*dmats0[8][28] + coeff0_9*dmats0[9][28] + coeff0_10*dmats0[10][28] + coeff0_11*dmats0[11][28] + coeff0_12*dmats0[12][28] + coeff0_13*dmats0[13][28] + coeff0_14*dmats0[14][28] + coeff0_15*dmats0[15][28] + coeff0_16*dmats0[16][28] + coeff0_17*dmats0[17][28] + coeff0_18*dmats0[18][28] + coeff0_19*dmats0[19][28] + coeff0_20*dmats0[20][28] + coeff0_21*dmats0[21][28] + coeff0_22*dmats0[22][28] + coeff0_23*dmats0[23][28] + coeff0_24*dmats0[24][28] + coeff0_25*dmats0[25][28] + coeff0_26*dmats0[26][28] + coeff0_27*dmats0[27][28] + coeff0_28*dmats0[28][28] + coeff0_29*dmats0[29][28] + coeff0_30*dmats0[30][28] + coeff0_31*dmats0[31][28] + coeff0_32*dmats0[32][28] + coeff0_33*dmats0[33][28] + coeff0_34*dmats0[34][28] + coeff0_35*dmats0[35][28] + coeff0_36*dmats0[36][28] + coeff0_37*dmats0[37][28] + coeff0_38*dmats0[38][28] + coeff0_39*dmats0[39][28] + coeff0_40*dmats0[40][28] + coeff0_41*dmats0[41][28] + coeff0_42*dmats0[42][28] + coeff0_43*dmats0[43][28] + coeff0_44*dmats0[44][28] + coeff0_45*dmats0[45][28] + coeff0_46*dmats0[46][28] + coeff0_47*dmats0[47][28] + coeff0_48*dmats0[48][28] + coeff0_49*dmats0[49][28] + coeff0_50*dmats0[50][28] + coeff0_51*dmats0[51][28] + coeff0_52*dmats0[52][28] + coeff0_53*dmats0[53][28] + coeff0_54*dmats0[54][28] + coeff0_55*dmats0[55][28];
          new_coeff0_29 = coeff0_0*dmats0[0][29] + coeff0_1*dmats0[1][29] + coeff0_2*dmats0[2][29] + coeff0_3*dmats0[3][29] + coeff0_4*dmats0[4][29] + coeff0_5*dmats0[5][29] + coeff0_6*dmats0[6][29] + coeff0_7*dmats0[7][29] + coeff0_8*dmats0[8][29] + coeff0_9*dmats0[9][29] + coeff0_10*dmats0[10][29] + coeff0_11*dmats0[11][29] + coeff0_12*dmats0[12][29] + coeff0_13*dmats0[13][29] + coeff0_14*dmats0[14][29] + coeff0_15*dmats0[15][29] + coeff0_16*dmats0[16][29] + coeff0_17*dmats0[17][29] + coeff0_18*dmats0[18][29] + coeff0_19*dmats0[19][29] + coeff0_20*dmats0[20][29] + coeff0_21*dmats0[21][29] + coeff0_22*dmats0[22][29] + coeff0_23*dmats0[23][29] + coeff0_24*dmats0[24][29] + coeff0_25*dmats0[25][29] + coeff0_26*dmats0[26][29] + coeff0_27*dmats0[27][29] + coeff0_28*dmats0[28][29] + coeff0_29*dmats0[29][29] + coeff0_30*dmats0[30][29] + coeff0_31*dmats0[31][29] + coeff0_32*dmats0[32][29] + coeff0_33*dmats0[33][29] + coeff0_34*dmats0[34][29] + coeff0_35*dmats0[35][29] + coeff0_36*dmats0[36][29] + coeff0_37*dmats0[37][29] + coeff0_38*dmats0[38][29] + coeff0_39*dmats0[39][29] + coeff0_40*dmats0[40][29] + coeff0_41*dmats0[41][29] + coeff0_42*dmats0[42][29] + coeff0_43*dmats0[43][29] + coeff0_44*dmats0[44][29] + coeff0_45*dmats0[45][29] + coeff0_46*dmats0[46][29] + coeff0_47*dmats0[47][29] + coeff0_48*dmats0[48][29] + coeff0_49*dmats0[49][29] + coeff0_50*dmats0[50][29] + coeff0_51*dmats0[51][29] + coeff0_52*dmats0[52][29] + coeff0_53*dmats0[53][29] + coeff0_54*dmats0[54][29] + coeff0_55*dmats0[55][29];
          new_coeff0_30 = coeff0_0*dmats0[0][30] + coeff0_1*dmats0[1][30] + coeff0_2*dmats0[2][30] + coeff0_3*dmats0[3][30] + coeff0_4*dmats0[4][30] + coeff0_5*dmats0[5][30] + coeff0_6*dmats0[6][30] + coeff0_7*dmats0[7][30] + coeff0_8*dmats0[8][30] + coeff0_9*dmats0[9][30] + coeff0_10*dmats0[10][30] + coeff0_11*dmats0[11][30] + coeff0_12*dmats0[12][30] + coeff0_13*dmats0[13][30] + coeff0_14*dmats0[14][30] + coeff0_15*dmats0[15][30] + coeff0_16*dmats0[16][30] + coeff0_17*dmats0[17][30] + coeff0_18*dmats0[18][30] + coeff0_19*dmats0[19][30] + coeff0_20*dmats0[20][30] + coeff0_21*dmats0[21][30] + coeff0_22*dmats0[22][30] + coeff0_23*dmats0[23][30] + coeff0_24*dmats0[24][30] + coeff0_25*dmats0[25][30] + coeff0_26*dmats0[26][30] + coeff0_27*dmats0[27][30] + coeff0_28*dmats0[28][30] + coeff0_29*dmats0[29][30] + coeff0_30*dmats0[30][30] + coeff0_31*dmats0[31][30] + coeff0_32*dmats0[32][30] + coeff0_33*dmats0[33][30] + coeff0_34*dmats0[34][30] + coeff0_35*dmats0[35][30] + coeff0_36*dmats0[36][30] + coeff0_37*dmats0[37][30] + coeff0_38*dmats0[38][30] + coeff0_39*dmats0[39][30] + coeff0_40*dmats0[40][30] + coeff0_41*dmats0[41][30] + coeff0_42*dmats0[42][30] + coeff0_43*dmats0[43][30] + coeff0_44*dmats0[44][30] + coeff0_45*dmats0[45][30] + coeff0_46*dmats0[46][30] + coeff0_47*dmats0[47][30] + coeff0_48*dmats0[48][30] + coeff0_49*dmats0[49][30] + coeff0_50*dmats0[50][30] + coeff0_51*dmats0[51][30] + coeff0_52*dmats0[52][30] + coeff0_53*dmats0[53][30] + coeff0_54*dmats0[54][30] + coeff0_55*dmats0[55][30];
          new_coeff0_31 = coeff0_0*dmats0[0][31] + coeff0_1*dmats0[1][31] + coeff0_2*dmats0[2][31] + coeff0_3*dmats0[3][31] + coeff0_4*dmats0[4][31] + coeff0_5*dmats0[5][31] + coeff0_6*dmats0[6][31] + coeff0_7*dmats0[7][31] + coeff0_8*dmats0[8][31] + coeff0_9*dmats0[9][31] + coeff0_10*dmats0[10][31] + coeff0_11*dmats0[11][31] + coeff0_12*dmats0[12][31] + coeff0_13*dmats0[13][31] + coeff0_14*dmats0[14][31] + coeff0_15*dmats0[15][31] + coeff0_16*dmats0[16][31] + coeff0_17*dmats0[17][31] + coeff0_18*dmats0[18][31] + coeff0_19*dmats0[19][31] + coeff0_20*dmats0[20][31] + coeff0_21*dmats0[21][31] + coeff0_22*dmats0[22][31] + coeff0_23*dmats0[23][31] + coeff0_24*dmats0[24][31] + coeff0_25*dmats0[25][31] + coeff0_26*dmats0[26][31] + coeff0_27*dmats0[27][31] + coeff0_28*dmats0[28][31] + coeff0_29*dmats0[29][31] + coeff0_30*dmats0[30][31] + coeff0_31*dmats0[31][31] + coeff0_32*dmats0[32][31] + coeff0_33*dmats0[33][31] + coeff0_34*dmats0[34][31] + coeff0_35*dmats0[35][31] + coeff0_36*dmats0[36][31] + coeff0_37*dmats0[37][31] + coeff0_38*dmats0[38][31] + coeff0_39*dmats0[39][31] + coeff0_40*dmats0[40][31] + coeff0_41*dmats0[41][31] + coeff0_42*dmats0[42][31] + coeff0_43*dmats0[43][31] + coeff0_44*dmats0[44][31] + coeff0_45*dmats0[45][31] + coeff0_46*dmats0[46][31] + coeff0_47*dmats0[47][31] + coeff0_48*dmats0[48][31] + coeff0_49*dmats0[49][31] + coeff0_50*dmats0[50][31] + coeff0_51*dmats0[51][31] + coeff0_52*dmats0[52][31] + coeff0_53*dmats0[53][31] + coeff0_54*dmats0[54][31] + coeff0_55*dmats0[55][31];
          new_coeff0_32 = coeff0_0*dmats0[0][32] + coeff0_1*dmats0[1][32] + coeff0_2*dmats0[2][32] + coeff0_3*dmats0[3][32] + coeff0_4*dmats0[4][32] + coeff0_5*dmats0[5][32] + coeff0_6*dmats0[6][32] + coeff0_7*dmats0[7][32] + coeff0_8*dmats0[8][32] + coeff0_9*dmats0[9][32] + coeff0_10*dmats0[10][32] + coeff0_11*dmats0[11][32] + coeff0_12*dmats0[12][32] + coeff0_13*dmats0[13][32] + coeff0_14*dmats0[14][32] + coeff0_15*dmats0[15][32] + coeff0_16*dmats0[16][32] + coeff0_17*dmats0[17][32] + coeff0_18*dmats0[18][32] + coeff0_19*dmats0[19][32] + coeff0_20*dmats0[20][32] + coeff0_21*dmats0[21][32] + coeff0_22*dmats0[22][32] + coeff0_23*dmats0[23][32] + coeff0_24*dmats0[24][32] + coeff0_25*dmats0[25][32] + coeff0_26*dmats0[26][32] + coeff0_27*dmats0[27][32] + coeff0_28*dmats0[28][32] + coeff0_29*dmats0[29][32] + coeff0_30*dmats0[30][32] + coeff0_31*dmats0[31][32] + coeff0_32*dmats0[32][32] + coeff0_33*dmats0[33][32] + coeff0_34*dmats0[34][32] + coeff0_35*dmats0[35][32] + coeff0_36*dmats0[36][32] + coeff0_37*dmats0[37][32] + coeff0_38*dmats0[38][32] + coeff0_39*dmats0[39][32] + coeff0_40*dmats0[40][32] + coeff0_41*dmats0[41][32] + coeff0_42*dmats0[42][32] + coeff0_43*dmats0[43][32] + coeff0_44*dmats0[44][32] + coeff0_45*dmats0[45][32] + coeff0_46*dmats0[46][32] + coeff0_47*dmats0[47][32] + coeff0_48*dmats0[48][32] + coeff0_49*dmats0[49][32] + coeff0_50*dmats0[50][32] + coeff0_51*dmats0[51][32] + coeff0_52*dmats0[52][32] + coeff0_53*dmats0[53][32] + coeff0_54*dmats0[54][32] + coeff0_55*dmats0[55][32];
          new_coeff0_33 = coeff0_0*dmats0[0][33] + coeff0_1*dmats0[1][33] + coeff0_2*dmats0[2][33] + coeff0_3*dmats0[3][33] + coeff0_4*dmats0[4][33] + coeff0_5*dmats0[5][33] + coeff0_6*dmats0[6][33] + coeff0_7*dmats0[7][33] + coeff0_8*dmats0[8][33] + coeff0_9*dmats0[9][33] + coeff0_10*dmats0[10][33] + coeff0_11*dmats0[11][33] + coeff0_12*dmats0[12][33] + coeff0_13*dmats0[13][33] + coeff0_14*dmats0[14][33] + coeff0_15*dmats0[15][33] + coeff0_16*dmats0[16][33] + coeff0_17*dmats0[17][33] + coeff0_18*dmats0[18][33] + coeff0_19*dmats0[19][33] + coeff0_20*dmats0[20][33] + coeff0_21*dmats0[21][33] + coeff0_22*dmats0[22][33] + coeff0_23*dmats0[23][33] + coeff0_24*dmats0[24][33] + coeff0_25*dmats0[25][33] + coeff0_26*dmats0[26][33] + coeff0_27*dmats0[27][33] + coeff0_28*dmats0[28][33] + coeff0_29*dmats0[29][33] + coeff0_30*dmats0[30][33] + coeff0_31*dmats0[31][33] + coeff0_32*dmats0[32][33] + coeff0_33*dmats0[33][33] + coeff0_34*dmats0[34][33] + coeff0_35*dmats0[35][33] + coeff0_36*dmats0[36][33] + coeff0_37*dmats0[37][33] + coeff0_38*dmats0[38][33] + coeff0_39*dmats0[39][33] + coeff0_40*dmats0[40][33] + coeff0_41*dmats0[41][33] + coeff0_42*dmats0[42][33] + coeff0_43*dmats0[43][33] + coeff0_44*dmats0[44][33] + coeff0_45*dmats0[45][33] + coeff0_46*dmats0[46][33] + coeff0_47*dmats0[47][33] + coeff0_48*dmats0[48][33] + coeff0_49*dmats0[49][33] + coeff0_50*dmats0[50][33] + coeff0_51*dmats0[51][33] + coeff0_52*dmats0[52][33] + coeff0_53*dmats0[53][33] + coeff0_54*dmats0[54][33] + coeff0_55*dmats0[55][33];
          new_coeff0_34 = coeff0_0*dmats0[0][34] + coeff0_1*dmats0[1][34] + coeff0_2*dmats0[2][34] + coeff0_3*dmats0[3][34] + coeff0_4*dmats0[4][34] + coeff0_5*dmats0[5][34] + coeff0_6*dmats0[6][34] + coeff0_7*dmats0[7][34] + coeff0_8*dmats0[8][34] + coeff0_9*dmats0[9][34] + coeff0_10*dmats0[10][34] + coeff0_11*dmats0[11][34] + coeff0_12*dmats0[12][34] + coeff0_13*dmats0[13][34] + coeff0_14*dmats0[14][34] + coeff0_15*dmats0[15][34] + coeff0_16*dmats0[16][34] + coeff0_17*dmats0[17][34] + coeff0_18*dmats0[18][34] + coeff0_19*dmats0[19][34] + coeff0_20*dmats0[20][34] + coeff0_21*dmats0[21][34] + coeff0_22*dmats0[22][34] + coeff0_23*dmats0[23][34] + coeff0_24*dmats0[24][34] + coeff0_25*dmats0[25][34] + coeff0_26*dmats0[26][34] + coeff0_27*dmats0[27][34] + coeff0_28*dmats0[28][34] + coeff0_29*dmats0[29][34] + coeff0_30*dmats0[30][34] + coeff0_31*dmats0[31][34] + coeff0_32*dmats0[32][34] + coeff0_33*dmats0[33][34] + coeff0_34*dmats0[34][34] + coeff0_35*dmats0[35][34] + coeff0_36*dmats0[36][34] + coeff0_37*dmats0[37][34] + coeff0_38*dmats0[38][34] + coeff0_39*dmats0[39][34] + coeff0_40*dmats0[40][34] + coeff0_41*dmats0[41][34] + coeff0_42*dmats0[42][34] + coeff0_43*dmats0[43][34] + coeff0_44*dmats0[44][34] + coeff0_45*dmats0[45][34] + coeff0_46*dmats0[46][34] + coeff0_47*dmats0[47][34] + coeff0_48*dmats0[48][34] + coeff0_49*dmats0[49][34] + coeff0_50*dmats0[50][34] + coeff0_51*dmats0[51][34] + coeff0_52*dmats0[52][34] + coeff0_53*dmats0[53][34] + coeff0_54*dmats0[54][34] + coeff0_55*dmats0[55][34];
          new_coeff0_35 = coeff0_0*dmats0[0][35] + coeff0_1*dmats0[1][35] + coeff0_2*dmats0[2][35] + coeff0_3*dmats0[3][35] + coeff0_4*dmats0[4][35] + coeff0_5*dmats0[5][35] + coeff0_6*dmats0[6][35] + coeff0_7*dmats0[7][35] + coeff0_8*dmats0[8][35] + coeff0_9*dmats0[9][35] + coeff0_10*dmats0[10][35] + coeff0_11*dmats0[11][35] + coeff0_12*dmats0[12][35] + coeff0_13*dmats0[13][35] + coeff0_14*dmats0[14][35] + coeff0_15*dmats0[15][35] + coeff0_16*dmats0[16][35] + coeff0_17*dmats0[17][35] + coeff0_18*dmats0[18][35] + coeff0_19*dmats0[19][35] + coeff0_20*dmats0[20][35] + coeff0_21*dmats0[21][35] + coeff0_22*dmats0[22][35] + coeff0_23*dmats0[23][35] + coeff0_24*dmats0[24][35] + coeff0_25*dmats0[25][35] + coeff0_26*dmats0[26][35] + coeff0_27*dmats0[27][35] + coeff0_28*dmats0[28][35] + coeff0_29*dmats0[29][35] + coeff0_30*dmats0[30][35] + coeff0_31*dmats0[31][35] + coeff0_32*dmats0[32][35] + coeff0_33*dmats0[33][35] + coeff0_34*dmats0[34][35] + coeff0_35*dmats0[35][35] + coeff0_36*dmats0[36][35] + coeff0_37*dmats0[37][35] + coeff0_38*dmats0[38][35] + coeff0_39*dmats0[39][35] + coeff0_40*dmats0[40][35] + coeff0_41*dmats0[41][35] + coeff0_42*dmats0[42][35] + coeff0_43*dmats0[43][35] + coeff0_44*dmats0[44][35] + coeff0_45*dmats0[45][35] + coeff0_46*dmats0[46][35] + coeff0_47*dmats0[47][35] + coeff0_48*dmats0[48][35] + coeff0_49*dmats0[49][35] + coeff0_50*dmats0[50][35] + coeff0_51*dmats0[51][35] + coeff0_52*dmats0[52][35] + coeff0_53*dmats0[53][35] + coeff0_54*dmats0[54][35] + coeff0_55*dmats0[55][35];
          new_coeff0_36 = coeff0_0*dmats0[0][36] + coeff0_1*dmats0[1][36] + coeff0_2*dmats0[2][36] + coeff0_3*dmats0[3][36] + coeff0_4*dmats0[4][36] + coeff0_5*dmats0[5][36] + coeff0_6*dmats0[6][36] + coeff0_7*dmats0[7][36] + coeff0_8*dmats0[8][36] + coeff0_9*dmats0[9][36] + coeff0_10*dmats0[10][36] + coeff0_11*dmats0[11][36] + coeff0_12*dmats0[12][36] + coeff0_13*dmats0[13][36] + coeff0_14*dmats0[14][36] + coeff0_15*dmats0[15][36] + coeff0_16*dmats0[16][36] + coeff0_17*dmats0[17][36] + coeff0_18*dmats0[18][36] + coeff0_19*dmats0[19][36] + coeff0_20*dmats0[20][36] + coeff0_21*dmats0[21][36] + coeff0_22*dmats0[22][36] + coeff0_23*dmats0[23][36] + coeff0_24*dmats0[24][36] + coeff0_25*dmats0[25][36] + coeff0_26*dmats0[26][36] + coeff0_27*dmats0[27][36] + coeff0_28*dmats0[28][36] + coeff0_29*dmats0[29][36] + coeff0_30*dmats0[30][36] + coeff0_31*dmats0[31][36] + coeff0_32*dmats0[32][36] + coeff0_33*dmats0[33][36] + coeff0_34*dmats0[34][36] + coeff0_35*dmats0[35][36] + coeff0_36*dmats0[36][36] + coeff0_37*dmats0[37][36] + coeff0_38*dmats0[38][36] + coeff0_39*dmats0[39][36] + coeff0_40*dmats0[40][36] + coeff0_41*dmats0[41][36] + coeff0_42*dmats0[42][36] + coeff0_43*dmats0[43][36] + coeff0_44*dmats0[44][36] + coeff0_45*dmats0[45][36] + coeff0_46*dmats0[46][36] + coeff0_47*dmats0[47][36] + coeff0_48*dmats0[48][36] + coeff0_49*dmats0[49][36] + coeff0_50*dmats0[50][36] + coeff0_51*dmats0[51][36] + coeff0_52*dmats0[52][36] + coeff0_53*dmats0[53][36] + coeff0_54*dmats0[54][36] + coeff0_55*dmats0[55][36];
          new_coeff0_37 = coeff0_0*dmats0[0][37] + coeff0_1*dmats0[1][37] + coeff0_2*dmats0[2][37] + coeff0_3*dmats0[3][37] + coeff0_4*dmats0[4][37] + coeff0_5*dmats0[5][37] + coeff0_6*dmats0[6][37] + coeff0_7*dmats0[7][37] + coeff0_8*dmats0[8][37] + coeff0_9*dmats0[9][37] + coeff0_10*dmats0[10][37] + coeff0_11*dmats0[11][37] + coeff0_12*dmats0[12][37] + coeff0_13*dmats0[13][37] + coeff0_14*dmats0[14][37] + coeff0_15*dmats0[15][37] + coeff0_16*dmats0[16][37] + coeff0_17*dmats0[17][37] + coeff0_18*dmats0[18][37] + coeff0_19*dmats0[19][37] + coeff0_20*dmats0[20][37] + coeff0_21*dmats0[21][37] + coeff0_22*dmats0[22][37] + coeff0_23*dmats0[23][37] + coeff0_24*dmats0[24][37] + coeff0_25*dmats0[25][37] + coeff0_26*dmats0[26][37] + coeff0_27*dmats0[27][37] + coeff0_28*dmats0[28][37] + coeff0_29*dmats0[29][37] + coeff0_30*dmats0[30][37] + coeff0_31*dmats0[31][37] + coeff0_32*dmats0[32][37] + coeff0_33*dmats0[33][37] + coeff0_34*dmats0[34][37] + coeff0_35*dmats0[35][37] + coeff0_36*dmats0[36][37] + coeff0_37*dmats0[37][37] + coeff0_38*dmats0[38][37] + coeff0_39*dmats0[39][37] + coeff0_40*dmats0[40][37] + coeff0_41*dmats0[41][37] + coeff0_42*dmats0[42][37] + coeff0_43*dmats0[43][37] + coeff0_44*dmats0[44][37] + coeff0_45*dmats0[45][37] + coeff0_46*dmats0[46][37] + coeff0_47*dmats0[47][37] + coeff0_48*dmats0[48][37] + coeff0_49*dmats0[49][37] + coeff0_50*dmats0[50][37] + coeff0_51*dmats0[51][37] + coeff0_52*dmats0[52][37] + coeff0_53*dmats0[53][37] + coeff0_54*dmats0[54][37] + coeff0_55*dmats0[55][37];
          new_coeff0_38 = coeff0_0*dmats0[0][38] + coeff0_1*dmats0[1][38] + coeff0_2*dmats0[2][38] + coeff0_3*dmats0[3][38] + coeff0_4*dmats0[4][38] + coeff0_5*dmats0[5][38] + coeff0_6*dmats0[6][38] + coeff0_7*dmats0[7][38] + coeff0_8*dmats0[8][38] + coeff0_9*dmats0[9][38] + coeff0_10*dmats0[10][38] + coeff0_11*dmats0[11][38] + coeff0_12*dmats0[12][38] + coeff0_13*dmats0[13][38] + coeff0_14*dmats0[14][38] + coeff0_15*dmats0[15][38] + coeff0_16*dmats0[16][38] + coeff0_17*dmats0[17][38] + coeff0_18*dmats0[18][38] + coeff0_19*dmats0[19][38] + coeff0_20*dmats0[20][38] + coeff0_21*dmats0[21][38] + coeff0_22*dmats0[22][38] + coeff0_23*dmats0[23][38] + coeff0_24*dmats0[24][38] + coeff0_25*dmats0[25][38] + coeff0_26*dmats0[26][38] + coeff0_27*dmats0[27][38] + coeff0_28*dmats0[28][38] + coeff0_29*dmats0[29][38] + coeff0_30*dmats0[30][38] + coeff0_31*dmats0[31][38] + coeff0_32*dmats0[32][38] + coeff0_33*dmats0[33][38] + coeff0_34*dmats0[34][38] + coeff0_35*dmats0[35][38] + coeff0_36*dmats0[36][38] + coeff0_37*dmats0[37][38] + coeff0_38*dmats0[38][38] + coeff0_39*dmats0[39][38] + coeff0_40*dmats0[40][38] + coeff0_41*dmats0[41][38] + coeff0_42*dmats0[42][38] + coeff0_43*dmats0[43][38] + coeff0_44*dmats0[44][38] + coeff0_45*dmats0[45][38] + coeff0_46*dmats0[46][38] + coeff0_47*dmats0[47][38] + coeff0_48*dmats0[48][38] + coeff0_49*dmats0[49][38] + coeff0_50*dmats0[50][38] + coeff0_51*dmats0[51][38] + coeff0_52*dmats0[52][38] + coeff0_53*dmats0[53][38] + coeff0_54*dmats0[54][38] + coeff0_55*dmats0[55][38];
          new_coeff0_39 = coeff0_0*dmats0[0][39] + coeff0_1*dmats0[1][39] + coeff0_2*dmats0[2][39] + coeff0_3*dmats0[3][39] + coeff0_4*dmats0[4][39] + coeff0_5*dmats0[5][39] + coeff0_6*dmats0[6][39] + coeff0_7*dmats0[7][39] + coeff0_8*dmats0[8][39] + coeff0_9*dmats0[9][39] + coeff0_10*dmats0[10][39] + coeff0_11*dmats0[11][39] + coeff0_12*dmats0[12][39] + coeff0_13*dmats0[13][39] + coeff0_14*dmats0[14][39] + coeff0_15*dmats0[15][39] + coeff0_16*dmats0[16][39] + coeff0_17*dmats0[17][39] + coeff0_18*dmats0[18][39] + coeff0_19*dmats0[19][39] + coeff0_20*dmats0[20][39] + coeff0_21*dmats0[21][39] + coeff0_22*dmats0[22][39] + coeff0_23*dmats0[23][39] + coeff0_24*dmats0[24][39] + coeff0_25*dmats0[25][39] + coeff0_26*dmats0[26][39] + coeff0_27*dmats0[27][39] + coeff0_28*dmats0[28][39] + coeff0_29*dmats0[29][39] + coeff0_30*dmats0[30][39] + coeff0_31*dmats0[31][39] + coeff0_32*dmats0[32][39] + coeff0_33*dmats0[33][39] + coeff0_34*dmats0[34][39] + coeff0_35*dmats0[35][39] + coeff0_36*dmats0[36][39] + coeff0_37*dmats0[37][39] + coeff0_38*dmats0[38][39] + coeff0_39*dmats0[39][39] + coeff0_40*dmats0[40][39] + coeff0_41*dmats0[41][39] + coeff0_42*dmats0[42][39] + coeff0_43*dmats0[43][39] + coeff0_44*dmats0[44][39] + coeff0_45*dmats0[45][39] + coeff0_46*dmats0[46][39] + coeff0_47*dmats0[47][39] + coeff0_48*dmats0[48][39] + coeff0_49*dmats0[49][39] + coeff0_50*dmats0[50][39] + coeff0_51*dmats0[51][39] + coeff0_52*dmats0[52][39] + coeff0_53*dmats0[53][39] + coeff0_54*dmats0[54][39] + coeff0_55*dmats0[55][39];
          new_coeff0_40 = coeff0_0*dmats0[0][40] + coeff0_1*dmats0[1][40] + coeff0_2*dmats0[2][40] + coeff0_3*dmats0[3][40] + coeff0_4*dmats0[4][40] + coeff0_5*dmats0[5][40] + coeff0_6*dmats0[6][40] + coeff0_7*dmats0[7][40] + coeff0_8*dmats0[8][40] + coeff0_9*dmats0[9][40] + coeff0_10*dmats0[10][40] + coeff0_11*dmats0[11][40] + coeff0_12*dmats0[12][40] + coeff0_13*dmats0[13][40] + coeff0_14*dmats0[14][40] + coeff0_15*dmats0[15][40] + coeff0_16*dmats0[16][40] + coeff0_17*dmats0[17][40] + coeff0_18*dmats0[18][40] + coeff0_19*dmats0[19][40] + coeff0_20*dmats0[20][40] + coeff0_21*dmats0[21][40] + coeff0_22*dmats0[22][40] + coeff0_23*dmats0[23][40] + coeff0_24*dmats0[24][40] + coeff0_25*dmats0[25][40] + coeff0_26*dmats0[26][40] + coeff0_27*dmats0[27][40] + coeff0_28*dmats0[28][40] + coeff0_29*dmats0[29][40] + coeff0_30*dmats0[30][40] + coeff0_31*dmats0[31][40] + coeff0_32*dmats0[32][40] + coeff0_33*dmats0[33][40] + coeff0_34*dmats0[34][40] + coeff0_35*dmats0[35][40] + coeff0_36*dmats0[36][40] + coeff0_37*dmats0[37][40] + coeff0_38*dmats0[38][40] + coeff0_39*dmats0[39][40] + coeff0_40*dmats0[40][40] + coeff0_41*dmats0[41][40] + coeff0_42*dmats0[42][40] + coeff0_43*dmats0[43][40] + coeff0_44*dmats0[44][40] + coeff0_45*dmats0[45][40] + coeff0_46*dmats0[46][40] + coeff0_47*dmats0[47][40] + coeff0_48*dmats0[48][40] + coeff0_49*dmats0[49][40] + coeff0_50*dmats0[50][40] + coeff0_51*dmats0[51][40] + coeff0_52*dmats0[52][40] + coeff0_53*dmats0[53][40] + coeff0_54*dmats0[54][40] + coeff0_55*dmats0[55][40];
          new_coeff0_41 = coeff0_0*dmats0[0][41] + coeff0_1*dmats0[1][41] + coeff0_2*dmats0[2][41] + coeff0_3*dmats0[3][41] + coeff0_4*dmats0[4][41] + coeff0_5*dmats0[5][41] + coeff0_6*dmats0[6][41] + coeff0_7*dmats0[7][41] + coeff0_8*dmats0[8][41] + coeff0_9*dmats0[9][41] + coeff0_10*dmats0[10][41] + coeff0_11*dmats0[11][41] + coeff0_12*dmats0[12][41] + coeff0_13*dmats0[13][41] + coeff0_14*dmats0[14][41] + coeff0_15*dmats0[15][41] + coeff0_16*dmats0[16][41] + coeff0_17*dmats0[17][41] + coeff0_18*dmats0[18][41] + coeff0_19*dmats0[19][41] + coeff0_20*dmats0[20][41] + coeff0_21*dmats0[21][41] + coeff0_22*dmats0[22][41] + coeff0_23*dmats0[23][41] + coeff0_24*dmats0[24][41] + coeff0_25*dmats0[25][41] + coeff0_26*dmats0[26][41] + coeff0_27*dmats0[27][41] + coeff0_28*dmats0[28][41] + coeff0_29*dmats0[29][41] + coeff0_30*dmats0[30][41] + coeff0_31*dmats0[31][41] + coeff0_32*dmats0[32][41] + coeff0_33*dmats0[33][41] + coeff0_34*dmats0[34][41] + coeff0_35*dmats0[35][41] + coeff0_36*dmats0[36][41] + coeff0_37*dmats0[37][41] + coeff0_38*dmats0[38][41] + coeff0_39*dmats0[39][41] + coeff0_40*dmats0[40][41] + coeff0_41*dmats0[41][41] + coeff0_42*dmats0[42][41] + coeff0_43*dmats0[43][41] + coeff0_44*dmats0[44][41] + coeff0_45*dmats0[45][41] + coeff0_46*dmats0[46][41] + coeff0_47*dmats0[47][41] + coeff0_48*dmats0[48][41] + coeff0_49*dmats0[49][41] + coeff0_50*dmats0[50][41] + coeff0_51*dmats0[51][41] + coeff0_52*dmats0[52][41] + coeff0_53*dmats0[53][41] + coeff0_54*dmats0[54][41] + coeff0_55*dmats0[55][41];
          new_coeff0_42 = coeff0_0*dmats0[0][42] + coeff0_1*dmats0[1][42] + coeff0_2*dmats0[2][42] + coeff0_3*dmats0[3][42] + coeff0_4*dmats0[4][42] + coeff0_5*dmats0[5][42] + coeff0_6*dmats0[6][42] + coeff0_7*dmats0[7][42] + coeff0_8*dmats0[8][42] + coeff0_9*dmats0[9][42] + coeff0_10*dmats0[10][42] + coeff0_11*dmats0[11][42] + coeff0_12*dmats0[12][42] + coeff0_13*dmats0[13][42] + coeff0_14*dmats0[14][42] + coeff0_15*dmats0[15][42] + coeff0_16*dmats0[16][42] + coeff0_17*dmats0[17][42] + coeff0_18*dmats0[18][42] + coeff0_19*dmats0[19][42] + coeff0_20*dmats0[20][42] + coeff0_21*dmats0[21][42] + coeff0_22*dmats0[22][42] + coeff0_23*dmats0[23][42] + coeff0_24*dmats0[24][42] + coeff0_25*dmats0[25][42] + coeff0_26*dmats0[26][42] + coeff0_27*dmats0[27][42] + coeff0_28*dmats0[28][42] + coeff0_29*dmats0[29][42] + coeff0_30*dmats0[30][42] + coeff0_31*dmats0[31][42] + coeff0_32*dmats0[32][42] + coeff0_33*dmats0[33][42] + coeff0_34*dmats0[34][42] + coeff0_35*dmats0[35][42] + coeff0_36*dmats0[36][42] + coeff0_37*dmats0[37][42] + coeff0_38*dmats0[38][42] + coeff0_39*dmats0[39][42] + coeff0_40*dmats0[40][42] + coeff0_41*dmats0[41][42] + coeff0_42*dmats0[42][42] + coeff0_43*dmats0[43][42] + coeff0_44*dmats0[44][42] + coeff0_45*dmats0[45][42] + coeff0_46*dmats0[46][42] + coeff0_47*dmats0[47][42] + coeff0_48*dmats0[48][42] + coeff0_49*dmats0[49][42] + coeff0_50*dmats0[50][42] + coeff0_51*dmats0[51][42] + coeff0_52*dmats0[52][42] + coeff0_53*dmats0[53][42] + coeff0_54*dmats0[54][42] + coeff0_55*dmats0[55][42];
          new_coeff0_43 = coeff0_0*dmats0[0][43] + coeff0_1*dmats0[1][43] + coeff0_2*dmats0[2][43] + coeff0_3*dmats0[3][43] + coeff0_4*dmats0[4][43] + coeff0_5*dmats0[5][43] + coeff0_6*dmats0[6][43] + coeff0_7*dmats0[7][43] + coeff0_8*dmats0[8][43] + coeff0_9*dmats0[9][43] + coeff0_10*dmats0[10][43] + coeff0_11*dmats0[11][43] + coeff0_12*dmats0[12][43] + coeff0_13*dmats0[13][43] + coeff0_14*dmats0[14][43] + coeff0_15*dmats0[15][43] + coeff0_16*dmats0[16][43] + coeff0_17*dmats0[17][43] + coeff0_18*dmats0[18][43] + coeff0_19*dmats0[19][43] + coeff0_20*dmats0[20][43] + coeff0_21*dmats0[21][43] + coeff0_22*dmats0[22][43] + coeff0_23*dmats0[23][43] + coeff0_24*dmats0[24][43] + coeff0_25*dmats0[25][43] + coeff0_26*dmats0[26][43] + coeff0_27*dmats0[27][43] + coeff0_28*dmats0[28][43] + coeff0_29*dmats0[29][43] + coeff0_30*dmats0[30][43] + coeff0_31*dmats0[31][43] + coeff0_32*dmats0[32][43] + coeff0_33*dmats0[33][43] + coeff0_34*dmats0[34][43] + coeff0_35*dmats0[35][43] + coeff0_36*dmats0[36][43] + coeff0_37*dmats0[37][43] + coeff0_38*dmats0[38][43] + coeff0_39*dmats0[39][43] + coeff0_40*dmats0[40][43] + coeff0_41*dmats0[41][43] + coeff0_42*dmats0[42][43] + coeff0_43*dmats0[43][43] + coeff0_44*dmats0[44][43] + coeff0_45*dmats0[45][43] + coeff0_46*dmats0[46][43] + coeff0_47*dmats0[47][43] + coeff0_48*dmats0[48][43] + coeff0_49*dmats0[49][43] + coeff0_50*dmats0[50][43] + coeff0_51*dmats0[51][43] + coeff0_52*dmats0[52][43] + coeff0_53*dmats0[53][43] + coeff0_54*dmats0[54][43] + coeff0_55*dmats0[55][43];
          new_coeff0_44 = coeff0_0*dmats0[0][44] + coeff0_1*dmats0[1][44] + coeff0_2*dmats0[2][44] + coeff0_3*dmats0[3][44] + coeff0_4*dmats0[4][44] + coeff0_5*dmats0[5][44] + coeff0_6*dmats0[6][44] + coeff0_7*dmats0[7][44] + coeff0_8*dmats0[8][44] + coeff0_9*dmats0[9][44] + coeff0_10*dmats0[10][44] + coeff0_11*dmats0[11][44] + coeff0_12*dmats0[12][44] + coeff0_13*dmats0[13][44] + coeff0_14*dmats0[14][44] + coeff0_15*dmats0[15][44] + coeff0_16*dmats0[16][44] + coeff0_17*dmats0[17][44] + coeff0_18*dmats0[18][44] + coeff0_19*dmats0[19][44] + coeff0_20*dmats0[20][44] + coeff0_21*dmats0[21][44] + coeff0_22*dmats0[22][44] + coeff0_23*dmats0[23][44] + coeff0_24*dmats0[24][44] + coeff0_25*dmats0[25][44] + coeff0_26*dmats0[26][44] + coeff0_27*dmats0[27][44] + coeff0_28*dmats0[28][44] + coeff0_29*dmats0[29][44] + coeff0_30*dmats0[30][44] + coeff0_31*dmats0[31][44] + coeff0_32*dmats0[32][44] + coeff0_33*dmats0[33][44] + coeff0_34*dmats0[34][44] + coeff0_35*dmats0[35][44] + coeff0_36*dmats0[36][44] + coeff0_37*dmats0[37][44] + coeff0_38*dmats0[38][44] + coeff0_39*dmats0[39][44] + coeff0_40*dmats0[40][44] + coeff0_41*dmats0[41][44] + coeff0_42*dmats0[42][44] + coeff0_43*dmats0[43][44] + coeff0_44*dmats0[44][44] + coeff0_45*dmats0[45][44] + coeff0_46*dmats0[46][44] + coeff0_47*dmats0[47][44] + coeff0_48*dmats0[48][44] + coeff0_49*dmats0[49][44] + coeff0_50*dmats0[50][44] + coeff0_51*dmats0[51][44] + coeff0_52*dmats0[52][44] + coeff0_53*dmats0[53][44] + coeff0_54*dmats0[54][44] + coeff0_55*dmats0[55][44];
          new_coeff0_45 = coeff0_0*dmats0[0][45] + coeff0_1*dmats0[1][45] + coeff0_2*dmats0[2][45] + coeff0_3*dmats0[3][45] + coeff0_4*dmats0[4][45] + coeff0_5*dmats0[5][45] + coeff0_6*dmats0[6][45] + coeff0_7*dmats0[7][45] + coeff0_8*dmats0[8][45] + coeff0_9*dmats0[9][45] + coeff0_10*dmats0[10][45] + coeff0_11*dmats0[11][45] + coeff0_12*dmats0[12][45] + coeff0_13*dmats0[13][45] + coeff0_14*dmats0[14][45] + coeff0_15*dmats0[15][45] + coeff0_16*dmats0[16][45] + coeff0_17*dmats0[17][45] + coeff0_18*dmats0[18][45] + coeff0_19*dmats0[19][45] + coeff0_20*dmats0[20][45] + coeff0_21*dmats0[21][45] + coeff0_22*dmats0[22][45] + coeff0_23*dmats0[23][45] + coeff0_24*dmats0[24][45] + coeff0_25*dmats0[25][45] + coeff0_26*dmats0[26][45] + coeff0_27*dmats0[27][45] + coeff0_28*dmats0[28][45] + coeff0_29*dmats0[29][45] + coeff0_30*dmats0[30][45] + coeff0_31*dmats0[31][45] + coeff0_32*dmats0[32][45] + coeff0_33*dmats0[33][45] + coeff0_34*dmats0[34][45] + coeff0_35*dmats0[35][45] + coeff0_36*dmats0[36][45] + coeff0_37*dmats0[37][45] + coeff0_38*dmats0[38][45] + coeff0_39*dmats0[39][45] + coeff0_40*dmats0[40][45] + coeff0_41*dmats0[41][45] + coeff0_42*dmats0[42][45] + coeff0_43*dmats0[43][45] + coeff0_44*dmats0[44][45] + coeff0_45*dmats0[45][45] + coeff0_46*dmats0[46][45] + coeff0_47*dmats0[47][45] + coeff0_48*dmats0[48][45] + coeff0_49*dmats0[49][45] + coeff0_50*dmats0[50][45] + coeff0_51*dmats0[51][45] + coeff0_52*dmats0[52][45] + coeff0_53*dmats0[53][45] + coeff0_54*dmats0[54][45] + coeff0_55*dmats0[55][45];
          new_coeff0_46 = coeff0_0*dmats0[0][46] + coeff0_1*dmats0[1][46] + coeff0_2*dmats0[2][46] + coeff0_3*dmats0[3][46] + coeff0_4*dmats0[4][46] + coeff0_5*dmats0[5][46] + coeff0_6*dmats0[6][46] + coeff0_7*dmats0[7][46] + coeff0_8*dmats0[8][46] + coeff0_9*dmats0[9][46] + coeff0_10*dmats0[10][46] + coeff0_11*dmats0[11][46] + coeff0_12*dmats0[12][46] + coeff0_13*dmats0[13][46] + coeff0_14*dmats0[14][46] + coeff0_15*dmats0[15][46] + coeff0_16*dmats0[16][46] + coeff0_17*dmats0[17][46] + coeff0_18*dmats0[18][46] + coeff0_19*dmats0[19][46] + coeff0_20*dmats0[20][46] + coeff0_21*dmats0[21][46] + coeff0_22*dmats0[22][46] + coeff0_23*dmats0[23][46] + coeff0_24*dmats0[24][46] + coeff0_25*dmats0[25][46] + coeff0_26*dmats0[26][46] + coeff0_27*dmats0[27][46] + coeff0_28*dmats0[28][46] + coeff0_29*dmats0[29][46] + coeff0_30*dmats0[30][46] + coeff0_31*dmats0[31][46] + coeff0_32*dmats0[32][46] + coeff0_33*dmats0[33][46] + coeff0_34*dmats0[34][46] + coeff0_35*dmats0[35][46] + coeff0_36*dmats0[36][46] + coeff0_37*dmats0[37][46] + coeff0_38*dmats0[38][46] + coeff0_39*dmats0[39][46] + coeff0_40*dmats0[40][46] + coeff0_41*dmats0[41][46] + coeff0_42*dmats0[42][46] + coeff0_43*dmats0[43][46] + coeff0_44*dmats0[44][46] + coeff0_45*dmats0[45][46] + coeff0_46*dmats0[46][46] + coeff0_47*dmats0[47][46] + coeff0_48*dmats0[48][46] + coeff0_49*dmats0[49][46] + coeff0_50*dmats0[50][46] + coeff0_51*dmats0[51][46] + coeff0_52*dmats0[52][46] + coeff0_53*dmats0[53][46] + coeff0_54*dmats0[54][46] + coeff0_55*dmats0[55][46];
          new_coeff0_47 = coeff0_0*dmats0[0][47] + coeff0_1*dmats0[1][47] + coeff0_2*dmats0[2][47] + coeff0_3*dmats0[3][47] + coeff0_4*dmats0[4][47] + coeff0_5*dmats0[5][47] + coeff0_6*dmats0[6][47] + coeff0_7*dmats0[7][47] + coeff0_8*dmats0[8][47] + coeff0_9*dmats0[9][47] + coeff0_10*dmats0[10][47] + coeff0_11*dmats0[11][47] + coeff0_12*dmats0[12][47] + coeff0_13*dmats0[13][47] + coeff0_14*dmats0[14][47] + coeff0_15*dmats0[15][47] + coeff0_16*dmats0[16][47] + coeff0_17*dmats0[17][47] + coeff0_18*dmats0[18][47] + coeff0_19*dmats0[19][47] + coeff0_20*dmats0[20][47] + coeff0_21*dmats0[21][47] + coeff0_22*dmats0[22][47] + coeff0_23*dmats0[23][47] + coeff0_24*dmats0[24][47] + coeff0_25*dmats0[25][47] + coeff0_26*dmats0[26][47] + coeff0_27*dmats0[27][47] + coeff0_28*dmats0[28][47] + coeff0_29*dmats0[29][47] + coeff0_30*dmats0[30][47] + coeff0_31*dmats0[31][47] + coeff0_32*dmats0[32][47] + coeff0_33*dmats0[33][47] + coeff0_34*dmats0[34][47] + coeff0_35*dmats0[35][47] + coeff0_36*dmats0[36][47] + coeff0_37*dmats0[37][47] + coeff0_38*dmats0[38][47] + coeff0_39*dmats0[39][47] + coeff0_40*dmats0[40][47] + coeff0_41*dmats0[41][47] + coeff0_42*dmats0[42][47] + coeff0_43*dmats0[43][47] + coeff0_44*dmats0[44][47] + coeff0_45*dmats0[45][47] + coeff0_46*dmats0[46][47] + coeff0_47*dmats0[47][47] + coeff0_48*dmats0[48][47] + coeff0_49*dmats0[49][47] + coeff0_50*dmats0[50][47] + coeff0_51*dmats0[51][47] + coeff0_52*dmats0[52][47] + coeff0_53*dmats0[53][47] + coeff0_54*dmats0[54][47] + coeff0_55*dmats0[55][47];
          new_coeff0_48 = coeff0_0*dmats0[0][48] + coeff0_1*dmats0[1][48] + coeff0_2*dmats0[2][48] + coeff0_3*dmats0[3][48] + coeff0_4*dmats0[4][48] + coeff0_5*dmats0[5][48] + coeff0_6*dmats0[6][48] + coeff0_7*dmats0[7][48] + coeff0_8*dmats0[8][48] + coeff0_9*dmats0[9][48] + coeff0_10*dmats0[10][48] + coeff0_11*dmats0[11][48] + coeff0_12*dmats0[12][48] + coeff0_13*dmats0[13][48] + coeff0_14*dmats0[14][48] + coeff0_15*dmats0[15][48] + coeff0_16*dmats0[16][48] + coeff0_17*dmats0[17][48] + coeff0_18*dmats0[18][48] + coeff0_19*dmats0[19][48] + coeff0_20*dmats0[20][48] + coeff0_21*dmats0[21][48] + coeff0_22*dmats0[22][48] + coeff0_23*dmats0[23][48] + coeff0_24*dmats0[24][48] + coeff0_25*dmats0[25][48] + coeff0_26*dmats0[26][48] + coeff0_27*dmats0[27][48] + coeff0_28*dmats0[28][48] + coeff0_29*dmats0[29][48] + coeff0_30*dmats0[30][48] + coeff0_31*dmats0[31][48] + coeff0_32*dmats0[32][48] + coeff0_33*dmats0[33][48] + coeff0_34*dmats0[34][48] + coeff0_35*dmats0[35][48] + coeff0_36*dmats0[36][48] + coeff0_37*dmats0[37][48] + coeff0_38*dmats0[38][48] + coeff0_39*dmats0[39][48] + coeff0_40*dmats0[40][48] + coeff0_41*dmats0[41][48] + coeff0_42*dmats0[42][48] + coeff0_43*dmats0[43][48] + coeff0_44*dmats0[44][48] + coeff0_45*dmats0[45][48] + coeff0_46*dmats0[46][48] + coeff0_47*dmats0[47][48] + coeff0_48*dmats0[48][48] + coeff0_49*dmats0[49][48] + coeff0_50*dmats0[50][48] + coeff0_51*dmats0[51][48] + coeff0_52*dmats0[52][48] + coeff0_53*dmats0[53][48] + coeff0_54*dmats0[54][48] + coeff0_55*dmats0[55][48];
          new_coeff0_49 = coeff0_0*dmats0[0][49] + coeff0_1*dmats0[1][49] + coeff0_2*dmats0[2][49] + coeff0_3*dmats0[3][49] + coeff0_4*dmats0[4][49] + coeff0_5*dmats0[5][49] + coeff0_6*dmats0[6][49] + coeff0_7*dmats0[7][49] + coeff0_8*dmats0[8][49] + coeff0_9*dmats0[9][49] + coeff0_10*dmats0[10][49] + coeff0_11*dmats0[11][49] + coeff0_12*dmats0[12][49] + coeff0_13*dmats0[13][49] + coeff0_14*dmats0[14][49] + coeff0_15*dmats0[15][49] + coeff0_16*dmats0[16][49] + coeff0_17*dmats0[17][49] + coeff0_18*dmats0[18][49] + coeff0_19*dmats0[19][49] + coeff0_20*dmats0[20][49] + coeff0_21*dmats0[21][49] + coeff0_22*dmats0[22][49] + coeff0_23*dmats0[23][49] + coeff0_24*dmats0[24][49] + coeff0_25*dmats0[25][49] + coeff0_26*dmats0[26][49] + coeff0_27*dmats0[27][49] + coeff0_28*dmats0[28][49] + coeff0_29*dmats0[29][49] + coeff0_30*dmats0[30][49] + coeff0_31*dmats0[31][49] + coeff0_32*dmats0[32][49] + coeff0_33*dmats0[33][49] + coeff0_34*dmats0[34][49] + coeff0_35*dmats0[35][49] + coeff0_36*dmats0[36][49] + coeff0_37*dmats0[37][49] + coeff0_38*dmats0[38][49] + coeff0_39*dmats0[39][49] + coeff0_40*dmats0[40][49] + coeff0_41*dmats0[41][49] + coeff0_42*dmats0[42][49] + coeff0_43*dmats0[43][49] + coeff0_44*dmats0[44][49] + coeff0_45*dmats0[45][49] + coeff0_46*dmats0[46][49] + coeff0_47*dmats0[47][49] + coeff0_48*dmats0[48][49] + coeff0_49*dmats0[49][49] + coeff0_50*dmats0[50][49] + coeff0_51*dmats0[51][49] + coeff0_52*dmats0[52][49] + coeff0_53*dmats0[53][49] + coeff0_54*dmats0[54][49] + coeff0_55*dmats0[55][49];
          new_coeff0_50 = coeff0_0*dmats0[0][50] + coeff0_1*dmats0[1][50] + coeff0_2*dmats0[2][50] + coeff0_3*dmats0[3][50] + coeff0_4*dmats0[4][50] + coeff0_5*dmats0[5][50] + coeff0_6*dmats0[6][50] + coeff0_7*dmats0[7][50] + coeff0_8*dmats0[8][50] + coeff0_9*dmats0[9][50] + coeff0_10*dmats0[10][50] + coeff0_11*dmats0[11][50] + coeff0_12*dmats0[12][50] + coeff0_13*dmats0[13][50] + coeff0_14*dmats0[14][50] + coeff0_15*dmats0[15][50] + coeff0_16*dmats0[16][50] + coeff0_17*dmats0[17][50] + coeff0_18*dmats0[18][50] + coeff0_19*dmats0[19][50] + coeff0_20*dmats0[20][50] + coeff0_21*dmats0[21][50] + coeff0_22*dmats0[22][50] + coeff0_23*dmats0[23][50] + coeff0_24*dmats0[24][50] + coeff0_25*dmats0[25][50] + coeff0_26*dmats0[26][50] + coeff0_27*dmats0[27][50] + coeff0_28*dmats0[28][50] + coeff0_29*dmats0[29][50] + coeff0_30*dmats0[30][50] + coeff0_31*dmats0[31][50] + coeff0_32*dmats0[32][50] + coeff0_33*dmats0[33][50] + coeff0_34*dmats0[34][50] + coeff0_35*dmats0[35][50] + coeff0_36*dmats0[36][50] + coeff0_37*dmats0[37][50] + coeff0_38*dmats0[38][50] + coeff0_39*dmats0[39][50] + coeff0_40*dmats0[40][50] + coeff0_41*dmats0[41][50] + coeff0_42*dmats0[42][50] + coeff0_43*dmats0[43][50] + coeff0_44*dmats0[44][50] + coeff0_45*dmats0[45][50] + coeff0_46*dmats0[46][50] + coeff0_47*dmats0[47][50] + coeff0_48*dmats0[48][50] + coeff0_49*dmats0[49][50] + coeff0_50*dmats0[50][50] + coeff0_51*dmats0[51][50] + coeff0_52*dmats0[52][50] + coeff0_53*dmats0[53][50] + coeff0_54*dmats0[54][50] + coeff0_55*dmats0[55][50];
          new_coeff0_51 = coeff0_0*dmats0[0][51] + coeff0_1*dmats0[1][51] + coeff0_2*dmats0[2][51] + coeff0_3*dmats0[3][51] + coeff0_4*dmats0[4][51] + coeff0_5*dmats0[5][51] + coeff0_6*dmats0[6][51] + coeff0_7*dmats0[7][51] + coeff0_8*dmats0[8][51] + coeff0_9*dmats0[9][51] + coeff0_10*dmats0[10][51] + coeff0_11*dmats0[11][51] + coeff0_12*dmats0[12][51] + coeff0_13*dmats0[13][51] + coeff0_14*dmats0[14][51] + coeff0_15*dmats0[15][51] + coeff0_16*dmats0[16][51] + coeff0_17*dmats0[17][51] + coeff0_18*dmats0[18][51] + coeff0_19*dmats0[19][51] + coeff0_20*dmats0[20][51] + coeff0_21*dmats0[21][51] + coeff0_22*dmats0[22][51] + coeff0_23*dmats0[23][51] + coeff0_24*dmats0[24][51] + coeff0_25*dmats0[25][51] + coeff0_26*dmats0[26][51] + coeff0_27*dmats0[27][51] + coeff0_28*dmats0[28][51] + coeff0_29*dmats0[29][51] + coeff0_30*dmats0[30][51] + coeff0_31*dmats0[31][51] + coeff0_32*dmats0[32][51] + coeff0_33*dmats0[33][51] + coeff0_34*dmats0[34][51] + coeff0_35*dmats0[35][51] + coeff0_36*dmats0[36][51] + coeff0_37*dmats0[37][51] + coeff0_38*dmats0[38][51] + coeff0_39*dmats0[39][51] + coeff0_40*dmats0[40][51] + coeff0_41*dmats0[41][51] + coeff0_42*dmats0[42][51] + coeff0_43*dmats0[43][51] + coeff0_44*dmats0[44][51] + coeff0_45*dmats0[45][51] + coeff0_46*dmats0[46][51] + coeff0_47*dmats0[47][51] + coeff0_48*dmats0[48][51] + coeff0_49*dmats0[49][51] + coeff0_50*dmats0[50][51] + coeff0_51*dmats0[51][51] + coeff0_52*dmats0[52][51] + coeff0_53*dmats0[53][51] + coeff0_54*dmats0[54][51] + coeff0_55*dmats0[55][51];
          new_coeff0_52 = coeff0_0*dmats0[0][52] + coeff0_1*dmats0[1][52] + coeff0_2*dmats0[2][52] + coeff0_3*dmats0[3][52] + coeff0_4*dmats0[4][52] + coeff0_5*dmats0[5][52] + coeff0_6*dmats0[6][52] + coeff0_7*dmats0[7][52] + coeff0_8*dmats0[8][52] + coeff0_9*dmats0[9][52] + coeff0_10*dmats0[10][52] + coeff0_11*dmats0[11][52] + coeff0_12*dmats0[12][52] + coeff0_13*dmats0[13][52] + coeff0_14*dmats0[14][52] + coeff0_15*dmats0[15][52] + coeff0_16*dmats0[16][52] + coeff0_17*dmats0[17][52] + coeff0_18*dmats0[18][52] + coeff0_19*dmats0[19][52] + coeff0_20*dmats0[20][52] + coeff0_21*dmats0[21][52] + coeff0_22*dmats0[22][52] + coeff0_23*dmats0[23][52] + coeff0_24*dmats0[24][52] + coeff0_25*dmats0[25][52] + coeff0_26*dmats0[26][52] + coeff0_27*dmats0[27][52] + coeff0_28*dmats0[28][52] + coeff0_29*dmats0[29][52] + coeff0_30*dmats0[30][52] + coeff0_31*dmats0[31][52] + coeff0_32*dmats0[32][52] + coeff0_33*dmats0[33][52] + coeff0_34*dmats0[34][52] + coeff0_35*dmats0[35][52] + coeff0_36*dmats0[36][52] + coeff0_37*dmats0[37][52] + coeff0_38*dmats0[38][52] + coeff0_39*dmats0[39][52] + coeff0_40*dmats0[40][52] + coeff0_41*dmats0[41][52] + coeff0_42*dmats0[42][52] + coeff0_43*dmats0[43][52] + coeff0_44*dmats0[44][52] + coeff0_45*dmats0[45][52] + coeff0_46*dmats0[46][52] + coeff0_47*dmats0[47][52] + coeff0_48*dmats0[48][52] + coeff0_49*dmats0[49][52] + coeff0_50*dmats0[50][52] + coeff0_51*dmats0[51][52] + coeff0_52*dmats0[52][52] + coeff0_53*dmats0[53][52] + coeff0_54*dmats0[54][52] + coeff0_55*dmats0[55][52];
          new_coeff0_53 = coeff0_0*dmats0[0][53] + coeff0_1*dmats0[1][53] + coeff0_2*dmats0[2][53] + coeff0_3*dmats0[3][53] + coeff0_4*dmats0[4][53] + coeff0_5*dmats0[5][53] + coeff0_6*dmats0[6][53] + coeff0_7*dmats0[7][53] + coeff0_8*dmats0[8][53] + coeff0_9*dmats0[9][53] + coeff0_10*dmats0[10][53] + coeff0_11*dmats0[11][53] + coeff0_12*dmats0[12][53] + coeff0_13*dmats0[13][53] + coeff0_14*dmats0[14][53] + coeff0_15*dmats0[15][53] + coeff0_16*dmats0[16][53] + coeff0_17*dmats0[17][53] + coeff0_18*dmats0[18][53] + coeff0_19*dmats0[19][53] + coeff0_20*dmats0[20][53] + coeff0_21*dmats0[21][53] + coeff0_22*dmats0[22][53] + coeff0_23*dmats0[23][53] + coeff0_24*dmats0[24][53] + coeff0_25*dmats0[25][53] + coeff0_26*dmats0[26][53] + coeff0_27*dmats0[27][53] + coeff0_28*dmats0[28][53] + coeff0_29*dmats0[29][53] + coeff0_30*dmats0[30][53] + coeff0_31*dmats0[31][53] + coeff0_32*dmats0[32][53] + coeff0_33*dmats0[33][53] + coeff0_34*dmats0[34][53] + coeff0_35*dmats0[35][53] + coeff0_36*dmats0[36][53] + coeff0_37*dmats0[37][53] + coeff0_38*dmats0[38][53] + coeff0_39*dmats0[39][53] + coeff0_40*dmats0[40][53] + coeff0_41*dmats0[41][53] + coeff0_42*dmats0[42][53] + coeff0_43*dmats0[43][53] + coeff0_44*dmats0[44][53] + coeff0_45*dmats0[45][53] + coeff0_46*dmats0[46][53] + coeff0_47*dmats0[47][53] + coeff0_48*dmats0[48][53] + coeff0_49*dmats0[49][53] + coeff0_50*dmats0[50][53] + coeff0_51*dmats0[51][53] + coeff0_52*dmats0[52][53] + coeff0_53*dmats0[53][53] + coeff0_54*dmats0[54][53] + coeff0_55*dmats0[55][53];
          new_coeff0_54 = coeff0_0*dmats0[0][54] + coeff0_1*dmats0[1][54] + coeff0_2*dmats0[2][54] + coeff0_3*dmats0[3][54] + coeff0_4*dmats0[4][54] + coeff0_5*dmats0[5][54] + coeff0_6*dmats0[6][54] + coeff0_7*dmats0[7][54] + coeff0_8*dmats0[8][54] + coeff0_9*dmats0[9][54] + coeff0_10*dmats0[10][54] + coeff0_11*dmats0[11][54] + coeff0_12*dmats0[12][54] + coeff0_13*dmats0[13][54] + coeff0_14*dmats0[14][54] + coeff0_15*dmats0[15][54] + coeff0_16*dmats0[16][54] + coeff0_17*dmats0[17][54] + coeff0_18*dmats0[18][54] + coeff0_19*dmats0[19][54] + coeff0_20*dmats0[20][54] + coeff0_21*dmats0[21][54] + coeff0_22*dmats0[22][54] + coeff0_23*dmats0[23][54] + coeff0_24*dmats0[24][54] + coeff0_25*dmats0[25][54] + coeff0_26*dmats0[26][54] + coeff0_27*dmats0[27][54] + coeff0_28*dmats0[28][54] + coeff0_29*dmats0[29][54] + coeff0_30*dmats0[30][54] + coeff0_31*dmats0[31][54] + coeff0_32*dmats0[32][54] + coeff0_33*dmats0[33][54] + coeff0_34*dmats0[34][54] + coeff0_35*dmats0[35][54] + coeff0_36*dmats0[36][54] + coeff0_37*dmats0[37][54] + coeff0_38*dmats0[38][54] + coeff0_39*dmats0[39][54] + coeff0_40*dmats0[40][54] + coeff0_41*dmats0[41][54] + coeff0_42*dmats0[42][54] + coeff0_43*dmats0[43][54] + coeff0_44*dmats0[44][54] + coeff0_45*dmats0[45][54] + coeff0_46*dmats0[46][54] + coeff0_47*dmats0[47][54] + coeff0_48*dmats0[48][54] + coeff0_49*dmats0[49][54] + coeff0_50*dmats0[50][54] + coeff0_51*dmats0[51][54] + coeff0_52*dmats0[52][54] + coeff0_53*dmats0[53][54] + coeff0_54*dmats0[54][54] + coeff0_55*dmats0[55][54];
          new_coeff0_55 = coeff0_0*dmats0[0][55] + coeff0_1*dmats0[1][55] + coeff0_2*dmats0[2][55] + coeff0_3*dmats0[3][55] + coeff0_4*dmats0[4][55] + coeff0_5*dmats0[5][55] + coeff0_6*dmats0[6][55] + coeff0_7*dmats0[7][55] + coeff0_8*dmats0[8][55] + coeff0_9*dmats0[9][55] + coeff0_10*dmats0[10][55] + coeff0_11*dmats0[11][55] + coeff0_12*dmats0[12][55] + coeff0_13*dmats0[13][55] + coeff0_14*dmats0[14][55] + coeff0_15*dmats0[15][55] + coeff0_16*dmats0[16][55] + coeff0_17*dmats0[17][55] + coeff0_18*dmats0[18][55] + coeff0_19*dmats0[19][55] + coeff0_20*dmats0[20][55] + coeff0_21*dmats0[21][55] + coeff0_22*dmats0[22][55] + coeff0_23*dmats0[23][55] + coeff0_24*dmats0[24][55] + coeff0_25*dmats0[25][55] + coeff0_26*dmats0[26][55] + coeff0_27*dmats0[27][55] + coeff0_28*dmats0[28][55] + coeff0_29*dmats0[29][55] + coeff0_30*dmats0[30][55] + coeff0_31*dmats0[31][55] + coeff0_32*dmats0[32][55] + coeff0_33*dmats0[33][55] + coeff0_34*dmats0[34][55] + coeff0_35*dmats0[35][55] + coeff0_36*dmats0[36][55] + coeff0_37*dmats0[37][55] + coeff0_38*dmats0[38][55] + coeff0_39*dmats0[39][55] + coeff0_40*dmats0[40][55] + coeff0_41*dmats0[41][55] + coeff0_42*dmats0[42][55] + coeff0_43*dmats0[43][55] + coeff0_44*dmats0[44][55] + coeff0_45*dmats0[45][55] + coeff0_46*dmats0[46][55] + coeff0_47*dmats0[47][55] + coeff0_48*dmats0[48][55] + coeff0_49*dmats0[49][55] + coeff0_50*dmats0[50][55] + coeff0_51*dmats0[51][55] + coeff0_52*dmats0[52][55] + coeff0_53*dmats0[53][55] + coeff0_54*dmats0[54][55] + coeff0_55*dmats0[55][55];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0] + coeff0_10*dmats1[10][0] + coeff0_11*dmats1[11][0] + coeff0_12*dmats1[12][0] + coeff0_13*dmats1[13][0] + coeff0_14*dmats1[14][0] + coeff0_15*dmats1[15][0] + coeff0_16*dmats1[16][0] + coeff0_17*dmats1[17][0] + coeff0_18*dmats1[18][0] + coeff0_19*dmats1[19][0] + coeff0_20*dmats1[20][0] + coeff0_21*dmats1[21][0] + coeff0_22*dmats1[22][0] + coeff0_23*dmats1[23][0] + coeff0_24*dmats1[24][0] + coeff0_25*dmats1[25][0] + coeff0_26*dmats1[26][0] + coeff0_27*dmats1[27][0] + coeff0_28*dmats1[28][0] + coeff0_29*dmats1[29][0] + coeff0_30*dmats1[30][0] + coeff0_31*dmats1[31][0] + coeff0_32*dmats1[32][0] + coeff0_33*dmats1[33][0] + coeff0_34*dmats1[34][0] + coeff0_35*dmats1[35][0] + coeff0_36*dmats1[36][0] + coeff0_37*dmats1[37][0] + coeff0_38*dmats1[38][0] + coeff0_39*dmats1[39][0] + coeff0_40*dmats1[40][0] + coeff0_41*dmats1[41][0] + coeff0_42*dmats1[42][0] + coeff0_43*dmats1[43][0] + coeff0_44*dmats1[44][0] + coeff0_45*dmats1[45][0] + coeff0_46*dmats1[46][0] + coeff0_47*dmats1[47][0] + coeff0_48*dmats1[48][0] + coeff0_49*dmats1[49][0] + coeff0_50*dmats1[50][0] + coeff0_51*dmats1[51][0] + coeff0_52*dmats1[52][0] + coeff0_53*dmats1[53][0] + coeff0_54*dmats1[54][0] + coeff0_55*dmats1[55][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1] + coeff0_10*dmats1[10][1] + coeff0_11*dmats1[11][1] + coeff0_12*dmats1[12][1] + coeff0_13*dmats1[13][1] + coeff0_14*dmats1[14][1] + coeff0_15*dmats1[15][1] + coeff0_16*dmats1[16][1] + coeff0_17*dmats1[17][1] + coeff0_18*dmats1[18][1] + coeff0_19*dmats1[19][1] + coeff0_20*dmats1[20][1] + coeff0_21*dmats1[21][1] + coeff0_22*dmats1[22][1] + coeff0_23*dmats1[23][1] + coeff0_24*dmats1[24][1] + coeff0_25*dmats1[25][1] + coeff0_26*dmats1[26][1] + coeff0_27*dmats1[27][1] + coeff0_28*dmats1[28][1] + coeff0_29*dmats1[29][1] + coeff0_30*dmats1[30][1] + coeff0_31*dmats1[31][1] + coeff0_32*dmats1[32][1] + coeff0_33*dmats1[33][1] + coeff0_34*dmats1[34][1] + coeff0_35*dmats1[35][1] + coeff0_36*dmats1[36][1] + coeff0_37*dmats1[37][1] + coeff0_38*dmats1[38][1] + coeff0_39*dmats1[39][1] + coeff0_40*dmats1[40][1] + coeff0_41*dmats1[41][1] + coeff0_42*dmats1[42][1] + coeff0_43*dmats1[43][1] + coeff0_44*dmats1[44][1] + coeff0_45*dmats1[45][1] + coeff0_46*dmats1[46][1] + coeff0_47*dmats1[47][1] + coeff0_48*dmats1[48][1] + coeff0_49*dmats1[49][1] + coeff0_50*dmats1[50][1] + coeff0_51*dmats1[51][1] + coeff0_52*dmats1[52][1] + coeff0_53*dmats1[53][1] + coeff0_54*dmats1[54][1] + coeff0_55*dmats1[55][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2] + coeff0_10*dmats1[10][2] + coeff0_11*dmats1[11][2] + coeff0_12*dmats1[12][2] + coeff0_13*dmats1[13][2] + coeff0_14*dmats1[14][2] + coeff0_15*dmats1[15][2] + coeff0_16*dmats1[16][2] + coeff0_17*dmats1[17][2] + coeff0_18*dmats1[18][2] + coeff0_19*dmats1[19][2] + coeff0_20*dmats1[20][2] + coeff0_21*dmats1[21][2] + coeff0_22*dmats1[22][2] + coeff0_23*dmats1[23][2] + coeff0_24*dmats1[24][2] + coeff0_25*dmats1[25][2] + coeff0_26*dmats1[26][2] + coeff0_27*dmats1[27][2] + coeff0_28*dmats1[28][2] + coeff0_29*dmats1[29][2] + coeff0_30*dmats1[30][2] + coeff0_31*dmats1[31][2] + coeff0_32*dmats1[32][2] + coeff0_33*dmats1[33][2] + coeff0_34*dmats1[34][2] + coeff0_35*dmats1[35][2] + coeff0_36*dmats1[36][2] + coeff0_37*dmats1[37][2] + coeff0_38*dmats1[38][2] + coeff0_39*dmats1[39][2] + coeff0_40*dmats1[40][2] + coeff0_41*dmats1[41][2] + coeff0_42*dmats1[42][2] + coeff0_43*dmats1[43][2] + coeff0_44*dmats1[44][2] + coeff0_45*dmats1[45][2] + coeff0_46*dmats1[46][2] + coeff0_47*dmats1[47][2] + coeff0_48*dmats1[48][2] + coeff0_49*dmats1[49][2] + coeff0_50*dmats1[50][2] + coeff0_51*dmats1[51][2] + coeff0_52*dmats1[52][2] + coeff0_53*dmats1[53][2] + coeff0_54*dmats1[54][2] + coeff0_55*dmats1[55][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3] + coeff0_10*dmats1[10][3] + coeff0_11*dmats1[11][3] + coeff0_12*dmats1[12][3] + coeff0_13*dmats1[13][3] + coeff0_14*dmats1[14][3] + coeff0_15*dmats1[15][3] + coeff0_16*dmats1[16][3] + coeff0_17*dmats1[17][3] + coeff0_18*dmats1[18][3] + coeff0_19*dmats1[19][3] + coeff0_20*dmats1[20][3] + coeff0_21*dmats1[21][3] + coeff0_22*dmats1[22][3] + coeff0_23*dmats1[23][3] + coeff0_24*dmats1[24][3] + coeff0_25*dmats1[25][3] + coeff0_26*dmats1[26][3] + coeff0_27*dmats1[27][3] + coeff0_28*dmats1[28][3] + coeff0_29*dmats1[29][3] + coeff0_30*dmats1[30][3] + coeff0_31*dmats1[31][3] + coeff0_32*dmats1[32][3] + coeff0_33*dmats1[33][3] + coeff0_34*dmats1[34][3] + coeff0_35*dmats1[35][3] + coeff0_36*dmats1[36][3] + coeff0_37*dmats1[37][3] + coeff0_38*dmats1[38][3] + coeff0_39*dmats1[39][3] + coeff0_40*dmats1[40][3] + coeff0_41*dmats1[41][3] + coeff0_42*dmats1[42][3] + coeff0_43*dmats1[43][3] + coeff0_44*dmats1[44][3] + coeff0_45*dmats1[45][3] + coeff0_46*dmats1[46][3] + coeff0_47*dmats1[47][3] + coeff0_48*dmats1[48][3] + coeff0_49*dmats1[49][3] + coeff0_50*dmats1[50][3] + coeff0_51*dmats1[51][3] + coeff0_52*dmats1[52][3] + coeff0_53*dmats1[53][3] + coeff0_54*dmats1[54][3] + coeff0_55*dmats1[55][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4] + coeff0_10*dmats1[10][4] + coeff0_11*dmats1[11][4] + coeff0_12*dmats1[12][4] + coeff0_13*dmats1[13][4] + coeff0_14*dmats1[14][4] + coeff0_15*dmats1[15][4] + coeff0_16*dmats1[16][4] + coeff0_17*dmats1[17][4] + coeff0_18*dmats1[18][4] + coeff0_19*dmats1[19][4] + coeff0_20*dmats1[20][4] + coeff0_21*dmats1[21][4] + coeff0_22*dmats1[22][4] + coeff0_23*dmats1[23][4] + coeff0_24*dmats1[24][4] + coeff0_25*dmats1[25][4] + coeff0_26*dmats1[26][4] + coeff0_27*dmats1[27][4] + coeff0_28*dmats1[28][4] + coeff0_29*dmats1[29][4] + coeff0_30*dmats1[30][4] + coeff0_31*dmats1[31][4] + coeff0_32*dmats1[32][4] + coeff0_33*dmats1[33][4] + coeff0_34*dmats1[34][4] + coeff0_35*dmats1[35][4] + coeff0_36*dmats1[36][4] + coeff0_37*dmats1[37][4] + coeff0_38*dmats1[38][4] + coeff0_39*dmats1[39][4] + coeff0_40*dmats1[40][4] + coeff0_41*dmats1[41][4] + coeff0_42*dmats1[42][4] + coeff0_43*dmats1[43][4] + coeff0_44*dmats1[44][4] + coeff0_45*dmats1[45][4] + coeff0_46*dmats1[46][4] + coeff0_47*dmats1[47][4] + coeff0_48*dmats1[48][4] + coeff0_49*dmats1[49][4] + coeff0_50*dmats1[50][4] + coeff0_51*dmats1[51][4] + coeff0_52*dmats1[52][4] + coeff0_53*dmats1[53][4] + coeff0_54*dmats1[54][4] + coeff0_55*dmats1[55][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5] + coeff0_10*dmats1[10][5] + coeff0_11*dmats1[11][5] + coeff0_12*dmats1[12][5] + coeff0_13*dmats1[13][5] + coeff0_14*dmats1[14][5] + coeff0_15*dmats1[15][5] + coeff0_16*dmats1[16][5] + coeff0_17*dmats1[17][5] + coeff0_18*dmats1[18][5] + coeff0_19*dmats1[19][5] + coeff0_20*dmats1[20][5] + coeff0_21*dmats1[21][5] + coeff0_22*dmats1[22][5] + coeff0_23*dmats1[23][5] + coeff0_24*dmats1[24][5] + coeff0_25*dmats1[25][5] + coeff0_26*dmats1[26][5] + coeff0_27*dmats1[27][5] + coeff0_28*dmats1[28][5] + coeff0_29*dmats1[29][5] + coeff0_30*dmats1[30][5] + coeff0_31*dmats1[31][5] + coeff0_32*dmats1[32][5] + coeff0_33*dmats1[33][5] + coeff0_34*dmats1[34][5] + coeff0_35*dmats1[35][5] + coeff0_36*dmats1[36][5] + coeff0_37*dmats1[37][5] + coeff0_38*dmats1[38][5] + coeff0_39*dmats1[39][5] + coeff0_40*dmats1[40][5] + coeff0_41*dmats1[41][5] + coeff0_42*dmats1[42][5] + coeff0_43*dmats1[43][5] + coeff0_44*dmats1[44][5] + coeff0_45*dmats1[45][5] + coeff0_46*dmats1[46][5] + coeff0_47*dmats1[47][5] + coeff0_48*dmats1[48][5] + coeff0_49*dmats1[49][5] + coeff0_50*dmats1[50][5] + coeff0_51*dmats1[51][5] + coeff0_52*dmats1[52][5] + coeff0_53*dmats1[53][5] + coeff0_54*dmats1[54][5] + coeff0_55*dmats1[55][5];
          new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6] + coeff0_10*dmats1[10][6] + coeff0_11*dmats1[11][6] + coeff0_12*dmats1[12][6] + coeff0_13*dmats1[13][6] + coeff0_14*dmats1[14][6] + coeff0_15*dmats1[15][6] + coeff0_16*dmats1[16][6] + coeff0_17*dmats1[17][6] + coeff0_18*dmats1[18][6] + coeff0_19*dmats1[19][6] + coeff0_20*dmats1[20][6] + coeff0_21*dmats1[21][6] + coeff0_22*dmats1[22][6] + coeff0_23*dmats1[23][6] + coeff0_24*dmats1[24][6] + coeff0_25*dmats1[25][6] + coeff0_26*dmats1[26][6] + coeff0_27*dmats1[27][6] + coeff0_28*dmats1[28][6] + coeff0_29*dmats1[29][6] + coeff0_30*dmats1[30][6] + coeff0_31*dmats1[31][6] + coeff0_32*dmats1[32][6] + coeff0_33*dmats1[33][6] + coeff0_34*dmats1[34][6] + coeff0_35*dmats1[35][6] + coeff0_36*dmats1[36][6] + coeff0_37*dmats1[37][6] + coeff0_38*dmats1[38][6] + coeff0_39*dmats1[39][6] + coeff0_40*dmats1[40][6] + coeff0_41*dmats1[41][6] + coeff0_42*dmats1[42][6] + coeff0_43*dmats1[43][6] + coeff0_44*dmats1[44][6] + coeff0_45*dmats1[45][6] + coeff0_46*dmats1[46][6] + coeff0_47*dmats1[47][6] + coeff0_48*dmats1[48][6] + coeff0_49*dmats1[49][6] + coeff0_50*dmats1[50][6] + coeff0_51*dmats1[51][6] + coeff0_52*dmats1[52][6] + coeff0_53*dmats1[53][6] + coeff0_54*dmats1[54][6] + coeff0_55*dmats1[55][6];
          new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7] + coeff0_10*dmats1[10][7] + coeff0_11*dmats1[11][7] + coeff0_12*dmats1[12][7] + coeff0_13*dmats1[13][7] + coeff0_14*dmats1[14][7] + coeff0_15*dmats1[15][7] + coeff0_16*dmats1[16][7] + coeff0_17*dmats1[17][7] + coeff0_18*dmats1[18][7] + coeff0_19*dmats1[19][7] + coeff0_20*dmats1[20][7] + coeff0_21*dmats1[21][7] + coeff0_22*dmats1[22][7] + coeff0_23*dmats1[23][7] + coeff0_24*dmats1[24][7] + coeff0_25*dmats1[25][7] + coeff0_26*dmats1[26][7] + coeff0_27*dmats1[27][7] + coeff0_28*dmats1[28][7] + coeff0_29*dmats1[29][7] + coeff0_30*dmats1[30][7] + coeff0_31*dmats1[31][7] + coeff0_32*dmats1[32][7] + coeff0_33*dmats1[33][7] + coeff0_34*dmats1[34][7] + coeff0_35*dmats1[35][7] + coeff0_36*dmats1[36][7] + coeff0_37*dmats1[37][7] + coeff0_38*dmats1[38][7] + coeff0_39*dmats1[39][7] + coeff0_40*dmats1[40][7] + coeff0_41*dmats1[41][7] + coeff0_42*dmats1[42][7] + coeff0_43*dmats1[43][7] + coeff0_44*dmats1[44][7] + coeff0_45*dmats1[45][7] + coeff0_46*dmats1[46][7] + coeff0_47*dmats1[47][7] + coeff0_48*dmats1[48][7] + coeff0_49*dmats1[49][7] + coeff0_50*dmats1[50][7] + coeff0_51*dmats1[51][7] + coeff0_52*dmats1[52][7] + coeff0_53*dmats1[53][7] + coeff0_54*dmats1[54][7] + coeff0_55*dmats1[55][7];
          new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8] + coeff0_10*dmats1[10][8] + coeff0_11*dmats1[11][8] + coeff0_12*dmats1[12][8] + coeff0_13*dmats1[13][8] + coeff0_14*dmats1[14][8] + coeff0_15*dmats1[15][8] + coeff0_16*dmats1[16][8] + coeff0_17*dmats1[17][8] + coeff0_18*dmats1[18][8] + coeff0_19*dmats1[19][8] + coeff0_20*dmats1[20][8] + coeff0_21*dmats1[21][8] + coeff0_22*dmats1[22][8] + coeff0_23*dmats1[23][8] + coeff0_24*dmats1[24][8] + coeff0_25*dmats1[25][8] + coeff0_26*dmats1[26][8] + coeff0_27*dmats1[27][8] + coeff0_28*dmats1[28][8] + coeff0_29*dmats1[29][8] + coeff0_30*dmats1[30][8] + coeff0_31*dmats1[31][8] + coeff0_32*dmats1[32][8] + coeff0_33*dmats1[33][8] + coeff0_34*dmats1[34][8] + coeff0_35*dmats1[35][8] + coeff0_36*dmats1[36][8] + coeff0_37*dmats1[37][8] + coeff0_38*dmats1[38][8] + coeff0_39*dmats1[39][8] + coeff0_40*dmats1[40][8] + coeff0_41*dmats1[41][8] + coeff0_42*dmats1[42][8] + coeff0_43*dmats1[43][8] + coeff0_44*dmats1[44][8] + coeff0_45*dmats1[45][8] + coeff0_46*dmats1[46][8] + coeff0_47*dmats1[47][8] + coeff0_48*dmats1[48][8] + coeff0_49*dmats1[49][8] + coeff0_50*dmats1[50][8] + coeff0_51*dmats1[51][8] + coeff0_52*dmats1[52][8] + coeff0_53*dmats1[53][8] + coeff0_54*dmats1[54][8] + coeff0_55*dmats1[55][8];
          new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9] + coeff0_10*dmats1[10][9] + coeff0_11*dmats1[11][9] + coeff0_12*dmats1[12][9] + coeff0_13*dmats1[13][9] + coeff0_14*dmats1[14][9] + coeff0_15*dmats1[15][9] + coeff0_16*dmats1[16][9] + coeff0_17*dmats1[17][9] + coeff0_18*dmats1[18][9] + coeff0_19*dmats1[19][9] + coeff0_20*dmats1[20][9] + coeff0_21*dmats1[21][9] + coeff0_22*dmats1[22][9] + coeff0_23*dmats1[23][9] + coeff0_24*dmats1[24][9] + coeff0_25*dmats1[25][9] + coeff0_26*dmats1[26][9] + coeff0_27*dmats1[27][9] + coeff0_28*dmats1[28][9] + coeff0_29*dmats1[29][9] + coeff0_30*dmats1[30][9] + coeff0_31*dmats1[31][9] + coeff0_32*dmats1[32][9] + coeff0_33*dmats1[33][9] + coeff0_34*dmats1[34][9] + coeff0_35*dmats1[35][9] + coeff0_36*dmats1[36][9] + coeff0_37*dmats1[37][9] + coeff0_38*dmats1[38][9] + coeff0_39*dmats1[39][9] + coeff0_40*dmats1[40][9] + coeff0_41*dmats1[41][9] + coeff0_42*dmats1[42][9] + coeff0_43*dmats1[43][9] + coeff0_44*dmats1[44][9] + coeff0_45*dmats1[45][9] + coeff0_46*dmats1[46][9] + coeff0_47*dmats1[47][9] + coeff0_48*dmats1[48][9] + coeff0_49*dmats1[49][9] + coeff0_50*dmats1[50][9] + coeff0_51*dmats1[51][9] + coeff0_52*dmats1[52][9] + coeff0_53*dmats1[53][9] + coeff0_54*dmats1[54][9] + coeff0_55*dmats1[55][9];
          new_coeff0_10 = coeff0_0*dmats1[0][10] + coeff0_1*dmats1[1][10] + coeff0_2*dmats1[2][10] + coeff0_3*dmats1[3][10] + coeff0_4*dmats1[4][10] + coeff0_5*dmats1[5][10] + coeff0_6*dmats1[6][10] + coeff0_7*dmats1[7][10] + coeff0_8*dmats1[8][10] + coeff0_9*dmats1[9][10] + coeff0_10*dmats1[10][10] + coeff0_11*dmats1[11][10] + coeff0_12*dmats1[12][10] + coeff0_13*dmats1[13][10] + coeff0_14*dmats1[14][10] + coeff0_15*dmats1[15][10] + coeff0_16*dmats1[16][10] + coeff0_17*dmats1[17][10] + coeff0_18*dmats1[18][10] + coeff0_19*dmats1[19][10] + coeff0_20*dmats1[20][10] + coeff0_21*dmats1[21][10] + coeff0_22*dmats1[22][10] + coeff0_23*dmats1[23][10] + coeff0_24*dmats1[24][10] + coeff0_25*dmats1[25][10] + coeff0_26*dmats1[26][10] + coeff0_27*dmats1[27][10] + coeff0_28*dmats1[28][10] + coeff0_29*dmats1[29][10] + coeff0_30*dmats1[30][10] + coeff0_31*dmats1[31][10] + coeff0_32*dmats1[32][10] + coeff0_33*dmats1[33][10] + coeff0_34*dmats1[34][10] + coeff0_35*dmats1[35][10] + coeff0_36*dmats1[36][10] + coeff0_37*dmats1[37][10] + coeff0_38*dmats1[38][10] + coeff0_39*dmats1[39][10] + coeff0_40*dmats1[40][10] + coeff0_41*dmats1[41][10] + coeff0_42*dmats1[42][10] + coeff0_43*dmats1[43][10] + coeff0_44*dmats1[44][10] + coeff0_45*dmats1[45][10] + coeff0_46*dmats1[46][10] + coeff0_47*dmats1[47][10] + coeff0_48*dmats1[48][10] + coeff0_49*dmats1[49][10] + coeff0_50*dmats1[50][10] + coeff0_51*dmats1[51][10] + coeff0_52*dmats1[52][10] + coeff0_53*dmats1[53][10] + coeff0_54*dmats1[54][10] + coeff0_55*dmats1[55][10];
          new_coeff0_11 = coeff0_0*dmats1[0][11] + coeff0_1*dmats1[1][11] + coeff0_2*dmats1[2][11] + coeff0_3*dmats1[3][11] + coeff0_4*dmats1[4][11] + coeff0_5*dmats1[5][11] + coeff0_6*dmats1[6][11] + coeff0_7*dmats1[7][11] + coeff0_8*dmats1[8][11] + coeff0_9*dmats1[9][11] + coeff0_10*dmats1[10][11] + coeff0_11*dmats1[11][11] + coeff0_12*dmats1[12][11] + coeff0_13*dmats1[13][11] + coeff0_14*dmats1[14][11] + coeff0_15*dmats1[15][11] + coeff0_16*dmats1[16][11] + coeff0_17*dmats1[17][11] + coeff0_18*dmats1[18][11] + coeff0_19*dmats1[19][11] + coeff0_20*dmats1[20][11] + coeff0_21*dmats1[21][11] + coeff0_22*dmats1[22][11] + coeff0_23*dmats1[23][11] + coeff0_24*dmats1[24][11] + coeff0_25*dmats1[25][11] + coeff0_26*dmats1[26][11] + coeff0_27*dmats1[27][11] + coeff0_28*dmats1[28][11] + coeff0_29*dmats1[29][11] + coeff0_30*dmats1[30][11] + coeff0_31*dmats1[31][11] + coeff0_32*dmats1[32][11] + coeff0_33*dmats1[33][11] + coeff0_34*dmats1[34][11] + coeff0_35*dmats1[35][11] + coeff0_36*dmats1[36][11] + coeff0_37*dmats1[37][11] + coeff0_38*dmats1[38][11] + coeff0_39*dmats1[39][11] + coeff0_40*dmats1[40][11] + coeff0_41*dmats1[41][11] + coeff0_42*dmats1[42][11] + coeff0_43*dmats1[43][11] + coeff0_44*dmats1[44][11] + coeff0_45*dmats1[45][11] + coeff0_46*dmats1[46][11] + coeff0_47*dmats1[47][11] + coeff0_48*dmats1[48][11] + coeff0_49*dmats1[49][11] + coeff0_50*dmats1[50][11] + coeff0_51*dmats1[51][11] + coeff0_52*dmats1[52][11] + coeff0_53*dmats1[53][11] + coeff0_54*dmats1[54][11] + coeff0_55*dmats1[55][11];
          new_coeff0_12 = coeff0_0*dmats1[0][12] + coeff0_1*dmats1[1][12] + coeff0_2*dmats1[2][12] + coeff0_3*dmats1[3][12] + coeff0_4*dmats1[4][12] + coeff0_5*dmats1[5][12] + coeff0_6*dmats1[6][12] + coeff0_7*dmats1[7][12] + coeff0_8*dmats1[8][12] + coeff0_9*dmats1[9][12] + coeff0_10*dmats1[10][12] + coeff0_11*dmats1[11][12] + coeff0_12*dmats1[12][12] + coeff0_13*dmats1[13][12] + coeff0_14*dmats1[14][12] + coeff0_15*dmats1[15][12] + coeff0_16*dmats1[16][12] + coeff0_17*dmats1[17][12] + coeff0_18*dmats1[18][12] + coeff0_19*dmats1[19][12] + coeff0_20*dmats1[20][12] + coeff0_21*dmats1[21][12] + coeff0_22*dmats1[22][12] + coeff0_23*dmats1[23][12] + coeff0_24*dmats1[24][12] + coeff0_25*dmats1[25][12] + coeff0_26*dmats1[26][12] + coeff0_27*dmats1[27][12] + coeff0_28*dmats1[28][12] + coeff0_29*dmats1[29][12] + coeff0_30*dmats1[30][12] + coeff0_31*dmats1[31][12] + coeff0_32*dmats1[32][12] + coeff0_33*dmats1[33][12] + coeff0_34*dmats1[34][12] + coeff0_35*dmats1[35][12] + coeff0_36*dmats1[36][12] + coeff0_37*dmats1[37][12] + coeff0_38*dmats1[38][12] + coeff0_39*dmats1[39][12] + coeff0_40*dmats1[40][12] + coeff0_41*dmats1[41][12] + coeff0_42*dmats1[42][12] + coeff0_43*dmats1[43][12] + coeff0_44*dmats1[44][12] + coeff0_45*dmats1[45][12] + coeff0_46*dmats1[46][12] + coeff0_47*dmats1[47][12] + coeff0_48*dmats1[48][12] + coeff0_49*dmats1[49][12] + coeff0_50*dmats1[50][12] + coeff0_51*dmats1[51][12] + coeff0_52*dmats1[52][12] + coeff0_53*dmats1[53][12] + coeff0_54*dmats1[54][12] + coeff0_55*dmats1[55][12];
          new_coeff0_13 = coeff0_0*dmats1[0][13] + coeff0_1*dmats1[1][13] + coeff0_2*dmats1[2][13] + coeff0_3*dmats1[3][13] + coeff0_4*dmats1[4][13] + coeff0_5*dmats1[5][13] + coeff0_6*dmats1[6][13] + coeff0_7*dmats1[7][13] + coeff0_8*dmats1[8][13] + coeff0_9*dmats1[9][13] + coeff0_10*dmats1[10][13] + coeff0_11*dmats1[11][13] + coeff0_12*dmats1[12][13] + coeff0_13*dmats1[13][13] + coeff0_14*dmats1[14][13] + coeff0_15*dmats1[15][13] + coeff0_16*dmats1[16][13] + coeff0_17*dmats1[17][13] + coeff0_18*dmats1[18][13] + coeff0_19*dmats1[19][13] + coeff0_20*dmats1[20][13] + coeff0_21*dmats1[21][13] + coeff0_22*dmats1[22][13] + coeff0_23*dmats1[23][13] + coeff0_24*dmats1[24][13] + coeff0_25*dmats1[25][13] + coeff0_26*dmats1[26][13] + coeff0_27*dmats1[27][13] + coeff0_28*dmats1[28][13] + coeff0_29*dmats1[29][13] + coeff0_30*dmats1[30][13] + coeff0_31*dmats1[31][13] + coeff0_32*dmats1[32][13] + coeff0_33*dmats1[33][13] + coeff0_34*dmats1[34][13] + coeff0_35*dmats1[35][13] + coeff0_36*dmats1[36][13] + coeff0_37*dmats1[37][13] + coeff0_38*dmats1[38][13] + coeff0_39*dmats1[39][13] + coeff0_40*dmats1[40][13] + coeff0_41*dmats1[41][13] + coeff0_42*dmats1[42][13] + coeff0_43*dmats1[43][13] + coeff0_44*dmats1[44][13] + coeff0_45*dmats1[45][13] + coeff0_46*dmats1[46][13] + coeff0_47*dmats1[47][13] + coeff0_48*dmats1[48][13] + coeff0_49*dmats1[49][13] + coeff0_50*dmats1[50][13] + coeff0_51*dmats1[51][13] + coeff0_52*dmats1[52][13] + coeff0_53*dmats1[53][13] + coeff0_54*dmats1[54][13] + coeff0_55*dmats1[55][13];
          new_coeff0_14 = coeff0_0*dmats1[0][14] + coeff0_1*dmats1[1][14] + coeff0_2*dmats1[2][14] + coeff0_3*dmats1[3][14] + coeff0_4*dmats1[4][14] + coeff0_5*dmats1[5][14] + coeff0_6*dmats1[6][14] + coeff0_7*dmats1[7][14] + coeff0_8*dmats1[8][14] + coeff0_9*dmats1[9][14] + coeff0_10*dmats1[10][14] + coeff0_11*dmats1[11][14] + coeff0_12*dmats1[12][14] + coeff0_13*dmats1[13][14] + coeff0_14*dmats1[14][14] + coeff0_15*dmats1[15][14] + coeff0_16*dmats1[16][14] + coeff0_17*dmats1[17][14] + coeff0_18*dmats1[18][14] + coeff0_19*dmats1[19][14] + coeff0_20*dmats1[20][14] + coeff0_21*dmats1[21][14] + coeff0_22*dmats1[22][14] + coeff0_23*dmats1[23][14] + coeff0_24*dmats1[24][14] + coeff0_25*dmats1[25][14] + coeff0_26*dmats1[26][14] + coeff0_27*dmats1[27][14] + coeff0_28*dmats1[28][14] + coeff0_29*dmats1[29][14] + coeff0_30*dmats1[30][14] + coeff0_31*dmats1[31][14] + coeff0_32*dmats1[32][14] + coeff0_33*dmats1[33][14] + coeff0_34*dmats1[34][14] + coeff0_35*dmats1[35][14] + coeff0_36*dmats1[36][14] + coeff0_37*dmats1[37][14] + coeff0_38*dmats1[38][14] + coeff0_39*dmats1[39][14] + coeff0_40*dmats1[40][14] + coeff0_41*dmats1[41][14] + coeff0_42*dmats1[42][14] + coeff0_43*dmats1[43][14] + coeff0_44*dmats1[44][14] + coeff0_45*dmats1[45][14] + coeff0_46*dmats1[46][14] + coeff0_47*dmats1[47][14] + coeff0_48*dmats1[48][14] + coeff0_49*dmats1[49][14] + coeff0_50*dmats1[50][14] + coeff0_51*dmats1[51][14] + coeff0_52*dmats1[52][14] + coeff0_53*dmats1[53][14] + coeff0_54*dmats1[54][14] + coeff0_55*dmats1[55][14];
          new_coeff0_15 = coeff0_0*dmats1[0][15] + coeff0_1*dmats1[1][15] + coeff0_2*dmats1[2][15] + coeff0_3*dmats1[3][15] + coeff0_4*dmats1[4][15] + coeff0_5*dmats1[5][15] + coeff0_6*dmats1[6][15] + coeff0_7*dmats1[7][15] + coeff0_8*dmats1[8][15] + coeff0_9*dmats1[9][15] + coeff0_10*dmats1[10][15] + coeff0_11*dmats1[11][15] + coeff0_12*dmats1[12][15] + coeff0_13*dmats1[13][15] + coeff0_14*dmats1[14][15] + coeff0_15*dmats1[15][15] + coeff0_16*dmats1[16][15] + coeff0_17*dmats1[17][15] + coeff0_18*dmats1[18][15] + coeff0_19*dmats1[19][15] + coeff0_20*dmats1[20][15] + coeff0_21*dmats1[21][15] + coeff0_22*dmats1[22][15] + coeff0_23*dmats1[23][15] + coeff0_24*dmats1[24][15] + coeff0_25*dmats1[25][15] + coeff0_26*dmats1[26][15] + coeff0_27*dmats1[27][15] + coeff0_28*dmats1[28][15] + coeff0_29*dmats1[29][15] + coeff0_30*dmats1[30][15] + coeff0_31*dmats1[31][15] + coeff0_32*dmats1[32][15] + coeff0_33*dmats1[33][15] + coeff0_34*dmats1[34][15] + coeff0_35*dmats1[35][15] + coeff0_36*dmats1[36][15] + coeff0_37*dmats1[37][15] + coeff0_38*dmats1[38][15] + coeff0_39*dmats1[39][15] + coeff0_40*dmats1[40][15] + coeff0_41*dmats1[41][15] + coeff0_42*dmats1[42][15] + coeff0_43*dmats1[43][15] + coeff0_44*dmats1[44][15] + coeff0_45*dmats1[45][15] + coeff0_46*dmats1[46][15] + coeff0_47*dmats1[47][15] + coeff0_48*dmats1[48][15] + coeff0_49*dmats1[49][15] + coeff0_50*dmats1[50][15] + coeff0_51*dmats1[51][15] + coeff0_52*dmats1[52][15] + coeff0_53*dmats1[53][15] + coeff0_54*dmats1[54][15] + coeff0_55*dmats1[55][15];
          new_coeff0_16 = coeff0_0*dmats1[0][16] + coeff0_1*dmats1[1][16] + coeff0_2*dmats1[2][16] + coeff0_3*dmats1[3][16] + coeff0_4*dmats1[4][16] + coeff0_5*dmats1[5][16] + coeff0_6*dmats1[6][16] + coeff0_7*dmats1[7][16] + coeff0_8*dmats1[8][16] + coeff0_9*dmats1[9][16] + coeff0_10*dmats1[10][16] + coeff0_11*dmats1[11][16] + coeff0_12*dmats1[12][16] + coeff0_13*dmats1[13][16] + coeff0_14*dmats1[14][16] + coeff0_15*dmats1[15][16] + coeff0_16*dmats1[16][16] + coeff0_17*dmats1[17][16] + coeff0_18*dmats1[18][16] + coeff0_19*dmats1[19][16] + coeff0_20*dmats1[20][16] + coeff0_21*dmats1[21][16] + coeff0_22*dmats1[22][16] + coeff0_23*dmats1[23][16] + coeff0_24*dmats1[24][16] + coeff0_25*dmats1[25][16] + coeff0_26*dmats1[26][16] + coeff0_27*dmats1[27][16] + coeff0_28*dmats1[28][16] + coeff0_29*dmats1[29][16] + coeff0_30*dmats1[30][16] + coeff0_31*dmats1[31][16] + coeff0_32*dmats1[32][16] + coeff0_33*dmats1[33][16] + coeff0_34*dmats1[34][16] + coeff0_35*dmats1[35][16] + coeff0_36*dmats1[36][16] + coeff0_37*dmats1[37][16] + coeff0_38*dmats1[38][16] + coeff0_39*dmats1[39][16] + coeff0_40*dmats1[40][16] + coeff0_41*dmats1[41][16] + coeff0_42*dmats1[42][16] + coeff0_43*dmats1[43][16] + coeff0_44*dmats1[44][16] + coeff0_45*dmats1[45][16] + coeff0_46*dmats1[46][16] + coeff0_47*dmats1[47][16] + coeff0_48*dmats1[48][16] + coeff0_49*dmats1[49][16] + coeff0_50*dmats1[50][16] + coeff0_51*dmats1[51][16] + coeff0_52*dmats1[52][16] + coeff0_53*dmats1[53][16] + coeff0_54*dmats1[54][16] + coeff0_55*dmats1[55][16];
          new_coeff0_17 = coeff0_0*dmats1[0][17] + coeff0_1*dmats1[1][17] + coeff0_2*dmats1[2][17] + coeff0_3*dmats1[3][17] + coeff0_4*dmats1[4][17] + coeff0_5*dmats1[5][17] + coeff0_6*dmats1[6][17] + coeff0_7*dmats1[7][17] + coeff0_8*dmats1[8][17] + coeff0_9*dmats1[9][17] + coeff0_10*dmats1[10][17] + coeff0_11*dmats1[11][17] + coeff0_12*dmats1[12][17] + coeff0_13*dmats1[13][17] + coeff0_14*dmats1[14][17] + coeff0_15*dmats1[15][17] + coeff0_16*dmats1[16][17] + coeff0_17*dmats1[17][17] + coeff0_18*dmats1[18][17] + coeff0_19*dmats1[19][17] + coeff0_20*dmats1[20][17] + coeff0_21*dmats1[21][17] + coeff0_22*dmats1[22][17] + coeff0_23*dmats1[23][17] + coeff0_24*dmats1[24][17] + coeff0_25*dmats1[25][17] + coeff0_26*dmats1[26][17] + coeff0_27*dmats1[27][17] + coeff0_28*dmats1[28][17] + coeff0_29*dmats1[29][17] + coeff0_30*dmats1[30][17] + coeff0_31*dmats1[31][17] + coeff0_32*dmats1[32][17] + coeff0_33*dmats1[33][17] + coeff0_34*dmats1[34][17] + coeff0_35*dmats1[35][17] + coeff0_36*dmats1[36][17] + coeff0_37*dmats1[37][17] + coeff0_38*dmats1[38][17] + coeff0_39*dmats1[39][17] + coeff0_40*dmats1[40][17] + coeff0_41*dmats1[41][17] + coeff0_42*dmats1[42][17] + coeff0_43*dmats1[43][17] + coeff0_44*dmats1[44][17] + coeff0_45*dmats1[45][17] + coeff0_46*dmats1[46][17] + coeff0_47*dmats1[47][17] + coeff0_48*dmats1[48][17] + coeff0_49*dmats1[49][17] + coeff0_50*dmats1[50][17] + coeff0_51*dmats1[51][17] + coeff0_52*dmats1[52][17] + coeff0_53*dmats1[53][17] + coeff0_54*dmats1[54][17] + coeff0_55*dmats1[55][17];
          new_coeff0_18 = coeff0_0*dmats1[0][18] + coeff0_1*dmats1[1][18] + coeff0_2*dmats1[2][18] + coeff0_3*dmats1[3][18] + coeff0_4*dmats1[4][18] + coeff0_5*dmats1[5][18] + coeff0_6*dmats1[6][18] + coeff0_7*dmats1[7][18] + coeff0_8*dmats1[8][18] + coeff0_9*dmats1[9][18] + coeff0_10*dmats1[10][18] + coeff0_11*dmats1[11][18] + coeff0_12*dmats1[12][18] + coeff0_13*dmats1[13][18] + coeff0_14*dmats1[14][18] + coeff0_15*dmats1[15][18] + coeff0_16*dmats1[16][18] + coeff0_17*dmats1[17][18] + coeff0_18*dmats1[18][18] + coeff0_19*dmats1[19][18] + coeff0_20*dmats1[20][18] + coeff0_21*dmats1[21][18] + coeff0_22*dmats1[22][18] + coeff0_23*dmats1[23][18] + coeff0_24*dmats1[24][18] + coeff0_25*dmats1[25][18] + coeff0_26*dmats1[26][18] + coeff0_27*dmats1[27][18] + coeff0_28*dmats1[28][18] + coeff0_29*dmats1[29][18] + coeff0_30*dmats1[30][18] + coeff0_31*dmats1[31][18] + coeff0_32*dmats1[32][18] + coeff0_33*dmats1[33][18] + coeff0_34*dmats1[34][18] + coeff0_35*dmats1[35][18] + coeff0_36*dmats1[36][18] + coeff0_37*dmats1[37][18] + coeff0_38*dmats1[38][18] + coeff0_39*dmats1[39][18] + coeff0_40*dmats1[40][18] + coeff0_41*dmats1[41][18] + coeff0_42*dmats1[42][18] + coeff0_43*dmats1[43][18] + coeff0_44*dmats1[44][18] + coeff0_45*dmats1[45][18] + coeff0_46*dmats1[46][18] + coeff0_47*dmats1[47][18] + coeff0_48*dmats1[48][18] + coeff0_49*dmats1[49][18] + coeff0_50*dmats1[50][18] + coeff0_51*dmats1[51][18] + coeff0_52*dmats1[52][18] + coeff0_53*dmats1[53][18] + coeff0_54*dmats1[54][18] + coeff0_55*dmats1[55][18];
          new_coeff0_19 = coeff0_0*dmats1[0][19] + coeff0_1*dmats1[1][19] + coeff0_2*dmats1[2][19] + coeff0_3*dmats1[3][19] + coeff0_4*dmats1[4][19] + coeff0_5*dmats1[5][19] + coeff0_6*dmats1[6][19] + coeff0_7*dmats1[7][19] + coeff0_8*dmats1[8][19] + coeff0_9*dmats1[9][19] + coeff0_10*dmats1[10][19] + coeff0_11*dmats1[11][19] + coeff0_12*dmats1[12][19] + coeff0_13*dmats1[13][19] + coeff0_14*dmats1[14][19] + coeff0_15*dmats1[15][19] + coeff0_16*dmats1[16][19] + coeff0_17*dmats1[17][19] + coeff0_18*dmats1[18][19] + coeff0_19*dmats1[19][19] + coeff0_20*dmats1[20][19] + coeff0_21*dmats1[21][19] + coeff0_22*dmats1[22][19] + coeff0_23*dmats1[23][19] + coeff0_24*dmats1[24][19] + coeff0_25*dmats1[25][19] + coeff0_26*dmats1[26][19] + coeff0_27*dmats1[27][19] + coeff0_28*dmats1[28][19] + coeff0_29*dmats1[29][19] + coeff0_30*dmats1[30][19] + coeff0_31*dmats1[31][19] + coeff0_32*dmats1[32][19] + coeff0_33*dmats1[33][19] + coeff0_34*dmats1[34][19] + coeff0_35*dmats1[35][19] + coeff0_36*dmats1[36][19] + coeff0_37*dmats1[37][19] + coeff0_38*dmats1[38][19] + coeff0_39*dmats1[39][19] + coeff0_40*dmats1[40][19] + coeff0_41*dmats1[41][19] + coeff0_42*dmats1[42][19] + coeff0_43*dmats1[43][19] + coeff0_44*dmats1[44][19] + coeff0_45*dmats1[45][19] + coeff0_46*dmats1[46][19] + coeff0_47*dmats1[47][19] + coeff0_48*dmats1[48][19] + coeff0_49*dmats1[49][19] + coeff0_50*dmats1[50][19] + coeff0_51*dmats1[51][19] + coeff0_52*dmats1[52][19] + coeff0_53*dmats1[53][19] + coeff0_54*dmats1[54][19] + coeff0_55*dmats1[55][19];
          new_coeff0_20 = coeff0_0*dmats1[0][20] + coeff0_1*dmats1[1][20] + coeff0_2*dmats1[2][20] + coeff0_3*dmats1[3][20] + coeff0_4*dmats1[4][20] + coeff0_5*dmats1[5][20] + coeff0_6*dmats1[6][20] + coeff0_7*dmats1[7][20] + coeff0_8*dmats1[8][20] + coeff0_9*dmats1[9][20] + coeff0_10*dmats1[10][20] + coeff0_11*dmats1[11][20] + coeff0_12*dmats1[12][20] + coeff0_13*dmats1[13][20] + coeff0_14*dmats1[14][20] + coeff0_15*dmats1[15][20] + coeff0_16*dmats1[16][20] + coeff0_17*dmats1[17][20] + coeff0_18*dmats1[18][20] + coeff0_19*dmats1[19][20] + coeff0_20*dmats1[20][20] + coeff0_21*dmats1[21][20] + coeff0_22*dmats1[22][20] + coeff0_23*dmats1[23][20] + coeff0_24*dmats1[24][20] + coeff0_25*dmats1[25][20] + coeff0_26*dmats1[26][20] + coeff0_27*dmats1[27][20] + coeff0_28*dmats1[28][20] + coeff0_29*dmats1[29][20] + coeff0_30*dmats1[30][20] + coeff0_31*dmats1[31][20] + coeff0_32*dmats1[32][20] + coeff0_33*dmats1[33][20] + coeff0_34*dmats1[34][20] + coeff0_35*dmats1[35][20] + coeff0_36*dmats1[36][20] + coeff0_37*dmats1[37][20] + coeff0_38*dmats1[38][20] + coeff0_39*dmats1[39][20] + coeff0_40*dmats1[40][20] + coeff0_41*dmats1[41][20] + coeff0_42*dmats1[42][20] + coeff0_43*dmats1[43][20] + coeff0_44*dmats1[44][20] + coeff0_45*dmats1[45][20] + coeff0_46*dmats1[46][20] + coeff0_47*dmats1[47][20] + coeff0_48*dmats1[48][20] + coeff0_49*dmats1[49][20] + coeff0_50*dmats1[50][20] + coeff0_51*dmats1[51][20] + coeff0_52*dmats1[52][20] + coeff0_53*dmats1[53][20] + coeff0_54*dmats1[54][20] + coeff0_55*dmats1[55][20];
          new_coeff0_21 = coeff0_0*dmats1[0][21] + coeff0_1*dmats1[1][21] + coeff0_2*dmats1[2][21] + coeff0_3*dmats1[3][21] + coeff0_4*dmats1[4][21] + coeff0_5*dmats1[5][21] + coeff0_6*dmats1[6][21] + coeff0_7*dmats1[7][21] + coeff0_8*dmats1[8][21] + coeff0_9*dmats1[9][21] + coeff0_10*dmats1[10][21] + coeff0_11*dmats1[11][21] + coeff0_12*dmats1[12][21] + coeff0_13*dmats1[13][21] + coeff0_14*dmats1[14][21] + coeff0_15*dmats1[15][21] + coeff0_16*dmats1[16][21] + coeff0_17*dmats1[17][21] + coeff0_18*dmats1[18][21] + coeff0_19*dmats1[19][21] + coeff0_20*dmats1[20][21] + coeff0_21*dmats1[21][21] + coeff0_22*dmats1[22][21] + coeff0_23*dmats1[23][21] + coeff0_24*dmats1[24][21] + coeff0_25*dmats1[25][21] + coeff0_26*dmats1[26][21] + coeff0_27*dmats1[27][21] + coeff0_28*dmats1[28][21] + coeff0_29*dmats1[29][21] + coeff0_30*dmats1[30][21] + coeff0_31*dmats1[31][21] + coeff0_32*dmats1[32][21] + coeff0_33*dmats1[33][21] + coeff0_34*dmats1[34][21] + coeff0_35*dmats1[35][21] + coeff0_36*dmats1[36][21] + coeff0_37*dmats1[37][21] + coeff0_38*dmats1[38][21] + coeff0_39*dmats1[39][21] + coeff0_40*dmats1[40][21] + coeff0_41*dmats1[41][21] + coeff0_42*dmats1[42][21] + coeff0_43*dmats1[43][21] + coeff0_44*dmats1[44][21] + coeff0_45*dmats1[45][21] + coeff0_46*dmats1[46][21] + coeff0_47*dmats1[47][21] + coeff0_48*dmats1[48][21] + coeff0_49*dmats1[49][21] + coeff0_50*dmats1[50][21] + coeff0_51*dmats1[51][21] + coeff0_52*dmats1[52][21] + coeff0_53*dmats1[53][21] + coeff0_54*dmats1[54][21] + coeff0_55*dmats1[55][21];
          new_coeff0_22 = coeff0_0*dmats1[0][22] + coeff0_1*dmats1[1][22] + coeff0_2*dmats1[2][22] + coeff0_3*dmats1[3][22] + coeff0_4*dmats1[4][22] + coeff0_5*dmats1[5][22] + coeff0_6*dmats1[6][22] + coeff0_7*dmats1[7][22] + coeff0_8*dmats1[8][22] + coeff0_9*dmats1[9][22] + coeff0_10*dmats1[10][22] + coeff0_11*dmats1[11][22] + coeff0_12*dmats1[12][22] + coeff0_13*dmats1[13][22] + coeff0_14*dmats1[14][22] + coeff0_15*dmats1[15][22] + coeff0_16*dmats1[16][22] + coeff0_17*dmats1[17][22] + coeff0_18*dmats1[18][22] + coeff0_19*dmats1[19][22] + coeff0_20*dmats1[20][22] + coeff0_21*dmats1[21][22] + coeff0_22*dmats1[22][22] + coeff0_23*dmats1[23][22] + coeff0_24*dmats1[24][22] + coeff0_25*dmats1[25][22] + coeff0_26*dmats1[26][22] + coeff0_27*dmats1[27][22] + coeff0_28*dmats1[28][22] + coeff0_29*dmats1[29][22] + coeff0_30*dmats1[30][22] + coeff0_31*dmats1[31][22] + coeff0_32*dmats1[32][22] + coeff0_33*dmats1[33][22] + coeff0_34*dmats1[34][22] + coeff0_35*dmats1[35][22] + coeff0_36*dmats1[36][22] + coeff0_37*dmats1[37][22] + coeff0_38*dmats1[38][22] + coeff0_39*dmats1[39][22] + coeff0_40*dmats1[40][22] + coeff0_41*dmats1[41][22] + coeff0_42*dmats1[42][22] + coeff0_43*dmats1[43][22] + coeff0_44*dmats1[44][22] + coeff0_45*dmats1[45][22] + coeff0_46*dmats1[46][22] + coeff0_47*dmats1[47][22] + coeff0_48*dmats1[48][22] + coeff0_49*dmats1[49][22] + coeff0_50*dmats1[50][22] + coeff0_51*dmats1[51][22] + coeff0_52*dmats1[52][22] + coeff0_53*dmats1[53][22] + coeff0_54*dmats1[54][22] + coeff0_55*dmats1[55][22];
          new_coeff0_23 = coeff0_0*dmats1[0][23] + coeff0_1*dmats1[1][23] + coeff0_2*dmats1[2][23] + coeff0_3*dmats1[3][23] + coeff0_4*dmats1[4][23] + coeff0_5*dmats1[5][23] + coeff0_6*dmats1[6][23] + coeff0_7*dmats1[7][23] + coeff0_8*dmats1[8][23] + coeff0_9*dmats1[9][23] + coeff0_10*dmats1[10][23] + coeff0_11*dmats1[11][23] + coeff0_12*dmats1[12][23] + coeff0_13*dmats1[13][23] + coeff0_14*dmats1[14][23] + coeff0_15*dmats1[15][23] + coeff0_16*dmats1[16][23] + coeff0_17*dmats1[17][23] + coeff0_18*dmats1[18][23] + coeff0_19*dmats1[19][23] + coeff0_20*dmats1[20][23] + coeff0_21*dmats1[21][23] + coeff0_22*dmats1[22][23] + coeff0_23*dmats1[23][23] + coeff0_24*dmats1[24][23] + coeff0_25*dmats1[25][23] + coeff0_26*dmats1[26][23] + coeff0_27*dmats1[27][23] + coeff0_28*dmats1[28][23] + coeff0_29*dmats1[29][23] + coeff0_30*dmats1[30][23] + coeff0_31*dmats1[31][23] + coeff0_32*dmats1[32][23] + coeff0_33*dmats1[33][23] + coeff0_34*dmats1[34][23] + coeff0_35*dmats1[35][23] + coeff0_36*dmats1[36][23] + coeff0_37*dmats1[37][23] + coeff0_38*dmats1[38][23] + coeff0_39*dmats1[39][23] + coeff0_40*dmats1[40][23] + coeff0_41*dmats1[41][23] + coeff0_42*dmats1[42][23] + coeff0_43*dmats1[43][23] + coeff0_44*dmats1[44][23] + coeff0_45*dmats1[45][23] + coeff0_46*dmats1[46][23] + coeff0_47*dmats1[47][23] + coeff0_48*dmats1[48][23] + coeff0_49*dmats1[49][23] + coeff0_50*dmats1[50][23] + coeff0_51*dmats1[51][23] + coeff0_52*dmats1[52][23] + coeff0_53*dmats1[53][23] + coeff0_54*dmats1[54][23] + coeff0_55*dmats1[55][23];
          new_coeff0_24 = coeff0_0*dmats1[0][24] + coeff0_1*dmats1[1][24] + coeff0_2*dmats1[2][24] + coeff0_3*dmats1[3][24] + coeff0_4*dmats1[4][24] + coeff0_5*dmats1[5][24] + coeff0_6*dmats1[6][24] + coeff0_7*dmats1[7][24] + coeff0_8*dmats1[8][24] + coeff0_9*dmats1[9][24] + coeff0_10*dmats1[10][24] + coeff0_11*dmats1[11][24] + coeff0_12*dmats1[12][24] + coeff0_13*dmats1[13][24] + coeff0_14*dmats1[14][24] + coeff0_15*dmats1[15][24] + coeff0_16*dmats1[16][24] + coeff0_17*dmats1[17][24] + coeff0_18*dmats1[18][24] + coeff0_19*dmats1[19][24] + coeff0_20*dmats1[20][24] + coeff0_21*dmats1[21][24] + coeff0_22*dmats1[22][24] + coeff0_23*dmats1[23][24] + coeff0_24*dmats1[24][24] + coeff0_25*dmats1[25][24] + coeff0_26*dmats1[26][24] + coeff0_27*dmats1[27][24] + coeff0_28*dmats1[28][24] + coeff0_29*dmats1[29][24] + coeff0_30*dmats1[30][24] + coeff0_31*dmats1[31][24] + coeff0_32*dmats1[32][24] + coeff0_33*dmats1[33][24] + coeff0_34*dmats1[34][24] + coeff0_35*dmats1[35][24] + coeff0_36*dmats1[36][24] + coeff0_37*dmats1[37][24] + coeff0_38*dmats1[38][24] + coeff0_39*dmats1[39][24] + coeff0_40*dmats1[40][24] + coeff0_41*dmats1[41][24] + coeff0_42*dmats1[42][24] + coeff0_43*dmats1[43][24] + coeff0_44*dmats1[44][24] + coeff0_45*dmats1[45][24] + coeff0_46*dmats1[46][24] + coeff0_47*dmats1[47][24] + coeff0_48*dmats1[48][24] + coeff0_49*dmats1[49][24] + coeff0_50*dmats1[50][24] + coeff0_51*dmats1[51][24] + coeff0_52*dmats1[52][24] + coeff0_53*dmats1[53][24] + coeff0_54*dmats1[54][24] + coeff0_55*dmats1[55][24];
          new_coeff0_25 = coeff0_0*dmats1[0][25] + coeff0_1*dmats1[1][25] + coeff0_2*dmats1[2][25] + coeff0_3*dmats1[3][25] + coeff0_4*dmats1[4][25] + coeff0_5*dmats1[5][25] + coeff0_6*dmats1[6][25] + coeff0_7*dmats1[7][25] + coeff0_8*dmats1[8][25] + coeff0_9*dmats1[9][25] + coeff0_10*dmats1[10][25] + coeff0_11*dmats1[11][25] + coeff0_12*dmats1[12][25] + coeff0_13*dmats1[13][25] + coeff0_14*dmats1[14][25] + coeff0_15*dmats1[15][25] + coeff0_16*dmats1[16][25] + coeff0_17*dmats1[17][25] + coeff0_18*dmats1[18][25] + coeff0_19*dmats1[19][25] + coeff0_20*dmats1[20][25] + coeff0_21*dmats1[21][25] + coeff0_22*dmats1[22][25] + coeff0_23*dmats1[23][25] + coeff0_24*dmats1[24][25] + coeff0_25*dmats1[25][25] + coeff0_26*dmats1[26][25] + coeff0_27*dmats1[27][25] + coeff0_28*dmats1[28][25] + coeff0_29*dmats1[29][25] + coeff0_30*dmats1[30][25] + coeff0_31*dmats1[31][25] + coeff0_32*dmats1[32][25] + coeff0_33*dmats1[33][25] + coeff0_34*dmats1[34][25] + coeff0_35*dmats1[35][25] + coeff0_36*dmats1[36][25] + coeff0_37*dmats1[37][25] + coeff0_38*dmats1[38][25] + coeff0_39*dmats1[39][25] + coeff0_40*dmats1[40][25] + coeff0_41*dmats1[41][25] + coeff0_42*dmats1[42][25] + coeff0_43*dmats1[43][25] + coeff0_44*dmats1[44][25] + coeff0_45*dmats1[45][25] + coeff0_46*dmats1[46][25] + coeff0_47*dmats1[47][25] + coeff0_48*dmats1[48][25] + coeff0_49*dmats1[49][25] + coeff0_50*dmats1[50][25] + coeff0_51*dmats1[51][25] + coeff0_52*dmats1[52][25] + coeff0_53*dmats1[53][25] + coeff0_54*dmats1[54][25] + coeff0_55*dmats1[55][25];
          new_coeff0_26 = coeff0_0*dmats1[0][26] + coeff0_1*dmats1[1][26] + coeff0_2*dmats1[2][26] + coeff0_3*dmats1[3][26] + coeff0_4*dmats1[4][26] + coeff0_5*dmats1[5][26] + coeff0_6*dmats1[6][26] + coeff0_7*dmats1[7][26] + coeff0_8*dmats1[8][26] + coeff0_9*dmats1[9][26] + coeff0_10*dmats1[10][26] + coeff0_11*dmats1[11][26] + coeff0_12*dmats1[12][26] + coeff0_13*dmats1[13][26] + coeff0_14*dmats1[14][26] + coeff0_15*dmats1[15][26] + coeff0_16*dmats1[16][26] + coeff0_17*dmats1[17][26] + coeff0_18*dmats1[18][26] + coeff0_19*dmats1[19][26] + coeff0_20*dmats1[20][26] + coeff0_21*dmats1[21][26] + coeff0_22*dmats1[22][26] + coeff0_23*dmats1[23][26] + coeff0_24*dmats1[24][26] + coeff0_25*dmats1[25][26] + coeff0_26*dmats1[26][26] + coeff0_27*dmats1[27][26] + coeff0_28*dmats1[28][26] + coeff0_29*dmats1[29][26] + coeff0_30*dmats1[30][26] + coeff0_31*dmats1[31][26] + coeff0_32*dmats1[32][26] + coeff0_33*dmats1[33][26] + coeff0_34*dmats1[34][26] + coeff0_35*dmats1[35][26] + coeff0_36*dmats1[36][26] + coeff0_37*dmats1[37][26] + coeff0_38*dmats1[38][26] + coeff0_39*dmats1[39][26] + coeff0_40*dmats1[40][26] + coeff0_41*dmats1[41][26] + coeff0_42*dmats1[42][26] + coeff0_43*dmats1[43][26] + coeff0_44*dmats1[44][26] + coeff0_45*dmats1[45][26] + coeff0_46*dmats1[46][26] + coeff0_47*dmats1[47][26] + coeff0_48*dmats1[48][26] + coeff0_49*dmats1[49][26] + coeff0_50*dmats1[50][26] + coeff0_51*dmats1[51][26] + coeff0_52*dmats1[52][26] + coeff0_53*dmats1[53][26] + coeff0_54*dmats1[54][26] + coeff0_55*dmats1[55][26];
          new_coeff0_27 = coeff0_0*dmats1[0][27] + coeff0_1*dmats1[1][27] + coeff0_2*dmats1[2][27] + coeff0_3*dmats1[3][27] + coeff0_4*dmats1[4][27] + coeff0_5*dmats1[5][27] + coeff0_6*dmats1[6][27] + coeff0_7*dmats1[7][27] + coeff0_8*dmats1[8][27] + coeff0_9*dmats1[9][27] + coeff0_10*dmats1[10][27] + coeff0_11*dmats1[11][27] + coeff0_12*dmats1[12][27] + coeff0_13*dmats1[13][27] + coeff0_14*dmats1[14][27] + coeff0_15*dmats1[15][27] + coeff0_16*dmats1[16][27] + coeff0_17*dmats1[17][27] + coeff0_18*dmats1[18][27] + coeff0_19*dmats1[19][27] + coeff0_20*dmats1[20][27] + coeff0_21*dmats1[21][27] + coeff0_22*dmats1[22][27] + coeff0_23*dmats1[23][27] + coeff0_24*dmats1[24][27] + coeff0_25*dmats1[25][27] + coeff0_26*dmats1[26][27] + coeff0_27*dmats1[27][27] + coeff0_28*dmats1[28][27] + coeff0_29*dmats1[29][27] + coeff0_30*dmats1[30][27] + coeff0_31*dmats1[31][27] + coeff0_32*dmats1[32][27] + coeff0_33*dmats1[33][27] + coeff0_34*dmats1[34][27] + coeff0_35*dmats1[35][27] + coeff0_36*dmats1[36][27] + coeff0_37*dmats1[37][27] + coeff0_38*dmats1[38][27] + coeff0_39*dmats1[39][27] + coeff0_40*dmats1[40][27] + coeff0_41*dmats1[41][27] + coeff0_42*dmats1[42][27] + coeff0_43*dmats1[43][27] + coeff0_44*dmats1[44][27] + coeff0_45*dmats1[45][27] + coeff0_46*dmats1[46][27] + coeff0_47*dmats1[47][27] + coeff0_48*dmats1[48][27] + coeff0_49*dmats1[49][27] + coeff0_50*dmats1[50][27] + coeff0_51*dmats1[51][27] + coeff0_52*dmats1[52][27] + coeff0_53*dmats1[53][27] + coeff0_54*dmats1[54][27] + coeff0_55*dmats1[55][27];
          new_coeff0_28 = coeff0_0*dmats1[0][28] + coeff0_1*dmats1[1][28] + coeff0_2*dmats1[2][28] + coeff0_3*dmats1[3][28] + coeff0_4*dmats1[4][28] + coeff0_5*dmats1[5][28] + coeff0_6*dmats1[6][28] + coeff0_7*dmats1[7][28] + coeff0_8*dmats1[8][28] + coeff0_9*dmats1[9][28] + coeff0_10*dmats1[10][28] + coeff0_11*dmats1[11][28] + coeff0_12*dmats1[12][28] + coeff0_13*dmats1[13][28] + coeff0_14*dmats1[14][28] + coeff0_15*dmats1[15][28] + coeff0_16*dmats1[16][28] + coeff0_17*dmats1[17][28] + coeff0_18*dmats1[18][28] + coeff0_19*dmats1[19][28] + coeff0_20*dmats1[20][28] + coeff0_21*dmats1[21][28] + coeff0_22*dmats1[22][28] + coeff0_23*dmats1[23][28] + coeff0_24*dmats1[24][28] + coeff0_25*dmats1[25][28] + coeff0_26*dmats1[26][28] + coeff0_27*dmats1[27][28] + coeff0_28*dmats1[28][28] + coeff0_29*dmats1[29][28] + coeff0_30*dmats1[30][28] + coeff0_31*dmats1[31][28] + coeff0_32*dmats1[32][28] + coeff0_33*dmats1[33][28] + coeff0_34*dmats1[34][28] + coeff0_35*dmats1[35][28] + coeff0_36*dmats1[36][28] + coeff0_37*dmats1[37][28] + coeff0_38*dmats1[38][28] + coeff0_39*dmats1[39][28] + coeff0_40*dmats1[40][28] + coeff0_41*dmats1[41][28] + coeff0_42*dmats1[42][28] + coeff0_43*dmats1[43][28] + coeff0_44*dmats1[44][28] + coeff0_45*dmats1[45][28] + coeff0_46*dmats1[46][28] + coeff0_47*dmats1[47][28] + coeff0_48*dmats1[48][28] + coeff0_49*dmats1[49][28] + coeff0_50*dmats1[50][28] + coeff0_51*dmats1[51][28] + coeff0_52*dmats1[52][28] + coeff0_53*dmats1[53][28] + coeff0_54*dmats1[54][28] + coeff0_55*dmats1[55][28];
          new_coeff0_29 = coeff0_0*dmats1[0][29] + coeff0_1*dmats1[1][29] + coeff0_2*dmats1[2][29] + coeff0_3*dmats1[3][29] + coeff0_4*dmats1[4][29] + coeff0_5*dmats1[5][29] + coeff0_6*dmats1[6][29] + coeff0_7*dmats1[7][29] + coeff0_8*dmats1[8][29] + coeff0_9*dmats1[9][29] + coeff0_10*dmats1[10][29] + coeff0_11*dmats1[11][29] + coeff0_12*dmats1[12][29] + coeff0_13*dmats1[13][29] + coeff0_14*dmats1[14][29] + coeff0_15*dmats1[15][29] + coeff0_16*dmats1[16][29] + coeff0_17*dmats1[17][29] + coeff0_18*dmats1[18][29] + coeff0_19*dmats1[19][29] + coeff0_20*dmats1[20][29] + coeff0_21*dmats1[21][29] + coeff0_22*dmats1[22][29] + coeff0_23*dmats1[23][29] + coeff0_24*dmats1[24][29] + coeff0_25*dmats1[25][29] + coeff0_26*dmats1[26][29] + coeff0_27*dmats1[27][29] + coeff0_28*dmats1[28][29] + coeff0_29*dmats1[29][29] + coeff0_30*dmats1[30][29] + coeff0_31*dmats1[31][29] + coeff0_32*dmats1[32][29] + coeff0_33*dmats1[33][29] + coeff0_34*dmats1[34][29] + coeff0_35*dmats1[35][29] + coeff0_36*dmats1[36][29] + coeff0_37*dmats1[37][29] + coeff0_38*dmats1[38][29] + coeff0_39*dmats1[39][29] + coeff0_40*dmats1[40][29] + coeff0_41*dmats1[41][29] + coeff0_42*dmats1[42][29] + coeff0_43*dmats1[43][29] + coeff0_44*dmats1[44][29] + coeff0_45*dmats1[45][29] + coeff0_46*dmats1[46][29] + coeff0_47*dmats1[47][29] + coeff0_48*dmats1[48][29] + coeff0_49*dmats1[49][29] + coeff0_50*dmats1[50][29] + coeff0_51*dmats1[51][29] + coeff0_52*dmats1[52][29] + coeff0_53*dmats1[53][29] + coeff0_54*dmats1[54][29] + coeff0_55*dmats1[55][29];
          new_coeff0_30 = coeff0_0*dmats1[0][30] + coeff0_1*dmats1[1][30] + coeff0_2*dmats1[2][30] + coeff0_3*dmats1[3][30] + coeff0_4*dmats1[4][30] + coeff0_5*dmats1[5][30] + coeff0_6*dmats1[6][30] + coeff0_7*dmats1[7][30] + coeff0_8*dmats1[8][30] + coeff0_9*dmats1[9][30] + coeff0_10*dmats1[10][30] + coeff0_11*dmats1[11][30] + coeff0_12*dmats1[12][30] + coeff0_13*dmats1[13][30] + coeff0_14*dmats1[14][30] + coeff0_15*dmats1[15][30] + coeff0_16*dmats1[16][30] + coeff0_17*dmats1[17][30] + coeff0_18*dmats1[18][30] + coeff0_19*dmats1[19][30] + coeff0_20*dmats1[20][30] + coeff0_21*dmats1[21][30] + coeff0_22*dmats1[22][30] + coeff0_23*dmats1[23][30] + coeff0_24*dmats1[24][30] + coeff0_25*dmats1[25][30] + coeff0_26*dmats1[26][30] + coeff0_27*dmats1[27][30] + coeff0_28*dmats1[28][30] + coeff0_29*dmats1[29][30] + coeff0_30*dmats1[30][30] + coeff0_31*dmats1[31][30] + coeff0_32*dmats1[32][30] + coeff0_33*dmats1[33][30] + coeff0_34*dmats1[34][30] + coeff0_35*dmats1[35][30] + coeff0_36*dmats1[36][30] + coeff0_37*dmats1[37][30] + coeff0_38*dmats1[38][30] + coeff0_39*dmats1[39][30] + coeff0_40*dmats1[40][30] + coeff0_41*dmats1[41][30] + coeff0_42*dmats1[42][30] + coeff0_43*dmats1[43][30] + coeff0_44*dmats1[44][30] + coeff0_45*dmats1[45][30] + coeff0_46*dmats1[46][30] + coeff0_47*dmats1[47][30] + coeff0_48*dmats1[48][30] + coeff0_49*dmats1[49][30] + coeff0_50*dmats1[50][30] + coeff0_51*dmats1[51][30] + coeff0_52*dmats1[52][30] + coeff0_53*dmats1[53][30] + coeff0_54*dmats1[54][30] + coeff0_55*dmats1[55][30];
          new_coeff0_31 = coeff0_0*dmats1[0][31] + coeff0_1*dmats1[1][31] + coeff0_2*dmats1[2][31] + coeff0_3*dmats1[3][31] + coeff0_4*dmats1[4][31] + coeff0_5*dmats1[5][31] + coeff0_6*dmats1[6][31] + coeff0_7*dmats1[7][31] + coeff0_8*dmats1[8][31] + coeff0_9*dmats1[9][31] + coeff0_10*dmats1[10][31] + coeff0_11*dmats1[11][31] + coeff0_12*dmats1[12][31] + coeff0_13*dmats1[13][31] + coeff0_14*dmats1[14][31] + coeff0_15*dmats1[15][31] + coeff0_16*dmats1[16][31] + coeff0_17*dmats1[17][31] + coeff0_18*dmats1[18][31] + coeff0_19*dmats1[19][31] + coeff0_20*dmats1[20][31] + coeff0_21*dmats1[21][31] + coeff0_22*dmats1[22][31] + coeff0_23*dmats1[23][31] + coeff0_24*dmats1[24][31] + coeff0_25*dmats1[25][31] + coeff0_26*dmats1[26][31] + coeff0_27*dmats1[27][31] + coeff0_28*dmats1[28][31] + coeff0_29*dmats1[29][31] + coeff0_30*dmats1[30][31] + coeff0_31*dmats1[31][31] + coeff0_32*dmats1[32][31] + coeff0_33*dmats1[33][31] + coeff0_34*dmats1[34][31] + coeff0_35*dmats1[35][31] + coeff0_36*dmats1[36][31] + coeff0_37*dmats1[37][31] + coeff0_38*dmats1[38][31] + coeff0_39*dmats1[39][31] + coeff0_40*dmats1[40][31] + coeff0_41*dmats1[41][31] + coeff0_42*dmats1[42][31] + coeff0_43*dmats1[43][31] + coeff0_44*dmats1[44][31] + coeff0_45*dmats1[45][31] + coeff0_46*dmats1[46][31] + coeff0_47*dmats1[47][31] + coeff0_48*dmats1[48][31] + coeff0_49*dmats1[49][31] + coeff0_50*dmats1[50][31] + coeff0_51*dmats1[51][31] + coeff0_52*dmats1[52][31] + coeff0_53*dmats1[53][31] + coeff0_54*dmats1[54][31] + coeff0_55*dmats1[55][31];
          new_coeff0_32 = coeff0_0*dmats1[0][32] + coeff0_1*dmats1[1][32] + coeff0_2*dmats1[2][32] + coeff0_3*dmats1[3][32] + coeff0_4*dmats1[4][32] + coeff0_5*dmats1[5][32] + coeff0_6*dmats1[6][32] + coeff0_7*dmats1[7][32] + coeff0_8*dmats1[8][32] + coeff0_9*dmats1[9][32] + coeff0_10*dmats1[10][32] + coeff0_11*dmats1[11][32] + coeff0_12*dmats1[12][32] + coeff0_13*dmats1[13][32] + coeff0_14*dmats1[14][32] + coeff0_15*dmats1[15][32] + coeff0_16*dmats1[16][32] + coeff0_17*dmats1[17][32] + coeff0_18*dmats1[18][32] + coeff0_19*dmats1[19][32] + coeff0_20*dmats1[20][32] + coeff0_21*dmats1[21][32] + coeff0_22*dmats1[22][32] + coeff0_23*dmats1[23][32] + coeff0_24*dmats1[24][32] + coeff0_25*dmats1[25][32] + coeff0_26*dmats1[26][32] + coeff0_27*dmats1[27][32] + coeff0_28*dmats1[28][32] + coeff0_29*dmats1[29][32] + coeff0_30*dmats1[30][32] + coeff0_31*dmats1[31][32] + coeff0_32*dmats1[32][32] + coeff0_33*dmats1[33][32] + coeff0_34*dmats1[34][32] + coeff0_35*dmats1[35][32] + coeff0_36*dmats1[36][32] + coeff0_37*dmats1[37][32] + coeff0_38*dmats1[38][32] + coeff0_39*dmats1[39][32] + coeff0_40*dmats1[40][32] + coeff0_41*dmats1[41][32] + coeff0_42*dmats1[42][32] + coeff0_43*dmats1[43][32] + coeff0_44*dmats1[44][32] + coeff0_45*dmats1[45][32] + coeff0_46*dmats1[46][32] + coeff0_47*dmats1[47][32] + coeff0_48*dmats1[48][32] + coeff0_49*dmats1[49][32] + coeff0_50*dmats1[50][32] + coeff0_51*dmats1[51][32] + coeff0_52*dmats1[52][32] + coeff0_53*dmats1[53][32] + coeff0_54*dmats1[54][32] + coeff0_55*dmats1[55][32];
          new_coeff0_33 = coeff0_0*dmats1[0][33] + coeff0_1*dmats1[1][33] + coeff0_2*dmats1[2][33] + coeff0_3*dmats1[3][33] + coeff0_4*dmats1[4][33] + coeff0_5*dmats1[5][33] + coeff0_6*dmats1[6][33] + coeff0_7*dmats1[7][33] + coeff0_8*dmats1[8][33] + coeff0_9*dmats1[9][33] + coeff0_10*dmats1[10][33] + coeff0_11*dmats1[11][33] + coeff0_12*dmats1[12][33] + coeff0_13*dmats1[13][33] + coeff0_14*dmats1[14][33] + coeff0_15*dmats1[15][33] + coeff0_16*dmats1[16][33] + coeff0_17*dmats1[17][33] + coeff0_18*dmats1[18][33] + coeff0_19*dmats1[19][33] + coeff0_20*dmats1[20][33] + coeff0_21*dmats1[21][33] + coeff0_22*dmats1[22][33] + coeff0_23*dmats1[23][33] + coeff0_24*dmats1[24][33] + coeff0_25*dmats1[25][33] + coeff0_26*dmats1[26][33] + coeff0_27*dmats1[27][33] + coeff0_28*dmats1[28][33] + coeff0_29*dmats1[29][33] + coeff0_30*dmats1[30][33] + coeff0_31*dmats1[31][33] + coeff0_32*dmats1[32][33] + coeff0_33*dmats1[33][33] + coeff0_34*dmats1[34][33] + coeff0_35*dmats1[35][33] + coeff0_36*dmats1[36][33] + coeff0_37*dmats1[37][33] + coeff0_38*dmats1[38][33] + coeff0_39*dmats1[39][33] + coeff0_40*dmats1[40][33] + coeff0_41*dmats1[41][33] + coeff0_42*dmats1[42][33] + coeff0_43*dmats1[43][33] + coeff0_44*dmats1[44][33] + coeff0_45*dmats1[45][33] + coeff0_46*dmats1[46][33] + coeff0_47*dmats1[47][33] + coeff0_48*dmats1[48][33] + coeff0_49*dmats1[49][33] + coeff0_50*dmats1[50][33] + coeff0_51*dmats1[51][33] + coeff0_52*dmats1[52][33] + coeff0_53*dmats1[53][33] + coeff0_54*dmats1[54][33] + coeff0_55*dmats1[55][33];
          new_coeff0_34 = coeff0_0*dmats1[0][34] + coeff0_1*dmats1[1][34] + coeff0_2*dmats1[2][34] + coeff0_3*dmats1[3][34] + coeff0_4*dmats1[4][34] + coeff0_5*dmats1[5][34] + coeff0_6*dmats1[6][34] + coeff0_7*dmats1[7][34] + coeff0_8*dmats1[8][34] + coeff0_9*dmats1[9][34] + coeff0_10*dmats1[10][34] + coeff0_11*dmats1[11][34] + coeff0_12*dmats1[12][34] + coeff0_13*dmats1[13][34] + coeff0_14*dmats1[14][34] + coeff0_15*dmats1[15][34] + coeff0_16*dmats1[16][34] + coeff0_17*dmats1[17][34] + coeff0_18*dmats1[18][34] + coeff0_19*dmats1[19][34] + coeff0_20*dmats1[20][34] + coeff0_21*dmats1[21][34] + coeff0_22*dmats1[22][34] + coeff0_23*dmats1[23][34] + coeff0_24*dmats1[24][34] + coeff0_25*dmats1[25][34] + coeff0_26*dmats1[26][34] + coeff0_27*dmats1[27][34] + coeff0_28*dmats1[28][34] + coeff0_29*dmats1[29][34] + coeff0_30*dmats1[30][34] + coeff0_31*dmats1[31][34] + coeff0_32*dmats1[32][34] + coeff0_33*dmats1[33][34] + coeff0_34*dmats1[34][34] + coeff0_35*dmats1[35][34] + coeff0_36*dmats1[36][34] + coeff0_37*dmats1[37][34] + coeff0_38*dmats1[38][34] + coeff0_39*dmats1[39][34] + coeff0_40*dmats1[40][34] + coeff0_41*dmats1[41][34] + coeff0_42*dmats1[42][34] + coeff0_43*dmats1[43][34] + coeff0_44*dmats1[44][34] + coeff0_45*dmats1[45][34] + coeff0_46*dmats1[46][34] + coeff0_47*dmats1[47][34] + coeff0_48*dmats1[48][34] + coeff0_49*dmats1[49][34] + coeff0_50*dmats1[50][34] + coeff0_51*dmats1[51][34] + coeff0_52*dmats1[52][34] + coeff0_53*dmats1[53][34] + coeff0_54*dmats1[54][34] + coeff0_55*dmats1[55][34];
          new_coeff0_35 = coeff0_0*dmats1[0][35] + coeff0_1*dmats1[1][35] + coeff0_2*dmats1[2][35] + coeff0_3*dmats1[3][35] + coeff0_4*dmats1[4][35] + coeff0_5*dmats1[5][35] + coeff0_6*dmats1[6][35] + coeff0_7*dmats1[7][35] + coeff0_8*dmats1[8][35] + coeff0_9*dmats1[9][35] + coeff0_10*dmats1[10][35] + coeff0_11*dmats1[11][35] + coeff0_12*dmats1[12][35] + coeff0_13*dmats1[13][35] + coeff0_14*dmats1[14][35] + coeff0_15*dmats1[15][35] + coeff0_16*dmats1[16][35] + coeff0_17*dmats1[17][35] + coeff0_18*dmats1[18][35] + coeff0_19*dmats1[19][35] + coeff0_20*dmats1[20][35] + coeff0_21*dmats1[21][35] + coeff0_22*dmats1[22][35] + coeff0_23*dmats1[23][35] + coeff0_24*dmats1[24][35] + coeff0_25*dmats1[25][35] + coeff0_26*dmats1[26][35] + coeff0_27*dmats1[27][35] + coeff0_28*dmats1[28][35] + coeff0_29*dmats1[29][35] + coeff0_30*dmats1[30][35] + coeff0_31*dmats1[31][35] + coeff0_32*dmats1[32][35] + coeff0_33*dmats1[33][35] + coeff0_34*dmats1[34][35] + coeff0_35*dmats1[35][35] + coeff0_36*dmats1[36][35] + coeff0_37*dmats1[37][35] + coeff0_38*dmats1[38][35] + coeff0_39*dmats1[39][35] + coeff0_40*dmats1[40][35] + coeff0_41*dmats1[41][35] + coeff0_42*dmats1[42][35] + coeff0_43*dmats1[43][35] + coeff0_44*dmats1[44][35] + coeff0_45*dmats1[45][35] + coeff0_46*dmats1[46][35] + coeff0_47*dmats1[47][35] + coeff0_48*dmats1[48][35] + coeff0_49*dmats1[49][35] + coeff0_50*dmats1[50][35] + coeff0_51*dmats1[51][35] + coeff0_52*dmats1[52][35] + coeff0_53*dmats1[53][35] + coeff0_54*dmats1[54][35] + coeff0_55*dmats1[55][35];
          new_coeff0_36 = coeff0_0*dmats1[0][36] + coeff0_1*dmats1[1][36] + coeff0_2*dmats1[2][36] + coeff0_3*dmats1[3][36] + coeff0_4*dmats1[4][36] + coeff0_5*dmats1[5][36] + coeff0_6*dmats1[6][36] + coeff0_7*dmats1[7][36] + coeff0_8*dmats1[8][36] + coeff0_9*dmats1[9][36] + coeff0_10*dmats1[10][36] + coeff0_11*dmats1[11][36] + coeff0_12*dmats1[12][36] + coeff0_13*dmats1[13][36] + coeff0_14*dmats1[14][36] + coeff0_15*dmats1[15][36] + coeff0_16*dmats1[16][36] + coeff0_17*dmats1[17][36] + coeff0_18*dmats1[18][36] + coeff0_19*dmats1[19][36] + coeff0_20*dmats1[20][36] + coeff0_21*dmats1[21][36] + coeff0_22*dmats1[22][36] + coeff0_23*dmats1[23][36] + coeff0_24*dmats1[24][36] + coeff0_25*dmats1[25][36] + coeff0_26*dmats1[26][36] + coeff0_27*dmats1[27][36] + coeff0_28*dmats1[28][36] + coeff0_29*dmats1[29][36] + coeff0_30*dmats1[30][36] + coeff0_31*dmats1[31][36] + coeff0_32*dmats1[32][36] + coeff0_33*dmats1[33][36] + coeff0_34*dmats1[34][36] + coeff0_35*dmats1[35][36] + coeff0_36*dmats1[36][36] + coeff0_37*dmats1[37][36] + coeff0_38*dmats1[38][36] + coeff0_39*dmats1[39][36] + coeff0_40*dmats1[40][36] + coeff0_41*dmats1[41][36] + coeff0_42*dmats1[42][36] + coeff0_43*dmats1[43][36] + coeff0_44*dmats1[44][36] + coeff0_45*dmats1[45][36] + coeff0_46*dmats1[46][36] + coeff0_47*dmats1[47][36] + coeff0_48*dmats1[48][36] + coeff0_49*dmats1[49][36] + coeff0_50*dmats1[50][36] + coeff0_51*dmats1[51][36] + coeff0_52*dmats1[52][36] + coeff0_53*dmats1[53][36] + coeff0_54*dmats1[54][36] + coeff0_55*dmats1[55][36];
          new_coeff0_37 = coeff0_0*dmats1[0][37] + coeff0_1*dmats1[1][37] + coeff0_2*dmats1[2][37] + coeff0_3*dmats1[3][37] + coeff0_4*dmats1[4][37] + coeff0_5*dmats1[5][37] + coeff0_6*dmats1[6][37] + coeff0_7*dmats1[7][37] + coeff0_8*dmats1[8][37] + coeff0_9*dmats1[9][37] + coeff0_10*dmats1[10][37] + coeff0_11*dmats1[11][37] + coeff0_12*dmats1[12][37] + coeff0_13*dmats1[13][37] + coeff0_14*dmats1[14][37] + coeff0_15*dmats1[15][37] + coeff0_16*dmats1[16][37] + coeff0_17*dmats1[17][37] + coeff0_18*dmats1[18][37] + coeff0_19*dmats1[19][37] + coeff0_20*dmats1[20][37] + coeff0_21*dmats1[21][37] + coeff0_22*dmats1[22][37] + coeff0_23*dmats1[23][37] + coeff0_24*dmats1[24][37] + coeff0_25*dmats1[25][37] + coeff0_26*dmats1[26][37] + coeff0_27*dmats1[27][37] + coeff0_28*dmats1[28][37] + coeff0_29*dmats1[29][37] + coeff0_30*dmats1[30][37] + coeff0_31*dmats1[31][37] + coeff0_32*dmats1[32][37] + coeff0_33*dmats1[33][37] + coeff0_34*dmats1[34][37] + coeff0_35*dmats1[35][37] + coeff0_36*dmats1[36][37] + coeff0_37*dmats1[37][37] + coeff0_38*dmats1[38][37] + coeff0_39*dmats1[39][37] + coeff0_40*dmats1[40][37] + coeff0_41*dmats1[41][37] + coeff0_42*dmats1[42][37] + coeff0_43*dmats1[43][37] + coeff0_44*dmats1[44][37] + coeff0_45*dmats1[45][37] + coeff0_46*dmats1[46][37] + coeff0_47*dmats1[47][37] + coeff0_48*dmats1[48][37] + coeff0_49*dmats1[49][37] + coeff0_50*dmats1[50][37] + coeff0_51*dmats1[51][37] + coeff0_52*dmats1[52][37] + coeff0_53*dmats1[53][37] + coeff0_54*dmats1[54][37] + coeff0_55*dmats1[55][37];
          new_coeff0_38 = coeff0_0*dmats1[0][38] + coeff0_1*dmats1[1][38] + coeff0_2*dmats1[2][38] + coeff0_3*dmats1[3][38] + coeff0_4*dmats1[4][38] + coeff0_5*dmats1[5][38] + coeff0_6*dmats1[6][38] + coeff0_7*dmats1[7][38] + coeff0_8*dmats1[8][38] + coeff0_9*dmats1[9][38] + coeff0_10*dmats1[10][38] + coeff0_11*dmats1[11][38] + coeff0_12*dmats1[12][38] + coeff0_13*dmats1[13][38] + coeff0_14*dmats1[14][38] + coeff0_15*dmats1[15][38] + coeff0_16*dmats1[16][38] + coeff0_17*dmats1[17][38] + coeff0_18*dmats1[18][38] + coeff0_19*dmats1[19][38] + coeff0_20*dmats1[20][38] + coeff0_21*dmats1[21][38] + coeff0_22*dmats1[22][38] + coeff0_23*dmats1[23][38] + coeff0_24*dmats1[24][38] + coeff0_25*dmats1[25][38] + coeff0_26*dmats1[26][38] + coeff0_27*dmats1[27][38] + coeff0_28*dmats1[28][38] + coeff0_29*dmats1[29][38] + coeff0_30*dmats1[30][38] + coeff0_31*dmats1[31][38] + coeff0_32*dmats1[32][38] + coeff0_33*dmats1[33][38] + coeff0_34*dmats1[34][38] + coeff0_35*dmats1[35][38] + coeff0_36*dmats1[36][38] + coeff0_37*dmats1[37][38] + coeff0_38*dmats1[38][38] + coeff0_39*dmats1[39][38] + coeff0_40*dmats1[40][38] + coeff0_41*dmats1[41][38] + coeff0_42*dmats1[42][38] + coeff0_43*dmats1[43][38] + coeff0_44*dmats1[44][38] + coeff0_45*dmats1[45][38] + coeff0_46*dmats1[46][38] + coeff0_47*dmats1[47][38] + coeff0_48*dmats1[48][38] + coeff0_49*dmats1[49][38] + coeff0_50*dmats1[50][38] + coeff0_51*dmats1[51][38] + coeff0_52*dmats1[52][38] + coeff0_53*dmats1[53][38] + coeff0_54*dmats1[54][38] + coeff0_55*dmats1[55][38];
          new_coeff0_39 = coeff0_0*dmats1[0][39] + coeff0_1*dmats1[1][39] + coeff0_2*dmats1[2][39] + coeff0_3*dmats1[3][39] + coeff0_4*dmats1[4][39] + coeff0_5*dmats1[5][39] + coeff0_6*dmats1[6][39] + coeff0_7*dmats1[7][39] + coeff0_8*dmats1[8][39] + coeff0_9*dmats1[9][39] + coeff0_10*dmats1[10][39] + coeff0_11*dmats1[11][39] + coeff0_12*dmats1[12][39] + coeff0_13*dmats1[13][39] + coeff0_14*dmats1[14][39] + coeff0_15*dmats1[15][39] + coeff0_16*dmats1[16][39] + coeff0_17*dmats1[17][39] + coeff0_18*dmats1[18][39] + coeff0_19*dmats1[19][39] + coeff0_20*dmats1[20][39] + coeff0_21*dmats1[21][39] + coeff0_22*dmats1[22][39] + coeff0_23*dmats1[23][39] + coeff0_24*dmats1[24][39] + coeff0_25*dmats1[25][39] + coeff0_26*dmats1[26][39] + coeff0_27*dmats1[27][39] + coeff0_28*dmats1[28][39] + coeff0_29*dmats1[29][39] + coeff0_30*dmats1[30][39] + coeff0_31*dmats1[31][39] + coeff0_32*dmats1[32][39] + coeff0_33*dmats1[33][39] + coeff0_34*dmats1[34][39] + coeff0_35*dmats1[35][39] + coeff0_36*dmats1[36][39] + coeff0_37*dmats1[37][39] + coeff0_38*dmats1[38][39] + coeff0_39*dmats1[39][39] + coeff0_40*dmats1[40][39] + coeff0_41*dmats1[41][39] + coeff0_42*dmats1[42][39] + coeff0_43*dmats1[43][39] + coeff0_44*dmats1[44][39] + coeff0_45*dmats1[45][39] + coeff0_46*dmats1[46][39] + coeff0_47*dmats1[47][39] + coeff0_48*dmats1[48][39] + coeff0_49*dmats1[49][39] + coeff0_50*dmats1[50][39] + coeff0_51*dmats1[51][39] + coeff0_52*dmats1[52][39] + coeff0_53*dmats1[53][39] + coeff0_54*dmats1[54][39] + coeff0_55*dmats1[55][39];
          new_coeff0_40 = coeff0_0*dmats1[0][40] + coeff0_1*dmats1[1][40] + coeff0_2*dmats1[2][40] + coeff0_3*dmats1[3][40] + coeff0_4*dmats1[4][40] + coeff0_5*dmats1[5][40] + coeff0_6*dmats1[6][40] + coeff0_7*dmats1[7][40] + coeff0_8*dmats1[8][40] + coeff0_9*dmats1[9][40] + coeff0_10*dmats1[10][40] + coeff0_11*dmats1[11][40] + coeff0_12*dmats1[12][40] + coeff0_13*dmats1[13][40] + coeff0_14*dmats1[14][40] + coeff0_15*dmats1[15][40] + coeff0_16*dmats1[16][40] + coeff0_17*dmats1[17][40] + coeff0_18*dmats1[18][40] + coeff0_19*dmats1[19][40] + coeff0_20*dmats1[20][40] + coeff0_21*dmats1[21][40] + coeff0_22*dmats1[22][40] + coeff0_23*dmats1[23][40] + coeff0_24*dmats1[24][40] + coeff0_25*dmats1[25][40] + coeff0_26*dmats1[26][40] + coeff0_27*dmats1[27][40] + coeff0_28*dmats1[28][40] + coeff0_29*dmats1[29][40] + coeff0_30*dmats1[30][40] + coeff0_31*dmats1[31][40] + coeff0_32*dmats1[32][40] + coeff0_33*dmats1[33][40] + coeff0_34*dmats1[34][40] + coeff0_35*dmats1[35][40] + coeff0_36*dmats1[36][40] + coeff0_37*dmats1[37][40] + coeff0_38*dmats1[38][40] + coeff0_39*dmats1[39][40] + coeff0_40*dmats1[40][40] + coeff0_41*dmats1[41][40] + coeff0_42*dmats1[42][40] + coeff0_43*dmats1[43][40] + coeff0_44*dmats1[44][40] + coeff0_45*dmats1[45][40] + coeff0_46*dmats1[46][40] + coeff0_47*dmats1[47][40] + coeff0_48*dmats1[48][40] + coeff0_49*dmats1[49][40] + coeff0_50*dmats1[50][40] + coeff0_51*dmats1[51][40] + coeff0_52*dmats1[52][40] + coeff0_53*dmats1[53][40] + coeff0_54*dmats1[54][40] + coeff0_55*dmats1[55][40];
          new_coeff0_41 = coeff0_0*dmats1[0][41] + coeff0_1*dmats1[1][41] + coeff0_2*dmats1[2][41] + coeff0_3*dmats1[3][41] + coeff0_4*dmats1[4][41] + coeff0_5*dmats1[5][41] + coeff0_6*dmats1[6][41] + coeff0_7*dmats1[7][41] + coeff0_8*dmats1[8][41] + coeff0_9*dmats1[9][41] + coeff0_10*dmats1[10][41] + coeff0_11*dmats1[11][41] + coeff0_12*dmats1[12][41] + coeff0_13*dmats1[13][41] + coeff0_14*dmats1[14][41] + coeff0_15*dmats1[15][41] + coeff0_16*dmats1[16][41] + coeff0_17*dmats1[17][41] + coeff0_18*dmats1[18][41] + coeff0_19*dmats1[19][41] + coeff0_20*dmats1[20][41] + coeff0_21*dmats1[21][41] + coeff0_22*dmats1[22][41] + coeff0_23*dmats1[23][41] + coeff0_24*dmats1[24][41] + coeff0_25*dmats1[25][41] + coeff0_26*dmats1[26][41] + coeff0_27*dmats1[27][41] + coeff0_28*dmats1[28][41] + coeff0_29*dmats1[29][41] + coeff0_30*dmats1[30][41] + coeff0_31*dmats1[31][41] + coeff0_32*dmats1[32][41] + coeff0_33*dmats1[33][41] + coeff0_34*dmats1[34][41] + coeff0_35*dmats1[35][41] + coeff0_36*dmats1[36][41] + coeff0_37*dmats1[37][41] + coeff0_38*dmats1[38][41] + coeff0_39*dmats1[39][41] + coeff0_40*dmats1[40][41] + coeff0_41*dmats1[41][41] + coeff0_42*dmats1[42][41] + coeff0_43*dmats1[43][41] + coeff0_44*dmats1[44][41] + coeff0_45*dmats1[45][41] + coeff0_46*dmats1[46][41] + coeff0_47*dmats1[47][41] + coeff0_48*dmats1[48][41] + coeff0_49*dmats1[49][41] + coeff0_50*dmats1[50][41] + coeff0_51*dmats1[51][41] + coeff0_52*dmats1[52][41] + coeff0_53*dmats1[53][41] + coeff0_54*dmats1[54][41] + coeff0_55*dmats1[55][41];
          new_coeff0_42 = coeff0_0*dmats1[0][42] + coeff0_1*dmats1[1][42] + coeff0_2*dmats1[2][42] + coeff0_3*dmats1[3][42] + coeff0_4*dmats1[4][42] + coeff0_5*dmats1[5][42] + coeff0_6*dmats1[6][42] + coeff0_7*dmats1[7][42] + coeff0_8*dmats1[8][42] + coeff0_9*dmats1[9][42] + coeff0_10*dmats1[10][42] + coeff0_11*dmats1[11][42] + coeff0_12*dmats1[12][42] + coeff0_13*dmats1[13][42] + coeff0_14*dmats1[14][42] + coeff0_15*dmats1[15][42] + coeff0_16*dmats1[16][42] + coeff0_17*dmats1[17][42] + coeff0_18*dmats1[18][42] + coeff0_19*dmats1[19][42] + coeff0_20*dmats1[20][42] + coeff0_21*dmats1[21][42] + coeff0_22*dmats1[22][42] + coeff0_23*dmats1[23][42] + coeff0_24*dmats1[24][42] + coeff0_25*dmats1[25][42] + coeff0_26*dmats1[26][42] + coeff0_27*dmats1[27][42] + coeff0_28*dmats1[28][42] + coeff0_29*dmats1[29][42] + coeff0_30*dmats1[30][42] + coeff0_31*dmats1[31][42] + coeff0_32*dmats1[32][42] + coeff0_33*dmats1[33][42] + coeff0_34*dmats1[34][42] + coeff0_35*dmats1[35][42] + coeff0_36*dmats1[36][42] + coeff0_37*dmats1[37][42] + coeff0_38*dmats1[38][42] + coeff0_39*dmats1[39][42] + coeff0_40*dmats1[40][42] + coeff0_41*dmats1[41][42] + coeff0_42*dmats1[42][42] + coeff0_43*dmats1[43][42] + coeff0_44*dmats1[44][42] + coeff0_45*dmats1[45][42] + coeff0_46*dmats1[46][42] + coeff0_47*dmats1[47][42] + coeff0_48*dmats1[48][42] + coeff0_49*dmats1[49][42] + coeff0_50*dmats1[50][42] + coeff0_51*dmats1[51][42] + coeff0_52*dmats1[52][42] + coeff0_53*dmats1[53][42] + coeff0_54*dmats1[54][42] + coeff0_55*dmats1[55][42];
          new_coeff0_43 = coeff0_0*dmats1[0][43] + coeff0_1*dmats1[1][43] + coeff0_2*dmats1[2][43] + coeff0_3*dmats1[3][43] + coeff0_4*dmats1[4][43] + coeff0_5*dmats1[5][43] + coeff0_6*dmats1[6][43] + coeff0_7*dmats1[7][43] + coeff0_8*dmats1[8][43] + coeff0_9*dmats1[9][43] + coeff0_10*dmats1[10][43] + coeff0_11*dmats1[11][43] + coeff0_12*dmats1[12][43] + coeff0_13*dmats1[13][43] + coeff0_14*dmats1[14][43] + coeff0_15*dmats1[15][43] + coeff0_16*dmats1[16][43] + coeff0_17*dmats1[17][43] + coeff0_18*dmats1[18][43] + coeff0_19*dmats1[19][43] + coeff0_20*dmats1[20][43] + coeff0_21*dmats1[21][43] + coeff0_22*dmats1[22][43] + coeff0_23*dmats1[23][43] + coeff0_24*dmats1[24][43] + coeff0_25*dmats1[25][43] + coeff0_26*dmats1[26][43] + coeff0_27*dmats1[27][43] + coeff0_28*dmats1[28][43] + coeff0_29*dmats1[29][43] + coeff0_30*dmats1[30][43] + coeff0_31*dmats1[31][43] + coeff0_32*dmats1[32][43] + coeff0_33*dmats1[33][43] + coeff0_34*dmats1[34][43] + coeff0_35*dmats1[35][43] + coeff0_36*dmats1[36][43] + coeff0_37*dmats1[37][43] + coeff0_38*dmats1[38][43] + coeff0_39*dmats1[39][43] + coeff0_40*dmats1[40][43] + coeff0_41*dmats1[41][43] + coeff0_42*dmats1[42][43] + coeff0_43*dmats1[43][43] + coeff0_44*dmats1[44][43] + coeff0_45*dmats1[45][43] + coeff0_46*dmats1[46][43] + coeff0_47*dmats1[47][43] + coeff0_48*dmats1[48][43] + coeff0_49*dmats1[49][43] + coeff0_50*dmats1[50][43] + coeff0_51*dmats1[51][43] + coeff0_52*dmats1[52][43] + coeff0_53*dmats1[53][43] + coeff0_54*dmats1[54][43] + coeff0_55*dmats1[55][43];
          new_coeff0_44 = coeff0_0*dmats1[0][44] + coeff0_1*dmats1[1][44] + coeff0_2*dmats1[2][44] + coeff0_3*dmats1[3][44] + coeff0_4*dmats1[4][44] + coeff0_5*dmats1[5][44] + coeff0_6*dmats1[6][44] + coeff0_7*dmats1[7][44] + coeff0_8*dmats1[8][44] + coeff0_9*dmats1[9][44] + coeff0_10*dmats1[10][44] + coeff0_11*dmats1[11][44] + coeff0_12*dmats1[12][44] + coeff0_13*dmats1[13][44] + coeff0_14*dmats1[14][44] + coeff0_15*dmats1[15][44] + coeff0_16*dmats1[16][44] + coeff0_17*dmats1[17][44] + coeff0_18*dmats1[18][44] + coeff0_19*dmats1[19][44] + coeff0_20*dmats1[20][44] + coeff0_21*dmats1[21][44] + coeff0_22*dmats1[22][44] + coeff0_23*dmats1[23][44] + coeff0_24*dmats1[24][44] + coeff0_25*dmats1[25][44] + coeff0_26*dmats1[26][44] + coeff0_27*dmats1[27][44] + coeff0_28*dmats1[28][44] + coeff0_29*dmats1[29][44] + coeff0_30*dmats1[30][44] + coeff0_31*dmats1[31][44] + coeff0_32*dmats1[32][44] + coeff0_33*dmats1[33][44] + coeff0_34*dmats1[34][44] + coeff0_35*dmats1[35][44] + coeff0_36*dmats1[36][44] + coeff0_37*dmats1[37][44] + coeff0_38*dmats1[38][44] + coeff0_39*dmats1[39][44] + coeff0_40*dmats1[40][44] + coeff0_41*dmats1[41][44] + coeff0_42*dmats1[42][44] + coeff0_43*dmats1[43][44] + coeff0_44*dmats1[44][44] + coeff0_45*dmats1[45][44] + coeff0_46*dmats1[46][44] + coeff0_47*dmats1[47][44] + coeff0_48*dmats1[48][44] + coeff0_49*dmats1[49][44] + coeff0_50*dmats1[50][44] + coeff0_51*dmats1[51][44] + coeff0_52*dmats1[52][44] + coeff0_53*dmats1[53][44] + coeff0_54*dmats1[54][44] + coeff0_55*dmats1[55][44];
          new_coeff0_45 = coeff0_0*dmats1[0][45] + coeff0_1*dmats1[1][45] + coeff0_2*dmats1[2][45] + coeff0_3*dmats1[3][45] + coeff0_4*dmats1[4][45] + coeff0_5*dmats1[5][45] + coeff0_6*dmats1[6][45] + coeff0_7*dmats1[7][45] + coeff0_8*dmats1[8][45] + coeff0_9*dmats1[9][45] + coeff0_10*dmats1[10][45] + coeff0_11*dmats1[11][45] + coeff0_12*dmats1[12][45] + coeff0_13*dmats1[13][45] + coeff0_14*dmats1[14][45] + coeff0_15*dmats1[15][45] + coeff0_16*dmats1[16][45] + coeff0_17*dmats1[17][45] + coeff0_18*dmats1[18][45] + coeff0_19*dmats1[19][45] + coeff0_20*dmats1[20][45] + coeff0_21*dmats1[21][45] + coeff0_22*dmats1[22][45] + coeff0_23*dmats1[23][45] + coeff0_24*dmats1[24][45] + coeff0_25*dmats1[25][45] + coeff0_26*dmats1[26][45] + coeff0_27*dmats1[27][45] + coeff0_28*dmats1[28][45] + coeff0_29*dmats1[29][45] + coeff0_30*dmats1[30][45] + coeff0_31*dmats1[31][45] + coeff0_32*dmats1[32][45] + coeff0_33*dmats1[33][45] + coeff0_34*dmats1[34][45] + coeff0_35*dmats1[35][45] + coeff0_36*dmats1[36][45] + coeff0_37*dmats1[37][45] + coeff0_38*dmats1[38][45] + coeff0_39*dmats1[39][45] + coeff0_40*dmats1[40][45] + coeff0_41*dmats1[41][45] + coeff0_42*dmats1[42][45] + coeff0_43*dmats1[43][45] + coeff0_44*dmats1[44][45] + coeff0_45*dmats1[45][45] + coeff0_46*dmats1[46][45] + coeff0_47*dmats1[47][45] + coeff0_48*dmats1[48][45] + coeff0_49*dmats1[49][45] + coeff0_50*dmats1[50][45] + coeff0_51*dmats1[51][45] + coeff0_52*dmats1[52][45] + coeff0_53*dmats1[53][45] + coeff0_54*dmats1[54][45] + coeff0_55*dmats1[55][45];
          new_coeff0_46 = coeff0_0*dmats1[0][46] + coeff0_1*dmats1[1][46] + coeff0_2*dmats1[2][46] + coeff0_3*dmats1[3][46] + coeff0_4*dmats1[4][46] + coeff0_5*dmats1[5][46] + coeff0_6*dmats1[6][46] + coeff0_7*dmats1[7][46] + coeff0_8*dmats1[8][46] + coeff0_9*dmats1[9][46] + coeff0_10*dmats1[10][46] + coeff0_11*dmats1[11][46] + coeff0_12*dmats1[12][46] + coeff0_13*dmats1[13][46] + coeff0_14*dmats1[14][46] + coeff0_15*dmats1[15][46] + coeff0_16*dmats1[16][46] + coeff0_17*dmats1[17][46] + coeff0_18*dmats1[18][46] + coeff0_19*dmats1[19][46] + coeff0_20*dmats1[20][46] + coeff0_21*dmats1[21][46] + coeff0_22*dmats1[22][46] + coeff0_23*dmats1[23][46] + coeff0_24*dmats1[24][46] + coeff0_25*dmats1[25][46] + coeff0_26*dmats1[26][46] + coeff0_27*dmats1[27][46] + coeff0_28*dmats1[28][46] + coeff0_29*dmats1[29][46] + coeff0_30*dmats1[30][46] + coeff0_31*dmats1[31][46] + coeff0_32*dmats1[32][46] + coeff0_33*dmats1[33][46] + coeff0_34*dmats1[34][46] + coeff0_35*dmats1[35][46] + coeff0_36*dmats1[36][46] + coeff0_37*dmats1[37][46] + coeff0_38*dmats1[38][46] + coeff0_39*dmats1[39][46] + coeff0_40*dmats1[40][46] + coeff0_41*dmats1[41][46] + coeff0_42*dmats1[42][46] + coeff0_43*dmats1[43][46] + coeff0_44*dmats1[44][46] + coeff0_45*dmats1[45][46] + coeff0_46*dmats1[46][46] + coeff0_47*dmats1[47][46] + coeff0_48*dmats1[48][46] + coeff0_49*dmats1[49][46] + coeff0_50*dmats1[50][46] + coeff0_51*dmats1[51][46] + coeff0_52*dmats1[52][46] + coeff0_53*dmats1[53][46] + coeff0_54*dmats1[54][46] + coeff0_55*dmats1[55][46];
          new_coeff0_47 = coeff0_0*dmats1[0][47] + coeff0_1*dmats1[1][47] + coeff0_2*dmats1[2][47] + coeff0_3*dmats1[3][47] + coeff0_4*dmats1[4][47] + coeff0_5*dmats1[5][47] + coeff0_6*dmats1[6][47] + coeff0_7*dmats1[7][47] + coeff0_8*dmats1[8][47] + coeff0_9*dmats1[9][47] + coeff0_10*dmats1[10][47] + coeff0_11*dmats1[11][47] + coeff0_12*dmats1[12][47] + coeff0_13*dmats1[13][47] + coeff0_14*dmats1[14][47] + coeff0_15*dmats1[15][47] + coeff0_16*dmats1[16][47] + coeff0_17*dmats1[17][47] + coeff0_18*dmats1[18][47] + coeff0_19*dmats1[19][47] + coeff0_20*dmats1[20][47] + coeff0_21*dmats1[21][47] + coeff0_22*dmats1[22][47] + coeff0_23*dmats1[23][47] + coeff0_24*dmats1[24][47] + coeff0_25*dmats1[25][47] + coeff0_26*dmats1[26][47] + coeff0_27*dmats1[27][47] + coeff0_28*dmats1[28][47] + coeff0_29*dmats1[29][47] + coeff0_30*dmats1[30][47] + coeff0_31*dmats1[31][47] + coeff0_32*dmats1[32][47] + coeff0_33*dmats1[33][47] + coeff0_34*dmats1[34][47] + coeff0_35*dmats1[35][47] + coeff0_36*dmats1[36][47] + coeff0_37*dmats1[37][47] + coeff0_38*dmats1[38][47] + coeff0_39*dmats1[39][47] + coeff0_40*dmats1[40][47] + coeff0_41*dmats1[41][47] + coeff0_42*dmats1[42][47] + coeff0_43*dmats1[43][47] + coeff0_44*dmats1[44][47] + coeff0_45*dmats1[45][47] + coeff0_46*dmats1[46][47] + coeff0_47*dmats1[47][47] + coeff0_48*dmats1[48][47] + coeff0_49*dmats1[49][47] + coeff0_50*dmats1[50][47] + coeff0_51*dmats1[51][47] + coeff0_52*dmats1[52][47] + coeff0_53*dmats1[53][47] + coeff0_54*dmats1[54][47] + coeff0_55*dmats1[55][47];
          new_coeff0_48 = coeff0_0*dmats1[0][48] + coeff0_1*dmats1[1][48] + coeff0_2*dmats1[2][48] + coeff0_3*dmats1[3][48] + coeff0_4*dmats1[4][48] + coeff0_5*dmats1[5][48] + coeff0_6*dmats1[6][48] + coeff0_7*dmats1[7][48] + coeff0_8*dmats1[8][48] + coeff0_9*dmats1[9][48] + coeff0_10*dmats1[10][48] + coeff0_11*dmats1[11][48] + coeff0_12*dmats1[12][48] + coeff0_13*dmats1[13][48] + coeff0_14*dmats1[14][48] + coeff0_15*dmats1[15][48] + coeff0_16*dmats1[16][48] + coeff0_17*dmats1[17][48] + coeff0_18*dmats1[18][48] + coeff0_19*dmats1[19][48] + coeff0_20*dmats1[20][48] + coeff0_21*dmats1[21][48] + coeff0_22*dmats1[22][48] + coeff0_23*dmats1[23][48] + coeff0_24*dmats1[24][48] + coeff0_25*dmats1[25][48] + coeff0_26*dmats1[26][48] + coeff0_27*dmats1[27][48] + coeff0_28*dmats1[28][48] + coeff0_29*dmats1[29][48] + coeff0_30*dmats1[30][48] + coeff0_31*dmats1[31][48] + coeff0_32*dmats1[32][48] + coeff0_33*dmats1[33][48] + coeff0_34*dmats1[34][48] + coeff0_35*dmats1[35][48] + coeff0_36*dmats1[36][48] + coeff0_37*dmats1[37][48] + coeff0_38*dmats1[38][48] + coeff0_39*dmats1[39][48] + coeff0_40*dmats1[40][48] + coeff0_41*dmats1[41][48] + coeff0_42*dmats1[42][48] + coeff0_43*dmats1[43][48] + coeff0_44*dmats1[44][48] + coeff0_45*dmats1[45][48] + coeff0_46*dmats1[46][48] + coeff0_47*dmats1[47][48] + coeff0_48*dmats1[48][48] + coeff0_49*dmats1[49][48] + coeff0_50*dmats1[50][48] + coeff0_51*dmats1[51][48] + coeff0_52*dmats1[52][48] + coeff0_53*dmats1[53][48] + coeff0_54*dmats1[54][48] + coeff0_55*dmats1[55][48];
          new_coeff0_49 = coeff0_0*dmats1[0][49] + coeff0_1*dmats1[1][49] + coeff0_2*dmats1[2][49] + coeff0_3*dmats1[3][49] + coeff0_4*dmats1[4][49] + coeff0_5*dmats1[5][49] + coeff0_6*dmats1[6][49] + coeff0_7*dmats1[7][49] + coeff0_8*dmats1[8][49] + coeff0_9*dmats1[9][49] + coeff0_10*dmats1[10][49] + coeff0_11*dmats1[11][49] + coeff0_12*dmats1[12][49] + coeff0_13*dmats1[13][49] + coeff0_14*dmats1[14][49] + coeff0_15*dmats1[15][49] + coeff0_16*dmats1[16][49] + coeff0_17*dmats1[17][49] + coeff0_18*dmats1[18][49] + coeff0_19*dmats1[19][49] + coeff0_20*dmats1[20][49] + coeff0_21*dmats1[21][49] + coeff0_22*dmats1[22][49] + coeff0_23*dmats1[23][49] + coeff0_24*dmats1[24][49] + coeff0_25*dmats1[25][49] + coeff0_26*dmats1[26][49] + coeff0_27*dmats1[27][49] + coeff0_28*dmats1[28][49] + coeff0_29*dmats1[29][49] + coeff0_30*dmats1[30][49] + coeff0_31*dmats1[31][49] + coeff0_32*dmats1[32][49] + coeff0_33*dmats1[33][49] + coeff0_34*dmats1[34][49] + coeff0_35*dmats1[35][49] + coeff0_36*dmats1[36][49] + coeff0_37*dmats1[37][49] + coeff0_38*dmats1[38][49] + coeff0_39*dmats1[39][49] + coeff0_40*dmats1[40][49] + coeff0_41*dmats1[41][49] + coeff0_42*dmats1[42][49] + coeff0_43*dmats1[43][49] + coeff0_44*dmats1[44][49] + coeff0_45*dmats1[45][49] + coeff0_46*dmats1[46][49] + coeff0_47*dmats1[47][49] + coeff0_48*dmats1[48][49] + coeff0_49*dmats1[49][49] + coeff0_50*dmats1[50][49] + coeff0_51*dmats1[51][49] + coeff0_52*dmats1[52][49] + coeff0_53*dmats1[53][49] + coeff0_54*dmats1[54][49] + coeff0_55*dmats1[55][49];
          new_coeff0_50 = coeff0_0*dmats1[0][50] + coeff0_1*dmats1[1][50] + coeff0_2*dmats1[2][50] + coeff0_3*dmats1[3][50] + coeff0_4*dmats1[4][50] + coeff0_5*dmats1[5][50] + coeff0_6*dmats1[6][50] + coeff0_7*dmats1[7][50] + coeff0_8*dmats1[8][50] + coeff0_9*dmats1[9][50] + coeff0_10*dmats1[10][50] + coeff0_11*dmats1[11][50] + coeff0_12*dmats1[12][50] + coeff0_13*dmats1[13][50] + coeff0_14*dmats1[14][50] + coeff0_15*dmats1[15][50] + coeff0_16*dmats1[16][50] + coeff0_17*dmats1[17][50] + coeff0_18*dmats1[18][50] + coeff0_19*dmats1[19][50] + coeff0_20*dmats1[20][50] + coeff0_21*dmats1[21][50] + coeff0_22*dmats1[22][50] + coeff0_23*dmats1[23][50] + coeff0_24*dmats1[24][50] + coeff0_25*dmats1[25][50] + coeff0_26*dmats1[26][50] + coeff0_27*dmats1[27][50] + coeff0_28*dmats1[28][50] + coeff0_29*dmats1[29][50] + coeff0_30*dmats1[30][50] + coeff0_31*dmats1[31][50] + coeff0_32*dmats1[32][50] + coeff0_33*dmats1[33][50] + coeff0_34*dmats1[34][50] + coeff0_35*dmats1[35][50] + coeff0_36*dmats1[36][50] + coeff0_37*dmats1[37][50] + coeff0_38*dmats1[38][50] + coeff0_39*dmats1[39][50] + coeff0_40*dmats1[40][50] + coeff0_41*dmats1[41][50] + coeff0_42*dmats1[42][50] + coeff0_43*dmats1[43][50] + coeff0_44*dmats1[44][50] + coeff0_45*dmats1[45][50] + coeff0_46*dmats1[46][50] + coeff0_47*dmats1[47][50] + coeff0_48*dmats1[48][50] + coeff0_49*dmats1[49][50] + coeff0_50*dmats1[50][50] + coeff0_51*dmats1[51][50] + coeff0_52*dmats1[52][50] + coeff0_53*dmats1[53][50] + coeff0_54*dmats1[54][50] + coeff0_55*dmats1[55][50];
          new_coeff0_51 = coeff0_0*dmats1[0][51] + coeff0_1*dmats1[1][51] + coeff0_2*dmats1[2][51] + coeff0_3*dmats1[3][51] + coeff0_4*dmats1[4][51] + coeff0_5*dmats1[5][51] + coeff0_6*dmats1[6][51] + coeff0_7*dmats1[7][51] + coeff0_8*dmats1[8][51] + coeff0_9*dmats1[9][51] + coeff0_10*dmats1[10][51] + coeff0_11*dmats1[11][51] + coeff0_12*dmats1[12][51] + coeff0_13*dmats1[13][51] + coeff0_14*dmats1[14][51] + coeff0_15*dmats1[15][51] + coeff0_16*dmats1[16][51] + coeff0_17*dmats1[17][51] + coeff0_18*dmats1[18][51] + coeff0_19*dmats1[19][51] + coeff0_20*dmats1[20][51] + coeff0_21*dmats1[21][51] + coeff0_22*dmats1[22][51] + coeff0_23*dmats1[23][51] + coeff0_24*dmats1[24][51] + coeff0_25*dmats1[25][51] + coeff0_26*dmats1[26][51] + coeff0_27*dmats1[27][51] + coeff0_28*dmats1[28][51] + coeff0_29*dmats1[29][51] + coeff0_30*dmats1[30][51] + coeff0_31*dmats1[31][51] + coeff0_32*dmats1[32][51] + coeff0_33*dmats1[33][51] + coeff0_34*dmats1[34][51] + coeff0_35*dmats1[35][51] + coeff0_36*dmats1[36][51] + coeff0_37*dmats1[37][51] + coeff0_38*dmats1[38][51] + coeff0_39*dmats1[39][51] + coeff0_40*dmats1[40][51] + coeff0_41*dmats1[41][51] + coeff0_42*dmats1[42][51] + coeff0_43*dmats1[43][51] + coeff0_44*dmats1[44][51] + coeff0_45*dmats1[45][51] + coeff0_46*dmats1[46][51] + coeff0_47*dmats1[47][51] + coeff0_48*dmats1[48][51] + coeff0_49*dmats1[49][51] + coeff0_50*dmats1[50][51] + coeff0_51*dmats1[51][51] + coeff0_52*dmats1[52][51] + coeff0_53*dmats1[53][51] + coeff0_54*dmats1[54][51] + coeff0_55*dmats1[55][51];
          new_coeff0_52 = coeff0_0*dmats1[0][52] + coeff0_1*dmats1[1][52] + coeff0_2*dmats1[2][52] + coeff0_3*dmats1[3][52] + coeff0_4*dmats1[4][52] + coeff0_5*dmats1[5][52] + coeff0_6*dmats1[6][52] + coeff0_7*dmats1[7][52] + coeff0_8*dmats1[8][52] + coeff0_9*dmats1[9][52] + coeff0_10*dmats1[10][52] + coeff0_11*dmats1[11][52] + coeff0_12*dmats1[12][52] + coeff0_13*dmats1[13][52] + coeff0_14*dmats1[14][52] + coeff0_15*dmats1[15][52] + coeff0_16*dmats1[16][52] + coeff0_17*dmats1[17][52] + coeff0_18*dmats1[18][52] + coeff0_19*dmats1[19][52] + coeff0_20*dmats1[20][52] + coeff0_21*dmats1[21][52] + coeff0_22*dmats1[22][52] + coeff0_23*dmats1[23][52] + coeff0_24*dmats1[24][52] + coeff0_25*dmats1[25][52] + coeff0_26*dmats1[26][52] + coeff0_27*dmats1[27][52] + coeff0_28*dmats1[28][52] + coeff0_29*dmats1[29][52] + coeff0_30*dmats1[30][52] + coeff0_31*dmats1[31][52] + coeff0_32*dmats1[32][52] + coeff0_33*dmats1[33][52] + coeff0_34*dmats1[34][52] + coeff0_35*dmats1[35][52] + coeff0_36*dmats1[36][52] + coeff0_37*dmats1[37][52] + coeff0_38*dmats1[38][52] + coeff0_39*dmats1[39][52] + coeff0_40*dmats1[40][52] + coeff0_41*dmats1[41][52] + coeff0_42*dmats1[42][52] + coeff0_43*dmats1[43][52] + coeff0_44*dmats1[44][52] + coeff0_45*dmats1[45][52] + coeff0_46*dmats1[46][52] + coeff0_47*dmats1[47][52] + coeff0_48*dmats1[48][52] + coeff0_49*dmats1[49][52] + coeff0_50*dmats1[50][52] + coeff0_51*dmats1[51][52] + coeff0_52*dmats1[52][52] + coeff0_53*dmats1[53][52] + coeff0_54*dmats1[54][52] + coeff0_55*dmats1[55][52];
          new_coeff0_53 = coeff0_0*dmats1[0][53] + coeff0_1*dmats1[1][53] + coeff0_2*dmats1[2][53] + coeff0_3*dmats1[3][53] + coeff0_4*dmats1[4][53] + coeff0_5*dmats1[5][53] + coeff0_6*dmats1[6][53] + coeff0_7*dmats1[7][53] + coeff0_8*dmats1[8][53] + coeff0_9*dmats1[9][53] + coeff0_10*dmats1[10][53] + coeff0_11*dmats1[11][53] + coeff0_12*dmats1[12][53] + coeff0_13*dmats1[13][53] + coeff0_14*dmats1[14][53] + coeff0_15*dmats1[15][53] + coeff0_16*dmats1[16][53] + coeff0_17*dmats1[17][53] + coeff0_18*dmats1[18][53] + coeff0_19*dmats1[19][53] + coeff0_20*dmats1[20][53] + coeff0_21*dmats1[21][53] + coeff0_22*dmats1[22][53] + coeff0_23*dmats1[23][53] + coeff0_24*dmats1[24][53] + coeff0_25*dmats1[25][53] + coeff0_26*dmats1[26][53] + coeff0_27*dmats1[27][53] + coeff0_28*dmats1[28][53] + coeff0_29*dmats1[29][53] + coeff0_30*dmats1[30][53] + coeff0_31*dmats1[31][53] + coeff0_32*dmats1[32][53] + coeff0_33*dmats1[33][53] + coeff0_34*dmats1[34][53] + coeff0_35*dmats1[35][53] + coeff0_36*dmats1[36][53] + coeff0_37*dmats1[37][53] + coeff0_38*dmats1[38][53] + coeff0_39*dmats1[39][53] + coeff0_40*dmats1[40][53] + coeff0_41*dmats1[41][53] + coeff0_42*dmats1[42][53] + coeff0_43*dmats1[43][53] + coeff0_44*dmats1[44][53] + coeff0_45*dmats1[45][53] + coeff0_46*dmats1[46][53] + coeff0_47*dmats1[47][53] + coeff0_48*dmats1[48][53] + coeff0_49*dmats1[49][53] + coeff0_50*dmats1[50][53] + coeff0_51*dmats1[51][53] + coeff0_52*dmats1[52][53] + coeff0_53*dmats1[53][53] + coeff0_54*dmats1[54][53] + coeff0_55*dmats1[55][53];
          new_coeff0_54 = coeff0_0*dmats1[0][54] + coeff0_1*dmats1[1][54] + coeff0_2*dmats1[2][54] + coeff0_3*dmats1[3][54] + coeff0_4*dmats1[4][54] + coeff0_5*dmats1[5][54] + coeff0_6*dmats1[6][54] + coeff0_7*dmats1[7][54] + coeff0_8*dmats1[8][54] + coeff0_9*dmats1[9][54] + coeff0_10*dmats1[10][54] + coeff0_11*dmats1[11][54] + coeff0_12*dmats1[12][54] + coeff0_13*dmats1[13][54] + coeff0_14*dmats1[14][54] + coeff0_15*dmats1[15][54] + coeff0_16*dmats1[16][54] + coeff0_17*dmats1[17][54] + coeff0_18*dmats1[18][54] + coeff0_19*dmats1[19][54] + coeff0_20*dmats1[20][54] + coeff0_21*dmats1[21][54] + coeff0_22*dmats1[22][54] + coeff0_23*dmats1[23][54] + coeff0_24*dmats1[24][54] + coeff0_25*dmats1[25][54] + coeff0_26*dmats1[26][54] + coeff0_27*dmats1[27][54] + coeff0_28*dmats1[28][54] + coeff0_29*dmats1[29][54] + coeff0_30*dmats1[30][54] + coeff0_31*dmats1[31][54] + coeff0_32*dmats1[32][54] + coeff0_33*dmats1[33][54] + coeff0_34*dmats1[34][54] + coeff0_35*dmats1[35][54] + coeff0_36*dmats1[36][54] + coeff0_37*dmats1[37][54] + coeff0_38*dmats1[38][54] + coeff0_39*dmats1[39][54] + coeff0_40*dmats1[40][54] + coeff0_41*dmats1[41][54] + coeff0_42*dmats1[42][54] + coeff0_43*dmats1[43][54] + coeff0_44*dmats1[44][54] + coeff0_45*dmats1[45][54] + coeff0_46*dmats1[46][54] + coeff0_47*dmats1[47][54] + coeff0_48*dmats1[48][54] + coeff0_49*dmats1[49][54] + coeff0_50*dmats1[50][54] + coeff0_51*dmats1[51][54] + coeff0_52*dmats1[52][54] + coeff0_53*dmats1[53][54] + coeff0_54*dmats1[54][54] + coeff0_55*dmats1[55][54];
          new_coeff0_55 = coeff0_0*dmats1[0][55] + coeff0_1*dmats1[1][55] + coeff0_2*dmats1[2][55] + coeff0_3*dmats1[3][55] + coeff0_4*dmats1[4][55] + coeff0_5*dmats1[5][55] + coeff0_6*dmats1[6][55] + coeff0_7*dmats1[7][55] + coeff0_8*dmats1[8][55] + coeff0_9*dmats1[9][55] + coeff0_10*dmats1[10][55] + coeff0_11*dmats1[11][55] + coeff0_12*dmats1[12][55] + coeff0_13*dmats1[13][55] + coeff0_14*dmats1[14][55] + coeff0_15*dmats1[15][55] + coeff0_16*dmats1[16][55] + coeff0_17*dmats1[17][55] + coeff0_18*dmats1[18][55] + coeff0_19*dmats1[19][55] + coeff0_20*dmats1[20][55] + coeff0_21*dmats1[21][55] + coeff0_22*dmats1[22][55] + coeff0_23*dmats1[23][55] + coeff0_24*dmats1[24][55] + coeff0_25*dmats1[25][55] + coeff0_26*dmats1[26][55] + coeff0_27*dmats1[27][55] + coeff0_28*dmats1[28][55] + coeff0_29*dmats1[29][55] + coeff0_30*dmats1[30][55] + coeff0_31*dmats1[31][55] + coeff0_32*dmats1[32][55] + coeff0_33*dmats1[33][55] + coeff0_34*dmats1[34][55] + coeff0_35*dmats1[35][55] + coeff0_36*dmats1[36][55] + coeff0_37*dmats1[37][55] + coeff0_38*dmats1[38][55] + coeff0_39*dmats1[39][55] + coeff0_40*dmats1[40][55] + coeff0_41*dmats1[41][55] + coeff0_42*dmats1[42][55] + coeff0_43*dmats1[43][55] + coeff0_44*dmats1[44][55] + coeff0_45*dmats1[45][55] + coeff0_46*dmats1[46][55] + coeff0_47*dmats1[47][55] + coeff0_48*dmats1[48][55] + coeff0_49*dmats1[49][55] + coeff0_50*dmats1[50][55] + coeff0_51*dmats1[51][55] + coeff0_52*dmats1[52][55] + coeff0_53*dmats1[53][55] + coeff0_54*dmats1[54][55] + coeff0_55*dmats1[55][55];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0] + coeff0_4*dmats2[4][0] + coeff0_5*dmats2[5][0] + coeff0_6*dmats2[6][0] + coeff0_7*dmats2[7][0] + coeff0_8*dmats2[8][0] + coeff0_9*dmats2[9][0] + coeff0_10*dmats2[10][0] + coeff0_11*dmats2[11][0] + coeff0_12*dmats2[12][0] + coeff0_13*dmats2[13][0] + coeff0_14*dmats2[14][0] + coeff0_15*dmats2[15][0] + coeff0_16*dmats2[16][0] + coeff0_17*dmats2[17][0] + coeff0_18*dmats2[18][0] + coeff0_19*dmats2[19][0] + coeff0_20*dmats2[20][0] + coeff0_21*dmats2[21][0] + coeff0_22*dmats2[22][0] + coeff0_23*dmats2[23][0] + coeff0_24*dmats2[24][0] + coeff0_25*dmats2[25][0] + coeff0_26*dmats2[26][0] + coeff0_27*dmats2[27][0] + coeff0_28*dmats2[28][0] + coeff0_29*dmats2[29][0] + coeff0_30*dmats2[30][0] + coeff0_31*dmats2[31][0] + coeff0_32*dmats2[32][0] + coeff0_33*dmats2[33][0] + coeff0_34*dmats2[34][0] + coeff0_35*dmats2[35][0] + coeff0_36*dmats2[36][0] + coeff0_37*dmats2[37][0] + coeff0_38*dmats2[38][0] + coeff0_39*dmats2[39][0] + coeff0_40*dmats2[40][0] + coeff0_41*dmats2[41][0] + coeff0_42*dmats2[42][0] + coeff0_43*dmats2[43][0] + coeff0_44*dmats2[44][0] + coeff0_45*dmats2[45][0] + coeff0_46*dmats2[46][0] + coeff0_47*dmats2[47][0] + coeff0_48*dmats2[48][0] + coeff0_49*dmats2[49][0] + coeff0_50*dmats2[50][0] + coeff0_51*dmats2[51][0] + coeff0_52*dmats2[52][0] + coeff0_53*dmats2[53][0] + coeff0_54*dmats2[54][0] + coeff0_55*dmats2[55][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1] + coeff0_4*dmats2[4][1] + coeff0_5*dmats2[5][1] + coeff0_6*dmats2[6][1] + coeff0_7*dmats2[7][1] + coeff0_8*dmats2[8][1] + coeff0_9*dmats2[9][1] + coeff0_10*dmats2[10][1] + coeff0_11*dmats2[11][1] + coeff0_12*dmats2[12][1] + coeff0_13*dmats2[13][1] + coeff0_14*dmats2[14][1] + coeff0_15*dmats2[15][1] + coeff0_16*dmats2[16][1] + coeff0_17*dmats2[17][1] + coeff0_18*dmats2[18][1] + coeff0_19*dmats2[19][1] + coeff0_20*dmats2[20][1] + coeff0_21*dmats2[21][1] + coeff0_22*dmats2[22][1] + coeff0_23*dmats2[23][1] + coeff0_24*dmats2[24][1] + coeff0_25*dmats2[25][1] + coeff0_26*dmats2[26][1] + coeff0_27*dmats2[27][1] + coeff0_28*dmats2[28][1] + coeff0_29*dmats2[29][1] + coeff0_30*dmats2[30][1] + coeff0_31*dmats2[31][1] + coeff0_32*dmats2[32][1] + coeff0_33*dmats2[33][1] + coeff0_34*dmats2[34][1] + coeff0_35*dmats2[35][1] + coeff0_36*dmats2[36][1] + coeff0_37*dmats2[37][1] + coeff0_38*dmats2[38][1] + coeff0_39*dmats2[39][1] + coeff0_40*dmats2[40][1] + coeff0_41*dmats2[41][1] + coeff0_42*dmats2[42][1] + coeff0_43*dmats2[43][1] + coeff0_44*dmats2[44][1] + coeff0_45*dmats2[45][1] + coeff0_46*dmats2[46][1] + coeff0_47*dmats2[47][1] + coeff0_48*dmats2[48][1] + coeff0_49*dmats2[49][1] + coeff0_50*dmats2[50][1] + coeff0_51*dmats2[51][1] + coeff0_52*dmats2[52][1] + coeff0_53*dmats2[53][1] + coeff0_54*dmats2[54][1] + coeff0_55*dmats2[55][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2] + coeff0_4*dmats2[4][2] + coeff0_5*dmats2[5][2] + coeff0_6*dmats2[6][2] + coeff0_7*dmats2[7][2] + coeff0_8*dmats2[8][2] + coeff0_9*dmats2[9][2] + coeff0_10*dmats2[10][2] + coeff0_11*dmats2[11][2] + coeff0_12*dmats2[12][2] + coeff0_13*dmats2[13][2] + coeff0_14*dmats2[14][2] + coeff0_15*dmats2[15][2] + coeff0_16*dmats2[16][2] + coeff0_17*dmats2[17][2] + coeff0_18*dmats2[18][2] + coeff0_19*dmats2[19][2] + coeff0_20*dmats2[20][2] + coeff0_21*dmats2[21][2] + coeff0_22*dmats2[22][2] + coeff0_23*dmats2[23][2] + coeff0_24*dmats2[24][2] + coeff0_25*dmats2[25][2] + coeff0_26*dmats2[26][2] + coeff0_27*dmats2[27][2] + coeff0_28*dmats2[28][2] + coeff0_29*dmats2[29][2] + coeff0_30*dmats2[30][2] + coeff0_31*dmats2[31][2] + coeff0_32*dmats2[32][2] + coeff0_33*dmats2[33][2] + coeff0_34*dmats2[34][2] + coeff0_35*dmats2[35][2] + coeff0_36*dmats2[36][2] + coeff0_37*dmats2[37][2] + coeff0_38*dmats2[38][2] + coeff0_39*dmats2[39][2] + coeff0_40*dmats2[40][2] + coeff0_41*dmats2[41][2] + coeff0_42*dmats2[42][2] + coeff0_43*dmats2[43][2] + coeff0_44*dmats2[44][2] + coeff0_45*dmats2[45][2] + coeff0_46*dmats2[46][2] + coeff0_47*dmats2[47][2] + coeff0_48*dmats2[48][2] + coeff0_49*dmats2[49][2] + coeff0_50*dmats2[50][2] + coeff0_51*dmats2[51][2] + coeff0_52*dmats2[52][2] + coeff0_53*dmats2[53][2] + coeff0_54*dmats2[54][2] + coeff0_55*dmats2[55][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3] + coeff0_4*dmats2[4][3] + coeff0_5*dmats2[5][3] + coeff0_6*dmats2[6][3] + coeff0_7*dmats2[7][3] + coeff0_8*dmats2[8][3] + coeff0_9*dmats2[9][3] + coeff0_10*dmats2[10][3] + coeff0_11*dmats2[11][3] + coeff0_12*dmats2[12][3] + coeff0_13*dmats2[13][3] + coeff0_14*dmats2[14][3] + coeff0_15*dmats2[15][3] + coeff0_16*dmats2[16][3] + coeff0_17*dmats2[17][3] + coeff0_18*dmats2[18][3] + coeff0_19*dmats2[19][3] + coeff0_20*dmats2[20][3] + coeff0_21*dmats2[21][3] + coeff0_22*dmats2[22][3] + coeff0_23*dmats2[23][3] + coeff0_24*dmats2[24][3] + coeff0_25*dmats2[25][3] + coeff0_26*dmats2[26][3] + coeff0_27*dmats2[27][3] + coeff0_28*dmats2[28][3] + coeff0_29*dmats2[29][3] + coeff0_30*dmats2[30][3] + coeff0_31*dmats2[31][3] + coeff0_32*dmats2[32][3] + coeff0_33*dmats2[33][3] + coeff0_34*dmats2[34][3] + coeff0_35*dmats2[35][3] + coeff0_36*dmats2[36][3] + coeff0_37*dmats2[37][3] + coeff0_38*dmats2[38][3] + coeff0_39*dmats2[39][3] + coeff0_40*dmats2[40][3] + coeff0_41*dmats2[41][3] + coeff0_42*dmats2[42][3] + coeff0_43*dmats2[43][3] + coeff0_44*dmats2[44][3] + coeff0_45*dmats2[45][3] + coeff0_46*dmats2[46][3] + coeff0_47*dmats2[47][3] + coeff0_48*dmats2[48][3] + coeff0_49*dmats2[49][3] + coeff0_50*dmats2[50][3] + coeff0_51*dmats2[51][3] + coeff0_52*dmats2[52][3] + coeff0_53*dmats2[53][3] + coeff0_54*dmats2[54][3] + coeff0_55*dmats2[55][3];
          new_coeff0_4 = coeff0_0*dmats2[0][4] + coeff0_1*dmats2[1][4] + coeff0_2*dmats2[2][4] + coeff0_3*dmats2[3][4] + coeff0_4*dmats2[4][4] + coeff0_5*dmats2[5][4] + coeff0_6*dmats2[6][4] + coeff0_7*dmats2[7][4] + coeff0_8*dmats2[8][4] + coeff0_9*dmats2[9][4] + coeff0_10*dmats2[10][4] + coeff0_11*dmats2[11][4] + coeff0_12*dmats2[12][4] + coeff0_13*dmats2[13][4] + coeff0_14*dmats2[14][4] + coeff0_15*dmats2[15][4] + coeff0_16*dmats2[16][4] + coeff0_17*dmats2[17][4] + coeff0_18*dmats2[18][4] + coeff0_19*dmats2[19][4] + coeff0_20*dmats2[20][4] + coeff0_21*dmats2[21][4] + coeff0_22*dmats2[22][4] + coeff0_23*dmats2[23][4] + coeff0_24*dmats2[24][4] + coeff0_25*dmats2[25][4] + coeff0_26*dmats2[26][4] + coeff0_27*dmats2[27][4] + coeff0_28*dmats2[28][4] + coeff0_29*dmats2[29][4] + coeff0_30*dmats2[30][4] + coeff0_31*dmats2[31][4] + coeff0_32*dmats2[32][4] + coeff0_33*dmats2[33][4] + coeff0_34*dmats2[34][4] + coeff0_35*dmats2[35][4] + coeff0_36*dmats2[36][4] + coeff0_37*dmats2[37][4] + coeff0_38*dmats2[38][4] + coeff0_39*dmats2[39][4] + coeff0_40*dmats2[40][4] + coeff0_41*dmats2[41][4] + coeff0_42*dmats2[42][4] + coeff0_43*dmats2[43][4] + coeff0_44*dmats2[44][4] + coeff0_45*dmats2[45][4] + coeff0_46*dmats2[46][4] + coeff0_47*dmats2[47][4] + coeff0_48*dmats2[48][4] + coeff0_49*dmats2[49][4] + coeff0_50*dmats2[50][4] + coeff0_51*dmats2[51][4] + coeff0_52*dmats2[52][4] + coeff0_53*dmats2[53][4] + coeff0_54*dmats2[54][4] + coeff0_55*dmats2[55][4];
          new_coeff0_5 = coeff0_0*dmats2[0][5] + coeff0_1*dmats2[1][5] + coeff0_2*dmats2[2][5] + coeff0_3*dmats2[3][5] + coeff0_4*dmats2[4][5] + coeff0_5*dmats2[5][5] + coeff0_6*dmats2[6][5] + coeff0_7*dmats2[7][5] + coeff0_8*dmats2[8][5] + coeff0_9*dmats2[9][5] + coeff0_10*dmats2[10][5] + coeff0_11*dmats2[11][5] + coeff0_12*dmats2[12][5] + coeff0_13*dmats2[13][5] + coeff0_14*dmats2[14][5] + coeff0_15*dmats2[15][5] + coeff0_16*dmats2[16][5] + coeff0_17*dmats2[17][5] + coeff0_18*dmats2[18][5] + coeff0_19*dmats2[19][5] + coeff0_20*dmats2[20][5] + coeff0_21*dmats2[21][5] + coeff0_22*dmats2[22][5] + coeff0_23*dmats2[23][5] + coeff0_24*dmats2[24][5] + coeff0_25*dmats2[25][5] + coeff0_26*dmats2[26][5] + coeff0_27*dmats2[27][5] + coeff0_28*dmats2[28][5] + coeff0_29*dmats2[29][5] + coeff0_30*dmats2[30][5] + coeff0_31*dmats2[31][5] + coeff0_32*dmats2[32][5] + coeff0_33*dmats2[33][5] + coeff0_34*dmats2[34][5] + coeff0_35*dmats2[35][5] + coeff0_36*dmats2[36][5] + coeff0_37*dmats2[37][5] + coeff0_38*dmats2[38][5] + coeff0_39*dmats2[39][5] + coeff0_40*dmats2[40][5] + coeff0_41*dmats2[41][5] + coeff0_42*dmats2[42][5] + coeff0_43*dmats2[43][5] + coeff0_44*dmats2[44][5] + coeff0_45*dmats2[45][5] + coeff0_46*dmats2[46][5] + coeff0_47*dmats2[47][5] + coeff0_48*dmats2[48][5] + coeff0_49*dmats2[49][5] + coeff0_50*dmats2[50][5] + coeff0_51*dmats2[51][5] + coeff0_52*dmats2[52][5] + coeff0_53*dmats2[53][5] + coeff0_54*dmats2[54][5] + coeff0_55*dmats2[55][5];
          new_coeff0_6 = coeff0_0*dmats2[0][6] + coeff0_1*dmats2[1][6] + coeff0_2*dmats2[2][6] + coeff0_3*dmats2[3][6] + coeff0_4*dmats2[4][6] + coeff0_5*dmats2[5][6] + coeff0_6*dmats2[6][6] + coeff0_7*dmats2[7][6] + coeff0_8*dmats2[8][6] + coeff0_9*dmats2[9][6] + coeff0_10*dmats2[10][6] + coeff0_11*dmats2[11][6] + coeff0_12*dmats2[12][6] + coeff0_13*dmats2[13][6] + coeff0_14*dmats2[14][6] + coeff0_15*dmats2[15][6] + coeff0_16*dmats2[16][6] + coeff0_17*dmats2[17][6] + coeff0_18*dmats2[18][6] + coeff0_19*dmats2[19][6] + coeff0_20*dmats2[20][6] + coeff0_21*dmats2[21][6] + coeff0_22*dmats2[22][6] + coeff0_23*dmats2[23][6] + coeff0_24*dmats2[24][6] + coeff0_25*dmats2[25][6] + coeff0_26*dmats2[26][6] + coeff0_27*dmats2[27][6] + coeff0_28*dmats2[28][6] + coeff0_29*dmats2[29][6] + coeff0_30*dmats2[30][6] + coeff0_31*dmats2[31][6] + coeff0_32*dmats2[32][6] + coeff0_33*dmats2[33][6] + coeff0_34*dmats2[34][6] + coeff0_35*dmats2[35][6] + coeff0_36*dmats2[36][6] + coeff0_37*dmats2[37][6] + coeff0_38*dmats2[38][6] + coeff0_39*dmats2[39][6] + coeff0_40*dmats2[40][6] + coeff0_41*dmats2[41][6] + coeff0_42*dmats2[42][6] + coeff0_43*dmats2[43][6] + coeff0_44*dmats2[44][6] + coeff0_45*dmats2[45][6] + coeff0_46*dmats2[46][6] + coeff0_47*dmats2[47][6] + coeff0_48*dmats2[48][6] + coeff0_49*dmats2[49][6] + coeff0_50*dmats2[50][6] + coeff0_51*dmats2[51][6] + coeff0_52*dmats2[52][6] + coeff0_53*dmats2[53][6] + coeff0_54*dmats2[54][6] + coeff0_55*dmats2[55][6];
          new_coeff0_7 = coeff0_0*dmats2[0][7] + coeff0_1*dmats2[1][7] + coeff0_2*dmats2[2][7] + coeff0_3*dmats2[3][7] + coeff0_4*dmats2[4][7] + coeff0_5*dmats2[5][7] + coeff0_6*dmats2[6][7] + coeff0_7*dmats2[7][7] + coeff0_8*dmats2[8][7] + coeff0_9*dmats2[9][7] + coeff0_10*dmats2[10][7] + coeff0_11*dmats2[11][7] + coeff0_12*dmats2[12][7] + coeff0_13*dmats2[13][7] + coeff0_14*dmats2[14][7] + coeff0_15*dmats2[15][7] + coeff0_16*dmats2[16][7] + coeff0_17*dmats2[17][7] + coeff0_18*dmats2[18][7] + coeff0_19*dmats2[19][7] + coeff0_20*dmats2[20][7] + coeff0_21*dmats2[21][7] + coeff0_22*dmats2[22][7] + coeff0_23*dmats2[23][7] + coeff0_24*dmats2[24][7] + coeff0_25*dmats2[25][7] + coeff0_26*dmats2[26][7] + coeff0_27*dmats2[27][7] + coeff0_28*dmats2[28][7] + coeff0_29*dmats2[29][7] + coeff0_30*dmats2[30][7] + coeff0_31*dmats2[31][7] + coeff0_32*dmats2[32][7] + coeff0_33*dmats2[33][7] + coeff0_34*dmats2[34][7] + coeff0_35*dmats2[35][7] + coeff0_36*dmats2[36][7] + coeff0_37*dmats2[37][7] + coeff0_38*dmats2[38][7] + coeff0_39*dmats2[39][7] + coeff0_40*dmats2[40][7] + coeff0_41*dmats2[41][7] + coeff0_42*dmats2[42][7] + coeff0_43*dmats2[43][7] + coeff0_44*dmats2[44][7] + coeff0_45*dmats2[45][7] + coeff0_46*dmats2[46][7] + coeff0_47*dmats2[47][7] + coeff0_48*dmats2[48][7] + coeff0_49*dmats2[49][7] + coeff0_50*dmats2[50][7] + coeff0_51*dmats2[51][7] + coeff0_52*dmats2[52][7] + coeff0_53*dmats2[53][7] + coeff0_54*dmats2[54][7] + coeff0_55*dmats2[55][7];
          new_coeff0_8 = coeff0_0*dmats2[0][8] + coeff0_1*dmats2[1][8] + coeff0_2*dmats2[2][8] + coeff0_3*dmats2[3][8] + coeff0_4*dmats2[4][8] + coeff0_5*dmats2[5][8] + coeff0_6*dmats2[6][8] + coeff0_7*dmats2[7][8] + coeff0_8*dmats2[8][8] + coeff0_9*dmats2[9][8] + coeff0_10*dmats2[10][8] + coeff0_11*dmats2[11][8] + coeff0_12*dmats2[12][8] + coeff0_13*dmats2[13][8] + coeff0_14*dmats2[14][8] + coeff0_15*dmats2[15][8] + coeff0_16*dmats2[16][8] + coeff0_17*dmats2[17][8] + coeff0_18*dmats2[18][8] + coeff0_19*dmats2[19][8] + coeff0_20*dmats2[20][8] + coeff0_21*dmats2[21][8] + coeff0_22*dmats2[22][8] + coeff0_23*dmats2[23][8] + coeff0_24*dmats2[24][8] + coeff0_25*dmats2[25][8] + coeff0_26*dmats2[26][8] + coeff0_27*dmats2[27][8] + coeff0_28*dmats2[28][8] + coeff0_29*dmats2[29][8] + coeff0_30*dmats2[30][8] + coeff0_31*dmats2[31][8] + coeff0_32*dmats2[32][8] + coeff0_33*dmats2[33][8] + coeff0_34*dmats2[34][8] + coeff0_35*dmats2[35][8] + coeff0_36*dmats2[36][8] + coeff0_37*dmats2[37][8] + coeff0_38*dmats2[38][8] + coeff0_39*dmats2[39][8] + coeff0_40*dmats2[40][8] + coeff0_41*dmats2[41][8] + coeff0_42*dmats2[42][8] + coeff0_43*dmats2[43][8] + coeff0_44*dmats2[44][8] + coeff0_45*dmats2[45][8] + coeff0_46*dmats2[46][8] + coeff0_47*dmats2[47][8] + coeff0_48*dmats2[48][8] + coeff0_49*dmats2[49][8] + coeff0_50*dmats2[50][8] + coeff0_51*dmats2[51][8] + coeff0_52*dmats2[52][8] + coeff0_53*dmats2[53][8] + coeff0_54*dmats2[54][8] + coeff0_55*dmats2[55][8];
          new_coeff0_9 = coeff0_0*dmats2[0][9] + coeff0_1*dmats2[1][9] + coeff0_2*dmats2[2][9] + coeff0_3*dmats2[3][9] + coeff0_4*dmats2[4][9] + coeff0_5*dmats2[5][9] + coeff0_6*dmats2[6][9] + coeff0_7*dmats2[7][9] + coeff0_8*dmats2[8][9] + coeff0_9*dmats2[9][9] + coeff0_10*dmats2[10][9] + coeff0_11*dmats2[11][9] + coeff0_12*dmats2[12][9] + coeff0_13*dmats2[13][9] + coeff0_14*dmats2[14][9] + coeff0_15*dmats2[15][9] + coeff0_16*dmats2[16][9] + coeff0_17*dmats2[17][9] + coeff0_18*dmats2[18][9] + coeff0_19*dmats2[19][9] + coeff0_20*dmats2[20][9] + coeff0_21*dmats2[21][9] + coeff0_22*dmats2[22][9] + coeff0_23*dmats2[23][9] + coeff0_24*dmats2[24][9] + coeff0_25*dmats2[25][9] + coeff0_26*dmats2[26][9] + coeff0_27*dmats2[27][9] + coeff0_28*dmats2[28][9] + coeff0_29*dmats2[29][9] + coeff0_30*dmats2[30][9] + coeff0_31*dmats2[31][9] + coeff0_32*dmats2[32][9] + coeff0_33*dmats2[33][9] + coeff0_34*dmats2[34][9] + coeff0_35*dmats2[35][9] + coeff0_36*dmats2[36][9] + coeff0_37*dmats2[37][9] + coeff0_38*dmats2[38][9] + coeff0_39*dmats2[39][9] + coeff0_40*dmats2[40][9] + coeff0_41*dmats2[41][9] + coeff0_42*dmats2[42][9] + coeff0_43*dmats2[43][9] + coeff0_44*dmats2[44][9] + coeff0_45*dmats2[45][9] + coeff0_46*dmats2[46][9] + coeff0_47*dmats2[47][9] + coeff0_48*dmats2[48][9] + coeff0_49*dmats2[49][9] + coeff0_50*dmats2[50][9] + coeff0_51*dmats2[51][9] + coeff0_52*dmats2[52][9] + coeff0_53*dmats2[53][9] + coeff0_54*dmats2[54][9] + coeff0_55*dmats2[55][9];
          new_coeff0_10 = coeff0_0*dmats2[0][10] + coeff0_1*dmats2[1][10] + coeff0_2*dmats2[2][10] + coeff0_3*dmats2[3][10] + coeff0_4*dmats2[4][10] + coeff0_5*dmats2[5][10] + coeff0_6*dmats2[6][10] + coeff0_7*dmats2[7][10] + coeff0_8*dmats2[8][10] + coeff0_9*dmats2[9][10] + coeff0_10*dmats2[10][10] + coeff0_11*dmats2[11][10] + coeff0_12*dmats2[12][10] + coeff0_13*dmats2[13][10] + coeff0_14*dmats2[14][10] + coeff0_15*dmats2[15][10] + coeff0_16*dmats2[16][10] + coeff0_17*dmats2[17][10] + coeff0_18*dmats2[18][10] + coeff0_19*dmats2[19][10] + coeff0_20*dmats2[20][10] + coeff0_21*dmats2[21][10] + coeff0_22*dmats2[22][10] + coeff0_23*dmats2[23][10] + coeff0_24*dmats2[24][10] + coeff0_25*dmats2[25][10] + coeff0_26*dmats2[26][10] + coeff0_27*dmats2[27][10] + coeff0_28*dmats2[28][10] + coeff0_29*dmats2[29][10] + coeff0_30*dmats2[30][10] + coeff0_31*dmats2[31][10] + coeff0_32*dmats2[32][10] + coeff0_33*dmats2[33][10] + coeff0_34*dmats2[34][10] + coeff0_35*dmats2[35][10] + coeff0_36*dmats2[36][10] + coeff0_37*dmats2[37][10] + coeff0_38*dmats2[38][10] + coeff0_39*dmats2[39][10] + coeff0_40*dmats2[40][10] + coeff0_41*dmats2[41][10] + coeff0_42*dmats2[42][10] + coeff0_43*dmats2[43][10] + coeff0_44*dmats2[44][10] + coeff0_45*dmats2[45][10] + coeff0_46*dmats2[46][10] + coeff0_47*dmats2[47][10] + coeff0_48*dmats2[48][10] + coeff0_49*dmats2[49][10] + coeff0_50*dmats2[50][10] + coeff0_51*dmats2[51][10] + coeff0_52*dmats2[52][10] + coeff0_53*dmats2[53][10] + coeff0_54*dmats2[54][10] + coeff0_55*dmats2[55][10];
          new_coeff0_11 = coeff0_0*dmats2[0][11] + coeff0_1*dmats2[1][11] + coeff0_2*dmats2[2][11] + coeff0_3*dmats2[3][11] + coeff0_4*dmats2[4][11] + coeff0_5*dmats2[5][11] + coeff0_6*dmats2[6][11] + coeff0_7*dmats2[7][11] + coeff0_8*dmats2[8][11] + coeff0_9*dmats2[9][11] + coeff0_10*dmats2[10][11] + coeff0_11*dmats2[11][11] + coeff0_12*dmats2[12][11] + coeff0_13*dmats2[13][11] + coeff0_14*dmats2[14][11] + coeff0_15*dmats2[15][11] + coeff0_16*dmats2[16][11] + coeff0_17*dmats2[17][11] + coeff0_18*dmats2[18][11] + coeff0_19*dmats2[19][11] + coeff0_20*dmats2[20][11] + coeff0_21*dmats2[21][11] + coeff0_22*dmats2[22][11] + coeff0_23*dmats2[23][11] + coeff0_24*dmats2[24][11] + coeff0_25*dmats2[25][11] + coeff0_26*dmats2[26][11] + coeff0_27*dmats2[27][11] + coeff0_28*dmats2[28][11] + coeff0_29*dmats2[29][11] + coeff0_30*dmats2[30][11] + coeff0_31*dmats2[31][11] + coeff0_32*dmats2[32][11] + coeff0_33*dmats2[33][11] + coeff0_34*dmats2[34][11] + coeff0_35*dmats2[35][11] + coeff0_36*dmats2[36][11] + coeff0_37*dmats2[37][11] + coeff0_38*dmats2[38][11] + coeff0_39*dmats2[39][11] + coeff0_40*dmats2[40][11] + coeff0_41*dmats2[41][11] + coeff0_42*dmats2[42][11] + coeff0_43*dmats2[43][11] + coeff0_44*dmats2[44][11] + coeff0_45*dmats2[45][11] + coeff0_46*dmats2[46][11] + coeff0_47*dmats2[47][11] + coeff0_48*dmats2[48][11] + coeff0_49*dmats2[49][11] + coeff0_50*dmats2[50][11] + coeff0_51*dmats2[51][11] + coeff0_52*dmats2[52][11] + coeff0_53*dmats2[53][11] + coeff0_54*dmats2[54][11] + coeff0_55*dmats2[55][11];
          new_coeff0_12 = coeff0_0*dmats2[0][12] + coeff0_1*dmats2[1][12] + coeff0_2*dmats2[2][12] + coeff0_3*dmats2[3][12] + coeff0_4*dmats2[4][12] + coeff0_5*dmats2[5][12] + coeff0_6*dmats2[6][12] + coeff0_7*dmats2[7][12] + coeff0_8*dmats2[8][12] + coeff0_9*dmats2[9][12] + coeff0_10*dmats2[10][12] + coeff0_11*dmats2[11][12] + coeff0_12*dmats2[12][12] + coeff0_13*dmats2[13][12] + coeff0_14*dmats2[14][12] + coeff0_15*dmats2[15][12] + coeff0_16*dmats2[16][12] + coeff0_17*dmats2[17][12] + coeff0_18*dmats2[18][12] + coeff0_19*dmats2[19][12] + coeff0_20*dmats2[20][12] + coeff0_21*dmats2[21][12] + coeff0_22*dmats2[22][12] + coeff0_23*dmats2[23][12] + coeff0_24*dmats2[24][12] + coeff0_25*dmats2[25][12] + coeff0_26*dmats2[26][12] + coeff0_27*dmats2[27][12] + coeff0_28*dmats2[28][12] + coeff0_29*dmats2[29][12] + coeff0_30*dmats2[30][12] + coeff0_31*dmats2[31][12] + coeff0_32*dmats2[32][12] + coeff0_33*dmats2[33][12] + coeff0_34*dmats2[34][12] + coeff0_35*dmats2[35][12] + coeff0_36*dmats2[36][12] + coeff0_37*dmats2[37][12] + coeff0_38*dmats2[38][12] + coeff0_39*dmats2[39][12] + coeff0_40*dmats2[40][12] + coeff0_41*dmats2[41][12] + coeff0_42*dmats2[42][12] + coeff0_43*dmats2[43][12] + coeff0_44*dmats2[44][12] + coeff0_45*dmats2[45][12] + coeff0_46*dmats2[46][12] + coeff0_47*dmats2[47][12] + coeff0_48*dmats2[48][12] + coeff0_49*dmats2[49][12] + coeff0_50*dmats2[50][12] + coeff0_51*dmats2[51][12] + coeff0_52*dmats2[52][12] + coeff0_53*dmats2[53][12] + coeff0_54*dmats2[54][12] + coeff0_55*dmats2[55][12];
          new_coeff0_13 = coeff0_0*dmats2[0][13] + coeff0_1*dmats2[1][13] + coeff0_2*dmats2[2][13] + coeff0_3*dmats2[3][13] + coeff0_4*dmats2[4][13] + coeff0_5*dmats2[5][13] + coeff0_6*dmats2[6][13] + coeff0_7*dmats2[7][13] + coeff0_8*dmats2[8][13] + coeff0_9*dmats2[9][13] + coeff0_10*dmats2[10][13] + coeff0_11*dmats2[11][13] + coeff0_12*dmats2[12][13] + coeff0_13*dmats2[13][13] + coeff0_14*dmats2[14][13] + coeff0_15*dmats2[15][13] + coeff0_16*dmats2[16][13] + coeff0_17*dmats2[17][13] + coeff0_18*dmats2[18][13] + coeff0_19*dmats2[19][13] + coeff0_20*dmats2[20][13] + coeff0_21*dmats2[21][13] + coeff0_22*dmats2[22][13] + coeff0_23*dmats2[23][13] + coeff0_24*dmats2[24][13] + coeff0_25*dmats2[25][13] + coeff0_26*dmats2[26][13] + coeff0_27*dmats2[27][13] + coeff0_28*dmats2[28][13] + coeff0_29*dmats2[29][13] + coeff0_30*dmats2[30][13] + coeff0_31*dmats2[31][13] + coeff0_32*dmats2[32][13] + coeff0_33*dmats2[33][13] + coeff0_34*dmats2[34][13] + coeff0_35*dmats2[35][13] + coeff0_36*dmats2[36][13] + coeff0_37*dmats2[37][13] + coeff0_38*dmats2[38][13] + coeff0_39*dmats2[39][13] + coeff0_40*dmats2[40][13] + coeff0_41*dmats2[41][13] + coeff0_42*dmats2[42][13] + coeff0_43*dmats2[43][13] + coeff0_44*dmats2[44][13] + coeff0_45*dmats2[45][13] + coeff0_46*dmats2[46][13] + coeff0_47*dmats2[47][13] + coeff0_48*dmats2[48][13] + coeff0_49*dmats2[49][13] + coeff0_50*dmats2[50][13] + coeff0_51*dmats2[51][13] + coeff0_52*dmats2[52][13] + coeff0_53*dmats2[53][13] + coeff0_54*dmats2[54][13] + coeff0_55*dmats2[55][13];
          new_coeff0_14 = coeff0_0*dmats2[0][14] + coeff0_1*dmats2[1][14] + coeff0_2*dmats2[2][14] + coeff0_3*dmats2[3][14] + coeff0_4*dmats2[4][14] + coeff0_5*dmats2[5][14] + coeff0_6*dmats2[6][14] + coeff0_7*dmats2[7][14] + coeff0_8*dmats2[8][14] + coeff0_9*dmats2[9][14] + coeff0_10*dmats2[10][14] + coeff0_11*dmats2[11][14] + coeff0_12*dmats2[12][14] + coeff0_13*dmats2[13][14] + coeff0_14*dmats2[14][14] + coeff0_15*dmats2[15][14] + coeff0_16*dmats2[16][14] + coeff0_17*dmats2[17][14] + coeff0_18*dmats2[18][14] + coeff0_19*dmats2[19][14] + coeff0_20*dmats2[20][14] + coeff0_21*dmats2[21][14] + coeff0_22*dmats2[22][14] + coeff0_23*dmats2[23][14] + coeff0_24*dmats2[24][14] + coeff0_25*dmats2[25][14] + coeff0_26*dmats2[26][14] + coeff0_27*dmats2[27][14] + coeff0_28*dmats2[28][14] + coeff0_29*dmats2[29][14] + coeff0_30*dmats2[30][14] + coeff0_31*dmats2[31][14] + coeff0_32*dmats2[32][14] + coeff0_33*dmats2[33][14] + coeff0_34*dmats2[34][14] + coeff0_35*dmats2[35][14] + coeff0_36*dmats2[36][14] + coeff0_37*dmats2[37][14] + coeff0_38*dmats2[38][14] + coeff0_39*dmats2[39][14] + coeff0_40*dmats2[40][14] + coeff0_41*dmats2[41][14] + coeff0_42*dmats2[42][14] + coeff0_43*dmats2[43][14] + coeff0_44*dmats2[44][14] + coeff0_45*dmats2[45][14] + coeff0_46*dmats2[46][14] + coeff0_47*dmats2[47][14] + coeff0_48*dmats2[48][14] + coeff0_49*dmats2[49][14] + coeff0_50*dmats2[50][14] + coeff0_51*dmats2[51][14] + coeff0_52*dmats2[52][14] + coeff0_53*dmats2[53][14] + coeff0_54*dmats2[54][14] + coeff0_55*dmats2[55][14];
          new_coeff0_15 = coeff0_0*dmats2[0][15] + coeff0_1*dmats2[1][15] + coeff0_2*dmats2[2][15] + coeff0_3*dmats2[3][15] + coeff0_4*dmats2[4][15] + coeff0_5*dmats2[5][15] + coeff0_6*dmats2[6][15] + coeff0_7*dmats2[7][15] + coeff0_8*dmats2[8][15] + coeff0_9*dmats2[9][15] + coeff0_10*dmats2[10][15] + coeff0_11*dmats2[11][15] + coeff0_12*dmats2[12][15] + coeff0_13*dmats2[13][15] + coeff0_14*dmats2[14][15] + coeff0_15*dmats2[15][15] + coeff0_16*dmats2[16][15] + coeff0_17*dmats2[17][15] + coeff0_18*dmats2[18][15] + coeff0_19*dmats2[19][15] + coeff0_20*dmats2[20][15] + coeff0_21*dmats2[21][15] + coeff0_22*dmats2[22][15] + coeff0_23*dmats2[23][15] + coeff0_24*dmats2[24][15] + coeff0_25*dmats2[25][15] + coeff0_26*dmats2[26][15] + coeff0_27*dmats2[27][15] + coeff0_28*dmats2[28][15] + coeff0_29*dmats2[29][15] + coeff0_30*dmats2[30][15] + coeff0_31*dmats2[31][15] + coeff0_32*dmats2[32][15] + coeff0_33*dmats2[33][15] + coeff0_34*dmats2[34][15] + coeff0_35*dmats2[35][15] + coeff0_36*dmats2[36][15] + coeff0_37*dmats2[37][15] + coeff0_38*dmats2[38][15] + coeff0_39*dmats2[39][15] + coeff0_40*dmats2[40][15] + coeff0_41*dmats2[41][15] + coeff0_42*dmats2[42][15] + coeff0_43*dmats2[43][15] + coeff0_44*dmats2[44][15] + coeff0_45*dmats2[45][15] + coeff0_46*dmats2[46][15] + coeff0_47*dmats2[47][15] + coeff0_48*dmats2[48][15] + coeff0_49*dmats2[49][15] + coeff0_50*dmats2[50][15] + coeff0_51*dmats2[51][15] + coeff0_52*dmats2[52][15] + coeff0_53*dmats2[53][15] + coeff0_54*dmats2[54][15] + coeff0_55*dmats2[55][15];
          new_coeff0_16 = coeff0_0*dmats2[0][16] + coeff0_1*dmats2[1][16] + coeff0_2*dmats2[2][16] + coeff0_3*dmats2[3][16] + coeff0_4*dmats2[4][16] + coeff0_5*dmats2[5][16] + coeff0_6*dmats2[6][16] + coeff0_7*dmats2[7][16] + coeff0_8*dmats2[8][16] + coeff0_9*dmats2[9][16] + coeff0_10*dmats2[10][16] + coeff0_11*dmats2[11][16] + coeff0_12*dmats2[12][16] + coeff0_13*dmats2[13][16] + coeff0_14*dmats2[14][16] + coeff0_15*dmats2[15][16] + coeff0_16*dmats2[16][16] + coeff0_17*dmats2[17][16] + coeff0_18*dmats2[18][16] + coeff0_19*dmats2[19][16] + coeff0_20*dmats2[20][16] + coeff0_21*dmats2[21][16] + coeff0_22*dmats2[22][16] + coeff0_23*dmats2[23][16] + coeff0_24*dmats2[24][16] + coeff0_25*dmats2[25][16] + coeff0_26*dmats2[26][16] + coeff0_27*dmats2[27][16] + coeff0_28*dmats2[28][16] + coeff0_29*dmats2[29][16] + coeff0_30*dmats2[30][16] + coeff0_31*dmats2[31][16] + coeff0_32*dmats2[32][16] + coeff0_33*dmats2[33][16] + coeff0_34*dmats2[34][16] + coeff0_35*dmats2[35][16] + coeff0_36*dmats2[36][16] + coeff0_37*dmats2[37][16] + coeff0_38*dmats2[38][16] + coeff0_39*dmats2[39][16] + coeff0_40*dmats2[40][16] + coeff0_41*dmats2[41][16] + coeff0_42*dmats2[42][16] + coeff0_43*dmats2[43][16] + coeff0_44*dmats2[44][16] + coeff0_45*dmats2[45][16] + coeff0_46*dmats2[46][16] + coeff0_47*dmats2[47][16] + coeff0_48*dmats2[48][16] + coeff0_49*dmats2[49][16] + coeff0_50*dmats2[50][16] + coeff0_51*dmats2[51][16] + coeff0_52*dmats2[52][16] + coeff0_53*dmats2[53][16] + coeff0_54*dmats2[54][16] + coeff0_55*dmats2[55][16];
          new_coeff0_17 = coeff0_0*dmats2[0][17] + coeff0_1*dmats2[1][17] + coeff0_2*dmats2[2][17] + coeff0_3*dmats2[3][17] + coeff0_4*dmats2[4][17] + coeff0_5*dmats2[5][17] + coeff0_6*dmats2[6][17] + coeff0_7*dmats2[7][17] + coeff0_8*dmats2[8][17] + coeff0_9*dmats2[9][17] + coeff0_10*dmats2[10][17] + coeff0_11*dmats2[11][17] + coeff0_12*dmats2[12][17] + coeff0_13*dmats2[13][17] + coeff0_14*dmats2[14][17] + coeff0_15*dmats2[15][17] + coeff0_16*dmats2[16][17] + coeff0_17*dmats2[17][17] + coeff0_18*dmats2[18][17] + coeff0_19*dmats2[19][17] + coeff0_20*dmats2[20][17] + coeff0_21*dmats2[21][17] + coeff0_22*dmats2[22][17] + coeff0_23*dmats2[23][17] + coeff0_24*dmats2[24][17] + coeff0_25*dmats2[25][17] + coeff0_26*dmats2[26][17] + coeff0_27*dmats2[27][17] + coeff0_28*dmats2[28][17] + coeff0_29*dmats2[29][17] + coeff0_30*dmats2[30][17] + coeff0_31*dmats2[31][17] + coeff0_32*dmats2[32][17] + coeff0_33*dmats2[33][17] + coeff0_34*dmats2[34][17] + coeff0_35*dmats2[35][17] + coeff0_36*dmats2[36][17] + coeff0_37*dmats2[37][17] + coeff0_38*dmats2[38][17] + coeff0_39*dmats2[39][17] + coeff0_40*dmats2[40][17] + coeff0_41*dmats2[41][17] + coeff0_42*dmats2[42][17] + coeff0_43*dmats2[43][17] + coeff0_44*dmats2[44][17] + coeff0_45*dmats2[45][17] + coeff0_46*dmats2[46][17] + coeff0_47*dmats2[47][17] + coeff0_48*dmats2[48][17] + coeff0_49*dmats2[49][17] + coeff0_50*dmats2[50][17] + coeff0_51*dmats2[51][17] + coeff0_52*dmats2[52][17] + coeff0_53*dmats2[53][17] + coeff0_54*dmats2[54][17] + coeff0_55*dmats2[55][17];
          new_coeff0_18 = coeff0_0*dmats2[0][18] + coeff0_1*dmats2[1][18] + coeff0_2*dmats2[2][18] + coeff0_3*dmats2[3][18] + coeff0_4*dmats2[4][18] + coeff0_5*dmats2[5][18] + coeff0_6*dmats2[6][18] + coeff0_7*dmats2[7][18] + coeff0_8*dmats2[8][18] + coeff0_9*dmats2[9][18] + coeff0_10*dmats2[10][18] + coeff0_11*dmats2[11][18] + coeff0_12*dmats2[12][18] + coeff0_13*dmats2[13][18] + coeff0_14*dmats2[14][18] + coeff0_15*dmats2[15][18] + coeff0_16*dmats2[16][18] + coeff0_17*dmats2[17][18] + coeff0_18*dmats2[18][18] + coeff0_19*dmats2[19][18] + coeff0_20*dmats2[20][18] + coeff0_21*dmats2[21][18] + coeff0_22*dmats2[22][18] + coeff0_23*dmats2[23][18] + coeff0_24*dmats2[24][18] + coeff0_25*dmats2[25][18] + coeff0_26*dmats2[26][18] + coeff0_27*dmats2[27][18] + coeff0_28*dmats2[28][18] + coeff0_29*dmats2[29][18] + coeff0_30*dmats2[30][18] + coeff0_31*dmats2[31][18] + coeff0_32*dmats2[32][18] + coeff0_33*dmats2[33][18] + coeff0_34*dmats2[34][18] + coeff0_35*dmats2[35][18] + coeff0_36*dmats2[36][18] + coeff0_37*dmats2[37][18] + coeff0_38*dmats2[38][18] + coeff0_39*dmats2[39][18] + coeff0_40*dmats2[40][18] + coeff0_41*dmats2[41][18] + coeff0_42*dmats2[42][18] + coeff0_43*dmats2[43][18] + coeff0_44*dmats2[44][18] + coeff0_45*dmats2[45][18] + coeff0_46*dmats2[46][18] + coeff0_47*dmats2[47][18] + coeff0_48*dmats2[48][18] + coeff0_49*dmats2[49][18] + coeff0_50*dmats2[50][18] + coeff0_51*dmats2[51][18] + coeff0_52*dmats2[52][18] + coeff0_53*dmats2[53][18] + coeff0_54*dmats2[54][18] + coeff0_55*dmats2[55][18];
          new_coeff0_19 = coeff0_0*dmats2[0][19] + coeff0_1*dmats2[1][19] + coeff0_2*dmats2[2][19] + coeff0_3*dmats2[3][19] + coeff0_4*dmats2[4][19] + coeff0_5*dmats2[5][19] + coeff0_6*dmats2[6][19] + coeff0_7*dmats2[7][19] + coeff0_8*dmats2[8][19] + coeff0_9*dmats2[9][19] + coeff0_10*dmats2[10][19] + coeff0_11*dmats2[11][19] + coeff0_12*dmats2[12][19] + coeff0_13*dmats2[13][19] + coeff0_14*dmats2[14][19] + coeff0_15*dmats2[15][19] + coeff0_16*dmats2[16][19] + coeff0_17*dmats2[17][19] + coeff0_18*dmats2[18][19] + coeff0_19*dmats2[19][19] + coeff0_20*dmats2[20][19] + coeff0_21*dmats2[21][19] + coeff0_22*dmats2[22][19] + coeff0_23*dmats2[23][19] + coeff0_24*dmats2[24][19] + coeff0_25*dmats2[25][19] + coeff0_26*dmats2[26][19] + coeff0_27*dmats2[27][19] + coeff0_28*dmats2[28][19] + coeff0_29*dmats2[29][19] + coeff0_30*dmats2[30][19] + coeff0_31*dmats2[31][19] + coeff0_32*dmats2[32][19] + coeff0_33*dmats2[33][19] + coeff0_34*dmats2[34][19] + coeff0_35*dmats2[35][19] + coeff0_36*dmats2[36][19] + coeff0_37*dmats2[37][19] + coeff0_38*dmats2[38][19] + coeff0_39*dmats2[39][19] + coeff0_40*dmats2[40][19] + coeff0_41*dmats2[41][19] + coeff0_42*dmats2[42][19] + coeff0_43*dmats2[43][19] + coeff0_44*dmats2[44][19] + coeff0_45*dmats2[45][19] + coeff0_46*dmats2[46][19] + coeff0_47*dmats2[47][19] + coeff0_48*dmats2[48][19] + coeff0_49*dmats2[49][19] + coeff0_50*dmats2[50][19] + coeff0_51*dmats2[51][19] + coeff0_52*dmats2[52][19] + coeff0_53*dmats2[53][19] + coeff0_54*dmats2[54][19] + coeff0_55*dmats2[55][19];
          new_coeff0_20 = coeff0_0*dmats2[0][20] + coeff0_1*dmats2[1][20] + coeff0_2*dmats2[2][20] + coeff0_3*dmats2[3][20] + coeff0_4*dmats2[4][20] + coeff0_5*dmats2[5][20] + coeff0_6*dmats2[6][20] + coeff0_7*dmats2[7][20] + coeff0_8*dmats2[8][20] + coeff0_9*dmats2[9][20] + coeff0_10*dmats2[10][20] + coeff0_11*dmats2[11][20] + coeff0_12*dmats2[12][20] + coeff0_13*dmats2[13][20] + coeff0_14*dmats2[14][20] + coeff0_15*dmats2[15][20] + coeff0_16*dmats2[16][20] + coeff0_17*dmats2[17][20] + coeff0_18*dmats2[18][20] + coeff0_19*dmats2[19][20] + coeff0_20*dmats2[20][20] + coeff0_21*dmats2[21][20] + coeff0_22*dmats2[22][20] + coeff0_23*dmats2[23][20] + coeff0_24*dmats2[24][20] + coeff0_25*dmats2[25][20] + coeff0_26*dmats2[26][20] + coeff0_27*dmats2[27][20] + coeff0_28*dmats2[28][20] + coeff0_29*dmats2[29][20] + coeff0_30*dmats2[30][20] + coeff0_31*dmats2[31][20] + coeff0_32*dmats2[32][20] + coeff0_33*dmats2[33][20] + coeff0_34*dmats2[34][20] + coeff0_35*dmats2[35][20] + coeff0_36*dmats2[36][20] + coeff0_37*dmats2[37][20] + coeff0_38*dmats2[38][20] + coeff0_39*dmats2[39][20] + coeff0_40*dmats2[40][20] + coeff0_41*dmats2[41][20] + coeff0_42*dmats2[42][20] + coeff0_43*dmats2[43][20] + coeff0_44*dmats2[44][20] + coeff0_45*dmats2[45][20] + coeff0_46*dmats2[46][20] + coeff0_47*dmats2[47][20] + coeff0_48*dmats2[48][20] + coeff0_49*dmats2[49][20] + coeff0_50*dmats2[50][20] + coeff0_51*dmats2[51][20] + coeff0_52*dmats2[52][20] + coeff0_53*dmats2[53][20] + coeff0_54*dmats2[54][20] + coeff0_55*dmats2[55][20];
          new_coeff0_21 = coeff0_0*dmats2[0][21] + coeff0_1*dmats2[1][21] + coeff0_2*dmats2[2][21] + coeff0_3*dmats2[3][21] + coeff0_4*dmats2[4][21] + coeff0_5*dmats2[5][21] + coeff0_6*dmats2[6][21] + coeff0_7*dmats2[7][21] + coeff0_8*dmats2[8][21] + coeff0_9*dmats2[9][21] + coeff0_10*dmats2[10][21] + coeff0_11*dmats2[11][21] + coeff0_12*dmats2[12][21] + coeff0_13*dmats2[13][21] + coeff0_14*dmats2[14][21] + coeff0_15*dmats2[15][21] + coeff0_16*dmats2[16][21] + coeff0_17*dmats2[17][21] + coeff0_18*dmats2[18][21] + coeff0_19*dmats2[19][21] + coeff0_20*dmats2[20][21] + coeff0_21*dmats2[21][21] + coeff0_22*dmats2[22][21] + coeff0_23*dmats2[23][21] + coeff0_24*dmats2[24][21] + coeff0_25*dmats2[25][21] + coeff0_26*dmats2[26][21] + coeff0_27*dmats2[27][21] + coeff0_28*dmats2[28][21] + coeff0_29*dmats2[29][21] + coeff0_30*dmats2[30][21] + coeff0_31*dmats2[31][21] + coeff0_32*dmats2[32][21] + coeff0_33*dmats2[33][21] + coeff0_34*dmats2[34][21] + coeff0_35*dmats2[35][21] + coeff0_36*dmats2[36][21] + coeff0_37*dmats2[37][21] + coeff0_38*dmats2[38][21] + coeff0_39*dmats2[39][21] + coeff0_40*dmats2[40][21] + coeff0_41*dmats2[41][21] + coeff0_42*dmats2[42][21] + coeff0_43*dmats2[43][21] + coeff0_44*dmats2[44][21] + coeff0_45*dmats2[45][21] + coeff0_46*dmats2[46][21] + coeff0_47*dmats2[47][21] + coeff0_48*dmats2[48][21] + coeff0_49*dmats2[49][21] + coeff0_50*dmats2[50][21] + coeff0_51*dmats2[51][21] + coeff0_52*dmats2[52][21] + coeff0_53*dmats2[53][21] + coeff0_54*dmats2[54][21] + coeff0_55*dmats2[55][21];
          new_coeff0_22 = coeff0_0*dmats2[0][22] + coeff0_1*dmats2[1][22] + coeff0_2*dmats2[2][22] + coeff0_3*dmats2[3][22] + coeff0_4*dmats2[4][22] + coeff0_5*dmats2[5][22] + coeff0_6*dmats2[6][22] + coeff0_7*dmats2[7][22] + coeff0_8*dmats2[8][22] + coeff0_9*dmats2[9][22] + coeff0_10*dmats2[10][22] + coeff0_11*dmats2[11][22] + coeff0_12*dmats2[12][22] + coeff0_13*dmats2[13][22] + coeff0_14*dmats2[14][22] + coeff0_15*dmats2[15][22] + coeff0_16*dmats2[16][22] + coeff0_17*dmats2[17][22] + coeff0_18*dmats2[18][22] + coeff0_19*dmats2[19][22] + coeff0_20*dmats2[20][22] + coeff0_21*dmats2[21][22] + coeff0_22*dmats2[22][22] + coeff0_23*dmats2[23][22] + coeff0_24*dmats2[24][22] + coeff0_25*dmats2[25][22] + coeff0_26*dmats2[26][22] + coeff0_27*dmats2[27][22] + coeff0_28*dmats2[28][22] + coeff0_29*dmats2[29][22] + coeff0_30*dmats2[30][22] + coeff0_31*dmats2[31][22] + coeff0_32*dmats2[32][22] + coeff0_33*dmats2[33][22] + coeff0_34*dmats2[34][22] + coeff0_35*dmats2[35][22] + coeff0_36*dmats2[36][22] + coeff0_37*dmats2[37][22] + coeff0_38*dmats2[38][22] + coeff0_39*dmats2[39][22] + coeff0_40*dmats2[40][22] + coeff0_41*dmats2[41][22] + coeff0_42*dmats2[42][22] + coeff0_43*dmats2[43][22] + coeff0_44*dmats2[44][22] + coeff0_45*dmats2[45][22] + coeff0_46*dmats2[46][22] + coeff0_47*dmats2[47][22] + coeff0_48*dmats2[48][22] + coeff0_49*dmats2[49][22] + coeff0_50*dmats2[50][22] + coeff0_51*dmats2[51][22] + coeff0_52*dmats2[52][22] + coeff0_53*dmats2[53][22] + coeff0_54*dmats2[54][22] + coeff0_55*dmats2[55][22];
          new_coeff0_23 = coeff0_0*dmats2[0][23] + coeff0_1*dmats2[1][23] + coeff0_2*dmats2[2][23] + coeff0_3*dmats2[3][23] + coeff0_4*dmats2[4][23] + coeff0_5*dmats2[5][23] + coeff0_6*dmats2[6][23] + coeff0_7*dmats2[7][23] + coeff0_8*dmats2[8][23] + coeff0_9*dmats2[9][23] + coeff0_10*dmats2[10][23] + coeff0_11*dmats2[11][23] + coeff0_12*dmats2[12][23] + coeff0_13*dmats2[13][23] + coeff0_14*dmats2[14][23] + coeff0_15*dmats2[15][23] + coeff0_16*dmats2[16][23] + coeff0_17*dmats2[17][23] + coeff0_18*dmats2[18][23] + coeff0_19*dmats2[19][23] + coeff0_20*dmats2[20][23] + coeff0_21*dmats2[21][23] + coeff0_22*dmats2[22][23] + coeff0_23*dmats2[23][23] + coeff0_24*dmats2[24][23] + coeff0_25*dmats2[25][23] + coeff0_26*dmats2[26][23] + coeff0_27*dmats2[27][23] + coeff0_28*dmats2[28][23] + coeff0_29*dmats2[29][23] + coeff0_30*dmats2[30][23] + coeff0_31*dmats2[31][23] + coeff0_32*dmats2[32][23] + coeff0_33*dmats2[33][23] + coeff0_34*dmats2[34][23] + coeff0_35*dmats2[35][23] + coeff0_36*dmats2[36][23] + coeff0_37*dmats2[37][23] + coeff0_38*dmats2[38][23] + coeff0_39*dmats2[39][23] + coeff0_40*dmats2[40][23] + coeff0_41*dmats2[41][23] + coeff0_42*dmats2[42][23] + coeff0_43*dmats2[43][23] + coeff0_44*dmats2[44][23] + coeff0_45*dmats2[45][23] + coeff0_46*dmats2[46][23] + coeff0_47*dmats2[47][23] + coeff0_48*dmats2[48][23] + coeff0_49*dmats2[49][23] + coeff0_50*dmats2[50][23] + coeff0_51*dmats2[51][23] + coeff0_52*dmats2[52][23] + coeff0_53*dmats2[53][23] + coeff0_54*dmats2[54][23] + coeff0_55*dmats2[55][23];
          new_coeff0_24 = coeff0_0*dmats2[0][24] + coeff0_1*dmats2[1][24] + coeff0_2*dmats2[2][24] + coeff0_3*dmats2[3][24] + coeff0_4*dmats2[4][24] + coeff0_5*dmats2[5][24] + coeff0_6*dmats2[6][24] + coeff0_7*dmats2[7][24] + coeff0_8*dmats2[8][24] + coeff0_9*dmats2[9][24] + coeff0_10*dmats2[10][24] + coeff0_11*dmats2[11][24] + coeff0_12*dmats2[12][24] + coeff0_13*dmats2[13][24] + coeff0_14*dmats2[14][24] + coeff0_15*dmats2[15][24] + coeff0_16*dmats2[16][24] + coeff0_17*dmats2[17][24] + coeff0_18*dmats2[18][24] + coeff0_19*dmats2[19][24] + coeff0_20*dmats2[20][24] + coeff0_21*dmats2[21][24] + coeff0_22*dmats2[22][24] + coeff0_23*dmats2[23][24] + coeff0_24*dmats2[24][24] + coeff0_25*dmats2[25][24] + coeff0_26*dmats2[26][24] + coeff0_27*dmats2[27][24] + coeff0_28*dmats2[28][24] + coeff0_29*dmats2[29][24] + coeff0_30*dmats2[30][24] + coeff0_31*dmats2[31][24] + coeff0_32*dmats2[32][24] + coeff0_33*dmats2[33][24] + coeff0_34*dmats2[34][24] + coeff0_35*dmats2[35][24] + coeff0_36*dmats2[36][24] + coeff0_37*dmats2[37][24] + coeff0_38*dmats2[38][24] + coeff0_39*dmats2[39][24] + coeff0_40*dmats2[40][24] + coeff0_41*dmats2[41][24] + coeff0_42*dmats2[42][24] + coeff0_43*dmats2[43][24] + coeff0_44*dmats2[44][24] + coeff0_45*dmats2[45][24] + coeff0_46*dmats2[46][24] + coeff0_47*dmats2[47][24] + coeff0_48*dmats2[48][24] + coeff0_49*dmats2[49][24] + coeff0_50*dmats2[50][24] + coeff0_51*dmats2[51][24] + coeff0_52*dmats2[52][24] + coeff0_53*dmats2[53][24] + coeff0_54*dmats2[54][24] + coeff0_55*dmats2[55][24];
          new_coeff0_25 = coeff0_0*dmats2[0][25] + coeff0_1*dmats2[1][25] + coeff0_2*dmats2[2][25] + coeff0_3*dmats2[3][25] + coeff0_4*dmats2[4][25] + coeff0_5*dmats2[5][25] + coeff0_6*dmats2[6][25] + coeff0_7*dmats2[7][25] + coeff0_8*dmats2[8][25] + coeff0_9*dmats2[9][25] + coeff0_10*dmats2[10][25] + coeff0_11*dmats2[11][25] + coeff0_12*dmats2[12][25] + coeff0_13*dmats2[13][25] + coeff0_14*dmats2[14][25] + coeff0_15*dmats2[15][25] + coeff0_16*dmats2[16][25] + coeff0_17*dmats2[17][25] + coeff0_18*dmats2[18][25] + coeff0_19*dmats2[19][25] + coeff0_20*dmats2[20][25] + coeff0_21*dmats2[21][25] + coeff0_22*dmats2[22][25] + coeff0_23*dmats2[23][25] + coeff0_24*dmats2[24][25] + coeff0_25*dmats2[25][25] + coeff0_26*dmats2[26][25] + coeff0_27*dmats2[27][25] + coeff0_28*dmats2[28][25] + coeff0_29*dmats2[29][25] + coeff0_30*dmats2[30][25] + coeff0_31*dmats2[31][25] + coeff0_32*dmats2[32][25] + coeff0_33*dmats2[33][25] + coeff0_34*dmats2[34][25] + coeff0_35*dmats2[35][25] + coeff0_36*dmats2[36][25] + coeff0_37*dmats2[37][25] + coeff0_38*dmats2[38][25] + coeff0_39*dmats2[39][25] + coeff0_40*dmats2[40][25] + coeff0_41*dmats2[41][25] + coeff0_42*dmats2[42][25] + coeff0_43*dmats2[43][25] + coeff0_44*dmats2[44][25] + coeff0_45*dmats2[45][25] + coeff0_46*dmats2[46][25] + coeff0_47*dmats2[47][25] + coeff0_48*dmats2[48][25] + coeff0_49*dmats2[49][25] + coeff0_50*dmats2[50][25] + coeff0_51*dmats2[51][25] + coeff0_52*dmats2[52][25] + coeff0_53*dmats2[53][25] + coeff0_54*dmats2[54][25] + coeff0_55*dmats2[55][25];
          new_coeff0_26 = coeff0_0*dmats2[0][26] + coeff0_1*dmats2[1][26] + coeff0_2*dmats2[2][26] + coeff0_3*dmats2[3][26] + coeff0_4*dmats2[4][26] + coeff0_5*dmats2[5][26] + coeff0_6*dmats2[6][26] + coeff0_7*dmats2[7][26] + coeff0_8*dmats2[8][26] + coeff0_9*dmats2[9][26] + coeff0_10*dmats2[10][26] + coeff0_11*dmats2[11][26] + coeff0_12*dmats2[12][26] + coeff0_13*dmats2[13][26] + coeff0_14*dmats2[14][26] + coeff0_15*dmats2[15][26] + coeff0_16*dmats2[16][26] + coeff0_17*dmats2[17][26] + coeff0_18*dmats2[18][26] + coeff0_19*dmats2[19][26] + coeff0_20*dmats2[20][26] + coeff0_21*dmats2[21][26] + coeff0_22*dmats2[22][26] + coeff0_23*dmats2[23][26] + coeff0_24*dmats2[24][26] + coeff0_25*dmats2[25][26] + coeff0_26*dmats2[26][26] + coeff0_27*dmats2[27][26] + coeff0_28*dmats2[28][26] + coeff0_29*dmats2[29][26] + coeff0_30*dmats2[30][26] + coeff0_31*dmats2[31][26] + coeff0_32*dmats2[32][26] + coeff0_33*dmats2[33][26] + coeff0_34*dmats2[34][26] + coeff0_35*dmats2[35][26] + coeff0_36*dmats2[36][26] + coeff0_37*dmats2[37][26] + coeff0_38*dmats2[38][26] + coeff0_39*dmats2[39][26] + coeff0_40*dmats2[40][26] + coeff0_41*dmats2[41][26] + coeff0_42*dmats2[42][26] + coeff0_43*dmats2[43][26] + coeff0_44*dmats2[44][26] + coeff0_45*dmats2[45][26] + coeff0_46*dmats2[46][26] + coeff0_47*dmats2[47][26] + coeff0_48*dmats2[48][26] + coeff0_49*dmats2[49][26] + coeff0_50*dmats2[50][26] + coeff0_51*dmats2[51][26] + coeff0_52*dmats2[52][26] + coeff0_53*dmats2[53][26] + coeff0_54*dmats2[54][26] + coeff0_55*dmats2[55][26];
          new_coeff0_27 = coeff0_0*dmats2[0][27] + coeff0_1*dmats2[1][27] + coeff0_2*dmats2[2][27] + coeff0_3*dmats2[3][27] + coeff0_4*dmats2[4][27] + coeff0_5*dmats2[5][27] + coeff0_6*dmats2[6][27] + coeff0_7*dmats2[7][27] + coeff0_8*dmats2[8][27] + coeff0_9*dmats2[9][27] + coeff0_10*dmats2[10][27] + coeff0_11*dmats2[11][27] + coeff0_12*dmats2[12][27] + coeff0_13*dmats2[13][27] + coeff0_14*dmats2[14][27] + coeff0_15*dmats2[15][27] + coeff0_16*dmats2[16][27] + coeff0_17*dmats2[17][27] + coeff0_18*dmats2[18][27] + coeff0_19*dmats2[19][27] + coeff0_20*dmats2[20][27] + coeff0_21*dmats2[21][27] + coeff0_22*dmats2[22][27] + coeff0_23*dmats2[23][27] + coeff0_24*dmats2[24][27] + coeff0_25*dmats2[25][27] + coeff0_26*dmats2[26][27] + coeff0_27*dmats2[27][27] + coeff0_28*dmats2[28][27] + coeff0_29*dmats2[29][27] + coeff0_30*dmats2[30][27] + coeff0_31*dmats2[31][27] + coeff0_32*dmats2[32][27] + coeff0_33*dmats2[33][27] + coeff0_34*dmats2[34][27] + coeff0_35*dmats2[35][27] + coeff0_36*dmats2[36][27] + coeff0_37*dmats2[37][27] + coeff0_38*dmats2[38][27] + coeff0_39*dmats2[39][27] + coeff0_40*dmats2[40][27] + coeff0_41*dmats2[41][27] + coeff0_42*dmats2[42][27] + coeff0_43*dmats2[43][27] + coeff0_44*dmats2[44][27] + coeff0_45*dmats2[45][27] + coeff0_46*dmats2[46][27] + coeff0_47*dmats2[47][27] + coeff0_48*dmats2[48][27] + coeff0_49*dmats2[49][27] + coeff0_50*dmats2[50][27] + coeff0_51*dmats2[51][27] + coeff0_52*dmats2[52][27] + coeff0_53*dmats2[53][27] + coeff0_54*dmats2[54][27] + coeff0_55*dmats2[55][27];
          new_coeff0_28 = coeff0_0*dmats2[0][28] + coeff0_1*dmats2[1][28] + coeff0_2*dmats2[2][28] + coeff0_3*dmats2[3][28] + coeff0_4*dmats2[4][28] + coeff0_5*dmats2[5][28] + coeff0_6*dmats2[6][28] + coeff0_7*dmats2[7][28] + coeff0_8*dmats2[8][28] + coeff0_9*dmats2[9][28] + coeff0_10*dmats2[10][28] + coeff0_11*dmats2[11][28] + coeff0_12*dmats2[12][28] + coeff0_13*dmats2[13][28] + coeff0_14*dmats2[14][28] + coeff0_15*dmats2[15][28] + coeff0_16*dmats2[16][28] + coeff0_17*dmats2[17][28] + coeff0_18*dmats2[18][28] + coeff0_19*dmats2[19][28] + coeff0_20*dmats2[20][28] + coeff0_21*dmats2[21][28] + coeff0_22*dmats2[22][28] + coeff0_23*dmats2[23][28] + coeff0_24*dmats2[24][28] + coeff0_25*dmats2[25][28] + coeff0_26*dmats2[26][28] + coeff0_27*dmats2[27][28] + coeff0_28*dmats2[28][28] + coeff0_29*dmats2[29][28] + coeff0_30*dmats2[30][28] + coeff0_31*dmats2[31][28] + coeff0_32*dmats2[32][28] + coeff0_33*dmats2[33][28] + coeff0_34*dmats2[34][28] + coeff0_35*dmats2[35][28] + coeff0_36*dmats2[36][28] + coeff0_37*dmats2[37][28] + coeff0_38*dmats2[38][28] + coeff0_39*dmats2[39][28] + coeff0_40*dmats2[40][28] + coeff0_41*dmats2[41][28] + coeff0_42*dmats2[42][28] + coeff0_43*dmats2[43][28] + coeff0_44*dmats2[44][28] + coeff0_45*dmats2[45][28] + coeff0_46*dmats2[46][28] + coeff0_47*dmats2[47][28] + coeff0_48*dmats2[48][28] + coeff0_49*dmats2[49][28] + coeff0_50*dmats2[50][28] + coeff0_51*dmats2[51][28] + coeff0_52*dmats2[52][28] + coeff0_53*dmats2[53][28] + coeff0_54*dmats2[54][28] + coeff0_55*dmats2[55][28];
          new_coeff0_29 = coeff0_0*dmats2[0][29] + coeff0_1*dmats2[1][29] + coeff0_2*dmats2[2][29] + coeff0_3*dmats2[3][29] + coeff0_4*dmats2[4][29] + coeff0_5*dmats2[5][29] + coeff0_6*dmats2[6][29] + coeff0_7*dmats2[7][29] + coeff0_8*dmats2[8][29] + coeff0_9*dmats2[9][29] + coeff0_10*dmats2[10][29] + coeff0_11*dmats2[11][29] + coeff0_12*dmats2[12][29] + coeff0_13*dmats2[13][29] + coeff0_14*dmats2[14][29] + coeff0_15*dmats2[15][29] + coeff0_16*dmats2[16][29] + coeff0_17*dmats2[17][29] + coeff0_18*dmats2[18][29] + coeff0_19*dmats2[19][29] + coeff0_20*dmats2[20][29] + coeff0_21*dmats2[21][29] + coeff0_22*dmats2[22][29] + coeff0_23*dmats2[23][29] + coeff0_24*dmats2[24][29] + coeff0_25*dmats2[25][29] + coeff0_26*dmats2[26][29] + coeff0_27*dmats2[27][29] + coeff0_28*dmats2[28][29] + coeff0_29*dmats2[29][29] + coeff0_30*dmats2[30][29] + coeff0_31*dmats2[31][29] + coeff0_32*dmats2[32][29] + coeff0_33*dmats2[33][29] + coeff0_34*dmats2[34][29] + coeff0_35*dmats2[35][29] + coeff0_36*dmats2[36][29] + coeff0_37*dmats2[37][29] + coeff0_38*dmats2[38][29] + coeff0_39*dmats2[39][29] + coeff0_40*dmats2[40][29] + coeff0_41*dmats2[41][29] + coeff0_42*dmats2[42][29] + coeff0_43*dmats2[43][29] + coeff0_44*dmats2[44][29] + coeff0_45*dmats2[45][29] + coeff0_46*dmats2[46][29] + coeff0_47*dmats2[47][29] + coeff0_48*dmats2[48][29] + coeff0_49*dmats2[49][29] + coeff0_50*dmats2[50][29] + coeff0_51*dmats2[51][29] + coeff0_52*dmats2[52][29] + coeff0_53*dmats2[53][29] + coeff0_54*dmats2[54][29] + coeff0_55*dmats2[55][29];
          new_coeff0_30 = coeff0_0*dmats2[0][30] + coeff0_1*dmats2[1][30] + coeff0_2*dmats2[2][30] + coeff0_3*dmats2[3][30] + coeff0_4*dmats2[4][30] + coeff0_5*dmats2[5][30] + coeff0_6*dmats2[6][30] + coeff0_7*dmats2[7][30] + coeff0_8*dmats2[8][30] + coeff0_9*dmats2[9][30] + coeff0_10*dmats2[10][30] + coeff0_11*dmats2[11][30] + coeff0_12*dmats2[12][30] + coeff0_13*dmats2[13][30] + coeff0_14*dmats2[14][30] + coeff0_15*dmats2[15][30] + coeff0_16*dmats2[16][30] + coeff0_17*dmats2[17][30] + coeff0_18*dmats2[18][30] + coeff0_19*dmats2[19][30] + coeff0_20*dmats2[20][30] + coeff0_21*dmats2[21][30] + coeff0_22*dmats2[22][30] + coeff0_23*dmats2[23][30] + coeff0_24*dmats2[24][30] + coeff0_25*dmats2[25][30] + coeff0_26*dmats2[26][30] + coeff0_27*dmats2[27][30] + coeff0_28*dmats2[28][30] + coeff0_29*dmats2[29][30] + coeff0_30*dmats2[30][30] + coeff0_31*dmats2[31][30] + coeff0_32*dmats2[32][30] + coeff0_33*dmats2[33][30] + coeff0_34*dmats2[34][30] + coeff0_35*dmats2[35][30] + coeff0_36*dmats2[36][30] + coeff0_37*dmats2[37][30] + coeff0_38*dmats2[38][30] + coeff0_39*dmats2[39][30] + coeff0_40*dmats2[40][30] + coeff0_41*dmats2[41][30] + coeff0_42*dmats2[42][30] + coeff0_43*dmats2[43][30] + coeff0_44*dmats2[44][30] + coeff0_45*dmats2[45][30] + coeff0_46*dmats2[46][30] + coeff0_47*dmats2[47][30] + coeff0_48*dmats2[48][30] + coeff0_49*dmats2[49][30] + coeff0_50*dmats2[50][30] + coeff0_51*dmats2[51][30] + coeff0_52*dmats2[52][30] + coeff0_53*dmats2[53][30] + coeff0_54*dmats2[54][30] + coeff0_55*dmats2[55][30];
          new_coeff0_31 = coeff0_0*dmats2[0][31] + coeff0_1*dmats2[1][31] + coeff0_2*dmats2[2][31] + coeff0_3*dmats2[3][31] + coeff0_4*dmats2[4][31] + coeff0_5*dmats2[5][31] + coeff0_6*dmats2[6][31] + coeff0_7*dmats2[7][31] + coeff0_8*dmats2[8][31] + coeff0_9*dmats2[9][31] + coeff0_10*dmats2[10][31] + coeff0_11*dmats2[11][31] + coeff0_12*dmats2[12][31] + coeff0_13*dmats2[13][31] + coeff0_14*dmats2[14][31] + coeff0_15*dmats2[15][31] + coeff0_16*dmats2[16][31] + coeff0_17*dmats2[17][31] + coeff0_18*dmats2[18][31] + coeff0_19*dmats2[19][31] + coeff0_20*dmats2[20][31] + coeff0_21*dmats2[21][31] + coeff0_22*dmats2[22][31] + coeff0_23*dmats2[23][31] + coeff0_24*dmats2[24][31] + coeff0_25*dmats2[25][31] + coeff0_26*dmats2[26][31] + coeff0_27*dmats2[27][31] + coeff0_28*dmats2[28][31] + coeff0_29*dmats2[29][31] + coeff0_30*dmats2[30][31] + coeff0_31*dmats2[31][31] + coeff0_32*dmats2[32][31] + coeff0_33*dmats2[33][31] + coeff0_34*dmats2[34][31] + coeff0_35*dmats2[35][31] + coeff0_36*dmats2[36][31] + coeff0_37*dmats2[37][31] + coeff0_38*dmats2[38][31] + coeff0_39*dmats2[39][31] + coeff0_40*dmats2[40][31] + coeff0_41*dmats2[41][31] + coeff0_42*dmats2[42][31] + coeff0_43*dmats2[43][31] + coeff0_44*dmats2[44][31] + coeff0_45*dmats2[45][31] + coeff0_46*dmats2[46][31] + coeff0_47*dmats2[47][31] + coeff0_48*dmats2[48][31] + coeff0_49*dmats2[49][31] + coeff0_50*dmats2[50][31] + coeff0_51*dmats2[51][31] + coeff0_52*dmats2[52][31] + coeff0_53*dmats2[53][31] + coeff0_54*dmats2[54][31] + coeff0_55*dmats2[55][31];
          new_coeff0_32 = coeff0_0*dmats2[0][32] + coeff0_1*dmats2[1][32] + coeff0_2*dmats2[2][32] + coeff0_3*dmats2[3][32] + coeff0_4*dmats2[4][32] + coeff0_5*dmats2[5][32] + coeff0_6*dmats2[6][32] + coeff0_7*dmats2[7][32] + coeff0_8*dmats2[8][32] + coeff0_9*dmats2[9][32] + coeff0_10*dmats2[10][32] + coeff0_11*dmats2[11][32] + coeff0_12*dmats2[12][32] + coeff0_13*dmats2[13][32] + coeff0_14*dmats2[14][32] + coeff0_15*dmats2[15][32] + coeff0_16*dmats2[16][32] + coeff0_17*dmats2[17][32] + coeff0_18*dmats2[18][32] + coeff0_19*dmats2[19][32] + coeff0_20*dmats2[20][32] + coeff0_21*dmats2[21][32] + coeff0_22*dmats2[22][32] + coeff0_23*dmats2[23][32] + coeff0_24*dmats2[24][32] + coeff0_25*dmats2[25][32] + coeff0_26*dmats2[26][32] + coeff0_27*dmats2[27][32] + coeff0_28*dmats2[28][32] + coeff0_29*dmats2[29][32] + coeff0_30*dmats2[30][32] + coeff0_31*dmats2[31][32] + coeff0_32*dmats2[32][32] + coeff0_33*dmats2[33][32] + coeff0_34*dmats2[34][32] + coeff0_35*dmats2[35][32] + coeff0_36*dmats2[36][32] + coeff0_37*dmats2[37][32] + coeff0_38*dmats2[38][32] + coeff0_39*dmats2[39][32] + coeff0_40*dmats2[40][32] + coeff0_41*dmats2[41][32] + coeff0_42*dmats2[42][32] + coeff0_43*dmats2[43][32] + coeff0_44*dmats2[44][32] + coeff0_45*dmats2[45][32] + coeff0_46*dmats2[46][32] + coeff0_47*dmats2[47][32] + coeff0_48*dmats2[48][32] + coeff0_49*dmats2[49][32] + coeff0_50*dmats2[50][32] + coeff0_51*dmats2[51][32] + coeff0_52*dmats2[52][32] + coeff0_53*dmats2[53][32] + coeff0_54*dmats2[54][32] + coeff0_55*dmats2[55][32];
          new_coeff0_33 = coeff0_0*dmats2[0][33] + coeff0_1*dmats2[1][33] + coeff0_2*dmats2[2][33] + coeff0_3*dmats2[3][33] + coeff0_4*dmats2[4][33] + coeff0_5*dmats2[5][33] + coeff0_6*dmats2[6][33] + coeff0_7*dmats2[7][33] + coeff0_8*dmats2[8][33] + coeff0_9*dmats2[9][33] + coeff0_10*dmats2[10][33] + coeff0_11*dmats2[11][33] + coeff0_12*dmats2[12][33] + coeff0_13*dmats2[13][33] + coeff0_14*dmats2[14][33] + coeff0_15*dmats2[15][33] + coeff0_16*dmats2[16][33] + coeff0_17*dmats2[17][33] + coeff0_18*dmats2[18][33] + coeff0_19*dmats2[19][33] + coeff0_20*dmats2[20][33] + coeff0_21*dmats2[21][33] + coeff0_22*dmats2[22][33] + coeff0_23*dmats2[23][33] + coeff0_24*dmats2[24][33] + coeff0_25*dmats2[25][33] + coeff0_26*dmats2[26][33] + coeff0_27*dmats2[27][33] + coeff0_28*dmats2[28][33] + coeff0_29*dmats2[29][33] + coeff0_30*dmats2[30][33] + coeff0_31*dmats2[31][33] + coeff0_32*dmats2[32][33] + coeff0_33*dmats2[33][33] + coeff0_34*dmats2[34][33] + coeff0_35*dmats2[35][33] + coeff0_36*dmats2[36][33] + coeff0_37*dmats2[37][33] + coeff0_38*dmats2[38][33] + coeff0_39*dmats2[39][33] + coeff0_40*dmats2[40][33] + coeff0_41*dmats2[41][33] + coeff0_42*dmats2[42][33] + coeff0_43*dmats2[43][33] + coeff0_44*dmats2[44][33] + coeff0_45*dmats2[45][33] + coeff0_46*dmats2[46][33] + coeff0_47*dmats2[47][33] + coeff0_48*dmats2[48][33] + coeff0_49*dmats2[49][33] + coeff0_50*dmats2[50][33] + coeff0_51*dmats2[51][33] + coeff0_52*dmats2[52][33] + coeff0_53*dmats2[53][33] + coeff0_54*dmats2[54][33] + coeff0_55*dmats2[55][33];
          new_coeff0_34 = coeff0_0*dmats2[0][34] + coeff0_1*dmats2[1][34] + coeff0_2*dmats2[2][34] + coeff0_3*dmats2[3][34] + coeff0_4*dmats2[4][34] + coeff0_5*dmats2[5][34] + coeff0_6*dmats2[6][34] + coeff0_7*dmats2[7][34] + coeff0_8*dmats2[8][34] + coeff0_9*dmats2[9][34] + coeff0_10*dmats2[10][34] + coeff0_11*dmats2[11][34] + coeff0_12*dmats2[12][34] + coeff0_13*dmats2[13][34] + coeff0_14*dmats2[14][34] + coeff0_15*dmats2[15][34] + coeff0_16*dmats2[16][34] + coeff0_17*dmats2[17][34] + coeff0_18*dmats2[18][34] + coeff0_19*dmats2[19][34] + coeff0_20*dmats2[20][34] + coeff0_21*dmats2[21][34] + coeff0_22*dmats2[22][34] + coeff0_23*dmats2[23][34] + coeff0_24*dmats2[24][34] + coeff0_25*dmats2[25][34] + coeff0_26*dmats2[26][34] + coeff0_27*dmats2[27][34] + coeff0_28*dmats2[28][34] + coeff0_29*dmats2[29][34] + coeff0_30*dmats2[30][34] + coeff0_31*dmats2[31][34] + coeff0_32*dmats2[32][34] + coeff0_33*dmats2[33][34] + coeff0_34*dmats2[34][34] + coeff0_35*dmats2[35][34] + coeff0_36*dmats2[36][34] + coeff0_37*dmats2[37][34] + coeff0_38*dmats2[38][34] + coeff0_39*dmats2[39][34] + coeff0_40*dmats2[40][34] + coeff0_41*dmats2[41][34] + coeff0_42*dmats2[42][34] + coeff0_43*dmats2[43][34] + coeff0_44*dmats2[44][34] + coeff0_45*dmats2[45][34] + coeff0_46*dmats2[46][34] + coeff0_47*dmats2[47][34] + coeff0_48*dmats2[48][34] + coeff0_49*dmats2[49][34] + coeff0_50*dmats2[50][34] + coeff0_51*dmats2[51][34] + coeff0_52*dmats2[52][34] + coeff0_53*dmats2[53][34] + coeff0_54*dmats2[54][34] + coeff0_55*dmats2[55][34];
          new_coeff0_35 = coeff0_0*dmats2[0][35] + coeff0_1*dmats2[1][35] + coeff0_2*dmats2[2][35] + coeff0_3*dmats2[3][35] + coeff0_4*dmats2[4][35] + coeff0_5*dmats2[5][35] + coeff0_6*dmats2[6][35] + coeff0_7*dmats2[7][35] + coeff0_8*dmats2[8][35] + coeff0_9*dmats2[9][35] + coeff0_10*dmats2[10][35] + coeff0_11*dmats2[11][35] + coeff0_12*dmats2[12][35] + coeff0_13*dmats2[13][35] + coeff0_14*dmats2[14][35] + coeff0_15*dmats2[15][35] + coeff0_16*dmats2[16][35] + coeff0_17*dmats2[17][35] + coeff0_18*dmats2[18][35] + coeff0_19*dmats2[19][35] + coeff0_20*dmats2[20][35] + coeff0_21*dmats2[21][35] + coeff0_22*dmats2[22][35] + coeff0_23*dmats2[23][35] + coeff0_24*dmats2[24][35] + coeff0_25*dmats2[25][35] + coeff0_26*dmats2[26][35] + coeff0_27*dmats2[27][35] + coeff0_28*dmats2[28][35] + coeff0_29*dmats2[29][35] + coeff0_30*dmats2[30][35] + coeff0_31*dmats2[31][35] + coeff0_32*dmats2[32][35] + coeff0_33*dmats2[33][35] + coeff0_34*dmats2[34][35] + coeff0_35*dmats2[35][35] + coeff0_36*dmats2[36][35] + coeff0_37*dmats2[37][35] + coeff0_38*dmats2[38][35] + coeff0_39*dmats2[39][35] + coeff0_40*dmats2[40][35] + coeff0_41*dmats2[41][35] + coeff0_42*dmats2[42][35] + coeff0_43*dmats2[43][35] + coeff0_44*dmats2[44][35] + coeff0_45*dmats2[45][35] + coeff0_46*dmats2[46][35] + coeff0_47*dmats2[47][35] + coeff0_48*dmats2[48][35] + coeff0_49*dmats2[49][35] + coeff0_50*dmats2[50][35] + coeff0_51*dmats2[51][35] + coeff0_52*dmats2[52][35] + coeff0_53*dmats2[53][35] + coeff0_54*dmats2[54][35] + coeff0_55*dmats2[55][35];
          new_coeff0_36 = coeff0_0*dmats2[0][36] + coeff0_1*dmats2[1][36] + coeff0_2*dmats2[2][36] + coeff0_3*dmats2[3][36] + coeff0_4*dmats2[4][36] + coeff0_5*dmats2[5][36] + coeff0_6*dmats2[6][36] + coeff0_7*dmats2[7][36] + coeff0_8*dmats2[8][36] + coeff0_9*dmats2[9][36] + coeff0_10*dmats2[10][36] + coeff0_11*dmats2[11][36] + coeff0_12*dmats2[12][36] + coeff0_13*dmats2[13][36] + coeff0_14*dmats2[14][36] + coeff0_15*dmats2[15][36] + coeff0_16*dmats2[16][36] + coeff0_17*dmats2[17][36] + coeff0_18*dmats2[18][36] + coeff0_19*dmats2[19][36] + coeff0_20*dmats2[20][36] + coeff0_21*dmats2[21][36] + coeff0_22*dmats2[22][36] + coeff0_23*dmats2[23][36] + coeff0_24*dmats2[24][36] + coeff0_25*dmats2[25][36] + coeff0_26*dmats2[26][36] + coeff0_27*dmats2[27][36] + coeff0_28*dmats2[28][36] + coeff0_29*dmats2[29][36] + coeff0_30*dmats2[30][36] + coeff0_31*dmats2[31][36] + coeff0_32*dmats2[32][36] + coeff0_33*dmats2[33][36] + coeff0_34*dmats2[34][36] + coeff0_35*dmats2[35][36] + coeff0_36*dmats2[36][36] + coeff0_37*dmats2[37][36] + coeff0_38*dmats2[38][36] + coeff0_39*dmats2[39][36] + coeff0_40*dmats2[40][36] + coeff0_41*dmats2[41][36] + coeff0_42*dmats2[42][36] + coeff0_43*dmats2[43][36] + coeff0_44*dmats2[44][36] + coeff0_45*dmats2[45][36] + coeff0_46*dmats2[46][36] + coeff0_47*dmats2[47][36] + coeff0_48*dmats2[48][36] + coeff0_49*dmats2[49][36] + coeff0_50*dmats2[50][36] + coeff0_51*dmats2[51][36] + coeff0_52*dmats2[52][36] + coeff0_53*dmats2[53][36] + coeff0_54*dmats2[54][36] + coeff0_55*dmats2[55][36];
          new_coeff0_37 = coeff0_0*dmats2[0][37] + coeff0_1*dmats2[1][37] + coeff0_2*dmats2[2][37] + coeff0_3*dmats2[3][37] + coeff0_4*dmats2[4][37] + coeff0_5*dmats2[5][37] + coeff0_6*dmats2[6][37] + coeff0_7*dmats2[7][37] + coeff0_8*dmats2[8][37] + coeff0_9*dmats2[9][37] + coeff0_10*dmats2[10][37] + coeff0_11*dmats2[11][37] + coeff0_12*dmats2[12][37] + coeff0_13*dmats2[13][37] + coeff0_14*dmats2[14][37] + coeff0_15*dmats2[15][37] + coeff0_16*dmats2[16][37] + coeff0_17*dmats2[17][37] + coeff0_18*dmats2[18][37] + coeff0_19*dmats2[19][37] + coeff0_20*dmats2[20][37] + coeff0_21*dmats2[21][37] + coeff0_22*dmats2[22][37] + coeff0_23*dmats2[23][37] + coeff0_24*dmats2[24][37] + coeff0_25*dmats2[25][37] + coeff0_26*dmats2[26][37] + coeff0_27*dmats2[27][37] + coeff0_28*dmats2[28][37] + coeff0_29*dmats2[29][37] + coeff0_30*dmats2[30][37] + coeff0_31*dmats2[31][37] + coeff0_32*dmats2[32][37] + coeff0_33*dmats2[33][37] + coeff0_34*dmats2[34][37] + coeff0_35*dmats2[35][37] + coeff0_36*dmats2[36][37] + coeff0_37*dmats2[37][37] + coeff0_38*dmats2[38][37] + coeff0_39*dmats2[39][37] + coeff0_40*dmats2[40][37] + coeff0_41*dmats2[41][37] + coeff0_42*dmats2[42][37] + coeff0_43*dmats2[43][37] + coeff0_44*dmats2[44][37] + coeff0_45*dmats2[45][37] + coeff0_46*dmats2[46][37] + coeff0_47*dmats2[47][37] + coeff0_48*dmats2[48][37] + coeff0_49*dmats2[49][37] + coeff0_50*dmats2[50][37] + coeff0_51*dmats2[51][37] + coeff0_52*dmats2[52][37] + coeff0_53*dmats2[53][37] + coeff0_54*dmats2[54][37] + coeff0_55*dmats2[55][37];
          new_coeff0_38 = coeff0_0*dmats2[0][38] + coeff0_1*dmats2[1][38] + coeff0_2*dmats2[2][38] + coeff0_3*dmats2[3][38] + coeff0_4*dmats2[4][38] + coeff0_5*dmats2[5][38] + coeff0_6*dmats2[6][38] + coeff0_7*dmats2[7][38] + coeff0_8*dmats2[8][38] + coeff0_9*dmats2[9][38] + coeff0_10*dmats2[10][38] + coeff0_11*dmats2[11][38] + coeff0_12*dmats2[12][38] + coeff0_13*dmats2[13][38] + coeff0_14*dmats2[14][38] + coeff0_15*dmats2[15][38] + coeff0_16*dmats2[16][38] + coeff0_17*dmats2[17][38] + coeff0_18*dmats2[18][38] + coeff0_19*dmats2[19][38] + coeff0_20*dmats2[20][38] + coeff0_21*dmats2[21][38] + coeff0_22*dmats2[22][38] + coeff0_23*dmats2[23][38] + coeff0_24*dmats2[24][38] + coeff0_25*dmats2[25][38] + coeff0_26*dmats2[26][38] + coeff0_27*dmats2[27][38] + coeff0_28*dmats2[28][38] + coeff0_29*dmats2[29][38] + coeff0_30*dmats2[30][38] + coeff0_31*dmats2[31][38] + coeff0_32*dmats2[32][38] + coeff0_33*dmats2[33][38] + coeff0_34*dmats2[34][38] + coeff0_35*dmats2[35][38] + coeff0_36*dmats2[36][38] + coeff0_37*dmats2[37][38] + coeff0_38*dmats2[38][38] + coeff0_39*dmats2[39][38] + coeff0_40*dmats2[40][38] + coeff0_41*dmats2[41][38] + coeff0_42*dmats2[42][38] + coeff0_43*dmats2[43][38] + coeff0_44*dmats2[44][38] + coeff0_45*dmats2[45][38] + coeff0_46*dmats2[46][38] + coeff0_47*dmats2[47][38] + coeff0_48*dmats2[48][38] + coeff0_49*dmats2[49][38] + coeff0_50*dmats2[50][38] + coeff0_51*dmats2[51][38] + coeff0_52*dmats2[52][38] + coeff0_53*dmats2[53][38] + coeff0_54*dmats2[54][38] + coeff0_55*dmats2[55][38];
          new_coeff0_39 = coeff0_0*dmats2[0][39] + coeff0_1*dmats2[1][39] + coeff0_2*dmats2[2][39] + coeff0_3*dmats2[3][39] + coeff0_4*dmats2[4][39] + coeff0_5*dmats2[5][39] + coeff0_6*dmats2[6][39] + coeff0_7*dmats2[7][39] + coeff0_8*dmats2[8][39] + coeff0_9*dmats2[9][39] + coeff0_10*dmats2[10][39] + coeff0_11*dmats2[11][39] + coeff0_12*dmats2[12][39] + coeff0_13*dmats2[13][39] + coeff0_14*dmats2[14][39] + coeff0_15*dmats2[15][39] + coeff0_16*dmats2[16][39] + coeff0_17*dmats2[17][39] + coeff0_18*dmats2[18][39] + coeff0_19*dmats2[19][39] + coeff0_20*dmats2[20][39] + coeff0_21*dmats2[21][39] + coeff0_22*dmats2[22][39] + coeff0_23*dmats2[23][39] + coeff0_24*dmats2[24][39] + coeff0_25*dmats2[25][39] + coeff0_26*dmats2[26][39] + coeff0_27*dmats2[27][39] + coeff0_28*dmats2[28][39] + coeff0_29*dmats2[29][39] + coeff0_30*dmats2[30][39] + coeff0_31*dmats2[31][39] + coeff0_32*dmats2[32][39] + coeff0_33*dmats2[33][39] + coeff0_34*dmats2[34][39] + coeff0_35*dmats2[35][39] + coeff0_36*dmats2[36][39] + coeff0_37*dmats2[37][39] + coeff0_38*dmats2[38][39] + coeff0_39*dmats2[39][39] + coeff0_40*dmats2[40][39] + coeff0_41*dmats2[41][39] + coeff0_42*dmats2[42][39] + coeff0_43*dmats2[43][39] + coeff0_44*dmats2[44][39] + coeff0_45*dmats2[45][39] + coeff0_46*dmats2[46][39] + coeff0_47*dmats2[47][39] + coeff0_48*dmats2[48][39] + coeff0_49*dmats2[49][39] + coeff0_50*dmats2[50][39] + coeff0_51*dmats2[51][39] + coeff0_52*dmats2[52][39] + coeff0_53*dmats2[53][39] + coeff0_54*dmats2[54][39] + coeff0_55*dmats2[55][39];
          new_coeff0_40 = coeff0_0*dmats2[0][40] + coeff0_1*dmats2[1][40] + coeff0_2*dmats2[2][40] + coeff0_3*dmats2[3][40] + coeff0_4*dmats2[4][40] + coeff0_5*dmats2[5][40] + coeff0_6*dmats2[6][40] + coeff0_7*dmats2[7][40] + coeff0_8*dmats2[8][40] + coeff0_9*dmats2[9][40] + coeff0_10*dmats2[10][40] + coeff0_11*dmats2[11][40] + coeff0_12*dmats2[12][40] + coeff0_13*dmats2[13][40] + coeff0_14*dmats2[14][40] + coeff0_15*dmats2[15][40] + coeff0_16*dmats2[16][40] + coeff0_17*dmats2[17][40] + coeff0_18*dmats2[18][40] + coeff0_19*dmats2[19][40] + coeff0_20*dmats2[20][40] + coeff0_21*dmats2[21][40] + coeff0_22*dmats2[22][40] + coeff0_23*dmats2[23][40] + coeff0_24*dmats2[24][40] + coeff0_25*dmats2[25][40] + coeff0_26*dmats2[26][40] + coeff0_27*dmats2[27][40] + coeff0_28*dmats2[28][40] + coeff0_29*dmats2[29][40] + coeff0_30*dmats2[30][40] + coeff0_31*dmats2[31][40] + coeff0_32*dmats2[32][40] + coeff0_33*dmats2[33][40] + coeff0_34*dmats2[34][40] + coeff0_35*dmats2[35][40] + coeff0_36*dmats2[36][40] + coeff0_37*dmats2[37][40] + coeff0_38*dmats2[38][40] + coeff0_39*dmats2[39][40] + coeff0_40*dmats2[40][40] + coeff0_41*dmats2[41][40] + coeff0_42*dmats2[42][40] + coeff0_43*dmats2[43][40] + coeff0_44*dmats2[44][40] + coeff0_45*dmats2[45][40] + coeff0_46*dmats2[46][40] + coeff0_47*dmats2[47][40] + coeff0_48*dmats2[48][40] + coeff0_49*dmats2[49][40] + coeff0_50*dmats2[50][40] + coeff0_51*dmats2[51][40] + coeff0_52*dmats2[52][40] + coeff0_53*dmats2[53][40] + coeff0_54*dmats2[54][40] + coeff0_55*dmats2[55][40];
          new_coeff0_41 = coeff0_0*dmats2[0][41] + coeff0_1*dmats2[1][41] + coeff0_2*dmats2[2][41] + coeff0_3*dmats2[3][41] + coeff0_4*dmats2[4][41] + coeff0_5*dmats2[5][41] + coeff0_6*dmats2[6][41] + coeff0_7*dmats2[7][41] + coeff0_8*dmats2[8][41] + coeff0_9*dmats2[9][41] + coeff0_10*dmats2[10][41] + coeff0_11*dmats2[11][41] + coeff0_12*dmats2[12][41] + coeff0_13*dmats2[13][41] + coeff0_14*dmats2[14][41] + coeff0_15*dmats2[15][41] + coeff0_16*dmats2[16][41] + coeff0_17*dmats2[17][41] + coeff0_18*dmats2[18][41] + coeff0_19*dmats2[19][41] + coeff0_20*dmats2[20][41] + coeff0_21*dmats2[21][41] + coeff0_22*dmats2[22][41] + coeff0_23*dmats2[23][41] + coeff0_24*dmats2[24][41] + coeff0_25*dmats2[25][41] + coeff0_26*dmats2[26][41] + coeff0_27*dmats2[27][41] + coeff0_28*dmats2[28][41] + coeff0_29*dmats2[29][41] + coeff0_30*dmats2[30][41] + coeff0_31*dmats2[31][41] + coeff0_32*dmats2[32][41] + coeff0_33*dmats2[33][41] + coeff0_34*dmats2[34][41] + coeff0_35*dmats2[35][41] + coeff0_36*dmats2[36][41] + coeff0_37*dmats2[37][41] + coeff0_38*dmats2[38][41] + coeff0_39*dmats2[39][41] + coeff0_40*dmats2[40][41] + coeff0_41*dmats2[41][41] + coeff0_42*dmats2[42][41] + coeff0_43*dmats2[43][41] + coeff0_44*dmats2[44][41] + coeff0_45*dmats2[45][41] + coeff0_46*dmats2[46][41] + coeff0_47*dmats2[47][41] + coeff0_48*dmats2[48][41] + coeff0_49*dmats2[49][41] + coeff0_50*dmats2[50][41] + coeff0_51*dmats2[51][41] + coeff0_52*dmats2[52][41] + coeff0_53*dmats2[53][41] + coeff0_54*dmats2[54][41] + coeff0_55*dmats2[55][41];
          new_coeff0_42 = coeff0_0*dmats2[0][42] + coeff0_1*dmats2[1][42] + coeff0_2*dmats2[2][42] + coeff0_3*dmats2[3][42] + coeff0_4*dmats2[4][42] + coeff0_5*dmats2[5][42] + coeff0_6*dmats2[6][42] + coeff0_7*dmats2[7][42] + coeff0_8*dmats2[8][42] + coeff0_9*dmats2[9][42] + coeff0_10*dmats2[10][42] + coeff0_11*dmats2[11][42] + coeff0_12*dmats2[12][42] + coeff0_13*dmats2[13][42] + coeff0_14*dmats2[14][42] + coeff0_15*dmats2[15][42] + coeff0_16*dmats2[16][42] + coeff0_17*dmats2[17][42] + coeff0_18*dmats2[18][42] + coeff0_19*dmats2[19][42] + coeff0_20*dmats2[20][42] + coeff0_21*dmats2[21][42] + coeff0_22*dmats2[22][42] + coeff0_23*dmats2[23][42] + coeff0_24*dmats2[24][42] + coeff0_25*dmats2[25][42] + coeff0_26*dmats2[26][42] + coeff0_27*dmats2[27][42] + coeff0_28*dmats2[28][42] + coeff0_29*dmats2[29][42] + coeff0_30*dmats2[30][42] + coeff0_31*dmats2[31][42] + coeff0_32*dmats2[32][42] + coeff0_33*dmats2[33][42] + coeff0_34*dmats2[34][42] + coeff0_35*dmats2[35][42] + coeff0_36*dmats2[36][42] + coeff0_37*dmats2[37][42] + coeff0_38*dmats2[38][42] + coeff0_39*dmats2[39][42] + coeff0_40*dmats2[40][42] + coeff0_41*dmats2[41][42] + coeff0_42*dmats2[42][42] + coeff0_43*dmats2[43][42] + coeff0_44*dmats2[44][42] + coeff0_45*dmats2[45][42] + coeff0_46*dmats2[46][42] + coeff0_47*dmats2[47][42] + coeff0_48*dmats2[48][42] + coeff0_49*dmats2[49][42] + coeff0_50*dmats2[50][42] + coeff0_51*dmats2[51][42] + coeff0_52*dmats2[52][42] + coeff0_53*dmats2[53][42] + coeff0_54*dmats2[54][42] + coeff0_55*dmats2[55][42];
          new_coeff0_43 = coeff0_0*dmats2[0][43] + coeff0_1*dmats2[1][43] + coeff0_2*dmats2[2][43] + coeff0_3*dmats2[3][43] + coeff0_4*dmats2[4][43] + coeff0_5*dmats2[5][43] + coeff0_6*dmats2[6][43] + coeff0_7*dmats2[7][43] + coeff0_8*dmats2[8][43] + coeff0_9*dmats2[9][43] + coeff0_10*dmats2[10][43] + coeff0_11*dmats2[11][43] + coeff0_12*dmats2[12][43] + coeff0_13*dmats2[13][43] + coeff0_14*dmats2[14][43] + coeff0_15*dmats2[15][43] + coeff0_16*dmats2[16][43] + coeff0_17*dmats2[17][43] + coeff0_18*dmats2[18][43] + coeff0_19*dmats2[19][43] + coeff0_20*dmats2[20][43] + coeff0_21*dmats2[21][43] + coeff0_22*dmats2[22][43] + coeff0_23*dmats2[23][43] + coeff0_24*dmats2[24][43] + coeff0_25*dmats2[25][43] + coeff0_26*dmats2[26][43] + coeff0_27*dmats2[27][43] + coeff0_28*dmats2[28][43] + coeff0_29*dmats2[29][43] + coeff0_30*dmats2[30][43] + coeff0_31*dmats2[31][43] + coeff0_32*dmats2[32][43] + coeff0_33*dmats2[33][43] + coeff0_34*dmats2[34][43] + coeff0_35*dmats2[35][43] + coeff0_36*dmats2[36][43] + coeff0_37*dmats2[37][43] + coeff0_38*dmats2[38][43] + coeff0_39*dmats2[39][43] + coeff0_40*dmats2[40][43] + coeff0_41*dmats2[41][43] + coeff0_42*dmats2[42][43] + coeff0_43*dmats2[43][43] + coeff0_44*dmats2[44][43] + coeff0_45*dmats2[45][43] + coeff0_46*dmats2[46][43] + coeff0_47*dmats2[47][43] + coeff0_48*dmats2[48][43] + coeff0_49*dmats2[49][43] + coeff0_50*dmats2[50][43] + coeff0_51*dmats2[51][43] + coeff0_52*dmats2[52][43] + coeff0_53*dmats2[53][43] + coeff0_54*dmats2[54][43] + coeff0_55*dmats2[55][43];
          new_coeff0_44 = coeff0_0*dmats2[0][44] + coeff0_1*dmats2[1][44] + coeff0_2*dmats2[2][44] + coeff0_3*dmats2[3][44] + coeff0_4*dmats2[4][44] + coeff0_5*dmats2[5][44] + coeff0_6*dmats2[6][44] + coeff0_7*dmats2[7][44] + coeff0_8*dmats2[8][44] + coeff0_9*dmats2[9][44] + coeff0_10*dmats2[10][44] + coeff0_11*dmats2[11][44] + coeff0_12*dmats2[12][44] + coeff0_13*dmats2[13][44] + coeff0_14*dmats2[14][44] + coeff0_15*dmats2[15][44] + coeff0_16*dmats2[16][44] + coeff0_17*dmats2[17][44] + coeff0_18*dmats2[18][44] + coeff0_19*dmats2[19][44] + coeff0_20*dmats2[20][44] + coeff0_21*dmats2[21][44] + coeff0_22*dmats2[22][44] + coeff0_23*dmats2[23][44] + coeff0_24*dmats2[24][44] + coeff0_25*dmats2[25][44] + coeff0_26*dmats2[26][44] + coeff0_27*dmats2[27][44] + coeff0_28*dmats2[28][44] + coeff0_29*dmats2[29][44] + coeff0_30*dmats2[30][44] + coeff0_31*dmats2[31][44] + coeff0_32*dmats2[32][44] + coeff0_33*dmats2[33][44] + coeff0_34*dmats2[34][44] + coeff0_35*dmats2[35][44] + coeff0_36*dmats2[36][44] + coeff0_37*dmats2[37][44] + coeff0_38*dmats2[38][44] + coeff0_39*dmats2[39][44] + coeff0_40*dmats2[40][44] + coeff0_41*dmats2[41][44] + coeff0_42*dmats2[42][44] + coeff0_43*dmats2[43][44] + coeff0_44*dmats2[44][44] + coeff0_45*dmats2[45][44] + coeff0_46*dmats2[46][44] + coeff0_47*dmats2[47][44] + coeff0_48*dmats2[48][44] + coeff0_49*dmats2[49][44] + coeff0_50*dmats2[50][44] + coeff0_51*dmats2[51][44] + coeff0_52*dmats2[52][44] + coeff0_53*dmats2[53][44] + coeff0_54*dmats2[54][44] + coeff0_55*dmats2[55][44];
          new_coeff0_45 = coeff0_0*dmats2[0][45] + coeff0_1*dmats2[1][45] + coeff0_2*dmats2[2][45] + coeff0_3*dmats2[3][45] + coeff0_4*dmats2[4][45] + coeff0_5*dmats2[5][45] + coeff0_6*dmats2[6][45] + coeff0_7*dmats2[7][45] + coeff0_8*dmats2[8][45] + coeff0_9*dmats2[9][45] + coeff0_10*dmats2[10][45] + coeff0_11*dmats2[11][45] + coeff0_12*dmats2[12][45] + coeff0_13*dmats2[13][45] + coeff0_14*dmats2[14][45] + coeff0_15*dmats2[15][45] + coeff0_16*dmats2[16][45] + coeff0_17*dmats2[17][45] + coeff0_18*dmats2[18][45] + coeff0_19*dmats2[19][45] + coeff0_20*dmats2[20][45] + coeff0_21*dmats2[21][45] + coeff0_22*dmats2[22][45] + coeff0_23*dmats2[23][45] + coeff0_24*dmats2[24][45] + coeff0_25*dmats2[25][45] + coeff0_26*dmats2[26][45] + coeff0_27*dmats2[27][45] + coeff0_28*dmats2[28][45] + coeff0_29*dmats2[29][45] + coeff0_30*dmats2[30][45] + coeff0_31*dmats2[31][45] + coeff0_32*dmats2[32][45] + coeff0_33*dmats2[33][45] + coeff0_34*dmats2[34][45] + coeff0_35*dmats2[35][45] + coeff0_36*dmats2[36][45] + coeff0_37*dmats2[37][45] + coeff0_38*dmats2[38][45] + coeff0_39*dmats2[39][45] + coeff0_40*dmats2[40][45] + coeff0_41*dmats2[41][45] + coeff0_42*dmats2[42][45] + coeff0_43*dmats2[43][45] + coeff0_44*dmats2[44][45] + coeff0_45*dmats2[45][45] + coeff0_46*dmats2[46][45] + coeff0_47*dmats2[47][45] + coeff0_48*dmats2[48][45] + coeff0_49*dmats2[49][45] + coeff0_50*dmats2[50][45] + coeff0_51*dmats2[51][45] + coeff0_52*dmats2[52][45] + coeff0_53*dmats2[53][45] + coeff0_54*dmats2[54][45] + coeff0_55*dmats2[55][45];
          new_coeff0_46 = coeff0_0*dmats2[0][46] + coeff0_1*dmats2[1][46] + coeff0_2*dmats2[2][46] + coeff0_3*dmats2[3][46] + coeff0_4*dmats2[4][46] + coeff0_5*dmats2[5][46] + coeff0_6*dmats2[6][46] + coeff0_7*dmats2[7][46] + coeff0_8*dmats2[8][46] + coeff0_9*dmats2[9][46] + coeff0_10*dmats2[10][46] + coeff0_11*dmats2[11][46] + coeff0_12*dmats2[12][46] + coeff0_13*dmats2[13][46] + coeff0_14*dmats2[14][46] + coeff0_15*dmats2[15][46] + coeff0_16*dmats2[16][46] + coeff0_17*dmats2[17][46] + coeff0_18*dmats2[18][46] + coeff0_19*dmats2[19][46] + coeff0_20*dmats2[20][46] + coeff0_21*dmats2[21][46] + coeff0_22*dmats2[22][46] + coeff0_23*dmats2[23][46] + coeff0_24*dmats2[24][46] + coeff0_25*dmats2[25][46] + coeff0_26*dmats2[26][46] + coeff0_27*dmats2[27][46] + coeff0_28*dmats2[28][46] + coeff0_29*dmats2[29][46] + coeff0_30*dmats2[30][46] + coeff0_31*dmats2[31][46] + coeff0_32*dmats2[32][46] + coeff0_33*dmats2[33][46] + coeff0_34*dmats2[34][46] + coeff0_35*dmats2[35][46] + coeff0_36*dmats2[36][46] + coeff0_37*dmats2[37][46] + coeff0_38*dmats2[38][46] + coeff0_39*dmats2[39][46] + coeff0_40*dmats2[40][46] + coeff0_41*dmats2[41][46] + coeff0_42*dmats2[42][46] + coeff0_43*dmats2[43][46] + coeff0_44*dmats2[44][46] + coeff0_45*dmats2[45][46] + coeff0_46*dmats2[46][46] + coeff0_47*dmats2[47][46] + coeff0_48*dmats2[48][46] + coeff0_49*dmats2[49][46] + coeff0_50*dmats2[50][46] + coeff0_51*dmats2[51][46] + coeff0_52*dmats2[52][46] + coeff0_53*dmats2[53][46] + coeff0_54*dmats2[54][46] + coeff0_55*dmats2[55][46];
          new_coeff0_47 = coeff0_0*dmats2[0][47] + coeff0_1*dmats2[1][47] + coeff0_2*dmats2[2][47] + coeff0_3*dmats2[3][47] + coeff0_4*dmats2[4][47] + coeff0_5*dmats2[5][47] + coeff0_6*dmats2[6][47] + coeff0_7*dmats2[7][47] + coeff0_8*dmats2[8][47] + coeff0_9*dmats2[9][47] + coeff0_10*dmats2[10][47] + coeff0_11*dmats2[11][47] + coeff0_12*dmats2[12][47] + coeff0_13*dmats2[13][47] + coeff0_14*dmats2[14][47] + coeff0_15*dmats2[15][47] + coeff0_16*dmats2[16][47] + coeff0_17*dmats2[17][47] + coeff0_18*dmats2[18][47] + coeff0_19*dmats2[19][47] + coeff0_20*dmats2[20][47] + coeff0_21*dmats2[21][47] + coeff0_22*dmats2[22][47] + coeff0_23*dmats2[23][47] + coeff0_24*dmats2[24][47] + coeff0_25*dmats2[25][47] + coeff0_26*dmats2[26][47] + coeff0_27*dmats2[27][47] + coeff0_28*dmats2[28][47] + coeff0_29*dmats2[29][47] + coeff0_30*dmats2[30][47] + coeff0_31*dmats2[31][47] + coeff0_32*dmats2[32][47] + coeff0_33*dmats2[33][47] + coeff0_34*dmats2[34][47] + coeff0_35*dmats2[35][47] + coeff0_36*dmats2[36][47] + coeff0_37*dmats2[37][47] + coeff0_38*dmats2[38][47] + coeff0_39*dmats2[39][47] + coeff0_40*dmats2[40][47] + coeff0_41*dmats2[41][47] + coeff0_42*dmats2[42][47] + coeff0_43*dmats2[43][47] + coeff0_44*dmats2[44][47] + coeff0_45*dmats2[45][47] + coeff0_46*dmats2[46][47] + coeff0_47*dmats2[47][47] + coeff0_48*dmats2[48][47] + coeff0_49*dmats2[49][47] + coeff0_50*dmats2[50][47] + coeff0_51*dmats2[51][47] + coeff0_52*dmats2[52][47] + coeff0_53*dmats2[53][47] + coeff0_54*dmats2[54][47] + coeff0_55*dmats2[55][47];
          new_coeff0_48 = coeff0_0*dmats2[0][48] + coeff0_1*dmats2[1][48] + coeff0_2*dmats2[2][48] + coeff0_3*dmats2[3][48] + coeff0_4*dmats2[4][48] + coeff0_5*dmats2[5][48] + coeff0_6*dmats2[6][48] + coeff0_7*dmats2[7][48] + coeff0_8*dmats2[8][48] + coeff0_9*dmats2[9][48] + coeff0_10*dmats2[10][48] + coeff0_11*dmats2[11][48] + coeff0_12*dmats2[12][48] + coeff0_13*dmats2[13][48] + coeff0_14*dmats2[14][48] + coeff0_15*dmats2[15][48] + coeff0_16*dmats2[16][48] + coeff0_17*dmats2[17][48] + coeff0_18*dmats2[18][48] + coeff0_19*dmats2[19][48] + coeff0_20*dmats2[20][48] + coeff0_21*dmats2[21][48] + coeff0_22*dmats2[22][48] + coeff0_23*dmats2[23][48] + coeff0_24*dmats2[24][48] + coeff0_25*dmats2[25][48] + coeff0_26*dmats2[26][48] + coeff0_27*dmats2[27][48] + coeff0_28*dmats2[28][48] + coeff0_29*dmats2[29][48] + coeff0_30*dmats2[30][48] + coeff0_31*dmats2[31][48] + coeff0_32*dmats2[32][48] + coeff0_33*dmats2[33][48] + coeff0_34*dmats2[34][48] + coeff0_35*dmats2[35][48] + coeff0_36*dmats2[36][48] + coeff0_37*dmats2[37][48] + coeff0_38*dmats2[38][48] + coeff0_39*dmats2[39][48] + coeff0_40*dmats2[40][48] + coeff0_41*dmats2[41][48] + coeff0_42*dmats2[42][48] + coeff0_43*dmats2[43][48] + coeff0_44*dmats2[44][48] + coeff0_45*dmats2[45][48] + coeff0_46*dmats2[46][48] + coeff0_47*dmats2[47][48] + coeff0_48*dmats2[48][48] + coeff0_49*dmats2[49][48] + coeff0_50*dmats2[50][48] + coeff0_51*dmats2[51][48] + coeff0_52*dmats2[52][48] + coeff0_53*dmats2[53][48] + coeff0_54*dmats2[54][48] + coeff0_55*dmats2[55][48];
          new_coeff0_49 = coeff0_0*dmats2[0][49] + coeff0_1*dmats2[1][49] + coeff0_2*dmats2[2][49] + coeff0_3*dmats2[3][49] + coeff0_4*dmats2[4][49] + coeff0_5*dmats2[5][49] + coeff0_6*dmats2[6][49] + coeff0_7*dmats2[7][49] + coeff0_8*dmats2[8][49] + coeff0_9*dmats2[9][49] + coeff0_10*dmats2[10][49] + coeff0_11*dmats2[11][49] + coeff0_12*dmats2[12][49] + coeff0_13*dmats2[13][49] + coeff0_14*dmats2[14][49] + coeff0_15*dmats2[15][49] + coeff0_16*dmats2[16][49] + coeff0_17*dmats2[17][49] + coeff0_18*dmats2[18][49] + coeff0_19*dmats2[19][49] + coeff0_20*dmats2[20][49] + coeff0_21*dmats2[21][49] + coeff0_22*dmats2[22][49] + coeff0_23*dmats2[23][49] + coeff0_24*dmats2[24][49] + coeff0_25*dmats2[25][49] + coeff0_26*dmats2[26][49] + coeff0_27*dmats2[27][49] + coeff0_28*dmats2[28][49] + coeff0_29*dmats2[29][49] + coeff0_30*dmats2[30][49] + coeff0_31*dmats2[31][49] + coeff0_32*dmats2[32][49] + coeff0_33*dmats2[33][49] + coeff0_34*dmats2[34][49] + coeff0_35*dmats2[35][49] + coeff0_36*dmats2[36][49] + coeff0_37*dmats2[37][49] + coeff0_38*dmats2[38][49] + coeff0_39*dmats2[39][49] + coeff0_40*dmats2[40][49] + coeff0_41*dmats2[41][49] + coeff0_42*dmats2[42][49] + coeff0_43*dmats2[43][49] + coeff0_44*dmats2[44][49] + coeff0_45*dmats2[45][49] + coeff0_46*dmats2[46][49] + coeff0_47*dmats2[47][49] + coeff0_48*dmats2[48][49] + coeff0_49*dmats2[49][49] + coeff0_50*dmats2[50][49] + coeff0_51*dmats2[51][49] + coeff0_52*dmats2[52][49] + coeff0_53*dmats2[53][49] + coeff0_54*dmats2[54][49] + coeff0_55*dmats2[55][49];
          new_coeff0_50 = coeff0_0*dmats2[0][50] + coeff0_1*dmats2[1][50] + coeff0_2*dmats2[2][50] + coeff0_3*dmats2[3][50] + coeff0_4*dmats2[4][50] + coeff0_5*dmats2[5][50] + coeff0_6*dmats2[6][50] + coeff0_7*dmats2[7][50] + coeff0_8*dmats2[8][50] + coeff0_9*dmats2[9][50] + coeff0_10*dmats2[10][50] + coeff0_11*dmats2[11][50] + coeff0_12*dmats2[12][50] + coeff0_13*dmats2[13][50] + coeff0_14*dmats2[14][50] + coeff0_15*dmats2[15][50] + coeff0_16*dmats2[16][50] + coeff0_17*dmats2[17][50] + coeff0_18*dmats2[18][50] + coeff0_19*dmats2[19][50] + coeff0_20*dmats2[20][50] + coeff0_21*dmats2[21][50] + coeff0_22*dmats2[22][50] + coeff0_23*dmats2[23][50] + coeff0_24*dmats2[24][50] + coeff0_25*dmats2[25][50] + coeff0_26*dmats2[26][50] + coeff0_27*dmats2[27][50] + coeff0_28*dmats2[28][50] + coeff0_29*dmats2[29][50] + coeff0_30*dmats2[30][50] + coeff0_31*dmats2[31][50] + coeff0_32*dmats2[32][50] + coeff0_33*dmats2[33][50] + coeff0_34*dmats2[34][50] + coeff0_35*dmats2[35][50] + coeff0_36*dmats2[36][50] + coeff0_37*dmats2[37][50] + coeff0_38*dmats2[38][50] + coeff0_39*dmats2[39][50] + coeff0_40*dmats2[40][50] + coeff0_41*dmats2[41][50] + coeff0_42*dmats2[42][50] + coeff0_43*dmats2[43][50] + coeff0_44*dmats2[44][50] + coeff0_45*dmats2[45][50] + coeff0_46*dmats2[46][50] + coeff0_47*dmats2[47][50] + coeff0_48*dmats2[48][50] + coeff0_49*dmats2[49][50] + coeff0_50*dmats2[50][50] + coeff0_51*dmats2[51][50] + coeff0_52*dmats2[52][50] + coeff0_53*dmats2[53][50] + coeff0_54*dmats2[54][50] + coeff0_55*dmats2[55][50];
          new_coeff0_51 = coeff0_0*dmats2[0][51] + coeff0_1*dmats2[1][51] + coeff0_2*dmats2[2][51] + coeff0_3*dmats2[3][51] + coeff0_4*dmats2[4][51] + coeff0_5*dmats2[5][51] + coeff0_6*dmats2[6][51] + coeff0_7*dmats2[7][51] + coeff0_8*dmats2[8][51] + coeff0_9*dmats2[9][51] + coeff0_10*dmats2[10][51] + coeff0_11*dmats2[11][51] + coeff0_12*dmats2[12][51] + coeff0_13*dmats2[13][51] + coeff0_14*dmats2[14][51] + coeff0_15*dmats2[15][51] + coeff0_16*dmats2[16][51] + coeff0_17*dmats2[17][51] + coeff0_18*dmats2[18][51] + coeff0_19*dmats2[19][51] + coeff0_20*dmats2[20][51] + coeff0_21*dmats2[21][51] + coeff0_22*dmats2[22][51] + coeff0_23*dmats2[23][51] + coeff0_24*dmats2[24][51] + coeff0_25*dmats2[25][51] + coeff0_26*dmats2[26][51] + coeff0_27*dmats2[27][51] + coeff0_28*dmats2[28][51] + coeff0_29*dmats2[29][51] + coeff0_30*dmats2[30][51] + coeff0_31*dmats2[31][51] + coeff0_32*dmats2[32][51] + coeff0_33*dmats2[33][51] + coeff0_34*dmats2[34][51] + coeff0_35*dmats2[35][51] + coeff0_36*dmats2[36][51] + coeff0_37*dmats2[37][51] + coeff0_38*dmats2[38][51] + coeff0_39*dmats2[39][51] + coeff0_40*dmats2[40][51] + coeff0_41*dmats2[41][51] + coeff0_42*dmats2[42][51] + coeff0_43*dmats2[43][51] + coeff0_44*dmats2[44][51] + coeff0_45*dmats2[45][51] + coeff0_46*dmats2[46][51] + coeff0_47*dmats2[47][51] + coeff0_48*dmats2[48][51] + coeff0_49*dmats2[49][51] + coeff0_50*dmats2[50][51] + coeff0_51*dmats2[51][51] + coeff0_52*dmats2[52][51] + coeff0_53*dmats2[53][51] + coeff0_54*dmats2[54][51] + coeff0_55*dmats2[55][51];
          new_coeff0_52 = coeff0_0*dmats2[0][52] + coeff0_1*dmats2[1][52] + coeff0_2*dmats2[2][52] + coeff0_3*dmats2[3][52] + coeff0_4*dmats2[4][52] + coeff0_5*dmats2[5][52] + coeff0_6*dmats2[6][52] + coeff0_7*dmats2[7][52] + coeff0_8*dmats2[8][52] + coeff0_9*dmats2[9][52] + coeff0_10*dmats2[10][52] + coeff0_11*dmats2[11][52] + coeff0_12*dmats2[12][52] + coeff0_13*dmats2[13][52] + coeff0_14*dmats2[14][52] + coeff0_15*dmats2[15][52] + coeff0_16*dmats2[16][52] + coeff0_17*dmats2[17][52] + coeff0_18*dmats2[18][52] + coeff0_19*dmats2[19][52] + coeff0_20*dmats2[20][52] + coeff0_21*dmats2[21][52] + coeff0_22*dmats2[22][52] + coeff0_23*dmats2[23][52] + coeff0_24*dmats2[24][52] + coeff0_25*dmats2[25][52] + coeff0_26*dmats2[26][52] + coeff0_27*dmats2[27][52] + coeff0_28*dmats2[28][52] + coeff0_29*dmats2[29][52] + coeff0_30*dmats2[30][52] + coeff0_31*dmats2[31][52] + coeff0_32*dmats2[32][52] + coeff0_33*dmats2[33][52] + coeff0_34*dmats2[34][52] + coeff0_35*dmats2[35][52] + coeff0_36*dmats2[36][52] + coeff0_37*dmats2[37][52] + coeff0_38*dmats2[38][52] + coeff0_39*dmats2[39][52] + coeff0_40*dmats2[40][52] + coeff0_41*dmats2[41][52] + coeff0_42*dmats2[42][52] + coeff0_43*dmats2[43][52] + coeff0_44*dmats2[44][52] + coeff0_45*dmats2[45][52] + coeff0_46*dmats2[46][52] + coeff0_47*dmats2[47][52] + coeff0_48*dmats2[48][52] + coeff0_49*dmats2[49][52] + coeff0_50*dmats2[50][52] + coeff0_51*dmats2[51][52] + coeff0_52*dmats2[52][52] + coeff0_53*dmats2[53][52] + coeff0_54*dmats2[54][52] + coeff0_55*dmats2[55][52];
          new_coeff0_53 = coeff0_0*dmats2[0][53] + coeff0_1*dmats2[1][53] + coeff0_2*dmats2[2][53] + coeff0_3*dmats2[3][53] + coeff0_4*dmats2[4][53] + coeff0_5*dmats2[5][53] + coeff0_6*dmats2[6][53] + coeff0_7*dmats2[7][53] + coeff0_8*dmats2[8][53] + coeff0_9*dmats2[9][53] + coeff0_10*dmats2[10][53] + coeff0_11*dmats2[11][53] + coeff0_12*dmats2[12][53] + coeff0_13*dmats2[13][53] + coeff0_14*dmats2[14][53] + coeff0_15*dmats2[15][53] + coeff0_16*dmats2[16][53] + coeff0_17*dmats2[17][53] + coeff0_18*dmats2[18][53] + coeff0_19*dmats2[19][53] + coeff0_20*dmats2[20][53] + coeff0_21*dmats2[21][53] + coeff0_22*dmats2[22][53] + coeff0_23*dmats2[23][53] + coeff0_24*dmats2[24][53] + coeff0_25*dmats2[25][53] + coeff0_26*dmats2[26][53] + coeff0_27*dmats2[27][53] + coeff0_28*dmats2[28][53] + coeff0_29*dmats2[29][53] + coeff0_30*dmats2[30][53] + coeff0_31*dmats2[31][53] + coeff0_32*dmats2[32][53] + coeff0_33*dmats2[33][53] + coeff0_34*dmats2[34][53] + coeff0_35*dmats2[35][53] + coeff0_36*dmats2[36][53] + coeff0_37*dmats2[37][53] + coeff0_38*dmats2[38][53] + coeff0_39*dmats2[39][53] + coeff0_40*dmats2[40][53] + coeff0_41*dmats2[41][53] + coeff0_42*dmats2[42][53] + coeff0_43*dmats2[43][53] + coeff0_44*dmats2[44][53] + coeff0_45*dmats2[45][53] + coeff0_46*dmats2[46][53] + coeff0_47*dmats2[47][53] + coeff0_48*dmats2[48][53] + coeff0_49*dmats2[49][53] + coeff0_50*dmats2[50][53] + coeff0_51*dmats2[51][53] + coeff0_52*dmats2[52][53] + coeff0_53*dmats2[53][53] + coeff0_54*dmats2[54][53] + coeff0_55*dmats2[55][53];
          new_coeff0_54 = coeff0_0*dmats2[0][54] + coeff0_1*dmats2[1][54] + coeff0_2*dmats2[2][54] + coeff0_3*dmats2[3][54] + coeff0_4*dmats2[4][54] + coeff0_5*dmats2[5][54] + coeff0_6*dmats2[6][54] + coeff0_7*dmats2[7][54] + coeff0_8*dmats2[8][54] + coeff0_9*dmats2[9][54] + coeff0_10*dmats2[10][54] + coeff0_11*dmats2[11][54] + coeff0_12*dmats2[12][54] + coeff0_13*dmats2[13][54] + coeff0_14*dmats2[14][54] + coeff0_15*dmats2[15][54] + coeff0_16*dmats2[16][54] + coeff0_17*dmats2[17][54] + coeff0_18*dmats2[18][54] + coeff0_19*dmats2[19][54] + coeff0_20*dmats2[20][54] + coeff0_21*dmats2[21][54] + coeff0_22*dmats2[22][54] + coeff0_23*dmats2[23][54] + coeff0_24*dmats2[24][54] + coeff0_25*dmats2[25][54] + coeff0_26*dmats2[26][54] + coeff0_27*dmats2[27][54] + coeff0_28*dmats2[28][54] + coeff0_29*dmats2[29][54] + coeff0_30*dmats2[30][54] + coeff0_31*dmats2[31][54] + coeff0_32*dmats2[32][54] + coeff0_33*dmats2[33][54] + coeff0_34*dmats2[34][54] + coeff0_35*dmats2[35][54] + coeff0_36*dmats2[36][54] + coeff0_37*dmats2[37][54] + coeff0_38*dmats2[38][54] + coeff0_39*dmats2[39][54] + coeff0_40*dmats2[40][54] + coeff0_41*dmats2[41][54] + coeff0_42*dmats2[42][54] + coeff0_43*dmats2[43][54] + coeff0_44*dmats2[44][54] + coeff0_45*dmats2[45][54] + coeff0_46*dmats2[46][54] + coeff0_47*dmats2[47][54] + coeff0_48*dmats2[48][54] + coeff0_49*dmats2[49][54] + coeff0_50*dmats2[50][54] + coeff0_51*dmats2[51][54] + coeff0_52*dmats2[52][54] + coeff0_53*dmats2[53][54] + coeff0_54*dmats2[54][54] + coeff0_55*dmats2[55][54];
          new_coeff0_55 = coeff0_0*dmats2[0][55] + coeff0_1*dmats2[1][55] + coeff0_2*dmats2[2][55] + coeff0_3*dmats2[3][55] + coeff0_4*dmats2[4][55] + coeff0_5*dmats2[5][55] + coeff0_6*dmats2[6][55] + coeff0_7*dmats2[7][55] + coeff0_8*dmats2[8][55] + coeff0_9*dmats2[9][55] + coeff0_10*dmats2[10][55] + coeff0_11*dmats2[11][55] + coeff0_12*dmats2[12][55] + coeff0_13*dmats2[13][55] + coeff0_14*dmats2[14][55] + coeff0_15*dmats2[15][55] + coeff0_16*dmats2[16][55] + coeff0_17*dmats2[17][55] + coeff0_18*dmats2[18][55] + coeff0_19*dmats2[19][55] + coeff0_20*dmats2[20][55] + coeff0_21*dmats2[21][55] + coeff0_22*dmats2[22][55] + coeff0_23*dmats2[23][55] + coeff0_24*dmats2[24][55] + coeff0_25*dmats2[25][55] + coeff0_26*dmats2[26][55] + coeff0_27*dmats2[27][55] + coeff0_28*dmats2[28][55] + coeff0_29*dmats2[29][55] + coeff0_30*dmats2[30][55] + coeff0_31*dmats2[31][55] + coeff0_32*dmats2[32][55] + coeff0_33*dmats2[33][55] + coeff0_34*dmats2[34][55] + coeff0_35*dmats2[35][55] + coeff0_36*dmats2[36][55] + coeff0_37*dmats2[37][55] + coeff0_38*dmats2[38][55] + coeff0_39*dmats2[39][55] + coeff0_40*dmats2[40][55] + coeff0_41*dmats2[41][55] + coeff0_42*dmats2[42][55] + coeff0_43*dmats2[43][55] + coeff0_44*dmats2[44][55] + coeff0_45*dmats2[45][55] + coeff0_46*dmats2[46][55] + coeff0_47*dmats2[47][55] + coeff0_48*dmats2[48][55] + coeff0_49*dmats2[49][55] + coeff0_50*dmats2[50][55] + coeff0_51*dmats2[51][55] + coeff0_52*dmats2[52][55] + coeff0_53*dmats2[53][55] + coeff0_54*dmats2[54][55] + coeff0_55*dmats2[55][55];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9 + new_coeff0_10*basisvalue10 + new_coeff0_11*basisvalue11 + new_coeff0_12*basisvalue12 + new_coeff0_13*basisvalue13 + new_coeff0_14*basisvalue14 + new_coeff0_15*basisvalue15 + new_coeff0_16*basisvalue16 + new_coeff0_17*basisvalue17 + new_coeff0_18*basisvalue18 + new_coeff0_19*basisvalue19 + new_coeff0_20*basisvalue20 + new_coeff0_21*basisvalue21 + new_coeff0_22*basisvalue22 + new_coeff0_23*basisvalue23 + new_coeff0_24*basisvalue24 + new_coeff0_25*basisvalue25 + new_coeff0_26*basisvalue26 + new_coeff0_27*basisvalue27 + new_coeff0_28*basisvalue28 + new_coeff0_29*basisvalue29 + new_coeff0_30*basisvalue30 + new_coeff0_31*basisvalue31 + new_coeff0_32*basisvalue32 + new_coeff0_33*basisvalue33 + new_coeff0_34*basisvalue34 + new_coeff0_35*basisvalue35 + new_coeff0_36*basisvalue36 + new_coeff0_37*basisvalue37 + new_coeff0_38*basisvalue38 + new_coeff0_39*basisvalue39 + new_coeff0_40*basisvalue40 + new_coeff0_41*basisvalue41 + new_coeff0_42*basisvalue42 + new_coeff0_43*basisvalue43 + new_coeff0_44*basisvalue44 + new_coeff0_45*basisvalue45 + new_coeff0_46*basisvalue46 + new_coeff0_47*basisvalue47 + new_coeff0_48*basisvalue48 + new_coeff0_49*basisvalue49 + new_coeff0_50*basisvalue50 + new_coeff0_51*basisvalue51 + new_coeff0_52*basisvalue52 + new_coeff0_53*basisvalue53 + new_coeff0_54*basisvalue54 + new_coeff0_55*basisvalue55;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    static const double X[56][1][3] = {{{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}, {{0, 0.8, 0.2}}, {{0, 0.6, 0.4}}, {{0, 0.4, 0.6}}, {{0, 0.2, 0.8}}, {{0.8, 0, 0.2}}, {{0.6, 0, 0.4}}, {{0.4, 0, 0.6}}, {{0.2, 0, 0.8}}, {{0.8, 0.2, 0}}, {{0.6, 0.4, 0}}, {{0.4, 0.6, 0}}, {{0.2, 0.8, 0}}, {{0, 0, 0.2}}, {{0, 0, 0.4}}, {{0, 0, 0.6}}, {{0, 0, 0.8}}, {{0, 0.2, 0}}, {{0, 0.4, 0}}, {{0, 0.6, 0}}, {{0, 0.8, 0}}, {{0.2, 0, 0}}, {{0.4, 0, 0}}, {{0.6, 0, 0}}, {{0.8, 0, 0}}, {{0.6, 0.2, 0.2}}, {{0.4, 0.4, 0.2}}, {{0.2, 0.6, 0.2}}, {{0.4, 0.2, 0.4}}, {{0.2, 0.4, 0.4}}, {{0.2, 0.2, 0.6}}, {{0, 0.2, 0.2}}, {{0, 0.4, 0.2}}, {{0, 0.6, 0.2}}, {{0, 0.2, 0.4}}, {{0, 0.4, 0.4}}, {{0, 0.2, 0.6}}, {{0.2, 0, 0.2}}, {{0.4, 0, 0.2}}, {{0.6, 0, 0.2}}, {{0.2, 0, 0.4}}, {{0.4, 0, 0.4}}, {{0.2, 0, 0.6}}, {{0.2, 0.2, 0}}, {{0.4, 0.2, 0}}, {{0.6, 0.2, 0}}, {{0.2, 0.4, 0}}, {{0.4, 0.4, 0}}, {{0.2, 0.6, 0}}, {{0.2, 0.2, 0.2}}, {{0.4, 0.2, 0.2}}, {{0.2, 0.4, 0.2}}, {{0.2, 0.2, 0.4}}};
    static const double W[56][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    static const double D[56][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new poisson3d_5_1_finite_element_0();
  }

};

/// This class defines the interface for a finite element.

class poisson3d_5_1_finite_element_1: public ufc::finite_element
{
public:

  /// Constructor
  poisson3d_5_1_finite_element_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_1_finite_element_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "FiniteElement('Lagrange', 'tetrahedron', 5)";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 56;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
    
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
    
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    static const double coefficients0[56][56] = \
    {{0.00945067404923492, 0.00609788120679252, 0.00352061335622803, 0.00248944957812476, 0.0070594623332504, 0.00546823600997146, 0.00315708753234943, 0.00446479600584755, 0.00257775117585285, 0.00182274533665715, -0.00298267418725716, -0.0025208197798293, -0.00195261860521383, -0.00112734487734488, -0.00218309396769447, -0.00169101731601731, -0.000976309302606919, -0.00138070985681772, -0.000797153207506487, -0.00056367243867244, 0.00278687112460004, 0.00245778931055954, 0.00207721109304566, 0.00160900079398463, 0.000928957041533343, 0.00219831358911339, 0.0018579140830667, 0.00143913406048032, 0.00083088443721827, 0.00160900079398463, 0.0012463266558274, 0.000719567030240151, 0.00101762145320217, 0.000587524019939407, 0.000415442218609132, -0.00705593524717798, -0.00638233357089081, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363026, -0.00582624677716565, -0.00513826668309062, -0.00434262794914383, -0.00336378514516178, -0.00194208225905522, -0.00459580543596519, -0.00388416451811043, -0.00300866089851429, -0.00173705117965752, -0.00336378514516178, -0.00260557676948629, -0.00150433044925714, -0.00212744452363027, -0.00122828066840393, -0.000868525589828762},
    {0.00945067404923495, -0.00609788120679253, 0.00352061335622802, 0.00248944957812476, 0.00705946233325039, -0.00546823600997146, 0.00315708753234944, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521383, -0.00112734487734489, -0.00218309396769448, 0.00169101731601732, -0.000976309302606925, 0.00138070985681772, -0.000797153207506488, -0.000563672438672441, 0.00278687112460004, -0.00245778931055954, 0.00207721109304566, -0.00160900079398463, 0.000928957041533346, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218268, 0.00160900079398463, -0.0012463266558274, 0.000719567030240148, -0.00101762145320216, 0.000587524019939402, 0.000415442218609128, 0.00705593524717798, -0.0063823335708908, 0.00562868913761196, -0.00475711057319694, 0.00368484200521179, -0.00212744452363027, -0.00582624677716565, 0.00513826668309062, -0.00434262794914382, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811043, 0.00300866089851429, -0.00173705117965752, -0.00336378514516178, 0.00260557676948629, -0.00150433044925714, 0.00212744452363026, -0.00122828066840393, -0.00086852558982876},
    {0.00945067404923495, 0, -0.00704122671245605, 0.00248944957812476, 0, 0, 0.00947126259704828, 0, -0.00515550235170572, 0.00182274533665715, 0, 0, 0, 0.00450937950937951, 0, 0, -0.00292892790782075, 0, 0.00159430641501296, -0.00056367243867244, 0, 0, 0, 0, 0.00464478520766673, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987881, 0.00041544221860913, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527608, 0, 0, 0, 0.00694820471863012, 0, 0, -0.00451299134777144, 0, 0.00245656133680786, -0.000868525589828763},
    {0.00945067404923493, 0, 0, -0.0074683487343743, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00563672438672439, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.018239037386404},
    {-0.010023442173431, 0, 0.0374718748311399, -0.0289614183593723, 0, 0, 0.0284137877911448, 0, 0.00515550235170572, -0.00619733414463434, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835494, -0.000187890812890802, 0, 0, 0, 0, 0.0232239260383337, 0, 0, 0, -0.00166176887443652, 0, 0, -0.00647610327216137, 0, 0.00646276421933342, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496074, 0, 0, 0, -0.0277928188745205, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.010023442173431, 0, -0.0287574853355259, 0.05360943398437, 0, 0, -0.0189425251940966, 0, 0.0154665070551171, 0.0153110608279201, 0, 0, 0, -0.00450937950937953, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144092, 0, -0.0123380044187274, 0.00872428659079174, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559316, 0, 0.041270230458372, -0.018239037386404},
    {0.010023442173431, 0, 0.0409576306293854, -0.0449826285156208, 0, 0, 0.0189425251940965, 0, -0.00515550235170569, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883226, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957583, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.055026973944496, 0.0303983956440067},
    {-0.0100234421734311, 0, -0.0148144621425437, 0.0449826285156207, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0187359374155699, -0.0289614183593723, 0.0211783869997512, -0.0164047080299144, 0.00947126259704827, 0.00446479600584757, -0.00257775117585285, -0.00619733414463432, 0.0208787193108001, -0.0176457384588052, 0.0136683302364968, -0.00789141414141413, -0.0084898098743674, 0.00657617845117844, -0.00379675839902689, 0.0023011830946962, -0.00132858867917746, -0.0001878908128908, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992314, 0.00464478520766674, -0.0010991567945567, 0.000928957041533349, -0.000719567030240152, 0.000415442218609127, -0.00482700238195388, 0.00373897996748219, -0.00215870109072045, 0.00559691799261185, -0.00323138210966672, -0.00290809553026392, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935591, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863011, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778515, 0.00607967912880135},
    {0.010023442173431, -0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, 0.0109364720199429, -0.00631417506469883, 0.0133943880175427, -0.00773325352755858, 0.0153110608279201, -0.00298267418725716, 0.00252081977982931, -0.00195261860521382, 0.00112734487734488, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476002, 0.0129522065443227, -0.00747795993496437, -0.00321800158796926, 0.0024926533116548, -0.00143913406048031, -0.0106850252586226, 0.00616900220936373, 0.00872428659079178, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845888, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.020635115229186, -0.0182390373864041},
    {0.0100234421734311, 0.0354703486038674, -0.0204788153146927, -0.0449826285156208, 0.0141189246665008, -0.0109364720199429, 0.00631417506469885, -0.00446479600584758, 0.00257775117585285, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974696, -0.00263047138047138, 0.00151870335961075, 0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620757, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, -0.0128297005588457, 0.00740723107127184, 0.0449826285156208, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0.00920473237878476, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489395, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.010023442173431, 0.0324515955311979, -0.0335503995581136, -0.00801060507812427, 0.0211783869997512, 0.00109364720199429, -0.00568275755822898, -0.0169662248222207, 0.0056710525868763, 0.00838462854862291, 0.0208787193108001, -0.0109235523792603, 0.00499002532443534, -0.00187890812890812, -0.0162519217595033, 0.00958243145743146, -0.00466458889023305, 0.0108922666482287, -0.00558007245254534, -0.00469727032227033, 0.0139343556230002, -0.00245778931055954, -0.00415442218609131, 0.00643600317593851, -0.00464478520766673, -0.0120907247401236, 0.00464478520766674, -0.000719567030240153, -0.000415442218609137, 0.00965400476390778, -0.00498530662330958, 0.00215870109072046, -0.006614539445814, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543313, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438609, 0.00310733161448835, -0.00481385743762287, 0.00347410235931505, -0.00201827108709706, -0.000521115353897262, 0.000902598269554289, 0.00170195561890421, -0.000491312267361565, -0.000868525589828764},
    {0.0100234421734309, -0.024904712849524, 0.0553363732971484, -0.0043134027343746, -0.0141189246665008, 0.0153110608279201, 0.00883984509057843, 0.00803663281052563, -0.0159820572902877, 0.000729098134662861, -0.00298267418725715, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846298, -0.0116492303992304, 0.00976309302606916, 0.00352848074520084, 0.00132858867917745, -0.00263047138047137, 0, 0.0196623144844763, -0.00727023882565982, -0.00724050357293082, 0.00928957041533346, -0.00219831358911338, -0.0148633126645336, 0.0129522065443228, -0.0066470754977461, 0.00402250198496157, 0.00685479660705067, -0.00575653624192123, -0.0045792965394097, -0.000293762009969684, 0.00290809553026392, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.00411061334647249, -0.00303983956440068, 0.00908221989193681, -0.00776832903622088, 0.00045958054359652, 0.00427258096992148, -0.00270779480866287, 0.000868525589828773, -0.00100913554354854, -0.00234501909253766, 0.00180519653910858, 0.00127646671417816, 0.000245656133680781, -0.000868525589828764},
    {0.0100234421734309, 0.0354703486038675, -0.0492363006502187, -0.00431340273437461, 0.0141189246665008, -0.00656188321196574, -0.016416855168217, -0.00982255121286466, 0.0149509568199466, 0.000729098134662872, 0, 0.0033610930397724, 0.0282044909641998, -0.0353234728234728, -0.000485131992820993, -0.0124007936507936, 0.00889526253486299, 0.00291483191994851, 0.00239145962251946, -0.00263047138047138, 0, 0, 0.0218107164769794, -0.00402250198496158, -0.00928957041533347, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992316, 0.00934744991870547, -0.0028782681209606, -0.00254405363300539, -0.00381890612960613, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320202, 0.00840946286290443, -0.00388416451811042, 0, 0.00233049871086626, 0.0030086608985143, -0.00347410235931507, -0.000336378514516178, -0.00286613444643493, 0.00120346435940572, 0.00085097780945211, 0.00098262453472315, -0.000868525589828765},
    {-0.010023442173431, -0.0128297005588457, 0.0448791059024117, -0.00801060507812424, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467804, -0.0175287079957994, 0.0083846285486229, 0, 0, 0.00867830491206149, 0.0225468975468975, 0, -0.00300625300625301, -0.019200749617936, 0.000613648825252316, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240152, -0.0108014976838374, 0, 0.00249265331165478, 0.0107935054536023, -0.000508810726601074, -0.00734405024924252, 0.00290809553026393, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931505, 0, -0.00208446141558903, -0.000902598269554287, 0.000425488904726055, 0.0017195929357655, -0.000868525589828767},
    {-0.0100234421734311, -0.0324515955311978, -0.0187359374155699, -0.0289614183593723, 0.0211783869997511, 0.0164047080299144, 0.00947126259704826, -0.00446479600584755, -0.00257775117585285, -0.00619733414463435, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141413, -0.00848980987436742, -0.00657617845117845, -0.0037967583990269, -0.00230118309469619, -0.00132858867917746, -0.00018789081289081, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992313, 0.00464478520766674, 0.00109915679455668, 0.000928957041533345, 0.000719567030240154, 0.000415442218609131, -0.00482700238195388, -0.00373897996748219, -0.00215870109072045, -0.00559691799261187, -0.00323138210966673, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.00955378148811641, 0.00740032731935591, 0.00427258096992148, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.0069482047186301, 0.0181644397838736, 0.014070114555226, 0.00812338442598859, 0.0136156449512337, 0.00786099627778516, 0.00607967912880135},
    {0.0100234421734308, 0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, -0.0109364720199429, -0.00631417506469886, -0.0133943880175427, -0.00773325352755858, 0.0153110608279201, 0.00298267418725716, 0.00252081977982933, 0.00195261860521383, 0.00112734487734487, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476002, -0.0129522065443228, -0.00747795993496438, -0.00321800158796927, -0.00249265331165479, -0.00143913406048031, 0.0106850252586227, 0.00616900220936374, 0.00872428659079177, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.0206351152291861, -0.0182390373864041},
    {0.010023442173431, -0.0354703486038674, -0.0204788153146927, -0.0449826285156208, 0.0141189246665008, 0.0109364720199429, 0.00631417506469886, 0.00446479600584758, 0.00257775117585287, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974696, 0.00263047138047138, 0.00151870335961075, -0.0308358534689291, -0.0178030883009781, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620757, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588457, 0.00740723107127183, 0.0449826285156208, 0, 0, 0, -0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878479, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489395, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.010023442173431, -0.0324515955311978, -0.0335503995581136, -0.00801060507812424, 0.0211783869997512, -0.00109364720199429, -0.00568275755822899, 0.0169662248222207, 0.00567105258687632, 0.0083846285486229, -0.0208787193108001, -0.0109235523792603, -0.00499002532443534, -0.00187890812890812, -0.0162519217595033, -0.00958243145743146, -0.00466458889023305, -0.0108922666482287, -0.00558007245254535, -0.00469727032227032, 0.0139343556230002, 0.00245778931055953, -0.00415442218609133, -0.00643600317593852, -0.00464478520766672, 0.0120907247401236, 0.00464478520766673, 0.000719567030240156, -0.000415442218609149, 0.00965400476390777, 0.00498530662330958, 0.00215870109072047, 0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543313, 0.00719357335632687, 0.0112908326677739, 0.0114368694935501, 0.00737991258440984, -0.00183832217438608, 0.00310733161448834, 0.00481385743762287, 0.00347410235931505, -0.00201827108709706, 0.000521115353897262, 0.000902598269554287, -0.00170195561890421, -0.000491312267361565, -0.000868525589828767},
    {0.010023442173431, 0.0249047128495239, 0.0553363732971484, -0.00431340273437459, -0.0141189246665008, -0.0153110608279201, 0.00883984509057843, -0.00803663281052563, -0.0159820572902878, 0.000729098134662854, 0.00298267418725716, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846298, 0.0116492303992304, 0.00976309302606918, -0.00352848074520084, 0.00132858867917746, -0.00263047138047138, 0, -0.0196623144844763, -0.0072702388256598, 0.00724050357293083, 0.00928957041533346, 0.00219831358911339, -0.0148633126645336, -0.0129522065443228, -0.00664707549774609, 0.00402250198496157, -0.00685479660705067, -0.00575653624192124, 0.0045792965394097, -0.000293762009969679, 0.00290809553026392, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.0041106133464725, -0.00303983956440067, -0.00908221989193682, -0.00776832903622089, -0.00045958054359652, 0.00427258096992148, 0.00270779480866287, 0.000868525589828776, -0.00100913554354853, 0.00234501909253766, 0.00180519653910858, -0.00127646671417816, 0.000245656133680779, -0.000868525589828761},
    {0.0100234421734309, -0.0354703486038674, -0.0492363006502187, -0.00431340273437459, 0.0141189246665008, 0.00656188321196577, -0.016416855168217, 0.00982255121286466, 0.0149509568199466, 0.000729098134662868, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820994, 0.0124007936507936, 0.00889526253486299, -0.00291483191994852, 0.00239145962251945, -0.00263047138047138, 0, 0, 0.0218107164769794, 0.00402250198496157, -0.00928957041533346, 0, -0.00557374224920009, 0.0143913406048031, 0.0141250354327105, 0.000804500396992309, -0.00934744991870548, -0.0028782681209606, 0.00254405363300539, -0.00381890612960614, 0.00290809553026393, 0, 0, 0, 0.0266398192099028, 0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320203, -0.00840946286290443, -0.00388416451811041, 0, 0.00233049871086626, -0.0030086608985143, -0.00347410235931507, -0.000336378514516177, 0.00286613444643493, 0.00120346435940572, -0.000850977809452108, 0.000982624534723152, -0.000868525589828766},
    {-0.010023442173431, 0.0128297005588457, 0.0448791059024117, -0.00801060507812424, 0, -0.0174983552319087, 0.0132597676358676, 0.00357183680467805, -0.0175287079957994, 0.0083846285486229, 0, 0, -0.00867830491206148, 0.0225468975468975, 0, 0.00300625300625301, -0.019200749617936, -0.000613648825252319, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766673, 0, 0, 0.00719567030240153, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601077, -0.00734405024924252, 0.00290809553026393, 0, 0, 0, 0, -0.0257938940364826, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702859, -0.00347410235931505, 0, 0.00208446141558903, -0.000902598269554286, -0.000425488904726054, 0.0017195929357655, -0.000868525589828767},
    {-0.0100234421734311, -0.0452812960900435, -0.0113287063442981, -0.00801060507812424, -0.0014118924666501, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.00838462854862289, -0.00894802256177147, -0.0142846454190328, -0.014536160727703, -0.00939454064454064, -0.0123708658169354, -0.0125886844636845, -0.00813591085505764, -0.0102786178229763, -0.00664294339588733, -0.00469727032227033, -0.00836061337380012, 0.00245778931055953, 0.00830884437218264, 0.00965400476390777, 0.00650269929073344, 0.00219831358911338, 0.00743165633226677, 0.00863480436288183, 0.00581619106052785, 0.00643600317593852, 0.00747795993496437, 0.00503696921168108, 0.00610572871921293, 0.00411266813957582, 0.00290809553026394, 0.0352796762358899, 0.0191470007126724, 0.00788016479265675, 0.000951422114639391, -0.00221090520312707, -0.00212744452363027, 0.017478740331497, 0.00719357335632686, 0.000868525589828764, -0.00201827108709706, -0.00194208225905521, 0.00643412761035128, 0.000776832903622088, -0.00180519653910857, -0.00173705117965753, 0.000672757029032363, -0.00156334606169177, -0.00150433044925715, -0.00127646671417816, -0.00122828066840393, -0.000868525589828768},
    {0.010023442173431, 0.0603750614533914, -0.00610007264692975, -0.0043134027343746, -0.00564756986660025, -0.0218729440398859, 0.00126283501293978, -0.0178591840233903, 0.00103110047034114, 0.000729098134662884, 0.0417574386216002, 0.0168054651988621, -0.000867830491206148, -0.00526094276094276, 0.0145539597846299, -0.000751563251563252, -0.00455611007883227, -0.000613648825252328, -0.0037200483016969, -0.00263047138047138, 0.00557374224920005, -0.0196623144844763, -0.0103860554652283, 0.00321800158796928, 0.00650269929073341, -0.0175865087129071, -0.00928957041533347, 0.00287826812096063, 0.00581619106052784, -0.00804500396992315, 0.0024926533116548, 0.00503696921168106, 0.00203524290640433, 0.00411266813957581, 0.00290809553026391, -0.0705593524717798, -0.0127646671417816, 0.00450295131008958, 0.00380568845855754, -0.000736968401042372, -0.00212744452363026, -0.0116524935543313, 0.00411061334647251, 0.00347410235931506, -0.000672757029032362, -0.00194208225905521, 0.00367664434877218, 0.00310733161448835, -0.000601732179702865, -0.00173705117965753, 0.00269102811612942, -0.000521115353897265, -0.00150433044925715, -0.00042548890472606, -0.00122828066840393, -0.000868525589828759},
    {0.0100234421734309, -0.0603750614533914, -0.00610007264692972, -0.00431340273437457, -0.00564756986660032, 0.0218729440398859, 0.00126283501293976, 0.0178591840233903, 0.00103110047034113, 0.000729098134662861, -0.0417574386216002, 0.016805465198862, 0.000867830491206157, -0.00526094276094276, 0.0145539597846298, 0.000751563251563257, -0.00455611007883228, 0.00061364882525233, -0.00372004830169691, -0.00263047138047138, 0.00557374224920011, 0.0196623144844763, -0.0103860554652283, -0.00321800158796927, 0.00650269929073344, 0.0175865087129071, -0.00928957041533346, -0.00287826812096062, 0.00581619106052786, -0.00804500396992314, -0.0024926533116548, 0.00503696921168108, -0.00203524290640433, 0.00411266813957583, 0.00290809553026393, 0.0705593524717798, -0.0127646671417816, -0.00450295131008958, 0.00380568845855756, 0.000736968401042369, -0.00212744452363027, -0.0116524935543313, -0.00411061334647251, 0.00347410235931506, 0.000672757029032361, -0.00194208225905522, -0.00367664434877217, 0.00310733161448835, 0.000601732179702866, -0.00173705117965753, 0.00269102811612942, 0.000521115353897263, -0.00150433044925715, 0.000425488904726061, -0.00122828066840394, -0.000868525589828769},
    {-0.0100234421734309, 0.0452812960900435, -0.0113287063442981, -0.00801060507812426, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089231, 0.00838462854862291, 0.00894802256177147, -0.0142846454190327, 0.014536160727703, -0.00939454064454063, -0.0123708658169354, 0.0125886844636845, -0.00813591085505763, 0.0102786178229763, -0.00664294339588732, -0.00469727032227032, -0.00836061337380015, -0.00245778931055953, 0.00830884437218263, -0.00965400476390777, 0.00650269929073343, -0.00219831358911339, 0.00743165633226677, -0.00863480436288183, 0.00581619106052785, 0.00643600317593852, -0.00747795993496437, 0.00503696921168107, -0.00610572871921293, 0.00411266813957582, 0.00290809553026393, -0.0352796762358899, 0.0191470007126724, -0.00788016479265675, 0.000951422114639385, 0.00221090520312707, -0.00212744452363027, 0.0174787403314969, -0.00719357335632687, 0.000868525589828768, 0.00201827108709706, -0.00194208225905521, -0.00643412761035128, 0.000776832903622089, 0.00180519653910857, -0.00173705117965753, 0.000672757029032358, 0.00156334606169177, -0.00150433044925715, 0.00127646671417816, -0.00122828066840393, -0.000868525589828765},
    {0.0787556170769579, 0.0679219441350653, -0.00435719474780691, -0.00308100195312472, 0.0705946233325039, -0.0328094160598287, 0.0126283501293977, -0.0267887760350854, 0.03608851646194, -0.00729098134662862, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015873, 0.0130985638061669, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, 0.0245778931055954, -0.036351194128299, 0.0362025178646541, -0.0232239260383337, 0.0219831358911338, -0.00743165633226679, 0, 0.00166176887443652, -0.0345935170706695, 0.0130864298861876, -0.00359783515120079, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596521, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598858, -0.0102117337134253, 0.00196524906944628, 0.00607967912880136},
    {-0.0214788046573522, 0.00377344134083702, 0.00217859737390344, -0.0215670136718729, 0, 0.0984282481794862, -0.0852413633734345, -0.0312535720409329, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, 0.0236742424242424, -0.0335199527228374, -0.00138070985681772, -0.00079715320750645, 0.00394570707070704, 0, 0, 0.036351194128299, -0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.00724050357293084, -0.0392592896585629, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724418, 0, 0.00694820471863008, 0.00302740663064561, 0.0164151336477636, -0.0108311792346515, -0.00680782247561686, -0.00393049813889256, 0.00607967912880134},
    {0.0787556170769579, 0.0301875307266957, 0.0610007264692975, -0.00308100195312472, 0, 0.0218729440398858, 0.0757701007763863, 0.0178591840233903, -0.0412440188136458, -0.00729098134662862, 0, 0, 0.0520698294723688, -0.00901875901875902, 0, -0.0127765752765753, 0.0286384062098028, 0.00184094647575699, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.0143913406048031, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515274, 0.00407048581280861, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967795, -0.0427258096992148, 0, 0, -0.0300866089851429, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723144, 0.00607967912880136},
    {-0.0214788046573522, 0.00377344134083686, -0.0196073763651313, 0.00924300585937417, 0, -0.0109364720199429, 0.00947126259704823, 0.102690308134494, -0.0695992817480272, -0.060150596109686, 0, -0.00504163955965862, 0.0065087286840461, -0.00450937950937951, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196709, -0.0263060558477138, 0.000563672438672465, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165478, -0.0043174021814409, -0.0605484764655283, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573521, -0.0150937653633479, 0.0130715842434208, 0.00924300585937414, 0, -0.0109364720199429, 0.00947126259704832, -0.00892959201169514, 0.123732056440937, -0.0601505961096861, 0, 0, -0.00650872868404609, 0.00676406926406927, 0, 0.0574945887445887, 0.0126920209338899, -0.0266937238984759, 0.00637722566005184, 0.000563672438672453, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097862, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448302, -0.0616900220936373, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769578, 0.0301875307266958, 0.0174287789912279, 0.0585390371093694, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094276, -0.00455611007883226, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700864, 0.0284934006896604, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769578, -0.0679219441350653, -0.00435719474780693, -0.00308100195312472, 0.0705946233325039, 0.0328094160598287, 0.0126283501293978, 0.0267887760350854, 0.03608851646194, -0.00729098134662859, -0.0298267418725716, 0.0151249186789758, 0.0325436434202305, 0.0248015873015873, 0.0130985638061669, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646541, -0.0232239260383337, -0.0219831358911339, -0.00743165633226677, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861877, -0.00359783515120079, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.0045958054359652, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.00908221989193679, -0.00703505727761299, -0.00812338442598858, 0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573524, -0.00377344134083698, 0.00217859737390344, -0.021567013671873, 0, -0.0984282481794863, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242424, -0.0335199527228374, 0.0013807098568177, -0.00079715320750645, 0.00394570707070706, 0, 0, 0.036351194128299, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838375, -0.00724050357293083, 0.039259289658563, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951694, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.00302740663064561, -0.0164151336477636, -0.0108311792346515, 0.00680782247561685, -0.00393049813889256, 0.00607967912880134},
    {0.0787556170769579, -0.0301875307266956, 0.0610007264692975, -0.00308100195312471, 0, -0.0218729440398858, 0.0757701007763863, -0.0178591840233903, -0.0412440188136458, -0.00729098134662861, 0, 0, -0.0520698294723689, -0.00901875901875903, 0, 0.0127765752765753, 0.0286384062098028, -0.00184094647575695, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766674, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515274, -0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967796, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780843, -0.00982624534723145, 0.00607967912880137},
    {-0.0214788046573523, -0.0037734413408369, -0.0196073763651314, 0.00924300585937414, 0, 0.0109364720199429, 0.00947126259704826, -0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965861, -0.00650872868404608, -0.00450937950937948, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196707, -0.0263060558477138, 0.000563672438672444, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.0024926533116548, -0.00431740218144091, 0.0605484764655282, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, 0.0150937653633478, 0.0130715842434208, 0.00924300585937414, 0, 0.0109364720199429, 0.00947126259704831, 0.00892959201169514, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404611, 0.00676406926406927, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005185, 0.000563672438672461, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097862, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448302, -0.0616900220936373, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769578, -0.0301875307266956, 0.0174287789912279, 0.0585390371093695, 0, -0.0218729440398858, -0.0189425251940966, -0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, -0.00526094276094275, -0.00455611007883226, -0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700864, -0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769576, -0.0377344134083696, -0.0566435317214905, -0.00308100195312473, 0.0423567739995023, 0.0546823600997146, 0.0378850503881931, 0.0446479600584756, 0.00515550235170574, -0.00729098134662862, 0.0417574386216002, -0.0050416395596586, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801536, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055954, -0.00519302773261415, 0.00402250198496157, 0.00464478520766674, -0.00219831358911335, -0.0130053985814669, -0.0158304746652834, -0.0108014976838374, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.00477689074405821, 0.00370016365967795, 0.00427258096992148, -0.0367664434877216, -0.00776832903622087, 0.00601732179702858, 0.00694820471863011, -0.00908221989193682, 0.00703505727761298, 0.0081233844259886, 0.00680782247561684, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573522, 0, -0.0043571947478069, -0.0215670136718729, -0.127070321998507, 0, 0.0284137877911448, 0, 0.03608851646194, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984381, 0, 0.00159430641501295, 0.00394570707070706, 0.0836061337380013, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377579, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.0095537814881164, 0, 0.00427258096992147, 0, -0.0155366580724417, 0, 0.00694820471863011, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778513, 0.00607967912880133},
    {0.0787556170769579, 0.0377344134083696, -0.0566435317214905, -0.0030810019531247, 0.0423567739995023, -0.0546823600997146, 0.0378850503881931, -0.0446479600584757, 0.00515550235170574, -0.00729098134662864, -0.0417574386216002, -0.00504163955965862, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801536, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261416, -0.00402250198496158, 0.00464478520766674, 0.00219831358911339, -0.0130053985814669, 0.0158304746652834, -0.0108014976838374, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405821, -0.00370016365967795, 0.00427258096992148, 0.0367664434877216, -0.00776832903622088, -0.00601732179702859, 0.00694820471863012, -0.00908221989193681, -0.00703505727761299, 0.00812338442598861, -0.00680782247561685, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573522, -0.0188672067041847, 0.00653579212171051, 0.00924300585937415, 0.0141189246665008, 0, -0.00315708753234942, -0.111619900146189, -0.0541327746929101, -0.0601505961096861, -0.00894802256177146, -0.00252081977982933, 0.000650872868404606, 0.00112734487734486, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672439, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476002, -0.00431740218144093, -0.00747795993496438, 0.00321800158796924, 0.0224338798048931, 0.018708742786244, 0.046301776120698, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.017478740331497, -0.00451299134777145, -0.00781673030845887, 0.0363288795677472, -0.00938007637015066, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {-0.0214788046573521, 0.0188672067041847, 0.00653579212171047, 0.00924300585937414, 0.0141189246665007, 0, -0.00315708753234944, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177148, -0.00252081977982932, -0.000650872868404592, 0.00112734487734487, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.019928830187662, 0.000563672438672456, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476002, 0.00431740218144092, -0.00747795993496438, 0.00321800158796927, -0.0224338798048931, 0.018708742786244, -0.0463017761206981, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777143, -0.00781673030845888, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769579, 0, -0.0348575579824557, 0.0585390371093695, -0.0282378493330015, 0, 0.00631417506469885, 0, -0.0206220094068228, 0.083846285486229, 0, 0, 0, 0, -0.0067918478994939, 0, 0.00151870335961074, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233621, 0, -0.0329013451166066, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.0275134869722481, 0.0303983956440068},
    {0.0787556170769578, -0.0377344134083696, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, 0.0546823600997145, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865145, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.0092175078635989, -0.00789141414141412, -0.00347132196482459, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, -0.00491557862111904, -0.0197335053839338, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097694, -0.000623163327913701, -0.00215870109072047, 0.00712335017241508, 0.00411266813957581, 0.00540074884191871, 0, -0.0510586685671264, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173251, -0.0143871467126537, -0.00564541633388696, 0.00571843474677502, 0.00737991258440982, 0.00367664434877216, -0.00155366580724418, 0.00240692871881142, 0.00347410235931506, 0.00100913554354853, 0.00026055767694863, 0.000902598269554296, -0.000850977809452108, -0.000491312267361565, -0.000868525589828763},
    {-0.0214788046573521, 0, -0.0217859737390348, 0.00308100195312476, -0.127070321998507, 0, 0.0473563129852414, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468977, 0.0509388592462045, 0, -0.00943765659186683, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533349, 0, -0.0141250354327105, -0.0144810071458616, 0, -0.00215870109072044, 0, 0.00411266813957584, 0.00540074884191874, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259877, 0, -0.0112908326677739, 0, 0.00737991258440982, 0, -0.00310733161448833, 0, 0.00347410235931505, 0.00201827108709706, 0, 0.000902598269554279, 0, -0.000491312267361592, -0.00086852558982878},
    {0.078755617076958, 0.0377344134083695, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584757, 0.0257775117585286, 0.0291639253865145, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.0092175078635989, 0.00789141414141413, -0.00347132196482461, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111908, -0.0197335053839338, 0.0394205194526234, -0.0325134964536672, 0.0241814494802473, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097695, 0.000623163327913697, -0.00215870109072046, -0.0071233501724151, 0.00411266813957581, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908062, -0.0114170653756727, -0.010317557614593, 0.0127646671417816, 0.00466099742173252, 0.0143871467126538, -0.00564541633388697, -0.00571843474677502, 0.00737991258440983, -0.00367664434877217, -0.00155366580724418, -0.00240692871881144, 0.00347410235931506, 0.00100913554354853, -0.000260557676948628, 0.000902598269554287, 0.000850977809452107, -0.000491312267361569, -0.000868525589828763},
    {-0.0214788046573521, -0.0188672067041848, 0.0108929868695174, 0.0030810019531247, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792643, 0.0127592173566001, -0.0089480225617715, -0.0277290175781224, -0.00585785581564147, 0.0191648629148629, 0.00145539597846298, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755831, -0.0129644660894661, 0, 0.0589869434534289, 0.00727023882565979, 0.0506835250105158, 0.0650269929073344, -0.00659494076734015, 0.0148633126645335, -0.00431740218144092, -0.00166176887443655, -0.00402250198496158, -0.00810112326287807, -0.0115130724838425, 0.00356167508620755, -0.00205633406978791, 0.00540074884191871, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440066, -0.00302740663064559, -0.00776832903622087, 0.00137874163078956, -0.00427258096992148, 0.000902598269554293, 0.000868525589828758, 0.00100913554354854, 0.000781673030845887, 0.00180519653910857, -0.000425488904726058, 0.00024565613368079, -0.000868525589828762},
    {-0.0214788046573521, 0.0188672067041848, 0.0108929868695174, 0.00308100195312469, 0.0141189246665007, 0.109364720199429, -0.0789271883087357, -0.0223239800292378, -0.0128887558792643, 0.0127592173566001, 0.00894802256177149, -0.0277290175781224, 0.00585785581564149, 0.0191648629148629, 0.00145539597846298, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755832, -0.0129644660894661, 0, -0.0589869434534289, 0.00727023882565983, -0.0506835250105158, 0.0650269929073343, 0.00659494076734015, 0.0148633126645336, 0.00431740218144092, -0.00166176887443653, -0.00402250198496157, 0.00810112326287806, -0.0115130724838425, -0.00356167508620755, -0.00205633406978791, 0.00540074884191872, 0, 0, -0.0607898426862092, 0.0399597288148543, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440068, 0.0030274066306456, -0.00776832903622088, -0.00137874163078956, -0.00427258096992148, -0.000902598269554298, 0.000868525589828757, 0.00100913554354854, -0.000781673030845891, 0.00180519653910858, 0.000425488904726056, 0.000245656133680792, -0.000868525589828762},
    {0.078755617076958, 0, 0.0435719474780696, -0.0523770332031201, -0.0282378493330015, 0, 0.0631417506469885, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.00672218607954481, 0, -0.0601250601250601, 0.000970263985641994, 0, -0.0125835421224892, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539589, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398463, 0, -0.0028782681209606, 0, -0.00822533627915163, 0.00540074884191871, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.018239037386404, 0, -0.00388416451811042, 0, -0.00466099742173252, 0, -0.00347410235931505, 0.000672757029032356, 0, 0.00120346435940571, 0, 0.000982624534723144, -0.000868525589828765},
    {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, -0.0546823600997146, -0.0315708753234943, -0.0446479600584757, -0.0257775117585285, -0.0182274533665716, 0.0894802256177147, -0.0151249186789759, 0.0585785581564149, 0.0744047619047619, -0.0130985638061669, -0.0281836219336219, 0.0097630930260691, 0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226673, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978424, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905521, -0.00902598269554287, -0.0191075629762328, -0.00908221989193678, -0.00234501909253766, -0.00812338442598858, 0.00340391123780844, 0.00196524906944629, 0.00607967912880135},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, 0.0546823600997146, -0.0315708753234943, 0.0446479600584756, -0.0257775117585286, -0.0182274533665715, -0.0894802256177148, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061669, 0.0281836219336219, 0.00976309302606918, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.0363511941282991, 0.0120675059548847, -0.0232239260383337, -0.0659494076734016, 0.00743165633226682, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978425, 0.0251848460584054, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554289, -0.0191075629762328, -0.00908221989193679, 0.00234501909253767, -0.00812338442598859, -0.00340391123780842, 0.00196524906944628, 0.00607967912880136},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281177, 0, 0, -0.0947126259704827, 0, 0.0515550235170571, -0.0182274533665715, 0, -0.0907495120738551, 0, -0.0946969696969696, 0.0232863356554078, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992785, 0, 0, -0.072702388256598, 0, 0.0464478520766673, 0, -0.048305766159734, 0, -0.0781031370985167, 0.0144810071458617, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992147, 0, 0.0310733161448835, 0, 0.00694820471863012, -0.00605481326129122, 0, -0.0108311792346515, 0, -0.00393049813889258, 0.00607967912880135},
    {0.107394023286761, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937952, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398575, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259878, 0, 0.0312669212338355, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    const double coeff0_6 = coefficients0[dof][6];
    const double coeff0_7 = coefficients0[dof][7];
    const double coeff0_8 = coefficients0[dof][8];
    const double coeff0_9 = coefficients0[dof][9];
    const double coeff0_10 = coefficients0[dof][10];
    const double coeff0_11 = coefficients0[dof][11];
    const double coeff0_12 = coefficients0[dof][12];
    const double coeff0_13 = coefficients0[dof][13];
    const double coeff0_14 = coefficients0[dof][14];
    const double coeff0_15 = coefficients0[dof][15];
    const double coeff0_16 = coefficients0[dof][16];
    const double coeff0_17 = coefficients0[dof][17];
    const double coeff0_18 = coefficients0[dof][18];
    const double coeff0_19 = coefficients0[dof][19];
    const double coeff0_20 = coefficients0[dof][20];
    const double coeff0_21 = coefficients0[dof][21];
    const double coeff0_22 = coefficients0[dof][22];
    const double coeff0_23 = coefficients0[dof][23];
    const double coeff0_24 = coefficients0[dof][24];
    const double coeff0_25 = coefficients0[dof][25];
    const double coeff0_26 = coefficients0[dof][26];
    const double coeff0_27 = coefficients0[dof][27];
    const double coeff0_28 = coefficients0[dof][28];
    const double coeff0_29 = coefficients0[dof][29];
    const double coeff0_30 = coefficients0[dof][30];
    const double coeff0_31 = coefficients0[dof][31];
    const double coeff0_32 = coefficients0[dof][32];
    const double coeff0_33 = coefficients0[dof][33];
    const double coeff0_34 = coefficients0[dof][34];
    const double coeff0_35 = coefficients0[dof][35];
    const double coeff0_36 = coefficients0[dof][36];
    const double coeff0_37 = coefficients0[dof][37];
    const double coeff0_38 = coefficients0[dof][38];
    const double coeff0_39 = coefficients0[dof][39];
    const double coeff0_40 = coefficients0[dof][40];
    const double coeff0_41 = coefficients0[dof][41];
    const double coeff0_42 = coefficients0[dof][42];
    const double coeff0_43 = coefficients0[dof][43];
    const double coeff0_44 = coefficients0[dof][44];
    const double coeff0_45 = coefficients0[dof][45];
    const double coeff0_46 = coefficients0[dof][46];
    const double coeff0_47 = coefficients0[dof][47];
    const double coeff0_48 = coefficients0[dof][48];
    const double coeff0_49 = coefficients0[dof][49];
    const double coeff0_50 = coefficients0[dof][50];
    const double coeff0_51 = coefficients0[dof][51];
    const double coeff0_52 = coefficients0[dof][52];
    const double coeff0_53 = coefficients0[dof][53];
    const double coeff0_54 = coefficients0[dof][54];
    const double coeff0_55 = coefficients0[dof][55];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9 + coeff0_10*basisvalue10 + coeff0_11*basisvalue11 + coeff0_12*basisvalue12 + coeff0_13*basisvalue13 + coeff0_14*basisvalue14 + coeff0_15*basisvalue15 + coeff0_16*basisvalue16 + coeff0_17*basisvalue17 + coeff0_18*basisvalue18 + coeff0_19*basisvalue19 + coeff0_20*basisvalue20 + coeff0_21*basisvalue21 + coeff0_22*basisvalue22 + coeff0_23*basisvalue23 + coeff0_24*basisvalue24 + coeff0_25*basisvalue25 + coeff0_26*basisvalue26 + coeff0_27*basisvalue27 + coeff0_28*basisvalue28 + coeff0_29*basisvalue29 + coeff0_30*basisvalue30 + coeff0_31*basisvalue31 + coeff0_32*basisvalue32 + coeff0_33*basisvalue33 + coeff0_34*basisvalue34 + coeff0_35*basisvalue35 + coeff0_36*basisvalue36 + coeff0_37*basisvalue37 + coeff0_38*basisvalue38 + coeff0_39*basisvalue39 + coeff0_40*basisvalue40 + coeff0_41*basisvalue41 + coeff0_42*basisvalue42 + coeff0_43*basisvalue43 + coeff0_44*basisvalue44 + coeff0_45*basisvalue45 + coeff0_46*basisvalue46 + coeff0_47*basisvalue47 + coeff0_48*basisvalue48 + coeff0_49*basisvalue49 + coeff0_50*basisvalue50 + coeff0_51*basisvalue51 + coeff0_52*basisvalue52 + coeff0_53*basisvalue53 + coeff0_54*basisvalue54 + coeff0_55*basisvalue55;
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
    
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
    
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-14)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-14)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
    
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
    
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
    
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    const double scalings_y_5 = scalings_y_4*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    const double scalings_z_3 = scalings_z_2*(0.5 - 0.5*z);
    const double scalings_z_4 = scalings_z_3*(0.5 - 0.5*z);
    const double scalings_z_5 = scalings_z_4*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    const double psitilde_a_5 = 1.8*x*psitilde_a_4 - 0.8*psitilde_a_3;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_0_5 = 0.0185185185185185*psitilde_bs_0_4 + 1.83333333333333*y*psitilde_bs_0_4 - 0.814814814814815*psitilde_bs_0_3;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_1_4 = 0.178571428571429*psitilde_bs_1_3 + 1.96428571428571*y*psitilde_bs_1_3 - 0.785714285714286*psitilde_bs_1_2;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_2_3 = 0.578703703703704*psitilde_bs_2_2 + 2.29166666666667*y*psitilde_bs_2_2 - 0.712962962962963*psitilde_bs_2_1;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_3_2 = 1.51234567901235*psitilde_bs_3_1 + 3.05555555555556*y*psitilde_bs_3_1 - 0.54320987654321*psitilde_bs_3_0;
    const double psitilde_bs_4_0 = 1;
    const double psitilde_bs_4_1 = 5.5*y + 4.5;
    const double psitilde_bs_5_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_00_3 = 0.155555555555556*psitilde_cs_00_2 + 1.86666666666667*z*psitilde_cs_00_2 - 0.711111111111111*psitilde_cs_00_1;
    const double psitilde_cs_00_4 = 0.09375*psitilde_cs_00_3 + 1.875*z*psitilde_cs_00_3 - 0.78125*psitilde_cs_00_2;
    const double psitilde_cs_00_5 = 0.0628571428571429*psitilde_cs_00_4 + 1.88571428571429*z*psitilde_cs_00_4 - 0.822857142857143*psitilde_cs_00_3;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_01_2 = 0.777777777777778*psitilde_cs_01_1 + 2.33333333333333*z*psitilde_cs_01_1 - 0.555555555555556*psitilde_cs_01_0;
    const double psitilde_cs_01_3 = 0.428571428571429*psitilde_cs_01_2 + 2.14285714285714*z*psitilde_cs_01_2 - 0.714285714285714*psitilde_cs_01_1;
    const double psitilde_cs_01_4 = 0.275*psitilde_cs_01_3 + 2.0625*z*psitilde_cs_01_3 - 0.7875*psitilde_cs_01_2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_02_1 = 4*z + 3;
    const double psitilde_cs_02_2 = 1.265625*psitilde_cs_02_1 + 2.8125*z*psitilde_cs_02_1 - 0.546875*psitilde_cs_02_0;
    const double psitilde_cs_02_3 = 0.733333333333333*psitilde_cs_02_2 + 2.44444444444444*z*psitilde_cs_02_2 - 0.711111111111111*psitilde_cs_02_1;
    const double psitilde_cs_03_0 = 1;
    const double psitilde_cs_03_1 = 5*z + 4;
    const double psitilde_cs_03_2 = 1.76*psitilde_cs_03_1 + 3.3*z*psitilde_cs_03_1 - 0.54*psitilde_cs_03_0;
    const double psitilde_cs_04_0 = 1;
    const double psitilde_cs_04_1 = 6*z + 5;
    const double psitilde_cs_05_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_10_2 = 0.777777777777778*psitilde_cs_10_1 + 2.33333333333333*z*psitilde_cs_10_1 - 0.555555555555556*psitilde_cs_10_0;
    const double psitilde_cs_10_3 = 0.428571428571429*psitilde_cs_10_2 + 2.14285714285714*z*psitilde_cs_10_2 - 0.714285714285714*psitilde_cs_10_1;
    const double psitilde_cs_10_4 = 0.275*psitilde_cs_10_3 + 2.0625*z*psitilde_cs_10_3 - 0.7875*psitilde_cs_10_2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_11_1 = 4*z + 3;
    const double psitilde_cs_11_2 = 1.265625*psitilde_cs_11_1 + 2.8125*z*psitilde_cs_11_1 - 0.546875*psitilde_cs_11_0;
    const double psitilde_cs_11_3 = 0.733333333333333*psitilde_cs_11_2 + 2.44444444444444*z*psitilde_cs_11_2 - 0.711111111111111*psitilde_cs_11_1;
    const double psitilde_cs_12_0 = 1;
    const double psitilde_cs_12_1 = 5*z + 4;
    const double psitilde_cs_12_2 = 1.76*psitilde_cs_12_1 + 3.3*z*psitilde_cs_12_1 - 0.54*psitilde_cs_12_0;
    const double psitilde_cs_13_0 = 1;
    const double psitilde_cs_13_1 = 6*z + 5;
    const double psitilde_cs_14_0 = 1;
    const double psitilde_cs_20_0 = 1;
    const double psitilde_cs_20_1 = 4*z + 3;
    const double psitilde_cs_20_2 = 1.265625*psitilde_cs_20_1 + 2.8125*z*psitilde_cs_20_1 - 0.546875*psitilde_cs_20_0;
    const double psitilde_cs_20_3 = 0.733333333333333*psitilde_cs_20_2 + 2.44444444444444*z*psitilde_cs_20_2 - 0.711111111111111*psitilde_cs_20_1;
    const double psitilde_cs_21_0 = 1;
    const double psitilde_cs_21_1 = 5*z + 4;
    const double psitilde_cs_21_2 = 1.76*psitilde_cs_21_1 + 3.3*z*psitilde_cs_21_1 - 0.54*psitilde_cs_21_0;
    const double psitilde_cs_22_0 = 1;
    const double psitilde_cs_22_1 = 6*z + 5;
    const double psitilde_cs_23_0 = 1;
    const double psitilde_cs_30_0 = 1;
    const double psitilde_cs_30_1 = 5*z + 4;
    const double psitilde_cs_30_2 = 1.76*psitilde_cs_30_1 + 3.3*z*psitilde_cs_30_1 - 0.54*psitilde_cs_30_0;
    const double psitilde_cs_31_0 = 1;
    const double psitilde_cs_31_1 = 6*z + 5;
    const double psitilde_cs_32_0 = 1;
    const double psitilde_cs_40_0 = 1;
    const double psitilde_cs_40_1 = 6*z + 5;
    const double psitilde_cs_41_0 = 1;
    const double psitilde_cs_50_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.866025403784439*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.11803398874989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.1234753829798*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.96862696659689*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.29128784747792*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.24037034920393*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.87082869338697*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.3228756555323*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    const double basisvalue10 = 7.93725393319377*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_0;
    const double basisvalue11 = 6.70820393249937*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_0;
    const double basisvalue12 = 5.19615242270663*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_0;
    const double basisvalue13 = 3*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_0;
    const double basisvalue14 = 5.80947501931113*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_1;
    const double basisvalue15 = 4.5*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_1;
    const double basisvalue16 = 2.59807621135332*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_1;
    const double basisvalue17 = 3.67423461417477*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_2;
    const double basisvalue18 = 2.12132034355964*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_2;
    const double basisvalue19 = 1.5*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_3;
    const double basisvalue20 = 11.1242977306435*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_0;
    const double basisvalue21 = 9.81070843517429*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_0;
    const double basisvalue22 = 8.2915619758885*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_0;
    const double basisvalue23 = 6.42261628933256*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_0;
    const double basisvalue24 = 3.70809924354783*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_0;
    const double basisvalue25 = 8.77496438739212*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_1;
    const double basisvalue26 = 7.41619848709566*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_1;
    const double basisvalue27 = 5.74456264653803*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_1;
    const double basisvalue28 = 3.3166247903554*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_1;
    const double basisvalue29 = 6.42261628933256*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_2;
    const double basisvalue30 = 4.9749371855331*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_2;
    const double basisvalue31 = 2.87228132326901*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_2;
    const double basisvalue32 = 4.06201920231798*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_3;
    const double basisvalue33 = 2.34520787991171*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_3;
    const double basisvalue34 = 1.6583123951777*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_4;
    const double basisvalue35 = 14.6458185158768*psitilde_a_5*scalings_y_5*psitilde_bs_5_0*scalings_z_5*psitilde_cs_50_0;
    const double basisvalue36 = 13.2476412994918*psitilde_a_4*scalings_y_4*psitilde_bs_4_1*scalings_z_5*psitilde_cs_41_0;
    const double basisvalue37 = 11.6833214455479*psitilde_a_3*scalings_y_3*psitilde_bs_3_2*scalings_z_5*psitilde_cs_32_0;
    const double basisvalue38 = 9.87420882906575*psitilde_a_2*scalings_y_2*psitilde_bs_2_3*scalings_z_5*psitilde_cs_23_0;
    const double basisvalue39 = 7.64852927038918*psitilde_a_1*scalings_y_1*psitilde_bs_1_4*scalings_z_5*psitilde_cs_14_0;
    const double basisvalue40 = 4.41588043316392*psitilde_a_0*scalings_y_0*psitilde_bs_0_5*scalings_z_5*psitilde_cs_05_0;
    const double basisvalue41 = 12.0933866224478*psitilde_a_4*scalings_y_4*psitilde_bs_4_0*scalings_z_4*psitilde_cs_40_1;
    const double basisvalue42 = 10.6653645038508*psitilde_a_3*scalings_y_3*psitilde_bs_3_1*scalings_z_4*psitilde_cs_31_1;
    const double basisvalue43 = 9.01387818865997*psitilde_a_2*scalings_y_2*psitilde_bs_2_2*scalings_z_4*psitilde_cs_22_1;
    const double basisvalue44 = 6.98212002188447*psitilde_a_1*scalings_y_1*psitilde_bs_1_3*scalings_z_4*psitilde_cs_13_1;
    const double basisvalue45 = 4.03112887414927*psitilde_a_0*scalings_y_0*psitilde_bs_0_4*scalings_z_4*psitilde_cs_04_1;
    const double basisvalue46 = 9.53939201416946*psitilde_a_3*scalings_y_3*psitilde_bs_3_0*scalings_z_3*psitilde_cs_30_2;
    const double basisvalue47 = 8.06225774829855*psitilde_a_2*scalings_y_2*psitilde_bs_2_1*scalings_z_3*psitilde_cs_21_2;
    const double basisvalue48 = 6.2449979983984*psitilde_a_1*scalings_y_1*psitilde_bs_1_2*scalings_z_3*psitilde_cs_12_2;
    const double basisvalue49 = 3.60555127546399*psitilde_a_0*scalings_y_0*psitilde_bs_0_3*scalings_z_3*psitilde_cs_03_2;
    const double basisvalue50 = 6.98212002188447*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_3;
    const double basisvalue51 = 5.40832691319598*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_3;
    const double basisvalue52 = 3.1224989991992*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_3;
    const double basisvalue53 = 4.41588043316392*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_4;
    const double basisvalue54 = 2.54950975679639*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_4;
    const double basisvalue55 = 1.80277563773199*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_5;
    
    // Table(s) of coefficients
    static const double coefficients0[56][56] = \
    {{0.00945067404923492, 0.00609788120679252, 0.00352061335622803, 0.00248944957812476, 0.0070594623332504, 0.00546823600997146, 0.00315708753234943, 0.00446479600584755, 0.00257775117585285, 0.00182274533665715, -0.00298267418725716, -0.0025208197798293, -0.00195261860521383, -0.00112734487734488, -0.00218309396769447, -0.00169101731601731, -0.000976309302606919, -0.00138070985681772, -0.000797153207506487, -0.00056367243867244, 0.00278687112460004, 0.00245778931055954, 0.00207721109304566, 0.00160900079398463, 0.000928957041533343, 0.00219831358911339, 0.0018579140830667, 0.00143913406048032, 0.00083088443721827, 0.00160900079398463, 0.0012463266558274, 0.000719567030240151, 0.00101762145320217, 0.000587524019939407, 0.000415442218609132, -0.00705593524717798, -0.00638233357089081, -0.00562868913761196, -0.00475711057319694, -0.00368484200521179, -0.00212744452363026, -0.00582624677716565, -0.00513826668309062, -0.00434262794914383, -0.00336378514516178, -0.00194208225905522, -0.00459580543596519, -0.00388416451811043, -0.00300866089851429, -0.00173705117965752, -0.00336378514516178, -0.00260557676948629, -0.00150433044925714, -0.00212744452363027, -0.00122828066840393, -0.000868525589828762},
    {0.00945067404923495, -0.00609788120679253, 0.00352061335622802, 0.00248944957812476, 0.00705946233325039, -0.00546823600997146, 0.00315708753234944, -0.00446479600584756, 0.00257775117585286, 0.00182274533665715, 0.00298267418725716, -0.00252081977982931, 0.00195261860521383, -0.00112734487734489, -0.00218309396769448, 0.00169101731601732, -0.000976309302606925, 0.00138070985681772, -0.000797153207506488, -0.000563672438672441, 0.00278687112460004, -0.00245778931055954, 0.00207721109304566, -0.00160900079398463, 0.000928957041533346, -0.00219831358911339, 0.00185791408306669, -0.00143913406048031, 0.000830884437218268, 0.00160900079398463, -0.0012463266558274, 0.000719567030240148, -0.00101762145320216, 0.000587524019939402, 0.000415442218609128, 0.00705593524717798, -0.0063823335708908, 0.00562868913761196, -0.00475711057319694, 0.00368484200521179, -0.00212744452363027, -0.00582624677716565, 0.00513826668309062, -0.00434262794914382, 0.00336378514516178, -0.00194208225905522, 0.0045958054359652, -0.00388416451811043, 0.00300866089851429, -0.00173705117965752, -0.00336378514516178, 0.00260557676948629, -0.00150433044925714, 0.00212744452363026, -0.00122828066840393, -0.00086852558982876},
    {0.00945067404923495, 0, -0.00704122671245605, 0.00248944957812476, 0, 0, 0.00947126259704828, 0, -0.00515550235170572, 0.00182274533665715, 0, 0, 0, 0.00450937950937951, 0, 0, -0.00292892790782075, 0, 0.00159430641501296, -0.00056367243867244, 0, 0, 0, 0, 0.00464478520766673, 0, 0, 0, -0.00332353774887306, 0, 0, 0.00215870109072046, 0, -0.00117504803987881, 0.00041544221860913, 0, 0, 0, 0, 0, 0.0127646671417816, 0, 0, 0, 0, -0.00971041129527608, 0, 0, 0, 0.00694820471863012, 0, 0, -0.00451299134777144, 0, 0.00245656133680786, -0.000868525589828763},
    {0.00945067404923493, 0, 0, -0.0074683487343743, 0, 0, 0, 0, 0, 0.0109364720199429, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00563672438672439, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.00623163327913698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.018239037386404},
    {-0.010023442173431, 0, 0.0374718748311399, -0.0289614183593723, 0, 0, 0.0284137877911448, 0, 0.00515550235170572, -0.00619733414463434, 0, 0, 0, 0.0315656565656566, 0, 0, -0.0113902751970807, 0, 0.00265717735835494, -0.000187890812890802, 0, 0, 0, 0, 0.0232239260383337, 0, 0, 0, -0.00166176887443652, 0, 0, -0.00647610327216137, 0, 0.00646276421933342, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0213629048496074, 0, 0, 0, -0.0277928188745205, 0, 0, 0.0243701532779658, 0, -0.0157219925555703, 0.00607967912880134},
    {0.010023442173431, 0, -0.0287574853355259, 0.05360943398437, 0, 0, -0.0189425251940966, 0, 0.0154665070551171, 0.0153110608279201, 0, 0, 0, -0.00450937950937953, 0, 0, 0.0370997534990628, 0, -0.0281660799985623, 0.0108976671476671, 0, 0, 0, 0, 0, 0, 0, 0, 0.0299118397398575, 0, 0, -0.00431740218144092, 0, -0.0123380044187274, 0.00872428659079174, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0312669212338355, 0, 0, -0.0487403065559316, 0, 0.041270230458372, -0.018239037386404},
    {0.010023442173431, 0, 0.0409576306293854, -0.0449826285156208, 0, 0, 0.0189425251940965, 0, -0.00515550235170569, -0.021143845905223, 0, 0, 0, 0, 0, 0, 0.00455611007883226, 0, 0.0356061766019561, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0302218152700864, 0, 0.00411266813957583, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0379091273212801, 0, -0.055026973944496, 0.0303983956440067},
    {-0.0100234421734311, 0, -0.0148144621425437, 0.0449826285156207, 0, 0, 0, 0, 0.0206220094068229, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0, 0.0106287094334197, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0205633406978791, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.03439185871531, -0.0303983956440067},
    {-0.010023442173431, 0.0324515955311979, -0.0187359374155699, -0.0289614183593723, 0.0211783869997512, -0.0164047080299144, 0.00947126259704827, 0.00446479600584757, -0.00257775117585285, -0.00619733414463432, 0.0208787193108001, -0.0176457384588052, 0.0136683302364968, -0.00789141414141413, -0.0084898098743674, 0.00657617845117844, -0.00379675839902689, 0.0023011830946962, -0.00132858867917746, -0.0001878908128908, 0.0139343556230002, -0.0122889465527977, 0.0103860554652283, -0.00804500396992314, 0.00464478520766674, -0.0010991567945567, 0.000928957041533349, -0.000719567030240152, 0.000415442218609127, -0.00482700238195388, 0.00373897996748219, -0.00215870109072045, 0.00559691799261185, -0.00323138210966672, -0.00290809553026392, 0, 0, 0, 0, 0, 0, 0.0128177429097644, -0.0113041867027994, 0.00955378148811641, -0.00740032731935591, 0.00427258096992148, -0.0183832217438608, 0.0155366580724417, -0.0120346435940572, 0.00694820471863011, 0.0181644397838736, -0.014070114555226, 0.00812338442598859, -0.0136156449512337, 0.00786099627778515, 0.00607967912880135},
    {0.010023442173431, -0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, 0.0109364720199429, -0.00631417506469883, 0.0133943880175427, -0.00773325352755858, 0.0153110608279201, -0.00298267418725716, 0.00252081977982931, -0.00195261860521382, 0.00112734487734488, 0.0276525235907967, -0.0214195526695527, 0.0123665844996876, -0.0243925408037797, 0.0140830399992811, 0.0108976671476671, 0, 0, 0, 0, 0, 0.0197848223020205, -0.0167212267476002, 0.0129522065443227, -0.00747795993496437, -0.00321800158796926, 0.0024926533116548, -0.00143913406048031, -0.0106850252586226, 0.00616900220936373, 0.00872428659079178, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0206811244618434, -0.017478740331497, 0.0135389740433143, -0.00781673030845888, -0.0363288795677472, 0.028140229110452, -0.0162467688519772, 0.0357410679969885, -0.020635115229186, -0.0182390373864041},
    {0.0100234421734311, 0.0354703486038674, -0.0204788153146927, -0.0449826285156208, 0.0141189246665008, -0.0109364720199429, 0.00631417506469885, -0.00446479600584758, 0.00257775117585285, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974696, -0.00263047138047138, 0.00151870335961075, 0.0308358534689291, -0.017803088300978, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, -0.0174485731815835, 0.0100739384233621, 0.00356167508620757, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, -0.0218868448636849, 0.01263637577376, -0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, -0.0128297005588457, 0.00740723107127184, 0.0449826285156208, 0, 0, 0, 0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, 0.00920473237878476, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0178083754310377, -0.0102816703489395, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.010023442173431, 0.0324515955311979, -0.0335503995581136, -0.00801060507812427, 0.0211783869997512, 0.00109364720199429, -0.00568275755822898, -0.0169662248222207, 0.0056710525868763, 0.00838462854862291, 0.0208787193108001, -0.0109235523792603, 0.00499002532443534, -0.00187890812890812, -0.0162519217595033, 0.00958243145743146, -0.00466458889023305, 0.0108922666482287, -0.00558007245254534, -0.00469727032227033, 0.0139343556230002, -0.00245778931055954, -0.00415442218609131, 0.00643600317593851, -0.00464478520766673, -0.0120907247401236, 0.00464478520766674, -0.000719567030240153, -0.000415442218609137, 0.00965400476390778, -0.00498530662330958, 0.00215870109072046, -0.006614539445814, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, -0.0202632808954031, 0.0228341307513453, -0.020635115229186, 0.0127646671417816, -0.00116524935543313, -0.00719357335632687, 0.0112908326677739, -0.0114368694935501, 0.00737991258440983, 0.00183832217438609, 0.00310733161448835, -0.00481385743762287, 0.00347410235931505, -0.00201827108709706, -0.000521115353897262, 0.000902598269554289, 0.00170195561890421, -0.000491312267361565, -0.000868525589828764},
    {0.0100234421734309, -0.024904712849524, 0.0553363732971484, -0.0043134027343746, -0.0141189246665008, 0.0153110608279201, 0.00883984509057843, 0.00803663281052563, -0.0159820572902877, 0.000729098134662861, -0.00298267418725715, 0.0277290175781224, -0.0301571095694136, 0.0191648629148629, -0.00145539597846298, -0.0116492303992304, 0.00976309302606916, 0.00352848074520084, 0.00132858867917745, -0.00263047138047137, 0, 0.0196623144844763, -0.00727023882565982, -0.00724050357293082, 0.00928957041533346, -0.00219831358911338, -0.0148633126645336, 0.0129522065443228, -0.0066470754977461, 0.00402250198496157, 0.00685479660705067, -0.00575653624192123, -0.0045792965394097, -0.000293762009969684, 0.00290809553026392, 0, 0, 0.0202632808954031, -0.0399597288148543, 0.0464290092656686, -0.031911667854454, 0, -0.00411061334647249, -0.00303983956440068, 0.00908221989193681, -0.00776832903622088, 0.00045958054359652, 0.00427258096992148, -0.00270779480866287, 0.000868525589828773, -0.00100913554354854, -0.00234501909253766, 0.00180519653910858, 0.00127646671417816, 0.000245656133680781, -0.000868525589828764},
    {0.0100234421734309, 0.0354703486038675, -0.0492363006502187, -0.00431340273437461, 0.0141189246665008, -0.00656188321196574, -0.016416855168217, -0.00982255121286466, 0.0149509568199466, 0.000729098134662872, 0, 0.0033610930397724, 0.0282044909641998, -0.0353234728234728, -0.000485131992820993, -0.0124007936507936, 0.00889526253486299, 0.00291483191994851, 0.00239145962251946, -0.00263047138047138, 0, 0, 0.0218107164769794, -0.00402250198496158, -0.00928957041533347, 0, -0.00557374224920008, -0.0143913406048031, 0.0141250354327105, 0.000804500396992316, 0.00934744991870547, -0.0028782681209606, -0.00254405363300539, -0.00381890612960613, 0.00290809553026392, 0, 0, 0, 0.0266398192099029, -0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320202, 0.00840946286290443, -0.00388416451811042, 0, 0.00233049871086626, 0.0030086608985143, -0.00347410235931507, -0.000336378514516178, -0.00286613444643493, 0.00120346435940572, 0.00085097780945211, 0.00098262453472315, -0.000868525589828765},
    {-0.010023442173431, -0.0128297005588457, 0.0448791059024117, -0.00801060507812424, 0, 0.0174983552319087, 0.0132597676358676, -0.00357183680467804, -0.0175287079957994, 0.0083846285486229, 0, 0, 0.00867830491206149, 0.0225468975468975, 0, -0.00300625300625301, -0.019200749617936, 0.000613648825252316, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, 0.0160900079398463, 0.00464478520766673, 0, 0, -0.00719567030240152, -0.0108014976838374, 0, 0.00249265331165478, 0.0107935054536023, -0.000508810726601074, -0.00734405024924252, 0.00290809553026393, 0, 0, 0, 0, 0.0257938940364825, -0.031911667854454, 0, 0, 0, -0.0134551405806471, 0.0135945758133865, 0, 0, 0.00601732179702858, -0.00347410235931505, 0, -0.00208446141558903, -0.000902598269554287, 0.000425488904726055, 0.0017195929357655, -0.000868525589828767},
    {-0.0100234421734311, -0.0324515955311978, -0.0187359374155699, -0.0289614183593723, 0.0211783869997511, 0.0164047080299144, 0.00947126259704826, -0.00446479600584755, -0.00257775117585285, -0.00619733414463435, -0.0208787193108001, -0.0176457384588052, -0.0136683302364968, -0.00789141414141413, -0.00848980987436742, -0.00657617845117845, -0.0037967583990269, -0.00230118309469619, -0.00132858867917746, -0.00018789081289081, 0.0139343556230002, 0.0122889465527977, 0.0103860554652283, 0.00804500396992313, 0.00464478520766674, 0.00109915679455668, 0.000928957041533345, 0.000719567030240154, 0.000415442218609131, -0.00482700238195388, -0.00373897996748219, -0.00215870109072045, -0.00559691799261187, -0.00323138210966673, -0.00290809553026391, 0, 0, 0, 0, 0, 0, 0.0128177429097644, 0.0113041867027994, 0.00955378148811641, 0.00740032731935591, 0.00427258096992148, 0.0183832217438608, 0.0155366580724417, 0.0120346435940572, 0.0069482047186301, 0.0181644397838736, 0.014070114555226, 0.00812338442598859, 0.0136156449512337, 0.00786099627778516, 0.00607967912880135},
    {0.0100234421734308, 0.024904712849524, 0.014378742667763, 0.05360943398437, -0.0141189246665008, -0.0109364720199429, -0.00631417506469886, -0.0133943880175427, -0.00773325352755858, 0.0153110608279201, 0.00298267418725716, 0.00252081977982933, 0.00195261860521383, 0.00112734487734487, 0.0276525235907967, 0.0214195526695527, 0.0123665844996876, 0.0243925408037797, 0.0140830399992812, 0.0108976671476671, 0, 0, 0, 0, 0, -0.0197848223020205, -0.0167212267476002, -0.0129522065443228, -0.00747795993496438, -0.00321800158796927, -0.00249265331165479, -0.00143913406048031, 0.0106850252586227, 0.00616900220936374, 0.00872428659079177, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0206811244618434, -0.017478740331497, -0.0135389740433143, -0.00781673030845888, -0.0363288795677472, -0.028140229110452, -0.0162467688519772, -0.0357410679969885, -0.0206351152291861, -0.0182390373864041},
    {0.010023442173431, -0.0354703486038674, -0.0204788153146927, -0.0449826285156208, 0.0141189246665008, 0.0109364720199429, 0.00631417506469886, 0.00446479600584758, 0.00257775117585287, -0.021143845905223, 0, 0, 0, 0, 0.00339592394974696, 0.00263047138047138, 0.00151870335961075, -0.0308358534689291, -0.0178030883009781, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0225260111157848, 0.0174485731815835, 0.0100739384233621, -0.00356167508620757, -0.00205633406978792, -0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0282557952193589, 0.0218868448636849, 0.01263637577376, 0.047654757329318, 0.027513486972248, 0.0303983956440067},
    {-0.010023442173431, 0.0128297005588457, 0.00740723107127183, 0.0449826285156208, 0, 0, 0, -0.0178591840233903, -0.0103110047034114, 0.021143845905223, 0, 0, 0, 0, 0, 0, 0, -0.00920473237878479, -0.00531435471670985, 0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0178083754310377, -0.0102816703489395, 0.0145404776513196, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0297842233308237, -0.017195929357655, -0.0303983956440068},
    {-0.010023442173431, -0.0324515955311978, -0.0335503995581136, -0.00801060507812424, 0.0211783869997512, -0.00109364720199429, -0.00568275755822899, 0.0169662248222207, 0.00567105258687632, 0.0083846285486229, -0.0208787193108001, -0.0109235523792603, -0.00499002532443534, -0.00187890812890812, -0.0162519217595033, -0.00958243145743146, -0.00466458889023305, -0.0108922666482287, -0.00558007245254535, -0.00469727032227032, 0.0139343556230002, 0.00245778931055953, -0.00415442218609133, -0.00643600317593852, -0.00464478520766672, 0.0120907247401236, 0.00464478520766673, 0.000719567030240156, -0.000415442218609149, 0.00965400476390777, 0.00498530662330958, 0.00215870109072047, 0.00661453944581401, 0.0032313821096667, 0.00290809553026393, 0, 0.0127646671417816, 0.0202632808954031, 0.0228341307513453, 0.020635115229186, 0.0127646671417816, -0.00116524935543313, 0.00719357335632687, 0.0112908326677739, 0.0114368694935501, 0.00737991258440984, -0.00183832217438608, 0.00310733161448834, 0.00481385743762287, 0.00347410235931505, -0.00201827108709706, 0.000521115353897262, 0.000902598269554287, -0.00170195561890421, -0.000491312267361565, -0.000868525589828767},
    {0.010023442173431, 0.0249047128495239, 0.0553363732971484, -0.00431340273437459, -0.0141189246665008, -0.0153110608279201, 0.00883984509057843, -0.00803663281052563, -0.0159820572902878, 0.000729098134662854, 0.00298267418725716, 0.0277290175781224, 0.0301571095694136, 0.0191648629148629, -0.00145539597846298, 0.0116492303992304, 0.00976309302606918, -0.00352848074520084, 0.00132858867917746, -0.00263047138047138, 0, -0.0196623144844763, -0.0072702388256598, 0.00724050357293083, 0.00928957041533346, 0.00219831358911339, -0.0148633126645336, -0.0129522065443228, -0.00664707549774609, 0.00402250198496157, -0.00685479660705067, -0.00575653624192124, 0.0045792965394097, -0.000293762009969679, 0.00290809553026392, 0, 0, -0.0202632808954031, -0.0399597288148543, -0.0464290092656686, -0.031911667854454, 0, 0.0041106133464725, -0.00303983956440067, -0.00908221989193682, -0.00776832903622089, -0.00045958054359652, 0.00427258096992148, 0.00270779480866287, 0.000868525589828776, -0.00100913554354853, 0.00234501909253766, 0.00180519653910858, -0.00127646671417816, 0.000245656133680779, -0.000868525589828761},
    {0.0100234421734309, -0.0354703486038674, -0.0492363006502187, -0.00431340273437459, 0.0141189246665008, 0.00656188321196577, -0.016416855168217, 0.00982255121286466, 0.0149509568199466, 0.000729098134662868, 0, 0.0033610930397724, -0.0282044909641998, -0.0353234728234728, -0.000485131992820994, 0.0124007936507936, 0.00889526253486299, -0.00291483191994852, 0.00239145962251945, -0.00263047138047138, 0, 0, 0.0218107164769794, 0.00402250198496157, -0.00928957041533346, 0, -0.00557374224920009, 0.0143913406048031, 0.0141250354327105, 0.000804500396992309, -0.00934744991870548, -0.0028782681209606, 0.00254405363300539, -0.00381890612960614, 0.00290809553026393, 0, 0, 0, 0.0266398192099028, 0.0515877880729651, 0.0425488904726053, 0, 0, -0.00911951869320203, -0.00840946286290443, -0.00388416451811041, 0, 0.00233049871086626, -0.0030086608985143, -0.00347410235931507, -0.000336378514516177, 0.00286613444643493, 0.00120346435940572, -0.000850977809452108, 0.000982624534723152, -0.000868525589828766},
    {-0.010023442173431, 0.0128297005588457, 0.0448791059024117, -0.00801060507812424, 0, -0.0174983552319087, 0.0132597676358676, 0.00357183680467805, -0.0175287079957994, 0.0083846285486229, 0, 0, -0.00867830491206148, 0.0225468975468975, 0, 0.00300625300625301, -0.019200749617936, -0.000613648825252319, 0.0122230158484327, -0.00469727032227032, 0, 0, 0, -0.0160900079398463, 0.00464478520766673, 0, 0, 0.00719567030240153, -0.0108014976838374, 0, -0.00249265331165479, 0.0107935054536023, 0.000508810726601077, -0.00734405024924252, 0.00290809553026393, 0, 0, 0, 0, -0.0257938940364826, -0.031911667854454, 0, 0, 0, 0.0134551405806471, 0.0135945758133865, 0, 0, -0.00601732179702859, -0.00347410235931505, 0, 0.00208446141558903, -0.000902598269554286, -0.000425488904726054, 0.0017195929357655, -0.000868525589828767},
    {-0.0100234421734311, -0.0452812960900435, -0.0113287063442981, -0.00801060507812424, -0.0014118924666501, 0.0164047080299144, 0.0145226026488073, 0.0133943880175427, 0.0118576554089232, 0.00838462854862289, -0.00894802256177147, -0.0142846454190328, -0.014536160727703, -0.00939454064454064, -0.0123708658169354, -0.0125886844636845, -0.00813591085505764, -0.0102786178229763, -0.00664294339588733, -0.00469727032227033, -0.00836061337380012, 0.00245778931055953, 0.00830884437218264, 0.00965400476390777, 0.00650269929073344, 0.00219831358911338, 0.00743165633226677, 0.00863480436288183, 0.00581619106052785, 0.00643600317593852, 0.00747795993496437, 0.00503696921168108, 0.00610572871921293, 0.00411266813957582, 0.00290809553026394, 0.0352796762358899, 0.0191470007126724, 0.00788016479265675, 0.000951422114639391, -0.00221090520312707, -0.00212744452363027, 0.017478740331497, 0.00719357335632686, 0.000868525589828764, -0.00201827108709706, -0.00194208225905521, 0.00643412761035128, 0.000776832903622088, -0.00180519653910857, -0.00173705117965753, 0.000672757029032363, -0.00156334606169177, -0.00150433044925715, -0.00127646671417816, -0.00122828066840393, -0.000868525589828768},
    {0.010023442173431, 0.0603750614533914, -0.00610007264692975, -0.0043134027343746, -0.00564756986660025, -0.0218729440398859, 0.00126283501293978, -0.0178591840233903, 0.00103110047034114, 0.000729098134662884, 0.0417574386216002, 0.0168054651988621, -0.000867830491206148, -0.00526094276094276, 0.0145539597846299, -0.000751563251563252, -0.00455611007883227, -0.000613648825252328, -0.0037200483016969, -0.00263047138047138, 0.00557374224920005, -0.0196623144844763, -0.0103860554652283, 0.00321800158796928, 0.00650269929073341, -0.0175865087129071, -0.00928957041533347, 0.00287826812096063, 0.00581619106052784, -0.00804500396992315, 0.0024926533116548, 0.00503696921168106, 0.00203524290640433, 0.00411266813957581, 0.00290809553026391, -0.0705593524717798, -0.0127646671417816, 0.00450295131008958, 0.00380568845855754, -0.000736968401042372, -0.00212744452363026, -0.0116524935543313, 0.00411061334647251, 0.00347410235931506, -0.000672757029032362, -0.00194208225905521, 0.00367664434877218, 0.00310733161448835, -0.000601732179702865, -0.00173705117965753, 0.00269102811612942, -0.000521115353897265, -0.00150433044925715, -0.00042548890472606, -0.00122828066840393, -0.000868525589828759},
    {0.0100234421734309, -0.0603750614533914, -0.00610007264692972, -0.00431340273437457, -0.00564756986660032, 0.0218729440398859, 0.00126283501293976, 0.0178591840233903, 0.00103110047034113, 0.000729098134662861, -0.0417574386216002, 0.016805465198862, 0.000867830491206157, -0.00526094276094276, 0.0145539597846298, 0.000751563251563257, -0.00455611007883228, 0.00061364882525233, -0.00372004830169691, -0.00263047138047138, 0.00557374224920011, 0.0196623144844763, -0.0103860554652283, -0.00321800158796927, 0.00650269929073344, 0.0175865087129071, -0.00928957041533346, -0.00287826812096062, 0.00581619106052786, -0.00804500396992314, -0.0024926533116548, 0.00503696921168108, -0.00203524290640433, 0.00411266813957583, 0.00290809553026393, 0.0705593524717798, -0.0127646671417816, -0.00450295131008958, 0.00380568845855756, 0.000736968401042369, -0.00212744452363027, -0.0116524935543313, -0.00411061334647251, 0.00347410235931506, 0.000672757029032361, -0.00194208225905522, -0.00367664434877217, 0.00310733161448835, 0.000601732179702866, -0.00173705117965753, 0.00269102811612942, 0.000521115353897263, -0.00150433044925715, 0.000425488904726061, -0.00122828066840394, -0.000868525589828769},
    {-0.0100234421734309, 0.0452812960900435, -0.0113287063442981, -0.00801060507812426, -0.00141189246665007, -0.0164047080299144, 0.0145226026488074, -0.0133943880175427, 0.0118576554089231, 0.00838462854862291, 0.00894802256177147, -0.0142846454190327, 0.014536160727703, -0.00939454064454063, -0.0123708658169354, 0.0125886844636845, -0.00813591085505763, 0.0102786178229763, -0.00664294339588732, -0.00469727032227032, -0.00836061337380015, -0.00245778931055953, 0.00830884437218263, -0.00965400476390777, 0.00650269929073343, -0.00219831358911339, 0.00743165633226677, -0.00863480436288183, 0.00581619106052785, 0.00643600317593852, -0.00747795993496437, 0.00503696921168107, -0.00610572871921293, 0.00411266813957582, 0.00290809553026393, -0.0352796762358899, 0.0191470007126724, -0.00788016479265675, 0.000951422114639385, 0.00221090520312707, -0.00212744452363027, 0.0174787403314969, -0.00719357335632687, 0.000868525589828768, 0.00201827108709706, -0.00194208225905521, -0.00643412761035128, 0.000776832903622089, 0.00180519653910857, -0.00173705117965753, 0.000672757029032358, 0.00156334606169177, -0.00150433044925715, 0.00127646671417816, -0.00122828066840393, -0.000868525589828765},
    {0.0787556170769579, 0.0679219441350653, -0.00435719474780691, -0.00308100195312472, 0.0705946233325039, -0.0328094160598287, 0.0126283501293977, -0.0267887760350854, 0.03608851646194, -0.00729098134662862, 0.0298267418725716, 0.0151249186789758, -0.0325436434202305, 0.0248015873015873, 0.0130985638061669, -0.0229226791726792, 0.0169226945785199, -0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, 0.0245778931055954, -0.036351194128299, 0.0362025178646541, -0.0232239260383337, 0.0219831358911338, -0.00743165633226679, 0, 0.00166176887443652, -0.0345935170706695, 0.0130864298861876, -0.00359783515120079, 0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0.0226083734055987, -0.0334382352084074, 0.0333014729371016, -0.0213629048496074, -0.00459580543596521, -0.0194208225905522, 0.0270779480866286, -0.0191075629762328, 0.0090822198919368, 0.00703505727761299, -0.00812338442598858, -0.0102117337134253, 0.00196524906944628, 0.00607967912880136},
    {-0.0214788046573522, 0.00377344134083702, 0.00217859737390344, -0.0215670136718729, 0, 0.0984282481794862, -0.0852413633734345, -0.0312535720409329, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, 0.0236742424242424, -0.0335199527228374, -0.00138070985681772, -0.00079715320750645, 0.00394570707070704, 0, 0, 0.036351194128299, -0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838374, -0.00724050357293084, -0.0392592896585629, 0.0259044130886455, 0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, -0.0555024548951693, 0.0427258096992148, 0, -0.0155366580724418, 0, 0.00694820471863008, 0.00302740663064561, 0.0164151336477636, -0.0108311792346515, -0.00680782247561686, -0.00393049813889256, 0.00607967912880134},
    {0.0787556170769579, 0.0301875307266957, 0.0610007264692975, -0.00308100195312472, 0, 0.0218729440398858, 0.0757701007763863, 0.0178591840233903, -0.0412440188136458, -0.00729098134662862, 0, 0, 0.0520698294723688, -0.00901875901875902, 0, -0.0127765752765753, 0.0286384062098028, 0.00184094647575699, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, 0.0402250198496157, -0.0464478520766674, 0, 0, 0.0143913406048031, 0.0182794576188018, 0, -0.0137095932141014, -0.0345392174515274, 0.00407048581280861, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0370016365967795, -0.0427258096992148, 0, 0, -0.0300866089851429, 0.0243187165152054, 0, 0.014070114555226, 0, -0.00340391123780844, -0.00982624534723144, 0.00607967912880136},
    {-0.0214788046573522, 0.00377344134083686, -0.0196073763651313, 0.00924300585937417, 0, -0.0109364720199429, 0.00947126259704823, 0.102690308134494, -0.0695992817480272, -0.060150596109686, 0, -0.00504163955965862, 0.0065087286840461, -0.00450937950937951, 0.0465726713108155, 0.0214195526695527, -0.0289638426440052, -0.00782402252196709, -0.0263060558477138, 0.000563672438672465, 0, 0, 0, 0, 0, 0, 0.0334424534952005, -0.0431740218144092, 0.0299118397398575, 0.0289620142917233, -0.00249265331165478, -0.0043174021814409, -0.0605484764655283, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, -0.0451299134777144, 0.0312669212338355, -0.0121096265225824, -0.028140229110452, 0.0270779480866286, 0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573521, -0.0150937653633479, 0.0130715842434208, 0.00924300585937414, 0, -0.0109364720199429, 0.00947126259704832, -0.00892959201169514, 0.123732056440937, -0.0601505961096861, 0, 0, -0.00650872868404609, 0.00676406926406927, 0, 0.0574945887445887, 0.0126920209338899, -0.0266937238984759, 0.00637722566005184, 0.000563672438672453, 0, 0, 0, 0, 0, 0, 0, 0.0431740218144092, -0.0448677596097862, 0, 0.0199412264932383, 0.0215870109072046, -0.0142467003448302, -0.0616900220936373, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0451299134777144, -0.0469003818507533, 0, -0.0375203054806026, 0.0162467688519772, 0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769578, 0.0301875307266958, 0.0174287789912279, 0.0585390371093694, 0, 0.0218729440398858, -0.0189425251940966, 0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, 0.00526094276094276, -0.00455611007883226, 0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0348971463631671, -0.0302218152700864, 0.0284934006896604, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0437736897273697, -0.0379091273212801, -0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769578, -0.0679219441350653, -0.00435719474780693, -0.00308100195312472, 0.0705946233325039, 0.0328094160598287, 0.0126283501293978, 0.0267887760350854, 0.03608851646194, -0.00729098134662859, -0.0298267418725716, 0.0151249186789758, 0.0325436434202305, 0.0248015873015873, 0.0130985638061669, 0.0229226791726792, 0.0169226945785199, 0.0248527774227189, 0.0164744996218006, 0.0142797017797018, 0, -0.0245778931055954, -0.036351194128299, -0.0362025178646541, -0.0232239260383337, -0.0219831358911339, -0.00743165633226677, 0, 0.00166176887443654, -0.0345935170706695, -0.0130864298861877, -0.00359783515120079, -0.0335815079556711, -0.014688100498485, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, -0.0226083734055987, -0.0334382352084074, -0.0333014729371016, -0.0213629048496074, 0.0045958054359652, -0.0194208225905522, -0.0270779480866286, -0.0191075629762328, 0.00908221989193679, -0.00703505727761299, -0.00812338442598858, 0.0102117337134253, 0.00196524906944628, 0.00607967912880137},
    {-0.0214788046573524, -0.00377344134083698, 0.00217859737390344, -0.021567013671873, 0, -0.0984282481794863, -0.0852413633734345, 0.031253572040933, -0.0180442582309699, 0.0309866707231716, 0, 0.0453747560369276, 0, -0.0202922077922078, -0.0116431678277039, -0.0236742424242424, -0.0335199527228374, 0.0013807098568177, -0.00079715320750645, 0.00394570707070706, 0, 0, 0.036351194128299, 0.0603375297744236, 0.0464478520766674, 0, 0.024152883079867, 0, -0.0108014976838375, -0.00724050357293083, 0.039259289658563, 0.0259044130886455, -0.0188259968842399, 0.0108691943688789, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0, 0, 0.0334382352084074, 0.0555024548951694, 0.0427258096992148, 0, -0.0155366580724417, 0, 0.00694820471863008, 0.00302740663064561, -0.0164151336477636, -0.0108311792346515, 0.00680782247561685, -0.00393049813889256, 0.00607967912880134},
    {0.0787556170769579, -0.0301875307266956, 0.0610007264692975, -0.00308100195312471, 0, -0.0218729440398858, 0.0757701007763863, -0.0178591840233903, -0.0412440188136458, -0.00729098134662861, 0, 0, -0.0520698294723689, -0.00901875901875903, 0, 0.0127765752765753, 0.0286384062098028, -0.00184094647575695, -0.0297603864135753, 0.0142797017797018, 0, 0, 0, -0.0402250198496157, -0.0464478520766674, 0, 0, -0.0143913406048031, 0.0182794576188018, 0, 0.0137095932141014, -0.0345392174515274, -0.00407048581280862, 0.036426489236243, -0.0178640154001927, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0370016365967796, -0.0427258096992148, 0, 0, 0.0300866089851429, 0.0243187165152054, 0, -0.014070114555226, 0, 0.00340391123780843, -0.00982624534723145, 0.00607967912880137},
    {-0.0214788046573523, -0.0037734413408369, -0.0196073763651314, 0.00924300585937414, 0, 0.0109364720199429, 0.00947126259704826, -0.102690308134494, -0.0695992817480271, -0.0601505961096861, 0, -0.00504163955965861, -0.00650872868404608, -0.00450937950937948, 0.0465726713108155, -0.0214195526695527, -0.0289638426440052, 0.00782402252196707, -0.0263060558477138, 0.000563672438672444, 0, 0, 0, 0, 0, 0, 0.0334424534952005, 0.0431740218144092, 0.0299118397398575, 0.0289620142917233, 0.0024926533116548, -0.00431740218144091, 0.0605484764655282, 0.0185070066280912, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0349574806629939, 0.0451299134777144, 0.0312669212338355, -0.0121096265225824, 0.028140229110452, 0.0270779480866286, -0.023827378664659, 0, -0.0182390373864041},
    {-0.0214788046573522, 0.0150937653633478, 0.0130715842434208, 0.00924300585937414, 0, 0.0109364720199429, 0.00947126259704831, 0.00892959201169514, 0.123732056440937, -0.060150596109686, 0, 0, 0.00650872868404611, 0.00676406926406927, 0, -0.0574945887445887, 0.0126920209338899, 0.0266937238984759, 0.00637722566005185, 0.000563672438672461, 0, 0, 0, 0, 0, 0, 0, -0.0431740218144092, -0.0448677596097862, 0, -0.0199412264932383, 0.0215870109072046, 0.0142467003448302, -0.0616900220936373, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0451299134777144, -0.0469003818507533, 0, 0.0375203054806026, 0.0162467688519772, -0.0119136893323295, 0.020635115229186, -0.0182390373864041},
    {0.0787556170769578, -0.0301875307266956, 0.0174287789912279, 0.0585390371093695, 0, -0.0218729440398858, -0.0189425251940966, -0.0178591840233903, 0.0103110047034114, 0.083846285486229, 0, 0, 0, 0, 0, -0.00526094276094275, -0.00455611007883226, -0.0533874477969518, 0.0308232573569172, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0348971463631671, -0.0302218152700864, -0.0284934006896603, 0.0164506725583033, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0437736897273697, -0.0379091273212801, 0.023827378664659, -0.013756743486124, 0.0303983956440068},
    {0.0787556170769576, -0.0377344134083696, -0.0566435317214905, -0.00308100195312473, 0.0423567739995023, 0.0546823600997146, 0.0378850503881931, 0.0446479600584756, 0.00515550235170574, -0.00729098134662862, 0.0417574386216002, -0.0050416395596586, -0.0247331689993752, -0.0202922077922078, 0.0295930515620807, 0.0101461038961039, 0.00216957622801536, 0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, -0.0245778931055954, -0.00519302773261415, 0.00402250198496157, 0.00464478520766674, -0.00219831358911335, -0.0130053985814669, -0.0158304746652834, -0.0108014976838374, -0.0168945083368386, -0.026796023100289, -0.0194283098164841, -0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, -0.0226083734055987, -0.00477689074405821, 0.00370016365967795, 0.00427258096992148, -0.0367664434877216, -0.00776832903622087, 0.00601732179702858, 0.00694820471863011, -0.00908221989193682, 0.00703505727761298, 0.0081233844259886, 0.00680782247561684, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573522, 0, -0.0043571947478069, -0.0215670136718729, -0.127070321998507, 0, 0.0284137877911448, 0, 0.03608851646194, 0.0309866707231716, 0, 0.0453747560369275, 0, -0.0202922077922078, -0.0422064833754266, 0, -0.00618329224984381, 0, 0.00159430641501295, 0.00394570707070706, 0.0836061337380013, 0, -0.0103860554652283, 0, 0.00464478520766673, 0, 0.024152883079867, 0, -0.0108014976838374, 0.043443021437585, 0, -0.0194283098164841, 0, -0.0217383887377579, -0.0178640154001926, 0, 0, 0, 0, 0, 0, 0.0769064574585866, 0, -0.0095537814881164, 0, 0.00427258096992147, 0, -0.0155366580724417, 0, 0.00694820471863011, -0.0181644397838736, 0, 0.00812338442598858, 0, 0.00786099627778513, 0.00607967912880133},
    {0.0787556170769579, 0.0377344134083696, -0.0566435317214905, -0.0030810019531247, 0.0423567739995023, -0.0546823600997146, 0.0378850503881931, -0.0446479600584757, 0.00515550235170574, -0.00729098134662864, -0.0417574386216002, -0.00504163955965862, 0.0247331689993752, -0.0202922077922078, 0.0295930515620807, -0.0101461038961039, 0.00216957622801536, -0.0266937238984759, 0.0132858867917747, 0.0142797017797018, -0.0557374224920008, 0.0245778931055954, -0.00519302773261416, -0.00402250198496158, 0.00464478520766674, 0.00219831358911339, -0.0130053985814669, 0.0158304746652834, -0.0108014976838374, -0.0168945083368386, 0.026796023100289, -0.0194283098164841, 0.0295110221428625, -0.0217383887377579, -0.0178640154001927, 0, 0, 0, 0, 0, 0, -0.0512709716390577, 0.0226083734055987, -0.00477689074405821, -0.00370016365967795, 0.00427258096992148, 0.0367664434877216, -0.00776832903622088, -0.00601732179702859, 0.00694820471863012, -0.00908221989193681, -0.00703505727761299, 0.00812338442598861, -0.00680782247561685, 0.00786099627778516, 0.00607967912880136},
    {-0.0214788046573522, -0.0188672067041847, 0.00653579212171051, 0.00924300585937415, 0.0141189246665008, 0, -0.00315708753234942, -0.111619900146189, -0.0541327746929101, -0.0601505961096861, -0.00894802256177146, -0.00252081977982933, 0.000650872868404606, 0.00112734487734486, -0.0276525235907967, 0.0360750360750361, 0.0374251899332651, -0.0188697013765088, 0.019928830187662, 0.000563672438672439, 0, 0, 0, 0, 0, 0.0593544669060614, 0.0167212267476002, -0.00431740218144093, -0.00747795993496438, 0.00321800158796924, 0.0224338798048931, 0.018708742786244, 0.046301776120698, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.0620433733855302, 0.017478740331497, -0.00451299134777145, -0.00781673030845887, 0.0363288795677472, -0.00938007637015066, -0.0162467688519772, -0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {-0.0214788046573521, 0.0188672067041847, 0.00653579212171047, 0.00924300585937414, 0.0141189246665007, 0, -0.00315708753234944, 0.111619900146189, -0.05413277469291, -0.060150596109686, 0.00894802256177148, -0.00252081977982932, -0.000650872868404592, 0.00112734487734487, -0.0276525235907967, -0.0360750360750361, 0.0374251899332651, 0.0188697013765088, 0.019928830187662, 0.000563672438672456, 0, 0, 0, 0, 0, -0.0593544669060614, 0.0167212267476002, 0.00431740218144092, -0.00747795993496438, 0.00321800158796927, -0.0224338798048931, 0.018708742786244, -0.0463017761206981, 0.0431830154655461, 0.0461140862656137, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0620433733855302, 0.017478740331497, 0.00451299134777143, -0.00781673030845888, 0.0363288795677472, 0.00938007637015065, -0.0162467688519772, 0.0119136893323295, -0.020635115229186, -0.0182390373864041},
    {0.0787556170769579, 0, -0.0348575579824557, 0.0585390371093695, -0.0282378493330015, 0, 0.00631417506469885, 0, -0.0206220094068228, 0.083846285486229, 0, 0, 0, 0, -0.0067918478994939, 0, 0.00151870335961074, 0, -0.0616465147138344, -0.0319414381914382, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0450520222315696, 0, 0.0100739384233621, 0, -0.0329013451166066, -0.0643935438844155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0565115904387179, 0, 0.01263637577376, 0, 0.0275134869722481, 0.0303983956440068},
    {0.0787556170769578, -0.0377344134083696, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, 0.0546823600997145, 0, 0.0446479600584757, 0.0257775117585286, 0.0291639253865145, 0.0417574386216002, 0.0285692908380655, 0.0273366604729936, 0.0187890812890813, -0.0092175078635989, -0.00789141414141412, -0.00347132196482459, -0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, -0.00491557862111904, -0.0197335053839338, -0.0394205194526234, -0.0325134964536672, -0.0241814494802472, -0.00928957041533347, -0.0158304746652834, -0.0141250354327105, -0.00241350119097694, -0.000623163327913701, -0.00215870109072047, 0.00712335017241508, 0.00411266813957581, 0.00540074884191871, 0, -0.0510586685671264, -0.0405265617908061, -0.0114170653756727, 0.010317557614593, 0.0127646671417816, 0.00466099742173251, -0.0143871467126537, -0.00564541633388696, 0.00571843474677502, 0.00737991258440982, 0.00367664434877216, -0.00155366580724418, 0.00240692871881142, 0.00347410235931506, 0.00100913554354853, 0.00026055767694863, 0.000902598269554296, -0.000850977809452108, -0.000491312267361565, -0.000868525589828763},
    {-0.0214788046573521, 0, -0.0217859737390348, 0.00308100195312476, -0.127070321998507, 0, 0.0473563129852414, 0, 0.0257775117585285, 0.0127592173566, 0, -0.0352914769176103, 0, 0.00225468975468977, 0.0509388592462045, 0, -0.00943765659186683, 0, -0.0143487577351166, -0.0129644660894661, 0.0836061337380013, 0, 0.0477758551400502, 0, -0.0325134964536671, 0, 0.00928957041533349, 0, -0.0141250354327105, -0.0144810071458616, 0, -0.00215870109072044, 0, 0.00411266813957584, 0.00540074884191874, 0, 0.0765880028506897, 0, -0.0228341307513453, 0, 0.0127646671417816, -0.00699149613259877, 0, -0.0112908326677739, 0, 0.00737991258440982, 0, -0.00310733161448833, 0, 0.00347410235931505, 0.00201827108709706, 0, 0.000902598269554279, 0, -0.000491312267361592, -0.00086852558982878},
    {0.078755617076958, 0.0377344134083695, -0.0217859737390348, -0.0523770332031201, 0.0423567739995023, -0.0546823600997146, 0, -0.0446479600584757, 0.0257775117585286, 0.0291639253865145, -0.0417574386216002, 0.0285692908380655, -0.0273366604729936, 0.0187890812890813, -0.0092175078635989, 0.00789141414141413, -0.00347132196482461, 0.0248527774227189, -0.0143487577351166, -0.015031265031265, -0.0557374224920008, 0.00491557862111908, -0.0197335053839338, 0.0394205194526234, -0.0325134964536672, 0.0241814494802473, -0.00928957041533348, 0.0158304746652834, -0.0141250354327105, -0.00241350119097695, 0.000623163327913697, -0.00215870109072046, -0.0071233501724151, 0.00411266813957581, 0.00540074884191871, 0, -0.0510586685671264, 0.0405265617908062, -0.0114170653756727, -0.010317557614593, 0.0127646671417816, 0.00466099742173252, 0.0143871467126538, -0.00564541633388697, -0.00571843474677502, 0.00737991258440983, -0.00367664434877217, -0.00155366580724418, -0.00240692871881144, 0.00347410235931506, 0.00100913554354853, -0.000260557676948628, 0.000902598269554287, 0.000850977809452107, -0.000491312267361569, -0.000868525589828763},
    {-0.0214788046573521, -0.0188672067041848, 0.0108929868695174, 0.0030810019531247, 0.0141189246665008, -0.109364720199429, -0.0789271883087357, 0.0223239800292378, -0.0128887558792643, 0.0127592173566001, -0.0089480225617715, -0.0277290175781224, -0.00585785581564147, 0.0191648629148629, 0.00145539597846298, 0.0383297258297258, 0.0348216984596467, -0.0124263887113595, 0.00717437886755831, -0.0129644660894661, 0, 0.0589869434534289, 0.00727023882565979, 0.0506835250105158, 0.0650269929073344, -0.00659494076734015, 0.0148633126645335, -0.00431740218144092, -0.00166176887443655, -0.00402250198496158, -0.00810112326287807, -0.0115130724838425, 0.00356167508620755, -0.00205633406978791, 0.00540074884191871, 0, 0, 0.0607898426862092, 0.0399597288148543, -0.0154763364218895, -0.031911667854454, 0, -0.0123318400394175, 0.00303983956440066, -0.00302740663064559, -0.00776832903622087, 0.00137874163078956, -0.00427258096992148, 0.000902598269554293, 0.000868525589828758, 0.00100913554354854, 0.000781673030845887, 0.00180519653910857, -0.000425488904726058, 0.00024565613368079, -0.000868525589828762},
    {-0.0214788046573521, 0.0188672067041848, 0.0108929868695174, 0.00308100195312469, 0.0141189246665007, 0.109364720199429, -0.0789271883087357, -0.0223239800292378, -0.0128887558792643, 0.0127592173566001, 0.00894802256177149, -0.0277290175781224, 0.00585785581564149, 0.0191648629148629, 0.00145539597846298, -0.0383297258297258, 0.0348216984596467, 0.0124263887113595, 0.00717437886755832, -0.0129644660894661, 0, -0.0589869434534289, 0.00727023882565983, -0.0506835250105158, 0.0650269929073343, 0.00659494076734015, 0.0148633126645336, 0.00431740218144092, -0.00166176887443653, -0.00402250198496157, 0.00810112326287806, -0.0115130724838425, -0.00356167508620755, -0.00205633406978791, 0.00540074884191872, 0, 0, -0.0607898426862092, 0.0399597288148543, 0.0154763364218895, -0.031911667854454, 0, 0.0123318400394175, 0.00303983956440068, 0.0030274066306456, -0.00776832903622088, -0.00137874163078956, -0.00427258096992148, -0.000902598269554298, 0.000868525589828757, 0.00100913554354854, -0.000781673030845891, 0.00180519653910858, 0.000425488904726056, 0.000245656133680792, -0.000868525589828762},
    {0.078755617076958, 0, 0.0435719474780696, -0.0523770332031201, -0.0282378493330015, 0, 0.0631417506469885, 0, -0.0515550235170572, 0.0291639253865144, 0, -0.00672218607954481, 0, -0.0601250601250601, 0.000970263985641994, 0, -0.0125835421224892, 0, 0.0286975154702333, -0.015031265031265, 0, 0, -0.0436214329539589, 0, -0.0650269929073343, 0, 0.0111474844984002, 0, 0.031573608614294, -0.00160900079398463, 0, -0.0028782681209606, 0, -0.00822533627915163, 0.00540074884191871, 0, 0, 0, -0.0532796384198057, 0, 0.0425488904726053, 0, 0, 0.018239037386404, 0, -0.00388416451811042, 0, -0.00466099742173252, 0, -0.00347410235931505, 0.000672757029032356, 0, 0.00120346435940571, 0, 0.000982624534723144, -0.000868525589828765},
    {0.107394023286761, -0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, -0.0546823600997146, -0.0315708753234943, -0.0446479600584757, -0.0257775117585285, -0.0182274533665716, 0.0894802256177147, -0.0151249186789759, 0.0585785581564149, 0.0744047619047619, -0.0130985638061669, -0.0281836219336219, 0.0097630930260691, 0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, 0.0737336793167861, 0.036351194128299, -0.0120675059548847, -0.0232239260383337, 0.0659494076734016, 0.00743165633226673, 0.0431740218144092, 0.0465295284842228, 0.0345935170706695, 0.0155790831978424, 0.0251848460584054, -0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0.0678251202167962, 0.0334382352084074, -0.0111004909790339, -0.0213629048496074, -0.0137874163078956, 0.0194208225905521, -0.00902598269554287, -0.0191075629762328, -0.00908221989193678, -0.00234501909253766, -0.00812338442598858, 0.00340391123780844, 0.00196524906944629, 0.00607967912880135},
    {0.107394023286761, 0.188672067041848, -0.108929868695174, -0.0770250488281178, -0.0705946233325038, 0.0546823600997146, -0.0315708753234943, 0.0446479600584756, -0.0257775117585286, -0.0182274533665715, -0.0894802256177148, -0.0151249186789758, -0.0585785581564149, 0.0744047619047619, -0.0130985638061669, 0.0281836219336219, 0.00976309302606918, -0.0690354928408859, 0.039857660375324, 0.0597492784992785, 0, -0.0737336793167861, 0.0363511941282991, 0.0120675059548847, -0.0232239260383337, -0.0659494076734016, 0.00743165633226682, -0.0431740218144092, 0.0465295284842228, 0.0345935170706695, -0.0155790831978425, 0.0251848460584054, 0.0254405363300539, -0.0146881004984851, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, -0.0678251202167962, 0.0334382352084074, 0.0111004909790339, -0.0213629048496074, 0.0137874163078956, 0.0194208225905522, 0.00902598269554289, -0.0191075629762328, -0.00908221989193679, 0.00234501909253767, -0.00812338442598859, -0.00340391123780842, 0.00196524906944628, 0.00607967912880136},
    {0.107394023286761, 0, 0.217859737390348, -0.0770250488281177, 0, 0, -0.0947126259704827, 0, 0.0515550235170571, -0.0182274533665715, 0, -0.0907495120738551, 0, -0.0946969696969696, 0.0232863356554078, 0, -0.0227805503941613, 0, -0.0797153207506479, 0.0597492784992785, 0, 0, -0.072702388256598, 0, 0.0464478520766673, 0, -0.048305766159734, 0, -0.0781031370985167, 0.0144810071458617, 0, 0.0431740218144092, 0, 0.0293762009969701, -0.0328199352701214, 0, 0, 0, 0, 0, 0, 0, 0, -0.0668764704168148, 0, 0.0427258096992147, 0, 0.0310733161448835, 0, 0.00694820471863012, -0.00605481326129122, 0, -0.0108311792346515, 0, -0.00393049813889258, 0.00607967912880135},
    {0.107394023286761, 0, 0, 0.231075146484353, 0, 0, 0, 0, 0, -0.109364720199429, 0, 0.0100832791193172, 0, -0.00450937950937952, -0.093145342621631, 0, -0.104139658944738, 0, 0, -0.09244227994228, 0, 0, 0, 0, 0, 0, -0.066884906990401, 0, 0.0299118397398575, -0.0579240285834467, 0, -0.0647610327216138, 0, 0, 0.0835038859404355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -0.0699149613259878, 0, 0.0312669212338355, 0.0242192530451648, 0, 0.0270779480866286, 0, 0, -0.018239037386404}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    static const double dmats0[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {6.32455532033677, 1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 11.2249721603218, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.58257569495583, 0, 8.36660026534075, -1.18321595661993, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.74165738677394, 0, 0, 8.69482604771365, 0, 0, 0, 0, 0, 0, 0, 0, 1.18793863634892e-14, 0, 0, 0, -2.04281036531029e-14, 1.06581410364015e-14, 1.4210854715202e-14, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.33226762955019e-14, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.49909083394701, -1.24344978758018e-14, -3.3466401061363, -2.36643191323985, 15.4919333848297, 0, 0.692820323027545, 0, 0.565685424949238, 0.400000000000001, -1.24344978758018e-14, 2.22044604925031e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 4.89897948556636, -1.4210854715202e-14, 0, 0, 14.1985914794391, 0, -0.828078671210839, 0, 0, 0, -1.24344978758018e-14, 1.80966353013901e-14, -1.06581410364015e-14, 0, 1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 2.48689957516035e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.6, 0, 8.76356092008264, -1.54919333848296, 0, 0, 9.52470471983252, 0, -1.48131215963607, 0.261861468283189, 0, -1.06581410364015e-14, 0, 0, 0, -1.06581410364015e-14, 2.48689957516035e-14, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 4.24264068711928, 0, -2.48689957516035e-14, 0, 0, 0, 14.3427433120127, 0, 0, 0, 0, 0, 0, 0, 2.22044604925031e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, -1.06581410364015e-14, 0},
    {3.11769145362397, 0, 3.1622776601684, 4.91934955049952, 0, 0, 0, 0, 10.690449676497, -2.41897262725905, 0, 0, 1.50990331349021e-14, 0, 0, 0, -2.30926389122033e-14, 0, 1.77635683940025e-14, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, -1.33226762955019e-14, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.37945210809676e-14, 0, 0, 1.95399252334028e-14, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.54558441227157, 0, 0, 7.66811580507232, 0, 0, 0, 0, 0, 10.3691851174526, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 11.6057691494799, 0, 0, 0, -4.20458932936041, 0, -3.43303281162798, 0, 1.25455201782643e-14, 19.6214168703486, 0, 0.611677741841184, 0, 0, 0.529728463363969, 0, 0.432521479156776, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.53137212479106, 0, 3.54562104171168, -2.50713268211204, 4.7871355387817, 0, -5.56626745075489, 0, -1.04880884817016, 0.741619848709568, 0, 18.8034951158403, -1.24344978758018e-14, 1.68183573174417, -0.603122387046461, 0, 0.701283781217647, 0, 0.13213749452868, -0.0934353184302319, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, -1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.08246744900953e-14, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 2.5951288749407, 0, 0, 0, 10.0285307284481, -1.4210854715202e-14, -0.767649473578743, 0, 0, 0, 0, 16.4130361329658, -1.06581410364015e-14, 0, -1.26347611061158, 1.4210854715202e-14, 0.0967147429123241, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.96647939483827, 0, 8.12403840463597, -1.64130361329658, 0, 0, 11.7728501222091, 0, -2.40312296813957, 0.485504156227611, 1.06581410364015e-14, -1.06581410364015e-14, 0, 10.2761860629321, 0, 0, -1.48323969741913, 0, 0.302765035409733, -0.0611677741841268, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.05298244095219, 0, -1.5856499343442, 4.4848952846511, 4.28174419288838, 0, 0.191485421551275, -1.06581410364015e-14, -3.75233260785875, -4.31161222746203, 0, 0, 0, 0, 18.8807171866294, 0, 0.844371341865028, 0, 1.18187368057054, 1.04463861754668, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, -1.15463194561016e-14, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0},
    {0, 2.32115382989599, -1.4210854715202e-14, -1.06581410364015e-14, 0, 3.92428337406972, 0, 5.49285249860473, -1.59872115546023e-14, 0, 0, 0, 0, 0, -1.68753899743024e-14, 17.3044631365566, -1.77635683940025e-14, -1.73008591662713, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.77635683940025e-14, 2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0},
    {2.65329983228431, 0, 4.15220619990511, 2.93605316083775, 0, 0, 2.63248931621763, 0, 9.82591326180786, -2.82261277137741, 0, 0, 0, 0, 0, 1.4210854715202e-14, 11.6081867662439, 0, -3.09487177319468, 0.683876505540245, 0, 0, 0, 0, 0, 0, 0, 0, 1.50990331349021e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.26565424807268e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 2.01017818278146, 0, 0, 0, 0, 0, 8.91928088068603, 0, 0, 0, 0, 0, 0, 0, 0, 0, 16.8558564884489, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.2978250586152, 0, 1.49829835452881, 5.50917601898534, 0, 0, 0, 0, 6.64803945320938, 4.13676892548487, 0, 0, 1.06581410364015e-14, 0, 0, 0, -2.57571741713036e-14, 1.24344978758018e-14, 12.5636136423842, -3.55352656109506, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87616630392937, -1.19904086659517e-14, 0, 6.22830929985771, 0, 0, 0, 0, 0, 11.0541524195338, 0, 0, -1.19904086659517e-14, 0, -1.4210854715202e-14, 0, 2.99760216648792e-14, -1.06581410364015e-14, 0, 11.60576914948, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.5709655798446e-14, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.83186700722508, 0, -4.41087092453693, -3.11895674167864, 16.6749979177077, 0, 2.13065307820925, 0, 1.739670953501, 1.23013312825382, 0, -4.36653943835009, 0, -0.557935943461983, -3.78153408023783, -1.06581410364015e-14, -0.483186700722515, 0, -0.394520289089675, -0.278967971730983, 23.6981011897578, 0, 0.490653381462667, 0, 0.0626933893908967, 0, 0.438853725736256, 0, 0.0560746721671611, 0.380058475033045, 0, 0.0485620906056452, 0, 0.0396507809422086, 0.0280373360835817, 0, 0, 0, 0, 0, 0, -1.46549439250521e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 6.91050664139875, 0, 0, 0, 10.5706241650196, 0, -2.72554057547698, 0, 1.15463194561016e-14, 4.67332857821916, 0, -7.86705867811458, -2.48689957516035e-14, 0, -2.39719223514613, 0, 0.618094504365252, 0, 0, 0, 23.10555488766, 0, 1.62065746295906, 0, -0.469687194424183, 0, 0.790669146636837, 0, 0, 0.240926884676763, 0, -0.0621208350291009, 0, 0, 0, 0, 0, 0, 0, -1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.85449644663775, 1.15463194561016e-14, 5.98171817966646, -2.4880667576406, 1.90029237516522, 0, -0.33993463423953, 0, -2.3592214660616, 0.981306762925316, 0, 9.95226703056241, 0, -6.45364422706459, -0.43094580368567, 0, 0.077089928932768, 0, 0.535021138879229, -0.222539456105679, 0, 0, 21.5273297444417, 0, 2.75066129614997, 0, -1.00024047132728, 0, 0.648615649442924, 0.0433116829031337, 0, -0.0077478293753126, 0, -0.0537716476536687, 0.0223660568773445, 0, 0, 0, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.06581410364015e-14, 1.54523626091315, -1.4210854715202e-14, 0, 0, 6.71778024562961, 0, -0.609449400220043, 0, 0, 0, 1.06581410364015e-14, 14.6593983227032, 1.4210854715202e-14, 0, -1.52344936219873, 0, 0.138210132827973, 0, 0, 0, 0, 0, 18.119501277094, 0, 0, 0, -1.47332496632642, 0, 0, 0.153112421864228, 0, -0.0138906409944104, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.52336024752224, 1.4210854715202e-14, 7.37120708415867, -1.62882203585591, 0, 0, 12.0171306297064, 0, -2.90724328050879, 0.642416074439625, 0, 0, 2.1316282072803e-14, 13.9858820944259, 0, 0, -2.72522906732402, 0, 0.659300804656272, -0.145686271816946, 0, 0, 0, 0, 10.8043830863937, 0, 0, 0, -1.40563403846548, 0, 0, 0.273895826790425, 0, -0.0662622240310384, 0.0146420212340128, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 6.30840061880559, 0, 0, 1.06581410364015e-14, -1.5236235005501, 0, 11.1963004093827, 0, 0, 4.26614580154029, 0, 0.132992762321608, 0, 0, -4.72217953828917, 0, -6.67684597013021, 0, 0, 0, 0, 0, 0, 0, 23.1532444221696, 0, 0.721778878560592, 0, 0, 1.11125127955292, 0, 1.1908743922773, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0},
    {3.51865775274499, -1.04360964314765e-14, 1.92724822331885, 2.725540575477, 1.73472166622177, 0, -2.01705689521107, -1.77635683940025e-14, 3.42052627529743, -4.03112887414928, -1.28785870856518e-14, 4.08831086321549, -2.30926389122033e-14, 0.365669640132028, 5.37643791520787, 0, -6.25148194071026, 0, -2.03981013744551, 1.8080332206528, 0, 0, 0, 0, 0, 0, 22.1880978975596, 1.765254609154e-14, 1.98456380761453, -1.26521523890981, 0, 1.4711357857929, 0, 0.363818136386342, -0.294009452979941, 0, 0, 0, 0, 0, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, -2.30926389122033e-14, 0},
    {0, 1.4106012612951, 0, 0, 0, 3.634054100636, 0, 2.50356888118883, -1.24344978758018e-14, 0, 0, 0, 3.56857028479908, 0, 0, 11.263055412878, 0, -1.4929881464507, 0, 0, 0, 0, 0, 0, 0, 0, 0, 19.3673596462205, 0, 0, -2.65048896124152, 0, 0.266287609379582, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, -1.24344978758018e-14, 0, 0, -1.59872115546023e-14, 0, -1.24344978758018e-14, 0, 0},
    {2.30350221379959, 0, 4.41588043316389, 1.78428514239955, 0, 0, 4.2661458015403, 0, 7.8374102865679, -2.63899331455872, 0, 0, 0, 2.23427840700302, 1.77635683940025e-14, 0, 13.2221027072096, -1.4210854715202e-14, -4.67379217909115, 1.18363558466232, 0, 0, 0, 0, 0, 0, 0, 0, 12.1258851598179, 0, 0, -3.11150358274815, 0, 0.833612074594101, -0.192474367616158, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.8421709430404e-14, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.14718316987778, 0, -0.861891607371343, 6.09449400220045, 1.55158222708545, 0, 0.0693888666488798, -1.95399252334028e-14, -3.05941170815566, 1.80277563773199, 0, -1.26842980563424e-14, 0, 0, 9.03122156537879, 0, 0.403888506800988, -1.4210854715202e-14, -3.67461994579465, -5.49650682721193, 0, 0, 0, 0, 0, 0, 0, 0, 0, 21.7841260588564, 0, 0.974215733960557, 0, 1.78974858569576, 1.8079190879493, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1.26168012376112, 1.24344978758018e-14, 0, 0, 1.42204860051343, 0, 4.47852016375307, 0, 0, 0, 0, 0, 0, 0, 8.27725128825318, 0, 5.37909280973869, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 19.9654813226339, 0, -2.61992366301004, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 1.06581410364015e-14, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 1.33226762955019e-14, 0, 0},
    {2.06031501455084, 0, 2.25696201807219, 3.98978286964828, 0, 0, 0.953939201416947, 0, 8.01142041980429, 1.18019368870418, 0, 0, 0, 0, 0, 1.77635683940025e-14, 5.55254896421454, 0, 9.62241374388481, -3.59830837050572, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 13.3932520322736, 0, -4.68666192508059, 1.18356086726904, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 1.09264703862702, 0, 0, 0, 0, 0, 5.38682254607085, 0, 0, 0, 0, 0, 0, 0, 0, 0, 13.4377817958113, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 18.907670401189, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.78428514239954, 0, 0.814411017927931, 5.0677048703591, 0, 0, 0, 0, 4.01510046524764, 6.58672305740272, 0, 0, -1.06581410364015e-14, 0, 0, 0, 1.59872115546023e-14, 0, 10.0159311874144, 2.83293314500743, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 14.0929454377398, -4.55552790213386, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.22124532708767e-14, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.45686271816936, 0, 0, 5.07816454066238, 0, 0, 0, 0, 1.06581410364015e-14, 10.0142755247553, 0, 0, 0, 0, 0, 0, 0, 0, 0, 13.8784813613723, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.5535587278944, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    static const double dmats1[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.47722557505167, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.95803989154981, 5.6124860801609, -1.08012344973464, -0.763762615825972, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.29128784747791, 7.24568837309471, 4.18330013267037, -0.591607978309964, 0, 0, 0, 0, 0, 0, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-2.64575131106461, 0, 9.66091783079295, 0.683130051063968, 0, 0, 0, 0, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338697, 0, 0, 4.34741302385683, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.02140518265514e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.24037034920393, 0, 0, 7.52994023880668, 0, 0, 0, 0, 0, 0, 0, 0, 1.0547118733939e-14, 0, 0, 0, -1.84297022087776e-14, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.7495454169735, 5.79655069847576, -1.67332005306815, -1.18321595661993, 7.74596669241484, -1.2, 0.346410161513775, -0.979795897113271, 0.28284271247462, 0.200000000000002, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.32379000772444, 2.44948974278317, 2.82842712474617, -1.00000000000001, 9.16515138991169, 7.09929573971954, -2.04939015319193, -0.414039335605414, -0.478091443733755, 0.169030850945704, -1.4210854715202e-14, 1.06581410364015e-14, 0, -1.15463194561016e-14, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.80000000000001, -5.69209978830307, 4.38178046004132, -0.77459666924149, 0, 10.998181667894, 4.76235235991626, 0.962140470884713, -0.740656079818035, 0.130930734141589, 1.19904086659517e-14, -1.4210854715202e-14, 1.53210777398272e-14, -1.33226762955019e-14, 0, 1.06581410364015e-14, 1.4210854715202e-14, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.19615242270665, 0, -3.1622776601684, -2.23606797749978, 0, 0, 13.7477270848675, -1.4210854715202e-14, 0.534522483824862, 0.377964473009222, 1.4210854715202e-14, 0, 0, -3.01980662698043e-14, 0, 0, 1.77635683940025e-14, 0, -1.39055433834301e-14, 0, 0, 0, 0, 0, -1.24344978758018e-14, 0, 0, 0, 1.14908083048704e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.01246117974981, 2.12132034355964, -0.40824829046386, 3.17542648054293, 0, 0, 0, 7.17137165600636, -1.38013111868471, -1.56144011671765, 0, 0, 0, 0, 0, 0, -1.55431223447522e-14, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.55884572681199, 2.73861278752584, 1.5811388300842, 2.45967477524976, 0, 0, 0, 9.25820099772551, 5.34522483824848, -1.20948631362952, 0, 0, 0, 0, 0, 1.35447209004269e-14, -1.33226762955019e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.80000000000002, 0, 3.65148371670112, -2.84018778721878, 0, 0, 0, 0, 12.3442679969673, 1.39659449751035, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.27279220613578, 0, 0, 3.83405790253616, 0, 0, 0, 0, 0, 5.18459255872629, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.20454076850486, 0, 0, 6.64078308635359, 0, 0, 0, 0, 0, 8.97997772825746, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.56904651573302, 5.80288457473996, -2.01017818278147, -1.42141062443803, 8.14215222512715, -2.10229466468021, 0.728256234341417, -1.71651640581399, 0.594618725379072, 0.42045893293605, 9.81070843517429, -1.18450885369836, 0.305838870920592, -0.105945692672798, -1.02581475831037, 0.264864231681986, -0.09175166127618, 0.216260739578392, -0.0749149177264366, -0.0529728463363877, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239553, 3.07059789431495, 1.77281052085583, -1.25356634105602, 2.39356776939085, 6.30376871403132, -2.78313372537745, -0.908295106229253, -0.524404424085079, 0.370809924354787, 11.1242977306435, 9.40174755792014, -2.42752078113807, 0.840917865872087, -0.301561193523231, -0.794200206656965, 0.350641890608825, 0.11443442705426, 0.0660687472643419, -0.0467176592151146, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551269, 1.29756443747037, 3.74574588632197, -1.05945692672795, -8.09173593712687, 5.01426536422408, 1.80936716113936, -0.383824736789376, -1.10800657553489, 0.313391585264003, 2.1316282072803e-14, 12.7134831207354, 8.20651806648291, -2.84282124887606, 1.01946290306866, -0.631738055305796, -0.227958835180072, 0.0483573714561647, 0.139595707137603, -0.0394836284566167, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741913, 7.03562363973513, 4.06201920231799, -0.820651806648276, 0, -6.79705818718656, 5.88642506110458, -2.08116553882674, -1.2015614840698, 0.242752078113814, 0, -1.06581410364015e-14, 14.8323969741913, 5.13809303146608, 0, 0.856348838577665, -0.741619848709565, 0.262202212042526, 0.151382517704867, -0.0305838870920541, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-3.4253953543107, 0, 9.38083151964687, 1.89521416591737, 0, 0, -3.39852909359329, 0, -2.77488738510233, -0.560611910581384, -1.06581410364015e-14, 0, 0, 17.7988763690296, 0, 0, 0.428174419288809, 0, 0.349602949390039, 0.0706304617818535, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 1.50990331349021e-14, 0, 0, 0, 0, -1.73194791841524e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047609, 2.74642624930236, -0.792824967172096, 2.24244764232554, 2.14087209644419, -0.331662479035538, 0.0957427107756352, 6.49923072370876, -1.87616630392937, -2.15580611373101, -1.06581410364015e-14, 0, 0, 0, 9.44035859331469, -1.46249406456535, 0.422185670932514, -2.04706526287665, 0.590936840285273, 0.522319308773341, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.71269767715535, 1.16057691494799, 1.34011878852097, 1.89521416591735, 2.53311402559511, 1.96214168703486, -0.566421515598876, 2.74642624930236, 3.17129986868836, -1.82198870938951, -1.77635683940025e-14, 0, 0, 0, 11.1699829238239, 8.6522315682783, -2.49768411251824, -0.865042958313563, -0.998865569685877, 0.441440386136644, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 1.59872115546023e-14, 0, -1.77635683940025e-14, 0, 0, 0, 0, 1.06581410364015e-14, -1.09356967925578e-14, 1.02140518265514e-14, -1.33226762955019e-14, 0},
    {1.32664991614216, -2.6969370381518, 2.07610309995255, 1.46802658041887, 0, 3.03973683071412, 1.31624465810881, -6.38211787508103, 4.91295663090393, -1.4113063856887, 0, 0, 0, 0, 0, 13.4039795085888, 5.80409338312195, 2.01017818278148, -1.54743588659734, 0.341938252770126, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.11022302462516e-14, 1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 1.6228338117763e-14, 0, 0, 0, 0},
    {3.82970843102536, 0, -1.4982983545288, 4.2378277069118, 0, 0, 3.79967103839266, -1.33226762955019e-14, -3.54562104171166, -4.07409060843206, 0, 0, -1.4210854715202e-14, 0, 1.4210854715202e-14, 1.25455201782643e-14, 16.7549743857359, -1.06581410364015e-14, 1.11676565710081, 0.987090711415277, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, 0, 0, -2.1316282072803e-14, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 1.00508909139073, -0.193429485824663, 3.55615782880926, 0, 0, 0, 4.45964044034302, -0.858258202906996, 2.67027285925187, 0, 0, 0, 0, 0, 0, 0, 8.42792824422443, -1.62195554683793, -2.29379153190449, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.14891252930761, 1.29756443747035, 0.749149177264408, 2.75458800949267, 0, 0, 0, 5.75737105184053, 3.32401972660469, 2.06838446274244, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 10.8804085776374, 6.2818068211921, -1.77676328054754, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.32664991614216, -1.06581410364015e-14, 1.73008591662715, -3.18072425757424, 0, 0, 0, 0, 7.67649473578738, -2.38836465270395, 0, 0, 0, 0, 0, 1.06581410364015e-14, -1.47659662275146e-14, 0, 14.5072114368499, 2.05162951662074, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.938083151964684, 0, 0, 3.11415464992885, 0, 0, 0, 0, 0, 5.52707620976691, 0, 0, 0, 0, 0, 0, 1.58761892521397e-14, 0, 0, 5.80288457473998, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.62480768092719, 0, 0, 5.39387407630365, 0, 0, 0, 0, 0, 9.57317681262151, 0, 0, -1.15463194561016e-14, 0, -1.24344978758018e-14, 0, 2.63122856836162e-14, 0, 0, 10.0508909139074, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.32116539930394e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.41593350361254, 5.72988941019469, -2.20543546226846, -1.55947837083932, 8.33749895885385, -2.76779953857108, 1.06532653910463, -2.25989885993663, 0.869835476750496, 0.615066564126918, 10.333118277332, -2.18326971917505, 0.724780051083766, -0.278967971730992, -1.89076704011891, 0.627677936394715, -0.241593350361261, 0.51249688899006, -0.197260144544837, -0.139483985865487, 11.8490505948789, -1.16109782023228, 0.245326690731332, -0.0814411017927985, 0.0313466946954515, -1.03851746182648, 0.21942686286813, -0.0728431359084674, 0.0280373360835804, 0.190029237516525, -0.0630840061880544, 0.0242810453028245, -0.0515078753637725, 0.0198253904711041, 0.0140186680417935, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.18529407725406, 3.45525332069938, 0.997445717412069, -1.41060126129511, 3.23209352764251, 5.28531208250978, -2.89087233497829, -1.36277028773849, -0.393397896234718, 0.556348640264195, 2.33666428910958, 9.02784807228869, -3.93352933905728, 1.64018416088946, -0.732969916135164, -1.19859611757308, 0.655588223176204, 0.309047252182618, 0.0892142571199805, -0.126168012376108, 13.0996183150503, 11.55277744383, -2.44096975264613, 0.810328731479533, -0.311895674167856, -0.234843597212092, -0.907332870306513, 0.395334573318419, -0.164844710568311, 0.0736662483163206, 0.120463442338383, -0.0658890955530712, -0.0310604175145501, -0.00896637020658453, 0.0126803623514089, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.92724822331887, 1.82834820066012, 2.99085908983323, -1.2440333788203, 0.9501461875826, 5.7406445631131, -0.169967317119757, -0.721110255092796, -1.1796107330308, 0.49065338146266, -10.3037210107158, 4.97613351528119, 5.7817446699566, -3.2268221135323, -0.215472901842833, -1.3018558182182, 0.0385449644663813, 0.163532434528613, 0.267510569439617, -0.111269728052837, 0, 14.555131461167, 10.7636648722209, -3.57321383950879, 1.37533064807499, 1.03556292536503, -0.500120235663638, -0.581087203147974, 0.32430782472146, 0.0216558414515668, 0.130841432732455, -0.00387391468765474, -0.0164356280722655, -0.0268858238268351, 0.0111830284386718, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.62882203585592, 0.772618130456564, 3.86595114186566, -1.05140010313427, 9.1544290607646, 3.3588901228148, 3.878512233171, -0.304724700110022, -1.52475169285615, 0.414677792940102, 0, -9.4626009282084, 7.32969916135161, 0.564240504518009, -2.07602937339135, -0.761724681099364, -0.879563899362182, 0.0691050664139832, 0.34578118203618, -0.0940400840863436, 0, 0, 16.3745228938128, 9.05975063854702, -3.48709964663735, 0, 0.951027176355639, -0.736662483163211, -0.0567083043941548, 0.208648802584721, 0.0765562109321125, 0.0883994979795828, -0.00694532049720742, -0.0347523163751395, 0.00945138406569365, 0, -1.4210854715202e-14, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376112, -6.78263087840204, 3.68560354207933, -0.814411017927966, 0, 10.0216907612581, 6.00856531485322, 2.67510569439611, -1.45362164025439, 0.321208037219813, 1.33226762955019e-14, -1.06581410364015e-14, -7.5700807425667, 6.99294104721295, 0, -2.27270584034421, -1.36261453366202, -0.606656948415834, 0.329650402328125, -0.0728431359084816, 0, 0, 0, 18.7137404500718, 5.40219154319687, 0, 0, 0.760821741084501, -0.702817019232737, 0, 0.228415531250638, 0.136947913395213, 0.0609713174047632, -0.0331311120155173, 0.00732101061700419, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.09901951359278, 0, -4.65474668125631, -3.2914029430219, 1.24344978758018e-14, 0, 13.490737563232, 0, 1.83585684909538, 1.29814682728311, -1.4210854715202e-14, -3.5527136788005e-14, 4.2632564145606e-14, -3.53270434653113, -2.1316282072803e-14, 0, -3.05941170815567, 0, -0.416333199893195, -0.29439202887759, 0, 0, 0, 0, 21.8326971917504, 0, 0, 0, 0.35505014583943, 0, 0, 0.307482445914323, 0, 0.0418430609640534, 0.0295875121532867, 0, 0, 0, -1.77635683940025e-14, 1.4210854715202e-14, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 1.15463194561016e-14, 0, 0, 1.15463194561016e-14, 0, 0, 0},
    {1.99489143482413, 3.15420030940279, -1.09264703862703, 1.54523626091314, 2.95048422176042, -0.761811750275057, 0.263899331455877, 5.59815020469134, -1.93925611658551, -2.28543525082516, 2.13307290077014, -0.257539376818858, 0.0664963811608033, -0.0230350221379969, 9.14446135479766, -2.36108976914457, 0.817905488277891, -3.3384229850651, 1.15646364545769, 1.02505848514082, 0, 0, 0, 0, 0, 11.5766222110848, -1.39771878815471, 0.360889439280296, -0.125015768949702, -2.15192884958005, 0.555625639776456, -0.192474367616153, 0.595437196138647, -0.206265495285698, -0.166687691932932, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.7593288763725, 1.66904592079255, 0.96362411165942, 1.3627702877385, 0.867360833110887, 2.28430636201792, -1.00852844760553, 2.96226264871971, 1.71026313764871, -2.01556443707464, 2.41867732448956, 2.04415543160774, -0.527798662911756, 0.182834820066013, 2.68821895760393, 7.07977053255619, -3.12574097035514, -1.76652739792485, -1.01990506872276, 0.904016610326401, 0, 0, 0, 0, 0, 13.1266557397187, 11.0940489487798, -2.8644711213756, 0.992281903807267, -0.6326076194549, -1.66605356688627, 0.735567892896449, 0.31507574846809, 0.181909068193174, -0.147004726489965, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.4210854715202e-14, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, -1.24344978758018e-14, 0},
    {1.48690428533296, 0.705300630647549, 2.03602754481988, 1.1517511068998, -2.93221479395416, 1.81702705031799, 0.655663160391206, 1.25178444059441, 3.61359041872287, -1.70346285967313, 1.06581410364015e-14, 2.7642026565595, 1.78428514239953, -0.618094504365255, -9.08783875864327, 5.63152770643899, 2.03210252328837, -0.74649407322535, -2.15494277062553, 0.764033484562597, 0, 0, 0, 0, 0, 0, 15.0018922739643, 9.68367982311024, -3.3545250915713, 2.13860408462397, -1.32524448062076, -0.4782064110178, 0.133143804689791, 0.384353057392901, -0.124241670058201, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 1.06581410364015e-14, 0, 0, 1.06581410364015e-14, -1.86517468137026e-14, 0, 1.13797860024079e-14, 0, 0, 0, 0, 0},
    {1.15175110689979, 3.82426463519456, 2.20794021658195, 0.89214257119979, 0, -2.46306042692148, 2.13307290077015, 6.78739640804928, 3.91870514328395, -1.31949665727937, 0, 0, 3.2249030993194, 1.11713920350152, 2.1316282072803e-14, -7.63378455726039, 6.61105135360481, -4.04762275910198, -2.33689608954558, 0.591817792331174, 0, 0, 0, 0, 0, 0, 0, 17.5022076529583, 6.06294257990893, 0, 1.79642743108413, -1.55575179137407, 0.721929233499942, 0.416806037297052, -0.0962371838080749, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2.1316282072803e-14, 0, 0, 0, -1.59872115546023e-14, 0, 0, -1.24344978758018e-14, 0, -2.44249065417534e-14, -1.06581410364015e-14, 0},
    {-2.65985524643218, 0, 5.09901951359278, -2.06031501455085, 0, 0, -1.23153021346075, 1.59872115546023e-14, 9.04986187739903, 3.04724700110024, 0, 2.1316282072803e-14, 0, 3.86988371918331, 0, -4.2632564145606e-14, -3.81689227863023, 0, -5.39683034546928, -1.36674464685442, 0, 0, 0, 0, 1.28785870856518e-14, 0, 0, 0, 21.00264918355, 0, 0, 0.898213715542074, 0, 0.962572311333254, 0.222250255910579, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -3.10862446895044e-14, 0, 0, 1.50990331349021e-14, 0, 0, 0, 0, 0, -2.30926389122033e-14, 0},
    {1.57359158493889, 1.49284005458435, -0.430945803685675, 3.04724700110022, 0.775791113542725, -0.120185042515472, 0.0346944333244361, 5.29905651979669, -1.52970585407783, 0.901387818865992, 0, 0, 0, 0, 4.51561078268939, -0.699555414372423, 0.201944253400492, 6.36462844462229, -1.83730997289733, -2.74825341360597, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.8920630294282, -1.68739114875268, 0.487107866980278, -3.09993548319961, 0.894874292847881, 0.903959543974651, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32992762321609, 0.630840061880562, 0.7284313590847, 2.57539376818856, 0.917928424547682, 0.711024300256705, -0.205255035576785, 2.23926008187652, 2.58567482211401, 0.761811750275061, 0, 0, 0, 0, 5.34294273196308, 4.13862564412659, -1.19471831485579, 2.68954640486935, 3.10562068169861, -2.32269806558124, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.8876627769294, 9.98274066131696, -2.88176900403079, -1.30996183150502, -1.51261363209513, 0.763985254692628, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.03015750727541, -1.46593983227032, 1.1284810090361, 1.99489143482414, 0, 1.10151410945721, 0.476969600708482, -5.20357020296093, 4.00571020990214, 0.590096844352088, 0, 0, 0, 0, 0, 6.4115312783557, 2.77627448210726, -6.24994106094658, 4.81120687194241, -1.79915418525285, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.4651953323153, 6.69662601613678, 3.0440762145518, -2.34333096254029, 0.591780433634522, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.01030295240889e-14, 0, 1.06581410364015e-14, 0, 0},
    {2.9738085706659, -2.1316282072803e-14, -0.814411017927963, 5.758755534499, 0, 0, 1.37689263682153, 0, -2.8908723349783, 1.70346285967311, 2.1316282072803e-14, 0, 0, 0, 0, 0, 8.01441409794462, 0, -3.47218947830368, -5.19371076584692, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 19.3314941653941, 0, 1.69115345252877, 1.70832296330021, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, -1.77635683940025e-14, 0, 0, 0, -1.4210854715202e-14, 0, 0, 0},
    {1.15175110689979, 0.546323519313513, -0.105140010313428, 3.27118942773249, 0, 0, 0, 2.69341127303542, -0.518347241175123, 4.25171145123352, 0, 0, 0, 0, 0, 0, 1.19904086659517e-14, 6.71889089790563, -1.29305115618717, 1.82865048192213, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9.45383520059452, -1.81939143264587, -2.94058061635795, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.892142571199769, 0.705300630647548, 0.407205508963976, 2.53385243517955, 0, 0, 0, 3.47717900165117, 2.00755023262382, 3.29336152870135, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 8.67405085085771, 5.0079655937072, 1.41646657250371, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.2048487632307, 7.0464727188699, -2.27776395106694, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-1.03015750727543, 0, 0.940400840863387, -2.92584077107541, 0, 0, 0, 0, 4.63623866886823, -3.80284633026897, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 11.5654011344769, -1.63559471386627, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 16.2731316843076, 2.63013526059784, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, -1.06581410364015e-14, -1.06581410364015e-14},
    {0.728431359084679, 0, 0, 2.53908227033119, 0, 0, 0, 0, 0, 5.00713776237767, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.93924068068616, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6.2767793639472, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376111, 0, 0, 4.39781949681096, 0, 0, 0, 0, 1.06581410364015e-14, 8.67261700493486, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12.0191174248973, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10.8717007662564, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    static const double dmats2[56][56] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766016838, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.82574185835056, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.16397779494323, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.95803989154981, 5.61248608016089, -1.08012344973464, -0.763762615825974, 0, 0, 0, 0, 0, 0, 0, 0, -1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.29128784747792, 1.44913767461895, 4.18330013267037, -0.591607978309966, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32287565553229, 0, 3.86436713231718, -0.341565025531991, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.87082869338697, 7.09929573971953, 0, 4.34741302385682, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.08012344973463, 1.06581410364015e-14, 7.09929573971953, 2.50998007960221, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-3.81881307912987, 0, 0, 8.87411967464941, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, -1.33226762955019e-14, 0, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.7495454169735, 5.79655069847577, -1.67332005306815, -1.18321595661993, 7.74596669241484, -1.2, 0.346410161513777, -0.979795897113275, 0.282842712474621, 0.200000000000002, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.32379000772445, 2.44948974278317, 2.82842712474618, -1.00000000000001, 1.30930734141595, 7.09929573971954, -2.04939015319192, -0.414039335605415, -0.478091443733755, 0.169030850945703, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.8, 0.632455532033677, 4.38178046004131, -0.774596669241482, 0, 3.14233761939829, 4.76235235991626, -0.106904496764974, -0.740656079818036, 0.130930734141593, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.03923048454133, 0, 3.16227766016837, -0.447213595499959, 0, 0, 5.89188303637179, 0, -0.534522483824841, 0.0755928946018432, 0, -1.24344978758018e-14, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.0124611797498, 2.12132034355962, -0.408248290463868, 3.17542648054291, 9.07114735222146, 0, 0, 7.17137165600636, -1.38013111868471, -1.56144011671765, -1.06581410364015e-14, 0, 0, 0, 0, 0, -2.33146835171283e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.11022302462516e-14, 0, 0, 0},
    {1.55884572681199, 0.547722557505173, 1.58113883008419, 2.45967477524976, 0, 9.07114735222146, 0, 1.8516401995451, 5.34522483824849, -1.20948631362953, 0, 0, 1.95399252334028e-14, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 1.02140518265514e-14, 0, 0, 0, 0, 0, 0, 0},
    {0.899999999999999, 1.4210854715202e-14, 1.46059348668045, 1.42009389360938, 0, 0, 9.07114735222145, 0, 4.93770719878694, -0.698297248755174, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.27279220613579, -6.2609903369994, 0, 3.83405790253615, 0, 0, 0, 10.5830052442584, 0, 5.18459255872628, 0, 1.06581410364015e-14, 1.95399252334028e-14, 0, 0, 0, 0, 1.06581410364015e-14, 0, -1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.734846922834958, 0, -6.26099033699939, 2.21359436211786, 0, 0, 0, 0, 10.5830052442584, 2.99332590941916, 0, 0, 0, 0, 0, 0, 0, 0, 1.68753899743024e-14, 1.4210854715202e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.7157676649773, 0, 0, -4.69574275274955, 0, 0, 0, 0, 0, 12.69960629311, 1.77635683940025e-14, 0, -1.33226762955019e-14, 0, -1.28785870856518e-14, 0, 2.79776202205539e-14, 0, 1.06581410364015e-14, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.49880108324396e-14, 0, 0, -1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0},
    {2.56904651573302, 5.80288457473996, -2.01017818278147, -1.42141062443803, 8.14215222512716, -2.10229466468021, 0.728256234341419, -1.71651640581399, 0.594618725379072, 0.420458932936049, 9.81070843517429, -1.18450885369835, 0.30583887092059, -0.105945692672798, -1.02581475831037, 0.264864231681985, -0.0917516612761808, 0.216260739578389, -0.0749149177264375, -0.0529728463363888, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.26568606239553, 3.07059789431495, 1.77281052085583, -1.25356634105602, 2.39356776939085, 6.30376871403131, -2.78313372537745, -0.908295106229252, -0.524404424085074, 0.370809924354786, 1.23603308118261, 9.40174755792014, -2.42752078113807, 0.840917865872084, -0.301561193523235, -0.794200206656964, 0.350641890608822, 0.114434427054266, 0.0660687472643433, -0.0467176592151159, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.91485421551268, 1.29756443747036, 3.74574588632196, -1.05945692672795, 0.404586796856345, 5.01426536422408, 1.80936716113937, -0.38382473678937, -1.10800657553489, 0.313391585264002, -1.06581410364015e-14, 2.82521847127454, 8.20651806648291, -2.84282124887606, -0.0509731451534328, -0.631738055305796, -0.227958835180075, 0.0483573714561643, 0.139595707137599, -0.0394836284566149, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 0.335029697130249, 4.06201920231797, -0.820651806648284, 0, 1.69926454679665, 5.88642506110456, -0.0991031208965145, -1.20156148406978, 0.242752078113805, 0, 0, 4.94413232473045, 5.13809303146606, 0, -0.214087209644424, -0.741619848709568, 0.0124858196210687, 0.151382517704866, -0.0305838870920656, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.856348838577684, 0, 2.68023757704195, -0.473803541479342, 0, 0, 5.09779364038993, 0, -0.792824967172093, 0.140152977645347, 0, -1.4210854715202e-14, 0, 7.91061171956872, 0, 0, -0.642261628933261, 0, 0.0998865569685847, -0.0176576154454708, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.02649122047609, 2.74642624930237, -0.792824967172094, 2.24244764232555, 2.14087209644419, -0.331662479035534, 0.0957427107756326, 6.49923072370876, -1.87616630392937, -2.15580611373101, 11.0554159678513, 0, 0, 0, 9.4403585933147, -1.46249406456535, 0.422185670932516, -2.04706526287665, 0.590936840285272, 0.522319308773344, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.71269767715536, 1.160576914948, 1.34011878852098, 1.89521416591736, 0.361873432227874, 1.96214168703486, -0.566421515598876, 2.74642624930237, 3.17129986868837, -1.82198870938951, 0, 11.0554159678513, 0, 0, 1.59571184626055, 8.6522315682783, -2.49768411251825, -0.865042958313561, -0.998865569685869, 0.441440386136648, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.04360964314765e-14, 1.95399252334028e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32664991614215, 0.299659670905754, 2.07610309995255, 1.46802658041887, 0, 0.868496237346892, 1.31624465810881, 0.709124208342329, 4.91295663090393, -1.4113063856887, 0, 0, 11.0554159678513, 0, 0, 3.82970843102537, 5.80409338312195, -0.223353131420178, -1.54743588659734, 0.341938252770122, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.76594168620508, 0, 1.49829835452878, 0.847565541382357, 0, 0, 1.62843044502542, 0, 3.54562104171167, -0.814818121686407, 0, 0, 0, 11.0554159678514, 0, 1.06581410364015e-14, 7.18070330817255, 0, -1.11676565710083, 0.197418142283059, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.48323969741914, 1.00508909139074, -0.193429485824666, 3.55615782880927, -8.46157280212812, 0, 0, 4.459640440343, -0.858258202906995, 2.67027285925187, 0, -1.29340982368831e-14, 0, 0, 12.4373429638327, 0, 0, 8.42792824422443, -1.62195554683794, -2.29379153190448, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.1489125293076, 0.259512887494068, 0.749149177264412, 2.75458800949266, -1.24344978758018e-14, -8.46157280212812, 0, 1.1514742103681, 3.32401972660468, 2.06838446274244, 0, 0, 0, 1.24344978758018e-14, 0, 12.4373429638328, -2.22044604925031e-14, 2.17608171552749, 6.2818068211921, -1.77676328054753, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.663324958071075, 0, 0.692034366650866, 1.59036212878711, 0, 0, -8.46157280212812, 0, 3.07059789431494, 1.19418232635199, 0, 0, -1.4210854715202e-14, 0, 0, 1.50990331349021e-14, 12.4373429638327, 0, 5.80288457473998, -1.02581475831035, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.37667655053519e-14, 0, 0, 0, 0, 0, 0, 0},
    {0.938083151964683, 7.62808987244124, 0, 3.11415464992887, 0, 0, 0, -7.5213980463361, 0, 5.52707620976692, 0, 0, 0, 0, 0, 0, 2.39808173319034e-14, 14.2141062443803, 1.59872115546023e-14, 5.80288457473999, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.541602560309058, 0, 7.62808987244124, 1.79795802543455, 0, 0, 0, 0, -7.5213980463361, 3.19105893754051, 0, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 14.2141062443803, 3.35029697130246, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-5.36159180343551, 0, 0, 8.89943818451482, 0, 0, 0, 0, 1.4210854715202e-14, -5.26497863243527, 0, 0, 0, 0, 1.4210854715202e-14, 0, 1.01585406753202e-14, -1.4210854715202e-14, -1.06581410364015e-14, 16.583123951777, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14},
    {2.41593350361254, 5.72988941019469, -2.20543546226846, -1.55947837083932, 8.33749895885385, -2.76779953857108, 1.06532653910463, -2.25989885993663, 0.869835476750495, 0.615066564126917, 10.333118277332, -2.18326971917505, 0.724780051083766, -0.27896797173099, -1.89076704011891, 0.627677936394712, -0.24159335036126, 0.512496888990059, -0.197260144544837, -0.139483985865486, 11.8490505948789, -1.16109782023228, 0.245326690731332, -0.0814411017927985, 0.031346694695451, -1.03851746182648, 0.21942686286813, -0.0728431359084674, 0.0280373360835801, 0.190029237516525, -0.063084006188054, 0.0242810453028244, -0.0515078753637721, 0.0198253904711043, 0.0140186680417933, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.18529407725405, 3.45525332069938, 0.997445717412057, -1.41060126129511, 3.23209352764252, 5.28531208250977, -2.89087233497829, -1.36277028773849, -0.393397896234717, 0.556348640264194, 2.33666428910958, 9.02784807228868, -3.93352933905729, 1.64018416088945, -0.732969916135168, -1.19859611757306, 0.655588223176204, 0.309047252182627, 0.0892142571199752, -0.126168012376108, 1.1908743922773, 11.55277744383, -2.44096975264614, 0.810328731479533, -0.311895674167854, -0.234843597212092, -0.907332870306512, 0.395334573318419, -0.164844710568313, 0.0736662483163242, 0.120463442338381, -0.0658890955530682, -0.0310604175145517, -0.00896637020658309, 0.0126803623514131, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.92724822331887, 1.82834820066013, 2.99085908983323, -1.2440333788203, 0.950146187582614, 5.7406445631131, -0.169967317119762, -0.721110255092803, -1.1796107330308, 0.490653381462659, 0.294392028877592, 4.9761335152812, 5.78174466995659, -3.22682211353229, -0.215472901842833, -1.30185581821819, 0.0385449644663778, 0.163532434528617, 0.267510569439617, -0.111269728052837, 0, 2.64638753839399, 10.7636648722209, -3.57321383950879, 1.37533064807499, -0.0295875121532885, -0.500120235663637, -0.581087203147972, 0.324307824721458, 0.0216558414515666, 0.130841432732452, -0.00387391468765502, -0.016435628072264, -0.0268858238268355, 0.0111830284386731, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.62882203585592, 0.772618130456575, 3.86595114186567, -1.05140010313427, 0.160604018609906, 3.35889012281481, 3.878512233171, -0.304724700110032, -1.52475169285615, 0.414677792940099, 0, 1.135512111385, 7.32969916135161, 0.564240504518033, -0.0364215679542301, -0.761724681099366, -0.879563899362179, 0.0691050664139837, 0.34578118203618, -0.0940400840863451, 0, 0, 4.46577897103986, 9.05975063854702, -3.48709964663735, 0, -0.114123261162675, -0.736662483163209, -0.0567083043941503, 0.00366050530850437, 0.0765562109321125, 0.0883994979795846, -0.00694532049720609, -0.0347523163751409, 0.00945138406569246, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.26168012376112, 0.199489143482424, 3.68560354207932, -0.814411017927961, 0, 1.02786571910339, 6.00856531485322, -0.078679579246943, -1.45362164025439, 0.321208037219817, 0, 0, 3.0280322970267, 6.99294104721296, 0, -0.233098034907105, -1.36261453366201, 0.0178428514239979, 0.329650402328133, -0.0728431359084714, 0, 0, 0, 6.80499652729884, 5.40219154319687, 0, 0, -0.304328696433807, -0.702817019232741, 0, 0.0234272339744261, 0.136947913395214, -0.00179327404131646, -0.0331311120155191, 0.00732101061700707, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.72843135908468, 0, 2.32737334062817, -0.470200420431707, 0, 0, 4.49691252107735, 0, -0.91792842454769, 0.185449546754732, 0, 0, 1.77635683940025e-14, 7.0654086930623, 0, 0, -1.01980390271858, 0, 0.208166599946617, -0.0420560041253766, 0, 0, 0, 0, 9.92395326897747, 0, 0, 0, -0.710100291678885, 0, 0, 0.102494148638109, 0, -0.02092153048203, 0.00422678745046823, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.99489143482414, 3.1542003094028, -1.09264703862703, 1.54523626091314, 2.95048422176041, -0.761811750275054, 0.263899331455879, 5.59815020469134, -1.93925611658551, -2.28543525082517, 2.13307290077014, -0.257539376818855, 0.0664963811608086, -0.023035022137995, 9.14446135479766, -2.36108976914458, 0.817905488277891, -3.3384229850651, 1.15646364545769, 1.02505848514082, 13.0453753561106, 0, 0, 0, 0, 11.5766222110848, -1.39771878815471, 0.360889439280295, -0.125015768949702, -2.15192884958005, 0.555625639776459, -0.192474367616154, 0.595437196138649, -0.206265495285697, -0.166687691932934, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.7593288763725, 1.66904592079255, 0.963624111659421, 1.36277028773849, 0.867360833110883, 2.28430636201793, -1.00852844760553, 2.96226264871971, 1.71026313764871, -2.01556443707463, 0.268741924943274, 2.04415543160775, -0.52779866291175, 0.182834820066012, 2.68821895760393, 7.07977053255619, -3.12574097035514, -1.76652739792483, -1.01990506872276, 0.904016610326405, 0, 13.0453753561106, 0, 0, 1.77635683940025e-14, 1.4585173044132, 11.0940489487798, -2.8644711213756, 0.992281903807275, -0.632607619454901, -1.66605356688627, 0.735567892896449, 0.31507574846809, 0.181909068193172, -0.147004726489962, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, -1.33226762955019e-14, 0},
    {1.48690428533297, 0.705300630647553, 2.03602754481991, 1.15175110689979, 0.146610739697709, 1.817027050318, 0.655663160391203, 1.25178444059441, 3.61359041872286, -1.70346285967311, 0, 0.614267257013221, 1.78428514239955, -0.618094504365256, 0.454391937932176, 5.63152770643899, 2.03210252328836, -0.746494073225344, -2.15494277062551, 0.764033484562614, 0, 0, 13.0453753561106, 0, 0, 0, 3.33375383865873, 9.68367982311023, -3.3545250915713, -0.106930204231195, -1.32524448062076, -0.478206411017788, 0.133143804689793, 0.384353057392904, -0.124241670058193, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.37667655053519e-14, 0},
    {1.15175110689979, 0.182107839771145, 2.20794021658194, 0.892142571199776, 0, 0.61576510673037, 2.13307290077016, 0.323209352764256, 3.91870514328395, -1.31949665727936, 0, -1.06581410364015e-14, 1.07496769977314, 1.11713920350151, 0, 1.90844613931508, 6.61105135360481, -0.192743940909633, -2.33689608954559, 0.591817792331167, 0, 0, 0, 13.0453753561106, 0, 0, 0, 5.83406921765278, 6.06294257990893, 0, -0.44910685777103, -1.55575179137407, 0.0343775825476159, 0.41680603729705, -0.0962371838080759, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.4210854715202e-14, 0},
    {0.66496381160804, 0, 1.45686271816934, 0.515078753637713, 0, -1.24344978758018e-14, 1.84729532019112, 0, 2.58567482211401, -0.761811750275046, 1.4210854715202e-14, -1.24344978758018e-14, 0, 1.71994831963701, 0, -1.4210854715202e-14, 5.72533841794525, 0, -1.54195152727693, 0.341686161713598, 0, 0, 0, 0, 13.0453753561106, 0, 0, 0, 9.33451074824443, 0, 0, -1.34732057331309, 0, 0.27502066038093, -0.0555625639776447, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0},
    {1.57359158493889, 1.49284005458435, -0.430945803685675, 3.04724700110023, 0.775791113542727, -0.120185042515474, 0.0346944333244412, 5.2990565197967, -1.52970585407783, 0.901387818865993, -10.5762837413611, 0, 0, 0, 4.5156107826894, -0.69955541437243, 0.201944253400492, 6.36462844462228, -1.83730997289733, -2.74825341360596, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 0, 10.8920630294282, -1.68739114875268, 0.487107866980279, -3.09993548319961, 0.894874292847885, 0.903959543974656, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.32992762321608, 0.63084006188056, 0.728431359084682, 2.57539376818856, 0.131132632078239, 0.71102430025671, -0.205255035576787, 2.23926008187652, 2.58567482211401, 0.761811750275053, 0, -10.576283741361, 0, 0, 0.763277533137579, 4.1386256441266, -1.19471831485579, 2.68954640486935, 3.1056206816986, -2.32269806558125, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 1.84109468241849, 9.98274066131697, -2.88176900403079, -1.30996183150502, -1.51261363209512, 0.763985254692621, 0, 0, 0, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.28785870856518e-14, 0, 0},
    {1.03015750727542, 0.162882203585586, 1.1284810090361, 1.99489143482414, 0, 0.314718316987764, 0.476969600708468, 0.57817446699566, 4.00571020990214, 0.590096844352083, 0, 0, -10.5762837413611, 0, 0, 1.83186607953019, 2.77627448210727, 0.694437895660723, 4.81120687194241, -1.79915418525285, 0, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 4.41862723780437, 6.69662601613678, -0.338230690505752, -2.34333096254029, 0.591780433634522, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.14352971536391e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.594761714133174, -1.19904086659517e-14, 0.814411017927931, 1.15175110689982, 0, 0, 0.590096844352076, 0, 2.89087233497831, 0.340692571934628, 0, 0, 0, -10.576283741361, 0, 0, 3.43474889911912, -1.24344978758018e-14, 3.47218947830365, -1.03874215316938, 0, 0, 0, 0, 0, 0, 0, 0, 14.3499128917217, 0, 0, 8.28492607088319, 0, -1.69115345252878, 0.341664592660043, 0, 0, 0, 0, 0, -1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.68753899743024e-14, 0},
    {1.15175110689979, 0.546323519313511, -0.105140010313427, 3.27118942773251, 9.65295620481434, 0, 0, 2.69341127303542, -0.518347241175114, 4.25171145123353, 0, 0, 0, 0, -9.91526600752598, 0, 1.53210777398272e-14, 6.71889089790563, -1.29305115618718, 1.82865048192216, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.9443476574685, 0, 0, 9.45383520059451, -1.81939143264587, -2.94058061635793, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.77635683940025e-14, 0, 0, 0, 0, 1.68753899743024e-14},
    {0.892142571199775, 0.141060126129508, 0.407205508963975, 2.53385243517955, 0, 9.65295620481433, 0, 0.69543580033025, 2.00755023262383, 3.29336152870135, 0, 0, 0, 0, 0, -9.91526600752598, 2.12052597703405e-14, 1.73481017017154, 5.0079655937072, 1.41646657250371, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.9443476574685, 0, 2.44096975264614, 7.0464727188699, -2.27776395106693, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0.515078753637709, 0, 0.376160336345346, 1.46292038553773, 0, 0, 9.65295620481433, 0, 1.8544954675473, 1.9014231651345, 0, 0, 0, -1.06581410364015e-14, 0, 0, -9.91526600752595, 0, 4.62616045379076, 0.81779735693317, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 15.9443476574685, 0, 6.50925267372302, -1.3150676302989, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 1.86517468137026e-14},
    {0.728431359084674, -7.94708263760857, 0, 2.53908227033117, 0, 0, 0, 10.2207771580387, 0, 5.00713776237767, 0, 0, 0, 0, -1.33226762955019e-14, 2.1316282072803e-14, 0, -8.49879943502227, -1.01030295240889e-14, 6.93924068068615, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 17.9373911146521, 0, 6.2767793639472, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.06581410364015e-14, 0, 0, 0, 0},
    {0.42056004125371, 0, -7.94708263760858, 1.46593983227034, 0, 0, 0, 0, 10.2207771580387, 2.89087233497828, 0, 0, 0, -1.06581410364015e-14, 2.1316282072803e-14, -1.77635683940025e-14, 2.62012633811537e-14, 0, -8.49879943502228, 4.00637247496576, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 17.9373911146521, 3.6239002554188, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.22124532708767e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {6.54237885546499, 0, 1.50990331349021e-14, -6.91050664139879, 0, 0, 0, 0, 0, 12.2649325896464, 0, -1.79856129989275e-14, 1.77635683940025e-14, 1.33226762955019e-14, 0, 0, -1.75415237890775e-14, 0, 1.06581410364015e-14, -5.66586629001485, 0, 0, 0, 0, 0, 0, 0, 0, -1.15463194561016e-14, 0, 0, 0, 0, 0, 20.4998755596024, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.46549439250521e-14, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, -1.59872115546023e-14}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    double coeff0_6 = 0;
    double coeff0_7 = 0;
    double coeff0_8 = 0;
    double coeff0_9 = 0;
    double coeff0_10 = 0;
    double coeff0_11 = 0;
    double coeff0_12 = 0;
    double coeff0_13 = 0;
    double coeff0_14 = 0;
    double coeff0_15 = 0;
    double coeff0_16 = 0;
    double coeff0_17 = 0;
    double coeff0_18 = 0;
    double coeff0_19 = 0;
    double coeff0_20 = 0;
    double coeff0_21 = 0;
    double coeff0_22 = 0;
    double coeff0_23 = 0;
    double coeff0_24 = 0;
    double coeff0_25 = 0;
    double coeff0_26 = 0;
    double coeff0_27 = 0;
    double coeff0_28 = 0;
    double coeff0_29 = 0;
    double coeff0_30 = 0;
    double coeff0_31 = 0;
    double coeff0_32 = 0;
    double coeff0_33 = 0;
    double coeff0_34 = 0;
    double coeff0_35 = 0;
    double coeff0_36 = 0;
    double coeff0_37 = 0;
    double coeff0_38 = 0;
    double coeff0_39 = 0;
    double coeff0_40 = 0;
    double coeff0_41 = 0;
    double coeff0_42 = 0;
    double coeff0_43 = 0;
    double coeff0_44 = 0;
    double coeff0_45 = 0;
    double coeff0_46 = 0;
    double coeff0_47 = 0;
    double coeff0_48 = 0;
    double coeff0_49 = 0;
    double coeff0_50 = 0;
    double coeff0_51 = 0;
    double coeff0_52 = 0;
    double coeff0_53 = 0;
    double coeff0_54 = 0;
    double coeff0_55 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    double new_coeff0_6 = 0;
    double new_coeff0_7 = 0;
    double new_coeff0_8 = 0;
    double new_coeff0_9 = 0;
    double new_coeff0_10 = 0;
    double new_coeff0_11 = 0;
    double new_coeff0_12 = 0;
    double new_coeff0_13 = 0;
    double new_coeff0_14 = 0;
    double new_coeff0_15 = 0;
    double new_coeff0_16 = 0;
    double new_coeff0_17 = 0;
    double new_coeff0_18 = 0;
    double new_coeff0_19 = 0;
    double new_coeff0_20 = 0;
    double new_coeff0_21 = 0;
    double new_coeff0_22 = 0;
    double new_coeff0_23 = 0;
    double new_coeff0_24 = 0;
    double new_coeff0_25 = 0;
    double new_coeff0_26 = 0;
    double new_coeff0_27 = 0;
    double new_coeff0_28 = 0;
    double new_coeff0_29 = 0;
    double new_coeff0_30 = 0;
    double new_coeff0_31 = 0;
    double new_coeff0_32 = 0;
    double new_coeff0_33 = 0;
    double new_coeff0_34 = 0;
    double new_coeff0_35 = 0;
    double new_coeff0_36 = 0;
    double new_coeff0_37 = 0;
    double new_coeff0_38 = 0;
    double new_coeff0_39 = 0;
    double new_coeff0_40 = 0;
    double new_coeff0_41 = 0;
    double new_coeff0_42 = 0;
    double new_coeff0_43 = 0;
    double new_coeff0_44 = 0;
    double new_coeff0_45 = 0;
    double new_coeff0_46 = 0;
    double new_coeff0_47 = 0;
    double new_coeff0_48 = 0;
    double new_coeff0_49 = 0;
    double new_coeff0_50 = 0;
    double new_coeff0_51 = 0;
    double new_coeff0_52 = 0;
    double new_coeff0_53 = 0;
    double new_coeff0_54 = 0;
    double new_coeff0_55 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
      new_coeff0_6 = coefficients0[dof][6];
      new_coeff0_7 = coefficients0[dof][7];
      new_coeff0_8 = coefficients0[dof][8];
      new_coeff0_9 = coefficients0[dof][9];
      new_coeff0_10 = coefficients0[dof][10];
      new_coeff0_11 = coefficients0[dof][11];
      new_coeff0_12 = coefficients0[dof][12];
      new_coeff0_13 = coefficients0[dof][13];
      new_coeff0_14 = coefficients0[dof][14];
      new_coeff0_15 = coefficients0[dof][15];
      new_coeff0_16 = coefficients0[dof][16];
      new_coeff0_17 = coefficients0[dof][17];
      new_coeff0_18 = coefficients0[dof][18];
      new_coeff0_19 = coefficients0[dof][19];
      new_coeff0_20 = coefficients0[dof][20];
      new_coeff0_21 = coefficients0[dof][21];
      new_coeff0_22 = coefficients0[dof][22];
      new_coeff0_23 = coefficients0[dof][23];
      new_coeff0_24 = coefficients0[dof][24];
      new_coeff0_25 = coefficients0[dof][25];
      new_coeff0_26 = coefficients0[dof][26];
      new_coeff0_27 = coefficients0[dof][27];
      new_coeff0_28 = coefficients0[dof][28];
      new_coeff0_29 = coefficients0[dof][29];
      new_coeff0_30 = coefficients0[dof][30];
      new_coeff0_31 = coefficients0[dof][31];
      new_coeff0_32 = coefficients0[dof][32];
      new_coeff0_33 = coefficients0[dof][33];
      new_coeff0_34 = coefficients0[dof][34];
      new_coeff0_35 = coefficients0[dof][35];
      new_coeff0_36 = coefficients0[dof][36];
      new_coeff0_37 = coefficients0[dof][37];
      new_coeff0_38 = coefficients0[dof][38];
      new_coeff0_39 = coefficients0[dof][39];
      new_coeff0_40 = coefficients0[dof][40];
      new_coeff0_41 = coefficients0[dof][41];
      new_coeff0_42 = coefficients0[dof][42];
      new_coeff0_43 = coefficients0[dof][43];
      new_coeff0_44 = coefficients0[dof][44];
      new_coeff0_45 = coefficients0[dof][45];
      new_coeff0_46 = coefficients0[dof][46];
      new_coeff0_47 = coefficients0[dof][47];
      new_coeff0_48 = coefficients0[dof][48];
      new_coeff0_49 = coefficients0[dof][49];
      new_coeff0_50 = coefficients0[dof][50];
      new_coeff0_51 = coefficients0[dof][51];
      new_coeff0_52 = coefficients0[dof][52];
      new_coeff0_53 = coefficients0[dof][53];
      new_coeff0_54 = coefficients0[dof][54];
      new_coeff0_55 = coefficients0[dof][55];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
        coeff0_6 = new_coeff0_6;
        coeff0_7 = new_coeff0_7;
        coeff0_8 = new_coeff0_8;
        coeff0_9 = new_coeff0_9;
        coeff0_10 = new_coeff0_10;
        coeff0_11 = new_coeff0_11;
        coeff0_12 = new_coeff0_12;
        coeff0_13 = new_coeff0_13;
        coeff0_14 = new_coeff0_14;
        coeff0_15 = new_coeff0_15;
        coeff0_16 = new_coeff0_16;
        coeff0_17 = new_coeff0_17;
        coeff0_18 = new_coeff0_18;
        coeff0_19 = new_coeff0_19;
        coeff0_20 = new_coeff0_20;
        coeff0_21 = new_coeff0_21;
        coeff0_22 = new_coeff0_22;
        coeff0_23 = new_coeff0_23;
        coeff0_24 = new_coeff0_24;
        coeff0_25 = new_coeff0_25;
        coeff0_26 = new_coeff0_26;
        coeff0_27 = new_coeff0_27;
        coeff0_28 = new_coeff0_28;
        coeff0_29 = new_coeff0_29;
        coeff0_30 = new_coeff0_30;
        coeff0_31 = new_coeff0_31;
        coeff0_32 = new_coeff0_32;
        coeff0_33 = new_coeff0_33;
        coeff0_34 = new_coeff0_34;
        coeff0_35 = new_coeff0_35;
        coeff0_36 = new_coeff0_36;
        coeff0_37 = new_coeff0_37;
        coeff0_38 = new_coeff0_38;
        coeff0_39 = new_coeff0_39;
        coeff0_40 = new_coeff0_40;
        coeff0_41 = new_coeff0_41;
        coeff0_42 = new_coeff0_42;
        coeff0_43 = new_coeff0_43;
        coeff0_44 = new_coeff0_44;
        coeff0_45 = new_coeff0_45;
        coeff0_46 = new_coeff0_46;
        coeff0_47 = new_coeff0_47;
        coeff0_48 = new_coeff0_48;
        coeff0_49 = new_coeff0_49;
        coeff0_50 = new_coeff0_50;
        coeff0_51 = new_coeff0_51;
        coeff0_52 = new_coeff0_52;
        coeff0_53 = new_coeff0_53;
        coeff0_54 = new_coeff0_54;
        coeff0_55 = new_coeff0_55;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0] + coeff0_10*dmats0[10][0] + coeff0_11*dmats0[11][0] + coeff0_12*dmats0[12][0] + coeff0_13*dmats0[13][0] + coeff0_14*dmats0[14][0] + coeff0_15*dmats0[15][0] + coeff0_16*dmats0[16][0] + coeff0_17*dmats0[17][0] + coeff0_18*dmats0[18][0] + coeff0_19*dmats0[19][0] + coeff0_20*dmats0[20][0] + coeff0_21*dmats0[21][0] + coeff0_22*dmats0[22][0] + coeff0_23*dmats0[23][0] + coeff0_24*dmats0[24][0] + coeff0_25*dmats0[25][0] + coeff0_26*dmats0[26][0] + coeff0_27*dmats0[27][0] + coeff0_28*dmats0[28][0] + coeff0_29*dmats0[29][0] + coeff0_30*dmats0[30][0] + coeff0_31*dmats0[31][0] + coeff0_32*dmats0[32][0] + coeff0_33*dmats0[33][0] + coeff0_34*dmats0[34][0] + coeff0_35*dmats0[35][0] + coeff0_36*dmats0[36][0] + coeff0_37*dmats0[37][0] + coeff0_38*dmats0[38][0] + coeff0_39*dmats0[39][0] + coeff0_40*dmats0[40][0] + coeff0_41*dmats0[41][0] + coeff0_42*dmats0[42][0] + coeff0_43*dmats0[43][0] + coeff0_44*dmats0[44][0] + coeff0_45*dmats0[45][0] + coeff0_46*dmats0[46][0] + coeff0_47*dmats0[47][0] + coeff0_48*dmats0[48][0] + coeff0_49*dmats0[49][0] + coeff0_50*dmats0[50][0] + coeff0_51*dmats0[51][0] + coeff0_52*dmats0[52][0] + coeff0_53*dmats0[53][0] + coeff0_54*dmats0[54][0] + coeff0_55*dmats0[55][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1] + coeff0_10*dmats0[10][1] + coeff0_11*dmats0[11][1] + coeff0_12*dmats0[12][1] + coeff0_13*dmats0[13][1] + coeff0_14*dmats0[14][1] + coeff0_15*dmats0[15][1] + coeff0_16*dmats0[16][1] + coeff0_17*dmats0[17][1] + coeff0_18*dmats0[18][1] + coeff0_19*dmats0[19][1] + coeff0_20*dmats0[20][1] + coeff0_21*dmats0[21][1] + coeff0_22*dmats0[22][1] + coeff0_23*dmats0[23][1] + coeff0_24*dmats0[24][1] + coeff0_25*dmats0[25][1] + coeff0_26*dmats0[26][1] + coeff0_27*dmats0[27][1] + coeff0_28*dmats0[28][1] + coeff0_29*dmats0[29][1] + coeff0_30*dmats0[30][1] + coeff0_31*dmats0[31][1] + coeff0_32*dmats0[32][1] + coeff0_33*dmats0[33][1] + coeff0_34*dmats0[34][1] + coeff0_35*dmats0[35][1] + coeff0_36*dmats0[36][1] + coeff0_37*dmats0[37][1] + coeff0_38*dmats0[38][1] + coeff0_39*dmats0[39][1] + coeff0_40*dmats0[40][1] + coeff0_41*dmats0[41][1] + coeff0_42*dmats0[42][1] + coeff0_43*dmats0[43][1] + coeff0_44*dmats0[44][1] + coeff0_45*dmats0[45][1] + coeff0_46*dmats0[46][1] + coeff0_47*dmats0[47][1] + coeff0_48*dmats0[48][1] + coeff0_49*dmats0[49][1] + coeff0_50*dmats0[50][1] + coeff0_51*dmats0[51][1] + coeff0_52*dmats0[52][1] + coeff0_53*dmats0[53][1] + coeff0_54*dmats0[54][1] + coeff0_55*dmats0[55][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2] + coeff0_10*dmats0[10][2] + coeff0_11*dmats0[11][2] + coeff0_12*dmats0[12][2] + coeff0_13*dmats0[13][2] + coeff0_14*dmats0[14][2] + coeff0_15*dmats0[15][2] + coeff0_16*dmats0[16][2] + coeff0_17*dmats0[17][2] + coeff0_18*dmats0[18][2] + coeff0_19*dmats0[19][2] + coeff0_20*dmats0[20][2] + coeff0_21*dmats0[21][2] + coeff0_22*dmats0[22][2] + coeff0_23*dmats0[23][2] + coeff0_24*dmats0[24][2] + coeff0_25*dmats0[25][2] + coeff0_26*dmats0[26][2] + coeff0_27*dmats0[27][2] + coeff0_28*dmats0[28][2] + coeff0_29*dmats0[29][2] + coeff0_30*dmats0[30][2] + coeff0_31*dmats0[31][2] + coeff0_32*dmats0[32][2] + coeff0_33*dmats0[33][2] + coeff0_34*dmats0[34][2] + coeff0_35*dmats0[35][2] + coeff0_36*dmats0[36][2] + coeff0_37*dmats0[37][2] + coeff0_38*dmats0[38][2] + coeff0_39*dmats0[39][2] + coeff0_40*dmats0[40][2] + coeff0_41*dmats0[41][2] + coeff0_42*dmats0[42][2] + coeff0_43*dmats0[43][2] + coeff0_44*dmats0[44][2] + coeff0_45*dmats0[45][2] + coeff0_46*dmats0[46][2] + coeff0_47*dmats0[47][2] + coeff0_48*dmats0[48][2] + coeff0_49*dmats0[49][2] + coeff0_50*dmats0[50][2] + coeff0_51*dmats0[51][2] + coeff0_52*dmats0[52][2] + coeff0_53*dmats0[53][2] + coeff0_54*dmats0[54][2] + coeff0_55*dmats0[55][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3] + coeff0_10*dmats0[10][3] + coeff0_11*dmats0[11][3] + coeff0_12*dmats0[12][3] + coeff0_13*dmats0[13][3] + coeff0_14*dmats0[14][3] + coeff0_15*dmats0[15][3] + coeff0_16*dmats0[16][3] + coeff0_17*dmats0[17][3] + coeff0_18*dmats0[18][3] + coeff0_19*dmats0[19][3] + coeff0_20*dmats0[20][3] + coeff0_21*dmats0[21][3] + coeff0_22*dmats0[22][3] + coeff0_23*dmats0[23][3] + coeff0_24*dmats0[24][3] + coeff0_25*dmats0[25][3] + coeff0_26*dmats0[26][3] + coeff0_27*dmats0[27][3] + coeff0_28*dmats0[28][3] + coeff0_29*dmats0[29][3] + coeff0_30*dmats0[30][3] + coeff0_31*dmats0[31][3] + coeff0_32*dmats0[32][3] + coeff0_33*dmats0[33][3] + coeff0_34*dmats0[34][3] + coeff0_35*dmats0[35][3] + coeff0_36*dmats0[36][3] + coeff0_37*dmats0[37][3] + coeff0_38*dmats0[38][3] + coeff0_39*dmats0[39][3] + coeff0_40*dmats0[40][3] + coeff0_41*dmats0[41][3] + coeff0_42*dmats0[42][3] + coeff0_43*dmats0[43][3] + coeff0_44*dmats0[44][3] + coeff0_45*dmats0[45][3] + coeff0_46*dmats0[46][3] + coeff0_47*dmats0[47][3] + coeff0_48*dmats0[48][3] + coeff0_49*dmats0[49][3] + coeff0_50*dmats0[50][3] + coeff0_51*dmats0[51][3] + coeff0_52*dmats0[52][3] + coeff0_53*dmats0[53][3] + coeff0_54*dmats0[54][3] + coeff0_55*dmats0[55][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4] + coeff0_10*dmats0[10][4] + coeff0_11*dmats0[11][4] + coeff0_12*dmats0[12][4] + coeff0_13*dmats0[13][4] + coeff0_14*dmats0[14][4] + coeff0_15*dmats0[15][4] + coeff0_16*dmats0[16][4] + coeff0_17*dmats0[17][4] + coeff0_18*dmats0[18][4] + coeff0_19*dmats0[19][4] + coeff0_20*dmats0[20][4] + coeff0_21*dmats0[21][4] + coeff0_22*dmats0[22][4] + coeff0_23*dmats0[23][4] + coeff0_24*dmats0[24][4] + coeff0_25*dmats0[25][4] + coeff0_26*dmats0[26][4] + coeff0_27*dmats0[27][4] + coeff0_28*dmats0[28][4] + coeff0_29*dmats0[29][4] + coeff0_30*dmats0[30][4] + coeff0_31*dmats0[31][4] + coeff0_32*dmats0[32][4] + coeff0_33*dmats0[33][4] + coeff0_34*dmats0[34][4] + coeff0_35*dmats0[35][4] + coeff0_36*dmats0[36][4] + coeff0_37*dmats0[37][4] + coeff0_38*dmats0[38][4] + coeff0_39*dmats0[39][4] + coeff0_40*dmats0[40][4] + coeff0_41*dmats0[41][4] + coeff0_42*dmats0[42][4] + coeff0_43*dmats0[43][4] + coeff0_44*dmats0[44][4] + coeff0_45*dmats0[45][4] + coeff0_46*dmats0[46][4] + coeff0_47*dmats0[47][4] + coeff0_48*dmats0[48][4] + coeff0_49*dmats0[49][4] + coeff0_50*dmats0[50][4] + coeff0_51*dmats0[51][4] + coeff0_52*dmats0[52][4] + coeff0_53*dmats0[53][4] + coeff0_54*dmats0[54][4] + coeff0_55*dmats0[55][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5] + coeff0_10*dmats0[10][5] + coeff0_11*dmats0[11][5] + coeff0_12*dmats0[12][5] + coeff0_13*dmats0[13][5] + coeff0_14*dmats0[14][5] + coeff0_15*dmats0[15][5] + coeff0_16*dmats0[16][5] + coeff0_17*dmats0[17][5] + coeff0_18*dmats0[18][5] + coeff0_19*dmats0[19][5] + coeff0_20*dmats0[20][5] + coeff0_21*dmats0[21][5] + coeff0_22*dmats0[22][5] + coeff0_23*dmats0[23][5] + coeff0_24*dmats0[24][5] + coeff0_25*dmats0[25][5] + coeff0_26*dmats0[26][5] + coeff0_27*dmats0[27][5] + coeff0_28*dmats0[28][5] + coeff0_29*dmats0[29][5] + coeff0_30*dmats0[30][5] + coeff0_31*dmats0[31][5] + coeff0_32*dmats0[32][5] + coeff0_33*dmats0[33][5] + coeff0_34*dmats0[34][5] + coeff0_35*dmats0[35][5] + coeff0_36*dmats0[36][5] + coeff0_37*dmats0[37][5] + coeff0_38*dmats0[38][5] + coeff0_39*dmats0[39][5] + coeff0_40*dmats0[40][5] + coeff0_41*dmats0[41][5] + coeff0_42*dmats0[42][5] + coeff0_43*dmats0[43][5] + coeff0_44*dmats0[44][5] + coeff0_45*dmats0[45][5] + coeff0_46*dmats0[46][5] + coeff0_47*dmats0[47][5] + coeff0_48*dmats0[48][5] + coeff0_49*dmats0[49][5] + coeff0_50*dmats0[50][5] + coeff0_51*dmats0[51][5] + coeff0_52*dmats0[52][5] + coeff0_53*dmats0[53][5] + coeff0_54*dmats0[54][5] + coeff0_55*dmats0[55][5];
          new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6] + coeff0_10*dmats0[10][6] + coeff0_11*dmats0[11][6] + coeff0_12*dmats0[12][6] + coeff0_13*dmats0[13][6] + coeff0_14*dmats0[14][6] + coeff0_15*dmats0[15][6] + coeff0_16*dmats0[16][6] + coeff0_17*dmats0[17][6] + coeff0_18*dmats0[18][6] + coeff0_19*dmats0[19][6] + coeff0_20*dmats0[20][6] + coeff0_21*dmats0[21][6] + coeff0_22*dmats0[22][6] + coeff0_23*dmats0[23][6] + coeff0_24*dmats0[24][6] + coeff0_25*dmats0[25][6] + coeff0_26*dmats0[26][6] + coeff0_27*dmats0[27][6] + coeff0_28*dmats0[28][6] + coeff0_29*dmats0[29][6] + coeff0_30*dmats0[30][6] + coeff0_31*dmats0[31][6] + coeff0_32*dmats0[32][6] + coeff0_33*dmats0[33][6] + coeff0_34*dmats0[34][6] + coeff0_35*dmats0[35][6] + coeff0_36*dmats0[36][6] + coeff0_37*dmats0[37][6] + coeff0_38*dmats0[38][6] + coeff0_39*dmats0[39][6] + coeff0_40*dmats0[40][6] + coeff0_41*dmats0[41][6] + coeff0_42*dmats0[42][6] + coeff0_43*dmats0[43][6] + coeff0_44*dmats0[44][6] + coeff0_45*dmats0[45][6] + coeff0_46*dmats0[46][6] + coeff0_47*dmats0[47][6] + coeff0_48*dmats0[48][6] + coeff0_49*dmats0[49][6] + coeff0_50*dmats0[50][6] + coeff0_51*dmats0[51][6] + coeff0_52*dmats0[52][6] + coeff0_53*dmats0[53][6] + coeff0_54*dmats0[54][6] + coeff0_55*dmats0[55][6];
          new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7] + coeff0_10*dmats0[10][7] + coeff0_11*dmats0[11][7] + coeff0_12*dmats0[12][7] + coeff0_13*dmats0[13][7] + coeff0_14*dmats0[14][7] + coeff0_15*dmats0[15][7] + coeff0_16*dmats0[16][7] + coeff0_17*dmats0[17][7] + coeff0_18*dmats0[18][7] + coeff0_19*dmats0[19][7] + coeff0_20*dmats0[20][7] + coeff0_21*dmats0[21][7] + coeff0_22*dmats0[22][7] + coeff0_23*dmats0[23][7] + coeff0_24*dmats0[24][7] + coeff0_25*dmats0[25][7] + coeff0_26*dmats0[26][7] + coeff0_27*dmats0[27][7] + coeff0_28*dmats0[28][7] + coeff0_29*dmats0[29][7] + coeff0_30*dmats0[30][7] + coeff0_31*dmats0[31][7] + coeff0_32*dmats0[32][7] + coeff0_33*dmats0[33][7] + coeff0_34*dmats0[34][7] + coeff0_35*dmats0[35][7] + coeff0_36*dmats0[36][7] + coeff0_37*dmats0[37][7] + coeff0_38*dmats0[38][7] + coeff0_39*dmats0[39][7] + coeff0_40*dmats0[40][7] + coeff0_41*dmats0[41][7] + coeff0_42*dmats0[42][7] + coeff0_43*dmats0[43][7] + coeff0_44*dmats0[44][7] + coeff0_45*dmats0[45][7] + coeff0_46*dmats0[46][7] + coeff0_47*dmats0[47][7] + coeff0_48*dmats0[48][7] + coeff0_49*dmats0[49][7] + coeff0_50*dmats0[50][7] + coeff0_51*dmats0[51][7] + coeff0_52*dmats0[52][7] + coeff0_53*dmats0[53][7] + coeff0_54*dmats0[54][7] + coeff0_55*dmats0[55][7];
          new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8] + coeff0_10*dmats0[10][8] + coeff0_11*dmats0[11][8] + coeff0_12*dmats0[12][8] + coeff0_13*dmats0[13][8] + coeff0_14*dmats0[14][8] + coeff0_15*dmats0[15][8] + coeff0_16*dmats0[16][8] + coeff0_17*dmats0[17][8] + coeff0_18*dmats0[18][8] + coeff0_19*dmats0[19][8] + coeff0_20*dmats0[20][8] + coeff0_21*dmats0[21][8] + coeff0_22*dmats0[22][8] + coeff0_23*dmats0[23][8] + coeff0_24*dmats0[24][8] + coeff0_25*dmats0[25][8] + coeff0_26*dmats0[26][8] + coeff0_27*dmats0[27][8] + coeff0_28*dmats0[28][8] + coeff0_29*dmats0[29][8] + coeff0_30*dmats0[30][8] + coeff0_31*dmats0[31][8] + coeff0_32*dmats0[32][8] + coeff0_33*dmats0[33][8] + coeff0_34*dmats0[34][8] + coeff0_35*dmats0[35][8] + coeff0_36*dmats0[36][8] + coeff0_37*dmats0[37][8] + coeff0_38*dmats0[38][8] + coeff0_39*dmats0[39][8] + coeff0_40*dmats0[40][8] + coeff0_41*dmats0[41][8] + coeff0_42*dmats0[42][8] + coeff0_43*dmats0[43][8] + coeff0_44*dmats0[44][8] + coeff0_45*dmats0[45][8] + coeff0_46*dmats0[46][8] + coeff0_47*dmats0[47][8] + coeff0_48*dmats0[48][8] + coeff0_49*dmats0[49][8] + coeff0_50*dmats0[50][8] + coeff0_51*dmats0[51][8] + coeff0_52*dmats0[52][8] + coeff0_53*dmats0[53][8] + coeff0_54*dmats0[54][8] + coeff0_55*dmats0[55][8];
          new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9] + coeff0_10*dmats0[10][9] + coeff0_11*dmats0[11][9] + coeff0_12*dmats0[12][9] + coeff0_13*dmats0[13][9] + coeff0_14*dmats0[14][9] + coeff0_15*dmats0[15][9] + coeff0_16*dmats0[16][9] + coeff0_17*dmats0[17][9] + coeff0_18*dmats0[18][9] + coeff0_19*dmats0[19][9] + coeff0_20*dmats0[20][9] + coeff0_21*dmats0[21][9] + coeff0_22*dmats0[22][9] + coeff0_23*dmats0[23][9] + coeff0_24*dmats0[24][9] + coeff0_25*dmats0[25][9] + coeff0_26*dmats0[26][9] + coeff0_27*dmats0[27][9] + coeff0_28*dmats0[28][9] + coeff0_29*dmats0[29][9] + coeff0_30*dmats0[30][9] + coeff0_31*dmats0[31][9] + coeff0_32*dmats0[32][9] + coeff0_33*dmats0[33][9] + coeff0_34*dmats0[34][9] + coeff0_35*dmats0[35][9] + coeff0_36*dmats0[36][9] + coeff0_37*dmats0[37][9] + coeff0_38*dmats0[38][9] + coeff0_39*dmats0[39][9] + coeff0_40*dmats0[40][9] + coeff0_41*dmats0[41][9] + coeff0_42*dmats0[42][9] + coeff0_43*dmats0[43][9] + coeff0_44*dmats0[44][9] + coeff0_45*dmats0[45][9] + coeff0_46*dmats0[46][9] + coeff0_47*dmats0[47][9] + coeff0_48*dmats0[48][9] + coeff0_49*dmats0[49][9] + coeff0_50*dmats0[50][9] + coeff0_51*dmats0[51][9] + coeff0_52*dmats0[52][9] + coeff0_53*dmats0[53][9] + coeff0_54*dmats0[54][9] + coeff0_55*dmats0[55][9];
          new_coeff0_10 = coeff0_0*dmats0[0][10] + coeff0_1*dmats0[1][10] + coeff0_2*dmats0[2][10] + coeff0_3*dmats0[3][10] + coeff0_4*dmats0[4][10] + coeff0_5*dmats0[5][10] + coeff0_6*dmats0[6][10] + coeff0_7*dmats0[7][10] + coeff0_8*dmats0[8][10] + coeff0_9*dmats0[9][10] + coeff0_10*dmats0[10][10] + coeff0_11*dmats0[11][10] + coeff0_12*dmats0[12][10] + coeff0_13*dmats0[13][10] + coeff0_14*dmats0[14][10] + coeff0_15*dmats0[15][10] + coeff0_16*dmats0[16][10] + coeff0_17*dmats0[17][10] + coeff0_18*dmats0[18][10] + coeff0_19*dmats0[19][10] + coeff0_20*dmats0[20][10] + coeff0_21*dmats0[21][10] + coeff0_22*dmats0[22][10] + coeff0_23*dmats0[23][10] + coeff0_24*dmats0[24][10] + coeff0_25*dmats0[25][10] + coeff0_26*dmats0[26][10] + coeff0_27*dmats0[27][10] + coeff0_28*dmats0[28][10] + coeff0_29*dmats0[29][10] + coeff0_30*dmats0[30][10] + coeff0_31*dmats0[31][10] + coeff0_32*dmats0[32][10] + coeff0_33*dmats0[33][10] + coeff0_34*dmats0[34][10] + coeff0_35*dmats0[35][10] + coeff0_36*dmats0[36][10] + coeff0_37*dmats0[37][10] + coeff0_38*dmats0[38][10] + coeff0_39*dmats0[39][10] + coeff0_40*dmats0[40][10] + coeff0_41*dmats0[41][10] + coeff0_42*dmats0[42][10] + coeff0_43*dmats0[43][10] + coeff0_44*dmats0[44][10] + coeff0_45*dmats0[45][10] + coeff0_46*dmats0[46][10] + coeff0_47*dmats0[47][10] + coeff0_48*dmats0[48][10] + coeff0_49*dmats0[49][10] + coeff0_50*dmats0[50][10] + coeff0_51*dmats0[51][10] + coeff0_52*dmats0[52][10] + coeff0_53*dmats0[53][10] + coeff0_54*dmats0[54][10] + coeff0_55*dmats0[55][10];
          new_coeff0_11 = coeff0_0*dmats0[0][11] + coeff0_1*dmats0[1][11] + coeff0_2*dmats0[2][11] + coeff0_3*dmats0[3][11] + coeff0_4*dmats0[4][11] + coeff0_5*dmats0[5][11] + coeff0_6*dmats0[6][11] + coeff0_7*dmats0[7][11] + coeff0_8*dmats0[8][11] + coeff0_9*dmats0[9][11] + coeff0_10*dmats0[10][11] + coeff0_11*dmats0[11][11] + coeff0_12*dmats0[12][11] + coeff0_13*dmats0[13][11] + coeff0_14*dmats0[14][11] + coeff0_15*dmats0[15][11] + coeff0_16*dmats0[16][11] + coeff0_17*dmats0[17][11] + coeff0_18*dmats0[18][11] + coeff0_19*dmats0[19][11] + coeff0_20*dmats0[20][11] + coeff0_21*dmats0[21][11] + coeff0_22*dmats0[22][11] + coeff0_23*dmats0[23][11] + coeff0_24*dmats0[24][11] + coeff0_25*dmats0[25][11] + coeff0_26*dmats0[26][11] + coeff0_27*dmats0[27][11] + coeff0_28*dmats0[28][11] + coeff0_29*dmats0[29][11] + coeff0_30*dmats0[30][11] + coeff0_31*dmats0[31][11] + coeff0_32*dmats0[32][11] + coeff0_33*dmats0[33][11] + coeff0_34*dmats0[34][11] + coeff0_35*dmats0[35][11] + coeff0_36*dmats0[36][11] + coeff0_37*dmats0[37][11] + coeff0_38*dmats0[38][11] + coeff0_39*dmats0[39][11] + coeff0_40*dmats0[40][11] + coeff0_41*dmats0[41][11] + coeff0_42*dmats0[42][11] + coeff0_43*dmats0[43][11] + coeff0_44*dmats0[44][11] + coeff0_45*dmats0[45][11] + coeff0_46*dmats0[46][11] + coeff0_47*dmats0[47][11] + coeff0_48*dmats0[48][11] + coeff0_49*dmats0[49][11] + coeff0_50*dmats0[50][11] + coeff0_51*dmats0[51][11] + coeff0_52*dmats0[52][11] + coeff0_53*dmats0[53][11] + coeff0_54*dmats0[54][11] + coeff0_55*dmats0[55][11];
          new_coeff0_12 = coeff0_0*dmats0[0][12] + coeff0_1*dmats0[1][12] + coeff0_2*dmats0[2][12] + coeff0_3*dmats0[3][12] + coeff0_4*dmats0[4][12] + coeff0_5*dmats0[5][12] + coeff0_6*dmats0[6][12] + coeff0_7*dmats0[7][12] + coeff0_8*dmats0[8][12] + coeff0_9*dmats0[9][12] + coeff0_10*dmats0[10][12] + coeff0_11*dmats0[11][12] + coeff0_12*dmats0[12][12] + coeff0_13*dmats0[13][12] + coeff0_14*dmats0[14][12] + coeff0_15*dmats0[15][12] + coeff0_16*dmats0[16][12] + coeff0_17*dmats0[17][12] + coeff0_18*dmats0[18][12] + coeff0_19*dmats0[19][12] + coeff0_20*dmats0[20][12] + coeff0_21*dmats0[21][12] + coeff0_22*dmats0[22][12] + coeff0_23*dmats0[23][12] + coeff0_24*dmats0[24][12] + coeff0_25*dmats0[25][12] + coeff0_26*dmats0[26][12] + coeff0_27*dmats0[27][12] + coeff0_28*dmats0[28][12] + coeff0_29*dmats0[29][12] + coeff0_30*dmats0[30][12] + coeff0_31*dmats0[31][12] + coeff0_32*dmats0[32][12] + coeff0_33*dmats0[33][12] + coeff0_34*dmats0[34][12] + coeff0_35*dmats0[35][12] + coeff0_36*dmats0[36][12] + coeff0_37*dmats0[37][12] + coeff0_38*dmats0[38][12] + coeff0_39*dmats0[39][12] + coeff0_40*dmats0[40][12] + coeff0_41*dmats0[41][12] + coeff0_42*dmats0[42][12] + coeff0_43*dmats0[43][12] + coeff0_44*dmats0[44][12] + coeff0_45*dmats0[45][12] + coeff0_46*dmats0[46][12] + coeff0_47*dmats0[47][12] + coeff0_48*dmats0[48][12] + coeff0_49*dmats0[49][12] + coeff0_50*dmats0[50][12] + coeff0_51*dmats0[51][12] + coeff0_52*dmats0[52][12] + coeff0_53*dmats0[53][12] + coeff0_54*dmats0[54][12] + coeff0_55*dmats0[55][12];
          new_coeff0_13 = coeff0_0*dmats0[0][13] + coeff0_1*dmats0[1][13] + coeff0_2*dmats0[2][13] + coeff0_3*dmats0[3][13] + coeff0_4*dmats0[4][13] + coeff0_5*dmats0[5][13] + coeff0_6*dmats0[6][13] + coeff0_7*dmats0[7][13] + coeff0_8*dmats0[8][13] + coeff0_9*dmats0[9][13] + coeff0_10*dmats0[10][13] + coeff0_11*dmats0[11][13] + coeff0_12*dmats0[12][13] + coeff0_13*dmats0[13][13] + coeff0_14*dmats0[14][13] + coeff0_15*dmats0[15][13] + coeff0_16*dmats0[16][13] + coeff0_17*dmats0[17][13] + coeff0_18*dmats0[18][13] + coeff0_19*dmats0[19][13] + coeff0_20*dmats0[20][13] + coeff0_21*dmats0[21][13] + coeff0_22*dmats0[22][13] + coeff0_23*dmats0[23][13] + coeff0_24*dmats0[24][13] + coeff0_25*dmats0[25][13] + coeff0_26*dmats0[26][13] + coeff0_27*dmats0[27][13] + coeff0_28*dmats0[28][13] + coeff0_29*dmats0[29][13] + coeff0_30*dmats0[30][13] + coeff0_31*dmats0[31][13] + coeff0_32*dmats0[32][13] + coeff0_33*dmats0[33][13] + coeff0_34*dmats0[34][13] + coeff0_35*dmats0[35][13] + coeff0_36*dmats0[36][13] + coeff0_37*dmats0[37][13] + coeff0_38*dmats0[38][13] + coeff0_39*dmats0[39][13] + coeff0_40*dmats0[40][13] + coeff0_41*dmats0[41][13] + coeff0_42*dmats0[42][13] + coeff0_43*dmats0[43][13] + coeff0_44*dmats0[44][13] + coeff0_45*dmats0[45][13] + coeff0_46*dmats0[46][13] + coeff0_47*dmats0[47][13] + coeff0_48*dmats0[48][13] + coeff0_49*dmats0[49][13] + coeff0_50*dmats0[50][13] + coeff0_51*dmats0[51][13] + coeff0_52*dmats0[52][13] + coeff0_53*dmats0[53][13] + coeff0_54*dmats0[54][13] + coeff0_55*dmats0[55][13];
          new_coeff0_14 = coeff0_0*dmats0[0][14] + coeff0_1*dmats0[1][14] + coeff0_2*dmats0[2][14] + coeff0_3*dmats0[3][14] + coeff0_4*dmats0[4][14] + coeff0_5*dmats0[5][14] + coeff0_6*dmats0[6][14] + coeff0_7*dmats0[7][14] + coeff0_8*dmats0[8][14] + coeff0_9*dmats0[9][14] + coeff0_10*dmats0[10][14] + coeff0_11*dmats0[11][14] + coeff0_12*dmats0[12][14] + coeff0_13*dmats0[13][14] + coeff0_14*dmats0[14][14] + coeff0_15*dmats0[15][14] + coeff0_16*dmats0[16][14] + coeff0_17*dmats0[17][14] + coeff0_18*dmats0[18][14] + coeff0_19*dmats0[19][14] + coeff0_20*dmats0[20][14] + coeff0_21*dmats0[21][14] + coeff0_22*dmats0[22][14] + coeff0_23*dmats0[23][14] + coeff0_24*dmats0[24][14] + coeff0_25*dmats0[25][14] + coeff0_26*dmats0[26][14] + coeff0_27*dmats0[27][14] + coeff0_28*dmats0[28][14] + coeff0_29*dmats0[29][14] + coeff0_30*dmats0[30][14] + coeff0_31*dmats0[31][14] + coeff0_32*dmats0[32][14] + coeff0_33*dmats0[33][14] + coeff0_34*dmats0[34][14] + coeff0_35*dmats0[35][14] + coeff0_36*dmats0[36][14] + coeff0_37*dmats0[37][14] + coeff0_38*dmats0[38][14] + coeff0_39*dmats0[39][14] + coeff0_40*dmats0[40][14] + coeff0_41*dmats0[41][14] + coeff0_42*dmats0[42][14] + coeff0_43*dmats0[43][14] + coeff0_44*dmats0[44][14] + coeff0_45*dmats0[45][14] + coeff0_46*dmats0[46][14] + coeff0_47*dmats0[47][14] + coeff0_48*dmats0[48][14] + coeff0_49*dmats0[49][14] + coeff0_50*dmats0[50][14] + coeff0_51*dmats0[51][14] + coeff0_52*dmats0[52][14] + coeff0_53*dmats0[53][14] + coeff0_54*dmats0[54][14] + coeff0_55*dmats0[55][14];
          new_coeff0_15 = coeff0_0*dmats0[0][15] + coeff0_1*dmats0[1][15] + coeff0_2*dmats0[2][15] + coeff0_3*dmats0[3][15] + coeff0_4*dmats0[4][15] + coeff0_5*dmats0[5][15] + coeff0_6*dmats0[6][15] + coeff0_7*dmats0[7][15] + coeff0_8*dmats0[8][15] + coeff0_9*dmats0[9][15] + coeff0_10*dmats0[10][15] + coeff0_11*dmats0[11][15] + coeff0_12*dmats0[12][15] + coeff0_13*dmats0[13][15] + coeff0_14*dmats0[14][15] + coeff0_15*dmats0[15][15] + coeff0_16*dmats0[16][15] + coeff0_17*dmats0[17][15] + coeff0_18*dmats0[18][15] + coeff0_19*dmats0[19][15] + coeff0_20*dmats0[20][15] + coeff0_21*dmats0[21][15] + coeff0_22*dmats0[22][15] + coeff0_23*dmats0[23][15] + coeff0_24*dmats0[24][15] + coeff0_25*dmats0[25][15] + coeff0_26*dmats0[26][15] + coeff0_27*dmats0[27][15] + coeff0_28*dmats0[28][15] + coeff0_29*dmats0[29][15] + coeff0_30*dmats0[30][15] + coeff0_31*dmats0[31][15] + coeff0_32*dmats0[32][15] + coeff0_33*dmats0[33][15] + coeff0_34*dmats0[34][15] + coeff0_35*dmats0[35][15] + coeff0_36*dmats0[36][15] + coeff0_37*dmats0[37][15] + coeff0_38*dmats0[38][15] + coeff0_39*dmats0[39][15] + coeff0_40*dmats0[40][15] + coeff0_41*dmats0[41][15] + coeff0_42*dmats0[42][15] + coeff0_43*dmats0[43][15] + coeff0_44*dmats0[44][15] + coeff0_45*dmats0[45][15] + coeff0_46*dmats0[46][15] + coeff0_47*dmats0[47][15] + coeff0_48*dmats0[48][15] + coeff0_49*dmats0[49][15] + coeff0_50*dmats0[50][15] + coeff0_51*dmats0[51][15] + coeff0_52*dmats0[52][15] + coeff0_53*dmats0[53][15] + coeff0_54*dmats0[54][15] + coeff0_55*dmats0[55][15];
          new_coeff0_16 = coeff0_0*dmats0[0][16] + coeff0_1*dmats0[1][16] + coeff0_2*dmats0[2][16] + coeff0_3*dmats0[3][16] + coeff0_4*dmats0[4][16] + coeff0_5*dmats0[5][16] + coeff0_6*dmats0[6][16] + coeff0_7*dmats0[7][16] + coeff0_8*dmats0[8][16] + coeff0_9*dmats0[9][16] + coeff0_10*dmats0[10][16] + coeff0_11*dmats0[11][16] + coeff0_12*dmats0[12][16] + coeff0_13*dmats0[13][16] + coeff0_14*dmats0[14][16] + coeff0_15*dmats0[15][16] + coeff0_16*dmats0[16][16] + coeff0_17*dmats0[17][16] + coeff0_18*dmats0[18][16] + coeff0_19*dmats0[19][16] + coeff0_20*dmats0[20][16] + coeff0_21*dmats0[21][16] + coeff0_22*dmats0[22][16] + coeff0_23*dmats0[23][16] + coeff0_24*dmats0[24][16] + coeff0_25*dmats0[25][16] + coeff0_26*dmats0[26][16] + coeff0_27*dmats0[27][16] + coeff0_28*dmats0[28][16] + coeff0_29*dmats0[29][16] + coeff0_30*dmats0[30][16] + coeff0_31*dmats0[31][16] + coeff0_32*dmats0[32][16] + coeff0_33*dmats0[33][16] + coeff0_34*dmats0[34][16] + coeff0_35*dmats0[35][16] + coeff0_36*dmats0[36][16] + coeff0_37*dmats0[37][16] + coeff0_38*dmats0[38][16] + coeff0_39*dmats0[39][16] + coeff0_40*dmats0[40][16] + coeff0_41*dmats0[41][16] + coeff0_42*dmats0[42][16] + coeff0_43*dmats0[43][16] + coeff0_44*dmats0[44][16] + coeff0_45*dmats0[45][16] + coeff0_46*dmats0[46][16] + coeff0_47*dmats0[47][16] + coeff0_48*dmats0[48][16] + coeff0_49*dmats0[49][16] + coeff0_50*dmats0[50][16] + coeff0_51*dmats0[51][16] + coeff0_52*dmats0[52][16] + coeff0_53*dmats0[53][16] + coeff0_54*dmats0[54][16] + coeff0_55*dmats0[55][16];
          new_coeff0_17 = coeff0_0*dmats0[0][17] + coeff0_1*dmats0[1][17] + coeff0_2*dmats0[2][17] + coeff0_3*dmats0[3][17] + coeff0_4*dmats0[4][17] + coeff0_5*dmats0[5][17] + coeff0_6*dmats0[6][17] + coeff0_7*dmats0[7][17] + coeff0_8*dmats0[8][17] + coeff0_9*dmats0[9][17] + coeff0_10*dmats0[10][17] + coeff0_11*dmats0[11][17] + coeff0_12*dmats0[12][17] + coeff0_13*dmats0[13][17] + coeff0_14*dmats0[14][17] + coeff0_15*dmats0[15][17] + coeff0_16*dmats0[16][17] + coeff0_17*dmats0[17][17] + coeff0_18*dmats0[18][17] + coeff0_19*dmats0[19][17] + coeff0_20*dmats0[20][17] + coeff0_21*dmats0[21][17] + coeff0_22*dmats0[22][17] + coeff0_23*dmats0[23][17] + coeff0_24*dmats0[24][17] + coeff0_25*dmats0[25][17] + coeff0_26*dmats0[26][17] + coeff0_27*dmats0[27][17] + coeff0_28*dmats0[28][17] + coeff0_29*dmats0[29][17] + coeff0_30*dmats0[30][17] + coeff0_31*dmats0[31][17] + coeff0_32*dmats0[32][17] + coeff0_33*dmats0[33][17] + coeff0_34*dmats0[34][17] + coeff0_35*dmats0[35][17] + coeff0_36*dmats0[36][17] + coeff0_37*dmats0[37][17] + coeff0_38*dmats0[38][17] + coeff0_39*dmats0[39][17] + coeff0_40*dmats0[40][17] + coeff0_41*dmats0[41][17] + coeff0_42*dmats0[42][17] + coeff0_43*dmats0[43][17] + coeff0_44*dmats0[44][17] + coeff0_45*dmats0[45][17] + coeff0_46*dmats0[46][17] + coeff0_47*dmats0[47][17] + coeff0_48*dmats0[48][17] + coeff0_49*dmats0[49][17] + coeff0_50*dmats0[50][17] + coeff0_51*dmats0[51][17] + coeff0_52*dmats0[52][17] + coeff0_53*dmats0[53][17] + coeff0_54*dmats0[54][17] + coeff0_55*dmats0[55][17];
          new_coeff0_18 = coeff0_0*dmats0[0][18] + coeff0_1*dmats0[1][18] + coeff0_2*dmats0[2][18] + coeff0_3*dmats0[3][18] + coeff0_4*dmats0[4][18] + coeff0_5*dmats0[5][18] + coeff0_6*dmats0[6][18] + coeff0_7*dmats0[7][18] + coeff0_8*dmats0[8][18] + coeff0_9*dmats0[9][18] + coeff0_10*dmats0[10][18] + coeff0_11*dmats0[11][18] + coeff0_12*dmats0[12][18] + coeff0_13*dmats0[13][18] + coeff0_14*dmats0[14][18] + coeff0_15*dmats0[15][18] + coeff0_16*dmats0[16][18] + coeff0_17*dmats0[17][18] + coeff0_18*dmats0[18][18] + coeff0_19*dmats0[19][18] + coeff0_20*dmats0[20][18] + coeff0_21*dmats0[21][18] + coeff0_22*dmats0[22][18] + coeff0_23*dmats0[23][18] + coeff0_24*dmats0[24][18] + coeff0_25*dmats0[25][18] + coeff0_26*dmats0[26][18] + coeff0_27*dmats0[27][18] + coeff0_28*dmats0[28][18] + coeff0_29*dmats0[29][18] + coeff0_30*dmats0[30][18] + coeff0_31*dmats0[31][18] + coeff0_32*dmats0[32][18] + coeff0_33*dmats0[33][18] + coeff0_34*dmats0[34][18] + coeff0_35*dmats0[35][18] + coeff0_36*dmats0[36][18] + coeff0_37*dmats0[37][18] + coeff0_38*dmats0[38][18] + coeff0_39*dmats0[39][18] + coeff0_40*dmats0[40][18] + coeff0_41*dmats0[41][18] + coeff0_42*dmats0[42][18] + coeff0_43*dmats0[43][18] + coeff0_44*dmats0[44][18] + coeff0_45*dmats0[45][18] + coeff0_46*dmats0[46][18] + coeff0_47*dmats0[47][18] + coeff0_48*dmats0[48][18] + coeff0_49*dmats0[49][18] + coeff0_50*dmats0[50][18] + coeff0_51*dmats0[51][18] + coeff0_52*dmats0[52][18] + coeff0_53*dmats0[53][18] + coeff0_54*dmats0[54][18] + coeff0_55*dmats0[55][18];
          new_coeff0_19 = coeff0_0*dmats0[0][19] + coeff0_1*dmats0[1][19] + coeff0_2*dmats0[2][19] + coeff0_3*dmats0[3][19] + coeff0_4*dmats0[4][19] + coeff0_5*dmats0[5][19] + coeff0_6*dmats0[6][19] + coeff0_7*dmats0[7][19] + coeff0_8*dmats0[8][19] + coeff0_9*dmats0[9][19] + coeff0_10*dmats0[10][19] + coeff0_11*dmats0[11][19] + coeff0_12*dmats0[12][19] + coeff0_13*dmats0[13][19] + coeff0_14*dmats0[14][19] + coeff0_15*dmats0[15][19] + coeff0_16*dmats0[16][19] + coeff0_17*dmats0[17][19] + coeff0_18*dmats0[18][19] + coeff0_19*dmats0[19][19] + coeff0_20*dmats0[20][19] + coeff0_21*dmats0[21][19] + coeff0_22*dmats0[22][19] + coeff0_23*dmats0[23][19] + coeff0_24*dmats0[24][19] + coeff0_25*dmats0[25][19] + coeff0_26*dmats0[26][19] + coeff0_27*dmats0[27][19] + coeff0_28*dmats0[28][19] + coeff0_29*dmats0[29][19] + coeff0_30*dmats0[30][19] + coeff0_31*dmats0[31][19] + coeff0_32*dmats0[32][19] + coeff0_33*dmats0[33][19] + coeff0_34*dmats0[34][19] + coeff0_35*dmats0[35][19] + coeff0_36*dmats0[36][19] + coeff0_37*dmats0[37][19] + coeff0_38*dmats0[38][19] + coeff0_39*dmats0[39][19] + coeff0_40*dmats0[40][19] + coeff0_41*dmats0[41][19] + coeff0_42*dmats0[42][19] + coeff0_43*dmats0[43][19] + coeff0_44*dmats0[44][19] + coeff0_45*dmats0[45][19] + coeff0_46*dmats0[46][19] + coeff0_47*dmats0[47][19] + coeff0_48*dmats0[48][19] + coeff0_49*dmats0[49][19] + coeff0_50*dmats0[50][19] + coeff0_51*dmats0[51][19] + coeff0_52*dmats0[52][19] + coeff0_53*dmats0[53][19] + coeff0_54*dmats0[54][19] + coeff0_55*dmats0[55][19];
          new_coeff0_20 = coeff0_0*dmats0[0][20] + coeff0_1*dmats0[1][20] + coeff0_2*dmats0[2][20] + coeff0_3*dmats0[3][20] + coeff0_4*dmats0[4][20] + coeff0_5*dmats0[5][20] + coeff0_6*dmats0[6][20] + coeff0_7*dmats0[7][20] + coeff0_8*dmats0[8][20] + coeff0_9*dmats0[9][20] + coeff0_10*dmats0[10][20] + coeff0_11*dmats0[11][20] + coeff0_12*dmats0[12][20] + coeff0_13*dmats0[13][20] + coeff0_14*dmats0[14][20] + coeff0_15*dmats0[15][20] + coeff0_16*dmats0[16][20] + coeff0_17*dmats0[17][20] + coeff0_18*dmats0[18][20] + coeff0_19*dmats0[19][20] + coeff0_20*dmats0[20][20] + coeff0_21*dmats0[21][20] + coeff0_22*dmats0[22][20] + coeff0_23*dmats0[23][20] + coeff0_24*dmats0[24][20] + coeff0_25*dmats0[25][20] + coeff0_26*dmats0[26][20] + coeff0_27*dmats0[27][20] + coeff0_28*dmats0[28][20] + coeff0_29*dmats0[29][20] + coeff0_30*dmats0[30][20] + coeff0_31*dmats0[31][20] + coeff0_32*dmats0[32][20] + coeff0_33*dmats0[33][20] + coeff0_34*dmats0[34][20] + coeff0_35*dmats0[35][20] + coeff0_36*dmats0[36][20] + coeff0_37*dmats0[37][20] + coeff0_38*dmats0[38][20] + coeff0_39*dmats0[39][20] + coeff0_40*dmats0[40][20] + coeff0_41*dmats0[41][20] + coeff0_42*dmats0[42][20] + coeff0_43*dmats0[43][20] + coeff0_44*dmats0[44][20] + coeff0_45*dmats0[45][20] + coeff0_46*dmats0[46][20] + coeff0_47*dmats0[47][20] + coeff0_48*dmats0[48][20] + coeff0_49*dmats0[49][20] + coeff0_50*dmats0[50][20] + coeff0_51*dmats0[51][20] + coeff0_52*dmats0[52][20] + coeff0_53*dmats0[53][20] + coeff0_54*dmats0[54][20] + coeff0_55*dmats0[55][20];
          new_coeff0_21 = coeff0_0*dmats0[0][21] + coeff0_1*dmats0[1][21] + coeff0_2*dmats0[2][21] + coeff0_3*dmats0[3][21] + coeff0_4*dmats0[4][21] + coeff0_5*dmats0[5][21] + coeff0_6*dmats0[6][21] + coeff0_7*dmats0[7][21] + coeff0_8*dmats0[8][21] + coeff0_9*dmats0[9][21] + coeff0_10*dmats0[10][21] + coeff0_11*dmats0[11][21] + coeff0_12*dmats0[12][21] + coeff0_13*dmats0[13][21] + coeff0_14*dmats0[14][21] + coeff0_15*dmats0[15][21] + coeff0_16*dmats0[16][21] + coeff0_17*dmats0[17][21] + coeff0_18*dmats0[18][21] + coeff0_19*dmats0[19][21] + coeff0_20*dmats0[20][21] + coeff0_21*dmats0[21][21] + coeff0_22*dmats0[22][21] + coeff0_23*dmats0[23][21] + coeff0_24*dmats0[24][21] + coeff0_25*dmats0[25][21] + coeff0_26*dmats0[26][21] + coeff0_27*dmats0[27][21] + coeff0_28*dmats0[28][21] + coeff0_29*dmats0[29][21] + coeff0_30*dmats0[30][21] + coeff0_31*dmats0[31][21] + coeff0_32*dmats0[32][21] + coeff0_33*dmats0[33][21] + coeff0_34*dmats0[34][21] + coeff0_35*dmats0[35][21] + coeff0_36*dmats0[36][21] + coeff0_37*dmats0[37][21] + coeff0_38*dmats0[38][21] + coeff0_39*dmats0[39][21] + coeff0_40*dmats0[40][21] + coeff0_41*dmats0[41][21] + coeff0_42*dmats0[42][21] + coeff0_43*dmats0[43][21] + coeff0_44*dmats0[44][21] + coeff0_45*dmats0[45][21] + coeff0_46*dmats0[46][21] + coeff0_47*dmats0[47][21] + coeff0_48*dmats0[48][21] + coeff0_49*dmats0[49][21] + coeff0_50*dmats0[50][21] + coeff0_51*dmats0[51][21] + coeff0_52*dmats0[52][21] + coeff0_53*dmats0[53][21] + coeff0_54*dmats0[54][21] + coeff0_55*dmats0[55][21];
          new_coeff0_22 = coeff0_0*dmats0[0][22] + coeff0_1*dmats0[1][22] + coeff0_2*dmats0[2][22] + coeff0_3*dmats0[3][22] + coeff0_4*dmats0[4][22] + coeff0_5*dmats0[5][22] + coeff0_6*dmats0[6][22] + coeff0_7*dmats0[7][22] + coeff0_8*dmats0[8][22] + coeff0_9*dmats0[9][22] + coeff0_10*dmats0[10][22] + coeff0_11*dmats0[11][22] + coeff0_12*dmats0[12][22] + coeff0_13*dmats0[13][22] + coeff0_14*dmats0[14][22] + coeff0_15*dmats0[15][22] + coeff0_16*dmats0[16][22] + coeff0_17*dmats0[17][22] + coeff0_18*dmats0[18][22] + coeff0_19*dmats0[19][22] + coeff0_20*dmats0[20][22] + coeff0_21*dmats0[21][22] + coeff0_22*dmats0[22][22] + coeff0_23*dmats0[23][22] + coeff0_24*dmats0[24][22] + coeff0_25*dmats0[25][22] + coeff0_26*dmats0[26][22] + coeff0_27*dmats0[27][22] + coeff0_28*dmats0[28][22] + coeff0_29*dmats0[29][22] + coeff0_30*dmats0[30][22] + coeff0_31*dmats0[31][22] + coeff0_32*dmats0[32][22] + coeff0_33*dmats0[33][22] + coeff0_34*dmats0[34][22] + coeff0_35*dmats0[35][22] + coeff0_36*dmats0[36][22] + coeff0_37*dmats0[37][22] + coeff0_38*dmats0[38][22] + coeff0_39*dmats0[39][22] + coeff0_40*dmats0[40][22] + coeff0_41*dmats0[41][22] + coeff0_42*dmats0[42][22] + coeff0_43*dmats0[43][22] + coeff0_44*dmats0[44][22] + coeff0_45*dmats0[45][22] + coeff0_46*dmats0[46][22] + coeff0_47*dmats0[47][22] + coeff0_48*dmats0[48][22] + coeff0_49*dmats0[49][22] + coeff0_50*dmats0[50][22] + coeff0_51*dmats0[51][22] + coeff0_52*dmats0[52][22] + coeff0_53*dmats0[53][22] + coeff0_54*dmats0[54][22] + coeff0_55*dmats0[55][22];
          new_coeff0_23 = coeff0_0*dmats0[0][23] + coeff0_1*dmats0[1][23] + coeff0_2*dmats0[2][23] + coeff0_3*dmats0[3][23] + coeff0_4*dmats0[4][23] + coeff0_5*dmats0[5][23] + coeff0_6*dmats0[6][23] + coeff0_7*dmats0[7][23] + coeff0_8*dmats0[8][23] + coeff0_9*dmats0[9][23] + coeff0_10*dmats0[10][23] + coeff0_11*dmats0[11][23] + coeff0_12*dmats0[12][23] + coeff0_13*dmats0[13][23] + coeff0_14*dmats0[14][23] + coeff0_15*dmats0[15][23] + coeff0_16*dmats0[16][23] + coeff0_17*dmats0[17][23] + coeff0_18*dmats0[18][23] + coeff0_19*dmats0[19][23] + coeff0_20*dmats0[20][23] + coeff0_21*dmats0[21][23] + coeff0_22*dmats0[22][23] + coeff0_23*dmats0[23][23] + coeff0_24*dmats0[24][23] + coeff0_25*dmats0[25][23] + coeff0_26*dmats0[26][23] + coeff0_27*dmats0[27][23] + coeff0_28*dmats0[28][23] + coeff0_29*dmats0[29][23] + coeff0_30*dmats0[30][23] + coeff0_31*dmats0[31][23] + coeff0_32*dmats0[32][23] + coeff0_33*dmats0[33][23] + coeff0_34*dmats0[34][23] + coeff0_35*dmats0[35][23] + coeff0_36*dmats0[36][23] + coeff0_37*dmats0[37][23] + coeff0_38*dmats0[38][23] + coeff0_39*dmats0[39][23] + coeff0_40*dmats0[40][23] + coeff0_41*dmats0[41][23] + coeff0_42*dmats0[42][23] + coeff0_43*dmats0[43][23] + coeff0_44*dmats0[44][23] + coeff0_45*dmats0[45][23] + coeff0_46*dmats0[46][23] + coeff0_47*dmats0[47][23] + coeff0_48*dmats0[48][23] + coeff0_49*dmats0[49][23] + coeff0_50*dmats0[50][23] + coeff0_51*dmats0[51][23] + coeff0_52*dmats0[52][23] + coeff0_53*dmats0[53][23] + coeff0_54*dmats0[54][23] + coeff0_55*dmats0[55][23];
          new_coeff0_24 = coeff0_0*dmats0[0][24] + coeff0_1*dmats0[1][24] + coeff0_2*dmats0[2][24] + coeff0_3*dmats0[3][24] + coeff0_4*dmats0[4][24] + coeff0_5*dmats0[5][24] + coeff0_6*dmats0[6][24] + coeff0_7*dmats0[7][24] + coeff0_8*dmats0[8][24] + coeff0_9*dmats0[9][24] + coeff0_10*dmats0[10][24] + coeff0_11*dmats0[11][24] + coeff0_12*dmats0[12][24] + coeff0_13*dmats0[13][24] + coeff0_14*dmats0[14][24] + coeff0_15*dmats0[15][24] + coeff0_16*dmats0[16][24] + coeff0_17*dmats0[17][24] + coeff0_18*dmats0[18][24] + coeff0_19*dmats0[19][24] + coeff0_20*dmats0[20][24] + coeff0_21*dmats0[21][24] + coeff0_22*dmats0[22][24] + coeff0_23*dmats0[23][24] + coeff0_24*dmats0[24][24] + coeff0_25*dmats0[25][24] + coeff0_26*dmats0[26][24] + coeff0_27*dmats0[27][24] + coeff0_28*dmats0[28][24] + coeff0_29*dmats0[29][24] + coeff0_30*dmats0[30][24] + coeff0_31*dmats0[31][24] + coeff0_32*dmats0[32][24] + coeff0_33*dmats0[33][24] + coeff0_34*dmats0[34][24] + coeff0_35*dmats0[35][24] + coeff0_36*dmats0[36][24] + coeff0_37*dmats0[37][24] + coeff0_38*dmats0[38][24] + coeff0_39*dmats0[39][24] + coeff0_40*dmats0[40][24] + coeff0_41*dmats0[41][24] + coeff0_42*dmats0[42][24] + coeff0_43*dmats0[43][24] + coeff0_44*dmats0[44][24] + coeff0_45*dmats0[45][24] + coeff0_46*dmats0[46][24] + coeff0_47*dmats0[47][24] + coeff0_48*dmats0[48][24] + coeff0_49*dmats0[49][24] + coeff0_50*dmats0[50][24] + coeff0_51*dmats0[51][24] + coeff0_52*dmats0[52][24] + coeff0_53*dmats0[53][24] + coeff0_54*dmats0[54][24] + coeff0_55*dmats0[55][24];
          new_coeff0_25 = coeff0_0*dmats0[0][25] + coeff0_1*dmats0[1][25] + coeff0_2*dmats0[2][25] + coeff0_3*dmats0[3][25] + coeff0_4*dmats0[4][25] + coeff0_5*dmats0[5][25] + coeff0_6*dmats0[6][25] + coeff0_7*dmats0[7][25] + coeff0_8*dmats0[8][25] + coeff0_9*dmats0[9][25] + coeff0_10*dmats0[10][25] + coeff0_11*dmats0[11][25] + coeff0_12*dmats0[12][25] + coeff0_13*dmats0[13][25] + coeff0_14*dmats0[14][25] + coeff0_15*dmats0[15][25] + coeff0_16*dmats0[16][25] + coeff0_17*dmats0[17][25] + coeff0_18*dmats0[18][25] + coeff0_19*dmats0[19][25] + coeff0_20*dmats0[20][25] + coeff0_21*dmats0[21][25] + coeff0_22*dmats0[22][25] + coeff0_23*dmats0[23][25] + coeff0_24*dmats0[24][25] + coeff0_25*dmats0[25][25] + coeff0_26*dmats0[26][25] + coeff0_27*dmats0[27][25] + coeff0_28*dmats0[28][25] + coeff0_29*dmats0[29][25] + coeff0_30*dmats0[30][25] + coeff0_31*dmats0[31][25] + coeff0_32*dmats0[32][25] + coeff0_33*dmats0[33][25] + coeff0_34*dmats0[34][25] + coeff0_35*dmats0[35][25] + coeff0_36*dmats0[36][25] + coeff0_37*dmats0[37][25] + coeff0_38*dmats0[38][25] + coeff0_39*dmats0[39][25] + coeff0_40*dmats0[40][25] + coeff0_41*dmats0[41][25] + coeff0_42*dmats0[42][25] + coeff0_43*dmats0[43][25] + coeff0_44*dmats0[44][25] + coeff0_45*dmats0[45][25] + coeff0_46*dmats0[46][25] + coeff0_47*dmats0[47][25] + coeff0_48*dmats0[48][25] + coeff0_49*dmats0[49][25] + coeff0_50*dmats0[50][25] + coeff0_51*dmats0[51][25] + coeff0_52*dmats0[52][25] + coeff0_53*dmats0[53][25] + coeff0_54*dmats0[54][25] + coeff0_55*dmats0[55][25];
          new_coeff0_26 = coeff0_0*dmats0[0][26] + coeff0_1*dmats0[1][26] + coeff0_2*dmats0[2][26] + coeff0_3*dmats0[3][26] + coeff0_4*dmats0[4][26] + coeff0_5*dmats0[5][26] + coeff0_6*dmats0[6][26] + coeff0_7*dmats0[7][26] + coeff0_8*dmats0[8][26] + coeff0_9*dmats0[9][26] + coeff0_10*dmats0[10][26] + coeff0_11*dmats0[11][26] + coeff0_12*dmats0[12][26] + coeff0_13*dmats0[13][26] + coeff0_14*dmats0[14][26] + coeff0_15*dmats0[15][26] + coeff0_16*dmats0[16][26] + coeff0_17*dmats0[17][26] + coeff0_18*dmats0[18][26] + coeff0_19*dmats0[19][26] + coeff0_20*dmats0[20][26] + coeff0_21*dmats0[21][26] + coeff0_22*dmats0[22][26] + coeff0_23*dmats0[23][26] + coeff0_24*dmats0[24][26] + coeff0_25*dmats0[25][26] + coeff0_26*dmats0[26][26] + coeff0_27*dmats0[27][26] + coeff0_28*dmats0[28][26] + coeff0_29*dmats0[29][26] + coeff0_30*dmats0[30][26] + coeff0_31*dmats0[31][26] + coeff0_32*dmats0[32][26] + coeff0_33*dmats0[33][26] + coeff0_34*dmats0[34][26] + coeff0_35*dmats0[35][26] + coeff0_36*dmats0[36][26] + coeff0_37*dmats0[37][26] + coeff0_38*dmats0[38][26] + coeff0_39*dmats0[39][26] + coeff0_40*dmats0[40][26] + coeff0_41*dmats0[41][26] + coeff0_42*dmats0[42][26] + coeff0_43*dmats0[43][26] + coeff0_44*dmats0[44][26] + coeff0_45*dmats0[45][26] + coeff0_46*dmats0[46][26] + coeff0_47*dmats0[47][26] + coeff0_48*dmats0[48][26] + coeff0_49*dmats0[49][26] + coeff0_50*dmats0[50][26] + coeff0_51*dmats0[51][26] + coeff0_52*dmats0[52][26] + coeff0_53*dmats0[53][26] + coeff0_54*dmats0[54][26] + coeff0_55*dmats0[55][26];
          new_coeff0_27 = coeff0_0*dmats0[0][27] + coeff0_1*dmats0[1][27] + coeff0_2*dmats0[2][27] + coeff0_3*dmats0[3][27] + coeff0_4*dmats0[4][27] + coeff0_5*dmats0[5][27] + coeff0_6*dmats0[6][27] + coeff0_7*dmats0[7][27] + coeff0_8*dmats0[8][27] + coeff0_9*dmats0[9][27] + coeff0_10*dmats0[10][27] + coeff0_11*dmats0[11][27] + coeff0_12*dmats0[12][27] + coeff0_13*dmats0[13][27] + coeff0_14*dmats0[14][27] + coeff0_15*dmats0[15][27] + coeff0_16*dmats0[16][27] + coeff0_17*dmats0[17][27] + coeff0_18*dmats0[18][27] + coeff0_19*dmats0[19][27] + coeff0_20*dmats0[20][27] + coeff0_21*dmats0[21][27] + coeff0_22*dmats0[22][27] + coeff0_23*dmats0[23][27] + coeff0_24*dmats0[24][27] + coeff0_25*dmats0[25][27] + coeff0_26*dmats0[26][27] + coeff0_27*dmats0[27][27] + coeff0_28*dmats0[28][27] + coeff0_29*dmats0[29][27] + coeff0_30*dmats0[30][27] + coeff0_31*dmats0[31][27] + coeff0_32*dmats0[32][27] + coeff0_33*dmats0[33][27] + coeff0_34*dmats0[34][27] + coeff0_35*dmats0[35][27] + coeff0_36*dmats0[36][27] + coeff0_37*dmats0[37][27] + coeff0_38*dmats0[38][27] + coeff0_39*dmats0[39][27] + coeff0_40*dmats0[40][27] + coeff0_41*dmats0[41][27] + coeff0_42*dmats0[42][27] + coeff0_43*dmats0[43][27] + coeff0_44*dmats0[44][27] + coeff0_45*dmats0[45][27] + coeff0_46*dmats0[46][27] + coeff0_47*dmats0[47][27] + coeff0_48*dmats0[48][27] + coeff0_49*dmats0[49][27] + coeff0_50*dmats0[50][27] + coeff0_51*dmats0[51][27] + coeff0_52*dmats0[52][27] + coeff0_53*dmats0[53][27] + coeff0_54*dmats0[54][27] + coeff0_55*dmats0[55][27];
          new_coeff0_28 = coeff0_0*dmats0[0][28] + coeff0_1*dmats0[1][28] + coeff0_2*dmats0[2][28] + coeff0_3*dmats0[3][28] + coeff0_4*dmats0[4][28] + coeff0_5*dmats0[5][28] + coeff0_6*dmats0[6][28] + coeff0_7*dmats0[7][28] + coeff0_8*dmats0[8][28] + coeff0_9*dmats0[9][28] + coeff0_10*dmats0[10][28] + coeff0_11*dmats0[11][28] + coeff0_12*dmats0[12][28] + coeff0_13*dmats0[13][28] + coeff0_14*dmats0[14][28] + coeff0_15*dmats0[15][28] + coeff0_16*dmats0[16][28] + coeff0_17*dmats0[17][28] + coeff0_18*dmats0[18][28] + coeff0_19*dmats0[19][28] + coeff0_20*dmats0[20][28] + coeff0_21*dmats0[21][28] + coeff0_22*dmats0[22][28] + coeff0_23*dmats0[23][28] + coeff0_24*dmats0[24][28] + coeff0_25*dmats0[25][28] + coeff0_26*dmats0[26][28] + coeff0_27*dmats0[27][28] + coeff0_28*dmats0[28][28] + coeff0_29*dmats0[29][28] + coeff0_30*dmats0[30][28] + coeff0_31*dmats0[31][28] + coeff0_32*dmats0[32][28] + coeff0_33*dmats0[33][28] + coeff0_34*dmats0[34][28] + coeff0_35*dmats0[35][28] + coeff0_36*dmats0[36][28] + coeff0_37*dmats0[37][28] + coeff0_38*dmats0[38][28] + coeff0_39*dmats0[39][28] + coeff0_40*dmats0[40][28] + coeff0_41*dmats0[41][28] + coeff0_42*dmats0[42][28] + coeff0_43*dmats0[43][28] + coeff0_44*dmats0[44][28] + coeff0_45*dmats0[45][28] + coeff0_46*dmats0[46][28] + coeff0_47*dmats0[47][28] + coeff0_48*dmats0[48][28] + coeff0_49*dmats0[49][28] + coeff0_50*dmats0[50][28] + coeff0_51*dmats0[51][28] + coeff0_52*dmats0[52][28] + coeff0_53*dmats0[53][28] + coeff0_54*dmats0[54][28] + coeff0_55*dmats0[55][28];
          new_coeff0_29 = coeff0_0*dmats0[0][29] + coeff0_1*dmats0[1][29] + coeff0_2*dmats0[2][29] + coeff0_3*dmats0[3][29] + coeff0_4*dmats0[4][29] + coeff0_5*dmats0[5][29] + coeff0_6*dmats0[6][29] + coeff0_7*dmats0[7][29] + coeff0_8*dmats0[8][29] + coeff0_9*dmats0[9][29] + coeff0_10*dmats0[10][29] + coeff0_11*dmats0[11][29] + coeff0_12*dmats0[12][29] + coeff0_13*dmats0[13][29] + coeff0_14*dmats0[14][29] + coeff0_15*dmats0[15][29] + coeff0_16*dmats0[16][29] + coeff0_17*dmats0[17][29] + coeff0_18*dmats0[18][29] + coeff0_19*dmats0[19][29] + coeff0_20*dmats0[20][29] + coeff0_21*dmats0[21][29] + coeff0_22*dmats0[22][29] + coeff0_23*dmats0[23][29] + coeff0_24*dmats0[24][29] + coeff0_25*dmats0[25][29] + coeff0_26*dmats0[26][29] + coeff0_27*dmats0[27][29] + coeff0_28*dmats0[28][29] + coeff0_29*dmats0[29][29] + coeff0_30*dmats0[30][29] + coeff0_31*dmats0[31][29] + coeff0_32*dmats0[32][29] + coeff0_33*dmats0[33][29] + coeff0_34*dmats0[34][29] + coeff0_35*dmats0[35][29] + coeff0_36*dmats0[36][29] + coeff0_37*dmats0[37][29] + coeff0_38*dmats0[38][29] + coeff0_39*dmats0[39][29] + coeff0_40*dmats0[40][29] + coeff0_41*dmats0[41][29] + coeff0_42*dmats0[42][29] + coeff0_43*dmats0[43][29] + coeff0_44*dmats0[44][29] + coeff0_45*dmats0[45][29] + coeff0_46*dmats0[46][29] + coeff0_47*dmats0[47][29] + coeff0_48*dmats0[48][29] + coeff0_49*dmats0[49][29] + coeff0_50*dmats0[50][29] + coeff0_51*dmats0[51][29] + coeff0_52*dmats0[52][29] + coeff0_53*dmats0[53][29] + coeff0_54*dmats0[54][29] + coeff0_55*dmats0[55][29];
          new_coeff0_30 = coeff0_0*dmats0[0][30] + coeff0_1*dmats0[1][30] + coeff0_2*dmats0[2][30] + coeff0_3*dmats0[3][30] + coeff0_4*dmats0[4][30] + coeff0_5*dmats0[5][30] + coeff0_6*dmats0[6][30] + coeff0_7*dmats0[7][30] + coeff0_8*dmats0[8][30] + coeff0_9*dmats0[9][30] + coeff0_10*dmats0[10][30] + coeff0_11*dmats0[11][30] + coeff0_12*dmats0[12][30] + coeff0_13*dmats0[13][30] + coeff0_14*dmats0[14][30] + coeff0_15*dmats0[15][30] + coeff0_16*dmats0[16][30] + coeff0_17*dmats0[17][30] + coeff0_18*dmats0[18][30] + coeff0_19*dmats0[19][30] + coeff0_20*dmats0[20][30] + coeff0_21*dmats0[21][30] + coeff0_22*dmats0[22][30] + coeff0_23*dmats0[23][30] + coeff0_24*dmats0[24][30] + coeff0_25*dmats0[25][30] + coeff0_26*dmats0[26][30] + coeff0_27*dmats0[27][30] + coeff0_28*dmats0[28][30] + coeff0_29*dmats0[29][30] + coeff0_30*dmats0[30][30] + coeff0_31*dmats0[31][30] + coeff0_32*dmats0[32][30] + coeff0_33*dmats0[33][30] + coeff0_34*dmats0[34][30] + coeff0_35*dmats0[35][30] + coeff0_36*dmats0[36][30] + coeff0_37*dmats0[37][30] + coeff0_38*dmats0[38][30] + coeff0_39*dmats0[39][30] + coeff0_40*dmats0[40][30] + coeff0_41*dmats0[41][30] + coeff0_42*dmats0[42][30] + coeff0_43*dmats0[43][30] + coeff0_44*dmats0[44][30] + coeff0_45*dmats0[45][30] + coeff0_46*dmats0[46][30] + coeff0_47*dmats0[47][30] + coeff0_48*dmats0[48][30] + coeff0_49*dmats0[49][30] + coeff0_50*dmats0[50][30] + coeff0_51*dmats0[51][30] + coeff0_52*dmats0[52][30] + coeff0_53*dmats0[53][30] + coeff0_54*dmats0[54][30] + coeff0_55*dmats0[55][30];
          new_coeff0_31 = coeff0_0*dmats0[0][31] + coeff0_1*dmats0[1][31] + coeff0_2*dmats0[2][31] + coeff0_3*dmats0[3][31] + coeff0_4*dmats0[4][31] + coeff0_5*dmats0[5][31] + coeff0_6*dmats0[6][31] + coeff0_7*dmats0[7][31] + coeff0_8*dmats0[8][31] + coeff0_9*dmats0[9][31] + coeff0_10*dmats0[10][31] + coeff0_11*dmats0[11][31] + coeff0_12*dmats0[12][31] + coeff0_13*dmats0[13][31] + coeff0_14*dmats0[14][31] + coeff0_15*dmats0[15][31] + coeff0_16*dmats0[16][31] + coeff0_17*dmats0[17][31] + coeff0_18*dmats0[18][31] + coeff0_19*dmats0[19][31] + coeff0_20*dmats0[20][31] + coeff0_21*dmats0[21][31] + coeff0_22*dmats0[22][31] + coeff0_23*dmats0[23][31] + coeff0_24*dmats0[24][31] + coeff0_25*dmats0[25][31] + coeff0_26*dmats0[26][31] + coeff0_27*dmats0[27][31] + coeff0_28*dmats0[28][31] + coeff0_29*dmats0[29][31] + coeff0_30*dmats0[30][31] + coeff0_31*dmats0[31][31] + coeff0_32*dmats0[32][31] + coeff0_33*dmats0[33][31] + coeff0_34*dmats0[34][31] + coeff0_35*dmats0[35][31] + coeff0_36*dmats0[36][31] + coeff0_37*dmats0[37][31] + coeff0_38*dmats0[38][31] + coeff0_39*dmats0[39][31] + coeff0_40*dmats0[40][31] + coeff0_41*dmats0[41][31] + coeff0_42*dmats0[42][31] + coeff0_43*dmats0[43][31] + coeff0_44*dmats0[44][31] + coeff0_45*dmats0[45][31] + coeff0_46*dmats0[46][31] + coeff0_47*dmats0[47][31] + coeff0_48*dmats0[48][31] + coeff0_49*dmats0[49][31] + coeff0_50*dmats0[50][31] + coeff0_51*dmats0[51][31] + coeff0_52*dmats0[52][31] + coeff0_53*dmats0[53][31] + coeff0_54*dmats0[54][31] + coeff0_55*dmats0[55][31];
          new_coeff0_32 = coeff0_0*dmats0[0][32] + coeff0_1*dmats0[1][32] + coeff0_2*dmats0[2][32] + coeff0_3*dmats0[3][32] + coeff0_4*dmats0[4][32] + coeff0_5*dmats0[5][32] + coeff0_6*dmats0[6][32] + coeff0_7*dmats0[7][32] + coeff0_8*dmats0[8][32] + coeff0_9*dmats0[9][32] + coeff0_10*dmats0[10][32] + coeff0_11*dmats0[11][32] + coeff0_12*dmats0[12][32] + coeff0_13*dmats0[13][32] + coeff0_14*dmats0[14][32] + coeff0_15*dmats0[15][32] + coeff0_16*dmats0[16][32] + coeff0_17*dmats0[17][32] + coeff0_18*dmats0[18][32] + coeff0_19*dmats0[19][32] + coeff0_20*dmats0[20][32] + coeff0_21*dmats0[21][32] + coeff0_22*dmats0[22][32] + coeff0_23*dmats0[23][32] + coeff0_24*dmats0[24][32] + coeff0_25*dmats0[25][32] + coeff0_26*dmats0[26][32] + coeff0_27*dmats0[27][32] + coeff0_28*dmats0[28][32] + coeff0_29*dmats0[29][32] + coeff0_30*dmats0[30][32] + coeff0_31*dmats0[31][32] + coeff0_32*dmats0[32][32] + coeff0_33*dmats0[33][32] + coeff0_34*dmats0[34][32] + coeff0_35*dmats0[35][32] + coeff0_36*dmats0[36][32] + coeff0_37*dmats0[37][32] + coeff0_38*dmats0[38][32] + coeff0_39*dmats0[39][32] + coeff0_40*dmats0[40][32] + coeff0_41*dmats0[41][32] + coeff0_42*dmats0[42][32] + coeff0_43*dmats0[43][32] + coeff0_44*dmats0[44][32] + coeff0_45*dmats0[45][32] + coeff0_46*dmats0[46][32] + coeff0_47*dmats0[47][32] + coeff0_48*dmats0[48][32] + coeff0_49*dmats0[49][32] + coeff0_50*dmats0[50][32] + coeff0_51*dmats0[51][32] + coeff0_52*dmats0[52][32] + coeff0_53*dmats0[53][32] + coeff0_54*dmats0[54][32] + coeff0_55*dmats0[55][32];
          new_coeff0_33 = coeff0_0*dmats0[0][33] + coeff0_1*dmats0[1][33] + coeff0_2*dmats0[2][33] + coeff0_3*dmats0[3][33] + coeff0_4*dmats0[4][33] + coeff0_5*dmats0[5][33] + coeff0_6*dmats0[6][33] + coeff0_7*dmats0[7][33] + coeff0_8*dmats0[8][33] + coeff0_9*dmats0[9][33] + coeff0_10*dmats0[10][33] + coeff0_11*dmats0[11][33] + coeff0_12*dmats0[12][33] + coeff0_13*dmats0[13][33] + coeff0_14*dmats0[14][33] + coeff0_15*dmats0[15][33] + coeff0_16*dmats0[16][33] + coeff0_17*dmats0[17][33] + coeff0_18*dmats0[18][33] + coeff0_19*dmats0[19][33] + coeff0_20*dmats0[20][33] + coeff0_21*dmats0[21][33] + coeff0_22*dmats0[22][33] + coeff0_23*dmats0[23][33] + coeff0_24*dmats0[24][33] + coeff0_25*dmats0[25][33] + coeff0_26*dmats0[26][33] + coeff0_27*dmats0[27][33] + coeff0_28*dmats0[28][33] + coeff0_29*dmats0[29][33] + coeff0_30*dmats0[30][33] + coeff0_31*dmats0[31][33] + coeff0_32*dmats0[32][33] + coeff0_33*dmats0[33][33] + coeff0_34*dmats0[34][33] + coeff0_35*dmats0[35][33] + coeff0_36*dmats0[36][33] + coeff0_37*dmats0[37][33] + coeff0_38*dmats0[38][33] + coeff0_39*dmats0[39][33] + coeff0_40*dmats0[40][33] + coeff0_41*dmats0[41][33] + coeff0_42*dmats0[42][33] + coeff0_43*dmats0[43][33] + coeff0_44*dmats0[44][33] + coeff0_45*dmats0[45][33] + coeff0_46*dmats0[46][33] + coeff0_47*dmats0[47][33] + coeff0_48*dmats0[48][33] + coeff0_49*dmats0[49][33] + coeff0_50*dmats0[50][33] + coeff0_51*dmats0[51][33] + coeff0_52*dmats0[52][33] + coeff0_53*dmats0[53][33] + coeff0_54*dmats0[54][33] + coeff0_55*dmats0[55][33];
          new_coeff0_34 = coeff0_0*dmats0[0][34] + coeff0_1*dmats0[1][34] + coeff0_2*dmats0[2][34] + coeff0_3*dmats0[3][34] + coeff0_4*dmats0[4][34] + coeff0_5*dmats0[5][34] + coeff0_6*dmats0[6][34] + coeff0_7*dmats0[7][34] + coeff0_8*dmats0[8][34] + coeff0_9*dmats0[9][34] + coeff0_10*dmats0[10][34] + coeff0_11*dmats0[11][34] + coeff0_12*dmats0[12][34] + coeff0_13*dmats0[13][34] + coeff0_14*dmats0[14][34] + coeff0_15*dmats0[15][34] + coeff0_16*dmats0[16][34] + coeff0_17*dmats0[17][34] + coeff0_18*dmats0[18][34] + coeff0_19*dmats0[19][34] + coeff0_20*dmats0[20][34] + coeff0_21*dmats0[21][34] + coeff0_22*dmats0[22][34] + coeff0_23*dmats0[23][34] + coeff0_24*dmats0[24][34] + coeff0_25*dmats0[25][34] + coeff0_26*dmats0[26][34] + coeff0_27*dmats0[27][34] + coeff0_28*dmats0[28][34] + coeff0_29*dmats0[29][34] + coeff0_30*dmats0[30][34] + coeff0_31*dmats0[31][34] + coeff0_32*dmats0[32][34] + coeff0_33*dmats0[33][34] + coeff0_34*dmats0[34][34] + coeff0_35*dmats0[35][34] + coeff0_36*dmats0[36][34] + coeff0_37*dmats0[37][34] + coeff0_38*dmats0[38][34] + coeff0_39*dmats0[39][34] + coeff0_40*dmats0[40][34] + coeff0_41*dmats0[41][34] + coeff0_42*dmats0[42][34] + coeff0_43*dmats0[43][34] + coeff0_44*dmats0[44][34] + coeff0_45*dmats0[45][34] + coeff0_46*dmats0[46][34] + coeff0_47*dmats0[47][34] + coeff0_48*dmats0[48][34] + coeff0_49*dmats0[49][34] + coeff0_50*dmats0[50][34] + coeff0_51*dmats0[51][34] + coeff0_52*dmats0[52][34] + coeff0_53*dmats0[53][34] + coeff0_54*dmats0[54][34] + coeff0_55*dmats0[55][34];
          new_coeff0_35 = coeff0_0*dmats0[0][35] + coeff0_1*dmats0[1][35] + coeff0_2*dmats0[2][35] + coeff0_3*dmats0[3][35] + coeff0_4*dmats0[4][35] + coeff0_5*dmats0[5][35] + coeff0_6*dmats0[6][35] + coeff0_7*dmats0[7][35] + coeff0_8*dmats0[8][35] + coeff0_9*dmats0[9][35] + coeff0_10*dmats0[10][35] + coeff0_11*dmats0[11][35] + coeff0_12*dmats0[12][35] + coeff0_13*dmats0[13][35] + coeff0_14*dmats0[14][35] + coeff0_15*dmats0[15][35] + coeff0_16*dmats0[16][35] + coeff0_17*dmats0[17][35] + coeff0_18*dmats0[18][35] + coeff0_19*dmats0[19][35] + coeff0_20*dmats0[20][35] + coeff0_21*dmats0[21][35] + coeff0_22*dmats0[22][35] + coeff0_23*dmats0[23][35] + coeff0_24*dmats0[24][35] + coeff0_25*dmats0[25][35] + coeff0_26*dmats0[26][35] + coeff0_27*dmats0[27][35] + coeff0_28*dmats0[28][35] + coeff0_29*dmats0[29][35] + coeff0_30*dmats0[30][35] + coeff0_31*dmats0[31][35] + coeff0_32*dmats0[32][35] + coeff0_33*dmats0[33][35] + coeff0_34*dmats0[34][35] + coeff0_35*dmats0[35][35] + coeff0_36*dmats0[36][35] + coeff0_37*dmats0[37][35] + coeff0_38*dmats0[38][35] + coeff0_39*dmats0[39][35] + coeff0_40*dmats0[40][35] + coeff0_41*dmats0[41][35] + coeff0_42*dmats0[42][35] + coeff0_43*dmats0[43][35] + coeff0_44*dmats0[44][35] + coeff0_45*dmats0[45][35] + coeff0_46*dmats0[46][35] + coeff0_47*dmats0[47][35] + coeff0_48*dmats0[48][35] + coeff0_49*dmats0[49][35] + coeff0_50*dmats0[50][35] + coeff0_51*dmats0[51][35] + coeff0_52*dmats0[52][35] + coeff0_53*dmats0[53][35] + coeff0_54*dmats0[54][35] + coeff0_55*dmats0[55][35];
          new_coeff0_36 = coeff0_0*dmats0[0][36] + coeff0_1*dmats0[1][36] + coeff0_2*dmats0[2][36] + coeff0_3*dmats0[3][36] + coeff0_4*dmats0[4][36] + coeff0_5*dmats0[5][36] + coeff0_6*dmats0[6][36] + coeff0_7*dmats0[7][36] + coeff0_8*dmats0[8][36] + coeff0_9*dmats0[9][36] + coeff0_10*dmats0[10][36] + coeff0_11*dmats0[11][36] + coeff0_12*dmats0[12][36] + coeff0_13*dmats0[13][36] + coeff0_14*dmats0[14][36] + coeff0_15*dmats0[15][36] + coeff0_16*dmats0[16][36] + coeff0_17*dmats0[17][36] + coeff0_18*dmats0[18][36] + coeff0_19*dmats0[19][36] + coeff0_20*dmats0[20][36] + coeff0_21*dmats0[21][36] + coeff0_22*dmats0[22][36] + coeff0_23*dmats0[23][36] + coeff0_24*dmats0[24][36] + coeff0_25*dmats0[25][36] + coeff0_26*dmats0[26][36] + coeff0_27*dmats0[27][36] + coeff0_28*dmats0[28][36] + coeff0_29*dmats0[29][36] + coeff0_30*dmats0[30][36] + coeff0_31*dmats0[31][36] + coeff0_32*dmats0[32][36] + coeff0_33*dmats0[33][36] + coeff0_34*dmats0[34][36] + coeff0_35*dmats0[35][36] + coeff0_36*dmats0[36][36] + coeff0_37*dmats0[37][36] + coeff0_38*dmats0[38][36] + coeff0_39*dmats0[39][36] + coeff0_40*dmats0[40][36] + coeff0_41*dmats0[41][36] + coeff0_42*dmats0[42][36] + coeff0_43*dmats0[43][36] + coeff0_44*dmats0[44][36] + coeff0_45*dmats0[45][36] + coeff0_46*dmats0[46][36] + coeff0_47*dmats0[47][36] + coeff0_48*dmats0[48][36] + coeff0_49*dmats0[49][36] + coeff0_50*dmats0[50][36] + coeff0_51*dmats0[51][36] + coeff0_52*dmats0[52][36] + coeff0_53*dmats0[53][36] + coeff0_54*dmats0[54][36] + coeff0_55*dmats0[55][36];
          new_coeff0_37 = coeff0_0*dmats0[0][37] + coeff0_1*dmats0[1][37] + coeff0_2*dmats0[2][37] + coeff0_3*dmats0[3][37] + coeff0_4*dmats0[4][37] + coeff0_5*dmats0[5][37] + coeff0_6*dmats0[6][37] + coeff0_7*dmats0[7][37] + coeff0_8*dmats0[8][37] + coeff0_9*dmats0[9][37] + coeff0_10*dmats0[10][37] + coeff0_11*dmats0[11][37] + coeff0_12*dmats0[12][37] + coeff0_13*dmats0[13][37] + coeff0_14*dmats0[14][37] + coeff0_15*dmats0[15][37] + coeff0_16*dmats0[16][37] + coeff0_17*dmats0[17][37] + coeff0_18*dmats0[18][37] + coeff0_19*dmats0[19][37] + coeff0_20*dmats0[20][37] + coeff0_21*dmats0[21][37] + coeff0_22*dmats0[22][37] + coeff0_23*dmats0[23][37] + coeff0_24*dmats0[24][37] + coeff0_25*dmats0[25][37] + coeff0_26*dmats0[26][37] + coeff0_27*dmats0[27][37] + coeff0_28*dmats0[28][37] + coeff0_29*dmats0[29][37] + coeff0_30*dmats0[30][37] + coeff0_31*dmats0[31][37] + coeff0_32*dmats0[32][37] + coeff0_33*dmats0[33][37] + coeff0_34*dmats0[34][37] + coeff0_35*dmats0[35][37] + coeff0_36*dmats0[36][37] + coeff0_37*dmats0[37][37] + coeff0_38*dmats0[38][37] + coeff0_39*dmats0[39][37] + coeff0_40*dmats0[40][37] + coeff0_41*dmats0[41][37] + coeff0_42*dmats0[42][37] + coeff0_43*dmats0[43][37] + coeff0_44*dmats0[44][37] + coeff0_45*dmats0[45][37] + coeff0_46*dmats0[46][37] + coeff0_47*dmats0[47][37] + coeff0_48*dmats0[48][37] + coeff0_49*dmats0[49][37] + coeff0_50*dmats0[50][37] + coeff0_51*dmats0[51][37] + coeff0_52*dmats0[52][37] + coeff0_53*dmats0[53][37] + coeff0_54*dmats0[54][37] + coeff0_55*dmats0[55][37];
          new_coeff0_38 = coeff0_0*dmats0[0][38] + coeff0_1*dmats0[1][38] + coeff0_2*dmats0[2][38] + coeff0_3*dmats0[3][38] + coeff0_4*dmats0[4][38] + coeff0_5*dmats0[5][38] + coeff0_6*dmats0[6][38] + coeff0_7*dmats0[7][38] + coeff0_8*dmats0[8][38] + coeff0_9*dmats0[9][38] + coeff0_10*dmats0[10][38] + coeff0_11*dmats0[11][38] + coeff0_12*dmats0[12][38] + coeff0_13*dmats0[13][38] + coeff0_14*dmats0[14][38] + coeff0_15*dmats0[15][38] + coeff0_16*dmats0[16][38] + coeff0_17*dmats0[17][38] + coeff0_18*dmats0[18][38] + coeff0_19*dmats0[19][38] + coeff0_20*dmats0[20][38] + coeff0_21*dmats0[21][38] + coeff0_22*dmats0[22][38] + coeff0_23*dmats0[23][38] + coeff0_24*dmats0[24][38] + coeff0_25*dmats0[25][38] + coeff0_26*dmats0[26][38] + coeff0_27*dmats0[27][38] + coeff0_28*dmats0[28][38] + coeff0_29*dmats0[29][38] + coeff0_30*dmats0[30][38] + coeff0_31*dmats0[31][38] + coeff0_32*dmats0[32][38] + coeff0_33*dmats0[33][38] + coeff0_34*dmats0[34][38] + coeff0_35*dmats0[35][38] + coeff0_36*dmats0[36][38] + coeff0_37*dmats0[37][38] + coeff0_38*dmats0[38][38] + coeff0_39*dmats0[39][38] + coeff0_40*dmats0[40][38] + coeff0_41*dmats0[41][38] + coeff0_42*dmats0[42][38] + coeff0_43*dmats0[43][38] + coeff0_44*dmats0[44][38] + coeff0_45*dmats0[45][38] + coeff0_46*dmats0[46][38] + coeff0_47*dmats0[47][38] + coeff0_48*dmats0[48][38] + coeff0_49*dmats0[49][38] + coeff0_50*dmats0[50][38] + coeff0_51*dmats0[51][38] + coeff0_52*dmats0[52][38] + coeff0_53*dmats0[53][38] + coeff0_54*dmats0[54][38] + coeff0_55*dmats0[55][38];
          new_coeff0_39 = coeff0_0*dmats0[0][39] + coeff0_1*dmats0[1][39] + coeff0_2*dmats0[2][39] + coeff0_3*dmats0[3][39] + coeff0_4*dmats0[4][39] + coeff0_5*dmats0[5][39] + coeff0_6*dmats0[6][39] + coeff0_7*dmats0[7][39] + coeff0_8*dmats0[8][39] + coeff0_9*dmats0[9][39] + coeff0_10*dmats0[10][39] + coeff0_11*dmats0[11][39] + coeff0_12*dmats0[12][39] + coeff0_13*dmats0[13][39] + coeff0_14*dmats0[14][39] + coeff0_15*dmats0[15][39] + coeff0_16*dmats0[16][39] + coeff0_17*dmats0[17][39] + coeff0_18*dmats0[18][39] + coeff0_19*dmats0[19][39] + coeff0_20*dmats0[20][39] + coeff0_21*dmats0[21][39] + coeff0_22*dmats0[22][39] + coeff0_23*dmats0[23][39] + coeff0_24*dmats0[24][39] + coeff0_25*dmats0[25][39] + coeff0_26*dmats0[26][39] + coeff0_27*dmats0[27][39] + coeff0_28*dmats0[28][39] + coeff0_29*dmats0[29][39] + coeff0_30*dmats0[30][39] + coeff0_31*dmats0[31][39] + coeff0_32*dmats0[32][39] + coeff0_33*dmats0[33][39] + coeff0_34*dmats0[34][39] + coeff0_35*dmats0[35][39] + coeff0_36*dmats0[36][39] + coeff0_37*dmats0[37][39] + coeff0_38*dmats0[38][39] + coeff0_39*dmats0[39][39] + coeff0_40*dmats0[40][39] + coeff0_41*dmats0[41][39] + coeff0_42*dmats0[42][39] + coeff0_43*dmats0[43][39] + coeff0_44*dmats0[44][39] + coeff0_45*dmats0[45][39] + coeff0_46*dmats0[46][39] + coeff0_47*dmats0[47][39] + coeff0_48*dmats0[48][39] + coeff0_49*dmats0[49][39] + coeff0_50*dmats0[50][39] + coeff0_51*dmats0[51][39] + coeff0_52*dmats0[52][39] + coeff0_53*dmats0[53][39] + coeff0_54*dmats0[54][39] + coeff0_55*dmats0[55][39];
          new_coeff0_40 = coeff0_0*dmats0[0][40] + coeff0_1*dmats0[1][40] + coeff0_2*dmats0[2][40] + coeff0_3*dmats0[3][40] + coeff0_4*dmats0[4][40] + coeff0_5*dmats0[5][40] + coeff0_6*dmats0[6][40] + coeff0_7*dmats0[7][40] + coeff0_8*dmats0[8][40] + coeff0_9*dmats0[9][40] + coeff0_10*dmats0[10][40] + coeff0_11*dmats0[11][40] + coeff0_12*dmats0[12][40] + coeff0_13*dmats0[13][40] + coeff0_14*dmats0[14][40] + coeff0_15*dmats0[15][40] + coeff0_16*dmats0[16][40] + coeff0_17*dmats0[17][40] + coeff0_18*dmats0[18][40] + coeff0_19*dmats0[19][40] + coeff0_20*dmats0[20][40] + coeff0_21*dmats0[21][40] + coeff0_22*dmats0[22][40] + coeff0_23*dmats0[23][40] + coeff0_24*dmats0[24][40] + coeff0_25*dmats0[25][40] + coeff0_26*dmats0[26][40] + coeff0_27*dmats0[27][40] + coeff0_28*dmats0[28][40] + coeff0_29*dmats0[29][40] + coeff0_30*dmats0[30][40] + coeff0_31*dmats0[31][40] + coeff0_32*dmats0[32][40] + coeff0_33*dmats0[33][40] + coeff0_34*dmats0[34][40] + coeff0_35*dmats0[35][40] + coeff0_36*dmats0[36][40] + coeff0_37*dmats0[37][40] + coeff0_38*dmats0[38][40] + coeff0_39*dmats0[39][40] + coeff0_40*dmats0[40][40] + coeff0_41*dmats0[41][40] + coeff0_42*dmats0[42][40] + coeff0_43*dmats0[43][40] + coeff0_44*dmats0[44][40] + coeff0_45*dmats0[45][40] + coeff0_46*dmats0[46][40] + coeff0_47*dmats0[47][40] + coeff0_48*dmats0[48][40] + coeff0_49*dmats0[49][40] + coeff0_50*dmats0[50][40] + coeff0_51*dmats0[51][40] + coeff0_52*dmats0[52][40] + coeff0_53*dmats0[53][40] + coeff0_54*dmats0[54][40] + coeff0_55*dmats0[55][40];
          new_coeff0_41 = coeff0_0*dmats0[0][41] + coeff0_1*dmats0[1][41] + coeff0_2*dmats0[2][41] + coeff0_3*dmats0[3][41] + coeff0_4*dmats0[4][41] + coeff0_5*dmats0[5][41] + coeff0_6*dmats0[6][41] + coeff0_7*dmats0[7][41] + coeff0_8*dmats0[8][41] + coeff0_9*dmats0[9][41] + coeff0_10*dmats0[10][41] + coeff0_11*dmats0[11][41] + coeff0_12*dmats0[12][41] + coeff0_13*dmats0[13][41] + coeff0_14*dmats0[14][41] + coeff0_15*dmats0[15][41] + coeff0_16*dmats0[16][41] + coeff0_17*dmats0[17][41] + coeff0_18*dmats0[18][41] + coeff0_19*dmats0[19][41] + coeff0_20*dmats0[20][41] + coeff0_21*dmats0[21][41] + coeff0_22*dmats0[22][41] + coeff0_23*dmats0[23][41] + coeff0_24*dmats0[24][41] + coeff0_25*dmats0[25][41] + coeff0_26*dmats0[26][41] + coeff0_27*dmats0[27][41] + coeff0_28*dmats0[28][41] + coeff0_29*dmats0[29][41] + coeff0_30*dmats0[30][41] + coeff0_31*dmats0[31][41] + coeff0_32*dmats0[32][41] + coeff0_33*dmats0[33][41] + coeff0_34*dmats0[34][41] + coeff0_35*dmats0[35][41] + coeff0_36*dmats0[36][41] + coeff0_37*dmats0[37][41] + coeff0_38*dmats0[38][41] + coeff0_39*dmats0[39][41] + coeff0_40*dmats0[40][41] + coeff0_41*dmats0[41][41] + coeff0_42*dmats0[42][41] + coeff0_43*dmats0[43][41] + coeff0_44*dmats0[44][41] + coeff0_45*dmats0[45][41] + coeff0_46*dmats0[46][41] + coeff0_47*dmats0[47][41] + coeff0_48*dmats0[48][41] + coeff0_49*dmats0[49][41] + coeff0_50*dmats0[50][41] + coeff0_51*dmats0[51][41] + coeff0_52*dmats0[52][41] + coeff0_53*dmats0[53][41] + coeff0_54*dmats0[54][41] + coeff0_55*dmats0[55][41];
          new_coeff0_42 = coeff0_0*dmats0[0][42] + coeff0_1*dmats0[1][42] + coeff0_2*dmats0[2][42] + coeff0_3*dmats0[3][42] + coeff0_4*dmats0[4][42] + coeff0_5*dmats0[5][42] + coeff0_6*dmats0[6][42] + coeff0_7*dmats0[7][42] + coeff0_8*dmats0[8][42] + coeff0_9*dmats0[9][42] + coeff0_10*dmats0[10][42] + coeff0_11*dmats0[11][42] + coeff0_12*dmats0[12][42] + coeff0_13*dmats0[13][42] + coeff0_14*dmats0[14][42] + coeff0_15*dmats0[15][42] + coeff0_16*dmats0[16][42] + coeff0_17*dmats0[17][42] + coeff0_18*dmats0[18][42] + coeff0_19*dmats0[19][42] + coeff0_20*dmats0[20][42] + coeff0_21*dmats0[21][42] + coeff0_22*dmats0[22][42] + coeff0_23*dmats0[23][42] + coeff0_24*dmats0[24][42] + coeff0_25*dmats0[25][42] + coeff0_26*dmats0[26][42] + coeff0_27*dmats0[27][42] + coeff0_28*dmats0[28][42] + coeff0_29*dmats0[29][42] + coeff0_30*dmats0[30][42] + coeff0_31*dmats0[31][42] + coeff0_32*dmats0[32][42] + coeff0_33*dmats0[33][42] + coeff0_34*dmats0[34][42] + coeff0_35*dmats0[35][42] + coeff0_36*dmats0[36][42] + coeff0_37*dmats0[37][42] + coeff0_38*dmats0[38][42] + coeff0_39*dmats0[39][42] + coeff0_40*dmats0[40][42] + coeff0_41*dmats0[41][42] + coeff0_42*dmats0[42][42] + coeff0_43*dmats0[43][42] + coeff0_44*dmats0[44][42] + coeff0_45*dmats0[45][42] + coeff0_46*dmats0[46][42] + coeff0_47*dmats0[47][42] + coeff0_48*dmats0[48][42] + coeff0_49*dmats0[49][42] + coeff0_50*dmats0[50][42] + coeff0_51*dmats0[51][42] + coeff0_52*dmats0[52][42] + coeff0_53*dmats0[53][42] + coeff0_54*dmats0[54][42] + coeff0_55*dmats0[55][42];
          new_coeff0_43 = coeff0_0*dmats0[0][43] + coeff0_1*dmats0[1][43] + coeff0_2*dmats0[2][43] + coeff0_3*dmats0[3][43] + coeff0_4*dmats0[4][43] + coeff0_5*dmats0[5][43] + coeff0_6*dmats0[6][43] + coeff0_7*dmats0[7][43] + coeff0_8*dmats0[8][43] + coeff0_9*dmats0[9][43] + coeff0_10*dmats0[10][43] + coeff0_11*dmats0[11][43] + coeff0_12*dmats0[12][43] + coeff0_13*dmats0[13][43] + coeff0_14*dmats0[14][43] + coeff0_15*dmats0[15][43] + coeff0_16*dmats0[16][43] + coeff0_17*dmats0[17][43] + coeff0_18*dmats0[18][43] + coeff0_19*dmats0[19][43] + coeff0_20*dmats0[20][43] + coeff0_21*dmats0[21][43] + coeff0_22*dmats0[22][43] + coeff0_23*dmats0[23][43] + coeff0_24*dmats0[24][43] + coeff0_25*dmats0[25][43] + coeff0_26*dmats0[26][43] + coeff0_27*dmats0[27][43] + coeff0_28*dmats0[28][43] + coeff0_29*dmats0[29][43] + coeff0_30*dmats0[30][43] + coeff0_31*dmats0[31][43] + coeff0_32*dmats0[32][43] + coeff0_33*dmats0[33][43] + coeff0_34*dmats0[34][43] + coeff0_35*dmats0[35][43] + coeff0_36*dmats0[36][43] + coeff0_37*dmats0[37][43] + coeff0_38*dmats0[38][43] + coeff0_39*dmats0[39][43] + coeff0_40*dmats0[40][43] + coeff0_41*dmats0[41][43] + coeff0_42*dmats0[42][43] + coeff0_43*dmats0[43][43] + coeff0_44*dmats0[44][43] + coeff0_45*dmats0[45][43] + coeff0_46*dmats0[46][43] + coeff0_47*dmats0[47][43] + coeff0_48*dmats0[48][43] + coeff0_49*dmats0[49][43] + coeff0_50*dmats0[50][43] + coeff0_51*dmats0[51][43] + coeff0_52*dmats0[52][43] + coeff0_53*dmats0[53][43] + coeff0_54*dmats0[54][43] + coeff0_55*dmats0[55][43];
          new_coeff0_44 = coeff0_0*dmats0[0][44] + coeff0_1*dmats0[1][44] + coeff0_2*dmats0[2][44] + coeff0_3*dmats0[3][44] + coeff0_4*dmats0[4][44] + coeff0_5*dmats0[5][44] + coeff0_6*dmats0[6][44] + coeff0_7*dmats0[7][44] + coeff0_8*dmats0[8][44] + coeff0_9*dmats0[9][44] + coeff0_10*dmats0[10][44] + coeff0_11*dmats0[11][44] + coeff0_12*dmats0[12][44] + coeff0_13*dmats0[13][44] + coeff0_14*dmats0[14][44] + coeff0_15*dmats0[15][44] + coeff0_16*dmats0[16][44] + coeff0_17*dmats0[17][44] + coeff0_18*dmats0[18][44] + coeff0_19*dmats0[19][44] + coeff0_20*dmats0[20][44] + coeff0_21*dmats0[21][44] + coeff0_22*dmats0[22][44] + coeff0_23*dmats0[23][44] + coeff0_24*dmats0[24][44] + coeff0_25*dmats0[25][44] + coeff0_26*dmats0[26][44] + coeff0_27*dmats0[27][44] + coeff0_28*dmats0[28][44] + coeff0_29*dmats0[29][44] + coeff0_30*dmats0[30][44] + coeff0_31*dmats0[31][44] + coeff0_32*dmats0[32][44] + coeff0_33*dmats0[33][44] + coeff0_34*dmats0[34][44] + coeff0_35*dmats0[35][44] + coeff0_36*dmats0[36][44] + coeff0_37*dmats0[37][44] + coeff0_38*dmats0[38][44] + coeff0_39*dmats0[39][44] + coeff0_40*dmats0[40][44] + coeff0_41*dmats0[41][44] + coeff0_42*dmats0[42][44] + coeff0_43*dmats0[43][44] + coeff0_44*dmats0[44][44] + coeff0_45*dmats0[45][44] + coeff0_46*dmats0[46][44] + coeff0_47*dmats0[47][44] + coeff0_48*dmats0[48][44] + coeff0_49*dmats0[49][44] + coeff0_50*dmats0[50][44] + coeff0_51*dmats0[51][44] + coeff0_52*dmats0[52][44] + coeff0_53*dmats0[53][44] + coeff0_54*dmats0[54][44] + coeff0_55*dmats0[55][44];
          new_coeff0_45 = coeff0_0*dmats0[0][45] + coeff0_1*dmats0[1][45] + coeff0_2*dmats0[2][45] + coeff0_3*dmats0[3][45] + coeff0_4*dmats0[4][45] + coeff0_5*dmats0[5][45] + coeff0_6*dmats0[6][45] + coeff0_7*dmats0[7][45] + coeff0_8*dmats0[8][45] + coeff0_9*dmats0[9][45] + coeff0_10*dmats0[10][45] + coeff0_11*dmats0[11][45] + coeff0_12*dmats0[12][45] + coeff0_13*dmats0[13][45] + coeff0_14*dmats0[14][45] + coeff0_15*dmats0[15][45] + coeff0_16*dmats0[16][45] + coeff0_17*dmats0[17][45] + coeff0_18*dmats0[18][45] + coeff0_19*dmats0[19][45] + coeff0_20*dmats0[20][45] + coeff0_21*dmats0[21][45] + coeff0_22*dmats0[22][45] + coeff0_23*dmats0[23][45] + coeff0_24*dmats0[24][45] + coeff0_25*dmats0[25][45] + coeff0_26*dmats0[26][45] + coeff0_27*dmats0[27][45] + coeff0_28*dmats0[28][45] + coeff0_29*dmats0[29][45] + coeff0_30*dmats0[30][45] + coeff0_31*dmats0[31][45] + coeff0_32*dmats0[32][45] + coeff0_33*dmats0[33][45] + coeff0_34*dmats0[34][45] + coeff0_35*dmats0[35][45] + coeff0_36*dmats0[36][45] + coeff0_37*dmats0[37][45] + coeff0_38*dmats0[38][45] + coeff0_39*dmats0[39][45] + coeff0_40*dmats0[40][45] + coeff0_41*dmats0[41][45] + coeff0_42*dmats0[42][45] + coeff0_43*dmats0[43][45] + coeff0_44*dmats0[44][45] + coeff0_45*dmats0[45][45] + coeff0_46*dmats0[46][45] + coeff0_47*dmats0[47][45] + coeff0_48*dmats0[48][45] + coeff0_49*dmats0[49][45] + coeff0_50*dmats0[50][45] + coeff0_51*dmats0[51][45] + coeff0_52*dmats0[52][45] + coeff0_53*dmats0[53][45] + coeff0_54*dmats0[54][45] + coeff0_55*dmats0[55][45];
          new_coeff0_46 = coeff0_0*dmats0[0][46] + coeff0_1*dmats0[1][46] + coeff0_2*dmats0[2][46] + coeff0_3*dmats0[3][46] + coeff0_4*dmats0[4][46] + coeff0_5*dmats0[5][46] + coeff0_6*dmats0[6][46] + coeff0_7*dmats0[7][46] + coeff0_8*dmats0[8][46] + coeff0_9*dmats0[9][46] + coeff0_10*dmats0[10][46] + coeff0_11*dmats0[11][46] + coeff0_12*dmats0[12][46] + coeff0_13*dmats0[13][46] + coeff0_14*dmats0[14][46] + coeff0_15*dmats0[15][46] + coeff0_16*dmats0[16][46] + coeff0_17*dmats0[17][46] + coeff0_18*dmats0[18][46] + coeff0_19*dmats0[19][46] + coeff0_20*dmats0[20][46] + coeff0_21*dmats0[21][46] + coeff0_22*dmats0[22][46] + coeff0_23*dmats0[23][46] + coeff0_24*dmats0[24][46] + coeff0_25*dmats0[25][46] + coeff0_26*dmats0[26][46] + coeff0_27*dmats0[27][46] + coeff0_28*dmats0[28][46] + coeff0_29*dmats0[29][46] + coeff0_30*dmats0[30][46] + coeff0_31*dmats0[31][46] + coeff0_32*dmats0[32][46] + coeff0_33*dmats0[33][46] + coeff0_34*dmats0[34][46] + coeff0_35*dmats0[35][46] + coeff0_36*dmats0[36][46] + coeff0_37*dmats0[37][46] + coeff0_38*dmats0[38][46] + coeff0_39*dmats0[39][46] + coeff0_40*dmats0[40][46] + coeff0_41*dmats0[41][46] + coeff0_42*dmats0[42][46] + coeff0_43*dmats0[43][46] + coeff0_44*dmats0[44][46] + coeff0_45*dmats0[45][46] + coeff0_46*dmats0[46][46] + coeff0_47*dmats0[47][46] + coeff0_48*dmats0[48][46] + coeff0_49*dmats0[49][46] + coeff0_50*dmats0[50][46] + coeff0_51*dmats0[51][46] + coeff0_52*dmats0[52][46] + coeff0_53*dmats0[53][46] + coeff0_54*dmats0[54][46] + coeff0_55*dmats0[55][46];
          new_coeff0_47 = coeff0_0*dmats0[0][47] + coeff0_1*dmats0[1][47] + coeff0_2*dmats0[2][47] + coeff0_3*dmats0[3][47] + coeff0_4*dmats0[4][47] + coeff0_5*dmats0[5][47] + coeff0_6*dmats0[6][47] + coeff0_7*dmats0[7][47] + coeff0_8*dmats0[8][47] + coeff0_9*dmats0[9][47] + coeff0_10*dmats0[10][47] + coeff0_11*dmats0[11][47] + coeff0_12*dmats0[12][47] + coeff0_13*dmats0[13][47] + coeff0_14*dmats0[14][47] + coeff0_15*dmats0[15][47] + coeff0_16*dmats0[16][47] + coeff0_17*dmats0[17][47] + coeff0_18*dmats0[18][47] + coeff0_19*dmats0[19][47] + coeff0_20*dmats0[20][47] + coeff0_21*dmats0[21][47] + coeff0_22*dmats0[22][47] + coeff0_23*dmats0[23][47] + coeff0_24*dmats0[24][47] + coeff0_25*dmats0[25][47] + coeff0_26*dmats0[26][47] + coeff0_27*dmats0[27][47] + coeff0_28*dmats0[28][47] + coeff0_29*dmats0[29][47] + coeff0_30*dmats0[30][47] + coeff0_31*dmats0[31][47] + coeff0_32*dmats0[32][47] + coeff0_33*dmats0[33][47] + coeff0_34*dmats0[34][47] + coeff0_35*dmats0[35][47] + coeff0_36*dmats0[36][47] + coeff0_37*dmats0[37][47] + coeff0_38*dmats0[38][47] + coeff0_39*dmats0[39][47] + coeff0_40*dmats0[40][47] + coeff0_41*dmats0[41][47] + coeff0_42*dmats0[42][47] + coeff0_43*dmats0[43][47] + coeff0_44*dmats0[44][47] + coeff0_45*dmats0[45][47] + coeff0_46*dmats0[46][47] + coeff0_47*dmats0[47][47] + coeff0_48*dmats0[48][47] + coeff0_49*dmats0[49][47] + coeff0_50*dmats0[50][47] + coeff0_51*dmats0[51][47] + coeff0_52*dmats0[52][47] + coeff0_53*dmats0[53][47] + coeff0_54*dmats0[54][47] + coeff0_55*dmats0[55][47];
          new_coeff0_48 = coeff0_0*dmats0[0][48] + coeff0_1*dmats0[1][48] + coeff0_2*dmats0[2][48] + coeff0_3*dmats0[3][48] + coeff0_4*dmats0[4][48] + coeff0_5*dmats0[5][48] + coeff0_6*dmats0[6][48] + coeff0_7*dmats0[7][48] + coeff0_8*dmats0[8][48] + coeff0_9*dmats0[9][48] + coeff0_10*dmats0[10][48] + coeff0_11*dmats0[11][48] + coeff0_12*dmats0[12][48] + coeff0_13*dmats0[13][48] + coeff0_14*dmats0[14][48] + coeff0_15*dmats0[15][48] + coeff0_16*dmats0[16][48] + coeff0_17*dmats0[17][48] + coeff0_18*dmats0[18][48] + coeff0_19*dmats0[19][48] + coeff0_20*dmats0[20][48] + coeff0_21*dmats0[21][48] + coeff0_22*dmats0[22][48] + coeff0_23*dmats0[23][48] + coeff0_24*dmats0[24][48] + coeff0_25*dmats0[25][48] + coeff0_26*dmats0[26][48] + coeff0_27*dmats0[27][48] + coeff0_28*dmats0[28][48] + coeff0_29*dmats0[29][48] + coeff0_30*dmats0[30][48] + coeff0_31*dmats0[31][48] + coeff0_32*dmats0[32][48] + coeff0_33*dmats0[33][48] + coeff0_34*dmats0[34][48] + coeff0_35*dmats0[35][48] + coeff0_36*dmats0[36][48] + coeff0_37*dmats0[37][48] + coeff0_38*dmats0[38][48] + coeff0_39*dmats0[39][48] + coeff0_40*dmats0[40][48] + coeff0_41*dmats0[41][48] + coeff0_42*dmats0[42][48] + coeff0_43*dmats0[43][48] + coeff0_44*dmats0[44][48] + coeff0_45*dmats0[45][48] + coeff0_46*dmats0[46][48] + coeff0_47*dmats0[47][48] + coeff0_48*dmats0[48][48] + coeff0_49*dmats0[49][48] + coeff0_50*dmats0[50][48] + coeff0_51*dmats0[51][48] + coeff0_52*dmats0[52][48] + coeff0_53*dmats0[53][48] + coeff0_54*dmats0[54][48] + coeff0_55*dmats0[55][48];
          new_coeff0_49 = coeff0_0*dmats0[0][49] + coeff0_1*dmats0[1][49] + coeff0_2*dmats0[2][49] + coeff0_3*dmats0[3][49] + coeff0_4*dmats0[4][49] + coeff0_5*dmats0[5][49] + coeff0_6*dmats0[6][49] + coeff0_7*dmats0[7][49] + coeff0_8*dmats0[8][49] + coeff0_9*dmats0[9][49] + coeff0_10*dmats0[10][49] + coeff0_11*dmats0[11][49] + coeff0_12*dmats0[12][49] + coeff0_13*dmats0[13][49] + coeff0_14*dmats0[14][49] + coeff0_15*dmats0[15][49] + coeff0_16*dmats0[16][49] + coeff0_17*dmats0[17][49] + coeff0_18*dmats0[18][49] + coeff0_19*dmats0[19][49] + coeff0_20*dmats0[20][49] + coeff0_21*dmats0[21][49] + coeff0_22*dmats0[22][49] + coeff0_23*dmats0[23][49] + coeff0_24*dmats0[24][49] + coeff0_25*dmats0[25][49] + coeff0_26*dmats0[26][49] + coeff0_27*dmats0[27][49] + coeff0_28*dmats0[28][49] + coeff0_29*dmats0[29][49] + coeff0_30*dmats0[30][49] + coeff0_31*dmats0[31][49] + coeff0_32*dmats0[32][49] + coeff0_33*dmats0[33][49] + coeff0_34*dmats0[34][49] + coeff0_35*dmats0[35][49] + coeff0_36*dmats0[36][49] + coeff0_37*dmats0[37][49] + coeff0_38*dmats0[38][49] + coeff0_39*dmats0[39][49] + coeff0_40*dmats0[40][49] + coeff0_41*dmats0[41][49] + coeff0_42*dmats0[42][49] + coeff0_43*dmats0[43][49] + coeff0_44*dmats0[44][49] + coeff0_45*dmats0[45][49] + coeff0_46*dmats0[46][49] + coeff0_47*dmats0[47][49] + coeff0_48*dmats0[48][49] + coeff0_49*dmats0[49][49] + coeff0_50*dmats0[50][49] + coeff0_51*dmats0[51][49] + coeff0_52*dmats0[52][49] + coeff0_53*dmats0[53][49] + coeff0_54*dmats0[54][49] + coeff0_55*dmats0[55][49];
          new_coeff0_50 = coeff0_0*dmats0[0][50] + coeff0_1*dmats0[1][50] + coeff0_2*dmats0[2][50] + coeff0_3*dmats0[3][50] + coeff0_4*dmats0[4][50] + coeff0_5*dmats0[5][50] + coeff0_6*dmats0[6][50] + coeff0_7*dmats0[7][50] + coeff0_8*dmats0[8][50] + coeff0_9*dmats0[9][50] + coeff0_10*dmats0[10][50] + coeff0_11*dmats0[11][50] + coeff0_12*dmats0[12][50] + coeff0_13*dmats0[13][50] + coeff0_14*dmats0[14][50] + coeff0_15*dmats0[15][50] + coeff0_16*dmats0[16][50] + coeff0_17*dmats0[17][50] + coeff0_18*dmats0[18][50] + coeff0_19*dmats0[19][50] + coeff0_20*dmats0[20][50] + coeff0_21*dmats0[21][50] + coeff0_22*dmats0[22][50] + coeff0_23*dmats0[23][50] + coeff0_24*dmats0[24][50] + coeff0_25*dmats0[25][50] + coeff0_26*dmats0[26][50] + coeff0_27*dmats0[27][50] + coeff0_28*dmats0[28][50] + coeff0_29*dmats0[29][50] + coeff0_30*dmats0[30][50] + coeff0_31*dmats0[31][50] + coeff0_32*dmats0[32][50] + coeff0_33*dmats0[33][50] + coeff0_34*dmats0[34][50] + coeff0_35*dmats0[35][50] + coeff0_36*dmats0[36][50] + coeff0_37*dmats0[37][50] + coeff0_38*dmats0[38][50] + coeff0_39*dmats0[39][50] + coeff0_40*dmats0[40][50] + coeff0_41*dmats0[41][50] + coeff0_42*dmats0[42][50] + coeff0_43*dmats0[43][50] + coeff0_44*dmats0[44][50] + coeff0_45*dmats0[45][50] + coeff0_46*dmats0[46][50] + coeff0_47*dmats0[47][50] + coeff0_48*dmats0[48][50] + coeff0_49*dmats0[49][50] + coeff0_50*dmats0[50][50] + coeff0_51*dmats0[51][50] + coeff0_52*dmats0[52][50] + coeff0_53*dmats0[53][50] + coeff0_54*dmats0[54][50] + coeff0_55*dmats0[55][50];
          new_coeff0_51 = coeff0_0*dmats0[0][51] + coeff0_1*dmats0[1][51] + coeff0_2*dmats0[2][51] + coeff0_3*dmats0[3][51] + coeff0_4*dmats0[4][51] + coeff0_5*dmats0[5][51] + coeff0_6*dmats0[6][51] + coeff0_7*dmats0[7][51] + coeff0_8*dmats0[8][51] + coeff0_9*dmats0[9][51] + coeff0_10*dmats0[10][51] + coeff0_11*dmats0[11][51] + coeff0_12*dmats0[12][51] + coeff0_13*dmats0[13][51] + coeff0_14*dmats0[14][51] + coeff0_15*dmats0[15][51] + coeff0_16*dmats0[16][51] + coeff0_17*dmats0[17][51] + coeff0_18*dmats0[18][51] + coeff0_19*dmats0[19][51] + coeff0_20*dmats0[20][51] + coeff0_21*dmats0[21][51] + coeff0_22*dmats0[22][51] + coeff0_23*dmats0[23][51] + coeff0_24*dmats0[24][51] + coeff0_25*dmats0[25][51] + coeff0_26*dmats0[26][51] + coeff0_27*dmats0[27][51] + coeff0_28*dmats0[28][51] + coeff0_29*dmats0[29][51] + coeff0_30*dmats0[30][51] + coeff0_31*dmats0[31][51] + coeff0_32*dmats0[32][51] + coeff0_33*dmats0[33][51] + coeff0_34*dmats0[34][51] + coeff0_35*dmats0[35][51] + coeff0_36*dmats0[36][51] + coeff0_37*dmats0[37][51] + coeff0_38*dmats0[38][51] + coeff0_39*dmats0[39][51] + coeff0_40*dmats0[40][51] + coeff0_41*dmats0[41][51] + coeff0_42*dmats0[42][51] + coeff0_43*dmats0[43][51] + coeff0_44*dmats0[44][51] + coeff0_45*dmats0[45][51] + coeff0_46*dmats0[46][51] + coeff0_47*dmats0[47][51] + coeff0_48*dmats0[48][51] + coeff0_49*dmats0[49][51] + coeff0_50*dmats0[50][51] + coeff0_51*dmats0[51][51] + coeff0_52*dmats0[52][51] + coeff0_53*dmats0[53][51] + coeff0_54*dmats0[54][51] + coeff0_55*dmats0[55][51];
          new_coeff0_52 = coeff0_0*dmats0[0][52] + coeff0_1*dmats0[1][52] + coeff0_2*dmats0[2][52] + coeff0_3*dmats0[3][52] + coeff0_4*dmats0[4][52] + coeff0_5*dmats0[5][52] + coeff0_6*dmats0[6][52] + coeff0_7*dmats0[7][52] + coeff0_8*dmats0[8][52] + coeff0_9*dmats0[9][52] + coeff0_10*dmats0[10][52] + coeff0_11*dmats0[11][52] + coeff0_12*dmats0[12][52] + coeff0_13*dmats0[13][52] + coeff0_14*dmats0[14][52] + coeff0_15*dmats0[15][52] + coeff0_16*dmats0[16][52] + coeff0_17*dmats0[17][52] + coeff0_18*dmats0[18][52] + coeff0_19*dmats0[19][52] + coeff0_20*dmats0[20][52] + coeff0_21*dmats0[21][52] + coeff0_22*dmats0[22][52] + coeff0_23*dmats0[23][52] + coeff0_24*dmats0[24][52] + coeff0_25*dmats0[25][52] + coeff0_26*dmats0[26][52] + coeff0_27*dmats0[27][52] + coeff0_28*dmats0[28][52] + coeff0_29*dmats0[29][52] + coeff0_30*dmats0[30][52] + coeff0_31*dmats0[31][52] + coeff0_32*dmats0[32][52] + coeff0_33*dmats0[33][52] + coeff0_34*dmats0[34][52] + coeff0_35*dmats0[35][52] + coeff0_36*dmats0[36][52] + coeff0_37*dmats0[37][52] + coeff0_38*dmats0[38][52] + coeff0_39*dmats0[39][52] + coeff0_40*dmats0[40][52] + coeff0_41*dmats0[41][52] + coeff0_42*dmats0[42][52] + coeff0_43*dmats0[43][52] + coeff0_44*dmats0[44][52] + coeff0_45*dmats0[45][52] + coeff0_46*dmats0[46][52] + coeff0_47*dmats0[47][52] + coeff0_48*dmats0[48][52] + coeff0_49*dmats0[49][52] + coeff0_50*dmats0[50][52] + coeff0_51*dmats0[51][52] + coeff0_52*dmats0[52][52] + coeff0_53*dmats0[53][52] + coeff0_54*dmats0[54][52] + coeff0_55*dmats0[55][52];
          new_coeff0_53 = coeff0_0*dmats0[0][53] + coeff0_1*dmats0[1][53] + coeff0_2*dmats0[2][53] + coeff0_3*dmats0[3][53] + coeff0_4*dmats0[4][53] + coeff0_5*dmats0[5][53] + coeff0_6*dmats0[6][53] + coeff0_7*dmats0[7][53] + coeff0_8*dmats0[8][53] + coeff0_9*dmats0[9][53] + coeff0_10*dmats0[10][53] + coeff0_11*dmats0[11][53] + coeff0_12*dmats0[12][53] + coeff0_13*dmats0[13][53] + coeff0_14*dmats0[14][53] + coeff0_15*dmats0[15][53] + coeff0_16*dmats0[16][53] + coeff0_17*dmats0[17][53] + coeff0_18*dmats0[18][53] + coeff0_19*dmats0[19][53] + coeff0_20*dmats0[20][53] + coeff0_21*dmats0[21][53] + coeff0_22*dmats0[22][53] + coeff0_23*dmats0[23][53] + coeff0_24*dmats0[24][53] + coeff0_25*dmats0[25][53] + coeff0_26*dmats0[26][53] + coeff0_27*dmats0[27][53] + coeff0_28*dmats0[28][53] + coeff0_29*dmats0[29][53] + coeff0_30*dmats0[30][53] + coeff0_31*dmats0[31][53] + coeff0_32*dmats0[32][53] + coeff0_33*dmats0[33][53] + coeff0_34*dmats0[34][53] + coeff0_35*dmats0[35][53] + coeff0_36*dmats0[36][53] + coeff0_37*dmats0[37][53] + coeff0_38*dmats0[38][53] + coeff0_39*dmats0[39][53] + coeff0_40*dmats0[40][53] + coeff0_41*dmats0[41][53] + coeff0_42*dmats0[42][53] + coeff0_43*dmats0[43][53] + coeff0_44*dmats0[44][53] + coeff0_45*dmats0[45][53] + coeff0_46*dmats0[46][53] + coeff0_47*dmats0[47][53] + coeff0_48*dmats0[48][53] + coeff0_49*dmats0[49][53] + coeff0_50*dmats0[50][53] + coeff0_51*dmats0[51][53] + coeff0_52*dmats0[52][53] + coeff0_53*dmats0[53][53] + coeff0_54*dmats0[54][53] + coeff0_55*dmats0[55][53];
          new_coeff0_54 = coeff0_0*dmats0[0][54] + coeff0_1*dmats0[1][54] + coeff0_2*dmats0[2][54] + coeff0_3*dmats0[3][54] + coeff0_4*dmats0[4][54] + coeff0_5*dmats0[5][54] + coeff0_6*dmats0[6][54] + coeff0_7*dmats0[7][54] + coeff0_8*dmats0[8][54] + coeff0_9*dmats0[9][54] + coeff0_10*dmats0[10][54] + coeff0_11*dmats0[11][54] + coeff0_12*dmats0[12][54] + coeff0_13*dmats0[13][54] + coeff0_14*dmats0[14][54] + coeff0_15*dmats0[15][54] + coeff0_16*dmats0[16][54] + coeff0_17*dmats0[17][54] + coeff0_18*dmats0[18][54] + coeff0_19*dmats0[19][54] + coeff0_20*dmats0[20][54] + coeff0_21*dmats0[21][54] + coeff0_22*dmats0[22][54] + coeff0_23*dmats0[23][54] + coeff0_24*dmats0[24][54] + coeff0_25*dmats0[25][54] + coeff0_26*dmats0[26][54] + coeff0_27*dmats0[27][54] + coeff0_28*dmats0[28][54] + coeff0_29*dmats0[29][54] + coeff0_30*dmats0[30][54] + coeff0_31*dmats0[31][54] + coeff0_32*dmats0[32][54] + coeff0_33*dmats0[33][54] + coeff0_34*dmats0[34][54] + coeff0_35*dmats0[35][54] + coeff0_36*dmats0[36][54] + coeff0_37*dmats0[37][54] + coeff0_38*dmats0[38][54] + coeff0_39*dmats0[39][54] + coeff0_40*dmats0[40][54] + coeff0_41*dmats0[41][54] + coeff0_42*dmats0[42][54] + coeff0_43*dmats0[43][54] + coeff0_44*dmats0[44][54] + coeff0_45*dmats0[45][54] + coeff0_46*dmats0[46][54] + coeff0_47*dmats0[47][54] + coeff0_48*dmats0[48][54] + coeff0_49*dmats0[49][54] + coeff0_50*dmats0[50][54] + coeff0_51*dmats0[51][54] + coeff0_52*dmats0[52][54] + coeff0_53*dmats0[53][54] + coeff0_54*dmats0[54][54] + coeff0_55*dmats0[55][54];
          new_coeff0_55 = coeff0_0*dmats0[0][55] + coeff0_1*dmats0[1][55] + coeff0_2*dmats0[2][55] + coeff0_3*dmats0[3][55] + coeff0_4*dmats0[4][55] + coeff0_5*dmats0[5][55] + coeff0_6*dmats0[6][55] + coeff0_7*dmats0[7][55] + coeff0_8*dmats0[8][55] + coeff0_9*dmats0[9][55] + coeff0_10*dmats0[10][55] + coeff0_11*dmats0[11][55] + coeff0_12*dmats0[12][55] + coeff0_13*dmats0[13][55] + coeff0_14*dmats0[14][55] + coeff0_15*dmats0[15][55] + coeff0_16*dmats0[16][55] + coeff0_17*dmats0[17][55] + coeff0_18*dmats0[18][55] + coeff0_19*dmats0[19][55] + coeff0_20*dmats0[20][55] + coeff0_21*dmats0[21][55] + coeff0_22*dmats0[22][55] + coeff0_23*dmats0[23][55] + coeff0_24*dmats0[24][55] + coeff0_25*dmats0[25][55] + coeff0_26*dmats0[26][55] + coeff0_27*dmats0[27][55] + coeff0_28*dmats0[28][55] + coeff0_29*dmats0[29][55] + coeff0_30*dmats0[30][55] + coeff0_31*dmats0[31][55] + coeff0_32*dmats0[32][55] + coeff0_33*dmats0[33][55] + coeff0_34*dmats0[34][55] + coeff0_35*dmats0[35][55] + coeff0_36*dmats0[36][55] + coeff0_37*dmats0[37][55] + coeff0_38*dmats0[38][55] + coeff0_39*dmats0[39][55] + coeff0_40*dmats0[40][55] + coeff0_41*dmats0[41][55] + coeff0_42*dmats0[42][55] + coeff0_43*dmats0[43][55] + coeff0_44*dmats0[44][55] + coeff0_45*dmats0[45][55] + coeff0_46*dmats0[46][55] + coeff0_47*dmats0[47][55] + coeff0_48*dmats0[48][55] + coeff0_49*dmats0[49][55] + coeff0_50*dmats0[50][55] + coeff0_51*dmats0[51][55] + coeff0_52*dmats0[52][55] + coeff0_53*dmats0[53][55] + coeff0_54*dmats0[54][55] + coeff0_55*dmats0[55][55];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0] + coeff0_10*dmats1[10][0] + coeff0_11*dmats1[11][0] + coeff0_12*dmats1[12][0] + coeff0_13*dmats1[13][0] + coeff0_14*dmats1[14][0] + coeff0_15*dmats1[15][0] + coeff0_16*dmats1[16][0] + coeff0_17*dmats1[17][0] + coeff0_18*dmats1[18][0] + coeff0_19*dmats1[19][0] + coeff0_20*dmats1[20][0] + coeff0_21*dmats1[21][0] + coeff0_22*dmats1[22][0] + coeff0_23*dmats1[23][0] + coeff0_24*dmats1[24][0] + coeff0_25*dmats1[25][0] + coeff0_26*dmats1[26][0] + coeff0_27*dmats1[27][0] + coeff0_28*dmats1[28][0] + coeff0_29*dmats1[29][0] + coeff0_30*dmats1[30][0] + coeff0_31*dmats1[31][0] + coeff0_32*dmats1[32][0] + coeff0_33*dmats1[33][0] + coeff0_34*dmats1[34][0] + coeff0_35*dmats1[35][0] + coeff0_36*dmats1[36][0] + coeff0_37*dmats1[37][0] + coeff0_38*dmats1[38][0] + coeff0_39*dmats1[39][0] + coeff0_40*dmats1[40][0] + coeff0_41*dmats1[41][0] + coeff0_42*dmats1[42][0] + coeff0_43*dmats1[43][0] + coeff0_44*dmats1[44][0] + coeff0_45*dmats1[45][0] + coeff0_46*dmats1[46][0] + coeff0_47*dmats1[47][0] + coeff0_48*dmats1[48][0] + coeff0_49*dmats1[49][0] + coeff0_50*dmats1[50][0] + coeff0_51*dmats1[51][0] + coeff0_52*dmats1[52][0] + coeff0_53*dmats1[53][0] + coeff0_54*dmats1[54][0] + coeff0_55*dmats1[55][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1] + coeff0_10*dmats1[10][1] + coeff0_11*dmats1[11][1] + coeff0_12*dmats1[12][1] + coeff0_13*dmats1[13][1] + coeff0_14*dmats1[14][1] + coeff0_15*dmats1[15][1] + coeff0_16*dmats1[16][1] + coeff0_17*dmats1[17][1] + coeff0_18*dmats1[18][1] + coeff0_19*dmats1[19][1] + coeff0_20*dmats1[20][1] + coeff0_21*dmats1[21][1] + coeff0_22*dmats1[22][1] + coeff0_23*dmats1[23][1] + coeff0_24*dmats1[24][1] + coeff0_25*dmats1[25][1] + coeff0_26*dmats1[26][1] + coeff0_27*dmats1[27][1] + coeff0_28*dmats1[28][1] + coeff0_29*dmats1[29][1] + coeff0_30*dmats1[30][1] + coeff0_31*dmats1[31][1] + coeff0_32*dmats1[32][1] + coeff0_33*dmats1[33][1] + coeff0_34*dmats1[34][1] + coeff0_35*dmats1[35][1] + coeff0_36*dmats1[36][1] + coeff0_37*dmats1[37][1] + coeff0_38*dmats1[38][1] + coeff0_39*dmats1[39][1] + coeff0_40*dmats1[40][1] + coeff0_41*dmats1[41][1] + coeff0_42*dmats1[42][1] + coeff0_43*dmats1[43][1] + coeff0_44*dmats1[44][1] + coeff0_45*dmats1[45][1] + coeff0_46*dmats1[46][1] + coeff0_47*dmats1[47][1] + coeff0_48*dmats1[48][1] + coeff0_49*dmats1[49][1] + coeff0_50*dmats1[50][1] + coeff0_51*dmats1[51][1] + coeff0_52*dmats1[52][1] + coeff0_53*dmats1[53][1] + coeff0_54*dmats1[54][1] + coeff0_55*dmats1[55][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2] + coeff0_10*dmats1[10][2] + coeff0_11*dmats1[11][2] + coeff0_12*dmats1[12][2] + coeff0_13*dmats1[13][2] + coeff0_14*dmats1[14][2] + coeff0_15*dmats1[15][2] + coeff0_16*dmats1[16][2] + coeff0_17*dmats1[17][2] + coeff0_18*dmats1[18][2] + coeff0_19*dmats1[19][2] + coeff0_20*dmats1[20][2] + coeff0_21*dmats1[21][2] + coeff0_22*dmats1[22][2] + coeff0_23*dmats1[23][2] + coeff0_24*dmats1[24][2] + coeff0_25*dmats1[25][2] + coeff0_26*dmats1[26][2] + coeff0_27*dmats1[27][2] + coeff0_28*dmats1[28][2] + coeff0_29*dmats1[29][2] + coeff0_30*dmats1[30][2] + coeff0_31*dmats1[31][2] + coeff0_32*dmats1[32][2] + coeff0_33*dmats1[33][2] + coeff0_34*dmats1[34][2] + coeff0_35*dmats1[35][2] + coeff0_36*dmats1[36][2] + coeff0_37*dmats1[37][2] + coeff0_38*dmats1[38][2] + coeff0_39*dmats1[39][2] + coeff0_40*dmats1[40][2] + coeff0_41*dmats1[41][2] + coeff0_42*dmats1[42][2] + coeff0_43*dmats1[43][2] + coeff0_44*dmats1[44][2] + coeff0_45*dmats1[45][2] + coeff0_46*dmats1[46][2] + coeff0_47*dmats1[47][2] + coeff0_48*dmats1[48][2] + coeff0_49*dmats1[49][2] + coeff0_50*dmats1[50][2] + coeff0_51*dmats1[51][2] + coeff0_52*dmats1[52][2] + coeff0_53*dmats1[53][2] + coeff0_54*dmats1[54][2] + coeff0_55*dmats1[55][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3] + coeff0_10*dmats1[10][3] + coeff0_11*dmats1[11][3] + coeff0_12*dmats1[12][3] + coeff0_13*dmats1[13][3] + coeff0_14*dmats1[14][3] + coeff0_15*dmats1[15][3] + coeff0_16*dmats1[16][3] + coeff0_17*dmats1[17][3] + coeff0_18*dmats1[18][3] + coeff0_19*dmats1[19][3] + coeff0_20*dmats1[20][3] + coeff0_21*dmats1[21][3] + coeff0_22*dmats1[22][3] + coeff0_23*dmats1[23][3] + coeff0_24*dmats1[24][3] + coeff0_25*dmats1[25][3] + coeff0_26*dmats1[26][3] + coeff0_27*dmats1[27][3] + coeff0_28*dmats1[28][3] + coeff0_29*dmats1[29][3] + coeff0_30*dmats1[30][3] + coeff0_31*dmats1[31][3] + coeff0_32*dmats1[32][3] + coeff0_33*dmats1[33][3] + coeff0_34*dmats1[34][3] + coeff0_35*dmats1[35][3] + coeff0_36*dmats1[36][3] + coeff0_37*dmats1[37][3] + coeff0_38*dmats1[38][3] + coeff0_39*dmats1[39][3] + coeff0_40*dmats1[40][3] + coeff0_41*dmats1[41][3] + coeff0_42*dmats1[42][3] + coeff0_43*dmats1[43][3] + coeff0_44*dmats1[44][3] + coeff0_45*dmats1[45][3] + coeff0_46*dmats1[46][3] + coeff0_47*dmats1[47][3] + coeff0_48*dmats1[48][3] + coeff0_49*dmats1[49][3] + coeff0_50*dmats1[50][3] + coeff0_51*dmats1[51][3] + coeff0_52*dmats1[52][3] + coeff0_53*dmats1[53][3] + coeff0_54*dmats1[54][3] + coeff0_55*dmats1[55][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4] + coeff0_10*dmats1[10][4] + coeff0_11*dmats1[11][4] + coeff0_12*dmats1[12][4] + coeff0_13*dmats1[13][4] + coeff0_14*dmats1[14][4] + coeff0_15*dmats1[15][4] + coeff0_16*dmats1[16][4] + coeff0_17*dmats1[17][4] + coeff0_18*dmats1[18][4] + coeff0_19*dmats1[19][4] + coeff0_20*dmats1[20][4] + coeff0_21*dmats1[21][4] + coeff0_22*dmats1[22][4] + coeff0_23*dmats1[23][4] + coeff0_24*dmats1[24][4] + coeff0_25*dmats1[25][4] + coeff0_26*dmats1[26][4] + coeff0_27*dmats1[27][4] + coeff0_28*dmats1[28][4] + coeff0_29*dmats1[29][4] + coeff0_30*dmats1[30][4] + coeff0_31*dmats1[31][4] + coeff0_32*dmats1[32][4] + coeff0_33*dmats1[33][4] + coeff0_34*dmats1[34][4] + coeff0_35*dmats1[35][4] + coeff0_36*dmats1[36][4] + coeff0_37*dmats1[37][4] + coeff0_38*dmats1[38][4] + coeff0_39*dmats1[39][4] + coeff0_40*dmats1[40][4] + coeff0_41*dmats1[41][4] + coeff0_42*dmats1[42][4] + coeff0_43*dmats1[43][4] + coeff0_44*dmats1[44][4] + coeff0_45*dmats1[45][4] + coeff0_46*dmats1[46][4] + coeff0_47*dmats1[47][4] + coeff0_48*dmats1[48][4] + coeff0_49*dmats1[49][4] + coeff0_50*dmats1[50][4] + coeff0_51*dmats1[51][4] + coeff0_52*dmats1[52][4] + coeff0_53*dmats1[53][4] + coeff0_54*dmats1[54][4] + coeff0_55*dmats1[55][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5] + coeff0_10*dmats1[10][5] + coeff0_11*dmats1[11][5] + coeff0_12*dmats1[12][5] + coeff0_13*dmats1[13][5] + coeff0_14*dmats1[14][5] + coeff0_15*dmats1[15][5] + coeff0_16*dmats1[16][5] + coeff0_17*dmats1[17][5] + coeff0_18*dmats1[18][5] + coeff0_19*dmats1[19][5] + coeff0_20*dmats1[20][5] + coeff0_21*dmats1[21][5] + coeff0_22*dmats1[22][5] + coeff0_23*dmats1[23][5] + coeff0_24*dmats1[24][5] + coeff0_25*dmats1[25][5] + coeff0_26*dmats1[26][5] + coeff0_27*dmats1[27][5] + coeff0_28*dmats1[28][5] + coeff0_29*dmats1[29][5] + coeff0_30*dmats1[30][5] + coeff0_31*dmats1[31][5] + coeff0_32*dmats1[32][5] + coeff0_33*dmats1[33][5] + coeff0_34*dmats1[34][5] + coeff0_35*dmats1[35][5] + coeff0_36*dmats1[36][5] + coeff0_37*dmats1[37][5] + coeff0_38*dmats1[38][5] + coeff0_39*dmats1[39][5] + coeff0_40*dmats1[40][5] + coeff0_41*dmats1[41][5] + coeff0_42*dmats1[42][5] + coeff0_43*dmats1[43][5] + coeff0_44*dmats1[44][5] + coeff0_45*dmats1[45][5] + coeff0_46*dmats1[46][5] + coeff0_47*dmats1[47][5] + coeff0_48*dmats1[48][5] + coeff0_49*dmats1[49][5] + coeff0_50*dmats1[50][5] + coeff0_51*dmats1[51][5] + coeff0_52*dmats1[52][5] + coeff0_53*dmats1[53][5] + coeff0_54*dmats1[54][5] + coeff0_55*dmats1[55][5];
          new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6] + coeff0_10*dmats1[10][6] + coeff0_11*dmats1[11][6] + coeff0_12*dmats1[12][6] + coeff0_13*dmats1[13][6] + coeff0_14*dmats1[14][6] + coeff0_15*dmats1[15][6] + coeff0_16*dmats1[16][6] + coeff0_17*dmats1[17][6] + coeff0_18*dmats1[18][6] + coeff0_19*dmats1[19][6] + coeff0_20*dmats1[20][6] + coeff0_21*dmats1[21][6] + coeff0_22*dmats1[22][6] + coeff0_23*dmats1[23][6] + coeff0_24*dmats1[24][6] + coeff0_25*dmats1[25][6] + coeff0_26*dmats1[26][6] + coeff0_27*dmats1[27][6] + coeff0_28*dmats1[28][6] + coeff0_29*dmats1[29][6] + coeff0_30*dmats1[30][6] + coeff0_31*dmats1[31][6] + coeff0_32*dmats1[32][6] + coeff0_33*dmats1[33][6] + coeff0_34*dmats1[34][6] + coeff0_35*dmats1[35][6] + coeff0_36*dmats1[36][6] + coeff0_37*dmats1[37][6] + coeff0_38*dmats1[38][6] + coeff0_39*dmats1[39][6] + coeff0_40*dmats1[40][6] + coeff0_41*dmats1[41][6] + coeff0_42*dmats1[42][6] + coeff0_43*dmats1[43][6] + coeff0_44*dmats1[44][6] + coeff0_45*dmats1[45][6] + coeff0_46*dmats1[46][6] + coeff0_47*dmats1[47][6] + coeff0_48*dmats1[48][6] + coeff0_49*dmats1[49][6] + coeff0_50*dmats1[50][6] + coeff0_51*dmats1[51][6] + coeff0_52*dmats1[52][6] + coeff0_53*dmats1[53][6] + coeff0_54*dmats1[54][6] + coeff0_55*dmats1[55][6];
          new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7] + coeff0_10*dmats1[10][7] + coeff0_11*dmats1[11][7] + coeff0_12*dmats1[12][7] + coeff0_13*dmats1[13][7] + coeff0_14*dmats1[14][7] + coeff0_15*dmats1[15][7] + coeff0_16*dmats1[16][7] + coeff0_17*dmats1[17][7] + coeff0_18*dmats1[18][7] + coeff0_19*dmats1[19][7] + coeff0_20*dmats1[20][7] + coeff0_21*dmats1[21][7] + coeff0_22*dmats1[22][7] + coeff0_23*dmats1[23][7] + coeff0_24*dmats1[24][7] + coeff0_25*dmats1[25][7] + coeff0_26*dmats1[26][7] + coeff0_27*dmats1[27][7] + coeff0_28*dmats1[28][7] + coeff0_29*dmats1[29][7] + coeff0_30*dmats1[30][7] + coeff0_31*dmats1[31][7] + coeff0_32*dmats1[32][7] + coeff0_33*dmats1[33][7] + coeff0_34*dmats1[34][7] + coeff0_35*dmats1[35][7] + coeff0_36*dmats1[36][7] + coeff0_37*dmats1[37][7] + coeff0_38*dmats1[38][7] + coeff0_39*dmats1[39][7] + coeff0_40*dmats1[40][7] + coeff0_41*dmats1[41][7] + coeff0_42*dmats1[42][7] + coeff0_43*dmats1[43][7] + coeff0_44*dmats1[44][7] + coeff0_45*dmats1[45][7] + coeff0_46*dmats1[46][7] + coeff0_47*dmats1[47][7] + coeff0_48*dmats1[48][7] + coeff0_49*dmats1[49][7] + coeff0_50*dmats1[50][7] + coeff0_51*dmats1[51][7] + coeff0_52*dmats1[52][7] + coeff0_53*dmats1[53][7] + coeff0_54*dmats1[54][7] + coeff0_55*dmats1[55][7];
          new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8] + coeff0_10*dmats1[10][8] + coeff0_11*dmats1[11][8] + coeff0_12*dmats1[12][8] + coeff0_13*dmats1[13][8] + coeff0_14*dmats1[14][8] + coeff0_15*dmats1[15][8] + coeff0_16*dmats1[16][8] + coeff0_17*dmats1[17][8] + coeff0_18*dmats1[18][8] + coeff0_19*dmats1[19][8] + coeff0_20*dmats1[20][8] + coeff0_21*dmats1[21][8] + coeff0_22*dmats1[22][8] + coeff0_23*dmats1[23][8] + coeff0_24*dmats1[24][8] + coeff0_25*dmats1[25][8] + coeff0_26*dmats1[26][8] + coeff0_27*dmats1[27][8] + coeff0_28*dmats1[28][8] + coeff0_29*dmats1[29][8] + coeff0_30*dmats1[30][8] + coeff0_31*dmats1[31][8] + coeff0_32*dmats1[32][8] + coeff0_33*dmats1[33][8] + coeff0_34*dmats1[34][8] + coeff0_35*dmats1[35][8] + coeff0_36*dmats1[36][8] + coeff0_37*dmats1[37][8] + coeff0_38*dmats1[38][8] + coeff0_39*dmats1[39][8] + coeff0_40*dmats1[40][8] + coeff0_41*dmats1[41][8] + coeff0_42*dmats1[42][8] + coeff0_43*dmats1[43][8] + coeff0_44*dmats1[44][8] + coeff0_45*dmats1[45][8] + coeff0_46*dmats1[46][8] + coeff0_47*dmats1[47][8] + coeff0_48*dmats1[48][8] + coeff0_49*dmats1[49][8] + coeff0_50*dmats1[50][8] + coeff0_51*dmats1[51][8] + coeff0_52*dmats1[52][8] + coeff0_53*dmats1[53][8] + coeff0_54*dmats1[54][8] + coeff0_55*dmats1[55][8];
          new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9] + coeff0_10*dmats1[10][9] + coeff0_11*dmats1[11][9] + coeff0_12*dmats1[12][9] + coeff0_13*dmats1[13][9] + coeff0_14*dmats1[14][9] + coeff0_15*dmats1[15][9] + coeff0_16*dmats1[16][9] + coeff0_17*dmats1[17][9] + coeff0_18*dmats1[18][9] + coeff0_19*dmats1[19][9] + coeff0_20*dmats1[20][9] + coeff0_21*dmats1[21][9] + coeff0_22*dmats1[22][9] + coeff0_23*dmats1[23][9] + coeff0_24*dmats1[24][9] + coeff0_25*dmats1[25][9] + coeff0_26*dmats1[26][9] + coeff0_27*dmats1[27][9] + coeff0_28*dmats1[28][9] + coeff0_29*dmats1[29][9] + coeff0_30*dmats1[30][9] + coeff0_31*dmats1[31][9] + coeff0_32*dmats1[32][9] + coeff0_33*dmats1[33][9] + coeff0_34*dmats1[34][9] + coeff0_35*dmats1[35][9] + coeff0_36*dmats1[36][9] + coeff0_37*dmats1[37][9] + coeff0_38*dmats1[38][9] + coeff0_39*dmats1[39][9] + coeff0_40*dmats1[40][9] + coeff0_41*dmats1[41][9] + coeff0_42*dmats1[42][9] + coeff0_43*dmats1[43][9] + coeff0_44*dmats1[44][9] + coeff0_45*dmats1[45][9] + coeff0_46*dmats1[46][9] + coeff0_47*dmats1[47][9] + coeff0_48*dmats1[48][9] + coeff0_49*dmats1[49][9] + coeff0_50*dmats1[50][9] + coeff0_51*dmats1[51][9] + coeff0_52*dmats1[52][9] + coeff0_53*dmats1[53][9] + coeff0_54*dmats1[54][9] + coeff0_55*dmats1[55][9];
          new_coeff0_10 = coeff0_0*dmats1[0][10] + coeff0_1*dmats1[1][10] + coeff0_2*dmats1[2][10] + coeff0_3*dmats1[3][10] + coeff0_4*dmats1[4][10] + coeff0_5*dmats1[5][10] + coeff0_6*dmats1[6][10] + coeff0_7*dmats1[7][10] + coeff0_8*dmats1[8][10] + coeff0_9*dmats1[9][10] + coeff0_10*dmats1[10][10] + coeff0_11*dmats1[11][10] + coeff0_12*dmats1[12][10] + coeff0_13*dmats1[13][10] + coeff0_14*dmats1[14][10] + coeff0_15*dmats1[15][10] + coeff0_16*dmats1[16][10] + coeff0_17*dmats1[17][10] + coeff0_18*dmats1[18][10] + coeff0_19*dmats1[19][10] + coeff0_20*dmats1[20][10] + coeff0_21*dmats1[21][10] + coeff0_22*dmats1[22][10] + coeff0_23*dmats1[23][10] + coeff0_24*dmats1[24][10] + coeff0_25*dmats1[25][10] + coeff0_26*dmats1[26][10] + coeff0_27*dmats1[27][10] + coeff0_28*dmats1[28][10] + coeff0_29*dmats1[29][10] + coeff0_30*dmats1[30][10] + coeff0_31*dmats1[31][10] + coeff0_32*dmats1[32][10] + coeff0_33*dmats1[33][10] + coeff0_34*dmats1[34][10] + coeff0_35*dmats1[35][10] + coeff0_36*dmats1[36][10] + coeff0_37*dmats1[37][10] + coeff0_38*dmats1[38][10] + coeff0_39*dmats1[39][10] + coeff0_40*dmats1[40][10] + coeff0_41*dmats1[41][10] + coeff0_42*dmats1[42][10] + coeff0_43*dmats1[43][10] + coeff0_44*dmats1[44][10] + coeff0_45*dmats1[45][10] + coeff0_46*dmats1[46][10] + coeff0_47*dmats1[47][10] + coeff0_48*dmats1[48][10] + coeff0_49*dmats1[49][10] + coeff0_50*dmats1[50][10] + coeff0_51*dmats1[51][10] + coeff0_52*dmats1[52][10] + coeff0_53*dmats1[53][10] + coeff0_54*dmats1[54][10] + coeff0_55*dmats1[55][10];
          new_coeff0_11 = coeff0_0*dmats1[0][11] + coeff0_1*dmats1[1][11] + coeff0_2*dmats1[2][11] + coeff0_3*dmats1[3][11] + coeff0_4*dmats1[4][11] + coeff0_5*dmats1[5][11] + coeff0_6*dmats1[6][11] + coeff0_7*dmats1[7][11] + coeff0_8*dmats1[8][11] + coeff0_9*dmats1[9][11] + coeff0_10*dmats1[10][11] + coeff0_11*dmats1[11][11] + coeff0_12*dmats1[12][11] + coeff0_13*dmats1[13][11] + coeff0_14*dmats1[14][11] + coeff0_15*dmats1[15][11] + coeff0_16*dmats1[16][11] + coeff0_17*dmats1[17][11] + coeff0_18*dmats1[18][11] + coeff0_19*dmats1[19][11] + coeff0_20*dmats1[20][11] + coeff0_21*dmats1[21][11] + coeff0_22*dmats1[22][11] + coeff0_23*dmats1[23][11] + coeff0_24*dmats1[24][11] + coeff0_25*dmats1[25][11] + coeff0_26*dmats1[26][11] + coeff0_27*dmats1[27][11] + coeff0_28*dmats1[28][11] + coeff0_29*dmats1[29][11] + coeff0_30*dmats1[30][11] + coeff0_31*dmats1[31][11] + coeff0_32*dmats1[32][11] + coeff0_33*dmats1[33][11] + coeff0_34*dmats1[34][11] + coeff0_35*dmats1[35][11] + coeff0_36*dmats1[36][11] + coeff0_37*dmats1[37][11] + coeff0_38*dmats1[38][11] + coeff0_39*dmats1[39][11] + coeff0_40*dmats1[40][11] + coeff0_41*dmats1[41][11] + coeff0_42*dmats1[42][11] + coeff0_43*dmats1[43][11] + coeff0_44*dmats1[44][11] + coeff0_45*dmats1[45][11] + coeff0_46*dmats1[46][11] + coeff0_47*dmats1[47][11] + coeff0_48*dmats1[48][11] + coeff0_49*dmats1[49][11] + coeff0_50*dmats1[50][11] + coeff0_51*dmats1[51][11] + coeff0_52*dmats1[52][11] + coeff0_53*dmats1[53][11] + coeff0_54*dmats1[54][11] + coeff0_55*dmats1[55][11];
          new_coeff0_12 = coeff0_0*dmats1[0][12] + coeff0_1*dmats1[1][12] + coeff0_2*dmats1[2][12] + coeff0_3*dmats1[3][12] + coeff0_4*dmats1[4][12] + coeff0_5*dmats1[5][12] + coeff0_6*dmats1[6][12] + coeff0_7*dmats1[7][12] + coeff0_8*dmats1[8][12] + coeff0_9*dmats1[9][12] + coeff0_10*dmats1[10][12] + coeff0_11*dmats1[11][12] + coeff0_12*dmats1[12][12] + coeff0_13*dmats1[13][12] + coeff0_14*dmats1[14][12] + coeff0_15*dmats1[15][12] + coeff0_16*dmats1[16][12] + coeff0_17*dmats1[17][12] + coeff0_18*dmats1[18][12] + coeff0_19*dmats1[19][12] + coeff0_20*dmats1[20][12] + coeff0_21*dmats1[21][12] + coeff0_22*dmats1[22][12] + coeff0_23*dmats1[23][12] + coeff0_24*dmats1[24][12] + coeff0_25*dmats1[25][12] + coeff0_26*dmats1[26][12] + coeff0_27*dmats1[27][12] + coeff0_28*dmats1[28][12] + coeff0_29*dmats1[29][12] + coeff0_30*dmats1[30][12] + coeff0_31*dmats1[31][12] + coeff0_32*dmats1[32][12] + coeff0_33*dmats1[33][12] + coeff0_34*dmats1[34][12] + coeff0_35*dmats1[35][12] + coeff0_36*dmats1[36][12] + coeff0_37*dmats1[37][12] + coeff0_38*dmats1[38][12] + coeff0_39*dmats1[39][12] + coeff0_40*dmats1[40][12] + coeff0_41*dmats1[41][12] + coeff0_42*dmats1[42][12] + coeff0_43*dmats1[43][12] + coeff0_44*dmats1[44][12] + coeff0_45*dmats1[45][12] + coeff0_46*dmats1[46][12] + coeff0_47*dmats1[47][12] + coeff0_48*dmats1[48][12] + coeff0_49*dmats1[49][12] + coeff0_50*dmats1[50][12] + coeff0_51*dmats1[51][12] + coeff0_52*dmats1[52][12] + coeff0_53*dmats1[53][12] + coeff0_54*dmats1[54][12] + coeff0_55*dmats1[55][12];
          new_coeff0_13 = coeff0_0*dmats1[0][13] + coeff0_1*dmats1[1][13] + coeff0_2*dmats1[2][13] + coeff0_3*dmats1[3][13] + coeff0_4*dmats1[4][13] + coeff0_5*dmats1[5][13] + coeff0_6*dmats1[6][13] + coeff0_7*dmats1[7][13] + coeff0_8*dmats1[8][13] + coeff0_9*dmats1[9][13] + coeff0_10*dmats1[10][13] + coeff0_11*dmats1[11][13] + coeff0_12*dmats1[12][13] + coeff0_13*dmats1[13][13] + coeff0_14*dmats1[14][13] + coeff0_15*dmats1[15][13] + coeff0_16*dmats1[16][13] + coeff0_17*dmats1[17][13] + coeff0_18*dmats1[18][13] + coeff0_19*dmats1[19][13] + coeff0_20*dmats1[20][13] + coeff0_21*dmats1[21][13] + coeff0_22*dmats1[22][13] + coeff0_23*dmats1[23][13] + coeff0_24*dmats1[24][13] + coeff0_25*dmats1[25][13] + coeff0_26*dmats1[26][13] + coeff0_27*dmats1[27][13] + coeff0_28*dmats1[28][13] + coeff0_29*dmats1[29][13] + coeff0_30*dmats1[30][13] + coeff0_31*dmats1[31][13] + coeff0_32*dmats1[32][13] + coeff0_33*dmats1[33][13] + coeff0_34*dmats1[34][13] + coeff0_35*dmats1[35][13] + coeff0_36*dmats1[36][13] + coeff0_37*dmats1[37][13] + coeff0_38*dmats1[38][13] + coeff0_39*dmats1[39][13] + coeff0_40*dmats1[40][13] + coeff0_41*dmats1[41][13] + coeff0_42*dmats1[42][13] + coeff0_43*dmats1[43][13] + coeff0_44*dmats1[44][13] + coeff0_45*dmats1[45][13] + coeff0_46*dmats1[46][13] + coeff0_47*dmats1[47][13] + coeff0_48*dmats1[48][13] + coeff0_49*dmats1[49][13] + coeff0_50*dmats1[50][13] + coeff0_51*dmats1[51][13] + coeff0_52*dmats1[52][13] + coeff0_53*dmats1[53][13] + coeff0_54*dmats1[54][13] + coeff0_55*dmats1[55][13];
          new_coeff0_14 = coeff0_0*dmats1[0][14] + coeff0_1*dmats1[1][14] + coeff0_2*dmats1[2][14] + coeff0_3*dmats1[3][14] + coeff0_4*dmats1[4][14] + coeff0_5*dmats1[5][14] + coeff0_6*dmats1[6][14] + coeff0_7*dmats1[7][14] + coeff0_8*dmats1[8][14] + coeff0_9*dmats1[9][14] + coeff0_10*dmats1[10][14] + coeff0_11*dmats1[11][14] + coeff0_12*dmats1[12][14] + coeff0_13*dmats1[13][14] + coeff0_14*dmats1[14][14] + coeff0_15*dmats1[15][14] + coeff0_16*dmats1[16][14] + coeff0_17*dmats1[17][14] + coeff0_18*dmats1[18][14] + coeff0_19*dmats1[19][14] + coeff0_20*dmats1[20][14] + coeff0_21*dmats1[21][14] + coeff0_22*dmats1[22][14] + coeff0_23*dmats1[23][14] + coeff0_24*dmats1[24][14] + coeff0_25*dmats1[25][14] + coeff0_26*dmats1[26][14] + coeff0_27*dmats1[27][14] + coeff0_28*dmats1[28][14] + coeff0_29*dmats1[29][14] + coeff0_30*dmats1[30][14] + coeff0_31*dmats1[31][14] + coeff0_32*dmats1[32][14] + coeff0_33*dmats1[33][14] + coeff0_34*dmats1[34][14] + coeff0_35*dmats1[35][14] + coeff0_36*dmats1[36][14] + coeff0_37*dmats1[37][14] + coeff0_38*dmats1[38][14] + coeff0_39*dmats1[39][14] + coeff0_40*dmats1[40][14] + coeff0_41*dmats1[41][14] + coeff0_42*dmats1[42][14] + coeff0_43*dmats1[43][14] + coeff0_44*dmats1[44][14] + coeff0_45*dmats1[45][14] + coeff0_46*dmats1[46][14] + coeff0_47*dmats1[47][14] + coeff0_48*dmats1[48][14] + coeff0_49*dmats1[49][14] + coeff0_50*dmats1[50][14] + coeff0_51*dmats1[51][14] + coeff0_52*dmats1[52][14] + coeff0_53*dmats1[53][14] + coeff0_54*dmats1[54][14] + coeff0_55*dmats1[55][14];
          new_coeff0_15 = coeff0_0*dmats1[0][15] + coeff0_1*dmats1[1][15] + coeff0_2*dmats1[2][15] + coeff0_3*dmats1[3][15] + coeff0_4*dmats1[4][15] + coeff0_5*dmats1[5][15] + coeff0_6*dmats1[6][15] + coeff0_7*dmats1[7][15] + coeff0_8*dmats1[8][15] + coeff0_9*dmats1[9][15] + coeff0_10*dmats1[10][15] + coeff0_11*dmats1[11][15] + coeff0_12*dmats1[12][15] + coeff0_13*dmats1[13][15] + coeff0_14*dmats1[14][15] + coeff0_15*dmats1[15][15] + coeff0_16*dmats1[16][15] + coeff0_17*dmats1[17][15] + coeff0_18*dmats1[18][15] + coeff0_19*dmats1[19][15] + coeff0_20*dmats1[20][15] + coeff0_21*dmats1[21][15] + coeff0_22*dmats1[22][15] + coeff0_23*dmats1[23][15] + coeff0_24*dmats1[24][15] + coeff0_25*dmats1[25][15] + coeff0_26*dmats1[26][15] + coeff0_27*dmats1[27][15] + coeff0_28*dmats1[28][15] + coeff0_29*dmats1[29][15] + coeff0_30*dmats1[30][15] + coeff0_31*dmats1[31][15] + coeff0_32*dmats1[32][15] + coeff0_33*dmats1[33][15] + coeff0_34*dmats1[34][15] + coeff0_35*dmats1[35][15] + coeff0_36*dmats1[36][15] + coeff0_37*dmats1[37][15] + coeff0_38*dmats1[38][15] + coeff0_39*dmats1[39][15] + coeff0_40*dmats1[40][15] + coeff0_41*dmats1[41][15] + coeff0_42*dmats1[42][15] + coeff0_43*dmats1[43][15] + coeff0_44*dmats1[44][15] + coeff0_45*dmats1[45][15] + coeff0_46*dmats1[46][15] + coeff0_47*dmats1[47][15] + coeff0_48*dmats1[48][15] + coeff0_49*dmats1[49][15] + coeff0_50*dmats1[50][15] + coeff0_51*dmats1[51][15] + coeff0_52*dmats1[52][15] + coeff0_53*dmats1[53][15] + coeff0_54*dmats1[54][15] + coeff0_55*dmats1[55][15];
          new_coeff0_16 = coeff0_0*dmats1[0][16] + coeff0_1*dmats1[1][16] + coeff0_2*dmats1[2][16] + coeff0_3*dmats1[3][16] + coeff0_4*dmats1[4][16] + coeff0_5*dmats1[5][16] + coeff0_6*dmats1[6][16] + coeff0_7*dmats1[7][16] + coeff0_8*dmats1[8][16] + coeff0_9*dmats1[9][16] + coeff0_10*dmats1[10][16] + coeff0_11*dmats1[11][16] + coeff0_12*dmats1[12][16] + coeff0_13*dmats1[13][16] + coeff0_14*dmats1[14][16] + coeff0_15*dmats1[15][16] + coeff0_16*dmats1[16][16] + coeff0_17*dmats1[17][16] + coeff0_18*dmats1[18][16] + coeff0_19*dmats1[19][16] + coeff0_20*dmats1[20][16] + coeff0_21*dmats1[21][16] + coeff0_22*dmats1[22][16] + coeff0_23*dmats1[23][16] + coeff0_24*dmats1[24][16] + coeff0_25*dmats1[25][16] + coeff0_26*dmats1[26][16] + coeff0_27*dmats1[27][16] + coeff0_28*dmats1[28][16] + coeff0_29*dmats1[29][16] + coeff0_30*dmats1[30][16] + coeff0_31*dmats1[31][16] + coeff0_32*dmats1[32][16] + coeff0_33*dmats1[33][16] + coeff0_34*dmats1[34][16] + coeff0_35*dmats1[35][16] + coeff0_36*dmats1[36][16] + coeff0_37*dmats1[37][16] + coeff0_38*dmats1[38][16] + coeff0_39*dmats1[39][16] + coeff0_40*dmats1[40][16] + coeff0_41*dmats1[41][16] + coeff0_42*dmats1[42][16] + coeff0_43*dmats1[43][16] + coeff0_44*dmats1[44][16] + coeff0_45*dmats1[45][16] + coeff0_46*dmats1[46][16] + coeff0_47*dmats1[47][16] + coeff0_48*dmats1[48][16] + coeff0_49*dmats1[49][16] + coeff0_50*dmats1[50][16] + coeff0_51*dmats1[51][16] + coeff0_52*dmats1[52][16] + coeff0_53*dmats1[53][16] + coeff0_54*dmats1[54][16] + coeff0_55*dmats1[55][16];
          new_coeff0_17 = coeff0_0*dmats1[0][17] + coeff0_1*dmats1[1][17] + coeff0_2*dmats1[2][17] + coeff0_3*dmats1[3][17] + coeff0_4*dmats1[4][17] + coeff0_5*dmats1[5][17] + coeff0_6*dmats1[6][17] + coeff0_7*dmats1[7][17] + coeff0_8*dmats1[8][17] + coeff0_9*dmats1[9][17] + coeff0_10*dmats1[10][17] + coeff0_11*dmats1[11][17] + coeff0_12*dmats1[12][17] + coeff0_13*dmats1[13][17] + coeff0_14*dmats1[14][17] + coeff0_15*dmats1[15][17] + coeff0_16*dmats1[16][17] + coeff0_17*dmats1[17][17] + coeff0_18*dmats1[18][17] + coeff0_19*dmats1[19][17] + coeff0_20*dmats1[20][17] + coeff0_21*dmats1[21][17] + coeff0_22*dmats1[22][17] + coeff0_23*dmats1[23][17] + coeff0_24*dmats1[24][17] + coeff0_25*dmats1[25][17] + coeff0_26*dmats1[26][17] + coeff0_27*dmats1[27][17] + coeff0_28*dmats1[28][17] + coeff0_29*dmats1[29][17] + coeff0_30*dmats1[30][17] + coeff0_31*dmats1[31][17] + coeff0_32*dmats1[32][17] + coeff0_33*dmats1[33][17] + coeff0_34*dmats1[34][17] + coeff0_35*dmats1[35][17] + coeff0_36*dmats1[36][17] + coeff0_37*dmats1[37][17] + coeff0_38*dmats1[38][17] + coeff0_39*dmats1[39][17] + coeff0_40*dmats1[40][17] + coeff0_41*dmats1[41][17] + coeff0_42*dmats1[42][17] + coeff0_43*dmats1[43][17] + coeff0_44*dmats1[44][17] + coeff0_45*dmats1[45][17] + coeff0_46*dmats1[46][17] + coeff0_47*dmats1[47][17] + coeff0_48*dmats1[48][17] + coeff0_49*dmats1[49][17] + coeff0_50*dmats1[50][17] + coeff0_51*dmats1[51][17] + coeff0_52*dmats1[52][17] + coeff0_53*dmats1[53][17] + coeff0_54*dmats1[54][17] + coeff0_55*dmats1[55][17];
          new_coeff0_18 = coeff0_0*dmats1[0][18] + coeff0_1*dmats1[1][18] + coeff0_2*dmats1[2][18] + coeff0_3*dmats1[3][18] + coeff0_4*dmats1[4][18] + coeff0_5*dmats1[5][18] + coeff0_6*dmats1[6][18] + coeff0_7*dmats1[7][18] + coeff0_8*dmats1[8][18] + coeff0_9*dmats1[9][18] + coeff0_10*dmats1[10][18] + coeff0_11*dmats1[11][18] + coeff0_12*dmats1[12][18] + coeff0_13*dmats1[13][18] + coeff0_14*dmats1[14][18] + coeff0_15*dmats1[15][18] + coeff0_16*dmats1[16][18] + coeff0_17*dmats1[17][18] + coeff0_18*dmats1[18][18] + coeff0_19*dmats1[19][18] + coeff0_20*dmats1[20][18] + coeff0_21*dmats1[21][18] + coeff0_22*dmats1[22][18] + coeff0_23*dmats1[23][18] + coeff0_24*dmats1[24][18] + coeff0_25*dmats1[25][18] + coeff0_26*dmats1[26][18] + coeff0_27*dmats1[27][18] + coeff0_28*dmats1[28][18] + coeff0_29*dmats1[29][18] + coeff0_30*dmats1[30][18] + coeff0_31*dmats1[31][18] + coeff0_32*dmats1[32][18] + coeff0_33*dmats1[33][18] + coeff0_34*dmats1[34][18] + coeff0_35*dmats1[35][18] + coeff0_36*dmats1[36][18] + coeff0_37*dmats1[37][18] + coeff0_38*dmats1[38][18] + coeff0_39*dmats1[39][18] + coeff0_40*dmats1[40][18] + coeff0_41*dmats1[41][18] + coeff0_42*dmats1[42][18] + coeff0_43*dmats1[43][18] + coeff0_44*dmats1[44][18] + coeff0_45*dmats1[45][18] + coeff0_46*dmats1[46][18] + coeff0_47*dmats1[47][18] + coeff0_48*dmats1[48][18] + coeff0_49*dmats1[49][18] + coeff0_50*dmats1[50][18] + coeff0_51*dmats1[51][18] + coeff0_52*dmats1[52][18] + coeff0_53*dmats1[53][18] + coeff0_54*dmats1[54][18] + coeff0_55*dmats1[55][18];
          new_coeff0_19 = coeff0_0*dmats1[0][19] + coeff0_1*dmats1[1][19] + coeff0_2*dmats1[2][19] + coeff0_3*dmats1[3][19] + coeff0_4*dmats1[4][19] + coeff0_5*dmats1[5][19] + coeff0_6*dmats1[6][19] + coeff0_7*dmats1[7][19] + coeff0_8*dmats1[8][19] + coeff0_9*dmats1[9][19] + coeff0_10*dmats1[10][19] + coeff0_11*dmats1[11][19] + coeff0_12*dmats1[12][19] + coeff0_13*dmats1[13][19] + coeff0_14*dmats1[14][19] + coeff0_15*dmats1[15][19] + coeff0_16*dmats1[16][19] + coeff0_17*dmats1[17][19] + coeff0_18*dmats1[18][19] + coeff0_19*dmats1[19][19] + coeff0_20*dmats1[20][19] + coeff0_21*dmats1[21][19] + coeff0_22*dmats1[22][19] + coeff0_23*dmats1[23][19] + coeff0_24*dmats1[24][19] + coeff0_25*dmats1[25][19] + coeff0_26*dmats1[26][19] + coeff0_27*dmats1[27][19] + coeff0_28*dmats1[28][19] + coeff0_29*dmats1[29][19] + coeff0_30*dmats1[30][19] + coeff0_31*dmats1[31][19] + coeff0_32*dmats1[32][19] + coeff0_33*dmats1[33][19] + coeff0_34*dmats1[34][19] + coeff0_35*dmats1[35][19] + coeff0_36*dmats1[36][19] + coeff0_37*dmats1[37][19] + coeff0_38*dmats1[38][19] + coeff0_39*dmats1[39][19] + coeff0_40*dmats1[40][19] + coeff0_41*dmats1[41][19] + coeff0_42*dmats1[42][19] + coeff0_43*dmats1[43][19] + coeff0_44*dmats1[44][19] + coeff0_45*dmats1[45][19] + coeff0_46*dmats1[46][19] + coeff0_47*dmats1[47][19] + coeff0_48*dmats1[48][19] + coeff0_49*dmats1[49][19] + coeff0_50*dmats1[50][19] + coeff0_51*dmats1[51][19] + coeff0_52*dmats1[52][19] + coeff0_53*dmats1[53][19] + coeff0_54*dmats1[54][19] + coeff0_55*dmats1[55][19];
          new_coeff0_20 = coeff0_0*dmats1[0][20] + coeff0_1*dmats1[1][20] + coeff0_2*dmats1[2][20] + coeff0_3*dmats1[3][20] + coeff0_4*dmats1[4][20] + coeff0_5*dmats1[5][20] + coeff0_6*dmats1[6][20] + coeff0_7*dmats1[7][20] + coeff0_8*dmats1[8][20] + coeff0_9*dmats1[9][20] + coeff0_10*dmats1[10][20] + coeff0_11*dmats1[11][20] + coeff0_12*dmats1[12][20] + coeff0_13*dmats1[13][20] + coeff0_14*dmats1[14][20] + coeff0_15*dmats1[15][20] + coeff0_16*dmats1[16][20] + coeff0_17*dmats1[17][20] + coeff0_18*dmats1[18][20] + coeff0_19*dmats1[19][20] + coeff0_20*dmats1[20][20] + coeff0_21*dmats1[21][20] + coeff0_22*dmats1[22][20] + coeff0_23*dmats1[23][20] + coeff0_24*dmats1[24][20] + coeff0_25*dmats1[25][20] + coeff0_26*dmats1[26][20] + coeff0_27*dmats1[27][20] + coeff0_28*dmats1[28][20] + coeff0_29*dmats1[29][20] + coeff0_30*dmats1[30][20] + coeff0_31*dmats1[31][20] + coeff0_32*dmats1[32][20] + coeff0_33*dmats1[33][20] + coeff0_34*dmats1[34][20] + coeff0_35*dmats1[35][20] + coeff0_36*dmats1[36][20] + coeff0_37*dmats1[37][20] + coeff0_38*dmats1[38][20] + coeff0_39*dmats1[39][20] + coeff0_40*dmats1[40][20] + coeff0_41*dmats1[41][20] + coeff0_42*dmats1[42][20] + coeff0_43*dmats1[43][20] + coeff0_44*dmats1[44][20] + coeff0_45*dmats1[45][20] + coeff0_46*dmats1[46][20] + coeff0_47*dmats1[47][20] + coeff0_48*dmats1[48][20] + coeff0_49*dmats1[49][20] + coeff0_50*dmats1[50][20] + coeff0_51*dmats1[51][20] + coeff0_52*dmats1[52][20] + coeff0_53*dmats1[53][20] + coeff0_54*dmats1[54][20] + coeff0_55*dmats1[55][20];
          new_coeff0_21 = coeff0_0*dmats1[0][21] + coeff0_1*dmats1[1][21] + coeff0_2*dmats1[2][21] + coeff0_3*dmats1[3][21] + coeff0_4*dmats1[4][21] + coeff0_5*dmats1[5][21] + coeff0_6*dmats1[6][21] + coeff0_7*dmats1[7][21] + coeff0_8*dmats1[8][21] + coeff0_9*dmats1[9][21] + coeff0_10*dmats1[10][21] + coeff0_11*dmats1[11][21] + coeff0_12*dmats1[12][21] + coeff0_13*dmats1[13][21] + coeff0_14*dmats1[14][21] + coeff0_15*dmats1[15][21] + coeff0_16*dmats1[16][21] + coeff0_17*dmats1[17][21] + coeff0_18*dmats1[18][21] + coeff0_19*dmats1[19][21] + coeff0_20*dmats1[20][21] + coeff0_21*dmats1[21][21] + coeff0_22*dmats1[22][21] + coeff0_23*dmats1[23][21] + coeff0_24*dmats1[24][21] + coeff0_25*dmats1[25][21] + coeff0_26*dmats1[26][21] + coeff0_27*dmats1[27][21] + coeff0_28*dmats1[28][21] + coeff0_29*dmats1[29][21] + coeff0_30*dmats1[30][21] + coeff0_31*dmats1[31][21] + coeff0_32*dmats1[32][21] + coeff0_33*dmats1[33][21] + coeff0_34*dmats1[34][21] + coeff0_35*dmats1[35][21] + coeff0_36*dmats1[36][21] + coeff0_37*dmats1[37][21] + coeff0_38*dmats1[38][21] + coeff0_39*dmats1[39][21] + coeff0_40*dmats1[40][21] + coeff0_41*dmats1[41][21] + coeff0_42*dmats1[42][21] + coeff0_43*dmats1[43][21] + coeff0_44*dmats1[44][21] + coeff0_45*dmats1[45][21] + coeff0_46*dmats1[46][21] + coeff0_47*dmats1[47][21] + coeff0_48*dmats1[48][21] + coeff0_49*dmats1[49][21] + coeff0_50*dmats1[50][21] + coeff0_51*dmats1[51][21] + coeff0_52*dmats1[52][21] + coeff0_53*dmats1[53][21] + coeff0_54*dmats1[54][21] + coeff0_55*dmats1[55][21];
          new_coeff0_22 = coeff0_0*dmats1[0][22] + coeff0_1*dmats1[1][22] + coeff0_2*dmats1[2][22] + coeff0_3*dmats1[3][22] + coeff0_4*dmats1[4][22] + coeff0_5*dmats1[5][22] + coeff0_6*dmats1[6][22] + coeff0_7*dmats1[7][22] + coeff0_8*dmats1[8][22] + coeff0_9*dmats1[9][22] + coeff0_10*dmats1[10][22] + coeff0_11*dmats1[11][22] + coeff0_12*dmats1[12][22] + coeff0_13*dmats1[13][22] + coeff0_14*dmats1[14][22] + coeff0_15*dmats1[15][22] + coeff0_16*dmats1[16][22] + coeff0_17*dmats1[17][22] + coeff0_18*dmats1[18][22] + coeff0_19*dmats1[19][22] + coeff0_20*dmats1[20][22] + coeff0_21*dmats1[21][22] + coeff0_22*dmats1[22][22] + coeff0_23*dmats1[23][22] + coeff0_24*dmats1[24][22] + coeff0_25*dmats1[25][22] + coeff0_26*dmats1[26][22] + coeff0_27*dmats1[27][22] + coeff0_28*dmats1[28][22] + coeff0_29*dmats1[29][22] + coeff0_30*dmats1[30][22] + coeff0_31*dmats1[31][22] + coeff0_32*dmats1[32][22] + coeff0_33*dmats1[33][22] + coeff0_34*dmats1[34][22] + coeff0_35*dmats1[35][22] + coeff0_36*dmats1[36][22] + coeff0_37*dmats1[37][22] + coeff0_38*dmats1[38][22] + coeff0_39*dmats1[39][22] + coeff0_40*dmats1[40][22] + coeff0_41*dmats1[41][22] + coeff0_42*dmats1[42][22] + coeff0_43*dmats1[43][22] + coeff0_44*dmats1[44][22] + coeff0_45*dmats1[45][22] + coeff0_46*dmats1[46][22] + coeff0_47*dmats1[47][22] + coeff0_48*dmats1[48][22] + coeff0_49*dmats1[49][22] + coeff0_50*dmats1[50][22] + coeff0_51*dmats1[51][22] + coeff0_52*dmats1[52][22] + coeff0_53*dmats1[53][22] + coeff0_54*dmats1[54][22] + coeff0_55*dmats1[55][22];
          new_coeff0_23 = coeff0_0*dmats1[0][23] + coeff0_1*dmats1[1][23] + coeff0_2*dmats1[2][23] + coeff0_3*dmats1[3][23] + coeff0_4*dmats1[4][23] + coeff0_5*dmats1[5][23] + coeff0_6*dmats1[6][23] + coeff0_7*dmats1[7][23] + coeff0_8*dmats1[8][23] + coeff0_9*dmats1[9][23] + coeff0_10*dmats1[10][23] + coeff0_11*dmats1[11][23] + coeff0_12*dmats1[12][23] + coeff0_13*dmats1[13][23] + coeff0_14*dmats1[14][23] + coeff0_15*dmats1[15][23] + coeff0_16*dmats1[16][23] + coeff0_17*dmats1[17][23] + coeff0_18*dmats1[18][23] + coeff0_19*dmats1[19][23] + coeff0_20*dmats1[20][23] + coeff0_21*dmats1[21][23] + coeff0_22*dmats1[22][23] + coeff0_23*dmats1[23][23] + coeff0_24*dmats1[24][23] + coeff0_25*dmats1[25][23] + coeff0_26*dmats1[26][23] + coeff0_27*dmats1[27][23] + coeff0_28*dmats1[28][23] + coeff0_29*dmats1[29][23] + coeff0_30*dmats1[30][23] + coeff0_31*dmats1[31][23] + coeff0_32*dmats1[32][23] + coeff0_33*dmats1[33][23] + coeff0_34*dmats1[34][23] + coeff0_35*dmats1[35][23] + coeff0_36*dmats1[36][23] + coeff0_37*dmats1[37][23] + coeff0_38*dmats1[38][23] + coeff0_39*dmats1[39][23] + coeff0_40*dmats1[40][23] + coeff0_41*dmats1[41][23] + coeff0_42*dmats1[42][23] + coeff0_43*dmats1[43][23] + coeff0_44*dmats1[44][23] + coeff0_45*dmats1[45][23] + coeff0_46*dmats1[46][23] + coeff0_47*dmats1[47][23] + coeff0_48*dmats1[48][23] + coeff0_49*dmats1[49][23] + coeff0_50*dmats1[50][23] + coeff0_51*dmats1[51][23] + coeff0_52*dmats1[52][23] + coeff0_53*dmats1[53][23] + coeff0_54*dmats1[54][23] + coeff0_55*dmats1[55][23];
          new_coeff0_24 = coeff0_0*dmats1[0][24] + coeff0_1*dmats1[1][24] + coeff0_2*dmats1[2][24] + coeff0_3*dmats1[3][24] + coeff0_4*dmats1[4][24] + coeff0_5*dmats1[5][24] + coeff0_6*dmats1[6][24] + coeff0_7*dmats1[7][24] + coeff0_8*dmats1[8][24] + coeff0_9*dmats1[9][24] + coeff0_10*dmats1[10][24] + coeff0_11*dmats1[11][24] + coeff0_12*dmats1[12][24] + coeff0_13*dmats1[13][24] + coeff0_14*dmats1[14][24] + coeff0_15*dmats1[15][24] + coeff0_16*dmats1[16][24] + coeff0_17*dmats1[17][24] + coeff0_18*dmats1[18][24] + coeff0_19*dmats1[19][24] + coeff0_20*dmats1[20][24] + coeff0_21*dmats1[21][24] + coeff0_22*dmats1[22][24] + coeff0_23*dmats1[23][24] + coeff0_24*dmats1[24][24] + coeff0_25*dmats1[25][24] + coeff0_26*dmats1[26][24] + coeff0_27*dmats1[27][24] + coeff0_28*dmats1[28][24] + coeff0_29*dmats1[29][24] + coeff0_30*dmats1[30][24] + coeff0_31*dmats1[31][24] + coeff0_32*dmats1[32][24] + coeff0_33*dmats1[33][24] + coeff0_34*dmats1[34][24] + coeff0_35*dmats1[35][24] + coeff0_36*dmats1[36][24] + coeff0_37*dmats1[37][24] + coeff0_38*dmats1[38][24] + coeff0_39*dmats1[39][24] + coeff0_40*dmats1[40][24] + coeff0_41*dmats1[41][24] + coeff0_42*dmats1[42][24] + coeff0_43*dmats1[43][24] + coeff0_44*dmats1[44][24] + coeff0_45*dmats1[45][24] + coeff0_46*dmats1[46][24] + coeff0_47*dmats1[47][24] + coeff0_48*dmats1[48][24] + coeff0_49*dmats1[49][24] + coeff0_50*dmats1[50][24] + coeff0_51*dmats1[51][24] + coeff0_52*dmats1[52][24] + coeff0_53*dmats1[53][24] + coeff0_54*dmats1[54][24] + coeff0_55*dmats1[55][24];
          new_coeff0_25 = coeff0_0*dmats1[0][25] + coeff0_1*dmats1[1][25] + coeff0_2*dmats1[2][25] + coeff0_3*dmats1[3][25] + coeff0_4*dmats1[4][25] + coeff0_5*dmats1[5][25] + coeff0_6*dmats1[6][25] + coeff0_7*dmats1[7][25] + coeff0_8*dmats1[8][25] + coeff0_9*dmats1[9][25] + coeff0_10*dmats1[10][25] + coeff0_11*dmats1[11][25] + coeff0_12*dmats1[12][25] + coeff0_13*dmats1[13][25] + coeff0_14*dmats1[14][25] + coeff0_15*dmats1[15][25] + coeff0_16*dmats1[16][25] + coeff0_17*dmats1[17][25] + coeff0_18*dmats1[18][25] + coeff0_19*dmats1[19][25] + coeff0_20*dmats1[20][25] + coeff0_21*dmats1[21][25] + coeff0_22*dmats1[22][25] + coeff0_23*dmats1[23][25] + coeff0_24*dmats1[24][25] + coeff0_25*dmats1[25][25] + coeff0_26*dmats1[26][25] + coeff0_27*dmats1[27][25] + coeff0_28*dmats1[28][25] + coeff0_29*dmats1[29][25] + coeff0_30*dmats1[30][25] + coeff0_31*dmats1[31][25] + coeff0_32*dmats1[32][25] + coeff0_33*dmats1[33][25] + coeff0_34*dmats1[34][25] + coeff0_35*dmats1[35][25] + coeff0_36*dmats1[36][25] + coeff0_37*dmats1[37][25] + coeff0_38*dmats1[38][25] + coeff0_39*dmats1[39][25] + coeff0_40*dmats1[40][25] + coeff0_41*dmats1[41][25] + coeff0_42*dmats1[42][25] + coeff0_43*dmats1[43][25] + coeff0_44*dmats1[44][25] + coeff0_45*dmats1[45][25] + coeff0_46*dmats1[46][25] + coeff0_47*dmats1[47][25] + coeff0_48*dmats1[48][25] + coeff0_49*dmats1[49][25] + coeff0_50*dmats1[50][25] + coeff0_51*dmats1[51][25] + coeff0_52*dmats1[52][25] + coeff0_53*dmats1[53][25] + coeff0_54*dmats1[54][25] + coeff0_55*dmats1[55][25];
          new_coeff0_26 = coeff0_0*dmats1[0][26] + coeff0_1*dmats1[1][26] + coeff0_2*dmats1[2][26] + coeff0_3*dmats1[3][26] + coeff0_4*dmats1[4][26] + coeff0_5*dmats1[5][26] + coeff0_6*dmats1[6][26] + coeff0_7*dmats1[7][26] + coeff0_8*dmats1[8][26] + coeff0_9*dmats1[9][26] + coeff0_10*dmats1[10][26] + coeff0_11*dmats1[11][26] + coeff0_12*dmats1[12][26] + coeff0_13*dmats1[13][26] + coeff0_14*dmats1[14][26] + coeff0_15*dmats1[15][26] + coeff0_16*dmats1[16][26] + coeff0_17*dmats1[17][26] + coeff0_18*dmats1[18][26] + coeff0_19*dmats1[19][26] + coeff0_20*dmats1[20][26] + coeff0_21*dmats1[21][26] + coeff0_22*dmats1[22][26] + coeff0_23*dmats1[23][26] + coeff0_24*dmats1[24][26] + coeff0_25*dmats1[25][26] + coeff0_26*dmats1[26][26] + coeff0_27*dmats1[27][26] + coeff0_28*dmats1[28][26] + coeff0_29*dmats1[29][26] + coeff0_30*dmats1[30][26] + coeff0_31*dmats1[31][26] + coeff0_32*dmats1[32][26] + coeff0_33*dmats1[33][26] + coeff0_34*dmats1[34][26] + coeff0_35*dmats1[35][26] + coeff0_36*dmats1[36][26] + coeff0_37*dmats1[37][26] + coeff0_38*dmats1[38][26] + coeff0_39*dmats1[39][26] + coeff0_40*dmats1[40][26] + coeff0_41*dmats1[41][26] + coeff0_42*dmats1[42][26] + coeff0_43*dmats1[43][26] + coeff0_44*dmats1[44][26] + coeff0_45*dmats1[45][26] + coeff0_46*dmats1[46][26] + coeff0_47*dmats1[47][26] + coeff0_48*dmats1[48][26] + coeff0_49*dmats1[49][26] + coeff0_50*dmats1[50][26] + coeff0_51*dmats1[51][26] + coeff0_52*dmats1[52][26] + coeff0_53*dmats1[53][26] + coeff0_54*dmats1[54][26] + coeff0_55*dmats1[55][26];
          new_coeff0_27 = coeff0_0*dmats1[0][27] + coeff0_1*dmats1[1][27] + coeff0_2*dmats1[2][27] + coeff0_3*dmats1[3][27] + coeff0_4*dmats1[4][27] + coeff0_5*dmats1[5][27] + coeff0_6*dmats1[6][27] + coeff0_7*dmats1[7][27] + coeff0_8*dmats1[8][27] + coeff0_9*dmats1[9][27] + coeff0_10*dmats1[10][27] + coeff0_11*dmats1[11][27] + coeff0_12*dmats1[12][27] + coeff0_13*dmats1[13][27] + coeff0_14*dmats1[14][27] + coeff0_15*dmats1[15][27] + coeff0_16*dmats1[16][27] + coeff0_17*dmats1[17][27] + coeff0_18*dmats1[18][27] + coeff0_19*dmats1[19][27] + coeff0_20*dmats1[20][27] + coeff0_21*dmats1[21][27] + coeff0_22*dmats1[22][27] + coeff0_23*dmats1[23][27] + coeff0_24*dmats1[24][27] + coeff0_25*dmats1[25][27] + coeff0_26*dmats1[26][27] + coeff0_27*dmats1[27][27] + coeff0_28*dmats1[28][27] + coeff0_29*dmats1[29][27] + coeff0_30*dmats1[30][27] + coeff0_31*dmats1[31][27] + coeff0_32*dmats1[32][27] + coeff0_33*dmats1[33][27] + coeff0_34*dmats1[34][27] + coeff0_35*dmats1[35][27] + coeff0_36*dmats1[36][27] + coeff0_37*dmats1[37][27] + coeff0_38*dmats1[38][27] + coeff0_39*dmats1[39][27] + coeff0_40*dmats1[40][27] + coeff0_41*dmats1[41][27] + coeff0_42*dmats1[42][27] + coeff0_43*dmats1[43][27] + coeff0_44*dmats1[44][27] + coeff0_45*dmats1[45][27] + coeff0_46*dmats1[46][27] + coeff0_47*dmats1[47][27] + coeff0_48*dmats1[48][27] + coeff0_49*dmats1[49][27] + coeff0_50*dmats1[50][27] + coeff0_51*dmats1[51][27] + coeff0_52*dmats1[52][27] + coeff0_53*dmats1[53][27] + coeff0_54*dmats1[54][27] + coeff0_55*dmats1[55][27];
          new_coeff0_28 = coeff0_0*dmats1[0][28] + coeff0_1*dmats1[1][28] + coeff0_2*dmats1[2][28] + coeff0_3*dmats1[3][28] + coeff0_4*dmats1[4][28] + coeff0_5*dmats1[5][28] + coeff0_6*dmats1[6][28] + coeff0_7*dmats1[7][28] + coeff0_8*dmats1[8][28] + coeff0_9*dmats1[9][28] + coeff0_10*dmats1[10][28] + coeff0_11*dmats1[11][28] + coeff0_12*dmats1[12][28] + coeff0_13*dmats1[13][28] + coeff0_14*dmats1[14][28] + coeff0_15*dmats1[15][28] + coeff0_16*dmats1[16][28] + coeff0_17*dmats1[17][28] + coeff0_18*dmats1[18][28] + coeff0_19*dmats1[19][28] + coeff0_20*dmats1[20][28] + coeff0_21*dmats1[21][28] + coeff0_22*dmats1[22][28] + coeff0_23*dmats1[23][28] + coeff0_24*dmats1[24][28] + coeff0_25*dmats1[25][28] + coeff0_26*dmats1[26][28] + coeff0_27*dmats1[27][28] + coeff0_28*dmats1[28][28] + coeff0_29*dmats1[29][28] + coeff0_30*dmats1[30][28] + coeff0_31*dmats1[31][28] + coeff0_32*dmats1[32][28] + coeff0_33*dmats1[33][28] + coeff0_34*dmats1[34][28] + coeff0_35*dmats1[35][28] + coeff0_36*dmats1[36][28] + coeff0_37*dmats1[37][28] + coeff0_38*dmats1[38][28] + coeff0_39*dmats1[39][28] + coeff0_40*dmats1[40][28] + coeff0_41*dmats1[41][28] + coeff0_42*dmats1[42][28] + coeff0_43*dmats1[43][28] + coeff0_44*dmats1[44][28] + coeff0_45*dmats1[45][28] + coeff0_46*dmats1[46][28] + coeff0_47*dmats1[47][28] + coeff0_48*dmats1[48][28] + coeff0_49*dmats1[49][28] + coeff0_50*dmats1[50][28] + coeff0_51*dmats1[51][28] + coeff0_52*dmats1[52][28] + coeff0_53*dmats1[53][28] + coeff0_54*dmats1[54][28] + coeff0_55*dmats1[55][28];
          new_coeff0_29 = coeff0_0*dmats1[0][29] + coeff0_1*dmats1[1][29] + coeff0_2*dmats1[2][29] + coeff0_3*dmats1[3][29] + coeff0_4*dmats1[4][29] + coeff0_5*dmats1[5][29] + coeff0_6*dmats1[6][29] + coeff0_7*dmats1[7][29] + coeff0_8*dmats1[8][29] + coeff0_9*dmats1[9][29] + coeff0_10*dmats1[10][29] + coeff0_11*dmats1[11][29] + coeff0_12*dmats1[12][29] + coeff0_13*dmats1[13][29] + coeff0_14*dmats1[14][29] + coeff0_15*dmats1[15][29] + coeff0_16*dmats1[16][29] + coeff0_17*dmats1[17][29] + coeff0_18*dmats1[18][29] + coeff0_19*dmats1[19][29] + coeff0_20*dmats1[20][29] + coeff0_21*dmats1[21][29] + coeff0_22*dmats1[22][29] + coeff0_23*dmats1[23][29] + coeff0_24*dmats1[24][29] + coeff0_25*dmats1[25][29] + coeff0_26*dmats1[26][29] + coeff0_27*dmats1[27][29] + coeff0_28*dmats1[28][29] + coeff0_29*dmats1[29][29] + coeff0_30*dmats1[30][29] + coeff0_31*dmats1[31][29] + coeff0_32*dmats1[32][29] + coeff0_33*dmats1[33][29] + coeff0_34*dmats1[34][29] + coeff0_35*dmats1[35][29] + coeff0_36*dmats1[36][29] + coeff0_37*dmats1[37][29] + coeff0_38*dmats1[38][29] + coeff0_39*dmats1[39][29] + coeff0_40*dmats1[40][29] + coeff0_41*dmats1[41][29] + coeff0_42*dmats1[42][29] + coeff0_43*dmats1[43][29] + coeff0_44*dmats1[44][29] + coeff0_45*dmats1[45][29] + coeff0_46*dmats1[46][29] + coeff0_47*dmats1[47][29] + coeff0_48*dmats1[48][29] + coeff0_49*dmats1[49][29] + coeff0_50*dmats1[50][29] + coeff0_51*dmats1[51][29] + coeff0_52*dmats1[52][29] + coeff0_53*dmats1[53][29] + coeff0_54*dmats1[54][29] + coeff0_55*dmats1[55][29];
          new_coeff0_30 = coeff0_0*dmats1[0][30] + coeff0_1*dmats1[1][30] + coeff0_2*dmats1[2][30] + coeff0_3*dmats1[3][30] + coeff0_4*dmats1[4][30] + coeff0_5*dmats1[5][30] + coeff0_6*dmats1[6][30] + coeff0_7*dmats1[7][30] + coeff0_8*dmats1[8][30] + coeff0_9*dmats1[9][30] + coeff0_10*dmats1[10][30] + coeff0_11*dmats1[11][30] + coeff0_12*dmats1[12][30] + coeff0_13*dmats1[13][30] + coeff0_14*dmats1[14][30] + coeff0_15*dmats1[15][30] + coeff0_16*dmats1[16][30] + coeff0_17*dmats1[17][30] + coeff0_18*dmats1[18][30] + coeff0_19*dmats1[19][30] + coeff0_20*dmats1[20][30] + coeff0_21*dmats1[21][30] + coeff0_22*dmats1[22][30] + coeff0_23*dmats1[23][30] + coeff0_24*dmats1[24][30] + coeff0_25*dmats1[25][30] + coeff0_26*dmats1[26][30] + coeff0_27*dmats1[27][30] + coeff0_28*dmats1[28][30] + coeff0_29*dmats1[29][30] + coeff0_30*dmats1[30][30] + coeff0_31*dmats1[31][30] + coeff0_32*dmats1[32][30] + coeff0_33*dmats1[33][30] + coeff0_34*dmats1[34][30] + coeff0_35*dmats1[35][30] + coeff0_36*dmats1[36][30] + coeff0_37*dmats1[37][30] + coeff0_38*dmats1[38][30] + coeff0_39*dmats1[39][30] + coeff0_40*dmats1[40][30] + coeff0_41*dmats1[41][30] + coeff0_42*dmats1[42][30] + coeff0_43*dmats1[43][30] + coeff0_44*dmats1[44][30] + coeff0_45*dmats1[45][30] + coeff0_46*dmats1[46][30] + coeff0_47*dmats1[47][30] + coeff0_48*dmats1[48][30] + coeff0_49*dmats1[49][30] + coeff0_50*dmats1[50][30] + coeff0_51*dmats1[51][30] + coeff0_52*dmats1[52][30] + coeff0_53*dmats1[53][30] + coeff0_54*dmats1[54][30] + coeff0_55*dmats1[55][30];
          new_coeff0_31 = coeff0_0*dmats1[0][31] + coeff0_1*dmats1[1][31] + coeff0_2*dmats1[2][31] + coeff0_3*dmats1[3][31] + coeff0_4*dmats1[4][31] + coeff0_5*dmats1[5][31] + coeff0_6*dmats1[6][31] + coeff0_7*dmats1[7][31] + coeff0_8*dmats1[8][31] + coeff0_9*dmats1[9][31] + coeff0_10*dmats1[10][31] + coeff0_11*dmats1[11][31] + coeff0_12*dmats1[12][31] + coeff0_13*dmats1[13][31] + coeff0_14*dmats1[14][31] + coeff0_15*dmats1[15][31] + coeff0_16*dmats1[16][31] + coeff0_17*dmats1[17][31] + coeff0_18*dmats1[18][31] + coeff0_19*dmats1[19][31] + coeff0_20*dmats1[20][31] + coeff0_21*dmats1[21][31] + coeff0_22*dmats1[22][31] + coeff0_23*dmats1[23][31] + coeff0_24*dmats1[24][31] + coeff0_25*dmats1[25][31] + coeff0_26*dmats1[26][31] + coeff0_27*dmats1[27][31] + coeff0_28*dmats1[28][31] + coeff0_29*dmats1[29][31] + coeff0_30*dmats1[30][31] + coeff0_31*dmats1[31][31] + coeff0_32*dmats1[32][31] + coeff0_33*dmats1[33][31] + coeff0_34*dmats1[34][31] + coeff0_35*dmats1[35][31] + coeff0_36*dmats1[36][31] + coeff0_37*dmats1[37][31] + coeff0_38*dmats1[38][31] + coeff0_39*dmats1[39][31] + coeff0_40*dmats1[40][31] + coeff0_41*dmats1[41][31] + coeff0_42*dmats1[42][31] + coeff0_43*dmats1[43][31] + coeff0_44*dmats1[44][31] + coeff0_45*dmats1[45][31] + coeff0_46*dmats1[46][31] + coeff0_47*dmats1[47][31] + coeff0_48*dmats1[48][31] + coeff0_49*dmats1[49][31] + coeff0_50*dmats1[50][31] + coeff0_51*dmats1[51][31] + coeff0_52*dmats1[52][31] + coeff0_53*dmats1[53][31] + coeff0_54*dmats1[54][31] + coeff0_55*dmats1[55][31];
          new_coeff0_32 = coeff0_0*dmats1[0][32] + coeff0_1*dmats1[1][32] + coeff0_2*dmats1[2][32] + coeff0_3*dmats1[3][32] + coeff0_4*dmats1[4][32] + coeff0_5*dmats1[5][32] + coeff0_6*dmats1[6][32] + coeff0_7*dmats1[7][32] + coeff0_8*dmats1[8][32] + coeff0_9*dmats1[9][32] + coeff0_10*dmats1[10][32] + coeff0_11*dmats1[11][32] + coeff0_12*dmats1[12][32] + coeff0_13*dmats1[13][32] + coeff0_14*dmats1[14][32] + coeff0_15*dmats1[15][32] + coeff0_16*dmats1[16][32] + coeff0_17*dmats1[17][32] + coeff0_18*dmats1[18][32] + coeff0_19*dmats1[19][32] + coeff0_20*dmats1[20][32] + coeff0_21*dmats1[21][32] + coeff0_22*dmats1[22][32] + coeff0_23*dmats1[23][32] + coeff0_24*dmats1[24][32] + coeff0_25*dmats1[25][32] + coeff0_26*dmats1[26][32] + coeff0_27*dmats1[27][32] + coeff0_28*dmats1[28][32] + coeff0_29*dmats1[29][32] + coeff0_30*dmats1[30][32] + coeff0_31*dmats1[31][32] + coeff0_32*dmats1[32][32] + coeff0_33*dmats1[33][32] + coeff0_34*dmats1[34][32] + coeff0_35*dmats1[35][32] + coeff0_36*dmats1[36][32] + coeff0_37*dmats1[37][32] + coeff0_38*dmats1[38][32] + coeff0_39*dmats1[39][32] + coeff0_40*dmats1[40][32] + coeff0_41*dmats1[41][32] + coeff0_42*dmats1[42][32] + coeff0_43*dmats1[43][32] + coeff0_44*dmats1[44][32] + coeff0_45*dmats1[45][32] + coeff0_46*dmats1[46][32] + coeff0_47*dmats1[47][32] + coeff0_48*dmats1[48][32] + coeff0_49*dmats1[49][32] + coeff0_50*dmats1[50][32] + coeff0_51*dmats1[51][32] + coeff0_52*dmats1[52][32] + coeff0_53*dmats1[53][32] + coeff0_54*dmats1[54][32] + coeff0_55*dmats1[55][32];
          new_coeff0_33 = coeff0_0*dmats1[0][33] + coeff0_1*dmats1[1][33] + coeff0_2*dmats1[2][33] + coeff0_3*dmats1[3][33] + coeff0_4*dmats1[4][33] + coeff0_5*dmats1[5][33] + coeff0_6*dmats1[6][33] + coeff0_7*dmats1[7][33] + coeff0_8*dmats1[8][33] + coeff0_9*dmats1[9][33] + coeff0_10*dmats1[10][33] + coeff0_11*dmats1[11][33] + coeff0_12*dmats1[12][33] + coeff0_13*dmats1[13][33] + coeff0_14*dmats1[14][33] + coeff0_15*dmats1[15][33] + coeff0_16*dmats1[16][33] + coeff0_17*dmats1[17][33] + coeff0_18*dmats1[18][33] + coeff0_19*dmats1[19][33] + coeff0_20*dmats1[20][33] + coeff0_21*dmats1[21][33] + coeff0_22*dmats1[22][33] + coeff0_23*dmats1[23][33] + coeff0_24*dmats1[24][33] + coeff0_25*dmats1[25][33] + coeff0_26*dmats1[26][33] + coeff0_27*dmats1[27][33] + coeff0_28*dmats1[28][33] + coeff0_29*dmats1[29][33] + coeff0_30*dmats1[30][33] + coeff0_31*dmats1[31][33] + coeff0_32*dmats1[32][33] + coeff0_33*dmats1[33][33] + coeff0_34*dmats1[34][33] + coeff0_35*dmats1[35][33] + coeff0_36*dmats1[36][33] + coeff0_37*dmats1[37][33] + coeff0_38*dmats1[38][33] + coeff0_39*dmats1[39][33] + coeff0_40*dmats1[40][33] + coeff0_41*dmats1[41][33] + coeff0_42*dmats1[42][33] + coeff0_43*dmats1[43][33] + coeff0_44*dmats1[44][33] + coeff0_45*dmats1[45][33] + coeff0_46*dmats1[46][33] + coeff0_47*dmats1[47][33] + coeff0_48*dmats1[48][33] + coeff0_49*dmats1[49][33] + coeff0_50*dmats1[50][33] + coeff0_51*dmats1[51][33] + coeff0_52*dmats1[52][33] + coeff0_53*dmats1[53][33] + coeff0_54*dmats1[54][33] + coeff0_55*dmats1[55][33];
          new_coeff0_34 = coeff0_0*dmats1[0][34] + coeff0_1*dmats1[1][34] + coeff0_2*dmats1[2][34] + coeff0_3*dmats1[3][34] + coeff0_4*dmats1[4][34] + coeff0_5*dmats1[5][34] + coeff0_6*dmats1[6][34] + coeff0_7*dmats1[7][34] + coeff0_8*dmats1[8][34] + coeff0_9*dmats1[9][34] + coeff0_10*dmats1[10][34] + coeff0_11*dmats1[11][34] + coeff0_12*dmats1[12][34] + coeff0_13*dmats1[13][34] + coeff0_14*dmats1[14][34] + coeff0_15*dmats1[15][34] + coeff0_16*dmats1[16][34] + coeff0_17*dmats1[17][34] + coeff0_18*dmats1[18][34] + coeff0_19*dmats1[19][34] + coeff0_20*dmats1[20][34] + coeff0_21*dmats1[21][34] + coeff0_22*dmats1[22][34] + coeff0_23*dmats1[23][34] + coeff0_24*dmats1[24][34] + coeff0_25*dmats1[25][34] + coeff0_26*dmats1[26][34] + coeff0_27*dmats1[27][34] + coeff0_28*dmats1[28][34] + coeff0_29*dmats1[29][34] + coeff0_30*dmats1[30][34] + coeff0_31*dmats1[31][34] + coeff0_32*dmats1[32][34] + coeff0_33*dmats1[33][34] + coeff0_34*dmats1[34][34] + coeff0_35*dmats1[35][34] + coeff0_36*dmats1[36][34] + coeff0_37*dmats1[37][34] + coeff0_38*dmats1[38][34] + coeff0_39*dmats1[39][34] + coeff0_40*dmats1[40][34] + coeff0_41*dmats1[41][34] + coeff0_42*dmats1[42][34] + coeff0_43*dmats1[43][34] + coeff0_44*dmats1[44][34] + coeff0_45*dmats1[45][34] + coeff0_46*dmats1[46][34] + coeff0_47*dmats1[47][34] + coeff0_48*dmats1[48][34] + coeff0_49*dmats1[49][34] + coeff0_50*dmats1[50][34] + coeff0_51*dmats1[51][34] + coeff0_52*dmats1[52][34] + coeff0_53*dmats1[53][34] + coeff0_54*dmats1[54][34] + coeff0_55*dmats1[55][34];
          new_coeff0_35 = coeff0_0*dmats1[0][35] + coeff0_1*dmats1[1][35] + coeff0_2*dmats1[2][35] + coeff0_3*dmats1[3][35] + coeff0_4*dmats1[4][35] + coeff0_5*dmats1[5][35] + coeff0_6*dmats1[6][35] + coeff0_7*dmats1[7][35] + coeff0_8*dmats1[8][35] + coeff0_9*dmats1[9][35] + coeff0_10*dmats1[10][35] + coeff0_11*dmats1[11][35] + coeff0_12*dmats1[12][35] + coeff0_13*dmats1[13][35] + coeff0_14*dmats1[14][35] + coeff0_15*dmats1[15][35] + coeff0_16*dmats1[16][35] + coeff0_17*dmats1[17][35] + coeff0_18*dmats1[18][35] + coeff0_19*dmats1[19][35] + coeff0_20*dmats1[20][35] + coeff0_21*dmats1[21][35] + coeff0_22*dmats1[22][35] + coeff0_23*dmats1[23][35] + coeff0_24*dmats1[24][35] + coeff0_25*dmats1[25][35] + coeff0_26*dmats1[26][35] + coeff0_27*dmats1[27][35] + coeff0_28*dmats1[28][35] + coeff0_29*dmats1[29][35] + coeff0_30*dmats1[30][35] + coeff0_31*dmats1[31][35] + coeff0_32*dmats1[32][35] + coeff0_33*dmats1[33][35] + coeff0_34*dmats1[34][35] + coeff0_35*dmats1[35][35] + coeff0_36*dmats1[36][35] + coeff0_37*dmats1[37][35] + coeff0_38*dmats1[38][35] + coeff0_39*dmats1[39][35] + coeff0_40*dmats1[40][35] + coeff0_41*dmats1[41][35] + coeff0_42*dmats1[42][35] + coeff0_43*dmats1[43][35] + coeff0_44*dmats1[44][35] + coeff0_45*dmats1[45][35] + coeff0_46*dmats1[46][35] + coeff0_47*dmats1[47][35] + coeff0_48*dmats1[48][35] + coeff0_49*dmats1[49][35] + coeff0_50*dmats1[50][35] + coeff0_51*dmats1[51][35] + coeff0_52*dmats1[52][35] + coeff0_53*dmats1[53][35] + coeff0_54*dmats1[54][35] + coeff0_55*dmats1[55][35];
          new_coeff0_36 = coeff0_0*dmats1[0][36] + coeff0_1*dmats1[1][36] + coeff0_2*dmats1[2][36] + coeff0_3*dmats1[3][36] + coeff0_4*dmats1[4][36] + coeff0_5*dmats1[5][36] + coeff0_6*dmats1[6][36] + coeff0_7*dmats1[7][36] + coeff0_8*dmats1[8][36] + coeff0_9*dmats1[9][36] + coeff0_10*dmats1[10][36] + coeff0_11*dmats1[11][36] + coeff0_12*dmats1[12][36] + coeff0_13*dmats1[13][36] + coeff0_14*dmats1[14][36] + coeff0_15*dmats1[15][36] + coeff0_16*dmats1[16][36] + coeff0_17*dmats1[17][36] + coeff0_18*dmats1[18][36] + coeff0_19*dmats1[19][36] + coeff0_20*dmats1[20][36] + coeff0_21*dmats1[21][36] + coeff0_22*dmats1[22][36] + coeff0_23*dmats1[23][36] + coeff0_24*dmats1[24][36] + coeff0_25*dmats1[25][36] + coeff0_26*dmats1[26][36] + coeff0_27*dmats1[27][36] + coeff0_28*dmats1[28][36] + coeff0_29*dmats1[29][36] + coeff0_30*dmats1[30][36] + coeff0_31*dmats1[31][36] + coeff0_32*dmats1[32][36] + coeff0_33*dmats1[33][36] + coeff0_34*dmats1[34][36] + coeff0_35*dmats1[35][36] + coeff0_36*dmats1[36][36] + coeff0_37*dmats1[37][36] + coeff0_38*dmats1[38][36] + coeff0_39*dmats1[39][36] + coeff0_40*dmats1[40][36] + coeff0_41*dmats1[41][36] + coeff0_42*dmats1[42][36] + coeff0_43*dmats1[43][36] + coeff0_44*dmats1[44][36] + coeff0_45*dmats1[45][36] + coeff0_46*dmats1[46][36] + coeff0_47*dmats1[47][36] + coeff0_48*dmats1[48][36] + coeff0_49*dmats1[49][36] + coeff0_50*dmats1[50][36] + coeff0_51*dmats1[51][36] + coeff0_52*dmats1[52][36] + coeff0_53*dmats1[53][36] + coeff0_54*dmats1[54][36] + coeff0_55*dmats1[55][36];
          new_coeff0_37 = coeff0_0*dmats1[0][37] + coeff0_1*dmats1[1][37] + coeff0_2*dmats1[2][37] + coeff0_3*dmats1[3][37] + coeff0_4*dmats1[4][37] + coeff0_5*dmats1[5][37] + coeff0_6*dmats1[6][37] + coeff0_7*dmats1[7][37] + coeff0_8*dmats1[8][37] + coeff0_9*dmats1[9][37] + coeff0_10*dmats1[10][37] + coeff0_11*dmats1[11][37] + coeff0_12*dmats1[12][37] + coeff0_13*dmats1[13][37] + coeff0_14*dmats1[14][37] + coeff0_15*dmats1[15][37] + coeff0_16*dmats1[16][37] + coeff0_17*dmats1[17][37] + coeff0_18*dmats1[18][37] + coeff0_19*dmats1[19][37] + coeff0_20*dmats1[20][37] + coeff0_21*dmats1[21][37] + coeff0_22*dmats1[22][37] + coeff0_23*dmats1[23][37] + coeff0_24*dmats1[24][37] + coeff0_25*dmats1[25][37] + coeff0_26*dmats1[26][37] + coeff0_27*dmats1[27][37] + coeff0_28*dmats1[28][37] + coeff0_29*dmats1[29][37] + coeff0_30*dmats1[30][37] + coeff0_31*dmats1[31][37] + coeff0_32*dmats1[32][37] + coeff0_33*dmats1[33][37] + coeff0_34*dmats1[34][37] + coeff0_35*dmats1[35][37] + coeff0_36*dmats1[36][37] + coeff0_37*dmats1[37][37] + coeff0_38*dmats1[38][37] + coeff0_39*dmats1[39][37] + coeff0_40*dmats1[40][37] + coeff0_41*dmats1[41][37] + coeff0_42*dmats1[42][37] + coeff0_43*dmats1[43][37] + coeff0_44*dmats1[44][37] + coeff0_45*dmats1[45][37] + coeff0_46*dmats1[46][37] + coeff0_47*dmats1[47][37] + coeff0_48*dmats1[48][37] + coeff0_49*dmats1[49][37] + coeff0_50*dmats1[50][37] + coeff0_51*dmats1[51][37] + coeff0_52*dmats1[52][37] + coeff0_53*dmats1[53][37] + coeff0_54*dmats1[54][37] + coeff0_55*dmats1[55][37];
          new_coeff0_38 = coeff0_0*dmats1[0][38] + coeff0_1*dmats1[1][38] + coeff0_2*dmats1[2][38] + coeff0_3*dmats1[3][38] + coeff0_4*dmats1[4][38] + coeff0_5*dmats1[5][38] + coeff0_6*dmats1[6][38] + coeff0_7*dmats1[7][38] + coeff0_8*dmats1[8][38] + coeff0_9*dmats1[9][38] + coeff0_10*dmats1[10][38] + coeff0_11*dmats1[11][38] + coeff0_12*dmats1[12][38] + coeff0_13*dmats1[13][38] + coeff0_14*dmats1[14][38] + coeff0_15*dmats1[15][38] + coeff0_16*dmats1[16][38] + coeff0_17*dmats1[17][38] + coeff0_18*dmats1[18][38] + coeff0_19*dmats1[19][38] + coeff0_20*dmats1[20][38] + coeff0_21*dmats1[21][38] + coeff0_22*dmats1[22][38] + coeff0_23*dmats1[23][38] + coeff0_24*dmats1[24][38] + coeff0_25*dmats1[25][38] + coeff0_26*dmats1[26][38] + coeff0_27*dmats1[27][38] + coeff0_28*dmats1[28][38] + coeff0_29*dmats1[29][38] + coeff0_30*dmats1[30][38] + coeff0_31*dmats1[31][38] + coeff0_32*dmats1[32][38] + coeff0_33*dmats1[33][38] + coeff0_34*dmats1[34][38] + coeff0_35*dmats1[35][38] + coeff0_36*dmats1[36][38] + coeff0_37*dmats1[37][38] + coeff0_38*dmats1[38][38] + coeff0_39*dmats1[39][38] + coeff0_40*dmats1[40][38] + coeff0_41*dmats1[41][38] + coeff0_42*dmats1[42][38] + coeff0_43*dmats1[43][38] + coeff0_44*dmats1[44][38] + coeff0_45*dmats1[45][38] + coeff0_46*dmats1[46][38] + coeff0_47*dmats1[47][38] + coeff0_48*dmats1[48][38] + coeff0_49*dmats1[49][38] + coeff0_50*dmats1[50][38] + coeff0_51*dmats1[51][38] + coeff0_52*dmats1[52][38] + coeff0_53*dmats1[53][38] + coeff0_54*dmats1[54][38] + coeff0_55*dmats1[55][38];
          new_coeff0_39 = coeff0_0*dmats1[0][39] + coeff0_1*dmats1[1][39] + coeff0_2*dmats1[2][39] + coeff0_3*dmats1[3][39] + coeff0_4*dmats1[4][39] + coeff0_5*dmats1[5][39] + coeff0_6*dmats1[6][39] + coeff0_7*dmats1[7][39] + coeff0_8*dmats1[8][39] + coeff0_9*dmats1[9][39] + coeff0_10*dmats1[10][39] + coeff0_11*dmats1[11][39] + coeff0_12*dmats1[12][39] + coeff0_13*dmats1[13][39] + coeff0_14*dmats1[14][39] + coeff0_15*dmats1[15][39] + coeff0_16*dmats1[16][39] + coeff0_17*dmats1[17][39] + coeff0_18*dmats1[18][39] + coeff0_19*dmats1[19][39] + coeff0_20*dmats1[20][39] + coeff0_21*dmats1[21][39] + coeff0_22*dmats1[22][39] + coeff0_23*dmats1[23][39] + coeff0_24*dmats1[24][39] + coeff0_25*dmats1[25][39] + coeff0_26*dmats1[26][39] + coeff0_27*dmats1[27][39] + coeff0_28*dmats1[28][39] + coeff0_29*dmats1[29][39] + coeff0_30*dmats1[30][39] + coeff0_31*dmats1[31][39] + coeff0_32*dmats1[32][39] + coeff0_33*dmats1[33][39] + coeff0_34*dmats1[34][39] + coeff0_35*dmats1[35][39] + coeff0_36*dmats1[36][39] + coeff0_37*dmats1[37][39] + coeff0_38*dmats1[38][39] + coeff0_39*dmats1[39][39] + coeff0_40*dmats1[40][39] + coeff0_41*dmats1[41][39] + coeff0_42*dmats1[42][39] + coeff0_43*dmats1[43][39] + coeff0_44*dmats1[44][39] + coeff0_45*dmats1[45][39] + coeff0_46*dmats1[46][39] + coeff0_47*dmats1[47][39] + coeff0_48*dmats1[48][39] + coeff0_49*dmats1[49][39] + coeff0_50*dmats1[50][39] + coeff0_51*dmats1[51][39] + coeff0_52*dmats1[52][39] + coeff0_53*dmats1[53][39] + coeff0_54*dmats1[54][39] + coeff0_55*dmats1[55][39];
          new_coeff0_40 = coeff0_0*dmats1[0][40] + coeff0_1*dmats1[1][40] + coeff0_2*dmats1[2][40] + coeff0_3*dmats1[3][40] + coeff0_4*dmats1[4][40] + coeff0_5*dmats1[5][40] + coeff0_6*dmats1[6][40] + coeff0_7*dmats1[7][40] + coeff0_8*dmats1[8][40] + coeff0_9*dmats1[9][40] + coeff0_10*dmats1[10][40] + coeff0_11*dmats1[11][40] + coeff0_12*dmats1[12][40] + coeff0_13*dmats1[13][40] + coeff0_14*dmats1[14][40] + coeff0_15*dmats1[15][40] + coeff0_16*dmats1[16][40] + coeff0_17*dmats1[17][40] + coeff0_18*dmats1[18][40] + coeff0_19*dmats1[19][40] + coeff0_20*dmats1[20][40] + coeff0_21*dmats1[21][40] + coeff0_22*dmats1[22][40] + coeff0_23*dmats1[23][40] + coeff0_24*dmats1[24][40] + coeff0_25*dmats1[25][40] + coeff0_26*dmats1[26][40] + coeff0_27*dmats1[27][40] + coeff0_28*dmats1[28][40] + coeff0_29*dmats1[29][40] + coeff0_30*dmats1[30][40] + coeff0_31*dmats1[31][40] + coeff0_32*dmats1[32][40] + coeff0_33*dmats1[33][40] + coeff0_34*dmats1[34][40] + coeff0_35*dmats1[35][40] + coeff0_36*dmats1[36][40] + coeff0_37*dmats1[37][40] + coeff0_38*dmats1[38][40] + coeff0_39*dmats1[39][40] + coeff0_40*dmats1[40][40] + coeff0_41*dmats1[41][40] + coeff0_42*dmats1[42][40] + coeff0_43*dmats1[43][40] + coeff0_44*dmats1[44][40] + coeff0_45*dmats1[45][40] + coeff0_46*dmats1[46][40] + coeff0_47*dmats1[47][40] + coeff0_48*dmats1[48][40] + coeff0_49*dmats1[49][40] + coeff0_50*dmats1[50][40] + coeff0_51*dmats1[51][40] + coeff0_52*dmats1[52][40] + coeff0_53*dmats1[53][40] + coeff0_54*dmats1[54][40] + coeff0_55*dmats1[55][40];
          new_coeff0_41 = coeff0_0*dmats1[0][41] + coeff0_1*dmats1[1][41] + coeff0_2*dmats1[2][41] + coeff0_3*dmats1[3][41] + coeff0_4*dmats1[4][41] + coeff0_5*dmats1[5][41] + coeff0_6*dmats1[6][41] + coeff0_7*dmats1[7][41] + coeff0_8*dmats1[8][41] + coeff0_9*dmats1[9][41] + coeff0_10*dmats1[10][41] + coeff0_11*dmats1[11][41] + coeff0_12*dmats1[12][41] + coeff0_13*dmats1[13][41] + coeff0_14*dmats1[14][41] + coeff0_15*dmats1[15][41] + coeff0_16*dmats1[16][41] + coeff0_17*dmats1[17][41] + coeff0_18*dmats1[18][41] + coeff0_19*dmats1[19][41] + coeff0_20*dmats1[20][41] + coeff0_21*dmats1[21][41] + coeff0_22*dmats1[22][41] + coeff0_23*dmats1[23][41] + coeff0_24*dmats1[24][41] + coeff0_25*dmats1[25][41] + coeff0_26*dmats1[26][41] + coeff0_27*dmats1[27][41] + coeff0_28*dmats1[28][41] + coeff0_29*dmats1[29][41] + coeff0_30*dmats1[30][41] + coeff0_31*dmats1[31][41] + coeff0_32*dmats1[32][41] + coeff0_33*dmats1[33][41] + coeff0_34*dmats1[34][41] + coeff0_35*dmats1[35][41] + coeff0_36*dmats1[36][41] + coeff0_37*dmats1[37][41] + coeff0_38*dmats1[38][41] + coeff0_39*dmats1[39][41] + coeff0_40*dmats1[40][41] + coeff0_41*dmats1[41][41] + coeff0_42*dmats1[42][41] + coeff0_43*dmats1[43][41] + coeff0_44*dmats1[44][41] + coeff0_45*dmats1[45][41] + coeff0_46*dmats1[46][41] + coeff0_47*dmats1[47][41] + coeff0_48*dmats1[48][41] + coeff0_49*dmats1[49][41] + coeff0_50*dmats1[50][41] + coeff0_51*dmats1[51][41] + coeff0_52*dmats1[52][41] + coeff0_53*dmats1[53][41] + coeff0_54*dmats1[54][41] + coeff0_55*dmats1[55][41];
          new_coeff0_42 = coeff0_0*dmats1[0][42] + coeff0_1*dmats1[1][42] + coeff0_2*dmats1[2][42] + coeff0_3*dmats1[3][42] + coeff0_4*dmats1[4][42] + coeff0_5*dmats1[5][42] + coeff0_6*dmats1[6][42] + coeff0_7*dmats1[7][42] + coeff0_8*dmats1[8][42] + coeff0_9*dmats1[9][42] + coeff0_10*dmats1[10][42] + coeff0_11*dmats1[11][42] + coeff0_12*dmats1[12][42] + coeff0_13*dmats1[13][42] + coeff0_14*dmats1[14][42] + coeff0_15*dmats1[15][42] + coeff0_16*dmats1[16][42] + coeff0_17*dmats1[17][42] + coeff0_18*dmats1[18][42] + coeff0_19*dmats1[19][42] + coeff0_20*dmats1[20][42] + coeff0_21*dmats1[21][42] + coeff0_22*dmats1[22][42] + coeff0_23*dmats1[23][42] + coeff0_24*dmats1[24][42] + coeff0_25*dmats1[25][42] + coeff0_26*dmats1[26][42] + coeff0_27*dmats1[27][42] + coeff0_28*dmats1[28][42] + coeff0_29*dmats1[29][42] + coeff0_30*dmats1[30][42] + coeff0_31*dmats1[31][42] + coeff0_32*dmats1[32][42] + coeff0_33*dmats1[33][42] + coeff0_34*dmats1[34][42] + coeff0_35*dmats1[35][42] + coeff0_36*dmats1[36][42] + coeff0_37*dmats1[37][42] + coeff0_38*dmats1[38][42] + coeff0_39*dmats1[39][42] + coeff0_40*dmats1[40][42] + coeff0_41*dmats1[41][42] + coeff0_42*dmats1[42][42] + coeff0_43*dmats1[43][42] + coeff0_44*dmats1[44][42] + coeff0_45*dmats1[45][42] + coeff0_46*dmats1[46][42] + coeff0_47*dmats1[47][42] + coeff0_48*dmats1[48][42] + coeff0_49*dmats1[49][42] + coeff0_50*dmats1[50][42] + coeff0_51*dmats1[51][42] + coeff0_52*dmats1[52][42] + coeff0_53*dmats1[53][42] + coeff0_54*dmats1[54][42] + coeff0_55*dmats1[55][42];
          new_coeff0_43 = coeff0_0*dmats1[0][43] + coeff0_1*dmats1[1][43] + coeff0_2*dmats1[2][43] + coeff0_3*dmats1[3][43] + coeff0_4*dmats1[4][43] + coeff0_5*dmats1[5][43] + coeff0_6*dmats1[6][43] + coeff0_7*dmats1[7][43] + coeff0_8*dmats1[8][43] + coeff0_9*dmats1[9][43] + coeff0_10*dmats1[10][43] + coeff0_11*dmats1[11][43] + coeff0_12*dmats1[12][43] + coeff0_13*dmats1[13][43] + coeff0_14*dmats1[14][43] + coeff0_15*dmats1[15][43] + coeff0_16*dmats1[16][43] + coeff0_17*dmats1[17][43] + coeff0_18*dmats1[18][43] + coeff0_19*dmats1[19][43] + coeff0_20*dmats1[20][43] + coeff0_21*dmats1[21][43] + coeff0_22*dmats1[22][43] + coeff0_23*dmats1[23][43] + coeff0_24*dmats1[24][43] + coeff0_25*dmats1[25][43] + coeff0_26*dmats1[26][43] + coeff0_27*dmats1[27][43] + coeff0_28*dmats1[28][43] + coeff0_29*dmats1[29][43] + coeff0_30*dmats1[30][43] + coeff0_31*dmats1[31][43] + coeff0_32*dmats1[32][43] + coeff0_33*dmats1[33][43] + coeff0_34*dmats1[34][43] + coeff0_35*dmats1[35][43] + coeff0_36*dmats1[36][43] + coeff0_37*dmats1[37][43] + coeff0_38*dmats1[38][43] + coeff0_39*dmats1[39][43] + coeff0_40*dmats1[40][43] + coeff0_41*dmats1[41][43] + coeff0_42*dmats1[42][43] + coeff0_43*dmats1[43][43] + coeff0_44*dmats1[44][43] + coeff0_45*dmats1[45][43] + coeff0_46*dmats1[46][43] + coeff0_47*dmats1[47][43] + coeff0_48*dmats1[48][43] + coeff0_49*dmats1[49][43] + coeff0_50*dmats1[50][43] + coeff0_51*dmats1[51][43] + coeff0_52*dmats1[52][43] + coeff0_53*dmats1[53][43] + coeff0_54*dmats1[54][43] + coeff0_55*dmats1[55][43];
          new_coeff0_44 = coeff0_0*dmats1[0][44] + coeff0_1*dmats1[1][44] + coeff0_2*dmats1[2][44] + coeff0_3*dmats1[3][44] + coeff0_4*dmats1[4][44] + coeff0_5*dmats1[5][44] + coeff0_6*dmats1[6][44] + coeff0_7*dmats1[7][44] + coeff0_8*dmats1[8][44] + coeff0_9*dmats1[9][44] + coeff0_10*dmats1[10][44] + coeff0_11*dmats1[11][44] + coeff0_12*dmats1[12][44] + coeff0_13*dmats1[13][44] + coeff0_14*dmats1[14][44] + coeff0_15*dmats1[15][44] + coeff0_16*dmats1[16][44] + coeff0_17*dmats1[17][44] + coeff0_18*dmats1[18][44] + coeff0_19*dmats1[19][44] + coeff0_20*dmats1[20][44] + coeff0_21*dmats1[21][44] + coeff0_22*dmats1[22][44] + coeff0_23*dmats1[23][44] + coeff0_24*dmats1[24][44] + coeff0_25*dmats1[25][44] + coeff0_26*dmats1[26][44] + coeff0_27*dmats1[27][44] + coeff0_28*dmats1[28][44] + coeff0_29*dmats1[29][44] + coeff0_30*dmats1[30][44] + coeff0_31*dmats1[31][44] + coeff0_32*dmats1[32][44] + coeff0_33*dmats1[33][44] + coeff0_34*dmats1[34][44] + coeff0_35*dmats1[35][44] + coeff0_36*dmats1[36][44] + coeff0_37*dmats1[37][44] + coeff0_38*dmats1[38][44] + coeff0_39*dmats1[39][44] + coeff0_40*dmats1[40][44] + coeff0_41*dmats1[41][44] + coeff0_42*dmats1[42][44] + coeff0_43*dmats1[43][44] + coeff0_44*dmats1[44][44] + coeff0_45*dmats1[45][44] + coeff0_46*dmats1[46][44] + coeff0_47*dmats1[47][44] + coeff0_48*dmats1[48][44] + coeff0_49*dmats1[49][44] + coeff0_50*dmats1[50][44] + coeff0_51*dmats1[51][44] + coeff0_52*dmats1[52][44] + coeff0_53*dmats1[53][44] + coeff0_54*dmats1[54][44] + coeff0_55*dmats1[55][44];
          new_coeff0_45 = coeff0_0*dmats1[0][45] + coeff0_1*dmats1[1][45] + coeff0_2*dmats1[2][45] + coeff0_3*dmats1[3][45] + coeff0_4*dmats1[4][45] + coeff0_5*dmats1[5][45] + coeff0_6*dmats1[6][45] + coeff0_7*dmats1[7][45] + coeff0_8*dmats1[8][45] + coeff0_9*dmats1[9][45] + coeff0_10*dmats1[10][45] + coeff0_11*dmats1[11][45] + coeff0_12*dmats1[12][45] + coeff0_13*dmats1[13][45] + coeff0_14*dmats1[14][45] + coeff0_15*dmats1[15][45] + coeff0_16*dmats1[16][45] + coeff0_17*dmats1[17][45] + coeff0_18*dmats1[18][45] + coeff0_19*dmats1[19][45] + coeff0_20*dmats1[20][45] + coeff0_21*dmats1[21][45] + coeff0_22*dmats1[22][45] + coeff0_23*dmats1[23][45] + coeff0_24*dmats1[24][45] + coeff0_25*dmats1[25][45] + coeff0_26*dmats1[26][45] + coeff0_27*dmats1[27][45] + coeff0_28*dmats1[28][45] + coeff0_29*dmats1[29][45] + coeff0_30*dmats1[30][45] + coeff0_31*dmats1[31][45] + coeff0_32*dmats1[32][45] + coeff0_33*dmats1[33][45] + coeff0_34*dmats1[34][45] + coeff0_35*dmats1[35][45] + coeff0_36*dmats1[36][45] + coeff0_37*dmats1[37][45] + coeff0_38*dmats1[38][45] + coeff0_39*dmats1[39][45] + coeff0_40*dmats1[40][45] + coeff0_41*dmats1[41][45] + coeff0_42*dmats1[42][45] + coeff0_43*dmats1[43][45] + coeff0_44*dmats1[44][45] + coeff0_45*dmats1[45][45] + coeff0_46*dmats1[46][45] + coeff0_47*dmats1[47][45] + coeff0_48*dmats1[48][45] + coeff0_49*dmats1[49][45] + coeff0_50*dmats1[50][45] + coeff0_51*dmats1[51][45] + coeff0_52*dmats1[52][45] + coeff0_53*dmats1[53][45] + coeff0_54*dmats1[54][45] + coeff0_55*dmats1[55][45];
          new_coeff0_46 = coeff0_0*dmats1[0][46] + coeff0_1*dmats1[1][46] + coeff0_2*dmats1[2][46] + coeff0_3*dmats1[3][46] + coeff0_4*dmats1[4][46] + coeff0_5*dmats1[5][46] + coeff0_6*dmats1[6][46] + coeff0_7*dmats1[7][46] + coeff0_8*dmats1[8][46] + coeff0_9*dmats1[9][46] + coeff0_10*dmats1[10][46] + coeff0_11*dmats1[11][46] + coeff0_12*dmats1[12][46] + coeff0_13*dmats1[13][46] + coeff0_14*dmats1[14][46] + coeff0_15*dmats1[15][46] + coeff0_16*dmats1[16][46] + coeff0_17*dmats1[17][46] + coeff0_18*dmats1[18][46] + coeff0_19*dmats1[19][46] + coeff0_20*dmats1[20][46] + coeff0_21*dmats1[21][46] + coeff0_22*dmats1[22][46] + coeff0_23*dmats1[23][46] + coeff0_24*dmats1[24][46] + coeff0_25*dmats1[25][46] + coeff0_26*dmats1[26][46] + coeff0_27*dmats1[27][46] + coeff0_28*dmats1[28][46] + coeff0_29*dmats1[29][46] + coeff0_30*dmats1[30][46] + coeff0_31*dmats1[31][46] + coeff0_32*dmats1[32][46] + coeff0_33*dmats1[33][46] + coeff0_34*dmats1[34][46] + coeff0_35*dmats1[35][46] + coeff0_36*dmats1[36][46] + coeff0_37*dmats1[37][46] + coeff0_38*dmats1[38][46] + coeff0_39*dmats1[39][46] + coeff0_40*dmats1[40][46] + coeff0_41*dmats1[41][46] + coeff0_42*dmats1[42][46] + coeff0_43*dmats1[43][46] + coeff0_44*dmats1[44][46] + coeff0_45*dmats1[45][46] + coeff0_46*dmats1[46][46] + coeff0_47*dmats1[47][46] + coeff0_48*dmats1[48][46] + coeff0_49*dmats1[49][46] + coeff0_50*dmats1[50][46] + coeff0_51*dmats1[51][46] + coeff0_52*dmats1[52][46] + coeff0_53*dmats1[53][46] + coeff0_54*dmats1[54][46] + coeff0_55*dmats1[55][46];
          new_coeff0_47 = coeff0_0*dmats1[0][47] + coeff0_1*dmats1[1][47] + coeff0_2*dmats1[2][47] + coeff0_3*dmats1[3][47] + coeff0_4*dmats1[4][47] + coeff0_5*dmats1[5][47] + coeff0_6*dmats1[6][47] + coeff0_7*dmats1[7][47] + coeff0_8*dmats1[8][47] + coeff0_9*dmats1[9][47] + coeff0_10*dmats1[10][47] + coeff0_11*dmats1[11][47] + coeff0_12*dmats1[12][47] + coeff0_13*dmats1[13][47] + coeff0_14*dmats1[14][47] + coeff0_15*dmats1[15][47] + coeff0_16*dmats1[16][47] + coeff0_17*dmats1[17][47] + coeff0_18*dmats1[18][47] + coeff0_19*dmats1[19][47] + coeff0_20*dmats1[20][47] + coeff0_21*dmats1[21][47] + coeff0_22*dmats1[22][47] + coeff0_23*dmats1[23][47] + coeff0_24*dmats1[24][47] + coeff0_25*dmats1[25][47] + coeff0_26*dmats1[26][47] + coeff0_27*dmats1[27][47] + coeff0_28*dmats1[28][47] + coeff0_29*dmats1[29][47] + coeff0_30*dmats1[30][47] + coeff0_31*dmats1[31][47] + coeff0_32*dmats1[32][47] + coeff0_33*dmats1[33][47] + coeff0_34*dmats1[34][47] + coeff0_35*dmats1[35][47] + coeff0_36*dmats1[36][47] + coeff0_37*dmats1[37][47] + coeff0_38*dmats1[38][47] + coeff0_39*dmats1[39][47] + coeff0_40*dmats1[40][47] + coeff0_41*dmats1[41][47] + coeff0_42*dmats1[42][47] + coeff0_43*dmats1[43][47] + coeff0_44*dmats1[44][47] + coeff0_45*dmats1[45][47] + coeff0_46*dmats1[46][47] + coeff0_47*dmats1[47][47] + coeff0_48*dmats1[48][47] + coeff0_49*dmats1[49][47] + coeff0_50*dmats1[50][47] + coeff0_51*dmats1[51][47] + coeff0_52*dmats1[52][47] + coeff0_53*dmats1[53][47] + coeff0_54*dmats1[54][47] + coeff0_55*dmats1[55][47];
          new_coeff0_48 = coeff0_0*dmats1[0][48] + coeff0_1*dmats1[1][48] + coeff0_2*dmats1[2][48] + coeff0_3*dmats1[3][48] + coeff0_4*dmats1[4][48] + coeff0_5*dmats1[5][48] + coeff0_6*dmats1[6][48] + coeff0_7*dmats1[7][48] + coeff0_8*dmats1[8][48] + coeff0_9*dmats1[9][48] + coeff0_10*dmats1[10][48] + coeff0_11*dmats1[11][48] + coeff0_12*dmats1[12][48] + coeff0_13*dmats1[13][48] + coeff0_14*dmats1[14][48] + coeff0_15*dmats1[15][48] + coeff0_16*dmats1[16][48] + coeff0_17*dmats1[17][48] + coeff0_18*dmats1[18][48] + coeff0_19*dmats1[19][48] + coeff0_20*dmats1[20][48] + coeff0_21*dmats1[21][48] + coeff0_22*dmats1[22][48] + coeff0_23*dmats1[23][48] + coeff0_24*dmats1[24][48] + coeff0_25*dmats1[25][48] + coeff0_26*dmats1[26][48] + coeff0_27*dmats1[27][48] + coeff0_28*dmats1[28][48] + coeff0_29*dmats1[29][48] + coeff0_30*dmats1[30][48] + coeff0_31*dmats1[31][48] + coeff0_32*dmats1[32][48] + coeff0_33*dmats1[33][48] + coeff0_34*dmats1[34][48] + coeff0_35*dmats1[35][48] + coeff0_36*dmats1[36][48] + coeff0_37*dmats1[37][48] + coeff0_38*dmats1[38][48] + coeff0_39*dmats1[39][48] + coeff0_40*dmats1[40][48] + coeff0_41*dmats1[41][48] + coeff0_42*dmats1[42][48] + coeff0_43*dmats1[43][48] + coeff0_44*dmats1[44][48] + coeff0_45*dmats1[45][48] + coeff0_46*dmats1[46][48] + coeff0_47*dmats1[47][48] + coeff0_48*dmats1[48][48] + coeff0_49*dmats1[49][48] + coeff0_50*dmats1[50][48] + coeff0_51*dmats1[51][48] + coeff0_52*dmats1[52][48] + coeff0_53*dmats1[53][48] + coeff0_54*dmats1[54][48] + coeff0_55*dmats1[55][48];
          new_coeff0_49 = coeff0_0*dmats1[0][49] + coeff0_1*dmats1[1][49] + coeff0_2*dmats1[2][49] + coeff0_3*dmats1[3][49] + coeff0_4*dmats1[4][49] + coeff0_5*dmats1[5][49] + coeff0_6*dmats1[6][49] + coeff0_7*dmats1[7][49] + coeff0_8*dmats1[8][49] + coeff0_9*dmats1[9][49] + coeff0_10*dmats1[10][49] + coeff0_11*dmats1[11][49] + coeff0_12*dmats1[12][49] + coeff0_13*dmats1[13][49] + coeff0_14*dmats1[14][49] + coeff0_15*dmats1[15][49] + coeff0_16*dmats1[16][49] + coeff0_17*dmats1[17][49] + coeff0_18*dmats1[18][49] + coeff0_19*dmats1[19][49] + coeff0_20*dmats1[20][49] + coeff0_21*dmats1[21][49] + coeff0_22*dmats1[22][49] + coeff0_23*dmats1[23][49] + coeff0_24*dmats1[24][49] + coeff0_25*dmats1[25][49] + coeff0_26*dmats1[26][49] + coeff0_27*dmats1[27][49] + coeff0_28*dmats1[28][49] + coeff0_29*dmats1[29][49] + coeff0_30*dmats1[30][49] + coeff0_31*dmats1[31][49] + coeff0_32*dmats1[32][49] + coeff0_33*dmats1[33][49] + coeff0_34*dmats1[34][49] + coeff0_35*dmats1[35][49] + coeff0_36*dmats1[36][49] + coeff0_37*dmats1[37][49] + coeff0_38*dmats1[38][49] + coeff0_39*dmats1[39][49] + coeff0_40*dmats1[40][49] + coeff0_41*dmats1[41][49] + coeff0_42*dmats1[42][49] + coeff0_43*dmats1[43][49] + coeff0_44*dmats1[44][49] + coeff0_45*dmats1[45][49] + coeff0_46*dmats1[46][49] + coeff0_47*dmats1[47][49] + coeff0_48*dmats1[48][49] + coeff0_49*dmats1[49][49] + coeff0_50*dmats1[50][49] + coeff0_51*dmats1[51][49] + coeff0_52*dmats1[52][49] + coeff0_53*dmats1[53][49] + coeff0_54*dmats1[54][49] + coeff0_55*dmats1[55][49];
          new_coeff0_50 = coeff0_0*dmats1[0][50] + coeff0_1*dmats1[1][50] + coeff0_2*dmats1[2][50] + coeff0_3*dmats1[3][50] + coeff0_4*dmats1[4][50] + coeff0_5*dmats1[5][50] + coeff0_6*dmats1[6][50] + coeff0_7*dmats1[7][50] + coeff0_8*dmats1[8][50] + coeff0_9*dmats1[9][50] + coeff0_10*dmats1[10][50] + coeff0_11*dmats1[11][50] + coeff0_12*dmats1[12][50] + coeff0_13*dmats1[13][50] + coeff0_14*dmats1[14][50] + coeff0_15*dmats1[15][50] + coeff0_16*dmats1[16][50] + coeff0_17*dmats1[17][50] + coeff0_18*dmats1[18][50] + coeff0_19*dmats1[19][50] + coeff0_20*dmats1[20][50] + coeff0_21*dmats1[21][50] + coeff0_22*dmats1[22][50] + coeff0_23*dmats1[23][50] + coeff0_24*dmats1[24][50] + coeff0_25*dmats1[25][50] + coeff0_26*dmats1[26][50] + coeff0_27*dmats1[27][50] + coeff0_28*dmats1[28][50] + coeff0_29*dmats1[29][50] + coeff0_30*dmats1[30][50] + coeff0_31*dmats1[31][50] + coeff0_32*dmats1[32][50] + coeff0_33*dmats1[33][50] + coeff0_34*dmats1[34][50] + coeff0_35*dmats1[35][50] + coeff0_36*dmats1[36][50] + coeff0_37*dmats1[37][50] + coeff0_38*dmats1[38][50] + coeff0_39*dmats1[39][50] + coeff0_40*dmats1[40][50] + coeff0_41*dmats1[41][50] + coeff0_42*dmats1[42][50] + coeff0_43*dmats1[43][50] + coeff0_44*dmats1[44][50] + coeff0_45*dmats1[45][50] + coeff0_46*dmats1[46][50] + coeff0_47*dmats1[47][50] + coeff0_48*dmats1[48][50] + coeff0_49*dmats1[49][50] + coeff0_50*dmats1[50][50] + coeff0_51*dmats1[51][50] + coeff0_52*dmats1[52][50] + coeff0_53*dmats1[53][50] + coeff0_54*dmats1[54][50] + coeff0_55*dmats1[55][50];
          new_coeff0_51 = coeff0_0*dmats1[0][51] + coeff0_1*dmats1[1][51] + coeff0_2*dmats1[2][51] + coeff0_3*dmats1[3][51] + coeff0_4*dmats1[4][51] + coeff0_5*dmats1[5][51] + coeff0_6*dmats1[6][51] + coeff0_7*dmats1[7][51] + coeff0_8*dmats1[8][51] + coeff0_9*dmats1[9][51] + coeff0_10*dmats1[10][51] + coeff0_11*dmats1[11][51] + coeff0_12*dmats1[12][51] + coeff0_13*dmats1[13][51] + coeff0_14*dmats1[14][51] + coeff0_15*dmats1[15][51] + coeff0_16*dmats1[16][51] + coeff0_17*dmats1[17][51] + coeff0_18*dmats1[18][51] + coeff0_19*dmats1[19][51] + coeff0_20*dmats1[20][51] + coeff0_21*dmats1[21][51] + coeff0_22*dmats1[22][51] + coeff0_23*dmats1[23][51] + coeff0_24*dmats1[24][51] + coeff0_25*dmats1[25][51] + coeff0_26*dmats1[26][51] + coeff0_27*dmats1[27][51] + coeff0_28*dmats1[28][51] + coeff0_29*dmats1[29][51] + coeff0_30*dmats1[30][51] + coeff0_31*dmats1[31][51] + coeff0_32*dmats1[32][51] + coeff0_33*dmats1[33][51] + coeff0_34*dmats1[34][51] + coeff0_35*dmats1[35][51] + coeff0_36*dmats1[36][51] + coeff0_37*dmats1[37][51] + coeff0_38*dmats1[38][51] + coeff0_39*dmats1[39][51] + coeff0_40*dmats1[40][51] + coeff0_41*dmats1[41][51] + coeff0_42*dmats1[42][51] + coeff0_43*dmats1[43][51] + coeff0_44*dmats1[44][51] + coeff0_45*dmats1[45][51] + coeff0_46*dmats1[46][51] + coeff0_47*dmats1[47][51] + coeff0_48*dmats1[48][51] + coeff0_49*dmats1[49][51] + coeff0_50*dmats1[50][51] + coeff0_51*dmats1[51][51] + coeff0_52*dmats1[52][51] + coeff0_53*dmats1[53][51] + coeff0_54*dmats1[54][51] + coeff0_55*dmats1[55][51];
          new_coeff0_52 = coeff0_0*dmats1[0][52] + coeff0_1*dmats1[1][52] + coeff0_2*dmats1[2][52] + coeff0_3*dmats1[3][52] + coeff0_4*dmats1[4][52] + coeff0_5*dmats1[5][52] + coeff0_6*dmats1[6][52] + coeff0_7*dmats1[7][52] + coeff0_8*dmats1[8][52] + coeff0_9*dmats1[9][52] + coeff0_10*dmats1[10][52] + coeff0_11*dmats1[11][52] + coeff0_12*dmats1[12][52] + coeff0_13*dmats1[13][52] + coeff0_14*dmats1[14][52] + coeff0_15*dmats1[15][52] + coeff0_16*dmats1[16][52] + coeff0_17*dmats1[17][52] + coeff0_18*dmats1[18][52] + coeff0_19*dmats1[19][52] + coeff0_20*dmats1[20][52] + coeff0_21*dmats1[21][52] + coeff0_22*dmats1[22][52] + coeff0_23*dmats1[23][52] + coeff0_24*dmats1[24][52] + coeff0_25*dmats1[25][52] + coeff0_26*dmats1[26][52] + coeff0_27*dmats1[27][52] + coeff0_28*dmats1[28][52] + coeff0_29*dmats1[29][52] + coeff0_30*dmats1[30][52] + coeff0_31*dmats1[31][52] + coeff0_32*dmats1[32][52] + coeff0_33*dmats1[33][52] + coeff0_34*dmats1[34][52] + coeff0_35*dmats1[35][52] + coeff0_36*dmats1[36][52] + coeff0_37*dmats1[37][52] + coeff0_38*dmats1[38][52] + coeff0_39*dmats1[39][52] + coeff0_40*dmats1[40][52] + coeff0_41*dmats1[41][52] + coeff0_42*dmats1[42][52] + coeff0_43*dmats1[43][52] + coeff0_44*dmats1[44][52] + coeff0_45*dmats1[45][52] + coeff0_46*dmats1[46][52] + coeff0_47*dmats1[47][52] + coeff0_48*dmats1[48][52] + coeff0_49*dmats1[49][52] + coeff0_50*dmats1[50][52] + coeff0_51*dmats1[51][52] + coeff0_52*dmats1[52][52] + coeff0_53*dmats1[53][52] + coeff0_54*dmats1[54][52] + coeff0_55*dmats1[55][52];
          new_coeff0_53 = coeff0_0*dmats1[0][53] + coeff0_1*dmats1[1][53] + coeff0_2*dmats1[2][53] + coeff0_3*dmats1[3][53] + coeff0_4*dmats1[4][53] + coeff0_5*dmats1[5][53] + coeff0_6*dmats1[6][53] + coeff0_7*dmats1[7][53] + coeff0_8*dmats1[8][53] + coeff0_9*dmats1[9][53] + coeff0_10*dmats1[10][53] + coeff0_11*dmats1[11][53] + coeff0_12*dmats1[12][53] + coeff0_13*dmats1[13][53] + coeff0_14*dmats1[14][53] + coeff0_15*dmats1[15][53] + coeff0_16*dmats1[16][53] + coeff0_17*dmats1[17][53] + coeff0_18*dmats1[18][53] + coeff0_19*dmats1[19][53] + coeff0_20*dmats1[20][53] + coeff0_21*dmats1[21][53] + coeff0_22*dmats1[22][53] + coeff0_23*dmats1[23][53] + coeff0_24*dmats1[24][53] + coeff0_25*dmats1[25][53] + coeff0_26*dmats1[26][53] + coeff0_27*dmats1[27][53] + coeff0_28*dmats1[28][53] + coeff0_29*dmats1[29][53] + coeff0_30*dmats1[30][53] + coeff0_31*dmats1[31][53] + coeff0_32*dmats1[32][53] + coeff0_33*dmats1[33][53] + coeff0_34*dmats1[34][53] + coeff0_35*dmats1[35][53] + coeff0_36*dmats1[36][53] + coeff0_37*dmats1[37][53] + coeff0_38*dmats1[38][53] + coeff0_39*dmats1[39][53] + coeff0_40*dmats1[40][53] + coeff0_41*dmats1[41][53] + coeff0_42*dmats1[42][53] + coeff0_43*dmats1[43][53] + coeff0_44*dmats1[44][53] + coeff0_45*dmats1[45][53] + coeff0_46*dmats1[46][53] + coeff0_47*dmats1[47][53] + coeff0_48*dmats1[48][53] + coeff0_49*dmats1[49][53] + coeff0_50*dmats1[50][53] + coeff0_51*dmats1[51][53] + coeff0_52*dmats1[52][53] + coeff0_53*dmats1[53][53] + coeff0_54*dmats1[54][53] + coeff0_55*dmats1[55][53];
          new_coeff0_54 = coeff0_0*dmats1[0][54] + coeff0_1*dmats1[1][54] + coeff0_2*dmats1[2][54] + coeff0_3*dmats1[3][54] + coeff0_4*dmats1[4][54] + coeff0_5*dmats1[5][54] + coeff0_6*dmats1[6][54] + coeff0_7*dmats1[7][54] + coeff0_8*dmats1[8][54] + coeff0_9*dmats1[9][54] + coeff0_10*dmats1[10][54] + coeff0_11*dmats1[11][54] + coeff0_12*dmats1[12][54] + coeff0_13*dmats1[13][54] + coeff0_14*dmats1[14][54] + coeff0_15*dmats1[15][54] + coeff0_16*dmats1[16][54] + coeff0_17*dmats1[17][54] + coeff0_18*dmats1[18][54] + coeff0_19*dmats1[19][54] + coeff0_20*dmats1[20][54] + coeff0_21*dmats1[21][54] + coeff0_22*dmats1[22][54] + coeff0_23*dmats1[23][54] + coeff0_24*dmats1[24][54] + coeff0_25*dmats1[25][54] + coeff0_26*dmats1[26][54] + coeff0_27*dmats1[27][54] + coeff0_28*dmats1[28][54] + coeff0_29*dmats1[29][54] + coeff0_30*dmats1[30][54] + coeff0_31*dmats1[31][54] + coeff0_32*dmats1[32][54] + coeff0_33*dmats1[33][54] + coeff0_34*dmats1[34][54] + coeff0_35*dmats1[35][54] + coeff0_36*dmats1[36][54] + coeff0_37*dmats1[37][54] + coeff0_38*dmats1[38][54] + coeff0_39*dmats1[39][54] + coeff0_40*dmats1[40][54] + coeff0_41*dmats1[41][54] + coeff0_42*dmats1[42][54] + coeff0_43*dmats1[43][54] + coeff0_44*dmats1[44][54] + coeff0_45*dmats1[45][54] + coeff0_46*dmats1[46][54] + coeff0_47*dmats1[47][54] + coeff0_48*dmats1[48][54] + coeff0_49*dmats1[49][54] + coeff0_50*dmats1[50][54] + coeff0_51*dmats1[51][54] + coeff0_52*dmats1[52][54] + coeff0_53*dmats1[53][54] + coeff0_54*dmats1[54][54] + coeff0_55*dmats1[55][54];
          new_coeff0_55 = coeff0_0*dmats1[0][55] + coeff0_1*dmats1[1][55] + coeff0_2*dmats1[2][55] + coeff0_3*dmats1[3][55] + coeff0_4*dmats1[4][55] + coeff0_5*dmats1[5][55] + coeff0_6*dmats1[6][55] + coeff0_7*dmats1[7][55] + coeff0_8*dmats1[8][55] + coeff0_9*dmats1[9][55] + coeff0_10*dmats1[10][55] + coeff0_11*dmats1[11][55] + coeff0_12*dmats1[12][55] + coeff0_13*dmats1[13][55] + coeff0_14*dmats1[14][55] + coeff0_15*dmats1[15][55] + coeff0_16*dmats1[16][55] + coeff0_17*dmats1[17][55] + coeff0_18*dmats1[18][55] + coeff0_19*dmats1[19][55] + coeff0_20*dmats1[20][55] + coeff0_21*dmats1[21][55] + coeff0_22*dmats1[22][55] + coeff0_23*dmats1[23][55] + coeff0_24*dmats1[24][55] + coeff0_25*dmats1[25][55] + coeff0_26*dmats1[26][55] + coeff0_27*dmats1[27][55] + coeff0_28*dmats1[28][55] + coeff0_29*dmats1[29][55] + coeff0_30*dmats1[30][55] + coeff0_31*dmats1[31][55] + coeff0_32*dmats1[32][55] + coeff0_33*dmats1[33][55] + coeff0_34*dmats1[34][55] + coeff0_35*dmats1[35][55] + coeff0_36*dmats1[36][55] + coeff0_37*dmats1[37][55] + coeff0_38*dmats1[38][55] + coeff0_39*dmats1[39][55] + coeff0_40*dmats1[40][55] + coeff0_41*dmats1[41][55] + coeff0_42*dmats1[42][55] + coeff0_43*dmats1[43][55] + coeff0_44*dmats1[44][55] + coeff0_45*dmats1[45][55] + coeff0_46*dmats1[46][55] + coeff0_47*dmats1[47][55] + coeff0_48*dmats1[48][55] + coeff0_49*dmats1[49][55] + coeff0_50*dmats1[50][55] + coeff0_51*dmats1[51][55] + coeff0_52*dmats1[52][55] + coeff0_53*dmats1[53][55] + coeff0_54*dmats1[54][55] + coeff0_55*dmats1[55][55];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0] + coeff0_4*dmats2[4][0] + coeff0_5*dmats2[5][0] + coeff0_6*dmats2[6][0] + coeff0_7*dmats2[7][0] + coeff0_8*dmats2[8][0] + coeff0_9*dmats2[9][0] + coeff0_10*dmats2[10][0] + coeff0_11*dmats2[11][0] + coeff0_12*dmats2[12][0] + coeff0_13*dmats2[13][0] + coeff0_14*dmats2[14][0] + coeff0_15*dmats2[15][0] + coeff0_16*dmats2[16][0] + coeff0_17*dmats2[17][0] + coeff0_18*dmats2[18][0] + coeff0_19*dmats2[19][0] + coeff0_20*dmats2[20][0] + coeff0_21*dmats2[21][0] + coeff0_22*dmats2[22][0] + coeff0_23*dmats2[23][0] + coeff0_24*dmats2[24][0] + coeff0_25*dmats2[25][0] + coeff0_26*dmats2[26][0] + coeff0_27*dmats2[27][0] + coeff0_28*dmats2[28][0] + coeff0_29*dmats2[29][0] + coeff0_30*dmats2[30][0] + coeff0_31*dmats2[31][0] + coeff0_32*dmats2[32][0] + coeff0_33*dmats2[33][0] + coeff0_34*dmats2[34][0] + coeff0_35*dmats2[35][0] + coeff0_36*dmats2[36][0] + coeff0_37*dmats2[37][0] + coeff0_38*dmats2[38][0] + coeff0_39*dmats2[39][0] + coeff0_40*dmats2[40][0] + coeff0_41*dmats2[41][0] + coeff0_42*dmats2[42][0] + coeff0_43*dmats2[43][0] + coeff0_44*dmats2[44][0] + coeff0_45*dmats2[45][0] + coeff0_46*dmats2[46][0] + coeff0_47*dmats2[47][0] + coeff0_48*dmats2[48][0] + coeff0_49*dmats2[49][0] + coeff0_50*dmats2[50][0] + coeff0_51*dmats2[51][0] + coeff0_52*dmats2[52][0] + coeff0_53*dmats2[53][0] + coeff0_54*dmats2[54][0] + coeff0_55*dmats2[55][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1] + coeff0_4*dmats2[4][1] + coeff0_5*dmats2[5][1] + coeff0_6*dmats2[6][1] + coeff0_7*dmats2[7][1] + coeff0_8*dmats2[8][1] + coeff0_9*dmats2[9][1] + coeff0_10*dmats2[10][1] + coeff0_11*dmats2[11][1] + coeff0_12*dmats2[12][1] + coeff0_13*dmats2[13][1] + coeff0_14*dmats2[14][1] + coeff0_15*dmats2[15][1] + coeff0_16*dmats2[16][1] + coeff0_17*dmats2[17][1] + coeff0_18*dmats2[18][1] + coeff0_19*dmats2[19][1] + coeff0_20*dmats2[20][1] + coeff0_21*dmats2[21][1] + coeff0_22*dmats2[22][1] + coeff0_23*dmats2[23][1] + coeff0_24*dmats2[24][1] + coeff0_25*dmats2[25][1] + coeff0_26*dmats2[26][1] + coeff0_27*dmats2[27][1] + coeff0_28*dmats2[28][1] + coeff0_29*dmats2[29][1] + coeff0_30*dmats2[30][1] + coeff0_31*dmats2[31][1] + coeff0_32*dmats2[32][1] + coeff0_33*dmats2[33][1] + coeff0_34*dmats2[34][1] + coeff0_35*dmats2[35][1] + coeff0_36*dmats2[36][1] + coeff0_37*dmats2[37][1] + coeff0_38*dmats2[38][1] + coeff0_39*dmats2[39][1] + coeff0_40*dmats2[40][1] + coeff0_41*dmats2[41][1] + coeff0_42*dmats2[42][1] + coeff0_43*dmats2[43][1] + coeff0_44*dmats2[44][1] + coeff0_45*dmats2[45][1] + coeff0_46*dmats2[46][1] + coeff0_47*dmats2[47][1] + coeff0_48*dmats2[48][1] + coeff0_49*dmats2[49][1] + coeff0_50*dmats2[50][1] + coeff0_51*dmats2[51][1] + coeff0_52*dmats2[52][1] + coeff0_53*dmats2[53][1] + coeff0_54*dmats2[54][1] + coeff0_55*dmats2[55][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2] + coeff0_4*dmats2[4][2] + coeff0_5*dmats2[5][2] + coeff0_6*dmats2[6][2] + coeff0_7*dmats2[7][2] + coeff0_8*dmats2[8][2] + coeff0_9*dmats2[9][2] + coeff0_10*dmats2[10][2] + coeff0_11*dmats2[11][2] + coeff0_12*dmats2[12][2] + coeff0_13*dmats2[13][2] + coeff0_14*dmats2[14][2] + coeff0_15*dmats2[15][2] + coeff0_16*dmats2[16][2] + coeff0_17*dmats2[17][2] + coeff0_18*dmats2[18][2] + coeff0_19*dmats2[19][2] + coeff0_20*dmats2[20][2] + coeff0_21*dmats2[21][2] + coeff0_22*dmats2[22][2] + coeff0_23*dmats2[23][2] + coeff0_24*dmats2[24][2] + coeff0_25*dmats2[25][2] + coeff0_26*dmats2[26][2] + coeff0_27*dmats2[27][2] + coeff0_28*dmats2[28][2] + coeff0_29*dmats2[29][2] + coeff0_30*dmats2[30][2] + coeff0_31*dmats2[31][2] + coeff0_32*dmats2[32][2] + coeff0_33*dmats2[33][2] + coeff0_34*dmats2[34][2] + coeff0_35*dmats2[35][2] + coeff0_36*dmats2[36][2] + coeff0_37*dmats2[37][2] + coeff0_38*dmats2[38][2] + coeff0_39*dmats2[39][2] + coeff0_40*dmats2[40][2] + coeff0_41*dmats2[41][2] + coeff0_42*dmats2[42][2] + coeff0_43*dmats2[43][2] + coeff0_44*dmats2[44][2] + coeff0_45*dmats2[45][2] + coeff0_46*dmats2[46][2] + coeff0_47*dmats2[47][2] + coeff0_48*dmats2[48][2] + coeff0_49*dmats2[49][2] + coeff0_50*dmats2[50][2] + coeff0_51*dmats2[51][2] + coeff0_52*dmats2[52][2] + coeff0_53*dmats2[53][2] + coeff0_54*dmats2[54][2] + coeff0_55*dmats2[55][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3] + coeff0_4*dmats2[4][3] + coeff0_5*dmats2[5][3] + coeff0_6*dmats2[6][3] + coeff0_7*dmats2[7][3] + coeff0_8*dmats2[8][3] + coeff0_9*dmats2[9][3] + coeff0_10*dmats2[10][3] + coeff0_11*dmats2[11][3] + coeff0_12*dmats2[12][3] + coeff0_13*dmats2[13][3] + coeff0_14*dmats2[14][3] + coeff0_15*dmats2[15][3] + coeff0_16*dmats2[16][3] + coeff0_17*dmats2[17][3] + coeff0_18*dmats2[18][3] + coeff0_19*dmats2[19][3] + coeff0_20*dmats2[20][3] + coeff0_21*dmats2[21][3] + coeff0_22*dmats2[22][3] + coeff0_23*dmats2[23][3] + coeff0_24*dmats2[24][3] + coeff0_25*dmats2[25][3] + coeff0_26*dmats2[26][3] + coeff0_27*dmats2[27][3] + coeff0_28*dmats2[28][3] + coeff0_29*dmats2[29][3] + coeff0_30*dmats2[30][3] + coeff0_31*dmats2[31][3] + coeff0_32*dmats2[32][3] + coeff0_33*dmats2[33][3] + coeff0_34*dmats2[34][3] + coeff0_35*dmats2[35][3] + coeff0_36*dmats2[36][3] + coeff0_37*dmats2[37][3] + coeff0_38*dmats2[38][3] + coeff0_39*dmats2[39][3] + coeff0_40*dmats2[40][3] + coeff0_41*dmats2[41][3] + coeff0_42*dmats2[42][3] + coeff0_43*dmats2[43][3] + coeff0_44*dmats2[44][3] + coeff0_45*dmats2[45][3] + coeff0_46*dmats2[46][3] + coeff0_47*dmats2[47][3] + coeff0_48*dmats2[48][3] + coeff0_49*dmats2[49][3] + coeff0_50*dmats2[50][3] + coeff0_51*dmats2[51][3] + coeff0_52*dmats2[52][3] + coeff0_53*dmats2[53][3] + coeff0_54*dmats2[54][3] + coeff0_55*dmats2[55][3];
          new_coeff0_4 = coeff0_0*dmats2[0][4] + coeff0_1*dmats2[1][4] + coeff0_2*dmats2[2][4] + coeff0_3*dmats2[3][4] + coeff0_4*dmats2[4][4] + coeff0_5*dmats2[5][4] + coeff0_6*dmats2[6][4] + coeff0_7*dmats2[7][4] + coeff0_8*dmats2[8][4] + coeff0_9*dmats2[9][4] + coeff0_10*dmats2[10][4] + coeff0_11*dmats2[11][4] + coeff0_12*dmats2[12][4] + coeff0_13*dmats2[13][4] + coeff0_14*dmats2[14][4] + coeff0_15*dmats2[15][4] + coeff0_16*dmats2[16][4] + coeff0_17*dmats2[17][4] + coeff0_18*dmats2[18][4] + coeff0_19*dmats2[19][4] + coeff0_20*dmats2[20][4] + coeff0_21*dmats2[21][4] + coeff0_22*dmats2[22][4] + coeff0_23*dmats2[23][4] + coeff0_24*dmats2[24][4] + coeff0_25*dmats2[25][4] + coeff0_26*dmats2[26][4] + coeff0_27*dmats2[27][4] + coeff0_28*dmats2[28][4] + coeff0_29*dmats2[29][4] + coeff0_30*dmats2[30][4] + coeff0_31*dmats2[31][4] + coeff0_32*dmats2[32][4] + coeff0_33*dmats2[33][4] + coeff0_34*dmats2[34][4] + coeff0_35*dmats2[35][4] + coeff0_36*dmats2[36][4] + coeff0_37*dmats2[37][4] + coeff0_38*dmats2[38][4] + coeff0_39*dmats2[39][4] + coeff0_40*dmats2[40][4] + coeff0_41*dmats2[41][4] + coeff0_42*dmats2[42][4] + coeff0_43*dmats2[43][4] + coeff0_44*dmats2[44][4] + coeff0_45*dmats2[45][4] + coeff0_46*dmats2[46][4] + coeff0_47*dmats2[47][4] + coeff0_48*dmats2[48][4] + coeff0_49*dmats2[49][4] + coeff0_50*dmats2[50][4] + coeff0_51*dmats2[51][4] + coeff0_52*dmats2[52][4] + coeff0_53*dmats2[53][4] + coeff0_54*dmats2[54][4] + coeff0_55*dmats2[55][4];
          new_coeff0_5 = coeff0_0*dmats2[0][5] + coeff0_1*dmats2[1][5] + coeff0_2*dmats2[2][5] + coeff0_3*dmats2[3][5] + coeff0_4*dmats2[4][5] + coeff0_5*dmats2[5][5] + coeff0_6*dmats2[6][5] + coeff0_7*dmats2[7][5] + coeff0_8*dmats2[8][5] + coeff0_9*dmats2[9][5] + coeff0_10*dmats2[10][5] + coeff0_11*dmats2[11][5] + coeff0_12*dmats2[12][5] + coeff0_13*dmats2[13][5] + coeff0_14*dmats2[14][5] + coeff0_15*dmats2[15][5] + coeff0_16*dmats2[16][5] + coeff0_17*dmats2[17][5] + coeff0_18*dmats2[18][5] + coeff0_19*dmats2[19][5] + coeff0_20*dmats2[20][5] + coeff0_21*dmats2[21][5] + coeff0_22*dmats2[22][5] + coeff0_23*dmats2[23][5] + coeff0_24*dmats2[24][5] + coeff0_25*dmats2[25][5] + coeff0_26*dmats2[26][5] + coeff0_27*dmats2[27][5] + coeff0_28*dmats2[28][5] + coeff0_29*dmats2[29][5] + coeff0_30*dmats2[30][5] + coeff0_31*dmats2[31][5] + coeff0_32*dmats2[32][5] + coeff0_33*dmats2[33][5] + coeff0_34*dmats2[34][5] + coeff0_35*dmats2[35][5] + coeff0_36*dmats2[36][5] + coeff0_37*dmats2[37][5] + coeff0_38*dmats2[38][5] + coeff0_39*dmats2[39][5] + coeff0_40*dmats2[40][5] + coeff0_41*dmats2[41][5] + coeff0_42*dmats2[42][5] + coeff0_43*dmats2[43][5] + coeff0_44*dmats2[44][5] + coeff0_45*dmats2[45][5] + coeff0_46*dmats2[46][5] + coeff0_47*dmats2[47][5] + coeff0_48*dmats2[48][5] + coeff0_49*dmats2[49][5] + coeff0_50*dmats2[50][5] + coeff0_51*dmats2[51][5] + coeff0_52*dmats2[52][5] + coeff0_53*dmats2[53][5] + coeff0_54*dmats2[54][5] + coeff0_55*dmats2[55][5];
          new_coeff0_6 = coeff0_0*dmats2[0][6] + coeff0_1*dmats2[1][6] + coeff0_2*dmats2[2][6] + coeff0_3*dmats2[3][6] + coeff0_4*dmats2[4][6] + coeff0_5*dmats2[5][6] + coeff0_6*dmats2[6][6] + coeff0_7*dmats2[7][6] + coeff0_8*dmats2[8][6] + coeff0_9*dmats2[9][6] + coeff0_10*dmats2[10][6] + coeff0_11*dmats2[11][6] + coeff0_12*dmats2[12][6] + coeff0_13*dmats2[13][6] + coeff0_14*dmats2[14][6] + coeff0_15*dmats2[15][6] + coeff0_16*dmats2[16][6] + coeff0_17*dmats2[17][6] + coeff0_18*dmats2[18][6] + coeff0_19*dmats2[19][6] + coeff0_20*dmats2[20][6] + coeff0_21*dmats2[21][6] + coeff0_22*dmats2[22][6] + coeff0_23*dmats2[23][6] + coeff0_24*dmats2[24][6] + coeff0_25*dmats2[25][6] + coeff0_26*dmats2[26][6] + coeff0_27*dmats2[27][6] + coeff0_28*dmats2[28][6] + coeff0_29*dmats2[29][6] + coeff0_30*dmats2[30][6] + coeff0_31*dmats2[31][6] + coeff0_32*dmats2[32][6] + coeff0_33*dmats2[33][6] + coeff0_34*dmats2[34][6] + coeff0_35*dmats2[35][6] + coeff0_36*dmats2[36][6] + coeff0_37*dmats2[37][6] + coeff0_38*dmats2[38][6] + coeff0_39*dmats2[39][6] + coeff0_40*dmats2[40][6] + coeff0_41*dmats2[41][6] + coeff0_42*dmats2[42][6] + coeff0_43*dmats2[43][6] + coeff0_44*dmats2[44][6] + coeff0_45*dmats2[45][6] + coeff0_46*dmats2[46][6] + coeff0_47*dmats2[47][6] + coeff0_48*dmats2[48][6] + coeff0_49*dmats2[49][6] + coeff0_50*dmats2[50][6] + coeff0_51*dmats2[51][6] + coeff0_52*dmats2[52][6] + coeff0_53*dmats2[53][6] + coeff0_54*dmats2[54][6] + coeff0_55*dmats2[55][6];
          new_coeff0_7 = coeff0_0*dmats2[0][7] + coeff0_1*dmats2[1][7] + coeff0_2*dmats2[2][7] + coeff0_3*dmats2[3][7] + coeff0_4*dmats2[4][7] + coeff0_5*dmats2[5][7] + coeff0_6*dmats2[6][7] + coeff0_7*dmats2[7][7] + coeff0_8*dmats2[8][7] + coeff0_9*dmats2[9][7] + coeff0_10*dmats2[10][7] + coeff0_11*dmats2[11][7] + coeff0_12*dmats2[12][7] + coeff0_13*dmats2[13][7] + coeff0_14*dmats2[14][7] + coeff0_15*dmats2[15][7] + coeff0_16*dmats2[16][7] + coeff0_17*dmats2[17][7] + coeff0_18*dmats2[18][7] + coeff0_19*dmats2[19][7] + coeff0_20*dmats2[20][7] + coeff0_21*dmats2[21][7] + coeff0_22*dmats2[22][7] + coeff0_23*dmats2[23][7] + coeff0_24*dmats2[24][7] + coeff0_25*dmats2[25][7] + coeff0_26*dmats2[26][7] + coeff0_27*dmats2[27][7] + coeff0_28*dmats2[28][7] + coeff0_29*dmats2[29][7] + coeff0_30*dmats2[30][7] + coeff0_31*dmats2[31][7] + coeff0_32*dmats2[32][7] + coeff0_33*dmats2[33][7] + coeff0_34*dmats2[34][7] + coeff0_35*dmats2[35][7] + coeff0_36*dmats2[36][7] + coeff0_37*dmats2[37][7] + coeff0_38*dmats2[38][7] + coeff0_39*dmats2[39][7] + coeff0_40*dmats2[40][7] + coeff0_41*dmats2[41][7] + coeff0_42*dmats2[42][7] + coeff0_43*dmats2[43][7] + coeff0_44*dmats2[44][7] + coeff0_45*dmats2[45][7] + coeff0_46*dmats2[46][7] + coeff0_47*dmats2[47][7] + coeff0_48*dmats2[48][7] + coeff0_49*dmats2[49][7] + coeff0_50*dmats2[50][7] + coeff0_51*dmats2[51][7] + coeff0_52*dmats2[52][7] + coeff0_53*dmats2[53][7] + coeff0_54*dmats2[54][7] + coeff0_55*dmats2[55][7];
          new_coeff0_8 = coeff0_0*dmats2[0][8] + coeff0_1*dmats2[1][8] + coeff0_2*dmats2[2][8] + coeff0_3*dmats2[3][8] + coeff0_4*dmats2[4][8] + coeff0_5*dmats2[5][8] + coeff0_6*dmats2[6][8] + coeff0_7*dmats2[7][8] + coeff0_8*dmats2[8][8] + coeff0_9*dmats2[9][8] + coeff0_10*dmats2[10][8] + coeff0_11*dmats2[11][8] + coeff0_12*dmats2[12][8] + coeff0_13*dmats2[13][8] + coeff0_14*dmats2[14][8] + coeff0_15*dmats2[15][8] + coeff0_16*dmats2[16][8] + coeff0_17*dmats2[17][8] + coeff0_18*dmats2[18][8] + coeff0_19*dmats2[19][8] + coeff0_20*dmats2[20][8] + coeff0_21*dmats2[21][8] + coeff0_22*dmats2[22][8] + coeff0_23*dmats2[23][8] + coeff0_24*dmats2[24][8] + coeff0_25*dmats2[25][8] + coeff0_26*dmats2[26][8] + coeff0_27*dmats2[27][8] + coeff0_28*dmats2[28][8] + coeff0_29*dmats2[29][8] + coeff0_30*dmats2[30][8] + coeff0_31*dmats2[31][8] + coeff0_32*dmats2[32][8] + coeff0_33*dmats2[33][8] + coeff0_34*dmats2[34][8] + coeff0_35*dmats2[35][8] + coeff0_36*dmats2[36][8] + coeff0_37*dmats2[37][8] + coeff0_38*dmats2[38][8] + coeff0_39*dmats2[39][8] + coeff0_40*dmats2[40][8] + coeff0_41*dmats2[41][8] + coeff0_42*dmats2[42][8] + coeff0_43*dmats2[43][8] + coeff0_44*dmats2[44][8] + coeff0_45*dmats2[45][8] + coeff0_46*dmats2[46][8] + coeff0_47*dmats2[47][8] + coeff0_48*dmats2[48][8] + coeff0_49*dmats2[49][8] + coeff0_50*dmats2[50][8] + coeff0_51*dmats2[51][8] + coeff0_52*dmats2[52][8] + coeff0_53*dmats2[53][8] + coeff0_54*dmats2[54][8] + coeff0_55*dmats2[55][8];
          new_coeff0_9 = coeff0_0*dmats2[0][9] + coeff0_1*dmats2[1][9] + coeff0_2*dmats2[2][9] + coeff0_3*dmats2[3][9] + coeff0_4*dmats2[4][9] + coeff0_5*dmats2[5][9] + coeff0_6*dmats2[6][9] + coeff0_7*dmats2[7][9] + coeff0_8*dmats2[8][9] + coeff0_9*dmats2[9][9] + coeff0_10*dmats2[10][9] + coeff0_11*dmats2[11][9] + coeff0_12*dmats2[12][9] + coeff0_13*dmats2[13][9] + coeff0_14*dmats2[14][9] + coeff0_15*dmats2[15][9] + coeff0_16*dmats2[16][9] + coeff0_17*dmats2[17][9] + coeff0_18*dmats2[18][9] + coeff0_19*dmats2[19][9] + coeff0_20*dmats2[20][9] + coeff0_21*dmats2[21][9] + coeff0_22*dmats2[22][9] + coeff0_23*dmats2[23][9] + coeff0_24*dmats2[24][9] + coeff0_25*dmats2[25][9] + coeff0_26*dmats2[26][9] + coeff0_27*dmats2[27][9] + coeff0_28*dmats2[28][9] + coeff0_29*dmats2[29][9] + coeff0_30*dmats2[30][9] + coeff0_31*dmats2[31][9] + coeff0_32*dmats2[32][9] + coeff0_33*dmats2[33][9] + coeff0_34*dmats2[34][9] + coeff0_35*dmats2[35][9] + coeff0_36*dmats2[36][9] + coeff0_37*dmats2[37][9] + coeff0_38*dmats2[38][9] + coeff0_39*dmats2[39][9] + coeff0_40*dmats2[40][9] + coeff0_41*dmats2[41][9] + coeff0_42*dmats2[42][9] + coeff0_43*dmats2[43][9] + coeff0_44*dmats2[44][9] + coeff0_45*dmats2[45][9] + coeff0_46*dmats2[46][9] + coeff0_47*dmats2[47][9] + coeff0_48*dmats2[48][9] + coeff0_49*dmats2[49][9] + coeff0_50*dmats2[50][9] + coeff0_51*dmats2[51][9] + coeff0_52*dmats2[52][9] + coeff0_53*dmats2[53][9] + coeff0_54*dmats2[54][9] + coeff0_55*dmats2[55][9];
          new_coeff0_10 = coeff0_0*dmats2[0][10] + coeff0_1*dmats2[1][10] + coeff0_2*dmats2[2][10] + coeff0_3*dmats2[3][10] + coeff0_4*dmats2[4][10] + coeff0_5*dmats2[5][10] + coeff0_6*dmats2[6][10] + coeff0_7*dmats2[7][10] + coeff0_8*dmats2[8][10] + coeff0_9*dmats2[9][10] + coeff0_10*dmats2[10][10] + coeff0_11*dmats2[11][10] + coeff0_12*dmats2[12][10] + coeff0_13*dmats2[13][10] + coeff0_14*dmats2[14][10] + coeff0_15*dmats2[15][10] + coeff0_16*dmats2[16][10] + coeff0_17*dmats2[17][10] + coeff0_18*dmats2[18][10] + coeff0_19*dmats2[19][10] + coeff0_20*dmats2[20][10] + coeff0_21*dmats2[21][10] + coeff0_22*dmats2[22][10] + coeff0_23*dmats2[23][10] + coeff0_24*dmats2[24][10] + coeff0_25*dmats2[25][10] + coeff0_26*dmats2[26][10] + coeff0_27*dmats2[27][10] + coeff0_28*dmats2[28][10] + coeff0_29*dmats2[29][10] + coeff0_30*dmats2[30][10] + coeff0_31*dmats2[31][10] + coeff0_32*dmats2[32][10] + coeff0_33*dmats2[33][10] + coeff0_34*dmats2[34][10] + coeff0_35*dmats2[35][10] + coeff0_36*dmats2[36][10] + coeff0_37*dmats2[37][10] + coeff0_38*dmats2[38][10] + coeff0_39*dmats2[39][10] + coeff0_40*dmats2[40][10] + coeff0_41*dmats2[41][10] + coeff0_42*dmats2[42][10] + coeff0_43*dmats2[43][10] + coeff0_44*dmats2[44][10] + coeff0_45*dmats2[45][10] + coeff0_46*dmats2[46][10] + coeff0_47*dmats2[47][10] + coeff0_48*dmats2[48][10] + coeff0_49*dmats2[49][10] + coeff0_50*dmats2[50][10] + coeff0_51*dmats2[51][10] + coeff0_52*dmats2[52][10] + coeff0_53*dmats2[53][10] + coeff0_54*dmats2[54][10] + coeff0_55*dmats2[55][10];
          new_coeff0_11 = coeff0_0*dmats2[0][11] + coeff0_1*dmats2[1][11] + coeff0_2*dmats2[2][11] + coeff0_3*dmats2[3][11] + coeff0_4*dmats2[4][11] + coeff0_5*dmats2[5][11] + coeff0_6*dmats2[6][11] + coeff0_7*dmats2[7][11] + coeff0_8*dmats2[8][11] + coeff0_9*dmats2[9][11] + coeff0_10*dmats2[10][11] + coeff0_11*dmats2[11][11] + coeff0_12*dmats2[12][11] + coeff0_13*dmats2[13][11] + coeff0_14*dmats2[14][11] + coeff0_15*dmats2[15][11] + coeff0_16*dmats2[16][11] + coeff0_17*dmats2[17][11] + coeff0_18*dmats2[18][11] + coeff0_19*dmats2[19][11] + coeff0_20*dmats2[20][11] + coeff0_21*dmats2[21][11] + coeff0_22*dmats2[22][11] + coeff0_23*dmats2[23][11] + coeff0_24*dmats2[24][11] + coeff0_25*dmats2[25][11] + coeff0_26*dmats2[26][11] + coeff0_27*dmats2[27][11] + coeff0_28*dmats2[28][11] + coeff0_29*dmats2[29][11] + coeff0_30*dmats2[30][11] + coeff0_31*dmats2[31][11] + coeff0_32*dmats2[32][11] + coeff0_33*dmats2[33][11] + coeff0_34*dmats2[34][11] + coeff0_35*dmats2[35][11] + coeff0_36*dmats2[36][11] + coeff0_37*dmats2[37][11] + coeff0_38*dmats2[38][11] + coeff0_39*dmats2[39][11] + coeff0_40*dmats2[40][11] + coeff0_41*dmats2[41][11] + coeff0_42*dmats2[42][11] + coeff0_43*dmats2[43][11] + coeff0_44*dmats2[44][11] + coeff0_45*dmats2[45][11] + coeff0_46*dmats2[46][11] + coeff0_47*dmats2[47][11] + coeff0_48*dmats2[48][11] + coeff0_49*dmats2[49][11] + coeff0_50*dmats2[50][11] + coeff0_51*dmats2[51][11] + coeff0_52*dmats2[52][11] + coeff0_53*dmats2[53][11] + coeff0_54*dmats2[54][11] + coeff0_55*dmats2[55][11];
          new_coeff0_12 = coeff0_0*dmats2[0][12] + coeff0_1*dmats2[1][12] + coeff0_2*dmats2[2][12] + coeff0_3*dmats2[3][12] + coeff0_4*dmats2[4][12] + coeff0_5*dmats2[5][12] + coeff0_6*dmats2[6][12] + coeff0_7*dmats2[7][12] + coeff0_8*dmats2[8][12] + coeff0_9*dmats2[9][12] + coeff0_10*dmats2[10][12] + coeff0_11*dmats2[11][12] + coeff0_12*dmats2[12][12] + coeff0_13*dmats2[13][12] + coeff0_14*dmats2[14][12] + coeff0_15*dmats2[15][12] + coeff0_16*dmats2[16][12] + coeff0_17*dmats2[17][12] + coeff0_18*dmats2[18][12] + coeff0_19*dmats2[19][12] + coeff0_20*dmats2[20][12] + coeff0_21*dmats2[21][12] + coeff0_22*dmats2[22][12] + coeff0_23*dmats2[23][12] + coeff0_24*dmats2[24][12] + coeff0_25*dmats2[25][12] + coeff0_26*dmats2[26][12] + coeff0_27*dmats2[27][12] + coeff0_28*dmats2[28][12] + coeff0_29*dmats2[29][12] + coeff0_30*dmats2[30][12] + coeff0_31*dmats2[31][12] + coeff0_32*dmats2[32][12] + coeff0_33*dmats2[33][12] + coeff0_34*dmats2[34][12] + coeff0_35*dmats2[35][12] + coeff0_36*dmats2[36][12] + coeff0_37*dmats2[37][12] + coeff0_38*dmats2[38][12] + coeff0_39*dmats2[39][12] + coeff0_40*dmats2[40][12] + coeff0_41*dmats2[41][12] + coeff0_42*dmats2[42][12] + coeff0_43*dmats2[43][12] + coeff0_44*dmats2[44][12] + coeff0_45*dmats2[45][12] + coeff0_46*dmats2[46][12] + coeff0_47*dmats2[47][12] + coeff0_48*dmats2[48][12] + coeff0_49*dmats2[49][12] + coeff0_50*dmats2[50][12] + coeff0_51*dmats2[51][12] + coeff0_52*dmats2[52][12] + coeff0_53*dmats2[53][12] + coeff0_54*dmats2[54][12] + coeff0_55*dmats2[55][12];
          new_coeff0_13 = coeff0_0*dmats2[0][13] + coeff0_1*dmats2[1][13] + coeff0_2*dmats2[2][13] + coeff0_3*dmats2[3][13] + coeff0_4*dmats2[4][13] + coeff0_5*dmats2[5][13] + coeff0_6*dmats2[6][13] + coeff0_7*dmats2[7][13] + coeff0_8*dmats2[8][13] + coeff0_9*dmats2[9][13] + coeff0_10*dmats2[10][13] + coeff0_11*dmats2[11][13] + coeff0_12*dmats2[12][13] + coeff0_13*dmats2[13][13] + coeff0_14*dmats2[14][13] + coeff0_15*dmats2[15][13] + coeff0_16*dmats2[16][13] + coeff0_17*dmats2[17][13] + coeff0_18*dmats2[18][13] + coeff0_19*dmats2[19][13] + coeff0_20*dmats2[20][13] + coeff0_21*dmats2[21][13] + coeff0_22*dmats2[22][13] + coeff0_23*dmats2[23][13] + coeff0_24*dmats2[24][13] + coeff0_25*dmats2[25][13] + coeff0_26*dmats2[26][13] + coeff0_27*dmats2[27][13] + coeff0_28*dmats2[28][13] + coeff0_29*dmats2[29][13] + coeff0_30*dmats2[30][13] + coeff0_31*dmats2[31][13] + coeff0_32*dmats2[32][13] + coeff0_33*dmats2[33][13] + coeff0_34*dmats2[34][13] + coeff0_35*dmats2[35][13] + coeff0_36*dmats2[36][13] + coeff0_37*dmats2[37][13] + coeff0_38*dmats2[38][13] + coeff0_39*dmats2[39][13] + coeff0_40*dmats2[40][13] + coeff0_41*dmats2[41][13] + coeff0_42*dmats2[42][13] + coeff0_43*dmats2[43][13] + coeff0_44*dmats2[44][13] + coeff0_45*dmats2[45][13] + coeff0_46*dmats2[46][13] + coeff0_47*dmats2[47][13] + coeff0_48*dmats2[48][13] + coeff0_49*dmats2[49][13] + coeff0_50*dmats2[50][13] + coeff0_51*dmats2[51][13] + coeff0_52*dmats2[52][13] + coeff0_53*dmats2[53][13] + coeff0_54*dmats2[54][13] + coeff0_55*dmats2[55][13];
          new_coeff0_14 = coeff0_0*dmats2[0][14] + coeff0_1*dmats2[1][14] + coeff0_2*dmats2[2][14] + coeff0_3*dmats2[3][14] + coeff0_4*dmats2[4][14] + coeff0_5*dmats2[5][14] + coeff0_6*dmats2[6][14] + coeff0_7*dmats2[7][14] + coeff0_8*dmats2[8][14] + coeff0_9*dmats2[9][14] + coeff0_10*dmats2[10][14] + coeff0_11*dmats2[11][14] + coeff0_12*dmats2[12][14] + coeff0_13*dmats2[13][14] + coeff0_14*dmats2[14][14] + coeff0_15*dmats2[15][14] + coeff0_16*dmats2[16][14] + coeff0_17*dmats2[17][14] + coeff0_18*dmats2[18][14] + coeff0_19*dmats2[19][14] + coeff0_20*dmats2[20][14] + coeff0_21*dmats2[21][14] + coeff0_22*dmats2[22][14] + coeff0_23*dmats2[23][14] + coeff0_24*dmats2[24][14] + coeff0_25*dmats2[25][14] + coeff0_26*dmats2[26][14] + coeff0_27*dmats2[27][14] + coeff0_28*dmats2[28][14] + coeff0_29*dmats2[29][14] + coeff0_30*dmats2[30][14] + coeff0_31*dmats2[31][14] + coeff0_32*dmats2[32][14] + coeff0_33*dmats2[33][14] + coeff0_34*dmats2[34][14] + coeff0_35*dmats2[35][14] + coeff0_36*dmats2[36][14] + coeff0_37*dmats2[37][14] + coeff0_38*dmats2[38][14] + coeff0_39*dmats2[39][14] + coeff0_40*dmats2[40][14] + coeff0_41*dmats2[41][14] + coeff0_42*dmats2[42][14] + coeff0_43*dmats2[43][14] + coeff0_44*dmats2[44][14] + coeff0_45*dmats2[45][14] + coeff0_46*dmats2[46][14] + coeff0_47*dmats2[47][14] + coeff0_48*dmats2[48][14] + coeff0_49*dmats2[49][14] + coeff0_50*dmats2[50][14] + coeff0_51*dmats2[51][14] + coeff0_52*dmats2[52][14] + coeff0_53*dmats2[53][14] + coeff0_54*dmats2[54][14] + coeff0_55*dmats2[55][14];
          new_coeff0_15 = coeff0_0*dmats2[0][15] + coeff0_1*dmats2[1][15] + coeff0_2*dmats2[2][15] + coeff0_3*dmats2[3][15] + coeff0_4*dmats2[4][15] + coeff0_5*dmats2[5][15] + coeff0_6*dmats2[6][15] + coeff0_7*dmats2[7][15] + coeff0_8*dmats2[8][15] + coeff0_9*dmats2[9][15] + coeff0_10*dmats2[10][15] + coeff0_11*dmats2[11][15] + coeff0_12*dmats2[12][15] + coeff0_13*dmats2[13][15] + coeff0_14*dmats2[14][15] + coeff0_15*dmats2[15][15] + coeff0_16*dmats2[16][15] + coeff0_17*dmats2[17][15] + coeff0_18*dmats2[18][15] + coeff0_19*dmats2[19][15] + coeff0_20*dmats2[20][15] + coeff0_21*dmats2[21][15] + coeff0_22*dmats2[22][15] + coeff0_23*dmats2[23][15] + coeff0_24*dmats2[24][15] + coeff0_25*dmats2[25][15] + coeff0_26*dmats2[26][15] + coeff0_27*dmats2[27][15] + coeff0_28*dmats2[28][15] + coeff0_29*dmats2[29][15] + coeff0_30*dmats2[30][15] + coeff0_31*dmats2[31][15] + coeff0_32*dmats2[32][15] + coeff0_33*dmats2[33][15] + coeff0_34*dmats2[34][15] + coeff0_35*dmats2[35][15] + coeff0_36*dmats2[36][15] + coeff0_37*dmats2[37][15] + coeff0_38*dmats2[38][15] + coeff0_39*dmats2[39][15] + coeff0_40*dmats2[40][15] + coeff0_41*dmats2[41][15] + coeff0_42*dmats2[42][15] + coeff0_43*dmats2[43][15] + coeff0_44*dmats2[44][15] + coeff0_45*dmats2[45][15] + coeff0_46*dmats2[46][15] + coeff0_47*dmats2[47][15] + coeff0_48*dmats2[48][15] + coeff0_49*dmats2[49][15] + coeff0_50*dmats2[50][15] + coeff0_51*dmats2[51][15] + coeff0_52*dmats2[52][15] + coeff0_53*dmats2[53][15] + coeff0_54*dmats2[54][15] + coeff0_55*dmats2[55][15];
          new_coeff0_16 = coeff0_0*dmats2[0][16] + coeff0_1*dmats2[1][16] + coeff0_2*dmats2[2][16] + coeff0_3*dmats2[3][16] + coeff0_4*dmats2[4][16] + coeff0_5*dmats2[5][16] + coeff0_6*dmats2[6][16] + coeff0_7*dmats2[7][16] + coeff0_8*dmats2[8][16] + coeff0_9*dmats2[9][16] + coeff0_10*dmats2[10][16] + coeff0_11*dmats2[11][16] + coeff0_12*dmats2[12][16] + coeff0_13*dmats2[13][16] + coeff0_14*dmats2[14][16] + coeff0_15*dmats2[15][16] + coeff0_16*dmats2[16][16] + coeff0_17*dmats2[17][16] + coeff0_18*dmats2[18][16] + coeff0_19*dmats2[19][16] + coeff0_20*dmats2[20][16] + coeff0_21*dmats2[21][16] + coeff0_22*dmats2[22][16] + coeff0_23*dmats2[23][16] + coeff0_24*dmats2[24][16] + coeff0_25*dmats2[25][16] + coeff0_26*dmats2[26][16] + coeff0_27*dmats2[27][16] + coeff0_28*dmats2[28][16] + coeff0_29*dmats2[29][16] + coeff0_30*dmats2[30][16] + coeff0_31*dmats2[31][16] + coeff0_32*dmats2[32][16] + coeff0_33*dmats2[33][16] + coeff0_34*dmats2[34][16] + coeff0_35*dmats2[35][16] + coeff0_36*dmats2[36][16] + coeff0_37*dmats2[37][16] + coeff0_38*dmats2[38][16] + coeff0_39*dmats2[39][16] + coeff0_40*dmats2[40][16] + coeff0_41*dmats2[41][16] + coeff0_42*dmats2[42][16] + coeff0_43*dmats2[43][16] + coeff0_44*dmats2[44][16] + coeff0_45*dmats2[45][16] + coeff0_46*dmats2[46][16] + coeff0_47*dmats2[47][16] + coeff0_48*dmats2[48][16] + coeff0_49*dmats2[49][16] + coeff0_50*dmats2[50][16] + coeff0_51*dmats2[51][16] + coeff0_52*dmats2[52][16] + coeff0_53*dmats2[53][16] + coeff0_54*dmats2[54][16] + coeff0_55*dmats2[55][16];
          new_coeff0_17 = coeff0_0*dmats2[0][17] + coeff0_1*dmats2[1][17] + coeff0_2*dmats2[2][17] + coeff0_3*dmats2[3][17] + coeff0_4*dmats2[4][17] + coeff0_5*dmats2[5][17] + coeff0_6*dmats2[6][17] + coeff0_7*dmats2[7][17] + coeff0_8*dmats2[8][17] + coeff0_9*dmats2[9][17] + coeff0_10*dmats2[10][17] + coeff0_11*dmats2[11][17] + coeff0_12*dmats2[12][17] + coeff0_13*dmats2[13][17] + coeff0_14*dmats2[14][17] + coeff0_15*dmats2[15][17] + coeff0_16*dmats2[16][17] + coeff0_17*dmats2[17][17] + coeff0_18*dmats2[18][17] + coeff0_19*dmats2[19][17] + coeff0_20*dmats2[20][17] + coeff0_21*dmats2[21][17] + coeff0_22*dmats2[22][17] + coeff0_23*dmats2[23][17] + coeff0_24*dmats2[24][17] + coeff0_25*dmats2[25][17] + coeff0_26*dmats2[26][17] + coeff0_27*dmats2[27][17] + coeff0_28*dmats2[28][17] + coeff0_29*dmats2[29][17] + coeff0_30*dmats2[30][17] + coeff0_31*dmats2[31][17] + coeff0_32*dmats2[32][17] + coeff0_33*dmats2[33][17] + coeff0_34*dmats2[34][17] + coeff0_35*dmats2[35][17] + coeff0_36*dmats2[36][17] + coeff0_37*dmats2[37][17] + coeff0_38*dmats2[38][17] + coeff0_39*dmats2[39][17] + coeff0_40*dmats2[40][17] + coeff0_41*dmats2[41][17] + coeff0_42*dmats2[42][17] + coeff0_43*dmats2[43][17] + coeff0_44*dmats2[44][17] + coeff0_45*dmats2[45][17] + coeff0_46*dmats2[46][17] + coeff0_47*dmats2[47][17] + coeff0_48*dmats2[48][17] + coeff0_49*dmats2[49][17] + coeff0_50*dmats2[50][17] + coeff0_51*dmats2[51][17] + coeff0_52*dmats2[52][17] + coeff0_53*dmats2[53][17] + coeff0_54*dmats2[54][17] + coeff0_55*dmats2[55][17];
          new_coeff0_18 = coeff0_0*dmats2[0][18] + coeff0_1*dmats2[1][18] + coeff0_2*dmats2[2][18] + coeff0_3*dmats2[3][18] + coeff0_4*dmats2[4][18] + coeff0_5*dmats2[5][18] + coeff0_6*dmats2[6][18] + coeff0_7*dmats2[7][18] + coeff0_8*dmats2[8][18] + coeff0_9*dmats2[9][18] + coeff0_10*dmats2[10][18] + coeff0_11*dmats2[11][18] + coeff0_12*dmats2[12][18] + coeff0_13*dmats2[13][18] + coeff0_14*dmats2[14][18] + coeff0_15*dmats2[15][18] + coeff0_16*dmats2[16][18] + coeff0_17*dmats2[17][18] + coeff0_18*dmats2[18][18] + coeff0_19*dmats2[19][18] + coeff0_20*dmats2[20][18] + coeff0_21*dmats2[21][18] + coeff0_22*dmats2[22][18] + coeff0_23*dmats2[23][18] + coeff0_24*dmats2[24][18] + coeff0_25*dmats2[25][18] + coeff0_26*dmats2[26][18] + coeff0_27*dmats2[27][18] + coeff0_28*dmats2[28][18] + coeff0_29*dmats2[29][18] + coeff0_30*dmats2[30][18] + coeff0_31*dmats2[31][18] + coeff0_32*dmats2[32][18] + coeff0_33*dmats2[33][18] + coeff0_34*dmats2[34][18] + coeff0_35*dmats2[35][18] + coeff0_36*dmats2[36][18] + coeff0_37*dmats2[37][18] + coeff0_38*dmats2[38][18] + coeff0_39*dmats2[39][18] + coeff0_40*dmats2[40][18] + coeff0_41*dmats2[41][18] + coeff0_42*dmats2[42][18] + coeff0_43*dmats2[43][18] + coeff0_44*dmats2[44][18] + coeff0_45*dmats2[45][18] + coeff0_46*dmats2[46][18] + coeff0_47*dmats2[47][18] + coeff0_48*dmats2[48][18] + coeff0_49*dmats2[49][18] + coeff0_50*dmats2[50][18] + coeff0_51*dmats2[51][18] + coeff0_52*dmats2[52][18] + coeff0_53*dmats2[53][18] + coeff0_54*dmats2[54][18] + coeff0_55*dmats2[55][18];
          new_coeff0_19 = coeff0_0*dmats2[0][19] + coeff0_1*dmats2[1][19] + coeff0_2*dmats2[2][19] + coeff0_3*dmats2[3][19] + coeff0_4*dmats2[4][19] + coeff0_5*dmats2[5][19] + coeff0_6*dmats2[6][19] + coeff0_7*dmats2[7][19] + coeff0_8*dmats2[8][19] + coeff0_9*dmats2[9][19] + coeff0_10*dmats2[10][19] + coeff0_11*dmats2[11][19] + coeff0_12*dmats2[12][19] + coeff0_13*dmats2[13][19] + coeff0_14*dmats2[14][19] + coeff0_15*dmats2[15][19] + coeff0_16*dmats2[16][19] + coeff0_17*dmats2[17][19] + coeff0_18*dmats2[18][19] + coeff0_19*dmats2[19][19] + coeff0_20*dmats2[20][19] + coeff0_21*dmats2[21][19] + coeff0_22*dmats2[22][19] + coeff0_23*dmats2[23][19] + coeff0_24*dmats2[24][19] + coeff0_25*dmats2[25][19] + coeff0_26*dmats2[26][19] + coeff0_27*dmats2[27][19] + coeff0_28*dmats2[28][19] + coeff0_29*dmats2[29][19] + coeff0_30*dmats2[30][19] + coeff0_31*dmats2[31][19] + coeff0_32*dmats2[32][19] + coeff0_33*dmats2[33][19] + coeff0_34*dmats2[34][19] + coeff0_35*dmats2[35][19] + coeff0_36*dmats2[36][19] + coeff0_37*dmats2[37][19] + coeff0_38*dmats2[38][19] + coeff0_39*dmats2[39][19] + coeff0_40*dmats2[40][19] + coeff0_41*dmats2[41][19] + coeff0_42*dmats2[42][19] + coeff0_43*dmats2[43][19] + coeff0_44*dmats2[44][19] + coeff0_45*dmats2[45][19] + coeff0_46*dmats2[46][19] + coeff0_47*dmats2[47][19] + coeff0_48*dmats2[48][19] + coeff0_49*dmats2[49][19] + coeff0_50*dmats2[50][19] + coeff0_51*dmats2[51][19] + coeff0_52*dmats2[52][19] + coeff0_53*dmats2[53][19] + coeff0_54*dmats2[54][19] + coeff0_55*dmats2[55][19];
          new_coeff0_20 = coeff0_0*dmats2[0][20] + coeff0_1*dmats2[1][20] + coeff0_2*dmats2[2][20] + coeff0_3*dmats2[3][20] + coeff0_4*dmats2[4][20] + coeff0_5*dmats2[5][20] + coeff0_6*dmats2[6][20] + coeff0_7*dmats2[7][20] + coeff0_8*dmats2[8][20] + coeff0_9*dmats2[9][20] + coeff0_10*dmats2[10][20] + coeff0_11*dmats2[11][20] + coeff0_12*dmats2[12][20] + coeff0_13*dmats2[13][20] + coeff0_14*dmats2[14][20] + coeff0_15*dmats2[15][20] + coeff0_16*dmats2[16][20] + coeff0_17*dmats2[17][20] + coeff0_18*dmats2[18][20] + coeff0_19*dmats2[19][20] + coeff0_20*dmats2[20][20] + coeff0_21*dmats2[21][20] + coeff0_22*dmats2[22][20] + coeff0_23*dmats2[23][20] + coeff0_24*dmats2[24][20] + coeff0_25*dmats2[25][20] + coeff0_26*dmats2[26][20] + coeff0_27*dmats2[27][20] + coeff0_28*dmats2[28][20] + coeff0_29*dmats2[29][20] + coeff0_30*dmats2[30][20] + coeff0_31*dmats2[31][20] + coeff0_32*dmats2[32][20] + coeff0_33*dmats2[33][20] + coeff0_34*dmats2[34][20] + coeff0_35*dmats2[35][20] + coeff0_36*dmats2[36][20] + coeff0_37*dmats2[37][20] + coeff0_38*dmats2[38][20] + coeff0_39*dmats2[39][20] + coeff0_40*dmats2[40][20] + coeff0_41*dmats2[41][20] + coeff0_42*dmats2[42][20] + coeff0_43*dmats2[43][20] + coeff0_44*dmats2[44][20] + coeff0_45*dmats2[45][20] + coeff0_46*dmats2[46][20] + coeff0_47*dmats2[47][20] + coeff0_48*dmats2[48][20] + coeff0_49*dmats2[49][20] + coeff0_50*dmats2[50][20] + coeff0_51*dmats2[51][20] + coeff0_52*dmats2[52][20] + coeff0_53*dmats2[53][20] + coeff0_54*dmats2[54][20] + coeff0_55*dmats2[55][20];
          new_coeff0_21 = coeff0_0*dmats2[0][21] + coeff0_1*dmats2[1][21] + coeff0_2*dmats2[2][21] + coeff0_3*dmats2[3][21] + coeff0_4*dmats2[4][21] + coeff0_5*dmats2[5][21] + coeff0_6*dmats2[6][21] + coeff0_7*dmats2[7][21] + coeff0_8*dmats2[8][21] + coeff0_9*dmats2[9][21] + coeff0_10*dmats2[10][21] + coeff0_11*dmats2[11][21] + coeff0_12*dmats2[12][21] + coeff0_13*dmats2[13][21] + coeff0_14*dmats2[14][21] + coeff0_15*dmats2[15][21] + coeff0_16*dmats2[16][21] + coeff0_17*dmats2[17][21] + coeff0_18*dmats2[18][21] + coeff0_19*dmats2[19][21] + coeff0_20*dmats2[20][21] + coeff0_21*dmats2[21][21] + coeff0_22*dmats2[22][21] + coeff0_23*dmats2[23][21] + coeff0_24*dmats2[24][21] + coeff0_25*dmats2[25][21] + coeff0_26*dmats2[26][21] + coeff0_27*dmats2[27][21] + coeff0_28*dmats2[28][21] + coeff0_29*dmats2[29][21] + coeff0_30*dmats2[30][21] + coeff0_31*dmats2[31][21] + coeff0_32*dmats2[32][21] + coeff0_33*dmats2[33][21] + coeff0_34*dmats2[34][21] + coeff0_35*dmats2[35][21] + coeff0_36*dmats2[36][21] + coeff0_37*dmats2[37][21] + coeff0_38*dmats2[38][21] + coeff0_39*dmats2[39][21] + coeff0_40*dmats2[40][21] + coeff0_41*dmats2[41][21] + coeff0_42*dmats2[42][21] + coeff0_43*dmats2[43][21] + coeff0_44*dmats2[44][21] + coeff0_45*dmats2[45][21] + coeff0_46*dmats2[46][21] + coeff0_47*dmats2[47][21] + coeff0_48*dmats2[48][21] + coeff0_49*dmats2[49][21] + coeff0_50*dmats2[50][21] + coeff0_51*dmats2[51][21] + coeff0_52*dmats2[52][21] + coeff0_53*dmats2[53][21] + coeff0_54*dmats2[54][21] + coeff0_55*dmats2[55][21];
          new_coeff0_22 = coeff0_0*dmats2[0][22] + coeff0_1*dmats2[1][22] + coeff0_2*dmats2[2][22] + coeff0_3*dmats2[3][22] + coeff0_4*dmats2[4][22] + coeff0_5*dmats2[5][22] + coeff0_6*dmats2[6][22] + coeff0_7*dmats2[7][22] + coeff0_8*dmats2[8][22] + coeff0_9*dmats2[9][22] + coeff0_10*dmats2[10][22] + coeff0_11*dmats2[11][22] + coeff0_12*dmats2[12][22] + coeff0_13*dmats2[13][22] + coeff0_14*dmats2[14][22] + coeff0_15*dmats2[15][22] + coeff0_16*dmats2[16][22] + coeff0_17*dmats2[17][22] + coeff0_18*dmats2[18][22] + coeff0_19*dmats2[19][22] + coeff0_20*dmats2[20][22] + coeff0_21*dmats2[21][22] + coeff0_22*dmats2[22][22] + coeff0_23*dmats2[23][22] + coeff0_24*dmats2[24][22] + coeff0_25*dmats2[25][22] + coeff0_26*dmats2[26][22] + coeff0_27*dmats2[27][22] + coeff0_28*dmats2[28][22] + coeff0_29*dmats2[29][22] + coeff0_30*dmats2[30][22] + coeff0_31*dmats2[31][22] + coeff0_32*dmats2[32][22] + coeff0_33*dmats2[33][22] + coeff0_34*dmats2[34][22] + coeff0_35*dmats2[35][22] + coeff0_36*dmats2[36][22] + coeff0_37*dmats2[37][22] + coeff0_38*dmats2[38][22] + coeff0_39*dmats2[39][22] + coeff0_40*dmats2[40][22] + coeff0_41*dmats2[41][22] + coeff0_42*dmats2[42][22] + coeff0_43*dmats2[43][22] + coeff0_44*dmats2[44][22] + coeff0_45*dmats2[45][22] + coeff0_46*dmats2[46][22] + coeff0_47*dmats2[47][22] + coeff0_48*dmats2[48][22] + coeff0_49*dmats2[49][22] + coeff0_50*dmats2[50][22] + coeff0_51*dmats2[51][22] + coeff0_52*dmats2[52][22] + coeff0_53*dmats2[53][22] + coeff0_54*dmats2[54][22] + coeff0_55*dmats2[55][22];
          new_coeff0_23 = coeff0_0*dmats2[0][23] + coeff0_1*dmats2[1][23] + coeff0_2*dmats2[2][23] + coeff0_3*dmats2[3][23] + coeff0_4*dmats2[4][23] + coeff0_5*dmats2[5][23] + coeff0_6*dmats2[6][23] + coeff0_7*dmats2[7][23] + coeff0_8*dmats2[8][23] + coeff0_9*dmats2[9][23] + coeff0_10*dmats2[10][23] + coeff0_11*dmats2[11][23] + coeff0_12*dmats2[12][23] + coeff0_13*dmats2[13][23] + coeff0_14*dmats2[14][23] + coeff0_15*dmats2[15][23] + coeff0_16*dmats2[16][23] + coeff0_17*dmats2[17][23] + coeff0_18*dmats2[18][23] + coeff0_19*dmats2[19][23] + coeff0_20*dmats2[20][23] + coeff0_21*dmats2[21][23] + coeff0_22*dmats2[22][23] + coeff0_23*dmats2[23][23] + coeff0_24*dmats2[24][23] + coeff0_25*dmats2[25][23] + coeff0_26*dmats2[26][23] + coeff0_27*dmats2[27][23] + coeff0_28*dmats2[28][23] + coeff0_29*dmats2[29][23] + coeff0_30*dmats2[30][23] + coeff0_31*dmats2[31][23] + coeff0_32*dmats2[32][23] + coeff0_33*dmats2[33][23] + coeff0_34*dmats2[34][23] + coeff0_35*dmats2[35][23] + coeff0_36*dmats2[36][23] + coeff0_37*dmats2[37][23] + coeff0_38*dmats2[38][23] + coeff0_39*dmats2[39][23] + coeff0_40*dmats2[40][23] + coeff0_41*dmats2[41][23] + coeff0_42*dmats2[42][23] + coeff0_43*dmats2[43][23] + coeff0_44*dmats2[44][23] + coeff0_45*dmats2[45][23] + coeff0_46*dmats2[46][23] + coeff0_47*dmats2[47][23] + coeff0_48*dmats2[48][23] + coeff0_49*dmats2[49][23] + coeff0_50*dmats2[50][23] + coeff0_51*dmats2[51][23] + coeff0_52*dmats2[52][23] + coeff0_53*dmats2[53][23] + coeff0_54*dmats2[54][23] + coeff0_55*dmats2[55][23];
          new_coeff0_24 = coeff0_0*dmats2[0][24] + coeff0_1*dmats2[1][24] + coeff0_2*dmats2[2][24] + coeff0_3*dmats2[3][24] + coeff0_4*dmats2[4][24] + coeff0_5*dmats2[5][24] + coeff0_6*dmats2[6][24] + coeff0_7*dmats2[7][24] + coeff0_8*dmats2[8][24] + coeff0_9*dmats2[9][24] + coeff0_10*dmats2[10][24] + coeff0_11*dmats2[11][24] + coeff0_12*dmats2[12][24] + coeff0_13*dmats2[13][24] + coeff0_14*dmats2[14][24] + coeff0_15*dmats2[15][24] + coeff0_16*dmats2[16][24] + coeff0_17*dmats2[17][24] + coeff0_18*dmats2[18][24] + coeff0_19*dmats2[19][24] + coeff0_20*dmats2[20][24] + coeff0_21*dmats2[21][24] + coeff0_22*dmats2[22][24] + coeff0_23*dmats2[23][24] + coeff0_24*dmats2[24][24] + coeff0_25*dmats2[25][24] + coeff0_26*dmats2[26][24] + coeff0_27*dmats2[27][24] + coeff0_28*dmats2[28][24] + coeff0_29*dmats2[29][24] + coeff0_30*dmats2[30][24] + coeff0_31*dmats2[31][24] + coeff0_32*dmats2[32][24] + coeff0_33*dmats2[33][24] + coeff0_34*dmats2[34][24] + coeff0_35*dmats2[35][24] + coeff0_36*dmats2[36][24] + coeff0_37*dmats2[37][24] + coeff0_38*dmats2[38][24] + coeff0_39*dmats2[39][24] + coeff0_40*dmats2[40][24] + coeff0_41*dmats2[41][24] + coeff0_42*dmats2[42][24] + coeff0_43*dmats2[43][24] + coeff0_44*dmats2[44][24] + coeff0_45*dmats2[45][24] + coeff0_46*dmats2[46][24] + coeff0_47*dmats2[47][24] + coeff0_48*dmats2[48][24] + coeff0_49*dmats2[49][24] + coeff0_50*dmats2[50][24] + coeff0_51*dmats2[51][24] + coeff0_52*dmats2[52][24] + coeff0_53*dmats2[53][24] + coeff0_54*dmats2[54][24] + coeff0_55*dmats2[55][24];
          new_coeff0_25 = coeff0_0*dmats2[0][25] + coeff0_1*dmats2[1][25] + coeff0_2*dmats2[2][25] + coeff0_3*dmats2[3][25] + coeff0_4*dmats2[4][25] + coeff0_5*dmats2[5][25] + coeff0_6*dmats2[6][25] + coeff0_7*dmats2[7][25] + coeff0_8*dmats2[8][25] + coeff0_9*dmats2[9][25] + coeff0_10*dmats2[10][25] + coeff0_11*dmats2[11][25] + coeff0_12*dmats2[12][25] + coeff0_13*dmats2[13][25] + coeff0_14*dmats2[14][25] + coeff0_15*dmats2[15][25] + coeff0_16*dmats2[16][25] + coeff0_17*dmats2[17][25] + coeff0_18*dmats2[18][25] + coeff0_19*dmats2[19][25] + coeff0_20*dmats2[20][25] + coeff0_21*dmats2[21][25] + coeff0_22*dmats2[22][25] + coeff0_23*dmats2[23][25] + coeff0_24*dmats2[24][25] + coeff0_25*dmats2[25][25] + coeff0_26*dmats2[26][25] + coeff0_27*dmats2[27][25] + coeff0_28*dmats2[28][25] + coeff0_29*dmats2[29][25] + coeff0_30*dmats2[30][25] + coeff0_31*dmats2[31][25] + coeff0_32*dmats2[32][25] + coeff0_33*dmats2[33][25] + coeff0_34*dmats2[34][25] + coeff0_35*dmats2[35][25] + coeff0_36*dmats2[36][25] + coeff0_37*dmats2[37][25] + coeff0_38*dmats2[38][25] + coeff0_39*dmats2[39][25] + coeff0_40*dmats2[40][25] + coeff0_41*dmats2[41][25] + coeff0_42*dmats2[42][25] + coeff0_43*dmats2[43][25] + coeff0_44*dmats2[44][25] + coeff0_45*dmats2[45][25] + coeff0_46*dmats2[46][25] + coeff0_47*dmats2[47][25] + coeff0_48*dmats2[48][25] + coeff0_49*dmats2[49][25] + coeff0_50*dmats2[50][25] + coeff0_51*dmats2[51][25] + coeff0_52*dmats2[52][25] + coeff0_53*dmats2[53][25] + coeff0_54*dmats2[54][25] + coeff0_55*dmats2[55][25];
          new_coeff0_26 = coeff0_0*dmats2[0][26] + coeff0_1*dmats2[1][26] + coeff0_2*dmats2[2][26] + coeff0_3*dmats2[3][26] + coeff0_4*dmats2[4][26] + coeff0_5*dmats2[5][26] + coeff0_6*dmats2[6][26] + coeff0_7*dmats2[7][26] + coeff0_8*dmats2[8][26] + coeff0_9*dmats2[9][26] + coeff0_10*dmats2[10][26] + coeff0_11*dmats2[11][26] + coeff0_12*dmats2[12][26] + coeff0_13*dmats2[13][26] + coeff0_14*dmats2[14][26] + coeff0_15*dmats2[15][26] + coeff0_16*dmats2[16][26] + coeff0_17*dmats2[17][26] + coeff0_18*dmats2[18][26] + coeff0_19*dmats2[19][26] + coeff0_20*dmats2[20][26] + coeff0_21*dmats2[21][26] + coeff0_22*dmats2[22][26] + coeff0_23*dmats2[23][26] + coeff0_24*dmats2[24][26] + coeff0_25*dmats2[25][26] + coeff0_26*dmats2[26][26] + coeff0_27*dmats2[27][26] + coeff0_28*dmats2[28][26] + coeff0_29*dmats2[29][26] + coeff0_30*dmats2[30][26] + coeff0_31*dmats2[31][26] + coeff0_32*dmats2[32][26] + coeff0_33*dmats2[33][26] + coeff0_34*dmats2[34][26] + coeff0_35*dmats2[35][26] + coeff0_36*dmats2[36][26] + coeff0_37*dmats2[37][26] + coeff0_38*dmats2[38][26] + coeff0_39*dmats2[39][26] + coeff0_40*dmats2[40][26] + coeff0_41*dmats2[41][26] + coeff0_42*dmats2[42][26] + coeff0_43*dmats2[43][26] + coeff0_44*dmats2[44][26] + coeff0_45*dmats2[45][26] + coeff0_46*dmats2[46][26] + coeff0_47*dmats2[47][26] + coeff0_48*dmats2[48][26] + coeff0_49*dmats2[49][26] + coeff0_50*dmats2[50][26] + coeff0_51*dmats2[51][26] + coeff0_52*dmats2[52][26] + coeff0_53*dmats2[53][26] + coeff0_54*dmats2[54][26] + coeff0_55*dmats2[55][26];
          new_coeff0_27 = coeff0_0*dmats2[0][27] + coeff0_1*dmats2[1][27] + coeff0_2*dmats2[2][27] + coeff0_3*dmats2[3][27] + coeff0_4*dmats2[4][27] + coeff0_5*dmats2[5][27] + coeff0_6*dmats2[6][27] + coeff0_7*dmats2[7][27] + coeff0_8*dmats2[8][27] + coeff0_9*dmats2[9][27] + coeff0_10*dmats2[10][27] + coeff0_11*dmats2[11][27] + coeff0_12*dmats2[12][27] + coeff0_13*dmats2[13][27] + coeff0_14*dmats2[14][27] + coeff0_15*dmats2[15][27] + coeff0_16*dmats2[16][27] + coeff0_17*dmats2[17][27] + coeff0_18*dmats2[18][27] + coeff0_19*dmats2[19][27] + coeff0_20*dmats2[20][27] + coeff0_21*dmats2[21][27] + coeff0_22*dmats2[22][27] + coeff0_23*dmats2[23][27] + coeff0_24*dmats2[24][27] + coeff0_25*dmats2[25][27] + coeff0_26*dmats2[26][27] + coeff0_27*dmats2[27][27] + coeff0_28*dmats2[28][27] + coeff0_29*dmats2[29][27] + coeff0_30*dmats2[30][27] + coeff0_31*dmats2[31][27] + coeff0_32*dmats2[32][27] + coeff0_33*dmats2[33][27] + coeff0_34*dmats2[34][27] + coeff0_35*dmats2[35][27] + coeff0_36*dmats2[36][27] + coeff0_37*dmats2[37][27] + coeff0_38*dmats2[38][27] + coeff0_39*dmats2[39][27] + coeff0_40*dmats2[40][27] + coeff0_41*dmats2[41][27] + coeff0_42*dmats2[42][27] + coeff0_43*dmats2[43][27] + coeff0_44*dmats2[44][27] + coeff0_45*dmats2[45][27] + coeff0_46*dmats2[46][27] + coeff0_47*dmats2[47][27] + coeff0_48*dmats2[48][27] + coeff0_49*dmats2[49][27] + coeff0_50*dmats2[50][27] + coeff0_51*dmats2[51][27] + coeff0_52*dmats2[52][27] + coeff0_53*dmats2[53][27] + coeff0_54*dmats2[54][27] + coeff0_55*dmats2[55][27];
          new_coeff0_28 = coeff0_0*dmats2[0][28] + coeff0_1*dmats2[1][28] + coeff0_2*dmats2[2][28] + coeff0_3*dmats2[3][28] + coeff0_4*dmats2[4][28] + coeff0_5*dmats2[5][28] + coeff0_6*dmats2[6][28] + coeff0_7*dmats2[7][28] + coeff0_8*dmats2[8][28] + coeff0_9*dmats2[9][28] + coeff0_10*dmats2[10][28] + coeff0_11*dmats2[11][28] + coeff0_12*dmats2[12][28] + coeff0_13*dmats2[13][28] + coeff0_14*dmats2[14][28] + coeff0_15*dmats2[15][28] + coeff0_16*dmats2[16][28] + coeff0_17*dmats2[17][28] + coeff0_18*dmats2[18][28] + coeff0_19*dmats2[19][28] + coeff0_20*dmats2[20][28] + coeff0_21*dmats2[21][28] + coeff0_22*dmats2[22][28] + coeff0_23*dmats2[23][28] + coeff0_24*dmats2[24][28] + coeff0_25*dmats2[25][28] + coeff0_26*dmats2[26][28] + coeff0_27*dmats2[27][28] + coeff0_28*dmats2[28][28] + coeff0_29*dmats2[29][28] + coeff0_30*dmats2[30][28] + coeff0_31*dmats2[31][28] + coeff0_32*dmats2[32][28] + coeff0_33*dmats2[33][28] + coeff0_34*dmats2[34][28] + coeff0_35*dmats2[35][28] + coeff0_36*dmats2[36][28] + coeff0_37*dmats2[37][28] + coeff0_38*dmats2[38][28] + coeff0_39*dmats2[39][28] + coeff0_40*dmats2[40][28] + coeff0_41*dmats2[41][28] + coeff0_42*dmats2[42][28] + coeff0_43*dmats2[43][28] + coeff0_44*dmats2[44][28] + coeff0_45*dmats2[45][28] + coeff0_46*dmats2[46][28] + coeff0_47*dmats2[47][28] + coeff0_48*dmats2[48][28] + coeff0_49*dmats2[49][28] + coeff0_50*dmats2[50][28] + coeff0_51*dmats2[51][28] + coeff0_52*dmats2[52][28] + coeff0_53*dmats2[53][28] + coeff0_54*dmats2[54][28] + coeff0_55*dmats2[55][28];
          new_coeff0_29 = coeff0_0*dmats2[0][29] + coeff0_1*dmats2[1][29] + coeff0_2*dmats2[2][29] + coeff0_3*dmats2[3][29] + coeff0_4*dmats2[4][29] + coeff0_5*dmats2[5][29] + coeff0_6*dmats2[6][29] + coeff0_7*dmats2[7][29] + coeff0_8*dmats2[8][29] + coeff0_9*dmats2[9][29] + coeff0_10*dmats2[10][29] + coeff0_11*dmats2[11][29] + coeff0_12*dmats2[12][29] + coeff0_13*dmats2[13][29] + coeff0_14*dmats2[14][29] + coeff0_15*dmats2[15][29] + coeff0_16*dmats2[16][29] + coeff0_17*dmats2[17][29] + coeff0_18*dmats2[18][29] + coeff0_19*dmats2[19][29] + coeff0_20*dmats2[20][29] + coeff0_21*dmats2[21][29] + coeff0_22*dmats2[22][29] + coeff0_23*dmats2[23][29] + coeff0_24*dmats2[24][29] + coeff0_25*dmats2[25][29] + coeff0_26*dmats2[26][29] + coeff0_27*dmats2[27][29] + coeff0_28*dmats2[28][29] + coeff0_29*dmats2[29][29] + coeff0_30*dmats2[30][29] + coeff0_31*dmats2[31][29] + coeff0_32*dmats2[32][29] + coeff0_33*dmats2[33][29] + coeff0_34*dmats2[34][29] + coeff0_35*dmats2[35][29] + coeff0_36*dmats2[36][29] + coeff0_37*dmats2[37][29] + coeff0_38*dmats2[38][29] + coeff0_39*dmats2[39][29] + coeff0_40*dmats2[40][29] + coeff0_41*dmats2[41][29] + coeff0_42*dmats2[42][29] + coeff0_43*dmats2[43][29] + coeff0_44*dmats2[44][29] + coeff0_45*dmats2[45][29] + coeff0_46*dmats2[46][29] + coeff0_47*dmats2[47][29] + coeff0_48*dmats2[48][29] + coeff0_49*dmats2[49][29] + coeff0_50*dmats2[50][29] + coeff0_51*dmats2[51][29] + coeff0_52*dmats2[52][29] + coeff0_53*dmats2[53][29] + coeff0_54*dmats2[54][29] + coeff0_55*dmats2[55][29];
          new_coeff0_30 = coeff0_0*dmats2[0][30] + coeff0_1*dmats2[1][30] + coeff0_2*dmats2[2][30] + coeff0_3*dmats2[3][30] + coeff0_4*dmats2[4][30] + coeff0_5*dmats2[5][30] + coeff0_6*dmats2[6][30] + coeff0_7*dmats2[7][30] + coeff0_8*dmats2[8][30] + coeff0_9*dmats2[9][30] + coeff0_10*dmats2[10][30] + coeff0_11*dmats2[11][30] + coeff0_12*dmats2[12][30] + coeff0_13*dmats2[13][30] + coeff0_14*dmats2[14][30] + coeff0_15*dmats2[15][30] + coeff0_16*dmats2[16][30] + coeff0_17*dmats2[17][30] + coeff0_18*dmats2[18][30] + coeff0_19*dmats2[19][30] + coeff0_20*dmats2[20][30] + coeff0_21*dmats2[21][30] + coeff0_22*dmats2[22][30] + coeff0_23*dmats2[23][30] + coeff0_24*dmats2[24][30] + coeff0_25*dmats2[25][30] + coeff0_26*dmats2[26][30] + coeff0_27*dmats2[27][30] + coeff0_28*dmats2[28][30] + coeff0_29*dmats2[29][30] + coeff0_30*dmats2[30][30] + coeff0_31*dmats2[31][30] + coeff0_32*dmats2[32][30] + coeff0_33*dmats2[33][30] + coeff0_34*dmats2[34][30] + coeff0_35*dmats2[35][30] + coeff0_36*dmats2[36][30] + coeff0_37*dmats2[37][30] + coeff0_38*dmats2[38][30] + coeff0_39*dmats2[39][30] + coeff0_40*dmats2[40][30] + coeff0_41*dmats2[41][30] + coeff0_42*dmats2[42][30] + coeff0_43*dmats2[43][30] + coeff0_44*dmats2[44][30] + coeff0_45*dmats2[45][30] + coeff0_46*dmats2[46][30] + coeff0_47*dmats2[47][30] + coeff0_48*dmats2[48][30] + coeff0_49*dmats2[49][30] + coeff0_50*dmats2[50][30] + coeff0_51*dmats2[51][30] + coeff0_52*dmats2[52][30] + coeff0_53*dmats2[53][30] + coeff0_54*dmats2[54][30] + coeff0_55*dmats2[55][30];
          new_coeff0_31 = coeff0_0*dmats2[0][31] + coeff0_1*dmats2[1][31] + coeff0_2*dmats2[2][31] + coeff0_3*dmats2[3][31] + coeff0_4*dmats2[4][31] + coeff0_5*dmats2[5][31] + coeff0_6*dmats2[6][31] + coeff0_7*dmats2[7][31] + coeff0_8*dmats2[8][31] + coeff0_9*dmats2[9][31] + coeff0_10*dmats2[10][31] + coeff0_11*dmats2[11][31] + coeff0_12*dmats2[12][31] + coeff0_13*dmats2[13][31] + coeff0_14*dmats2[14][31] + coeff0_15*dmats2[15][31] + coeff0_16*dmats2[16][31] + coeff0_17*dmats2[17][31] + coeff0_18*dmats2[18][31] + coeff0_19*dmats2[19][31] + coeff0_20*dmats2[20][31] + coeff0_21*dmats2[21][31] + coeff0_22*dmats2[22][31] + coeff0_23*dmats2[23][31] + coeff0_24*dmats2[24][31] + coeff0_25*dmats2[25][31] + coeff0_26*dmats2[26][31] + coeff0_27*dmats2[27][31] + coeff0_28*dmats2[28][31] + coeff0_29*dmats2[29][31] + coeff0_30*dmats2[30][31] + coeff0_31*dmats2[31][31] + coeff0_32*dmats2[32][31] + coeff0_33*dmats2[33][31] + coeff0_34*dmats2[34][31] + coeff0_35*dmats2[35][31] + coeff0_36*dmats2[36][31] + coeff0_37*dmats2[37][31] + coeff0_38*dmats2[38][31] + coeff0_39*dmats2[39][31] + coeff0_40*dmats2[40][31] + coeff0_41*dmats2[41][31] + coeff0_42*dmats2[42][31] + coeff0_43*dmats2[43][31] + coeff0_44*dmats2[44][31] + coeff0_45*dmats2[45][31] + coeff0_46*dmats2[46][31] + coeff0_47*dmats2[47][31] + coeff0_48*dmats2[48][31] + coeff0_49*dmats2[49][31] + coeff0_50*dmats2[50][31] + coeff0_51*dmats2[51][31] + coeff0_52*dmats2[52][31] + coeff0_53*dmats2[53][31] + coeff0_54*dmats2[54][31] + coeff0_55*dmats2[55][31];
          new_coeff0_32 = coeff0_0*dmats2[0][32] + coeff0_1*dmats2[1][32] + coeff0_2*dmats2[2][32] + coeff0_3*dmats2[3][32] + coeff0_4*dmats2[4][32] + coeff0_5*dmats2[5][32] + coeff0_6*dmats2[6][32] + coeff0_7*dmats2[7][32] + coeff0_8*dmats2[8][32] + coeff0_9*dmats2[9][32] + coeff0_10*dmats2[10][32] + coeff0_11*dmats2[11][32] + coeff0_12*dmats2[12][32] + coeff0_13*dmats2[13][32] + coeff0_14*dmats2[14][32] + coeff0_15*dmats2[15][32] + coeff0_16*dmats2[16][32] + coeff0_17*dmats2[17][32] + coeff0_18*dmats2[18][32] + coeff0_19*dmats2[19][32] + coeff0_20*dmats2[20][32] + coeff0_21*dmats2[21][32] + coeff0_22*dmats2[22][32] + coeff0_23*dmats2[23][32] + coeff0_24*dmats2[24][32] + coeff0_25*dmats2[25][32] + coeff0_26*dmats2[26][32] + coeff0_27*dmats2[27][32] + coeff0_28*dmats2[28][32] + coeff0_29*dmats2[29][32] + coeff0_30*dmats2[30][32] + coeff0_31*dmats2[31][32] + coeff0_32*dmats2[32][32] + coeff0_33*dmats2[33][32] + coeff0_34*dmats2[34][32] + coeff0_35*dmats2[35][32] + coeff0_36*dmats2[36][32] + coeff0_37*dmats2[37][32] + coeff0_38*dmats2[38][32] + coeff0_39*dmats2[39][32] + coeff0_40*dmats2[40][32] + coeff0_41*dmats2[41][32] + coeff0_42*dmats2[42][32] + coeff0_43*dmats2[43][32] + coeff0_44*dmats2[44][32] + coeff0_45*dmats2[45][32] + coeff0_46*dmats2[46][32] + coeff0_47*dmats2[47][32] + coeff0_48*dmats2[48][32] + coeff0_49*dmats2[49][32] + coeff0_50*dmats2[50][32] + coeff0_51*dmats2[51][32] + coeff0_52*dmats2[52][32] + coeff0_53*dmats2[53][32] + coeff0_54*dmats2[54][32] + coeff0_55*dmats2[55][32];
          new_coeff0_33 = coeff0_0*dmats2[0][33] + coeff0_1*dmats2[1][33] + coeff0_2*dmats2[2][33] + coeff0_3*dmats2[3][33] + coeff0_4*dmats2[4][33] + coeff0_5*dmats2[5][33] + coeff0_6*dmats2[6][33] + coeff0_7*dmats2[7][33] + coeff0_8*dmats2[8][33] + coeff0_9*dmats2[9][33] + coeff0_10*dmats2[10][33] + coeff0_11*dmats2[11][33] + coeff0_12*dmats2[12][33] + coeff0_13*dmats2[13][33] + coeff0_14*dmats2[14][33] + coeff0_15*dmats2[15][33] + coeff0_16*dmats2[16][33] + coeff0_17*dmats2[17][33] + coeff0_18*dmats2[18][33] + coeff0_19*dmats2[19][33] + coeff0_20*dmats2[20][33] + coeff0_21*dmats2[21][33] + coeff0_22*dmats2[22][33] + coeff0_23*dmats2[23][33] + coeff0_24*dmats2[24][33] + coeff0_25*dmats2[25][33] + coeff0_26*dmats2[26][33] + coeff0_27*dmats2[27][33] + coeff0_28*dmats2[28][33] + coeff0_29*dmats2[29][33] + coeff0_30*dmats2[30][33] + coeff0_31*dmats2[31][33] + coeff0_32*dmats2[32][33] + coeff0_33*dmats2[33][33] + coeff0_34*dmats2[34][33] + coeff0_35*dmats2[35][33] + coeff0_36*dmats2[36][33] + coeff0_37*dmats2[37][33] + coeff0_38*dmats2[38][33] + coeff0_39*dmats2[39][33] + coeff0_40*dmats2[40][33] + coeff0_41*dmats2[41][33] + coeff0_42*dmats2[42][33] + coeff0_43*dmats2[43][33] + coeff0_44*dmats2[44][33] + coeff0_45*dmats2[45][33] + coeff0_46*dmats2[46][33] + coeff0_47*dmats2[47][33] + coeff0_48*dmats2[48][33] + coeff0_49*dmats2[49][33] + coeff0_50*dmats2[50][33] + coeff0_51*dmats2[51][33] + coeff0_52*dmats2[52][33] + coeff0_53*dmats2[53][33] + coeff0_54*dmats2[54][33] + coeff0_55*dmats2[55][33];
          new_coeff0_34 = coeff0_0*dmats2[0][34] + coeff0_1*dmats2[1][34] + coeff0_2*dmats2[2][34] + coeff0_3*dmats2[3][34] + coeff0_4*dmats2[4][34] + coeff0_5*dmats2[5][34] + coeff0_6*dmats2[6][34] + coeff0_7*dmats2[7][34] + coeff0_8*dmats2[8][34] + coeff0_9*dmats2[9][34] + coeff0_10*dmats2[10][34] + coeff0_11*dmats2[11][34] + coeff0_12*dmats2[12][34] + coeff0_13*dmats2[13][34] + coeff0_14*dmats2[14][34] + coeff0_15*dmats2[15][34] + coeff0_16*dmats2[16][34] + coeff0_17*dmats2[17][34] + coeff0_18*dmats2[18][34] + coeff0_19*dmats2[19][34] + coeff0_20*dmats2[20][34] + coeff0_21*dmats2[21][34] + coeff0_22*dmats2[22][34] + coeff0_23*dmats2[23][34] + coeff0_24*dmats2[24][34] + coeff0_25*dmats2[25][34] + coeff0_26*dmats2[26][34] + coeff0_27*dmats2[27][34] + coeff0_28*dmats2[28][34] + coeff0_29*dmats2[29][34] + coeff0_30*dmats2[30][34] + coeff0_31*dmats2[31][34] + coeff0_32*dmats2[32][34] + coeff0_33*dmats2[33][34] + coeff0_34*dmats2[34][34] + coeff0_35*dmats2[35][34] + coeff0_36*dmats2[36][34] + coeff0_37*dmats2[37][34] + coeff0_38*dmats2[38][34] + coeff0_39*dmats2[39][34] + coeff0_40*dmats2[40][34] + coeff0_41*dmats2[41][34] + coeff0_42*dmats2[42][34] + coeff0_43*dmats2[43][34] + coeff0_44*dmats2[44][34] + coeff0_45*dmats2[45][34] + coeff0_46*dmats2[46][34] + coeff0_47*dmats2[47][34] + coeff0_48*dmats2[48][34] + coeff0_49*dmats2[49][34] + coeff0_50*dmats2[50][34] + coeff0_51*dmats2[51][34] + coeff0_52*dmats2[52][34] + coeff0_53*dmats2[53][34] + coeff0_54*dmats2[54][34] + coeff0_55*dmats2[55][34];
          new_coeff0_35 = coeff0_0*dmats2[0][35] + coeff0_1*dmats2[1][35] + coeff0_2*dmats2[2][35] + coeff0_3*dmats2[3][35] + coeff0_4*dmats2[4][35] + coeff0_5*dmats2[5][35] + coeff0_6*dmats2[6][35] + coeff0_7*dmats2[7][35] + coeff0_8*dmats2[8][35] + coeff0_9*dmats2[9][35] + coeff0_10*dmats2[10][35] + coeff0_11*dmats2[11][35] + coeff0_12*dmats2[12][35] + coeff0_13*dmats2[13][35] + coeff0_14*dmats2[14][35] + coeff0_15*dmats2[15][35] + coeff0_16*dmats2[16][35] + coeff0_17*dmats2[17][35] + coeff0_18*dmats2[18][35] + coeff0_19*dmats2[19][35] + coeff0_20*dmats2[20][35] + coeff0_21*dmats2[21][35] + coeff0_22*dmats2[22][35] + coeff0_23*dmats2[23][35] + coeff0_24*dmats2[24][35] + coeff0_25*dmats2[25][35] + coeff0_26*dmats2[26][35] + coeff0_27*dmats2[27][35] + coeff0_28*dmats2[28][35] + coeff0_29*dmats2[29][35] + coeff0_30*dmats2[30][35] + coeff0_31*dmats2[31][35] + coeff0_32*dmats2[32][35] + coeff0_33*dmats2[33][35] + coeff0_34*dmats2[34][35] + coeff0_35*dmats2[35][35] + coeff0_36*dmats2[36][35] + coeff0_37*dmats2[37][35] + coeff0_38*dmats2[38][35] + coeff0_39*dmats2[39][35] + coeff0_40*dmats2[40][35] + coeff0_41*dmats2[41][35] + coeff0_42*dmats2[42][35] + coeff0_43*dmats2[43][35] + coeff0_44*dmats2[44][35] + coeff0_45*dmats2[45][35] + coeff0_46*dmats2[46][35] + coeff0_47*dmats2[47][35] + coeff0_48*dmats2[48][35] + coeff0_49*dmats2[49][35] + coeff0_50*dmats2[50][35] + coeff0_51*dmats2[51][35] + coeff0_52*dmats2[52][35] + coeff0_53*dmats2[53][35] + coeff0_54*dmats2[54][35] + coeff0_55*dmats2[55][35];
          new_coeff0_36 = coeff0_0*dmats2[0][36] + coeff0_1*dmats2[1][36] + coeff0_2*dmats2[2][36] + coeff0_3*dmats2[3][36] + coeff0_4*dmats2[4][36] + coeff0_5*dmats2[5][36] + coeff0_6*dmats2[6][36] + coeff0_7*dmats2[7][36] + coeff0_8*dmats2[8][36] + coeff0_9*dmats2[9][36] + coeff0_10*dmats2[10][36] + coeff0_11*dmats2[11][36] + coeff0_12*dmats2[12][36] + coeff0_13*dmats2[13][36] + coeff0_14*dmats2[14][36] + coeff0_15*dmats2[15][36] + coeff0_16*dmats2[16][36] + coeff0_17*dmats2[17][36] + coeff0_18*dmats2[18][36] + coeff0_19*dmats2[19][36] + coeff0_20*dmats2[20][36] + coeff0_21*dmats2[21][36] + coeff0_22*dmats2[22][36] + coeff0_23*dmats2[23][36] + coeff0_24*dmats2[24][36] + coeff0_25*dmats2[25][36] + coeff0_26*dmats2[26][36] + coeff0_27*dmats2[27][36] + coeff0_28*dmats2[28][36] + coeff0_29*dmats2[29][36] + coeff0_30*dmats2[30][36] + coeff0_31*dmats2[31][36] + coeff0_32*dmats2[32][36] + coeff0_33*dmats2[33][36] + coeff0_34*dmats2[34][36] + coeff0_35*dmats2[35][36] + coeff0_36*dmats2[36][36] + coeff0_37*dmats2[37][36] + coeff0_38*dmats2[38][36] + coeff0_39*dmats2[39][36] + coeff0_40*dmats2[40][36] + coeff0_41*dmats2[41][36] + coeff0_42*dmats2[42][36] + coeff0_43*dmats2[43][36] + coeff0_44*dmats2[44][36] + coeff0_45*dmats2[45][36] + coeff0_46*dmats2[46][36] + coeff0_47*dmats2[47][36] + coeff0_48*dmats2[48][36] + coeff0_49*dmats2[49][36] + coeff0_50*dmats2[50][36] + coeff0_51*dmats2[51][36] + coeff0_52*dmats2[52][36] + coeff0_53*dmats2[53][36] + coeff0_54*dmats2[54][36] + coeff0_55*dmats2[55][36];
          new_coeff0_37 = coeff0_0*dmats2[0][37] + coeff0_1*dmats2[1][37] + coeff0_2*dmats2[2][37] + coeff0_3*dmats2[3][37] + coeff0_4*dmats2[4][37] + coeff0_5*dmats2[5][37] + coeff0_6*dmats2[6][37] + coeff0_7*dmats2[7][37] + coeff0_8*dmats2[8][37] + coeff0_9*dmats2[9][37] + coeff0_10*dmats2[10][37] + coeff0_11*dmats2[11][37] + coeff0_12*dmats2[12][37] + coeff0_13*dmats2[13][37] + coeff0_14*dmats2[14][37] + coeff0_15*dmats2[15][37] + coeff0_16*dmats2[16][37] + coeff0_17*dmats2[17][37] + coeff0_18*dmats2[18][37] + coeff0_19*dmats2[19][37] + coeff0_20*dmats2[20][37] + coeff0_21*dmats2[21][37] + coeff0_22*dmats2[22][37] + coeff0_23*dmats2[23][37] + coeff0_24*dmats2[24][37] + coeff0_25*dmats2[25][37] + coeff0_26*dmats2[26][37] + coeff0_27*dmats2[27][37] + coeff0_28*dmats2[28][37] + coeff0_29*dmats2[29][37] + coeff0_30*dmats2[30][37] + coeff0_31*dmats2[31][37] + coeff0_32*dmats2[32][37] + coeff0_33*dmats2[33][37] + coeff0_34*dmats2[34][37] + coeff0_35*dmats2[35][37] + coeff0_36*dmats2[36][37] + coeff0_37*dmats2[37][37] + coeff0_38*dmats2[38][37] + coeff0_39*dmats2[39][37] + coeff0_40*dmats2[40][37] + coeff0_41*dmats2[41][37] + coeff0_42*dmats2[42][37] + coeff0_43*dmats2[43][37] + coeff0_44*dmats2[44][37] + coeff0_45*dmats2[45][37] + coeff0_46*dmats2[46][37] + coeff0_47*dmats2[47][37] + coeff0_48*dmats2[48][37] + coeff0_49*dmats2[49][37] + coeff0_50*dmats2[50][37] + coeff0_51*dmats2[51][37] + coeff0_52*dmats2[52][37] + coeff0_53*dmats2[53][37] + coeff0_54*dmats2[54][37] + coeff0_55*dmats2[55][37];
          new_coeff0_38 = coeff0_0*dmats2[0][38] + coeff0_1*dmats2[1][38] + coeff0_2*dmats2[2][38] + coeff0_3*dmats2[3][38] + coeff0_4*dmats2[4][38] + coeff0_5*dmats2[5][38] + coeff0_6*dmats2[6][38] + coeff0_7*dmats2[7][38] + coeff0_8*dmats2[8][38] + coeff0_9*dmats2[9][38] + coeff0_10*dmats2[10][38] + coeff0_11*dmats2[11][38] + coeff0_12*dmats2[12][38] + coeff0_13*dmats2[13][38] + coeff0_14*dmats2[14][38] + coeff0_15*dmats2[15][38] + coeff0_16*dmats2[16][38] + coeff0_17*dmats2[17][38] + coeff0_18*dmats2[18][38] + coeff0_19*dmats2[19][38] + coeff0_20*dmats2[20][38] + coeff0_21*dmats2[21][38] + coeff0_22*dmats2[22][38] + coeff0_23*dmats2[23][38] + coeff0_24*dmats2[24][38] + coeff0_25*dmats2[25][38] + coeff0_26*dmats2[26][38] + coeff0_27*dmats2[27][38] + coeff0_28*dmats2[28][38] + coeff0_29*dmats2[29][38] + coeff0_30*dmats2[30][38] + coeff0_31*dmats2[31][38] + coeff0_32*dmats2[32][38] + coeff0_33*dmats2[33][38] + coeff0_34*dmats2[34][38] + coeff0_35*dmats2[35][38] + coeff0_36*dmats2[36][38] + coeff0_37*dmats2[37][38] + coeff0_38*dmats2[38][38] + coeff0_39*dmats2[39][38] + coeff0_40*dmats2[40][38] + coeff0_41*dmats2[41][38] + coeff0_42*dmats2[42][38] + coeff0_43*dmats2[43][38] + coeff0_44*dmats2[44][38] + coeff0_45*dmats2[45][38] + coeff0_46*dmats2[46][38] + coeff0_47*dmats2[47][38] + coeff0_48*dmats2[48][38] + coeff0_49*dmats2[49][38] + coeff0_50*dmats2[50][38] + coeff0_51*dmats2[51][38] + coeff0_52*dmats2[52][38] + coeff0_53*dmats2[53][38] + coeff0_54*dmats2[54][38] + coeff0_55*dmats2[55][38];
          new_coeff0_39 = coeff0_0*dmats2[0][39] + coeff0_1*dmats2[1][39] + coeff0_2*dmats2[2][39] + coeff0_3*dmats2[3][39] + coeff0_4*dmats2[4][39] + coeff0_5*dmats2[5][39] + coeff0_6*dmats2[6][39] + coeff0_7*dmats2[7][39] + coeff0_8*dmats2[8][39] + coeff0_9*dmats2[9][39] + coeff0_10*dmats2[10][39] + coeff0_11*dmats2[11][39] + coeff0_12*dmats2[12][39] + coeff0_13*dmats2[13][39] + coeff0_14*dmats2[14][39] + coeff0_15*dmats2[15][39] + coeff0_16*dmats2[16][39] + coeff0_17*dmats2[17][39] + coeff0_18*dmats2[18][39] + coeff0_19*dmats2[19][39] + coeff0_20*dmats2[20][39] + coeff0_21*dmats2[21][39] + coeff0_22*dmats2[22][39] + coeff0_23*dmats2[23][39] + coeff0_24*dmats2[24][39] + coeff0_25*dmats2[25][39] + coeff0_26*dmats2[26][39] + coeff0_27*dmats2[27][39] + coeff0_28*dmats2[28][39] + coeff0_29*dmats2[29][39] + coeff0_30*dmats2[30][39] + coeff0_31*dmats2[31][39] + coeff0_32*dmats2[32][39] + coeff0_33*dmats2[33][39] + coeff0_34*dmats2[34][39] + coeff0_35*dmats2[35][39] + coeff0_36*dmats2[36][39] + coeff0_37*dmats2[37][39] + coeff0_38*dmats2[38][39] + coeff0_39*dmats2[39][39] + coeff0_40*dmats2[40][39] + coeff0_41*dmats2[41][39] + coeff0_42*dmats2[42][39] + coeff0_43*dmats2[43][39] + coeff0_44*dmats2[44][39] + coeff0_45*dmats2[45][39] + coeff0_46*dmats2[46][39] + coeff0_47*dmats2[47][39] + coeff0_48*dmats2[48][39] + coeff0_49*dmats2[49][39] + coeff0_50*dmats2[50][39] + coeff0_51*dmats2[51][39] + coeff0_52*dmats2[52][39] + coeff0_53*dmats2[53][39] + coeff0_54*dmats2[54][39] + coeff0_55*dmats2[55][39];
          new_coeff0_40 = coeff0_0*dmats2[0][40] + coeff0_1*dmats2[1][40] + coeff0_2*dmats2[2][40] + coeff0_3*dmats2[3][40] + coeff0_4*dmats2[4][40] + coeff0_5*dmats2[5][40] + coeff0_6*dmats2[6][40] + coeff0_7*dmats2[7][40] + coeff0_8*dmats2[8][40] + coeff0_9*dmats2[9][40] + coeff0_10*dmats2[10][40] + coeff0_11*dmats2[11][40] + coeff0_12*dmats2[12][40] + coeff0_13*dmats2[13][40] + coeff0_14*dmats2[14][40] + coeff0_15*dmats2[15][40] + coeff0_16*dmats2[16][40] + coeff0_17*dmats2[17][40] + coeff0_18*dmats2[18][40] + coeff0_19*dmats2[19][40] + coeff0_20*dmats2[20][40] + coeff0_21*dmats2[21][40] + coeff0_22*dmats2[22][40] + coeff0_23*dmats2[23][40] + coeff0_24*dmats2[24][40] + coeff0_25*dmats2[25][40] + coeff0_26*dmats2[26][40] + coeff0_27*dmats2[27][40] + coeff0_28*dmats2[28][40] + coeff0_29*dmats2[29][40] + coeff0_30*dmats2[30][40] + coeff0_31*dmats2[31][40] + coeff0_32*dmats2[32][40] + coeff0_33*dmats2[33][40] + coeff0_34*dmats2[34][40] + coeff0_35*dmats2[35][40] + coeff0_36*dmats2[36][40] + coeff0_37*dmats2[37][40] + coeff0_38*dmats2[38][40] + coeff0_39*dmats2[39][40] + coeff0_40*dmats2[40][40] + coeff0_41*dmats2[41][40] + coeff0_42*dmats2[42][40] + coeff0_43*dmats2[43][40] + coeff0_44*dmats2[44][40] + coeff0_45*dmats2[45][40] + coeff0_46*dmats2[46][40] + coeff0_47*dmats2[47][40] + coeff0_48*dmats2[48][40] + coeff0_49*dmats2[49][40] + coeff0_50*dmats2[50][40] + coeff0_51*dmats2[51][40] + coeff0_52*dmats2[52][40] + coeff0_53*dmats2[53][40] + coeff0_54*dmats2[54][40] + coeff0_55*dmats2[55][40];
          new_coeff0_41 = coeff0_0*dmats2[0][41] + coeff0_1*dmats2[1][41] + coeff0_2*dmats2[2][41] + coeff0_3*dmats2[3][41] + coeff0_4*dmats2[4][41] + coeff0_5*dmats2[5][41] + coeff0_6*dmats2[6][41] + coeff0_7*dmats2[7][41] + coeff0_8*dmats2[8][41] + coeff0_9*dmats2[9][41] + coeff0_10*dmats2[10][41] + coeff0_11*dmats2[11][41] + coeff0_12*dmats2[12][41] + coeff0_13*dmats2[13][41] + coeff0_14*dmats2[14][41] + coeff0_15*dmats2[15][41] + coeff0_16*dmats2[16][41] + coeff0_17*dmats2[17][41] + coeff0_18*dmats2[18][41] + coeff0_19*dmats2[19][41] + coeff0_20*dmats2[20][41] + coeff0_21*dmats2[21][41] + coeff0_22*dmats2[22][41] + coeff0_23*dmats2[23][41] + coeff0_24*dmats2[24][41] + coeff0_25*dmats2[25][41] + coeff0_26*dmats2[26][41] + coeff0_27*dmats2[27][41] + coeff0_28*dmats2[28][41] + coeff0_29*dmats2[29][41] + coeff0_30*dmats2[30][41] + coeff0_31*dmats2[31][41] + coeff0_32*dmats2[32][41] + coeff0_33*dmats2[33][41] + coeff0_34*dmats2[34][41] + coeff0_35*dmats2[35][41] + coeff0_36*dmats2[36][41] + coeff0_37*dmats2[37][41] + coeff0_38*dmats2[38][41] + coeff0_39*dmats2[39][41] + coeff0_40*dmats2[40][41] + coeff0_41*dmats2[41][41] + coeff0_42*dmats2[42][41] + coeff0_43*dmats2[43][41] + coeff0_44*dmats2[44][41] + coeff0_45*dmats2[45][41] + coeff0_46*dmats2[46][41] + coeff0_47*dmats2[47][41] + coeff0_48*dmats2[48][41] + coeff0_49*dmats2[49][41] + coeff0_50*dmats2[50][41] + coeff0_51*dmats2[51][41] + coeff0_52*dmats2[52][41] + coeff0_53*dmats2[53][41] + coeff0_54*dmats2[54][41] + coeff0_55*dmats2[55][41];
          new_coeff0_42 = coeff0_0*dmats2[0][42] + coeff0_1*dmats2[1][42] + coeff0_2*dmats2[2][42] + coeff0_3*dmats2[3][42] + coeff0_4*dmats2[4][42] + coeff0_5*dmats2[5][42] + coeff0_6*dmats2[6][42] + coeff0_7*dmats2[7][42] + coeff0_8*dmats2[8][42] + coeff0_9*dmats2[9][42] + coeff0_10*dmats2[10][42] + coeff0_11*dmats2[11][42] + coeff0_12*dmats2[12][42] + coeff0_13*dmats2[13][42] + coeff0_14*dmats2[14][42] + coeff0_15*dmats2[15][42] + coeff0_16*dmats2[16][42] + coeff0_17*dmats2[17][42] + coeff0_18*dmats2[18][42] + coeff0_19*dmats2[19][42] + coeff0_20*dmats2[20][42] + coeff0_21*dmats2[21][42] + coeff0_22*dmats2[22][42] + coeff0_23*dmats2[23][42] + coeff0_24*dmats2[24][42] + coeff0_25*dmats2[25][42] + coeff0_26*dmats2[26][42] + coeff0_27*dmats2[27][42] + coeff0_28*dmats2[28][42] + coeff0_29*dmats2[29][42] + coeff0_30*dmats2[30][42] + coeff0_31*dmats2[31][42] + coeff0_32*dmats2[32][42] + coeff0_33*dmats2[33][42] + coeff0_34*dmats2[34][42] + coeff0_35*dmats2[35][42] + coeff0_36*dmats2[36][42] + coeff0_37*dmats2[37][42] + coeff0_38*dmats2[38][42] + coeff0_39*dmats2[39][42] + coeff0_40*dmats2[40][42] + coeff0_41*dmats2[41][42] + coeff0_42*dmats2[42][42] + coeff0_43*dmats2[43][42] + coeff0_44*dmats2[44][42] + coeff0_45*dmats2[45][42] + coeff0_46*dmats2[46][42] + coeff0_47*dmats2[47][42] + coeff0_48*dmats2[48][42] + coeff0_49*dmats2[49][42] + coeff0_50*dmats2[50][42] + coeff0_51*dmats2[51][42] + coeff0_52*dmats2[52][42] + coeff0_53*dmats2[53][42] + coeff0_54*dmats2[54][42] + coeff0_55*dmats2[55][42];
          new_coeff0_43 = coeff0_0*dmats2[0][43] + coeff0_1*dmats2[1][43] + coeff0_2*dmats2[2][43] + coeff0_3*dmats2[3][43] + coeff0_4*dmats2[4][43] + coeff0_5*dmats2[5][43] + coeff0_6*dmats2[6][43] + coeff0_7*dmats2[7][43] + coeff0_8*dmats2[8][43] + coeff0_9*dmats2[9][43] + coeff0_10*dmats2[10][43] + coeff0_11*dmats2[11][43] + coeff0_12*dmats2[12][43] + coeff0_13*dmats2[13][43] + coeff0_14*dmats2[14][43] + coeff0_15*dmats2[15][43] + coeff0_16*dmats2[16][43] + coeff0_17*dmats2[17][43] + coeff0_18*dmats2[18][43] + coeff0_19*dmats2[19][43] + coeff0_20*dmats2[20][43] + coeff0_21*dmats2[21][43] + coeff0_22*dmats2[22][43] + coeff0_23*dmats2[23][43] + coeff0_24*dmats2[24][43] + coeff0_25*dmats2[25][43] + coeff0_26*dmats2[26][43] + coeff0_27*dmats2[27][43] + coeff0_28*dmats2[28][43] + coeff0_29*dmats2[29][43] + coeff0_30*dmats2[30][43] + coeff0_31*dmats2[31][43] + coeff0_32*dmats2[32][43] + coeff0_33*dmats2[33][43] + coeff0_34*dmats2[34][43] + coeff0_35*dmats2[35][43] + coeff0_36*dmats2[36][43] + coeff0_37*dmats2[37][43] + coeff0_38*dmats2[38][43] + coeff0_39*dmats2[39][43] + coeff0_40*dmats2[40][43] + coeff0_41*dmats2[41][43] + coeff0_42*dmats2[42][43] + coeff0_43*dmats2[43][43] + coeff0_44*dmats2[44][43] + coeff0_45*dmats2[45][43] + coeff0_46*dmats2[46][43] + coeff0_47*dmats2[47][43] + coeff0_48*dmats2[48][43] + coeff0_49*dmats2[49][43] + coeff0_50*dmats2[50][43] + coeff0_51*dmats2[51][43] + coeff0_52*dmats2[52][43] + coeff0_53*dmats2[53][43] + coeff0_54*dmats2[54][43] + coeff0_55*dmats2[55][43];
          new_coeff0_44 = coeff0_0*dmats2[0][44] + coeff0_1*dmats2[1][44] + coeff0_2*dmats2[2][44] + coeff0_3*dmats2[3][44] + coeff0_4*dmats2[4][44] + coeff0_5*dmats2[5][44] + coeff0_6*dmats2[6][44] + coeff0_7*dmats2[7][44] + coeff0_8*dmats2[8][44] + coeff0_9*dmats2[9][44] + coeff0_10*dmats2[10][44] + coeff0_11*dmats2[11][44] + coeff0_12*dmats2[12][44] + coeff0_13*dmats2[13][44] + coeff0_14*dmats2[14][44] + coeff0_15*dmats2[15][44] + coeff0_16*dmats2[16][44] + coeff0_17*dmats2[17][44] + coeff0_18*dmats2[18][44] + coeff0_19*dmats2[19][44] + coeff0_20*dmats2[20][44] + coeff0_21*dmats2[21][44] + coeff0_22*dmats2[22][44] + coeff0_23*dmats2[23][44] + coeff0_24*dmats2[24][44] + coeff0_25*dmats2[25][44] + coeff0_26*dmats2[26][44] + coeff0_27*dmats2[27][44] + coeff0_28*dmats2[28][44] + coeff0_29*dmats2[29][44] + coeff0_30*dmats2[30][44] + coeff0_31*dmats2[31][44] + coeff0_32*dmats2[32][44] + coeff0_33*dmats2[33][44] + coeff0_34*dmats2[34][44] + coeff0_35*dmats2[35][44] + coeff0_36*dmats2[36][44] + coeff0_37*dmats2[37][44] + coeff0_38*dmats2[38][44] + coeff0_39*dmats2[39][44] + coeff0_40*dmats2[40][44] + coeff0_41*dmats2[41][44] + coeff0_42*dmats2[42][44] + coeff0_43*dmats2[43][44] + coeff0_44*dmats2[44][44] + coeff0_45*dmats2[45][44] + coeff0_46*dmats2[46][44] + coeff0_47*dmats2[47][44] + coeff0_48*dmats2[48][44] + coeff0_49*dmats2[49][44] + coeff0_50*dmats2[50][44] + coeff0_51*dmats2[51][44] + coeff0_52*dmats2[52][44] + coeff0_53*dmats2[53][44] + coeff0_54*dmats2[54][44] + coeff0_55*dmats2[55][44];
          new_coeff0_45 = coeff0_0*dmats2[0][45] + coeff0_1*dmats2[1][45] + coeff0_2*dmats2[2][45] + coeff0_3*dmats2[3][45] + coeff0_4*dmats2[4][45] + coeff0_5*dmats2[5][45] + coeff0_6*dmats2[6][45] + coeff0_7*dmats2[7][45] + coeff0_8*dmats2[8][45] + coeff0_9*dmats2[9][45] + coeff0_10*dmats2[10][45] + coeff0_11*dmats2[11][45] + coeff0_12*dmats2[12][45] + coeff0_13*dmats2[13][45] + coeff0_14*dmats2[14][45] + coeff0_15*dmats2[15][45] + coeff0_16*dmats2[16][45] + coeff0_17*dmats2[17][45] + coeff0_18*dmats2[18][45] + coeff0_19*dmats2[19][45] + coeff0_20*dmats2[20][45] + coeff0_21*dmats2[21][45] + coeff0_22*dmats2[22][45] + coeff0_23*dmats2[23][45] + coeff0_24*dmats2[24][45] + coeff0_25*dmats2[25][45] + coeff0_26*dmats2[26][45] + coeff0_27*dmats2[27][45] + coeff0_28*dmats2[28][45] + coeff0_29*dmats2[29][45] + coeff0_30*dmats2[30][45] + coeff0_31*dmats2[31][45] + coeff0_32*dmats2[32][45] + coeff0_33*dmats2[33][45] + coeff0_34*dmats2[34][45] + coeff0_35*dmats2[35][45] + coeff0_36*dmats2[36][45] + coeff0_37*dmats2[37][45] + coeff0_38*dmats2[38][45] + coeff0_39*dmats2[39][45] + coeff0_40*dmats2[40][45] + coeff0_41*dmats2[41][45] + coeff0_42*dmats2[42][45] + coeff0_43*dmats2[43][45] + coeff0_44*dmats2[44][45] + coeff0_45*dmats2[45][45] + coeff0_46*dmats2[46][45] + coeff0_47*dmats2[47][45] + coeff0_48*dmats2[48][45] + coeff0_49*dmats2[49][45] + coeff0_50*dmats2[50][45] + coeff0_51*dmats2[51][45] + coeff0_52*dmats2[52][45] + coeff0_53*dmats2[53][45] + coeff0_54*dmats2[54][45] + coeff0_55*dmats2[55][45];
          new_coeff0_46 = coeff0_0*dmats2[0][46] + coeff0_1*dmats2[1][46] + coeff0_2*dmats2[2][46] + coeff0_3*dmats2[3][46] + coeff0_4*dmats2[4][46] + coeff0_5*dmats2[5][46] + coeff0_6*dmats2[6][46] + coeff0_7*dmats2[7][46] + coeff0_8*dmats2[8][46] + coeff0_9*dmats2[9][46] + coeff0_10*dmats2[10][46] + coeff0_11*dmats2[11][46] + coeff0_12*dmats2[12][46] + coeff0_13*dmats2[13][46] + coeff0_14*dmats2[14][46] + coeff0_15*dmats2[15][46] + coeff0_16*dmats2[16][46] + coeff0_17*dmats2[17][46] + coeff0_18*dmats2[18][46] + coeff0_19*dmats2[19][46] + coeff0_20*dmats2[20][46] + coeff0_21*dmats2[21][46] + coeff0_22*dmats2[22][46] + coeff0_23*dmats2[23][46] + coeff0_24*dmats2[24][46] + coeff0_25*dmats2[25][46] + coeff0_26*dmats2[26][46] + coeff0_27*dmats2[27][46] + coeff0_28*dmats2[28][46] + coeff0_29*dmats2[29][46] + coeff0_30*dmats2[30][46] + coeff0_31*dmats2[31][46] + coeff0_32*dmats2[32][46] + coeff0_33*dmats2[33][46] + coeff0_34*dmats2[34][46] + coeff0_35*dmats2[35][46] + coeff0_36*dmats2[36][46] + coeff0_37*dmats2[37][46] + coeff0_38*dmats2[38][46] + coeff0_39*dmats2[39][46] + coeff0_40*dmats2[40][46] + coeff0_41*dmats2[41][46] + coeff0_42*dmats2[42][46] + coeff0_43*dmats2[43][46] + coeff0_44*dmats2[44][46] + coeff0_45*dmats2[45][46] + coeff0_46*dmats2[46][46] + coeff0_47*dmats2[47][46] + coeff0_48*dmats2[48][46] + coeff0_49*dmats2[49][46] + coeff0_50*dmats2[50][46] + coeff0_51*dmats2[51][46] + coeff0_52*dmats2[52][46] + coeff0_53*dmats2[53][46] + coeff0_54*dmats2[54][46] + coeff0_55*dmats2[55][46];
          new_coeff0_47 = coeff0_0*dmats2[0][47] + coeff0_1*dmats2[1][47] + coeff0_2*dmats2[2][47] + coeff0_3*dmats2[3][47] + coeff0_4*dmats2[4][47] + coeff0_5*dmats2[5][47] + coeff0_6*dmats2[6][47] + coeff0_7*dmats2[7][47] + coeff0_8*dmats2[8][47] + coeff0_9*dmats2[9][47] + coeff0_10*dmats2[10][47] + coeff0_11*dmats2[11][47] + coeff0_12*dmats2[12][47] + coeff0_13*dmats2[13][47] + coeff0_14*dmats2[14][47] + coeff0_15*dmats2[15][47] + coeff0_16*dmats2[16][47] + coeff0_17*dmats2[17][47] + coeff0_18*dmats2[18][47] + coeff0_19*dmats2[19][47] + coeff0_20*dmats2[20][47] + coeff0_21*dmats2[21][47] + coeff0_22*dmats2[22][47] + coeff0_23*dmats2[23][47] + coeff0_24*dmats2[24][47] + coeff0_25*dmats2[25][47] + coeff0_26*dmats2[26][47] + coeff0_27*dmats2[27][47] + coeff0_28*dmats2[28][47] + coeff0_29*dmats2[29][47] + coeff0_30*dmats2[30][47] + coeff0_31*dmats2[31][47] + coeff0_32*dmats2[32][47] + coeff0_33*dmats2[33][47] + coeff0_34*dmats2[34][47] + coeff0_35*dmats2[35][47] + coeff0_36*dmats2[36][47] + coeff0_37*dmats2[37][47] + coeff0_38*dmats2[38][47] + coeff0_39*dmats2[39][47] + coeff0_40*dmats2[40][47] + coeff0_41*dmats2[41][47] + coeff0_42*dmats2[42][47] + coeff0_43*dmats2[43][47] + coeff0_44*dmats2[44][47] + coeff0_45*dmats2[45][47] + coeff0_46*dmats2[46][47] + coeff0_47*dmats2[47][47] + coeff0_48*dmats2[48][47] + coeff0_49*dmats2[49][47] + coeff0_50*dmats2[50][47] + coeff0_51*dmats2[51][47] + coeff0_52*dmats2[52][47] + coeff0_53*dmats2[53][47] + coeff0_54*dmats2[54][47] + coeff0_55*dmats2[55][47];
          new_coeff0_48 = coeff0_0*dmats2[0][48] + coeff0_1*dmats2[1][48] + coeff0_2*dmats2[2][48] + coeff0_3*dmats2[3][48] + coeff0_4*dmats2[4][48] + coeff0_5*dmats2[5][48] + coeff0_6*dmats2[6][48] + coeff0_7*dmats2[7][48] + coeff0_8*dmats2[8][48] + coeff0_9*dmats2[9][48] + coeff0_10*dmats2[10][48] + coeff0_11*dmats2[11][48] + coeff0_12*dmats2[12][48] + coeff0_13*dmats2[13][48] + coeff0_14*dmats2[14][48] + coeff0_15*dmats2[15][48] + coeff0_16*dmats2[16][48] + coeff0_17*dmats2[17][48] + coeff0_18*dmats2[18][48] + coeff0_19*dmats2[19][48] + coeff0_20*dmats2[20][48] + coeff0_21*dmats2[21][48] + coeff0_22*dmats2[22][48] + coeff0_23*dmats2[23][48] + coeff0_24*dmats2[24][48] + coeff0_25*dmats2[25][48] + coeff0_26*dmats2[26][48] + coeff0_27*dmats2[27][48] + coeff0_28*dmats2[28][48] + coeff0_29*dmats2[29][48] + coeff0_30*dmats2[30][48] + coeff0_31*dmats2[31][48] + coeff0_32*dmats2[32][48] + coeff0_33*dmats2[33][48] + coeff0_34*dmats2[34][48] + coeff0_35*dmats2[35][48] + coeff0_36*dmats2[36][48] + coeff0_37*dmats2[37][48] + coeff0_38*dmats2[38][48] + coeff0_39*dmats2[39][48] + coeff0_40*dmats2[40][48] + coeff0_41*dmats2[41][48] + coeff0_42*dmats2[42][48] + coeff0_43*dmats2[43][48] + coeff0_44*dmats2[44][48] + coeff0_45*dmats2[45][48] + coeff0_46*dmats2[46][48] + coeff0_47*dmats2[47][48] + coeff0_48*dmats2[48][48] + coeff0_49*dmats2[49][48] + coeff0_50*dmats2[50][48] + coeff0_51*dmats2[51][48] + coeff0_52*dmats2[52][48] + coeff0_53*dmats2[53][48] + coeff0_54*dmats2[54][48] + coeff0_55*dmats2[55][48];
          new_coeff0_49 = coeff0_0*dmats2[0][49] + coeff0_1*dmats2[1][49] + coeff0_2*dmats2[2][49] + coeff0_3*dmats2[3][49] + coeff0_4*dmats2[4][49] + coeff0_5*dmats2[5][49] + coeff0_6*dmats2[6][49] + coeff0_7*dmats2[7][49] + coeff0_8*dmats2[8][49] + coeff0_9*dmats2[9][49] + coeff0_10*dmats2[10][49] + coeff0_11*dmats2[11][49] + coeff0_12*dmats2[12][49] + coeff0_13*dmats2[13][49] + coeff0_14*dmats2[14][49] + coeff0_15*dmats2[15][49] + coeff0_16*dmats2[16][49] + coeff0_17*dmats2[17][49] + coeff0_18*dmats2[18][49] + coeff0_19*dmats2[19][49] + coeff0_20*dmats2[20][49] + coeff0_21*dmats2[21][49] + coeff0_22*dmats2[22][49] + coeff0_23*dmats2[23][49] + coeff0_24*dmats2[24][49] + coeff0_25*dmats2[25][49] + coeff0_26*dmats2[26][49] + coeff0_27*dmats2[27][49] + coeff0_28*dmats2[28][49] + coeff0_29*dmats2[29][49] + coeff0_30*dmats2[30][49] + coeff0_31*dmats2[31][49] + coeff0_32*dmats2[32][49] + coeff0_33*dmats2[33][49] + coeff0_34*dmats2[34][49] + coeff0_35*dmats2[35][49] + coeff0_36*dmats2[36][49] + coeff0_37*dmats2[37][49] + coeff0_38*dmats2[38][49] + coeff0_39*dmats2[39][49] + coeff0_40*dmats2[40][49] + coeff0_41*dmats2[41][49] + coeff0_42*dmats2[42][49] + coeff0_43*dmats2[43][49] + coeff0_44*dmats2[44][49] + coeff0_45*dmats2[45][49] + coeff0_46*dmats2[46][49] + coeff0_47*dmats2[47][49] + coeff0_48*dmats2[48][49] + coeff0_49*dmats2[49][49] + coeff0_50*dmats2[50][49] + coeff0_51*dmats2[51][49] + coeff0_52*dmats2[52][49] + coeff0_53*dmats2[53][49] + coeff0_54*dmats2[54][49] + coeff0_55*dmats2[55][49];
          new_coeff0_50 = coeff0_0*dmats2[0][50] + coeff0_1*dmats2[1][50] + coeff0_2*dmats2[2][50] + coeff0_3*dmats2[3][50] + coeff0_4*dmats2[4][50] + coeff0_5*dmats2[5][50] + coeff0_6*dmats2[6][50] + coeff0_7*dmats2[7][50] + coeff0_8*dmats2[8][50] + coeff0_9*dmats2[9][50] + coeff0_10*dmats2[10][50] + coeff0_11*dmats2[11][50] + coeff0_12*dmats2[12][50] + coeff0_13*dmats2[13][50] + coeff0_14*dmats2[14][50] + coeff0_15*dmats2[15][50] + coeff0_16*dmats2[16][50] + coeff0_17*dmats2[17][50] + coeff0_18*dmats2[18][50] + coeff0_19*dmats2[19][50] + coeff0_20*dmats2[20][50] + coeff0_21*dmats2[21][50] + coeff0_22*dmats2[22][50] + coeff0_23*dmats2[23][50] + coeff0_24*dmats2[24][50] + coeff0_25*dmats2[25][50] + coeff0_26*dmats2[26][50] + coeff0_27*dmats2[27][50] + coeff0_28*dmats2[28][50] + coeff0_29*dmats2[29][50] + coeff0_30*dmats2[30][50] + coeff0_31*dmats2[31][50] + coeff0_32*dmats2[32][50] + coeff0_33*dmats2[33][50] + coeff0_34*dmats2[34][50] + coeff0_35*dmats2[35][50] + coeff0_36*dmats2[36][50] + coeff0_37*dmats2[37][50] + coeff0_38*dmats2[38][50] + coeff0_39*dmats2[39][50] + coeff0_40*dmats2[40][50] + coeff0_41*dmats2[41][50] + coeff0_42*dmats2[42][50] + coeff0_43*dmats2[43][50] + coeff0_44*dmats2[44][50] + coeff0_45*dmats2[45][50] + coeff0_46*dmats2[46][50] + coeff0_47*dmats2[47][50] + coeff0_48*dmats2[48][50] + coeff0_49*dmats2[49][50] + coeff0_50*dmats2[50][50] + coeff0_51*dmats2[51][50] + coeff0_52*dmats2[52][50] + coeff0_53*dmats2[53][50] + coeff0_54*dmats2[54][50] + coeff0_55*dmats2[55][50];
          new_coeff0_51 = coeff0_0*dmats2[0][51] + coeff0_1*dmats2[1][51] + coeff0_2*dmats2[2][51] + coeff0_3*dmats2[3][51] + coeff0_4*dmats2[4][51] + coeff0_5*dmats2[5][51] + coeff0_6*dmats2[6][51] + coeff0_7*dmats2[7][51] + coeff0_8*dmats2[8][51] + coeff0_9*dmats2[9][51] + coeff0_10*dmats2[10][51] + coeff0_11*dmats2[11][51] + coeff0_12*dmats2[12][51] + coeff0_13*dmats2[13][51] + coeff0_14*dmats2[14][51] + coeff0_15*dmats2[15][51] + coeff0_16*dmats2[16][51] + coeff0_17*dmats2[17][51] + coeff0_18*dmats2[18][51] + coeff0_19*dmats2[19][51] + coeff0_20*dmats2[20][51] + coeff0_21*dmats2[21][51] + coeff0_22*dmats2[22][51] + coeff0_23*dmats2[23][51] + coeff0_24*dmats2[24][51] + coeff0_25*dmats2[25][51] + coeff0_26*dmats2[26][51] + coeff0_27*dmats2[27][51] + coeff0_28*dmats2[28][51] + coeff0_29*dmats2[29][51] + coeff0_30*dmats2[30][51] + coeff0_31*dmats2[31][51] + coeff0_32*dmats2[32][51] + coeff0_33*dmats2[33][51] + coeff0_34*dmats2[34][51] + coeff0_35*dmats2[35][51] + coeff0_36*dmats2[36][51] + coeff0_37*dmats2[37][51] + coeff0_38*dmats2[38][51] + coeff0_39*dmats2[39][51] + coeff0_40*dmats2[40][51] + coeff0_41*dmats2[41][51] + coeff0_42*dmats2[42][51] + coeff0_43*dmats2[43][51] + coeff0_44*dmats2[44][51] + coeff0_45*dmats2[45][51] + coeff0_46*dmats2[46][51] + coeff0_47*dmats2[47][51] + coeff0_48*dmats2[48][51] + coeff0_49*dmats2[49][51] + coeff0_50*dmats2[50][51] + coeff0_51*dmats2[51][51] + coeff0_52*dmats2[52][51] + coeff0_53*dmats2[53][51] + coeff0_54*dmats2[54][51] + coeff0_55*dmats2[55][51];
          new_coeff0_52 = coeff0_0*dmats2[0][52] + coeff0_1*dmats2[1][52] + coeff0_2*dmats2[2][52] + coeff0_3*dmats2[3][52] + coeff0_4*dmats2[4][52] + coeff0_5*dmats2[5][52] + coeff0_6*dmats2[6][52] + coeff0_7*dmats2[7][52] + coeff0_8*dmats2[8][52] + coeff0_9*dmats2[9][52] + coeff0_10*dmats2[10][52] + coeff0_11*dmats2[11][52] + coeff0_12*dmats2[12][52] + coeff0_13*dmats2[13][52] + coeff0_14*dmats2[14][52] + coeff0_15*dmats2[15][52] + coeff0_16*dmats2[16][52] + coeff0_17*dmats2[17][52] + coeff0_18*dmats2[18][52] + coeff0_19*dmats2[19][52] + coeff0_20*dmats2[20][52] + coeff0_21*dmats2[21][52] + coeff0_22*dmats2[22][52] + coeff0_23*dmats2[23][52] + coeff0_24*dmats2[24][52] + coeff0_25*dmats2[25][52] + coeff0_26*dmats2[26][52] + coeff0_27*dmats2[27][52] + coeff0_28*dmats2[28][52] + coeff0_29*dmats2[29][52] + coeff0_30*dmats2[30][52] + coeff0_31*dmats2[31][52] + coeff0_32*dmats2[32][52] + coeff0_33*dmats2[33][52] + coeff0_34*dmats2[34][52] + coeff0_35*dmats2[35][52] + coeff0_36*dmats2[36][52] + coeff0_37*dmats2[37][52] + coeff0_38*dmats2[38][52] + coeff0_39*dmats2[39][52] + coeff0_40*dmats2[40][52] + coeff0_41*dmats2[41][52] + coeff0_42*dmats2[42][52] + coeff0_43*dmats2[43][52] + coeff0_44*dmats2[44][52] + coeff0_45*dmats2[45][52] + coeff0_46*dmats2[46][52] + coeff0_47*dmats2[47][52] + coeff0_48*dmats2[48][52] + coeff0_49*dmats2[49][52] + coeff0_50*dmats2[50][52] + coeff0_51*dmats2[51][52] + coeff0_52*dmats2[52][52] + coeff0_53*dmats2[53][52] + coeff0_54*dmats2[54][52] + coeff0_55*dmats2[55][52];
          new_coeff0_53 = coeff0_0*dmats2[0][53] + coeff0_1*dmats2[1][53] + coeff0_2*dmats2[2][53] + coeff0_3*dmats2[3][53] + coeff0_4*dmats2[4][53] + coeff0_5*dmats2[5][53] + coeff0_6*dmats2[6][53] + coeff0_7*dmats2[7][53] + coeff0_8*dmats2[8][53] + coeff0_9*dmats2[9][53] + coeff0_10*dmats2[10][53] + coeff0_11*dmats2[11][53] + coeff0_12*dmats2[12][53] + coeff0_13*dmats2[13][53] + coeff0_14*dmats2[14][53] + coeff0_15*dmats2[15][53] + coeff0_16*dmats2[16][53] + coeff0_17*dmats2[17][53] + coeff0_18*dmats2[18][53] + coeff0_19*dmats2[19][53] + coeff0_20*dmats2[20][53] + coeff0_21*dmats2[21][53] + coeff0_22*dmats2[22][53] + coeff0_23*dmats2[23][53] + coeff0_24*dmats2[24][53] + coeff0_25*dmats2[25][53] + coeff0_26*dmats2[26][53] + coeff0_27*dmats2[27][53] + coeff0_28*dmats2[28][53] + coeff0_29*dmats2[29][53] + coeff0_30*dmats2[30][53] + coeff0_31*dmats2[31][53] + coeff0_32*dmats2[32][53] + coeff0_33*dmats2[33][53] + coeff0_34*dmats2[34][53] + coeff0_35*dmats2[35][53] + coeff0_36*dmats2[36][53] + coeff0_37*dmats2[37][53] + coeff0_38*dmats2[38][53] + coeff0_39*dmats2[39][53] + coeff0_40*dmats2[40][53] + coeff0_41*dmats2[41][53] + coeff0_42*dmats2[42][53] + coeff0_43*dmats2[43][53] + coeff0_44*dmats2[44][53] + coeff0_45*dmats2[45][53] + coeff0_46*dmats2[46][53] + coeff0_47*dmats2[47][53] + coeff0_48*dmats2[48][53] + coeff0_49*dmats2[49][53] + coeff0_50*dmats2[50][53] + coeff0_51*dmats2[51][53] + coeff0_52*dmats2[52][53] + coeff0_53*dmats2[53][53] + coeff0_54*dmats2[54][53] + coeff0_55*dmats2[55][53];
          new_coeff0_54 = coeff0_0*dmats2[0][54] + coeff0_1*dmats2[1][54] + coeff0_2*dmats2[2][54] + coeff0_3*dmats2[3][54] + coeff0_4*dmats2[4][54] + coeff0_5*dmats2[5][54] + coeff0_6*dmats2[6][54] + coeff0_7*dmats2[7][54] + coeff0_8*dmats2[8][54] + coeff0_9*dmats2[9][54] + coeff0_10*dmats2[10][54] + coeff0_11*dmats2[11][54] + coeff0_12*dmats2[12][54] + coeff0_13*dmats2[13][54] + coeff0_14*dmats2[14][54] + coeff0_15*dmats2[15][54] + coeff0_16*dmats2[16][54] + coeff0_17*dmats2[17][54] + coeff0_18*dmats2[18][54] + coeff0_19*dmats2[19][54] + coeff0_20*dmats2[20][54] + coeff0_21*dmats2[21][54] + coeff0_22*dmats2[22][54] + coeff0_23*dmats2[23][54] + coeff0_24*dmats2[24][54] + coeff0_25*dmats2[25][54] + coeff0_26*dmats2[26][54] + coeff0_27*dmats2[27][54] + coeff0_28*dmats2[28][54] + coeff0_29*dmats2[29][54] + coeff0_30*dmats2[30][54] + coeff0_31*dmats2[31][54] + coeff0_32*dmats2[32][54] + coeff0_33*dmats2[33][54] + coeff0_34*dmats2[34][54] + coeff0_35*dmats2[35][54] + coeff0_36*dmats2[36][54] + coeff0_37*dmats2[37][54] + coeff0_38*dmats2[38][54] + coeff0_39*dmats2[39][54] + coeff0_40*dmats2[40][54] + coeff0_41*dmats2[41][54] + coeff0_42*dmats2[42][54] + coeff0_43*dmats2[43][54] + coeff0_44*dmats2[44][54] + coeff0_45*dmats2[45][54] + coeff0_46*dmats2[46][54] + coeff0_47*dmats2[47][54] + coeff0_48*dmats2[48][54] + coeff0_49*dmats2[49][54] + coeff0_50*dmats2[50][54] + coeff0_51*dmats2[51][54] + coeff0_52*dmats2[52][54] + coeff0_53*dmats2[53][54] + coeff0_54*dmats2[54][54] + coeff0_55*dmats2[55][54];
          new_coeff0_55 = coeff0_0*dmats2[0][55] + coeff0_1*dmats2[1][55] + coeff0_2*dmats2[2][55] + coeff0_3*dmats2[3][55] + coeff0_4*dmats2[4][55] + coeff0_5*dmats2[5][55] + coeff0_6*dmats2[6][55] + coeff0_7*dmats2[7][55] + coeff0_8*dmats2[8][55] + coeff0_9*dmats2[9][55] + coeff0_10*dmats2[10][55] + coeff0_11*dmats2[11][55] + coeff0_12*dmats2[12][55] + coeff0_13*dmats2[13][55] + coeff0_14*dmats2[14][55] + coeff0_15*dmats2[15][55] + coeff0_16*dmats2[16][55] + coeff0_17*dmats2[17][55] + coeff0_18*dmats2[18][55] + coeff0_19*dmats2[19][55] + coeff0_20*dmats2[20][55] + coeff0_21*dmats2[21][55] + coeff0_22*dmats2[22][55] + coeff0_23*dmats2[23][55] + coeff0_24*dmats2[24][55] + coeff0_25*dmats2[25][55] + coeff0_26*dmats2[26][55] + coeff0_27*dmats2[27][55] + coeff0_28*dmats2[28][55] + coeff0_29*dmats2[29][55] + coeff0_30*dmats2[30][55] + coeff0_31*dmats2[31][55] + coeff0_32*dmats2[32][55] + coeff0_33*dmats2[33][55] + coeff0_34*dmats2[34][55] + coeff0_35*dmats2[35][55] + coeff0_36*dmats2[36][55] + coeff0_37*dmats2[37][55] + coeff0_38*dmats2[38][55] + coeff0_39*dmats2[39][55] + coeff0_40*dmats2[40][55] + coeff0_41*dmats2[41][55] + coeff0_42*dmats2[42][55] + coeff0_43*dmats2[43][55] + coeff0_44*dmats2[44][55] + coeff0_45*dmats2[45][55] + coeff0_46*dmats2[46][55] + coeff0_47*dmats2[47][55] + coeff0_48*dmats2[48][55] + coeff0_49*dmats2[49][55] + coeff0_50*dmats2[50][55] + coeff0_51*dmats2[51][55] + coeff0_52*dmats2[52][55] + coeff0_53*dmats2[53][55] + coeff0_54*dmats2[54][55] + coeff0_55*dmats2[55][55];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9 + new_coeff0_10*basisvalue10 + new_coeff0_11*basisvalue11 + new_coeff0_12*basisvalue12 + new_coeff0_13*basisvalue13 + new_coeff0_14*basisvalue14 + new_coeff0_15*basisvalue15 + new_coeff0_16*basisvalue16 + new_coeff0_17*basisvalue17 + new_coeff0_18*basisvalue18 + new_coeff0_19*basisvalue19 + new_coeff0_20*basisvalue20 + new_coeff0_21*basisvalue21 + new_coeff0_22*basisvalue22 + new_coeff0_23*basisvalue23 + new_coeff0_24*basisvalue24 + new_coeff0_25*basisvalue25 + new_coeff0_26*basisvalue26 + new_coeff0_27*basisvalue27 + new_coeff0_28*basisvalue28 + new_coeff0_29*basisvalue29 + new_coeff0_30*basisvalue30 + new_coeff0_31*basisvalue31 + new_coeff0_32*basisvalue32 + new_coeff0_33*basisvalue33 + new_coeff0_34*basisvalue34 + new_coeff0_35*basisvalue35 + new_coeff0_36*basisvalue36 + new_coeff0_37*basisvalue37 + new_coeff0_38*basisvalue38 + new_coeff0_39*basisvalue39 + new_coeff0_40*basisvalue40 + new_coeff0_41*basisvalue41 + new_coeff0_42*basisvalue42 + new_coeff0_43*basisvalue43 + new_coeff0_44*basisvalue44 + new_coeff0_45*basisvalue45 + new_coeff0_46*basisvalue46 + new_coeff0_47*basisvalue47 + new_coeff0_48*basisvalue48 + new_coeff0_49*basisvalue49 + new_coeff0_50*basisvalue50 + new_coeff0_51*basisvalue51 + new_coeff0_52*basisvalue52 + new_coeff0_53*basisvalue53 + new_coeff0_54*basisvalue54 + new_coeff0_55*basisvalue55;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    static const double X[56][1][3] = {{{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}, {{0, 0.8, 0.2}}, {{0, 0.6, 0.4}}, {{0, 0.4, 0.6}}, {{0, 0.2, 0.8}}, {{0.8, 0, 0.2}}, {{0.6, 0, 0.4}}, {{0.4, 0, 0.6}}, {{0.2, 0, 0.8}}, {{0.8, 0.2, 0}}, {{0.6, 0.4, 0}}, {{0.4, 0.6, 0}}, {{0.2, 0.8, 0}}, {{0, 0, 0.2}}, {{0, 0, 0.4}}, {{0, 0, 0.6}}, {{0, 0, 0.8}}, {{0, 0.2, 0}}, {{0, 0.4, 0}}, {{0, 0.6, 0}}, {{0, 0.8, 0}}, {{0.2, 0, 0}}, {{0.4, 0, 0}}, {{0.6, 0, 0}}, {{0.8, 0, 0}}, {{0.6, 0.2, 0.2}}, {{0.4, 0.4, 0.2}}, {{0.2, 0.6, 0.2}}, {{0.4, 0.2, 0.4}}, {{0.2, 0.4, 0.4}}, {{0.2, 0.2, 0.6}}, {{0, 0.2, 0.2}}, {{0, 0.4, 0.2}}, {{0, 0.6, 0.2}}, {{0, 0.2, 0.4}}, {{0, 0.4, 0.4}}, {{0, 0.2, 0.6}}, {{0.2, 0, 0.2}}, {{0.4, 0, 0.2}}, {{0.6, 0, 0.2}}, {{0.2, 0, 0.4}}, {{0.4, 0, 0.4}}, {{0.2, 0, 0.6}}, {{0.2, 0.2, 0}}, {{0.4, 0.2, 0}}, {{0.6, 0.2, 0}}, {{0.2, 0.4, 0}}, {{0.4, 0.4, 0}}, {{0.2, 0.6, 0}}, {{0.2, 0.2, 0.2}}, {{0.4, 0.2, 0.2}}, {{0.2, 0.4, 0.2}}, {{0.2, 0.2, 0.4}}};
    static const double W[56][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    static const double D[56][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new poisson3d_5_1_finite_element_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class poisson3d_5_1_dof_map_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  poisson3d_5_1_dof_map_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~poisson3d_5_1_dof_map_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for FiniteElement('Lagrange', 'tetrahedron', 5)";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return true;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + 4*m.num_entities[1] + 6*m.num_entities[2] + 4*m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space for a cell
  virtual unsigned int local_dimension(const ufc::cell& c) const
  {
    return 56;
  }

  /// Return the maximum dimension of the local finite element function space
  virtual unsigned int max_local_dimension() const
  {
    return 56;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    return 3;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 21;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + 4*c.entity_indices[1][0];
    dofs[5] = offset + 4*c.entity_indices[1][0] + 1;
    dofs[6] = offset + 4*c.entity_indices[1][0] + 2;
    dofs[7] = offset + 4*c.entity_indices[1][0] + 3;
    dofs[8] = offset + 4*c.entity_indices[1][1];
    dofs[9] = offset + 4*c.entity_indices[1][1] + 1;
    dofs[10] = offset + 4*c.entity_indices[1][1] + 2;
    dofs[11] = offset + 4*c.entity_indices[1][1] + 3;
    dofs[12] = offset + 4*c.entity_indices[1][2];
    dofs[13] = offset + 4*c.entity_indices[1][2] + 1;
    dofs[14] = offset + 4*c.entity_indices[1][2] + 2;
    dofs[15] = offset + 4*c.entity_indices[1][2] + 3;
    dofs[16] = offset + 4*c.entity_indices[1][3];
    dofs[17] = offset + 4*c.entity_indices[1][3] + 1;
    dofs[18] = offset + 4*c.entity_indices[1][3] + 2;
    dofs[19] = offset + 4*c.entity_indices[1][3] + 3;
    dofs[20] = offset + 4*c.entity_indices[1][4];
    dofs[21] = offset + 4*c.entity_indices[1][4] + 1;
    dofs[22] = offset + 4*c.entity_indices[1][4] + 2;
    dofs[23] = offset + 4*c.entity_indices[1][4] + 3;
    dofs[24] = offset + 4*c.entity_indices[1][5];
    dofs[25] = offset + 4*c.entity_indices[1][5] + 1;
    dofs[26] = offset + 4*c.entity_indices[1][5] + 2;
    dofs[27] = offset + 4*c.entity_indices[1][5] + 3;
    offset = offset + 4*m.num_entities[1];
    dofs[28] = offset + 6*c.entity_indices[2][0];
    dofs[29] = offset + 6*c.entity_indices[2][0] + 1;
    dofs[30] = offset + 6*c.entity_indices[2][0] + 2;
    dofs[31] = offset + 6*c.entity_indices[2][0] + 3;
    dofs[32] = offset + 6*c.entity_indices[2][0] + 4;
    dofs[33] = offset + 6*c.entity_indices[2][0] + 5;
    dofs[34] = offset + 6*c.entity_indices[2][1];
    dofs[35] = offset + 6*c.entity_indices[2][1] + 1;
    dofs[36] = offset + 6*c.entity_indices[2][1] + 2;
    dofs[37] = offset + 6*c.entity_indices[2][1] + 3;
    dofs[38] = offset + 6*c.entity_indices[2][1] + 4;
    dofs[39] = offset + 6*c.entity_indices[2][1] + 5;
    dofs[40] = offset + 6*c.entity_indices[2][2];
    dofs[41] = offset + 6*c.entity_indices[2][2] + 1;
    dofs[42] = offset + 6*c.entity_indices[2][2] + 2;
    dofs[43] = offset + 6*c.entity_indices[2][2] + 3;
    dofs[44] = offset + 6*c.entity_indices[2][2] + 4;
    dofs[45] = offset + 6*c.entity_indices[2][2] + 5;
    dofs[46] = offset + 6*c.entity_indices[2][3];
    dofs[47] = offset + 6*c.entity_indices[2][3] + 1;
    dofs[48] = offset + 6*c.entity_indices[2][3] + 2;
    dofs[49] = offset + 6*c.entity_indices[2][3] + 3;
    dofs[50] = offset + 6*c.entity_indices[2][3] + 4;
    dofs[51] = offset + 6*c.entity_indices[2][3] + 5;
    offset = offset + 6*m.num_entities[2];
    dofs[52] = offset + 4*c.entity_indices[3][0];
    dofs[53] = offset + 4*c.entity_indices[3][0] + 1;
    dofs[54] = offset + 4*c.entity_indices[3][0] + 2;
    dofs[55] = offset + 4*c.entity_indices[3][0] + 3;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 8;
      dofs[8] = 9;
      dofs[9] = 10;
      dofs[10] = 11;
      dofs[11] = 12;
      dofs[12] = 13;
      dofs[13] = 14;
      dofs[14] = 15;
      dofs[15] = 28;
      dofs[16] = 29;
      dofs[17] = 30;
      dofs[18] = 31;
      dofs[19] = 32;
      dofs[20] = 33;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 20;
      dofs[12] = 21;
      dofs[13] = 22;
      dofs[14] = 23;
      dofs[15] = 34;
      dofs[16] = 35;
      dofs[17] = 36;
      dofs[18] = 37;
      dofs[19] = 38;
      dofs[20] = 39;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 8;
      dofs[4] = 9;
      dofs[5] = 10;
      dofs[6] = 11;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 40;
      dofs[16] = 41;
      dofs[17] = 42;
      dofs[18] = 43;
      dofs[19] = 44;
      dofs[20] = 45;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 12;
      dofs[4] = 13;
      dofs[5] = 14;
      dofs[6] = 15;
      dofs[7] = 20;
      dofs[8] = 21;
      dofs[9] = 22;
      dofs[10] = 23;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 46;
      dofs[16] = 47;
      dofs[17] = 48;
      dofs[18] = 49;
      dofs[19] = 50;
      dofs[20] = 51;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.8*x[2][0] + 0.2*x[3][0];
    coordinates[4][1] = 0.8*x[2][1] + 0.2*x[3][1];
    coordinates[4][2] = 0.8*x[2][2] + 0.2*x[3][2];
    coordinates[5][0] = 0.6*x[2][0] + 0.4*x[3][0];
    coordinates[5][1] = 0.6*x[2][1] + 0.4*x[3][1];
    coordinates[5][2] = 0.6*x[2][2] + 0.4*x[3][2];
    coordinates[6][0] = 0.4*x[2][0] + 0.6*x[3][0];
    coordinates[6][1] = 0.4*x[2][1] + 0.6*x[3][1];
    coordinates[6][2] = 0.4*x[2][2] + 0.6*x[3][2];
    coordinates[7][0] = 0.2*x[2][0] + 0.8*x[3][0];
    coordinates[7][1] = 0.2*x[2][1] + 0.8*x[3][1];
    coordinates[7][2] = 0.2*x[2][2] + 0.8*x[3][2];
    coordinates[8][0] = 0.8*x[1][0] + 0.2*x[3][0];
    coordinates[8][1] = 0.8*x[1][1] + 0.2*x[3][1];
    coordinates[8][2] = 0.8*x[1][2] + 0.2*x[3][2];
    coordinates[9][0] = 0.6*x[1][0] + 0.4*x[3][0];
    coordinates[9][1] = 0.6*x[1][1] + 0.4*x[3][1];
    coordinates[9][2] = 0.6*x[1][2] + 0.4*x[3][2];
    coordinates[10][0] = 0.4*x[1][0] + 0.6*x[3][0];
    coordinates[10][1] = 0.4*x[1][1] + 0.6*x[3][1];
    coordinates[10][2] = 0.4*x[1][2] + 0.6*x[3][2];
    coordinates[11][0] = 0.2*x[1][0] + 0.8*x[3][0];
    coordinates[11][1] = 0.2*x[1][1] + 0.8*x[3][1];
    coordinates[11][2] = 0.2*x[1][2] + 0.8*x[3][2];
    coordinates[12][0] = 0.8*x[1][0] + 0.2*x[2][0];
    coordinates[12][1] = 0.8*x[1][1] + 0.2*x[2][1];
    coordinates[12][2] = 0.8*x[1][2] + 0.2*x[2][2];
    coordinates[13][0] = 0.6*x[1][0] + 0.4*x[2][0];
    coordinates[13][1] = 0.6*x[1][1] + 0.4*x[2][1];
    coordinates[13][2] = 0.6*x[1][2] + 0.4*x[2][2];
    coordinates[14][0] = 0.4*x[1][0] + 0.6*x[2][0];
    coordinates[14][1] = 0.4*x[1][1] + 0.6*x[2][1];
    coordinates[14][2] = 0.4*x[1][2] + 0.6*x[2][2];
    coordinates[15][0] = 0.2*x[1][0] + 0.8*x[2][0];
    coordinates[15][1] = 0.2*x[1][1] + 0.8*x[2][1];
    coordinates[15][2] = 0.2*x[1][2] + 0.8*x[2][2];
    coordinates[16][0] = 0.8*x[0][0] + 0.2*x[3][0];
    coordinates[16][1] = 0.8*x[0][1] + 0.2*x[3][1];
    coordinates[16][2] = 0.8*x[0][2] + 0.2*x[3][2];
    coordinates[17][0] = 0.6*x[0][0] + 0.4*x[3][0];
    coordinates[17][1] = 0.6*x[0][1] + 0.4*x[3][1];
    coordinates[17][2] = 0.6*x[0][2] + 0.4*x[3][2];
    coordinates[18][0] = 0.4*x[0][0] + 0.6*x[3][0];
    coordinates[18][1] = 0.4*x[0][1] + 0.6*x[3][1];
    coordinates[18][2] = 0.4*x[0][2] + 0.6*x[3][2];
    coordinates[19][0] = 0.2*x[0][0] + 0.8*x[3][0];
    coordinates[19][1] = 0.2*x[0][1] + 0.8*x[3][1];
    coordinates[19][2] = 0.2*x[0][2] + 0.8*x[3][2];
    coordinates[20][0] = 0.8*x[0][0] + 0.2*x[2][0];
    coordinates[20][1] = 0.8*x[0][1] + 0.2*x[2][1];
    coordinates[20][2] = 0.8*x[0][2] + 0.2*x[2][2];
    coordinates[21][0] = 0.6*x[0][0] + 0.4*x[2][0];
    coordinates[21][1] = 0.6*x[0][1] + 0.4*x[2][1];
    coordinates[21][2] = 0.6*x[0][2] + 0.4*x[2][2];
    coordinates[22][0] = 0.4*x[0][0] + 0.6*x[2][0];
    coordinates[22][1] = 0.4*x[0][1] + 0.6*x[2][1];
    coordinates[22][2] = 0.4*x[0][2] + 0.6*x[2][2];
    coordinates[23][0] = 0.2*x[0][0] + 0.8*x[2][0];
    coordinates[23][1] = 0.2*x[0][1] + 0.8*x[2][1];
    coordinates[23][2] = 0.2*x[0][2] + 0.8*x[2][2];
    coordinates[24][0] = 0.8*x[0][0] + 0.2*x[1][0];
    coordinates[24][1] = 0.8*x[0][1] + 0.2*x[1][1];
    coordinates[24][2] = 0.8*x[0][2] + 0.2*x[1][2];
    coordinates[25][0] = 0.6*x[0][0] + 0.4*x[1][0];
    coordinates[25][1] = 0.6*x[0][1] + 0.4*x[1][1];
    coordinates[25][2] = 0.6*x[0][2] + 0.4*x[1][2];
    coordinates[26][0] = 0.4*x[0][0] + 0.6*x[1][0];
    coordinates[26][1] = 0.4*x[0][1] + 0.6*x[1][1];
    coordinates[26][2] = 0.4*x[0][2] + 0.6*x[1][2];
    coordinates[27][0] = 0.2*x[0][0] + 0.8*x[1][0];
    coordinates[27][1] = 0.2*x[0][1] + 0.8*x[1][1];
    coordinates[27][2] = 0.2*x[0][2] + 0.8*x[1][2];
    coordinates[28][0] = 0.6*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[28][1] = 0.6*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[28][2] = 0.6*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[29][0] = 0.4*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[29][1] = 0.4*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[29][2] = 0.4*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[30][0] = 0.2*x[1][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[30][1] = 0.2*x[1][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[30][2] = 0.2*x[1][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[31][0] = 0.4*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[31][1] = 0.4*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[31][2] = 0.4*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[32][0] = 0.2*x[1][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[32][1] = 0.2*x[1][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[32][2] = 0.2*x[1][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[33][0] = 0.2*x[1][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[33][1] = 0.2*x[1][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[33][2] = 0.2*x[1][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[34][0] = 0.6*x[0][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[34][1] = 0.6*x[0][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[34][2] = 0.6*x[0][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[35][0] = 0.4*x[0][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[35][1] = 0.4*x[0][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[35][2] = 0.4*x[0][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[36][0] = 0.2*x[0][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[36][1] = 0.2*x[0][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[36][2] = 0.2*x[0][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[37][0] = 0.4*x[0][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[37][1] = 0.4*x[0][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[37][2] = 0.4*x[0][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[38][0] = 0.2*x[0][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[38][1] = 0.2*x[0][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[38][2] = 0.2*x[0][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[39][0] = 0.2*x[0][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[39][1] = 0.2*x[0][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[39][2] = 0.2*x[0][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[40][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[3][0];
    coordinates[40][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[3][1];
    coordinates[40][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[3][2];
    coordinates[41][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[3][0];
    coordinates[41][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[3][1];
    coordinates[41][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[3][2];
    coordinates[42][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[3][0];
    coordinates[42][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[3][1];
    coordinates[42][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[3][2];
    coordinates[43][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[3][0];
    coordinates[43][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[3][1];
    coordinates[43][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[3][2];
    coordinates[44][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[3][0];
    coordinates[44][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[3][1];
    coordinates[44][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[3][2];
    coordinates[45][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[3][0];
    coordinates[45][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[3][1];
    coordinates[45][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[3][2];
    coordinates[46][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0];
    coordinates[46][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1];
    coordinates[46][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2];
    coordinates[47][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0];
    coordinates[47][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1];
    coordinates[47][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2];
    coordinates[48][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[2][0];
    coordinates[48][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[2][1];
    coordinates[48][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[2][2];
    coordinates[49][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0];
    coordinates[49][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1];
    coordinates[49][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2];
    coordinates[50][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[2][0];
    coordinates[50][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[2][1];
    coordinates[50][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[2][2];
    coordinates[51][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[2][0];
    coordinates[51][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[2][1];
    coordinates[51][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[2][2];
    coordinates[52][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[52][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[52][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[53][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[53][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[53][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[54][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[54][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[54][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[55][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[55][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[55][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new poisson3d_5_1_dof_map_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class poisson3d_5_1_dof_map_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  poisson3d_5_1_dof_map_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~poisson3d_5_1_dof_map_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for FiniteElement('Lagrange', 'tetrahedron', 5)";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return true;
      break;
    case 3:
      return true;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + 4*m.num_entities[1] + 6*m.num_entities[2] + 4*m.num_entities[3];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space for a cell
  virtual unsigned int local_dimension(const ufc::cell& c) const
  {
    return 56;
  }

  /// Return the maximum dimension of the local finite element function space
  virtual unsigned int max_local_dimension() const
  {
    return 56;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    return 3;
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 21;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + 4*c.entity_indices[1][0];
    dofs[5] = offset + 4*c.entity_indices[1][0] + 1;
    dofs[6] = offset + 4*c.entity_indices[1][0] + 2;
    dofs[7] = offset + 4*c.entity_indices[1][0] + 3;
    dofs[8] = offset + 4*c.entity_indices[1][1];
    dofs[9] = offset + 4*c.entity_indices[1][1] + 1;
    dofs[10] = offset + 4*c.entity_indices[1][1] + 2;
    dofs[11] = offset + 4*c.entity_indices[1][1] + 3;
    dofs[12] = offset + 4*c.entity_indices[1][2];
    dofs[13] = offset + 4*c.entity_indices[1][2] + 1;
    dofs[14] = offset + 4*c.entity_indices[1][2] + 2;
    dofs[15] = offset + 4*c.entity_indices[1][2] + 3;
    dofs[16] = offset + 4*c.entity_indices[1][3];
    dofs[17] = offset + 4*c.entity_indices[1][3] + 1;
    dofs[18] = offset + 4*c.entity_indices[1][3] + 2;
    dofs[19] = offset + 4*c.entity_indices[1][3] + 3;
    dofs[20] = offset + 4*c.entity_indices[1][4];
    dofs[21] = offset + 4*c.entity_indices[1][4] + 1;
    dofs[22] = offset + 4*c.entity_indices[1][4] + 2;
    dofs[23] = offset + 4*c.entity_indices[1][4] + 3;
    dofs[24] = offset + 4*c.entity_indices[1][5];
    dofs[25] = offset + 4*c.entity_indices[1][5] + 1;
    dofs[26] = offset + 4*c.entity_indices[1][5] + 2;
    dofs[27] = offset + 4*c.entity_indices[1][5] + 3;
    offset = offset + 4*m.num_entities[1];
    dofs[28] = offset + 6*c.entity_indices[2][0];
    dofs[29] = offset + 6*c.entity_indices[2][0] + 1;
    dofs[30] = offset + 6*c.entity_indices[2][0] + 2;
    dofs[31] = offset + 6*c.entity_indices[2][0] + 3;
    dofs[32] = offset + 6*c.entity_indices[2][0] + 4;
    dofs[33] = offset + 6*c.entity_indices[2][0] + 5;
    dofs[34] = offset + 6*c.entity_indices[2][1];
    dofs[35] = offset + 6*c.entity_indices[2][1] + 1;
    dofs[36] = offset + 6*c.entity_indices[2][1] + 2;
    dofs[37] = offset + 6*c.entity_indices[2][1] + 3;
    dofs[38] = offset + 6*c.entity_indices[2][1] + 4;
    dofs[39] = offset + 6*c.entity_indices[2][1] + 5;
    dofs[40] = offset + 6*c.entity_indices[2][2];
    dofs[41] = offset + 6*c.entity_indices[2][2] + 1;
    dofs[42] = offset + 6*c.entity_indices[2][2] + 2;
    dofs[43] = offset + 6*c.entity_indices[2][2] + 3;
    dofs[44] = offset + 6*c.entity_indices[2][2] + 4;
    dofs[45] = offset + 6*c.entity_indices[2][2] + 5;
    dofs[46] = offset + 6*c.entity_indices[2][3];
    dofs[47] = offset + 6*c.entity_indices[2][3] + 1;
    dofs[48] = offset + 6*c.entity_indices[2][3] + 2;
    dofs[49] = offset + 6*c.entity_indices[2][3] + 3;
    dofs[50] = offset + 6*c.entity_indices[2][3] + 4;
    dofs[51] = offset + 6*c.entity_indices[2][3] + 5;
    offset = offset + 6*m.num_entities[2];
    dofs[52] = offset + 4*c.entity_indices[3][0];
    dofs[53] = offset + 4*c.entity_indices[3][0] + 1;
    dofs[54] = offset + 4*c.entity_indices[3][0] + 2;
    dofs[55] = offset + 4*c.entity_indices[3][0] + 3;
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 8;
      dofs[8] = 9;
      dofs[9] = 10;
      dofs[10] = 11;
      dofs[11] = 12;
      dofs[12] = 13;
      dofs[13] = 14;
      dofs[14] = 15;
      dofs[15] = 28;
      dofs[16] = 29;
      dofs[17] = 30;
      dofs[18] = 31;
      dofs[19] = 32;
      dofs[20] = 33;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 7;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 20;
      dofs[12] = 21;
      dofs[13] = 22;
      dofs[14] = 23;
      dofs[15] = 34;
      dofs[16] = 35;
      dofs[17] = 36;
      dofs[18] = 37;
      dofs[19] = 38;
      dofs[20] = 39;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 8;
      dofs[4] = 9;
      dofs[5] = 10;
      dofs[6] = 11;
      dofs[7] = 16;
      dofs[8] = 17;
      dofs[9] = 18;
      dofs[10] = 19;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 40;
      dofs[16] = 41;
      dofs[17] = 42;
      dofs[18] = 43;
      dofs[19] = 44;
      dofs[20] = 45;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 12;
      dofs[4] = 13;
      dofs[5] = 14;
      dofs[6] = 15;
      dofs[7] = 20;
      dofs[8] = 21;
      dofs[9] = 22;
      dofs[10] = 23;
      dofs[11] = 24;
      dofs[12] = 25;
      dofs[13] = 26;
      dofs[14] = 27;
      dofs[15] = 46;
      dofs[16] = 47;
      dofs[17] = 48;
      dofs[18] = 49;
      dofs[19] = 50;
      dofs[20] = 51;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.8*x[2][0] + 0.2*x[3][0];
    coordinates[4][1] = 0.8*x[2][1] + 0.2*x[3][1];
    coordinates[4][2] = 0.8*x[2][2] + 0.2*x[3][2];
    coordinates[5][0] = 0.6*x[2][0] + 0.4*x[3][0];
    coordinates[5][1] = 0.6*x[2][1] + 0.4*x[3][1];
    coordinates[5][2] = 0.6*x[2][2] + 0.4*x[3][2];
    coordinates[6][0] = 0.4*x[2][0] + 0.6*x[3][0];
    coordinates[6][1] = 0.4*x[2][1] + 0.6*x[3][1];
    coordinates[6][2] = 0.4*x[2][2] + 0.6*x[3][2];
    coordinates[7][0] = 0.2*x[2][0] + 0.8*x[3][0];
    coordinates[7][1] = 0.2*x[2][1] + 0.8*x[3][1];
    coordinates[7][2] = 0.2*x[2][2] + 0.8*x[3][2];
    coordinates[8][0] = 0.8*x[1][0] + 0.2*x[3][0];
    coordinates[8][1] = 0.8*x[1][1] + 0.2*x[3][1];
    coordinates[8][2] = 0.8*x[1][2] + 0.2*x[3][2];
    coordinates[9][0] = 0.6*x[1][0] + 0.4*x[3][0];
    coordinates[9][1] = 0.6*x[1][1] + 0.4*x[3][1];
    coordinates[9][2] = 0.6*x[1][2] + 0.4*x[3][2];
    coordinates[10][0] = 0.4*x[1][0] + 0.6*x[3][0];
    coordinates[10][1] = 0.4*x[1][1] + 0.6*x[3][1];
    coordinates[10][2] = 0.4*x[1][2] + 0.6*x[3][2];
    coordinates[11][0] = 0.2*x[1][0] + 0.8*x[3][0];
    coordinates[11][1] = 0.2*x[1][1] + 0.8*x[3][1];
    coordinates[11][2] = 0.2*x[1][2] + 0.8*x[3][2];
    coordinates[12][0] = 0.8*x[1][0] + 0.2*x[2][0];
    coordinates[12][1] = 0.8*x[1][1] + 0.2*x[2][1];
    coordinates[12][2] = 0.8*x[1][2] + 0.2*x[2][2];
    coordinates[13][0] = 0.6*x[1][0] + 0.4*x[2][0];
    coordinates[13][1] = 0.6*x[1][1] + 0.4*x[2][1];
    coordinates[13][2] = 0.6*x[1][2] + 0.4*x[2][2];
    coordinates[14][0] = 0.4*x[1][0] + 0.6*x[2][0];
    coordinates[14][1] = 0.4*x[1][1] + 0.6*x[2][1];
    coordinates[14][2] = 0.4*x[1][2] + 0.6*x[2][2];
    coordinates[15][0] = 0.2*x[1][0] + 0.8*x[2][0];
    coordinates[15][1] = 0.2*x[1][1] + 0.8*x[2][1];
    coordinates[15][2] = 0.2*x[1][2] + 0.8*x[2][2];
    coordinates[16][0] = 0.8*x[0][0] + 0.2*x[3][0];
    coordinates[16][1] = 0.8*x[0][1] + 0.2*x[3][1];
    coordinates[16][2] = 0.8*x[0][2] + 0.2*x[3][2];
    coordinates[17][0] = 0.6*x[0][0] + 0.4*x[3][0];
    coordinates[17][1] = 0.6*x[0][1] + 0.4*x[3][1];
    coordinates[17][2] = 0.6*x[0][2] + 0.4*x[3][2];
    coordinates[18][0] = 0.4*x[0][0] + 0.6*x[3][0];
    coordinates[18][1] = 0.4*x[0][1] + 0.6*x[3][1];
    coordinates[18][2] = 0.4*x[0][2] + 0.6*x[3][2];
    coordinates[19][0] = 0.2*x[0][0] + 0.8*x[3][0];
    coordinates[19][1] = 0.2*x[0][1] + 0.8*x[3][1];
    coordinates[19][2] = 0.2*x[0][2] + 0.8*x[3][2];
    coordinates[20][0] = 0.8*x[0][0] + 0.2*x[2][0];
    coordinates[20][1] = 0.8*x[0][1] + 0.2*x[2][1];
    coordinates[20][2] = 0.8*x[0][2] + 0.2*x[2][2];
    coordinates[21][0] = 0.6*x[0][0] + 0.4*x[2][0];
    coordinates[21][1] = 0.6*x[0][1] + 0.4*x[2][1];
    coordinates[21][2] = 0.6*x[0][2] + 0.4*x[2][2];
    coordinates[22][0] = 0.4*x[0][0] + 0.6*x[2][0];
    coordinates[22][1] = 0.4*x[0][1] + 0.6*x[2][1];
    coordinates[22][2] = 0.4*x[0][2] + 0.6*x[2][2];
    coordinates[23][0] = 0.2*x[0][0] + 0.8*x[2][0];
    coordinates[23][1] = 0.2*x[0][1] + 0.8*x[2][1];
    coordinates[23][2] = 0.2*x[0][2] + 0.8*x[2][2];
    coordinates[24][0] = 0.8*x[0][0] + 0.2*x[1][0];
    coordinates[24][1] = 0.8*x[0][1] + 0.2*x[1][1];
    coordinates[24][2] = 0.8*x[0][2] + 0.2*x[1][2];
    coordinates[25][0] = 0.6*x[0][0] + 0.4*x[1][0];
    coordinates[25][1] = 0.6*x[0][1] + 0.4*x[1][1];
    coordinates[25][2] = 0.6*x[0][2] + 0.4*x[1][2];
    coordinates[26][0] = 0.4*x[0][0] + 0.6*x[1][0];
    coordinates[26][1] = 0.4*x[0][1] + 0.6*x[1][1];
    coordinates[26][2] = 0.4*x[0][2] + 0.6*x[1][2];
    coordinates[27][0] = 0.2*x[0][0] + 0.8*x[1][0];
    coordinates[27][1] = 0.2*x[0][1] + 0.8*x[1][1];
    coordinates[27][2] = 0.2*x[0][2] + 0.8*x[1][2];
    coordinates[28][0] = 0.6*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[28][1] = 0.6*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[28][2] = 0.6*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[29][0] = 0.4*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[29][1] = 0.4*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[29][2] = 0.4*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[30][0] = 0.2*x[1][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[30][1] = 0.2*x[1][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[30][2] = 0.2*x[1][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[31][0] = 0.4*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[31][1] = 0.4*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[31][2] = 0.4*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[32][0] = 0.2*x[1][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[32][1] = 0.2*x[1][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[32][2] = 0.2*x[1][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[33][0] = 0.2*x[1][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[33][1] = 0.2*x[1][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[33][2] = 0.2*x[1][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[34][0] = 0.6*x[0][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[34][1] = 0.6*x[0][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[34][2] = 0.6*x[0][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[35][0] = 0.4*x[0][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[35][1] = 0.4*x[0][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[35][2] = 0.4*x[0][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[36][0] = 0.2*x[0][0] + 0.6*x[2][0] + 0.2*x[3][0];
    coordinates[36][1] = 0.2*x[0][1] + 0.6*x[2][1] + 0.2*x[3][1];
    coordinates[36][2] = 0.2*x[0][2] + 0.6*x[2][2] + 0.2*x[3][2];
    coordinates[37][0] = 0.4*x[0][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[37][1] = 0.4*x[0][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[37][2] = 0.4*x[0][2] + 0.2*x[2][2] + 0.4*x[3][2];
    coordinates[38][0] = 0.2*x[0][0] + 0.4*x[2][0] + 0.4*x[3][0];
    coordinates[38][1] = 0.2*x[0][1] + 0.4*x[2][1] + 0.4*x[3][1];
    coordinates[38][2] = 0.2*x[0][2] + 0.4*x[2][2] + 0.4*x[3][2];
    coordinates[39][0] = 0.2*x[0][0] + 0.2*x[2][0] + 0.6*x[3][0];
    coordinates[39][1] = 0.2*x[0][1] + 0.2*x[2][1] + 0.6*x[3][1];
    coordinates[39][2] = 0.2*x[0][2] + 0.2*x[2][2] + 0.6*x[3][2];
    coordinates[40][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[3][0];
    coordinates[40][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[3][1];
    coordinates[40][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[3][2];
    coordinates[41][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[3][0];
    coordinates[41][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[3][1];
    coordinates[41][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[3][2];
    coordinates[42][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[3][0];
    coordinates[42][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[3][1];
    coordinates[42][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[3][2];
    coordinates[43][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[3][0];
    coordinates[43][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[3][1];
    coordinates[43][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[3][2];
    coordinates[44][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[3][0];
    coordinates[44][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[3][1];
    coordinates[44][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[3][2];
    coordinates[45][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[3][0];
    coordinates[45][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[3][1];
    coordinates[45][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[3][2];
    coordinates[46][0] = 0.6*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0];
    coordinates[46][1] = 0.6*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1];
    coordinates[46][2] = 0.6*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2];
    coordinates[47][0] = 0.4*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0];
    coordinates[47][1] = 0.4*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1];
    coordinates[47][2] = 0.4*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2];
    coordinates[48][0] = 0.2*x[0][0] + 0.6*x[1][0] + 0.2*x[2][0];
    coordinates[48][1] = 0.2*x[0][1] + 0.6*x[1][1] + 0.2*x[2][1];
    coordinates[48][2] = 0.2*x[0][2] + 0.6*x[1][2] + 0.2*x[2][2];
    coordinates[49][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0];
    coordinates[49][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1];
    coordinates[49][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2];
    coordinates[50][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.4*x[2][0];
    coordinates[50][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.4*x[2][1];
    coordinates[50][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.4*x[2][2];
    coordinates[51][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.6*x[2][0];
    coordinates[51][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.6*x[2][1];
    coordinates[51][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.6*x[2][2];
    coordinates[52][0] = 0.4*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[52][1] = 0.4*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[52][2] = 0.4*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[53][0] = 0.2*x[0][0] + 0.4*x[1][0] + 0.2*x[2][0] + 0.2*x[3][0];
    coordinates[53][1] = 0.2*x[0][1] + 0.4*x[1][1] + 0.2*x[2][1] + 0.2*x[3][1];
    coordinates[53][2] = 0.2*x[0][2] + 0.4*x[1][2] + 0.2*x[2][2] + 0.2*x[3][2];
    coordinates[54][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.4*x[2][0] + 0.2*x[3][0];
    coordinates[54][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.4*x[2][1] + 0.2*x[3][1];
    coordinates[54][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.4*x[2][2] + 0.2*x[3][2];
    coordinates[55][0] = 0.2*x[0][0] + 0.2*x[1][0] + 0.2*x[2][0] + 0.4*x[3][0];
    coordinates[55][1] = 0.2*x[0][1] + 0.2*x[1][1] + 0.2*x[2][1] + 0.4*x[3][1];
    coordinates[55][2] = 0.2*x[0][2] + 0.2*x[1][2] + 0.2*x[2][2] + 0.4*x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new poisson3d_5_1_dof_map_1();
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class poisson3d_5_1_cell_integral_0_quadrature: public ufc::cell_integral
{
public:

  /// Constructor
  poisson3d_5_1_cell_integral_0_quadrature() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_1_cell_integral_0_quadrature()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
    
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
    
    // Compute inverse of Jacobian
    
    // Set scale factor
    const double det = std::abs(detJ);
    
    
    // Array of quadrature weights
    static const double W216[216] = {0.000387381319499512, 0.000665118532009356, 0.000585836191028465, 0.00031770121340093, 9.73756921862106e-05, 1.13421694285829e-05, 0.000726129045448854, 0.00124673509136236, 0.00109812387115231, 0.000595516787239015, 0.00018252640191439, 2.12603918824395e-05, 0.000754254678618672, 0.00129502570039305, 0.00114065822419689, 0.000618583330037891, 0.000189596316905709, 2.20838846030239e-05, 0.00052854808478811, 0.000907496331275901, 0.000799322479379511, 0.000433475646411833, 0.000132860654397154, 1.54754027286732e-05, 0.000235476865600463, 0.000404304542544008, 0.000356111312225707, 0.000193120530504076, 5.91916068934585e-05, 6.89454646290829e-06, 4.6812880715354e-05, 8.03758801295647e-05, 7.07950496032822e-05, 3.83924269380772e-05, 1.17673115182154e-05, 1.37063817429187e-06, 0.000815716960554665, 0.00140055402785094, 0.00123360754139117, 0.000668990101264451, 0.00020504603516934, 2.38834437972721e-05, 0.00152902514424097, 0.00262527620241696, 0.00231234244244452, 0.00125399217565136, 0.00038434966864935, 4.47684526165347e-05, 0.00158824987927105, 0.00272696274959673, 0.00240190792079416, 0.00130256387809358, 0.000399236936769426, 4.65024984913897e-05, 0.00111297477582909, 0.00191093403786039, 0.00168315009155545, 0.000912778750463146, 0.000279767463547727, 3.25868797532674e-05, 0.000495848569406277, 0.000851352545880463, 0.000749871051095567, 0.00040665794717984, 0.000124641006767853, 1.45179909356304e-05, 9.85748637060563e-05, 0.00016924917475603, 0.00014907461918741, 8.08437377690454e-05, 2.47786744026275e-05, 2.88618152005384e-06, 0.00105799885859761, 0.00181654254419696, 0.00160001009401815, 0.000867691610910741, 0.000265948216917809, 3.09772353632468e-05, 0.00198317178090058, 0.00340502816537243, 0.002999147722918, 0.00162644931353069, 0.000498508098270848, 5.80654492426738e-05, 0.00205998727584867, 0.00353691735740022, 0.00311531568122456, 0.00168944764288879, 0.000517817139813846, 6.03145364200509e-05, 0.00144354733248948, 0.00247851415218444, 0.00218307447537104, 0.00118388966129318, 0.000362863188360062, 4.22657407549443e-05, 0.000643124080823797, 0.00110421882265784, 0.0009725955870956, 0.000527442317324585, 0.000161661518972436, 1.88300827146993e-05, 0.000127853285306839, 0.000219519076308906, 0.000193352332454762, 0.000104855711503479, 3.21383647800037e-05, 3.74342682766598e-06, 0.00105799885859761, 0.00181654254419696, 0.00160001009401815, 0.000867691610910741, 0.000265948216917809, 3.09772353632468e-05, 0.00198317178090058, 0.00340502816537242, 0.002999147722918, 0.00162644931353069, 0.000498508098270848, 5.80654492426738e-05, 0.00205998727584867, 0.00353691735740022, 0.00311531568122456, 0.00168944764288879, 0.000517817139813846, 6.03145364200509e-05, 0.00144354733248948, 0.00247851415218444, 0.00218307447537103, 0.00118388966129318, 0.000362863188360062, 4.22657407549443e-05, 0.000643124080823797, 0.00110421882265784, 0.000972595587095599, 0.000527442317324584, 0.000161661518972436, 1.88300827146993e-05, 0.000127853285306839, 0.000219519076308906, 0.000193352332454762, 0.000104855711503479, 3.21383647800037e-05, 3.74342682766598e-06, 0.000815716960554665, 0.00140055402785094, 0.00123360754139117, 0.000668990101264451, 0.00020504603516934, 2.38834437972721e-05, 0.00152902514424097, 0.00262527620241696, 0.00231234244244452, 0.00125399217565136, 0.00038434966864935, 4.47684526165347e-05, 0.00158824987927105, 0.00272696274959673, 0.00240190792079416, 0.00130256387809358, 0.000399236936769426, 4.65024984913897e-05, 0.00111297477582909, 0.00191093403786039, 0.00168315009155545, 0.000912778750463146, 0.000279767463547727, 3.25868797532674e-05, 0.000495848569406277, 0.000851352545880463, 0.000749871051095567, 0.00040665794717984, 0.000124641006767853, 1.45179909356304e-05, 9.85748637060563e-05, 0.00016924917475603, 0.00014907461918741, 8.08437377690454e-05, 2.47786744026275e-05, 2.88618152005384e-06, 0.000387381319499513, 0.000665118532009358, 0.000585836191028466, 0.00031770121340093, 9.73756921862108e-05, 1.1342169428583e-05, 0.000726129045448856, 0.00124673509136237, 0.00109812387115231, 0.000595516787239017, 0.00018252640191439, 2.12603918824395e-05, 0.000754254678618674, 0.00129502570039306, 0.00114065822419689, 0.000618583330037893, 0.000189596316905709, 2.2083884603024e-05, 0.000528548084788111, 0.000907496331275903, 0.000799322479379513, 0.000433475646411835, 0.000132860654397154, 1.54754027286732e-05, 0.000235476865600464, 0.000404304542544009, 0.000356111312225708, 0.000193120530504076, 5.91916068934587e-05, 6.8945464629083e-06, 4.68128807153541e-05, 8.03758801295649e-05, 7.07950496032824e-05, 3.83924269380773e-05, 1.17673115182155e-05, 1.37063817429188e-06};
    // Quadrature points on the UFC reference element: (0.0319263353242805, 0.0285569981572747, 0.0259045550936672), (0.028463310193376, 0.0254594424473144, 0.131563941657985), (0.0228628857485123, 0.0204500572821369, 0.302436918022891), (0.0160915115528755, 0.0143932982315623, 0.509036413164752), (0.00931865528776854, 0.00833521352131827, 0.715681127311714), (0.00370998742847828, 0.00331845490822724, 0.886805616177562), (0.0280201798587476, 0.144242689425179, 0.0259045550936672), (0.0249808524183223, 0.128596795420904, 0.131563941657985), (0.0200656343503386, 0.103294164359602, 0.302436918022891), (0.0141227310723564, 0.0727012004267798, 0.509036413164752), (0.00817852705463396, 0.0421015405270034, 0.715681127311714), (0.00325607414580387, 0.0167616658467636, 0.886805616177562), (0.0218069509935963, 0.328255251806044, 0.0259045550936672), (0.0194415677276443, 0.29264965614937, 0.131563941657985), (0.0156162561103851, 0.235068079131624, 0.302436918022891), (0.010991139455319, 0.165447212249001, 0.509036413164752), (0.00636501048812954, 0.095811107253949, 0.715681127311714), (0.00253406829245939, 0.0381447743739704, 0.886805616177562), (0.0145155449974574, 0.54419938163427, 0.0259045550936672), (0.0129410549532857, 0.485170491669912, 0.131563941657985), (0.0103947804683323, 0.389708626447115, 0.302436918022891), (0.0073161249999536, 0.274287372718776, 0.509036413164752), (0.00423679569770502, 0.15884085642018, 0.715681127311714), (0.00168677328328137, 0.0632384783264913, 0.886805616177562), (0.00759002965380401, 0.749307201071183, 0.0259045550936672), (0.00676674495268004, 0.668030423084586, 0.131563941657985), (0.00543532413100883, 0.536589143558839, 0.302436918022891), (0.00382552675151443, 0.377665816017406, 0.509036413164752), (0.00221537703119812, 0.218707704486042, 0.715681127311714), (0.000881996455633749, 0.0870729530278464, 0.886805616177562), (0.0024027984603795, 0.902933556108202, 0.0259045550936672), (0.00214216875238843, 0.804993045098109, 0.131563941657985), (0.00172067686812083, 0.646603079311139, 0.302436918022891), (0.00121105848171127, 0.455096571592547, 0.509036413164752), (0.000701328553710634, 0.263548148312915, 0.715681127311714), (0.000279216264273074, 0.104925044101914, 0.886805616177562), (0.160169775246924, 0.0285569981572747, 0.0259045550936672), (0.142796282446779, 0.0254594424473144, 0.131563941657985), (0.114699768534046, 0.0204500572821369, 0.302436918022891), (0.080728770233996, 0.0143932982315623, 0.509036413164752), (0.0467503366072312, 0.00833521352131827, 0.715681127311714), (0.0186124666847173, 0.00331845490822724, 0.886805616177562), (0.140573162086061, 0.144242689425179, 0.0259045550936672), (0.125325298900695, 0.128596795420904, 0.131563941657985), (0.100666365601832, 0.103294164359602, 0.302436918022891), (0.0708516852547046, 0.0727012004267798, 0.509036413164752), (0.0410304792857127, 0.0421015405270034, 0.715681127311714), (0.0163352498438522, 0.0167616658467636, 0.886805616177562), (0.109402297632597, 0.328255251806044, 0.0259045550936672), (0.0975355142316141, 0.29264965614937, 0.131563941657985), (0.0783444828851544, 0.235068079131624, 0.302436918022891), (0.055140946130676, 0.165447212249001, 0.509036413164752), (0.0319323307536863, 0.095811107253949, 0.715681127311714), (0.0127130516152568, 0.0381447743739704, 0.886805616177562), (0.0728223755158399, 0.54419938163427, 0.0259045550936672), (0.0649233882395983, 0.485170491669912, 0.131563941657985), (0.0521490999340504, 0.389708626447115, 0.302436918022891), (0.0367039337593433, 0.274287372718776, 0.509036413164752), (0.0212553870582339, 0.15884085642018, 0.715681127311714), (0.00846229593630254, 0.0632384783264913, 0.886805616177562), (0.0380780735220407, 0.749307201071183, 0.0259045550936672), (0.0339477740622412, 0.668030423084586, 0.131563941657985), (0.0272682296798337, 0.536589143558839, 0.302436918022891), (0.0191921106436906, 0.377665816017406, 0.509036413164752), (0.0111142239649518, 0.218707704486042, 0.715681127311714), (0.00442484778263935, 0.0870729530278464, 0.886805616177562), (0.012054489983069, 0.902933556108202, 0.0259045550936672), (0.0107469486906659, 0.804993045098109, 0.131563941657985), (0.00863238528444238, 0.646603079311139, 0.302436918022891), (0.00607570404984925, 0.455096571592547, 0.509036413164752), (0.00351846322733596, 0.263548148312915, 0.715681127311714), (0.00140078733871757, 0.104925044101914, 0.886805616177562), (0.359957416087714, 0.0285569981572747, 0.0259045550936672), (0.32091311096137, 0.0254594424473144, 0.131563941657985), (0.257770432927963, 0.0204500572821369, 0.302436918022891), (0.18142573711282, 0.0143932982315623, 0.509036413164752), (0.105064331522142, 0.00833521352131827, 0.715681127311714), (0.0418287120932833, 0.00331845490822724, 0.886805616177562), (0.315916983199671, 0.144242689425179, 0.0259045550936672), (0.281649710085242, 0.128596795420904, 0.131563941657985), (0.226232476090537, 0.103294164359602, 0.302436918022891), (0.159228478097232, 0.0727012004267798, 0.509036413164752), (0.0922098147528569, 0.0421015405270034, 0.715681127311714), (0.0367110106502643, 0.0167616658467636, 0.886805616177562), (0.245865165941442, 0.328255251806044, 0.0259045550936672), (0.219196359771832, 0.29264965614937, 0.131563941657985), (0.176067410849475, 0.235068079131624, 0.302436918022891), (0.123920961112862, 0.165447212249001, 0.509036413164752), (0.0717630979380167, 0.095811107253949, 0.715681127311714), (0.0285706663630063, 0.0381447743739704, 0.886805616177562), (0.163657307276856, 0.54419938163427, 0.0259045550936672), (0.145905524549545, 0.485170491669912, 0.131563941657985), (0.117197237959671, 0.389708626447115, 0.302436918022891), (0.0824865561302055, 0.274287372718776, 0.509036413164752), (0.0477682770774383, 0.15884085642018, 0.715681127311714), (0.0190177339932274, 0.0632384783264913, 0.886805616177562), (0.0855747280250677, 0.749307201071183, 0.0259045550936672), (0.0762925028429077, 0.668030423084586, 0.131563941657985), (0.0612812341261478, 0.536589143558839, 0.302436918022891), (0.0431313744801241, 0.377665816017406, 0.509036413164752), (0.0249775423239286, 0.218707704486042, 0.715681127311714), (0.00994417811952886, 0.0870729530278464, 0.886805616177562), (0.0270906484064887, 0.902933556108202, 0.0259045550936672), (0.0241521465304899, 0.804993045098109, 0.131563941657985), (0.0193999841535094, 0.646603079311139, 0.302436918022891), (0.0136542286291271, 0.455096571592547, 0.509036413164752), (0.00790721551528067, 0.263548148312915, 0.715681127311714), (0.00314805830348347, 0.104925044101914, 0.886805616177562), (0.585581030661344, 0.0285569981572747, 0.0259045550936672), (0.522063504933331, 0.0254594424473144, 0.131563941657985), (0.419342591767009, 0.0204500572821369, 0.302436918022891), (0.295144551490866, 0.0143932982315623, 0.509036413164752), (0.170919327644826, 0.00833521352131827, 0.715681127311714), (0.0680472168209276, 0.00331845490822724, 0.886805616177562), (0.513935772281482, 0.144242689425179, 0.0259045550936672), (0.45818955283587, 0.128596795420904, 0.131563941657985), (0.368036441526969, 0.103294164359602, 0.302436918022891), (0.259033908311236, 0.0727012004267798, 0.509036413164752), (0.150007517408426, 0.0421015405270034, 0.715681127311714), (0.0597217073254103, 0.0167616658467636, 0.886805616177562), (0.399975027158847, 0.328255251806044, 0.0259045550936672), (0.356590042420813, 0.29264965614937, 0.131563941657985), (0.286427591996011, 0.235068079131624, 0.302436918022891), (0.201595413473385, 0.165447212249001, 0.509036413164752), (0.11674466749632, 0.095811107253949, 0.715681127311714), (0.0464789430854615, 0.0381447743739704, 0.886805616177562), (0.266238755995207, 0.54419938163427, 0.0259045550936672), (0.237360042122558, 0.485170491669912, 0.131563941657985), (0.190657217570324, 0.389708626447115, 0.302436918022891), (0.134189657986267, 0.274287372718776, 0.509036413164752), (0.077709739190668, 0.15884085642018, 0.715681127311714), (0.0309381715027194, 0.0632384783264913, 0.886805616177562), (0.139213515810082, 0.749307201071183, 0.0259045550936672), (0.124113132414521, 0.668030423084586, 0.131563941657985), (0.0996927042921221, 0.536589143558839, 0.302436918022891), (0.0701663963377181, 0.377665816017406, 0.509036413164752), (0.0406336258783158, 0.218707704486042, 0.715681127311714), (0.0161772526750629, 0.0870729530278464, 0.886805616177562), (0.0440712403916416, 0.902933556108202, 0.0259045550936672), (0.0392908667134162, 0.804993045098109, 0.131563941657985), (0.0315600185124607, 0.646603079311139, 0.302436918022891), (0.022212786613574, 0.455096571592547, 0.509036413164752), (0.0128635088600901, 0.263548148312915, 0.715681127311714), (0.00512128141704032, 0.104925044101914, 0.886805616177562), (0.785368671502134, 0.0285569981572747, 0.0259045550936672), (0.700180333447922, 0.0254594424473144, 0.131563941657985), (0.562413256160926, 0.0204500572821369, 0.302436918022891), (0.39584151836969, 0.0143932982315623, 0.509036413164752), (0.229233322559737, 0.00833521352131827, 0.715681127311714), (0.0912634622294936, 0.00331845490822724, 0.886805616177562), (0.689279593395093, 0.144242689425179, 0.0259045550936672), (0.614513964020416, 0.128596795420904, 0.131563941657985), (0.493602552015675, 0.103294164359602, 0.302436918022891), (0.347410701153764, 0.0727012004267798, 0.509036413164752), (0.20118685287557, 0.0421015405270034, 0.715681127311714), (0.0800974681318223, 0.0167616658467636, 0.886805616177562), (0.536437895467693, 0.328255251806044, 0.0259045550936672), (0.47825088796103, 0.29264965614937, 0.131563941657985), (0.384150519960331, 0.235068079131624, 0.302436918022891), (0.270375428455571, 0.165447212249001, 0.509036413164752), (0.156575434680651, 0.095811107253949, 0.715681127311714), (0.0623365578332109, 0.0381447743739704, 0.886805616177562), (0.357073687756223, 0.54419938163427, 0.0259045550936672), (0.318342178432505, 0.485170491669912, 0.131563941657985), (0.255705355595944, 0.389708626447115, 0.302436918022891), (0.179972280357129, 0.274287372718776, 0.509036413164752), (0.104222629209872, 0.15884085642018, 0.715681127311714), (0.0414936095596443, 0.0632384783264913, 0.886805616177562), (0.18671017031311, 0.749307201071183, 0.0259045550936672), (0.166457861195188, 0.668030423084586, 0.131563941657985), (0.133705708738436, 0.536589143558839, 0.302436918022891), (0.0941056601741515, 0.377665816017406, 0.509036413164752), (0.0544969442372926, 0.218707704486042, 0.715681127311714), (0.0216965830119524, 0.0870729530278464, 0.886805616177562), (0.0591073988150613, 0.902933556108202, 0.0259045550936672), (0.0526960645532402, 0.804993045098109, 0.131563941657985), (0.0423276173815277, 0.646603079311139, 0.302436918022891), (0.0297913111928518, 0.455096571592547, 0.509036413164752), (0.0172522611480347, 0.263548148312915, 0.715681127311714), (0.00686855238180623, 0.104925044101914, 0.886805616177562), (0.913612111424777, 0.0285569981572747, 0.0259045550936672), (0.814513305701324, 0.0254594424473144, 0.131563941657985), (0.65425013894646, 0.0204500572821369, 0.302436918022891), (0.46047877705081, 0.0143932982315623, 0.509036413164752), (0.266665003879199, 0.00833521352131827, 0.715681127311714), (0.106165941485733, 0.00331845490822724, 0.886805616177562), (0.801832575622406, 0.144242689425179, 0.0259045550936672), (0.714858410502789, 0.128596795420904, 0.131563941657985), (0.574203283267168, 0.103294164359602, 0.302436918022891), (0.404139655336112, 0.0727012004267798, 0.509036413164752), (0.234038805106649, 0.0421015405270034, 0.715681127311714), (0.0931766438298707, 0.0167616658467636, 0.886805616177562), (0.624033242106693, 0.328255251806044, 0.0259045550936672), (0.556344834465, 0.29264965614937, 0.131563941657985), (0.4468787467351, 0.235068079131624, 0.302436918022891), (0.314525235130928, 0.165447212249001, 0.509036413164752), (0.182142754946207, 0.095811107253949, 0.715681127311714), (0.0725155411560083, 0.0381447743739704, 0.886805616177562), (0.415380518274606, 0.54419938163427, 0.0259045550936672), (0.370324511718817, 0.485170491669912, 0.131563941657985), (0.297459675061662, 0.389708626447115, 0.302436918022891), (0.209360089116518, 0.274287372718776, 0.509036413164752), (0.121241220570401, 0.15884085642018, 0.715681127311714), (0.0482691322126655, 0.0632384783264913, 0.886805616177562), (0.217198214181346, 0.749307201071183, 0.0259045550936672), (0.193638890304749, 0.668030423084586, 0.131563941657985), (0.155538614287261, 0.536589143558839, 0.302436918022891), (0.109472244066328, 0.377665816017406, 0.509036413164752), (0.0633957911710462, 0.218707704486042, 0.715681127311714), (0.025239434338958, 0.0870729530278464, 0.886805616177562), (0.0687590903377507, 0.902933556108202, 0.0259045550936672), (0.0613008444915176, 0.804993045098109, 0.131563941657985), (0.0492393257978493, 0.646603079311139, 0.302436918022891), (0.0346559567609898, 0.455096571592547, 0.509036413164752), (0.0200693958216601, 0.263548148312915, 0.715681127311714), (0.00799012345625072, 0.104925044101914, 0.886805616177562)
    
    // Value of basis functions at quadrature points.
    static const double FE0[216][56] = \
    {{0.310703073843137, 0.0224419087355454, 0.0208770933958504, 0.0195251039488212, -0.00350519620228991, -0.00213578148775574, -0.00215103292273904, -0.00360148326124408, -0.00378445250751075, -0.00231961445844539, -0.00235756395016157, -0.00402640927558309, -0.00417195365419327, -0.00251816728181517, -0.00254121852244879, -0.00432001510649592, 0.354215091736513, -0.393270742690671, 0.286443388274845, -0.115220749339439, 0.390484209646641, -0.426933671469902, 0.308757369600584, -0.123622648285321, 0.436556032512803, -0.467925628811811, 0.33533302108994, -0.13347157028838, 0.000760975979021677, 0.000531676205177356, 0.000783336703543941, 0.000539901917789621, 0.000550725217214193, 0.000801136210248132, 0.129016952515557, -0.0645985164753274, 0.0224161618429517, -0.0655979383537516, 0.0157596925360572, 0.0229255170425749, 0.144239197195841, -0.0708009310625633, 0.0243455865742153, -0.0733376023953636, 0.0172728564941386, 0.025630416073592, 0.159008269921432, -0.078050445204567, 0.0268384022973471, -0.0796151059527957, 0.0187513670337174, 0.0276270291882513, 0.0240592369041033, -0.0056665575752356, -0.00578015385508157, -0.00586958024656276},
    {0.0168212127674999, 0.020831167572953, 0.019287579585398, 0.0197079603842469, -0.0163810014243473, -0.00390242657177244, -0.00279689872237231, -0.00375319132544435, -0.0177608566866899, -0.00425338581439347, -0.00307308040887677, -0.00419601682685231, -0.00343697142949154, -0.00209922099594013, -0.00211619316158232, -0.0035439613540267, 0.762426252400217, -0.486472830393038, 0.315036567668793, -0.120074282056387, 0.147540025395148, -0.240094368128956, 0.21694135572351, -0.101414897218028, 0.164947740605516, -0.263802651514285, 0.236451671291947, -0.109957591036382, 0.00316467178184315, 0.00223004992142851, 0.00324612435929954, 0.000874386037654523, 0.000889697864058403, 0.000912212004253246, 0.361953254691466, -0.228613371439197, 0.0928919182149795, -0.0896376076996035, 0.0254598197962893, 0.0261040901440183, 0.404658813134062, -0.251187955916101, 0.101246022171265, -0.100213625582169, 0.0279738671992902, 0.0291840175456563, 0.0783070774100657, -0.0486083437947556, 0.0195925056820643, -0.0494595496579393, 0.0138062550459612, 0.0200967791728532, 0.0745625360302081, -0.0208135617171249, -0.0211780387674468, -0.00830375195874419},
    {-0.015558863517762, 0.0178396083059577, 0.0163965439250797, -0.0116539403654932, -0.0318063021955915, 0.0112436886986536, 0.00289018334287365, 0.00239490987846146, -0.0347130258988212, 0.0123225323775474, 0.00318776431988747, 0.00267747665318658, -0.0023472113547114, -0.00146045594323523, -0.00146979957328538, -0.00240441497001839, 0.161426388205, 0.609621358264414, -0.233929227326291, 0.0766193511896089, 0.0109152642713772, -0.0722517992667382, 0.107859002109445, -0.0688053487876893, 0.0122031169159266, -0.0796910370126102, 0.118208186061103, -0.075093352246887, 0.00492003269613239, 0.00351354277547211, 0.00501895556935264, -0.00200454808179203, -0.00203185252724611, -0.000736039572724612, 0.243404271380666, -0.25783598289135, 0.143623705893241, 0.147100706594461, -0.0581440074053685, -0.0210626951480254, 0.272122663056644, -0.284383462616915, 0.157404550542626, 0.164456588164782, -0.0641306693151905, -0.0235478065455436, 0.0184002802424233, -0.0192293260314474, 0.010643317277893, -0.0194912534396812, 0.00760071179165113, 0.0108573133203486, 0.0656627847324155, -0.0256055314109388, -0.0259543106904588, 0.0148074655791842},
    {0.00894872042728936, 0.0135427001430301, 0.0123418011338163, 0.0118223838397412, -0.0399845505836983, 0.0422013001459018, -0.0119330872043856, -0.00292413044299179, -0.0439702312640462, 0.0465429183672366, -0.0132189650130184, -0.00326913804247667, -0.00124328365422732, -0.000790403917700966, -0.000793900257330511, -0.0012639800238532, -0.0670830605102294, 0.297174981210493, 0.535772970512814, -0.0935504836114047, -0.00189681223432868, -0.00504670264847047, 0.0321772770214551, -0.0361703730372398, -0.0021206102653668, -0.00559052193272506, 0.0354876359854277, -0.0397758545272402, 0.00433545094422108, 0.00314406469764678, 0.00439483944231581, -0.00523488897578106, -0.00528322800716191, 0.00206912152181218, 0.0276817096582159, -0.127430920717604, 0.125763840462251, 0.212173344440878, -0.151186193020126, 0.0592105064091016, 0.0309477746936238, -0.141162538552777, 0.13870227078189, 0.237206911741454, -0.167477615955912, 0.0661965404040856, 0.000875066182395603, -0.0039914522104273, 0.00392188671994384, -0.00402830933090287, 0.00284414833496883, 0.00397561007306036, 0.0220957122063313, -0.0156004611164018, -0.0157445159727609, 0.0262147255291869},
    {-0.0109738205691141, 0.00844321113047129, 0.00763236993753088, -0.0295941247801588, -0.0344990911194155, 0.0601371945948342, -0.0484704728084982, 0.0146274102792546, -0.0382116443014455, 0.0667196416010177, -0.0539112807906321, 0.0163532456363014, -0.000445033691819755, -0.000288809823713002, -0.000289536829614523, -0.000449201642483951, 0.0736287355315533, -0.227812559637078, 0.539364215954558, 0.467968625984661, 0.000857520491375045, 0.000986133972089243, 0.00289671851242861, -0.0128544681648156, 0.000958696239860321, 0.00109682745775005, 0.003213785117021, -0.0142377769749927, 0.0021568462732364, 0.00158719434401838, 0.00217342755715769, -0.00427040450999733, -0.00429242918242179, 0.00471322495942957, -0.00736449804673683, -0.0158793451087479, 0.0621953543792267, 0.0427239595603105, -0.122833242483405, 0.134874841195124, -0.00823340859708807, -0.0176618007484954, 0.0690030817265088, 0.0477648053829494, -0.136621267387051, 0.150788237023003, -9.58907759980449e-05, -0.000205698983516037, 0.000803647599287693, -0.000206759878969849, 0.000591393526743575, 0.000809825837014444, 0.00154409316372173, -0.00441655656921419, -0.00443933502297741, 0.011944193459961},
    {0.0261188035535262, 0.0035684642286854, 0.00320507108782466, 0.192230415140459, -0.0178382131382485, 0.0410650708630083, -0.0503948504088267, 0.0367434731025106, -0.0198704873172242, 0.0457735732543992, -0.0562285998280996, 0.0410787029080853, -7.43559974856201e-05, -4.90514755035476e-05, -4.90999381122584e-05, -7.46268914880513e-05, -0.166915283235763, 0.464278848231276, -0.769187724303446, 1.17551858417977, -0.000624602315104125, -0.000497526431336839, -0.00067167010694609, -0.00213554205983519, -0.00069829688843745, -0.000555120501094609, -0.000748683496630237, -0.00237884036290422, 0.000442330508822864, 0.000329297655088434, 0.000443650701341725, -0.00114989679790878, -0.00115219045506196, 0.00190118764207903, 0.00448653414127226, 0.00450467759663876, 0.0126956210248066, -0.0157301889762824, -0.0329713743754397, 0.0544048679014514, 0.00501588411531277, 0.00502614278802677, 0.0141512951707229, -0.0175861372125055, -0.0367881678484819, 0.0608239019496676, 1.87695983853431e-05, 1.88079866661889e-05, 5.29545980094692e-05, 1.88455022702011e-05, 3.94226140922604e-05, 5.31126478447164e-05, -0.000169942708606369, -0.000355500518009422, -0.000356209621910847, 0.00124387251864877},
    {0.00186778322923252, 0.0206121464512043, 0.0160091858217584, 0.0195251039488212, -0.00316207558373215, -0.00241577235429526, -0.00353354291864167, -0.0181912548602126, -0.00345830596482601, -0.00210523271754555, -0.0021172061694168, -0.00353378209371138, -0.0192566655330663, -0.00375432783799054, -0.0025813141973566, -0.00342032226620971, 0.132011178619608, -0.227738706732717, 0.212018681376458, -0.101123604922342, 0.735069464402225, -0.406133611428703, 0.258494822208368, -0.0978768097135354, 0.142792530306415, -0.243344599888119, 0.225266158072228, -0.107046130267053, 0.00348848218525216, 0.000784350551183088, 0.000777623744280614, 0.00244903953872555, 0.000793999307933946, 0.00355148076900435, 0.377375106660751, -0.0785454775264237, 0.0222526783512815, -0.245248734459552, 0.0227212856352968, 0.101630074704723, 0.0733078286669996, -0.0470623884943937, 0.0193921693138193, -0.0476413305733007, 0.0136139979708022, 0.0197423729662323, 0.408195327988216, -0.262054509821074, 0.107980185164327, -0.0849602859197391, 0.0242782715384176, 0.0240700543779731, 0.0789442096360897, -0.0225591161468237, -0.00731385603414876, -0.0228366290826982},
    {-0.0295118525675891, 0.0190287287217082, 0.0205403356838656, 0.0197079603842469, -0.0201247738104083, -0.00584323645334017, -0.00577935460978638, -0.0189575391547738, -0.0161511787365525, -0.00384447375345874, -0.00275184717215821, -0.00368263832928284, -0.0157869230855841, -0.00392069343601914, -0.00283742515434127, -0.00382121421853439, 0.228014045330252, -0.271715104548501, 0.231653750460453, -0.105383312724627, 0.222871671150234, -0.277102067847941, 0.238857806241638, -0.109348835528473, 0.043294425082725, -0.131942507994804, 0.157153073341714, -0.0877581335269879, 0.0144481437637404, 0.00412636436673866, 0.0042658513953787, 0.00395489410044242, 0.00161349267462764, 0.00404388585725964, 1.02114857456486, -0.347362233990544, 0.122072685786528, -0.332927664119839, 0.0461721157239614, 0.115720863634553, 0.198365455024692, -0.165396976974349, 0.0803159756186612, -0.0646735932737137, 0.0219849126185114, 0.0224796100611513, 0.193891742045086, -0.161666798236395, 0.0785046187846286, -0.0659557975662844, 0.0224207806119896, 0.0231786894608932, 0.243053648758977, -0.0826227979472469, -0.0337079263968614, -0.0323072000911185},
    {0.00567780449458319, 0.0161577642134766, 0.0267098865309273, -0.0116539403654932, -0.0579732173172456, 0.0239119304978302, 0.00786274267358259, 0.0120967981262488, -0.0313276643913105, 0.0110671468330951, 0.00284192495165232, 0.00234989003992667, -0.0106996359300077, -0.00356839426992839, -0.00278670080802812, -0.00384632070848081, -0.0380248596464324, 0.301987998765445, -0.169129303002115, 0.0672450495550815, -0.0129869928834949, -0.0973703756382163, 0.165842790839071, -0.110067289214354, -0.00252281677407701, -0.0351937372957312, 0.0767569359401079, -0.0594783463215323, 0.0223195251484257, 0.00852158369598076, 0.00936787367759256, -0.00902659870223738, -0.00485135103202024, -0.00326290380377039, 0.609030388285103, -0.507138483772255, 0.268073449809222, 0.537193055045898, -0.138827491931281, -0.0933720830549344, 0.118308533258758, -0.183301116519788, 0.124072300702037, 0.104353614600787, -0.0501780777616423, -0.0181381986942381, 0.0404070414401215, -0.0626045781079558, 0.0423755958927613, -0.0336468690624237, 0.0161789816165827, 0.0177857381237385, 0.210453801737447, -0.101195988938319, -0.0543878465817404, 0.0576110360335692},
    {0.00068957310734765, 0.0121457494920626, 0.0302516266319403, 0.0118223838397412, -0.105865563934372, 0.12409022809239, -0.041339453116336, -0.0147699151361855, -0.0393358416198599, 0.041497453639969, -0.0117258358154837, -0.00286916224498854, -0.00561799280287631, -0.00244134318822822, -0.0020707327412002, -0.00293713936881876, -0.00443360677459508, 0.013991049973439, 0.368517130449206, -0.082104674715699, -0.0006332131187422, -0.000823109650176566, 0.0650785581277397, -0.084049925337174, -0.000123006202579788, -0.000233473059959181, 0.0217631516226339, -0.0312299337797291, 0.0195246551264626, 0.00966161533660757, 0.0113416597229395, -0.0234549811860987, -0.0160632387501084, 0.00917252921469192, 0.00658282380308282, -0.303643238348031, 0.324555812036961, 0.7371382729091, -0.459669715353957, 0.262483423099521, 0.00127876086944764, -0.0861276695976373, 0.108535861128071, 0.143194411238384, -0.130384202142739, 0.0509892927988105, 0.000182634184634378, -0.012300858657203, 0.0155012238600134, -0.00842429281329383, 0.00767065339756552, 0.00900449227761779, 0.0673733265593303, -0.0613460913255654, -0.0420131188138637, 0.10199297705579},
    {-0.00713114387718264, 0.00750148626092429, 0.0261981528295216, -0.0295941247801588, -0.123694165861968, 0.228667944176925, -0.201694509957147, 0.0738837109693798, -0.033900773893903, 0.0590785907491439, -0.0475981981984883, 0.0143524422502372, -0.00199428872932866, -0.00106415577163227, -0.000972023581156795, -0.00141352907516575, 0.0450881185414648, -0.127068618871478, 0.241702073048267, 0.4107131286806, 0.00265240926107728, 0.00228882920954538, 0.00493590353226529, -0.0404499066305402, 0.000515249575909579, 0.000540144332047434, 0.00127523875639663, -0.011086077739848, 0.00964665779284895, 0.00583119804449707, 0.00725319143528796, -0.019044129950352, -0.0156762365286426, 0.0208939848046807, -0.0207484211610253, -0.0296106304241312, 0.207559166263658, 0.0967054607656748, -0.448595161601129, 0.597907570023056, -0.00403053003957324, -0.00698785830119739, 0.0536249323623455, 0.0187857312892299, -0.105864663432386, 0.116147845812616, -0.000237104930297521, -0.000411076369401915, 0.00315460639795229, -0.000338378829323367, 0.00190689200901604, 0.00237190585575591, 0.00306743134590159, -0.0172861296712997, -0.0142291329716045, 0.0464709748021344},
    {0.02849036321081, 0.0031468891813878, 0.014002682081672, 0.192230415140459, -0.0792712548026677, 0.186695351837455, -0.237148368754727, 0.185592944676423, -0.0175130233754289, 0.0403122421171736, -0.0494632043101821, 0.0360527643450903, -0.000331016674263732, -0.000203286338883911, -0.000196366252368532, -0.00029105936922343, -0.178725093630068, 0.48438729810939, -0.768528380851087, 1.03169505116885, -0.00337811380894421, -0.00244266543518652, -0.00305101620671766, -0.00832902875213198, -0.000656222903823837, -0.000509478771102155, -0.000658791463301694, -0.00184009292641842, 0.00196766105572205, 0.00136316239720277, 0.00177020657356562, -0.00510934349238279, -0.00475860988386897, 0.00842806911326788, 0.0236431905624579, 0.0211799660893323, 0.050656680414662, -0.0793857886114539, -0.136173587707137, 0.241179763965931, 0.00459285981587519, 0.00441760993533497, 0.0109380568162515, -0.0154212604048491, -0.0284023964650369, 0.0468509037896236, 8.68104352411136e-05, 8.34980070342009e-05, 0.000206742097729723, 7.77662418171773e-05, 0.000143227438853893, 0.000185995560246247, -0.000752719569419495, -0.00138633542760996, -0.00129116969293919, 0.00483950370236119},
    {-0.00652877742132652, 0.0172198053622791, -0.0100835618309505, 0.0195251039488212, 0.00276855422334095, 0.00354739278284653, 0.0184970121822559, -0.041398111534134, -0.00286607158261651, -0.00172596335485683, -0.00170726307853504, -0.00275019694120751, -0.0363180547067536, 0.0161122007935753, 0.00305654953686335, 0.00233062201042271, 0.00480559727387782, -0.069622949057169, 0.116257872529598, -0.0787003334009246, 0.0608951799214983, 0.649943657200637, -0.208138951111156, 0.0666936707344562, 0.00404544389461567, -0.0599894079180595, 0.10126879776982, -0.0690428357230213, 0.00650857391428433, -0.00331084397955435, -0.000888682878661127, 0.00449418520894312, -0.00323471010148813, 0.00629001046750739, 0.262546773541793, 0.225455398280341, -0.0254307747166693, -0.30603626219929, -0.0925652849176253, 0.179996535328407, 0.0174417456922686, -0.0208093974068782, 0.012373195733733, -0.0203308788978238, 0.00854372002011952, 0.011957693299125, 0.221016906233311, -0.26369084359981, 0.156789663691808, 0.189792675603336, -0.0797572741627937, -0.0214081135912332, 0.0766673450374758, -0.0322182004023549, 0.0231891952485369, -0.0314773329510054},
    {0.00858574731449889, 0.0157648753004524, -0.0118196797021562, 0.0197079603842469, 0.0153251463175155, 0.0068247368416488, 0.0170656692030986, -0.0431419561966856, -0.0132859312546022, -0.00313228338759675, -0.00220943154814895, -0.00286604561350592, -0.0295531067577097, 0.00943259119854763, 0.00266080533945908, 0.00226463927967869, -0.0231797827124803, -0.0726756131892215, 0.124780298683106, -0.0820154883984918, -0.0515609015277078, 0.218856107347192, -0.150272175335546, 0.0648054921715142, -0.00342534063541736, -0.0283345225038208, 0.0689689653012089, -0.0561822573222245, 0.0267888501632131, -0.00978290406589548, -0.00387759279960774, 0.00722618325024973, -0.00370795971687465, 0.00716210674532233, 0.621558100379749, 0.552501249632886, -0.110962179307842, -0.408107372713323, -0.106107915976052, 0.204952663667211, 0.0412919121935717, -0.071530373453446, 0.050927237043988, -0.0271117596075609, 0.013737414822459, 0.0136156014808827, 0.0918493604928062, -0.159111523503952, 0.113282090987398, 0.0816446385611459, -0.0413689957486714, -0.0163971882951673, 0.231873293669398, -0.117489224878567, 0.0602870558259816, -0.0445312874461567},
    {0.00728244989005014, 0.0132117409898728, -0.00729939888431999, -0.0116539403654932, 0.0195387056426204, -0.0109690858690787, -0.00648860340814748, 0.027528864934527, -0.0254704298617906, 0.00892944871598925, 0.00226644479881711, 0.00182882255572043, -0.019796806095659, 0.00237595918769125, 0.00101947993108492, 0.00100887627534109, -0.0311859124881201, 0.0417262619381511, -0.0868398367617211, 0.0523340502310095, -0.0242391457778128, 0.0111025773900735, -0.0390618253060839, 0.0288702594494082, -0.00161027694513706, -0.00387808858029436, 0.0317985080969107, -0.0376349350795188, 0.0409818725101211, -0.0056082843161018, -0.00334442030343123, -0.0163822876805217, 0.00311575927800615, -0.00577891319567472, 0.191503462471457, 0.214883898684752, -0.0957047798901592, 0.627694611711972, 0.0891613579747631, -0.165370846131076, 0.0127221319331293, -0.0750581386916097, 0.0779090376449027, 0.041699578487278, -0.0311437274442792, -0.010986066232021, 0.00988823433174058, -0.0583386862978687, 0.060554537937708, 0.01109547742318, -0.00828676301457337, -0.00494169284465333, 0.19138099827276, -0.142934721751187, 0.0271848592901337, 0.0794093452371633},
    {-0.0111121414793535, 0.00978040546374068, 0.00962857943557122, 0.0118223838397412, -0.0386200930353358, 0.0549301045311348, -0.0255353246076025, -0.0336121174078451, -0.0315495982714478, 0.0331063769654701, -0.00927948168620291, -0.00223295070854476, -0.0102542626550717, -0.00120308329737539, -0.000725398750303436, -0.000833886962407522, 0.0582572468288328, -0.126131012729572, 0.160899807538944, -0.0638986839804832, 0.0189347929379127, 0.0045835917006036, 0.0125779136442869, -0.0238627208753058, 0.00125789336011876, 0.00166566730191947, 0.00758070923223652, -0.0194939783441421, 0.0354479766349184, 0.00472287345005595, 0.0039072705576289, -0.0422408716007053, -0.00772207115915256, 0.0162454222693436, -0.135052204499447, -0.0818913657941363, 0.111811445560696, 0.732427557989167, -0.22097674739764, 0.464883125160772, -0.00897191069721803, -0.029759145059726, 0.0673887642738489, 0.0486572926877846, -0.080302471656953, 0.0308835379551549, -0.00291605389125923, -0.00967232885843016, 0.0219027222761859, -0.00176820242786563, 0.00291818589781653, 0.00241423827269643, 0.0520988018747552, -0.0859822306134803, -0.0157184470409668, 0.140584097846636},
    {0.00476412582200832, 0.00595231255869341, 0.0280757815135756, -0.0295941247801588, -0.142669865790111, 0.291843625603676, -0.302869456861528, 0.168138269227149, -0.0268382683375482, 0.0466279404547274, -0.037393933972244, 0.01116991419638, -0.00359294678603303, -0.00126119637132608, -0.000974613380291073, -0.00126885446246872, -0.0275920932079784, 0.0681032668396037, -0.0986835027874708, 0.319641098476688, -0.00369386434939364, -0.00184206276700017, -0.0025720284552583, -0.0363098612093177, -0.000245394151048781, -0.000227431231249626, -0.000410933730046616, -0.00683041084421027, 0.017326500301564, 0.00687907171497615, 0.00720440458057823, -0.0340478467842393, -0.0183200746196732, 0.0370052357530363, 0.0253065278441707, 0.0181540378518065, 0.206163069251945, -0.0898530971770411, -0.524251903161903, 1.05895121462285, 0.00168118623991392, 0.00224139766285651, 0.0329387331902803, -0.00596920255192246, -0.0647270320843951, 0.0703492087783398, 0.000225067154909109, 0.000300064908350173, 0.00440964052015409, 0.000161455481944413, 0.00175074209144622, 0.00183354017309741, -0.00221809903691528, -0.0240519510403532, -0.0129415977638843, 0.0640542148802874},
    {0.030253888096142, 0.00246776876963701, 0.0249036082205118, 0.192230415140459, -0.14494000231401, 0.354346112869683, -0.476703569824698, 0.422356647889497, -0.0137211579663085, 0.0315457931627576, -0.0386364416503142, 0.0280583803965706, -0.000590197519305787, -0.000319772383935504, -0.000291122581603, -0.000414169528781396, -0.184397876468037, 0.480186648330888, -0.713797405978017, 0.802925731904298, -0.0079316315375073, -0.00486754060925036, -0.00537840791475002, -0.0118519803113753, -0.00052692134942571, -0.000394510258857454, -0.000478814744853832, -0.00112200145901491, 0.00350406908242337, 0.00214037926285277, 0.00261482209238985, -0.00908234190642532, -0.00744444616732409, 0.0149269125729192, 0.0533385602171293, 0.0395429056190284, 0.0748264123822478, -0.167793715365594, -0.213032160197368, 0.427152317178935, 0.00354343567185668, 0.00320492075650963, 0.00666144891220991, -0.0111470244813417, -0.0172660855678773, 0.028376970654002, 0.000152416213593867, 0.000137855440824324, 0.000286533442193787, 0.00011299480009257, 0.000175022302174668, 0.000213818265916807, -0.00123819818524013, -0.0019178961930274, -0.00157202570775301, 0.00667062852425415},
    {0.00254449201900164, 0.0124299526187294, 0.0100401662714875, 0.0195251039488212, -0.00508376488671095, -0.0317222402070974, 0.0822967853898158, -0.0686320373360359, -0.00204964204175445, -0.00121893362021503, -0.00118292063155351, -0.00183063682142863, -0.0430586021516741, 0.050627285175583, -0.0189383076837505, -0.0028486734361088, -0.000856872010174092, -0.00161605386869118, 0.0393067632837015, -0.0523859677188016, -0.0180010510271378, 0.0671213089149897, 0.62929291895221, -0.0815183617627335, -0.000480145834420655, -0.000964790796582488, 0.0241807101537084, -0.0328660874700561, 0.00762045121549466, -0.0102064605800903, 0.00528980646672308, 0.00516240990083429, -0.00957976934285645, 0.00694122508762249, 0.010103142175862, 0.339146109455536, 0.151374443887892, -0.171539508681194, -0.274137109924834, 0.198631858112276, 0.000269483244227353, -0.00487483111382478, 0.00581659421565783, -0.00457550953039673, 0.00394040230937928, 0.00529814948649655, 0.00566126746200279, -0.102409791178523, 0.1221942227509, 0.190039573917173, -0.163660980479209, 0.084822246271929, 0.0286048777260461, -0.02463435503793, 0.0457134252619624, -0.0231217705022783},
    {-0.00481040329364101, 0.0112741294025221, 0.0110939404354585, 0.0197079603842469, -0.0231802053744836, -0.0276298646655858, 0.0870824133797188, -0.0715230777133198, -0.00942285779415132, -0.00219692843858377, -0.0015236304910657, -0.00190775015176707, -0.0347488262460901, 0.0337592671031061, -0.00742856718436129, -0.00228007999607879, 0.00736457670424279, 0.00438882359425972, 0.0397000546457264, -0.0545926623435964, 0.0271584695282763, -0.0674411898835215, 0.19356039727999, -0.0652473476293611, 0.000724403591411443, 0.00117997939276469, 0.0153905329075907, -0.0265233404201762, 0.0311498025027812, -0.0344250066126233, 0.0104494503526857, 0.00826141494223322, -0.0125944886537098, 0.00790361085687716, -0.0622282391086319, 0.896985621982871, 0.299024122342194, -0.215261263528709, -0.360407082565477, 0.226171733444769, -0.00165982695934236, -0.0156940372993719, 0.0237762510293563, -0.0057417091279616, 0.00630583373704124, 0.00603272639544607, -0.0061209709271592, -0.057875157105101, 0.0876800683880994, 0.088230407816175, -0.0968990713114823, 0.0294129801131075, 0.0814105285467182, -0.0894091368997129, 0.136303813341331, -0.0327105924159354},
    {-0.0117698440494143, 0.00930958844834866, -0.00170955493305554, -0.0116539403654932, 0.00630079692715634, -0.00613847546289087, -0.0581932607657199, 0.0456388471838216, -0.0178289300066307, 0.00619513050775155, 0.00155135505195033, 0.00121733460618681, -0.0229736761944516, 0.0147838058786251, 0.000390512398712469, 0.000216558881969404, 0.0354164151936631, -0.0239832387341913, -0.0228191073912387, 0.0348355559312484, 0.0456362358439275, -0.0572326192875783, -0.00574410374506546, 0.00619710390792822, 0.00121726495336645, 0.00152574562586519, 0.00579711893708314, -0.0175355170299285, 0.047137220592618, -0.0344284557477548, -0.00124580290150357, -0.0185903243576045, 0.0186004631507439, -0.00637721311597525, -0.182482550339248, 0.506413169593175, -0.0356502118925021, 0.273447788383992, 0.532275572502242, -0.18249194844047, -0.00486739559083218, -0.0135003025897686, 0.0359792454393946, 0.00729373058973905, -0.0141897599063139, -0.00486764626837001, -0.00627193949230204, -0.017395972731348, 0.046361477336999, 0.0174054601488664, -0.033861862257262, -0.00122530056414615, 0.0554961977510376, -0.107966384569833, 0.108025267288504, 0.058330375675975},
    {-0.00224468746929321, 0.00677269421349494, -0.011422245664625, 0.0118223838397412, 0.0552995939385817, -0.104936601954282, 0.0910163993406658, -0.0557239934719332, -0.0217460457985154, 0.0226775867458487, -0.00629687867371578, -0.00148633784230101, -0.0117175620738285, 0.00293737332109392, 0.000940361805028107, 0.000794793321731137, 0.00967279417619873, -0.0153043533659061, 0.00875331817503941, -0.0425334207826389, 0.00521205405512003, -0.00198233611792224, -0.00130720099680894, 0.0227440073355649, 0.00013902221817863, 0.00013714594419802, 0.000282495939903825, -0.00894386721374991, 0.0402552930537988, -0.0114231579753208, -0.0049685361611975, -0.0475204821291955, 0.0183210544613898, 0.0179273362417272, -0.027166966073064, 0.0158793811192679, -0.142180891317948, 0.0660584269543603, 0.524279942561051, 0.513013202099966, -0.00072463022154462, -0.00109859912110419, 0.0307263570233541, 0.0017619903683786, -0.0362717841321437, 0.013683709446727, -0.000390457174614223, -0.000591965248076018, 0.01655648107521, 0.000228226378675297, -0.00469819704416935, -0.00204349462353736, 0.00312773309752856, -0.064386537958712, 0.0248235963876546, 0.103266475966667},
    {0.0225257270787522, 0.00405256796561407, 0.0116134748343783, -0.0295941247801588, -0.0648165501864932, 0.151531129548398, -0.19835664154622, 0.278748753109168, -0.018223677561725, 0.0315481269099229, -0.0251644118483799, 0.00743512434099136, -0.00404462887250183, -0.000558857812259082, -0.00034054118705874, -0.000383710664554989, -0.118755279768058, 0.25582760745598, -0.289712660115728, 0.212765404450057, -0.0263569760650688, -0.00453184613547787, -0.0039205801342066, -0.0109803617251882, -0.000703025187056512, -0.000574930295742444, -0.000816247856154013, -0.00308721416081524, 0.0194350005513992, 0.003031843168363, 0.00248993575190867, -0.0379858317474886, -0.00798650751792702, 0.0408364456798985, 0.157600788983966, 0.0349049822683103, 0.0712526331790854, -0.43732301110713, -0.228543925328445, 1.16858608987677, 0.00420371911717604, 0.0044282023657649, 0.0148344905822272, -0.0116648090026962, -0.0289941058569006, 0.0311699434866482, 0.00093298861635522, 0.00098281124000304, 0.00329242046313162, 0.00020663571115582, 0.000513614726283404, 0.00042181194694528, -0.00718602312101305, -0.0178616139374966, -0.00375539793211324, 0.0470512180454073},
    {0.0278235382849624, 0.00165731029056243, 0.0299947250976383, 0.192230415140459, -0.180516285730209, 0.461954122435896, -0.667777779660335, 0.700205786033841, -0.00920510912404789, 0.0211331468589337, -0.0258282524616005, 0.018676736758009, -0.00065642017057042, -0.000300087035939547, -0.000253714313457498, -0.000343356021220723, -0.164115051478879, 0.408587395180542, -0.565496681950333, 0.534458237398515, -0.0117031127641393, -0.00580187045310396, -0.00555494811880203, -0.00982556301346881, -0.00031215959751384, -0.000224404254493859, -0.000254123794305346, -0.000501037229845007, 0.00389172137830533, 0.00200434369960645, 0.00226908998258699, -0.0100656565735361, -0.0069415163699908, 0.0164723191745265, 0.075242381617107, 0.0438841037852106, 0.0649328546150946, -0.220382521134524, -0.198640193522426, 0.471376064563758, 0.00200695593001038, 0.00169734565320812, 0.00297050180073005, -0.00587830951327907, -0.00768298859828448, 0.01257311328681, 0.000143116864357681, 0.000121038426398017, 0.000211827722239877, 8.34709799705236e-05, 0.000109097111329684, 0.0001235073418277, -0.00108250526785379, -0.0014148413947441, -0.000975708303854961, 0.0048999304383196},
    {-0.00396719586028167, 0.00700570982272352, -0.02833860959432, 0.0195251039488212, 0.0724066424646239, -0.168855440911288, 0.180838102102827, -0.0944993352356273, -0.00114511302775321, -0.00067304391220226, -0.000641631662810973, -0.00095722122472303, -0.0331231875874076, 0.061426398524528, -0.054679251196284, 0.0212151322982276, 0.000881675779233149, 0.000801959357219065, 0.0016411729305169, -0.0273920854159157, 0.0255030826818176, -0.073192066942304, 0.139859224732048, 0.607097608174253, 0.000258330833524133, 0.000259692769779562, 0.000557467377202737, -0.00960126525101308, 0.00579356513688972, -0.0121650083715792, 0.0147231629685929, 0.00385553391728582, -0.0110070831616922, 0.00499744607324111, -0.00690326988915843, 0.0311157999146768, 0.421321766809783, -0.00986172950072129, -0.314981484277462, 0.143008184695501, -6.99259570603739e-05, -0.000110401968427987, 0.00167935394144988, -9.98933671546853e-05, 0.00111758579171929, 0.00144858658908363, -0.0020226567481075, -0.00319345341606735, 0.0485764761033611, 0.00911692338279008, -0.101998203955027, 0.123447196538076, 0.000859882570604459, -0.00962018371015555, 0.0274644613171975, -0.0087044874030528},
    {0.00163104362451023, 0.00630096536900634, -0.0195929661997851, 0.0197079603842469, 0.0976637162240657, -0.196492083346238, 0.196789453010076, -0.0984800038037923, -0.0052253220280552, -0.00120559810201227, -0.000822987370205606, -0.0009975429945274, -0.0265321488636248, 0.0418649924033504, -0.0285354595072402, 0.00502315034416612, -0.00176946150425576, -0.000595996850338502, -0.000775277112656243, -0.0285459434103635, -0.00898463593005338, 0.0206962864077098, -0.0268812006793831, 0.14374374463353, -9.10089387106891e-05, -8.65533292723625e-05, -0.000164932469374757, -0.00769075132780619, 0.0235366042308366, -0.0420202768766619, 0.0388570541739825, 0.00614426323170701, -0.0148820249349659, 0.00569033110762512, 0.0116355043739916, -0.0395842756636942, 1.11194332036726, 0.00578806775186309, -0.425867801143153, 0.162835958625975, 0.000117860635943615, 0.000165544232333913, 0.0068224466540862, 5.8629632562094e-05, 0.00178100917684906, 0.00164942997065806, 0.000598450377074136, 0.000840569096452741, 0.0346417252885993, -0.00203594308727173, -0.0618464275405144, 0.0571907222901663, -0.00114461233433269, -0.0347702174190693, 0.0842169716874299, -0.0123143225387019},
    {0.0126104679035144, 0.00513339611045793, 0.010730605592219, -0.0116539403654932, -0.0616020791184594, 0.199029865952139, -0.142163327346853, 0.0628400509032686, -0.00976967983719916, 0.00336641557499317, 0.000832405683526324, 0.000636531784454386, -0.0173335456893195, 0.0196253949660864, -0.00679385676003048, -0.00110709786801743, -0.0295896009607121, 0.0136392839755105, 0.00544335694053336, 0.0182151550335338, -0.052498414352161, 0.0795137525688747, -0.0444271201886022, -0.0316809934644757, -0.000531777248558407, -0.00046557505927923, -0.000751445762371083, -0.00502439475260475, 0.0352681842662437, -0.0451290574006877, 0.0211211319918737, -0.0137344939427511, 0.0237601277454772, -0.00459137662910272, 0.14289168101562, -0.295112794978876, 0.604407671559529, -0.0898140827333989, 0.679925843563978, -0.131387998461278, 0.0014474064771296, 0.00172796720790312, 0.0102230255215699, -0.000909762444964116, -0.00398115426194416, -0.00133088111664943, 0.00256801519808443, 0.00306579120779263, 0.018137879942237, -0.00530369673915512, -0.0232091739926472, 0.0108622704650949, -0.00953110774137439, -0.0417084816104071, 0.0721540127553231, 0.0219592189204036},
    {0.0253972624653878, 0.00367511242526292, -0.00366448453027248, 0.0118223838397412, 0.0220838867907565, -0.0613915956629153, 0.29971631803346, -0.0767262716388461, -0.0117484678910899, 0.0121800061564543, -0.00335222048188705, -0.000777190818574965, -0.00871645838744095, 0.00519517947069486, 0.000292876250127014, 0.000165965533133847, -0.0936105122386033, 0.117950744221066, -0.0442674609131471, -0.022240289639452, -0.06945179083084, 0.0503099322824042, 0.00386755227615351, 0.00474931180199949, -0.000703504718975545, -0.000562698709719906, -0.000767316927102128, -0.0025265994948824, 0.0297697731931803, -0.0200255369950428, -0.0015199192167781, -0.03483292545168, 0.0315465107267411, 0.0129070725951028, 0.288289419424248, -0.264445516332456, -0.0434943939143449, -0.459983218363609, 0.902742954379207, 0.369352063935649, 0.00292019780295694, 0.00295772910200331, 0.00862922652404655, -0.00465935234926482, -0.0100968590612785, 0.00374131346123147, 0.00216656187585751, 0.00219440720934239, 0.00640222151602415, -0.00198736941186252, -0.00430664765188624, -0.000326870461831583, -0.0113881603082128, -0.0246782473142257, 0.0223499055425402, 0.038875990881448},
    {0.0300055156673332, 0.00216464827329578, -0.00428139506100082, -0.0295941247801588, 0.0263560489909357, -0.0712908034861193, 0.124137438646549, 0.383808683073226, -0.0097093743841581, 0.0167516558155944, -0.0132940824930264, 0.00388775028686347, -0.00296712446719288, 0.000185712664972354, 8.46394591313866e-05, 8.15846378218058e-05, -0.145766077772663, 0.28016483684933, -0.262749987106196, 0.111252848540127, -0.0445452074184814, 0.00310597107865769, 0.00167285081968765, 0.00233464669412121, -0.000451216062995051, -0.000332623551692533, -0.000393080450660128, -0.000860066651710737, 0.0142092312311995, -0.00100238451978058, -0.000612533599599344, -0.0276309035962125, 0.00261350337251348, 0.0294008525198291, 0.237643788379266, -0.0198115605034007, -0.0175284168873228, -0.54610911038394, 0.074788675559698, 0.841342254771774, 0.00240718813093238, 0.00212165260149833, 0.00411876416494383, -0.00553175564855263, -0.00800924228238423, 0.00852228919405443, 0.000735621731929415, 0.000648363848762594, 0.00125866873032966, -6.13263008014085e-05, -8.87922808971656e-05, -5.42588740760214e-05, -0.00469219258024622, -0.00679366725474126, 0.000642587464437334, 0.0177130351991935},
    {0.0191690324088708, 0.000873918129073743, 0.0293017672320873, 0.192230415140459, -0.182241449919052, 0.488039973885573, -0.759220407776989, 0.964112153400919, -0.00484906798185385, 0.011117566500962, -0.013560134503665, 0.0097658741614185, -0.000476116367454549, -0.000179485226862473, -0.000140724751873898, -0.000181253151723376, -0.109706187629544, 0.262185091653748, -0.340572586312384, 0.279462732628028, -0.010771742474417, -0.00423279236974588, -0.00353440393167955, -0.00518678617405977, -0.000109111249280883, -7.56002252730393e-05, -8.0513836682017e-05, -0.000138009650255276, 0.00281896164772159, 0.00119640185951769, 0.00125348284008009, -0.00727633935672289, -0.00412667615030049, 0.0118595097748233, 0.0664794510482458, 0.0300484980775632, 0.035869982963235, -0.182750526031733, -0.118090011664295, 0.339374740501438, 0.000673396711126234, 0.000536684303255052, 0.000817119380216155, -0.00185115251774727, -0.00210915884939877, 0.00343766126960388, 6.6118931959e-05, 5.26955245610423e-05, 8.02306572194184e-05, 2.98855446087612e-05, 3.40508738617129e-05, 3.56754594920405e-05, -0.000469376814715695, -0.00053479670257825, -0.000303302620946022, 0.00184464173156562},
    {0.0302387678330058, 0.00234316248688225, 0.259211232824319, 0.0195251039488212, 0.326169225832269, -0.374897896525802, 0.278859093198177, -0.113874016814714, -0.000380508993873697, -0.00022170412581788, -0.000208599314241373, -0.000303030395125532, -0.0132630858830491, 0.0312018466643442, -0.0394682247241064, 0.0302540966567124, -0.00535611326543896, -0.00351055389355523, -0.00396473437419387, -0.00867159466642653, -0.186693590381845, 0.494062814074892, -0.750151206494619, 0.865758905462843, -0.000496810721561596, -0.000369581508087647, -0.000443618433201171, -0.00100999427280183, 0.00229970513974524, -0.00609865929245065, 0.0103483935384562, 0.00151045400183977, -0.00537329878448933, 0.00190641688386352, 0.0364144726787385, -0.115913919569766, 0.296132254907867, 0.028708382496437, -0.153763681235287, 0.0545545092110109, 9.69026328636047e-05, 8.67088960806589e-05, 0.000175124329342011, 7.63959394307345e-05, 0.000115022243287938, 0.000145175123742181, 0.00337765456969319, 0.00302233995531915, 0.00610416325940225, -0.0107516916029518, -0.0161878196213953, 0.027467992543724, -0.000792444136599846, -0.00119310925359822, 0.00424437453528148, -0.00105120358339417},
    {0.0298426277029343, 0.00209472514255055, 0.00525730855264615, 0.0197079603842469, 0.692635678336204, -0.462466964864018, 0.306530718648313, -0.118670820076184, -0.0017270639307576, -0.000395389812588229, -0.000266770877594601, -0.000315795178772636, -0.0105672909703009, 0.0213868154213814, -0.0217703682626921, 0.0112777292031757, -0.0265751876169889, -0.00675219580951297, -0.00535145363896158, -0.00903687495372637, -0.162604137077821, 0.36522935308263, -0.436716022046421, 0.32272636006408, -0.000432706224703858, -0.000317855761650155, -0.000373373839063385, -0.000804707400159043, 0.00930170605810038, -0.0212166270111519, 0.028952055543369, 0.0023999983730728, -0.00733851328906358, 0.0021707374405572, 0.158848151032726, -0.425607910612668, 0.828499366632536, 0.0481442357685088, -0.210000758077653, 0.0621183733200973, 0.000422711162026648, 0.000370402667393299, 0.000708332127892959, 0.000128116731068662, 0.000182761736816868, 0.00016530333906084, 0.00258641951000171, 0.00226636240431682, 0.00433403278578941, -0.00692989245680937, -0.00988566575807058, 0.0134899078897135, -0.00301399817492398, -0.00429954414710401, 0.0131467846872079, -0.00148714787905985},
    {0.0280378223106107, 0.00169002122151258, -0.0132573892568216, -0.0116539403654932, 0.130691148487769, 0.574537076169525, -0.227302951843894, 0.0757238026633065, -0.00320141581284019, 0.00109628782576135, 0.000268548259483248, 0.000201508776833691, -0.00684455236573051, 0.0102186321500794, -0.00632708254785063, 0.000743550878448018, -0.0564738728270903, 0.0210073440323389, 0.0058431328785942, 0.00576642628111652, -0.120739823396356, 0.195812008554932, -0.137666072131895, 0.0212776405782488, -0.000321301008031821, -0.000231343120081738, -0.000262683898321788, -0.00052121796919942, 0.0138399924095757, -0.023278825445249, 0.0193014916091437, -0.00533944176444635, 0.0120265244814163, -0.0017515102309489, 0.265205427918547, -0.506505871964673, 0.552336380720342, -0.116176763860625, 0.3441540757173, -0.0501216611310331, 0.000705738744092459, 0.000598414007520604, 0.00105392615206973, -0.000309158240324237, -0.000406602630006522, -0.000133378862024455, 0.00150885298032614, 0.00127939519584879, 0.00225326954047083, -0.00288170155665654, -0.00378999256368272, 0.00314244848129011, -0.00390294190895962, -0.00513312031819352, 0.0115618073754219, 0.00265192061850403},
    {0.0235888506057285, 0.00119584539165626, 0.00832487477036396, 0.0118223838397412, -0.0614326220634956, 0.26203544501963, 0.518569562880659, -0.0924570392473859, -0.003810323461109, 0.00393303420879119, -0.00107536568428442, -0.000246037629293962, -0.00340656404721305, 0.0029025345964173, -0.00040101338964554, -0.000146155552097418, -0.0784456611970351, 0.0857621692652027, -0.0255955981627664, -0.00704067522019316, -0.0701331978298506, 0.0632915073048822, -0.00954482528990459, -0.00418242435882016, -0.000186631606087884, -0.00013124857287077, -0.000143263535890484, -0.000259412493288236, 0.0115838154219246, -0.0111148703955156, 0.00205374017223567, -0.0134650954278906, 0.0172791285751942, 0.00492376721147823, 0.252592041211182, -0.264553500667241, 0.0587703498073264, -0.320492997714484, 0.494463927062682, 0.140899771694796, 0.000672173233146917, 0.000548608666298252, 0.000882116525256795, -0.000852864616960197, -0.00102537745625853, 0.000374948690527344, 0.000600946663166296, 0.00049047556662225, 0.000788643397617979, -0.000629404405192859, -0.000756716922147384, 0.000139821688128011, -0.00251190957878634, -0.00302000505476237, 0.00387543154955192, 0.00469488656026307},
    {0.0161601125382385, 0.000696217557866089, -0.0107414954878784, -0.0295941247801588, 0.0716512169321643, -0.219640225699807, 0.508137289414865, 0.462498877065499, -0.00311691852162235, 0.0053640568002262, -0.00424076416664772, 0.00123075677299037, -0.00114779903153429, 0.000243419538752891, 8.3183464345972e-05, 7.02142929650505e-05, -0.0741443373095675, 0.131860036949592, -0.10956139003762, 0.0352196480617276, -0.0273035043961887, 0.00598377507353535, 0.00214906927708208, 0.00200927002101725, -7.26574152465354e-05, -4.99388244904336e-05, -5.24846012754307e-05, -8.74057861344202e-05, 0.00548279610120531, -0.00130887565395665, -0.000597423305482524, -0.0106212767903116, 0.00338668826964639, 0.0112157851875391, 0.134778903993248, -0.0338151876404043, -0.0170960168773588, -0.274403811056705, 0.0969143307348482, 0.320953754397271, 0.000358660436103719, 0.000282211597182808, 0.000417519173892655, -0.000730216581573684, -0.000808818462207872, 0.000854090737576335, 0.000132075990550803, 0.000103923858030729, 0.000153750603397791, -3.31370434908315e-05, -3.67039769223268e-05, -1.67531660865139e-05, -0.00074638072928615, -0.000826722549058301, 0.000263607814241562, 0.00213912723541274},
    {0.00734348804657581, 0.000278404956314872, 0.0263784345157957, 0.192230415140459, -0.168273534214414, 0.466883678827404, -0.770250380155342, 1.16177879234705, -0.0015436079932699, 0.00353550741513575, -0.00430576776528935, 0.00309161208448752, -0.000182636570873296, -5.79074942609892e-05, -4.27474677586887e-05, -5.29819689256e-05, -0.0411121542477163, 0.0953904978590438, -0.118457806464409, 0.0884703557588252, -0.00486430680960252, -0.00156238526998144, -0.0011760437484389, -0.0015161454644233, -1.2944417486678e-05, -8.73387187564842e-06, -8.90566876010808e-06, -1.39079164691892e-05, 0.00108025695398444, 0.000385414218265712, 0.000379616920949843, -0.00278416799838679, -0.00132537357759288, 0.00452414480070673, 0.0291460988640957, 0.0106032943169834, 0.0108632141195972, -0.0765964287661006, -0.0379272265467476, 0.129464075403039, 7.75607474134132e-05, 5.92733532525508e-05, 8.2262404563882e-05, -0.000203830924063772, -0.000212016551629293, 0.000344517134121813, 9.17683051884083e-06, 7.01310308668865e-06, 9.73312106897217e-06, 3.33851316927647e-06, 3.47258421640878e-06, 3.42035053585432e-06, -6.22795393424545e-05, -6.47806177063691e-05, -3.08381243865341e-05, 0.000222769464588879},
    {-0.0125131983511541, 0.011212845430362, 0.0208770933958504, 0.0195251039488212, -0.00350519620228991, -0.00213578148775574, -0.00215103292273904, -0.00360148326124408, -0.00226985035225306, -0.0017969233795524, -0.00280290297783925, -0.020199909014669, -0.00250226696008488, -0.00195073523785069, -0.00302124952471561, -0.0216728867138269, 0.110772181805886, -0.208071106282045, 0.201984382314542, -0.0990472496003531, 0.122114469068079, -0.22588143914298, 0.217718994875168, -0.10626977667799, 0.684912572298823, -0.294334759695953, 0.183176207073003, -0.0688168296738453, 0.00058950120913559, 0.000632108757277131, 0.00392988617312104, 0.000641888290245373, 0.00276290821881131, 0.00401918370631295, 0.0682600995319012, -0.0455513793747797, 0.0192696123733745, -0.0462561176199211, 0.0135475095344601, 0.01970746954651, 0.382855534749969, -0.0593557148961155, 0.0162123406290092, -0.259440152714044, 0.0176530793253, 0.110534761061572, 0.422057231279378, -0.0654333199232003, 0.0178723695424869, -0.281647539217126, 0.0191641359271614, 0.119145434915934, 0.085112301093968, -0.00579129401155841, -0.0249277235077286, -0.0253133873528489},
    {-0.0273770165770784, 0.0164398153896087, 0.019287579585398, 0.0197079603842469, -0.0163810014243473, -0.00390242657177244, -0.00279689872237231, -0.00375319132544435, -0.016455224270786, -0.00492616592994066, -0.00514129646939641, -0.0210508054013366, -0.00318431349806179, -0.00243126567889089, -0.00354041384624266, -0.0177795380457134, 0.180416762354003, -0.246495727242804, 0.220428635877593, -0.103219493481902, 0.0349131389634353, -0.121655788733429, 0.151792179115675, -0.0871793205263412, 0.195819938431222, -0.223629290575918, 0.191612435862306, -0.0875743328428926, 0.00366524434684604, 0.00373089742608153, 0.0162853332138362, 0.00146285721496177, 0.00446348462723381, 0.00457643479011267, 0.18340167254765, -0.15995899772086, 0.0798527093604428, -0.0627187368589139, 0.0218860330331138, 0.022439867358159, 1.02865870250385, -0.294038759305839, 0.100800793821525, -0.351775278729851, 0.0402311973122855, 0.125860165397376, 0.199059686904237, -0.0569005669447607, 0.0195063478381806, -0.173615581371304, 0.0198557520467495, 0.086670176465348, 0.261733439364735, -0.0299334554728397, -0.0913332599904377, -0.035811093976741},
    {0.00769851183855554, 0.0241754756606097, 0.0163965439250797, -0.0116539403654932, -0.0318063021955915, 0.0112436886986536, 0.00289018334287365, 0.00239490987846146, -0.0533456433751071, 0.0225203407828072, 0.00770119898836347, 0.0134325104780699, -0.00360710415218171, -0.00266909142798141, -0.00355083307641826, -0.0120625997765331, -0.0489992445747003, 0.267079884396059, -0.160437252917034, 0.0658643173647256, -0.00331321111484881, -0.0316540782798458, 0.073973663738371, -0.0591471639811745, -0.0185830554278955, -0.0843456371815979, 0.148164197799284, -0.0992018341832284, 0.00899172423205336, 0.00848823481059446, 0.0251793692370997, -0.00484271172850354, -0.0101935082532452, -0.00369260335515115, 0.106637314753757, -0.176833383630821, 0.123463292225494, 0.100887065451003, -0.0499823516793694, -0.0181061313655989, 0.598104697243667, -0.471191240681806, 0.247288544676479, 0.565852843131988, -0.133183258819302, -0.101553215623277, 0.0404424016727429, -0.0318608188635712, 0.0167210568633937, -0.0670643924827068, 0.0157847653325304, 0.0468236851945558, 0.225928762376926, -0.0531762439644614, -0.11193160198611, 0.063859270369354},
    {-0.000692705759853925, 0.0299005753431564, 0.0123418011338163, 0.0118223838397412, -0.0399845505836983, 0.0422013001459018, -0.0119330872043856, -0.00292413044299179, -0.105804882736616, 0.125936349182971, -0.0430093491732371, -0.0164007895111112, -0.00299169409535095, -0.00213868376258172, -0.00258304136084948, -0.00634120372040312, 0.00436230428657875, -0.0132065134800401, 0.346278038065547, -0.0804188321427702, 0.000123346670198293, 0.000224276439037019, 0.0207966526318344, -0.0310931493406899, 0.000691823711102499, 0.000808339326836501, 0.0620609909795394, -0.0822769537703657, 0.0117309116645637, 0.0102295585373868, 0.0220482695108045, -0.017032284117612, -0.0265051855745514, 0.0103804813719165, -0.00123018051608058, -0.0823604990239475, 0.108110410393762, 0.137130787640276, -0.129964235452736, 0.0508991465589973, -0.00689980563393294, -0.296844513069608, 0.32262111230204, 0.769135723398206, -0.468418363737866, 0.285481857011572, -0.000195095984610733, -0.00839344984860806, 0.0091222980617277, -0.0130616624471412, 0.00795480220858049, 0.0171453755662, 0.0716446307025766, -0.0436329501586935, -0.0679004315061573, 0.11305467747162},
    {-0.0063064816626327, 0.0274659679169433, 0.00763236993753088, -0.0295941247801588, -0.0344990911194155, 0.0601371945948342, -0.0484704728084982, 0.0146274102792546, -0.130480475709523, 0.243240342500804, -0.217371432971372, 0.0820418520171366, -0.00151964692640051, -0.0010529163339581, -0.00116741866689652, -0.00225357922813266, 0.0395383094710885, -0.109983778158357, 0.203317202134924, 0.402280019603826, 0.000460484759394756, 0.00047608762305933, 0.00109193877883999, -0.0110500905791669, 0.00258275537262994, 0.00213506992707138, 0.00441662708652146, -0.0417930162395376, 0.00786323208030548, 0.00639960141743295, 0.0109037695623235, -0.0172183619845815, -0.0215344921497767, 0.0236455632872443, -0.00355544628755455, -0.00598583280785839, 0.053465012374061, 0.0161051023871777, -0.10559117951605, 0.115942502867309, -0.0199416979909375, -0.0268441585068067, 0.216252986347212, 0.090329894461468, -0.473535838844077, 0.650295403023377, -0.000232251912742933, -0.0003126417400369, 0.00251860045911445, -0.000391011706139525, 0.00204979821318826, 0.0034924874078851, 0.00292009506579979, -0.0153079960375438, -0.0191452543972815, 0.0515110081076783},
    {0.0287678230747216, 0.0152327680704991, 0.00320507108782466, 0.192230415140459, -0.0178382131382485, 0.0410650708630083, -0.0503948504088267, 0.0367434731025106, -0.0864392103719314, 0.204225002221374, -0.260674168295516, 0.206085870657986, -0.000323458282953187, -0.000218849807463874, -0.000227625898028619, -0.000374392247516078, -0.179978494886957, 0.485949924576904, -0.76622985466315, 1.01051141642987, -0.000673485269869302, -0.000520749400288676, -0.000669087236009829, -0.00183577670380717, -0.00377742729515867, -0.00269364658708948, -0.00332750776727585, -0.00889568296255177, 0.00197351752822306, 0.00152661443863603, 0.00222573096474407, -0.0053308884150961, -0.00578037173244178, 0.00953798155162833, 0.00469595144355349, 0.00448735510346647, 0.0109135407614043, -0.0156696993884498, -0.0283431930980597, 0.0467680739919022, 0.0263385347137001, 0.0232114501770192, 0.0542752718892444, -0.0878878170391845, -0.146608993334405, 0.262311600776789, 9.85596371985836e-05, 8.68579870966363e-05, 0.000203099798998407, 9.41815724237148e-05, 0.000157107844850943, 0.000229055736824829, -0.000849299280487138, -0.00141674826780969, -0.00153620391228614, 0.00536437454800331},
    {-0.0251141783214145, 0.0170982512251828, 0.0160091858217584, 0.0195251039488212, -0.00316207558373215, -0.00241577235429526, -0.00353354291864167, -0.0181912548602126, -0.00335839069175041, -0.00254526063085192, -0.00367028490867623, -0.0177284701790032, -0.0187003136617075, -0.0045390434804233, -0.00447483985261535, -0.0171592587463128, 0.0293790293907046, -0.114578371119062, 0.148172383047726, -0.0869289168370503, 0.163589384055957, -0.204331219412855, 0.180652919655244, -0.0841378732334323, 0.159427677701374, -0.212238483655012, 0.190336131394753, -0.089361510440975, 0.00421763175802373, 0.00135971169586768, 0.00390122508841131, 0.00424553498359215, 0.00398337890666897, 0.0178172618556883, 0.189862433322843, -0.0548926656233986, 0.0191290770071509, -0.171395695833932, 0.0195319060365618, 0.0873642936180388, 0.185032341811612, -0.0570169166961805, 0.0201544183268214, -0.167035397269227, 0.0202877569663799, 0.0851417500342238, 1.03030384104418, -0.317483677185369, 0.11222456794757, -0.297879486990314, 0.036179796235578, 0.103805482586356, 0.2767865057501, -0.0336178884956984, -0.0315420290818583, -0.0984861631531845},
    {-0.00383082689433907, 0.0214382923559291, 0.0205403356838656, 0.0197079603842469, -0.0201247738104083, -0.00584323645334017, -0.00577935460978638, -0.0189575391547738, -0.0209027123750645, -0.00602728243276228, -0.00589038572642277, -0.0184752602365972, -0.0204312959399314, -0.00614677799527293, -0.00607356716536563, -0.019170475293716, 0.0135858929586521, -0.128120058977522, 0.160325430484326, -0.0905906908173123, 0.0132794918987443, -0.130660138804077, 0.165311291244429, -0.0939995744532915, 0.0129416622398114, -0.133170339698606, 0.170518145142878, -0.097633200088908, 0.0226514859192027, 0.00883256817957487, 0.0214011552624065, 0.00846553253191364, 0.00809465779372059, 0.0202875864800224, 0.481495556956455, -0.240406207919515, 0.1049373819195, -0.230416174847411, 0.0396909506048684, 0.09947716301179, 0.469246332285472, -0.245024815275158, 0.108242622668801, -0.224554398029832, 0.0404534804829548, 0.0969464685915432, 0.458663474463193, -0.23949879918387, 0.105801439419834, -0.229006363638902, 0.041255502199997, 0.0999613464691265, 0.843911139358477, -0.152030613093807, -0.145370156280727, -0.139329327795018},
    {0.0115162223864431, 0.0272195064369147, 0.0267098865309273, -0.0116539403654932, -0.0579732173172456, 0.0239119304978302, 0.00786274267358259, 0.0120967981262488, -0.0588573409883909, 0.0241488423712532, 0.00787093469350963, 0.0117890561421181, -0.0201021088746972, -0.00778634204845772, -0.0077179870839259, -0.0192964308977999, -0.056836811610785, 0.109442245504125, -0.114072987754541, 0.0578058834528901, -0.0194120182105408, -0.035287602815314, 0.111856327157735, -0.094617179025035, -0.0189181773662325, -0.0353243681982647, 0.112964564404558, -0.0960601434065916, 0.0487018653261889, 0.0236011963360773, 0.0469972581018548, -0.0249998751193355, -0.0243385216796769, -0.0163695132433427, 0.220716232256438, -0.3420507329146, 0.23044406538496, 0.362321701221159, -0.119340321283624, -0.080265473615362, 0.215101221529171, -0.342407108105343, 0.232727232578612, 0.353104253921159, -0.119464659358852, -0.0782235236836696, 0.0734655711869931, -0.116945564495709, 0.0794855508053786, -0.113851854988648, 0.0385191424984337, 0.0767034880809724, 0.712118434341345, -0.240928805691316, -0.234555210079491, 0.248455666275406},
    {-0.00801784374588152, 0.0302616520385568, 0.0302516266319403, 0.0118223838397412, -0.105865563934372, 0.12409022809239, -0.041339453116336, -0.0147699151361855, -0.105632026424746, 0.123383715498899, -0.0408731176137138, -0.0143941691791134, -0.0150864946514284, -0.00725880667245088, -0.00721801875870378, -0.014735200510621, 0.0450891617645782, -0.110330824686386, 0.228755234415404, -0.0705796677815742, 0.00643968899227084, 0.00649089001066514, 0.0403972016220275, -0.0722518641953717, 0.00627586360252846, 0.00641766861494255, 0.040167198565973, -0.0720924778963431, 0.0580523449520112, 0.0336777988542341, 0.0568994552709836, -0.0817577714486635, -0.0805869296995662, 0.0460172433773116, -0.0519109273669999, -0.188485078243955, 0.278998016488917, 0.457575033785653, -0.395146024404499, 0.225638708936901, -0.050590315778771, -0.186358845866078, 0.277409530310443, 0.445934346116379, -0.39068852421984, 0.219898470630843, -0.00722537050781903, -0.0266159973122159, 0.0396199669450188, -0.0262348332920278, 0.0229846576996804, 0.0388331348008336, 0.209813218721953, -0.183819922143654, -0.181187462435566, 0.439859006441802},
    {-0.000295237250846771, 0.0258677865545684, 0.0261981528295216, -0.0295941247801588, -0.123694165861968, 0.228667944176925, -0.201694509957147, 0.0738837109693798, -0.121961990153499, 0.2250337245264, -0.197896745384657, 0.072004112783863, -0.00717468642842263, -0.00405343007898568, -0.00404133581582028, -0.00709146953368313, 0.00176428765404592, -0.00456258703054652, 0.00724460448462809, 0.353061458146974, 0.000103788161142541, 8.21838039899575e-05, 0.000147945230318219, -0.0347719661720228, 0.000101147795129399, 8.0636341249031e-05, 0.00014559393676393, -0.0342850300686167, 0.0367446701898161, 0.0242440923895312, 0.036388205229727, -0.0791788655558157, -0.0786453959092751, 0.104822079207859, -0.000745002803480166, -0.000887527786824341, 0.178424152469219, 0.00289858008214555, -0.385626002220497, 0.513979475619877, -0.000726049966660758, -0.000870816268069355, 0.17558845740332, 0.00282484033908847, -0.378364938100432, 0.500903861562423, -4.27115106568579e-05, -5.12277116198022e-05, 0.0103293831195581, -5.08825635930771e-05, 0.00681531545619461, 0.0102291763923664, 0.000461254537812761, -0.0617813836177232, -0.0613651299539455, 0.200412591091127},
    {0.0299453386201052, 0.013710902622998, 0.014002682081672, 0.192230415140459, -0.0792712548026677, 0.186695351837455, -0.237148368754727, 0.185592944676423, -0.0775772130015774, 0.182572154863326, -0.231653156327066, 0.180871468758637, -0.00146630027813818, -0.000920673795231253, -0.000919650531979711, -0.00146020247167447, -0.184439512927778, 0.487299390017091, -0.741538613135988, 0.886876346755305, -0.00348612303332124, -0.00245735051523463, -0.00294386828509748, -0.00715988564968094, -0.00339743622479551, -0.00240040868105137, -0.00287885247894114, -0.00700687753076342, 0.00891143980381032, 0.00638415719932237, 0.0088808686042026, -0.0239288085613582, -0.0238732528442571, 0.0422823954558635, 0.023785331250659, 0.020436151834442, 0.043546018384025, -0.0765978577452829, -0.117058944746748, 0.207325437623335, 0.0231802335251374, 0.0199626044256031, 0.0425842975405801, -0.0746492117914788, -0.11434644973226, 0.202051088091408, 0.000438133590394865, 0.00037731662802893, 0.000804893149832731, 0.000376440608824124, 0.000576625608242402, 0.000802131918237234, -0.00364366602223468, -0.00558130841108428, -0.0055683502401685, 0.0208710379051658},
    {0.0107425993815028, 0.0254129579368256, -0.0100835618309505, 0.0195251039488212, 0.00276855422334095, 0.00354739278284653, 0.0184970121822559, -0.041398111534134, -0.00476624355041166, -0.00338273614995222, -0.00435470127501963, -0.0137973375736295, -0.0603965005827018, 0.0315784943674165, 0.00779631465864154, 0.0116923912438848, -0.00527925397357774, -0.0289195606742811, 0.0792937759693208, -0.0676531927685026, -0.0668972246841806, 0.26996967643304, -0.141961339914384, 0.0573319015009941, -0.0222957897716069, -0.0635582267339031, 0.135372028365772, -0.098700543214283, 0.0127562315865689, -0.00844494131011642, -0.00445839259329853, 0.0114632796231687, -0.0162280695445271, 0.0315560665716391, 0.109055095339958, 0.15377203670886, -0.0218610650020318, -0.208732280105511, -0.0795719254745864, 0.154730479224276, 0.0363463430762801, -0.036202132415786, 0.0208463565738799, -0.0695671764823022, 0.0187334020168054, 0.0515692280558872, 0.460570658542209, -0.458743261526413, 0.264159179791141, 0.649423009457742, -0.174879920823443, -0.0923254899097837, 0.262336456257311, -0.0706432910926394, 0.10000665415058, -0.135750409459011},
    {0.0106226666604082, 0.0277849136316368, -0.0118196797021562, 0.0197079603842469, 0.0153251463175155, 0.0068247368416488, 0.0170656692030986, -0.0431419561966856, -0.026019057404776, -0.00708763218492734, -0.00629024608249975, -0.0143785334928047, -0.057876558781076, 0.0213437702446786, 0.00757530613557591, 0.0113613654920689, -0.0217181624512596, -0.0244158730639455, 0.0837575252759868, -0.070503000519193, -0.0483096864799013, 0.0735262174169138, -0.100868692067401, 0.055708765959124, -0.0161008564820601, -0.0271010761760102, 0.104754249861408, -0.094582430040685, 0.0606169663210653, -0.0278519033674185, -0.0194533184251786, 0.0205729256103859, -0.0186022939508754, 0.0359312466038147, 0.208816727025423, 0.370860928123472, -0.095386453682271, -0.273937984971186, -0.091213581742051, 0.176183523808719, 0.0695953213086698, -0.13669587008386, 0.0990608304482568, -0.091299209379899, 0.0336204732668517, 0.0587191894223527, 0.154807210805021, -0.304065071880222, 0.220350026027013, 0.274939401154231, -0.101245047457608, -0.0707151723594293, 0.780836385935845, -0.28753905994854, 0.259996376636918, -0.19204741754843},
    {-0.00262086915298648, 0.0300547935497049, -0.00729939888431999, -0.0116539403654932, 0.0195387056426204, -0.0109690858690787, -0.00648860340814748, 0.027528864934527, -0.0629779646757725, 0.024733826799805, 0.00750215134912613, 0.00917493645108298, -0.048949411602034, 0.00658120841508754, 0.00337457711054255, 0.00506138536202273, 0.00953046266998923, -0.00904585198043121, -0.055682659374989, 0.0449879363356469, 0.0074075201126751, -0.00240693191785913, -0.0250468724285161, 0.0248177503627265, 0.00246880960967878, 0.00278290510283732, 0.0564773593641095, -0.0799936001014606, 0.113516362413503, -0.0185639631595197, -0.0167784696518062, -0.054226955665696, 0.0156313105845951, -0.0289919659842307, -0.0415161074774569, 0.137785921551927, -0.0822707305417843, 0.402484695490358, 0.0766458066681741, -0.14215779334252, -0.0138366637603085, -0.159308678962164, 0.185509533264767, 0.134141800340848, -0.0886182134630823, -0.0473789501690623, -0.0107545004524866, -0.123822135860404, 0.144186661900358, 0.0356926225918727, -0.0235796481027297, -0.0213117429016787, 0.615646310841459, -0.406714954274355, 0.117238515266376, 0.342463929443952},
    {-0.0112132650061804, 0.0291131058185437, 0.00962857943557122, 0.0118223838397412, -0.0386200930353358, 0.0549301045311348, -0.0255353246076025, -0.0336121174078451, -0.0994794309404829, 0.112846675863041, -0.0356794946677951, -0.0112023885451423, -0.0323328431907149, -0.00410083987857553, -0.00278914940712228, -0.00418348946077049, 0.0538868521712184, -0.101042221662702, 0.0849937731483466, -0.0549292461438857, 0.0175143255728478, 0.00367186688349743, 0.00664416170108377, -0.0205131183769428, 0.00583724844800665, 0.00513055372915046, 0.0136495564365595, -0.0528386438915602, 0.120828272253816, 0.018159391200534, 0.0196022074155316, -0.162415639580523, -0.0387405065266481, 0.0815009178862219, -0.108188894132857, -0.043258325000982, 0.0961165087027929, 0.386897800957663, -0.189958312030145, 0.399627629543894, -0.0360576518771371, -0.0604431390604202, 0.197458726781112, 0.128946934255838, -0.265421203189447, 0.133189585327345, -0.0117194719847802, -0.0196452524780159, 0.0641781118900007, -0.00468592208119778, 0.00964538694945505, 0.0104117408727183, 0.138067294932718, -0.284194329656931, -0.0677880055845992, 0.606288622857958},
    {0.0122972972658715, 0.0224445447174005, 0.0280757815135756, -0.0295941247801588, -0.142669865790111, 0.291843625603676, -0.302869456861528, 0.168138269227149, -0.104568501634171, 0.189850631265638, -0.162829463999105, 0.0560378329736127, -0.0139990053434686, -0.00513509550091871, -0.00424389085237572, -0.0063656580691272, -0.06839141714894, 0.159071760349638, -0.206289585296184, 0.274773179699455, -0.00915583372768281, -0.00430258606699973, -0.00537660975155994, -0.0312130576026592, -0.00305149496021592, -0.0023131674958557, -0.00349760168066449, -0.0228773094223895, 0.0705466934159362, 0.0299544723214308, 0.0361434486839319, -0.14825914401829, -0.0919091466194162, 0.185649879114551, 0.0591095569907137, 0.0379494934220747, 0.177224025148592, -0.187830363035947, -0.45066283116216, 0.910306571261337, 0.0197002829695772, 0.0204025052145788, 0.115288086146479, -0.0626008972232353, -0.242286521734026, 0.303390821313914, 0.0026373560130351, 0.00273136532565275, 0.0154340791804503, 0.00169323422072122, 0.00655338578226156, 0.00790739894148369, -0.0232618994971412, -0.0900313728410426, -0.0558124539406071, 0.276242777973244},
    {0.0299283676160423, 0.0111030516956888, 0.0249036082205118, 0.192230415140459, -0.14494000231401, 0.354346112869683, -0.476703569824698, 0.422356647889497, -0.0625327818570342, 0.146258539859399, -0.183841668082121, 0.140764808621599, -0.00268976516544261, -0.0014825888739096, -0.00138523266460874, -0.00207782821502175, -0.179896704701628, 0.459596310372118, -0.662594871472996, 0.690219303679269, -0.00773801956853012, -0.00465882113210356, -0.00499260080123195, -0.0101883216251349, -0.00257895986512175, -0.00179668352082423, -0.00206072672104517, -0.0043956401511129, 0.0162462241769459, 0.0101844496336541, 0.0131181816702361, -0.0432160109691644, -0.037347702370065, 0.0748861467390758, 0.0510514100330865, 0.0367063907025683, 0.0643230528044016, -0.155757438084702, -0.183128903994627, 0.367193083012779, 0.0170146296952577, 0.0141558916760698, 0.0265497360935653, -0.0519114972451437, -0.0706240215419474, 0.122379662580787, 0.000731861863459665, 0.00060889701665811, 0.00114200189365379, 0.0005262147996119, 0.000715899315482608, 0.000922121235404486, -0.00576626720338016, -0.00784483208536707, -0.00677958116209419, 0.0287680203061013},
    {-0.00674620730936926, 0.0302500012368685, 0.0100401662714875, 0.0195251039488212, -0.00508376488671095, -0.0317222402070974, 0.0822967853898158, -0.0686320373360359, -0.00538804273701245, -0.00355870072593606, -0.00406902438226485, -0.00918403835795022, -0.113191271384467, 0.147807356790049, -0.0651442148090813, -0.014291379808519, 0.00197276041282763, 0.00282759490804076, 0.0246420043988713, -0.04503256618228, 0.0414434833137069, -0.117441550053527, 0.394513248650745, -0.0700756553903233, 0.00554578525152279, 0.00580669741102569, 0.0442577376376677, -0.0742698835373353, 0.0222480575010769, -0.0351083038449275, 0.0265381887482214, 0.0177577186477456, -0.0480603077984889, 0.0348231154916576, -0.0176773769273056, 0.212615825443168, 0.130126061606394, -0.107540712446678, -0.235656571469202, 0.170749967708241, -0.00236550908394924, -0.010512427352851, 0.0145979510551169, -0.0143906266881722, 0.0116516377961438, 0.0228490121220381, -0.0496942941533087, -0.220843648702358, 0.306671776781995, 0.597701424519648, -0.483940043727635, 0.365807823698725, 0.0899663991696788, -0.0728429636639771, 0.197145552539167, -0.0997158754837051},
    {-0.0108682780791435, 0.0300983577029971, 0.0110939404354585, 0.0197079603842469, -0.0231802053744836, -0.0276298646655858, 0.0870824133797188, -0.0715230777133198, -0.026935005912887, -0.00688995031957133, -0.00551966637194162, -0.00957090470710664, -0.0993286602484081, 0.105874942979701, -0.0269115200303897, -0.0114388293177889, 0.0148432501155472, 0.00721310365350005, 0.0237118389394892, -0.0469295077882568, 0.0547376953425479, -0.110840703140009, 0.115608731683716, -0.056088598307651, 0.00732475842426723, 0.00702557127231344, 0.0288288932084989, -0.0651740009484614, 0.0976912073872446, -0.124711701733312, 0.0524233707828876, 0.029928683173966, -0.0631847155811979, 0.0396512647545375, -0.102273132930751, 0.535746834338047, 0.257050201911992, -0.128570110450796, -0.309816855637989, 0.194424079547109, -0.0136857422900748, -0.0339579907193822, 0.0640995944875302, -0.0172046885375793, 0.0196375549684063, 0.0260169779825591, -0.0504691348713492, -0.125227435768644, 0.236381119183879, 0.264377148370073, -0.301761974485488, 0.126847598555409, 0.243941787386094, -0.278436906801433, 0.587829296106843, -0.141069013725581},
    {-0.0100304424671005, 0.0286160770881658, -0.00170955493305554, -0.0116539403654932, 0.00630079692715634, -0.00613847546289087, -0.0581932607657199, 0.0456388471838216, -0.0578628584901557, 0.0216383431733441, 0.00606898946781378, 0.00610719045242115, -0.0745598627704426, 0.0516368564971357, 0.00152770678244023, 0.0010864443757939, 0.0278670398224955, -0.0165823879066068, -0.0112119814106073, 0.0299457000850141, 0.035908399951192, -0.0395715317875216, -0.00282231830132855, 0.00532721841410373, 0.00480510466139414, 0.00412692477588739, 0.00994877186000764, -0.0489220790330338, 0.164640818162663, -0.134686083023702, -0.00625001174450558, -0.0727264094622996, 0.0933158150499534, -0.0319935495606552, -0.126171301109485, 0.248821960746169, -0.0306460030495001, 0.134356329875994, 0.457560220603032, -0.15687561199579, -0.0168836903877483, -0.0259497034408979, 0.108028244942133, 0.0179789750546522, -0.0477190678644039, -0.0209924066648649, -0.0217556766329356, -0.0334377937414931, 0.139201057956504, 0.0429042901956706, -0.113874830422634, -0.00528428039161075, 0.136797588383317, -0.36308262200192, 0.465874103411457, 0.251557919288668},
    {0.00537863848136859, 0.0243807773710823, -0.011422245664625, 0.0118223838397412, 0.0552995939385817, -0.104936601954282, 0.0910163993406658, -0.0557239934719332, -0.0812967864767637, 0.0892023530796901, -0.0267723918023991, -0.00745674051607513, -0.0438056716503967, 0.0115541664574629, 0.00399812922953607, 0.00398736235826677, -0.0220790357715202, 0.0324894082074785, -0.0161003738189845, -0.0365630181088648, -0.0118969892080667, 0.00420827497900537, 0.00240439388633077, 0.0195514382990292, -0.00159200294020184, -0.00123786029074245, -0.00204387781190569, -0.0287428711768686, 0.15834431170101, -0.0485677550391043, -0.0249264224083989, -0.202042477253413, 0.0919140623426249, 0.0899388353673571, 0.0576723909467996, -0.0292076635307709, -0.122223005070747, -0.121504250912767, 0.450686934679816, 0.441001702974336, 0.00771746652451476, 0.00859140789289676, 0.103896823889261, -0.016259166190466, -0.132569155480525, 0.0590129145694342, 0.00415845224881611, 0.00462936371142332, 0.0559833955320353, -0.00210601072884665, -0.017171364169944, -0.00881285693118499, -0.0288618673204911, -0.235325313207767, 0.107055238101477, 0.445351148958013},
    {0.0265225731168814, 0.0168888172596302, 0.0116134748343783, -0.0295941247801588, -0.0648165501864932, 0.151531129548398, -0.19835664154622, 0.278748753109168, -0.0776057232015647, 0.138298675897828, -0.115271030719205, 0.0373009361247863, -0.0172240947344067, -0.00244988539799068, -0.00155992255535781, -0.00192502052858147, -0.136406455745618, 0.28376541171853, -0.302860847075892, 0.182899592666262, -0.0302745418665179, -0.00502674905678499, -0.00409850995120617, -0.00943905186116172, -0.00405120647096817, -0.00292119558198592, -0.00374060763022854, -0.0113015031487309, 0.0851979215757075, 0.0138880133222224, 0.0124916450858425, -0.174002317425163, -0.0400671452317111, 0.204870501421384, 0.174811675791853, 0.0364890940310535, 0.0612509238451516, -0.457170278774762, -0.196463287614661, 1.00455203413528, 0.0233925320915244, 0.0212049137663457, 0.0559021878246979, -0.0611765225010694, -0.114170745608983, 0.134424749317638, 0.00519182313107982, 0.0047062952122467, 0.0124071336395174, 0.00108370863424564, 0.00202247231023228, 0.00181912313224393, -0.0376873641955267, -0.0703340806951783, -0.0161956798487159, 0.202914971390714},
    {0.0260124381207849, 0.00773818110957634, 0.0299947250976383, 0.192230415140459, -0.180516285730209, 0.461954122435896, -0.667777779660335, 0.700205786033841, -0.0433476663950503, 0.100657728235379, -0.125149498779013, 0.093698468630728, -0.00309114016851071, -0.00142932235848192, -0.00122935995028981, -0.0017225671593687, -0.152061844875105, 0.37398651422484, -0.507825635341389, 0.459436505525796, -0.010843593574508, -0.005310543918717, -0.00498843785959978, -0.00844635187532084, -0.00145104215453649, -0.000995259096535661, -0.00108695820213263, -0.0020282360777897, 0.0185363701620086, 0.00971194662742106, 0.0113836940204542, -0.0487726328729603, -0.034824576812883, 0.082639226619067, 0.068870543621452, 0.039408671345784, 0.0558182505772274, -0.197907251068728, -0.170757133079533, 0.405209157119218, 0.00921595422346839, 0.00738565375592512, 0.0121625460717833, -0.0264830807255861, -0.0320019178067211, 0.0542233129952793, 0.000657193539132905, 0.000526674049476931, 0.000867316232694579, 0.000376055172972421, 0.000454421706483191, 0.000532642719457799, -0.00487692496111071, -0.00589323248953554, -0.00420787879224302, 0.0211316366719472},
    {0.00349338776214513, 0.0248800215131901, -0.02833860959432, 0.0195251039488212, 0.0724066424646239, -0.168855440911288, 0.180838102102827, -0.0944993352356273, -0.00422947460640322, -0.00262076359232678, -0.00270891496297762, -0.00480223949501897, -0.122340482894467, 0.239188061792398, -0.23085120375936, 0.106433229417867, -0.000737780502525553, -0.000621569734380975, -0.00109018984479878, -0.0235470671456198, -0.0213408121217803, 0.0567285276972927, -0.0929049606346436, 0.521879511054613, -0.001084491129715, -0.00084978085768393, -0.00144195494716553, -0.0304844426488356, 0.0225595452321252, -0.0513596430982692, 0.0738639646439453, 0.0162777402116468, -0.0552209333838475, 0.0250714592272949, 0.0053504751740065, -0.0206694422318369, 0.362180965134431, 0.00655089853965136, -0.270767634055307, 0.122934171541447, 0.000271898877739651, 0.000309623694825095, 0.00562132130380544, 0.000332901640170635, 0.00405603955611847, 0.00624723266456596, 0.00786486339244155, 0.00895607985992302, 0.162600612797479, -0.0303827855030739, -0.37018075297663, 0.532383334467844, -0.00286561887202116, -0.0349144074259302, 0.118444338190401, -0.0375393217377863},
    {0.00932686977392249, 0.02330143749075, -0.0195929661997851, 0.0197079603842469, 0.0976637162240657, -0.196492083346238, 0.196789453010076, -0.0984800038037923, -0.020009268681567, -0.004838265503167, -0.00354803175299672, -0.00500452794251927, -0.101599268420437, 0.168011170718831, -0.123020984383693, 0.0252004138115011, -0.00968427887924619, -0.0030573908049978, -0.00351420065220267, -0.0245389584623717, -0.0491729940243791, 0.106169413017208, -0.121847957868656, 0.123566481166195, -0.00249885878459795, -0.00191419023133717, -0.00300028765102186, -0.0253162076693629, 0.0944563035738695, -0.181156214573332, 0.19494018247284, 0.0264889132374169, -0.0746609519959466, 0.0285475625477052, 0.0596887115499931, -0.179428858511902, 0.955860192068403, 0.0262363368356125, -0.366088874082172, 0.139978727185895, 0.00303324343284998, 0.00323502747564214, 0.0235363446423738, 0.00133327046843723, 0.00660042969678799, 0.00711339789299559, 0.0154016280466348, 0.016426208777337, 0.119508385589293, -0.0462984786883526, -0.229203196864413, 0.246643004355891, -0.0260291213928502, -0.128858614879586, 0.363197492232307, -0.0531072416281612},
    {0.0190977808372876, 0.0202336576842804, 0.010730605592219, -0.0116539403654932, -0.0616020791184594, 0.199029865952139, -0.142163327346853, 0.0628400509032686, -0.039595919990636, 0.0141639916635123, 0.00370744537983262, 0.00319338727160602, -0.0702518096504063, 0.0825726724762285, -0.0302591072534673, -0.00555414878956782, -0.0433432647799655, 0.0190435548721706, 0.00697704744749578, 0.0156582995463822, -0.0769004177114559, 0.111019355037725, -0.0569446628060191, -0.0272339425429253, -0.00390790286721713, -0.00289524874373817, -0.00405247587043015, -0.0175051398490236, 0.148388770431383, -0.200999967524955, 0.105961643569102, -0.0611719587217119, 0.119201100977507, -0.0230342679574021, 0.19950941508979, -0.378262163481966, 0.519567159982301, -0.115119607905565, 0.584484870331949, -0.112945107132979, 0.0101386109265571, 0.00986461372661274, 0.036975078526123, -0.00585011446225268, -0.0152426492298348, -0.00573961131991814, 0.0179881100148822, 0.0175019791423124, 0.0656018645113012, -0.034104763467522, -0.0888609872432726, 0.0468450635759487, -0.0612886051163676, -0.159689304474723, 0.31117429138268, 0.0947022088715309},
    {0.0283483789400612, 0.015605997316105, -0.00366448453027248, 0.0118223838397412, 0.0220838867907565, -0.0613915956629153, 0.29971631803346, -0.0767262716388461, -0.0508705214556543, 0.0541344006325358, -0.015500258893724, -0.00389905316318783, -0.0377420092156797, 0.0230901301043675, 0.00135422408082107, 0.000832624912029102, -0.102213293977142, 0.124878367918978, -0.0445130476343746, -0.0191184272948391, -0.0758343923526162, 0.0532647951908287, 0.00388900865659831, 0.00408265242310424, -0.00385372990326825, -0.00275466521408041, -0.00342928295650859, -0.0094044431423382, 0.13231264484422, -0.092595642078496, -0.00762521338174643, -0.161063201374778, 0.158264250550675, 0.0647529036970648, 0.305221577224943, -0.265912605385222, -0.0373890997493766, -0.462535110161498, 0.776025214555274, 0.317506232833687, 0.0155106605694831, 0.0137520514519181, 0.0329692767107086, -0.0235049735356563, -0.0401332560488801, 0.016134938594296, 0.0115077156161065, 0.0102029630870848, 0.0244606642480476, -0.0100256563422968, -0.0171181742635493, -0.00140967467297037, -0.0574497025764958, -0.098091734497678, 0.096387099678759, 0.167658158602435},
    {0.0290142634016862, 0.00987675823579523, -0.00428139506100082, -0.0295941247801588, 0.0263560489909357, -0.0712908034861193, 0.124137438646549, 0.383808683073226, -0.0448012227061955, 0.0784639261814977, -0.0636937172730277, 0.0195042770596191, -0.0136909752155198, 0.000869868924945432, 0.000405518905338962, 0.000409298247694177, -0.139268112846256, 0.263309019711057, -0.240581629561091, 0.0956363217673718, -0.0425594697223883, 0.00291910365758022, 0.00153171149747913, 0.00200693308424884, -0.00216277464680906, -0.00149776380988467, -0.00168582891470653, -0.00341147704251061, 0.0665553353467772, -0.00480255754658157, -0.00307299187277484, -0.132383333906543, 0.013111565844646, 0.147499795774797, 0.223346204549638, -0.0181400485021904, -0.0150679586141473, -0.500033591404991, 0.0642906130875655, 0.723243311516806, 0.0113499419004006, 0.00930748316716329, 0.0165840632254412, -0.0254105603546717, -0.0329868908038958, 0.0367535664289513, 0.00346847170388689, 0.00284430900906819, 0.00506798665030044, -0.000281707249352406, -0.000365700171241807, -0.000233998998075136, -0.0215539605020869, -0.0279804196188504, 0.00277124849002808, 0.0763899465305751},
    {0.0171539738450263, 0.00422403562884661, 0.0293017672320873, 0.192230415140459, -0.182241449919052, 0.488039973885573, -0.759220407776989, 0.964112153400919, -0.0235420783828846, 0.0542965286233318, -0.0668187951761436, 0.0489939685728521, -0.00231153056297324, -0.000876578948905533, -0.000693434004591417, -0.000909320668327648, -0.0977269276576106, 0.232125215917278, -0.298701660335599, 0.240234638216594, -0.00959553257924229, -0.00374749699368054, -0.00309987463794573, -0.00445871865745549, -0.000487622959599563, -0.000329816258202443, -0.000344874274678643, -0.000575980404938949, 0.0137673862153514, 0.0058953788974484, 0.00628854087800174, -0.0358548234884832, -0.0207029333243838, 0.0594974335725776, 0.0588574919775216, 0.026354253475136, 0.0308349249253133, -0.160282676136513, -0.101513754490212, 0.291736816703684, 0.0029910027604684, 0.00231943115192509, 0.00343051691130132, -0.00814519801425978, -0.00893419973878654, 0.0148253959652971, 0.00029367816138888, 0.000227738431127063, 0.000336832821567559, 0.000131498445572864, 0.000144236318875335, 0.00015385541847579, -0.00206529017058808, -0.00226534884361136, -0.00130803505644944, 0.00795527596810652},
    {0.0296266593660007, 0.0106036055567763, 0.259211232824319, 0.0195251039488212, 0.326169225832269, -0.374897896525802, 0.278859093198177, -0.113874016814714, -0.00174302461810547, -0.00103224751682512, -0.000995395188472054, -0.0015202593654192, -0.0607551610563968, 0.145274827975367, -0.188334660306985, 0.151780397361144, -0.00517932982837496, -0.00333411005738375, -0.00365885144162152, -0.00745436569613287, -0.180531596982921, 0.469230738890095, -0.692276345467664, 0.744232604758411, -0.00241016220156659, -0.00167493200389353, -0.00190612042688981, -0.00397712490619116, 0.0107073556307269, -0.0291016110858973, 0.0519163834616734, 0.00720759150771036, -0.0269570575484029, 0.00956421589618278, 0.0345842460403174, -0.106971053214144, 0.25456426498465, 0.0264935041720337, -0.132179922471374, 0.0468967101986916, 0.000461712209768428, 0.000381836111040756, 0.000700919724644963, 0.000353697875660614, 0.00047181986189386, 0.000626088065400543, 0.0160935189181037, 0.0133093441036108, 0.0244313765371238, -0.0497781754902045, -0.066402241860369, 0.118459567099648, -0.00366885739980116, -0.0048941198429574, 0.018304459973106, -0.0045334627648615},
    {0.0287142087112329, 0.00958841840105437, 0.00525730855264615, 0.0197079603842469, 0.692635678336204, -0.462466964864018, 0.306530718648313, -0.118670820076184, -0.00799167084001716, -0.00185631097078687, -0.00128014616296254, -0.00158429842618396, -0.0488981962979723, 0.100408707642264, -0.104468874747729, 0.0565787251628148, -0.0252758904406239, -0.00632110314145261, -0.004885668829183, -0.00776837170631504, -0.154654198992137, 0.341911353913747, -0.398704725867906, 0.277425364104441, -0.00206468956655645, -0.00142790422348551, -0.00160037367631403, -0.00320095002602278, 0.043670470135928, -0.101811651647421, 0.145248246697174, 0.0115168069922353, -0.0368162525456522, 0.0108902736390702, 0.148706520787247, -0.388563452590706, 0.712203175478731, 0.0439538129017001, -0.180523018821064, 0.0533988371215843, 0.00198528590849253, 0.00162273463193141, 0.00285873514978347, 0.000586799320677608, 0.000753907637346223, 0.000712893814650173, 0.0121472595661696, 0.00992893703459015, 0.0174915853413359, -0.0317402430744735, -0.0407792081924056, 0.0581771181962357, -0.0138046925395084, -0.0177359836148675, 0.0566973513015149, -0.00641353363141378},
    {0.0262660176362191, 0.00787931986789131, -0.0132573892568216, -0.0116539403654932, 0.130691148487769, 0.574537076169525, -0.227302951843894, 0.0757238026633065, -0.01505619621431, 0.00521607980877915, 0.00130030386482768, 0.00101094018990619, -0.0321897964657499, 0.0486197142564781, -0.0306355733076926, 0.00373028648217903, -0.0524224823609095, 0.0192586102361634, 0.00525321344488675, 0.00495699486804402, -0.112078045039238, 0.179511848166742, -0.123767382336513, 0.0182909049745178, -0.00149628249178237, -0.00102691121281123, -0.00112365340857564, -0.00210719285445267, 0.0658499558828103, -0.112715482697028, 0.0968327727428127, -0.0258534416707461, 0.0603353220350685, -0.00878707177583321, 0.243128686850451, -0.455369321869516, 0.474805099586673, -0.10444762263269, 0.295845278163648, -0.0430860995861488, 0.00324585602164065, 0.00260497492156441, 0.00431063789576671, -0.00139441358919901, -0.00169240546797397, -0.000575215033661466, 0.0069395644960075, 0.00556937564640963, 0.00921604331712943, -0.0129974986479507, -0.0157751171905967, 0.0135522494457506, -0.0176036557524687, -0.0213656288798233, 0.049861914532043, 0.0114367792968685},
    {0.0214698480958179, 0.00569928745007793, 0.00832487477036396, 0.0118223838397412, -0.0614326220634956, 0.26203544501963, 0.518569562880659, -0.0924570392473859, -0.0182709161061165, 0.0190137982809253, -0.00526293094004182, -0.00123433495846093, -0.0163348457295087, 0.0140319672776631, -0.00196259356847574, -0.000733241121874825, -0.0709478783134303, 0.0769033452699533, -0.0226501029881941, -0.0060523778910262, -0.0634299145349295, 0.0567537957659614, -0.00844642404705154, -0.00359533878904277, -0.000846812331002415, -0.000575990819905163, -0.0006128888517879, -0.00106930375395291, 0.0560006137916713, -0.0543971191883836, 0.0103033153809036, -0.0658993199929184, 0.086686871895592, 0.0247018231068586, 0.226500485285503, -0.234109161969773, 0.0505207745986585, -0.283611242802991, 0.425056183742284, 0.121121715799415, 0.00302386350863112, 0.00237595963988631, 0.00366588503758726, -0.00378631280488674, -0.00431386934521675, 0.00161701877171255, 0.00270344101158925, 0.00212419201925463, 0.00327743098393716, -0.0027942558659869, -0.00318358660367831, 0.000603000624105149, -0.0111516824754385, -0.0127054746021772, 0.0167133416449516, 0.0202473562137981},
    {0.0143344773818663, 0.00339108928439715, -0.0107414954878784, -0.0295941247801588, 0.0716512169321643, -0.219640225699807, 0.508137289414865, 0.462498877065499, -0.0152353726346625, 0.0263431259843042, -0.0209745648569958, 0.00617452750875364, -0.00561039559867238, 0.00119544438383654, 0.000411420418441708, 0.000352254883283904, -0.0655314248951066, 0.115979237022268, -0.0956568846043597, 0.0302758773259643, -0.0241318165707197, 0.00526310839581209, 0.00187632953336965, 0.00172722943069841, -0.000322168490867553, -0.000217247209950859, -0.000225042499060239, -0.000367264997427105, 0.026926260071335, -0.006473620370019, -0.00299718572753369, -0.0525321970635773, 0.0169905218832957, 0.0562679610565776, 0.118546565081049, -0.0295236807499636, -0.0146962544553076, -0.239579049519497, 0.0833104971211989, 0.275901578528233, 0.00158263957699806, 0.00121865954262168, 0.00176263378613845, -0.00319846707772597, -0.00343883722280913, 0.00368338599466447, 0.000582803868995854, 0.000448768946986602, 0.000649086377665954, -0.000145145625740528, -0.000156053562033028, -7.22503764697957e-05, -0.00326926866673882, -0.00351495966953633, 0.00113684563986322, 0.00922528597146999},
    {0.00638882353038111, 0.00138044771170165, 0.0263784345157957, 0.192230415140459, -0.168273534214414, 0.466883678827404, -0.770250380155342, 1.16177879234705, -0.00766460049150915, 0.0175880306956049, -0.0214801029408944, 0.0155101676309963, -0.000906860003956126, -0.000288071461031755, -0.000213253491124332, -0.00026580282292845, -0.0357169332052306, 0.0827155276170139, -0.102424732574839, 0.0760518002123164, -0.00422595518496751, -0.00135478401778058, -0.00101686811553746, -0.00130332461042046, -5.6418032203541e-05, -3.77810915961271e-05, -3.83065599431215e-05, -5.93644300409396e-05, 0.00537393653439492, 0.00192270868623445, 0.00190448281304134, -0.0138893267054968, -0.00664920623945975, 0.02269697566456, 0.0252733238596128, 0.00916815545757754, 0.00933834822750573, -0.0662292251284654, -0.0326033938848808, 0.111291244539186, 0.000337408026585495, 0.000255673905629333, 0.000351786028768619, -0.000884184141232522, -0.000909216372899799, 0.00148577842015651, 3.99214342172854e-05, 3.02508185948576e-05, 4.16226103101686e-05, 1.4481906575825e-05, 1.48919053797741e-05, 1.47507410001302e-05, -0.00027015812866913, -0.000277806604307325, -0.000132993732949449, 0.000960724534063706},
    {0.00337366876311364, -0.00634204581202926, 0.0208770933958504, 0.0195251039488212, -0.00350519620228991, -0.00213578148775574, -0.00215103292273904, -0.00360148326124408, 0.00186672700723231, 0.00270775846140311, 0.0252970714511431, -0.0453962493417858, 0.00205786663823846, 0.00293953537827712, 0.0272677169715594, -0.0487065445939726, -0.00203791177178541, -0.0492117221773477, 0.0992013086517272, -0.0738509092732363, -0.00224657951086715, -0.0534241146055897, 0.10692910492615, -0.0792361187978445, -0.0283178557954039, 0.628290543629044, -0.135565454749909, 0.0421979810464191, -0.00088831104609588, -0.00570497820444094, 0.00883182654289624, -0.00579324153221626, 0.00620921957215449, 0.00903250928257807, 0.0161444667353695, -0.0223718110929615, 0.0143676720035994, -0.0227179316958211, 0.0101011981811169, 0.014694143970245, 0.203498998675296, 0.263102111421653, -0.0182154333958999, -0.286356708329364, -0.118794431005032, 0.185217860295405, 0.224335855573074, 0.290041905135055, -0.0200805648288411, -0.310868080347518, -0.128962918095965, 0.199646358368677, 0.093942584155313, 0.0389718679788593, -0.0417702048278289, -0.0424164434544871},
    {0.0115772632577271, -0.0106853276693848, 0.019287579585398, 0.0197079603842469, -0.0163810014243473, -0.00390242657177244, -0.00279689872237231, -0.00375319132544435, 0.0146717000428919, 0.00719541733809581, 0.0244226254420095, -0.0473085106547024, 0.00283917689101551, 0.00355123466570265, 0.016817976125512, -0.0399568305836848, -0.0274010505067603, -0.048121740386427, 0.105826965373029, -0.0769617882285365, -0.00530248227273661, -0.0237500598790073, 0.0728748586553496, -0.0650020279883699, -0.0668371306827621, 0.207386421256723, -0.134368965523375, 0.0582192890483208, -0.00535364888165075, -0.0177228274895518, 0.0365988305517864, -0.00694898923818512, 0.0100310086012753, 0.0102848470593367, 0.0358043028635586, -0.0767957177864591, 0.0595392120224926, -0.0301110314791141, 0.0163185090590721, 0.0167314550889352, 0.451308792072928, 0.670583112662057, -0.109780279168362, -0.379545813158116, -0.142493838383863, 0.210897913990534, 0.0873344935774071, 0.129767107520561, -0.021244002453216, -0.187321483324288, -0.0703265751992813, 0.145229106953919, 0.282395714198688, 0.106020532605796, -0.153042815009205, -0.0600069529033938},
    {0.00318614435970195, -0.0102352418729336, 0.0163965439250797, -0.0116539403654932, -0.0318063021955915, 0.0112436886986536, 0.00289018334287365, 0.00239490987846146, 0.0285443214023187, -0.0170879000112395, -0.0117215303676542, 0.0301875416619842, 0.00193009838737005, 0.00202524321821193, 0.00540450880422605, -0.0271088739443334, -0.0126571512820065, 0.0143537935251019, -0.0723998288978306, 0.0491092861808114, -0.000855846139544765, -0.00170119926809071, 0.0333817769889968, -0.0441008898133743, -0.0107878343256711, 0.00689944426242837, -0.0507328584660814, 0.0395780045482836, -0.00682270690696087, -0.0129194301108834, 0.0565868352835748, 0.0073707993616605, -0.0229083726068188, -0.00829856919202057, 0.00573105683907255, -0.0797988402663343, 0.0920558261790192, 0.0455268719942118, -0.0372674873257959, -0.0135001655287293, 0.0722392598788913, 0.323635014986764, -0.139904331697434, 0.573860568794342, 0.151143347183347, -0.170167910286462, 0.00488464507640123, 0.0218834216346687, -0.00945999455336019, -0.0680134612436616, -0.0179133795644633, 0.0784602300632857, 0.229126016875703, 0.0603471905904601, -0.187558480429118, 0.10700595273598},
    {-0.011801561675532, 0.00496641091476397, 0.0123418011338163, 0.0118223838397412, -0.0399845505836983, 0.0422013001459018, -0.0119330872043856, -0.00292413044299179, -0.0204347654762904, 0.0301742710270641, -0.0150525237472739, -0.0368583011689746, -0.000577804782105243, -0.000512427300870599, -0.00090401952533086, -0.014250899101052, 0.0594964820405094, -0.120624902691672, 0.125434652322348, -0.0599613204849069, 0.00168229735160553, 0.00204848339993455, 0.00753331313448471, -0.023183453960041, 0.0212051492401301, 0.00258397885603979, 0.00538637983819935, -0.011848287343186, 0.00281071914707328, 0.00358016746981978, 0.0495501608220378, -0.00596100303954864, -0.0595664075674236, 0.023328566495279, -0.011236152923305, -0.0298340045411961, 0.0806085190825288, 0.0496738192420156, -0.0969030134598642, 0.0379510614356348, -0.141630312498688, -0.0376329322111705, 0.0576357432941008, 0.62613232396099, -0.122234496933111, 0.478368417328807, -0.00400467878859442, -0.00106409286769063, 0.00162968389014105, -0.0106331416082053, 0.00207581794703586, 0.0287296932280311, 0.0583239313380668, -0.0113860858694687, -0.113777534921342, 0.189440364791817},
    {0.00800383521292127, 0.0263496071105023, 0.00763236993753088, -0.0295941247801588, -0.0344990911194155, 0.0601371945948342, -0.0484704728084982, 0.0146274102792546, -0.135681023269177, 0.282736312981598, -0.302623682224074, 0.184376690406239, -0.00158021534532795, -0.00122388284394244, -0.00162527582784931, -0.00506457947298111, -0.0455282927907745, 0.109434345863475, -0.15080433156421, 0.299945181214723, -0.000530247378602331, -0.000473709291275571, -0.000809912274627619, -0.00823909033431843, -0.00668370237087965, -0.00295758720150996, -0.0038078199674851, -0.0324034103825531, 0.00914001856619172, 0.00890950075286587, 0.0245045777907788, -0.02397133806597, -0.0483955236813235, 0.0531398377109131, 0.00353768478630589, 0.00443980886007413, 0.0398642041456056, -0.011945468336023, -0.0787301479845029, 0.0864482284436403, 0.0445920774865157, 0.0277197895493028, 0.187422412635166, -0.150571145206208, -0.491548893679026, 1.08966918597521, 0.000519343703536485, 0.000322839817681629, 0.00218282384204108, 0.000651778469724282, 0.0021277714616437, 0.00585219562230551, -0.00486751435712141, -0.0158903041743695, -0.0320807954006469, 0.0863145549122675},
    {0.0302224845101156, 0.0261153899063016, 0.00320507108782466, 0.192230415140459, -0.0178382131382485, 0.0410650708630083, -0.0503948504088267, 0.0367434731025106, -0.15273114932588, 0.375858140305486, -0.510844558915112, 0.463146916325963, -0.000571524833485264, -0.000402773807050394, -0.000446079687283208, -0.000841390117526122, -0.183081952923274, 0.472755167242471, -0.693290980381532, 0.753450370761892, -0.000685098508853392, -0.000506609750046235, -0.000605395551988712, -0.00136877883379712, -0.0086355816411185, -0.0051354222148678, -0.00554103138111584, -0.01171951264617, 0.00363208529783313, 0.00299171446345375, 0.00500199470056434, -0.0104469704798225, -0.0129905137821949, 0.0214351751990888, 0.00456844460258446, 0.0040601952535117, 0.00813727702558389, -0.0141780709602839, -0.0211330510483064, 0.0348708803444415, 0.0575846769898423, 0.0411575515467008, 0.0744784252336857, -0.178712824081338, -0.214222367041931, 0.439543117114767, 0.000215483698467328, 0.000154012870971259, 0.000278700643371266, 0.000191510670662762, 0.000229563095955929, 0.000383817842058293, -0.00172698194151784, -0.0020701265354201, -0.00257414408714914, 0.00898882818456415},
    {0.0117859544381791, -0.0110271230805097, 0.0160091858217584, 0.0195251039488212, -0.00316207558373215, -0.00241577235429526, -0.00353354291864167, -0.0181912548602126, 0.00295045175569064, 0.00361626834906969, 0.016089212404908, -0.0398420632543444, 0.0164288131844466, 0.00644900529020177, 0.0196160926625834, -0.0385628463970881, -0.00533638736500265, -0.0215894746779392, 0.0708866405450278, -0.0648153237617092, -0.0297142669526367, -0.0385011904458149, 0.0864255424365169, -0.062734285582657, -0.0650794963063692, 0.175306723361565, -0.129373925944914, 0.0585357554359557, -0.00599234830794817, -0.00596048830774341, 0.00876741507713452, -0.0186109023749111, 0.0089520433947813, 0.0400416091578109, 0.0357749037316649, -0.0262610115094858, 0.0142628870184278, -0.0819968258027645, 0.0145632415484494, 0.0651399463159163, 0.0783533620054186, 0.119573754125016, -0.0213506983799166, -0.179587540573384, -0.0663105251505073, 0.142667995223338, 0.436290050931204, 0.665814943251965, -0.118885738214351, -0.320264119644222, -0.11825364884818, 0.173941927289884, 0.297586072438545, 0.109880054473007, -0.0528534831921601, -0.165028595825849},
    {0.00868942826669689, -0.0116958193796847, 0.0205403356838656, 0.0197079603842469, -0.0201247738104083, -0.00584323645334017, -0.00577935460978638, -0.0189575391547738, 0.0148427436855896, 0.00638151686995207, 0.0144742293073116, -0.0415203612920179, 0.0145079970177744, 0.00650803540571881, 0.0149243543543366, -0.0430827522936908, -0.0167229445882328, -0.0161405937908936, 0.0744586814191187, -0.0675455897618916, -0.0163457939686741, -0.0164605936176501, 0.0767742256020669, -0.0700872974533167, -0.0358001777363248, 0.0412250887240751, -0.0838466866383685, 0.0516918998203714, -0.0239827552691316, -0.0217039465904849, 0.0480958691343093, -0.0208020433238823, 0.0181915227080136, 0.0455932912233091, 0.0606589183534295, -0.111649968396047, 0.0782426680475976, -0.107010375739815, 0.0295940856905752, 0.0741714582685035, 0.132853752011542, 0.279624171526345, -0.0854504023725405, -0.234371635813971, -0.0741175462222495, 0.162448602623016, 0.129857516831937, 0.273317840187487, -0.083523249410554, -0.239018235798383, -0.0755869842279404, 0.167500490591787, 0.880805880216167, 0.278545525839862, -0.243589881034135, -0.233467551046818},
    {-0.00515804432869904, -0.00575929267843153, 0.0267098865309273, -0.0116539403654932, -0.0579732173172456, 0.0239119304978302, 0.00786274267358259, 0.0120967981262488, 0.0151788194266244, -0.00831999241146501, -0.00467132225531164, 0.0264941258766691, 0.00518416692938194, 0.00268262576567047, 0.00458054935471386, -0.0433658185196246, 0.0180568823364596, -0.0159481203670532, -0.0486788032710565, 0.0431008137183391, 0.00616713919741463, 0.00514217279233525, 0.0477328791988052, -0.0705477914032103, 0.0135071248184838, -0.00305500856414899, -0.016608328329972, 0.0184711533395948, -0.0167792370213337, -0.0140070776838757, 0.105619250345273, 0.0148371797724691, -0.0546971571990114, -0.0367880124736601, -0.0321631653551498, -0.145964620231556, 0.171822073141542, 0.15461492822938, -0.08898168576429, -0.0598469743850445, -0.0704430166245739, 0.0867188215718508, -0.0597843133070969, 0.338634019363569, 0.0528647758526336, -0.131075451218333, -0.0240590751445701, 0.0296179059986184, -0.0204187065694202, -0.109186198802953, -0.0170452571090385, 0.128528399601262, 0.68293577606614, 0.106614352543, -0.393033048742583, 0.416325384375381},
    {-0.0103543496131472, 0.0114965700457507, 0.0302516266319403, 0.0118223838397412, -0.105865563934372, 0.12409022809239, -0.041339453116336, -0.0147699151361855, -0.0456650504879514, 0.0640339164783739, -0.0289985647742542, -0.032348724573381, -0.0065219381210536, -0.00376718935977035, -0.00512102322348483, -0.0331151410630412, 0.0487160015083047, -0.0883080091353926, 0.0683057070994246, -0.0526251123873067, 0.00695767866119288, 0.00519526230305947, 0.012062497405501, -0.0538719236429515, 0.0152385459635856, 0.00364434309255477, 0.00622457516001475, -0.0232376234689826, 0.0301281169318005, 0.023893646693619, 0.127872945220575, -0.0580053142400196, -0.181106971901399, 0.10341681503917, -0.0415491378876404, -0.0562811451289651, 0.208024526539326, 0.136630693123355, -0.294625982202666, 0.168239137275043, -0.0909999553413712, -0.0394797780221855, 0.10734628634863, 0.299245365959432, -0.20667256059336, 0.368473445107362, -0.0129967244405359, -0.00563854997514032, 0.0153313273413633, -0.0176049509478884, 0.0121587857504662, 0.0650708434821411, 0.140795688796734, -0.0972399536775844, -0.303607243389941, 0.737050889895457},
    {0.0142799617871942, 0.0286331249853376, 0.0261981528295216, -0.0295941247801588, -0.123694165861968, 0.228667944176925, -0.201694509957147, 0.0738837109693798, -0.144761735858263, 0.294022599521717, -0.301560576395922, 0.161818385181666, -0.00851593238442563, -0.00529609529109965, -0.00615830015627299, -0.0159370083754821, -0.0786155180524537, 0.180183183238993, -0.22752856333055, 0.263247185749171, -0.00462473340853539, -0.0032455576878773, -0.0046464601038415, -0.0259264273302238, -0.0101289835370485, -0.00485254778273549, -0.00597444597327726, -0.0303422121712233, 0.0480095304404493, 0.0369438237145959, 0.0817770038721055, -0.120654962207571, -0.176743667493169, 0.235571815733862, 0.029421241886308, 0.0278742507862932, 0.133035353826841, -0.091034612474503, -0.287527730636603, 0.383229739093874, 0.0644377196220507, 0.0416757755850919, 0.171057647372711, -0.199381890716993, -0.429892852309207, 0.839340860388867, 0.00379069275492264, 0.0024516705664447, 0.0100628481015951, 0.00359137526936534, 0.00774346432721741, 0.0171405460669789, -0.0325561061198112, -0.0701951279005206, -0.102826639852128, 0.335821880299531},
    {0.0296919171784481, 0.0243834140019697, 0.014002682081672, 0.192230415140459, -0.0792712548026677, 0.186695351837455, -0.237148368754727, 0.185592944676423, -0.141645882360695, 0.345411287467402, -0.462849575846567, 0.406481350416947, -0.0026772732438634, -0.00174183802117238, -0.00183748784347617, -0.00328158485493611, -0.177845002139631, 0.452155668420226, -0.646858537591111, 0.661266465096995, -0.00336147904794568, -0.00228012796141555, -0.00256799349356308, -0.00533850326641928, -0.00736223322060817, -0.00445019405091739, -0.00475113027769324, -0.00953910730364736, 0.0168597007475027, 0.0127557271340271, 0.0199584129431104, -0.0478104381081744, -0.0536515356544616, 0.0950233075549246, 0.0220699483121886, 0.0178268522440284, 0.0324684740451171, -0.0668177993242804, -0.0872806619365436, 0.154584525524274, 0.048337087431914, 0.0347932015858052, 0.0600710050445718, -0.146342789854298, -0.170348282676651, 0.338567432066033, 0.00091362762341507, 0.000657632301917815, 0.00113541242327322, 0.000737976565159913, 0.000859031324028661, 0.00134409443819929, -0.00714306605782539, -0.00831478638071548, -0.00933062058445968, 0.0349726090309795},
    {-4.1621401110515e-06, -0.00888246788236116, -0.0100835618309505, 0.0195251039488212, 0.00276855422334095, 0.00354739278284653, 0.0184970121822559, -0.041398111534134, 0.00207617681188797, 0.00204132940906948, 0.00495444159751799, -0.0310074355429239, 0.0263087298629871, -0.0190562037323708, -0.00887004256151634, 0.0262768860950637, 1.34764571128876e-06, 2.34589687808998e-06, 0.0351417498325346, -0.0504430947992082, 1.7076988225069e-05, -2.18993997978083e-05, -0.0629150249458782, 0.0427474066498151, 1.27907673087603e-05, -5.86577422620124e-06, -0.0362041359827394, 0.0320568669764366, -0.00769781311112103, 0.00960799969344636, -0.0100195650228907, -0.0130420310882866, -0.0364701390904391, 0.0709175009299217, -8.84633105615457e-06, 0.0681493393296446, -0.0162998925724397, -0.0925068516383988, -0.0593298559286744, 0.115369044865993, -6.62595538441016e-06, 0.0182538627477391, -0.00937969153956938, -0.0692881905275171, -0.0158915560681607, 0.0864121113201357, -8.39622469953893e-05, 0.231307825632447, -0.11885681869661, 0.646818621763509, 0.148350740803415, -0.154705447658132, 0.261284405700932, 0.0599267458324432, 0.167576410526233, -0.227470527214667},
    {-0.0068129511163931, -0.00438186087957976, -0.0118196797021562, 0.0197079603842469, 0.0153251463175155, 0.0068247368416488, 0.0170656692030986, -0.0431419561966856, 0.00496290698942556, 0.00178381589036705, 0.00264840080550213, -0.0323135857262808, 0.0110394459580145, -0.00537180196847265, -0.00318945341855556, 0.0255329556369081, 0.0101073362006734, 0.00568346715796609, 0.0351451051021586, -0.0525679482857169, 0.0224826683241414, -0.0171152528866811, -0.0423250421086932, 0.0415371758142848, 0.0168396543479754, -0.00265609563809129, -0.0110627097316331, 0.0134514304724193, -0.0152560834038925, 0.0117265687773735, -0.0437183995785951, -0.00866187936743286, -0.0418058504080334, 0.0807500582702354, -0.0486078465011176, 0.155615226861536, -0.0711213725288545, -0.114945842078814, -0.0680100252848931, 0.131364712142299, -0.0364075705728996, 0.0241497410540253, -0.0185893518541938, -0.0860951298768104, -0.0105543945334429, 0.098392962708179, -0.0809846746389744, 0.0537184681949511, -0.0413499881472977, 0.259267781302797, 0.031783614910571, -0.118494062764005, 0.736328501815995, 0.0902664474228086, 0.435663605754062, -0.321804755462983},
    {-0.0117909391625216, 0.00649963867312389, -0.00729939888431999, -0.0116539403654932, 0.0195387056426204, -0.0109690858690787, -0.00648860340814748, 0.027528864934527, -0.0157527489567286, 0.0076128285242191, 0.0033167616031395, 0.0206192860832164, -0.0122437712383401, 0.00202563119535561, 0.00149192775061576, 0.0113747003386201, 0.0347177461769007, -0.0226829837593206, -0.0194855585338848, 0.0335435867035136, 0.0269842516546398, -0.00603551746377202, -0.00876488846392666, 0.0185044353861292, 0.0202113674476524, 0.00308515890396066, 0.00608305867108546, -0.0149188861614174, 0.0349392194237379, -0.00820727779862277, -0.0377070803306021, -0.02397417434508, 0.0351290133199049, -0.0651550715289333, -0.104103979779745, 0.0482167279666834, -0.0613421198629883, 0.140845268185824, 0.0571481039328643, -0.105994687797817, -0.0779745095406278, -0.0246468125557043, 0.0425730134126714, 0.105493956438009, -0.0292122395057679, -0.0793906612641798, -0.0606054257488449, -0.0191566523097159, 0.0330897317403693, 0.0280699674769364, -0.00777283022214865, -0.0357110775063413, 0.484166493457688, -0.13407012159853, 0.196451023490723, 0.57385057542787},
    {-0.0003961809862403, 0.0218166735098089, 0.00962857943557122, 0.0118223838397412, -0.0386200930353358, 0.0549301045311348, -0.0255353246076025, -0.0336121174078451, -0.0821315304265297, 0.106627813860814, -0.0421122676094755, -0.0251756789225904, -0.0266944218436793, -0.00387484689204713, -0.00329201428801519, -0.00940176168823116, 0.00168506853674359, -0.00261416436528553, 0.00143665582940332, -0.0409559557664377, 0.000547681628745361, 9.49985402433941e-05, 0.00011230674066787, -0.0152948461494822, 0.000410216847388328, 0.000156669398596571, 0.000218004723301832, -0.0325268279583356, 0.114169552842125, 0.021433407310903, 0.0440530051317887, -0.191698087141148, -0.0870634463072911, 0.183161022520475, -0.00279906307588779, -0.000731198562916304, 0.0716657109865359, 0.00653976121473041, -0.141635372249502, 0.297967524909641, -0.00209651514742682, -0.0012058757830718, 0.139114209894575, 0.00489832064355568, -0.233581784872354, 0.223179475587969, -0.00068141016557749, -0.000391934214268572, 0.0452149544041175, -0.00017800461094241, 0.00848834483592531, 0.0174464607140272, 0.00524477673604143, -0.250102923105532, -0.113589148133034, 1.01592911016151},
    {0.023673129138354, 0.0302602001667236, 0.0280757815135756, -0.0295941247801588, -0.142669865790111, 0.291843625603676, -0.302869456861528, 0.168138269227149, -0.148692369932632, 0.290316139028518, -0.279210915864665, 0.125936579040831, -0.0199060448288925, -0.00785249482412045, -0.00727718818584341, -0.0143058565620665, -0.123983018812677, 0.264604231380493, -0.294894932128272, 0.204874433632237, -0.0165981047421918, -0.00715703702973118, -0.00768596711021545, -0.02327285910972, -0.0124320806881217, -0.00659796760180466, -0.00764573936306601, -0.0242552732279389, 0.10787872295819, 0.0513642644623301, 0.0812269504449895, -0.254226540885044, -0.206551947025893, 0.417220107102282, 0.0983244220117469, 0.0542495311672528, 0.132140523387534, -0.268507118668093, -0.336020030755683, 0.678736343273606, 0.073645585869555, 0.0500118481387927, 0.131448910284485, -0.201113453401185, -0.309771944349812, 0.508377620011562, 0.00985923039920341, 0.0066952869974271, 0.0175975936224406, 0.00543973324107434, 0.00837873704784167, 0.0132500536336069, -0.074731851268179, -0.115108315633292, -0.0935222838266846, 0.462886930472144},
    {0.0273991946635943, 0.0208837824772988, 0.0249036082205118, 0.192230415140459, -0.14494000231401, 0.354346112869683, -0.476703569824698, 0.422356647889497, -0.120035982431373, 0.28854446919111, -0.378174407346313, 0.316347679887784, -0.00516318952324476, -0.00292490831688955, -0.00284951473428297, -0.00466960557446201, -0.161228140099431, 0.400101823491238, -0.550948270007868, 0.514636432413085, -0.00693501587567336, -0.00405573932646444, -0.00415135234621378, -0.00759654426569465, -0.00519436877155121, -0.00321746450671809, -0.00338045124712302, -0.0062912835618737, 0.0320511755142852, 0.0209500829955213, 0.0294811350675108, -0.0888981780171576, -0.0839333290017819, 0.16829532188209, 0.0444428333845822, 0.0305213990124237, 0.0479600994071269, -0.129512458891891, -0.13654327736291, 0.273783907869764, 0.0332879505960334, 0.0242129757642642, 0.0390539670767369, -0.0970056138378372, -0.108321347400085, 0.205065800379851, 0.00143183730650082, 0.00104148922898282, 0.00167985490323329, 0.000983323393772473, 0.00109802835866288, 0.00154515485006053, -0.0107752678944201, -0.0120322060831619, -0.0113602228338168, 0.0482052081612837},
    {-0.0109437322728922, 0.0101644112236962, 0.0100401662714875, 0.0195251039488212, -0.00508376488671095, -0.0317222402070974, 0.0822967853898158, -0.0686320373360359, -0.00206888955864449, -0.00165092370827359, -0.00261316748166393, -0.020639741246323, -0.0434629513771953, 0.0685695955840055, -0.0418362555160875, -0.0321177209637211, 0.00265561165013069, 0.00277042286983613, 0.00774815294057041, -0.0335768632939073, 0.0557887295356482, -0.115066962106936, 0.124046280414075, -0.0522493142351212, 0.0167773679286114, 0.00365371796926725, 0.00645575293301402, -0.0212633870913166, 0.0103211392078777, -0.0225468980583688, 0.0596405772233314, 0.0114041815824651, -0.108008294229385, 0.0782597007030628, -0.017319952436912, 0.0668525135558669, 0.0970236731312838, -0.0338138843694443, -0.175708585038306, 0.127313382496836, -0.00520863652853863, -0.00212276595816228, 0.00504941268934664, -0.0101688635658782, 0.00557926969439204, 0.0382870066797381, -0.109422330078203, -0.0445947794744642, 0.106077377249782, 0.422354381832924, -0.231729827781458, 0.612966832642402, 0.0635730506039554, -0.0348801212954483, 0.330347458639909, -0.167089166495659},
    {-0.00766332463985577, 0.0155122697307478, 0.0110939404354585, 0.0197079603842469, -0.0231802053744836, -0.0276298646655858, 0.0870824133797188, -0.0715230777133198, -0.0156006160152442, -0.00470230194600604, -0.00496770894873693, -0.0215091650261775, -0.0575306459131932, 0.0722582786979664, -0.0242204129508018, -0.0257070439035827, 0.00895970133038244, 0.00338168273878017, 0.00558144257295909, -0.034991247469186, 0.0330408366068701, -0.0519648837128462, 0.027212714225664, -0.0418203837218571, 0.00993638458944256, 0.00296439209504614, 0.00463130748587514, -0.0281457276807841, 0.0666729850431346, -0.112240739741459, 0.117813620332084, 0.0269358648165765, -0.141998119943555, 0.0891102380811218, -0.0479481933002691, 0.126107477245155, 0.191659952362795, -0.0302636455110634, -0.231003451275632, 0.144965106220525, -0.0144194801926218, -0.00719393524937954, 0.0326184358075965, -0.00910119875154596, 0.0131778377235275, 0.0435954169258846, -0.0531749520918535, -0.0265291922543363, 0.120287537290538, 0.139854259344909, -0.202498240605752, 0.212552508933874, 0.129044050170326, -0.186845886872037, 0.984997691207882, -0.236382660307924},
    {0.00242217246301147, 0.0235602360807905, -0.00170955493305554, -0.0116539403654932, 0.00630079692715634, -0.00613847546289087, -0.0581932607657199, 0.0456388471838216, -0.0521782100505412, 0.0221414434455203, 0.00763849492268669, 0.0137249895707229, -0.0672348425655045, 0.0528374343948943, 0.00192278806264398, 0.00244161989758005, -0.00601102036289123, 0.00300848299555957, 0.00140208737896736, 0.0223279009667123, -0.00774557056222423, 0.00717932068418087, 0.000352938229631446, 0.00397204289231757, -0.00232932866944791, -0.000942363494893933, -0.00127304603535361, -0.0328933134587859, 0.168468784091727, -0.169517341690776, -0.0140459588870732, -0.0915342352082088, 0.209713222196366, -0.0719006779748152, 0.0228908053564657, -0.0311158320724684, -0.0228500559069324, -0.0168016078072763, 0.34116281345662, -0.11696848358163, 0.00688396145322232, 0.00408428952379189, 0.0824200118822444, -0.005052754531626, -0.0447813094524682, -0.0351759809093124, 0.00887040900954815, 0.00526285901448311, 0.106203269866522, -0.0120576866063493, -0.106864283990075, -0.00885461820277436, -0.038445163448872, -0.340729942564746, 0.780643154897168, 0.421523682717848},
    {0.0189589162037065, 0.0297613960356094, -0.011422245664625, 0.0118223838397412, 0.0552995939385817, -0.104936601954282, 0.0910163993406658, -0.0557239934719332, -0.105570386298514, 0.126084042471053, -0.0432981721365901, -0.016757899825139, -0.0568851719559672, 0.0163313630643681, 0.00646605237524107, 0.00896099560153867, -0.0724739320858043, 0.0961639620233353, -0.0394468229313448, -0.027261858799801, -0.0390515961300795, 0.0124558869364595, 0.00589090048204114, 0.0145778050557573, -0.0117440028107332, -0.00592549409332401, -0.00707806935557176, -0.0278299423469547, 0.223813500790995, -0.0785471478787501, -0.0560183754307317, -0.326757131883804, 0.206562994372508, 0.202123969612012, 0.17070195853941, -0.0715604211734565, -0.0911310520484137, -0.297692260133122, 0.336038002649934, 0.328816568729681, 0.0513352712705217, 0.0340426061301113, 0.109496317042074, -0.0895251176953564, -0.159859447182854, 0.0988851440159856, 0.027661315223121, 0.0183433969642289, 0.059000606532508, -0.0115959733825858, -0.0207062100805642, -0.0147672866726003, -0.158917255565657, -0.283768681645197, 0.179387388583808, 0.746253812808758},
    {0.0300889454319043, 0.0277086251948064, 0.0116134748343783, -0.0295941247801588, -0.0648165501864932, 0.151531129548398, -0.19835664154622, 0.278748753109168, -0.131811378803804, 0.246174341648927, -0.220629172869922, 0.0838282289179196, -0.0292546938798931, -0.00436084381177458, -0.0029856974556608, -0.00432619334275608, -0.149068120911646, 0.294363584413148, -0.288327136953214, 0.136372299873129, -0.0330847176025081, -0.00521448988916367, -0.00390183033368262, -0.00703787904698708, -0.00994958093958401, -0.00580000035399194, -0.00633883292814328, -0.0143122788603934, 0.151653962827897, 0.0265817081097539, 0.028073088576498, -0.333041069655042, -0.0900448667383323, 0.460415557245728, 0.181340605155926, 0.0347380525200536, 0.045669480354496, -0.43523155552094, -0.14648556610804, 0.749006978310935, 0.0545346359098206, 0.0386386245243292, 0.074193694016686, -0.130887367428771, -0.162933739126997, 0.225249181345676, 0.0121036142241399, 0.00857559599677565, 0.0164668166434049, 0.00231859812224145, 0.00288628206839904, 0.00304821841502335, -0.0806322373880537, -0.100374178118915, -0.0271383331252475, 0.34001500036677},
    {0.0220003110095952, 0.0154943147771694, 0.0299947250976383, 0.192230415140459, -0.180516285730209, 0.461954122435896, -0.667777779660335, 0.700205786033841, -0.0879689972569602, 0.207984322071836, -0.265754984143154, 0.210573178414572, -0.00627310588132717, -0.00295334145686079, -0.00261054608514565, -0.00387120992564302, -0.126842853448363, 0.306175461989327, -0.403856272644198, 0.342561795741952, -0.00904521677844054, -0.00434763868718076, -0.0039671331655812, -0.00629770910904651, -0.00272017181873595, -0.00173022626023817, -0.00178611135162366, -0.0030689926568001, 0.0383008284550467, 0.0206233204859007, 0.0255831356372873, -0.103568695058546, -0.078262984802148, 0.185719199739326, 0.0563829702642086, 0.0313403617540285, 0.0416188089603943, -0.157388834244543, -0.127318725090268, 0.302129183998959, 0.0169560741910446, 0.0124724984787889, 0.0187379208165114, -0.047331609842247, -0.0506689302910036, 0.0908594356622283, 0.0012091447219905, 0.00088941906810188, 0.00133620894796112, 0.00067210068612189, 0.000719490060176343, 0.000892524160294977, -0.00871623327667437, -0.00933080910976648, -0.00705094305896725, 0.0354092820333483},
    {0.0174985944065183, 0.029466788535953, -0.02833860959432, 0.0195251039488212, 0.0724066424646239, -0.168855440911288, 0.180838102102827, -0.0944993352356273, -0.00534222456152022, -0.00361590696451125, -0.00430211705354463, -0.0107923090820133, -0.154527546186846, 0.330011368059036, -0.366622398302526, 0.239192632867617, -0.00342995289485309, -0.00259182591384826, -0.00371794677692987, -0.0175569975586255, -0.0992137635313647, 0.2365470195983, -0.316839952784666, 0.389120107604863, -0.0113307209892054, -0.00562742560967782, -0.00678487933658837, -0.0287096173150645, 0.0311257440235671, -0.0815659404066927, 0.165998121769478, 0.0258512152336891, -0.124100720402198, 0.0563443238094873, 0.0223104495607475, -0.0704903705473233, 0.270046808008899, 0.0223409640327253, -0.201887847036957, 0.0916613069592546, 0.00254796784356058, 0.00167695757540017, 0.00578284080485399, 0.00255145275286987, 0.00480288799507102, 0.010468191687264, 0.073701734940998, 0.0485071595553231, 0.167272676255486, -0.232862300323701, -0.438343036323648, 0.892089521161032, -0.0219629172026514, -0.0413432822734828, 0.198471563814311, -0.0629028622529026},
    {0.0217651615841542, 0.0301544408087982, -0.0195929661997851, 0.0197079603842469, 0.0976637162240657, -0.196492083346238, 0.196789453010076, -0.0984800038037923, -0.0274091197540595, -0.0071634342787704, -0.00594031364386978, -0.0112469218624479, -0.139172828321605, 0.248753810380151, -0.205968627929981, 0.0566341297909968, -0.0211833561662756, -0.00609264721184827, -0.00592810457681625, -0.018296564542443, -0.10756082710638, 0.211570198073956, -0.205545302674773, 0.092132765186699, -0.0122839984890329, -0.00638648725849446, -0.00749348392029718, -0.0258568697967753, 0.139850019066564, -0.303301889052389, 0.438098662911987, 0.044349223363801, -0.167789230656524, 0.0641563418218738, 0.118945298523638, -0.302678515721806, 0.712701711629257, 0.0442580728498245, -0.272960595421595, 0.104369947911727, 0.0135841635533056, 0.00913669553498087, 0.0259826848221013, 0.00505449906476703, 0.00823962628291217, 0.0119195837084822, 0.0689750885493812, 0.0463925792045305, 0.131929947642522, -0.175519988457197, -0.286125111813636, 0.413287992708298, -0.0986777798289385, -0.160860258909838, 0.608592823921269, -0.0889892877687575},
    {0.0274003458448222, 0.0298409049274021, 0.010730605592219, -0.0116539403654932, -0.0616020791184594, 0.199029865952139, -0.142163327346853, 0.0628400509032686, -0.0610854342286923, 0.0232306873822269, 0.00669126628283972, 0.00717665632659224, -0.108378901130757, 0.135429332788424, -0.0546121987974093, -0.0124821118327561, -0.0591659989897215, 0.0242282398847579, 0.00787120988341587, 0.011675030491396, -0.104973403820015, 0.141244824496285, -0.0642425604898817, -0.020305979499737, -0.0119885014703203, -0.00664803468950693, -0.00749836629912185, -0.0201356771221163, 0.243375823624968, -0.362768474720545, 0.238132814788185, -0.110404287296158, 0.267886498788715, -0.0517660436411061, 0.253826662117954, -0.426739376810484, 0.387395988763217, -0.129873073436885, 0.435799472520741, -0.0842133314492745, 0.0289883075262092, 0.020085537226104, 0.0452167068747919, -0.0148321715329214, -0.0205119728904679, -0.00961759465884535, 0.0514315884793259, 0.0356361296383087, 0.080224313139771, -0.0864680007722811, -0.119579879709269, 0.0784960528037533, -0.155388943236062, -0.214893266576778, 0.521420011906788, 0.158688002977296},
    {0.030258215665097, 0.026502069349489, -0.00366448453027248, 0.0118223838397412, 0.0220838867907565, -0.0613915956629153, 0.29971631803346, -0.0767262716388461, -0.0891206073830918, 0.0989156565214026, -0.0302223496874193, -0.00876253400272336, -0.0661206272102359, 0.0421908315553681, 0.00264046129850585, 0.00187119892902529, -0.105520857724535, 0.123094447614705, -0.0406925701304423, -0.0142549464553036, -0.0782883499269335, 0.0525038935933198, 0.00355522180364742, 0.00304407840610805, -0.00894093136026364, -0.00529430977445553, -0.00572826125448683, -0.0122845275854093, 0.241764792404171, -0.180542653746288, -0.0171365171848402, -0.314040457459371, 0.355675036688131, 0.145522386249459, 0.300861406780796, -0.243089788685928, -0.0278777959462829, -0.422836525655092, 0.578614428417817, 0.236736750281293, 0.034359916762711, 0.0245123276814269, 0.0449173939628028, -0.0482901013499826, -0.0583454638191479, 0.0270365518840589, 0.0254924120689171, 0.0181862593654885, 0.0333252470857067, -0.0205973412450427, -0.0248862892142951, -0.0023621250377042, -0.118028295404669, -0.142605118785308, 0.161511294647257, 0.280936830180623},
    {0.0257416392900366, 0.0190269258810564, -0.00428139506100082, -0.0295941247801588, 0.0263560489909357, -0.0712908034861193, 0.124137438646549, 0.383808683073226, -0.0878504334226812, 0.157568839051331, -0.132636080114132, 0.0438329214248813, -0.0268465464558911, 0.00174684397405047, 0.000844454371940265, 0.000919836089063018, -0.121303800469527, 0.223660496831055, -0.196471909613941, 0.0713076774021096, -0.0370696875098228, 0.00247955111857613, 0.00125087806345138, 0.00149639524288001, -0.00423354856605207, -0.00264930503155428, -0.0027647241244966, -0.00498781022541945, 0.133654373845705, -0.0100008671933938, -0.00690608582347866, -0.275675643274785, 0.0294662669971707, 0.331483547871064, 0.189715199007308, -0.0148141401162546, -0.0112348646634435, -0.408354348391796, 0.0479359119350409, 0.539259559420539, 0.0216664494002769, 0.0158283391111046, 0.0248315981213801, -0.0466361623797635, -0.0512176787686351, 0.0615862093229059, 0.00662113227785815, 0.00483704205785011, 0.00758838205535305, -0.000517019099184866, -0.000567810831510311, -0.000392101030648698, -0.0395581202410412, -0.0434442928364937, 0.00464364975090457, 0.128003012885621},
    {0.013601987832951, 0.00894945296698991, 0.0293017672320873, 0.192230415140459, -0.182241449919052, 0.488039973885573, -0.759220407776989, 0.964112153400919, -0.0502269898723961, 0.116925136398512, -0.145927351620368, 0.110106556022686, -0.00493164708263818, -0.00188767340681881, -0.00151440904534089, -0.00204356107550113, -0.0769454525440011, 0.181036347722612, -0.229609921292738, 0.17912205076676, -0.00755505789865077, -0.00292270344760462, -0.00238285241145429, -0.00332447825028202, -0.000862826386783019, -0.000561763579208118, -0.000570886316972352, -0.000916248940524103, 0.0296474480394173, 0.0128750754489869, 0.0141325473043702, -0.0783043067552676, -0.0465267206212462, 0.133711509675399, 0.0459034376839121, 0.0202583342166768, 0.0229909184989447, -0.123208195799601, -0.075689967193349, 0.217522740600862, 0.0052424081733108, 0.00389379030146964, 0.0055081887248163, -0.014071008300651, -0.0145481290329095, 0.0248422133666612, 0.000514737336232626, 0.000382320334733097, 0.000540833582191091, 0.000227166450212583, 0.000234869226072958, 0.000257808232733803, -0.00356783408859089, -0.00368881245606512, -0.00219181234952881, 0.0133302788979784},
    {0.0267678423103473, 0.0201430117194883, 0.259211232824319, 0.0195251039488212, 0.326169225832269, -0.374897896525802, 0.278859093198177, -0.113874016814714, -0.00337553010430065, -0.00205150924371669, -0.00205803437210193, -0.00341655366698131, -0.117658048548054, 0.288722082265175, -0.389392282440997, 0.341103554416912, -0.0045864708615284, -0.00287261154019174, -0.00301923029022304, -0.00555807139457076, -0.15986680450647, 0.404281086211673, -0.571256238409763, 0.554909447702643, -0.00479647185936978, -0.00298367308690558, -0.00312601768238479, -0.00574276601693965, 0.0212800115225837, -0.0601691836488128, 0.116674242781906, 0.0149020923897652, -0.0605819216858597, 0.021494132932306, 0.0297971880275009, -0.0882709367117858, 0.189806405664418, 0.0218620492158882, -0.098555058333917, 0.0349667931625684, 0.000894002818805528, 0.000651456689925417, 0.00103865505607176, 0.000655925438529117, 0.000727355508724839, 0.00104910609763184, 0.0311615135421641, 0.0227072846284331, 0.0362035364034469, -0.0923126030276814, -0.102365415934278, 0.198497082175759, -0.00680382061772633, -0.00754475450407255, 0.0306719160336634, -0.00759650869077078},
    {0.0252996401097378, 0.0185719699093626, 0.00525730855264615, 0.0197079603842469, 0.692635678336204, -0.462466964864018, 0.306530718648313, -0.118670820076184, -0.015746654158432, -0.00374279051726993, -0.00267315597091034, -0.00356047179888873, -0.0963481856909961, 0.202449247313067, -0.218148211810369, 0.127152152668922, -0.0218777468924619, -0.0053404757434516, -0.00397432758405999, -0.00579219833361028, -0.133862165186806, 0.28886877039115, -0.324332910255172, 0.206851936598334, -0.00401625659770311, -0.002519131791377, -0.00262485806158846, -0.00470265394852329, 0.0880506682779475, -0.212599492451486, 0.326423530855205, 0.0240489893012457, -0.0827389756646376, 0.0244742477384673, 0.125636862646552, -0.31608332487961, 0.5310278913207, 0.0357549512968219, -0.134600295702079, 0.0398148630221872, 0.00376947345663158, 0.00275646118252999, 0.00429766082129862, 0.00107275314758302, 0.00117380595892291, 0.00119456237747495, 0.0230640696685629, 0.0168658072497395, 0.0262958606114859, -0.0580257073642747, -0.0634917000508116, 0.0974846396461412, -0.0252369538466303, -0.0276142623090705, 0.0950050643944836, -0.0107468543355459},
    {0.0222798637815405, 0.0157384636757173, -0.0132573892568216, -0.0116539403654932, 0.130691148487769, 0.574537076169525, -0.227302951843894, 0.0757238026633065, -0.030488677631767, 0.0107584088389603, 0.00275781261113257, 0.00227193562591236, -0.0651840819226079, 0.100280437182776, -0.0649749437055043, 0.00838325633735879, -0.0438437190827519, 0.0158018985425532, 0.00418435023440379, 0.00369599943203786, -0.0937368491673829, 0.147291418079095, -0.0985846245778231, 0.0136379351193381, -0.0028123797220151, -0.00178705171664072, -0.00184603177137818, -0.00318156671073533, 0.135818617311492, -0.239058106385729, 0.217617054249947, -0.0548325275417144, 0.135594537640337, -0.0197476187160158, 0.199489724091347, -0.362716030615443, 0.354020818079539, -0.0831958264843425, 0.220586062558379, -0.0321255526459662, 0.00598527537215401, 0.0044007472642863, 0.00662916434183093, -0.00249611820153066, -0.00267631819221324, -0.000963860569483388, 0.0127963791968913, 0.00940869504609308, 0.0141729988015652, -0.0232666213244205, -0.0249462873527773, 0.0227088620848719, -0.0315120319233674, -0.0337869513785023, 0.083551246966671, 0.0191640690194896},
    {0.0174601641969977, 0.0118031792992787, 0.00832487477036396, 0.0118223838397412, -0.0614326220634956, 0.26203544501963, 0.518569562880659, -0.0924570392473859, -0.0382036012222131, 0.0402707906090505, -0.0113654215697486, -0.00277398168006028, -0.0341553717751246, 0.0297193862963567, -0.00423826638234889, -0.00164784884783918, -0.0571355455781428, 0.0611192621110199, -0.0176401553036295, -0.00451273116942685, -0.051081200158203, 0.0451053215830697, -0.00657817017555996, -0.00268073106307841, -0.00153258545360959, -0.00098856745849668, -0.00101096290126262, -0.00166708789366002, 0.118608021325498, -0.117471841982964, 0.023155147567194, -0.142311111700305, 0.194815672108919, 0.0555136223703439, 0.180012228074643, -0.182326851992764, 0.037668942412368, -0.220879630062071, 0.316927383528957, 0.09030991653593, 0.00540089350611553, 0.00399603386838143, 0.00578913319241683, -0.00662703513197355, -0.0069460561873218, 0.0027095617168546, 0.00482858996842363, 0.00357259572485545, 0.00517568999039133, -0.00489067669414921, -0.00512611061131792, 0.00101041956648673, -0.0195183534360829, -0.0204579539645001, 0.0280057544625229, 0.0339275351802638},
    {0.0112160500006485, 0.00727374512952912, -0.0107414954878784, -0.0295941247801588, 0.0716512169321643, -0.219640225699807, 0.508137289414865, 0.462498877065499, -0.0328603051059602, 0.0572391342198016, -0.0460842819145469, 0.0138763194503273, -0.0121007418432333, 0.00259749741088059, 0.00090395270071815, 0.000791639729753525, -0.0509893480914856, 0.0895677407263753, -0.0730358568886421, 0.0225740853843906, -0.0187767257796372, 0.00406456138455537, 0.00143261340615383, 0.00128784458422881, -0.000563356708674904, -0.000368625559680022, -0.000373344867856833, -0.00059062452500809, 0.0585061854534171, -0.0142235201623364, -0.00673572294370002, -0.115421158702178, 0.0381836357431634, 0.126453757203726, 0.0915504212460301, -0.0225418936754921, -0.0109577172391413, -0.182923176377682, 0.0621173832613314, 0.205715782381084, 0.00274678048751648, 0.00204438250187339, 0.00285562558404517, -0.00548823047180258, -0.0056335857683375, 0.00617207533649786, 0.00101149643840161, 0.000752839780526246, 0.00105157842819907, -0.000249054508513001, -0.00025565069504787, -0.000121066531530378, -0.00560971849366762, -0.00575829138967057, 0.0019049583577091, 0.0154583744682552},
    {0.00485293934295211, 0.00304595863694757, 0.0263784345157957, 0.192230415140459, -0.168273534214414, 0.466883678827404, -0.770250380155342, 1.16177879234705, -0.0169490283234976, 0.0390069530753062, -0.0478485849051355, 0.034856762800183, -0.00200537469754978, -0.000638888466666515, -0.00047503858824402, -0.000597351761171212, -0.0270708836687132, 0.0625083209661624, -0.077060120670354, 0.0567052050431297, -0.00320297211812957, -0.00102381350470693, -0.000765049394997414, -0.000971775672177697, -9.60985345168711e-05, -6.36001340202051e-05, -6.39179461700487e-05, -9.78802368935099e-05, 0.0119183832377096, 0.0042829817936294, 0.00428003792419035, -0.0309395457727478, -0.0149430883155106, 0.0510080300769959, 0.0190991109554803, 0.00689773991227427, 0.00696279311635672, -0.049828122095164, -0.0243095118088297, 0.0829801901267495, 0.000573029207155878, 0.000428493256672421, 0.000581723792626193, -0.00149498944557329, -0.00151012679740893, 0.00248964847991301, 6.77996550040028e-05, 5.06984541297113e-05, 6.88283807443596e-05, 2.44861861607715e-05, 2.47341183559485e-05, 2.47171175797589e-05, -0.000456786694266239, -0.000461411837892375, -0.000222851295041071, 0.00160984056801339},
    {-0.00634204581202934, 0.00337366876311365, 0.0208770933958504, 0.0195251039488212, -0.00350519620228991, -0.00213578148775574, -0.00215103292273904, -0.00360148326124408, -0.00203791177178534, -0.0492117221773475, 0.0992013086517273, -0.0738509092732363, -0.00224657951086704, -0.0534241146055898, 0.106929104926151, -0.0792361187978446, 0.00186672700723228, 0.00270775846140296, 0.0252970714511431, -0.0453962493417858, 0.00205786663823839, 0.00293953537827706, 0.0272677169715593, -0.0487065445939725, 0.0421979810464191, -0.135565454749909, 0.628290543629045, -0.0283178557954035, 0.0161444667353696, -0.0223718110929613, 0.0143676720035994, -0.0227179316958212, 0.0101011981811169, 0.014694143970245, -0.00088831104609588, -0.00570497820444096, 0.00883182654289621, -0.00579324153221612, 0.00620921957215441, 0.00903250928257811, -0.0182154333959001, 0.263102111421653, 0.203498998675296, -0.118794431005032, -0.286356708329364, 0.185217860295405, -0.0200805648288412, 0.290041905135055, 0.224335855573075, -0.128962918095965, -0.310868080347518, 0.199646358368677, 0.0389718679788597, 0.093942584155313, -0.0417702048278289, -0.0424164434544871},
    {-0.0106853276693849, 0.0115772632577271, 0.019287579585398, 0.0197079603842469, -0.0163810014243473, -0.00390242657177244, -0.00279689872237231, -0.00375319132544435, -0.0274010505067602, -0.0481217403864267, 0.105826965373029, -0.0769617882285366, -0.0053024822727365, -0.0237500598790074, 0.0728748586553496, -0.0650020279883699, 0.0146717000428918, 0.00719541733809569, 0.0244226254420096, -0.0473085106547024, 0.00283917689101547, 0.00355123466570265, 0.016817976125512, -0.0399568305836848, 0.0582192890483207, -0.134368965523375, 0.207386421256723, -0.0668371306827619, 0.0358043028635586, -0.0767957177864589, 0.0595392120224926, -0.0301110314791141, 0.0163185090590721, 0.0167314550889353, -0.00535364888165092, -0.0177228274895519, 0.0365988305517864, -0.00694898923818503, 0.0100310086012752, 0.0102848470593368, -0.109780279168362, 0.670583112662057, 0.451308792072928, -0.142493838383863, -0.379545813158116, 0.210897913990534, -0.0212440024532162, 0.129767107520561, 0.0873344935774071, -0.0703265751992815, -0.187321483324288, 0.145229106953919, 0.106020532605796, 0.282395714198688, -0.153042815009205, -0.0600069529033939},
    {-0.0102352418729336, 0.00318614435970196, 0.0163965439250797, -0.0116539403654932, -0.0318063021955915, 0.0112436886986536, 0.00289018334287365, 0.00239490987846146, -0.0126571512820064, 0.014353793525102, -0.0723998288978306, 0.0491092861808114, -0.000855846139544752, -0.00170119926809078, 0.0333817769889969, -0.0441008898133743, 0.0285443214023187, -0.0170879000112398, -0.0117215303676543, 0.0301875416619842, 0.00193009838737002, 0.00202524321821195, 0.00540450880422602, -0.0271088739443333, 0.0395780045482835, -0.0507328584660815, 0.00689944426242844, -0.010787834325671, 0.00573105683907265, -0.0797988402663342, 0.0920558261790191, 0.0455268719942119, -0.0372674873257958, -0.0135001655287294, -0.00682270690696095, -0.0129194301108834, 0.0565868352835748, 0.00737079936166045, -0.0229083726068188, -0.00829856919202052, -0.139904331697434, 0.323635014986763, 0.0722392598788917, 0.151143347183347, 0.573860568794342, -0.170167910286462, -0.00945999455336036, 0.0218834216346687, 0.00488464507640135, -0.0179133795644634, -0.0680134612436615, 0.0784602300632857, 0.0603471905904602, 0.229126016875704, -0.187558480429118, 0.10700595273598},
    {0.00496641091476394, -0.011801561675532, 0.0123418011338163, 0.0118223838397412, -0.0399845505836983, 0.0422013001459018, -0.0119330872043856, -0.00292413044299179, 0.0594964820405095, -0.120624902691672, 0.125434652322348, -0.0599613204849069, 0.00168229735160551, 0.00204848339993448, 0.00753331313448472, -0.0231834539600411, -0.0204347654762904, 0.0301742710270639, -0.015052523747274, -0.0368583011689746, -0.000577804782105243, -0.000512427300870581, -0.000904019525330866, -0.0142508991010519, -0.0118482873431861, 0.00538637983819928, 0.0025839788560398, 0.0212051492401301, -0.011236152923305, -0.0298340045411959, 0.0806085190825288, 0.0496738192420157, -0.0969030134598643, 0.0379510614356349, 0.00281071914707334, 0.00358016746981964, 0.0495501608220378, -0.00596100303954884, -0.0595664075674237, 0.023328566495279, 0.0576357432941004, -0.0376329322111707, -0.141630312498688, -0.122234496933111, 0.626132323960991, 0.478368417328807, 0.00162968389014089, -0.00106409286769063, -0.00400467878859417, 0.00207581794703584, -0.0106331416082052, 0.0287296932280311, -0.0113860858694686, 0.0583239313380668, -0.113777534921342, 0.189440364791817},
    {0.0263496071105022, 0.00800383521292129, 0.00763236993753088, -0.0295941247801588, -0.0344990911194155, 0.0601371945948342, -0.0484704728084982, 0.0146274102792546, -0.0455282927907743, 0.109434345863475, -0.15080433156421, 0.299945181214723, -0.000530247378602366, -0.000473709291275561, -0.00080991227462764, -0.00823909033431851, -0.135681023269177, 0.282736312981597, -0.302623682224074, 0.184376690406239, -0.00158021534532793, -0.00122388284394246, -0.00162527582784928, -0.00506457947298101, -0.0324034103825531, -0.003807819967485, -0.0029575872015101, -0.00668370237087951, 0.00353768478630586, 0.00443980886007431, 0.0398642041456057, -0.0119454683360227, -0.0787301479845028, 0.0864482284436403, 0.00914001856619179, 0.00890950075286562, 0.0245045777907789, -0.0239713380659704, -0.0483955236813235, 0.0531398377109133, 0.187422412635165, 0.0277197895493028, 0.0445920774865158, -0.491548893679026, -0.150571145206208, 1.08966918597521, 0.00218282384204085, 0.000322839817681629, 0.000519343703536707, 0.0021277714616438, 0.000651778469724241, 0.00585219562230551, -0.0158903041743695, -0.00486751435712143, -0.0320807954006469, 0.0863145549122675},
    {0.0261153899063015, 0.0302224845101156, 0.00320507108782466, 0.192230415140459, -0.0178382131382485, 0.0410650708630083, -0.0503948504088267, 0.0367434731025106, -0.183081952923273, 0.472755167242471, -0.693290980381532, 0.753450370761893, -0.000685098508853437, -0.000506609750046193, -0.000605395551988722, -0.00136877883379721, -0.15273114932588, 0.375858140305486, -0.510844558915112, 0.463146916325962, -0.000571524833485222, -0.000402773807050422, -0.000446079687283173, -0.000841390117526007, -0.0117195126461702, -0.0055410313811158, -0.00513542221486786, -0.00863558164111834, 0.0045684446025844, 0.00406019525351187, 0.00813727702558381, -0.0141780709602837, -0.0211330510483063, 0.0348708803444417, 0.00363208529783313, 0.00299171446345356, 0.00500199470056441, -0.0104469704798229, -0.0129905137821948, 0.0214351751990887, 0.0744784252336854, 0.0411575515467008, 0.0575846769898424, -0.214222367041932, -0.178712824081337, 0.439543117114766, 0.00027870064337096, 0.000154012870971259, 0.000215483698467536, 0.000229563095956058, 0.000191510670662632, 0.000383817842058265, -0.00207012653542002, -0.00172698194151785, -0.00257414408714912, 0.00898882818456403},
    {-0.0110271230805097, 0.0117859544381791, 0.0160091858217584, 0.0195251039488212, -0.00316207558373215, -0.00241577235429526, -0.00353354291864167, -0.0181912548602126, -0.00533638736500254, -0.021589474677939, 0.0708866405450278, -0.0648153237617092, -0.0297142669526366, -0.0385011904458149, 0.0864255424365169, -0.0627342855826571, 0.00295045175569059, 0.00361626834906953, 0.0160892124049081, -0.0398420632543444, 0.0164288131844466, 0.00644900529020178, 0.0196160926625834, -0.038562846397088, 0.0585357554359555, -0.129373925944914, 0.175306723361565, -0.0650794963063691, 0.0357749037316649, -0.0262610115094856, 0.0142628870184278, -0.0819968258027645, 0.0145632415484494, 0.0651399463159163, -0.00599234830794818, -0.00596048830774358, 0.00876741507713458, -0.0186109023749111, 0.00895204339478121, 0.0400416091578109, -0.0213506983799168, 0.119573754125017, 0.0783533620054189, -0.0663105251505076, -0.179587540573384, 0.142667995223338, -0.118885738214351, 0.665814943251966, 0.436290050931204, -0.11825364884818, -0.320264119644221, 0.173941927289884, 0.109880054473008, 0.297586072438545, -0.0528534831921601, -0.165028595825849},
    {-0.0116958193796847, 0.00868942826669692, 0.0205403356838656, 0.0197079603842469, -0.0201247738104083, -0.00584323645334017, -0.00577935460978638, -0.0189575391547738, -0.0167229445882327, -0.0161405937908933, 0.0744586814191187, -0.0675455897618916, -0.0163457939686739, -0.0164605936176502, 0.0767742256020669, -0.0700872974533167, 0.0148427436855896, 0.00638151686995194, 0.0144742293073116, -0.0415203612920179, 0.0145079970177743, 0.00650803540571885, 0.0149243543543366, -0.0430827522936908, 0.0516918998203713, -0.0838466866383685, 0.041225088724075, -0.0358001777363246, 0.0606589183534294, -0.111649968396047, 0.0782426680475976, -0.107010375739815, 0.0295940856905751, 0.0741714582685035, -0.0239827552691317, -0.021703946590485, 0.0480958691343093, -0.0208020433238823, 0.0181915227080135, 0.0455932912233092, -0.0854504023725406, 0.279624171526345, 0.132853752011542, -0.07411754622225, -0.23437163581397, 0.162448602623016, -0.0835232494105542, 0.273317840187487, 0.129857516831937, -0.0755869842279407, -0.239018235798383, 0.167500490591787, 0.278545525839863, 0.880805880216167, -0.243589881034135, -0.233467551046818},
    {-0.00575929267843156, -0.00515804432869902, 0.0267098865309273, -0.0116539403654932, -0.0579732173172456, 0.0239119304978302, 0.00786274267358259, 0.0120967981262488, 0.0180568823364596, -0.015948120367053, -0.0486788032710565, 0.0431008137183392, 0.00616713919741468, 0.00514217279233513, 0.0477328791988053, -0.0705477914032103, 0.0151788194266244, -0.00831999241146525, -0.00467132225531166, 0.0264941258766691, 0.00518416692938189, 0.00268262576567053, 0.0045805493547138, -0.0433658185196245, 0.0184711533395948, -0.0166083283299721, -0.00305500856414893, 0.0135071248184839, -0.0321631653551498, -0.145964620231556, 0.171822073141542, 0.15461492822938, -0.08898168576429, -0.0598469743850445, -0.0167792370213338, -0.0140070776838758, 0.105619250345273, 0.014837179772469, -0.0546971571990115, -0.03678801247366, -0.0597843133070972, 0.0867188215718506, -0.0704430166245736, 0.0528647758526334, 0.33863401936357, -0.131075451218333, -0.0204187065694203, 0.0296179059986184, -0.02405907514457, -0.0170452571090387, -0.109186198802953, 0.128528399601262, 0.106614352543, 0.68293577606614, -0.393033048742583, 0.416325384375381},
    {0.0114965700457506, -0.0103543496131472, 0.0302516266319403, 0.0118223838397412, -0.105865563934372, 0.12409022809239, -0.041339453116336, -0.0147699151361855, 0.0487160015083048, -0.0883080091353924, 0.0683057070994247, -0.0526251123873066, 0.00695767866119288, 0.00519526230305939, 0.012062497405501, -0.0538719236429516, -0.0456650504879514, 0.0640339164783736, -0.0289985647742542, -0.0323487245733811, -0.00652193812105361, -0.00376718935977031, -0.00512102322348484, -0.0331151410630411, -0.0232376234689826, 0.00622457516001471, 0.00364434309255474, 0.0152385459635857, -0.0415491378876403, -0.0562811451289647, 0.208024526539326, 0.136630693123355, -0.294625982202666, 0.168239137275043, 0.0301281169318005, 0.0238936466936188, 0.127872945220575, -0.0580053142400197, -0.181106971901399, 0.10341681503917, 0.107346286348629, -0.0394797780221855, -0.0909999553413711, -0.20667256059336, 0.299245365959432, 0.368473445107362, 0.0153313273413631, -0.00563854997514032, -0.0129967244405357, 0.0121587857504661, -0.0176049509478883, 0.0650708434821411, -0.0972399536775843, 0.140795688796734, -0.303607243389941, 0.737050889895457},
    {0.0286331249853375, 0.0142799617871943, 0.0261981528295216, -0.0295941247801588, -0.123694165861968, 0.228667944176925, -0.201694509957147, 0.0738837109693798, -0.0786155180524536, 0.180183183238994, -0.227528563330551, 0.263247185749171, -0.00462473340853541, -0.00324555768787732, -0.00464646010384153, -0.0259264273302239, -0.144761735858263, 0.294022599521717, -0.301560576395923, 0.161818385181666, -0.0085159323844256, -0.00529609529109965, -0.00615830015627296, -0.015937008375482, -0.0303422121712233, -0.00597444597327716, -0.00485254778273565, -0.0101289835370484, 0.0294212418863081, 0.0278742507862935, 0.133035353826841, -0.0910346124745029, -0.287527730636603, 0.383229739093874, 0.0480095304404493, 0.0369438237145957, 0.0817770038721057, -0.120654962207571, -0.176743667493169, 0.235571815733863, 0.17105764737271, 0.0416757755850919, 0.064437719622051, -0.429892852309207, -0.199381890716993, 0.839340860388868, 0.0100628481015949, 0.00245167056644469, 0.00379069275492287, 0.00774346432721751, 0.00359137526936527, 0.0171405460669789, -0.0701951279005205, -0.0325561061198115, -0.102826639852128, 0.335821880299532},
    {0.0243834140019696, 0.0296919171784482, 0.014002682081672, 0.192230415140459, -0.0792712548026677, 0.186695351837455, -0.237148368754727, 0.185592944676423, -0.177845002139631, 0.452155668420226, -0.646858537591112, 0.661266465096996, -0.00336147904794571, -0.00228012796141551, -0.0025679934935631, -0.00533850326641937, -0.141645882360695, 0.345411287467401, -0.462849575846567, 0.406481350416946, -0.00267727324386337, -0.0017418380211724, -0.00183748784347614, -0.00328158485493601, -0.0095391073036475, -0.00475113027769318, -0.00445019405091743, -0.007362233220608, 0.0220699483121886, 0.0178268522440286, 0.032468474045117, -0.0668177993242803, -0.0872806619365435, 0.154584525524274, 0.0168597007475027, 0.0127557271340269, 0.0199584129431104, -0.0478104381081746, -0.0536515356544614, 0.0950233075549246, 0.0600710050445716, 0.0347932015858052, 0.0483370874319142, -0.170348282676652, -0.146342789854298, 0.338567432066033, 0.00113541242327302, 0.000657632301917819, 0.000913627623415306, 0.000859031324028783, 0.000737976565159795, 0.00134409443819929, -0.00831478638071545, -0.00714306605782546, -0.00933062058445968, 0.0349726090309795},
    {-0.00888246788236121, -4.16214011103241e-06, -0.0100835618309505, 0.0195251039488212, 0.00276855422334095, 0.00354739278284653, 0.0184970121822559, -0.041398111534134, 1.34764571132345e-06, 2.345896878201e-06, 0.0351417498325346, -0.0504430947992082, 1.70769882250968e-05, -2.18993997978534e-05, -0.0629150249458781, 0.042747406649815, 0.00207617681188791, 0.00204132940906936, 0.00495444159751796, -0.0310074355429239, 0.0263087298629871, -0.0190562037323708, -0.0088700425615163, 0.0262768860950637, 0.0320568669764365, -0.0362041359827395, -5.86577422616655e-06, 1.27907673088089e-05, -8.8463310561164e-06, 0.0681493393296448, -0.0162998925724396, -0.0925068516383988, -0.0593298559286743, 0.115369044865993, -0.00769781311112107, 0.00960799969344611, -0.0100195650228907, -0.0130420310882867, -0.0364701390904393, 0.0709175009299217, -0.00937969153956957, 0.0182538627477391, -6.6259553841699e-06, -0.0158915560681609, -0.069288190527517, 0.0864121113201357, -0.11885681869661, 0.231307825632447, -8.39622469953338e-05, 0.148350740803415, 0.646818621763509, -0.154705447658132, 0.0599267458324434, 0.261284405700932, 0.167576410526233, -0.227470527214667},
    {-0.00438186087957976, -0.00681295111639304, -0.0118196797021562, 0.0197079603842469, 0.0153251463175155, 0.0068247368416488, 0.0170656692030986, -0.0431419561966856, 0.0101073362006734, 0.00568346715796623, 0.0351451051021587, -0.0525679482857169, 0.0224826683241414, -0.0171152528866811, -0.0423250421086932, 0.0415371758142848, 0.00496290698942548, 0.0017838158903669, 0.00264840080550209, -0.0323135857262808, 0.0110394459580145, -0.00537180196847265, -0.00318945341855553, 0.0255329556369081, 0.0134514304724192, -0.0110627097316332, -0.00265609563809126, 0.0168396543479754, -0.0486078465011174, 0.155615226861536, -0.0711213725288545, -0.114945842078814, -0.0680100252848931, 0.131364712142299, -0.0152560834038925, 0.0117265687773733, -0.043718399578595, -0.00866187936743298, -0.0418058504080335, 0.0807500582702354, -0.0185893518541939, 0.0241497410540252, -0.0364075705728993, -0.0105543945334431, -0.0860951298768103, 0.098392962708179, -0.0413499881472978, 0.0537184681949508, -0.0809846746389742, 0.0317836149105708, 0.259267781302797, -0.118494062764005, 0.0902664474228087, 0.736328501815995, 0.435663605754062, -0.321804755462983},
    {0.00649963867312386, -0.0117909391625216, -0.00729939888431999, -0.0116539403654932, 0.0195387056426204, -0.0109690858690787, -0.00648860340814748, 0.027528864934527, 0.0347177461769007, -0.0226829837593203, -0.0194855585338848, 0.0335435867035136, 0.0269842516546397, -0.00603551746377206, -0.00876488846392664, 0.0185044353861291, -0.0157527489567287, 0.00761282852421888, 0.00331676160313951, 0.0206192860832164, -0.01224377123834, 0.0020256311953556, 0.0014919277506158, 0.0113747003386201, -0.0149188861614174, 0.00608305867108536, 0.00308515890396074, 0.0202113674476524, -0.104103979779745, 0.0482167279666835, -0.0613421198629883, 0.140845268185824, 0.0571481039328644, -0.105994687797817, 0.0349392194237378, -0.00820727779862292, -0.0377070803306021, -0.0239741743450801, 0.0351290133199048, -0.0651550715289332, 0.0425730134126711, -0.0246468125557044, -0.0779745095406277, -0.029212239505768, 0.105493956438009, -0.0793906612641798, 0.0330897317403691, -0.019156652309716, -0.0606054257488448, -0.00777283022214868, 0.0280699674769364, -0.0357110775063413, -0.134070121598529, 0.484166493457688, 0.196451023490723, 0.57385057542787},
    {0.0218166735098088, -0.000396180986240273, 0.00962857943557122, 0.0118223838397412, -0.0386200930353358, 0.0549301045311348, -0.0255353246076025, -0.0336121174078451, 0.00168506853674372, -0.00261416436528547, 0.00143665582940347, -0.0409559557664377, 0.000547681628745338, 9.49985402433524e-05, 0.000112306740667862, -0.0152948461494822, -0.0821315304265297, 0.106627813860813, -0.0421122676094755, -0.0251756789225904, -0.0266944218436792, -0.00387484689204712, -0.00329201428801516, -0.0094017616882311, -0.0325268279583357, 0.000218004723301847, 0.000156669398596515, 0.000410216847388413, -0.00279906307588783, -0.0007311985629161, 0.0716657109865359, 0.00653976121473068, -0.141635372249502, 0.297967524909641, 0.114169552842125, 0.0214334073109028, 0.0440530051317888, -0.191698087141148, -0.0870634463072912, 0.183161022520475, 0.139114209894574, -0.00120587578307186, -0.00209651514742671, -0.233581784872354, 0.00489832064355597, 0.223179475587969, 0.0452149544041173, -0.000391934214268599, -0.000681410165577337, 0.00848834483592534, -0.000178004610942403, 0.0174464607140272, -0.250102923105532, 0.0052447767360409, -0.113589148133034, 1.01592911016151},
    {0.0302602001667236, 0.023673129138354, 0.0280757815135756, -0.0295941247801588, -0.142669865790111, 0.291843625603676, -0.302869456861528, 0.168138269227149, -0.123983018812676, 0.264604231380493, -0.294894932128271, 0.204874433632237, -0.0165981047421918, -0.00715703702973119, -0.00768596711021547, -0.02327285910972, -0.148692369932632, 0.290316139028518, -0.279210915864665, 0.125936579040831, -0.0199060448288925, -0.00785249482412044, -0.00727718818584339, -0.0143058565620664, -0.024255273227939, -0.0076457393630659, -0.00659796760180478, -0.0124320806881215, 0.0983244220117468, 0.0542495311672531, 0.132140523387534, -0.268507118668092, -0.336020030755683, 0.678736343273606, 0.10787872295819, 0.0513642644623298, 0.0812269504449895, -0.254226540885044, -0.206551947025893, 0.417220107102282, 0.131448910284485, 0.0500118481387927, 0.0736455858695551, -0.309771944349812, -0.201113453401184, 0.508377620011561, 0.0175975936224403, 0.00669528699742708, 0.00985923039920365, 0.00837873704784177, 0.00543973324107423, 0.0132500536336069, -0.115108315633291, -0.0747318512681792, -0.0935222838266846, 0.462886930472143},
    {0.0208837824772988, 0.0273991946635943, 0.0249036082205118, 0.192230415140459, -0.14494000231401, 0.354346112869683, -0.476703569824698, 0.422356647889497, -0.161228140099431, 0.400101823491238, -0.550948270007869, 0.514636432413085, -0.00693501587567338, -0.00405573932646442, -0.0041513523462138, -0.00759654426569474, -0.120035982431373, 0.28854446919111, -0.378174407346312, 0.316347679887783, -0.00516318952324473, -0.00292490831688956, -0.00284951473428294, -0.00466960557446191, -0.00629128356187384, -0.003380451247123, -0.00321746450671813, -0.00519436877155106, 0.0444428333845823, 0.0305213990124238, 0.0479600994071269, -0.129512458891891, -0.13654327736291, 0.273783907869765, 0.0320511755142852, 0.020950082995521, 0.0294811350675108, -0.0888981780171578, -0.0839333290017816, 0.16829532188209, 0.0390539670767367, 0.0242129757642642, 0.0332879505960336, -0.108321347400085, -0.0970056138378368, 0.205065800379851, 0.00167985490323307, 0.00104148922898282, 0.00143183730650098, 0.00109802835866301, 0.000983323393772348, 0.00154515485006053, -0.0120322060831618, -0.0107752678944203, -0.0113602228338168, 0.0482052081612835},
    {0.0101644112236962, -0.0109437322728922, 0.0100401662714875, 0.0195251039488212, -0.00508376488671095, -0.0317222402070974, 0.0822967853898158, -0.0686320373360359, 0.00265561165013069, 0.00277042286983623, 0.00774815294057044, -0.0335768632939073, 0.0557887295356483, -0.115066962106936, 0.124046280414075, -0.0522493142351213, -0.00206888955864461, -0.00165092370827371, -0.00261316748166397, -0.0206397412463229, -0.0434629513771954, 0.0685695955840057, -0.0418362555160877, -0.032117720963721, -0.0212633870913167, 0.00645575293301406, 0.00365371796926722, 0.0167773679286116, -0.017319952436912, 0.0668525135558671, 0.0970236731312839, -0.0338138843694441, -0.175708585038306, 0.127313382496836, 0.0103211392078777, -0.022546898058369, 0.0596405772233313, 0.0114041815824649, -0.108008294229385, 0.0782597007030627, 0.00504941268934637, -0.00212276595816229, -0.00520863652853839, 0.00557926969439193, -0.0101688635658782, 0.0382870066797381, 0.106077377249782, -0.0445947794744644, -0.109422330078203, -0.231729827781459, 0.422354381832925, 0.612966832642402, -0.0348801212954481, 0.0635730506039554, 0.330347458639909, -0.167089166495659},
    {0.0155122697307477, -0.00766332463985573, 0.0110939404354585, 0.0197079603842469, -0.0231802053744836, -0.0276298646655858, 0.0870824133797188, -0.0715230777133198, 0.00895970133038251, 0.00338168273878032, 0.00558144257295907, -0.034991247469186, 0.03304083660687, -0.0519648837128462, 0.0272127142256641, -0.0418203837218572, -0.0156006160152443, -0.00470230194600613, -0.00496770894873694, -0.0215091650261775, -0.0575306459131931, 0.0722582786979663, -0.0242204129508018, -0.0257070439035827, -0.0281457276807841, 0.00463130748587515, 0.00296439209504608, 0.00993638458944258, -0.047948193300269, 0.126107477245155, 0.191659952362795, -0.0302636455110631, -0.231003451275631, 0.144965106220525, 0.0666729850431344, -0.112240739741459, 0.117813620332084, 0.0269358648165762, -0.141998119943556, 0.0891102380811218, 0.0326184358075962, -0.00719393524937952, -0.0144194801926216, 0.0131778377235274, -0.00910119875154587, 0.0435954169258845, 0.120287537290538, -0.0265291922543363, -0.0531749520918535, -0.202498240605751, 0.139854259344908, 0.212552508933874, -0.186845886872036, 0.129044050170325, 0.984997691207882, -0.236382660307924},
    {0.0235602360807905, 0.00242217246301152, -0.00170955493305554, -0.0116539403654932, 0.00630079692715634, -0.00613847546289087, -0.0581932607657199, 0.0456388471838216, -0.00601102036289123, 0.0030084829955598, 0.00140208737896731, 0.0223279009667123, -0.00774557056222436, 0.00717932068418092, 0.000352938229631426, 0.00397204289231752, -0.0521782100505412, 0.0221414434455201, 0.00763849492268674, 0.0137249895707229, -0.0672348425655043, 0.0528374343948943, 0.00192278806264404, 0.0024416198975801, -0.0328933134587858, -0.00127304603535361, -0.000942363494893978, -0.0023293286694479, 0.022890805356466, -0.0311158320724685, -0.0228500559069324, -0.0168016078072762, 0.34116281345662, -0.11696848358163, 0.168468784091727, -0.169517341690776, -0.0140459588870732, -0.0915342352082089, 0.209713222196366, -0.0719006779748152, 0.082420011882244, 0.00408428952379192, 0.0068839614532226, -0.0447813094524682, -0.00505275453162593, -0.0351759809093124, 0.106203269866522, 0.00526285901448312, 0.00887040900954829, -0.106864283990074, -0.0120576866063495, -0.00885461820277434, -0.340729942564745, -0.0384451634488731, 0.780643154897168, 0.421523682717848},
    {0.0297613960356094, 0.0189589162037065, -0.011422245664625, 0.0118223838397412, 0.0552995939385817, -0.104936601954282, 0.0910163993406658, -0.0557239934719332, -0.0724739320858042, 0.0961639620233354, -0.0394468229313447, -0.027261858799801, -0.0390515961300795, 0.0124558869364595, 0.00589090048204109, 0.0145778050557573, -0.105570386298514, 0.126084042471053, -0.0432981721365902, -0.016757899825139, -0.0568851719559671, 0.0163313630643681, 0.00646605237524114, 0.00896099560153871, -0.0278299423469547, -0.0070780693555717, -0.00592549409332408, -0.0117440028107332, 0.17070195853941, -0.0715604211734564, -0.0911310520484137, -0.297692260133122, 0.336038002649934, 0.328816568729681, 0.223813500790995, -0.0785471478787503, -0.0560183754307316, -0.326757131883804, 0.206562994372507, 0.202123969612012, 0.109496317042074, 0.0340426061301113, 0.0513352712705219, -0.159859447182854, -0.0895251176953563, 0.0988851440159855, 0.0590006065325077, 0.0183433969642289, 0.0276613152231212, -0.020706210080564, -0.0115959733825859, -0.0147672866726003, -0.283768681645196, -0.158917255565658, 0.179387388583808, 0.746253812808758},
    {0.0277086251948063, 0.0300889454319043, 0.0116134748343783, -0.0295941247801588, -0.0648165501864932, 0.151531129548398, -0.19835664154622, 0.278748753109168, -0.149068120911646, 0.294363584413148, -0.288327136953214, 0.136372299873129, -0.0330847176025081, -0.00521448988916367, -0.00390183033368265, -0.00703787904698712, -0.131811378803804, 0.246174341648927, -0.220629172869922, 0.0838282289179194, -0.029254693879893, -0.00436084381177457, -0.00298569745566078, -0.00432619334275604, -0.0143122788603935, -0.0063388329281432, -0.00580000035399203, -0.0099495809395839, 0.181340605155926, 0.0347380525200538, 0.045669480354496, -0.43523155552094, -0.14648556610804, 0.749006978310935, 0.151653962827897, 0.0265817081097537, 0.0280730885764982, -0.333041069655042, -0.0900448667383323, 0.460415557245728, 0.0741936940166857, 0.0386386245243292, 0.0545346359098208, -0.162933739126998, -0.130887367428771, 0.225249181345676, 0.0164668166434046, 0.00857559599677565, 0.0121036142241402, 0.00288628206839916, 0.00231859812224133, 0.00304821841502335, -0.100374178118915, -0.0806322373880542, -0.0271383331252475, 0.34001500036677},
    {0.0154943147771694, 0.0220003110095952, 0.0299947250976383, 0.192230415140459, -0.180516285730209, 0.461954122435896, -0.667777779660335, 0.700205786033841, -0.126842853448363, 0.306175461989328, -0.403856272644199, 0.342561795741953, -0.00904521677844057, -0.00434763868718075, -0.00396713316558123, -0.00629770910904658, -0.0879689972569602, 0.207984322071835, -0.265754984143153, 0.210573178414572, -0.00627310588132714, -0.00295334145686079, -0.00261054608514562, -0.00387120992564294, -0.00306899265680024, -0.00178611135162363, -0.0017302262602382, -0.0027201718187358, 0.0563829702642087, 0.0313403617540288, 0.0416188089603943, -0.157388834244543, -0.127318725090268, 0.302129183998959, 0.0383008284550466, 0.0206233204859005, 0.0255831356372873, -0.103568695058546, -0.0782629848021475, 0.185719199739325, 0.0187379208165112, 0.0124724984787888, 0.0169560741910448, -0.0506689302910036, -0.0473316098422469, 0.0908594356622281, 0.00133620894796083, 0.000889419068101882, 0.00120914472199075, 0.000719490060176447, 0.00067210068612179, 0.000892524160294977, -0.00933080910976625, -0.00871623327667453, -0.00705094305896725, 0.0354092820333481},
    {0.0294667885359529, 0.0174985944065183, -0.02833860959432, 0.0195251039488212, 0.0724066424646239, -0.168855440911288, 0.180838102102827, -0.0944993352356273, -0.00342995289485307, -0.00259182591384814, -0.0037179467769299, -0.0175569975586255, -0.0992137635313648, 0.2365470195983, -0.316839952784666, 0.389120107604863, -0.00534222456152027, -0.00361590696451142, -0.00430211705354463, -0.0107923090820133, -0.154527546186846, 0.330011368059036, -0.366622398302526, 0.239192632867617, -0.0287096173150646, -0.0067848793365882, -0.00562742560967802, -0.0113307209892053, 0.0223104495607476, -0.0704903705473232, 0.270046808008898, 0.0223409640327254, -0.201887847036957, 0.0916613069592546, 0.0311257440235671, -0.0815659404066929, 0.165998121769478, 0.0258512152336889, -0.124100720402198, 0.0563443238094873, 0.00578284080485374, 0.00167695757540016, 0.00254796784356083, 0.00480288799507095, 0.00255145275286991, 0.010468191687264, 0.167272676255486, 0.0485071595553232, 0.0737017349409982, -0.438343036323648, -0.232862300323702, 0.892089521161032, -0.0413432822734823, -0.021962917202652, 0.198471563814311, -0.0629028622529026},
    {0.0301544408087981, 0.0217651615841543, -0.0195929661997851, 0.0197079603842469, 0.0976637162240657, -0.196492083346238, 0.196789453010076, -0.0984800038037923, -0.0211833561662755, -0.00609264721184809, -0.0059281045768163, -0.018296564542443, -0.10756082710638, 0.211570198073956, -0.205545302674773, 0.092132765186699, -0.0274091197540597, -0.00716343427877054, -0.00594031364386975, -0.0112469218624479, -0.139172828321605, 0.248753810380151, -0.205968627929981, 0.0566341297909968, -0.0258568697967753, -0.00749348392029705, -0.0063864872584946, -0.0122839984890328, 0.118945298523638, -0.302678515721806, 0.712701711629256, 0.0442580728498247, -0.272960595421595, 0.104369947911727, 0.139850019066564, -0.303301889052389, 0.438098662911987, 0.0443492233638008, -0.167789230656524, 0.0641563418218738, 0.025982684822101, 0.00913669553498087, 0.0135841635533059, 0.0082396262829122, 0.00505449906476702, 0.0119195837084822, 0.131929947642522, 0.0463925792045305, 0.0689750885493814, -0.286125111813635, -0.175519988457197, 0.413287992708298, -0.160860258909837, -0.0986777798289391, 0.608592823921269, -0.0889892877687574},
    {0.0298409049274021, 0.0274003458448222, 0.010730605592219, -0.0116539403654932, -0.0616020791184594, 0.199029865952139, -0.142163327346853, 0.0628400509032686, -0.0591659989897214, 0.0242282398847581, 0.00787120988341579, 0.0116750304913959, -0.104973403820015, 0.141244824496285, -0.0642425604898817, -0.020305979499737, -0.0610854342286924, 0.0232306873822266, 0.00669126628283979, 0.00717665632659229, -0.108378901130757, 0.135429332788424, -0.0546121987974093, -0.012482111832756, -0.0201356771221164, -0.00749836629912176, -0.00664803468950701, -0.0119885014703202, 0.253826662117954, -0.426739376810484, 0.387395988763217, -0.129873073436885, 0.435799472520741, -0.0842133314492744, 0.243375823624968, -0.362768474720545, 0.238132814788185, -0.110404287296158, 0.267886498788714, -0.0517660436411062, 0.0452167068747916, 0.020085537226104, 0.0289883075262095, -0.0205119728904679, -0.0148321715329214, -0.00961759465884535, 0.0802243131397707, 0.0356361296383087, 0.0514315884793261, -0.119579879709269, -0.0864680007722812, 0.0784960528037533, -0.214893266576777, -0.155388943236063, 0.521420011906788, 0.158688002977296},
    {0.026502069349489, 0.030258215665097, -0.00366448453027248, 0.0118223838397412, 0.0220838867907565, -0.0613915956629153, 0.29971631803346, -0.0767262716388461, -0.105520857724535, 0.123094447614705, -0.0406925701304423, -0.0142549464553037, -0.0782883499269335, 0.0525038935933199, 0.00355522180364739, 0.00304407840610801, -0.0891206073830918, 0.0989156565214024, -0.0302223496874192, -0.00876253400272332, -0.0661206272102358, 0.042190831555368, 0.00264046129850589, 0.00187119892902533, -0.0122845275854093, -0.00572826125448676, -0.00529430977445559, -0.00894093136026359, 0.300861406780796, -0.243089788685928, -0.0278777959462829, -0.422836525655093, 0.578614428417818, 0.236736750281293, 0.241764792404171, -0.180542653746288, -0.0171365171848402, -0.314040457459371, 0.355675036688131, 0.145522386249459, 0.0449173939628025, 0.024512327681427, 0.0343599167627112, -0.058345463819148, -0.0482901013499825, 0.0270365518840589, 0.0333252470857065, 0.0181862593654885, 0.0254924120689173, -0.0248862892142949, -0.0205973412450427, -0.0023621250377042, -0.142605118785308, -0.118028295404669, 0.161511294647256, 0.280936830180622},
    {0.0190269258810564, 0.0257416392900366, -0.00428139506100082, -0.0295941247801588, 0.0263560489909357, -0.0712908034861193, 0.124137438646549, 0.383808683073226, -0.121303800469527, 0.223660496831055, -0.196471909613941, 0.0713076774021096, -0.0370696875098228, 0.00247955111857612, 0.00125087806345139, 0.00149639524287998, -0.0878504334226814, 0.157568839051331, -0.132636080114133, 0.0438329214248812, -0.026846546455891, 0.00174684397405048, 0.000844454371940279, 0.000919836089063056, -0.00498781022541955, -0.00276472412449655, -0.00264930503155434, -0.00423354856605195, 0.189715199007309, -0.0148141401162544, -0.0112348646634435, -0.408354348391796, 0.0479359119350409, 0.539259559420539, 0.133654373845705, -0.010000867193394, -0.00690608582347857, -0.275675643274785, 0.0294662669971707, 0.331483547871064, 0.0248315981213798, 0.0158283391111046, 0.0216664494002771, -0.0512176787686353, -0.0466361623797635, 0.0615862093229059, 0.00758838205535277, 0.00483704205785011, 0.00662113227785838, -0.000567810831510235, -0.000517019099184951, -0.000392101030648698, -0.0434442928364933, -0.0395581202410417, 0.00464364975090457, 0.128003012885621},
    {0.00894945296698984, 0.0136019878329511, 0.0293017672320873, 0.192230415140459, -0.182241449919052, 0.488039973885573, -0.759220407776989, 0.964112153400919, -0.0769454525440012, 0.181036347722612, -0.229609921292739, 0.179122050766761, -0.00755505789865079, -0.00292270344760463, -0.00238285241145429, -0.00332447825028208, -0.050226989872396, 0.116925136398511, -0.145927351620368, 0.110106556022686, -0.00493164708263815, -0.00188767340681879, -0.00151440904534088, -0.00204356107550104, -0.000916248940524252, -0.000570886316972339, -0.000561763579208133, -0.000862826386782874, 0.0459034376839123, 0.0202583342166771, 0.0229909184989448, -0.123208195799601, -0.0756899671933493, 0.217522740600863, 0.0296474480394172, 0.0128750754489866, 0.0141325473043703, -0.0783043067552675, -0.0465267206212459, 0.133711509675399, 0.00550818872481608, 0.00389379030146964, 0.00524240817331101, -0.0145481290329096, -0.0140710083006508, 0.0248422133666611, 0.000540833582190897, 0.000382320334733087, 0.000514737336232834, 0.000234869226073006, 0.000227166450212521, 0.000257808232733803, -0.00368881245606489, -0.00356783408859105, -0.00219181234952881, 0.0133302788979783},
    {0.0201430117194883, 0.0267678423103474, 0.259211232824319, 0.0195251039488212, 0.326169225832269, -0.374897896525802, 0.278859093198177, -0.113874016814714, -0.00458647086152834, -0.0028726115401916, -0.00301923029022305, -0.00555807139457076, -0.15986680450647, 0.404281086211673, -0.571256238409763, 0.554909447702643, -0.00337553010430074, -0.00205150924371685, -0.00205803437210193, -0.00341655366698131, -0.117658048548054, 0.288722082265175, -0.389392282440996, 0.341103554416912, -0.0057427660169397, -0.00312601768238469, -0.00298367308690569, -0.00479647185936969, 0.029797188027501, -0.0882709367117858, 0.189806405664418, 0.0218620492158882, -0.0985550583339168, 0.0349667931625685, 0.0212800115225836, -0.0601691836488128, 0.116674242781906, 0.0149020923897652, -0.0605819216858598, 0.021494132932306, 0.00103865505607159, 0.000651456689925417, 0.000894002818805806, 0.000727355508724811, 0.000655925438529201, 0.00104910609763184, 0.0362035364034465, 0.0227072846284331, 0.0311615135421645, -0.102365415934278, -0.0923126030276816, 0.198497082175759, -0.00754475450407244, -0.00680382061772644, 0.0306719160336634, -0.00759650869077078},
    {0.0185719699093626, 0.0252996401097378, 0.00525730855264615, 0.0197079603842469, 0.692635678336204, -0.462466964864018, 0.306530718648313, -0.118670820076184, -0.0218777468924617, -0.00534047574345136, -0.00397432758406008, -0.00579219833361026, -0.133862165186805, 0.28886877039115, -0.324332910255171, 0.206851936598334, -0.0157466541584322, -0.00374279051727014, -0.00267315597091026, -0.00356047179888874, -0.0963481856909963, 0.202449247313067, -0.218148211810369, 0.127152152668922, -0.00470265394852335, -0.00262485806158838, -0.00251913179137709, -0.00401625659770305, 0.125636862646552, -0.316083324879609, 0.531027891320699, 0.0357549512968218, -0.134600295702078, 0.0398148630221872, 0.0880506682779476, -0.212599492451486, 0.326423530855206, 0.0240489893012458, -0.0827389756646381, 0.0244742477384673, 0.00429766082129837, 0.00275646118253001, 0.00376947345663182, 0.00117380595892297, 0.00107275314758299, 0.00119456237747495, 0.0262958606114857, 0.0168658072497395, 0.0230640696685631, -0.0634917000508115, -0.0580257073642749, 0.0974846396461413, -0.0276142623090702, -0.0252369538466308, 0.0950050643944836, -0.0107468543355458},
    {0.0157384636757173, 0.0222798637815405, -0.0132573892568216, -0.0116539403654932, 0.130691148487769, 0.574537076169525, -0.227302951843894, 0.0757238026633065, -0.0438437190827519, 0.0158018985425534, 0.00418435023440368, 0.00369599943203781, -0.0937368491673829, 0.147291418079095, -0.0985846245778231, 0.0136379351193381, -0.0304886776317671, 0.01075840883896, 0.00275781261113268, 0.00227193562591241, -0.0651840819226079, 0.100280437182776, -0.0649749437055042, 0.00838325633735881, -0.00318156671073536, -0.0018460317713781, -0.00178705171664079, -0.00281237972201506, 0.199489724091347, -0.362716030615443, 0.354020818079539, -0.0831958264843426, 0.220586062558379, -0.032125552645966, 0.135818617311492, -0.239058106385729, 0.217617054249947, -0.0548325275417143, 0.135594537640337, -0.0197476187160159, 0.00662916434183065, 0.0044007472642863, 0.00598527537215425, -0.00267631819221317, -0.00249611820153071, -0.000963860569483388, 0.014172998801565, 0.00940869504609308, 0.0127963791968915, -0.0249462873527773, -0.0232666213244205, 0.0227088620848719, -0.0337869513785021, -0.0315120319233677, 0.083551246966671, 0.0191640690194896},
    {0.0118031792992786, 0.0174601641969977, 0.00832487477036396, 0.0118223838397412, -0.0614326220634956, 0.26203544501963, 0.518569562880659, -0.0924570392473859, -0.0571355455781428, 0.0611192621110202, -0.0176401553036296, -0.0045127311694269, -0.0510812001582031, 0.0451053215830698, -0.00657817017556001, -0.00268073106307841, -0.0382036012222131, 0.0402707906090502, -0.0113654215697485, -0.00277398168006024, -0.0341553717751245, 0.0297193862963566, -0.00423826638234887, -0.00164784884783918, -0.0016670878936601, -0.00101096290126257, -0.000988567458496728, -0.00153258545360951, 0.180012228074643, -0.182326851992764, 0.0376689424123682, -0.220879630062072, 0.316927383528957, 0.0903099165359304, 0.118608021325498, -0.117471841982963, 0.0231551475671938, -0.142311111700304, 0.194815672108918, 0.0555136223703436, 0.00578913319241656, 0.00399603386838145, 0.00540089350611576, -0.00694605618732188, -0.00662703513197352, 0.0027095617168546, 0.00517568999039118, 0.00357259572485543, 0.00482858996842375, -0.00512611061131785, -0.0048906766941493, 0.00101041956648676, -0.0204579539644997, -0.0195183534360832, 0.0280057544625228, 0.0339275351802638},
    {0.0072737451295291, 0.0112160500006485, -0.0107414954878784, -0.0295941247801588, 0.0716512169321643, -0.219640225699807, 0.508137289414865, 0.462498877065499, -0.0509893480914855, 0.0895677407263755, -0.073035856888642, 0.0225740853843906, -0.0187767257796372, 0.00406456138455535, 0.00143261340615383, 0.00128784458422875, -0.0328603051059604, 0.0572391342198014, -0.0460842819145469, 0.0138763194503273, -0.0121007418432333, 0.00259749741088061, 0.000903952700718136, 0.000791639729753556, -0.000590624525008208, -0.000373344867856812, -0.000368625559680043, -0.000563356708674793, 0.0915504212460301, -0.0225418936754919, -0.0109577172391413, -0.182923176377682, 0.0621173832613313, 0.205715782381084, 0.0585061854534171, -0.0142235201623365, -0.00673572294369999, -0.115421158702178, 0.0381836357431632, 0.126453757203726, 0.00285562558404501, 0.00204438250187339, 0.00274678048751675, -0.00563358576833767, -0.00548823047180241, 0.00617207533649786, 0.00105157842819885, 0.000752839780526246, 0.00101149643840173, -0.000255650695047835, -0.00024905450851305, -0.000121066531530378, -0.00575829138967029, -0.0056097184936679, 0.0019049583577091, 0.0154583744682552},
    {0.00304595863694749, 0.00485293934295218, 0.0263784345157957, 0.192230415140459, -0.168273534214414, 0.466883678827404, -0.770250380155342, 1.16177879234705, -0.0270708836687134, 0.0625083209661633, -0.0770601206703548, 0.0567052050431303, -0.0032029721181296, -0.00102381350470696, -0.000765049394997455, -0.000971775672177766, -0.0169490283234974, 0.0390069530753053, -0.0478485849051347, 0.0348567628001824, -0.00200537469754974, -0.000638888466666501, -0.000475038588243992, -0.000597351761171136, -9.78802368936556e-05, -6.39179461700314e-05, -6.36001340202155e-05, -9.60985345167253e-05, 0.0190991109554806, 0.00689773991227449, 0.00696279311635672, -0.0498281220951642, -0.0243095118088303, 0.0829801901267505, 0.0119183832377093, 0.00428298179362907, 0.00428003792419035, -0.0309395457727476, -0.0149430883155103, 0.0510080300769948, 0.000581723792625943, 0.000428493256672421, 0.000573029207156101, -0.00151012679740932, -0.00149498944557296, 0.00248964847991301, 6.88283807441931e-05, 5.06984541297079e-05, 6.77996550041554e-05, 2.47341183559693e-05, 2.44861861607368e-05, 2.47171175797589e-05, -0.00046141183789225, -0.000456786694266379, -0.000222851295041071, 0.00160984056801328},
    {0.011212845430362, -0.012513198351154, 0.0208770933958504, 0.0195251039488212, -0.00350519620228991, -0.00213578148775574, -0.00215103292273904, -0.00360148326124408, 0.110772181805886, -0.208071106282045, 0.201984382314542, -0.0990472496003532, 0.12211446906808, -0.225881439142981, 0.217718994875169, -0.10626977667799, -0.00226985035225309, -0.00179692337955266, -0.00280290297783916, -0.020199909014669, -0.00250226696008475, -0.00195073523785083, -0.00302124952471564, -0.0216728867138268, -0.0688168296738457, 0.183176207073004, -0.294334759695954, 0.684912572298823, 0.0682600995319013, -0.0455513793747795, 0.0192696123733746, -0.046256117619921, 0.01354750953446, 0.0197074695465102, 0.000589501209135618, 0.000632108757277121, 0.00392988617312101, 0.000641888290245429, 0.0027629082188112, 0.00401918370631309, 0.0162123406290093, -0.0593557148961158, 0.382855534749969, 0.0176530793253, -0.259440152714044, 0.110534761061572, 0.0178723695424869, -0.0654333199232007, 0.422057231279378, 0.0191641359271616, -0.281647539217126, 0.119145434915933, -0.00579129401155807, 0.0851123010939676, -0.0249277235077287, -0.0253133873528488},
    {0.0164398153896086, -0.0273770165770783, 0.019287579585398, 0.0197079603842469, -0.0163810014243473, -0.00390242657177244, -0.00279689872237231, -0.00375319132544435, 0.180416762354003, -0.246495727242803, 0.220428635877593, -0.103219493481902, 0.0349131389634355, -0.121655788733429, 0.151792179115675, -0.0871793205263413, -0.0164552242707861, -0.00492616592994091, -0.0051412964693963, -0.0210508054013366, -0.00318431349806171, -0.00243126567889094, -0.00354041384624268, -0.0177795380457134, -0.0875743328428928, 0.191612435862307, -0.223629290575918, 0.195819938431223, 0.18340167254765, -0.159958997720859, 0.0798527093604428, -0.0627187368589139, 0.0218860330331136, 0.0224398673581593, 0.00366524434684593, 0.00373089742608158, 0.0162853332138362, 0.00146285721496194, 0.00446348462723356, 0.00457643479011293, 0.100800793821525, -0.294038759305839, 1.02865870250385, 0.0402311973122853, -0.351775278729851, 0.125860165397376, 0.0195063478381805, -0.0569005669447607, 0.199059686904238, 0.0198557520467494, -0.173615581371304, 0.086670176465348, -0.0299334554728395, 0.261733439364735, -0.0913332599904377, -0.035811093976741},
    {0.0241754756606097, 0.00769851183855549, 0.0163965439250797, -0.0116539403654932, -0.0318063021955915, 0.0112436886986536, 0.00289018334287365, 0.00239490987846146, -0.0489992445746998, 0.267079884396059, -0.160437252917034, 0.0658643173647257, -0.00331321111484878, -0.0316540782798459, 0.0739736637383711, -0.0591471639811746, -0.0533456433751073, 0.0225203407828069, 0.00770119898836343, 0.0134325104780699, -0.00360710415218165, -0.00266909142798146, -0.00355083307641825, -0.0120625997765329, -0.0992018341832288, 0.148164197799285, -0.0843456371815984, -0.0185830554278951, 0.106637314753757, -0.176833383630821, 0.123463292225494, 0.100887065451003, -0.0499823516793694, -0.0181061313655988, 0.00899172423205338, 0.00848823481059452, 0.0251793692370997, -0.0048427117285036, -0.0101935082532452, -0.00369260335515106, 0.247288544676479, -0.471191240681806, 0.598104697243668, -0.133183258819302, 0.565852843131987, -0.101553215623276, 0.0167210568633936, -0.0318608188635712, 0.040442401672743, 0.0157847653325304, -0.0670643924827068, 0.0468236851945558, -0.0531762439644611, 0.225928762376926, -0.11193160198611, 0.0638592703693539},
    {0.0299005753431563, -0.000692705759853969, 0.0123418011338163, 0.0118223838397412, -0.0399845505836983, 0.0422013001459018, -0.0119330872043856, -0.00292413044299179, 0.00436230428657888, -0.0132065134800396, 0.346278038065547, -0.0804188321427703, 0.000123346670198285, 0.00022427643903693, 0.0207966526318344, -0.03109314934069, -0.105804882736616, 0.125936349182971, -0.0430093491732371, -0.0164007895111114, -0.00299169409535092, -0.00213868376258172, -0.00258304136084945, -0.006341203720403, -0.0822769537703659, 0.0620609909795396, 0.00080833932683623, 0.000691823711102744, -0.00123018051608045, -0.0823604990239474, 0.108110410393762, 0.137130787640276, -0.129964235452736, 0.0508991465589973, 0.0117309116645637, 0.0102295585373867, 0.0220482695108046, -0.0170322841176122, -0.0265051855745516, 0.0103804813719166, 0.32262111230204, -0.296844513069609, -0.00689980563393293, -0.468418363737866, 0.769135723398206, 0.285481857011572, 0.00912229806172754, -0.00839344984860805, -0.000195095984610497, 0.00795480220858049, -0.0130616624471412, 0.0171453755662, -0.0436329501586934, 0.0716446307025767, -0.0679004315061572, 0.11305467747162},
    {0.0274659679169433, -0.0063064816626327, 0.00763236993753088, -0.0295941247801588, -0.0344990911194155, 0.0601371945948342, -0.0484704728084982, 0.0146274102792546, 0.0395383094710887, -0.109983778158357, 0.203317202134924, 0.402280019603826, 0.000460484759394759, 0.000476087623059326, 0.00109193877883999, -0.0110500905791669, -0.130480475709523, 0.243240342500803, -0.217371432971372, 0.0820418520171363, -0.00151964692640046, -0.00105291633395813, -0.00116741866689648, -0.00225357922813258, -0.0417930162395376, 0.00441662708652153, 0.00213506992707123, 0.00258275537263011, -0.0035554462875545, -0.00598583280785825, 0.0534650123740611, 0.0161051023871778, -0.10559117951605, 0.115942502867309, 0.00786323208030551, 0.00639960141743276, 0.0109037695623235, -0.017218361984582, -0.0215344921497768, 0.0236455632872445, 0.216252986347212, -0.0268441585068067, -0.0199416979909376, -0.473535838844077, 0.0903298944614686, 0.650295403023377, 0.00251860045911427, -0.0003126417400369, -0.000232251912742697, 0.00204979821318837, -0.000391011706139539, 0.0034924874078851, -0.0153079960375439, 0.00292009506579971, -0.0191452543972815, 0.0515110081076783},
    {0.0152327680704991, 0.0287678230747217, 0.00320507108782466, 0.192230415140459, -0.0178382131382485, 0.0410650708630083, -0.0503948504088267, 0.0367434731025106, -0.179978494886957, 0.485949924576904, -0.766229854663151, 1.01051141642987, -0.000673485269869333, -0.000520749400288633, -0.000669087236009835, -0.00183577670380725, -0.0864392103719315, 0.204225002221373, -0.260674168295515, 0.206085870657986, -0.000323458282953146, -0.000218849807463908, -0.000227625898028591, -0.000374392247515946, -0.00889568296255187, -0.0033275077672758, -0.00269364658708953, -0.00377742729515851, 0.00469595144355351, 0.00448735510346658, 0.0109135407614041, -0.0156696993884495, -0.0283431930980594, 0.0467680739919021, 0.00197351752822317, 0.00152661443863583, 0.00222573096474409, -0.00533088841509655, -0.00578037173244153, 0.00953798155162844, 0.0542752718892438, 0.0232114501770192, 0.0263385347137001, -0.146608993334406, -0.0878878170391841, 0.262311600776788, 0.000203099798998185, 8.68579870966363e-05, 9.85596371987502e-05, 0.000157107844851068, 9.41815724235986e-05, 0.000229055736824829, -0.00141674826780969, -0.000849299280487138, -0.00153620391228615, 0.00536437454800343},
    {0.0170982512251828, -0.0251141783214145, 0.0160091858217584, 0.0195251039488212, -0.00316207558373215, -0.00241577235429526, -0.00353354291864167, -0.0181912548602126, 0.0293790293907047, -0.114578371119062, 0.148172383047726, -0.0869289168370504, 0.163589384055957, -0.204331219412855, 0.180652919655244, -0.0841378732334324, -0.00335839069175048, -0.00254526063085208, -0.00367028490867618, -0.0177284701790032, -0.0187003136617075, -0.00453904348042347, -0.00447483985261538, -0.0171592587463126, -0.0893615104409752, 0.190336131394753, -0.212238483655012, 0.159427677701374, 0.189862433322843, -0.0548926656233985, 0.019129077007151, -0.171395695833932, 0.0195319060365618, 0.087364293618039, 0.00421763175802373, 0.00135971169586752, 0.00390122508841143, 0.00424553498359206, 0.00398337890666881, 0.0178172618556884, 0.0201544183268212, -0.0570169166961804, 0.185032341811612, 0.0202877569663798, -0.167035397269227, 0.0851417500342238, 0.11222456794757, -0.317483677185369, 1.03030384104418, 0.0361797962355779, -0.297879486990314, 0.103805482586356, -0.0336178884956979, 0.2767865057501, -0.0315420290818583, -0.0984861631531846},
    {0.021438292355929, -0.00383082689433904, 0.0205403356838656, 0.0197079603842469, -0.0201247738104083, -0.00584323645334017, -0.00577935460978638, -0.0189575391547738, 0.0135858929586521, -0.128120058977522, 0.160325430484326, -0.0905906908173124, 0.0132794918987443, -0.130660138804077, 0.165311291244429, -0.0939995744532915, -0.0209027123750646, -0.00602728243276247, -0.0058903857264227, -0.0184752602365972, -0.0204312959399314, -0.006146777995273, -0.00607356716536564, -0.0191704752937159, -0.0976332000889082, 0.170518145142878, -0.133170339698606, 0.0129416622398117, 0.481495556956455, -0.240406207919514, 0.1049373819195, -0.230416174847411, 0.0396909506048682, 0.0994771630117902, 0.0226514859192026, 0.00883256817957496, 0.0214011552624064, 0.00846553253191366, 0.00809465779372033, 0.0202875864800227, 0.108242622668801, -0.245024815275157, 0.469246332285473, 0.0404534804829545, -0.224554398029832, 0.0969464685915433, 0.105801439419834, -0.23949879918387, 0.458663474463193, 0.0412555021999969, -0.229006363638902, 0.0999613464691265, -0.152030613093807, 0.843911139358477, -0.145370156280727, -0.139329327795018},
    {0.0272195064369147, 0.0115162223864431, 0.0267098865309273, -0.0116539403654932, -0.0579732173172456, 0.0239119304978302, 0.00786274267358259, 0.0120967981262488, -0.0568368116107849, 0.109442245504126, -0.114072987754541, 0.0578058834528901, -0.0194120182105407, -0.0352876028153142, 0.111856327157735, -0.0946171790250351, -0.058857340988391, 0.0241488423712529, 0.0078709346935096, 0.0117890561421181, -0.0201021088746972, -0.00778634204845774, -0.00771798708392591, -0.0192964308977998, -0.0960601434065918, 0.112964564404558, -0.0353243681982651, -0.0189181773662322, 0.220716232256439, -0.3420507329146, 0.23044406538496, 0.36232170122116, -0.119340321283624, -0.080265473615362, 0.0487018653261888, 0.0236011963360773, 0.0469972581018547, -0.0249998751193356, -0.024338521679677, -0.0163695132433425, 0.232727232578612, -0.342407108105344, 0.215101221529172, -0.119464659358852, 0.353104253921159, -0.0782235236836696, 0.0794855508053784, -0.116945564495709, 0.0734655711869932, 0.0385191424984336, -0.113851854988648, 0.0767034880809724, -0.240928805691316, 0.712118434341345, -0.234555210079491, 0.248455666275406},
    {0.0302616520385567, -0.00801784374588158, 0.0302516266319403, 0.0118223838397412, -0.105865563934372, 0.12409022809239, -0.041339453116336, -0.0147699151361855, 0.0450891617645784, -0.110330824686386, 0.228755234415404, -0.0705796677815743, 0.00643968899227087, 0.00649089001066503, 0.0403972016220275, -0.0722518641953718, -0.105632026424746, 0.123383715498899, -0.0408731176137138, -0.0143941691791135, -0.0150864946514284, -0.00725880667245086, -0.00721801875870379, -0.0147352005106209, -0.0720924778963432, 0.040167198565973, 0.00641766861494237, 0.00627586360252868, -0.0519109273669998, -0.188485078243955, 0.278998016488917, 0.457575033785653, -0.395146024404499, 0.225638708936901, 0.0580523449520112, 0.033677798854234, 0.0568994552709837, -0.0817577714486638, -0.0805869296995663, 0.0460172433773117, 0.277409530310442, -0.186358845866078, -0.0505903157787711, -0.39068852421984, 0.44593434611638, 0.219898470630842, 0.0396199669450186, -0.0266159973122159, -0.00722537050781884, 0.0229846576996804, -0.0262348332920277, 0.0388331348008336, -0.183819922143654, 0.209813218721952, -0.181187462435566, 0.439859006441802},
    {0.0258677865545683, -0.000295237250846746, 0.0261981528295216, -0.0295941247801588, -0.123694165861968, 0.228667944176925, -0.201694509957147, 0.0738837109693798, 0.00176428765404604, -0.00456258703054617, 0.0072446044846279, 0.353061458146974, 0.000103788161142548, 8.21838039899297e-05, 0.000147945230318202, -0.0347719661720228, -0.121961990153499, 0.2250337245264, -0.197896745384657, 0.072004112783863, -0.00717468642842259, -0.00405343007898567, -0.00404133581582028, -0.00709146953368306, -0.0342850300686168, 0.000145593936763992, 8.06363412488749e-05, 0.000101147795129559, -0.00074500280348011, -0.000887527786824063, 0.178424152469219, 0.00289858008214561, -0.385626002220496, 0.513979475619877, 0.0367446701898162, 0.024244092389531, 0.0363882052297272, -0.0791788655558164, -0.0786453959092754, 0.104822079207859, 0.175588457403319, -0.000870816268069313, -0.000726049966660647, -0.378364938100432, 0.00282484033908878, 0.500903861562424, 0.0103293831195579, -5.12277116198022e-05, -4.2711510656615e-05, 0.00681531545619469, -5.08825635931187e-05, 0.0102291763923665, -0.0617813836177233, 0.00046125453781265, -0.0613651299539456, 0.200412591091128},
    {0.0137109026229979, 0.0299453386201052, 0.014002682081672, 0.192230415140459, -0.0792712548026677, 0.186695351837455, -0.237148368754727, 0.185592944676423, -0.184439512927778, 0.48729939001709, -0.741538613135988, 0.886876346755306, -0.00348612303332125, -0.0024573505152346, -0.00294386828509749, -0.007159885649681, -0.0775772130015774, 0.182572154863325, -0.231653156327065, 0.180871468758637, -0.00146630027813814, -0.000920673795231267, -0.00091965053197969, -0.00146020247167436, -0.00700687753076355, -0.00287885247894111, -0.00240040868105143, -0.00339743622479534, 0.023785331250659, 0.0204361518344421, 0.0435460183840248, -0.0765978577452828, -0.117058944746748, 0.207325437623335, 0.00891143980381032, 0.00638415719932214, 0.00888086860420264, -0.0239288085613586, -0.0238732528442568, 0.0422823954558634, 0.0425842975405796, 0.019962604425603, 0.0231802335251375, -0.11434644973226, -0.0746492117914783, 0.202051088091408, 0.000804893149832453, 0.000377316628028927, 0.000438133590395032, 0.000576625608242523, 0.000376440608824006, 0.000802131918237234, -0.00558130841108412, -0.00364366602223467, -0.00556835024016848, 0.0208710379051656},
    {0.0254129579368256, 0.0107425993815027, -0.0100835618309505, 0.0195251039488212, 0.00276855422334095, 0.00354739278284653, 0.0184970121822559, -0.041398111534134, -0.0052792539735776, -0.0289195606742811, 0.0792937759693209, -0.0676531927685027, -0.0668972246841804, 0.26996967643304, -0.141961339914384, 0.057331901500994, -0.00476624355041176, -0.00338273614995236, -0.00435470127501964, -0.0137973375736295, -0.0603965005827018, 0.0315784943674165, 0.0077963146586417, 0.0116923912438848, -0.0987005432142833, 0.135372028365773, -0.0635582267339036, -0.0222957897716066, 0.109055095339958, 0.15377203670886, -0.0218610650020318, -0.208732280105511, -0.0795719254745864, 0.154730479224276, 0.0127562315865689, -0.00844494131011669, -0.00445839259329854, 0.0114632796231686, -0.0162280695445273, 0.0315560665716392, 0.0208463565738798, -0.0362021324157861, 0.0363463430762804, 0.0187334020168052, -0.0695671764823022, 0.0515692280558871, 0.26415917979114, -0.458743261526414, 0.460570658542209, -0.174879920823443, 0.649423009457742, -0.0923254899097836, -0.070643291092639, 0.26233645625731, 0.10000665415058, -0.135750409459011},
    {0.0277849136316368, 0.0106226666604082, -0.0118196797021562, 0.0197079603842469, 0.0153251463175155, 0.0068247368416488, 0.0170656692030986, -0.0431419561966856, -0.0217181624512596, -0.0244158730639453, 0.0837575252759869, -0.0705030005191931, -0.0483096864799012, 0.0735262174169138, -0.100868692067401, 0.0557087659591239, -0.0260190574047761, -0.00708763218492746, -0.00629024608249978, -0.0143785334928047, -0.0578765587810759, 0.0213437702446785, 0.00757530613557601, 0.0113613654920689, -0.0945824300406851, 0.104754249861408, -0.0271010761760106, -0.0161008564820599, 0.208816727025423, 0.370860928123472, -0.0953864536822709, -0.273937984971186, -0.0912135817420511, 0.176183523808719, 0.0606169663210653, -0.0278519033674187, -0.0194533184251786, 0.0205729256103857, -0.0186022939508756, 0.0359312466038149, 0.0990608304482566, -0.13669587008386, 0.0695953213086701, 0.0336204732668515, -0.0912992093798989, 0.0587191894223527, 0.220350026027013, -0.304065071880222, 0.154807210805021, -0.101245047457608, 0.27493940115423, -0.0707151723594293, -0.28753905994854, 0.780836385935845, 0.259996376636918, -0.19204741754843},
    {0.0300547935497049, -0.00262086915298649, -0.00729939888431999, -0.0116539403654932, 0.0195387056426204, -0.0109690858690787, -0.00648860340814748, 0.027528864934527, 0.00953046266998924, -0.00904585198043095, -0.055682659374989, 0.044987936335647, 0.00740752011267507, -0.00240693191785919, -0.0250468724285161, 0.0248177503627264, -0.0629779646757726, 0.0247338267998048, 0.00750215134912612, 0.00917493645108301, -0.0489494116020339, 0.00658120841508748, 0.00337457711054262, 0.00506138536202277, -0.0799936001014607, 0.0564773593641095, 0.00278290510283716, 0.00246880960967895, -0.0415161074774566, 0.137785921551927, -0.0822707305417843, 0.402484695490358, 0.0766458066681742, -0.14215779334252, 0.113516362413503, -0.0185639631595198, -0.0167784696518061, -0.0542269556656961, 0.015631310584595, -0.0289919659842306, 0.185509533264766, -0.159308678962164, -0.0138366637603084, -0.0886182134630823, 0.134141800340848, -0.0473789501690622, 0.144186661900357, -0.123822135860404, -0.0107545004524866, -0.0235796481027297, 0.0356926225918727, -0.0213117429016787, -0.406714954274354, 0.615646310841458, 0.117238515266376, 0.342463929443951},
    {0.0291131058185436, -0.0112132650061804, 0.00962857943557122, 0.0118223838397412, -0.0386200930353358, 0.0549301045311348, -0.0255353246076025, -0.0336121174078451, 0.0538868521712186, -0.101042221662702, 0.0849937731483468, -0.0549292461438858, 0.0175143255728478, 0.00367186688349735, 0.00664416170108381, -0.0205131183769429, -0.0994794309404829, 0.112846675863041, -0.0356794946677951, -0.0112023885451424, -0.0323328431907148, -0.00410083987857552, -0.00278914940712226, -0.00418348946077043, -0.0528386438915602, 0.0136495564365595, 0.00513055372915034, 0.00583724844800681, -0.108188894132857, -0.0432583250009818, 0.096116508702793, 0.386897800957663, -0.189958312030145, 0.399627629543894, 0.120828272253816, 0.0181593912005338, 0.0196022074155318, -0.162415639580523, -0.0387405065266482, 0.081500917886222, 0.197458726781111, -0.0604431390604202, -0.036057651877137, -0.265421203189447, 0.128946934255838, 0.133189585327345, 0.0641781118900005, -0.0196452524780159, -0.01171947198478, 0.00964538694945504, -0.00468592208119773, 0.0104117408727183, -0.284194329656931, 0.138067294932717, -0.0677880055845992, 0.606288622857957},
    {0.0224445447174005, 0.0122972972658715, 0.0280757815135756, -0.0295941247801588, -0.142669865790111, 0.291843625603676, -0.302869456861528, 0.168138269227149, -0.0683914171489397, 0.159071760349638, -0.206289585296184, 0.274773179699455, -0.00915583372768278, -0.00430258606699974, -0.00537660975155995, -0.0312130576026593, -0.104568501634171, 0.189850631265637, -0.162829463999105, 0.0560378329736125, -0.0139990053434685, -0.00513509550091868, -0.00424389085237572, -0.00636565806912712, -0.0228773094223896, -0.00349760168066443, -0.00231316749585582, -0.00305149496021578, 0.0591095569907137, 0.0379494934220749, 0.177224025148592, -0.187830363035947, -0.45066283116216, 0.910306571261337, 0.0705466934159361, 0.0299544723214305, 0.036143448683932, -0.148259144018291, -0.0919091466194161, 0.185649879114551, 0.115288086146478, 0.0204025052145787, 0.0197002829695774, -0.242286521734026, -0.0626008972232349, 0.303390821313913, 0.0154340791804501, 0.00273136532565274, 0.00263735601303532, 0.00655338578226163, 0.00169323422072116, 0.00790739894148369, -0.0900313728410425, -0.0232618994971414, -0.0558124539406071, 0.276242777973244},
    {0.0111030516956888, 0.0299283676160423, 0.0249036082205118, 0.192230415140459, -0.14494000231401, 0.354346112869683, -0.476703569824698, 0.422356647889497, -0.179896704701628, 0.459596310372118, -0.662594871472996, 0.69021930367927, -0.00773801956853011, -0.00465882113210353, -0.00499260080123196, -0.010188321625135, -0.0625327818570342, 0.146258539859399, -0.18384166808212, 0.140764808621599, -0.00268976516544258, -0.00148258887390959, -0.00138523266460874, -0.00207782821502164, -0.00439564015111303, -0.00206072672104513, -0.00179668352082427, -0.00257895986512159, 0.0510514100330865, 0.0367063907025685, 0.0643230528044015, -0.155757438084702, -0.183128903994627, 0.367193083012779, 0.016246224176946, 0.0101844496336538, 0.0131181816702361, -0.0432160109691646, -0.0373477023700646, 0.0748861467390755, 0.0265497360935649, 0.0141558916760697, 0.0170146296952578, -0.0706240215419473, -0.0519114972451433, 0.122379662580786, 0.00114200189365346, 0.000608897016658106, 0.000731861863459846, 0.000715899315482698, 0.000526214799611786, 0.000922121235404486, -0.00784483208536685, -0.00576626720338018, -0.00677958116209418, 0.0287680203061011},
    {0.0302500012368685, -0.00674620730936927, 0.0100401662714875, 0.0195251039488212, -0.00508376488671095, -0.0317222402070974, 0.0822967853898158, -0.0686320373360359, 0.00197276041282772, 0.00282759490804086, 0.0246420043988714, -0.04503256618228, 0.0414434833137069, -0.117441550053527, 0.394513248650746, -0.0700756553903234, -0.00538804273701254, -0.00355870072593614, -0.00406902438226489, -0.00918403835795023, -0.113191271384467, 0.147807356790049, -0.0651442148090813, -0.0142913798085189, -0.0742698835373354, 0.0442577376376679, 0.00580669741102551, 0.0055457852515229, -0.0176773769273056, 0.212615825443168, 0.130126061606394, -0.107540712446679, -0.235656571469202, 0.170749967708241, 0.0222480575010768, -0.0351083038449277, 0.0265381887482213, 0.0177577186477453, -0.048060307798489, 0.0348231154916576, 0.0145979510551167, -0.010512427352851, -0.00236550908394901, 0.0116516377961437, -0.0143906266881723, 0.0228490121220381, 0.306671776781994, -0.220843648702358, -0.0496942941533089, -0.483940043727634, 0.597701424519648, 0.365807823698724, -0.0728429636639768, 0.0899663991696786, 0.197145552539167, -0.0997158754837049},
    {0.0300983577029971, -0.0108682780791435, 0.0110939404354585, 0.0197079603842469, -0.0231802053744836, -0.0276298646655858, 0.0870824133797188, -0.0715230777133198, 0.0148432501155472, 0.0072131036535002, 0.0237118389394892, -0.0469295077882568, 0.054737695342548, -0.11084070314001, 0.115608731683716, -0.056088598307651, -0.0269350059128872, -0.0068899503195714, -0.00551966637194164, -0.00957090470710664, -0.0993286602484081, 0.105874942979701, -0.0269115200303896, -0.0114388293177888, -0.0651740009484615, 0.028828893208499, 0.00702557127231329, 0.00732475842426733, -0.102273132930751, 0.535746834338047, 0.257050201911992, -0.128570110450796, -0.309816855637989, 0.194424079547109, 0.0976912073872445, -0.124711701733312, 0.0524233707828877, 0.0299286831739657, -0.0631847155811979, 0.0396512647545375, 0.0640995944875299, -0.0339579907193821, -0.0136857422900746, 0.0196375549684062, -0.0172046885375793, 0.0260169779825591, 0.236381119183878, -0.125227435768644, -0.0504691348713493, -0.301761974485488, 0.264377148370073, 0.126847598555409, -0.278436906801433, 0.243941787386094, 0.587829296106842, -0.141069013725581},
    {0.0286160770881658, -0.0100304424671004, -0.00170955493305554, -0.0116539403654932, 0.00630079692715634, -0.00613847546289087, -0.0581932607657199, 0.0456388471838216, 0.0278670398224955, -0.0165823879066066, -0.0112119814106073, 0.0299457000850141, 0.0359083999511919, -0.0395715317875216, -0.00282231830132849, 0.00532721841410369, -0.0578628584901558, 0.0216383431733439, 0.00606898946781382, 0.0061071904524212, -0.0745598627704425, 0.0516368564971356, 0.00152770678244034, 0.00108644437579395, -0.0489220790330338, 0.00994877186000767, 0.00412692477588731, 0.00480510466139422, -0.126171301109485, 0.248821960746169, -0.0306460030495001, 0.134356329875995, 0.457560220603032, -0.15687561199579, 0.164640818162663, -0.134686083023702, -0.00625001174450549, -0.0727264094622996, 0.0933158150499533, -0.0319935495606552, 0.108028244942132, -0.0259497034408979, -0.0168836903877481, -0.0477190678644039, 0.0179789750546524, -0.0209924066648649, 0.139201057956504, -0.0334377937414931, -0.0217556766329356, -0.113874830422633, 0.0429042901956706, -0.00528428039161075, -0.36308262200192, 0.136797588383316, 0.465874103411457, 0.251557919288668},
    {0.0243807773710822, 0.00537863848136858, -0.011422245664625, 0.0118223838397412, 0.0552995939385817, -0.104936601954282, 0.0910163993406658, -0.0557239934719332, -0.0220790357715199, 0.0324894082074784, -0.0161003738189843, -0.0365630181088649, -0.0118969892080667, 0.00420827497900532, 0.00240439388633077, 0.0195514382990292, -0.0812967864767635, 0.0892023530796897, -0.026772391802399, -0.00745674051607515, -0.0438056716503965, 0.0115541664574629, 0.00399812922953614, 0.0039873623582668, -0.0287428711768686, -0.00204387781190561, -0.00123786029074251, -0.00159200294020172, 0.0576723909467993, -0.0292076635307706, -0.122223005070747, -0.121504250912767, 0.450686934679817, 0.441001702974337, 0.158344311701009, -0.0485677550391042, -0.0249264224083987, -0.202042477253413, 0.0919140623426243, 0.0899388353673568, 0.10389682388926, 0.00859140789289665, 0.00771746652451483, -0.132569155480525, -0.0162591661904657, 0.0590129145694339, 0.0559833955320349, 0.00462936371142327, 0.00415845224881624, -0.0171713641699439, -0.00210601072884667, -0.00881285693118496, -0.235325313207766, -0.0288618673204915, 0.107055238101476, 0.445351148958011},
    {0.0168888172596302, 0.0265225731168815, 0.0116134748343783, -0.0295941247801588, -0.0648165501864932, 0.151531129548398, -0.19835664154622, 0.278748753109168, -0.136406455745618, 0.28376541171853, -0.302860847075892, 0.182899592666262, -0.0302745418665179, -0.00502674905678499, -0.0040985099512062, -0.00943905186116176, -0.0776057232015649, 0.138298675897828, -0.115271030719205, 0.0373009361247861, -0.0172240947344067, -0.00244988539799066, -0.00155992255535781, -0.00192502052858141, -0.011301503148731, -0.00374060763022849, -0.00292119558198601, -0.00405120647096804, 0.174811675791853, 0.0364890940310537, 0.0612509238451515, -0.457170278774762, -0.196463287614661, 1.00455203413528, 0.0851979215757075, 0.0138880133222222, 0.0124916450858427, -0.174002317425163, -0.0400671452317112, 0.204870501421384, 0.0559021878246976, 0.0212049137663457, 0.0233925320915246, -0.114170745608983, -0.0611765225010693, 0.134424749317638, 0.0124071336395171, 0.0047062952122467, 0.00519182313108008, 0.00202247231023233, 0.00108370863424555, 0.00181912313224393, -0.070334080695178, -0.0376873641955271, -0.0161956798487159, 0.202914971390714},
    {0.0077381811095763, 0.0260124381207849, 0.0299947250976383, 0.192230415140459, -0.180516285730209, 0.461954122435896, -0.667777779660335, 0.700205786033841, -0.152061844875105, 0.373986514224841, -0.50782563534139, 0.459436505525797, -0.010843593574508, -0.00531054391871699, -0.00498843785959981, -0.0084463518753209, -0.0433476663950503, 0.100657728235379, -0.125149498779013, 0.0936984686307274, -0.00309114016851067, -0.00142932235848191, -0.00122935995028978, -0.0017225671593686, -0.00202823607778984, -0.0010869582021326, -0.000995259096535694, -0.00145104215453633, 0.0688705436214522, 0.0394086713457842, 0.0558182505772274, -0.197907251068728, -0.170757133079533, 0.405209157119218, 0.0185363701620084, 0.00971194662742078, 0.0113836940204542, -0.0487726328729605, -0.0348245768128825, 0.0826392266190663, 0.012162546071783, 0.00738565375592508, 0.00921595422346853, -0.032001917806721, -0.0264830807255858, 0.054223312995279, 0.000867316232694329, 0.000526674049476932, 0.0006571935391331, 0.000454421706483275, 0.000376055172972307, 0.000532642719457771, -0.00589323248953531, -0.00487692496111082, -0.00420787879224299, 0.021131636671947},
    {0.02488002151319, 0.00349338776214513, -0.02833860959432, 0.0195251039488212, 0.0724066424646239, -0.168855440911288, 0.180838102102827, -0.0944993352356273, -0.000737780502525525, -0.000621569734380864, -0.00109018984479878, -0.0235470671456198, -0.0213408121217801, 0.0567285276972925, -0.0929049606346431, 0.521879511054613, -0.00422947460640336, -0.00262076359232691, -0.00270891496297763, -0.00480223949501897, -0.122340482894467, 0.239188061792398, -0.23085120375936, 0.106433229417867, -0.0304844426488356, -0.00144195494716539, -0.000849780857684078, -0.00108449112971489, 0.00535047517400664, -0.0206694422318368, 0.362180965134431, 0.00655089853965132, -0.270767634055307, 0.122934171541447, 0.0225595452321252, -0.0513596430982694, 0.0738639646439452, 0.0162777402116466, -0.0552209333838475, 0.0250714592272948, 0.00562132130380526, 0.000309623694825109, 0.000271898877739804, 0.0040560395561185, 0.000332901640170677, 0.00624723266456594, 0.162600612797479, 0.00895607985992297, 0.00786486339244143, -0.37018075297663, -0.0303827855030737, 0.532383334467843, -0.0349144074259299, -0.00286561887202161, 0.118444338190401, -0.0375393217377863},
    {0.02330143749075, 0.0093268697739225, -0.0195929661997851, 0.0197079603842469, 0.0976637162240657, -0.196492083346238, 0.196789453010076, -0.0984800038037923, -0.00968427887924612, -0.00305739080499761, -0.00351420065220275, -0.0245389584623717, -0.0491729940243791, 0.106169413017208, -0.121847957868656, 0.123566481166195, -0.0200092686815672, -0.00483826550316711, -0.00354803175299672, -0.00500452794251928, -0.101599268420437, 0.168011170718831, -0.123020984383693, 0.0252004138115012, -0.025316207669363, -0.00300028765102176, -0.00191419023133729, -0.00249885878459787, 0.0596887115499932, -0.179428858511902, 0.955860192068403, 0.0262363368356126, -0.366088874082172, 0.139978727185895, 0.0944563035738696, -0.181156214573332, 0.19494018247284, 0.0264889132374166, -0.0746609519959467, 0.0285475625477052, 0.0235363446423736, 0.00323502747564214, 0.00303324343285019, 0.00660042969678805, 0.00133327046843725, 0.00711339789299559, 0.119508385589292, 0.016426208777337, 0.0154016280466348, -0.229203196864412, -0.0462984786883525, 0.246643004355891, -0.128858614879586, -0.0260291213928504, 0.363197492232307, -0.0531072416281612},
    {0.0202336576842804, 0.0190977808372876, 0.010730605592219, -0.0116539403654932, -0.0616020791184594, 0.199029865952139, -0.142163327346853, 0.0628400509032686, -0.0433432647799655, 0.0190435548721708, 0.00697704744749569, 0.0156582995463821, -0.0769004177114559, 0.111019355037725, -0.0569446628060191, -0.0272339425429253, -0.039595919990636, 0.014163991663512, 0.00370744537983269, 0.00319338727160606, -0.0702518096504061, 0.0825726724762284, -0.0302591072534671, -0.00555414878956771, -0.0175051398490236, -0.00405247587043005, -0.00289524874373822, -0.00390790286721708, 0.19950941508979, -0.378262163481966, 0.519567159982301, -0.115119607905565, 0.584484870331949, -0.112945107132979, 0.148388770431383, -0.200999967524955, 0.105961643569101, -0.0611719587217117, 0.119201100977506, -0.0230342679574021, 0.0369750785261226, 0.00986461372661273, 0.0101386109265573, -0.0152426492298347, -0.00585011446225261, -0.00573961131991814, 0.0656018645113008, 0.0175019791423124, 0.0179881100148822, -0.0888609872432724, -0.0341047634675219, 0.0468450635759486, -0.159689304474722, -0.061288605116368, 0.31117429138268, 0.0947022088715308},
    {0.015605997316105, 0.0283483789400612, -0.00366448453027248, 0.0118223838397412, 0.0220838867907565, -0.0613915956629153, 0.29971631803346, -0.0767262716388461, -0.102213293977142, 0.124878367918978, -0.0445130476343746, -0.0191184272948392, -0.0758343923526162, 0.0532647951908287, 0.00388900865659832, 0.0040826524231042, -0.0508705214556543, 0.0541344006325355, -0.0155002588937239, -0.00389905316318783, -0.0377420092156796, 0.0230901301043675, 0.0013542240808211, 0.000832624912029147, -0.0094044431423382, -0.00342928295650852, -0.00275466521408045, -0.00385372990326818, 0.305221577224944, -0.265912605385222, -0.0373890997493766, -0.462535110161498, 0.776025214555274, 0.317506232833688, 0.13231264484422, -0.0925956420784959, -0.00762521338174646, -0.161063201374778, 0.158264250550674, 0.0647529036970647, 0.0329692767107082, 0.0137520514519181, 0.0155106605694832, -0.0401332560488801, -0.0235049735356561, 0.0161349385942959, 0.0244606642480474, 0.0102029630870847, 0.0115077156161066, -0.0171181742635492, -0.0100256563422968, -0.00140967467297037, -0.0980917344976772, -0.0574497025764963, 0.0963870996787588, 0.167658158602434},
    {0.00987675823579522, 0.0290142634016863, -0.00428139506100082, -0.0295941247801588, 0.0263560489909357, -0.0712908034861193, 0.124137438646549, 0.383808683073226, -0.139268112846256, 0.263309019711057, -0.240581629561091, 0.0956363217673718, -0.0425594697223883, 0.00291910365758021, 0.00153171149747913, 0.00200693308424883, -0.0448012227061956, 0.0784639261814977, -0.0636937172730278, 0.019504277059619, -0.0136909752155197, 0.00086986892494546, 0.000405518905338967, 0.000409298247694226, -0.0034114770425107, -0.00168582891470649, -0.00149776380988472, -0.00216277464680895, 0.223346204549639, -0.0181400485021903, -0.0150679586141474, -0.500033591404991, 0.0642906130875656, 0.723243311516806, 0.0665553353467771, -0.00480255754658172, -0.00307299187277474, -0.132383333906543, 0.013111565844646, 0.147499795774797, 0.016584063225441, 0.00930748316716329, 0.0113499419004008, -0.0329868908038961, -0.0254105603546717, 0.0367535664289513, 0.00506798665030017, 0.00284430900906819, 0.00346847170388705, -0.000365700171241753, -0.000281707249352494, -0.000233998998075136, -0.02798041961885, -0.0215539605020873, 0.00277124849002808, 0.0763899465305751},
    {0.00422403562884654, 0.0171539738450264, 0.0293017672320873, 0.192230415140459, -0.182241449919052, 0.488039973885573, -0.759220407776989, 0.964112153400919, -0.0977269276576107, 0.232125215917279, -0.298701660335599, 0.240234638216595, -0.0095955325792423, -0.00374749699368054, -0.00309987463794578, -0.00445871865745555, -0.0235420783828845, 0.054296528623331, -0.0668187951761429, 0.0489939685728514, -0.00231153056297321, -0.000876578948905533, -0.000693434004591424, -0.000909320668327572, -0.000575980404939083, -0.000344874274678625, -0.000329816258202458, -0.000487622959599403, 0.0588574919775217, 0.0263542534751362, 0.0308349249253133, -0.160282676136514, -0.101513754490212, 0.291736816703685, 0.0137673862153512, 0.00589537889744807, 0.00628854087800174, -0.0358548234884831, -0.0207029333243833, 0.0594974335725768, 0.00343051691130107, 0.00231943115192508, 0.00299100276046851, -0.00893419973878676, -0.00814519801425953, 0.014825395965297, 0.000336832821567337, 0.000227738431127059, 0.00029367816138906, 0.000144236318875383, 0.000131498445572802, 0.000153855418475762, -0.00226534884361104, -0.00206529017058826, -0.00130803505644933, 0.00795527596810641},
    {0.0106036055567762, 0.0296266593660007, 0.259211232824319, 0.0195251039488212, 0.326169225832269, -0.374897896525802, 0.278859093198177, -0.113874016814714, -0.00517932982837491, -0.00333411005738363, -0.00365885144162154, -0.00745436569613287, -0.180531596982921, 0.469230738890095, -0.692276345467664, 0.744232604758411, -0.00174302461810558, -0.00103224751682526, -0.000995395188472067, -0.0015202593654192, -0.0607551610563967, 0.145274827975367, -0.188334660306985, 0.151780397361144, -0.00397712490619118, -0.00190612042688972, -0.00167493200389363, -0.00241016220156647, 0.0345842460403176, -0.106971053214144, 0.25456426498465, 0.0264935041720337, -0.132179922471374, 0.0468967101986917, 0.0107073556307269, -0.0291016110858974, 0.0519163834616734, 0.00720759150771039, -0.0269570575484031, 0.00956421589618275, 0.00070091972464463, 0.000381836111040756, 0.000461712209768622, 0.000471819861893832, 0.000353697875660655, 0.000626088065400543, 0.0244313765371234, 0.0133093441036108, 0.0160935189181038, -0.0664022418603689, -0.0497781754902047, 0.118459567099648, -0.00489411984295707, -0.00366885739980127, 0.018304459973106, -0.0045334627648615},
    {0.00958841840105439, 0.0287142087112329, 0.00525730855264615, 0.0197079603842469, 0.692635678336204, -0.462466964864018, 0.306530718648313, -0.118670820076184, -0.0252758904406238, -0.00632110314145237, -0.0048856688291831, -0.00776837170631502, -0.154654198992137, 0.341911353913746, -0.398704725867906, 0.277425364104441, -0.00799167084001736, -0.00185631097078705, -0.00128014616296248, -0.00158429842618399, -0.0488981962979725, 0.100408707642264, -0.104468874747729, 0.0565787251628151, -0.00320095002602286, -0.00160037367631397, -0.0014279042234856, -0.0020646895665564, 0.148706520787247, -0.388563452590706, 0.71220317547873, 0.0439538129017, -0.180523018821064, 0.0533988371215844, 0.0436704701359282, -0.101811651647421, 0.145248246697175, 0.0115168069922354, -0.0368162525456527, 0.0108902736390702, 0.00285873514978316, 0.00162273463193142, 0.0019852859084928, 0.000753907637346327, 0.000586799320677601, 0.000712893814650173, 0.0174915853413357, 0.0099289370345902, 0.0121472595661698, -0.0407792081924056, -0.0317402430744738, 0.0581771181962361, -0.0177359836148673, -0.0138046925395088, 0.0566973513015151, -0.00641353363141375},
    {0.00787931986789134, 0.0262660176362191, -0.0132573892568216, -0.0116539403654932, 0.130691148487769, 0.574537076169525, -0.227302951843894, 0.0757238026633065, -0.0524224823609094, 0.0192586102361636, 0.00525321344488663, 0.00495699486804396, -0.112078045039238, 0.179511848166742, -0.123767382336513, 0.0182909049745178, -0.0150561962143102, 0.00521607980877888, 0.0013003038648278, 0.00101094018990625, -0.0321897964657501, 0.0486197142564783, -0.0306355733076927, 0.00373028648217909, -0.0021071928544527, -0.00112365340857558, -0.00102691121281128, -0.00149628249178237, 0.243128686850451, -0.455369321869515, 0.474805099586672, -0.10444762263269, 0.295845278163648, -0.0430860995861486, 0.0658499558828106, -0.112715482697029, 0.0968327727428132, -0.0258534416707461, 0.0603353220350685, -0.00878707177583335, 0.00431063789576645, 0.0026049749215644, 0.00324585602164089, -0.00169240546797392, -0.00139441358919907, -0.000575215033661453, 0.00921604331712928, 0.00556937564640964, 0.00693956449600766, -0.0157751171905968, -0.0129974986479508, 0.0135522494457507, -0.0213656288798231, -0.0176036557524692, 0.0498619145320434, 0.0114367792968686},
    {0.00569928745007792, 0.0214698480958179, 0.00832487477036396, 0.0118223838397412, -0.0614326220634956, 0.26203544501963, 0.518569562880659, -0.0924570392473859, -0.0709478783134302, 0.0769033452699535, -0.0226501029881942, -0.00605237789102625, -0.0634299145349295, 0.0567537957659614, -0.00844642404705155, -0.00359533878904277, -0.0182709161061166, 0.0190137982809251, -0.00526293094004177, -0.0012343349584609, -0.0163348457295087, 0.0140319672776631, -0.00196259356847569, -0.000733241121874798, -0.00106930375395296, -0.000612888851787838, -0.000575990819905201, -0.000846812331002344, 0.226500485285503, -0.234109161969772, 0.0505207745986585, -0.283611242802991, 0.425056183742284, 0.121121715799415, 0.0560006137916712, -0.0543971191883837, 0.0103033153809034, -0.0658993199929182, 0.0866868718955917, 0.0247018231068585, 0.003665885037587, 0.00237595963988631, 0.00302386350863133, -0.00431386934521683, -0.00378631280488674, 0.00161701877171255, 0.00327743098393697, 0.00212419201925463, 0.00270344101158938, -0.00318358660367827, -0.00279425586598695, 0.000603000624105149, -0.0127054746021769, -0.0111516824754388, 0.0167133416449516, 0.0202473562137981},
    {0.00339108928439719, 0.0143344773818663, -0.0107414954878784, -0.0295941247801588, 0.0716512169321643, -0.219640225699807, 0.508137289414865, 0.462498877065499, -0.0655314248951063, 0.115979237022268, -0.0956568846043594, 0.0302758773259642, -0.0241318165707196, 0.00526310839581205, 0.00187632953336964, 0.00172722943069838, -0.0152353726346629, 0.0263431259843043, -0.0209745648569962, 0.00617452750875366, -0.00561039559867247, 0.00119544438383659, 0.000411420418441701, 0.00035225488328396, -0.000367264997427223, -0.000225042499060218, -0.000217247209950887, -0.000322168490867469, 0.118546565081049, -0.0295236807499632, -0.0146962544553076, -0.239579049519496, 0.0833104971211988, 0.275901578528232, 0.0269262600713353, -0.00647362037001933, -0.00299718572753368, -0.0525321970635782, 0.0169905218832959, 0.0562679610565784, 0.00176263378613828, 0.00121865954262174, 0.00158263957699833, -0.00343883722280938, -0.00319846707772592, 0.00368338599466453, 0.000649086377665708, 0.00044876894698663, 0.000582803868996042, -0.000156053562033, -0.000145145625740577, -7.22503764697957e-05, -0.00351495966953594, -0.0032692686667391, 0.00113684563986322, 0.00922528597147008},
    {0.00138044771170157, 0.00638882353038118, 0.0263784345157957, 0.192230415140459, -0.168273534214414, 0.466883678827404, -0.770250380155342, 1.16177879234705, -0.0357169332052308, 0.0827155276170149, -0.102424732574839, 0.076051800212317, -0.00422595518496755, -0.00135478401778062, -0.00101686811553749, -0.00130332461042054, -0.00766460049150895, 0.017588030695604, -0.0214801029408936, 0.0155101676309956, -0.000906860003956084, -0.000288071461031734, -0.000213253491124332, -0.000265802822928374, -5.93644300410853e-05, -3.83065599431041e-05, -3.7781091596148e-05, -5.64180322033744e-05, 0.0252733238596131, 0.00916815545757776, 0.00933834822750568, -0.0662292251284659, -0.0326033938848811, 0.111291244539186, 0.00537393653439462, 0.00192270868623429, 0.00190448281304134, -0.0138893267054964, -0.00664920623945942, 0.022696975664559, 0.000351786028768369, 0.000255673905629319, 0.000337408026585634, -0.00090921637290009, -0.000884184141232217, 0.00148577842015651, 4.16226103099882e-05, 3.02508185948611e-05, 3.99214342174242e-05, 1.4891905379795e-05, 1.44819065758112e-05, 1.47507410001302e-05, -0.000277806604307127, -0.000270158128669287, -0.000132993732949449, 0.000960724534063817},
    {0.0224419087355453, 0.310703073843137, 0.0208770933958504, 0.0195251039488212, -0.00350519620228991, -0.00213578148775574, -0.00215103292273904, -0.00360148326124408, 0.354215091736513, -0.39327074269067, 0.286443388274845, -0.115220749339439, 0.390484209646641, -0.426933671469902, 0.308757369600584, -0.123622648285321, -0.00378445250751072, -0.00231961445844581, -0.00235756395016146, -0.00402640927558312, -0.00417195365419305, -0.00251816728181528, -0.00254121852244889, -0.0043200151064958, -0.13347157028838, 0.33533302108994, -0.46792562881181, 0.436556032512803, 0.129016952515557, -0.064598516475327, 0.0224161618429518, -0.0655979383537513, 0.0157596925360571, 0.0229255170425751, 0.000760975979021777, 0.000531676205177384, 0.000783336703543885, 0.000539901917789586, 0.000550725217214117, 0.000801136210248271, 0.0243455865742151, -0.0708009310625633, 0.144239197195841, 0.0172728564941387, -0.0733376023953635, 0.025630416073592, 0.0268384022973471, -0.078050445204567, 0.159008269921432, 0.0187513670337175, -0.0796151059527957, 0.0276270291882513, -0.00566655757523526, 0.0240592369041034, -0.00578015385508157, -0.00586958024656276},
    {0.020831167572953, 0.0168212127674999, 0.019287579585398, 0.0197079603842469, -0.0163810014243473, -0.00390242657177244, -0.00279689872237231, -0.00375319132544435, 0.762426252400217, -0.486472830393037, 0.315036567668793, -0.120074282056387, 0.147540025395148, -0.240094368128956, 0.21694135572351, -0.101414897218028, -0.0177608566866899, -0.00425338581439391, -0.00307308040887658, -0.00419601682685231, -0.00343697142949136, -0.00209922099594019, -0.00211619316158239, -0.0035439613540266, -0.109957591036383, 0.236451671291948, -0.263802651514286, 0.164947740605517, 0.361953254691466, -0.228613371439197, 0.0928919182149795, -0.0896376076996035, 0.0254598197962891, 0.0261040901440187, 0.00316467178184315, 0.00223004992142867, 0.00324612435929955, 0.000874386037654717, 0.000889697864058014, 0.000912212004253607, 0.101246022171265, -0.251187955916101, 0.404658813134064, 0.0279738671992905, -0.10021362558217, 0.0291840175456564, 0.0195925056820641, -0.0486083437947558, 0.0783070774100661, 0.0138062550459614, -0.0494595496579394, 0.0200967791728532, -0.0208135617171248, 0.0745625360302081, -0.0211780387674468, -0.00830375195874425},
    {0.0178396083059577, -0.015558863517762, 0.0163965439250797, -0.0116539403654932, -0.0318063021955915, 0.0112436886986536, 0.00289018334287365, 0.00239490987846146, 0.161426388205, 0.609621358264413, -0.233929227326291, 0.076619351189609, 0.0109152642713773, -0.0722517992667381, 0.107859002109445, -0.0688053487876893, -0.0347130258988215, 0.0123225323775471, 0.0031877643198875, 0.00267747665318665, -0.00234721135471117, -0.00146045594323535, -0.00146979957328539, -0.0024044149700183, -0.0750933522468873, 0.118208186061103, -0.0796910370126105, 0.012203116915927, 0.243404271380666, -0.257835982891349, 0.143623705893241, 0.147100706594461, -0.0581440074053685, -0.0210626951480253, 0.00492003269613239, 0.00351354277547225, 0.00501895556935265, -0.00200454808179221, -0.00203185252724626, -0.000736039572724473, 0.157404550542626, -0.284383462616916, 0.272122663056645, -0.0641306693151905, 0.164456588164782, -0.0235478065455437, 0.0106433172778928, -0.0192293260314475, 0.0184002802424236, 0.0076007117916512, -0.0194912534396814, 0.0108573133203486, -0.0256055314109384, 0.0656627847324157, -0.025954310690459, 0.0148074655791843},
    {0.01354270014303, 0.0089487204272893, 0.0123418011338163, 0.0118223838397412, -0.0399845505836983, 0.0422013001459018, -0.0119330872043856, -0.00292413044299179, -0.0670830605102293, 0.297174981210493, 0.535772970512815, -0.0935504836114048, -0.00189681223432859, -0.00504670264847054, 0.0321772770214551, -0.0361703730372399, -0.0439702312640463, 0.0465429183672363, -0.0132189650130183, -0.00326913804247689, -0.0012432836542272, -0.000790403917701007, -0.00079390025733049, -0.00126398002385308, -0.0397758545272403, 0.0354876359854278, -0.0055905219327252, -0.00212061026536656, 0.0276817096582159, -0.127430920717604, 0.125763840462251, 0.212173344440878, -0.151186193020126, 0.0592105064091017, 0.00433545094422108, 0.00314406469764661, 0.00439483944231597, -0.00523488897578139, -0.00528322800716227, 0.00206912152181227, 0.13870227078189, -0.141162538552777, 0.0309477746936239, -0.167477615955912, 0.237206911741454, 0.0661965404040856, 0.00392188671994365, -0.0039914522104273, 0.000875066182395912, 0.00284414833496884, -0.00402830933090283, 0.00397561007306036, -0.0156004611164018, 0.0220957122063312, -0.0157445159727609, 0.0262147255291869},
    {0.00844321113047126, -0.0109738205691141, 0.00763236993753088, -0.0295941247801588, -0.0344990911194155, 0.0601371945948342, -0.0484704728084982, 0.0146274102792546, 0.0736287355315533, -0.227812559637078, 0.539364215954558, 0.467968625984661, 0.000857520491375101, 0.000986133972089222, 0.00289671851242862, -0.0128544681648157, -0.0382116443014457, 0.0667196416010178, -0.0539112807906323, 0.0163532456363012, -0.000445033691819665, -0.000288809823713043, -0.000289536829614474, -0.000449201642483861, -0.0142377769749928, 0.00321378511702103, 0.00109682745774996, 0.000958696239860543, -0.00736449804673689, -0.0158793451087477, 0.0621953543792269, 0.0427239595603106, -0.122833242483405, 0.134874841195124, 0.00215684627323642, 0.00158719434401822, 0.00217342755715777, -0.00427040450999766, -0.00429242918242198, 0.0047132249594298, 0.0690030817265086, -0.0176618007484954, -0.00823340859708799, -0.136621267387052, 0.0477648053829501, 0.150788237023004, 0.000803647599287444, -0.000205698983516078, -9.58907759977985e-05, 0.000591393526743637, -0.000206759878969835, 0.000809825837014434, -0.0044165565692143, 0.00154409316372162, -0.00443933502297747, 0.011944193459961},
    {0.00356846422868539, 0.0261188035535262, 0.00320507108782466, 0.192230415140459, -0.0178382131382485, 0.0410650708630083, -0.0503948504088267, 0.0367434731025106, -0.166915283235763, 0.464278848231276, -0.769187724303446, 1.17551858417977, -0.000624602315104132, -0.000497526431336808, -0.000671670106946097, -0.00213554205983528, -0.0198704873172245, 0.0457735732543992, -0.0562285998280999, 0.0410787029080855, -7.43559974855368e-05, -4.90514755035476e-05, -4.90999381122237e-05, -7.46268914879368e-05, -0.00237884036290435, -0.000748683496630209, -0.000555120501094651, -0.000698296888437294, 0.00448653414127237, 0.0045046775966389, 0.0126956210248065, -0.0157301889762824, -0.0329713743754392, 0.0544048679014511, 0.000442330508822919, 0.000329297655088212, 0.000443650701341738, -0.00114989679790933, -0.00115219045506149, 0.00190118764207914, 0.0141512951707223, 0.00502614278802671, 0.00501588411531291, -0.0367881678484823, -0.0175861372125052, 0.0608239019496677, 5.29545980091362e-05, 1.88079866661819e-05, 1.87695983854819e-05, 3.94226140923784e-05, 1.88455022700797e-05, 5.31126478447302e-05, -0.000355500518009366, -0.00016994270860627, -0.000356209621910858, 0.00124387251864899},
    {0.0206121464512043, 0.00186778322923233, 0.0160091858217584, 0.0195251039488212, -0.00316207558373215, -0.00241577235429526, -0.00353354291864167, -0.0181912548602126, 0.132011178619608, -0.227738706732716, 0.212018681376458, -0.101123604922342, 0.735069464402224, -0.406133611428702, 0.258494822208368, -0.0978768097135354, -0.00345830596482613, -0.00210523271754576, -0.00211720616941673, -0.00353378209371142, -0.0192566655330663, -0.00375432783799079, -0.00258131419735663, -0.00342032226620963, -0.107046130267054, 0.22526615807223, -0.243344599888121, 0.142792530306415, 0.377375106660751, -0.0785454775264234, 0.0222526783512817, -0.245248734459552, 0.0227212856352967, 0.101630074704723, 0.00348848218525233, 0.000784350551182977, 0.000777623744280809, 0.00244903953872561, 0.000793999307933779, 0.00355148076900451, 0.019392169313819, -0.047062388494394, 0.0733078286670002, 0.0136139979708023, -0.047641330573301, 0.0197423729662324, 0.107980185164328, -0.262054509821076, 0.408195327988218, 0.0242782715384179, -0.0849602859197397, 0.0240700543779733, -0.022559116146823, 0.0789442096360895, -0.00731385603414883, -0.0228366290826983},
    {0.0190287287217083, -0.0295118525675891, 0.0205403356838656, 0.0197079603842469, -0.0201247738104083, -0.00584323645334017, -0.00577935460978638, -0.0189575391547738, 0.228014045330251, -0.2717151045485, 0.231653750460453, -0.105383312724627, 0.222871671150233, -0.27710206784794, 0.238857806241638, -0.109348835528473, -0.0161511787365527, -0.00384447375345909, -0.00275184717215812, -0.00368263832928291, -0.0157869230855841, -0.00392069343601931, -0.00283742515434127, -0.00382121421853435, -0.0877581335269887, 0.157153073341715, -0.131942507994805, 0.0432944250827255, 1.02114857456486, -0.347362233990544, 0.122072685786528, -0.332927664119839, 0.0461721157239612, 0.115720863634553, 0.0144481437637405, 0.00412636436673883, 0.00426585139537876, 0.00395489410044252, 0.00161349267462729, 0.00404388585725999, 0.0803159756186617, -0.165396976974351, 0.198365455024694, 0.0219849126185115, -0.0646735932737143, 0.0224796100611515, 0.0785046187846291, -0.161666798236396, 0.193891742045088, 0.0224207806119899, -0.065955797566285, 0.0231786894608934, -0.0826227979472468, 0.243053648758979, -0.0337079263968618, -0.0323072000911189},
    {0.0161577642134767, 0.00567780449458311, 0.0267098865309273, -0.0116539403654932, -0.0579732173172456, 0.0239119304978302, 0.00786274267358259, 0.0120967981262488, -0.0380248596464323, 0.301987998765445, -0.169129303002115, 0.0672450495550815, -0.0129869928834948, -0.0973703756382163, 0.165842790839071, -0.110067289214354, -0.0313276643913107, 0.0110671468330949, 0.00284192495165231, 0.00234989003992674, -0.0106996359300076, -0.00356839426992844, -0.00278670080802812, -0.00384632070848072, -0.0594783463215326, 0.0767569359401082, -0.0351937372957314, -0.00252281677407681, 0.609030388285103, -0.507138483772254, 0.268073449809222, 0.537193055045898, -0.138827491931281, -0.0933720830549342, 0.0223195251484257, 0.00852158369598089, 0.00936787367759252, -0.0090265987022376, -0.00485135103202042, -0.00326290380377027, 0.124072300702037, -0.183301116519788, 0.118308533258758, -0.0501780777616423, 0.104353614600787, -0.0181381986942382, 0.0423755958927612, -0.062604578107956, 0.0404070414401219, 0.0161789816165828, -0.0336468690624238, 0.0177857381237386, -0.101195988938319, 0.210453801737447, -0.0543878465817405, 0.0576110360335694},
    {0.0121457494920626, 0.00068957310734755, 0.0302516266319403, 0.0118223838397412, -0.105865563934372, 0.12409022809239, -0.041339453116336, -0.0147699151361855, -0.00443360677459487, 0.0139910499734393, 0.368517130449206, -0.0821046747156991, -0.000633213118742084, -0.000823109650176664, 0.0650785581277397, -0.084049925337174, -0.0393358416198601, 0.0414974536399688, -0.0117258358154837, -0.00286916224498876, -0.00561799280287625, -0.00244134318822826, -0.00207073274120021, -0.00293713936881869, -0.0312299337797293, 0.021763151622634, -0.000233473059959272, -0.000123006202579615, 0.00658282380308265, -0.303643238348031, 0.324555812036961, 0.7371382729091, -0.459669715353957, 0.262483423099521, 0.0195246551264626, 0.00966161533660755, 0.0113416597229397, -0.0234549811860991, -0.0160632387501087, 0.00917252921469214, 0.108535861128071, -0.0861276695976375, 0.00127876086944764, -0.130384202142739, 0.143194411238384, 0.0509892927988106, 0.0155012238600132, -0.0123008586572031, 0.000182634184634628, 0.00767065339756554, -0.00842429281329378, 0.00900449227761782, -0.0613460913255659, 0.0673733265593304, -0.0420131188138639, 0.101992977055791},
    {0.00750148626092435, -0.00713114387718262, 0.0261981528295216, -0.0295941247801588, -0.123694165861968, 0.228667944176925, -0.201694509957147, 0.0738837109693798, 0.0450881185414648, -0.127068618871477, 0.241702073048266, 0.4107131286806, 0.00265240926107733, 0.00228882920954533, 0.00493590353226528, -0.0404499066305402, -0.0339007738939036, 0.0590785907491445, -0.047598198198489, 0.0143524422502372, -0.00199428872932863, -0.00106415577163226, -0.00097202358115682, -0.00141352907516568, -0.0110860777398482, 0.00127523875639669, 0.000540144332047333, 0.000515249575909739, -0.0207484211610251, -0.0296106304241309, 0.207559166263658, 0.0967054607656747, -0.448595161601129, 0.597907570023055, 0.00964665779284909, 0.00583119804449696, 0.00725319143528827, -0.0190441299503528, -0.0156762365286429, 0.0208939848046813, 0.0536249323623457, -0.00698785830119744, -0.00403053003957313, -0.105864663432388, 0.0187857312892304, 0.116147845812617, 0.00315460639795213, -0.000411076369401915, -0.000237104930297313, 0.00190689200901614, -0.00033837882932336, 0.0023719058557559, -0.0172861296713002, 0.00306743134590157, -0.0142291329716047, 0.046470974802135},
    {0.00314688918138774, 0.02849036321081, 0.014002682081672, 0.192230415140459, -0.0792712548026677, 0.186695351837455, -0.237148368754727, 0.185592944676423, -0.178725093630068, 0.48438729810939, -0.768528380851087, 1.03169505116885, -0.00337811380894423, -0.00244266543518648, -0.00305101620671766, -0.00832902875213204, -0.0175130233754289, 0.0403122421171728, -0.0494632043101814, 0.0360527643450898, -0.000331016674263683, -0.000203286338883911, -0.000196366252368518, -0.000291059369223311, -0.00184009292641853, -0.000658791463301681, -0.000509478771102183, -0.00065622290382367, 0.023643190562458, 0.0211799660893324, 0.0506566804146618, -0.0793857886114538, -0.136173587707136, 0.241179763965931, 0.00196766105572199, 0.00136316239720261, 0.0017702065735656, -0.00510934349238301, -0.00475860988386853, 0.00842806911326777, 0.0109380568162509, 0.00441760993533491, 0.00459285981587523, -0.0284023964650367, -0.0154212604048487, 0.046850903789623, 0.00020674209772939, 8.3498007034194e-05, 8.68104352412663e-05, 0.000143227438853991, 7.77662418170802e-05, 0.000185995560246247, -0.00138633542760969, -0.000752719569419343, -0.00129116969293916, 0.00483950370236119},
    {0.0172198053622791, -0.00652877742132654, -0.0100835618309505, 0.0195251039488212, 0.00276855422334095, 0.00354739278284653, 0.0184970121822559, -0.041398111534134, 0.00480559727387792, -0.069622949057169, 0.116257872529598, -0.0787003334009246, 0.0608951799214984, 0.649943657200637, -0.208138951111156, 0.066693670734456, -0.00286607158261663, -0.00172596335485698, -0.00170726307853506, -0.00275019694120753, -0.0363180547067535, 0.0161122007935752, 0.00305654953686361, 0.00233062201042265, -0.0690428357230215, 0.10126879776982, -0.0599894079180596, 0.00404544389461591, 0.262546773541793, 0.225455398280341, -0.0254307747166692, -0.30603626219929, -0.0925652849176254, 0.179996535328407, 0.00650857391428436, -0.00331084397955464, -0.000888682878661058, 0.0044941852089429, -0.00323471010148835, 0.0062900104675075, 0.0123731957337328, -0.0208093974068782, 0.0174417456922688, 0.00854372002011934, -0.0203308788978239, 0.011957693299125, 0.156789663691807, -0.26369084359981, 0.221016906233311, -0.0797572741627934, 0.189792675603336, -0.0214081135912332, -0.0322182004023545, 0.0766673450374754, 0.0231891952485369, -0.0314773329510054},
    {0.0157648753004524, 0.0085857473144989, -0.0118196797021562, 0.0197079603842469, 0.0153251463175155, 0.0068247368416488, 0.0170656692030986, -0.0431419561966856, -0.0231797827124803, -0.0726756131892214, 0.124780298683106, -0.0820154883984918, -0.0515609015277078, 0.218856107347192, -0.150272175335546, 0.0648054921715141, -0.0132859312546024, -0.00313228338759691, -0.00220943154814895, -0.00286604561350593, -0.0295531067577096, 0.00943259119854756, 0.00266080533945924, 0.00226463927967868, -0.0561822573222246, 0.068968965301209, -0.028334522503821, -0.00342534063541716, 0.621558100379749, 0.552501249632886, -0.110962179307842, -0.408107372713323, -0.106107915976052, 0.204952663667212, 0.0267888501632131, -0.00978290406589569, -0.00387759279960767, 0.00722618325024954, -0.00370795971687495, 0.00716210674532261, 0.0509272370439877, -0.071530373453446, 0.0412919121935718, 0.0137374148224589, -0.027111759607561, 0.0136156014808827, 0.113282090987397, -0.159111523503952, 0.0918493604928063, -0.0413689957486713, 0.0816446385611457, -0.0163971882951673, -0.117489224878566, 0.231873293669398, 0.0602870558259816, -0.0445312874461567},
    {0.0132117409898728, 0.00728244989005006, -0.00729939888431999, -0.0116539403654932, 0.0195387056426204, -0.0109690858690787, -0.00648860340814748, 0.027528864934527, -0.03118591248812, 0.0417262619381513, -0.0868398367617212, 0.0523340502310095, -0.0242391457778127, 0.0111025773900734, -0.0390618253060837, 0.0288702594494081, -0.0254704298617907, 0.00892944871598905, 0.00226644479881709, 0.00182882255572049, -0.0197968060956589, 0.00237595918769118, 0.00101947993108501, 0.00100887627534114, -0.0376349350795189, 0.0317985080969108, -0.00387808858029446, -0.00161027694513688, 0.191503462471457, 0.214883898684752, -0.0957047798901591, 0.627694611711972, 0.0891613579747631, -0.165370846131076, 0.040981872510121, -0.005608284316102, -0.00334442030343114, -0.0163822876805221, 0.00311575927800604, -0.00577891319567461, 0.0779090376449023, -0.0750581386916097, 0.0127221319331292, -0.0311437274442791, 0.0416995784872783, -0.010986066232021, 0.0605545379377077, -0.0583386862978686, 0.00988823433174066, -0.00828676301457338, 0.01109547742318, -0.00494169284465334, -0.142934721751187, 0.19138099827276, 0.0271848592901337, 0.0794093452371635},
    {0.00978040546374066, -0.0111121414793536, 0.00962857943557122, 0.0118223838397412, -0.0386200930353358, 0.0549301045311348, -0.0255353246076025, -0.0336121174078451, 0.0582572468288329, -0.126131012729572, 0.160899807538945, -0.0638986839804833, 0.0189347929379127, 0.00458359170060351, 0.0125779136442869, -0.0238627208753059, -0.0315495982714479, 0.0331063769654699, -0.00927948168620292, -0.0022329507085449, -0.0102542626550716, -0.00120308329737537, -0.00072539875030345, -0.000833886962407467, -0.0194939783441422, 0.00758070923223656, 0.00166566730191939, 0.00125789336011894, -0.135052204499447, -0.0818913657941362, 0.111811445560696, 0.732427557989167, -0.22097674739764, 0.464883125160772, 0.0354479766349183, 0.00472287345005579, 0.00390727055762914, -0.0422408716007057, -0.00772207115915273, 0.0162454222693438, 0.0673887642738485, -0.029759145059726, -0.00897191069721784, -0.0803024716569531, 0.0486572926877849, 0.0308835379551549, 0.0219027222761858, -0.00967232885843017, -0.00291605389125908, 0.00291818589781651, -0.00176820242786556, 0.00241423827269643, -0.0859822306134806, 0.052098801874755, -0.0157184470409668, 0.140584097846636},
    {0.00595231255869339, 0.00476412582200835, 0.0280757815135756, -0.0295941247801588, -0.142669865790111, 0.291843625603676, -0.302869456861528, 0.168138269227149, -0.0275920932079783, 0.0681032668396039, -0.0986835027874707, 0.319641098476688, -0.00369386434939359, -0.00184206276700021, -0.00257202845525831, -0.0363098612093178, -0.0268382683375484, 0.0466279404547273, -0.0373939339722442, 0.0111699141963798, -0.00359294678603298, -0.00126119637132604, -0.000974613380291096, -0.00126885446246864, -0.00683041084421036, -0.000410933730046588, -0.00022743123124971, -0.000245394151048635, 0.0253065278441708, 0.0181540378518067, 0.206163069251945, -0.089853097177041, -0.524251903161903, 1.05895121462285, 0.017326500301564, 0.006879071714976, 0.00720440458057845, -0.0340478467842399, -0.0183200746196732, 0.0370052357530366, 0.03293873319028, 0.00224139766285651, 0.00168118623991409, -0.0647270320843953, -0.00596920255192238, 0.0703492087783398, 0.00440964052015384, 0.000300064908350173, 0.000225067154909331, 0.00175074209144624, 0.000161455481944364, 0.00183354017309741, -0.0240519510403532, -0.00221809903691547, -0.0129415977638843, 0.0640542148802874},
    {0.00246776876963696, 0.030253888096142, 0.0249036082205118, 0.192230415140459, -0.14494000231401, 0.354346112869683, -0.476703569824698, 0.422356647889497, -0.184397876468037, 0.480186648330888, -0.713797405978018, 0.802925731904298, -0.00793163153750731, -0.00486754060925033, -0.00537840791475004, -0.0118519803113753, -0.0137211579663084, 0.0315457931627569, -0.0386364416503135, 0.0280583803965701, -0.000590197519305752, -0.000319772383935504, -0.000291122581602986, -0.000414169528781283, -0.00112200145901502, -0.000478814744853805, -0.000394510258857471, -0.000526921349425547, 0.0533385602171293, 0.0395429056190285, 0.0748264123822477, -0.167793715365594, -0.213032160197368, 0.427152317178935, 0.00350406908242337, 0.00214037926285249, 0.00261482209238981, -0.00908234190642554, -0.00744444616732359, 0.0149269125729188, 0.00666144891220946, 0.00320492075650958, 0.00354343567185679, -0.0172660855678771, -0.0111470244813413, 0.0283769706540014, 0.000286533442193482, 0.000137855440824331, 0.00015241621359402, 0.000175022302174747, 0.000112994800092435, 0.000213818265916821, -0.00191789619302718, -0.00123819818524007, -0.00157202570775299, 0.00667062852425393},
    {0.0124299526187294, 0.0025444920190016, 0.0100401662714875, 0.0195251039488212, -0.00508376488671095, -0.0317222402070974, 0.0822967853898158, -0.0686320373360359, -0.00085687201017403, -0.00161605386869115, 0.0393067632837015, -0.0523859677188016, -0.0180010510271378, 0.0671213089149897, 0.629292918952211, -0.0815183617627336, -0.00204964204175454, -0.00121893362021515, -0.00118292063155352, -0.00183063682142864, -0.043058602151674, 0.0506272851755831, -0.0189383076837504, -0.00284867343610872, -0.0328660874700562, 0.0241807101537085, -0.000964790796582522, -0.0004801458344206, 0.0101031421758621, 0.339146109455536, 0.151374443887892, -0.171539508681194, -0.274137109924834, 0.198631858112276, 0.00762045121549466, -0.0102064605800905, 0.00528980646672306, 0.00516240990083402, -0.00957976934285662, 0.00694122508762252, 0.00581659421565761, -0.0048748311138247, 0.00026948324422754, 0.00394040230937913, -0.00457550953039679, 0.00529814948649652, 0.122194222750899, -0.102409791178522, 0.00566126746200271, -0.163660980479209, 0.190039573917172, 0.0848222462719287, -0.0246343550379298, 0.0286048777260458, 0.0457134252619621, -0.0231217705022782},
    {0.0112741294025221, -0.00481040329364098, 0.0110939404354585, 0.0197079603842469, -0.0231802053744836, -0.0276298646655858, 0.0870824133797188, -0.0715230777133198, 0.00736457670424282, 0.00438882359425984, 0.0397000546457263, -0.0545926623435964, 0.0271584695282764, -0.0674411898835216, 0.19356039727999, -0.0652473476293611, -0.00942285779415148, -0.0021969284385838, -0.00152363049106575, -0.00190775015176708, -0.0347488262460901, 0.0337592671031061, -0.00742856718436123, -0.0022800799960787, -0.0265233404201762, 0.0153905329075907, 0.0011799793927646, 0.00072440359141155, -0.0622282391086317, 0.896985621982871, 0.299024122342194, -0.215261263528709, -0.360407082565477, 0.22617173344477, 0.0311498025027812, -0.0344250066126238, 0.0104494503526857, 0.00826141494223287, -0.0125944886537099, 0.00790361085687727, 0.023776251029356, -0.0156940372993719, -0.00165982695934224, 0.00630583373704123, -0.00574170912796156, 0.00603272639544607, 0.0876800683880991, -0.0578751571051009, -0.00612097092715924, -0.096899071311482, 0.0882304078161749, 0.0294129801131074, -0.0894091368997127, 0.081410528546718, 0.136303813341331, -0.0327105924159354},
    {0.00930958844834864, -0.0117698440494143, -0.00170955493305554, -0.0116539403654932, 0.00630079692715634, -0.00613847546289087, -0.0581932607657199, 0.0456388471838216, 0.0354164151936632, -0.0239832387341911, -0.0228191073912387, 0.0348355559312484, 0.0456362358439276, -0.0572326192875784, -0.00574410374506534, 0.00619710390792818, -0.0178289300066308, 0.00619513050775135, 0.00155135505195035, 0.00121733460618687, -0.0229736761944515, 0.0147838058786252, 0.000390512398712576, 0.000216558881969468, -0.0175355170299285, 0.0057971189370832, 0.00152574562586517, 0.00121726495336655, -0.182482550339248, 0.506413169593175, -0.0356502118925021, 0.273447788383992, 0.532275572502242, -0.18249194844047, 0.0471372205926179, -0.0344284557477551, -0.00124580290150348, -0.0185903243576046, 0.0186004631507438, -0.00637721311597518, 0.0359792454393943, -0.0135003025897686, -0.00486739559083205, -0.0141897599063138, 0.0072937305897393, -0.00486764626837001, 0.0463614773369988, -0.017395972731348, -0.00627193949230205, -0.033861862257262, 0.0174054601488664, -0.00122530056414615, -0.107966384569832, 0.0554961977510375, 0.108025267288504, 0.058330375675975},
    {0.00677269421349488, -0.00224468746929321, -0.011422245664625, 0.0118223838397412, 0.0552995939385817, -0.104936601954282, 0.0910163993406658, -0.0557239934719332, 0.00967279417619889, -0.015304353365906, 0.00875331817503955, -0.042533420782639, 0.00521205405512006, -0.0019823361179223, -0.00130720099680889, 0.0227440073355648, -0.0217460457985154, 0.0226775867458484, -0.00629687867371578, -0.00148633784230109, -0.0117175620738283, 0.00293737332109392, 0.00094036180502817, 0.000794793321731182, -0.00894386721374986, 0.000282495939903871, 0.000137145944197982, 0.000139022218178741, -0.027166966073064, 0.015879381119268, -0.142180891317948, 0.0660584269543609, 0.524279942561052, 0.513013202099966, 0.0402552930537984, -0.0114231579753208, -0.00496853616119737, -0.0475204821291951, 0.0183210544613896, 0.0179273362417273, 0.0307263570233536, -0.00109859912110422, -0.000724630221544398, -0.0362717841321436, 0.00176199036837885, 0.0136837094467269, 0.0165564810752096, -0.000591965248076004, -0.000390457174614098, -0.00469819704416935, 0.000228226378675325, -0.00204349462353737, -0.0643865379587115, 0.00312773309752809, 0.0248235963876544, 0.103266475966666},
    {0.00405256796561407, 0.0225257270787522, 0.0116134748343783, -0.0295941247801588, -0.0648165501864932, 0.151531129548398, -0.19835664154622, 0.278748753109168, -0.118755279768057, 0.25582760745598, -0.289712660115728, 0.212765404450057, -0.0263569760650687, -0.00453184613547789, -0.00392058013420659, -0.0109803617251882, -0.0182236775617253, 0.0315481269099228, -0.0251644118483801, 0.00743512434099122, -0.00404462887250178, -0.000558857812259055, -0.00034054118705875, -0.000383710664554921, -0.00308721416081532, -0.000816247856153973, -0.000574930295742508, -0.000703025187056366, 0.157600788983966, 0.0349049822683105, 0.0712526331790854, -0.43732301110713, -0.228543925328445, 1.16858608987677, 0.0194350005513992, 0.0030318431683629, 0.00248993575190885, -0.037985831747489, -0.00798650751792706, 0.0408364456798987, 0.0148344905822269, 0.0044282023657649, 0.00420371911717626, -0.0289941058569008, -0.0116648090026963, 0.0311699434866482, 0.0032924204631314, 0.00098281124000304, 0.000932988616355414, 0.000513614726283418, 0.000206635711155743, 0.00042181194694528, -0.0178616139374965, -0.0071860231210133, -0.00375539793211324, 0.0470512180454073},
    {0.00165731029056231, 0.0278235382849624, 0.0299947250976383, 0.192230415140459, -0.180516285730209, 0.461954122435896, -0.667777779660335, 0.700205786033841, -0.164115051478879, 0.408587395180543, -0.565496681950334, 0.534458237398516, -0.0117031127641394, -0.00580187045310396, -0.00555494811880206, -0.00982556301346887, -0.00920510912404753, 0.0211331468589321, -0.0258282524615988, 0.0186767367580078, -0.000656420170570347, -0.00030008703593952, -0.000253714313457484, -0.000343356021220619, -0.000501037229845128, -0.000254123794305314, -0.000224404254493875, -0.000312159597513673, 0.0752423816171071, 0.0438841037852108, 0.0649328546150946, -0.220382521134524, -0.198640193522426, 0.471376064563759, 0.00389172137830511, 0.00200434369960606, 0.00226908998258696, -0.010065656573536, -0.00694151636999013, 0.0164723191745254, 0.00297050180072966, 0.00169734565320802, 0.0020069559300103, -0.00768298859828426, -0.00587830951327847, 0.0125731132868091, 0.000211827722239627, 0.000121038426397995, 0.000143116864357806, 0.000109097111329767, 8.34709799704021e-05, 0.000123507341827728, -0.00141484139474379, -0.00108250526785383, -0.000975708303854878, 0.00489993043831916},
    {0.00700570982272352, -0.00396719586028167, -0.02833860959432, 0.0195251039488212, 0.0724066424646239, -0.168855440911288, 0.180838102102827, -0.0944993352356273, 0.00088167577923319, 0.000801959357219176, 0.00164117293051685, -0.0273920854159158, 0.0255030826818176, -0.0731920669423039, 0.139859224732048, 0.607097608174253, -0.00114511302775338, -0.00067304391220236, -0.00064163166281098, -0.00095722122472304, -0.0331231875874076, 0.0614263985245281, -0.0546792511962839, 0.0212151322982277, -0.00960126525101311, 0.000557467377202806, 0.000259692769779479, 0.000258330833524202, -0.00690326988915813, 0.0311157999146768, 0.421321766809783, -0.00986172950072131, -0.314981484277462, 0.143008184695501, 0.00579356513688986, -0.0121650083715794, 0.0147231629685932, 0.0038555339172856, -0.0110070831616925, 0.00499744607324112, 0.00167935394144966, -0.000110401968427987, -6.99259570601796e-05, 0.00111758579171922, -9.98933671546298e-05, 0.00144858658908363, 0.0485764761033606, -0.00319345341606735, -0.00202265674810748, -0.101998203955027, 0.00911692338279008, 0.123447196538076, -0.00962018371015533, 0.000859882570604181, 0.0274644613171975, -0.0087044874030528},
    {0.00630096536900632, 0.00163104362451023, -0.0195929661997851, 0.0197079603842469, 0.0976637162240657, -0.196492083346238, 0.196789453010076, -0.0984800038037923, -0.00176946150425569, -0.000595996850338294, -0.000775277112656344, -0.0285459434103635, -0.0089846359300533, 0.0206962864077096, -0.0268812006793829, 0.14374374463353, -0.00522532202805538, -0.00120559810201234, -0.000822987370205613, -0.000997542994527409, -0.0265321488636247, 0.0418649924033504, -0.0285354595072401, 0.0050231503441662, -0.00769075132780626, -0.000164932469374685, -8.65533292724353e-05, -9.10089387106683e-05, 0.0116355043739916, -0.0395842756636937, 1.11194332036726, 0.00578806775186312, -0.425867801143152, 0.162835958625975, 0.0235366042308366, -0.042020276876662, 0.0388570541739825, 0.00614426323170673, -0.014882024934966, 0.0056903311076251, 0.00682244665408591, 0.00016554423233392, 0.000117860635943795, 0.00178100917684918, 5.86296325621322e-05, 0.00164942997065806, 0.034641725288599, 0.000840569096452769, 0.000598450377074178, -0.0618464275405141, -0.00203594308727165, 0.0571907222901661, -0.034770217419069, -0.00114461233433283, 0.0842169716874297, -0.0123143225387019},
    {0.00513339611045791, 0.0126104679035144, 0.010730605592219, -0.0116539403654932, -0.0616020791184594, 0.199029865952139, -0.142163327346853, 0.0628400509032686, -0.0295896009607121, 0.0136392839755107, 0.00544335694053327, 0.0182151550335338, -0.052498414352161, 0.0795137525688747, -0.0444271201886021, -0.0316809934644757, -0.00976967983719927, 0.00336641557499294, 0.000832405683526393, 0.000636531784454442, -0.0173335456893194, 0.0196253949660864, -0.00679385676003039, -0.00110709786801733, -0.00502439475260476, -0.00075144576237101, -0.000465575059279249, -0.000531777248558352, 0.14289168101562, -0.295112794978876, 0.604407671559529, -0.0898140827333987, 0.679925843563978, -0.131387998461278, 0.0352681842662437, -0.0451290574006878, 0.0211211319918736, -0.013734493942751, 0.0237601277454771, -0.00459137662910276, 0.0102230255215696, 0.00172796720790312, 0.00144740647712978, -0.00398115426194409, -0.000909762444964012, -0.00133088111664943, 0.0181378799422368, 0.00306579120779264, 0.00256801519808451, -0.023209173992647, -0.00530369673915506, 0.0108622704650948, -0.0417084816104068, -0.00953110774137492, 0.0721540127553231, 0.0219592189204036},
    {0.00367511242526286, 0.0253972624653879, -0.00366448453027248, 0.0118223838397412, 0.0220838867907565, -0.0613915956629153, 0.29971631803346, -0.0767262716388461, -0.0936105122386032, 0.117950744221066, -0.0442674609131471, -0.0222402896394521, -0.06945179083084, 0.0503099322824041, 0.00386755227615358, 0.00474931180199947, -0.0117484678910899, 0.0121800061564539, -0.00335222048188698, -0.000777190818574965, -0.00871645838744077, 0.00519517947069479, 0.000292876250127097, 0.000165965533133881, -0.0025265994948824, -0.000767316927102061, -0.000562698709719922, -0.000703504718975456, 0.288289419424248, -0.264445516332456, -0.043494393914345, -0.459983218363609, 0.902742954379208, 0.369352063935649, 0.0297697731931798, -0.0200255369950426, -0.0015199192167781, -0.0348329254516792, 0.0315465107267404, 0.0129070725951027, 0.00862922652404613, 0.00295772910200325, 0.00292019780295708, -0.0100968590612786, -0.00465935234926476, 0.00374131346123147, 0.00640222151602382, 0.00219440720934235, 0.00216656187585758, -0.00430664765188615, -0.00198736941186251, -0.000326870461831576, -0.024678247314225, -0.0113881603082132, 0.0223499055425398, 0.0388759908814474},
    {0.00216464827329585, 0.0300055156673333, -0.00428139506100082, -0.0295941247801588, 0.0263560489909357, -0.0712908034861193, 0.124137438646549, 0.383808683073226, -0.145766077772663, 0.28016483684933, -0.262749987106196, 0.111252848540127, -0.0445452074184814, 0.00310597107865766, 0.00167285081968764, 0.00233464669412119, -0.00970937438415855, 0.0167516558155947, -0.0132940824930269, 0.0038877502868635, -0.0029671244671929, 0.000185712664972379, 8.46394591313936e-05, 8.15846378218604e-05, -0.000860066651710846, -0.000393080450660097, -0.000332623551692574, -0.00045121606299494, 0.237643788379266, -0.0198115605034005, -0.0175284168873228, -0.54610911038394, 0.074788675559698, 0.841342254771773, 0.0142092312311999, -0.0010023845197807, -0.000612533599599251, -0.0276309035962134, 0.00261350337251345, 0.0294008525198299, 0.00411876416494368, 0.00212165260149841, 0.00240718813093271, -0.00800924228238475, -0.00553175564855268, 0.00852228919405462, 0.00125866873032942, 0.000648363848762601, 0.000735621731929627, -8.87922808971535e-05, -6.13263008014883e-05, -5.4258874076063e-05, -0.00679366725474095, -0.00469219258024664, 0.000642587464437379, 0.0177130351991939},
    {0.000873918129073691, 0.0191690324088708, 0.0293017672320873, 0.192230415140459, -0.182241449919052, 0.488039973885573, -0.759220407776989, 0.964112153400919, -0.109706187629544, 0.262185091653748, -0.340572586312385, 0.279462732628028, -0.010771742474417, -0.0042327923697459, -0.00353440393167959, -0.00518678617405983, -0.00484906798185377, 0.0111175665009612, -0.0135601345036643, 0.00976587416141797, -0.000476116367454519, -0.000179485226862473, -0.000140724751873898, -0.000181253151723279, -0.00013800965025541, -8.05138366819971e-05, -7.56002252730471e-05, -0.00010911124928073, 0.0664794510482461, 0.0300484980775636, 0.035869982963235, -0.182750526031733, -0.118090011664295, 0.339374740501439, 0.00281896164772135, 0.00119640185951736, 0.00125348284008009, -0.00727633935672273, -0.00412667615030005, 0.0118595097748223, 0.000817119380215923, 0.000536684303254997, 0.000673396711126373, -0.0021091588493988, -0.00185115251774694, 0.00343766126960371, 8.02306572191408e-05, 5.26955245610458e-05, 6.61189319591596e-05, 3.40508738617615e-05, 2.98855446086987e-05, 3.56754594920128e-05, -0.000534796702578028, -0.000469376814715887, -0.000303302620946022, 0.00184464173156551},
    {0.00234316248688227, 0.0302387678330058, 0.259211232824319, 0.0195251039488212, 0.326169225832269, -0.374897896525802, 0.278859093198177, -0.113874016814714, -0.0053561132654389, -0.00351055389355512, -0.00396473437419388, -0.00867159466642652, -0.186693590381845, 0.494062814074892, -0.750151206494618, 0.865758905462842, -0.000380508993873849, -0.000221704125817991, -0.000208599314241359, -0.00030303039512555, -0.0132630858830493, 0.0312018466643449, -0.0394682247241072, 0.0302540966567131, -0.00100999427280186, -0.000443618433201126, -0.000369581508087748, -0.000496810721561436, 0.0364144726787388, -0.115913919569766, 0.296132254907867, 0.028708382496437, -0.153763681235287, 0.054554509211011, 0.00229970513974539, -0.00609865929245096, 0.0103483935384566, 0.0015104540018398, -0.00537329878448961, 0.00190641688386353, 0.000175124329341789, 8.67088960806867e-05, 9.69026328638267e-05, 0.000115022243287896, 7.63959394307484e-05, 0.000145175123742181, 0.0061041632594019, 0.00302233995531921, 0.0033776545696933, -0.0161878196213956, -0.0107516916029522, 0.0274679925437245, -0.001193109253598, -0.000792444136600179, 0.00424437453528148, -0.00105120358339417},
    {0.00209472514255059, 0.0298426277029344, 0.00525730855264615, 0.0197079603842469, 0.692635678336204, -0.462466964864018, 0.306530718648313, -0.118670820076184, -0.0265751876169888, -0.00675219580951274, -0.0053514536389617, -0.00903687495372635, -0.16260413707782, 0.365229353082629, -0.43671602204642, 0.32272636006408, -0.00172706393075782, -0.000395389812588404, -0.000266770877594545, -0.000315795178772657, -0.0105672909703012, 0.021386815421382, -0.0217703682626926, 0.011277729203176, -0.000804707400159116, -0.000373373839063326, -0.000317855761650245, -0.000432706224703788, 0.158848151032727, -0.425607910612668, 0.828499366632536, 0.0481442357685086, -0.210000758077652, 0.0621183733200974, 0.00930170605810062, -0.0212166270111526, 0.0289520555433697, 0.00239999837307292, -0.00733851328906402, 0.00217073744055717, 0.000708332127892661, 0.000370402667393313, 0.000422711162026884, 0.000182761736816986, 0.000128116731068682, 0.00016530333906084, 0.00433403278578929, 0.00226636240431688, 0.00258641951000199, -0.00988566575807079, -0.00692989245680962, 0.0134899078897138, -0.00429954414710382, -0.00301399817492443, 0.0131467846872082, -0.00148714787905987},
    {0.00169002122151257, 0.0280378223106107, -0.0132573892568216, -0.0116539403654932, 0.130691148487769, 0.574537076169525, -0.227302951843894, 0.0757238026633065, -0.0564738728270902, 0.0210073440323392, 0.00584313287859409, 0.00576642628111648, -0.120739823396356, 0.195812008554932, -0.137666072131895, 0.0212776405782488, -0.0032014158128403, 0.00109628782576105, 0.000268548259483359, 0.000201508776833746, -0.00684455236573051, 0.0102186321500795, -0.00632708254785052, 0.000743550878448081, -0.000521217969199468, -0.000262683898321719, -0.000231343120081794, -0.000321301008031794, 0.265205427918547, -0.506505871964673, 0.552336380720342, -0.116176763860625, 0.3441540757173, -0.0501216611310329, 0.0138399924095756, -0.0232788254452492, 0.0193014916091435, -0.00533944176444615, 0.0120265244814158, -0.001751510230949, 0.00105392615206942, 0.000598414007520597, 0.000705738744092677, -0.000406602630006461, -0.000309158240324233, -0.000133378862024455, 0.00225326954047066, 0.00127939519584879, 0.00150885298032624, -0.00378999256368269, -0.00288170155665656, 0.00314244848129014, -0.00513312031819317, -0.00390294190896012, 0.0115618073754219, 0.00265192061850403},
    {0.00119584539165626, 0.0235888506057285, 0.00832487477036396, 0.0118223838397412, -0.0614326220634956, 0.26203544501963, 0.518569562880659, -0.0924570392473859, -0.078445661197035, 0.0857621692652029, -0.0255955981627665, -0.00704067522019321, -0.0701331978298506, 0.0632915073048822, -0.00954482528990458, -0.0041824243588202, -0.00381032346110911, 0.00393303420879096, -0.00107536568428437, -0.000246037629293941, -0.00340656404721303, 0.00290253459641734, -0.000401013389645499, -0.000146155552097404, -0.00025941249328829, -0.000143263535890422, -0.000131248572870803, -0.000186631606087798, 0.252592041211182, -0.26455350066724, 0.0587703498073265, -0.320492997714484, 0.494463927062682, 0.140899771694796, 0.0115838154219245, -0.0111148703955158, 0.00205374017223553, -0.0134650954278903, 0.017279128575194, 0.00492376721147816, 0.000882116525256514, 0.000548608666298252, 0.000672173233147159, -0.00102537745625865, -0.000852864616960224, 0.000374948690527344, 0.000788643397617785, 0.00049047556662225, 0.000600946663166412, -0.000756716922147342, -0.000629404405192879, 0.000139821688128011, -0.00302000505476203, -0.00251190957878664, 0.00387543154955192, 0.00469488656026307},
    {0.000696217557866079, 0.0161601125382385, -0.0107414954878784, -0.0295941247801588, 0.0716512169321643, -0.219640225699807, 0.508137289414865, 0.462498877065499, -0.0741443373095674, 0.131860036949593, -0.10956139003762, 0.0352196480617276, -0.0273035043961887, 0.00598377507353532, 0.00214906927708211, 0.00200927002101722, -0.00311691852162252, 0.00536405680022606, -0.00424076416664773, 0.00123075677299028, -0.00114779903153431, 0.000243419538752912, 8.31834643459581e-05, 7.0214292965113e-05, -8.74057861345451e-05, -5.2484601275396e-05, -4.99388244904406e-05, -7.26574152464174e-05, 0.134778903993248, -0.033815187640404, -0.0170960168773588, -0.274403811056705, 0.0969143307348483, 0.320953754397271, 0.00548279610120528, -0.00130887565395685, -0.000597423305482469, -0.0106212767903117, 0.00338668826964617, 0.0112157851875391, 0.000417519173892461, 0.000282211597182808, 0.00035866043610383, -0.000808818462208122, -0.000730216581573628, 0.000854090737576335, 0.000153750603397625, 0.000103923858030729, 0.00013207599055098, -3.67039769223199e-05, -3.3137043490894e-05, -1.67531660865139e-05, -0.00082672254905794, -0.000746380729286372, 0.000263607814241562, 0.00213912723541274},
    {0.000278404956314803, 0.00734348804657588, 0.0263784345157957, 0.192230415140459, -0.168273534214414, 0.466883678827404, -0.770250380155342, 1.16177879234705, -0.0411121542477164, 0.0953904978590446, -0.11845780646441, 0.0884703557588259, -0.00486430680960254, -0.00156238526998145, -0.00117604374843892, -0.00151614546442337, -0.00154360799326977, 0.00353550741513475, -0.00430576776528852, 0.00309161208448686, -0.000182636570873262, -5.79074942609822e-05, -4.27474677586748e-05, -5.29819689255306e-05, -1.3907916469335e-05, -8.90566876009594e-06, -8.73387187566924e-06, -1.29444174865254e-05, 0.0291460988640959, 0.0106032943169836, 0.0108632141195972, -0.0765964287661009, -0.037927226546748, 0.12946407540304, 0.00108025695398424, 0.00038541421826549, 0.000379616920949843, -0.00278416799838657, -0.00132537357759244, 0.00452414480070562, 8.22624045636877e-05, 5.92733532525092e-05, 7.7560747413663e-05, -0.000212016551629432, -0.000203830924063578, 0.000344517134121702, 9.73312106873625e-06, 7.01310308668171e-06, 9.17683051902124e-06, 3.47258421642266e-06, 3.33851316924871e-06, 3.42035053585432e-06, -6.47806177060777e-05, -6.22795393426488e-05, -3.08381243864786e-05, 0.000222769464588879}};
    
    
    // Compute element tensor using UFL quadrature representation
    // Optimisations: ('simplify expressions', True), ('ignore zero tables', True), ('non zero columns', True), ('remove zero terms', True), ('ignore ones', True)
    // Total number of operations to compute element tensor: 48816
    
    // Loop quadrature points for integral
    // Number of operations to compute element tensor for following IP loop = 48816
    for (unsigned int ip = 0; ip < 216; ip++)
    {
      
      // Function declarations
      double F0 = 0;
      
      // Total number of operations to compute function values = 112
      for (unsigned int r = 0; r < 56; r++)
      {
        F0 += FE0[ip][r]*w[0][r];
      }// end loop over 'r'
      
      // Number of operations to compute ip constants: 2
      // Number of operations: 2
      const double Gip0 = F0*W216[ip]*det;
      
      
      // Number of operations for primary indices = 112
      for (unsigned int j = 0; j < 56; j++)
      {
        // Number of operations to compute entry = 2
        A[j] += FE0[ip][j]*Gip0;
      }// end loop over 'j'
    }// end loop over 'ip'
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class poisson3d_5_1_cell_integral_0: public ufc::cell_integral
{
private:

  poisson3d_5_1_cell_integral_0_quadrature integral_0_quadrature;

public:

  /// Constructor
  poisson3d_5_1_cell_integral_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_1_cell_integral_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Reset values of the element tensor block
    for (unsigned int j = 0; j < 56; j++)
      A[j] = 0;
    
    // Add all contributions to element tensor
    integral_0_quadrature.tabulate_tensor(A, w, c);
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class poisson3d_5_form_1: public ufc::form
{
public:

  /// Constructor
  poisson3d_5_form_1() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~poisson3d_5_form_1()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "Form([Integral(Product(BasisFunction(FiniteElement('Lagrange', Cell('tetrahedron', 1, Space(3)), 5), 0), Function(FiniteElement('Lagrange', Cell('tetrahedron', 1, Space(3)), 5), 0)), Measure('cell', 0, None))])";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 1;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 1;
  }

  /// Return the number of cell integrals
  virtual unsigned int num_cell_integrals() const
  {
    return 1;
  }

  /// Return the number of exterior facet integrals
  virtual unsigned int num_exterior_facet_integrals() const
  {
    return 0;
  }

  /// Return the number of interior facet integrals
  virtual unsigned int num_interior_facet_integrals() const
  {
    return 0;
  }

  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new poisson3d_5_1_finite_element_0();
      break;
    case 1:
      return new poisson3d_5_1_finite_element_1();
      break;
    }
    return 0;
  }

  /// Create a new dof map for argument function i
  virtual ufc::dof_map* create_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new poisson3d_5_1_dof_map_0();
      break;
    case 1:
      return new poisson3d_5_1_dof_map_1();
      break;
    }
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    return new poisson3d_5_1_cell_integral_0();
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return 0;
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

// DOLFIN wrappers

// Standard library includes
#include <string>

// DOLFIN includes
#include <dolfin/common/NoDeleter.h>
#include <dolfin/fem/FiniteElement.h>
#include <dolfin/fem/DofMap.h>
#include <dolfin/fem/Form.h>
#include <dolfin/function/FunctionSpace.h>
#include <dolfin/function/Function.h>
#include <dolfin/function/Coefficient.h>

namespace Poisson3D_5
{

class CoefficientReference
{
public:
  
  CoefficientReference(const std::string & name):
    _name(name)
  {}
  
  ~CoefficientReference()
  {}

  CoefficientReference& operator=(dolfin::Function & f)
  {
    _function_pointer = dolfin::reference_to_no_delete_pointer(f);
    return *this;
  }
  
  CoefficientReference& operator=(const boost::shared_ptr<dolfin::Function> f)
  {
    _function_pointer = f;
    return *this;
  }

  std::string name() const
  {
    return _name;
  }

  dolfin::Function & function() const
  {
    return *_function_pointer;
  }

  boost::shared_ptr<dolfin::Function> function_pointer() const
  {
    return _function_pointer;
  }

private:

  boost::shared_ptr<dolfin::Function> _function_pointer;
  std::string _name;

};

class CoefficientSet
{
public:

  CoefficientSet():
    f("f")
  {}
  
  ~CoefficientSet()
  {}

  dolfin::uint num_coefficients() const
  {
    return 1;
  }

  std::string name(dolfin::uint i) const
  {
    switch(i)
    {
    case 0: return "f";
    default:
        throw std::runtime_error("Invalid coefficient number.");
    }
  }

  void disp() const
  {
    for(dolfin::uint i = 0; i < num_coefficients(); ++i)
    {
        dolfin::cout << "Coefficient " << i << ": \t" << name(i) << dolfin::endl;
    }
  }

  CoefficientReference f;
};

class CoefficientSpace_f: public dolfin::FunctionSpace
{
public:


  CoefficientSpace_f(const dolfin::Mesh & mesh):
      dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                            boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_1_finite_element_1()))),
                            boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson3d_5_1_dof_map_1()), dolfin::reference_to_no_delete_pointer(mesh))))
  {
    // Do nothing
  }

  CoefficientSpace_f(dolfin::Mesh & mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_1_finite_element_1()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson3d_5_1_dof_map_1()), dolfin::reference_to_no_delete_pointer(mesh))))
  {
    // Do nothing
  }

  CoefficientSpace_f(boost::shared_ptr<dolfin::Mesh> mesh):
      dolfin::FunctionSpace(mesh,
                            boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_1_finite_element_1()))),
                            boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson3d_5_1_dof_map_1()), mesh)))
  {
      // Do nothing
  }

  CoefficientSpace_f(boost::shared_ptr<const dolfin::Mesh> mesh):
      dolfin::FunctionSpace(mesh,
                            boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_1_finite_element_1()))),
                            boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson3d_5_1_dof_map_1()), mesh)))
  {
      // Do nothing
  }
 

  ~CoefficientSpace_f()
  {
  }
  
};

class Form_0_FunctionSpace_0: public dolfin::FunctionSpace
{
public:


  Form_0_FunctionSpace_0(const dolfin::Mesh & mesh):
      dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                            boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_0_finite_element_0()))),
                            boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson3d_5_0_dof_map_0()), dolfin::reference_to_no_delete_pointer(mesh))))
  {
    // Do nothing
  }

  Form_0_FunctionSpace_0(dolfin::Mesh & mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_0_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson3d_5_0_dof_map_0()), dolfin::reference_to_no_delete_pointer(mesh))))
  {
    // Do nothing
  }

  Form_0_FunctionSpace_0(boost::shared_ptr<dolfin::Mesh> mesh):
      dolfin::FunctionSpace(mesh,
                            boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_0_finite_element_0()))),
                            boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson3d_5_0_dof_map_0()), mesh)))
  {
      // Do nothing
  }

  Form_0_FunctionSpace_0(boost::shared_ptr<const dolfin::Mesh> mesh):
      dolfin::FunctionSpace(mesh,
                            boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_0_finite_element_0()))),
                            boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson3d_5_0_dof_map_0()), mesh)))
  {
      // Do nothing
  }
 

  ~Form_0_FunctionSpace_0()
  {
  }
  
};

class Form_0_FunctionSpace_1: public dolfin::FunctionSpace
{
public:


  Form_0_FunctionSpace_1(const dolfin::Mesh & mesh):
      dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                            boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_0_finite_element_1()))),
                            boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson3d_5_0_dof_map_1()), dolfin::reference_to_no_delete_pointer(mesh))))
  {
    // Do nothing
  }

  Form_0_FunctionSpace_1(dolfin::Mesh & mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_0_finite_element_1()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson3d_5_0_dof_map_1()), dolfin::reference_to_no_delete_pointer(mesh))))
  {
    // Do nothing
  }

  Form_0_FunctionSpace_1(boost::shared_ptr<dolfin::Mesh> mesh):
      dolfin::FunctionSpace(mesh,
                            boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_0_finite_element_1()))),
                            boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson3d_5_0_dof_map_1()), mesh)))
  {
      // Do nothing
  }

  Form_0_FunctionSpace_1(boost::shared_ptr<const dolfin::Mesh> mesh):
      dolfin::FunctionSpace(mesh,
                            boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_0_finite_element_1()))),
                            boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson3d_5_0_dof_map_1()), mesh)))
  {
      // Do nothing
  }
 

  ~Form_0_FunctionSpace_1()
  {
  }
  
};

class Form_0: public dolfin::Form
{
public:

  // Constructor
  Form_0(const dolfin::FunctionSpace & _V0, const dolfin::FunctionSpace & _V1):
    dolfin::Form(2, 0)
  {
    boost::shared_ptr<const dolfin::FunctionSpace> _pV0 = dolfin::reference_to_no_delete_pointer(_V0);
    boost::shared_ptr<const dolfin::FunctionSpace> _pV1 = dolfin::reference_to_no_delete_pointer(_V1);

    _function_spaces[0] = _pV0;
    _function_spaces[1] = _pV1;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_0());
  }

  // Constructor
  Form_0(const dolfin::FunctionSpace & _V0, const dolfin::FunctionSpace & _V1, const CoefficientSet & coefficients):
    dolfin::Form(2, 0)
  {
    boost::shared_ptr<const dolfin::FunctionSpace> _pV0 = dolfin::reference_to_no_delete_pointer(_V0);
    boost::shared_ptr<const dolfin::FunctionSpace> _pV1 = dolfin::reference_to_no_delete_pointer(_V1);

    _function_spaces[0] = _pV0;
    _function_spaces[1] = _pV1;

    update_coefficients(coefficients);

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_0());
  }

  // Constructor
  Form_0(boost::shared_ptr<const dolfin::FunctionSpace> _pV0, boost::shared_ptr<const dolfin::FunctionSpace> _pV1):
    dolfin::Form(2, 0)
  {
    _function_spaces[0] = _pV0;
    _function_spaces[1] = _pV1;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_0());
  }

  // Constructor
  Form_0(boost::shared_ptr<const dolfin::FunctionSpace> _pV0, boost::shared_ptr<const dolfin::FunctionSpace> _pV1, const CoefficientSet & coefficients):
    dolfin::Form(2, 0)
  {
    _function_spaces[0] = _pV0;
    _function_spaces[1] = _pV1;

    update_coefficients(coefficients);

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_0());
  }

  // Destructor
  ~Form_0()
  {}

  void update_coefficients(const CoefficientSet & coefficients)
  {

  }

  /// Return the number of the coefficient with this name
  virtual dolfin::uint coefficient_number(const std::string & name) const
  {
    dolfin::error("No coefficients.");
    return 0;
  }
  
  /// Return the name of the coefficient with this number
  virtual std::string coefficient_name(dolfin::uint i) const
  {
    dolfin::error("No coefficients.");
    return "unnamed";
  }

  // Typedefs
  typedef Form_0_FunctionSpace_0 TestSpace;
  typedef Form_0_FunctionSpace_1 TrialSpace;

  // Coefficients
};

class Form_1_FunctionSpace_0: public dolfin::FunctionSpace
{
public:


  Form_1_FunctionSpace_0(const dolfin::Mesh & mesh):
      dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                            boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_1_finite_element_0()))),
                            boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson3d_5_1_dof_map_0()), dolfin::reference_to_no_delete_pointer(mesh))))
  {
    // Do nothing
  }

  Form_1_FunctionSpace_0(dolfin::Mesh & mesh):
    dolfin::FunctionSpace(dolfin::reference_to_no_delete_pointer(mesh),
                          boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_1_finite_element_0()))),
                          boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson3d_5_1_dof_map_0()), dolfin::reference_to_no_delete_pointer(mesh))))
  {
    // Do nothing
  }

  Form_1_FunctionSpace_0(boost::shared_ptr<dolfin::Mesh> mesh):
      dolfin::FunctionSpace(mesh,
                            boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_1_finite_element_0()))),
                            boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson3d_5_1_dof_map_0()), mesh)))
  {
      // Do nothing
  }

  Form_1_FunctionSpace_0(boost::shared_ptr<const dolfin::Mesh> mesh):
      dolfin::FunctionSpace(mesh,
                            boost::shared_ptr<const dolfin::FiniteElement>(new dolfin::FiniteElement(boost::shared_ptr<ufc::finite_element>(new poisson3d_5_1_finite_element_0()))),
                            boost::shared_ptr<const dolfin::DofMap>(new dolfin::DofMap(boost::shared_ptr<ufc::dof_map>(new poisson3d_5_1_dof_map_0()), mesh)))
  {
      // Do nothing
  }
 

  ~Form_1_FunctionSpace_0()
  {
  }
  
};

typedef CoefficientSpace_f Form_1_FunctionSpace_1;

class Form_1_Coefficient_f: public dolfin::Coefficient
{
public:

  Form_1_Coefficient_f(dolfin::Form & form):
    dolfin::Coefficient(form)
  {}
  
  ~Form_1_Coefficient_f()
  {}

  const Form_1_Coefficient_f& operator=(dolfin::Function & f)
  {
    attach(f);
    return *this;
  }
  
  const Form_1_Coefficient_f& operator=(boost::shared_ptr<dolfin::Function> f)
  {
    attach(f);
    return *this;
  }
  
  const dolfin::FunctionSpace * create_function_space() const
  {
    return new Form_1_FunctionSpace_1(form.mesh());
  }
  
  dolfin::uint number() const
  {
    return 0;
  }

  std::string name() const
  {
    return "f";
  }

};

class Form_1: public dolfin::Form
{
public:

  // Constructor
  Form_1(const dolfin::FunctionSpace & _V0):
    dolfin::Form(1, 1), f(*this)
  {
    boost::shared_ptr<const dolfin::FunctionSpace> _pV0 = dolfin::reference_to_no_delete_pointer(_V0);

    _function_spaces[0] = _pV0;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Constructor
  Form_1(const dolfin::FunctionSpace & _V0, const CoefficientSet & coefficients):
    dolfin::Form(1, 1), f(*this)
  {
    boost::shared_ptr<const dolfin::FunctionSpace> _pV0 = dolfin::reference_to_no_delete_pointer(_V0);

    _function_spaces[0] = _pV0;

    update_coefficients(coefficients);

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Constructor
  Form_1(const dolfin::FunctionSpace & _V0, dolfin::Function & _f):
    dolfin::Form(1, 1), f(*this)
  {
    boost::shared_ptr<const dolfin::FunctionSpace> _pV0 = dolfin::reference_to_no_delete_pointer(_V0);

    _function_spaces[0] = _pV0;

    boost::shared_ptr<dolfin::Function> _pf = dolfin::reference_to_no_delete_pointer(_f);

    f = _pf;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Constructor
  Form_1(const dolfin::FunctionSpace & _V0, boost::shared_ptr<dolfin::Function> _pf):
    dolfin::Form(1, 1), f(*this)
  {
    boost::shared_ptr<const dolfin::FunctionSpace> _pV0 = dolfin::reference_to_no_delete_pointer(_V0);

    _function_spaces[0] = _pV0;

    f = _pf;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Constructor
  Form_1(boost::shared_ptr<const dolfin::FunctionSpace> _pV0):
    dolfin::Form(1, 1), f(*this)
  {
    _function_spaces[0] = _pV0;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Constructor
  Form_1(boost::shared_ptr<const dolfin::FunctionSpace> _pV0, const CoefficientSet & coefficients):
    dolfin::Form(1, 1), f(*this)
  {
    _function_spaces[0] = _pV0;

    update_coefficients(coefficients);

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Constructor
  Form_1(boost::shared_ptr<const dolfin::FunctionSpace> _pV0, dolfin::Function & _f):
    dolfin::Form(1, 1), f(*this)
  {
    _function_spaces[0] = _pV0;

    boost::shared_ptr<dolfin::Function> _pf = dolfin::reference_to_no_delete_pointer(_f);

    f = _pf;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Constructor
  Form_1(boost::shared_ptr<const dolfin::FunctionSpace> _pV0, boost::shared_ptr<dolfin::Function> _pf):
    dolfin::Form(1, 1), f(*this)
  {
    _function_spaces[0] = _pV0;

    f = _pf;

    _ufc_form = boost::shared_ptr<const ufc::form>(new poisson3d_5_form_1());
  }

  // Destructor
  ~Form_1()
  {}

  void update_coefficients(const CoefficientSet & coefficients)
  {
    f = coefficients.f.function_pointer();
  }

  /// Return the number of the coefficient with this name
  virtual dolfin::uint coefficient_number(const std::string & name) const
  {
    if(name == "f") return 0;
    dolfin::error("Invalid coefficient.");
    return 0;
  }
  
  /// Return the name of the coefficient with this number
  virtual std::string coefficient_name(dolfin::uint i) const
  {
    switch(i)
    {
      case 0: return "f";
    }
    dolfin::error("Invalid coefficient.");
    return "unnamed";
  }

  // Typedefs
  typedef Form_1_FunctionSpace_0 TestSpace;
  typedef Form_1_FunctionSpace_1 CoefficientSpace_f;

  // Coefficients
  Form_1_Coefficient_f f;
};

// Class typedefs
typedef Form_0 BilinearForm;
typedef Form_1 LinearForm;
typedef Form_0::TestSpace FunctionSpace;

} // namespace Poisson3D_5

#endif
